/*   ---  */

-- Need to call the custom attribute store procedure to drop the staging table. 
-- By this time the procedure is already loaded.
BEGIN
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vStagingTableName VARCHAR(1024);
  DECLARE @vStagingColExists INTEGER;   


FOR AttributesLOOP as AttributesCUR dynamic scroll cursor for
      select CUSTOM_ATTRIBUTE_KEY as CUSTOM_ATTRIBUTE_KEY_R,
        CUSTOM_ATTRIBUTE_CONTEXT as CUSTOM_ATTRIBUTE_CONTEXT_R,
        TYPE as TYPE_R,
        DISPLAY_NAME as DISPLAY_NAME_R,
        COLUMN_NAME as COLUMN_NAME_R,
        COLUMN_TYPE as COLUMN_TYPE_R,
        SOURCE_NAME as SOURCE_NAME_R    
      from SAHARA.CUSTOM_ATTRIBUTE CONFIGURED
      where CONFIGURED.SOURCE_NAME not in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY')
	    and exists (select 1
                         from SAHARA.P_CUSTOM_ATTRIBUTE PUBLISHED
                        where PUBLISHED.CUSTOM_ATTRIBUTE_KEY =
                              CONFIGURED.CUSTOM_ATTRIBUTE_KEY)
     DO            

            set @vStagingTableName = CUSTOM_ATTRIBUTE_CONTEXT_R || '_';

    
                select SAHARA.fn_HELPER_ColumnExists('SAHARA_APP',@vStagingTableName,COLUMN_NAME_R) into 
                        @vStagingColExists ;

                if @vStagingColExists = 1 then        
                    SET @vSQL = 'lock table ' ||
                    -- source table or view
                    'SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R || '_ ' ||
                            'in exclusive mode';

                    EXECUTE IMMEDIATE @vSQL;
	                SET @vSQL = 'alter table SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R ||
                        '_ drop  "' || COLUMN_NAME_R || '"';
                    EXECUTE IMMEDIATE @vSQL;                    
                end if;            
      COMMIT ;
     END FOR;
 END;

/* Sahara Tables*/

/* Table: DATABASE */
if not exists (select 1 from sys.syscolumns where tname='DATABASES' and cname='IS_BASED_ON_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASES ADD IS_BASED_ON_DB_VM	char(1)	null default 'F';
	comment on column SAHARA.DATABASES.IS_BASED_ON_DB_VM is 
	'A Boolean flag that indicates whether a database is using database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_BASED_ON_DB_VM_DATABASE' and table_name='DATABASES')
then
	alter table SAHARA.DATABASES
	   alter IS_BASED_ON_DB_VM add constraint CKC_IS_BASED_ON_DB_VM_DATABASE check (IS_BASED_ON_DB_VM is null or (IS_BASED_ON_DB_VM in ('T','F')));
end if;

/* Index: I_DATABASES_ISBASEDONTHIN */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_ISBASEDONTHIN')
then
	create index I_DATABASES_ISBASEDONTHIN on SAHARA.DATABASES (
	IS_BASED_ON_THIN ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_ISBASEDONTHIN is 
	'An index on Is Based On Thin for dimensional queries.';   
end if;

/* Index: I_DATABASES_APPLICATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_APPLICATIONBE')
then
	create index I_DATABASES_APPLICATIONBE on SAHARA.DATABASES (
	APPLICATION_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_APPLICATIONBE is 
	'An index on Application Business Entity for dimensional queries.';
end if;

/* Index: I_DATABASES_PPLCTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_PPLCTNCMPSTB')
then
	create index I_DATABASES_PPLCTNCMPSTB on SAHARA.DATABASES (
	APPLICATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_PPLCTNCMPSTB is 
	'An index on Composite Application Business Entity for dimensional queries.'; 
end if;

/* Index: I_DATABASES_ORGANIZATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_ORGANIZATIONBE')
then
	create index I_DATABASES_ORGANIZATIONBE on SAHARA.DATABASES (
	ORGANIZATION_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_ORGANIZATIONBE is 
	'An index on Organization Business Entity for dimensional queries.';  
end if;

/* Index: I_DATABASES_RGNZTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_RGNZTNCMPSTB')
then
	create index I_DATABASES_RGNZTNCMPSTB on SAHARA.DATABASES (
	ORGANIZATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_RGNZTNCMPSTB is 
	'An index on Composite Organization Business Entity for dimensional queries.';
end if;

/* Index: I_DATABASES_ISBASEDONDBVM */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_ISBASEDONDBVM')
then
	create index I_DATABASES_ISBASEDONDBVM on SAHARA.DATABASES (
	IS_BASED_ON_DB_VM ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_ISBASEDONDBVM is 
	'An index on Is Based on DB VM for dimensional queries.';   
end if;

/* Table: DATABASE_ */

if not exists (select 1 from sys.syscolumns where tname='DATABASES_' and cname='IS_BASED_ON_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASES_ ADD IS_BASED_ON_DB_VM	char(1)	null default 'F';
	comment on column SAHARA_APP.DATABASES_.IS_BASED_ON_DB_VM is 
	'A Boolean flag that indicates whether a database is using database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_BASED_ON_DB_VM_DATABASE' and table_name='DATABASES_')
then
	alter table SAHARA_APP.DATABASES_
	alter IS_BASED_ON_DB_VM add constraint CKC_IS_BASED_ON_DB_VM_DATABASE check (IS_BASED_ON_DB_VM is null or (IS_BASED_ON_DB_VM in ('T','F')));
end if;


/* Table: SAHARA.DATABASE_CAPACITY  */
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY' and cname='CAPACITY_FROM_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASE_CAPACITY ADD CAPACITY_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA.DATABASE_CAPACITY.CAPACITY_FROM_DB_VM is 
	'Capacity from database files deriving storage from database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY' and cname='AVAILABLE_FROM_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASE_CAPACITY ADD AVAILABLE_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA.DATABASE_CAPACITY.AVAILABLE_FROM_DB_VM is 
	'Capacity available for database,  based on database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY' and cname='USED_FROM_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASE_CAPACITY ADD USED_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA.DATABASE_CAPACITY.USED_FROM_DB_VM is 
	'Capacity used by database,  based on database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_CAPACITY_FROM_DB__DATABASE' and table_name='DATABASE_CAPACITY')
then
	alter table SAHARA.DATABASE_CAPACITY
	alter CAPACITY_FROM_DB_VM add constraint CKC_CAPACITY_FROM_DB__DATABASE check (CAPACITY_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AVAILABLE_FROM_DB_DATABASE' and table_name='DATABASE_CAPACITY')
then
	alter table SAHARA.DATABASE_CAPACITY
	alter AVAILABLE_FROM_DB_VM add constraint CKC_AVAILABLE_FROM_DB_DATABASE check (AVAILABLE_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_FROM_DB_VM_DATABASE' and table_name='DATABASE_CAPACITY')
then
	alter table SAHARA.DATABASE_CAPACITY
	alter USED_FROM_DB_VM add constraint CKC_USED_FROM_DB_VM_DATABASE check (USED_FROM_DB_VM >= 0);
end if;

/* Table: DATABASE_CAPACITY_  */

if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY_' and cname='CAPACITY_FROM_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASE_CAPACITY_ ADD CAPACITY_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DATABASE_CAPACITY_.CAPACITY_FROM_DB_VM is 
	'Capacity from database files deriving storage from database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY_' and cname='AVAILABLE_FROM_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASE_CAPACITY_ ADD AVAILABLE_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DATABASE_CAPACITY_.AVAILABLE_FROM_DB_VM is 
	'Capacity available for database,  based on database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY_' and cname='USED_FROM_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASE_CAPACITY_ ADD USED_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DATABASE_CAPACITY_.USED_FROM_DB_VM is 
	'Capacity used by database,  based on database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_CAPACITY_FROM_DB__DATABASE' and table_name='DATABASE_CAPACITY_')
then
	alter table SAHARA_APP.DATABASE_CAPACITY_
	alter CAPACITY_FROM_DB_VM add constraint CKC_CAPACITY_FROM_DB__DATABASE check (CAPACITY_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AVAILABLE_FROM_DB_DATABASE' and table_name='DATABASE_CAPACITY_')
then
	alter table SAHARA_APP.DATABASE_CAPACITY_
	alter AVAILABLE_FROM_DB_VM add constraint CKC_AVAILABLE_FROM_DB_DATABASE check (AVAILABLE_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_FROM_DB_VM_DATABASE' and table_name='DATABASE_CAPACITY_')
then
	alter table SAHARA_APP.DATABASE_CAPACITY_
	alter USED_FROM_DB_VM add constraint CKC_USED_FROM_DB_VM_DATABASE check (USED_FROM_DB_VM >= 0);
end if;

/* Table: DISKGROUP  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP' and cname='REDUNDANCY_TYPE') 
then
	ALTER TABLE SAHARA.DISKGROUP ADD REDUNDANCY_TYPE varchar(256) null;
	comment on column SAHARA.DISKGROUP.REDUNDANCY_TYPE is 
	'This attribute distinguishes between mirrored disk groups and non-mirrored disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP' and cname='IS_USED_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA.DISKGROUP ADD IS_USED_BY_CLUSTERED_DB char(1) null default 'F';
	comment on column SAHARA.DISKGROUP.IS_USED_BY_CLUSTERED_DB is 
	'A Boolean flag that indicates whether a disk group is utilized by a clustered database like Oracle RAC.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP' and cname='USES_VM_VOLUMES') 
then
	ALTER TABLE SAHARA.DISKGROUP ADD USES_VM_VOLUMES char(1) null default 'F';
	comment on column SAHARA.DISKGROUP.USES_VM_VOLUMES is 
	'A Boolean flag that indicates whether a disk group uses volume managers volume.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_CLUSTE_DISKGROU' and table_name='DISKGROUP')
then
	alter table SAHARA.DISKGROUP
	alter IS_USED_BY_CLUSTERED_DB add constraint CKC_IS_USED_BY_CLUSTE_DISKGROU check (IS_USED_BY_CLUSTERED_DB is null or (IS_USED_BY_CLUSTERED_DB in ('T','F')));
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USES_VM_VOLUMES_DISKGROU' and table_name='DISKGROUP')
then
	alter table SAHARA.DISKGROUP
	alter USES_VM_VOLUMES add constraint CKC_USES_VM_VOLUMES_DISKGROU check (USES_VM_VOLUMES is null or (USES_VM_VOLUMES in ('T','F')));
end if;

/* Index: I_DISKGROUP_DISKGROUPOID */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_DISKGROUPOID')
then
	create index I_DISKGROUP_DISKGROUPOID on SAHARA.DISKGROUP (
	DISKGROUP_OID ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_DISKGROUPOID is 
	'An index on Diskgroup Object Identifier for dimensional queries.';
end if;

/* Index: I_DISKGROUP_FAMILY */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_FAMILY')
then
	create index I_DISKGROUP_FAMILY on SAHARA.DISKGROUP (
	FAMILY ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_FAMILY is 
	'An index on Family for dimensional queries.';
end if;

/* Index: I_DISKGROUP_VERSION */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_VERSION')
then
	create index I_DISKGROUP_VERSION on SAHARA.DISKGROUP (
	VERSION ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_VERSION is 
	'An index on Version for dimensional queries.';
end if;

/* Index: I_DISKGROUP_REDUNDANCYTYPE */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_REDUNDANCYTYPE')
then
	create index I_DISKGROUP_REDUNDANCYTYPE on SAHARA.DISKGROUP (
	REDUNDANCY_TYPE ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_REDUNDANCYTYPE is 
	'An index on Redundancy Type for dimensional queries.';
end if;

/* Index: I_DISKGROUP_SSDBCLSTRDDB */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_SSDBCLSTRDDB')
then
	create index I_DISKGROUP_SSDBCLSTRDDB on SAHARA.DISKGROUP (
	IS_USED_BY_CLUSTERED_DB ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_SSDBCLSTRDDB is 
	'An index on Is Used By Clustered DB for dimensional queries.';
end if;

/* Index: I_DISKGROUP_USESVMVOLUMES */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_USESVMVOLUMES')
then
	create index I_DISKGROUP_USESVMVOLUMES on SAHARA.DISKGROUP (
	USES_VM_VOLUMES ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_USESVMVOLUMES is 
	'An index on Uses VM Volumes for dimensional queries.';
end if;

/* Index: I_DISKGROUP_ISBASEDONTHIN */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_ISBASEDONTHIN')
then
	create index I_DISKGROUP_ISBASEDONTHIN on SAHARA.DISKGROUP (
	IS_BASED_ON_THIN ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_ISBASEDONTHIN is 
	'An index on Is Based On Thin for dimensional queries.';
end if;

/* Index: I_DISKGROUP_APPLICATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_APPLICATIONBE')
then
	create index I_DISKGROUP_APPLICATIONBE on SAHARA.DISKGROUP (
	APPLICATION_BE ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_APPLICATIONBE is 
	'An index on Application Business Entity for dimensional queries.';
end if;

/* Index: I_DISKGROUP_ORGANIZATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_ORGANIZATIONBE')
then
	create index I_DISKGROUP_ORGANIZATIONBE on SAHARA.DISKGROUP (
	ORGANIZATION_BE ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_ORGANIZATIONBE is 
	'An index on Organization Business Entity for dimensional queries.';
end if;

/* Index: I_DISKGROUP_PPLCTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_PPLCTNCMPSTB')
then
	create index I_DISKGROUP_PPLCTNCMPSTB on SAHARA.DISKGROUP (
	APPLICATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_PPLCTNCMPSTB is 
	'An index on Composite Application Business Entity for dimensional queries.';
end if;

/* Index: I_DISKGROUP_RGNZTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_RGNZTNCMPSTB')
then
	create index I_DISKGROUP_RGNZTNCMPSTB on SAHARA.DISKGROUP (
	ORGANIZATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_RGNZTNCMPSTB is 
	'An index on Composite Organization Business Entity for dimensional queries.';
end if;

/* Index: I_DISKGROUP_SBSDNDRCTTTCHD */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_SBSDNDRCTTTCHD')
then
	create index I_DISKGROUP_SBSDNDRCTTTCHD on SAHARA.DISKGROUP (
	IS_BASED_ON_DIRECT_ATTACHED ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_SBSDNDRCTTTCHD is 
	'An index on Is Based On Direct Attached for dimensional queries.';
end if;

/* Index: I_DISKGROUP_STATE */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_STATE')
then
	create index I_DISKGROUP_STATE on SAHARA.DISKGROUP (
	STATE ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_STATE is 
	'An index on State for dimensional queries.';
end if;

/* Index: I_DISKGROUP_T2SCRRNTT2SDLTDD */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_T2SCRRNTT2SDLTDD')
then
	create index I_DISKGROUP_T2SCRRNTT2SDLTDD on SAHARA.DISKGROUP (
	T2_IS_CURRENT ASC,
	T2_IS_DELETED ASC,
	DISKGROUP_OID ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_T2SCRRNTT2SDLTDD is 
	'An index on Type 2 - Is Current, Type 2 - Is Deleted and Diskgroup Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'FK_FILE_CAT_REFERENCE_BVIEW_IM' and table_name='FILE_CATEGORY')
then
	alter table SAHARA.FILE_CATEGORY
	add constraint FK_FILE_CAT_REFERENCE_BVIEW_IM foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete restrict;
end if;


/* Table: SAHARA_APP.DISKGROUP_  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_' and cname='REDUNDANCY_TYPE') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_ ADD REDUNDANCY_TYPE varchar(256) null;
	comment on column SAHARA_APP.DISKGROUP_.REDUNDANCY_TYPE is 
	'This attribute distinguishes between mirrored disk groups and non-mirrored disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_' and cname='IS_USED_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_ ADD IS_USED_BY_CLUSTERED_DB char(1) null default 'F';
	comment on column SAHARA_APP.DISKGROUP_.IS_USED_BY_CLUSTERED_DB is 
	'A Boolean flag that indicates whether a disk group is utilized by a clustered database like Oracle RAC.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_' and cname='USES_VM_VOLUMES') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_ ADD USES_VM_VOLUMES char(1) null default 'F';
	comment on column SAHARA_APP.DISKGROUP_.USES_VM_VOLUMES is 
	'A Boolean flag that indicates whether a disk group uses volume managers volume.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_CLUSTE_DISKGROU' and table_name='DISKGROUP_')
then
	alter table SAHARA_APP.DISKGROUP_
	alter IS_USED_BY_CLUSTERED_DB add constraint CKC_IS_USED_BY_CLUSTE_DISKGROU check (IS_USED_BY_CLUSTERED_DB is null or (IS_USED_BY_CLUSTERED_DB in ('T','F')));
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USES_VM_VOLUMES_DISKGROU' and table_name='DISKGROUP_')
then
	alter table SAHARA_APP.DISKGROUP_
	alter USES_VM_VOLUMES add constraint CKC_USES_VM_VOLUMES_DISKGROU check (USES_VM_VOLUMES is null or (USES_VM_VOLUMES in ('T','F')));
end if;

/* Table: SAHARA.DISKGROUP_CAPACITY  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.OVERHEAD_CAPACITY is 
	'Capacity of disk groups gone into mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.UNKNOWN_CAPACITY is 
	'Capacity of disk groups gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.DB_USED_CAPACITY is 
	'Capacity of database volume managers disk groups used by databases.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='USED_CAPACITY_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD USED_CAPACITY_BY_CLUSTERED_DB numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.USED_CAPACITY_BY_CLUSTERED_DB is 
	'Capacity of database volume managers disk groups used by clustered databases like Oracle RAC.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERHEAD_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_DISKGROU check (OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_UNKNOWN_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter UNKNOWN_CAPACITY add constraint CKC_UNKNOWN_CAPACITY_DISKGROU check (UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_USED_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter DB_USED_CAPACITY add constraint CKC_DB_USED_CAPACITY_DISKGROU check (DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_CAPACITY_BY__DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter USED_CAPACITY_BY_CLUSTERED_DB add constraint CKC_USED_CAPACITY_BY__DISKGROU check (USED_CAPACITY_BY_CLUSTERED_DB >= 0);
end if;

/* Table: SAHARA_APP.DISKGROUP_CAPACITY_  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.OVERHEAD_CAPACITY is 
	'Capacity of disk groups gone into mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.UNKNOWN_CAPACITY is 
	'Capacity of disk groups gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.DB_USED_CAPACITY is 
	'Capacity of database volume managers disk groups used by databases.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='USED_CAPACITY_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD USED_CAPACITY_BY_CLUSTERED_DB numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.USED_CAPACITY_BY_CLUSTERED_DB is 
	'Capacity of database volume managers disk groups used by clustered databases like Oracle RAC.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERHEAD_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_DISKGROU check (OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_UNKNOWN_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter UNKNOWN_CAPACITY add constraint CKC_UNKNOWN_CAPACITY_DISKGROU check (UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_USED_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter DB_USED_CAPACITY add constraint CKC_DB_USED_CAPACITY_DISKGROU check (DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_CAPACITY_BY__DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter USED_CAPACITY_BY_CLUSTERED_DB add constraint CKC_USED_CAPACITY_BY__DISKGROU check (USED_CAPACITY_BY_CLUSTERED_DB >= 0);
end if;

/* Table: SAHARA.HOST_CAPACITY  */
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_OVERHEAD_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='APPS_ON_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD APPS_ON_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.APPS_ON_DB_VM_CAPACITY is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='APPS_ON_DB_VM_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD APPS_ON_DB_VM_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.APPS_ON_DB_VM_OTHER_CAPACITY is 
	'For shared storage, amount of storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_DB_USED_CAPACITY is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_DB_USED_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_DB_USED_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_DB_USED_OTHER_CAPACITY is 
	'For shared storage, amount of used storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_UNKNOWN_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_OVERHEAD_CAP') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_OVERHEAD_CAP numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_OVERHEAD_CAP is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_APPS_ON_DB_VM_CAP') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_APPS_ON_DB_VM_CAP numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_APPS_ON_DB_VM_CAP is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_DB_USED_CAP') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_DB_USED_CAP numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_DB_USED_CAP is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_CAPACITY_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_CAPACITY add constraint CKC_DB_VM_CAPACITY_HOST_CAP check (DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_USED_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_USED_CAPACITY add constraint CKC_DB_VM_USED_CAPACI_HOST_CAP check (DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_FREE_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_FREE_CAPACITY add constraint CKC_DB_VM_FREE_CAPACI_HOST_CAP check (DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_OVERHEAD_CA_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_OVERHEAD_CAPACITY add constraint CKC_DB_VM_OVERHEAD_CA_HOST_CAP check (DB_VM_OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_CAP_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter APPS_ON_DB_VM_CAPACITY add constraint CKC_APPS_ON_DB_VM_CAP_HOST_CAP check (APPS_ON_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_OTH_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter APPS_ON_DB_VM_OTHER_CAPACITY add constraint CKC_APPS_ON_DB_VM_OTH_HOST_CAP check (APPS_ON_DB_VM_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_CAP_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_DB_USED_CAPACITY add constraint CKC_DB_VM_DB_USED_CAP_HOST_CAP check (DB_VM_DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_OTH_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_DB_USED_OTHER_CAPACITY add constraint CKC_DB_VM_DB_USED_OTH_HOST_CAP check (DB_VM_DB_USED_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_UNKNOWN_CAP_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_UNKNOWN_CAPACITY add constraint CKC_DB_VM_UNKNOWN_CAP_HOST_CAP check (DB_VM_UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_CA_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_CAPACITY add constraint CKC_ADJUSTED_DB_VM_CA_HOST_CAP check (ADJUSTED_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_US_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_USED_CAPACITY add constraint CKC_ADJUSTED_DB_VM_US_HOST_CAP check (ADJUSTED_DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_FR_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_FREE_CAPACITY add constraint CKC_ADJUSTED_DB_VM_FR_HOST_CAP check (ADJUSTED_DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_OV_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_OVERHEAD_CAP add constraint CKC_ADJUSTED_DB_VM_OV_HOST_CAP check (ADJUSTED_DB_VM_OVERHEAD_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_APPS_ON__HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_APPS_ON_DB_VM_CAP add constraint CKC_ADJUSTED_APPS_ON__HOST_CAP check (ADJUSTED_APPS_ON_DB_VM_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_DB_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_DB_USED_CAP add constraint CKC_ADJUSTED_DB_VM_DB_HOST_CAP check (ADJUSTED_DB_VM_DB_USED_CAP >= 0);
end if;

/* Table SAHARA_APP.HOST_CAPACITY_ */

if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_OVERHEAD_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='APPS_ON_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD APPS_ON_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.APPS_ON_DB_VM_CAPACITY is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='APPS_ON_DB_VM_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD APPS_ON_DB_VM_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.APPS_ON_DB_VM_OTHER_CAPACITY is 
	'For shared storage, amount of storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_DB_USED_CAPACITY is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_DB_USED_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_DB_USED_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_DB_USED_OTHER_CAPACITY is 
	'For shared storage, amount of used storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_UNKNOWN_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_OVERHEAD_CAP') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_OVERHEAD_CAP numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_OVERHEAD_CAP is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_APPS_ON_DB_VM_CAP') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_APPS_ON_DB_VM_CAP numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_APPS_ON_DB_VM_CAP is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_DB_USED_CAP') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_DB_USED_CAP numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_DB_USED_CAP is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_CAPACITY_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_CAPACITY add constraint CKC_DB_VM_CAPACITY_HOST_CAP check (DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_USED_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_USED_CAPACITY add constraint CKC_DB_VM_USED_CAPACI_HOST_CAP check (DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_FREE_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_FREE_CAPACITY add constraint CKC_DB_VM_FREE_CAPACI_HOST_CAP check (DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_OVERHEAD_CA_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_OVERHEAD_CAPACITY add constraint CKC_DB_VM_OVERHEAD_CA_HOST_CAP check (DB_VM_OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_CAP_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter APPS_ON_DB_VM_CAPACITY add constraint CKC_APPS_ON_DB_VM_CAP_HOST_CAP check (APPS_ON_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_OTH_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter APPS_ON_DB_VM_OTHER_CAPACITY add constraint CKC_APPS_ON_DB_VM_OTH_HOST_CAP check (APPS_ON_DB_VM_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_CAP_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_DB_USED_CAPACITY add constraint CKC_DB_VM_DB_USED_CAP_HOST_CAP check (DB_VM_DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_OTH_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_DB_USED_OTHER_CAPACITY add constraint CKC_DB_VM_DB_USED_OTH_HOST_CAP check (DB_VM_DB_USED_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_UNKNOWN_CAP_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_UNKNOWN_CAPACITY add constraint CKC_DB_VM_UNKNOWN_CAP_HOST_CAP check (DB_VM_UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_CA_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_CAPACITY add constraint CKC_ADJUSTED_DB_VM_CA_HOST_CAP check (ADJUSTED_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_US_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_USED_CAPACITY add constraint CKC_ADJUSTED_DB_VM_US_HOST_CAP check (ADJUSTED_DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_FR_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_FREE_CAPACITY add constraint CKC_ADJUSTED_DB_VM_FR_HOST_CAP check (ADJUSTED_DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_OV_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_OVERHEAD_CAP add constraint CKC_ADJUSTED_DB_VM_OV_HOST_CAP check (ADJUSTED_DB_VM_OVERHEAD_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_APPS_ON__HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_APPS_ON_DB_VM_CAP add constraint CKC_ADJUSTED_APPS_ON__HOST_CAP check (ADJUSTED_APPS_ON_DB_VM_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_DB_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_DB_USED_CAP add constraint CKC_ADJUSTED_DB_VM_DB_HOST_CAP check (ADJUSTED_DB_VM_DB_USED_CAP >= 0);
end if;

/* table SAHARA.VOLUME */
if not exists (select 1 from sys.syscolumns where tname='VOLUME' and cname='IS_USED_BY_DB_VM') 
then
	ALTER TABLE SAHARA.VOLUME ADD IS_USED_BY_DB_VM char(1) null default 'F';
	comment on column SAHARA.VOLUME.IS_USED_BY_DB_VM is 
	'A Boolean flag that indicates whether a volume is utilized by a database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='VOLUME' and cname='DB_VM_NAME') 
then
	ALTER TABLE SAHARA.VOLUME ADD DB_VM_NAME varchar(256) null;
	comment on column SAHARA.VOLUME.DB_VM_NAME is 
	'This attribute represents name of the database volume manager that is using the volume.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_DB_VM_VOLUME' and table_name='VOLUME')
then
	alter table SAHARA.VOLUME
	alter IS_USED_BY_DB_VM add constraint CKC_IS_USED_BY_DB_VM_VOLUME check (IS_USED_BY_DB_VM is null or (IS_USED_BY_DB_VM in ('T','F')));
end if;

/* Index: I_VOLUME_ISCOMPONENT */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISCOMPONENT')
then
	create index I_VOLUME_ISCOMPONENT on SAHARA.VOLUME (
	IS_COMPONENT ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISCOMPONENT is 
	'An index on Is Component for dimensional queries.';
end if;

/* Index: I_VOLUME_ISBASEDONTHIN */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISBASEDONTHIN')
then
	create index I_VOLUME_ISBASEDONTHIN on SAHARA.VOLUME (
	IS_BASED_ON_THIN ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISBASEDONTHIN is 
	'An index on Is Based On Thin for dimensional queries.';
end if;
/* Index: I_VOLUME_ISEXPORTED */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISEXPORTED')
then
	create index I_VOLUME_ISEXPORTED on SAHARA.VOLUME (
	IS_EXPORTED ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISEXPORTED is 
	'An index on Is Exported for dimensional queries.';
end if;
/* Index: I_VOLUME_VOLUMETYPE */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_VOLUMETYPE')
then
	create index I_VOLUME_VOLUMETYPE on SAHARA.VOLUME (
	VOLUME_TYPE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_VOLUMETYPE is 
	'An index on Volume Type for dimensional queries.';
end if;
/* Index: I_VOLUME_APPLICATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_APPLICATIONBE')
then
	create index I_VOLUME_APPLICATIONBE on SAHARA.VOLUME (
	APPLICATION_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_APPLICATIONBE is 
	'An index on Application Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_PPLCTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_PPLCTNCMPSTB')
then
	create index I_VOLUME_PPLCTNCMPSTB on SAHARA.VOLUME (
	APPLICATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_PPLCTNCMPSTB is 
	'An index on Composite Application Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_ORGANIZATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ORGANIZATIONBE')
then
	create index I_VOLUME_ORGANIZATIONBE on SAHARA.VOLUME (
	ORGANIZATION_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ORGANIZATIONBE is 
	'An index on Organization Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_RGNZTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_RGNZTNCMPSTB')
then
	create index I_VOLUME_RGNZTNCMPSTB on SAHARA.VOLUME (
	ORGANIZATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_RGNZTNCMPSTB is 
	'An index on Composite Organization Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_ISUSEDBYDBVM */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISUSEDBYDBVM')
then
	create index I_VOLUME_ISUSEDBYDBVM on SAHARA.VOLUME (
	IS_USED_BY_DB_VM ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISUSEDBYDBVM is 
	'An index on Is Used By DB VM for dimensional queries.';
end if;

/* Index: I_DATABASES_ISBASEDONTHIN */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_DBVMNAME')
then
	create index I_VOLUME_DBVMNAME on SAHARA.VOLUME (
	DB_VM_NAME ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_DBVMNAME is 
	'An index on DB VM Name for dimensional queries.';   
end if;

/* TABLE SAHARA_APP.VOLUME_  */

if not exists (select 1 from sys.syscolumns where tname='VOLUME_' and cname='IS_USED_BY_DB_VM') 
then
	ALTER TABLE SAHARA_APP.VOLUME_ ADD IS_USED_BY_DB_VM char(1) null default 'F';
	comment on column SAHARA_APP.VOLUME_.IS_USED_BY_DB_VM is 
	'A Boolean flag that indicates whether a VOLUME_ is utilized by a database VOLUME_ manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='VOLUME_' and cname='DB_VM_NAME') 
then
	ALTER TABLE SAHARA_APP.VOLUME_ ADD DB_VM_NAME varchar(256) null;
	comment on column SAHARA_APP.VOLUME_.DB_VM_NAME is 
	'This attribute represents name of the database VOLUME_ manager that is using the VOLUME_.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_DB_VM_VOLUME_' and table_name='VOLUME_')
then
	alter table SAHARA_APP.VOLUME_
	alter IS_USED_BY_DB_VM add constraint CKC_IS_USED_BY_DB_VM_VOLUME_ check (IS_USED_BY_DB_VM is null or (IS_USED_BY_DB_VM in ('T','F')));
end if;

if exists (select 1 from sys.syscolumns where tname='DATABASES' and cname='IS_RAC_AWARE') 
then
	comment on column SAHARA.DATABASES.IS_RAC_AWARE is 
	'In oracle RAC scenarios, value will be True. 
	False otherwise for standalone database.';
end if;

if not exists(select 1 from sys.systable where table_name='DISKGROUP_BVIEW_NODE' and creator=user_id('SAHARA')) then
/*==============================================================*/
/* Table: DISKGROUP_BVIEW_NODE                                  */
/*==============================================================*/
create table SAHARA.DISKGROUP_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   DISKGROUP_KEY        numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.DISKGROUP_BVIEW_NODE is 
'This is a join table between diskgroup objects and business view nodes.';

comment on column SAHARA.DISKGROUP_BVIEW_NODE.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.DISKGROUP_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.DISKGROUP_BVIEW_NODE
   add constraint PK_CLUSTER_BVIEWNODE primary key (BVIEW_NODE_KEY, DISKGROUP_KEY);

/*==============================================================*/
/* Index: I_DISKGROUP_BVIEWNODE_BVIEWKEY                        */
/*==============================================================*/
create index I_DISKGROUP_BVIEWNODE_BVIEWKEY on SAHARA.DISKGROUP_BVIEW_NODE (
BVIEW_NODE_KEY ASC,
DISKGROUP_KEY ASC,
BVIEW_KEY ASC
);

comment on index SAHARA.DISKGROUP_BVIEW_NODE.I_DISKGROUP_BVIEWNODE_BVIEWKEY is 
'An index on Business View Key for dimensional queries.';

alter table SAHARA.DISKGROUP_BVIEW_NODE
   add constraint FK_DISKGROU_DG_BVND_T_DISKGROU foreign key (DISKGROUP_KEY)
      references SAHARA.DISKGROUP (DISKGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DISKGROUP_BVIEW_NODE
   add constraint FK_DG_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DISKGROUP_BVIEW_NODE
   add constraint FK_DGBVWND_BVIEW_NODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

grant SELECT on SAHARA.DISKGROUP_BVIEW_NODE to SAHARA_APP;

end if;

/* BV Calls */

call sahara.SP_DataDict_BViewDimension('DISKGROUP',       'Disk Group');

BEGIN
DECLARE @CONFIG long varchar;
SET @CONFIG = 'insert into SAHARA.COGNOS_FACT_MAPPING(OBJECT_TYPE, NAMESPACE, FACT_DIMENSION_NAME, FACT_NAME) on existing update values';

call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Percent Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Percent Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Overhead Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Used Capacity by Clustered DB''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Used Capacity by DB''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Used Unknown Capacity''');

SET @CONFIG = 'insert into SAHARA.COGNOS_PRESENTATION_SHORTCUT(OBJECT_TYPE, NAMESPACE, FOLDER_NAME) on existing update values';
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''Disk Group Analysis'', ''Disk Group Analysis Views''');

SET @CONFIG = 'insert into SAHARA.COGNOS_RELATIONSHIP(LHS_OBJECT_TYPE, RHS_TABLE, LHS_CARDINALITY_A, LHS_CARDINALITY_B, RHS_CARDINALITY_A, RHS_CARDINALITY_B) on existing update values';
call sahara.SP_DataDict_CONFIG(@CONFIG,'''DISKGROUP'', ''DISKGROUP'', ''zero'', ''one'', ''one'', ''one''');

END;

-- Trigger
CREATE OR REPLACE TRIGGER SAHARA.T_DG_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.DISKGROUP_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;
-- Call
call SAHARA.SP_BVIEW_AddRuleKey();

if not exists(select 1 from sys.systable where table_name='INSTALLED_ADDONS' and creator=user_id('SAHARA')) then
/*==============================================================*/
/* Table: INSTALLED_ADDONS                                      */
/*==============================================================*/
create table SAHARA.INSTALLED_ADDONS 
(
   ID                   varchar(64)                    not null,
   DISPLAY_NAME         varchar(256)                   null,
   VERSION              varchar(128)                   null,
   DESCRIPTION          varchar(10240)                 null,
   INSTALL_DATE         date                           null
);

comment on table SAHARA.INSTALLED_ADDONS is 
'This table records what addons/hf are currently installed on VOM-ES instance.
';

comment on column SAHARA.INSTALLED_ADDONS.ID is 
'Unique identifier of the hf or addon.';

comment on column SAHARA.INSTALLED_ADDONS.DISPLAY_NAME is 
'User friendly name to be displayed in GUI.';

comment on column SAHARA.INSTALLED_ADDONS.VERSION is 
'ES version on which this addon is installed.';

comment on column SAHARA.INSTALLED_ADDONS.DESCRIPTION is 
'Description of addon or hf.';

comment on column SAHARA.INSTALLED_ADDONS.INSTALL_DATE is 
'stores date when this addon was installed.';

alter table SAHARA.INSTALLED_ADDONS
   add constraint PK_INSTALLED_ADDONS primary key (ID);

grant SELECT on SAHARA.INSTALLED_ADDONS to SAHARA_APP;


end if;

commit;

/* Sahara PD Dump*/

/* HOST_CAPACITY*/
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_CAPACITY', 'DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_USED_CAPACITY', 'DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_FREE_CAPACITY', 'DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_OVERHEAD_CAPACITY', 'DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_DB_VM_CAPACITY', 'Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_DB_VM_OTHER_CAPACITY', 'Apps on DB VM Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_DB_USED_CAPACITY', 'DB VM DB Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_DB_USED_OTHER_CAPACITY', 'DB VM DB Used Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_UNKNOWN_CAPACITY', 'DB VM Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_CAPACITY', 'Adjusted DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_USED_CAPACITY', 'Adjusted DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_FREE_CAPACITY', 'Adjusted DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_OVERHEAD_CAP', 'Adjusted DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_APPS_ON_DB_VM_CAP', 'Adjusted Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_DB_USED_CAP', 'Adjusted DB VM DB Used Capacity');

/* HOST_CAPACITY_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_CAPACITY', 'DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_USED_CAPACITY', 'DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_FREE_CAPACITY', 'DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_OVERHEAD_CAPACITY', 'DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_DB_VM_CAPACITY', 'Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_DB_VM_OTHER_CAPACITY', 'Apps on DB VM Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_DB_USED_CAPACITY', 'DB VM DB Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_DB_USED_OTHER_CAPACITY', 'DB VM DB Used Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_UNKNOWN_CAPACITY', 'DB VM Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_CAPACITY', 'Adjusted DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_USED_CAPACITY', 'Adjusted DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_FREE_CAPACITY', 'Adjusted DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_OVERHEAD_CAP', 'Adjusted DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_APPS_ON_DB_VM_CAP', 'Adjusted Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_DB_USED_CAP', 'Adjusted DB VM: DB Used Capacity');

/* DATABASE_CAPACITY */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'CAPACITY_FROM_DB_VM', 'Capacity From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'AVAILABLE_FROM_DB_VM', 'Available From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'USED_FROM_DB_VM', 'Used From DB VM');

/* DATABASE_CAPACITY_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'CAPACITY_FROM_DB_VM', 'Capacity From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'AVAILABLE_FROM_DB_VM', 'Available From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'USED_FROM_DB_VM', 'Used From DB VM');

/* DATABASES */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');
/* DATABASES_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');

/* VOLUME */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_USED_BY_DB_VM', 'Is Used By DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'DB_VM_NAME', 'DB VM Name');

/* VOLUME_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_USED_BY_DB_VM', 'Is Used by DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'DB_VM_NAME', 'DB VM Name');

/* DISKGROUP */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'IS_USED_BY_CLUSTERED_DB', 'Is Used By Clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'USES_VM_VOLUMES', 'Uses VM Volumes');
/* DISKGROUP_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'IS_USED_BY_CLUSTERED_DB', 'Is Used by clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'USES_VM_VOLUMES', 'Uses VM Volumes');

/* DISKGROUP_CAPACITY */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'OVERHEAD_CAPACITY', 'Overhead capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'DB_USED_CAPACITY', 'DB Used capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'USED_CAPACITY_BY_CLUSTERED_DB', 'Used Capacity By Clustered DB');
/* DISKGROUP_CAPACITY_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'OVERHEAD_CAPACITY', 'Overhead capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'DB_USED_CAPACITY', 'DB Used capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'USED_CAPACITY_BY_CLUSTERED_DB', 'Used Capacity By Clustered DB');

/* DATABASES_CURRENT */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'IS_RAC_AWARE', 'Is RAC');

/* VOLUME_CURRENT */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_USED_BY_DB_VM', 'Is Used By DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'DB_VM_NAME', 'DB VM Name');

/* DISKGROUP_CURRENT */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'IS_USED_BY_CLUSTERED_DB', 'Is Used By Clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'USES_VM_VOLUMES', 'Uses VM Volumes');

/* DATBASES RAC changes*/
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'IS_RAC_AWARE', 'Is RAC');

/* DISKGROUP_BVIEW_NODE */
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DISKGROUP_BVIEW_NODE', 'Diskgroup Bview node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_BVIEW_NODE', 'DISKGROUP_KEY', 'diskgroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_BVIEW_NODE', 'STATUS', 'Status');

/* INSTALLED_ADDONS*/
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'INSTALLED_ADDONS', 'Installed Addons');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'INSTALLED_ADDONS', 'ID', 'ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'INSTALLED_ADDONS', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'INSTALLED_ADDONS', 'VERSION', 'Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'INSTALLED_ADDONS', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'INSTALLED_ADDONS', 'INSTALL_DATE', 'Install date');

commit;

/* insert addon entry*/
insert into SAHARA.INSTALLED_ADDONS 
on existing update
VALUES ('VRTSvomerhsclpack1-5.0.405.0','HSCL Pack 1 Add-on for Enterprise Server (5.0.405.0)','5.0.405.0','VOM Enterprise Server - Add-on to support  Oracle Automatic Storage Management (ASM) and fixes for forecasting reports.', getDate());

/* Sahara views */

/* View: DATABASE_CURRENT*/
if exists(select 1 from sys.systable where table_name='DATABASES_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.DATABASES_CURRENT
end if;

create view SAHARA.DATABASES_CURRENT  as
select DATABASES_KEY,
       HOST_KEY,
       DATABASES_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       INSTANCE_DISPLAY_NAME,
       DATABASE_TYPE,
       DATABASE_VERSION,
       DATABASE_STATE,
       DISCOVERY_STATE,
       IS_CLUSTERED,
       IS_RAC_AWARE,
       IS_BASED_ON_THIN,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_BASED_ON_DB_VM
  from SAHARA.DATABASES
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.DATABASES_CURRENT is 
'This view filters the DATABASES table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.DATABASES_CURRENT to SAHARA_APP;

/* View: DISKGROUP_CURRENT */
if exists(select 1 from sys.systable where table_name='DISKGROUP_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.DISKGROUP_CURRENT
end if;

create view SAHARA.DISKGROUP_CURRENT (DISKGROUP_KEY, DISKGROUP_OID, HOST_KEY, CC_STORAGE_URL, DISPLAY_NAME, FAMILY, VERSION, TYPE, IS_SHARED, IS_BASED_ON_DIRECT_ATTACHED, IS_BASED_ON_THIN, STATE, APPLICATION_BE, APPLICATION_COMPOSITE_BE, ORGANIZATION_BE, ORGANIZATION_COMPOSITE_BE, REDUNDANCY_TYPE, IS_USED_BY_CLUSTERED_DB, USES_VM_VOLUMES) as
select DISKGROUP_KEY,
       DISKGROUP_OID,       
       HOST_KEY,
       CC_STORAGE_URL,
       DISPLAY_NAME,       
       FAMILY,
       VERSION,
       TYPE,
       IS_SHARED,
       IS_BASED_ON_DIRECT_ATTACHED,
       IS_BASED_ON_THIN,
       STATE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       REDUNDANCY_TYPE,
       IS_USED_BY_CLUSTERED_DB,
       USES_VM_VOLUMES
  from SAHARA.DISKGROUP 
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.DISKGROUP_CURRENT is 
'This view filters the DISKGROUP table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.DISKGROUP_CURRENT to SAHARA_APP;

/* View: VOLUME_CURRENT */
if exists(select 1 from sys.systable where table_name='VOLUME_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.VOLUME_CURRENT
end if;

create view SAHARA.VOLUME_CURRENT  as
select VOLUME_KEY,
       VOLUME_OID,
       CC_STORAGE_URL,
       DISKGROUP_KEY,
       HOST_KEY,
       VOLUME_NAME,
       VOLUME_NAME AS DISPLAY_NAME,
       VM_VERSION,
       VM_TYPE,
       DISK_GROUP_NAME,
       VOLUME_LAYOUT,
       IS_VCS_CVM,
       IS_USED_BY_FILESYSTEM,
       IS_USED_BY_APPLICATION,
       IS_DIRECT_ATTACHED,
       IS_COMPONENT,
       IS_BASED_ON_THIN,
       IS_EXPORTED,
       VOLUME_TYPE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_USED_BY_DB_VM,
       DB_VM_NAME
  from SAHARA.VOLUME
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.VOLUME_CURRENT is 
'This view filters the VOLUME table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.VOLUME_CURRENT to SAHARA_APP;

commit;

begin
	message '-- configuring data set DISKGROUP_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DG_DIM',                       -- name
        'DIMENSION',                        -- stereotype
        'DISKGROUP',                           -- destination table
        'DISKGROUP_',                          -- staging table
		null,


        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
		'diskgroup_dim.csv',
		'vrts_vxvm_diskgroup'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'DISKGROUP_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'DISKGROUP_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || http_encode(SRC.DISKGROUP_OID) || @URL2');
end;

commit;

begin
                message '-- configuring data set VOLUME_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'VOLUME_DIM',                       -- name
        'DIMENSION',                        -- stereotype
        'VOLUME',                           -- destination table
        'VOLUME_',                          -- staging table
                                null,


--        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
--          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
--          'join SAHARA.HOST_CURRENT ' ||
--            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID'

        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY, DISKGROUP_CURRENT.DISKGROUP_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID ' ||
          'left outer join SAHARA.DISKGROUP_CURRENT ' ||
            'on SRC.DISKGROUP_OID = DISKGROUP_CURRENT.DISKGROUP_OID',
                                'volume_dim.csv',
                                'vrts_vxvm_volume'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'VOLUME_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'VOLUME_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.VOLUME_OID || @URL2');
end;

commit;

begin
                message '-- configuring data set DB_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DB_DIM',                           -- name
        'DIMENSION',                        -- stereotype
        'DATABASES',                        -- destination table
        'DATABASES_',                       -- staging table
                                null,

        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
                                'database_dim.csv',
                                'vrts_dbed_basic'
    );
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'DATABASES_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'DATABASES_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.DATABASES_OID || @URL2');
end;

commit;

call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DG_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'VOLUME_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DB_DIM');

commit;

/* Enable DG Forecasting */

call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DB_VM_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DB_VM_USED_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DB_VM_FREE_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DB_VM_OVERHEAD_CAP');

call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP','REDUNDANCY_TYPE');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP','USES_VM_VOLUMES');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP_CAPACITY','OVERHEAD_CAPACITY');

call sahara.SP_DataDict_EnableFCastSchema('DATABASES','IS_RAC_AWARE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASE_CAPACITY','CAPACITY_FROM_DB_VM');
call sahara.SP_DataDict_EnableFCastSchema('DATABASE_CAPACITY','AVAILABLE_FROM_DB_VM');
call sahara.SP_DataDict_EnableFCastSchema('DATABASE_CAPACITY','USED_FROM_DB_VM');
commit;

/*BV Import Export*/

-- staging table for Diskgroup objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_DISKGROUP_');
create table SAHARA_APP.IE_DISKGROUP_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    DISKGROUP_KEY NUMERIC(10),
    HOST_NAME VARCHAR(256),
    DISKGROUP_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_DISKGROUP_ is
'Import/Export staging table for diskgroup objects.';

comment on column SAHARA_APP.IE_DISKGROUP_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_DISKGROUP_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_DISKGROUP_.DISKGROUP_KEY is
'The sahara.diskgroup object unique key for this object.';

comment on column SAHARA_APP.IE_DISKGROUP_.HOST_NAME is
'The name of the owner of this diskgroup';

comment on column SAHARA_APP.IE_DISKGROUP_.DISKGROUP_NAME is
'The fully qualified name for this diskgroup.';

comment on column SAHARA_APP.IE_DISKGROUP_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_DISKGROUP_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_DISKGROUP_
    add constraint C_IE_DG_REJECTED check(REJECTED in ('T', 'F'));

CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjectByObjectName');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjectByObjectName(IN @pType  VARCHAR(255))
BEGIN
  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  BEGIN 

    SET @vSQL_TEMPLATE ='begin FOR StageCursorLOOP AS StageCursor CURSOR FOR select HOST_NAME AS HOST_NAME_R, $TYPE$_NAME AS $TYPE$_NAME_R '||
                        ' from SAHARA_APP.IE_$TYPE$_ where HOST_NAME is not null ' ||
                   'and $TYPE$_NAME is not null and $TYPE$_KEY is null and BVIEW_NODE_KEY is not null for update of $TYPE$_KEY ' ||
                   'DO ' ||
                'update SAHARA_APP.IE_$TYPE$_ ' ||
                    'set $TYPE$_KEY = ' ||
                        '(select FIRST $TYPE$_KEY ' ||
                           'from SAHARA.$TYPE$_CURRENT OT ' ||
                           'join SAHARA.HOST_CURRENT ON HOST_CURRENT.HOST_KEY = OT.HOST_KEY ' ||
                          'where OT.$PATH_COLUMN$ = $PATH_VALUE$ ' ||
                            'and HOST_CURRENT.DISPLAY_NAME = HOST_NAME_R ' ||
                            ') '|| 
	          'WHERE HOST_NAME = HOST_NAME_R '||
		  ' AND $TYPE$_NAME = $TYPE$_NAME_R ; ' ||
            'end for; ' ||
        'end;';
    SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);
   
    IF @pType = 'FILESYSTEM' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'DISPLAY_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'FILESYSTEM_NAME_R');
    END IF;
    
    IF @pType = 'VOLUME' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'VOLUME_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'VOLUME_NAME_R');
    END IF;

    IF @pType = 'DISKGROUP' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'DISPLAY_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'DISKGROUP_NAME_R');
    END IF;

 EXECUTE IMMEDIATE @vSQL;
  END;
END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjects');
CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjects
(
IN @viewname LONG VARCHAR,
IN @session_key numeric(10)
)
BEGIN
    DECLARE @vRootKey NUMERIC(10);
    DECLARE @vStagingTable VARCHAR(61);
    DECLARE @vType VARCHAR(30);
    DECLARE @vSQL LONG VARCHAR;
   DECLARE @FUNC varchar(64);
    
    BEGIN
    -- get the node key for the root of the view
     SET @vRootKey = 0 ; 

    set @FUNC = 'SP_IE_CorrelateObjects';
    
    call SAHARA.SP_Log_LogDebug(@session_key, @FUNC, 'entry');


     select BVIEW_KEY
      into @vRootKey
      from SAHARA.BVIEW
     where NAME = @viewname;

    if @vRootKey =0  then
      
      CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewNotFound,
            'SP_IE_CorrelateObjects',
            'the business view not found ' || @viewname
        );

    end if;

    -- Iterate over the supported types.  For each, join the node and object
    -- staging tables and set the BVIEW_NODE_KEY column in the object staging
    -- table.  Use this join to call AssociateDimensionRows.  Select all the
    -- objects that don't have BVIEW_NODE_KEY values and mark them
    -- as rejected.




    FOR IETypesLOOP AS IETypesCUR CURSOR FOR 
    select TYPE_NAME from SAHARA_APP.IE_TYPES_
    
    DO
      SET @vType = UPPER(TYPE_NAME);
      SET @vStagingTable = 'SAHARA_APP.IE_' || @vType || '_';

        -- Set the business view node keys for the objects based on
        -- the IE node IDs previously populated by CorrelateNodesAtLevel.

        SET @vSQL =
            'update ' || @vStagingTable || ' STAGE ' ||
               'set BVIEW_NODE_KEY = ' ||
                    '(select BVIEW_NODE_KEY ' ||
                       'from SAHARA_APP.IE_NODE_ ' ||
                      'where STAGE.NODE_ID = IE_NODE_.NODE_ID)';
        execute immediate @vSQL;

        -- Now do the object correlation by filling in the <type>_KEY column
        -- of the object staging table.  If multiple object records match
        -- a criteria, associate the node with first object one we find.

        -- First we'll try based on the object's OID.
        SET @vSQL =
            'update ' || @vStagingTable || ' STAGE ' ||
               'set ' || @vType || '_KEY = ' ||
                    '(select ' || @vType || '_KEY ' ||
                       'from SAHARA.' || @vType || '_CURRENT OBJECT_TYPE ' ||
                      'where OBJECT_TYPE.' || @vType || '_OID = STAGE.OBJECT_UNIQUE_KEY) ' ||
             'where ' || @vType || '_KEY is null ' ||
               'and BVIEW_NODE_KEY is not null';
        execute immediate @vSQL;

        -- try matching the IP address for object types that have them
  
	if @vType in ('HOST', 'ARRAY', 'SWITCH') then
            CALL SAHARA_APP.SP_IE_CorrelateObjectsByIPAddress(@vType);
        end if;

        -- try matching the filer name and fully qualified path
        -- for netapp objects

        if @vType in ('NETAPP_VOLUME', 'NETAPP_SHARE', 'NETAPP_QTREE') then
            CALL SP_IE_CorrelateObjectByNetAppPath(@vType);
        end if;

		-- try matching the host name and object display name
        -- for filesystem, volume and diskgroup objects
        if @vType in ('FILESYSTEM', 'VOLUME', 'DISKGROUP') then
            CALL SP_IE_CorrelateObjectByObjectName(@vType);
        end if;

        -- match against the object name

        if @vType in ('APP_GROUP', 'HOST', 'ARRAY', 'SWITCH') then
            SET @vSQL =
                'update ' || @vStagingTable || ' STAGE ' ||
                    'set ' || @vType || '_KEY = ' ||
                        '(select FIRST ' || @vType || '_KEY ' ||
                            'from SAHARA.' || @vType || '_CURRENT OT ' ||
                           'where OT.DISPLAY_NAME = STAGE."NAME" ' ||
                             ') ' ||
                    'where ' || @vType || '_KEY is null ' ||
                      'and BVIEW_NODE_KEY is not null';
            execute immediate @vSQL;
        end if;

        -- Object corellation is complete and recorded in the staging tables.
        -- Create the node to object associations.

        CALL SP_IE_CreateNodeObjectAssociations(@vType);

        -- Anything that correlated is NOT rejected.  Everything else is
        -- already marked rejected.

        SET @vSQL = 'update ' || @vStagingTable || ' ' ||
                   'set REJECTED = ''F'' ' ||
                 'where ' || @vType || '_KEY is not null ' ||
                   'and BVIEW_NODE_KEY is not null';
        execute immediate @vSQL;

	-- Anything that is NOT correlated is rejected.  Everything else is
        -- already marked NOT rejected.

        SET @vSQL = 'update ' || @vStagingTable || ' ' ||
                   'set REJECTED = ''T'' ' ||
                 'where ' || @vType || '_KEY is null ' ||
                   'and BVIEW_NODE_KEY is not null';
   
	execute immediate @vSQL;

        CALL SAHARA.SP_BVIEW_AddDimensionSupport(@vRootKey, @vType);

        
    end for;

    call SAHARA.SP_BVIEW_PublishBV();
  END ;

END;

commit;

/*Custom Attr*/

BEGIN
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vStagingTableName VARCHAR(1024);
  DECLARE @vStagingColExists INTEGER;  
  
  

FOR AttributesLOOP as AttributesCUR dynamic scroll cursor for
      select CUSTOM_ATTRIBUTE_KEY as CUSTOM_ATTRIBUTE_KEY_R,
        CUSTOM_ATTRIBUTE_CONTEXT as CUSTOM_ATTRIBUTE_CONTEXT_R,
        TYPE as TYPE_R,
        DISPLAY_NAME as DISPLAY_NAME_R,
        COLUMN_NAME as COLUMN_NAME_R,
        COLUMN_TYPE as COLUMN_TYPE_R,
        SOURCE_NAME as SOURCE_NAME_R    
      from SAHARA.CUSTOM_ATTRIBUTE CONFIGURED
      where CONFIGURED.SOURCE_NAME not in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY')
	    and exists (select 1
                         from SAHARA.P_CUSTOM_ATTRIBUTE PUBLISHED
                        where PUBLISHED.CUSTOM_ATTRIBUTE_KEY =
                              CONFIGURED.CUSTOM_ATTRIBUTE_KEY)
     DO            

            

                if SAHARA.fn_HELPER_ColumnExists('SAHARA_APP',CUSTOM_ATTRIBUTE_CONTEXT_R || '_',
                    COLUMN_NAME_R) = 0 THEN
                    SET @vSQL = 'lock table ' ||
                    -- source table or view
                    'SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R || '_ ' ||
                            'in exclusive mode';

                    EXECUTE IMMEDIATE @vSQL;

                    SET @vSQL = 'alter table SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R
                        || '_ add ("' || COLUMN_NAME_R || '" ' || COLUMN_TYPE_R || ')';
                    EXECUTE IMMEDIATE @vSQL;                    
                end if;    	        
            -- release the table locks
      COMMIT ;
     END FOR;     
END;

call sahara.SP_recompile_views();

commit;

-- Store Procedures

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DAILY_SUMMARIZATION' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DAILY_SUMMARIZATION
end if;

CREATE PROCEDURE SAHARA.SP_DAILY_SUMMARIZATION(@pADATE date)

BEGIN

   DECLARE @FUNC                       varchar(256);

   DECLARE @pRollupBatch               numeric(10);
   DECLARE @pDates_key                 numeric(10);
   DECLARE @pYear_Period_key           numeric(10);
   DECLARE @pRollup_source_key         numeric(10);

   DECLARE @pADATE_Yesterday           date;
   DECLARE @pADATE_PrevSuccRollup      date;

   DECLARE @error                      integer;
   DECLARE @sqlstate                   varchar(5);
   DECLARE @serror                     varchar(512);
   DECLARE @section                    varchar(512);

   DECLARE @pQuarterNumber             int;
   DECLARE @pMonthNumber               int;
   DECLARE @pYearNumber                int;
   DECLARE @pWeekNumber                int;
   DECLARE @last_summarization_time    date;

   set @FUNC = 'SP_DAILY_SUMMARIZATION';

   call SAHARA.SP_saharaAPIInit();
   call SP_Log_LogDebug(null, @FUNC, 'enter');

   set @last_summarization_time =
       sahara.fn_conf_GetConfigDate('TREND_DATA_RETENTION', 'LAST_SUMMARIZATION_TIME');

   set @pADATE_Yesterday = @pADATE;

   if @pADATE_Yesterday = @last_summarization_time
   then
       return; -- summarization has been executed
   end if;

   for rollup_source_entry as rollup_source_loop cursor for
          select rs.rollup_source_key rsk
          from sahara.rollup_source rs
            inner join sahara.rollup_batch rb on rs.rollup_source_key = rb.rollup_source_key
            inner join sahara.dates dates     on rb.dates_key = dates.dates_key
          where dates.adate = @pADATE_Yesterday and
                rb.rollup_batch_key in (select max(rollup_batch_key) from sahara.rollup_batch where is_success = 'T' group by dates_key)
          order by rsk asc
   do

       select max(dates.adate) into @pADATE_PrevSuccRollup
       from sahara.rollup_batch rb
            inner join sahara.dates dates on rb.dates_key = dates.dates_key
       where dates.adate < @pADATE_Yesterday and
             rb.rollup_source_key = rsk      and
             rb.rollup_batch_key in
               (select max(rollup_batch_key) from sahara.rollup_batch where is_success = 'T' and rollup_source_key = rsk group by dates_key);

       if @pADATE_PrevSuccRollup is null
       then
           set @pADATE_PrevSuccRollup = @pADATE_Yesterday;
       end if;

       -- This is to adjust the case that rollup is fine, but unable to process DAILY_SUMMARIZATION.
       if @pADATE_PrevSuccRollup > @last_summarization_time
       then
           set @pADATE_PrevSuccRollup = @last_summarization_time;
       end if;

       set @pYearNumber = 0;

       for adate_entry as adate_loop cursor for
           select adate as adate
           from sahara.dates da
           where adate >   @pADATE_PrevSuccRollup and adate <= @pADATE_Yesterday
           order by adate asc
       do

       begin

           savepoint SP_DAILY_SUMMARIZATION;

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','week');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'week');

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','quarter');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'month');

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','month');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'quarter');

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','year');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'year');

           commit;
       EXCEPTION
             WHEN OTHERS THEN
            SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
            IF ( @error <> 0 )
            THEN
                BEGIN
                    call SP_Log_LogError(null, @FUNC,
                                               string ('          @error    = ', @error ) +
                                               string ('         @serror    = ', @serror ) +
                                               string ('       @sqlstate    = ', @sqlstate ) +
                                               string (' Error failed at    = ', @FUNC+':'+@section ) );
                    rollback to savepoint SP_DAILY_SUMMARIZATION;
                END;
            END IF;

       end;

       end for;

   end for;

   call SAHARA.fn_Configuration_InstallDate (
        'TREND_DATA_RETENTION',                     -- section
        'LAST_SUMMARIZATION_TIME',                  -- name
        date(dateadd( day, -1, now(*))),            -- value
        'Last Time the Data Summarization is processed.',       -- display name
        -- description
        '',
        0,
        1
    );

   call SP_Log_LogDebug(null, @FUNC, 'exit');


END;

IF EXISTS (select * from sys.sysprocedure sp key join sysusers u
 where sp.proc_name = 'SP_fix_duplicate_summary_data' and u.name = 'SAHARA' ) THEN
  DROP PROCEDURE SAHARA.SP_fix_duplicate_summary_data;
END IF;
create procedure SAHARA.SP_fix_duplicate_summary_data()
BEGIN
  declare @ObjKeyColNames varchar(1024);
  declare @deleteStmt varchar(1024);

  if exists (select 1 from sahara.rollup_batch)
  then
     message 'Resetting last_summarization_time' to log;
     update sahara.configuration
     set DATE_VALUE = (select datetime(min(adate)) from sahara.dates)
     where section='TREND_DATA_RETENTION' and NAME='LAST_SUMMARIZATION_TIME';

     message 'deleting summary batches as necessary' to log;

     // Foar each table delete any duplicate rows in it for summary samples
     for facttables as ft_loop cursor for
         SELECT DEST_TABLE as fact_table FROM "SAHARA"."ROLLUP_DATA_SET_TYPE"
         where stereotype='FACT'
     do
        message 'Fact table=' || fact_table to log;

        // delete all summary rows from the fact table
        set @deleteStmt = 'delete from ' || fact_table ||
            ' F from rollup_batch B  where B.rollup_batch_key = F.rollup_batch_key and B.period_type in (''week'', ''month'', ''quarter'', ''year'')';
        message @deleteStmt to log;
        execute immediate @deleteStmt;
     end for;

     message 'Repopulating summary data in fact tables...' to log;

     update sahara.configuration
     set BOOLEAN_VALUE = 'F'
     where section='TREND_DATA_RETENTION' and NAME='MIGRATE_SUMMARIZED_DATA';

     call SAHARA.SP_Missing_And_Summarization_after_Db_Upgrade();

     message 'Done!' to log;
     end if;

  commit;

END;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_DelColumnMetadata' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_DelColumnMetadata
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_DelColumnMetadata" (@pOwner varchar(32), @pTableName varchar(128), @pColumnName varchar(128), @pDisplayName varchar(128))
BEGIN
	delete from sahara.COLUMN_METADATA_BASE
	where OWNER=@pOwner and TABLE_NAME=@pTableName and COLUMN_NAME=@pColumnName
END;

grant execute on sahara.SP_DataDict_DelColumnMetadata to sahara;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_DelTableMetadata' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_DelTableMetadata
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_DelTableMetadata" (@pOwner varchar(32), @pTableName varchar(128), @pDisplayName varchar(128))
BEGIN
		delete from sahara.TABLE_METADATA_BASE
		where OWNER=@pOwner and TABLE_NAME=@pTableName
END;

grant execute on sahara.SP_DataDict_DelTableMetadata to sahara;
