/**
* The DataSourcePanel is the GUI used to add new data-sources for rollup. 
* @module    Rollup
* @title     DataSourcePanel
* @requires  yahoo.js, container.js, dom.js, event.js
*/

/** This class renders and displays the DataSourcePanel UI
*   @return{Object} A DataSourcePanel object
*/
var jj;
var myObj;
function DataSourcePanel(){
	this.PanelWindow    = {};
	this.Title          = "DataSourcePanel";
	this.ErrorPane      = document.getElementById('divDSVersionCheck');
	this.SuccessImage   = '/sm/plugins/ER/img/Rollup/' + "tick.gif";
	this.FailureImage   = '/sm/plugins/ER/img/Rollup/' + "cross.gif";
	this.ProgressImage  = '/sm/plugins/ER/img/Rollup/' + "progressbar_green.gif";
	this.SpacerImage  = '/sm/images/' + "spacer.gif";
	this.CloseOnSuccess = false;
	this.validateFlag	= true;
	this.UIHostTxtField = 'txtHost';
	this.UIUserTxtField = 'txtUser';
	this.UIHostPasswordField = 'txtPassword';
	this.UISaveBtn      = 'btnSaveDS';
	this.UICancelBtn    = 'btnCancelAddDS';
	this.UIValidateBtn  = 'imgDSValidate';
	this.UIValidateWSBtn  = 'imgWSValidate';
	this.vomDBPort = '5636';
	this.baseURL = "";
	
	/** Initialises the panel and adds listeners
	*/
	this.initPanel = function(){
		this.PanelWindow = this._makePanel();
		YAHOO.util.Event.addListener('imgAddNew', 'click', this.openPanel, this, true);
		YAHOO.util.Event.addListener('btnCancelAddDS', 'click', this.handleCancel, this, true);
		YAHOO.util.Event.addListener(this.UISaveBtn , 'click', this.startVersionCheck, this, true);
		YAHOO.util.Event.addListener('imgDSValidate', 'click', this.startVersionCheck,this, true);
		YAHOO.util.Event.addListener('imgDSServerValidate', 'click', this.openPanelValidate,this, true);
		//var el = document.getElementById('imgDSServerValidate');
		//alert(el);
		//el.onclick = this.openPanelValidate;

		this.CloseOnSuccess = false;
		YAHOO.util.Event.addListener(this.UIValidateWSBtn, 'click', webServerButtonClick);
		YAHOO.util.Event.addListener(this.UIHostTxtField, 'keypress', this.handleKeyPress, this, true);
		YAHOO.util.Event.addListener(this.UIUserTxtField, 'keypress', this.handleKeyPress, this, true);
		YAHOO.util.Event.addListener(this.UIHostPasswordField, 'keypress', this.handleKeyPress, this, true);
		
		document.getElementById(this.UIHostTxtField).value="";
		document.getElementById(this.UIUserTxtField).value="";
		document.getElementById(this.UIHostPasswordField).value="";
	}	
	
	this.reInitPanel = function(){
		//YAHOO.util.Event.addListener('imgAddNew', 'click', this.openPanel, this, true);
		//YAHOO.util.Event.addListener('btnCancelAddDS', 'click', this.handleCancel, this, true);
		//YAHOO.util.Event.addListener(this.UISaveBtn , 'click', this.startVersionCheck, this, true);
		//YAHOO.util.Event.addListener('imgDSValidate', 'click', this.startVersionCheck,this, true);
		//YAHOO.util.Event.addListener('imgDSServerValidate', 'click', this.openPanelValidate,this, true);
		//var el = document.getElementById('imgDSServerValidate');
		//alert(el);
		//el.onclick = this.openPanelValidate;
		
		YAHOO.util.Event.removeListener(this.UIValidateWSBtn, 'click');
		YAHOO.util.Event.addListener(this.UIValidateWSBtn, 'click', webServerButtonClick);
		//YAHOO.util.Event.addListener(this.UIHostTxtField, 'keypress', this.handleKeyPress, this, true);
		//YAHOO.util.Event.addListener(this.UIUserTxtField, 'keypress', this.handleKeyPress, this, true);
		//YAHOO.util.Event.addListener(this.UIHostPasswordField, 'keypress', this.handleKeyPress, this, true);
		
		this.CloseOnSuccess = false;
		document.getElementById(this.UIHostTxtField).value="";
		document.getElementById(this.UIUserTxtField).value="";
		document.getElementById(this.UIHostPasswordField).value="";
	}
	/** Renders the panel UI
	*   @return{Object} A YAHOO panel object.
	*/
	this._makePanel = function(){
		var dsPanel = new YAHOO.widget.Panel("addDSPanel",
											{width  : "650px", 
											 
	             							 fixedcenter : true, 
	             							 close	   : false,
	              							 visible   : true, 
											 underlay: "none", 
	              							 draggable : true,
	              							 modal 	   :true,											
	              							 constraintoviewport : true
	            							 } );
			dsPanel.setHeader('<div align="left"><label>Add New Data Source</label></div>');
			dsPanel.setBody(document.getElementById('divAddDSBody'));
			dsPanel.setFooter(document.getElementById('divAddDSFooter'));
			dsPanel.render(document.body);
			dsPanel.hide();
		return dsPanel;
	}
	
	/** Displays the panel UI. Also calls the 'add' method of the underlyig datasource table
	 * @see DRUHTMLWebDataHandler.java
	*/
	this.openPanel = function(){
		document.getElementById(this.UIHostTxtField).value="";
		document.getElementById(this.UIUserTxtField).value="";
		document.getElementById(this.UIHostPasswordField).value="";
		this.CloseOnSuccess = false;
		
		this.checkLicenseValidity();
		
	}

	this.openPanelValidate = function(){
		this.validateFlag=true;
		document.getElementById(this.UIHostTxtField).value = ERForm.dbHost.value;
		document.getElementById(this.UIHostTxtField).disabled = true;
		this.PanelWindow.setHeader("Verify Data Source");
		this.resetStyles();
		//performTask('Add');
		
		this.PanelWindow.show();	
		this.startVersionCheck();

	}

	/** Pre-checks input and disabled Save button accordingly
	*/
	this.handleKeyPress = function(e){
		if(this.isHostNameValid()){
			this.disableSave(false);
		}
		if(this.isUserNameValid()){
			this.disableSave(false);
		}
	}
	
	
	/** Callback object for checking for version 5. Updates result status based on server response. 
	 * A successful result from server is assumed to be of the form 'Major:x|Minor:y'
	 * Connection times out after 9 seconds.
	*/
	this.versionCheckResult = { 
	    success: function(o){
	    	var result = o.responseText;
	    	if(result.indexOf('Error') == -1){
				var Version = (result.split(':'))[1];

				
				//alert(majorVersion + ", " + halschemaVersion);
					//var res = Version.split('.');
				//	var mainVersion = parseInt(halschemaVersion.split('.')[0]); 
				//	var gaVersion = parseInt(halschemaVersion.split('.')[1]);
					//var halLSD = parseInt(halschemaVersion.split('.')[2]);
				//	alert(Version);
					if(Version.indexOf('3.1')!=-1)
					  this.setStatus(true,'VOM4.0');
					else
					{
						this.setStatus(true,'VOM4.0');
						}		
				
	    	}
	    	else{
	    		//Found error message
	    		this.setStatus(false);
				alert("Unable to connect to datasource server. Please verify that database on datasource server is running.");
			//	this.checkForVersion4();
	    	}
	    }, 
	    failure: function(response){
	    	this.setStatus(false);
			
			this._onConnectionFailure(response.status);
	    },
	    timeout: 300000, 
	    scope: this
	};
	
	/** Updates result status bar with appropriate error text. 
	*/
	this._onConnectionFailure = function(connStatus){
		switch(connStatus){
			case 0 :
				this.setResultStatus(false, 'Error establishing HTTP connection.');
				break;
			case -1:
				this.setResultStatus(false, 'No Veritas Operations Manager database could be reached.  Please verify the host name and that the Veritas Operations Manager database on the host is started.');
				break;
			default :
				this.setResultStatus(false, 'Could not connect to server.');
				break;
		}
	}
	
	
	this.checkForVersion = function(){
		YAHOO.util.Dom.removeClass('spnDSVersion5Header','invisible');
		var url = '/sm/plugins/ER/versionChecker.jsp';
		var hostName = document.getElementById(this.UIHostTxtField).value;
		var inputDBPassword = encodeURIComponent(document.getElementById(this.UIHostPasswordField).value);
		var params = 'hostName='+hostName+
					 '&version=5'+'&dbPassword='+inputDBPassword+'&dbPort='+this.vomDBPort;
		var conn = YAHOO.util.Connect.asyncRequest('POST', url, this.versionCheckResult, params); 
	}
		
	

	
	this.setStatus = function(successFlag, versionNumer){
		//var resultImg = document.getElementById('imgDSV5Status');
		if(!successFlag){
			resultImg.src = this.FailureImage;
			return;
		}
		else{
			//resultImg.src = this.SuccessImage;
			YAHOO.util.Dom.removeClass('divDSV5Subversions','invisible');
			switch(versionNumer){

				case 'VOM3.1' :
					//document.getElementById('imgDSV52GAStatus').src =this.SuccessImage;
					this.setResultStatus(true, 'Veritas Operations Manager 3.1','SFMdb3',this.vomDBPort);
					break;
				case 'VOM4.0' :
					//document.getElementById('imgDSV51GAStatus').src =this.SuccessImage;
					this.setResultStatus(true, 'Veritas Operations Manager 4.0','SFMdb3',this.vomDBPort);
					break;
				case 'Other':
					//document.getElementById('imgDSV5MP1Status').src =this.SuccessImage;
					this.setResultStatus(true, 'A database was contacted on the specified host, but does not report a compatible version of Veritas Operations Manager.','SFMdb3',this.vomDBPort);
					break;
				
			}
		}
	}

	this.disableSave = function( disableFlag ){		
		var okButton = document.getElementById(this.UISaveBtn);
			okButton.disabled = (disableFlag) ? true : false;	
	}
	
	this.disableCancel = function( disableFlag ){
		var cancelButton = document.getElementById(this.UICancelBtn);
		cancelButton.disabled = (disableFlag) ? true : false;	
	}
	
	this.setResultStatus = function(successFlag, version, dbName, dbHostPort){
		// console.log('Checking version');
		YAHOO.util.Dom.removeClass('divDSVersionCheck', 'hidden');	
		YAHOO.util.Dom.removeClass('divVersionCheckResults','invisible');
		var resultLabel = document.getElementById('lblDSResult');
		if(successFlag){
			//YAHOO.util.Dom.addClass('divVersionCheckResults','success');
			var lblDSV5Header = document.getElementById('lblDSV5Header');
			lblDSV5Header.innerHTML = "Adding source " + document.getElementById(this.UIHostTxtField).value;
								
			YAHOO.util.Dom.removeClass('spnDSVersion5Header','invisible');			
			//YAHOO.util.Dom.removeClass('divStatus','invisible');
			//YAHOO.util.Dom.removeClass('divStatus','result');
			//YAHOO.util.Dom.addClass('divVersionCheckResults','success');
			
			var progImg = document.getElementById('imgDSV5Status');
			progImg.src = this.ProgressImage;
			
			this.disableSave(true);
						
			var url = "plugins/springs/vomer/source/addSource?ms=" + new Date().getTime();
		    url = encodeURI(url);
		    
			var sourceName = YAHOO.lang.trim(document.getElementById(this.UIHostTxtField).value);			
			var kvp = "source="+sourceName;
			var postdata = encodeURI(kvp);
			
			var adminName = YAHOO.lang.trim(document.getElementById(this.UIUserTxtField).value);
			var kvp = "&admin="+adminName;
			postdata+= encodeURI(kvp);
			
			var adminPassword = encodeURIComponent(document.getElementById(this.UIHostPasswordField).value);
			var kvp = "&passwd="+adminPassword;
			postdata+= encodeURI(kvp);
			
			//myObj=this;
			
			var request = YAHOO.util.Connect.asyncRequest('POST', url,
				        { success:this.sourceSuccessHandler, failure:this.sourceFailureHandler, argument:{obj:this} }, postdata);
			
			/*var url="/sm/plugins/ER/getPortForWebServer.jsp";
			url=url+"?ccserver="+webServerName;
			var webServerPort="8443";
			xmlHttpPort=this.GetXmlHttpObjectForPort();	
			xmlHttpPort.open("POST",url,true);
			xmlHttpPort.send(null);  
			xmlHttpPort.onreadystatechange=function(){
			if(xmlHttpPort.readyState==1){ }
			if (xmlHttpPort.readyState==4) {
					webServerPort=RTrim(LTrim( xmlHttpPort.responseText));
					
					if(webServerPort==0){
						webServerPort='8443'
						alert("Webserver for "+webServerName+" is down, please configure the server after starting the webserver.");
					}
					resultLabel.innerHTML =  "Found " +version;
					var dsf = new DataSourceForm(webServerName,
													dbHostDBPassword,
													version, 
													dbName, 
													dbHostPort, 
													webServerPort
													);
					if(myObj.CloseOnSuccess){
						myObj.PanelWindow.hide();
					}
					
			}	
		} */
		
		}
		else {
			//YAHOO.util.Dom.removeClass('divDSVersionCheck', 'hidden');					
			//YAHOO.util.Dom.removeClass('divVersionCheckResults','success');
			var failText = (version) ? version : 'Veritas Operations Manager server could be reached';
			//document.getElementById(this.UIHostTxtField).value="";
			document.getElementById(this.UIHostTxtField).focus();
			resultLabel.innerHTML =failText  ;
			this.disableSave(true);
			this.CloseOnSuccess = false;
		}
	}
	
	this.sourceSuccessHandler = function(o) {
		
		var obj = o.argument.obj;
		var resultLabel = document.getElementById('lblDSResult');
		var progImg = document.getElementById('imgDSV5Status');
		
	    var resultTxt = o.responseText;
	    var succIndex = resultTxt.indexOf("Success:");
	    var failIndex = resultTxt.indexOf("Failure:"); 
	    var urlIndex = resultTxt.indexOf(":URL:"); 

	    var msg = resultTxt.substring(8); 
	    if(succIndex != -1) {
	    	//YAHOO.util.Dom.addClass('divVersionCheckResults','success');
	    	progImg.src = obj.SuccessImage;
	    	obj.CloseOnSuccess = true;
	    	msg = resultTxt.substring(8, urlIndex); 
	    	obj.baseURL = resultTxt.substring(urlIndex+5);
	    	obj.disableCancel(true);
	    	obj.disableSave(false);
	    }else {
	    	//YAHOO.util.Dom.addClass('divVersionCheckResults','result');
	    	progImg.src = obj.FailureImage;    	
	    	obj.CloseOnSuccess = false;
	    	obj.disableSave(true);
	    }
	  	resultLabel.innerHTML = msg;
	}
	
	this.sourceFailureHandler = function(o) {
		//YAHOO.util.Dom.addClass('divVersionCheckResults','result');
	    var resultTxt = o.responseText;
	    var succIndex = resultTxt.indexOf("Success:");
	    var failIndex = resultTxt.indexOf("Failure:");       
	  	var msg = resultTxt.substring(8);
	  	var obj = o.argument.obj;
	  	obj.setResultStatus(false, msg);	
	  	obj.CloseOnSuccess = false;
	  	obj.disableSave(false);
	  	obj.disableCancel(false);
	}
	
	this.GetXmlHttpObjectForPort= function() {
		 var xmlHttpPort=null; 
		 try  { 
			xmlHttpPort=new XMLHttpRequest(); 
		 }catch (e)  {
		try  {
		 xmlHttpPort=new ActiveXObject("Msxml2.XMLHTTP");    }  catch (e)    {
		 xmlHttpPort=new ActiveXObject("Microsoft.XMLHTTP");    }  }return xmlHttpPort;
	 }		

	this.handleCancel = function(override){
		/*var confirmChoice = confirm('Do you want to cancel your changes?');
		if(!confirmChoice){
			return;
		}
		else{*/
			this.resetStyles();
			this.PanelWindow.hide();
			if(document.getElementById('isNewDS')!= null){
				document.getElementById('isNewDS').value='false';
			}
			//cancelChanges();
		//}
	}
	

//Check if valid license is installed or not.
	this.checkLicenseValidity = function(){
		//YAHOO.util.Dom.removeClass('spnDSVersion4Header','invisible');
		var url = '/sm/plugins/ER/checkLicenseValidity.jsp';
		var params = '';
		var conn = YAHOO.util.Connect.asyncRequest('POST', url, this.licneseCheckResult, params); 
	}

	this.setValidationStatus = function(validFlag){
		if(validFlag === true){
			this.PanelWindow.hide();
		}
		else{
			this._animateShowErrorPane();
		}
	}
	
	this.blank = function(str) {
		var fld = str.replace(/^\s*/,'').replace(/\s*$/,'');
		if (fld.length <= 0) { 
			return true;
		}else { 
			return false;
		}
	}
	
	this.isHostNameValid = function(){
		var inputHostName = document.getElementById(this.UIHostTxtField).value;
		if(this.blank(inputHostName)){
			return false;
		}	
		if(inputHostName.match(/^[A-Za-z0-9\-\_\.\:]+$/)) {		    
	        return true;
	    }else {
	    	return false;
	    }
	}
	
	this.isUserNameValid = function(){
		var userName = document.getElementById(this.UIUserTxtField).value;
		if(this.blank(userName)){
			return false;
		}			
		return true;
	}
	
	this.startVersionCheck = function(event){		
		if(event!=null){
			var eventTarget = YAHOO.util.Event.getTarget(event);
			if(eventTarget.id == this.UIValidateBtn){
				this.CloseOnSuccess = false;
			}
			/*else if(eventTarget.id == this.UISaveBtn && !this.CloseOnSuccess){
				this.CloseOnSuccess = true;
			}*/
			else if(eventTarget.id == this.UISaveBtn && this.CloseOnSuccess) {
				this.PanelWindow.hide();		
				document.location.href = this.baseURL;//document.location.href;
				return;
			}
		}else{
			this.CloseOnSuccess = false;
		}
		this.resetStyles();
		this._animateShowErrorPane();
		//var inputHostName = document.getElementById(this.UIHostTxtField).value;		
		if(!this.isHostNameValid()){
			this.setResultStatus(false, ' Please enter a valid host name.');
			return;
		}
		if(!this.isUserNameValid()){
			this.setResultStatus(false, ' Please enter a valid user name.');
			return;
		}		
		this.disableSave(true);
		this.checkWhetherHostNameExists();		
	}	
	
	this.checkWhetherHostNameExists = function(){
		
		var url = "plugins/springs/vomer/source/checkSource?ms=" + new Date().getTime();
	    url = encodeURI(url);
	    
		var sourceName = YAHOO.lang.trim(document.getElementById(this.UIHostTxtField).value);			
		var kvp = "source="+sourceName;
		var postdata = encodeURI(kvp);
		
		YAHOO.util.Dom.removeClass('divDSVersionCheck', 'hidden');	
		YAHOO.util.Dom.removeClass('divVersionCheckResults','invisible');
		
		YAHOO.util.Dom.removeClass('divStatus','invisible');
		//YAHOO.util.Dom.removeClass('divStatus','result');
		//YAHOO.util.Dom.addClass('divVersionCheckResults','success');
		
		var lblDSV4Header = document.getElementById('lblDSV4Header');
		lblDSV4Header.innerHTML = "Verifying source " + document.getElementById(this.UIHostTxtField).value;							
		YAHOO.util.Dom.removeClass('spnDSVersion4Header','invisible');	
		
		var progImg = document.getElementById('imgDSV4Status');
		progImg.src = this.ProgressImage;
		
		var conn = YAHOO.util.Connect.asyncRequest('POST', url, this.checkHostNameExistResult, postdata); 		
		//}
	}
	
	this.checkHostNameExistResult = { 
	    success: function(o){
			var result = true;
	    	var resultTxt = o.responseText;
	    	
	    	var succIndex = resultTxt.indexOf("Success:");
	 	    var failIndex = resultTxt.indexOf("Failure:");  
	 	    var resultMsg = resultTxt.substring(8); 
	 	    
	 	    var progImg = document.getElementById('imgDSV4Status');
	 	    
	 	    if(failIndex != -1) {
	 	    	result = false;
	 	    	//YAHOO.util.Dom.addClass('divVersionCheckResults','result');
		    	progImg.src = "/sm/plugins/ER/img/Rollup/cross.gif";	
	 	    }else {
	 	    	//YAHOO.util.Dom.addClass('divVersionCheckResults','success');
		    	progImg.src = "/sm/plugins/ER/img/Rollup/tick.gif";
	 	    }
	 	
	    	if(result){
	    		this.setStatus(true,'VOM4.0');
	    	} else {
		    	document.getElementById(this.UIHostTxtField).focus();
	    		this.setResultStatus(false, resultMsg);
		    	//document.getElementById(this.UIHostTxtField).value="";
				return;
	    	}
	    }, 
	    failure: function(response){
	    	this.setResultStatus(false, "Communication Error");
	    	
	    },
	    timeout: 300000, 
	    scope: this
	};
	
	this.licneseCheckResult = { 
	    success: function(o){
	    	var result = o.responseText;
			if(result.indexOf('timeout') != -1){
				window.location.href ='/sm/Login?sessionExpire=true';
				return;
			}
	    	if(result.indexOf('true') != -1){
				this.validateFlag=false;
				document.getElementById(this.UIHostTxtField).value = '';
				document.getElementById(this.UIHostTxtField).disabled = false;
				this.resetStyles();
				//this.PanelWindow.setHeader("Add New Data Source");
				this.PanelWindow.setHeader('<div align="left"><label>Add New Data Source</label></div>');

				//performTask('Add');
				this.PanelWindow.show();	
				//this.PanelWindow.moveTo(0,0);
	    	} else {
				alert("You are currently using Veritas Operation Manager Enterprise Server with an evaluation license. You "+
						"cannot add more than two data sources when using an evaluation license. If you "+
							"want to add more data sources, you need to obtain a permanent license. For "+
								"information about obtaining a permanent license, go to the Symantec Licensing "+
									"Portal: www.symantec.com/business/products/licensing/activation");
				return;
	    	}
	    }, 
	    failure: function(response){
			//this.checkForVersion5();
			return;
	    	
	    },
	    timeout: 300000, 
	    scope: this
	};



	this._animateShowErrorPane = function(){
		YAHOO.util.Dom.setStyle(this.ErrorPane,'display','');
		//YAHOO.util.Dom.removeClass(this.ErrorPane,'hidden');
		var anim =  new YAHOO.util.Anim(this.ErrorPane, {height: {to:160}},0.2);
			anim.animate();
	}
	
	this.resetStyles = function(){
		YAHOO.util.Dom.setStyle(this.ErrorPane,'display','none');
		

		YAHOO.util.Dom.addClass('spnDSVersion4Header','invisible');	
		YAHOO.util.Dom.addClass('spnDSVersion5Header','invisible');
		
		var lblDSV4Header = document.getElementById('lblDSV4Header');
		lblDSV4Header.innerHTML = "";
		var lblDSV5Header = document.getElementById('lblDSV5Header');
		lblDSV5Header.innerHTML = "";
		
		document.getElementById('imgDSV4Status').src = this.SpacerImage;
		document.getElementById('imgDSV5Status').src = this.SpacerImage;
		
		//document.getElementById('imgDSV5Status').src =this.ProgressImage;
		//YAHOO.util.Dom.addClass('divDSV5Subversions','invisible');
		//document.getElementById('imgDSV5MP1Status').src = this.FailureImage;
		//document.getElementById('imgDSV5GAStatus').src  = this.FailureImage;
		//document.getElementById('imgDSV51GAStatus').src  = this.FailureImage;
		//document.getElementById('imgDSV52GAStatus').src  = this.FailureImage;
		//YAHOO.util.Dom.addClass('spnDSVersion4Header','invisible');
		
		//YAHOO.util.Dom.addClass('divDSV4Subversions','invisible');
		//document.getElementById('imgDSV43Status').src = this.FailureImage;

		YAHOO.util.Dom.addClass('divVersionCheckResults','invisible');
		document.getElementById('lblDSResult').innerHTML = '';
		
		document.getElementById(this.UISaveBtn).disabled = false;
		document.getElementById(this.UICancelBtn).disabled = false;
	}
	
	this.initPanel();
}

function initRollupPanel(){
		jj = new DataSourcePanel();
}

function reInitRollupPanel(){
		jj.reInitPanel();
}

function RTrim( value ) {	
	var re = /((\s*\S+)*)\s*/;
	return value.replace(re, "$1");	
}

function LTrim( value ) {	
	var re = /\s*((\S+\s*)*)/;
	return value.replace(re, "$1");	
}

function preSelectSourceTable(event, objectArray) {
	refreshPanel(objectArray);
}

//YAHOO.util.Event.onDOMReady(initRollupPanel); 
YAHOO.util.Event.addListener(window, 'load', initRollupPanel);

function deleteDataSource(dataSourceId)
{
	if(confirm("Do you want to delete selected data source?"))
	{
		var statusDiv = YAHOO.util.Dom.get("statusPanel");
		statusDiv.style.display="inline";
		YAHOO.util.Dom.get('statusPanelText').innerHTML = "Deleting data source";
		var tempUrl = new String(window.location.href);
		tempUrl = tempUrl.replace(/&action(\=[^&]*)?(?=&|$)|^action(\=[^&]*)?(&|$)/g,''); 
		tempUrl = tempUrl.replace(/&dataSourcesTable(\=[^&]*)?(?=&|$)|^dataSourcesTable(\=[^&]*)?(&|$)/g,'');
		tempUrl = tempUrl.replace(/#/g,'');
		var url = tempUrl.toString();
		url += "&action=delete";
		url += "&dataSourcesTable="+dataSourceId;
		YAHOO.util.Connect.asyncRequest('POST', url , {
        success:function(obj)
		{
			window.location.href = obj.argument;
        }
        ,
		failure:function()
		{
            alert("couldn't delete data source.");
        }
		,
		argument:tempUrl.toString()
    }
    , null);
	}
}

function checkDeleteAllowed()
{
	var selectedObjects = dataSourcesTable.getSelectedObjects();
	if(selectedObjects == "")
	{
		alert("Please select a data source."); 
		return;
	}
	
	var url = "/sm/plugins/springs/vomer/source/isRollupInProgress?dataSourceId="+selectedObjects+"&ms="+new Date().getTime();
	
	YAHOO.util.Connect.asyncRequest('POST', url , {
        success:function(obj)
		{
			var resText = YAHOO.lang.trim(obj.responseText);
			if(YAHOO.lang.JSON.isSafe(resText))
			{
				var isRollupInProgress = YAHOO.lang.JSON.parse(resText);
				if(isRollupInProgress)
				{
					alert("Cannnot delete selected data source while its rollup is in progress.\n please try after rollup is complete");
				}
				else
				{
					deleteDataSource(obj.argument);
				}
			}
        }
        ,
		failure:function()
		{
            alert("couldn't check if a roll up is progress for current datasource.");
        }
		,
		argument:selectedObjects        
    }
    , null);
}