if exists(select 1 from sys.sysprocedure where proc_name = 'sahara_database_data' and user_name(creator) = 'sahara') then
   drop procedure sahara.sahara_database_data
end if;

CREATE PROCEDURE sahara."sahara_database_data" (@batch_id integer)
/* RESULT ( column_name, ... ) 
 */
BEGIN

/* Variables for logging purpose */
/* 
To record initial time when this stored procedure was called 
*/
DECLARE @initialtime timestamp; 

/* 
Total number of dimension this stored procedure is inserting 
*/
DECLARE @noDims bigint; 

/* 
Total number of facts this stored procedure is inserting 
*/
DECLARE @noFacts bigint; 

/* Variables for handling RAC business logic */
DECLARE @l_cnt INTEGER;
DECLARE @l_parentid INTEGER;
DECLARE @l_sqlstr LONG VARCHAR;

/* Log detail error info */
declare @err_code integer;
declare @error    integer;
declare @serror         varchar(32767);
declare @sqlstate       varchar(5);
declare @tablename      varchar(128);
declare @attributenames varchar(256);
declare @count    integer;
declare @error_count    integer;
declare @is_ASM int;

/* 
Store the current time 
*/
Set @initialtime=GETDATE();

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_database_data', 'I','starting dimension/facts rollup -> sahara_database_dimension/sahara_database_facts';

if exists (SELECT 1 FROM sys.systable where table_name = 'sf_view_storage_Assignment_r' and table_type='VIEW' and user_name(creator)='habdbsync')
then 
set @is_ASM = 1;
end if;

if exists(select 1 from sys.systable where table_name='sahara_database_dimension' and creator=user_id('sahara')) then
   drop table sahara.sahara_database_dimension
end if;

if exists(select 1 from sys.systable where table_name='sahara_database_facts' and creator=user_id('sahara')) then
   drop table sahara.sahara_database_facts
end if;

if exists(select 1 from sys.systable where table_name='sahara_database_to_host' and creator=user_id('sahara')) then
   drop table sahara.sahara_database_to_host
end if;

/*==============================================================*/
/* Table: sahara_database_dimension                                                */
/*==============================================================*/
create table sahara.sahara_database_dimension  (
	database_id           		unsigned int, // REMOVE
	host_id           			unsigned int, // REMOVE
	host_unique_identifier 		varchar(256),	
	ccstor_unique_identifier    varchar(256),
	display_name        	 	varchar(256),
	instance_display_name        	 	varchar(256),
	type	 					varchar(256),
	version	 					varchar(256),
	database_state				varchar(256),
	discovery_state			unsigned int ,
	discovery_state_str		varchar(256),
	is_clustered bit default 0,
	is_rac_aware bit default 0, /*Stores IS_RAC for VOMA*/
	is_based_on_thin	bit default 0,
	is_based_on_db_vm	bit default 0,
	primary key(ccstor_unique_identifier)
);
grant select on sahara.sahara_database_dimension to public;

/*==============================================================*/
/* Table: sahara_database_facts                                          */
/*==============================================================*/
create table sahara.sahara_database_facts  (
	database_id    	        unsigned int, // REMOVE

	capacity 			NUMERIC(22,9) default 0,
	capacity_avaialble	NUMERIC(22,9) default 0,
	capacity_used		NUMERIC(22,9) default 0,
	percent_used		NUMERIC(20,9) default 0,
	percent_free		NUMERIC(20,9) default 0,
	
	capacity_from_raw   NUMERIC(22,9) default 0, /*Not available in VOM 4.0*/
	available_from_raw  NUMERIC(22,9) default 0, /*Not available in VOM 4.0*/
	used_from_raw       NUMERIC(22,9) default 0 , /*Not available in VOM 4.0*/
	
	capacity_from_file_system    NUMERIC(22,9) default 0,
	available_from_file_system   NUMERIC(22,9) default 0,
	used_from_file_system        NUMERIC(22,9) default 0,
	
	capacity_from_volume         NUMERIC(22,9) default 0,
	available_from_volume        NUMERIC(22,9) default 0,
	used_from_volume             NUMERIC(22,9) default 0,
	
	batch_id 			int default 0,
	ccstor_unique_identifier varchar(256),
	no_of_hosts        int           default 0 not null,

	capacity_from_dbvm         NUMERIC(22,9) default 0,
	available_from_dbvm        NUMERIC(22,9) default 0,
	used_from_dbvm             NUMERIC(22,9) default 0,

	primary key(ccstor_unique_identifier)
);
grant select on sahara.sahara_database_facts to public;

create table sahara.sahara_database_to_host  (
        database_unique_identifier varchar(256),
        host_unique_identifier varchar(256),
	capacity_from_dbvm	   NUMERIC(22,9) default 0,	
	used_from_dbvm             NUMERIC(22,9) default 0,
        primary key ( database_unique_identifier,host_unique_identifier)
);
grant select on sahara.sahara_database_to_host to public;

set @tablename = 'sahara.sahara_database_dimension';
set @attributenames = 'host_unique_identifier,ccstor_unique_identifier,display_name,type,version,database_state,instance_display_name,is_clustered,is_rac_aware';
insert into sahara.sahara_database_dimension 
(	
	host_unique_identifier,
	ccstor_unique_identifier,
	display_name,
	type,
	version,
	database_state,
	discovery_state, 
	instance_display_name,
    is_clustered,
    is_rac_aware
) on existing update
select
HOST.ID,
DB.ID,
DB.name,
IF DB.TYPE IS NULL THEN SYSMGR.DATABASETYPE ELSE DB.TYPE ENDIF as TYPE,
DB.version,
RB1.DISPLAY as display_status,
1 as discovery_state,
SYSMGR.NAME as sysmgr_name,
isnull((if sg.id = '' then 0 else 1 endif),0) as IsClustered,
(if isnull(DB.DB_SUBTYPE,0) = 1 then 1 else 0 endif) as is_rac_aware
--HABDBSYNC.SF_VIEW_BASE_UI_DATABASE DB
FROM
    HABDBSYNC.SF_VIEW_DBED_DB_R DB
    LEFT OUTER JOIN HABDBSYNC.SF_VIEW_DB_SYSMANAGER_R SYSMGR ON SYSMGR.ID=DB.DBSYS_ID
    LEFT OUTER JOIN HABDBSYNC.SF_VIEW_DBED_TO_HOST_R HOSTLIST ON HOSTLIST.DB_ID=DB.ID
    LEFT OUTER JOIN HABDBSYNC.SF_VIEW_ALL_HOSTS_S HOST ON HOST.ID=HOSTLIST.HOST_ID
    LEFT OUTER JOIN HABDBSYNC.VCS_VIEW_REL_HOST_RES_R hres on hres.storage_id=DB.ID and hres.host_id=host.id
    LEFT OUTER JOIN HABDBSYNC.VCS_VIEW_RESOURCE_R res on res.id = hres.res_id
    left outer join HABDBSYNC.VCS_VIEW_SERVICEGROUP_R sg on sg.id = res.sg_id
    left outer join HABDBSYNC.UI_VIEW_ENUM_DEF_R ENUM1 on ENUM1.OBJECT='DB' and ENUM1.PROPERTY='STATE' and ENUM1.INT_VALUE=DB.STATUS    
    left outer join HABDBSYNC.UI_VIEW_RES_BUNDLE_R RB1 ON ENUM1.RESOURCE_ID=RB1.ID and locale_id = 'en_US'
WHERE     
    (SYSMGR.AUTH_REQD IS NULL OR SYSMGR.AUTH_REQD <> 1)
    AND TYPE != 'OracleASM';

if (@is_ASM=1) THEN
set @attributenames = 'ccstor_unique_identifier,is_based_on_db_vm';
insert into sahara.sahara_database_dimension 
(	
	ccstor_unique_identifier,	
	is_based_on_db_vm
) on existing update
select
DB.ID,
1 as is_based_on_db_vm
FROM
    HABDBSYNC.SF_VIEW_DBED_DB_R DB
join HABDBSYNC.SF_VIEW_DB_TO_STORAGE_R DBS on db.id = dbs.db_id and dbs.type = 'OracleASM'
join sahara.sahara_database_dimension dbd on db.id=dbd.ccstor_unique_identifier;

end if; /*for ASM*/


set @attributenames = 'ccstor_unique_identifier,is_based_on_thin';
update sahara.sahara_database_dimension as db set is_based_on_thin = 1
where ccstor_unique_identifier in (
SELECT dbtoLUN.id FROM "HABDBSYNC"."SF_VIEW_UI_DB_DISK" dbtoLUN
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = dbtoLUN.lun_id
where lun.thin_status='ThinLun')
;

set @tablename = 'sahara.sahara_database_to_host';
set @attributenames = 'database_unique_identifier, host_unique_identifier';
insert into sahara.sahara_database_to_host
(	database_unique_identifier,
    host_unique_identifier
) on existing update
select
DB.ID,
HOSTLIST.HOST_ID
FROM
sahara.sahara_database_dimension DBD
    Join HABDBSYNC.SF_VIEW_DBED_DB_R DB on DB.ID = DBD.ccstor_unique_identifier
    JOIN HABDBSYNC.SF_VIEW_DB_SYSMANAGER_R SYSMGR ON SYSMGR.ID=DB.DBSYS_ID
    JOIN HABDBSYNC.SF_VIEW_DBED_TO_HOST_R HOSTLIST ON HOSTLIST.DB_ID=DB.ID
WHERE     
    SYSMGR.AUTH_REQD IS NULL OR SYSMGR.AUTH_REQD <> 1;

set @tablename = 'sahara.sahara_database_facts';
set @attributenames = 'ccstor_unique_identifier, no_of_hosts';
insert into sahara.sahara_database_facts 
(
   ccstor_unique_identifier,
   no_of_hosts
) on existing update
select 
DBD.ccstor_unique_identifier,
count(DBTH.host_unique_identifier) as no_of_hosts
from sahara.sahara_database_dimension DBD
join sahara.sahara_database_to_host DBTH on DBD.ccstor_unique_identifier = DBTH.database_unique_identifier
group by DBD.ccstor_unique_identifier; 

set @attributenames = 'host_unique_identifier,capacity,capacity_used,capacity_avaialble,percent_used';

insert into sahara.sahara_database_facts  
(
   ccstor_unique_identifier,
   capacity,
   capacity_used,
   capacity_avaialble,
   percent_used
) on existing update
select
ID,
isnull(DB.totalsize,0)/(1024.0*1024.0*1024.0),
isnull(DB.usedsize,0)/(1024.0*1024.0*1024.0),
(IF (isnull(DB.TOTALSIZE,0) - isnull(DB.USEDSIZE,0)>0)
     THEN (isnull(DB.TOTALSIZE,0) - isnull(DB.USEDSIZE,0)) ELSE 0
     ENDIF)/(1024.0*1024.0*1024.0) AS FREESPACE,
    (IF isnull(DB.TOTALSIZE,0) = 0 THEN 0 ELSE (USEDSIZE * 100 / TOTALSIZE) ENDIF) AS USAGE
from
sahara.sahara_database_dimension DBD
join HABDBSYNC.SF_VIEW_DBED_DB_R DB on DBD.ccstor_unique_identifier = DB.ID
;

/*
calculate percent free
*/
set @attributenames = 'percent_free';
insert into sahara.sahara_database_facts (ccstor_unique_identifier, percent_free)
on existing update 
select ccstor_unique_identifier, (if capacity_avaialble =0 then 0 else (100 - percent_used) endif) as percent_free
from sahara.sahara_database_facts;

/*ASSUMPTION: We have seen a configuration when ts is based off multiple files. Ts capacities will be double-counted in such cases. We are ignoring it as the configuration is not prevalant and recommended.*/
set @attributenames = 'ccstor_unique_identifier, capacity_from_volume, used_from_volume, available_from_volume, capacity_from_file_system, used_from_file_system, available_from_file_system';
insert into sahara.sahara_database_facts (ccstor_unique_identifier, capacity_from_volume, used_from_volume, available_from_volume, capacity_from_file_system, used_from_file_system, available_from_file_system, capacity_from_dbvm, used_from_dbvm, available_from_dbvm)
on existing update 
select
distinctTblspc.db_id,
isnull(sum(if distinctTblspc.israw=1 then distinctTblspc.size else 0 endif),0)/(1024.0*1024.0*1024.0) as dbOnRawVol,
isnull(sum(if distinctTblspc.israw=1 then distinctTblspc.usedsize + distinctTblspc.log_file_size else 0 endif),0)/(1024.0*1024.0*1024.0) as dbOnRawVolUsed,
isnull(sum(if distinctTblspc.israw=1 then distinctTblspc.freesize else 0 endif),0)/(1024.0*1024.0*1024.0) as dbOnRawVolFree,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='FILESYSTEM') then distinctTblspc.size else 0 endif),0)/(1024.0*1024.0*1024.0) as dbOnFS,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='FILESYSTEM') then distinctTblspc.usedsize + distinctTblspc.log_file_size else 0 endif),0)/(1024.0*1024.0*1024.0) as dbOnFSUsed,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='FILESYSTEM') then distinctTblspc.freesize else 0 endif),0)/(1024.0*1024.0*1024.0) as dbOnFSFree,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='OracleASM') then distinctTblspc.size else 0 endif),0)/(1024.0*1024.0*1024.0) as capacity_from_dbvm,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='OracleASM') then distinctTblspc.usedsize + distinctTblspc.log_file_size else 0 endif),0)/(1024.0*1024.0*1024.0) as used_from_dbvm,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='OracleASM') then distinctTblspc.freesize else 0 endif),0)/(1024.0*1024.0*1024.0) as available_from_dbvm
from
(
select 
distinct 
file.id,
file.db_id,
file.file_storage_type, 
file.israw,
isnull(file.size,0) as size,
file.name,
ts.name,
isnull(ts.totalsize,0) as totalsize,
isnull(ts.usedsize,0) as usedsize,
(if ts.totalsize!=0 then (ts.totalsize - ts.usedsize) else 0 endif) as freesize,
isnull((if file.tbs_id = '-' then file.size else 0 endif),0) as log_file_size
from HABDBSYNC.SF_VIEW_UI_DB_FILE file 
left outer join HABDBSYNC.SF_VIEW_DB_TABLESPACE_R ts on tbs_id = ts.id
) as distinctTblspc
join sahara.sahara_database_dimension DBD on DBD.ccstor_unique_identifier = distinctTblspc.db_id
group by distinctTblspc.db_id
;

set @tablename = 'sahara.sahara_database_to_host';
set @attributenames = 'database_unique_identifier, host_unique_identifier,used_from_dbvm';
insert into sahara.sahara_database_to_host
(	database_unique_identifier,
    host_unique_identifier,
    capacity_from_dbvm,
    used_from_dbvm
) on existing update
select
distinctTblspc.db_id,
distinctTblspc.host_id,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='OracleASM') then distinctTblspc.size else 0 endif),0)/(1024.0*1024.0*1024.0) as capacity_from_dbvm,
isnull(sum(if ((distinctTblspc.israw is null or distinctTblspc.israw=0) and distinctTblspc.file_storage_type='OracleASM') then distinctTblspc.usedsize + distinctTblspc.log_file_size else 0 endif),0)/(1024.0*1024.0*1024.0) as used_from_dbvm
from
(
select 
distinct 
file.id,
file.db_id,
file.file_storage_type, 
file.israw,
isnull(file.size,0) as size,
file.name,
ts.name,
isnull(ts.totalsize,0) as totalsize,
isnull(ts.usedsize,0) as usedsize,
(if ts.totalsize!=0 then (ts.totalsize - ts.usedsize) else 0 endif) as freesize,
isnull((if file.tbs_id = '-' then file.size else 0 endif),0) as log_file_size
,HOSTLIST.Host_ID
from HABDBSYNC.SF_VIEW_UI_DB_FILE file 
left outer join HABDBSYNC.SF_VIEW_DB_TABLESPACE_R ts on tbs_id = ts.id
 join sahara.sahara_database_dimension DBD on DBD.ccstor_unique_identifier = file.db_id
  Join HABDBSYNC.SF_VIEW_DBED_DB_R DB on DB.ID = DBD.ccstor_unique_identifier
    JOIN HABDBSYNC.SF_VIEW_DB_SYSMANAGER_R SYSMGR ON SYSMGR.ID=DB.DBSYS_ID
    JOIN HABDBSYNC.SF_VIEW_DBED_TO_HOST_R HOSTLIST ON HOSTLIST.DB_ID=DB.ID
) as distinctTblspc
group by distinctTblspc.db_id,
distinctTblspc.host_id;

select count(1) into @noDims from sahara.sahara_database_dimension;
select count(1) into @noFacts from sahara.sahara_database_facts;

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_database_data', 'I','completed dimension rollup -> sahara_database_dimension: ' || @noDims || ' rows inserted';

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_database_data', 'I','completed facts rollup -> sahara_database_facts: ' || @noFacts || ' rows inserted';

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_database_data', 'I','roll-up batch: ' || @batch_id || ', data set: sahara_database_dimension/sahara_database_facts -> Succeeded: ' ||datediff(second,@initialtime,GETDATE()) || ' secs';

commit;


EXCEPTION
    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 ) then
            SET @ERROR_COUNT=@ERROR_COUNT + 1;
            BEGIN
                insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message)
                select GETDATE(), @batch_id, 'sahara.sahara_database_data', 'E', string (' Error failed at = ', @tablename+':'+@attributenames ) +
                                                                             string (': SQLCODE = ', @error ) +
                                                                             string (': ERRORMSG = ', @serror ) +
                                                                             string (': SQLSTATE = ', @sqlstate );
                message string (' Error failed at = ', @tablename+':'+@attributenames ) to LOG;
                message string ('          @error = ', @error ) to LOG;
                message string ('      @serrir()  = ', @serror ) to LOG;
                message string ('     @sqlstate() = ', @sqlstate ) to LOG;
            END;
        END IF;
        -- RESIGNAL;

END;
grant execute on sahara.sahara_database_data to public;