if exists(select 1 from sys.sysprocedure where proc_name = 'sahara_dg_data' and user_name(creator) = 'sahara') then
   drop procedure sahara.sahara_dg_data
end if;

CREATE PROCEDURE sahara."sahara_dg_data" (@batch_id integer)
/* RESULT ( column_name, ... ) 
 */
BEGIN

/* Variables for logging purpose */
/* 
To record initial time when this stored procedure was called 
*/
DECLARE @initialtime timestamp; 

/* 
Total number of dimension this stored procedure is inserting 
*/
DECLARE @noDims bigint; 

/* 
Total number of facts this stored procedure is inserting 
*/
DECLARE @noFacts bigint; 


DECLARE @noFactsdgtolun bigint;

/* Variables for handling CFS business logic */
DECLARE @l_cnt INTEGER;
DECLARE @slice VARCHAR(100);

/* Log detail error info */
declare @err_code integer;
declare @error    integer;
declare @serror         varchar(32767);
declare @sqlstate       varchar(5);
declare @tablename      varchar(128);
declare @attributenames varchar(256);
declare @count    integer;
declare @error_count    integer;
declare @is_ASM int;

/* 
Store the current time 
*/
Set @initialtime=GETDATE();

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_dg_data', 'I','starting dimension/facts rollup -> sahara_dg_dimension/sahara_dg_facts';

if exists (SELECT 1 FROM sys.systable where table_name = 'sf_view_storage_Assignment_r' and table_type='VIEW' and user_name(creator)='habdbsync')
then 
set @is_ASM = 1;
end if;

if exists(select 1 from sys.systable where table_name='sahara_dg_dimension' and creator=user_id('sahara')) then
   drop table sahara.sahara_dg_dimension
end if;

if exists(select 1 from sys.systable where table_name='sahara_dg_facts' and creator=user_id('sahara')) then
   drop table sahara.sahara_dg_facts
end if;

if exists(select 1 from sys.systable where table_name='sahara_dg_to_lun' and creator=user_id('sahara')) then
   drop table sahara.sahara_dg_to_lun
end if;

if exists(select 1 from sys.systable where table_name='sahara_dg_to_host' and creator=user_id('sahara')) then
   drop table sahara.sahara_dg_to_host
end if;

/*==============================================================*/
/* Table: Disk Group                                                */
/*==============================================================*/
create table sahara.sahara_dg_dimension  (
	host_unique_identifier 		varchar(256) ,
	ccstor_unique_identifier	varchar(256) ,
	display_name        		varchar(1024) default '',
	type	 			varchar(256) default '',
	version				varchar(256) default '',
	is_shared			bit default 0,	/*To be tested for ASM*/
	family				varchar(256) default '',
	batch_id 			int default 0,
	is_based_on_direct_attached	bit default 0,
	is_based_on_thin		bit default 0,
	state				varchar(256) default '', /*Imported / Deported etc. To be deprecated when added in correlation table*/
	/*ASM support*/
	redundancy_type			varchar(256) default '',
	is_used_by_clustered_DB		bit default 0, 
	uses_vm_volumes			bit default 0,
	primary key(ccstor_unique_identifier)
);
grant select on sahara.sahara_dg_dimension to public;

/*==============================================================*/
/* Table: sahara_dg_facts                                          */
/*==============================================================*/
create table sahara.sahara_dg_facts  (
   capacity 			NUMERIC(22,9) default 0,
   capacity_free		NUMERIC(22,9) default 0,
   capacity_used		NUMERIC(22,9) default 0,
   percent_free			NUMERIC(20,9) default 0,
   percent_used			NUMERIC(20,9) default 0,
   batch_id 			int default 0,
   ccstor_unique_identifier	varchar(256),
   no_of_hosts			int default 0 not null,
   capacity_overhead		NUMERIC(20,9) default 0, /*Testing pending*/
   capacity_unknown		NUMERIC(20,9) default 0,
   capacity_used_db		NUMERIC(20,9) default 0, 
   capacity_used_clustered_db	NUMERIC(20,9) default 0, 
   capacity_used_db_tbsp_used	NUMERIC(20,9) default 0, 
   primary key(ccstor_unique_identifier)
);
grant select on sahara.sahara_dg_facts to public;

create table sahara.sahara_dg_to_host  (
        dg_unique_identifier varchar(256),
        host_unique_identifier varchar(256),
	state  varchar(256) default '', /*Imported / Deported etc*/
	capacity_used_db		NUMERIC(20,9) default 0, /*Sum of file size per host per ASM DG*/
	capacity_used_db_tbsp_used      NUMERIC(20,9) default 0, /*Sum of tablespace used size per host per ASM DG*/
        primary key ( dg_unique_identifier,host_unique_identifier)
);

create index INDEX_1 on sahara.sahara_dg_to_host (
       state ASC
);

grant select on sahara.sahara_dg_to_host to public;

--- Create a brand new table, so that it does not mess with the existing
--- table fs_to_vm. The only thing to be done here is used the sahara_fs_to_vm
--- to get to the lunid and array id.
create table sahara.sahara_dg_to_lun  (
   dg_ccstor_unique_identifier varchar(256),
   lun_ccstor_unique_identifier varchar(256),
   array_ccstor_unique_identifier varchar(256),   
   primary key(dg_ccstor_unique_identifier,lun_ccstor_unique_identifier)
);
grant select on sahara.sahara_dg_to_lun to public;


/*
if  exists (select 1 from syscolumn cl,systable as tbl where tbl.table_id = cl.table_id and tbl.table_name = 'Apps_filesystem'  and cl.column_name = 'ignorecapacity') then
*/
/*
Get DG Dimension.
*/
set @tablename = 'sahara.sahara_dg_dimension';
set @attributenames = 'host_unique_identifier,ccstor_unique_identifier,display_name,type,version,state,is_shared,family';

insert into sahara.sahara_dg_dimension 
(	host_unique_identifier,
	ccstor_unique_identifier,
	display_name,	
	type,
    version,
	state,
    is_shared,
    family
) on existing update
select SF_VIEW_ALL_HOSTS_S.id as HOST_ID, SF_VIEW_DISKGROUP_R.ID, SF_VIEW_DISKGROUP_R.name as DG_NAME, RB_TYPE.DISPLAY as DISPLAY_TYPE, 
isnull((if (SF_VIEW_HOST_TO_DG_R.DISABLED!=1 and SF_VIEW_HOST_TO_DG_R.PERMISSION=0) then '' else SF_VIEW_DISKGROUP_R.VERSION endif),'') as DG_VERSION, 
RB_STATE.DISPLAY,
(if SF_VIEW_DISKGROUP_R.TYPE in ('shared','ASM-SHARED') then 1 else 0 endif) as isShared, isnull(SF_VIEW_DISKGROUP_R.FAMILY,'') as FAMILY
FROM
    HABDBSYNC.SF_VIEW_DISKGROUP_R
    INNER JOIN HABDBSYNC.SF_VIEW_HOST_TO_DG_R ON SF_VIEW_DISKGROUP_R.ID=SF_VIEW_HOST_TO_DG_R.DG_ID AND SF_VIEW_DISKGROUP_R.TYPE != 'DUMMY'
    INNER JOIN HABDBSYNC.SF_VIEW_ALL_HOSTS_S ON SF_VIEW_HOST_TO_DG_R.HOST_ID=SF_VIEW_ALL_HOSTS_S.ID
    LEFT OUTER JOIN HABDBSYNC.SF_VIEW_HOST_TO_HC ON SF_VIEW_ALL_HOSTS_S.ID=SF_VIEW_HOST_TO_HC.HOST_ID
    left outer join HABDBSYNC.SF_VIEW_UI_ENUM_BUNDLE RB_TYPE on RB_TYPE.OBJECT='DG' and RB_TYPE.PROPERTY='TYPE' 
    and RB_TYPE.VALUE=SF_VIEW_DISKGROUP_R.TYPE and RB_TYPE.LOCALE_ID = 'en_US'
    left outer join HABDBSYNC.SF_VIEW_UI_ENUM_BUNDLE RB_STATE on RB_STATE.OBJECT='DG' and RB_STATE.PROPERTY='STATE' 
        and RB_STATE.INT_VALUE=(CASE 
    /* VxVM */
    WHEN SF_VIEW_HOST_TO_DG_R.DISABLED=1 AND SF_VIEW_DISKGROUP_R.FAMILY = 'VxVM'     
    THEN 2     
    WHEN SF_VIEW_HOST_TO_DG_R.DISABLED=0 AND SF_VIEW_DISKGROUP_R.FAMILY = 'VxVM' 
        THEN SF_VIEW_HOST_TO_DG_R.PERMISSION
   /* other than VxVM */
    WHEN  SF_VIEW_DISKGROUP_R.FAMILY <> 'VxVM' 
    THEN SF_VIEW_DISKGROUP_R.STATE 
    END) and RB_STATE.LOCALE_ID = 'en_US'
;

set @tablename = 'sahara.sahara_dg_to_host';
set @attributenames = 'dg_unique_identifier, host_unique_identifier, state';
insert into sahara.sahara_dg_to_host
(	dg_unique_identifier,
    host_unique_identifier,
    state
) on existing update
select dg.ccstor_unique_identifier, host.ccstor_unique_identifier, RB_STATE.DISPLAY
FROM
    sahara.sahara_dg_dimension dg
    join HABDBSYNC.SF_VIEW_DISKGROUP_R on dg.ccstor_unique_identifier = SF_VIEW_DISKGROUP_R.id
    join HABDBSYNC.SF_VIEW_HOST_TO_DG_R ON dg.ccstor_unique_identifier=SF_VIEW_HOST_TO_DG_R.DG_ID
    JOIN sahara.sahara_host_dimension host ON SF_VIEW_HOST_TO_DG_R.HOST_ID=host.ccstor_unique_identifier
    left outer join HABDBSYNC.SF_VIEW_UI_ENUM_BUNDLE RB_STATE on RB_STATE.OBJECT='DG' and RB_STATE.PROPERTY='STATE' 
        and RB_STATE.INT_VALUE=(CASE 
    /* VxVM */
    WHEN SF_VIEW_HOST_TO_DG_R.DISABLED=1 AND SF_VIEW_DISKGROUP_R.FAMILY = 'VxVM'     
    THEN 2     
    WHEN SF_VIEW_HOST_TO_DG_R.DISABLED=0 AND SF_VIEW_DISKGROUP_R.FAMILY = 'VxVM' 
        THEN SF_VIEW_HOST_TO_DG_R.PERMISSION
   /* other than VxVM */
    WHEN  SF_VIEW_DISKGROUP_R.FAMILY <> 'VxVM' 
    THEN SF_VIEW_DISKGROUP_R.STATE 
    END) and RB_STATE.LOCALE_ID = 'en_US';

/*
	Get DG facts
*/

set @tablename = 'sahara.sahara_dg_facts';
set @attributenames = 'ccstor_unique_identifier, no_of_hosts';
insert into sahara.sahara_dg_facts 
(
   ccstor_unique_identifier,
   no_of_hosts
) on existing update
select 
DGD.ccstor_unique_identifier,
count(DGTH.host_unique_identifier) as no_of_hosts
from sahara.sahara_dg_dimension DGD
join sahara.sahara_dg_to_host DGTH on DGD.ccstor_unique_identifier = DGTH.dg_unique_identifier
group by DGD.ccstor_unique_identifier; 


set @attributenames = 'ccstor_unique_identifier,capacity,capacity_free,capacity_used,percent_free,percent_used,capacity_overhead';
insert into sahara.sahara_dg_facts 
(
   ccstor_unique_identifier,
   capacity,   
   capacity_free,
   capacity_used,
   percent_free,
   percent_used,
   capacity_overhead
) on existing update
select 
DG.ID as ccstor_unique_identifier,
isnull(DG.TotalSize,0)/(1024.0*1024.0*1024.0) as capacity,
isnull(DG.FREESpace,0)/(1024.0*1024.0*1024.0) as capacity_free,
(if capacity > capacity_free then (capacity - capacity_free) else 0 endif) as capacity_used,
(if capacity > 0 and (capacity > capacity_free) then (capacity_free/capacity) * 100 else 0 endif) as percent_free,
(if capacity > 0 and (capacity > capacity_free) then (100-percent_free) else 0 endif) as percent_used,
isnull(DG.OVERHEAD_SIZE,0)/(1024.0*1024.0*1024.0) as capacity_overhead
from sahara.sahara_dg_dimension 
join HABDBSYNC.SF_VIEW_DISKGROUP_R as DG on DG.ID=sahara_dg_dimension.ccstor_unique_identifier
where sahara_dg_dimension.family!='OracleASM'; 


-- Calculation pertaining to the dg to lun
set @tablename = 'sahara.sahara_dg_to_lun';
set @attributenames = 'dg_ccstor_unique_identifier,lun_ccstor_unique_identifier,array_ccstor_unique_identifier';
insert into  sahara.sahara_dg_to_lun  
(
   dg_ccstor_unique_identifier,   
   lun_ccstor_unique_identifier,
   array_ccstor_unique_identifier
) on existing update
select DG.ccstor_unique_identifier, disk.ID, enc_ccstor_unique_identifier from sahara.sahara_dg_dimension dg 
join HABDBSYNC.SF_VIEW_DISK_R disk ON DG.ccstor_unique_identifier = disk.DISK_TO_DG
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = disk.ID;

if (@is_ASM=1) THEN
set @attributenames = 'ASM->dg_ccstor_unique_identifier,lun_ccstor_unique_identifier,array_ccstor_unique_identifier';
insert into  sahara.sahara_dg_to_lun  
(
   dg_ccstor_unique_identifier,   
   lun_ccstor_unique_identifier,
   array_ccstor_unique_identifier
) on existing update
select asm_dg.ccstor_unique_identifier, disk.ID, enc_ccstor_unique_identifier from sahara.sahara_dg_dimension asm_dg 
join habdbsync.sf_view_storage_Assignment_r dg_to_vol on asm_dg.ccstor_unique_identifier = dg_to_vol.dest_id
join sahara.sahara_volume_dimension vol on dg_to_vol.source_id=vol.ccstor_unique_identifier
join sahara.sahara_dg_dimension dg on vol.dg_unique_identifier = dg.ccstor_unique_identifier
join HABDBSYNC.SF_VIEW_DISK_R disk ON DG.ccstor_unique_identifier = disk.DISK_TO_DG
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = disk.ID
;
end if; /*for ASM*/

/****************************************************************/
/*RDM VMDK Disk->ESX LUN to be associated with Virtual Machine Host and DG_TO_LUN tuple to be deleted for RDM VMDK Disk. This is being handled for the case when array is not configured but guest is.*/
set @attributenames = 'ESX RDM LUN->fs_ccstor_unique_identifier,lun_ccstor_unique_identifier, array_ccstor_unique_identifier';
insert into  sahara.sahara_dg_to_lun  
(
   dg_ccstor_unique_identifier,   
   lun_ccstor_unique_identifier,
   array_ccstor_unique_identifier
) on existing update
select distinct dg.ccstor_unique_identifier, VMDK.object_id, enc_ccstor_unique_identifier
from sahara.sahara_dg_dimension dg 
join HABDBSYNC.SF_VIEW_DISK_R disk ON DG.ccstor_unique_identifier = disk.DISK_TO_DG
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on da.source_type in ('VDISK','DISK') and da.dest_id = disk.ID
join HABDBSYNC.VT_VIEW_VDISK_R VMDK on VMDK.id = da.source_id
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = VMDK.object_id
where VMDK.object_id is not null
and is_raw=1;


set @attributenames = 'Delete RDM LUNs';
delete from sahara.sahara_dg_to_lun
where lun_ccstor_unique_identifier in
(select da.dest_id as VMDK_OID
from HABDBSYNC.VT_VIEW_VDISK_R VMDK
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on VMDK.id = da.source_id and da.source_type in  ('VDISK','DISK')
where VMDK.object_id is not null
and VMDK_OID<>VMDK.object_id
and da.dest_id is not null
and is_raw=1
);
/****************************************************************/

set @tablename = 'sahara.sahara_dg_dimension';
set @attributenames = 'ccstor_unique_identifier,is_based_on_direct_attached';
/* setting the is direct attached flag */
update sahara.sahara_dg_dimension as dg set is_based_on_direct_attached = 1
where ccstor_unique_identifier in (
select dgtolun.dg_ccstor_unique_identifier
from 
sahara.sahara_dg_to_lun as dgtolun
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = dgtolun.lun_ccstor_unique_identifier
where lun.is_direct_attached=1)
;

set @attributenames = 'ccstor_unique_identifier,is_based_on_thin';
update sahara.sahara_dg_dimension as dg set is_based_on_thin = 1
where ccstor_unique_identifier in (
select dgtolun.dg_ccstor_unique_identifier
from 
sahara.sahara_dg_to_lun as dgtolun
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = dgtolun.lun_ccstor_unique_identifier
where lun.thin_status='ThinLun')
;

if (@is_ASM=1) THEN
set @attributenames = 'ccstor_unique_identifier,uses_vm_volumes';
update sahara.sahara_dg_dimension as dg set uses_vm_volumes = 1
where ccstor_unique_identifier in (
select dgtovol.dest_ID
from 
habdbsync.sf_view_storage_Assignment_r dgtovol
join sahara.sahara_volume_dimension vol on vol.ccstor_unique_identifier = dgtovol.source_ID
);

set @attributenames = 'ccstor_unique_identifier,is_used_by_clustered_DB';
update sahara.sahara_dg_dimension as dg set is_used_by_clustered_DB = 1
where ccstor_unique_identifier in (
select db_to_dg.storage_id from  
HABDBSYNC.SF_VIEW_DBED_DB_R DB
join HABDBSYNC.SF_VIEW_DB_TO_STORAGE_R db_to_dg on db.id=db_to_dg.db_id and db_to_dg.type='OracleASM'
where db.db_subtype = 1 AND db.TYPE != 'OracleASM'
);

set @attributenames = 'ccstor_unique_identifier,redundancy_type';
insert into sahara.sahara_dg_dimension 
(
ccstor_unique_identifier,
redundancy_type
) on existing update
select sahara_dg_dimension.ccstor_unique_identifier, (if dgp_red_type.prop_value = '0' then 'Unknown' else dgp_red_type.prop_value endif) as redundancy_type
from sahara.sahara_dg_dimension 
join habdbsync.SF_VIEW_DISKGROUP_PROPS_R dgp_red_type on sahara_dg_dimension.ccstor_unique_identifier=dgp_red_type.id and dgp_red_type.prop_name = 'REDUNDANCY_TYPE'
where sahara_dg_dimension.family='OracleASM';

set @tablename = 'sahara.sahara_dg_facts';

set @attributenames = 'ccstor_unique_identifier,capacity,capacity_free,capacity_used,percent_free,percent_used,capacity_overhead';
insert into sahara.sahara_dg_facts 
(
   ccstor_unique_identifier,
   capacity,   
   capacity_free,
   capacity_used,
   percent_free,
   percent_used,
   capacity_overhead
) on existing update
select dg.id, isnull(DG.TotalSize,0)/(1024.0*1024.0*1024.0) as capacity,
isnull(dgp_free.prop_value,0)/(1024.0*1024.0*1024.0) as capacity_free, 
isnull(dgp_used.prop_value,0)/(1024.0*1024.0*1024.0) as capacity_used,
(if capacity > 0 and (capacity > capacity_free) then (capacity_free/capacity) * 100 else 0 endif) as percent_free,
(if capacity > 0 and (capacity > capacity_used) then (capacity_used/capacity) * 100 else 0 endif) as percent_used,
isnull(DG.OVERHEAD_SIZE,0)/(1024.0*1024.0*1024.0) as capacity_overhead
from sahara.sahara_dg_dimension 
join HABDBSYNC.SF_VIEW_DISKGROUP_R as DG on DG.ID=sahara_dg_dimension.ccstor_unique_identifier
join habdbsync.SF_VIEW_DISKGROUP_PROPS_R dgp_free on dg.id=dgp_free.id and dgp_free.prop_name = 'ACTUAL_USABLE_FREE_SIZE'
join habdbsync.SF_VIEW_DISKGROUP_PROPS_R dgp_used on dg.id=dgp_used.id and dgp_used.prop_name = 'USABLE_USED_SIZE'
where sahara_dg_dimension.family='OracleASM'; 

set @attributenames = 'ccstor_unique_identifier, capacity_used_db,capacity_used_clustered_db';
insert into sahara.sahara_dg_facts 
(
   ccstor_unique_identifier,
   capacity_used_db,
   capacity_used_clustered_db,
   capacity_used_db_tbsp_used
) on existing update
select ccstor_unique_identifier, isnull(sum(distinctTblspc.fileCap),0)/(1024.0*1024.0*1024.0) as capacity,
isnull(sum(if is_rac = 1 then distinctTblspc.fileCap else 0 endif),0)/(1024.0*1024.0*1024.0) as capacity_RAC,
isnull(sum(distinctTblspc.usedsize + distinctTblspc.log_file_size),0)/(1024.0*1024.0*1024.0) as used_capacity
from
(
select 
distinct 
dg.ccstor_unique_identifier, 
file.id,
file.israw,
isnull(file.size,0) as fileCap,
isnull(ts.usedsize,0) as usedsize,
isnull((if file.tbs_id = '-' then file.size else 0 endif),0) as log_file_size,
(if isnull(DB.DB_SUBTYPE,0) = 1 then 1 else 0 endif) as is_rac
from     HABDBSYNC.SF_VIEW_DBED_DB_R DB
JOIN HABDBSYNC.SF_VIEW_DB_SYSMANAGER_R SYSMGR ON SYSMGR.ID=DB.DBSYS_ID
join HABDBSYNC.SF_VIEW_UI_DB_FILE file on file.db_id = db.id AND db.TYPE != 'OracleASM'
left outer join HABDBSYNC.SF_VIEW_DB_TABLESPACE_R ts on tbs_id = ts.id
join sahara.sahara_dg_dimension dg on file.file_storage_ID = dg.ccstor_unique_identifier
WHERE     
    SYSMGR.AUTH_REQD IS NULL OR SYSMGR.AUTH_REQD <> 1
) as distinctTblspc
group by distinctTblspc.ccstor_unique_identifier
;

set @attributenames = 'ccstor_unique_identifier, capacity_unknown';
insert into sahara.sahara_dg_facts 
(
   ccstor_unique_identifier,
   capacity_unknown
) on existing update
select dg.ccstor_unique_identifier, isnull(capacity_used-capacity_used_db,0) as capacity_unknown
from sahara.sahara_dg_dimension dg
join sahara.sahara_dg_facts df on dg.ccstor_unique_identifier=df.ccstor_unique_identifier
where dg.family='OracleASM'; 

set @tablename = 'sahara.sahara_dg_to_host';
set @attributenames = 'dg_unique_identifier,host_unique_identifier, capacity_used_db, capacity_used_db_tbsp_used';
insert into sahara.sahara_dg_to_host
(	dg_unique_identifier,
	host_unique_identifier,
	capacity_used_db,
	capacity_used_db_tbsp_used
) on existing update
select ccstor_unique_identifier, HOST_ID, isnull(sum(distinctTblspc.fileCap),0)/(1024.0*1024.0*1024.0) as capacity,
isnull(sum(distinctTblspc.usedsize + distinctTblspc.log_file_size),0)/(1024.0*1024.0*1024.0) as used_capacity
from
(
select 
distinct 
dg.ccstor_unique_identifier, 
file.id,
file.israw,
isnull(file.size,0) as fileCap,
isnull(ts.usedsize,0) as usedsize,
isnull((if file.tbs_id = '-' then file.size else 0 endif),0) as log_file_size,
(if isnull(DB.DB_SUBTYPE,0) = 1 then 1 else 0 endif) as is_rac,
HOSTLIST.HOST_ID
from     HABDBSYNC.SF_VIEW_DBED_DB_R DB
JOIN HABDBSYNC.SF_VIEW_DB_SYSMANAGER_R SYSMGR ON SYSMGR.ID=DB.DBSYS_ID
    JOIN HABDBSYNC.SF_VIEW_DBED_TO_HOST_R HOSTLIST ON HOSTLIST.DB_ID=DB.ID
join HABDBSYNC.SF_VIEW_UI_DB_FILE file on file.db_id = db.id AND db.TYPE != 'OracleASM'
left outer join HABDBSYNC.SF_VIEW_DB_TABLESPACE_R ts on tbs_id = ts.id
join sahara.sahara_dg_dimension dg on file.file_storage_ID = dg.ccstor_unique_identifier
WHERE     
    SYSMGR.AUTH_REQD IS NULL OR SYSMGR.AUTH_REQD <> 1
) as distinctTblspc
group by distinctTblspc.ccstor_unique_identifier, distinctTblspc.HOST_ID
;

end if; /*for ASM*/

update sahara.sahara_dg_dimension set batch_id = @batch_id ;
update sahara.sahara_dg_facts set batch_id = @batch_id ;

select count(1) into @noDims from sahara.sahara_dg_dimension;
select count(1) into @noFacts from sahara.sahara_dg_facts;
select count(1) into @noFactsdgtolun from sahara.sahara_dg_to_lun;

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_dg_data', 'I','completed dimension rollup -> sahara_dg_dimension: ' || @noDims || ' rows inserted';

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_dg_to_lun', 'I','completed facts rollup -> sahara_dg_to_lun: ' || @noFactsdgtolun|| ' rows inserted';

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_dg_data', 'I','completed facts rollup -> sahara_dg_facts: ' || @noFacts || ' rows inserted';


insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_dg_data', 'I','roll-up batch: ' || @batch_id || ', data set: sahara_dg_dimension/sahara_dg_facts -> Succeeded: ' ||datediff(second,@initialtime,GETDATE()) || ' secs';

commit;

EXCEPTION
    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 ) then
            SET @ERROR_COUNT=@ERROR_COUNT + 1;
            BEGIN
                insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message)
                select GETDATE(), @batch_id, 'sahara.sahara_dg_data', 'E', string (' Error failed at = ', @tablename+':'+@attributenames ) +
                                                                             string (': SQLCODE = ', @error ) +
                                                                             string (': ERRORMSG = ', @serror ) +
                                                                             string (': SQLSTATE = ', @sqlstate );
                message string (' Error failed at = ', @tablename+':'+@attributenames ) to LOG;
                message string ('          @error = ', @error ) to LOG;
                message string ('      @serrir()  = ', @serror ) to LOG;
                message string ('     @sqlstate() = ', @sqlstate ) to LOG;
            END;
        END IF;
        -- RESIGNAL;

END;
grant execute on sahara.sahara_dg_data to public;

commit;
