if exists(select 1 from sys.sysprocedure where proc_name = 'sahara_host_consumption_data' and user_name(creator) = 'sahara') then
   drop procedure sahara.sahara_host_consumption_data
end if;

CREATE PROCEDURE sahara."sahara_host_consumption_data" (@batch_id integer)
/* RESULT ( column_name, ... )
 */
BEGIN

/* Variables for logging purpose */
/* 
To record initial time when this stored procedure was called 
*/
DECLARE @initialtime timestamp; 

/* 
Total number of dimension this stored procedure is inserting 
*/
DECLARE @noDims bigint; 

/* 
Total number of facts this stored procedure is inserting 
*/
DECLARE @noFacts bigint; 
/*declare @tmpstr long varchar;
declare @hostlist long varchar;
DECLARE ccs52 int; */

/* Log detail error info */
declare @err_code integer;
declare @error    integer;
declare @serror         varchar(32767);
declare @extraerrorinfo long varchar;
declare @sqlstate       varchar(5);
declare @tablename      varchar(128);
declare @attributenames varchar(2048);
declare @count    integer;
declare @error_count    integer;

/* 
Store the current time 
*/
Set @initialtime=GETDATE();

/*Temp table for Assigned to VM Host To LUN*/
if exists(select 1 from sys.systable where table_name='tmp_ass_to_host' and creator=user_id('sahara')) then
   drop table sahara.tmp_ass_to_host
end if;

/*Temp table for host count per SP*/
if exists(select 1 from sys.systable where table_name='tmp_storage_pool_host_count' and creator=user_id('sahara')) then
   drop table sahara.tmp_storage_pool_host_count
end if;

/*Temp table for ESX count per VMDK*/
if exists(select 1 from sys.systable where table_name='tmp_VMDK_ESX_count' and creator=user_id('sahara')) then
   drop table sahara.tmp_VMDK_ESX_count
end if;

/*Temp table for GZ count per Exported Disk*/
if exists(select 1 from sys.systable where table_name='tmp_LUN_GZ_count' and creator=user_id('sahara')) then
   drop table sahara.tmp_LUN_GZ_count
end if;

/*Temp table for VM host count per VMDK*/
if exists(select 1 from sys.systable where table_name='tmp_VMDK_VM_count' and creator=user_id('sahara')) then
   drop table sahara.tmp_VMDK_VM_count
end if;

/*Temp table for VM host count per Exported Volume*/
if exists(select 1 from sys.systable where table_name='tmp_volume_VM_count' and creator=user_id('sahara')) then
   drop table sahara.tmp_volume_VM_count
end if;

/*Temp table for VM host count per Exported FS*/
if exists(select 1 from sys.systable where table_name='tmp_FS_VM_count' and creator=user_id('sahara')) then
   drop table sahara.tmp_FS_VM_count
end if;

/*Temp table for GZ host count per Exported FS*/
if exists(select 1 from sys.systable where table_name='tmp_FS_GZ_count' and creator=user_id('sahara')) then
   drop table sahara.tmp_FS_GZ_count
end if;

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_host_consumption_data', 'I','starting dimension/facts rollup -> sahara_lun_dimension/sahara_host_facts';

/*Temporarily store assigned tuple with type*/
create table sahara.tmp_ass_to_host  (
   host_unique_identifier    		varchar(256),  
   lun_unique_identifier    		varchar(256),
   thin_status VARCHAR(256),
   type int, /* 1 for VM, 2 for FS, 3 for DB, 4 for SP. */
   primary key (host_unique_identifier, lun_unique_identifier)
   );

create index lun_unique_identifier on sahara.tmp_ass_to_host (lun_unique_identifier);

grant select on sahara.tmp_ass_to_host to public; 

/*Temporarily store host count per SP*/
create table sahara.tmp_storage_pool_host_count  (
   pool_unique_identifier    		varchar(256),  
   host_count int,
   primary key (pool_unique_identifier)
   );
grant select on sahara.tmp_storage_pool_host_count to public; 


/*Temporarily store ESX count per VMDK*/
create table sahara.tmp_VMDK_ESX_count  (
   VMDK_unique_identifier    		varchar(256),  
   host_count int,
   primary key (VMDK_unique_identifier)
   );
grant select on sahara.tmp_VMDK_ESX_count to public;

/*Temporarily store GZ count per Exported LUN*/
create table sahara.tmp_LUN_GZ_count  (
   LUN_unique_identifier    		varchar(256),  
   host_count int,
   primary key (LUN_unique_identifier)
   );
grant select on sahara.tmp_LUN_GZ_count to public;

/*Temporarily store virt host count per VMDK*/
create table sahara.tmp_VMDK_VM_count  (
   VMDK_unique_identifier    		varchar(256),  
   host_count int,
   primary key (VMDK_unique_identifier)
   );
grant select on sahara.tmp_VMDK_VM_count to public;

/*Temporarily store virt host count per Volume*/
create table sahara.tmp_volume_VM_count  (
   vol_unique_identifier    		varchar(256),  
   host_count int,
   primary key (vol_unique_identifier)
   );
grant select on sahara.tmp_volume_VM_count to public;

/*Temporarily store virt host count per FS*/
create table sahara.tmp_FS_VM_count  (
   FS_unique_identifier    		varchar(256),  
   host_count int,
   primary key (FS_unique_identifier)
   );
grant select on sahara.tmp_FS_VM_count to public;

/*Temporarily store virt host count per FS*/
create table sahara.tmp_FS_GZ_count  (
   FS_unique_identifier    		varchar(256),  
   host_count int,
   primary key (FS_unique_identifier)
   );
grant select on sahara.tmp_FS_GZ_count to public;



/****************************************************************/
/*RDM VMDK->ESX LUN are now associated with Virtual Machine Host and related objects and LUN_TO_HOST and LUN_TO_<Object> tuples have to been deleted for RDM VMDK. Deleting RDM VMDK objects in LUN Dimension and LUN facts.*/

set @tablename='sahara.sahara_lun_dimension';
set @attributenames = 'Delete RDM LUNs';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

select isnull(da.dest_id, VMDK.id) as VMDK_OID into #tmp_table
from HABDBSYNC.VT_VIEW_VDISK_R VMDK
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on VMDK.id = da.source_id and da.source_type in ('VDISK','DISK')
where VMDK.object_id is not null and VMDK_OID<>VMDK.object_id
and is_raw=1;

      delete from sahara.sahara_lun_dimension
      where ccstor_unique_identifier in (select VMDK_OID from #tmp_table);

set @tablename='sahara.sahara_lun_facts';
set @attributenames = 'Delete RDM LUNs';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

      delete from sahara.sahara_lun_facts
      where ccstor_unique_identifier in (select VMDK_OID from #tmp_table);

/****************************************************************/

/* sahara_lun_dimension:is_assigned:assigned */
/* lun that is assigned to VM, FS or DB. DB case is not yet available in VOM.*/
set @tablename='sahara.sahara_lun_dimension';
set @attributenames='is_assigned';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_lun_dimension (ccstor_unique_identifier, is_assigned) on existing update
/*Imported DG*/
select lun.ccstor_unique_identifier as lun_id, 
     1  as is_assigned
from sahara.sahara_lun_dimension lun
join sahara.sahara_dg_to_lun dg_lun on lun.ccstor_unique_identifier = dg_lun.lun_ccstor_unique_identifier
join sahara.sahara.sahara_dg_to_host dg_host on dg_lun.dg_ccstor_unique_identifier = dg_host.dg_unique_identifier
and dg_host.state = 'Imported'
UNION
select lun.ccstor_unique_identifier as lun_id, 
     1  as is_assigned
from sahara.sahara_lun_dimension  lun
join sahara.sahara_fs_to_lun fs_lun on lun.ccstor_unique_identifier = fs_lun.lun_ccstor_unique_identifier
UNION /*Assigned to SP*/
select
lun.ccstor_unique_identifier as lun_id, 
     1  as is_assigned
from sahara.sahara_lun_dimension  lun
join habdbsync.SF_VIEW_DISK_R on lun.ccstor_unique_identifier = SF_VIEW_DISK_R.id
where SF_VIEW_DISK_R.SP_ID is not null
and (sub_type is null OR sub_type<>'RAW') /* RDM LUN is not assigned unless used by VM*/
;


set @tablename = 'sahara.tmp_ass_to_host';
set @attributenames = 'host_unique_identifier,lun_unique_identifier,thin_status, type';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Assigned to VMs*/
insert into sahara.tmp_ass_to_host(host_unique_identifier,lun_unique_identifier,thin_status, type)
on existing update
select all_luns.host_oid, all_luns.lun_oid, isnull(luns.thin_status,'') as thin_status, 1
from sahara.sahara_all_luns_to_host all_luns
join sahara.sahara_lun_dimension as luns
	on luns.ccstor_unique_identifier = all_luns.lun_oid
join sahara.sahara_dg_to_lun dg_lun on all_luns.lun_oid = dg_lun.lun_ccstor_unique_identifier
join sahara.sahara.sahara_dg_to_host dg_host on dg_lun.dg_ccstor_unique_identifier = dg_host.dg_unique_identifier
and dg_host.state = 'Imported'
where luns.is_secondary = 0 and luns.is_assigned = 1
and all_luns.host_oid = dg_host.host_unique_identifier
;

/*Assigned to VM host Count*/
set @tablename = 'sahara.sahara_lun_dimension';
set @attributenames = 'ccstor_unique_identifier,no_of_Ass_To_VM_hosts';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, 
                        no_of_Ass_To_VM_hosts)
on existing update
select all_luns.lun_unique_identifier, count(all_luns.host_unique_identifier) as cnt 
from sahara.tmp_ass_to_host all_luns
join sahara.sahara_lun_dimension as luns
	on luns.ccstor_unique_identifier = all_luns.lun_unique_identifier
group by all_luns.lun_unique_identifier
;

set @tablename = 'sahara.tmp_ass_to_host';
set @attributenames = 'host_unique_identifier,lun_unique_identifier,thin_status, type';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Directly Assigned to FSs*/
insert into sahara.tmp_ass_to_host(host_unique_identifier,lun_unique_identifier,thin_status, type)
on existing update
select all_luns.host_oid, all_luns.lun_oid, isnull(luns.thin_status,'') as thin_status, 2
from sahara.sahara_all_luns_to_host all_luns
join sahara.sahara_lun_dimension as luns
	on luns.ccstor_unique_identifier = all_luns.lun_oid
join sahara.sahara_fs_to_lun fs_lun on all_luns.lun_oid = fs_lun.lun_ccstor_unique_identifier
join sahara.sahara_fs_to_host fs_host on fs_lun.fs_ccstor_unique_identifier = fs_host.fs_unique_identifier
left outer join sahara.tmp_ass_to_host assToDG on all_luns.lun_oid = assToDG.lun_unique_identifier /*Till now contains ass to VM*/
where luns.is_secondary = 0  and luns.is_assigned = 1
and all_luns.host_oid = fs_host.host_unique_identifier
and assToDG.lun_unique_identifier is null /*Directly Assigned to FS if LUN is not assigned to VM*/
;

/*Directly Assigned to FS host Count*/
set @tablename = 'sahara.sahara_lun_dimension';
set @attributenames = 'ccstor_unique_identifier,no_of_Ass_To_FS_hosts';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, 
                        no_of_Ass_To_FS_hosts)
on existing update
select all_luns.lun_unique_identifier, count(all_luns.host_unique_identifier) as cnt 
from sahara.tmp_ass_to_host all_luns
join sahara.sahara_lun_dimension as luns
	on luns.ccstor_unique_identifier = all_luns.lun_unique_identifier
where all_luns.type = 2
group by all_luns.lun_unique_identifier;


set @tablename = 'sahara.tmp_ass_to_host';
set @attributenames = 'host_unique_identifier,lun_unique_identifier,thin_status, type';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Assigned to SPs*/
insert into sahara.tmp_ass_to_host(host_unique_identifier,lun_unique_identifier,thin_status, type)
on existing update
select
all_luns.host_oid, all_luns.lun_oid, isnull(lun.thin_status,'') as thin_status, 4
from sahara.sahara_lun_dimension  lun
join sahara.sahara_all_luns_to_host all_luns on all_luns.lun_oid = lun.ccstor_unique_identifier
join habdbsync.SF_VIEW_DISK_R on lun.ccstor_unique_identifier = SF_VIEW_DISK_R.id
join habdbsync.VT_VIEW_SP_TO_HOST_R sp on SF_VIEW_DISK_R.sp_id = sp.sp_id
where SF_VIEW_DISK_R.SP_ID is not null
and lun.is_secondary = 0 and lun.is_assigned = 1
and sp.Host_ID=all_luns.host_oid
and (lun.sub_type is null OR lun.sub_type<>'RAW')
;


set @tablename = 'sahara.sahara_host_facts';
set @attributenames = 'capacity_assigned_VM,capacity_assigned_VM_consumed,capacity_assigned_VM_distributed, capacity_assigned_VM_consumed_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_assigned_VM, capacity_assigned_VM_consumed, capacity_assigned_VM_distributed, capacity_assigned_VM_consumed_distributed)
on existing update
select ass.host_unique_identifier, sum(ass.lun_size), sum(ass.consumed_lun_size),
sum(ass.lun_size/no_of_Ass_To_VM_hosts)  as capacity_assigned_VM_distributed, 
sum(ass.consumed_lun_size/no_of_Ass_To_VM_hosts) as capacity_assigned_VM_consumed_distributed 
from
(select all_luns.host_unique_identifier, all_luns.lun_unique_identifier, lunf.lun_size, 
(if all_luns.thin_status = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif) as consumed_lun_size,
no_of_Ass_To_VM_hosts
from sahara.tmp_ass_to_host all_luns
join sahara.sahara_lun_dimension as luns
	on luns.ccstor_unique_identifier = all_luns.lun_unique_identifier
join sahara.sahara_lun_facts lunf on luns.ccstor_unique_identifier=lunf.ccstor_unique_identifier
where all_luns.type = 1
) as ass
group by ass.host_unique_identifier;

set @attributenames = 'capacity_directly_assigned_FS,capacity_directly_assigned_FS_consumed,capacity_directly_assigned_FS_distributed,capacity_directly_assigned_FS_consumed_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_directly_assigned_FS, capacity_directly_assigned_FS_consumed,capacity_directly_assigned_FS_distributed,capacity_directly_assigned_FS_consumed_distributed)
on existing update
/*select ass.host_oid, sum(ass.lun_size),sum(ass.consumed_lun_size) from
(
select distinct all_luns.host_oid, all_luns.lun_oid, lunf.lun_size, (if luns.thin_status = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif) as consumed_lun_size
from sahara.sahara_all_luns_to_host all_luns
join sahara.sahara_lun_dimension as luns
	on luns.ccstor_unique_identifier = all_luns.lun_oid
join sahara.sahara_lun_facts lunf on lunf.ccstor_unique_identifier = all_luns.lun_oid
join sahara.sahara_fs_to_lun fs_lun on all_luns.lun_oid = fs_lun.lun_ccstor_unique_identifier
join sahara.sahara_fs_to_host fs_host on fs_lun.fs_ccstor_unique_identifier = fs_host.fs_unique_identifier
left outer join sahara.sahara_dg_to_lun dg_lun on all_luns.lun_oid = dg_lun.lun_ccstor_unique_identifier
left outer join sahara.sahara.sahara_dg_to_host dg_host on dg_lun.dg_ccstor_unique_identifier = dg_host.dg_unique_identifier
and dg_host.state = 'Imported'
where luns.is_secondary = 0  and luns.is_assigned = 1
and all_luns.host_oid = fs_host.host_unique_identifier
and dg_host.dg_unique_identifier is null
) as ass
group by ass.host_oid;  */
select ass.host_unique_identifier, sum(ass.lun_size), sum(ass.consumed_lun_size),
sum(ass.lun_size/no_of_Ass_To_FS_hosts)  as capacity_assigned_FS_distributed, 
sum(ass.consumed_lun_size/no_of_Ass_To_FS_hosts) as capacity_assigned_FS_consumed_distributed 
from
(select all_luns.host_unique_identifier, all_luns.lun_unique_identifier, lunf.lun_size, 
(if all_luns.thin_status = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif) as consumed_lun_size,
no_of_Ass_To_FS_hosts
from sahara.tmp_ass_to_host all_luns
join sahara.sahara_lun_dimension as luns
	on luns.ccstor_unique_identifier = all_luns.lun_unique_identifier
join sahara.sahara_lun_facts lunf on luns.ccstor_unique_identifier=lunf.ccstor_unique_identifier
where all_luns.type = 2
) as ass
group by ass.host_unique_identifier;

set @attributenames = 'capacity_directly_assigned_DB, capacity_directly_assigned_DB_consumed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

/*
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_directly_assigned_DB)
on existing update
*/
/*Not supported in VOM 4.0*/

set @attributenames = 'capacity_assigned_SP, capacity_assigned_SP_consumed, capacity_assigned_SP_distributed, capacity_assigned_SP_consumed_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_assigned_SP, capacity_assigned_SP_consumed, capacity_assigned_SP_distributed, capacity_assigned_SP_consumed_distributed)
on existing update
select ass.host_oid, sum(ass.lun_size), sum(ass.consumed_lun_size),
sum(if no_of_allocated_hosts = 0 then 0 else ass.lun_size/no_of_allocated_hosts endif)  as capacity_assigned_SP_distributed, 
sum(if no_of_allocated_hosts = 0 then 0 else ass.consumed_lun_size/no_of_allocated_hosts endif) as capacity_assigned_SP_consumed_distributed 
from
(
select
all_luns.host_oid, all_luns.lun_oid, lunf.lun_size, lunf.consumed_lun_size, lunf.no_of_allocated_hosts
from sahara.sahara_lun_dimension  lun
join sahara.sahara_all_luns_to_host all_luns on all_luns.lun_oid = lun.ccstor_unique_identifier
join sahara.sahara_lun_facts lunf on lun.ccstor_unique_identifier=lunf.ccstor_unique_identifier
join habdbsync.SF_VIEW_DISK_R on lun.ccstor_unique_identifier = SF_VIEW_DISK_R.id
where SF_VIEW_DISK_R.SP_ID is not null
and lun.is_secondary = 0 and lun.is_assigned = 1
and (lun.sub_type is null OR lun.sub_type<>'RAW'))
as ass
group by ass.host_oid
;

set @attributenames = 'capacity_claimed_assigned_to_other_hosts,capacity_claimed_assigned_to_other_hosts_consumed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_claimed_assigned_to_other_hosts,
                        capacity_claimed_assigned_to_other_hosts_consumed)
on existing update
select ass.host_oid, sum(ass.lun_size), sum(ass.consumed_lun_size) from
(
select Ass_To_Other.host_oid, Ass_To_Other.lun_oid, lunf.lun_size, (if isnull(luns.thin_status,'') = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif) as consumed_lun_size
from
(/*Host To LUN*/
select 
all_luns.host_oid, all_luns.lun_oid
from sahara.sahara_all_luns_to_host all_luns
EXCEPT    /*Assigned Host to LUN*/
    (select host_unique_identifier, lun_unique_identifier
        from sahara.tmp_ass_to_host 
    )
) as Ass_To_Other 
join sahara.sahara_lun_dimension as luns
on luns.ccstor_unique_identifier = Ass_To_Other.lun_oid
join sahara.sahara_lun_facts lunf on lunf.ccstor_unique_identifier = Ass_To_Other.lun_oid
where luns.is_secondary = 0 and luns.is_assigned = 1
) as ass
group by ass.host_oid;  

set @attributenames = 'capacity_unknown_assignment, capacity_unknown_assignment_consumed, capacity_unknown_assignment_distributed, capacity_unknown_assignment_consumed_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_unknown_assignment, capacity_unknown_assignment_consumed, capacity_unknown_assignment_distributed, capacity_unknown_assignment_consumed_distributed)
on existing update
select ass.host_oid, sum(ass.lun_size), sum(ass.consumed_lun_size), sum(ass.lun_size/no_of_claimed_hosts),
sum(ass.consumed_lun_size/no_of_claimed_hosts) from
(
select
all_luns.host_oid, all_luns.lun_oid, lun_size, (if isnull(luns.thin_status,'') = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif) as consumed_lun_size, (if no_of_claimed_hosts = 0 then 1 else no_of_claimed_hosts endif) as no_of_claimed_hosts /*Defensive check*/
from sahara.sahara_all_luns_to_host all_luns
join sahara.sahara_lun_dimension as luns
on luns.ccstor_unique_identifier = all_luns.lun_oid
join sahara.sahara_lun_facts lunf on lunf.ccstor_unique_identifier = all_luns.lun_oid
where luns.is_secondary = 0 and luns.is_assigned = 0 and luns.is_claimed = 2
) as ass
group by ass.host_oid;  

set @tablename = 'sahara.tmp_storage_pool_host_count';
set @attributenames = 'pool_unique_identifier,host_count';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Host Count Per Pool*/
insert into sahara.tmp_storage_pool_host_count(pool_unique_identifier,host_count)
on existing update
select sptohost.SP_ID, count(sptohost.Host_ID) as host_count
from
   HABDBSYNC.VT_VIEW_SP_TO_HOST_R sptohost
group by sptohost.SP_ID
;

set @tablename = 'sahara.tmp_VMDK_ESX_count';
set @attributenames = 'VMDK_unique_identifier,host_count';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Host Count Per VMDK */
insert into sahara.tmp_VMDK_ESX_count(VMDK_unique_identifier,host_count)
on existing update
select vdisk.id, count(distinct sph.host_id) as host_count from
HABDBSYNC.VT_VIEW_SP_TO_HOST_R sph
JOIN HABDBSYNC.VT_VIEW_STORAGE_POOL_R sp  on sp.ID = sph.SP_ID
JOIN HABDBSYNC.VT_VIEW_VDISK_R vdisk ON vdisk.SP_ID = sp.ID
left outer join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da ON vdisk.ID = da.SOURCE_ID
where da.SOURCE_ID is null
group by vdisk.id
;

set @tablename = 'sahara.tmp_LUN_GZ_count';
set @attributenames = 'LUN_unique_identifier,host_count';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Host Count Per LUN */
insert into sahara.tmp_LUN_GZ_count(LUN_unique_identifier,host_count)
on existing update
select obj.id, count(distinct da.source_host_id) as host_count 
from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
     inner join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and da.source_type = 'DISK'
group by obj.id
;

set @tablename = 'sahara.tmp_VMDK_VM_count';
set @attributenames = 'VMDK_unique_identifier,host_count';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Host Count Per VMDK */
insert into sahara.tmp_VMDK_VM_count(VMDK_unique_identifier,host_count)
on existing update
select obj.id, count(distinct da.dest_host_id) as host_count 
from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
     inner join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and da.source_type in ('VDISK','DISK','FILE','VOLUME')
group by obj.id
;

set @tablename = 'sahara.tmp_volume_VM_count';
set @attributenames = 'vol_unique_identifier,host_count';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Host Count Per volume */
insert into sahara.tmp_volume_VM_count(vol_unique_identifier,host_count)
on existing update
select obj.id, count(distinct da.dest_host_id) as host_count 
from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
     inner join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and da.source_type = 'VOLUME'
group by obj.id
;

set @tablename = 'sahara.tmp_FS_VM_count';
set @attributenames = 'FS_unique_identifier,host_count';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Host Count Per FS */
insert into sahara.tmp_FS_VM_count(FS_unique_identifier,host_count)
on existing update
select obj.id, count(distinct da.dest_host_id) as host_count 
from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
     inner join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and da.source_type = 'FILESYSTEM'
group by obj.id
;

set @tablename = 'sahara.tmp_FS_GZ_count';
set @attributenames = 'FS_unique_identifier,host_count';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
/*Host Count Per FS */
insert into sahara.tmp_FS_GZ_count(FS_unique_identifier,host_count)
on existing update
select obj.id, count(distinct da.source_host_id) as host_count 
from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
     inner join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and da.source_type = 'FILESYSTEM'
group by obj.id
;



set @tablename = 'sahara.sahara_host_facts';
set @attributenames = 'capacity_diskgroup, capacity_diskgroup_available, capacity_diskgroup_used, capacity_diskgroup_distributed, capacity_diskgroup_available_distributed, capacity_diskgroup_used_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_diskgroup, capacity_diskgroup_available, capacity_diskgroup_used,
			capacity_diskgroup_distributed, capacity_diskgroup_available_distributed, capacity_diskgroup_used_distributed)
on existing update
select hostd.host_unique_identifier, sum(capacity), sum(capacity_free), sum(capacity_used), sum(capacity/no_of_hosts), sum(capacity_free/no_of_hosts), sum(capacity_used/no_of_hosts) from 
sahara.sahara_dg_to_host hostd
join sahara.sahara_dg_dimension dgd on dgd.ccstor_unique_identifier = hostd.dg_unique_identifier
join sahara.sahara_dg_facts dgf on dgd.ccstor_unique_identifier = dgf.ccstor_unique_identifier
where dgd.family!='OracleASM'
group by hostd.host_unique_identifier
;

set @attributenames = 'capacity_volume, capacity_unused_volume, capacity_volume_distributed, capacity_unused_volume_distributed,capacity_exported_Volumes,capacity_exported_Volumes_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_volume, capacity_unused_volume, capacity_volume_distributed, capacity_unused_volume_distributed,capacity_exported_Volumes,capacity_exported_Volumes_distributed)
on existing update
select hostd.host_unique_identifier, sum(capacity), sum(if vold.is_used_by_filesystem =0 and vold.is_used_by_application = 0 then capacity else 0 endif), sum(capacity/no_of_hosts), sum(if vold.is_used_by_filesystem =0 and vold.is_used_by_application = 0 then capacity/no_of_hosts else 0 endif)as capacity_unused_volume_distributed,sum(if vold.is_exported=1 and host.is_virtual=0 then capacity else 0 endif) as capacity_exported_Volumes,sum(if vold.is_exported=1 and host.is_virtual=0 then capacity/no_of_hosts else 0 endif) as capacity_exported_Volumes_distributed 
from 
sahara.sahara_host_dimension host
join sahara.sahara_volume_to_host hostd on  host.ccstor_unique_identifier = hostd.host_unique_identifier
join sahara.sahara_volume_dimension vold on vold.ccstor_unique_identifier = hostd.volume_unique_identifier
join sahara.sahara_volume_facts volf on vold.ccstor_unique_identifier = volf.ccstor_unique_identifier
where vold.is_segment=0 /*Ignore segment volumes*/
and vold.type <> 'Snapshot' /*= 'Data' */ /*Ignore snapshots and clones*/
group by hostd.host_unique_identifier
;

set @attributenames = 'capacity_volume_manager_overhead, capacity_volume_manager_overhead_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, capacity_volume_manager_overhead, capacity_volume_manager_overhead_distributed)
on existing update
select all_overheads.host_unique_identifier, sum (overhead) /(1024.0*1024.0*1024.0) as overhead,
sum(all_overheads.overhead_distributed) /(1024.0*1024.0*1024.0) as overhead_distributed
from 
(
/*ZFS*/
select hostd.host_unique_identifier, isnull(sum(OVERHEAD_SIZE),0) as overhead,
isnull(sum(if no_of_hosts>0 then OVERHEAD_SIZE/no_of_hosts else 0 endif),0) as overhead_distributed
from sahara.sahara_dg_to_host hostd
join sahara.sahara_dg_dimension dgd on dgd.ccstor_unique_identifier = hostd.dg_unique_identifier
join HABDBSYNC.SF_VIEW_DISKGROUP_R dg on dg.id = dgd.ccstor_unique_identifier
join sahara.sahara_dg_facts dgf on dgd.ccstor_unique_identifier = dgf.ccstor_unique_identifier
where dgd.family = 'ZFS'
group by hostd.host_unique_identifier
/*Non-ZFS*/
UNION ALL
select hostd.ccstor_unique_identifier, 
sum(
IF vmvol.vol_type like '%mirror%' 
THEN (vmvol.vol_size * (isnull(vol_nredundancy,1)-1))
ELSE 
    (IF vmvol.vol_type = 'raid5' and isnull(vmvol.VOL_NCOL,0)!=0
    THEN
    (1.0/(vmvol.VOL_NCOL+1)) * vmvol.vol_size 
    ELSE 0 
    ENDIF)
ENDIF) as Overhead_Capacity,
sum(
IF vmvol.vol_type like '%mirror%' 
THEN (vmvol.vol_size * (isnull(vol_nredundancy,1)-1))/no_of_hosts
ELSE 
    (IF vmvol.vol_type = 'raid5' and isnull(vmvol.VOL_NCOL,0)!=0
    THEN
    ((1.0/(vmvol.VOL_NCOL+1)) * vmvol.vol_size)/no_of_hosts
    ELSE 0 
    ENDIF)
ENDIF) as Overhead_Capacity_distributed
from 
sahara.sahara_volume_dimension vold
join sahara.sahara_volume_facts volf on vold.ccstor_unique_identifier = volf.ccstor_unique_identifier
join HABDBSYNC.SF_VIEW_HOST_VOLUME vmvol on vmvol.vol_id = vold.ccstor_unique_identifier
join sahara.sahara.sahara_host_dimension hostd on vmvol.host_id = hostd.ccstor_unique_identifier
where vold.vendor!='ZFS'
group by hostd.ccstor_unique_identifier) as all_overheads
group by all_overheads.host_unique_identifier
;

-- If the result is negative, reset to 0 and write to the sahara_log;
select count(*) into @count
from sahara.sahara_host_facts
where capacity_volume_manager_overhead < 0;

if ( @count > 0 ) then
      select list(ccstor_unique_identifier) into @extraerrorinfo
      from sahara.sahara_host_facts
      where capacity_volume_manager_overhead < 0;

      insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message) 
      select GETDATE(), @batch_id, 'sahara.sahara_host_consumption_data', 'W', string (' Error failed at = ', @tablename+':'+@attributenames ) + 
                                                                    string (': count_of negative occurence = ', @count );

      message string (' Warning, failed at = ', @tablename+':'+@attributenames );
      message string (' List of hosts  = ', @extraerrorinfo );
      message string (': count_of negative occurence = ', @count );

      update sahara.sahara_host_facts set capacity_volume_manager_overhead = 0.0
      from sahara.sahara_host_facts
      where capacity_volume_manager_overhead < 0;

      update sahara.sahara_host_facts set capacity_volume_manager_overhead_distributed = 0.0
      from sahara.sahara_host_facts
      where capacity_volume_manager_overhead_distributed < 0;
end if;

set @attributenames = 'capacity_filesystem, capacity_filesystem_available, capacity_filesystem_used, capacity_filesystem_on_volumes, capacity_filesystem_on_non_volumes, capacity_filesystem_used_by_apps, capacity_filesystem_used_by_non_apps, capacity_filesystem_distributed, capacity_filesystem_available_distributed, capacity_filesystem_used_distributed, capacity_filesystem_on_volumes_distributed, capacity_filesystem_on_non_volumes_distributed, capacity_filesystem_used_by_apps_distributed, capacity_filesystem_used_by_non_apps_distributed,capacity_exported_FSs,capacity_exported_FSs_distributed,capacity_unexported_filesystem_used,capacity_unexported_filesystem_used_by_non_apps,capacity_filesystem_ignore_size_zero,capacity_filesystem_used_ignore_size_zero,capacity_unexported_filesystem_used_ignore_size_zero,capacity_filesystem_available_ignore_size_zero,capacity_filesystem_used_by_apps_ignore_size_zero,capacity_filesystem_used_by_non_apps_ignore_size_zero,capacity_unexported_filesystem_used_by_non_apps_ignore_size_zero,capacity_exported_FSs_ignore_size_zero
';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_filesystem, capacity_filesystem_available, capacity_filesystem_used, capacity_filesystem_on_volumes, capacity_filesystem_on_non_volumes, capacity_filesystem_used_by_apps, capacity_filesystem_used_by_non_apps, capacity_filesystem_distributed, capacity_filesystem_available_distributed, capacity_filesystem_used_distributed, capacity_filesystem_on_volumes_distributed, capacity_filesystem_on_non_volumes_distributed, capacity_filesystem_used_by_apps_distributed, capacity_filesystem_used_by_non_apps_distributed, capacity_exported_FSs,capacity_exported_FSs_distributed, 
			capacity_unexported_filesystem_used,			capacity_unexported_filesystem_used_by_non_apps,capacity_filesystem_ignore_size_zero,capacity_filesystem_used_ignore_size_zero,capacity_unexported_filesystem_used_ignore_size_zero,capacity_filesystem_available_ignore_size_zero,capacity_filesystem_used_by_apps_ignore_size_zero,capacity_filesystem_used_by_non_apps_ignore_size_zero,capacity_unexported_filesystem_used_by_non_apps_ignore_size_zero,capacity_exported_FSs_ignore_size_zero)
on existing update
select hostd.host_unique_identifier, sum(capacity), sum(capacity_free), sum(capacity_used), 
sum(if is_created_from_volume = 1 then capacity else 0 endif), 
sum(if is_created_from_volume = 0 then capacity else 0 endif),
sum(capacity_used_by_apps), sum(used_by_non_apps), sum(capacity/no_of_hosts), sum(capacity_free/no_of_hosts), sum(capacity_used/no_of_hosts), 
sum(if is_created_from_volume = 1 then capacity/no_of_hosts else 0 endif), 
sum(if is_created_from_volume = 0 then capacity/no_of_hosts else 0 endif),
sum(capacity_used_by_apps/no_of_hosts), sum(used_by_non_apps/no_of_hosts),
sum(if fsd.is_exported=1 and host.is_virtual=0 then capacity else 0 endif) as capacity_exported_FSs,
sum(if fsd.is_exported=1 and host.is_virtual=0 then capacity/isnull(cnt.host_count,1) else 0 endif) as  capacity_exported_FSs_distributed,
sum(if is_server_virtualized = 1 and fsd.is_exported = 1 then 0 else capacity_used endif) as capacity_unexported_filesystem_used,
sum(if is_server_virtualized = 1 and fsd.is_exported = 1 then 0 else used_by_non_apps endif) as capacity_unexported_filesystem_used_by_non_apps,
/***************Workaround for size 0 LUN issue in percentage calaculation******************/
sum(if is_based_on_size_zero_LUN=0 then capacity else 0 endif) as capacity_filesystem_ignore_size_zero,
sum(if is_based_on_size_zero_LUN=0 then capacity_used else 0 endif) as capacity_filesystem_used_ignore_size_zero,
sum(if (is_server_virtualized = 1 and fsd.is_exported = 1) OR is_based_on_size_zero_LUN=1 then 0 else capacity endif) as capacity_unexported_filesystem_used_ignore_size_zero,
sum(if is_based_on_size_zero_LUN=0 then capacity_free else 0 endif) as capacity_filesystem_available_ignore_size_zero,
sum(if is_based_on_size_zero_LUN=0 then capacity_used_by_apps else 0 endif) as capacity_filesystem_used_by_apps_ignore_size_zero,
sum(if is_based_on_size_zero_LUN=0 then used_by_non_apps else 0 endif) as capacity_filesystem_used_by_non_apps_ignore_size_zero,
sum(if (is_server_virtualized = 1 and fsd.is_exported = 1) OR is_based_on_size_zero_LUN=1 then 0 else capacity endif) as capacity_unexported_filesystem_used_by_non_apps_ignore_size_zero,
sum(if fsd.is_exported=1 and host.is_virtual=0 and is_based_on_size_zero_LUN=0 then capacity else 0 endif) as capacity_exported_FSs_ignore_size_zero
from 
sahara.sahara_host_dimension host
join sahara.sahara_fs_to_host hostd on  host.ccstor_unique_identifier = hostd.host_unique_identifier
join sahara.sahara_fs_dimension fsd on fsd.ccstor_unique_identifier = hostd.fs_unique_identifier
join sahara.sahara_fs_facts fsf on fsd.ccstor_unique_identifier = fsf.ccstor_unique_identifier
left outer join sahara.tmp_FS_GZ_count cnt on fsd.ccstor_unique_identifier=cnt.FS_unique_identifier
where fsd.is_child = 0 /*Ignore ZFS Child FSs*/
group by hostd.host_unique_identifier
;

set @attributenames = 'DB Capacities';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_database, capacity_database_available, capacity_database_used, capacity_apps_on_raw_volume, capacity_apps_on_filesystem, capacity_database_distributed, capacity_database_available_distributed, capacity_database_used_distributed, capacity_apps_on_raw_volume_distributed, capacity_apps_on_filesystem_distributed
			/*,capacity_dbvm_db_used, capacity_dbvm_db_used_by_other, capacity_dbvm_db_used_distributed*/
			)
on existing update
select hostd.host_unique_identifier, sum(capacity), sum(capacity_avaialble), sum(capacity_used), sum(capacity_from_volume), sum(capacity_from_file_system), sum(capacity/no_of_hosts), sum(capacity_avaialble/no_of_hosts), sum(capacity_used/no_of_hosts), sum(capacity_from_volume/no_of_hosts), sum(capacity_from_file_system/no_of_hosts)
/*, sum(dbf.used_from_dbvm), sum(dbf.used_from_dbvm-hostd.used_from_dbvm),
sum(dbf.used_from_dbvm/no_of_hosts)*/
from 
sahara.sahara_database_to_host hostd
join sahara.sahara_database_facts dbf on hostd.database_unique_identifier = dbf.ccstor_unique_identifier
group by hostd.host_unique_identifier
;

/* Calculate the capacity allocated to VMs through the storage pool and RDM Disks; to Zones through FS*/


set @attributenames = 'capacity_exported_LUNs,capacity_exported_LUNs_consumed,capacity_exported_LUNs_distributed,capacity_exported_LUNs_consumed_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier,capacity_exported_LUNs,capacity_exported_LUNs_consumed,capacity_exported_LUNs_distributed,capacity_exported_LUNs_consumed_distributed)
on existing update
select host.host_oid, 
sum(if lund.is_exported=1 then lunf.lun_size else 0 endif) as capacity_exported_LUNs,
sum(if lund.is_exported=1 then (if lund.thin_status = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif) else 0 endif) as capacity_exported_LUNs_consumed,
sum(if lund.is_exported=1 then lunf.lun_size/isnull(tmp_LUN_GZ_count.host_count,1) else 0 endif) as capacity_exported_LUNs_distributed,
sum(if lund.is_exported=1 then (if lund.thin_status = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif)/isnull(tmp_LUN_GZ_count.host_count,1) else 0 endif) as capacity_exported_LUNs_consumed_distributed 
from sahara.sahara_all_luns_to_host host
join sahara.sahara_lun_facts lunf on host.lun_oid = lunf.ccstor_unique_identifier
join sahara.sahara_lun_dimension lund on lund.ccstor_unique_identifier = lunf.ccstor_unique_identifier
join sahara.tmp_LUN_GZ_count on tmp_LUN_GZ_count.LUN_unique_identifier = lund.ccstor_unique_identifier
join sahara.sahara_host_dimension h on host.host_oid = h.ccstor_unique_identifier
where is_server_virtualized=1 or control_guest='T' /*This capacity is not applicable on guests*/
group by host.host_oid
;

set @attributenames = 'dtmapd_allocated_capacity,VirtualPoolAllocatedCapacity, capacity_FileStorage_allocated, capacity_FileStorage_claimed, capacity_allocated, capacity_total_allocated,
dtmapd_allocated_capacity_distributed,VirtualPoolAllocatedCapacity_distributed, capacity_FileStorage_allocated_distributed, capacity_FileStorage_claimed_distributed, capacity_allocated_distributed, capacity_total_allocated_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, dtmapd_allocated_capacity ,VirtualPoolAllocatedCapacity, capacity_FileStorage_allocated, capacity_FileStorage_claimed, capacity_allocated, capacity_total_allocated,dtmapd_allocated_capacity_distributed,VirtualPoolAllocatedCapacity_distributed, capacity_FileStorage_allocated_distributed, capacity_FileStorage_claimed_distributed, capacity_allocated_distributed, capacity_total_allocated_distributed,capacity_allocated_consumed,capacity_total_allocated_consumed,capacity_allocated_consumed_distributed,capacity_total_allocated_consumed_distributed)
on existing update


select hd.ccstor_unique_identifier,
    isnull(VirtualizationLayer.dtmapd_allocated_capacity,0)/(1024.0*1024.0*1024.0), /*RDM capacity in VMWare*/
    isnull(VirtualizationLayer.VirtualPoolAllocatedCapacity,0)/(1024.0*1024.0*1024.0), /*Non RDM virtualized Block storage.*/
isnull(VirtualizationLayer.capacity_FileStorage_allocated,0)/(1024.0*1024.0*1024.0) as FileStorage_allocated,
FileStorage_allocated as capacity_FileStorage_claimed,
/*Block Allocated.
This will overwrite capacity discovered from host side. Claimed still remains from host side. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedCapacity,0))/(1024.0*1024.0*1024.0) as capacity_allocated,
/*Assuming that total VM allocated capacity will be sum of above RDM + Non-RDM + File. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedCapacity,0)+isnull(VirtualizationLayer.capacity_FileStorage_allocated,0))/(1024.0*1024.0*1024.0) as capacity_total_allocated,
    isnull(VirtualizationLayer.dtmapd_allocated_capacity_distributed,0)/(1024.0*1024.0*1024.0), /*RDM capacity in VMWare*/
    isnull(VirtualizationLayer.VirtualPoolAllocatedCapacity_distributed,0)/(1024.0*1024.0*1024.0), /*Non RDM virtualized Block storage.*/
isnull(VirtualizationLayer.capacity_FileStorage_allocated_distributed,0)/(1024.0*1024.0*1024.0) as FileStorage_allocated_distributed,
FileStorage_allocated_distributed as capacity_FileStorage_claimed_distributed,
/*Block Allocated.
This will overwrite capacity discovered from host side. Claimed still remains from host side. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity_distributed,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedCapacity_distributed,0))/(1024.0*1024.0*1024.0) as capacity_allocated_distributed,
/*Assuming that total VM allocated capacity will be sum of above RDM + Non-RDM + File. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity_distributed,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedCapacity_distributed,0)+isnull(VirtualizationLayer.capacity_FileStorage_allocated_distributed,0))/(1024.0*1024.0*1024.0) as capacity_total_allocated_distributed,
/*Block Allocated.
This will overwrite capacity discovered from host side. Claimed still remains from host side. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedConsumedCapacity,0))/(1024.0*1024.0*1024.0) as capacity_allocated_consumed,
/*Assuming that total VM allocated capacity will be sum of above RDM + Non-RDM + File. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedConsumedCapacity,0)+isnull(VirtualizationLayer.capacity_FileStorage_allocated,0))/(1024.0*1024.0*1024.0) as capacity_total_allocated_consumed,
/*Block Allocated.
This will overwrite capacity discovered from host side. Claimed still remains from host side. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity_distributed,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedConsumedCapacity_distributed,0))/(1024.0*1024.0*1024.0) as capacity_allocated_consumed_distributed,
/*Assuming that total VM allocated capacity will be sum of above RDM + Non-RDM + File. */
(isnull(VirtualizationLayer.dtmapd_allocated_capacity_distributed,0)+isnull(VirtualizationLayer.VirtualPoolAllocatedConsumedCapacity_distributed,0)+isnull(VirtualizationLayer.capacity_FileStorage_allocated_distributed,0))/(1024.0*1024.0*1024.0) as capacity_total_allocated_consumed_distributed
from sahara.sahara_host_dimension hd
join (
    select v.DEST_HOST_ID, 
       sum(if v.IS_RAW = 1 and v.type=0 then isnull(v.size,0) else 0 endif) as dtmapd_allocated_capacity , /*Block RDM*/
       sum(if v.IS_RAW = 0 and v.type!=3 then isnull(v.size,0) else 0 endif) as VirtualPoolAllocatedCapacity, /*Block Non-RDM*/
       sum(if v.type = 3 then isnull(v.size,0) else 0 endif) as capacity_FileStorage_allocated, /*File*/
       sum(if v.IS_RAW = 1 and v.type=0 then isnull(v.size,0)/host_count else 0 endif) as dtmapd_allocated_capacity_distributed , /*Block RDM*/
       sum(if v.IS_RAW = 0 and v.type!=3 then isnull(v.size,0)/host_count else 0 endif) as VirtualPoolAllocatedCapacity_distributed, /*Block Non-RDM*/
       sum(if v.type = 3 then isnull(v.size,0)/host_count else 0 endif) as capacity_FileStorage_allocated_distributed /*File*/,
sum(if v.IS_RAW = 0 and v.type!=3 then isnull(v.consumed_size,0) else 0 endif) as VirtualPoolAllocatedConsumedCapacity, /*Block Non-RDM*/
sum(if v.IS_RAW = 0 and v.type!=3 then isnull(v.consumed_size,0)/host_count else 0 endif) as VirtualPoolAllocatedConsumedCapacity_distributed /*Block Non-RDM*/
from (
      (
	select host.id as DEST_HOST_ID, (if (obj.type is not null and obj.type = 'SPARSE') then physical_alloc else size endif) as size, physical_alloc as consumed_size, obj.id as storage_id, isnull(obj.IS_RAW,0) as IS_RAW, 0 as type, host_count
	   from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
    /* LPAR requires special handling, there can some luns directly given to the VM, those need to be handled, hence we cannot overwrite the total capacity */
	join HABDBSYNC.SF_VIEW_ALL_HOSTS_S host on da.dest_host_id = host.vm_id and (host.vmm is null OR host.vmm <> 'LPAR')
            join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and (da.source_type in ('VDISK','FILE','DISK') OR (da.source_type = 'VOLUME' AND (host.vmm is null OR host.vmm<>'ZONE')))
	    join sahara.tmp_VMDK_VM_count cnt on obj.id=cnt.VMDK_unique_identifier)
      union all /*Seen for zone only*/
      (select da.dest_host_id, lf.lun_size * (1024.0*1024.0*1024.0) as size, (if thin_status = 'ThinLun' then lf.consumed_lun_size else lf.lun_size endif) * (1024.0*1024.0*1024.0) as capacity_allocated_consumed, l.ccstor_unique_identifier, 0 as IS_RAW, 1 as type, host_count
       from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da             
	    join sahara.sahara_lun_dimension l on l.ccstor_unique_identifier = da.source_id and da.source_type in ('DISK','VDISK') and l.type='Disk' 
        join sahara.sahara_lun_facts lf on l.ccstor_unique_identifier = lf.ccstor_unique_identifier
	    join sahara.tmp_VMDK_VM_count cnt on l.ccstor_unique_identifier=cnt.VMDK_unique_identifier 
        join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and isnull(obj.IS_RAW,0) != 1)
      union all
      (select da.dest_host_id, size, size as consumed_size, obj.id as storage_id, 0 as IS_RAW, 2 as type, host_count
       from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
            join  HABDBSYNC.SF_VIEW_VOLUME_R obj on obj.id = da.source_id and da.source_type in ('VOLUME')
	    join sahara.tmp_volume_VM_count cnt on obj.id=cnt.vol_unique_identifier)
      union all
      (select da.dest_host_id, obj.TOTAL_SIZE as size , obj.TOTAL_SIZE as consumed_size, obj.id as storage_id, 0 as IS_RAW, 3 as type, host_count
       from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
            join  HABDBSYNC.SF_VIEW_FILESYSTEM_R obj on obj.id = da.source_id and da.source_type = 'FILESYSTEM'
	    join sahara.tmp_FS_VM_count cnt on obj.id=cnt.FS_unique_identifier)) as v
group by v.dest_host_id
) as VirtualizationLayer on VirtualizationLayer.DEST_HOST_ID = hd.ccstor_unique_identifier;



    /* LPAR requires special handling, there can some luns directly given to the VM, those need to be handled, hence we cannot overwrite the total capacity */
    /* We only supoort RDM based storage for LPAR */
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                            dtmapd_allocated_capacity ,                            
                            dtmapd_allocated_capacity_distributed
)
on existing update
select hd.ccstor_unique_identifier,
    isnull(VirtualizationLayer.dtmapd_allocated_capacity,0)/(1024.0*1024.0*1024.0), /*RDM capacity in VMWare*/    
    isnull(VirtualizationLayer.dtmapd_allocated_capacity_distributed,0)/(1024.0*1024.0*1024.0) /*RDM capacity in VMWare*/
from sahara.sahara_host_dimension hd
join (
    select v.DEST_HOST_ID, 
       sum(if v.IS_RAW = 1 and v.type=0 then isnull(v.size,0) else 0 endif) as dtmapd_allocated_capacity , /*Block RDM*/
       sum(if v.IS_RAW = 1 and v.type=0 then isnull(v.size,0)/host_count else 0 endif) as dtmapd_allocated_capacity_distributed  /*Block RDM*/
        from  (
	            select host.id as DEST_HOST_ID, (if (obj.type is not null and obj.type = 'SPARSE') then physical_alloc else size endif) as size, physical_alloc as consumed_size, obj.id as storage_id, isnull(obj.IS_RAW,0) as IS_RAW, 0 as type, host_count
	                from HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da 
            /* LPAR requires special handling, there can some luns directly given to the VM, those need to be handled, hence we cannot overwrite the total capacity */
	                join HABDBSYNC.SF_VIEW_ALL_HOSTS_S host on da.dest_host_id = host.vm_id and host.vmm =  'LPAR'
                    join  HABDBSYNC.VT_VIEW_VDISK_R obj on obj.id = da.source_id and da.source_type in ('VDISK','FILE','DISK','VOLUME')
	                join sahara.tmp_VMDK_VM_count cnt on obj.id=cnt.VMDK_unique_identifier
            ) as v
            group by v.dest_host_id
) as VirtualizationLayer on VirtualizationLayer.DEST_HOST_ID = hd.ccstor_unique_identifier;

/* LPAR requires special handling, there can some luns directly given to the VM, those need to be handled, hence we cannot overwrite the total capacity */
    /* We only supoort RDM based storage for LPAR */

set @attributenames = 'ccstor_unique_identifier,capacity_total_allocated';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
update sahara.sahara_host_facts hf, sahara.sahara_host_dimension hd
set hf.capacity_total_allocated = hf.dtmapd_allocated_capacity,hf.capacity_allocated = hf.dtmapd_allocated_capacity 
 where hf.ccstor_unique_identifier = hf.ccstor_unique_identifier and hd.virtualizationTechnology  = 'LPAR' 
and hf.capacity_total_allocated < hf.dtmapd_allocated_capacity;

/* LPAR requires special handling, there can some luns directly given to the VM, those need to be handled, hence we cannot overwrite the total capacity */
    /* We only supoort RDM based storage for LPAR */
set @attributenames = 'ccstor_unique_identifier,capacity_total_allocated_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
update sahara.sahara_host_facts hf, sahara.sahara_host_dimension hd
set hf.capacity_total_allocated_distributed = hf.dtmapd_allocated_capacity_distributed 
 where hf.ccstor_unique_identifier = hf.ccstor_unique_identifier and hd.virtualizationTechnology  = 'LPAR' and hf.capacity_total_allocated_distributed < hf.dtmapd_allocated_capacity_distributed;
;

set @tablename = 'sahara.sahara_host_facts';
set @attributenames = 'ccstor_unique_identifier,capacity_virtualization,capacity_storage_pool_used,capacity_storage_pool_unused,capacity_virtualization_distributed,capacity_storage_pool_used_distributed,capacity_storage_pool_unused_distributed,capacity_NAS_storage_pool,capacity_NAS_storage_pool_used,capacity_NAS_storage_pool_unused,capacity_NAS_storage_pool_distributed,capacity_NAS_storage_pool_used_distributed,capacity_NAS_storage_pool_unused_distributed,capacity_storage_pool_NON_RDM_used,capacity_storage_pool_NON_RDM_used_distributed,sp_overhead,sp_overhead_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);

if exists (select 1 from sys.syscolumns where tname='VT_VIEW_STORAGE_POOL_R' and cname='PARENT_SP_ID') then

insert into sahara.sahara_host_facts(ccstor_unique_identifier,capacity_virtualization,capacity_storage_pool_used,capacity_storage_pool_unused,capacity_virtualization_distributed,capacity_storage_pool_used_distributed,capacity_storage_pool_unused_distributed,capacity_NAS_storage_pool,capacity_NAS_storage_pool_used,capacity_NAS_storage_pool_unused,capacity_NAS_storage_pool_distributed,capacity_NAS_storage_pool_used_distributed,capacity_NAS_storage_pool_unused_distributed,capacity_storage_pool_NON_RDM_used,capacity_storage_pool_NON_RDM_used_distributed,sp_overhead,sp_overhead_distributed)
on existing update

select sptohost.Host_ID, 
sum(Total_Size)/(1024.0*1024.0*1024.0) as capacity_virtualization,
sum(Total_Used)/(1024.0*1024.0*1024.0) as capacity_storage_pool_used, 
sum(Total_Free)/(1024.0*1024.0*1024.0) as capacity_storage_pool_unused,
sum(Total_Size / cnt.host_count)/(1024.0*1024.0*1024.0) as capacity_virtualization_distributed,
sum(Total_Used / cnt.host_count)/(1024.0*1024.0*1024.0) as capacity_storage_pool_used_distributed, 
sum(Total_Free / cnt.host_count)/(1024.0*1024.0*1024.0) as capacity_storage_pool_unused_distributed,
sum(if type = 'NFS' then Total_Size else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool,
sum(if type = 'NFS' then Total_Used else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_used, 
sum(if type = 'NFS' then Total_Free else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_unused,
sum(if type = 'NFS' then Total_Size / cnt.host_count else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_distributed,
sum(if type = 'NFS' then Total_Used / cnt.host_count else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_used_distributed, 
sum(if type = 'NFS' then Total_Free / cnt.host_count else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_unused_distributed,
sum(Non_RDM_Used.cap) /(1024.0*1024.0*1024.0) as capacity_storage_pool_NON_RDM_used,
sum(Non_RDM_Used.cap / cnt.host_count) /(1024.0*1024.0*1024.0) as capacity_storage_pool_NON_RDM_used_distributed,
sum(if type = 'NFS' then 0 else isnull((Total_Size-(Total_Free+Non_RDM_Used.cap))/ cnt.host_count,0) endif)/(1024.0*1024.0*1024.0) as sp_overhead,
sum(if type = 'NFS' then 0 else isnull((Total_Size-(Total_Free+Non_RDM_Used.cap)),0) endif)/(1024.0*1024.0*1024.0) as sp_overhead_distributed
from
   
   (select SP.* from HABDBSYNC.VT_VIEW_STORAGE_POOL_R sp where sp.ID not in (select parent_sp_id from HABDBSYNC.VT_VIEW_STORAGE_POOL_R  where parent_sp_id is not null)) spool   
   JOIN HABDBSYNC.VT_VIEW_SP_TO_HOST_R sptohost on spool.ID = sptohost.SP_ID   
   join sahara.sahara_host_dimension on sptohost.Host_ID = ccstor_unique_identifier
   join sahara.tmp_storage_pool_host_count cnt on spool.ID = cnt.pool_unique_identifier
/*TODO: Ignore snapshots*/
   left outer join (select sp_id, sum(vdisk.physical_alloc) as cap from habdbsync.VT_VIEW_VDISK_R vdisk where isnull(IS_RAW,0) = 0 group by sp_id) as Non_RDM_Used on spool.ID = Non_RDM_Used.SP_ID
group by sptohost.Host_ID;


else // Below code for < 5.0

insert into sahara.sahara_host_facts(ccstor_unique_identifier,capacity_virtualization,capacity_storage_pool_used,capacity_storage_pool_unused,capacity_virtualization_distributed,capacity_storage_pool_used_distributed,capacity_storage_pool_unused_distributed,capacity_NAS_storage_pool,capacity_NAS_storage_pool_used,capacity_NAS_storage_pool_unused,capacity_NAS_storage_pool_distributed,capacity_NAS_storage_pool_used_distributed,capacity_NAS_storage_pool_unused_distributed,capacity_storage_pool_NON_RDM_used,capacity_storage_pool_NON_RDM_used_distributed,sp_overhead,sp_overhead_distributed)
on existing update
select sptohost.Host_ID, 
sum(Total_Size)/(1024.0*1024.0*1024.0) as capacity_virtualization,
sum(Total_Used)/(1024.0*1024.0*1024.0) as capacity_storage_pool_used, 
sum(Total_Free)/(1024.0*1024.0*1024.0) as capacity_storage_pool_unused,
sum(Total_Size / cnt.host_count)/(1024.0*1024.0*1024.0) as capacity_virtualization_distributed,
sum(Total_Used / cnt.host_count)/(1024.0*1024.0*1024.0) as capacity_storage_pool_used_distributed, 
sum(Total_Free / cnt.host_count)/(1024.0*1024.0*1024.0) as capacity_storage_pool_unused_distributed,
sum(if type = 'NFS' then Total_Size else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool,
sum(if type = 'NFS' then Total_Used else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_used, 
sum(if type = 'NFS' then Total_Free else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_unused,
sum(if type = 'NFS' then Total_Size / cnt.host_count else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_distributed,
sum(if type = 'NFS' then Total_Used / cnt.host_count else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_used_distributed, 
sum(if type = 'NFS' then Total_Free / cnt.host_count else 0 endif)/(1024.0*1024.0*1024.0) as capacity_NFS_storage_pool_unused_distributed,
sum(Non_RDM_Used.cap) /(1024.0*1024.0*1024.0) as capacity_storage_pool_NON_RDM_used,
sum(Non_RDM_Used.cap / cnt.host_count) /(1024.0*1024.0*1024.0) as capacity_storage_pool_NON_RDM_used_distributed,
sum(if type = 'NFS' then 0 else isnull((Total_Size-(Total_Free+Non_RDM_Used.cap))/ cnt.host_count,0) endif)/(1024.0*1024.0*1024.0) as sp_overhead,
sum(if type = 'NFS' then 0 else isnull((Total_Size-(Total_Free+Non_RDM_Used.cap)),0) endif)/(1024.0*1024.0*1024.0) as sp_overhead_distributed
from
   HABDBSYNC.VT_VIEW_STORAGE_POOL_R spool
   JOIN HABDBSYNC.VT_VIEW_SP_TO_HOST_R sptohost on spool.ID = sptohost.SP_ID
   join sahara.sahara_host_dimension on sptohost.Host_ID = ccstor_unique_identifier
   join sahara.tmp_storage_pool_host_count cnt on spool.ID = cnt.pool_unique_identifier
/*TODO: Ignore snapshots*/
   left outer join (select sp_id, sum(vdisk.physical_alloc) as cap from habdbsync.VT_VIEW_VDISK_R vdisk where isnull(IS_RAW,0) = 0 group by sp_id) as Non_RDM_Used on spool.ID = Non_RDM_Used.SP_ID
group by sptohost.Host_ID;

end if;



-- If the result is negative, reset to 0 and write to the sahara_log;
select count(*) into @count
from sahara.sahara_host_facts
where sp_overhead < 0;


if ( @count > 0 ) then
      select list(ccstor_unique_identifier) into @extraerrorinfo
      from sahara.sahara_host_facts
      where sp_overhead < 0;

      insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message) 
      select GETDATE(), @batch_id, 'sahara.sahara_host_consumption_data', 'W', string (' Error failed at = ', @tablename+':'+@attributenames ) + 
                                                                    string (': count_of negative occurence = ', @count );

      message string (' Warning, failed at = ', @tablename+':'+@attributenames );
      message string (' List of hosts  = ', @extraerrorinfo );
      message string (': count_of negative occurence = ', @count );

      update sahara.sahara_host_facts set sp_overhead = 0.0
      from sahara.sahara_host_facts
      where sp_overhead < 0;

      update sahara.sahara_host_facts set sp_overhead_distributed = 0.0
      from sahara.sahara_host_facts
      where sp_overhead_distributed < 0;
end if;

/*Add NAS SP to total allocated capacity and File Storage allocated for ESX servers*/
set @attributenames = 'ccstor_unique_identifier,capacity_total_allocated';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
update sahara.sahara_host_facts
set capacity_total_allocated = capacity_total_allocated+capacity_NAS_storage_pool,
capacity_total_allocated_distributed = capacity_total_allocated_distributed+capacity_NAS_storage_pool_distributed,
capacity_FileStorage_allocated = capacity_FileStorage_allocated+capacity_NAS_storage_pool,
capacity_FileStorage_allocated_distributed = capacity_FileStorage_allocated_distributed+capacity_NAS_storage_pool_distributed
from
sahara.sahara_host_facts
;


set @attributenames = 'ccstor_unique_identifier,capacity_unused_VMDK,capacity_unused_VMDK_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier,capacity_unused_VMDK,capacity_unused_VMDK_distributed)
on existing update
select UnusedVDisks.host_id, sum(UnusedVDisks.physical_alloc)/(1024.0*1024.0*1024.0) as unusedVMDK, sum(UnusedVDisks.physical_alloc/host_count)/(1024.0*1024.0*1024.0) as unusedVMDKDistributed from
(select sph.host_id, vdisk.id, vdisk.physical_alloc, cnt.host_count from
HABDBSYNC.VT_VIEW_SP_TO_HOST_R sph
join sahara.sahara_host_dimension on sph.Host_ID = ccstor_unique_identifier
JOIN HABDBSYNC.VT_VIEW_STORAGE_POOL_R sp  on sp.ID = sph.SP_ID
JOIN HABDBSYNC.VT_VIEW_VDISK_R vdisk ON vdisk.SP_ID = sp.ID
join sahara.tmp_VMDK_ESX_count cnt on cnt.VMDK_unique_identifier = vdisk.id /*Gives Unused VMDK*/
) as UnusedVDisks
group by UnusedVDisks.host_id;

set @attributenames = 'ccstor_unique_identifier,capacity_virtualization_unused,capacity_virtualization_unused_distributed,capacity_exported_to_VMs,capacity_exported_to_VMs_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier,capacity_virtualization_unused,capacity_virtualization_unused_distributed,capacity_exported_to_VMs,capacity_exported_to_VMs_distributed)
on existing update
select sahara.sahara_host_facts.ccstor_unique_identifier, (capacity_storage_pool_unused+capacity_unused_VMDK) as capacity_virtualization_unused, 
(capacity_storage_pool_unused_distributed+capacity_unused_VMDK_distributed)
as capacity_virtualization_unused_distributed,
(capacity_exported_LUNs+capacity_exported_Volumes+capacity_exported_FSs) as capacity_exported_to_VMs,
(capacity_exported_LUNs_distributed+capacity_exported_Volumes_distributed+capacity_exported_FSs_distributed) as capacity_exported_to_VMs_distributed
from
sahara.sahara_host_facts, sahara.sahara_host_dimension 
where sahara.sahara_host_dimension.ccstor_unique_identifier = sahara.sahara_host_facts.ccstor_unique_identifier 
and sahara.sahara_host_dimension.VirtualizationTechnology <> 'Hyper-V';
;

set @attributenames = 'ccstor_unique_identifier,capacity_virtualization_unused,capacity_virtualization_unused_distributed,capacity_exported_to_VMs,capacity_exported_to_VMs_distributed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier,capacity_virtualization_unused,capacity_virtualization_unused_distributed)
on existing update
select sahara.sahara_host_facts.ccstor_unique_identifier, (capacity_storage_pool_unused+capacity_unused_VMDK) as capacity_virtualization_unused, 
(capacity_storage_pool_unused_distributed+capacity_unused_VMDK_distributed)
as capacity_virtualization_unused_distributed
from
sahara.sahara_host_facts, sahara.sahara_host_dimension 
where sahara.sahara_host_dimension.ccstor_unique_identifier = sahara.sahara_host_facts.ccstor_unique_identifier 
and sahara.sahara_host_dimension.VirtualizationTechnology = 'Hyper-V';
;


set @attributenames = 'ccstor_unique_identifier,exchange_capacity';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier,exchange_capacity)
on existing update
select exc.host_id, sum(dbfile_size)/(1024*1024*1024.0) as cap
from
HABDBSYNC.SF_VIEW_UI_EXCH_DBFILE exc
join sahara.sahara_host_dimension hd on exc.host_id = hd.ccstor_unique_identifier
group by exc.host_id
;

/*
percent claimed vs allocated
*/
set @attributenames = 'per_allocated_capacity_claimed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, per_allocated_capacity_claimed)
on existing update
Select percentage.ccstor_unique_identifier, 
(if percentage.per_allocated_capacity_claimed>999 then 999 else percentage.per_allocated_capacity_claimed endif) as Host_Claimed_Percentage
from
(select ccstor_unique_identifier,
(case 
    when capacity_allocated > 0 then
        ( capacity_claimed * 100) /capacity_allocated 
    else 0
end
) as  per_allocated_capacity_claimed 
from sahara.sahara_host_facts) as percentage;

/*
percent unclaimed vs allocated
*/
set @attributenames = 'per_allocated_capacity_unclaimed';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, per_allocated_capacity_unclaimed)
on existing update
select ccstor_unique_identifier,
(case 
    when capacity_allocated > 0 then
        ( capacity_unclaimed * 100) /capacity_allocated 
    else 0
end
) as  per_allocated_capacity_unclaimed 
from sahara.sahara_host_facts;

set @attributenames = 'DB VM Capacities';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_dbvm, capacity_dbvm_free, capacity_dbvm_used, capacity_dbvm_distributed, capacity_dbvm_free_distributed, capacity_dbvm_used_distributed, capacity_dbvm_overhead, capacity_dbvm_unknown, capacity_apps_on_dbvm, capacity_apps_on_dbvm_distributed, capacity_dbvm_overhead_distributed,capacity_apps_on_dbvm_other,capacity_dbvm_db_used, capacity_dbvm_db_used_by_other, capacity_dbvm_db_used_distributed)
on existing update
select hostd.host_unique_identifier, sum(capacity), sum(capacity_free), sum(capacity_used), sum(capacity/no_of_hosts), sum(capacity_free/no_of_hosts), sum(capacity_used/no_of_hosts), sum(capacity_overhead), sum(capacity_unknown), sum(dgf.capacity_used_db), sum(dgf.capacity_used_db/no_of_hosts), sum(capacity_overhead/no_of_hosts),
sum(dgf.capacity_used_db-hostd.capacity_used_db) as capacity_apps_on_dbvm_other,
sum(dgf.capacity_used_db_tbsp_used), sum(dgf.capacity_used_db_tbsp_used - hostd.capacity_used_db_tbsp_used) as capacity_dbvm_db_used_by_other, sum(dgf.capacity_used_db_tbsp_used/no_of_hosts)
from 
sahara.sahara_dg_to_host hostd
join sahara.sahara_dg_dimension dgd on dgd.ccstor_unique_identifier = hostd.dg_unique_identifier
join sahara.sahara_dg_facts dgf on dgd.ccstor_unique_identifier = dgf.ccstor_unique_identifier
where dgd.family='OracleASM'
group by hostd.host_unique_identifier
;

set @attributenames = 'ccstor_unique_identifier,per_host_utilization';
call sahara.debug_etl_log(@tablename,@attributenames,@sfm_debug_flag);
insert into sahara.sahara_host_facts(ccstor_unique_identifier,per_host_utilization)
on existing update
select percentage.ccstor_unique_identifier, (if percentage.Host_Utilization_Percentage>999 then 999 else percentage.Host_Utilization_Percentage endif) as Host_Utilization_Percentage
from
(
select
host.ccstor_unique_identifier,
            isnull((IF host.is_server_virtualized != 1 AND apps_visibility.host_unique_identifier IS NOT NULL 
               THEN (IF capacity_total_allocated <> 0 THEN ((capacity_filesystem_used_by_non_apps_ignore_size_zero + capacity_database_used)/capacity_total_allocated) * 100 ELSE 0 ENDIF)
            ELSE
             IF host.is_server_virtualized != 1 AND fs_visibility.host_unique_identifier IS NOT NULL 
                  THEN (IF capacity_total_allocated <> 0 THEN (capacity_filesystem_used_ignore_size_zero/capacity_total_allocated) * 100 ELSE 0 ENDIF) 
                 ELSE 
                    IF (host.is_server_virtualized = 1  and host.VirtualizationTechnology in ('VMware','LPAR','LDOM','Hyper-V') and  capacity_total_allocated <> 0) THEN (capacity_storage_pool_used/capacity_total_allocated)*100
                     ELSE 
                        IF (host.is_server_virtualized = 1 and host.VirtualizationTechnology = 'Solaris Zones' and  capacity_total_allocated <> 0) 
                        THEN 
                            if (apps_visibility.host_unique_identifier IS NOT NULL)
                                Then 
                                    (((capacity_unexported_filesystem_used_by_non_apps_ignore_size_zero + capacity_database_used) + capacity_exported_LUNs+capacity_exported_Volumes+capacity_exported_FSs_ignore_size_zero) /capacity_total_allocated)*100
                                Else       
                                    ((capacity_unexported_filesystem_used_ignore_size_zero + capacity_exported_LUNs+capacity_exported_Volumes+capacity_exported_FSs_ignore_size_zero) /capacity_total_allocated)*100                               
                            endif
                        ENDIF 
                    ENDIF                
                ENDIF  
            ENDIF),0) as Host_Utilization_Percentage
from
sahara.sahara_host_dimension host
join sahara.sahara_host_facts hostf on host.ccstor_unique_identifier = hostf.ccstor_unique_identifier
left outer join
(
select distinct hostd.host_unique_identifier  
from 
sahara.sahara_database_to_host hostd
join sahara.sahara_database_dimension dbd on dbd.host_unique_identifier = hostd.host_unique_identifier
) as apps_visibility on host.ccstor_unique_identifier = apps_visibility.host_unique_identifier  
left outer join
(
select distinct hostd.host_unique_identifier  
from 
sahara.sahara_host_dimension host
join sahara.sahara_fs_to_host hostd on  host.ccstor_unique_identifier = hostd.host_unique_identifier
) as fs_visibility on host.ccstor_unique_identifier = fs_visibility.host_unique_identifier
)as percentage
;

-- If the per_host_utilization>100 write to the sahara_log;
select count(*) into @count
from sahara.sahara_host_facts
where per_host_utilization > 100;

if ( @count > 0 ) then
      select list(ccstor_unique_identifier) into @extraerrorinfo
      from sahara.sahara_host_facts
      where per_host_utilization > 100;

      insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message) 
      select GETDATE(), @batch_id, 'sahara.sahara_host_consumption_data', 'W', string (' Error failed at = ', @tablename+':'+@attributenames ) + 
                                                                    string (': count_of hosts with more than 100 percent utilization = ', @count );

      message string (' Warning, failed at = ', @tablename+':'+@attributenames );
      message string (' List of hosts  = ', @extraerrorinfo );
      message string (': count_of hosts with more than 100 percent utilization = ', @count );
      
end if;

select count(1) into @noDims from sahara.sahara_lun_dimension;
select count(1) into @noFacts from sahara.sahara_host_facts;


insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_host_consumption_data', 'I','completed dimension rollup -> sahara_lun_dimension: ' || @noDims || ' are the total rows';

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_host_consumption_data', 'I','completed facts rollup -> sahara_host_facts: ' || @noFacts || '  are the total rows';

commit;

EXCEPTION
    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 ) then
            SET @ERROR_COUNT=@ERROR_COUNT + 1;
            BEGIN
                insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message)
                select GETDATE(), @batch_id, 'sahara.sahara_host_consumption_data', 'E', string (' Error failed at = ', @tablename+':'+@attributenames ) +
                                                                             string (': SQLCODE = ', @error ) +
                                                                             string (': ERRORMSG = ', @serror ) +
                                                                             string (': SQLSTATE = ', @sqlstate );
                message string (' Error failed at = ', @tablename+':'+@attributenames ) to LOG;
                message string ('          @error = ', @error ) to LOG;
                message string ('      @serrir()  = ', @serror ) to LOG;
                message string ('     @sqlstate() = ', @sqlstate ) to LOG;
            END;
        END IF;
        -- RESIGNAL;


END;
grant execute on sahara.sahara_host_consumption_data to public;
