if exists(select 1 from sys.sysprocedure where proc_name = 'sahara_host_data' and user_name(creator) = 'sahara') then
   drop procedure sahara.sahara_host_data
end if;

CREATE PROCEDURE sahara."sahara_host_data" (@batch_id integer)
/* RESULT ( column_name, ... )
 */
BEGIN

/* Variables for logging purpose */
/* 
To record initial time when this stored procedure was called 
*/
DECLARE @initialtime timestamp; 

/* 
Total number of dimension this stored procedure is inserting 
*/
DECLARE @noDims bigint; 

/* 
Total number of facts this stored procedure is inserting 
*/
DECLARE @noFacts bigint; 
/*declare @tmpstr long varchar;
declare @hostlist long varchar;
DECLARE ccs52 int; */
declare @si_installed int;

/* Log detail error info */
declare @err_code integer;
declare @error    integer;
declare @serror         varchar(32767);
declare @sqlstate       varchar(5);
declare @tablename      varchar(128);
declare @attributenames varchar(256);
declare @count    integer;
declare @error_count    integer;

/* 
Store the current time 
*/
/*set ccs52 = 0;*/
Set @initialtime=GETDATE();

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_host_data', 'I','starting dimension/facts rollup -> sahara_host_dimension/sahara_host_facts';

select count(*) into @si_installed 
from habdbsync.SF_VIEW_ADD_ON_STATUS_R 
where addon_id like '%sfmsi%' and status = 'enabled' and operation_status = 'install success';

if exists(select 1 from sys.systable where table_name='sahara_host_dimension' and creator=user_id('sahara')) then
   drop table sahara.sahara_host_dimension
end if;

if exists(select 1 from sys.systable where table_name='sahara_host_facts' and creator=user_id('sahara')) then
   drop table sahara.sahara_host_facts
end if;

if exists(select 1 from sys.systable where table_name='host_array_capacity' and creator=user_id('sahara')) then
   drop table sahara.host_array_capacity
end if;

if exists(select 1 from sys.systable where table_name='sahara_all_luns_to_host' and creator=user_id('sahara')) then
   drop table sahara.sahara_all_luns_to_host
end if;

/*==============================================================*/
/* Table: HOST                                                */
/*==============================================================*/
/* Host_OID (ccstor_unique_identifier) changes in vm_oid_data.sql for VMs discovered by both VC and VOM.*/
create table sahara.sahara_host_dimension  (
   host_id           			unsigned int,
   ccstor_unique_identifier    		varchar(256),   
   display_name         		varchar(256) not null,
   os_details 					varchar(1024),
   os_type              		unsigned int default 0, /* Deprecated */
   os_type_str             		varchar(256),
   guest_os_type              		unsigned int default 0,/* Deprecated */
   guest_os_type_str             		varchar(256) default 'N/A',
   ip_address  				long varchar default 'UNKNOWN',
   is_host_user_created 		bit default 0,/* Deprecated */
   is_server_virtualized			bit default 0,
   is_virtual				bit default 0,
   parent_virtualization_server         varchar(256) default 'N/A',
   parent_virtualization_server_oid     varchar(256) default 'N/A',
   is_clustered				bit default 0,
   cluster_type				varchar(256) default 'N/A',
   cluster_version			varchar(256) default 'N/A',
   is_active				bit default 0, /* Deprecated */
   cluster_name	      		varchar(256) default 'N/A',
/*By default we assume that OS is installed on GOS. In VMWare, if we cannot find IP address, it's 0*/
   is_guestos_installed				bit default 0, 
   VirtualizationTechnology    varchar(256) default 'N/A',
   batch_id 				int default 0,
   control_guest				char(1) default 'F', 
   SF_version             		varchar(32) default 'N/A',
   MH_version             		varchar(32) default 'N/A',
   MH_Kit_Differntiator			varchar(32) default 'N/A', /*Not Exposed*/
   vm_host_unique_identifier    	varchar(256),
   vom_license_required                 char(1) default 'F',
   configuration_type			varchar(256) default 'VSERVER', /*Agent, Agentless Or Vserver*/
   primary key(ccstor_unique_identifier)
);
grant select on sahara.sahara_host_dimension to public;

/*==============================================================*/
/* Table: sahara_host_facts                                          */
/*==============================================================*/
create table sahara.sahara_host_facts  (
   host_id					unsigned int,
   capacity_allocated 				NUMERIC(22,9) default 0, /*Block*/
   capacity_fc_fibre_allocated 			NUMERIC(22,9) default 0,
   capacity_nonfc_fibre_allocated 		NUMERIC(22,9) default 0, // DEPRECATED
   capacity_IDE_allocated 			NUMERIC(22,9) default 0,
   capacity_ISCSI_allocated 			NUMERIC(22,9) default 0,
   capacity_FileStorage_allocated 		NUMERIC(22,9) default 0,
   capacity_DAS_allocated 			NUMERIC(22,9) default 0,
   capacity_claimed				NUMERIC(22,9) default 0,
   capacity_fc_fibre_claimed 			NUMERIC(22,9) default 0, 
   capacity_nonfc_fibre_claimed 		NUMERIC(22,9) default 0, // DEPRECATED  
   capacity_ISCSI_claimed 			NUMERIC(22,9) default 0,
   capacity_FileStorage_claimed 		NUMERIC(22,9) default 0,
   capacity_unclaimed				NUMERIC(22,9) default 0,
   capacity_claimed_unknown			NUMERIC(22,9) default 0, // DEPRECATED
   per_allocated_capacity_claimed		NUMERIC(20,9) default 0,
   per_allocated_capacity_unclaimed  		NUMERIC(20,9) default 0,
   per_allocated_capacity_claimed_unknown	NUMERIC(20,9) default 0,  // DEPRECATED
   ccstor_unique_identifier			varchar(256),
   dtmapd_allocated_capacity			NUMERIC(22,9) default 0,   
   VirtualPoolAllocatedCapacity			NUMERIC(22,9) default 0,
/************************NEW BELOW*****************************/
   capacity_total_allocated			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_allocated_claimed_by_other_hosts	NUMERIC(22,9) default 0, /*Implemented*/
   capacity_assigned_VM				NUMERIC(22,9) default 0, /*Implemented*/
   capacity_directly_assigned_FS		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_directly_assigned_DB		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_assigned_SP				NUMERIC(22,9) default 0, /* In ESX server case, Assigned to data Store. Need to be included in LUN Used. */ 
   capacity_unknown_assignment			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_claimed_assigned_to_other_hosts	NUMERIC(22,9) default 0, /*Implemented*/
   capacity_diskgroup				NUMERIC(22,9) default 0, /*equal to volume manager*/ /*Implemented*/
   capacity_diskgroup_used			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_diskgroup_available			NUMERIC(22,9) default 0, /*equal to volume manager unused as we do not add unused vols*/ /*Implemented*/
   capacity_volume				NUMERIC(22,9) default 0, /*Implemented*/
   capacity_unused_volume			NUMERIC(22,9) default 0, /*Capacity of potentially unused Volumes*/ /*Implemented*/
   capacity_volume_manager_overhead		NUMERIC(22,9) default 0, /*gone in protection initiated by user like Mirror OR RAID-5*/
   capacity_volume_manager_administrative	NUMERIC(22,9) default 0, /*The area reserved by volume manager like private region or used for book-keeping in VVR etc. TODO: Not implemented */
   capacity_filesystem				NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_used			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_unexported_filesystem_used		NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_filesystem_available		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_on_volumes		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_on_non_volumes		NUMERIC(22,9) default 0, /* FS On Raw LUN*/ /*Implemented*/
   capacity_filesystem_used_by_apps		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_used_by_non_apps		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_unexported_filesystem_used_by_non_apps	NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_database				NUMERIC(22,9) default 0, /*Implemented*/
   capacity_database_used			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_database_available			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_apps_on_raw_volume			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_apps_on_filesystem			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_apps_on_raw_lun			NUMERIC(22,9) default 0, /*This isn't supported in VOM today*/

   per_host_utilization  		NUMERIC(20,9) default 0,
/*****************************************************/
/* Thin-Aware Capacities*/
   capacity_thin_subscribed			NUMERIC(22,9) default 0, /*Amount of thin storage allocated*/ /*Implemented*/
   capacity_thin_consumed			NUMERIC(22,9) default 0, /*Amount of thin storage consumed*/ /*Implemented*/
   capacity_allocated_consumed 			NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_total_allocated_consumed 		NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_claimed_consumed 			NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_unclaimed_consumed 			NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_allocated_claimed_by_other_hosts_consumed NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_assigned_VM_consumed 		NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_directly_assigned_FS_consumed 	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_directly_assigned_DB_consumed 	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_assigned_SP_consumed		NUMERIC(22,9) default 0, /* In ESX server case, Assigned to data Store. Need to be included in LUN Used. */ 
   capacity_unknown_assignment_consumed 	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_claimed_assigned_to_other_hosts_consumed	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/

/*****************************************************/
/* Virtualization Related Capacities: Same capacities would be listed per ESX server if storage pool is shared*/   
   /*ESX*/
   capacity_virtualization  		NUMERIC(22,9) default 0, /*Equal to Storage Pool capacity*/ /*Implemented*/
   capacity_virtualization_unused  	NUMERIC(22,9) default 0, /*SP Unused + unassigned VMDKs*/ /*Implemented*/
   capacity_storage_pool_used		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_storage_pool_unused		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_unused_VMDK			NUMERIC(22,9) default 0, /*Implemented with snapshots not handled*/   
   capacity_NAS_storage_pool		NUMERIC(22,9) default 0,
   capacity_NAS_storage_pool_used	NUMERIC(22,9) default 0,
   capacity_NAS_storage_pool_unused	NUMERIC(22,9) default 0,
   capacity_storage_pool_NON_RDM_used	NUMERIC(22,9) default 0, /*Not Exposed.*/

   sp_overhead				NUMERIC(22,9) default 0,

   /*Zones*/	
   capacity_exported_to_VMs		NUMERIC(22,9) default 0,
   capacity_exported_LUNs		NUMERIC(22,9) default 0,
   capacity_exported_LUNs_consumed	NUMERIC(22,9) default 0,
   capacity_exported_Volumes		NUMERIC(22,9) default 0,
   capacity_exported_FSs		NUMERIC(22,9) default 0,

/*****************************************************/
/*Cluster-aware Capacities*/
   capacity_allocated_distributed 			NUMERIC(22,9) default 0, /*Block*/ /*Implemented*/
   capacity_fc_fibre_allocated_distributed 		NUMERIC(22,9) default 0,  /*Implemented*/
   capacity_ISCSI_allocated_distributed 		NUMERIC(22,9) default 0,  /*Implemented*/
   capacity_FileStorage_allocated_distributed 		NUMERIC(22,9) default 0, 
   capacity_claimed_distributed				NUMERIC(22,9) default 0, /*Implemented*/
   capacity_fc_fibre_claimed_distributed 		NUMERIC(22,9) default 0,  /*Implemented*/
   capacity_ISCSI_claimed_distributed 			NUMERIC(22,9) default 0,  /*Implemented*/
   capacity_FileStorage_claimed_distributed 		NUMERIC(22,9) default 0,
   capacity_unclaimed_distributed			NUMERIC(22,9) default 0, /*Implemented*/
   dtmapd_allocated_capacity_distributed		NUMERIC(22,9) default 0,   
   VirtualPoolAllocatedCapacity_distributed		NUMERIC(22,9) default 0,
   capacity_total_allocated_distributed			NUMERIC(22,9) default 0,
   capacity_assigned_VM_distributed			NUMERIC(22,9) default 0, /* In ESX server case, Assigned to data Store. Need to be included in LUN Used. */ /*Implemented*/
   capacity_directly_assigned_FS_distributed		NUMERIC(22,9) default 0,/*Implemented*/
   capacity_directly_assigned_DB_distributed		NUMERIC(22,9) default 0, /*Not in VOM*/
   capacity_assigned_SP_distributed			NUMERIC(22,9) default 0, /* In ESX server case, Assigned to data Store. Need to be included in LUN Used. */ 
   capacity_unknown_assignment_distributed		NUMERIC(22,9) default 0,   /*Implemented*/
   capacity_diskgroup_distributed			NUMERIC(22,9) default 0, /*equal to volume manager*/ /*Implemented*/
   capacity_diskgroup_used_distributed			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_diskgroup_available_distributed		NUMERIC(22,9) default 0, /*equal to volume manager unused as we do not add unused vols*/ /*Implemented*/
   capacity_volume_distributed				NUMERIC(22,9) default 0, /*Implemented*/
   capacity_unused_volume_distributed			NUMERIC(22,9) default 0, /*Capacity of potentially unused Volumes*//*Implemented*/
   capacity_volume_manager_overhead_distributed		NUMERIC(22,9) default 0,
   capacity_volume_manager_administrative_distributed	NUMERIC(22,9) default 0, /*The area reserved by volume manager like private region or used for book-keeping in VVR etc. TODO: Not implemented  */
   capacity_filesystem_distributed			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_used_distributed			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_available_distributed			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_on_volumes_distributed		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_filesystem_on_non_volumes_distributed	NUMERIC(22,9) default 0, /* FS On Raw LUN*/ /*Implemented*/
   capacity_filesystem_used_by_apps_distributed		NUMERIC(22,9) default 0, /*Implemented*/ 
   capacity_filesystem_used_by_non_apps_distributed	NUMERIC(22,9) default 0, /*Implemented*/
   capacity_database_distributed			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_database_used_distributed			NUMERIC(22,9) default 0, /*Implemented*/
   capacity_database_available_distributed		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_apps_on_raw_volume_distributed		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_apps_on_filesystem_distributed		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_apps_on_raw_lun_distributed			NUMERIC(22,9) default 0, /*This isn't supported in VOM today*/

/*****************************************************/
/* Thin-Aware Cluster-Aware Capacities*/
   capacity_thin_subscribed_distributed			NUMERIC(22,9) default 0, /*Amount of thin storage allocated*/ /*Implemented*/
   capacity_thin_consumed_distributed			NUMERIC(22,9) default 0, /*Amount of thin storage consumed*/ /*Implemented*/
   capacity_allocated_consumed_distributed 		NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_total_allocated_consumed_distributed 	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/
   capacity_claimed_consumed_distributed 		NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_unclaimed_consumed_distributed 		NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_assigned_VM_consumed_distributed 		NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_directly_assigned_FS_consumed_distributed 	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/
   capacity_directly_assigned_DB_consumed_distributed 	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Not in VOM*/
   capacity_assigned_SP_consumed_distributed		NUMERIC(22,9) default 0, /* In ESX server case, Assigned to data Store. Need to be included in LUN Used. */ 
   capacity_unknown_assignment_consumed_distributed 	NUMERIC(22,9) default 0, /*For "Use Consumed" filter*/ /*Implemented*/

/*****************************************************/
/* Cluster-Aware Virtualization Related Capacities */   
   /*ESX*/
   capacity_virtualization_distributed  		NUMERIC(22,9) default 0, /*Equal to Storage Pool capacity*/ /*Implemented*/
   capacity_virtualization_unused_distributed  		NUMERIC(22,9) default 0, /*SP Unused + unassigned VMDKs*/ /*Implemented*/
   capacity_storage_pool_used_distributed		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_storage_pool_unused_distributed		NUMERIC(22,9) default 0, /*Implemented*/
   capacity_unused_VMDK_distributed			NUMERIC(22,9) default 0, /*Implemented with snapshots not handled*/ 
   capacity_NAS_storage_pool_distributed		NUMERIC(22,9) default 0,
   capacity_NAS_storage_pool_used_distributed		NUMERIC(22,9) default 0,
   capacity_NAS_storage_pool_unused_distributed		NUMERIC(22,9) default 0,
   capacity_storage_pool_NON_RDM_used_distributed	NUMERIC(22,9) default 0, /*Not Exposed.*/
 
   sp_overhead_distributed				NUMERIC(22,9) default 0,

   /*Zones*/	
   capacity_exported_to_VMs_distributed			NUMERIC(22,9) default 0, /*For Global Zones*/
   capacity_exported_LUNs_distributed			NUMERIC(22,9) default 0,
   capacity_exported_LUNs_consumed_distributed		NUMERIC(22,9) default 0,
   capacity_exported_Volumes_distributed		NUMERIC(22,9) default 0,
   capacity_exported_FSs_distributed			NUMERIC(22,9) default 0,

   /*****************************************************/
   exchange_capacity					NUMERIC(22,9) default 0,
   /*****************************************************/
   capacity_unsupported_allocated 			NUMERIC(22,9) default 0, /*Block FCDisk, NCDisk for waterfall*/
   capacity_unsupported_allocated_distributed		NUMERIC(22,9) default 0, /*Block FCDisk, NCDisk for waterfall*/
   /*****************************************************/
   /*Workaround for size 0 LUN issue.*/
   capacity_filesystem_ignore_size_zero			NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_filesystem_used_ignore_size_zero		NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_unexported_filesystem_used_ignore_size_zero	NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_filesystem_available_ignore_size_zero	NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_filesystem_used_by_apps_ignore_size_zero	NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_filesystem_used_by_non_apps_ignore_size_zero NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_unexported_filesystem_used_by_non_apps_ignore_size_zero NUMERIC(22,9) default 0, /*Not Exposed: For percent host*/
   capacity_exported_FSs_ignore_size_zero		NUMERIC(22,9) default 0,
   /*********************** NAS ******************************/
   NAS_allocated					NUMERIC(22,9) default 0,
   NAS_consumed						NUMERIC(22,9) default 0,
   NAS_allocated_distributed				NUMERIC(22,9) default 0,	
   NAS_consumed_distributed				NUMERIC(22,9) default 0,
   /*****************************************************/
   /*********************** ASM ******************************/
   capacity_dbvm					NUMERIC(22,9) default 0,
   capacity_dbvm_used					NUMERIC(22,9) default 0,
   capacity_dbvm_free					NUMERIC(22,9) default 0,	
   capacity_dbvm_overhead				NUMERIC(22,9) default 0,
   capacity_apps_on_dbvm				NUMERIC(22,9) default 0,
   capacity_apps_on_dbvm_other				NUMERIC(22,9) default 0,
   capacity_dbvm_db_used				NUMERIC(22,9) default 0, 
   capacity_dbvm_db_used_by_other			NUMERIC(22,9) default 0,
   capacity_dbvm_unknown				NUMERIC(22,9) default 0,
   capacity_dbvm_distributed				NUMERIC(22,9) default 0,
   capacity_dbvm_used_distributed			NUMERIC(22,9) default 0,
   capacity_dbvm_free_distributed			NUMERIC(22,9) default 0,	
   capacity_dbvm_overhead_distributed			NUMERIC(22,9) default 0,
   capacity_apps_on_dbvm_distributed			NUMERIC(22,9) default 0,
   capacity_dbvm_db_used_distributed			NUMERIC(22,9) default 0, 
   /*****************************************************/
   batch_id 			int default 0,
   primary key(ccstor_unique_identifier)
);
grant select on sahara.sahara_host_facts to public;




/*
Table for Host Array Capacity facts
*/
create table sahara.host_array_capacity (
  host_id                      		unsigned int,
  host_ccstor_unique_identifier		varchar(256),
  array_id                     		unsigned int,
  array_ccstor_unique_identifier	varchar(256),
  allocated_capacity           		NUMERIC(22,9) default 0,
  claimed_capacity             		NUMERIC(22,9) default 0,
  unknown_capacity            		NUMERIC(22,9) default 0, // (claimed_capacity - (assignedToVM_capacity + assignedToDB_capacity))
  unclaimed_capacity        		NUMERIC(22,9) default 0, 
  
  assignedVM_capacity        		bigint default 0,
  assignedFS_capacity        		bigint default 0,
  assignedDB_capacity			bigint default 0,

  batch_id 							int default 0,
  primary key(host_ccstor_unique_identifier, array_ccstor_unique_identifier)
);
grant select on sahara.host_array_capacity to public;

/*
Table for all luns to host mapping.
*/
create table sahara.sahara_all_luns_to_host
(
	lun_oid	varchar(256),
	host_oid varchar(256),
	primary key(lun_oid,host_oid)
);

create index INDEX_1 on sahara.sahara_all_luns_to_host (
       host_oid ASC
);

grant select on sahara.sahara_all_luns_to_host to public;

/*
Get Host Dimension.
*/
set @tablename = 'sahara.sahara_host_dimension';
set @attributenames = 'ccstor_unique_identifier,display_name,os_details,os_type_str,is_host_user_created, is_virtual,guest_os_type_str,ip_address,VirtualizationTechnology,sf_version, mh_version, MH_Kit_Differntiator';
insert into sahara.sahara_host_dimension
(	
	ccstor_unique_identifier,
	display_name,
	os_details,
	os_type_str,
	/*is_host_user_created,
	is_server_virtualized,*/
	is_virtual,
    guest_os_type_str,
	ip_address,
	VirtualizationTechnology,
    is_guestos_installed,
    sf_version, 
    mh_version,
    MH_Kit_Differntiator,
    vm_host_unique_identifier /*Required for URL changes*/,
    control_guest,
    configuration_type
) on existing update

select 
host.id as host_id,
coalesce(host.name,host.id) as display_name,
replace((host.Platform+' '+host.os_version+' '+host.arch), '"', '$DQ$') as os_details,
replace(host.Platform, '"', '$DQ$') as os_type_str,
host.is_virtual,
isnull((if host.is_virtual = 1 then os_type_str endif),'N/A') as guest_os_type_str,
isnull(host.ip,'UNKNOWN') as IP,
isnull(host.VMM,'N/A') as VirtualizationTechnology, /*Here not populated for Virtual machines without agent */
isnull(
(if host.is_virtual = 1 then 
(if (host.VMM IS NOT NULL and host.VMM = 'VMWARE') and host.ip is null then 0 else 1 endif) 
else 0
endif)
,0) 
as is_guestos_installed,
(if (sf_version is null OR sf_version = '') then 'N/A' else sf_version endif) as sf_version, isnull(mh_version,'N/A'),
(if mh_version is null or mh_version in ('N/A','-') then 'N/A' else
    if HABDBSYNC.fn_convert_version_to_number(mh_version)>4010970 then 'Post 4.0 GA' else 'Pre 4.0 GA' endif endif) as MH_Kit_Differntiator,
host.id as vm_host_unique_identifier,
(if host.vm_type is not null and host.vm_type = 'VIOS' then  'T' else 'F' endif) as cntl_gst,
isnull((if c.type = 'VMAPI' then 'VSERVER' else c.type endif),'VSERVER')
from 
HABDBSYNC.SF_VIEW_ALL_HOSTS_S as host
left outer join habdbsync.HOST_VIEW_CONFIG_MAPPING_R c
on host.id=c.object_id
;

/*
Populate Correlation table LUN to Host
Copy CCStor unique identifiers for luns and hosts.
*/
set @tablename='sahara.sahara_all_luns_to_host';
set @attributenames = 'lun_oid,host_oid';
insert into sahara.sahara_all_luns_to_host (lun_oid, host_oid) on existing update
select lm.ldev_id, hba.host_id
from habdbsync.sf_view_hba_port_r hba
     inner join habdbsync.LUN_VIEW_MAPPING_R lm on hba.id = lm.hba_port
union 
select id, host_id from habdbsync.SF_VIEW_DEVICEPATH_R where type = 'DISK'
union ALL /*VMDK to Host*/
select VMDK.id, host.id
from HABDBSYNC.VT_VIEW_VDISK_R VMDK
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on VMDK.id = da.source_id and da.source_type in ('VDISK','DISK','FILE')
join HABDBSYNC.SF_VIEW_ALL_HOSTS_S host on da.dest_host_id = host.vm_id
where VMDK.id not in
(select obj.id
	       from HABDBSYNC.VT_VIEW_VDISK_R obj 	
            join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on obj.id = da.source_id and da.source_type in ('VDISK','DISK','FILE')
	    join habdbsync.SF_VIEW_UI_DISK disk on disk.id = da.dest_id and da.dest_type = 'DISK'
);
--union ALL /*VMDK to Host*/
--select VMDK.id, host.id
--from HABDBSYNC.VT_VIEW_VDISK_R VMDK
--join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on VMDK.id = da.source_id and da.source_type != 'VDISK' and da.dest_type = 'DISK'
--join HABDBSYNC.SF_VIEW_ALL_HOSTS_S host on da.dest_host_id = host.vm_id ;

/****************************************************************/
/*RDM VMDK->ESX LUN to be associated with Virtual Machine Host and LUN_TO_HOST tuple to be deleted for RDM VMDK*/
set @attributenames = 'ESX RDM LUN->lun_oid,host_oid';
insert into sahara.sahara_all_luns_to_host (lun_oid, host_oid) on existing update
select VMDK.object_id, host.host_oid
from HABDBSYNC.VT_VIEW_VDISK_R VMDK
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on VMDK.id = da.source_id and da.source_type in  ('VDISK','DISK')
join sahara.sahara_all_luns_to_host host on isnull(da.dest_id,VMDK.id) = host.lun_oid
where VMDK.object_id is not null and is_raw=1;


set @attributenames = 'Delete RDM LUNs';
delete from sahara.sahara_all_luns_to_host
where lun_oid in
(select isnull(da.dest_id, VMDK.id) as VMDK_OID
from HABDBSYNC.VT_VIEW_VDISK_R VMDK
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on VMDK.id = da.source_id and da.source_type in ('VDISK','DISK')
where VMDK.object_id is not null and is_raw=1
and VMDK_OID<>VMDK.object_id
);
/****************************************************************/


/* LUN FACT FOR VMDK*/
set @tablename='sahara.sahara_lun_facts';
set @attributenames = 'ccstor_unique_identifier, no_of_allocated_hosts';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, no_of_allocated_hosts) on existing update
select lundim.ccstor_unique_identifier, count(hostdim.ccstor_unique_identifier) as cnt
from sahara.sahara_host_dimension as hostdim
     join sahara.sahara_all_luns_to_host as lunhost on hostdim.ccstor_unique_identifier=lunhost.host_oid
     join sahara.sahara_lun_dimension as lundim on lundim.ccstor_unique_identifier=lunhost.lun_oid     
where (lundim.type = 'VDISK')  OR (lundim.type = 'Disk' AND lundim.sub_type = 'RAW')
group by lundim.ccstor_unique_identifier;

set @attributenames = 'ccstor_unique_identifier, no_of_allocated_VMs';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, no_of_allocated_VMs) on existing update
select lundim.ccstor_unique_identifier, count(hostdim.ccstor_unique_identifier) as cnt
from sahara.sahara_host_dimension as hostdim
     join sahara.sahara_all_luns_to_host as lunhost on hostdim.ccstor_unique_identifier=lunhost.host_oid
     join sahara.sahara_lun_dimension as lundim on lundim.ccstor_unique_identifier=lunhost.lun_oid     
where (lundim.type = 'VDISK')  OR (lundim.type = 'Disk' AND lundim.sub_type = 'RAW')
and hostdim.is_virtual=1
group by lundim.ccstor_unique_identifier;


/*
Populate is_server_virtualized 
*/
set @tablename = 'sahara.sahara_host_dimension';
set @attributenames = 'is_server_virtualized';
insert into sahara.sahara_host_dimension
(	ccstor_unique_identifier,
	is_server_virtualized
) on existing update
select 
VSERVER.id,
1 as is_server_virtualized
FROM 
HABDBSYNC.SF_VIEW_HOST_CONTAINER_R VSERVER
join sahara.sahara_host_dimension hd on VSERVER.id = hd.ccstor_unique_identifier
where Type = 'VSERVER';

/*
Populate parent_virtualization_server 
*/
set @attributenames = 'parent_virtualization_server,parent_virtualization_server_oid,VirtualizationTechnology';
insert into sahara.sahara_host_dimension
(	ccstor_unique_identifier,
	parent_virtualization_server,
	parent_virtualization_server_oid,
    VirtualizationTechnology
) on existing update
select 
h.id,
VSERVER.Name as parent_virtualization_server,
HC_ID, 
sub_type as virtTech
from HABDBSYNC.VM_VIEW_HC_REL_R HTOHC
join HABDBSYNC.SF_VIEW_HOST_CONTAINER_R VSERVER ON HTOHC.HC_ID=VSERVER.ID and VSERVER.TYPE = 'VSERVER'
join HABDBSYNC.SF_VIEW_ALL_HOSTS_S h on HTOHC.VM_ID  = h.vm_id
join sahara.sahara_host_dimension hd on h.id = hd.ccstor_unique_identifier
;


set @attributenames = 'VirtualizationTechnology';
update sahara.sahara_host_dimension set VirtualizationTechnology = 'Solaris Zones'
where VirtualizationTechnology in ('ZONES','ZONE')
;
update sahara.sahara_host_dimension set VirtualizationTechnology = 'VMware'
where VirtualizationTechnology = 'vmware'
;

update sahara.sahara_host_dimension set VirtualizationTechnology = 'LPAR'
where VirtualizationTechnology = 'LPAR'
;

update sahara.sahara_host_dimension set VirtualizationTechnology = 'LDOM'
where VirtualizationTechnology = 'LDOM'
;

/*
ER LICENSING NEEDS TO SKIP HOSTS WHOSE AGENT ARE EITHER STOPPED OR DETACHED

In addition to MIGA hosts, in CCS 5.1 one more condition is introduced for checking Agent condition.
One more host dimension is added called as IS_ACTIVE, this flag will get updated as F' if the agent
condition is stopped or detached
*/
--set @attributenames = 'is_active';
/*insert into sahara.sahara_host_dimension
(	ccstor_unique_identifier,
	is_active
) on existing update
select sahara_host_dimension.host_id,1 
from sahara.sahara_host_dimension
join halchm.hal_host on hal_host._objectlink=sahara_host_dimension.ccstor_unique_identifier
join halchm.agent on hal_host.objectkey=agent.HostOsUuid
where AgentType in ('HALLocalManager','RHExplorer','MHExplorer','RHExplorerManual') and agent.agentcondition not in (7,9);
*/
/* Insert Cluster Name custom attribute - this is required to satisfy cluster based chargeback requirement*/
set @attributenames = 'cluster_name,cluster_version,cluster_type,is_clustered';

insert into sahara.sahara_host_dimension
(
ccstor_unique_identifier,
cluster_name,
cluster_version,
cluster_type,
is_clustered
) on existing update
SELECT
    h.id,
    hc.name as cluster_name,
    isnull(hc.version,'UNKNOWN') as cluster_version,
    isnull(hc.ha_type,'UNKNOWN') as cluster_type,
    1 as is_clustered
FROM
    HABDBSYNC.SF_VIEW_HOST_CONTAINER_R as hc
    JOIN HABDBSYNC.SF_VIEW_HOST_TO_HC_R as hc2h ON hc.ID=hc2h.HC_ID
    JOIN HABDBSYNC.SF_VIEW_HOST_R as h ON hc2h.HOST_ID=h.ID
    JOIN sahara.sahara_host_dimension on sahara_host_dimension.ccstor_unique_identifier = h.ID
WHERE 
    hc.TYPE='CLUSTER';

-- no_of_claimed_hosts
set @tablename = 'sahara.sahara_lun_facts';
set @attributenames='no_of_claimed_hosts';

/*Claimed count for non-RDM*/
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, lun_oid, no_of_claimed_hosts) on existing update
select l.ccstor_unique_identifier, l.lun_oid, 
       coalesce(lm.count,0) as no_of_claimed_hosts
from sahara.sahara_lun_dimension as l
left outer join (select id, count(distinct host.ccstor_unique_identifier) as count
                      from habdbsync.SF_VIEW_DEVICEPATH_R dp
                           inner join sahara.sahara_host_dimension host on host.ccstor_unique_identifier = dp.host_id
                      where type = 'DISK'
                      group by id) as lm on l.ccstor_unique_identifier=lm.id
		      where (l.sub_type is null OR l.sub_type<>'RAW');

/*Claimed count for RDM*/
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, lun_oid, no_of_claimed_hosts) on existing update
select l.ccstor_unique_identifier, l.lun_oid, count(distinct LH.host_id) as count
from sahara.sahara_lun_dimension as l
join
(
select VMDK.object_id as id, da.dest_host_id as host_id
from habdbsync.SF_VIEW_DEVICEPATH_R dp
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on da.source_type = 'VDISK' and da.dest_id = dp.id and dp.type = 'DISK'
			   join HABDBSYNC.VT_VIEW_VDISK_R VMDK on VMDK.id = da.source_id
where VMDK.object_id is not null
UNION ALL
select dp.id, dp.host_id
from habdbsync.SF_VIEW_DEVICEPATH_R dp
join HABDBSYNC.VT_VIEW_VDISK_R VMDK on VMDK.object_id = dp.id
where dp.type = 'DISK'
) as LH on l.ccstor_unique_identifier=LH.id
/*join sahara.sahara_host_dimension host on host.ccstor_unique_identifier =  LH.host_id*/
group by l.ccstor_unique_identifier, l.lun_oid
;

set @attributenames='no_of_claimed_VMs';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, lun_oid, no_of_claimed_VMs) on existing update
select l.ccstor_unique_identifier, l.lun_oid, count(distinct LH.host_id) as count
from sahara.sahara_lun_dimension as l
join
(
select VMDK.object_id as id, da.dest_host_id as host_id
from habdbsync.SF_VIEW_DEVICEPATH_R dp
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on da.source_type = 'VDISK' and da.dest_id = dp.id and dp.type = 'DISK'
			   join HABDBSYNC.VT_VIEW_VDISK_R VMDK on VMDK.id = da.source_id
where VMDK.object_id is not null
) as LH on l.ccstor_unique_identifier=LH.id
/*join sahara.sahara_host_dimension host on host.ccstor_unique_identifier =  LH.host_id*/
group by l.ccstor_unique_identifier, l.lun_oid
;

set @attributenames='no_of_allocated_hosts';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, lun_oid, no_of_allocated_hosts) on existing update
select l.ccstor_unique_identifier, l.lun_oid, 
       coalesce(lm.count,0) as no_of_claimed_hosts
from sahara.sahara_lun_dimension as l
     left outer join (select id, count(distinct host.ccstor_unique_identifier) as count
                      from habdbsync.SF_VIEW_DEVICEPATH_R dp
                           inner join sahara.sahara_host_dimension host on host.ccstor_unique_identifier = dp.host_id
                      where type = 'DISK'
                      group by id) as lm on l.ccstor_unique_identifier=lm.id
where l.vom_discover_type = 1
;

-- no_of_allocated_hosts
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, lun_oid, no_of_allocated_hosts) on existing update
select l.ccstor_unique_identifier, l.lun_oid, 
       fact.no_of_claimed_hosts+coalesce(lm.count,0) as no_of_allocated_hosts
from sahara.sahara_lun_dimension as l
     inner join sahara.sahara_lun_facts fact on l.ccstor_unique_identifier = fact.ccstor_unique_identifier
     left outer join (select lm.ldev_id, count(distinct host.ccstor_unique_identifier) as count
                      from habdbsync.LUN_VIEW_MAPPING_R  lm
                           inner join habdbsync.SF_VIEW_HBA_PORT_R hba on lm.hba_port = hba.id
                           inner join sahara.sahara_host_dimension host on host.ccstor_unique_identifier = hba.host_id
                      group by lm.ldev_id) as lm on l.ccstor_unique_identifier=lm.ldev_id and l.is_claimed = 1
		      where l.type<>'VDISK' AND (l.sub_type is null OR l.sub_type<>'RAW');

set @tablename = 'sahara.sahara_host_facts';
set @attributenames = 'Basic capacities';

if ( @si_installed = 0 ) then

insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_allocated,
                        capacity_total_allocated,
                        capacity_allocated_consumed,
                        capacity_total_allocated_consumed,
                        capacity_fc_fibre_allocated,                                                
                         capacity_ISCSI_allocated,
                        capacity_DAS_allocated,
                        capacity_IDE_allocated ,
                        capacity_claimed,
                        capacity_claimed_consumed,
                        capacity_fc_fibre_claimed,                    
                        capacity_ISCSI_claimed,
                        capacity_allocated_distributed,
                        capacity_total_allocated_distributed,
                        capacity_fc_fibre_allocated_distributed,
                        capacity_ISCSI_allocated_distributed,
                        capacity_claimed_distributed,
                        capacity_fc_fibre_claimed_distributed,
                        capacity_ISCSI_claimed_distributed,
                        capacity_allocated_consumed_distributed,
                        capacity_total_allocated_consumed_distributed,
                        capacity_claimed_consumed_distributed,
			capacity_unsupported_allocated,
			capacity_unsupported_allocated_distributed,
			capacity_thin_subscribed, 
			capacity_thin_consumed,
			capacity_thin_subscribed_distributed, 
			capacity_thin_consumed_distributed
                        )                                                
                        
on existing update

select distinctLUN_HOST.host_id, 
sum((if disk.is_VMDK_snapshot=1 then disk.consumed_lun_size else disk.lun_size endif)) as capacity_allocated,
capacity_allocated as capacity_total_allocated,
sum(if thin_status = 'ThinLun' then disk.consumed_lun_size else disk.lun_size endif) as capacity_allocated_consumed, 
capacity_allocated_consumed as capacity_total_allocated_consumed, 
sum(if port_type='FC_HBA' then disk.lun_size else 0 endif) as capacity_fc_fibre_allocated,
sum(if port_type='Initiator' then disk.lun_size else 0 endif) as capacity_ISCSI_allocated,
sum(if (port_type is null or port_type= '') and (disk.storage_technology<>'NAS') then disk.lun_size else 0 endif) as capacity_DAS_allocated,
 /*I don't think we can exactly say whether a disk is IDE. Will following be same as DAS?*/
//sum(if disk.enc_ccstor_unique_identifier is null then disk.lun_size else 0 endif) as capacity_IDE_allocated,
capacity_DAS_allocated as capacity_IDE_allocated,
capacity_allocated as capacity_claimed,
capacity_allocated_consumed as capacity_claimed_consumed,
capacity_fc_fibre_allocated as capacity_fc_fibre_claimed,
capacity_ISCSI_allocated as capacity_ISCSI_claimed,
/*unclaimed will be 0*/
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if disk.is_VMDK_snapshot=1 then disk.consumed_lun_size else disk.lun_size endif) / no_of_allocated_hosts endif) as capacity_allocated_distributed,
capacity_allocated_distributed as capacity_total_allocated_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if port_type='FC_HBA' then disk.lun_size / no_of_allocated_hosts else 0 endif)endif) as capacity_fc_fibre_allocated_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if port_type='Initiator' then disk.lun_size / no_of_allocated_hosts else 0 endif)endif) as capacity_ISCSI_allocated_distributed,
capacity_allocated_distributed as capacity_claimed_distributed,
capacity_fc_fibre_allocated_distributed as capacity_fc_fibre_claimed_distributed,
capacity_ISCSI_allocated_distributed as capacity_ISCSI_claimed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if thin_status = 'ThinLun' then disk.consumed_lun_size / no_of_allocated_hosts else disk.lun_size / no_of_allocated_hosts endif)endif) as capacity_allocated_consumed_distributed, 
capacity_allocated_consumed_distributed as capacity_total_allocated_consumed_distributed,
capacity_allocated_consumed_distributed as capacity_claimed_consumed_distributed, 
sum(if subsys_id in ('NCDISK', 'FCDISK') then disk.lun_size else 0 endif) as capacity_unsupported_allocated,
sum(if subsys_id in ('NCDISK', 'FCDISK') then (disk.lun_size / no_of_allocated_hosts) else 0 endif) as capacity_unsupported_allocated_distributed,
sum(if thin_status = 'ThinLun' then disk.lun_size else 0 endif) as capacity_thin_subscribed, 
sum(if thin_status = 'ThinLun' then disk.consumed_lun_size else 0 endif) as capacity_thin_consumed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if thin_status = 'ThinLun' then disk.lun_size / no_of_allocated_hosts else 0 endif)endif) capacity_thin_subscribed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if thin_status = 'ThinLun' then disk.consumed_lun_size / no_of_allocated_hosts else 0 endif)endif) as capacity_thin_consumed_distributed
from 
(
    select LUN_HOST.host_id, LUN_HOST.LDEV_ID, list(distinct LUN_HOST.port_type) as port_type, list(distinct LUN_HOST.subsys_id) as subsys_id
    from
    (
    /*Get Claimed LUNs*/
    select  h_port.host_id, parent as LDEV_ID, h_port.type as port_type, '' as subsys_id
    from HABDBSYNC.sf_view_devicepath_r dp 
    join HABDBSYNC.sf_view_hba_port_r as h_port on dp.hba_port = h_port.id 
    where dp.type = 'path'
    UNION
    select dp.host_id, disk.id as LDEV_ID, null as port_type, disk.subsys_id from habdbsync.SF_VIEW_DEVICEPATH_R dp
    join habdbsync.sf_view_disk_r disk on dp.id = disk.id
    where dp.type = 'DISK' and (disk.subsys_id is null or disk.subsys_id in ('NCDISK', 'FCDISK', 'DISK'))
    ) as LUN_HOST
    group by LUN_HOST.host_id, LUN_HOST.LDEV_ID
) as distinctLUN_HOST
join 
    ( /* Get selected LUNs */
    select d.ccstor_unique_identifier, d.enc_ccstor_unique_identifier, lun_size, 
    isnull((if d.sub_type = 'RAW' then 1 else 0 endif),0) as is_raw,
    isnull((if d.type = 'VDISK' and d.sub_type = 'SPARSE' then 1 else 0 endif),0) as is_VMDK_snapshot,
    (if is_raw=1 then (if no_of_allocated_VMs = 0 then 1 else no_of_allocated_VMs endif) else (if no_of_allocated_hosts = 0 then 1 else no_of_allocated_hosts endif) endif) as no_of_allocated_hosts, /*Defensive check*/
    (if is_raw=1 then (if no_of_claimed_VMs = 0 then 1 else no_of_claimed_VMs endif) else (if no_of_claimed_hosts = 0 then 1 else no_of_claimed_hosts endif) endif) as no_of_claimed_hosts, /*Defensive check*/
    thin_status, consumed_lun_size, storage_technology
    from sahara.sahara_lun_dimension d 
    join sahara.sahara_lun_facts f on d.ccstor_unique_identifier = f.ccstor_unique_identifier
    where resource_type = '0x1' /* Considering data devices*/
    and is_secondary = 0
    and thin_status != 'ThinSourceLun'
    and is_administrative = 0
    ) disk on distinctLUN_HOST.LDEV_ID = disk.ccstor_unique_identifier
JOIN sahara.sahara_host_dimension hd on hd.ccstor_unique_identifier = distinctLUN_HOST.host_id
group by distinctLUN_HOST.host_id;

else

insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_allocated,
                        capacity_total_allocated,
                        capacity_allocated_consumed,
                        capacity_total_allocated_consumed,
                        capacity_fc_fibre_allocated,                                                
                        capacity_ISCSI_allocated,
                        capacity_DAS_allocated,
                        capacity_IDE_allocated ,
                        capacity_claimed,
                        capacity_claimed_consumed,
                        capacity_fc_fibre_claimed,                    
                        capacity_ISCSI_claimed,                        
                        capacity_unclaimed,
                        capacity_unclaimed_consumed,
                        capacity_thin_subscribed, 
                        capacity_thin_consumed,
                        capacity_allocated_distributed,
                        capacity_total_allocated_distributed,
                        capacity_fc_fibre_allocated_distributed,
                        capacity_ISCSI_allocated_distributed,
                        capacity_claimed_distributed,
                        capacity_fc_fibre_claimed_distributed,
                        capacity_ISCSI_claimed_distributed,
                        capacity_unclaimed_distributed,
                        capacity_thin_subscribed_distributed,
                        capacity_thin_consumed_distributed,
                        capacity_allocated_consumed_distributed,
                        capacity_total_allocated_consumed_distributed,
                        capacity_claimed_consumed_distributed,
                        capacity_unclaimed_consumed_distributed,
			capacity_unsupported_allocated,
			capacity_unsupported_allocated_distributed)
on existing update

select distinctLUN_HOST.host_id, 
sum((if disk.is_VMDK_snapshot=1 then disk.consumed_lun_size else disk.lun_size endif)) as capacity_allocated,
capacity_allocated as capacity_total_allocated,
sum(if thin_status = 'ThinLun' then disk.consumed_lun_size else disk.lun_size endif) as capacity_allocated_consumed,
capacity_allocated_consumed as capacity_total_allocated_consumed,
sum(if port_type='FC_HBA' then disk.lun_size else 0 endif) as capacity_fc_fibre_allocated,
sum(if port_type='Initiator' then disk.lun_size else 0 endif) as capacity_ISCSI_allocated,
sum(if (port_type is null or port_type= '') and (disk.storage_technology<>'NAS') then disk.lun_size else 0 endif) as capacity_DAS_allocated,
 /*I don't think we can exactly say whether a disk is IDE. Will following be same as DAS?*/
//sum(if disk.enc_ccstor_unique_identifier is null then disk.lun_size else 0 endif) as capacity_IDE_allocated,
capacity_DAS_allocated as capacity_IDE_allocated,
sum(if charindex('1', isUnClaimed)=0 then disk.lun_size else 0 endif) as capacity_claimed,
sum(if charindex('1', isUnClaimed)=0 then 
        if thin_status = 'ThinLun' 
            then disk.consumed_lun_size 
            else disk.lun_size     
        endif 
    else 0 
endif
) as capacity_claimed_consumed,
sum(if charindex('1', isUnClaimed)=0 and port_type='FC_HBA' then disk.lun_size else 0 endif) as capacity_fc_fibre_claimed,
sum(if charindex('1', isUnClaimed)=0 and port_type='Initiator' then disk.lun_size else 0 endif) as capacity_ISCSI_claimed,
sum(if charindex('1', isUnClaimed)<>0 then disk.lun_size else 0 endif) as capacity_unclaimed,
sum(if charindex('1', isUnClaimed)<>0 then 
        if thin_status = 'ThinLun' 
            then disk.consumed_lun_size 
            else disk.lun_size     
        endif 
    else 0 
endif) as capacity_unclaimed_consumed,
sum(if thin_status = 'ThinLun' then disk.lun_size else 0 endif) as capacity_thin_subscribed, 
sum(if thin_status = 'ThinLun' then disk.consumed_lun_size else 0 endif) as capacity_thin_consumed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if disk.is_VMDK_snapshot=1 then disk.consumed_lun_size else disk.lun_size endif)/no_of_allocated_hosts endif) as capacity_allocated_distributed,
capacity_allocated_distributed as capacity_total_allocated_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if port_type='FC_HBA' then disk.lun_size / no_of_allocated_hosts else 0 endif)endif) as capacity_fc_fibre_allocated_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if port_type='Initiator' then disk.lun_size / no_of_allocated_hosts else 0 endif)endif) as capacity_ISCSI_allocated_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if charindex('1', isUnClaimed)=0 then disk.lun_size / no_of_claimed_hosts else 0 endif)endif) as capacity_claimed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if charindex('1', isUnClaimed)=0 and port_type='FC_HBA' then disk.lun_size / no_of_claimed_hosts else 0 endif)endif) as capacity_fc_fibre_claimed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if charindex('1', isUnClaimed)=0 and port_type='Initiator' then disk.lun_size / no_of_claimed_hosts else 0 endif)endif) as capacity_ISCSI_claimed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if charindex('1', isUnClaimed)<>0 then disk.lun_size / no_of_allocated_hosts else 0 endif)endif) as capacity_unclaimed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if thin_status = 'ThinLun' then disk.lun_size / no_of_allocated_hosts else 0 endif)endif) capacity_thin_subscribed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if thin_status = 'ThinLun' then disk.consumed_lun_size / no_of_allocated_hosts else 0 endif)endif) as capacity_thin_consumed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if thin_status = 'ThinLun' then disk.consumed_lun_size / no_of_allocated_hosts else disk.lun_size / no_of_allocated_hosts endif)endif) as capacity_allocated_consumed_distributed,
capacity_allocated_consumed_distributed as capacity_total_allocated_consumed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if charindex('1', isUnClaimed)=0 then 
        if thin_status = 'ThinLun' 
            then disk.consumed_lun_size / no_of_claimed_hosts
            else disk.lun_size / no_of_claimed_hosts
        endif 
    else 0 
endif)endif
) as capacity_claimed_consumed_distributed,
sum(if (disk.is_raw = 1 and hd.is_server_virtualized = 1) then 0 else (if charindex('1', isUnClaimed)<>0 then 
        if thin_status = 'ThinLun' 
            then disk.consumed_lun_size / no_of_allocated_hosts
            else disk.lun_size / no_of_allocated_hosts
        endif 
    else 0 
endif)endif) as capacity_unclaimed_consumed_distributed,
sum(if subsys_id in ('NCDISK', 'FCDISK') then disk.lun_size else 0 endif) as capacity_unsupported_allocated,
sum(if subsys_id in ('NCDISK', 'FCDISK') then (disk.lun_size / no_of_allocated_hosts) else 0 endif) as capacity_unsupported_allocated_distributed
from 
(
    select LUN_HOST.host_id, LUN_HOST.LDEV_ID, list(distinct LUN_HOST.port_type) as port_type, 
    isnull(list(isUnClaimed),'0') as isUnClaimed, list(distinct LUN_HOST.subsys_id) as subsys_id
    from
    (
    /*Get Unclaimed LUNs*/
    select hbaport.host_id, map.LDEV_ID, hbaport.type as port_type, 1 as isUnClaimed, subsys_id
    from HABDBSYNC.sf_view_hba_port_r hbaport
    join HABDBSYNC.LUN_VIEW_MAPPING_R map on map.hba_port = hbaport.id    /*Only available in SI Add-On*/
    where map.ldev_id not in (select id from habdbsync.SF_VIEW_DISK_R disk)
    UNION ALL
    /*Get Claimed LUNs*/
    select  h_port.host_id, parent as LDEV_ID, h_port.type as port_type, 0 as isUnClaimed, '' as subsys_id
    from HABDBSYNC.sf_view_devicepath_r dp 
    join HABDBSYNC.sf_view_hba_port_r as h_port on dp.hba_port = h_port.id 
    where dp.type = 'path'
    UNION
    select dp.host_id, disk.id as LDEV_ID, null as port_type, 0 as isUnClaimed, disk.subsys_id from habdbsync.SF_VIEW_DEVICEPATH_R dp
    join habdbsync.sf_view_disk_r disk on dp.id = disk.id
    where dp.type = 'DISK' and (disk.subsys_id is null or disk.subsys_id in ('NCDISK', 'FCDISK', 'DISK'))
    ) as LUN_HOST
    group by LUN_HOST.host_id, LUN_HOST.LDEV_ID
) as distinctLUN_HOST
join 
    ( /* Get selected LUNs */
    select d.ccstor_unique_identifier, d.enc_ccstor_unique_identifier, lun_size, isnull(thin_status,'') as thin_status, consumed_lun_size, isnull((if d.sub_type = 'RAW' then 1 else 0 endif),0) as is_raw,
    isnull((if d.type = 'VDISK' and d.sub_type = 'SPARSE' then 1 else 0 endif),0) as is_VMDK_snapshot,
    (if is_raw=1 then (if no_of_allocated_VMs = 0 then 1 else no_of_allocated_VMs endif) else (if no_of_allocated_hosts = 0 then 1 else no_of_allocated_hosts endif) endif) as no_of_allocated_hosts, /*Defensive check*/
    (if is_raw=1 then (if no_of_claimed_VMs = 0 then 1 else no_of_claimed_VMs endif) else (if no_of_claimed_hosts = 0 then 1 else no_of_claimed_hosts endif) endif) as no_of_claimed_hosts, /*Defensive check*/
    storage_technology
    from sahara.sahara_lun_dimension d 
    join sahara.sahara_lun_facts f on d.ccstor_unique_identifier = f.ccstor_unique_identifier
    where resource_type = '0x1' /* Considering data devices*/
    and is_secondary = 0
    and thin_status != 'ThinSourceLun'
    and is_administrative = 0
    ) disk on distinctLUN_HOST.LDEV_ID = disk.ccstor_unique_identifier
JOIN sahara.sahara_host_dimension hd on hd.ccstor_unique_identifier = distinctLUN_HOST.host_id
group by distinctLUN_HOST.host_id;


set @attributenames = 'capacity_allocated_claimed_by_other_hosts,capacity_allocated_claimed_by_other_hosts_consumed';

insert into sahara.sahara_host_facts(ccstor_unique_identifier, 
                        capacity_allocated_claimed_by_other_hosts,
                        capacity_allocated_claimed_by_other_hosts_consumed)
on existing update
select claimed.host_oid, sum(claimed.lun_size), sum(claimed.consumed_lun_size) from
(
select Claimed_By_Other.host_oid, Claimed_By_Other.lun_oid, lunf.lun_size, (if isnull(luns.thin_status,'') = 'ThinLun' then lunf.consumed_lun_size else lunf.lun_size endif) as consumed_lun_size
from
(/*Host To LUN*/
select 
all_luns.host_oid, all_luns.lun_oid
from sahara.sahara_all_luns_to_host all_luns
EXCEPT    /*Claimed Host to LUN*/
    (select  h_port.host_id, parent as LDEV_ID
    from HABDBSYNC.sf_view_devicepath_r dp 
    join HABDBSYNC.sf_view_hba_port_r as h_port on dp.hba_port = h_port.id 
    where dp.type = 'path'
    UNION
    select dp.host_id, disk.id as LDEV_ID 
    from habdbsync.SF_VIEW_DEVICEPATH_R dp
    join habdbsync.sf_view_disk_r disk on dp.id = disk.id
    where dp.type = 'DISK' and (disk.subsys_id is null or disk.subsys_id in ('NCDISK', 'FCDISK', 'DISK'))
    )
) as Claimed_By_Other 
join sahara.sahara_lun_dimension as luns
on luns.ccstor_unique_identifier = Claimed_By_Other.lun_oid
join sahara.sahara_lun_facts lunf on lunf.ccstor_unique_identifier = Claimed_By_Other.lun_oid
where luns.is_secondary = 0 and luns.is_claimed = 2
) as claimed
group by claimed.host_oid; 

end if;

/*
Calculate allocated_to_hosts_directly for LUNS which are masked to discovered hosts.
Allocated lun capacity for lun
*/

/*
Calculate allocated_to_hosts_directly for LUNS which are masked to undiscovered hosts.
Allocated lun capacity for lun
*/

/*
insert into sahara.sahara_lun_facts(lun_id, allocated_to_host_directly)
on existing update
select distinct sahara_lun_facts.lun_id, sahara_lun_facts.lun_size
from sahara.sahara_lun_dimension 
join sahara.sahara_lun_facts on sahara_lun_dimension.lun_id=sahara_lun_facts.lun_id
where masking_status = 2 
    and resource_type != 2
    and thin_status != 'ThinSourceLun'
    and is_administrative = 0 
    and capacity_unallocated_to_hosts=0 
    and allocated_to_host_directly =0;

else
*/

-- calculate allocated_to_host_directly
-- (TODO: currently is update in host_data.sql)
set @attributenames='allocated_to_host_directly';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, lun_oid, allocated_to_host_directly) on existing update
select l.ccstor_unique_identifier,
       l.lun_oid,
       coalesce(lf.lun_size,0)
from sahara.sahara_lun_dimension as l
     inner join sahara.sahara_lun_facts as lf on l.ccstor_unique_identifier=lf.lun_oid
where (masking_status = 2 or l.vom_discover_type in (1,3))
      and is_segment != 1
      and resource_type != '0x2'
      and thin_status != 'ThinSourceLun'
      and is_administrative = 0;

-- calculate consumed_allocated_to_host_directly
set @attributenames='consumed_allocated_to_host_directly';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, lun_oid, consumed_allocated_to_host_directly) on existing update
select l.ccstor_unique_identifier,
       l.lun_oid,
       coalesce(lf.consumed_lun_size,0)
from sahara.sahara_lun_dimension as l
     inner join sahara.sahara_lun_facts as lf on l.ccstor_unique_identifier=lf.lun_oid
where (masking_status = 2 or l.vom_discover_type in (1,3))
      and is_segment != 1
      and resource_type != '0x2'
      and thin_status != 'ThinSourceLun'
      and is_administrative = 0;

-- Adding the shared capacity for the host facts table..
set @attributenames='adjusted_allocated_capacity';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, adjusted_allocated_capacity) on existing update
select luns.ccstor_unique_identifier,
       case when no_of_allocated_hosts = 0 then lun_size
            else (lun_size/no_of_allocated_hosts) end as distributed_allocated_capacity
from sahara.sahara_host_dimension as hostdim
     join sahara.sahara_all_luns_to_host as lunhost on hostdim.ccstor_unique_identifier=lunhost.host_oid
     join sahara.sahara_lun_facts as luns on luns.ccstor_unique_identifier = lunhost.lun_oid
     join sahara.sahara_lun_dimension as lunsd on luns.ccstor_unique_identifier = lunsd.ccstor_unique_identifier
where (sub_type is null OR sub_type<>'RAW');

/*RDM*/
set @attributenames='adjusted_allocated_capacity-RDM';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, adjusted_allocated_capacity) on existing update
select luns.ccstor_unique_identifier,
       (lun_size / count(hostdim.ccstor_unique_identifier)) as distributed_allocated_capacity
from sahara.sahara_host_dimension as hostdim
     join sahara.sahara_all_luns_to_host as lunhost on hostdim.ccstor_unique_identifier=lunhost.host_oid
     join sahara.sahara_lun_facts as luns on luns.ccstor_unique_identifier = lunhost.lun_oid
     join sahara.sahara_lun_dimension as lunsd on luns.ccstor_unique_identifier = lunsd.ccstor_unique_identifier
where sub_type='RAW' and is_server_virtualized=0 /*Just count Virtual Machines. Not ESX servers*/
group by luns.ccstor_unique_identifier, lun_size;

set @attributenames='consumed_adjusted_allocated_capacity';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, consumed_adjusted_allocated_capacity) on existing update
select luns.ccstor_unique_identifier,
       case when no_of_allocated_hosts = 0 then consumed_lun_size
            else (consumed_lun_size/no_of_allocated_hosts) end as consumed_distributed_allocated_capacity
from sahara.sahara_host_dimension as hostdim
     join sahara.sahara_all_luns_to_host as lunhost on hostdim.ccstor_unique_identifier=lunhost.host_oid
     join sahara.sahara_lun_facts as luns on luns.ccstor_unique_identifier = lunhost.lun_oid
     join sahara.sahara_lun_dimension as lunsd on luns.ccstor_unique_identifier = lunsd.ccstor_unique_identifier
where (sub_type is null OR sub_type<>'RAW');

/*RDM*/
set @attributenames='consumed_adjusted_allocated_capacity-RDM';
insert into sahara.sahara_lun_facts(ccstor_unique_identifier, consumed_adjusted_allocated_capacity) on existing update
select luns.ccstor_unique_identifier,
       (consumed_lun_size / count(hostdim.ccstor_unique_identifier)) as consumed_adjusted_allocated_capacity
from sahara.sahara_host_dimension as hostdim
     join sahara.sahara_all_luns_to_host as lunhost on hostdim.ccstor_unique_identifier=lunhost.host_oid
     join sahara.sahara_lun_facts as luns on luns.ccstor_unique_identifier = lunhost.lun_oid
     join sahara.sahara_lun_dimension as lunsd on luns.ccstor_unique_identifier = lunsd.ccstor_unique_identifier
where sub_type='RAW' and is_server_virtualized=0 /*Just count Virtual Machines. Not ESX servers*/
group by luns.ccstor_unique_identifier, consumed_lun_size;

update sahara.sahara_host_dimension set batch_id = @batch_id ;
update sahara.sahara_host_facts set batch_id = @batch_id ;

set @attributenames = '';
select count(1) into @noDims from sahara.sahara_host_dimension;
select count(1) into @noFacts from sahara.sahara_host_facts;

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_host_data', 'I','completed dimension rollup -> sahara_host_dimension: ' || @noDims || ' rows inserted';

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_host_data', 'I','completed facts rollup -> sahara_host_facts: ' || @noFacts || ' rows inserted';

insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 
select GETDATE(), @batch_id, 'sahara.sahara_host_data', 'I','roll-up batch: ' || @batch_id || ', data set: sahara_host_dimension/sahara_host_facts -> Succeeded: ' ||datediff(second,@initialtime,GETDATE()) || ' secs';

commit;

EXCEPTION
    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 ) then
            SET @ERROR_COUNT=@ERROR_COUNT + 1;
            BEGIN
                insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message)
                select GETDATE(), @batch_id, 'sahara.sahara_host_data', 'E', string (' Error failed at = ', @tablename+':'+@attributenames ) +
                                                                             string (': SQLCODE = ', @error ) +
                                                                             string (': ERRORMSG = ', @serror ) +
                                                                             string (': SQLSTATE = ', @sqlstate );
                message string (' Error failed at = ', @tablename+':'+@attributenames ) to LOG;
                message string ('          @error = ', @error ) to LOG;
                message string ('      @serrir()  = ', @serror ) to LOG;
                message string ('     @sqlstate() = ', @sqlstate ) to LOG;
            END;
        END IF;
        -- RESIGNAL;


END;
grant execute on sahara.sahara_host_data to public;
