

if exists(select 1 from sys.sysprocedure where proc_name = 'sahara_lic_data' 
    and user_name(creator) = 'sahara') 
then
   drop procedure sahara.sahara_lic_data
end if;

CREATE PROCEDURE sahara."sahara_lic_data" (@batch_id integer)
BEGIN

  /* Variables for logging purpose */
  DECLARE @initialtime timestamp; 

  /* Total number of dimension this stored procedure is inserting */
  DECLARE @noLicDims bigint; 
  DECLARE @noProdDims bigint; 
  DECLARE @noLicHostDims bigint; 
  DECLARE @noLicDeplDims bigint; 

  /* Total number of facts this stored procedure is inserting */
  DECLARE @noFacts bigint; 

  /* Log detail error info */
  declare @err_code integer;
  declare @error    integer;
  declare @serror         varchar(32767);
  declare @sqlstate       varchar(5);
  declare @tablename      varchar(128);
  declare @attributenames varchar(256);
  declare @count    integer;
  declare @error_count    integer;
  declare @dbmajorver integer;
  
  /* Store the current time */
  Set @initialtime=GETDATE();

  /* Get db major version.  fix 2941938 */
  select  major into @DBMajorVer from habdbsync.p_sf_db_version;
  
  insert into sahara.sahara_log (date_time, batch_id, procedure_name, 
	severity, log_message) 
    select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'I',
           'starting dimension/facts rollup for licensing';

  /*
   * Drop licensing related dimension/fact tables
   */
  if exists(select 1 from sys.systable 
      where table_name='sahara_sf_license_dimension' 
      and creator=user_id('sahara')) 
  then
     drop table sahara.sahara_sf_license_dimension
  end if;

  if exists(select 1 from sys.systable 
      where table_name='sahara_sf_product_dimension' 
      and creator=user_id('sahara')) 
  then
     drop table sahara.sahara_sf_product_dimension
  end if;

  if exists(select 1 from sys.systable 
      where table_name='sahara_sf_license_host_dimension' 
      and creator=user_id('sahara')) 
  then
     drop table sahara.sahara_sf_license_host_dimension
  end if;


  if exists(select 1 from sys.systable 
      where table_name='sahara_sf_deployed_licenses_fact' 
      and creator=user_id('sahara')) 
  then
     drop table sahara.sahara_sf_deployed_licenses_fact
  end if;

  if exists(select 1 from sys.systable 
      where table_name='sahara_sf_license_deployment' 
      and creator=user_id('sahara')) 
  then
     drop table sahara.sahara_sf_license_deployment
  end if;

  if exists(select 1 from sys.systable 
      where table_name='sahara_ldr_entitlement_report_view' 
      and table_type='VIEW' and creator=user_id('sahara')) 
  then
     drop view sahara.sahara_ldr_entitlement_report_view
  end if;

  if exists(select 1 from sys.systable 
      where table_name='sahara_ldr_licensable_hosts_view' 
      and table_type='VIEW' and creator=user_id('sahara')) 
  then
     drop view sahara.sahara_ldr_licensable_hosts_view
  end if;

  /*==============================================================*/
  /* Table: SF_LICENSE                                            */
  /*==============================================================*/
  create table sahara.sahara_sf_license_dimension  (
        ccstor_unique_identifier  varchar(512) ,
        type                      varchar(128) ,
        "key"                     varchar(128) ,
        is_site_license           char(1) default 'F',
        serial_number             varchar(128) ,
        is_expired                char(1) default 'F',
        is_keyless                char(1) default 'F',
        batch_id                  int default 0,
	primary key(ccstor_unique_identifier)
  );
  grant select on sahara.sahara_sf_license_dimension to public;

  /*==============================================================*/
  /* Table: SF_PRODUCT                                            */
  /*==============================================================*/
  create table sahara.sahara_sf_product_dimension  (
        ccstor_unique_identifier  varchar(512) ,
        product_name              varchar(512) ,
        short_name                varchar(512) ,
        product_category          varchar(128) default 'Unknown',
        product_edition           varchar(512) ,
        product_version           varchar(512) ,
        platform                  varchar(16) ,
        license_type              varchar(128) ,
        batch_id                  int default 0,
	primary key(ccstor_unique_identifier)
  );
  grant select on sahara.sahara_sf_product_dimension to public;

  /*==============================================================*/
  /* Table: SF_LICENSE_HOST                                       */
  /*==============================================================*/
  create table sahara.sahara_sf_license_host_dimension  (
        ccstor_unique_identifier  varchar(512) ,
        host_unique_identifier    varchar(512) ,
        id                        varchar(38) ,
        name                      varchar(255) ,
        model                     varchar(128) ,
        processor_type            varchar(64) ,
        num_processors            numeric(4) ,
        platform                  varchar(16) ,
        server_price_tier         varchar(16) ,
        processor_price_tier      varchar(16) ,
        num_processor_price_tier  varchar(16) ,
        os_price_tier             varchar(32) ,
        assigned_price_tier       varchar(16) ,
        os_edition                varchar(128) ,
        os                        varchar(128) ,
        has_enterprise_edition    char(1) default 'F',
		spvu					  numeric(10,2) default 0,
		ncore					  integer default 0,
        batch_id                  int default 0,
	primary key(ccstor_unique_identifier)
  );
  grant select on sahara.sahara_sf_license_host_dimension to public;

  /*==============================================================*/
  /* Table: SF_DEPLOYED_LICENSES fact                             */
  /*==============================================================*/
  create table sahara.sahara_sf_deployed_licenses_fact  (
        license_unique_identifier     varchar(512) ,
        product_unique_identifier     varchar(512) ,
        license_host_unique_identifier varchar(512) ,
        license_deployment_unique_identifier varchar(512) ,
        license_deployments           numeric(10),
        license_deployments_expired   numeric(10),
        accounted_deployments         numeric(10),
        accounted_deployments_expired numeric(10),
        keyless_deployments           numeric(10),
		total_spvu					  numeric(10,2) default 0,	
        batch_id                      int default 0,
	primary key(license_unique_identifier, product_unique_identifier, 
                    license_host_unique_identifier)
  );
  grant select on sahara.sahara_sf_deployed_licenses_fact to public;

  /*==============================================================*/
  /* Table: SF_LICENSE_DEPLOYMENT                                 */
  /*==============================================================*/
  create table sahara.sahara_sf_license_deployment  (
        ccstor_unique_identifier      varchar(512) ,
        license_unique_identifier     varchar(512) ,
        product_unique_identifier     varchar(512) ,
        license_host_unique_identifier varchar(512) ,
        expiry_date                   date,
        is_expired                    char(1) default 'F',
        is_accountable                char(1) default 'F',
        batch_id                  int default 0,
	primary key(license_unique_identifier, product_unique_identifier, 
                    license_host_unique_identifier)
  );
  grant select on sahara.sahara_sf_license_deployment to public;


  
  /*==============================================================*/
  /* VIEW: SAHARA_LDR_ENTITLEMENT_REPORT_VIEW                     */
  /* VIEW: SAHARA_LDR_LICENSABLE_HOSTS_VIEW                       */
  /* For older versions of VOM we need to implement this logic    */
  /*==============================================================*/
  if not exists(select 1 from sys.systable 
      where table_name='LDR_VOM_ENTITLEMENT_REPORT_VIEW' 
      and creator=user_id('HABDBSYNC')) 
  then
	CREATE VIEW sahara.sahara_ldr_entitlement_report_view as
		WITH tmp_view AS
			(SELECT HOSTID, list(product_edition) as EDITION ,list(PRODUCT_NAME) as PRODUCT_NAME ,
                CASE WHEN EDITION LIKE '%ENTERPRISE%' THEN 3 
                WHEN EDITION LIKE '%STANDARD%' THEN 2
                WHEN EDITION LIKE '%BASIC%' THEN 1
                ELSE 0 END AS LICENSE_EDITION,
                CASE WHEN PRODUCT_NAME LIKE '%Application HA%' THEN 'APPHA'
                ELSE  list(PRODUCT_CATEGORY) END AS PRODUCT_CATEGORY,
                CASE WHEN PRODUCT_CATEGORY LIKE '%Oracle RAC%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%DB Editions%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%SF CFS%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%SFW%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%VCS%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%VVR%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%VM%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%DMP%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%FS%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%SF%' then 1
                WHEN PRODUCT_CATEGORY LIKE '%APPHA%' then 1
                ELSE 0 END AS IS_SF_LICENSE
                FROM HABDBSYNC.LDR_VIEW_LDR_PRODUCT_INFO_ACCOUNTED 
                GROUP BY HOSTID)
	SELECT DISTINCT H1.HOSTID,H1.NAME, H1.PLATFORM,  H1.CONFIG_TYPE,H1.VSERVER_ID, H1.is_virtual, H1.HOST_TYPE,H1.VSERVER_NVMS,H1.IS_VSERVER,ISNULL(H1.VM_TYPE,'') as VM_TYPE,				  DISPLAY_HOST_TYPE, ISNULL(LICENSE_EDITION,0) AS LICENSE_EDITION, ISNULL(PRODUCT_CATEGORY,'NA') AS PRODUCT_CATEGORY, 
				    ISNULL(IS_SF_LICENSE,0) as IS_SF_LICENSE
	FROM HABDBSYNC.SF_VIEW_UI_HOST AS H1 LEFT OUTER JOIN tmp_view AS H2
	ON H1.ID = H2.HOSTID;

	grant select on sahara.sahara_ldr_entitlement_report_view to public;


	CREATE VIEW sahara.sahara_ldr_licensable_hosts_view as
		select HOSTID,H1.NAME,PLATFORM,CONFIG_TYPE,VSERVER_ID,IS_VIRTUAL,HOST_TYPE,VSERVER_NVMS,IS_VSERVER,VM_TYPE,ISNULL(IS_SF_LICENSE,0) as IS_SF_LICENSE,
			   LICENSE_EDITION, PRODUCT_CATEGORY, convert(integer,ISNULL(B.VALUE,0)) AS IS_ROLLEDUP,DISPLAY_HOST_TYPE,
		    CASE 
			-- Chargeable Hosts IS 1 AND not Chargeable Hosts IS O
			WHEN H1.VM_TYPE = 'VIOS' THEN 0 -- DONOT CHARGE FOR VIOS
			WHEN H1.CONFIG_TYPE IN ('VMAPI') THEN 0  -- DONOT CHARGE FOR ESX AND HMC
			WHEN H1.CONFIG_TYPE IN ('AGENTLESS', 'VSERVER') THEN 1  -- Agentless and NO Configuration
			-- VSERVER Has NO VMs AND SF basic/std
			WHEN H1.HOST_TYPE IN ('LDOM','KVM' ,'ZONE') AND VSERVER_NVMS = 0 AND IS_SF_LICENSE = 1 AND license_edition != 3 AND IS_ROLLEDUP = 1 THEN 1  
			WHEN H1.HOST_TYPE IN ('LDOM','KVM' ,'ZONE') AND VSERVER_NVMS > 0 THEN 0 -- Have VM's
			WHEN IS_SF_LICENSE = 1 AND LICENSE_EDITION = 3 then 0 -- Enterprise SF Host is always free (Virtual/physical) 
		   
			-- Non SF VM with NON SF Vserver, and is not rolling up 
			WHEN is_virtual = 1 and IS_SF_LICENSE = 0 and IS_ROLLEDUP = 0 and 
			(SELECT count(*) as CNT
				 from sahara.sahara_ldr_entitlement_report_view b
				 where b.hostid = H1.vserver_id
				and  is_sf_license = 0 ) > 0 then 1
			-- Non SF VM with any Vserver, and is rolling up
			WHEN is_virtual = 1 and IS_SF_LICENSE = 0 and IS_ROLLEDUP = 1 then 1  
	
		    -- Non SF VM with SF Vserver/SFE, and is not rolling up 
			WHEN is_virtual = 1 and IS_SF_LICENSE = 0 and IS_ROLLEDUP = 0 and 
				(SELECT count(*) as CNT
				from sahara.sahara_ldr_entitlement_report_view b
				where b.hostid = H1.vserver_id 	and  is_sf_license = 1 ) > 0 then 0 
		   
			-- Basic/std SF VM with NON SF or Basic/Std Vserver, and is rolling up 
			-- YES(Propogated)
			WHEN is_virtual = 1 and IS_SF_LICENSE = 1 and  license_edition !=3 and IS_ROLLEDUP = 1 and 
			(SELECT count(*) as CNT
				 from sahara.sahara_ldr_entitlement_report_view b
				 where b.hostid = H1.vserver_id
				and  license_edition != 3) > 0 then 1

			-- NO (Propogated) case, where VM has SF basic/std but Vserver has SF ENT.
			WHEN is_virtual = 1 and IS_SF_LICENSE = 1 and  license_edition !=3 and 
			(SELECT count(*) as CNT
				 from sahara.sahara_ldr_entitlement_report_view b
				 where b.hostid = H1.vserver_id
				and  license_edition = 3) > 0 then 3

			WHEN H1.CONFIG_TYPE IN ('AGENT') and IS_SF_LICENSE = 0 then 1 -- Physical hosts with out SF License.
			WHEN H1.CONFIG_TYPE = 'AGENT' AND LICENSE_EDITION !=3 THEN IS_ROLLEDUP -- SF Basic/Stand Hosts and Virtual Machines
			ELSE 0 END AS CHARGEABLE
		 -- Covers following
			-- Rollup = 0 , all VM's expect the NON SF VM with NON SF Vserver is free.
			-- Rollup = 1, SF Basic/Std VM with SF Enterprise VServer
			-- Rollup = 1, SF Enterprise VM with any VServer
		FROM sahara.sahara_ldr_entitlement_report_view as H1 
		left outer join HABDBSYNC.SF_VIEW_DB_DEFAULT_R B on B.NAME = 'VOM_ADVANCED';

	grant select on sahara.sahara_ldr_licensable_hosts_view to public;

  end if;

  /*
   **********************************************************************
   * Extraction Logic:
   **********************************************************************
   */

  /*Updating vom advanced flag to 1 everytime during rollup*/
  update habdbsync.sf_view_db_default_r set value='1' where name='VOM_ADVANCED';  
  commit;

  /* 
   * Get licenses and populate sahara_sf_license_dimension
   */
  set @tablename = 'sahara.sahara_sf_license_dimension';
  set @attributenames = 'ccstor_unique_identifier,' ||
          'key,type,serial_number,is_sitelicense,is_keyless';

  insert into sahara.sahara_sf_license_dimension
  (
      ccstor_unique_identifier,
      type,
      "key",
      serial_number,
      is_site_license,
      is_keyless,
      is_expired
  ) on existing update
  (
      SELECT distinct
        UPPER(lic_info.licensekey) || '_' || lic_info.serialno 
          as 'ccstor_unique_identifier',
        lic_info.type as 'TYPE', 
        UPPER(lic_info.licensekey) as 'KEY',
        lic_info.serialno as 'SERIAL_NUMBER',
        case 
           when lic_info.site_license = 1 then 'T' else 'F'
        END as 'IS_SITE_LICENSE',
        case 
           when lic_info.keyless = 1 then 'T' else 'F'
        END as 'IS_KEYLESS',
        case 
           when lic_info.is_expired = 'Yes' then 'T' else 'F'
        END as 'IS_EXPIRED'
      FROM "HABDBSYNC"."LDR_VIEW_LDR_PRODUCT_INFO_ACCOUNTED" lic_info 
  );
    
  /* 
   * Get products and populate sahara_sf_product_dimension
   *
   * This table will have product_name repeated in multiple rows.
   * Note that different versions of a product can have different licensekey,
   * and in some cases there may be multiple licenses per product,version
   * combination (For example, DEMO/PERMANENT license keys).
   *
   * The primary key of this table is 
   *     <product_name:product_edition:product_version:licensekey> 
   */
  set @tablename = 'sahara.sahara_sf_product_dimension';
  set @attributenames = 'ccstor_unique_identifier,product_name,short_name,' ||
          'product_category,product_edition,product_version';

  insert into sahara.sahara_sf_product_dimension
  (
      ccstor_unique_identifier,
      product_name,
      short_name,
      product_edition,
      product_version,
      platform,
      license_type
  ) on existing update
  (
   SELECT 
       product_name + '_' + product_edition + '_' + product_version + '_' + 
         platform + '_' + lic_type
                        as 'ccstor_unique_identifier',
       product_name     as 'product_name',
       short_name       as 'short_name',
       product_edition  as 'product_edition',
       product_version  as 'product_version',
       platform,
       lic_type
   FROM 
      (
        SELECT 
           product_name    as 'product_name',
           product_edition as 'product_edition',
           product_version as 'product_version',
           platform,
           type as 'lic_type',
           case 
             when product_category like '%Oracle RAC%' then 'SFRAC'
             when product_category like '%DB Editions%' then 'SF'
             when product_category like '%SF%' then 'SF'
             when product_category like '%VCS%' then 'VCS'
             when product_category like '%VVR%' then 'VVR'
             when product_category like '%VM%' then 'VM'
             when product_category like '%DMP%' then 'DMP'
             when product_category like '%FS%' then 'FS'
             when product_category like '%APPHA%' then 'APPHA'
             when product_category like '%OTHER%' then 'OTHERS'
             else product_category 
           end as 'short_name',
           product_category
        FROM "HABDBSYNC"."LDR_VIEW_LDR_PRODUCT_INFO"
        GROUP BY 
            PRODUCT_NAME, PRODUCT_EDITION, PRODUCT_VERSION, PLATFORM, 
            TYPE, short_pname, product_category
      ) prod_list
  );

  /*
   * Map product_category column to friendly names and set them
   */
  insert into sahara.sahara_sf_product_dimension
  (
      ccstor_unique_identifier,
      product_category
  ) on existing update
  (
   SELECT 
       product_name + '_' + product_edition + '_' + product_version + '_' + 
         platform + '_' + lic_type
                         as 'ccstor_unique_identifier',
       str_prod_cat.str_value as 'product_category'
   FROM
      (
        SELECT
           product_name    as 'product_name',
           product_edition as 'product_edition',
           product_version as 'product_version',
           product_category as 'product_category',
           platform,
           type as 'lic_type',
           short_pname as 'short_name'
        FROM "HABDBSYNC"."LDR_VIEW_LDR_PRODUCT_INFO"
        GROUP BY
            PRODUCT_NAME, PRODUCT_EDITION, PRODUCT_VERSION, PLATFORM,
            TYPE, short_pname, product_category
      ) prod_list
      join sahara.sahara_constant_license_product_category str_prod_cat 
      on str_prod_cat.prod_category = prod_list.product_category
  );

  /*
   * For any non-edition products (like SF File System, Volume Manager etc.)
   * set product eidtion to Basic.
   *
   * Any product that is not in the 'Others' category and has
   * product edition set to 'N/A' will be set to Basic edition.
   */
  insert into sahara.sahara_sf_product_dimension
  (
      ccstor_unique_identifier,
      product_edition
  ) on existing update
  (
    SELECT 
      ccstor_unique_identifier,
      prod.product_edition
      --'Basic'
    FROM
      sahara.sahara_sf_product_dimension prod
    where
      prod.product_edition = 'N/A' AND
      prod.product_category <> 'Others'
  );

  /* 
   * Get license hosts and populate sahara_sf_license_host_dimension
   *
   * This table will have product_name repeated in multiple rows.
   * Note that different versions of a product can have different licensekey,
   * and in some cases there may be multiple licenses per product,version
   * combination (For example, DEMO/PERMANENT license keys).
   *
   * The primary key of this table is <product_name:product_version:licensekey> 
   */
  set @tablename = 'sahara.sahara_sf_license_host_dimension';
  set @attributenames = 'ccstor_unique_identifier,host_unique_identifier,' ||
          'id,name,model,processor_type,num_processors,platform,' ||
          'server_price_tier,num_processor_price_tier,os_price_tier,' ||
          'assigned_price_tier,os_edition,os';

if exists (select 1 from sys.syscolumns where tname='LDR_VIEW_LDR_HOST_INFO' and cname='SPVU') then
	insert into sahara.sahara_sf_license_host_dimension
	(
	  ccstor_unique_identifier,
	  host_unique_identifier,
	  id,
	  name,
	  model,
	  processor_type,
	  num_processors,
	  platform,
	  server_price_tier,
	  processor_price_tier,
	  num_processor_price_tier,
	  os_price_tier,
	  assigned_price_tier,
	  os_edition,
	  os,
	  spvu,
	  ncore
	) on existing update
	(
	SELECT 
		"id" as 'ccstor_unique_identifier',
		"id" as 'host_unique_identifier',
		"id" as 'id',
		"name",
		model,
		processor_type,
		nprocessors as 'num_processors',
		platform,
		server_price_tier,
		processor_price_tier,
		nprocessor_price_tier as 'num_processor_price_tier',
		os_version_price_tier as 'os_price_tier',
		tier_assigned as 'assigned_price_tier',
		os_edition,
		os,
		CASE WHEN ISNUMERIC(SPVU) = 0 THEN 0 ELSE cast(SPVU as numeric(10,2)) END AS SPVU,
		ncore
	FROM
	   "HABDBSYNC"."LDR_VIEW_LDR_HOST_INFO" lh join
	   "sahara"."sahara_host_dimension" h on lh.id = h.ccstor_unique_identifier
	);
else
	insert into sahara.sahara_sf_license_host_dimension
	  (
		  ccstor_unique_identifier,
		  host_unique_identifier,
		  id,
		  name,
		  model,
		  processor_type,
		  num_processors,
		  platform,
		  server_price_tier,
		  processor_price_tier,
		  num_processor_price_tier,
		  os_price_tier,
		  assigned_price_tier,
		  os_edition,
		  os
	  ) on existing update
	  (
	   SELECT 
			"id" as 'ccstor_unique_identifier',
			"id" as 'host_unique_identifier',
			"id" as 'id',
			"name",
			model,
			processor_type,
			nprocessors as 'num_processors',
			platform,
			server_price_tier,
			processor_price_tier,
			nprocessor_price_tier as 'num_processor_price_tier',
			os_version_price_tier as 'os_price_tier',
			tier_assigned as 'assigned_price_tier',
			os_edition,
			os
	   FROM
		   "HABDBSYNC"."LDR_VIEW_LDR_HOST_INFO" lh join
		   "sahara"."sahara_host_dimension" h on lh.id = h.ccstor_unique_identifier
	  );
end if;

  /*
   * Get all license deployments
   */
  set @tablename = 'sahara.sahara_sf_license_deployment';
  set @attributenames = 
          'license_deploymeent_unique_identifier,' ||
          'license_unique_identifier,product_unique_identifier,' ||
          'license_host_unique_identifier,expiry_date,' ||
          'is_expired,is_accountable';

  insert into sahara.sahara_sf_license_deployment
  (
    ccstor_unique_identifier,
    license_unique_identifier,
    product_unique_identifier,
    license_host_unique_identifier,
    expiry_date,
    is_expired,
    is_accountable
  ) on existing update
  (
   select
     UPPER(alldepl.licensekey) + '_' + alldepl.serialno + '_' +
       alldepl.product_name + '_' + alldepl.product_edition + '_' +
       alldepl.product_version + '_' + alldepl.platform + '_' + 
       alldepl.type + '_' + alldepl.hostid
     as 'sf_license_deployment_unique_identifier',
     UPPER(alldepl.licensekey) || '_' || alldepl.serialno 
          as 'license_unique_identifier',
     alldepl.product_name + '_' + alldepl.product_edition + '_' +
     alldepl.product_version + '_' + alldepl.platform + '_' + alldepl.type
     as 'product_unique_identifier',
     alldepl.hostid as 'license_host_unique_identifier',
     alldepl.expire_date as 'expiry_date',
     case
        when alldepl.is_expired = 'Yes' then 'T' else 'F'
     END as 'is_expired',
     if @dbmajorver < 6 then
     	case 
     	  when alldepl.higher_edition = 'Yes' then 'T' -- VOM MH <= 2.1
     	  when alldepl.higher_edition = '0' then 'T' -- VOM MS >= 5.0
     	  when alldepl.higher_edition is NULL then 'T' -- VOM MS < 5.0
     	  else 'F'
     	end 
     else 
        case
          when alldepl.higher_edition = 0 then 'T' -- VOM MS >= 5.0
          when alldepl.higher_edition is NULL then 'T' -- VOM MS < 5.0
     	  else 'F'
     	end  
     endif as 'is_accountable'
   from
     "HABDBSYNC"."LDR_VIEW_LDR_PRODUCT_INFO_ACCOUNTED" alldepl
  );

  /*
   * Set has_enterprise_edition for license_host dimension
   * Set it to true if at least one Enterprise edition product is present on
   * the host.
   */
  insert into sahara.sahara_sf_license_host_dimension
  (
      ccstor_unique_identifier,
      has_enterprise_edition
  ) on existing update
  (
   SELECT 
        lh.ccstor_unique_identifier,
        'T' as has_enterprise_edition
   FROM sahara.sahara_sf_license_host_dimension lh
   JOIN sahara.sahara_sf_license_deployment ld 
     ON ld.license_host_unique_identifier = lh.ccstor_unique_identifier
   JOIN sahara.sahara_sf_product_dimension p
     ON p.ccstor_unique_identifier = ld.product_unique_identifier
   WHERE p.product_edition = 'Enterprise'
   GROUP BY lh.ccstor_unique_identifier
  );



  /*
   * Set charegeable for license_host dimension
   * Set it to true based on entitlement logic from VOM for new versions of VOM else copy the logic in VOM A
   * the host.
   */
  if exists(select 1 from sys.systable 
      where table_name='LDR_VOM_ENTITLEMENT_REPORT_VIEW' 
      and creator=user_id('HABDBSYNC')) 
  then
      insert into sahara.sahara_host_dimension
      (
         ccstor_unique_identifier,
	 vom_license_required
      )on existing update
      (
	 SELECT 
	    h.ccstor_unique_identifier,
	    if (lh.CHARGEABLE=1) THEN 'T' ELSE 'F' END IF as vom_license_required
         FROM "HABDBSYNC"."LDR_VOM_LICENSABLE_HOSTS_VIEW" lh join
	      "sahara"."sahara_host_dimension" h on lh.hostid = h.ccstor_unique_identifier
      )
  else 
      insert into sahara.sahara_host_dimension
      (
         ccstor_unique_identifier,
	 vom_license_required
      )on existing update
      (
	 SELECT 
	    h.ccstor_unique_identifier,
	    if (lh.CHARGEABLE=1) THEN 'T' ELSE 'F' END IF as vom_license_required
         FROM "sahara"."sahara_ldr_licensable_hosts_view" lh join
	      "sahara"."sahara_host_dimension" h on lh.hostid = h.ccstor_unique_identifier
      )
  end if;

  
  /* 
   * Get deployed licenses and cookup fact table data
   */
  set @tablename = 'sahara.sahara_sf_deployed_licenses_fact';
  set @attributenames = 
          'license_unique_identifier,product_unique_identifier,' ||
          'license_host_unique_identifier, license_deployment_unique_identifier,'||
          'license_deployments,' ||
          'license_deployments_expired,accounted_deployments,' ||
          'accounted_deployments_expired';

  insert into sahara.sahara_sf_deployed_licenses_fact
  (
    license_unique_identifier,
    product_unique_identifier,
    license_host_unique_identifier,
    license_deployment_unique_identifier,
    license_deployments,
    license_deployments_expired,
    accounted_deployments,
    accounted_deployments_expired,
    keyless_deployments
  ) on existing update
  (
    select
      sfld.license_unique_identifier,
      sfld.product_unique_identifier,
      sfld.license_host_unique_identifier,
      sfld.ccstor_unique_identifier as license_deployment_unique_identifier,
      1 as 'license_deployments',
      case 
         when sfld.is_expired = 'T' then 1 else 0
      END as 'license_deployments_expired',
      case 
         when sfld.is_accountable = 'T' then 1 else 0
      END as 'accounted_license_deployments',
      case 
         when sfld.is_expired = 'T' and sfld.is_accountable = 'T' then 1 else 0
      END as 'accounted_license_deployments_expired',
      case 
         when lic.is_keyless = 'T' then 1 else 0
      END as 'keyless_deployments'
    from "sahara"."sahara_sf_license_deployment" sfld
    join "sahara"."sahara_sf_license_dimension" lic 
            on sfld.license_unique_identifier = lic.ccstor_unique_identifier
  );
    
  /*
   * Get SPVU details per product per host
   */
  set @tablename = 'sahara_sf_deployed_licenses_fact';
  set @attributenames = 
          'ccstor_unique_identifier,' ||
          'license_unique_identifier,product_unique_identifier,' ||
          'license_host_unique_identifier,' ||
          'total_spvu';

if exists (select 1 from sys.syscolumns where tname='LDR_VIEW_LDR_PRODUCT_INFO_ACCOUNTED' and cname='HC_ID') then
  insert into sahara.sahara_sf_deployed_licenses_fact
  (
    license_unique_identifier,
	product_unique_identifier,
	license_host_unique_identifier,
	license_deployment_unique_identifier,
    total_spvu
  ) on existing update
  (
	SELECT 
		   sld.license_unique_identifier,
		   (PRODUCT_NAME||'_'||PRODUCT_EDITION||'_'||PRODUCT_VERSION||'_'||PLATFORM||'_'||TYPE) as PRODUCT_ID,
		    HOSTID,
			sld.ccstor_unique_identifier,
		   CASE WHEN ISNUMERIC(TIER_NAME_KEY) = 0 THEN 0 ELSE convert(DECIMAL(10,2),TIER_NAME_KEY) * NCORE_TO_LICENSE END AS TOTAL_SPVU       	
	FROM
	(
		SELECT HOSTID,PLATFORM,PRODUCT_NAME,PRODUCT_EDITION,PRODUCT_VERSION,TYPE,OS_EDITION,IS_VIRTUAL, HC_ID, NCORE_TO_LICENSE,
			CASE WHEN ISNUMERIC(SPVU)= 1 then SPVU END as TIER_NAME_KEY,
			'SPVU' as TIER_TYPE
		FROM "HABDBSYNC"."LDR_VIEW_LDR_PRODUCT_INFO_ACCOUNTED"
		WHERE NCORE_TO_LICENSE != 0 AND TIER_NAME_KEY is not NULL AND (isNULL(HIGHER_EDITION,'-1') != '1')
	) pl
	join sahara.sahara_sf_license_deployment sld on sld.product_unique_identifier = (PRODUCT_NAME||'_'||PRODUCT_EDITION||'_'||PRODUCT_VERSION||'_'||PLATFORM||'_'||TYPE)
	where TIER_TYPE = 'SPVU'
  );
end if;

  /*
   * Set batch Id on all dimension/fact tables
   */
  update sahara.sahara_sf_license_dimension set batch_id = @batch_id ;
  update sahara.sahara_sf_product_dimension set batch_id = @batch_id ;
  update sahara.sahara_sf_license_host_dimension set batch_id = @batch_id ;
  update sahara.sahara_sf_license_deployment set batch_id = @batch_id ;
  update sahara.sahara_sf_deployed_licenses_fact set batch_id = @batch_id ;

  /*
   * Log counts 
   */
  select count(1) into @noLicDims from sahara.sahara_sf_license_dimension;
  select count(1) into @noProdDims from sahara.sahara_sf_product_dimension;
  select count(1) into @noLicHostDims from 
         sahara.sahara_sf_license_host_dimension;
  select count(1) into @noLicDeplDims from sahara.sahara_sf_license_deployment;

  select count(1) into @noFacts from sahara.sahara_sf_deployed_licenses_fact;

  insert into sahara.sahara_log 
     (date_time, batch_id, procedure_name, severity, log_message) 
     select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'I',
          'completed dimension rollup -> sahara_sf_license_dimension: ' || 
          @noLicDims || ' rows inserted';

  insert into sahara.sahara_log 
     (date_time, batch_id, procedure_name, severity, log_message) 
     select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'I',
          'completed dimension rollup -> sahara_sf_product_dimension: ' || 
          @noProdDims || ' rows inserted';

  insert into sahara.sahara_log 
     (date_time, batch_id, procedure_name, severity, log_message) 
     select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'I',
          'completed dimension rollup -> sahara_sf_license_host_dimension: ' || 
          @noLicHostDims || ' rows inserted';

  insert into sahara.sahara_log 
     (date_time, batch_id, procedure_name, severity, log_message) 
     select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'I',
          'completed dimension rollup -> sahara_sf_license_deployment: ' || 
          @noLicDeplDims || ' rows inserted';

  insert into sahara.sahara_log 
     (date_time, batch_id, procedure_name, severity, log_message) 
     select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'I',
            'completed facts rollup -> sahara_sf_deployed_licenses_fact: ' || 
            @noFacts|| ' rows inserted';


  insert into sahara.sahara_log 
      (date_time, batch_id, procedure_name, severity, log_message) 
      select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'I',
      'roll-up batch: ' || @batch_id || 
      ', data set: license data -> Succeeded: ' ||
      datediff(second,@initialtime,GETDATE()) || ' secs';

  commit;

  EXCEPTION
    when others then
      SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
      IF ( @error <> 0 ) then
         SET @ERROR_COUNT=@ERROR_COUNT + 1;
         BEGIN
           insert into sahara.sahara_log 
              (date_time, batch_id, procedure_name, severity, log_message)
              select GETDATE(), @batch_id, 'sahara.sahara_lic_data', 'E', 
              string (' Error failed at = ', @tablename+':'+@attributenames ) +
              string (': SQLCODE = ', @error ) +
              string (': ERRORMSG = ', @serror ) +
              string (': SQLSTATE = ', @sqlstate );
           message string (' Error failed at = ', @tablename+':'+
                        @attributenames ) to LOG;
           message string ('          @error = ', @error ) to LOG;
           message string ('      @serrir()  = ', @serror ) to LOG;
           message string ('     @sqlstate() = ', @sqlstate ) to LOG;
         END;
      END IF;
END;

grant execute on sahara.sahara_lic_data to public;


commit;
