if exists(select 1 from sys.sysprocedure where proc_name = 'sahara_volume_data' and user_name(creator) = 'sahara') then

   drop procedure sahara.sahara_volume_data

end if;



CREATE PROCEDURE sahara."sahara_volume_data" (@batch_id integer)

/* RESULT ( column_name, ... ) 

 */

BEGIN



/* Variables for logging purpose */

/* 

To record initial time when this stored procedure was called 

*/

DECLARE @initialtime timestamp; 



/* 

Total number of dimension this stored procedure is inserting 

*/

DECLARE @noDims bigint; 



/* 

Total number of facts this stored procedure is inserting 

*/

DECLARE @noFacts bigint; 

DECLARE @noFactsvmtolun bigint;

/* Log detail error info */
declare @err_code integer;
declare @error    integer;
declare @serror         varchar(32767);
declare @sqlstate       varchar(5);
declare @tablename      varchar(128);
declare @attributenames varchar(256);
declare @count    integer;
declare @error_count    integer;
declare @is_ASM int;


/* 

Store the current time 

*/

Set @initialtime=GETDATE();



insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 

select GETDATE(), @batch_id, 'sahara.sahara_volume_data', 'I','starting dimension/facts rollup -> sahara_volume_dimension/sahara_volume_facts';


if exists (SELECT 1 FROM sys.systable where table_name = 'sf_view_storage_Assignment_r' and table_type='VIEW' and user_name(creator)='habdbsync')
then 
set @is_ASM = 1;
end if;



if exists(select 1 from sys.systable where table_name='sahara_volume_dimension' and creator=user_id('sahara')) then

   drop table sahara.sahara_volume_dimension

end if;



if exists(select 1 from sys.systable where table_name='sahara_volume_facts' and creator=user_id('sahara')) then

   drop table sahara.sahara_volume_facts

end if;



if exists(select 1 from sys.systable where table_name='sahara_volume_to_lun' and creator=user_id('sahara')) then

   drop table sahara.sahara_volume_to_lun

end if;


if exists(select 1 from sys.systable where table_name='sahara_volume_to_host' and creator=user_id('sahara')) then
   drop table sahara.sahara_volume_to_host
end if;


/*==============================================================*/

/* Table: Volume                                                */

/*==============================================================*/

create table sahara.sahara_volume_dimension  (
	volume_id          			unsigned int, // Remove
	host_id           			unsigned int, // Remove
	host_unique_identifier 			varchar(256),
	ccstor_unique_identifier		varchar(256),
        dg_unique_identifier 			varchar(256),
	display_name        	 		varchar(256),
	version				 	varchar(256),
	vendor		 			varchar(1024),
	layout		              		unsigned int, // REMOVE
	layout_str				varchar(256),
	disk_group_name				varchar(256) default null,
	is_vcs_cvm				bit default 0,
	is_used_by_filesystem			bit default 0,
	is_used_by_application			bit default 0,
        is_direct_attached			bit default 0,
        is_segment				bit default 0,
        is_exported				bit default 0,
	is_based_on_thin			bit default 0,
	batch_id 				int default 0,	
	type					varchar(256) default 'Data', /*For differentiating between snapshots, clones and data volumes*/
	used_by_dbvm				bit default 0,
	dbvm_name        	 		varchar(256),
	primary key(ccstor_unique_identifier)

);

grant select on sahara.sahara_volume_dimension to public;



/*==============================================================*/

/* Table: Volume Facts                                          */

/*==============================================================*/

create table sahara.sahara_volume_facts  (

   volume_id    	        unsigned int, // REMOVE

   capacity 			NUMERIC(22,9) default 0,

   capacity_used		NUMERIC(22,9) default 0, // REMOVE

   capacity_unknown		NUMERIC(22,9) default 0, // REMOVE

   capacity_used_by_apps NUMERIC(22,9) default 0, // REMOVE

   used_by_non_apps 	NUMERIC(22,9) default 0, // REMOVE

   batch_id 			int default 0,

   ccstor_unique_identifier varchar(256),

   no_of_hosts        int           default 0 not null,

   primary key(ccstor_unique_identifier)

);

grant select on sahara.sahara_volume_facts to public;





 -- Create a brand new table, to capture the volume to lun information. 

---  We will make use of the existing sahara_volume_facts table

--- to get to the lunid and array id.

create table sahara.sahara_volume_to_lun  (

   volume_id    	        unsigned int, // REMOVE

   vm_ccstor_unique_identifier varchar(256),

   lun_id				unsigned int, // REMOVE

   lun_ccstor_unique_identifier varchar(256),

   array_id             unsigned int, // REMOVE

   array_ccstor_unique_identifier varchar(256),

   primary key(vm_ccstor_unique_identifier,lun_ccstor_unique_identifier)

);

grant select on sahara.sahara_volume_to_lun to public;

create table sahara.sahara_volume_to_host  (
        volume_unique_identifier varchar(256),
        host_unique_identifier varchar(256),
        primary key ( volume_unique_identifier,host_unique_identifier)
);
grant select on sahara.sahara_volume_to_host to public;

set @tablename = 'sahara.sahara_volume_dimension';
set @attributenames = 'host_unique_identifier,ccstor_unique_identifier,dg_unique_identifier,display_name,version,vendor,layout_str,is_vcs_cvm,disk_group_name';

insert into sahara.sahara_volume_dimension 
(	host_unique_identifier,
	ccstor_unique_identifier,  
    dg_unique_identifier,  
	display_name,        	 	
	version,			 		
	vendor,		 				
	layout_str,		              	
	is_vcs_cvm,
	disk_group_name
) on existing update

select volumes.host_id, volumes.vol_id, (if volumes.type = 'DUMMY' then null else volumes.ID endif) as DG_ID, volumes.vol_name, 
(if volumes.family = 'VxVM' then volumes.sf_version else (if (volumes.version is null or volumes.version = '0') then 'N/A' else volumes.version endif) endif) as version, volumes.family as vendor, 
RB_LAYOUT.DISPLAY as layout_str, (if dg_type='shared' then 1 else 0 endif) as is_vcs_cvm, (if volumes.type = 'DUMMY' then null else DG_Name endif) as DG_Name
from 
(
/*Non ZFS*/
select vol.host_id, vol.vol_id, vol.vol_name, vol.dg_type, vol.dg_name, vol.vol_type, dg.version, dg.family, dg.type, dg.id, host.sf_version from
HABDBSYNC.SF_VIEW_HOST_VOLUME vol 
join HABDBSYNC.SF_VIEW_DISKGROUP_R dg on vol.DG_ID = dg.ID
join HABDBSYNC.SF_VIEW_HOST_R host on vol.host_id = host.id
where 
dg.family!='ZFS'
and ( 
            (dg.type is null or dg.type != 'DUMMY')/*Ignoring FS on Raw dummy vols*/
            OR 
            (dg.type = 'DUMMY' and host.PLATFORM = 'Windows') /*Do not ignore volume created for windows LDM support. Ignore DG.*/
      )
UNION ALL /*ZFS*/
select vol.host_id, vol.vol_id, vol.vol_name, vol.dg_type, vol.dg_name, vol.vol_type, dg.version, dg.family, dg.type, dg.id, host.sf_version from
HABDBSYNC.SF_VIEW_HOST_VOLUME vol 
join HABDBSYNC.SF_VIEW_DISKGROUP_R dg on vol.DG_ID = dg.ID
join (
select
ccstor_unique_identifier,
sf_version
from sahara.sahara_host_dimension
) host on vol.host_id = host.ccstor_unique_identifier
where 
dg.family='ZFS' AND vol.vol_id in (select id from "HABDBSYNC"."P_SF_VOLUME_PROPS" where PROP_NAME = 'zfs_volsize')    
) as volumes
left outer join HABDBSYNC.SF_VIEW_UI_ENUM_BUNDLE RB_LAYOUT on RB_LAYOUT.OBJECT='VOLUME' and RB_LAYOUT.PROPERTY='LAYOUT' 
        and RB_LAYOUT.VALUE=volumes.vol_type and RB_LAYOUT.LOCALE_ID = 'en_US'
;

set @attributenames = 'ccstor_unique_identifier, is_segment';
insert into sahara.sahara_volume_dimension 
(	ccstor_unique_identifier,  
	is_segment
) on existing update
select vol_id, 1 from HABDBSYNC.SF_VIEW_HOST_VOLUME vold
join sahara.sahara_volume_dimension on vol_id=ccstor_unique_identifier
where vold.vol_type!='VOLUMESET'
and pvol_id!=vol_id
;

set @attributenames = 'ccstor_unique_identifier,is_exported';
insert into sahara.sahara_volume_dimension 
(	ccstor_unique_identifier,
	is_exported
) on existing update
select vol.ccstor_unique_identifier, 1 as is_exported
FROM
    sahara.sahara_volume_dimension vol 
    JOIN HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on vol.ccstor_unique_identifier = da.source_id and da.source_type = 'VOLUME'
;

set @attributenames = 'ccstor_unique_identifier, type';
insert into sahara.sahara_volume_dimension 
(	ccstor_unique_identifier,  
	type
) on existing update
select ccstor_unique_identifier, (if snap.type = 'zfs-clone' then 'Clone' else 'Snapshot' endif) as type from 
sahara.sahara_volume_dimension 
join HABDBSYNC.SF_VIEW_VOL_SNAPSHOT_REL_R snap
on ccstor_unique_identifier = snapshot_vol_id
;

set @tablename = 'sahara.sahara_volume_to_host';
set @attributenames = 'volume_unique_identifier, host_unique_identifier';

insert into sahara.sahara_volume_to_host
(	volume_unique_identifier,
    host_unique_identifier,
) on existing update
select vol.vol_id, vol.host_id
from sahara.sahara_volume_dimension vold
join HABDBSYNC.SF_VIEW_HOST_VOLUME vol on vold.ccstor_unique_identifier = vol.vol_id
join sahara.sahara_host_dimension host on vol.host_id = host.ccstor_unique_identifier
;

/*
Insert volume layout reading from constants file
*/
/*insert into sahara.sahara_volume_dimension
(
volume_id,
layout_str
) on existing update
select vol.volume_id , vol_layout.str_value
from sahara.sahara_volume_dimension as vol,sahara.sahara_constant_volume_layout as vol_layout
where vol.layout_str=vol_layout.str_dbvalue;
*/
/*
Insert VM type reading from constants file
*/
/*
insert into sahara.sahara_volume_dimension
(
volume_id,
vendor
) on existing update
select vol.volume_id , vm_type.str_value
from sahara.sahara_volume_dimension as vol,sahara.sahara_constant_vm_type as vm_type
where vol.vendor=vm_type.str_dbvalue;
*/

set @tablename = 'sahara.sahara_volume_facts';
set @attributenames = 'ccstor_unique_identifier, no_of_hosts';
insert into sahara.sahara_volume_facts 
(
   ccstor_unique_identifier,
   no_of_hosts
) on existing update
select 
VolD.ccstor_unique_identifier,
count(VolTH.host_unique_identifier) as no_of_hosts
from sahara.sahara_volume_dimension VolD
join sahara.sahara_volume_to_host volTH on VolD.ccstor_unique_identifier = VolTH.volume_unique_identifier
group by volD.ccstor_unique_identifier; 

set @attributenames = 'capacity';
/*
Get Volume Capacity. For non sun volumes.
*/
insert into sahara.sahara_volume_facts  (
   ccstor_unique_identifier,
   capacity   
) on existing update
select 
vol.ccstor_unique_identifier, 
coalesce(size,0) / (1024 * 1024 * 1024.0) as volume_size
from sahara.sahara_volume_dimension as vol
join HABDBSYNC.SF_VIEW_Volume_R on HABDBSYNC.SF_VIEW_Volume_R.id=vol.ccstor_unique_identifier;
;

set @tablename = 'sahara.sahara_volume_to_lun';
set @attributenames = 'vm_ccstor_unique_identifier, lun_ccstor_unique_identifier, array_ccstor_unique_identifier';
-- Calculation pertaining to the volume to lun
insert into  sahara.sahara_volume_to_lun  
(
   vm_ccstor_unique_identifier,
   lun_ccstor_unique_identifier,
   array_ccstor_unique_identifier
) on existing update
select dToV.VOL_GUID, DtoV.DISK_ID, enc_ccstor_unique_identifier from HABDBSYNC.SF_VIEW_DISK_TO_VOL_R DtoV
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = DtoV.DISK_ID
join sahara.sahara_volume_dimension vol on vol.ccstor_unique_identifier = DtoV.VOL_GUID
;

/****************************************************************/
/*RDM VMDK Disk->ESX LUN to be associated with Virtual Machine Host and Vol_TO_LUN tuple to be deleted for RDM VMDK Disk. This is being handled for the case when array is not configured but guest is.*/
set @attributenames = 'ESX RDM LUN->vm_ccstor_unique_identifier,lun_ccstor_unique_identifier, array_ccstor_unique_identifier';
insert into  sahara.sahara_volume_to_lun  
(
   vm_ccstor_unique_identifier,
   lun_ccstor_unique_identifier,
   array_ccstor_unique_identifier
) on existing update
select distinct vol.ccstor_unique_identifier, VMDK.object_id, enc_ccstor_unique_identifier
from HABDBSYNC.SF_VIEW_DISK_TO_VOL_R DtoV
join sahara.sahara_volume_dimension vol on vol.ccstor_unique_identifier = DtoV.VOL_GUID
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on da.source_type in  ('VDISK','DISK') and da.dest_id = DtoV.DISK_ID
join HABDBSYNC.VT_VIEW_VDISK_R VMDK on VMDK.id = da.source_id
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = VMDK.object_id
where VMDK.object_id is not null
and is_raw=1;


set @attributenames = 'Delete RDM LUNs';
delete from sahara.sahara_volume_to_lun
where lun_ccstor_unique_identifier in
(select da.dest_id as VMDK_OID
from HABDBSYNC.VT_VIEW_VDISK_R VMDK
join HABDBSYNC.VT_VIEW_DEVICE_ASSIGNMENT_R da on VMDK.id = da.source_id and da.source_type in  ('VDISK','DISK')
where VMDK.object_id is not null
and VMDK_OID<>VMDK.object_id
and da.dest_id is not null
and is_raw=1
);
/****************************************************************/

set @tablename = 'sahara.sahara_volume_dimension';
set @attributenames = 'is_direct_attached';
/* setting the is direct attached flag */
update sahara.sahara_volume_dimension as vm set is_direct_attached = 1
where ccstor_unique_identifier in (
select voltolun.vm_ccstor_unique_identifier
from 
sahara.sahara_volume_to_lun as voltolun
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = voltolun.lun_ccstor_unique_identifier
where lun.is_direct_attached=1)
;

set @attributenames = 'ccstor_unique_identifier,is_based_on_thin';
update sahara.sahara_volume_dimension as vm set is_based_on_thin = 1
where ccstor_unique_identifier in (
select voltolun.vm_ccstor_unique_identifier
from 
sahara.sahara_volume_to_lun as voltolun
join sahara.sahara_lun_dimension lun on lun.ccstor_unique_identifier = voltolun.lun_ccstor_unique_identifier
where lun.thin_status='ThinLun')
;

set @attributenames = 'is_used_by_filesystem';
/*
populate is_used_by_filesystem column
*/
update sahara.sahara_volume_dimension
set is_used_by_filesystem = 1
where ccstor_unique_identifier in (
select FS.ID
from HABDBSYNC.SF_VIEW_FILESYSTEM_R as FS
join HABDBSYNC.SF_VIEW_BASE_UI_VOLUME_FS ON SF_VIEW_BASE_UI_VOLUME_FS.ID=FS.ID
);

set @attributenames = 'is_used_by_application';
/*
populate is_used_by_application column
*/
update sahara.sahara_volume_dimension
set is_used_by_application = 1
where ccstor_unique_identifier in (
SELECT s.storage_id FROM HABDBSYNC.SF_VIEW_UI_DB_STORAGE s
where storage_type = 'VOLUME' 
);

if (@is_ASM=1) THEN
set @attributenames = 'ccstor_unique_identifier,used_by_dbvm,dbvm_name';
insert into sahara.sahara_volume_dimension
(	ccstor_unique_identifier,
	used_by_dbvm,
	dbvm_name
) on existing update
select dgtovol.source_ID, 1 as used_by_dbvm, dg.name
from 
habdbsync.sf_view_storage_Assignment_r dgtovol
join sahara.sahara_volume_dimension vol on vol.ccstor_unique_identifier = dgtovol.source_ID
join HABDBSYNC.SF_VIEW_DISKGROUP_R as DG on DG.ID = dgtovol.dest_ID
;

end if; /*for ASM*/

select count(1) into @noDims from sahara.sahara_volume_dimension;
select count(1) into @noFacts from sahara.sahara_volume_facts;
select count(1) into @noFactsvmtolun from sahara.sahara_volume_to_lun;


insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 

select GETDATE(), @batch_id, 'sahara.sahara_volume_data', 'I','completed dimension rollup -> sahara_volume_dimension: ' || @noDims || ' rows inserted';



insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 

select GETDATE(), @batch_id, 'sahara.sahara_volume_data', 'I','completed facts rollup -> sahara_volume_facts: ' || @noFacts || ' rows inserted';



insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 

select GETDATE(), @batch_id, 'sahara.sahara_volume_to_lun', 'I','completed facts rollup -> sahara_volume_to_lun: ' || @noFactsvmtolun || ' rows inserted';





insert into sahara.sahara_log (	date_time, batch_id, procedure_name, severity, log_message) 

select GETDATE(), @batch_id, 'sahara.sahara_volume_data', 'I','roll-up batch: ' || @batch_id || ', data set: sahara_volume_dimension/sahara_volume_facts -> Succeeded: ' ||datediff(second,@initialtime,GETDATE()) || ' secs';

commit;

EXCEPTION
    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 ) then
            SET @ERROR_COUNT=@ERROR_COUNT + 1;
            BEGIN
                insert into sahara.sahara_log (date_time, batch_id, procedure_name, severity, log_message)
                select GETDATE(), @batch_id, 'sahara.sahara_volume_data', 'E', string (' Error failed at = ', @tablename+':'+@attributenames ) +
                                                                             string (': SQLCODE = ', @error ) +
                                                                             string (': ERRORMSG = ', @serror ) +
                                                                             string (': SQLSTATE = ', @sqlstate );
                message string (' Error failed at = ', @tablename+':'+@attributenames ) to LOG;
                message string ('          @error = ', @error ) to LOG;
                message string ('      @serrir()  = ', @serror ) to LOG;
                message string ('     @sqlstate() = ', @sqlstate ) to LOG;
            END;
        END IF;
        -- RESIGNAL;

END;

grant execute on sahara.sahara_volume_data to public;

