#
# $Id: CSVResources.pm,v 1.5 2012/06/01 15:19:06 dkumkar Exp $
# $Source: /project/sahara/src/import/CSVResources.pm,v $
#
# $Copyright: Copyright (c) 2012 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CSVResources;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw( );
our $VERSION = '0.01';

our $unassigned_header =
"
# Unassigned objects
# This CSV file contains all %s objects that are not currently assigned to 
# any business view in Enterprise Reporter.

#Format:
";

our $unassigned_to_view_header =
"
# Unassigned objects
# This CSV file contains all '%s' objects that are not currently assigned to 
# the business view '%s' in Enterprise Reporter.

#Format:
";


our $view_section_comment =
";
; This definition file defines the overall structure of a business view. 
; Section names are enclosed in square brackets ( [ ] ).
; Parameter values should not be quoted. Multi-line parameters are
; supported by ending each line with the backslash ( \ ) character.
;

;
; [Business View] section
;
; The Business View section defines the view name and description. 
; A business view defines the logical structure used to organized 
; objects into different business categories. 
;
; This section has the following required prameters:
;     Name - the name of the business view
;     Description - a description for the business view
;
";

our $level_sections_comment =
";
; [Level] Sections
;
; The hierarchy defines a data demension that allows the user to create 
; drill-down reports. This hierarchy is defined by specifying a collection
; of one or more [Level] sections. 
; 
; Each level in the view hierarchy is defined by one of these sections. 
; The level highest in the view is [Level1], the next level in the 
; hierarchy is [Level2], etc.
;
; Each [Level] section has the following required parameters:
;     Name  - the name for the level
;     Description - the description of the level
;
";

our $types_section_comment = 
";
; [Object Types] section
;
; Each business view can support one more more object types. The Object Types
; section has a single parameter, containing a comma separated list of 
; supported object types. 
;
; The support types are:
;     Host - a host object identified by name, ip, or unique id
;     Array - an array object identified by name, ip, or unique id
;     Filesystem - a filesystem object identified by hostname + filesystem name or unique id
;     Volume - a volume object identified by identified by hostname + volume name or unique id
;     NAS_Share - a NAS share identified by filer name + path or unique id
;     NAS_Filesystem - a NAS Filesystem identified by filer name + NAS filesystem name or unique id
;     Cluster - a Cluster object identified by name or unique id
;     ServiceGroup - a ServiceGroup object identified by cluster name + servicegroup name or unique id
;     DiskGroup - a Disk group object identified by name or unique id
; 
";

our $assignment_file_header =
"#\n" .
"# This assignment file contains %s objects.\n" .
"# The definition for this view is found in the %s.view file\n" .
"# and has the following levels defined:\n" .
"#\n";


our $assign_file_header_end =
"#
# If you assigning an object to a node in the view, at least one
# of these columns must have data to allow ER to associate the object
# with that node.If you're only defining a node but not assigning
# objects, each of these columns should be left empty.
#
";


sub get_assignment_file_header {

   my $view_name = shift;
   my $type = shift;
   my $type_columns_ref = shift;
   my $view_column_display_names_ref = shift;

   my $rval = "";
   my $tmp = "";

   $tmp = sprintf( $assignment_file_header, $type, $view_name );

   $rval .= $tmp;
   $rval .= "#     ";
   $rval .= join( "\n#     ", @$view_column_display_names_ref );
   $rval .= "\n#\n";
   $rval .= "# These are represented by the first " . 
            scalar(@$view_column_display_names_ref) .
            " values in this file.\n";

   $tmp = sprintf(
            "#\n# The $type assignment file also has %d required columns. These\n" .
            "# columns are :\n" .
            "#\n",
            scalar( @$type_columns_ref ) );
   $rval .= $tmp;

   $rval .= "#     ";
   $rval .= join( "\n#     ", @$type_columns_ref );

   $tmp = sprintf(
            "\n#\n# There should be a total of %d columns in each row of this CSV file.\n",
            scalar( @$type_columns_ref ) + 
            scalar( @$view_column_display_names_ref ) );  
   $rval .= $tmp;

   $rval .= $assign_file_header_end;

   $rval .= "\n# ";

   $rval .= join( ", ", @$view_column_display_names_ref );
   $rval .= ", ";
   $rval .= join( ", ", @$type_columns_ref );
   $rval .= "\n";

   return $rval;
}



# avoid annoying Perl module bug
1;
