#
# $Id: bv_create.pl,v 1.12 2012/03/21 02:30:10 jeevan Exp $
# $Source: /project/sahara/src/import/bv_create.pl,v $
#
# $Copyright: Copyright (c) 2012 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
use strict;
use ERPERLLIB;
use warnings;
use File::Spec;
use File::Path;
use DBI;
#use OracleHelpers;
use SybaseHelpers;
use Getopt::Std;
use PromptUser;
use CSVResources;

our $dbh = undef;

sub create_view_file {
   my $basename = ( shift );
   my $bvname = ( shift );
   my $bvdesc = ( shift );
   my $level_count = ( shift );
   my $level_names_ref = ( shift );
   my $level_descs_ref = ( shift );
   my $types_ref = ( shift );

   open( VIEW, ">${basename}.view" ) or die "Couldn't open ${basename}.view $!\n";

   print VIEW $CSVResources::view_section_comment;
   print VIEW "[Business View]\n";
   print VIEW "Name=${bvname}\n";
   print VIEW "Description=${bvdesc}\n\n";

   print VIEW $CSVResources::level_sections_comment;
   my $vc = 1;
   for( my $i = 0; $i < $level_count; ++$i ) {
      print VIEW "[Level" . ($i+1) . "]\n";
      print VIEW "Name=" . @$level_names_ref[ $i ] . "\n";
      print VIEW "Description=" . @$level_descs_ref[ $i ] . "\n\n";
   }

   print VIEW $CSVResources::types_section_comment;
   print VIEW "[Object Types]\n";
   print VIEW "Types=" . join( ",", @$types_ref ) . "\n\n";
   close VIEW;
}



sub create_type_csv_file {
   my $basename = ( shift );
   my $bvname = ( shift );
   my $type = ( shift );
   my $level_names_ref = ( shift );

   my @type_columns = ( 'name' );
   my $sql = "";

   if( $type =~ /APP_GROUP/i ) {
      $sql = "SELECT DISPLAY_NAME, APP_GROUP_OID from SAHARA.APP_GROUP_CURRENT";
      @type_columns = ( 'name', 'object_unique_id' );
   }
   elsif( $type =~ /SWITCH/i || $type =~ /ARRAY/i ) {
      $sql = "SELECT DISPLAY_NAME, SNMP_IP_ADDRESS, ${type}_OID from SAHARA.${type}_CURRENT";
      @type_columns = ( 'name', 'ip', 'object_unique_id' );
   }
   elsif( $type =~ /HOST/i ) {
      $sql = "SELECT DISPLAY_NAME, IP_ADDRESS, HOST_OID from SAHARA.HOST_CURRENT";
      @type_columns = ( 'name', 'ip', 'object_unique_id' );
   }
   elsif( $type =~ /NAS_VOLUME/i ) {
      $sql = "SELECT t1.DISPLAY_NAME, CONCAT( '/vol/', t2.DISPLAY_NAME ), t2.NETAPP_VOLUME_OID from\n"
           .  "SAHARA.ARRAY_CURRENT t1,\n"
           .  "SAHARA.NETAPP_VOLUME_CURRENT t2\n"
           .  "WHERE t2.ARRAY_KEY = t1.ARRAY_KEY\n"
           .  "ORDER BY t1.DISPLAY_NAME\n";
      @type_columns = ( 'filer_name', 'path', 'object_unique_id' )
   }   
   elsif( $type =~ /NAS_SHARE/i ) {
      $sql = "SELECT t1.DISPLAY_NAME, t2.SHARE_PATH, t2.NETAPP_SHARE_OID from\n"
           .  "SAHARA.ARRAY_CURRENT t1,\n"
           .  "SAHARA.NETAPP_SHARE_CURRENT t2\n"
           .  "WHERE t2.ARRAY_KEY = t1.ARRAY_KEY\n"
           .  "ORDER BY t1.DISPLAY_NAME\n";
      @type_columns = ( 'filer_name', 'path', 'object_unique_id' )
   }
   elsif( $type =~ /NAS_QTREE/i ) {
      $sql = "SELECT t1.DISPLAY_NAME, t2.QTREE_PATH, t2.NETAPP_QTREE_OID from\n"
           .  "SAHARA.ARRAY_CURRENT t1,\n"
           .  "SAHARA.NETAPP_QTREE_CURRENT t2\n"
           .  "WHERE t2.ARRAY_KEY = t1.ARRAY_KEY\n"
           .  "ORDER BY t1.DISPLAY_NAME\n";
      @type_columns = ( 'filer_name', 'path', 'object_unique_id' )
   }
   elsif( $type =~ /NAS_FILESYSTEM/i ) {
      $sql = "SELECT t1.DISPLAY_NAME, t2.DISPLAY_NAME , t2.NAS_FILESYSTEM_OID from\n"
           .  "SAHARA.ARRAY_CURRENT t1,\n"
           .  "SAHARA.NAS_FILESYSTEM_CURRENT t2\n"
           .  "WHERE t2.ARRAY_KEY = t1.ARRAY_KEY\n"
           .  "ORDER BY t1.DISPLAY_NAME\n";
      @type_columns = ( 'filer_name', 'nas_filesystem_name', 'object_unique_id' )
   }
   elsif( $type =~ /FILESYSTEM/i ) {
      $sql = "SELECT t1.DISPLAY_NAME, t2.DISPLAY_NAME , t2.FILESYSTEM_OID from\n"
           .  "SAHARA.HOST_CURRENT t1,\n"
           .  "SAHARA.FILESYSTEM_CURRENT t2\n"
           .  "WHERE t2.HOST_KEY = t1.HOST_KEY\n"
           .  "ORDER BY t1.DISPLAY_NAME\n";
      @type_columns = ( 'host_name', 'filesystem_name', 'object_unique_id' )
   }
   elsif( $type =~ /VOLUME/i ) {
      $sql = "SELECT t1.DISPLAY_NAME, t2.VOLUME_NAME , t2.VOLUME_OID from\n"
           .  "SAHARA.HOST_CURRENT t1,\n"
           .  "SAHARA.VOLUME_CURRENT t2\n"
           .  "WHERE t2.HOST_KEY = t1.HOST_KEY\n"
           .  "ORDER BY t1.DISPLAY_NAME\n";
      @type_columns = ( 'host_name', 'volume_name', 'object_unique_id' )
   }
   elsif( $type =~ /DISKGROUP/i ) {
      $sql = "SELECT t1.DISPLAY_NAME, t2.DISPLAY_NAME , t2.DISKGROUP_OID from\n"
           .  "SAHARA.HOST_CURRENT t1,\n"
           .  "SAHARA.DISKGROUP_CURRENT t2\n"
           .  "WHERE t2.HOST_KEY = t1.HOST_KEY\n"
           .  "ORDER BY t1.DISPLAY_NAME\n";
      @type_columns = ( 'host_name', 'filesystem_name', 'object_unique_id' )
   }
   elsif( $type =~ /CLUSTER/i ) {
      $sql = "SELECT DISPLAY_NAME, ${type}_OID from SAHARA.${type}_CURRENT";
      @type_columns = ( 'name', 'object_unique_id' );
   }
   elsif( $type =~ /SERVICEGROUP/i ) {
      $sql = "SELECT c.DISPLAY_NAME, sg.DISPLAY_NAME, sg.SERVICEGROUP_OID from \n"
           . " SAHARA.SERVICEGROUP_CURRENT sg, \n"
           . " SAHARA.CLUSTER_CURRENT c \n"
           . "WHERE sg.CLUSTER_KEY = c.CLUSTER_KEY \n"
           . "ORDER BY c.DISPLAY_NAME\n";
      @type_columns = ( 'cluster_name', 'servicegroup_name', 'object_unique_id' );
   }

   my $level_count = scalar( @$level_names_ref );

   print "Creating ${basename}_${type}.csv file...\n\n";

   open( CSV, ">${basename}_${type}.csv" ) or die "Couldn't open ${basename}_${type}.csv. $!\n";

   my $hdr = CSVResources::get_assignment_file_header( $bvname,
                                                       $type,
                                                       \@type_columns,
                                                       $level_names_ref );

   print CSV $hdr;
   print CSV "\n";

   # get a list of all known objects
   my $sth = $dbh->prepare( $sql );

   my $name = undef;
   my $ip = undef;
   my $path = undef;
   my $oid = undef;

      $sth->execute( );

   $sth->bind_col( 1, \$name );

   if( $type =~ /APP_GROUP/i || $type =~ /CLUSTER/i ) {
      $sth->bind_col( 2, \$oid );
   }
   elsif( $type =~ /SWITCH/i || $type =~ /ARRAY/i || $type =~ /HOST/i ) {
      $sth->bind_col( 2, \$ip );
      $sth->bind_col( 3, \$oid );
   }
   elsif( $type =~ /NAS_VOLUME/i || $type =~ /NAS_SHARE/i || $type =~ /NAS_QTREE/i || $type =~ /NAS_FILESYSTEM/i || $type =~ /SERVICEGROUP/i || $type =~ /FILESYSTEM/i || $type =~ /VOLUME/i || $type =~ /DISKGROUP/i ) {
      $sth->bind_col( 2, \$path );
      $sth->bind_col( 3, \$oid );
   }

   #$sth->execute( );
   while( $sth->fetch( ) ) {
      my $line = "," x $level_count;
      $line .= "$name,";

      if( $type =~ /SWITCH/i || $type =~ /ARRAY/i || $type =~ /HOST/i ) {

         if( $ip ) {
            $line .= "\"$ip\"";
         }
         $line .= ",";
      }
      elsif( $type =~ /NAS_VOLUME/i || $type =~ /NAS_SHARE/i || $type =~ /NAS_QTREE/i || $type =~ /NAS_FILESYSTEM/i || $type =~ /SERVICEGROUP/i || $type =~ /FILESYSTEM/i || $type =~ /VOLUME/i || $type =~ /DISKGROUP/i) {
         $line .= "$path,";
      }
      
      $line .= "$oid\n";
      print CSV $line;
   }
   close CSV;
}

###############################################################################
#
# main entry point
#
my $usage =
q(Usage: bv_create.pl -d directory [-?] [-h]

Flags:
   -d  directory     : This required flag lets you define the directory
                       were you want to write the business view CSV files.

   -?                : Print this usage message.
   -h                : Print this usage message.

The bv_create.pl script is an interactive script that helps in the creation
of business view files.  It will ask a series of questions to help define
your business view.  Once all the questions are answered, it will generate
the business view .view and .csv files you'll need to further develop
your business view.  These business view files can later be imported using
the bv_import_csv.pl script.

Business views are written to a set of comma separated value (CSV) files.
An example import set and options file can be found in the VRTSccer/examples
directory.

This script will create a '.view' file for each business view, and CSV
files for each supported objec type.  All business view files are created in
the directory specified by the -d command line option.  If this option is not
given, this script will exit.
);

my %options = ( );
getopts("d:?h", \%options );

if( $options{ '?' } || $options{ 'h' } ) {
   print($usage);
   exit(0);
}

# the directory option is required
my $output_directory = $options{'d'};
if(!defined($output_directory))
{
    print(STDERR "The business view directory option (-d) is required.\n\n$usage");
    exit(-1);
}

PromptUser::clear_screen( );

print "Welcome to the VOM Advanced business view creation helper script.\n";
print "This script will as a series of questions to help define your business view.\n";
print "It concludes by generating appropriate .view and .csv files that you can\n";
print "later import using the bv_import_csv.pl script.\n";

print "\n\nConnecting to the VOM Advanced database.\n";
$dbh = SybaseHelpers::connect();

if ( !$dbh ) {
   print "\n\nUnable to connect to the database.\n\n\t";
   print SybaseHelpers::sybase_error_message( "", $DBI::err );
   exit;
}
print "\n\nSuccessfully connected to the database.\n\n";

if( ! -d $output_directory ) {
   if( PromptUser::prompt_user_yn( "Output directory \"$output_directory\" does not exist. Create it" ) ) {
      mkpath( $output_directory, { verbose => 1, mode => 0700 } ) or die "Couldn't create $output_directory. $!\n";
   }
   else {
      print "Output directory was not created. Exiting...\n";
      exit 7;
   }
}

my $bvname = PromptUser::prompt_user( "What is the name of your business view" );
my $bvdesc = PromptUser::prompt_user( "Please enter a description" );
print "\n";

my $level_count = PromptUser::prompt_user( "How many levels does this view support" );
print "\n";

my @level_names = ( );;
my @level_descs = ( );
for( my $i = 0; $i < $level_count; ++$i ) {
    my $level_name = PromptUser::prompt_user( "Level " . ($i + 1) . " name" );
    my $level_desc = PromptUser::prompt_user( "Level " . ($i + 1) . " description" );
    print "\n";

    push( @level_names, $level_name );
    push( @level_descs, $level_desc );
}

my @types = ( );

if( PromptUser::prompt_user_yn( "Does this view support Host objects" ) ) {
   push( @types, "Host" );
}
if( PromptUser::prompt_user_yn( "Does this view support Filesystem objects" ) ) {
   push( @types, "Filesystem" );
}
if( PromptUser::prompt_user_yn( "Does this view support Volume objects" ) ) {
   push( @types, "Volume" );
}
if( PromptUser::prompt_user_yn( "Does this view support Disk Group objects" ) ) {
   push( @types, "Diskgroup" );
}
if( PromptUser::prompt_user_yn( "Does this view support Array objects" ) ) {
   push( @types, "Array" );
}
#if( PromptUser::prompt_user_yn( "Does this view support Switch objects" ) ) {
#   push( @types, "Switch" );
#}
#if( PromptUser::prompt_user_yn( "Does this view support Application Group objects" ) ) {
#   push( @types, "App_Group" );
#}
#if( PromptUser::prompt_user_yn( "Does this view support NAS Volume objects" ) ) {
#   push( @types, "NAS_Volume" );
#}
if( PromptUser::prompt_user_yn( "Does this view support NAS Share objects" ) ) {
   push( @types, "NAS_Share" );
}
#if( PromptUser::prompt_user_yn( "Does this view support NAS Qtree objects" ) ) {
#   push( @types, "NAS_Qtree" );
#}
if( PromptUser::prompt_user_yn( "Does this view support NAS Filesystem objects" ) ) {
   push( @types, "NAS_Filesystem" );
}
if( PromptUser::prompt_user_yn( "Does this view support Cluster objects" ) ) {
   push( @types, "Cluster" );
}
if( PromptUser::prompt_user_yn( "Does this view support Service Group objects" ) ) {
   push( @types, "ServiceGroup" );
}

print "\n";

GET_BASENAME: my $basename = PromptUser::prompt_user( "Please enter a base name for the generated .view and .csv files" );
$basename = File::Spec->catfile( $output_directory, $basename );

if( -f "${basename}.view" ) {
   if( ! PromptUser::prompt_user_yn( "${basename}.view exists. Do you want to overwrite it" ) ) {
      print "\n";
      goto GET_BASENAME;
   }
}

print "Creating ${basename}.view file...\n\n";
create_view_file( $basename,
                  $bvname,
                  $bvdesc,
                  $level_count,
                  \@level_names,
                  \@level_descs,
                  \@types );


foreach my $type ( @types ) {
   create_type_csv_file( $basename, $bvname, $type, \@level_names );
}

print "Creation of business view csv files for view $bvname complete.\n";
$dbh->disconnect( );

