#
# $Id: bv_export_csv.pl,v 1.34 2012/07/11 23:30:05 sajeevm Exp $
#
# $Source: /project/sahara/src/import/bv_export_csv.pl,v $
#
# $Copyright: Copyright (c) 2012 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
use strict;
use warnings;

use ERPERLLIB;
require VRTS::VOMERPaths;

use CSVResources;
use DBI;
use ERLogging;
use File::Basename;
use File::Spec;
use Getopt::Std;
use SybaseHelpers;
use PromptUser;

my @view_column_names;
my @view_column_display_names;
my @view_types;

my %map_nas_type_to_netapp_type =
(
   'NETAPP_VOLUME' => 'NAS_VOLUME',
   'NETAPP_SHARE' => 'NAS_SHARE',
   'NETAPP_QTREE' => 'NAS_QTREE'
);

sub column_definition_get {

   my( $view_to_get, $dbh ) = @_;
   my $view_key;
  
   my $sql = "select BVIEW_KEY from SAHARA.BVIEW  ";
		$sql .= "where DISPLAY_NAME = \'" . $view_to_get . "\' ";

 	    my $sth = $dbh->prepare( $sql );
		$sth->execute( );
		$sth->bind_col( 1, \$view_key );
        $sth->fetch( );

   $sql = "select NAME, DISPLAY_NAME  "
           . "from SAHARA.BVIEW_LEVEL "
           . "where BVIEW_KEY = ".$view_key
           . "order by \"LEVEL\" ";

   $sth = $dbh->prepare( $sql );
   $sth->execute();
      $sth->execute();
   my $i = 1;
   
   @view_column_names = ( );
  
   while( my @data = $sth->fetchrow_array( ) ) {
      push( @view_column_names, $data[0] );
      push( @view_column_display_names, $data[1] );
      $i++;
   }

}

sub export_view_definition {
   my( $view_key, $view_name, $display_name, $note, $dir, $dbh ) = @_;

   my $fullpath = File::Spec->catfile( $dir, $view_name . ".view" );

   if( -e "$fullpath" ) {
      if( !PromptUser::prompt_user_yn("$fullpath exists. Do you want to overwrite this view set") ) {
         return undef;
      }
   }

   if( !open( DEF, ">${fullpath}" ) ) {
      ERLogging::write_log( "Could not open ${fullpath} file. $!\n" );
      return undef;
   }

   print DEF $CSVResources::view_section_comment;

   print DEF "[Business View]\n";
   print DEF "Name=${display_name}\n";
   print DEF "Description=${note}\n\n";

   my $sql = "select NAME, DISPLAY_NAME, NOTE "
           . "from SAHARA.BVIEW_LEVEL "
           . "where BVIEW_KEY=".$view_key
           . " order by \"LEVEL\" ";

   my $sth = $dbh->prepare( $sql );
   #$sth->execute( $view_key );
   $sth->execute();
   print DEF $CSVResources::level_sections_comment;

   my $i = 1;
   @view_column_names = ( );
   while( my @data = $sth->fetchrow_array( ) ) {
      print DEF "[Level${i}]\n";
      print DEF "Name=${data[1]}\n";
      print DEF "Description=${data[2]}\n\n";

      push( @view_column_names, $data[0] );
      push( @view_column_display_names, $data[1] );
      $i++;
   }

    print(DEF $CSVResources::types_section_comment);

    print(DEF "[Object Types]\n");
    print(DEF "Types=");

    $sql = "select DIMENSION, ALIAS "
         .   "from SAHARA.BVIEW_SUPPORTED_DIMENSION "
         .  "where BVIEW_KEY = ".$view_key;

    $sth = $dbh->prepare($sql);
    #$sth->execute($view_key);
    $sth->execute();
    @view_types = ();
    while(my @data = $sth->fetchrow_array())
    {
		if( $data[0] =~ /NETAPP_VOLUME/i || $data[0] =~ /NETAPP_SHARE/i || $data[0] =~ /NETAPP_QTREE/i ) {
			push(@view_types, $map_nas_type_to_netapp_type{$data[0]});
		}
		else {
			push(@view_types, $data[0]);
		}
		
        export_view_assigned_objects($view_name,
                                     $data[0],    # typename
                                     $data[1],    # alias
                                     $dir,
                                     $dbh);
    }

	print(DEF join(",", @view_types));
    close(DEF);
    

    return 1;
}

#
# Objects assigned to view
#
sub export_view_assigned_objects
{
    my ($view_name, $type, $alias, $dir, $dbh) = @_;

	my $fullpath;

	my @type_columns;

	if( uc($type) eq 'NETAPP_SHARE' ||  uc($type) eq 'NAS_FILESYSTEM')
	{
		push( @type_columns, 'Filer Name' );
	}
	elsif( uc($type) eq 'SERVICEGROUP')
	{
		push( @type_columns, 'Cluster Name' );
	}
	else
	{
		push( @type_columns, 'Name' );
	}

    if( uc($type) ne uc('App_Group') )
    {
		if( uc($type) eq 'NETAPP_SHARE' ||  uc($type) eq 'NETAPP_QTREE')
		{
			push( @type_columns, 'Share Path' );
		}
		elsif( uc($type) eq 'NETAPP_VOLUME')
		{
			push( @type_columns, 'Volume Name' );
		}
		elsif( uc($type) eq 'NAS_FILESYSTEM')
		{
			push( @type_columns, 'NAS Filesystem Name' );
		}
		elsif( uc($type) eq 'FILESYSTEM')
		{
			push( @type_columns, 'Filesystem Name' );
		}
		elsif( uc($type) eq 'VOLUME')
		{
			push( @type_columns, 'Volume Name' );
		}
		elsif( uc($type) eq 'CLUSTER')
		{
		}
		elsif( uc($type) eq 'SERVICEGROUP')
		{
			#push( @type_columns, 'Servicegroup Name' );
		}
		elsif( uc($type) eq 'DISKGROUP')
		{
			push( @type_columns, 'Diskgroup Name' );
		}
		else 
		{
			push( @type_columns, 'ip' );
		}
    }
    push( @type_columns, 'unique id' );

	if( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i ) {
        $fullpath = File::Spec->catfile( $dir, $view_name . "_${map_nas_type_to_netapp_type{$type}}" . ".csv" );
    }
    else {
		$fullpath = File::Spec->catfile( $dir, $view_name . "_${type}" . ".csv" );
	}	

   if( !open( CSV, ">$fullpath" ) ) {
      ERLogging::write_log( "Could not open $fullpath. $!\n" );
      return;
   }


   print CSV CSVResources::get_assignment_file_header($view_name,
                                                      $type,
                                                      \@type_columns,
                                                      \@view_column_display_names);

   print CSV "\n# ";

   my $sql = "select ";

   for( my $i = 0; $i < scalar( @view_column_names ); ++$i ) {
      $sql .= "T1.\"" . $view_column_names[$i] . "\",";
      print CSV $view_column_display_names[$i] . ",";
   }
#   $sql .= "T2.DISPLAY_NAME, ";
#   print CSV "name, ";

   if( $type eq 'HOST' ) {
      $sql .= "T2.DISPLAY_NAME, T2.IP_ADDRESS, ";
      print CSV "name, ip, ";
   }
   elsif( $type eq 'SWITCH' || $type eq 'ARRAY' ) {
      $sql .= "T2.DISPLAY_NAME, T2.SNMP_IP_ADDRESS, ";
      print CSV "name, ip, ";
   }
   elsif( $type eq 'FILESYSTEM' ) {
      $sql .= "T3.DISPLAY_NAME, T2.DISPLAY_NAME, ";
      print CSV "host name, filesystem name, ";
   }
   elsif( $type eq 'VOLUME' ) {
      $sql .= "T3.DISPLAY_NAME, T2.VOLUME_NAME, ";
      print CSV "host name, volume name, ";
   }
   elsif( $type eq 'NETAPP_VOLUME' ) {
      $sql .= "T3.DISPLAY_NAME, Fn_IE_CONCAT( '/vol/', T2.DISPLAY_NAME ), ";
      print CSV "Filer name, Volume Name, ";
   }
   elsif( $type eq 'NETAPP_SHARE' ) {
      $sql .= "T3.DISPLAY_NAME, T2.SHARE_PATH, ";
      print CSV "Filer name, Share Path, ";
   }
   elsif( $type eq 'NETAPP_QTREE' ) {
      $sql .= "T3.DISPLAY_NAME, T2.QTREE_PATH, ";
      print CSV "Filer name, Qtree Path, ";
   }
   elsif( $type eq 'NAS_FILESYSTEM' ) {
      $sql .= "T3.DISPLAY_NAME, T2.DISPLAY_NAME, ";
      print CSV "Filer name, NAS Filesystem Name, ";
   }
   elsif( $type eq 'CLUSTER' ) {
      $sql .= "T2.DISPLAY_NAME, ";
      print CSV "Cluster name, ";
   }
   elsif( $type eq 'SERVICEGROUP' ) {
      $sql .= "T3.DISPLAY_NAME, T2.DISPLAY_NAME,";
      print CSV "Cluster name, Servicegroup Name, ";
   }
   elsif( $type eq 'DISKGROUP' ) {
      $sql .= "T3.DISPLAY_NAME, T2.DISPLAY_NAME,";
      print CSV "Host name, Diskgroup Name, ";
   }
   $sql .= $type . "_OID ";
   print CSV "object unique id\n";

   $sql .= "from SAHARA.BV_${alias}_$view_name T1 "
        .  "join SAHARA.$type T2 on T1.${type}_KEY=T2.${type}_KEY";

	if( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i || $type =~ /NAS_FILESYSTEM/i ) {
	   $sql .= " join SAHARA.ARRAY_CURRENT T3 on T2.ARRAY_KEY=T3.ARRAY_KEY";
    }
	elsif( $type =~ /FILESYSTEM/i || $type =~ /VOLUME/i || $type =~ /DISKGROUP/i ) {
	   $sql .= " join SAHARA.HOST_CURRENT T3 on T2.HOST_KEY=T3.HOST_KEY";
    }	
	elsif( $type =~ /SERVICEGROUP/i ) {
	   $sql .= " join SAHARA.CLUSTER_CURRENT T3 on T2.CLUSTER_KEY=T3.CLUSTER_KEY";
    }	

   my $sth =$dbh->prepare( $sql );
   $sth->execute( );

 my @data = ();

   while( @data = $sth->fetchrow_array( ) ) {
      my $tmp = "";
      foreach my $entry ( @data ) {
         if( $entry && $entry ne 'N/A' ) {
            # make sure to quote all of our strings
            $tmp .= "\"" . $entry . "\"";
         }
         $tmp .= ",";
      }
      chop( $tmp );

      print CSV $tmp . "\n";
   }
   close( CSV );
}


#
# export all the nodes that have nothing assigned to them to the .Host file.
#
sub export_view_empty_nodes {
   my( $view_name, $dir, $dbh ) = ( @_ );

   my $fullpath = File::Spec->catfile( $dir, $view_name . "_HOST.csv" );

   if( !open( CSV, ">>$fullpath" ) ) {
      ERLogging::write_log( "Could not open $fullpath. $!\n" );
      return;
   }

   print CSV "\n\n# Additional Paths\n";

   # select all the branches that have nothing assigned to them.
   my $sql = "select ";

   foreach my $col ( @view_column_names ) {
      $sql .= "T1.\"" . $col . "\",";
   }
   chop( $sql );

   $sql .= " from SAHARA.BVS_$view_name t1 "
         . "left join SAHARA.HOST_BVIEW_NODE t2 "
         . "   ON t1.BVIEW_NODE_KEY=t2.BVIEW_NODE_KEY "
         . "left join SAHARA.ARRAY_BVIEW_NODE t3 "
         . "   ON t1.BVIEW_NODE_KEY=t3.BVIEW_NODE_KEY "
         . "left join SAHARA.SWITCH_BVIEW_NODE t4 "
         . "   ON t1.BVIEW_NODE_KEY=t4.BVIEW_NODE_KEY "
         . "left join SAHARA.APP_GROUP_BVIEW_NODE t5 "
         . "   ON t1.BVIEW_NODE_KEY=t5.BVIEW_NODE_KEY "
         . "left join SAHARA.LUN_BVIEW_NODE t6 "
         . "   ON t1.BVIEW_NODE_KEY=t6.BVIEW_NODE_KEY "
         . "where ( t2.bview_node_key is null AND "
         . "      t3.bview_node_key is null AND "
         . "      t4.bview_node_key is null AND "
         . "      t5.bview_node_key is null AND "
         . "      t6.bview_node_key is null ) "
         . "and \"" . $view_column_names[ $#view_column_names ] . "\" is not null ";

   my $sth = $dbh->prepare( $sql );
   $sth->execute( );

   while( my @data = $sth->fetchrow_array( ) ) {
      my $tmp = "";
      foreach my $entry ( @data ) {
         if( $entry && $entry ne 'N/A' ) {
            $tmp .= $entry;
         }
         $tmp .= ",";
      }
      $tmp .= ",,";

      print CSV $tmp . "\n";
   }

   close( CSV );
}

sub formatColumn
{
    my ($column, $terminator) = @_;

    if(defined($column))
    {
        chomp($column);
        return "\"" . $column . "\"" . $terminator;
    }
    # else

    return $terminator;
}

#
# Export all unassigned objects to the unassigned_<type>.csv file.   There
# will be one file per supported object type.  If given a $view_name, then
# only objects not assigned to that view are exported and the name is called
# <view>_unassigned_<type>.csv file.
#
sub export_unassigned_objects
{
    my ($dir, $dbh, $view_name) = @_;
	my $level_count = scalar( @view_column_names );

    # determine the supported dimensions (object types)
    my $sth;
    if(defined($view_name))
    {
        my $sql = "select DIMENSION, ALIAS "
                .   "from SAHARA.BVIEW_SUPPORTED_DIMENSION "
                .  "where BVIEW_DISPLAY_NAME = '".$view_name."\'";
        $sth = $dbh->prepare($sql);
      #  $sth->execute($view_name);
	  $sth->execute();
    }
    else
    {
        my $sql = "select DIMENSION, ALIAS from SAHARA.BVIEW_DIMENSION";
        $sth = $dbh->prepare($sql);
        $sth->execute();
    }

    # iterate across them and dump all the unassigned objects
    my @data;
    while(@data = $sth->fetchrow_array())
    {
        my ($type, $alias) = @data;

        if($type eq 'LUN')
        {
            # skip luns for now
            next;
        }

        my $type_table = $type . "_CURRENT";
        my $type_key = $type . "_KEY";
        my $join_table = $alias . "_BVIEW_NODE";

#        my $sql = "select T1.DISPLAY_NAME, ";
        my $sql = "select ";

        if($type eq 'APP_GROUP')
        {
            $sql .= "T1.DISPLAY_NAME, ";
        }
		if($type eq 'HOST')
        {
            $sql .= "T1.DISPLAY_NAME, T1.IP_ADDRESS, ";
        }
        elsif(($type eq 'ARRAY') || ($type eq 'SWITCH'))
        {
            $sql .= "T1.DISPLAY_NAME, T1.SNMP_IP_ADDRESS, ";
        }
		elsif( $type eq 'NETAPP_VOLUME' ) {
			$sql .= "T2.DISPLAY_NAME, Fn_IE_CONCAT( '/vol/', T1.DISPLAY_NAME ), ";
		}
		elsif( $type eq 'NETAPP_SHARE' ) {
			$sql .= "T2.DISPLAY_NAME, T1.SHARE_PATH, ";
		}
		elsif( $type eq 'NETAPP_QTREE' ) {
			$sql .= "T2.DISPLAY_NAME, T1.QTREE_PATH, ";
		}
		elsif( $type eq 'NAS_FILESYSTEM' ) {
			$sql .= "T2.DISPLAY_NAME, T1.DISPLAY_NAME, ";
		}
		elsif( $type eq 'FILESYSTEM' ) {
			$sql .= "T2.DISPLAY_NAME, T1.DISPLAY_NAME, ";
		}
		elsif( $type eq 'VOLUME' ) {
			$sql .= "T2.DISPLAY_NAME, T1.VOLUME_NAME, ";
		}
		elsif( $type eq 'CLUSTER' ) {
			$sql .= "T1.DISPLAY_NAME, ";

		}
		elsif( $type eq 'SERVICEGROUP' ) {
			$sql .= "T2.DISPLAY_NAME, T1.DISPLAY_NAME, ";

		}
		elsif( $type eq 'DISKGROUP' ) {
			$sql .= "T2.DISPLAY_NAME, T1.DISPLAY_NAME, ";

		}
        else
        {
            $sql .= "null, ";
        }

        $sql .= "T1.${type}_OID ";

		if( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i || $type =~ /NAS_FILESYSTEM/i ) {
			$sql .= " from SAHARA.$type_table T1 join SAHARA.ARRAY_CURRENT T2 on T1.ARRAY_KEY=T2.ARRAY_KEY ";
	    }
		elsif( $type =~ /FILESYSTEM/i || $type =~ /VOLUME/i  || $type =~ /DISKGROUP/i ) {
			$sql .= " from SAHARA.$type_table T1 join SAHARA.HOST_CURRENT T2 on T1.HOST_KEY=T2.HOST_KEY ";
	    }
		elsif( $type =~ /SERVICEGROUP/i ) {
			$sql .= " from SAHARA.$type_table T1 join SAHARA.CLUSTER_CURRENT T2 on T2.CLUSTER_KEY=T1.CLUSTER_KEY ";
		}	
		else {
            $sql .= " from SAHARA.$type_table T1 "			
		}

        my ($fullpath, $sth2);
        if(defined($view_name))
        {
#            $sql .= "from SAHARA.$type_table T1 "
#                 . "where not exists (select 1 "
            $sql .= "where not exists (select 1 "
                 .                     "from SAHARA.$join_table "
                 .                    "where $type_key = T1.$type_key "
                 .                      "and BVIEW_KEY = (select BVIEW_KEY "
                 .                                         "from SAHARA.BVIEW "
                 .                                        "where DISPLAY_NAME = '".$view_name."'\))";
            $sth2 = $dbh->prepare($sql);
         #   $sth2->execute($view_name);
             $sth2->execute();
            # tbd - it might be better to get the oracle column name than
            #       to use this generation function
            my $sybase_view_name = SybaseHelpers::GetSybaseName($view_name);
			if( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i ) {
	            $fullpath = File::Spec->catfile($dir, $sybase_view_name . '_UNASSIGNED_' . $map_nas_type_to_netapp_type{$type} . ".csv");
			}
			else {
	            $fullpath = File::Spec->catfile($dir, $sybase_view_name . '_UNASSIGNED_' . $type . ".csv");
			}	
        }
        else
        {
#            $sql .= "from SAHARA.$type_table T1 "
#                 . "where not exists (select 1 "
           $sql .= "where not exists (select 1 "
                 .                     "from SAHARA.$join_table "
                 .                    "where $type_key = T1.$type_key)";

            #ERLogging::write_log("SQL $sql\n");
		#	 print $sql;
		#	 print "\n";
			$sth2 = $dbh->prepare($sql);
            $sth2->execute();
			if( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i ) {
	            $fullpath = File::Spec->catfile($dir, 'UNASSIGNED_' . $map_nas_type_to_netapp_type{$type} . ".csv");
			}
			else {
	            $fullpath = File::Spec->catfile($dir, 'UNASSIGNED_' . $type . ".csv");
			}	
        }

        if(!open(CSV, ">$fullpath"))
        {
            # tbd - this should probably be a fatal error
            ERLogging::write_log("Could not open file $fullpath. $!\n");
            return;
        }
		#print $sql;
		#print "\n";
        my $tmp;
        if($view_name)
        {
           $tmp = sprintf($CSVResources::unassigned_to_view_header, $type, $view_name);
		   print(CSV $tmp);
		   print(CSV "# ");
			
		   for( my $i = 0; $i < scalar( @view_column_names ); ++$i ) {
    
             print CSV $view_column_display_names[$i] . ",";
        
		   }
        
       }
        else
        {
           $tmp = sprintf($CSVResources::unassigned_header, $type);
		   print(CSV $tmp);
		   print(CSV "# ");

        }

	    if(($type eq 'HOST') || ($type eq 'ARRAY') || ($type eq 'SWITCH') || ($type eq 'APP_GROUP'))
		{
	        print(CSV " display name, ");
        }
		elsif(($type eq 'FILESYSTEM') || ($type eq 'VOLUME') || ($type eq 'DISKGROUP'))
		{
	        print(CSV " host name, ");
        }
		elsif(($type eq 'NETAPP_SHARE') || ($type eq 'NETAPP_QTREE')|| ($type eq 'NETAPP_VOLUME')|| ($type eq 'NAS_FILESYSTEM'))
		{
	        print(CSV " filer name, ");
        }

        if(($type eq 'HOST') || ($type eq 'ARRAY') || ($type eq 'SWITCH'))
        {
            print(CSV "ip address, ");
        }
		elsif( $type eq 'FILESYSTEM' ) {
			print CSV "Filesystem Name, ";
		}
		elsif( $type eq 'VOLUME' ) {
			print CSV "Volume Name, ";
		}
		elsif( $type eq 'DISKGROUP' ) {
			print CSV "Diskgroup Name, ";
		}
		elsif( $type eq 'NETAPP_SHARE' ) {
			print CSV "Share Path, ";
		}
		elsif( $type eq 'NETAPP_QTREE' ) {
			print CSV "Qtree Path, ";
		}
		elsif( $type eq 'NETAPP_VOLUME' ) {
			print CSV "Volume Name, ";
		}
		elsif( $type eq 'NAS_FILESYSTEM' ) {
			print CSV "NAS Filesystem Name, ";
		}
		elsif( $type eq 'CLUSTER' ) {		
			print CSV "Cluster name, ";
		}
		elsif( $type eq 'SERVICEGROUP' ) {
			print CSV "Cluster name, Servicegroup Name, ";
		}
        print(CSV " object_unique_id\n");
       # print $type;
		#print "\n";

        my ($col1, $col2, $col3);
		$sth2->execute();
        $sth2->bind_col(1, \$col1);
        $sth2->bind_col(2, \$col2);
		if ($type ne 'CLUSTER') {
	        $sth2->bind_col(3, \$col3);
		}
        while($sth2->fetch())
        {

            if($view_name)
            {
               print (CSV "," x $level_count);
			}

			if ($type ne 'CLUSTER') {
            print(CSV formatColumn($col1, ",") .
                      formatColumn($col2, ",") .
                      formatColumn($col3, "\n"));
			}			else {
				print(CSV formatColumn($col1, ",") .
                      formatColumn($col2, "\n"));
			}
        }

        close(CSV);
    }
}

#
# Export all objects objects to the all_<type>.csv file. There will
# be one file per supported object type.
#
sub export_all_objects {
   my ($dir, $dbh, $view_name) = @_;

   my @level_names = ( );

   my $sql = "select DIMENSION from SAHARA.BVIEW_DIMENSION";
   my $sth = $dbh->prepare( $sql );
   my $type = undef;
   my $fullpath;
   $sth->execute( );
   $sth->bind_col( 1, \$type );


   while( $sth->fetch( ) ) {

      my @type_columns;

      if( $type =~ /APP_GROUP/i ) {
         $sql = "select DISPLAY_NAME, APP_GROUP_OID from SAHARA.APP_GROUP_CURRENT";
         @type_columns = ( 'name', 'object_unique_id' );
      }
      elsif( $type =~ /SWITCH/i || $type =~ /ARRAY/i ) {
         $sql = "select DISPLAY_NAME, SNMP_IP_ADDRESS, ${type}_OID from SAHARA.${type}_CURRENT";
         @type_columns = ( 'name', 'ip', 'object_unique_id' );
      }
      elsif( $type =~ /HOST/i ) {
         $sql = "select DISPLAY_NAME, IP_ADDRESS, HOST_OID from SAHARA.HOST_CURRENT";
         @type_columns = ( 'name', 'ip', 'object_unique_id' );
      }
      elsif( $type =~ /NETAPP_VOLUME/i ) {
         $sql = "select t1.DISPLAY_NAME, Fn_IE_CONCAT( '/vol/', t2.DISPLAY_NAME ), t2.NETAPP_VOLUME_OID from "
              .  "SAHARA.ARRAY_CURRENT t1, "
              .  "SAHARA.NETAPP_VOLUME_CURRENT t2 "
              .  "where t2.ARRAY_KEY = t1.ARRAY_KEY "
              .  "order by t1.DISPLAY_NAME ";
         @type_columns = ( 'filer_name', 'path', 'object_unique_id' )
      }
      elsif( $type =~ /NETAPP_SHARE/i ) {
         $sql = "select t1.DISPLAY_NAME, t2.SHARE_PATH, t2.NETAPP_SHARE_OID from "
              .  "SAHARA.ARRAY_CURRENT t1, "
              .  "SAHARA.NETAPP_SHARE_CURRENT t2 "
              .  "where t2.ARRAY_KEY = t1.ARRAY_KEY "
              .  "order by t1.DISPLAY_NAME ";
         @type_columns = ( 'filer_name', 'path', 'object_unique_id' )
      }
      elsif( $type =~ /NETAPP_QTREE/i ) {
         $sql = "select t1.DISPLAY_NAME, t2.QTREE_PATH, t2.NETAPP_QTREE_OID from "
              .  "SAHARA.ARRAY_CURRENT t1, "
              .  "SAHARA.NETAPP_QTREE_CURRENT t2 "
              .  "where t2.ARRAY_KEY = t1.ARRAY_KEY "
              .  "order by t1.DISPLAY_NAME ";
         @type_columns = ( 'filer_name', 'path', 'object_unique_id' )
      }
	  elsif( $type =~ /NAS_FILESYSTEM/i ) {
         $sql = "select t1.DISPLAY_NAME, t2.DISPLAY_NAME, t2.NAS_FILESYSTEM_OID from "
              .  "SAHARA.ARRAY_CURRENT t1, "
              .  "SAHARA.NAS_FILESYSTEM_CURRENT t2 "
              .  "where t2.ARRAY_KEY = t1.ARRAY_KEY "
              .  "order by t1.DISPLAY_NAME ";
         @type_columns = ( 'filer_name', 'nas_filesystem_name', 'object_unique_id' )
      }
 	  elsif( $type =~ /FILESYSTEM/i ) {
         $sql = "select t1.DISPLAY_NAME, t2.DISPLAY_NAME, t2.FILESYSTEM_OID from "
              .  "SAHARA.HOST_CURRENT t1, "
              .  "SAHARA.FILESYSTEM_CURRENT t2 "
              .  "where t2.HOST_KEY = t1.HOST_KEY "
              .  "order by t1.DISPLAY_NAME ";
         @type_columns = ( 'host_name', 'filesystem_name', 'object_unique_id' )
      }
 	  elsif( $type =~ /VOLUME/i ) {
         $sql = "select t1.DISPLAY_NAME, t2.VOLUME_NAME, t2.VOLUME_OID from "
              .  "SAHARA.HOST_CURRENT t1, "
              .  "SAHARA.VOLUME_CURRENT t2 "
              .  "where t2.HOST_KEY = t1.HOST_KEY "
              .  "order by t1.DISPLAY_NAME ";
         @type_columns = ( 'host_name', 'volume_name', 'object_unique_id' )
      }
	  elsif( $type =~ /DISKGROUP/i ) {
         $sql = "select t1.DISPLAY_NAME, t2.DISPLAY_NAME, t2.DISKGROUP_OID from "
              .  "SAHARA.HOST_CURRENT t1, "
              .  "SAHARA.DISKGROUP_CURRENT t2 "
              .  "where t2.HOST_KEY = t1.HOST_KEY "
              .  "order by t1.DISPLAY_NAME ";
         @type_columns = ( 'host_name', 'diskgroup_name', 'object_unique_id' )
      } 
       elsif( $type =~ /CLUSTER/i ) {
         $sql = "select DISPLAY_NAME, CLUSTER_OID from SAHARA.CLUSTER_CURRENT";
         @type_columns = ( 'name', 'object_unique_id' );
      }
	  elsif( $type =~ /SERVICEGROUP/i ) {
         $sql = "select T2.DISPLAY_NAME, T1.DISPLAY_NAME,T1.SERVICEGROUP_OID from SAHARA.SERVICEGROUP_CURRENT T1 join SAHARA.CLUSTER_CURRENT T2 on T2.CLUSTER_KEY=T1.CLUSTER_KEY ";
         @type_columns = ( 'Cluster Name','ServiceGroup Name', 'object_unique_id' );
      }

	  if( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i ) {
        print("Creating all_$map_nas_type_to_netapp_type{$type}.csv file...\n\n");
		$fullpath = File::Spec->catfile( $dir, "all_$map_nas_type_to_netapp_type{$type}.csv" );
	  }
	  else {
        print("Creating all_$type.csv file...\n\n"); 
		$fullpath = File::Spec->catfile( $dir, "all_$type.csv" );
	  }	

      open( CSV, ">$fullpath" ) or die "Couldn't open $fullpath. $!\n";
      my $hdr = CSVResources::get_assignment_file_header('All',
                                                         $type,
                                                         \@type_columns,
                                                         \@level_names);

      print CSV $hdr;
      print CSV "\n";

      # get a list of all known objects
      my $sth2 = $dbh->prepare( $sql );

      my $name = undef;
      my $ip = undef;
      my $path = undef;
      my $oid = undef;
	  my $SGName = undef;
      $sth2->execute( );
      $sth2->bind_col( 1, \$name );

      if( $type =~ /APP_GROUP/i ) {
         $sth2->bind_col( 2, \$oid );
      }
      elsif( $type =~ /SWITCH/i || $type =~ /ARRAY/i || $type =~ /HOST/i ) {
         $sth2->bind_col( 2, \$ip );
         $sth2->bind_col( 3, \$oid );
      }
      elsif( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i || $type =~ /NAS_FILESYSTEM/i || $type =~ /FILESYSTEM/i || $type =~ /VOLUME/i || $type =~ /DISKGROUP/i ) {
         $sth2->bind_col( 2, \$path );
         $sth2->bind_col( 3, \$oid );
      }
      elsif( $type =~ /SERVICEGROUP/i ) {
         $sth2->bind_col( 2, \$SGName );
		 $sth2->bind_col( 3, \$oid );
      }
      elsif( $type =~ /CLUSTER/i ) {
         $sth2->bind_col( 2, \$oid );
      }


      while( $sth2->fetch( ) ) {
         my $line .= "$name,";

         if( $type =~ /SWITCH/i || $type =~ /ARRAY/i || $type =~ /HOST/i ) {

            if( $ip ) {
               $line .= "\"$ip\"";
            }
            $line .= ",";
         }
         elsif( $type =~ /NETAPP_VOLUME/i || $type =~ /NETAPP_SHARE/i || $type =~ /NETAPP_QTREE/i || $type =~ /NAS_FILESYSTEM/i || $type =~ /FILESYSTEM/i || $type =~ /VOLUME/i || $type =~ /DISKGROUP/i ) {
            $line .= "$path,";
         }
		 elsif( $type =~ /SERVICEGROUP/i ) {
			 $line .= "$SGName,";
	     }

         $line .= "$oid\n";
         print CSV $line;
      }
      close CSV;
   }
}

###############################################################################
#
# main entry point
#
my $usage =
q(Usage: bv_export_csv.pl -d directory [-u] [-v view_name] [-?] [-h]

Flags:
   -a             This optional flag tells the script to export all objects
                  found in the ER database that are supported by business
                  views, regardless of their assignment state.
   -d directory   This required flag lets you define the directory where
                  you want to write the business view export CSV files.
   -u             This optional flag tells the script to only export
                  objects that are not assigned to a business view.
                  If -u is specified with -v, then objects that are
                  not assigned to the given view are exported.
   -v view_name   This optonal flag lets you export a specific view
                  by providing the view name.  By default, this script
                  will export all business views.
   -?             Print this usage message.
   -h             Print this usage message.

The bv_export_csv.pl script is used to export Enterprise Reporter business
views to a set of comma separated value (CSV) files.  An example
export set and options file can be found in the VRTSccer/examples directory.

This script will create a 'view' file for each business view unless the -u
option is specified.  Each '.view' file defines the structure of a single
Enterprise Reporter business view.

This script can export all of the business views known to Enterprise Reporter
or a single business view by specifying the business view display name using
the -v option.

The view files are written to the directory specified by the -d option.  If
this option is not specified, the script will exit.

By default, the script generates an assignment file for each supported object
type in the business view and an 'UNASSIGNED' file for each of these types.
If you provide the '-u' switch, only the unassigned objects are exported.
You can combine this with the '-v' option to export only objects that are
not assigned to the provided business view.
);

my %options;
getopts("ad:huv:?", \%options);

if($options{'?'} || $options{'h'})
{
    print($usage);
    exit 0;
}

# the directory option is required
my $export_dir = $options{'d'};
if(!defined($export_dir))
{
    print(STDERR "The business view directory option (-d) is required.\n\n$usage");
    exit(-1);
}

my $view_to_get = $options{ 'v' };

my $log_dir = VRTS::VOMERPaths::get_apppath();
$log_dir = $log_dir  . "\\logs\\business_view";

if(! $log_dir)
{
    $log_dir = "log";
}

my $all = $options{'a'} || 0;

ERLogging::init($log_dir, 'export', undef, 2);

if(! -e $export_dir)
{
    if(PromptUser::prompt_user_yn("Directory $export_dir doesn't exist. Create it"))
    {
        mkdir($export_dir, 755);
    }
    else
    {
        exit;
    }
}

my $unassigned_only = $options{ 'u' } || 0;

my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
my $start_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d\n",
                          $year+1900,$mon+1,$mday,$hour,$min,$sec );

ERLogging::write_log("Enterprise Reporter export started at $start_time\n");
ERLogging::write_log("Connecting to the ER database...\n");

my $dbh = SybaseHelpers::connect();


ERLogging::write_log("Connected to the ER database.\n");

# iterate across all of our views.
my $sth;
my $rows = 0;

if($all)
{
    ERLogging::write_log( "Exporting all known objects to $export_dir.\n\n" );
    export_all_objects( $export_dir, $dbh );
    ERLogging::write_log( "Export complete.\n" );
    goto EXIT;
}

if( ! $unassigned_only ) {

   my ( $view_key, $view_name, $view_display_name, $view_note );

   my $count_sql = "select COUNT(*) from SAHARA.BVIEW ";

   if( $view_to_get ) {
      $count_sql .= "where DISPLAY_NAME='" . $view_to_get . "\' ";
   }

   $sth = $dbh->prepare( $count_sql );
   $sth->execute( );
   $sth->bind_col( 1, \$rows );
   $sth->fetch( );

   if( $rows == 0 ) {
      if( $view_to_get ) {
         print "\nView \'$view_to_get' was not found in the ER database - skipping.\n";
      }
      else {
         print "\nNo views were found in the ER database - export skipped. If you wish to save\n";
         print "all unassigned objects, provide the -u command line switch.\n";
      }
   }
   else {
      my $sql = "select BVIEW_KEY, NAME, DISPLAY_NAME, NOTE "
              . "from SAHARA.BVIEW ";

      if( $view_to_get ) {
         $sql .= "where DISPLAY_NAME=\'" . $view_to_get . "\' ";
      }

      $sth = $dbh->prepare( $sql );
      $sth->execute( );
      $sth->bind_col( 1, \$view_key );
      $sth->bind_col( 2, \$view_name );
      $sth->bind_col( 3, \$view_display_name );
      $sth->bind_col( 4, \$view_note );


      while( $sth->fetch( ) ) {
         # reset after the last run
         @view_column_names = ( );
         @view_column_display_names = ( );
         @view_types = ( );

         ERLogging::write_log( "Found business view '$view_display_name'. Exporting the view definition.\n" );

         if( export_view_definition( $view_key,
                                     $view_name,
                                     $view_display_name,
                                     $view_note,
                                     $export_dir,
                                     $dbh ) ) {
             export_view_empty_nodes( $view_name, $export_dir, $dbh );
             ERLogging::write_log( "Exporting all objects that are not assigned to business view \'$view_display_name'\n" );
             export_unassigned_objects( $export_dir, $dbh, $view_display_name );
         }
      }
   }

   # export objects not assigned to any view
   ERLogging::write_log( "Exporting all objects that are not assigned to any business view.\n" );
   export_unassigned_objects( $export_dir, $dbh, undef );
}

# unassigned objects only
else {

   if( ! $view_to_get ) {
      ERLogging::write_log( "Exporting all objects that are not assigned to any business view.\n" );
   }
   else {
      column_definition_get($view_to_get, $dbh);
      ERLogging::write_log( "Exporting all objects that are not assigned to business view \'$view_to_get\'\n" );
   }

   export_unassigned_objects( $export_dir, $dbh, $view_to_get );
}

EXIT: if( $sth ) {
   $sth->finish( );
}
if( $dbh ) {
   $dbh->disconnect( );
}

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
my $end_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d\n",
                          $year+1900,$mon+1,$mday,$hour,$min,$sec );

ERLogging::write_log( "\nER Export script completed at $end_time\n" );
