/*==============================================================*/
/* DBMS name:      Sybase SQL Anywhere 11                       */
/* Created on:     25/09/2012 4:12:03 PM                        */
/*==============================================================*/


/*==============================================================*/
/* Table: ACTUAL_ARRAY_CLAIMED_CAPACITY                         */
/*==============================================================*/
create table SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY 
(
   ARRAY_KEY            numeric(10)                    not null,
   CLAIMED_BY_HOSTS     numeric(22,9)                  null,
   PERCENT_ALLOCATED_HOST_CLAIMED numeric(7,4)                   null
);

comment on table SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY is 
'Result join table which maintains relationships between duplicate Arrays and its claimed facts across multi-site rollup.';

comment on column SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY.ARRAY_KEY is 
'The surrogate key of the Array for which claimed facts is being stored.  This key may be used in joins to the Array table.';

comment on column SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY.CLAIMED_BY_HOSTS is 
'The total capacity, in gigabytes, of LUNs in arrays and unified storage devices that have been claimed by at least one host.

A LUN is considered claimed when a host''s operating system has written a device handle for it.';

comment on column SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY.PERCENT_ALLOCATED_HOST_CLAIMED is 
'The percentage of storage allocated to host in which a host''s O/S has written a device handle to the LUN.  This value is calculated by dividing "Claimed Capacity (GB)" by the sum of "Capacity Allocated to Hosts Directly (GB)" and "Capacity Allocated to Hosts Through Virtualizer (GB)".';

/*==============================================================*/
/* Index: I_AACC_ARRAYKEY                                       */
/*==============================================================*/
create index I_AACC_ARRAYKEY on SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY (
ARRAY_KEY ASC
);

grant SELECT on SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: ACTUAL_LUN_CLAIMED_STATES                             */
/*==============================================================*/
create table SAHARA.ACTUAL_LUN_CLAIMED_STATES 
(
   LUN_KEY              numeric(10)                    not null,
   CLAIMED_STATUS       numeric(1)                     null
);

comment on table SAHARA.ACTUAL_LUN_CLAIMED_STATES is 
'Result join table which maintains relationships between duplicate LUNs and its claimed state across multi-site rollup.';

comment on column SAHARA.ACTUAL_LUN_CLAIMED_STATES.LUN_KEY is 
'The surrogate key of the LUN for which claimed status is being stored.  This key may be used in joins to the LUN table.';

comment on column SAHARA.ACTUAL_LUN_CLAIMED_STATES.CLAIMED_STATUS is 
'The Claimed Status of the LUN.

Possible values:
- 0
- 1
- 2';

/*==============================================================*/
/* Index: I_LACC_LUNKEY                                         */
/*==============================================================*/
create index I_LACC_LUNKEY on SAHARA.ACTUAL_LUN_CLAIMED_STATES (
LUN_KEY ASC
);

grant SELECT on SAHARA.ACTUAL_LUN_CLAIMED_STATES to SAHARA_APP;

/*==============================================================*/
/* Table: APP_GROUP                                             */
/*==============================================================*/
create table SAHARA.APP_GROUP 
(
   APP_GROUP_KEY        numeric(10)                    not null default autoincrement,
   APP_GROUP_OID        varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DESCRIPTION          varchar(2048)                  null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.APP_GROUP is 
'Application groups contain collections of databases and file systems that make up a business application.
For example: an ERP application may contain several Oracle instances and file systems used by web application.
Application groups track the capacity usage for this ERP.

Application groups are created in CommandCentral Storage GUI.

NOTE: When you add an Oracle instance to the group, note it''s file systems.  Do not include those file systems explicitly.
Including a file system that is used by and Oracle instance, which is already in the group will result in double counting.';

comment on column SAHARA.APP_GROUP.APP_GROUP_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.APP_GROUP.CC_STORAGE_URL is 
'The URL for the application group object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.APP_GROUP.DISPLAY_NAME is 
'Application group display name';

comment on column SAHARA.APP_GROUP.DESCRIPTION is 
'User typed description of this application group';

comment on column SAHARA.APP_GROUP.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.APP_GROUP.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.APP_GROUP
   alter T2_IS_CURRENT add constraint C_APPGROUP_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.APP_GROUP
   alter T2_IS_DELETED add constraint C_APPGROUP_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.APP_GROUP
   add constraint PK_APP_GROUP primary key (APP_GROUP_KEY);

/*==============================================================*/
/* Index: I_APPGROUP_APPGROUPOID                                */
/*==============================================================*/
create index I_APPGROUP_APPGROUPOID on SAHARA.APP_GROUP (
APP_GROUP_OID ASC
);

comment on index SAHARA.APP_GROUP.I_APPGROUP_APPGROUPOID is 
'An index on Application Group Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_APPGROUP_DISPLAYNAME                                */
/*==============================================================*/
create index I_APPGROUP_DISPLAYNAME on SAHARA.APP_GROUP (
DISPLAY_NAME ASC
);

comment on index SAHARA.APP_GROUP.I_APPGROUP_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_APPGROUP_DESCRIPTION                                */
/*==============================================================*/
create index I_APPGROUP_DESCRIPTION on SAHARA.APP_GROUP (
DESCRIPTION ASC
);

comment on index SAHARA.APP_GROUP.I_APPGROUP_DESCRIPTION is 
'An index on Description for dimensional queries.';

/*==============================================================*/
/* Index: I_APPGROUP_T2SCRRNTPPGRPK                             */
/*==============================================================*/
create index I_APPGROUP_T2SCRRNTPPGRPK on SAHARA.APP_GROUP (
T2_IS_CURRENT ASC,
APP_GROUP_KEY ASC
);

comment on index SAHARA.APP_GROUP.I_APPGROUP_T2SCRRNTPPGRPK is 
'An index on Type 2 - Is Current and Application Group Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_APPGROUP_T2SCRRNTT2SDLTDD                           */
/*==============================================================*/
create index I_APPGROUP_T2SCRRNTT2SDLTDD on SAHARA.APP_GROUP (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
APP_GROUP_OID ASC
);

comment on index SAHARA.APP_GROUP.I_APPGROUP_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Application Group Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_APPGROUP_T2ISDELETED                                */
/*==============================================================*/
create index I_APPGROUP_T2ISDELETED on SAHARA.APP_GROUP (
T2_IS_DELETED ASC
);

comment on index SAHARA.APP_GROUP.I_APPGROUP_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

grant SELECT on SAHARA.APP_GROUP to SAHARA_APP;

/*==============================================================*/
/* Table: APP_GROUP_                                            */
/*==============================================================*/
create table SAHARA_APP.APP_GROUP_ 
(
   APP_GROUP_OID        varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DESCRIPTION          varchar(2048)                  null
);

comment on table SAHARA_APP.APP_GROUP_ is 
'This is a staging table for loading data into warehouse table SAHARA.APP_GROUP.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  Application groups contain collections of databases and file systems that make up a business application.
For example: an ERP application may contain several Oracle instances and file systems used by web application.
Application groups track the capacity usage for this ERP.

Application groups are created in CommandCentral Storage GUI.

NOTE: When you add an Oracle instance to the group, note it''s file systems.  Do not include those file systems explicitly.
Including a file system that is used by and Oracle instance, which is already in the group will result in double counting.';

comment on column SAHARA_APP.APP_GROUP_.APP_GROUP_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.APP_GROUP_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.APP_GROUP, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application group display name';

comment on column SAHARA_APP.APP_GROUP_.DESCRIPTION is 
'This column is for staging data that will end up in table SAHARA.APP_GROUP, column DESCRIPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

User typed description of this application group';

alter table SAHARA_APP.APP_GROUP_
   add constraint PK_APP_GROUP_ primary key (APP_GROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.APP_GROUP_ to SAHARA;

/*==============================================================*/
/* Table: APP_GROUP_BVIEW_NODE                                  */
/*==============================================================*/
create table SAHARA.APP_GROUP_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   APP_GROUP_KEY        numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.APP_GROUP_BVIEW_NODE is 
'This is a join table between application group objects and business view nodes.';

comment on column SAHARA.APP_GROUP_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.APP_GROUP_BVIEW_NODE.APP_GROUP_KEY is 
'The unique identifier of the application group object with which the business view node is associated.';

comment on column SAHARA.APP_GROUP_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.APP_GROUP_BVIEW_NODE
   add constraint PK_APP_GROUP_BVIEW_NODE primary key (BVIEW_NODE_KEY, APP_GROUP_KEY);

/*==============================================================*/
/* Index: I_APPGRPBVIEWNODE_BVIEWAPPGRP                         */
/*==============================================================*/
create unique index I_APPGRPBVIEWNODE_BVIEWAPPGRP on SAHARA.APP_GROUP_BVIEW_NODE (
BVIEW_KEY ASC,
APP_GROUP_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.APP_GROUP_BVIEW_NODE.I_APPGRPBVIEWNODE_BVIEWAPPGRP is 
'This index exists to accelerate queries for which application groups are associated with a particular business view.';

grant SELECT on SAHARA.APP_GROUP_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: APP_GROUP_CAPACITY                                    */
/*==============================================================*/
create table SAHARA.APP_GROUP_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   APP_GROUP_KEY        numeric(10)                    not null,
   ALLOCATED            numeric(22,9)                  not null default 0,
   USED                 numeric(22,9)                  not null default 0,
   UNUSED               numeric(22,9)                  not null default 0,
   "UNKNOWN"            numeric(22,9)                  not null default 0
);

comment on table SAHARA.APP_GROUP_CAPACITY is 
'Application groups can spawn hosts,';

comment on column SAHARA.APP_GROUP_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.APP_GROUP_CAPACITY.ALLOCATED is 
'The amount of storage that is apportioned into LUNs and designated for use by specific application groups.
This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.
("Claimed" means that the operating system has written a device handle for the LUN.)

This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is
also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.';

comment on column SAHARA.APP_GROUP_CAPACITY.USED is 
'Capacity that has data written into it.';

comment on column SAHARA.APP_GROUP_CAPACITY.UNUSED is 
'Capacity still available for writing into.';

comment on column SAHARA.APP_GROUP_CAPACITY."UNKNOWN" is 
'Unknown capacity for application groups is calculated as
Total capacity - used capacity - available/free capacity';

alter table SAHARA.APP_GROUP_CAPACITY
   alter ALLOCATED add constraint C_PPGRPCPCT_ALLOCATED check (ALLOCATED >= 0);

alter table SAHARA.APP_GROUP_CAPACITY
   alter USED add constraint C_PPGRPCPCT_USED check (USED >= 0);

alter table SAHARA.APP_GROUP_CAPACITY
   alter UNUSED add constraint C_PPGRPCPCT_UNUSED check (UNUSED >= 0);

alter table SAHARA.APP_GROUP_CAPACITY
   alter "UNKNOWN" add constraint C_PPGRPCPCT_UNKNOWN check ("UNKNOWN" >= 0);

/*==============================================================*/
/* Index: I_APPGROUPCAP_ROLLUPBATCHKEY                          */
/*==============================================================*/
create index I_APPGROUPCAP_ROLLUPBATCHKEY on SAHARA.APP_GROUP_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_APPGROUPCAP_DATESKEY                                */
/*==============================================================*/
create index I_APPGROUPCAP_DATESKEY on SAHARA.APP_GROUP_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_APPGROUPCAP_APPGROUPKEY                             */
/*==============================================================*/
create index I_APPGROUPCAP_APPGROUPKEY on SAHARA.APP_GROUP_CAPACITY (
APP_GROUP_KEY ASC
);

grant SELECT on SAHARA.APP_GROUP_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: APP_GROUP_CAPACITY_                                   */
/*==============================================================*/
create table SAHARA_APP.APP_GROUP_CAPACITY_ 
(
   APP_GROUP_OID        varchar(512)                   not null,
   ALLOCATED            numeric(22,9)                  not null default 0,
   USED                 numeric(22,9)                  not null default 0,
   UNUSED               numeric(22,9)                  not null default 0,
   "UNKNOWN"            numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.APP_GROUP_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.APP_GROUP_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  Application groups can spawn hosts,';

comment on column SAHARA_APP.APP_GROUP_CAPACITY_.APP_GROUP_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.APP_GROUP_CAPACITY_.ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.APP_GROUP_CAPACITY, column ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific application groups.
This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.
("Claimed" means that the operating system has written a device handle for the LUN.)

This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is
also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.';

comment on column SAHARA_APP.APP_GROUP_CAPACITY_.USED is 
'This column is for staging data that will end up in table SAHARA.APP_GROUP_CAPACITY, column USED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity that has data written into it.';

comment on column SAHARA_APP.APP_GROUP_CAPACITY_.UNUSED is 
'This column is for staging data that will end up in table SAHARA.APP_GROUP_CAPACITY, column UNUSED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity still available for writing into.';

comment on column SAHARA_APP.APP_GROUP_CAPACITY_."UNKNOWN" is 
'This column is for staging data that will end up in table SAHARA.APP_GROUP_CAPACITY, column UNKNOWN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Unknown capacity for application groups is calculated as
Total capacity - used capacity - available/free capacity';

alter table SAHARA_APP.APP_GROUP_CAPACITY_
   alter ALLOCATED add constraint C_PPGRPCPCT_ALLOCATED check (ALLOCATED >= 0);

alter table SAHARA_APP.APP_GROUP_CAPACITY_
   alter USED add constraint C_PPGRPCPCT_USED check (USED >= 0);

alter table SAHARA_APP.APP_GROUP_CAPACITY_
   alter UNUSED add constraint C_PPGRPCPCT_UNUSED check (UNUSED >= 0);

alter table SAHARA_APP.APP_GROUP_CAPACITY_
   alter "UNKNOWN" add constraint C_PPGRPCPCT_UNKNOWN check ("UNKNOWN" >= 0);

alter table SAHARA_APP.APP_GROUP_CAPACITY_
   add constraint PK_APP_GROUP_CAPACITY_ primary key (APP_GROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.APP_GROUP_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: APP_GROUP_HOST                                        */
/*==============================================================*/
create table SAHARA.APP_GROUP_HOST 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   APP_GROUP_KEY        numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null
);

comment on table SAHARA.APP_GROUP_HOST is 
'A join table for forming the many-to-many relationship between application groups and hosts.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA.APP_GROUP_HOST.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.APP_GROUP_HOST.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.APP_GROUP_HOST.APP_GROUP_KEY is 
'The surrogate key of the application group which is related to a host.  This key may be used in joins to the APP_GROUP table.';

comment on column SAHARA.APP_GROUP_HOST.HOST_KEY is 
'The surrogate key of the host which is related to an application group.  This key may be used in joins to the HOST table.';

/*==============================================================*/
/* Index: I_APPGROUPHOST_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_APPGROUPHOST_ROLLUPBATCHKEY on SAHARA.APP_GROUP_HOST (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_APPGROUPHOST_DATESKEY                               */
/*==============================================================*/
create index I_APPGROUPHOST_DATESKEY on SAHARA.APP_GROUP_HOST (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_APPGROUPHOST_APPGROUPKEY                            */
/*==============================================================*/
create index I_APPGROUPHOST_APPGROUPKEY on SAHARA.APP_GROUP_HOST (
APP_GROUP_KEY ASC
);

/*==============================================================*/
/* Index: I_APPGROUPHOST_HOSTKEY                                */
/*==============================================================*/
create index I_APPGROUPHOST_HOSTKEY on SAHARA.APP_GROUP_HOST (
HOST_KEY ASC
);

grant SELECT on SAHARA.APP_GROUP_HOST to SAHARA_APP;

/*==============================================================*/
/* Table: APP_GROUP_HOST_                                       */
/*==============================================================*/
create table SAHARA_APP.APP_GROUP_HOST_ 
(
   APP_GROUP_OID        varchar(512)                   not null,
   HOST_OID             varchar(512)                   not null
);

comment on table SAHARA_APP.APP_GROUP_HOST_ is 
'This is a staging table for loading data into warehouse table SAHARA.APP_GROUP_HOST.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  A join table for forming the many-to-many relationship between application groups and hosts.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA_APP.APP_GROUP_HOST_.APP_GROUP_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.APP_GROUP_HOST_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

alter table SAHARA_APP.APP_GROUP_HOST_
   add constraint PK_APP_GROUP_HOST_ primary key (APP_GROUP_OID, HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.APP_GROUP_HOST_ to SAHARA;

/*==============================================================*/
/* Table: ARRAY                                                 */
/*==============================================================*/
create table SAHARA.ARRAY 
(
   ARRAY_KEY            numeric(10)                    not null default autoincrement,
   PARENT_ARRAY_KEY     numeric(10)                    null,
   ARRAY_OID            varchar(512)                   not null,
   PARENT_ARRAY_OID     varchar(512)                   null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   VENDOR               varchar(256)                   null,
   FAMILY               varchar(32)                    not null,
   TYPE                 varchar(32)                    not null,
   DISCOVERY_STATE      varchar(32)                    null,
   FIRMWARE_VERSION     varchar(256)                   null,
   MODEL                varchar(256)                   null,
   SERIAL_NUMBER        varchar(256)                   null,
   SNMP_IP_ADDRESS      varchar(4000)                  null,
   NIS_DOMAIN           varchar(256)                   null,
   NIS_SERVER           varchar(256)                   null,
   IS_AUTO_DISCOVERED   char(1)                        not null,
   IS_VIRTUALIZER       char(1)                        not null,
   IS_NIS_ENABLED       char(1)                        null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   VOM_DISCOVER_TYPE    VARCHAR(32)                    null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   SUB_TYPE             varchar(32)                    null,
   IP_ADDRESSES         varchar(4000)                  null,
   CLUSTER_ARRAY_KEY    numeric(10)                    null,
   CLUSTER_ARRAY_NAME   varchar(256)                   null,
   CLUSTER_ARRAY_OID    varchar(512)                   null,
   FILER_TYPE           varchar(32)                    null,
   PARENT_ARRAY_NAME    varchar(256)                   null
);

comment on table SAHARA.ARRAY is 
'This is the dimension table for arrays.  It defines the attributes of arrays that may be used to qualify array queries.';

comment on column SAHARA.ARRAY.ARRAY_KEY is 
'A unique database generated identifier for the array record used as a surrogate key.';

comment on column SAHARA.ARRAY.PARENT_ARRAY_KEY is 
'This unique identifier of the host pfiler''s array record for a NetApp vfiler.  This will be null for all other arrays and NetApp filers.';

comment on column SAHARA.ARRAY.ARRAY_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.ARRAY.PARENT_ARRAY_OID is 
'This unique identifier of the host pfiler array in CommandCentral Storage for a NetApp vfiler.  This will be null for all other arrays and NetApp filers.';

comment on column SAHARA.ARRAY.CC_STORAGE_URL is 
'The URL for the array object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.ARRAY.DISPLAY_NAME is 
'Array display name.';

comment on column SAHARA.ARRAY.ADVISORY_DISPLAY_NAME is 
'The array name discovered by CommandCentral Storage.

This may be null.';

comment on column SAHARA.ARRAY.VENDOR is 
'The array vendor name.

This may be null.';

comment on column SAHARA.ARRAY.FAMILY is 
'Indicates the Storage Array Family.  Values include:
Symmetrix (Symm and DMX),
CLARiiON
Thunder Series
Lightning Series
Enterprise Virtual Array
Enterprise Storage Server
IBM DS Array';

comment on column SAHARA.ARRAY.TYPE is 
'Array type.  This can be:
Filer
Enclosure
Unenclosed Device';

comment on column SAHARA.ARRAY.DISCOVERY_STATE is 
'The discovery state of the array:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the array was initially discovered by two or more explorers and then one of them stopped discovering the array.

Note: An array that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.ARRAY.FIRMWARE_VERSION is 
'The array firmware or microcode version.

This may be null.';

comment on column SAHARA.ARRAY.MODEL is 
'The array vendor model.

This may be null.';

comment on column SAHARA.ARRAY.SERIAL_NUMBER is 
'The array serial number assigned by the vendor.

This may be null.';

comment on column SAHARA.ARRAY.SNMP_IP_ADDRESS is 
'One or more IP addresses of the SNMP agent providing array information.  Multiple addresses are stored as a comma separated list.

This may be null.';

comment on column SAHARA.ARRAY.NIS_DOMAIN is 
'The NIS domain for a NAS device.  This will be null for non-NAS arrays.';

comment on column SAHARA.ARRAY.NIS_SERVER is 
'The NIS server for a NAS device.  This will be null for non-NAS arrays.';

comment on column SAHARA.ARRAY.IS_AUTO_DISCOVERED is 
'A boolean flag that indicates if the array has been discovered by CommandCentral Storage agents.

Possible values:
''T'' - the array has been discovered
''F'' - the array has not been discovered';

comment on column SAHARA.ARRAY.IS_VIRTUALIZER is 
'A boolean flag that indicates if the array is a virtualizer.  Virtualizer arrays present external disks as LUNs to hosts.  HDS Tagma and USP arrays are examples of virtualizers.

Possible values:
''T'' - the array is a virtualizer
''F'' - the array is not a virtualizer';

comment on column SAHARA.ARRAY.IS_NIS_ENABLED is 
'Is NIS enabled for a NAS device.  This will be null for non-NAS arrays.';

comment on column SAHARA.ARRAY.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.ARRAY.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.ARRAY.APPLICATION_BE is 
'Application business entity to which this array belongs.

This may be null.';

comment on column SAHARA.ARRAY.APPLICATION_COMPOSITE_BE is 
'Composite application business entity to which this array belongs.

This may be null.';

comment on column SAHARA.ARRAY.ORGANIZATION_BE is 
'Organization business entity to which this array belongs.

This may be null.';

comment on column SAHARA.ARRAY.ORGANIZATION_COMPOSITE_BE is 
'Composite organization business entity to which this array belongs.

This may be null.';

comment on column SAHARA.ARRAY.SUB_TYPE is 
'sub-type of array. Example: PFiler, Vfiler, PDM, VDM etc.';

comment on column SAHARA.ARRAY.CLUSTER_ARRAY_OID is 
'This unique identifier of the host pfiler array in CommandCentral Storage for a NetApp vfiler.  This will be null for all other arrays and NetApp filers.';

comment on column SAHARA.ARRAY.FILER_TYPE is 
'Type of Filer: Physical Filer, Virtual Filer, PDM, VDM';

comment on column SAHARA.ARRAY.PARENT_ARRAY_NAME is 
'Name of parent array of component';

alter table SAHARA.ARRAY
   alter IS_AUTO_DISCOVERED add constraint C_ARRAY_ISAUTODISCOVERED check (IS_AUTO_DISCOVERED in ('T','F'));

alter table SAHARA.ARRAY
   alter IS_VIRTUALIZER add constraint C_ARRAY_ISVIRTUALIZER check (IS_VIRTUALIZER in ('T','F'));

alter table SAHARA.ARRAY
   alter IS_NIS_ENABLED add constraint C_ARRAY_ISNISENABLED check (IS_NIS_ENABLED is null or (IS_NIS_ENABLED in ('T','F')));

alter table SAHARA.ARRAY
   alter T2_IS_CURRENT add constraint C_ARRAY_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.ARRAY
   alter T2_IS_DELETED add constraint C_ARRAY_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.ARRAY
   add constraint PK_ARRAY primary key (ARRAY_KEY);

/*==============================================================*/
/* Index: I_ARRAY_PARENTARRAYKEY                                */
/*==============================================================*/
create index I_ARRAY_PARENTARRAYKEY on SAHARA.ARRAY (
PARENT_ARRAY_KEY ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_PARENTARRAYKEY is 
'An index on Parent Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_ARRAYOID                                      */
/*==============================================================*/
create index I_ARRAY_ARRAYOID on SAHARA.ARRAY (
ARRAY_OID ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_ARRAYOID is 
'An index on Array Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_PARENTARRAYOID                                */
/*==============================================================*/
create index I_ARRAY_PARENTARRAYOID on SAHARA.ARRAY (
PARENT_ARRAY_OID ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_PARENTARRAYOID is 
'An index on Parent Array Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_DISPLAYNAME                                   */
/*==============================================================*/
create index I_ARRAY_DISPLAYNAME on SAHARA.ARRAY (
DISPLAY_NAME ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_ADVISORYDISPLAYNAME                           */
/*==============================================================*/
create index I_ARRAY_ADVISORYDISPLAYNAME on SAHARA.ARRAY (
ADVISORY_DISPLAY_NAME ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_ADVISORYDISPLAYNAME is 
'An index on Advisory Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_VENDOR                                        */
/*==============================================================*/
create index I_ARRAY_VENDOR on SAHARA.ARRAY (
VENDOR ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_VENDOR is 
'An index on Vendor for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_FAMILY                                        */
/*==============================================================*/
create index I_ARRAY_FAMILY on SAHARA.ARRAY (
FAMILY ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_FAMILY is 
'An index on Family for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_TYPE                                          */
/*==============================================================*/
create index I_ARRAY_TYPE on SAHARA.ARRAY (
TYPE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_DISCOVERYSTATE                                */
/*==============================================================*/
create index I_ARRAY_DISCOVERYSTATE on SAHARA.ARRAY (
DISCOVERY_STATE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_FIRMWAREVERSION                               */
/*==============================================================*/
create index I_ARRAY_FIRMWAREVERSION on SAHARA.ARRAY (
FIRMWARE_VERSION ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_FIRMWAREVERSION is 
'An index on Firmware Version for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_MODEL                                         */
/*==============================================================*/
create index I_ARRAY_MODEL on SAHARA.ARRAY (
MODEL ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_MODEL is 
'An index on Model for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_SERIALNUMBER                                  */
/*==============================================================*/
create index I_ARRAY_SERIALNUMBER on SAHARA.ARRAY (
SERIAL_NUMBER ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_SERIALNUMBER is 
'An index on Serial Number for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_SNMPIPADDRESS                                 */
/*==============================================================*/
create index I_ARRAY_SNMPIPADDRESS on SAHARA.ARRAY (
SNMP_IP_ADDRESS ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_SNMPIPADDRESS is 
'An index on SNMP IP Address(es) for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_NISDOMAIN                                     */
/*==============================================================*/
create index I_ARRAY_NISDOMAIN on SAHARA.ARRAY (
NIS_DOMAIN ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_NISDOMAIN is 
'An index on NIS Domain for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_NISSERVER                                     */
/*==============================================================*/
create index I_ARRAY_NISSERVER on SAHARA.ARRAY (
NIS_SERVER ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_NISSERVER is 
'An index on NIS Server for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_ISAUTODISCOVERED                              */
/*==============================================================*/
create index I_ARRAY_ISAUTODISCOVERED on SAHARA.ARRAY (
IS_AUTO_DISCOVERED ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_ISAUTODISCOVERED is 
'An index on Is Auto Discovered for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_ISVIRTUALIZER                                 */
/*==============================================================*/
create index I_ARRAY_ISVIRTUALIZER on SAHARA.ARRAY (
IS_VIRTUALIZER ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_ISVIRTUALIZER is 
'An index on Is Virtualizer for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_ISNISENABLED                                  */
/*==============================================================*/
create index I_ARRAY_ISNISENABLED on SAHARA.ARRAY (
IS_NIS_ENABLED ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_ISNISENABLED is 
'An index on Is NAS NIS Enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_T2ISCURRENTARRAYKEY                           */
/*==============================================================*/
create index I_ARRAY_T2ISCURRENTARRAYKEY on SAHARA.ARRAY (
T2_IS_CURRENT ASC,
ARRAY_KEY ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_T2ISCURRENTARRAYKEY is 
'An index on Type 2 - Is Current and Array Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_ARRAY_T2SCRRNTT2SDLTDD                              */
/*==============================================================*/
create index I_ARRAY_T2SCRRNTT2SDLTDD on SAHARA.ARRAY (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
ARRAY_OID ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Array Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_ARRAY_T2ISDELETED                                   */
/*==============================================================*/
create index I_ARRAY_T2ISDELETED on SAHARA.ARRAY (
T2_IS_DELETED ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_VOMDISCOVERTYPE                               */
/*==============================================================*/
create index I_ARRAY_VOMDISCOVERTYPE on SAHARA.ARRAY (
VOM_DISCOVER_TYPE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_VOMDISCOVERTYPE is 
'An index on VOM Discover type for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_APPLICATIONBE                                 */
/*==============================================================*/
create index I_ARRAY_APPLICATIONBE on SAHARA.ARRAY (
APPLICATION_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_APPLICATIONBE is 
'An index on Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_APPLICATIONCOMPOSITEBE                        */
/*==============================================================*/
create index I_ARRAY_APPLICATIONCOMPOSITEBE on SAHARA.ARRAY (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_APPLICATIONCOMPOSITEBE is 
'An index on Composite Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_ORGANIZATIONBE                                */
/*==============================================================*/
create index I_ARRAY_ORGANIZATIONBE on SAHARA.ARRAY (
ORGANIZATION_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_ORGANIZATIONBE is 
'An index on Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_RGNZTNCMPSTB                                  */
/*==============================================================*/
create index I_ARRAY_RGNZTNCMPSTB on SAHARA.ARRAY (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_RGNZTNCMPSTB is 
'An index on Composite Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_SUBTYPE                                       */
/*==============================================================*/
create index I_ARRAY_SUBTYPE on SAHARA.ARRAY (
SUB_TYPE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_SUBTYPE is 
'An index on Sub Type for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_IPADDRESSES                                   */
/*==============================================================*/
create index I_ARRAY_IPADDRESSES on SAHARA.ARRAY (
IP_ADDRESSES ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_IPADDRESSES is 
'An index on IP Addresses for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_CLUSTERARRAYKEY                               */
/*==============================================================*/
create index I_ARRAY_CLUSTERARRAYKEY on SAHARA.ARRAY (
CLUSTER_ARRAY_KEY ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_CLUSTERARRAYKEY is 
'An index on Cluster Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_CLUSTERARRAYNAME                              */
/*==============================================================*/
create index I_ARRAY_CLUSTERARRAYNAME on SAHARA.ARRAY (
CLUSTER_ARRAY_NAME ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_CLUSTERARRAYNAME is 
'An index on Cluster Array Name for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_CLUSTERARRAYOID                               */
/*==============================================================*/
create index I_ARRAY_CLUSTERARRAYOID on SAHARA.ARRAY (
CLUSTER_ARRAY_OID ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_CLUSTERARRAYOID is 
'An index on Cluster Array Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_FILERTYPE                                     */
/*==============================================================*/
create index I_ARRAY_FILERTYPE on SAHARA.ARRAY (
FILER_TYPE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_FILERTYPE is 
'An index on Filer Type for dimensional queries.';

/*==============================================================*/
/* Index: I_ARRAY_PARENTARRAYNAME                               */
/*==============================================================*/
create index I_ARRAY_PARENTARRAYNAME on SAHARA.ARRAY (
PARENT_ARRAY_NAME ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_PARENTARRAYNAME is 
'An index on Parent Array Name for dimensional queries.';

grant SELECT on SAHARA.ARRAY to SAHARA_APP;

/*==============================================================*/
/* Table: ARRAY_                                                */
/*==============================================================*/
create table SAHARA_APP.ARRAY_ 
(
   ARRAY_OID            varchar(512)                   not null,
   PARENT_ARRAY_OID     varchar(512)                   null,
   DM_ID                numeric(10)                    null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   VENDOR               varchar(256)                   null,
   FAMILY               varchar(32)                    not null,
   TYPE                 varchar(32)                    not null,
   DISCOVERY_STATE      varchar(32)                    null,
   FIRMWARE_VERSION     varchar(256)                   null,
   MODEL                varchar(256)                   null,
   SERIAL_NUMBER        varchar(256)                   null,
   SNMP_IP_ADDRESS      varchar(4000)                  null,
   NIS_DOMAIN           varchar(256)                   null,
   NIS_SERVER           varchar(256)                   null,
   IS_AUTO_DISCOVERED   char(1)                        not null,
   IS_VIRTUALIZER       char(1)                        not null,
   IS_NIS_ENABLED       char(1)                        null,
   VOM_DISCOVER_TYPE    varchar(32)                    null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   SUB_TYPE             varchar(32)                    not null default ' ',
   IP_ADDRESSES         varchar(4000)                  null,
   CLUSTER_ARRAY_NAME   varchar(256)                   null,
   CLUSTER_ARRAY_OID    varchar(512)                   null,
   FILER_TYPE           varchar(32)                    null,
   PARENT_ARRAY_NAME    varchar(256)                   null
);

comment on table SAHARA_APP.ARRAY_ is 
'This is a staging table for loading data into warehouse table SAHARA.ARRAY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for arrays.  It defines the attributes of arrays that may be used to qualify array queries.';

comment on column SAHARA_APP.ARRAY_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_.PARENT_ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Array display name.';

comment on column SAHARA_APP.ARRAY_.ADVISORY_DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column ADVISORY_DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The array name discovered by CommandCentral Storage.

This may be null.';

comment on column SAHARA_APP.ARRAY_.VENDOR is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column VENDOR.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The array vendor name.

This may be null.';

comment on column SAHARA_APP.ARRAY_.FAMILY is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column FAMILY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates the Storage Array Family.  Values include:
Symmetrix (Symm and DMX),
CLARiiON
Thunder Series
Lightning Series
Enterprise Virtual Array
Enterprise Storage Server
IBM DS Array';

comment on column SAHARA_APP.ARRAY_.TYPE is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Array type.  This can be:
NetApp
Enclosure
Unenclosed Device';

comment on column SAHARA_APP.ARRAY_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the array:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the array was initially discovered by two or more explorers and then one of them stopped discovering the array.

Note: An array that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.ARRAY_.FIRMWARE_VERSION is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column FIRMWARE_VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The array firmware or microcode version.

This may be null.';

comment on column SAHARA_APP.ARRAY_.MODEL is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column MODEL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The array vendor model.

This may be null.';

comment on column SAHARA_APP.ARRAY_.SERIAL_NUMBER is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column SERIAL_NUMBER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The array serial number assigned by the vendor.

This may be null.';

comment on column SAHARA_APP.ARRAY_.SNMP_IP_ADDRESS is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column SNMP_IP_ADDRESS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

One or more IP addresses of the SNMP agent providing array information.  Multiple addresses are stored as a comma separated list.

This may be null.';

comment on column SAHARA_APP.ARRAY_.NIS_DOMAIN is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column NIS_DOMAIN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The NIS domain for a NAS device.  This will be null for non-NAS arrays.';

comment on column SAHARA_APP.ARRAY_.NIS_SERVER is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column NIS_SERVER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The NIS server for a NAS device.  This will be null for non-NAS arrays.';

comment on column SAHARA_APP.ARRAY_.IS_AUTO_DISCOVERED is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column IS_AUTO_DISCOVERED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the array has been discovered by CommandCentral Storage agents.

Possible values:
''T'' - the array has been discovered
''F'' - the array has not been discovered';

comment on column SAHARA_APP.ARRAY_.IS_VIRTUALIZER is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column IS_VIRTUALIZER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the array is a virtualizer.  Virtualizer arrays present external disks as LUNs to hosts.  HDS Tagma and USP arrays are examples of virtualizers.

Possible values:
''T'' - the array is a virtualizer
''F'' - the array is not a virtualizer';

comment on column SAHARA_APP.ARRAY_.IS_NIS_ENABLED is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column IS_NIS_ENABLED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Is NIS enabled for a NAS device.  This will be null for non-NAS arrays.';

comment on column SAHARA_APP.ARRAY_.APPLICATION_BE is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column APPLICATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application business entity to which this array belongs.

This may be null.';

comment on column SAHARA_APP.ARRAY_.APPLICATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column APPLICATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite application business entity to which this array belongs.

This may be null.';

comment on column SAHARA_APP.ARRAY_.ORGANIZATION_BE is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column ORGANIZATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Organization business entity to which this array belongs.

This may be null.';

comment on column SAHARA_APP.ARRAY_.ORGANIZATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.ARRAY, column ORGANIZATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite organization business entity to which this array belongs.

This may be null.';

comment on column SAHARA_APP.ARRAY_.SUB_TYPE is 
'sub-type of array. Example: PFiler, Vfiler, PDM, VDM etc.';

comment on column SAHARA_APP.ARRAY_.CLUSTER_ARRAY_OID is 
'This unique identifier of the host pfiler array in CommandCentral Storage for a NetApp vfiler.  This will be null for all other arrays and NetApp filers.';

comment on column SAHARA_APP.ARRAY_.FILER_TYPE is 
'Type of filer';

comment on column SAHARA_APP.ARRAY_.PARENT_ARRAY_NAME is 
'Name of parent array of component';

alter table SAHARA_APP.ARRAY_
   alter DM_ID add constraint C_ARRAY_DMID check (DM_ID is null or (DM_ID >= 0));

alter table SAHARA_APP.ARRAY_
   alter IS_AUTO_DISCOVERED add constraint C_ARRAY_ISAUTODISCOVERED check (IS_AUTO_DISCOVERED in ('T','F'));

alter table SAHARA_APP.ARRAY_
   alter IS_VIRTUALIZER add constraint C_ARRAY_ISVIRTUALIZER check (IS_VIRTUALIZER in ('T','F'));

alter table SAHARA_APP.ARRAY_
   alter IS_NIS_ENABLED add constraint C_ARRAY_ISNISENABLED check (IS_NIS_ENABLED is null or (IS_NIS_ENABLED in ('T','F')));

alter table SAHARA_APP.ARRAY_
   add constraint PK_ARRAY_ primary key (ARRAY_OID);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.ARRAY_ to SAHARA;

/*==============================================================*/
/* Table: ARRAY_BVIEW_NODE                                      */
/*==============================================================*/
create table SAHARA.ARRAY_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.ARRAY_BVIEW_NODE is 
'This is a join table between array objects and business view nodes.';

comment on column SAHARA.ARRAY_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.ARRAY_BVIEW_NODE.ARRAY_KEY is 
'The unique identifier of the array object with which the business view node is associated.';

comment on column SAHARA.ARRAY_BVIEW_NODE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the join table record.';

alter table SAHARA.ARRAY_BVIEW_NODE
   add constraint PK_ARRAY_BVIEW_NODE primary key (BVIEW_NODE_KEY, ARRAY_KEY);

/*==============================================================*/
/* Index: I_ARRAYBVIEWNODE_BVIEWARRAY                           */
/*==============================================================*/
create unique index I_ARRAYBVIEWNODE_BVIEWARRAY on SAHARA.ARRAY_BVIEW_NODE (
BVIEW_KEY ASC,
ARRAY_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.ARRAY_BVIEW_NODE.I_ARRAYBVIEWNODE_BVIEWARRAY is 
'This index exists to accelerate queries for which arrays are associated with a particular business view.';

grant SELECT on SAHARA.ARRAY_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: ARRAY_CLAIMED_CAPACITY                                */
/*==============================================================*/
create table SAHARA.ARRAY_CLAIMED_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null,
   ROLLUP_SOURCE_KEY    numeric(10)                    not null,
   CLAIMED_BY_HOSTS     numeric(22,9)                  null,
   PERCENT_ALLOCATED_HOST_CLAIMED numeric(7,4)                   null
);

comment on table SAHARA.ARRAY_CLAIMED_CAPACITY is 
'A join table for forming the many-to-many relationships between duplicate Arrays being rolled up in multi-site rollup and its claimed  level capacities.';

comment on column SAHARA.ARRAY_CLAIMED_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.ARRAY_CLAIMED_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.ARRAY_CLAIMED_CAPACITY.ARRAY_KEY is 
'The surrogate key of the Array for which claimed facts is being stored.  This key may be used in joins to the Array table.';

comment on column SAHARA.ARRAY_CLAIMED_CAPACITY.ROLLUP_SOURCE_KEY is 
'The Claimed Status of the LUN.

Possible values:
- 0
- 1
- 2';

comment on column SAHARA.ARRAY_CLAIMED_CAPACITY.CLAIMED_BY_HOSTS is 
'The total capacity, in gigabytes, of LUNs in arrays and unified storage devices that have been claimed by at least one host.

A LUN is considered claimed when a host''s operating system has written a device handle for it.';

comment on column SAHARA.ARRAY_CLAIMED_CAPACITY.PERCENT_ALLOCATED_HOST_CLAIMED is 
'The percentage of storage allocated to host in which a host''s O/S has written a device handle to the LUN.  This value is calculated by dividing "Claimed Capacity (GB)" by the sum of "Capacity Allocated to Hosts Directly (GB)" and "Capacity Allocated to Hosts Through Virtualizer (GB)".';

/*==============================================================*/
/* Index: I_ACC_ROLLUPBATCHKEY                                  */
/*==============================================================*/
create index I_ACC_ROLLUPBATCHKEY on SAHARA.ARRAY_CLAIMED_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_ACC_DATESKEY                                        */
/*==============================================================*/
create index I_ACC_DATESKEY on SAHARA.ARRAY_CLAIMED_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_ACC_ARRAYKEY                                        */
/*==============================================================*/
create index I_ACC_ARRAYKEY on SAHARA.ARRAY_CLAIMED_CAPACITY (
ARRAY_KEY ASC
);

/*==============================================================*/
/* Index: I_ACC_ROLLUPSOURCEKEY                                 */
/*==============================================================*/
create index I_ACC_ROLLUPSOURCEKEY on SAHARA.ARRAY_CLAIMED_CAPACITY (
ROLLUP_SOURCE_KEY ASC
);

grant SELECT on SAHARA.ARRAY_CLAIMED_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: ARRAY_STORAGE                                         */
/*==============================================================*/
create table SAHARA.ARRAY_STORAGE 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null,
   TOTAL_PHYSICAL       numeric(22,9)                  not null default 0,
   V_PHYSICAL_EXTERNAL  numeric(22,9)                  not null default 0,
   LOCAL_LOGICAL        numeric(22,9)                  not null default 0,
   V_VIRTUAL_LOGICAL    numeric(22,9)                  not null default 0,
   LOGICAL_AVAILABLE    numeric(22,9)                  not null default 0,
   ALLOCATED            numeric(22,9)                  not null default 0,
   A_ALLOCATED_TO_HOSTS_DIRECTLY numeric(22,9)                  not null default 0,
   A_ALLOCATED_TO_VIRTUALIZER numeric(22,9)                  not null default 0,
   A_ALLOC_HOST_VIA_VIRTUALIZER numeric(22,9)                  not null default 0,
   V_LOCAL_LOGICAL_ALLOCATED numeric(22,9)                  not null default 0,
   V_VIRTUAL_LOGICAL_ALLOCATED numeric(22,9)                  not null default 0,
   UNALLOCATED          numeric(22,9)                  not null default 0,
   A_UNALLOCATED_VIA_VIRUALIZER numeric(22,9)                  not null default 0,
   V_LOCAL_LOGICAL_UNALLOCATED numeric(22,9)                  not null default 0,
   V_VIRTUAL_LOGICAL_UNALLOCATED numeric(22,9)                  not null default 0,
   CLAIMED_BY_HOSTS     numeric(22,9)                  not null default 0,
   REPLICATION          numeric(22,9)                  not null default 0,
   RAID_OVERHEAD        numeric(22,9)                  not null default 0,
   ADMINISTRATIVE       numeric(22,9)                  not null default 0,
   SNAPSHOT_RESERVE     numeric(22,9)                  not null default 0,
   WAFL_SPARE           numeric(22,9)                  not null default 0,
   DAS_CAPACITY         numeric(22,9)                  not null default 0,
   NAS_USED_CAPACITY    numeric(22,9)                  not null default 0,
   UNCONFIGURED_CAPACITY numeric(22,9)                  not null default 0,
   INTERNAL_PHYSICAL    numeric(22,9)                  not null default 0,
   LOGICAL_UNKNOWN      numeric(22,9)                  not null default 0,
   TOTAL_UNCLAIMED      numeric(22,9)                  not null default 0,
   UNDISCOVERED_ALLOCATED numeric(22,9)                  not null default 0,
   PERCENT_ALLOCATED_HOST_CLAIMED numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED   numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_ALLOCATED numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_AVAILABLE numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_OVERHEAD numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_UNDISCOVERD numeric(7,4)                   not null default 0,
   CONSUMPTION_ASSIGNED numeric(22,9)                  not null default 0,
   CONSUMPTION_UNKNOWN  numeric(22,9)                  not null default 0,
   HOST_USAGE_APPS_ON_RAW numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_UN_USED numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_USED   numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_AUSED_APPS numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_NAUSED_NON_APPS numeric(22,9)                  not null default 0,
   HOST_USAGE_OTHER     numeric(22,9)                  not null default 0,
   APPL_USED_CAPACITY   numeric(22,9)                  not null default 0,
   APPL_UN_USED_CAPACITY numeric(22,9)                  not null default 0,
   PERCENT_UNCONFIGURED numeric(7,4)                   not null default 0,
   THINAWARE_CLAIMED_BY_HOST numeric(22,9)                  not null default 0,
   THINAWARE_TOTAL_UNCLAIMED numeric(22,9)                  not null default 0,
   THINAWARE_ALLOCATED_TOHOST numeric(22,9)                  not null default 0,
   THINAWARE_ALC_THGVIRTUALIZER numeric(22,9)                  not null default 0,
   THINAWARE_UNDISC_ALLOCATED numeric(22,9)                  not null default 0,
   NA_VOLUME_CAPACITY   numeric(22,9)                  null default 0,
   NA_VOLUME_USED_CAPACITY numeric(22,9)                  null default 0,
   NA_VOLUME_AVAILABLE_CAPACITY numeric(22,9)                  null default 0,
   NA_QTREES            numeric(10)                    null default 0,
   NA_SHARES            numeric(10)                    null default 0,
   NA_VOLUMES           numeric(10)                    null default 0,
   NA_FILESYSTEMS       numeric(10)                    null default 0,
   NA_DM_BATCH_KEY      numeric(10)                    null,
   NA_DM_DATA_AGE       integer                        null,
   NA_CONSUMPTION       numeric(22,9)                  null,
   NA_FILES             numeric(10)                    null default 0,
   NA_FOLDERS           numeric(10)                    null default 0,
   NA_USERS             numeric(10)                    null default 0,
   SNAPSHOT_USED        numeric(22,9)                  not null default 0,
   OVERWRITE_RESERVE    numeric(22,9)                  not null default 0,
   NETAPP_REDUNDANT_CAPACITY numeric(22,9)                  not null default 0,
   NUMBER_OF_STORAGE_POOLS numeric(10)                    not null default 0,
   WAFL_SPARE_NEW       numeric(22,9)                  not null default 0
);

comment on table SAHARA.ARRAY_STORAGE is 
'This is the fact table for array storage measures.  Each rows represents one sampling of values for an array.';

comment on column SAHARA.ARRAY_STORAGE.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.ARRAY_STORAGE.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.ARRAY_STORAGE.ARRAY_KEY is 
'The surrogate key from the array dimension that identifies the array for which the array storage data was collected.  This key may be used in joins to the ARRAY table.';

comment on column SAHARA.ARRAY_STORAGE.TOTAL_PHYSICAL is 
'The total installed storage capacity of the array in gigabytes.  This is the sum of capacity from all installed physical disks on the array.';

comment on column SAHARA.ARRAY_STORAGE.V_PHYSICAL_EXTERNAL is 
'For virtualizer only.  This is the physical external capacity in gigabytes available to a virtualizer from backend arrays.';

comment on column SAHARA.ARRAY_STORAGE.LOCAL_LOGICAL is 
'The amount of physical storage, in gigabytes, that has been formatted and is apportioned into LUNs, or is ready to be apportioned into LUNs.';

comment on column SAHARA.ARRAY_STORAGE.V_VIRTUAL_LOGICAL is 
'For virtualizer only.  The amount of virtual storage in gigabytes that has been formatted and is apportioned into LUNs, or is ready to be apportioned into LUNs.';

comment on column SAHARA.ARRAY_STORAGE.LOGICAL_AVAILABLE is 
'The amount of configured storage that has not yet been apportioned into addressable units (LUNs).
This storage is typically part of raw storage volumes such as RAID groups.
Available storage reflects configured storage in arrays and unified storage devices only.
It does not include storage in JBODs, in SCSI disks, or in IDE disks.';

comment on column SAHARA.ARRAY_STORAGE.ALLOCATED is 
'The amount of storage, in gigabytes, that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs or SCSI disks) claimed by hosts.  "Claimed" means that the operating system has written a device handle for the LUN.';

comment on column SAHARA.ARRAY_STORAGE.A_ALLOCATED_TO_HOSTS_DIRECTLY is 
'For real array only.  Here LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.

In gigabytes.

TODO:  This needs a better description.';

comment on column SAHARA.ARRAY_STORAGE.A_ALLOCATED_TO_VIRTUALIZER is 
'For real arrays only.  LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.

In gigabytes.

TODO:  This needs a better description.';

comment on column SAHARA.ARRAY_STORAGE.A_ALLOC_HOST_VIA_VIRTUALIZER is 
'For real arrays only.  LUN capacity in gigabytes allocated to hosts through virtualizers, such as TagmaStore.';

comment on column SAHARA.ARRAY_STORAGE.V_LOCAL_LOGICAL_ALLOCATED is 
'For virtualizer only.  LUN capacity in gigabytes that is local to virtualizer arrays, such as TagmaStore, allocated to hosts.  These LUNs have been created from the array''s local physical disks, as opposed to virtual disks.';

comment on column SAHARA.ARRAY_STORAGE.V_VIRTUAL_LOGICAL_ALLOCATED is 
'For virtualizer only.  Virtual LUN capacity in gigabytes allocated to hosts from virtualizers, like TagmaStore.';

comment on column SAHARA.ARRAY_STORAGE.UNALLOCATED is 
'The amount of storage, in gigabytes, for arrays and unified storage devices that is apportioned into LUNs but not designated for use by specific hosts.  These are LUNs that have not been masked to a host.';

comment on column SAHARA.ARRAY_STORAGE.A_UNALLOCATED_VIA_VIRUALIZER is 
'For real arrays only.  LUN capacity in gigabytes allocated to virtualizers, such as TagmaStore, but NOT allocated to hosts from virtualizers.';

comment on column SAHARA.ARRAY_STORAGE.V_LOCAL_LOGICAL_UNALLOCATED is 
'For virtualizer only.  LUN capacity in gigabytes that is local to virtualizer arrays, such as TagmaStore, and is not allocated to hosts.';

comment on column SAHARA.ARRAY_STORAGE.V_VIRTUAL_LOGICAL_UNALLOCATED is 
'For virtualizer only.  Virtual LUN capacity in gigabytes unallocated to hosts from virtualizers, like TagmaStore.';

comment on column SAHARA.ARRAY_STORAGE.CLAIMED_BY_HOSTS is 
'The total capacity, in gigabytes, of LUNs in arrays and unified storage devices that have been claimed by at least one host.

A LUN is considered claimed when a host''s operating system has written a device handle for it.';

comment on column SAHARA.ARRAY_STORAGE.REPLICATION is 
'LUNs that are designated as target LUNs for snapshot or replication techniques.
An example is a business continuance volume (BCV) on a Symmetrix or DMX array, which contains a copy of a
standard device (STD).';

comment on column SAHARA.ARRAY_STORAGE.RAID_OVERHEAD is 
'The amount of configured storage, in gigabytes, that is reserved for overhead -- most commonly for RAID techniques such as mirroring and striping.

Depending on the array type, this value can also include additional storage such as disk-format capacity and internal configuration capacity for the array that is not discovered by CommandCentral Storage.';

comment on column SAHARA.ARRAY_STORAGE.ADMINISTRATIVE is 
'The total capacity, in gigabytes, of LUNs that are designated for administrative uses, not including RAID overhead.

In snapshot or replication scenarios, this value includes target LUNs for storing copied or replicated data.  Examples include:

- For EMC CLARiiON arrays: PSM, snapshot cache, and MirrorView
- For EMC Symmetrix and DMX arrays: BCV, VCMDB, Symmetrix file system device, and save device
- For Hitachi arrays managed by HiCommand: OnDemandDevice and CommandDevice';

comment on column SAHARA.ARRAY_STORAGE.SNAPSHOT_RESERVE is 
'For unified storage devices, the total amount of storage, in gigabytes, that is reserved for snapshots in the file system.';

comment on column SAHARA.ARRAY_STORAGE.WAFL_SPARE is 
'For unified storage devices, the total amount of storage, in gigabytes, that is devoted to the file system.

This value is calculated as follows: Physical capacity minus the sum of logical capacity, snapshot reserve, and RAID overhead.';

comment on column SAHARA.ARRAY_STORAGE.DAS_CAPACITY is 
'Direct attached storage capacity.  This capacity applies to JBODs.';

comment on column SAHARA.ARRAY_STORAGE.NAS_USED_CAPACITY is 
'Capacity used from NAS.';

comment on column SAHARA.ARRAY_STORAGE.UNCONFIGURED_CAPACITY is 
'The amount of physical storage that has not been formatted.  It includes:

- Space on array disks that has not been converted into raw storage volumes and has not been designated for hot spares
- Disks in NetApp filers on which no file systems have been created and that have not been designated for hot spares

Unconfigured storage does not include storage in SCSI disks, nor does it include storage in JBODs.';

comment on column SAHARA.ARRAY_STORAGE.INTERNAL_PHYSICAL is 
'The total amount of storage physically located on the array.';

comment on column SAHARA.ARRAY_STORAGE.LOGICAL_UNKNOWN is 
'Configured array storage that is not discovered by CommandCentral Storage.  CommandCentral Storage calls this amount "Logical: Unknown".';

comment on column SAHARA.ARRAY_STORAGE.TOTAL_UNCLAIMED is 
'The amount of array storage allocated to hosts that in which the host''s O/S has not written a device handle to the LUN.  CommandCentral Storage calls this amount "Claimed Status: Unclaimed".';

comment on column SAHARA.ARRAY_STORAGE.UNDISCOVERED_ALLOCATED is 
'The amount of array storage allocated to a host that is currently not being discovered by CommandCentral Storage.  CommandCentral Storage calls this amount "Claimed Status: Unknown".';

comment on column SAHARA.ARRAY_STORAGE.PERCENT_ALLOCATED_HOST_CLAIMED is 
'The percentage of storage allocated to host in which a host''s O/S has written a device handle to the LUN.  This value is calculated by dividing "Claimed Capacity (GB)" by the sum of "Capacity Allocated to Hosts Directly (GB)" and "Capacity Allocated to Hosts Through Virtualizer (GB)".';

comment on column SAHARA.ARRAY_STORAGE.PERCENT_CONFIGURED is 
'The percentage of an array''s total storage that has been formatted and usable.  This value is calculated by dividing Configured Capacity (GB) by Total Array Capacity (GB).';

comment on column SAHARA.ARRAY_STORAGE.PERCENT_CONFIGURED_ALLOCATED is 
'The percentage of configured array storage that is allocated to either hosts or virtualizers.  This value is calculated by dividing Total Allocated Capacity (GB) by Configured Capacity (GB).';

comment on column SAHARA.ARRAY_STORAGE.PERCENT_CONFIGURED_AVAILABLE is 
'The percentage of configured array storage that is currently not yet in use but is available for allocation.  This value is calculated by dividing Capacity Available for Allocation (GB) by Configured Capacity (GB).';

comment on column SAHARA.ARRAY_STORAGE.PERCENT_CONFIGURED_OVERHEAD is 
'The percentage of configured array storage that is reserved for RAID, replication and other administrative uses.  This value is calculated by dividing Capacity Reserved for Overhead (GB) by Configured Capacity (GB).';

comment on column SAHARA.ARRAY_STORAGE.PERCENT_CONFIGURED_UNDISCOVERD is 
'The percentage of configured array storage that is not known because CommandCentral is not able to discover it.  This value is calculated by dividing Undiscovered Configured Capacity (GB) by Configured Capacity (GB)';

comment on column SAHARA.ARRAY_STORAGE.PERCENT_UNCONFIGURED is 
'The percentage of an array''s total storage that has not been formatted yet.  This value is calculated by dividing Unconfigured Capacity (GB) by Total Array Capacity (GB).';

comment on column SAHARA.ARRAY_STORAGE.THINAWARE_CLAIMED_BY_HOST is 
'The total capacity, in gigabytes, of LUNs in arrays and unified storage devices that have been claimed by at least one host

A LUN is considered claimed when a host''s operating system has written a device handle for it.  This capacity will 
include consumed size  capacity from the  thin aware luns.';

comment on column SAHARA.ARRAY_STORAGE.THINAWARE_TOTAL_UNCLAIMED is 
'The amount of array storage allocated to hosts that in which the host''s O/S has not written a device handle to the LUN.  CommandCentral Storage calls this amount "Claimed Status: Unclaimed". This capacity will 
include consumed size  capacity from the   thin aware luns.';

comment on column SAHARA.ARRAY_STORAGE.THINAWARE_ALLOCATED_TOHOST is 
'For real array only .This capacity will  include consumed size  capacity from the   aware luns allocated to host  Here LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.

In gigabytes.


';

comment on column SAHARA.ARRAY_STORAGE.THINAWARE_ALC_THGVIRTUALIZER is 
'For real arrays only.  LUN capacity in gigabytes allocated to hosts through virtualizers, such as TagmaStore. This capacity will include consumed size  capacity from the  thinaware  the thin aware luns.';

comment on column SAHARA.ARRAY_STORAGE.THINAWARE_UNDISC_ALLOCATED is 
'The amount of array storage allocated to a host that is currently not being discovered by VOM.  VOM calls this amount "Claimed Status: Unknown".This capacity will  include consumed size  capacity from the  thinaware  the thin aware luns.';

comment on column SAHARA.ARRAY_STORAGE.NA_VOLUME_CAPACITY is 
'The aggregate capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_VOLUME_USED_CAPACITY is 
'The aggregate used capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_VOLUME_AVAILABLE_CAPACITY is 
'The aggregate free local capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_QTREES is 
'The number of qtrees for the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_SHARES is 
'The number of shares on the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_VOLUMES is 
'The number of volumes on the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_FILESYSTEMS is 
'The number of filesystems on the device.

This is value is only available for Celerra devices and will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_DM_DATA_AGE is 
'The age of the DM data, in seconds, relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_CONSUMPTION is 
'The aggregate of the size of all of the files on the device, in gigabytes.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_FILES is 
'The number of files on the device.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_FOLDERS is 
'The number of directories on the device.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.NA_USERS is 
'The number of users that have files on the device.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.ARRAY_STORAGE.WAFL_SPARE_NEW is 
'For unified storage devices, the total amount of storage, in gigabytes, that is devoted to the file system.

This value is calculated as follows: Physical capacity minus the sum of logical capacity, snapshot reserve, and RAID overhead.';

alter table SAHARA.ARRAY_STORAGE
   alter TOTAL_PHYSICAL add constraint C_ARRAYSTORAGE_TOTALPHYSICAL check (TOTAL_PHYSICAL >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter V_PHYSICAL_EXTERNAL add constraint C_ARRAYSTORAGE_VPHSCLXTRNL check (V_PHYSICAL_EXTERNAL >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter LOCAL_LOGICAL add constraint C_ARRAYSTORAGE_LOCALLOGICAL check (LOCAL_LOGICAL >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter V_VIRTUAL_LOGICAL add constraint C_ARRAYSTORAGE_VVIRTUALLOGICAL check (V_VIRTUAL_LOGICAL >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter LOGICAL_AVAILABLE add constraint C_ARRAYSTORAGE_LGCLVLBL check (LOGICAL_AVAILABLE >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter ALLOCATED add constraint C_ARRAYSTORAGE_ALLOCATED check (ALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter A_ALLOCATED_TO_HOSTS_DIRECTLY add constraint C_ARRAYSTORAGE_LLCTDTHSTSDRCTL check (A_ALLOCATED_TO_HOSTS_DIRECTLY >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter A_ALLOCATED_TO_VIRTUALIZER add constraint C_ARRAYSTORAGE_LLCTDTVRTLZR check (A_ALLOCATED_TO_VIRTUALIZER >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter A_ALLOC_HOST_VIA_VIRTUALIZER add constraint C_ARRAYSTORAGE_LLCHSTVVRTLZR check (A_ALLOC_HOST_VIA_VIRTUALIZER >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter V_LOCAL_LOGICAL_ALLOCATED add constraint C_ARRAYSTORAGE_VLCLLGCLLLCTD check (V_LOCAL_LOGICAL_ALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter V_VIRTUAL_LOGICAL_ALLOCATED add constraint C_ARRAYSTORAGE_VVRTLLGCLLLCTD check (V_VIRTUAL_LOGICAL_ALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter UNALLOCATED add constraint C_ARRAYSTORAGE_UNALLOCATED check (UNALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter A_UNALLOCATED_VIA_VIRUALIZER add constraint C_ARRAYSTORAGE_NLLCTDVVRLZR check (A_UNALLOCATED_VIA_VIRUALIZER >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter V_LOCAL_LOGICAL_UNALLOCATED add constraint C_ARRAYSTORAGE_VLCLLGCLNLLCTD check (V_LOCAL_LOGICAL_UNALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter V_VIRTUAL_LOGICAL_UNALLOCATED add constraint C_ARRAYSTORAGE_VVRTLLGCLNLLCTD check (V_VIRTUAL_LOGICAL_UNALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter CLAIMED_BY_HOSTS add constraint C_ARRAYSTORAGE_CLAIMEDBYHOSTS check (CLAIMED_BY_HOSTS >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter REPLICATION add constraint C_ARRAYSTORAGE_REPLICATION check (REPLICATION >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter RAID_OVERHEAD add constraint C_ARRAYSTORAGE_RAIDOVERHEAD check (RAID_OVERHEAD >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter ADMINISTRATIVE add constraint C_ARRAYSTORAGE_ADMINISTRATIVE check (ADMINISTRATIVE >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter SNAPSHOT_RESERVE add constraint C_ARRAYSTORAGE_SNAPSHOTRESERVE check (SNAPSHOT_RESERVE >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter WAFL_SPARE add constraint C_ARRAYSTORAGE_WAFLSPARE check (WAFL_SPARE >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter DAS_CAPACITY add constraint C_ARRAYSTORAGE_DASCAPACITY check (DAS_CAPACITY >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter NAS_USED_CAPACITY add constraint C_ARRAYSTORAGE_NASUSEDCAPACITY check (NAS_USED_CAPACITY >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter UNCONFIGURED_CAPACITY add constraint C_ARRAYSTORAGE_NCNFGRDCPCT check (UNCONFIGURED_CAPACITY >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter INTERNAL_PHYSICAL add constraint C_ARRAYSTORAGE_NTRNLPHSCL check (INTERNAL_PHYSICAL >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter LOGICAL_UNKNOWN add constraint C_ARRAYSTORAGE_LOGICALUNKNOWN check (LOGICAL_UNKNOWN >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter TOTAL_UNCLAIMED add constraint C_ARRAYSTORAGE_TOTALUNCLAIMED check (TOTAL_UNCLAIMED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter UNDISCOVERED_ALLOCATED add constraint C_ARRAYSTORAGE_NDSCVRDLLCTD check (UNDISCOVERED_ALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter CONSUMPTION_ASSIGNED add constraint CKC_CONSUMPTION_ASSIG_ARRAY_ST check (CONSUMPTION_ASSIGNED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter CONSUMPTION_UNKNOWN add constraint CKC_CONSUMPTION_UNKNO_ARRAY_ST check (CONSUMPTION_UNKNOWN >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter HOST_USAGE_APPS_ON_RAW add constraint CKC_HOST_USAGE_APPS_O_ARRAY_ST check (HOST_USAGE_APPS_ON_RAW >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter HOST_USAGE_FS_UN_USED add constraint CKC_HOST_USAGE_FS_UN__ARRAY_ST check (HOST_USAGE_FS_UN_USED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter HOST_USAGE_FS_USED add constraint CKC_HOST_USAGE_FS_USE_ARRAY_ST check (HOST_USAGE_FS_USED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter HOST_USAGE_FS_AUSED_APPS add constraint CKC_HOST_USAGE_FS_AUS_ARRAY_ST check (HOST_USAGE_FS_AUSED_APPS >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter HOST_USAGE_FS_NAUSED_NON_APPS add constraint CKC_HOST_USAGE_FS_NAU_ARRAY_ST check (HOST_USAGE_FS_NAUSED_NON_APPS >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter HOST_USAGE_OTHER add constraint CKC_HOST_USAGE_OTHER_ARRAY_ST check (HOST_USAGE_OTHER >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter APPL_USED_CAPACITY add constraint CKC_APPL_USED_CAPACIT_ARRAY_ST check (APPL_USED_CAPACITY >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter APPL_UN_USED_CAPACITY add constraint CKC_APPL_UN_USED_CAPA_ARRAY_ST check (APPL_UN_USED_CAPACITY >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter THINAWARE_CLAIMED_BY_HOST add constraint CKC_THINAWARE_CLAIMED_ARRAY_ST check (THINAWARE_CLAIMED_BY_HOST >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter THINAWARE_TOTAL_UNCLAIMED add constraint CKC_THINAWARE_TOTAL_U_ARRAY_ST check (THINAWARE_TOTAL_UNCLAIMED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter THINAWARE_ALLOCATED_TOHOST add constraint CKC_THINAWARE_ALLOCAT_ARRAY_ST check (THINAWARE_ALLOCATED_TOHOST >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter THINAWARE_ALC_THGVIRTUALIZER add constraint CKC_THINAWARE_ALC_THG_ARRAY_ST check (THINAWARE_ALC_THGVIRTUALIZER >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter THINAWARE_UNDISC_ALLOCATED add constraint CKC_THINAWARE_UNDISC__ARRAY_ST check (THINAWARE_UNDISC_ALLOCATED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter NA_VOLUME_CAPACITY add constraint C_ARRAYSTORAGE_NVLMCPCT check (NA_VOLUME_CAPACITY is null or (NA_VOLUME_CAPACITY >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_VOLUME_USED_CAPACITY add constraint C_ARRAYSTORAGE_NVLMSDCPCT check (NA_VOLUME_USED_CAPACITY is null or (NA_VOLUME_USED_CAPACITY >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_VOLUME_AVAILABLE_CAPACITY add constraint C_ARRAYSTORAGE_NVLMVLBLCPCT check (NA_VOLUME_AVAILABLE_CAPACITY is null or (NA_VOLUME_AVAILABLE_CAPACITY >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_QTREES add constraint C_ARRAYSTORAGE_NAQTREES check (NA_QTREES is null or (NA_QTREES >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_SHARES add constraint C_ARRAYSTORAGE_NASHARES check (NA_SHARES is null or (NA_SHARES >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_VOLUMES add constraint C_ARRAYSTORAGE_NAVOLUMES check (NA_VOLUMES is null or (NA_VOLUMES >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_FILESYSTEMS add constraint C_ARRAYSTORAGE_NAFILESYSTEMS check (NA_FILESYSTEMS is null or (NA_FILESYSTEMS >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_CONSUMPTION add constraint C_ARRAYSTORAGE_NACONSUMPTION check (NA_CONSUMPTION is null or (NA_CONSUMPTION >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_FILES add constraint C_ARRAYSTORAGE_NAFILES check (NA_FILES is null or (NA_FILES >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_FOLDERS add constraint C_ARRAYSTORAGE_NAFOLDERS check (NA_FOLDERS is null or (NA_FOLDERS >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter NA_USERS add constraint C_ARRAYSTORAGE_NAUSERS check (NA_USERS is null or (NA_USERS >= 0));

alter table SAHARA.ARRAY_STORAGE
   alter SNAPSHOT_USED add constraint CKC_SNAPSHOT_USED_ARRAY_ST check (SNAPSHOT_USED >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter OVERWRITE_RESERVE add constraint CKC_OVERWRITE_RESERVE_ARRAY_ST check (OVERWRITE_RESERVE >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter NETAPP_REDUNDANT_CAPACITY add constraint CKC_NETAPP_REDUNDANT__ARRAY_ST check (NETAPP_REDUNDANT_CAPACITY >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter NUMBER_OF_STORAGE_POOLS add constraint CKC_NUMBER_OF_STORAGE_ARRAY_ST check (NUMBER_OF_STORAGE_POOLS >= 0);

alter table SAHARA.ARRAY_STORAGE
   alter WAFL_SPARE_NEW add constraint C_ARRAYSTORAGE_WAFLSPARE_NEW check (WAFL_SPARE_NEW >= 0);

/*==============================================================*/
/* Index: I_ARRAYSTORAGE_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_ARRAYSTORAGE_ROLLUPBATCHKEY on SAHARA.ARRAY_STORAGE (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_ARRAYSTORAGE_DATESKEY                               */
/*==============================================================*/
create index I_ARRAYSTORAGE_DATESKEY on SAHARA.ARRAY_STORAGE (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_ARRAYSTORAGE_ARRAYKEY                               */
/*==============================================================*/
create index I_ARRAYSTORAGE_ARRAYKEY on SAHARA.ARRAY_STORAGE (
ARRAY_KEY ASC
);

/*==============================================================*/
/* Index: I_ARRAYSTORAGE_DMBATCHKEY                             */
/*==============================================================*/
create index I_ARRAYSTORAGE_DMBATCHKEY on SAHARA.ARRAY_STORAGE (
NA_DM_BATCH_KEY ASC
);

grant SELECT on SAHARA.ARRAY_STORAGE to SAHARA_APP;

/*==============================================================*/
/* Table: ARRAY_STORAGE_ARRAY_                                  */
/*==============================================================*/
create table SAHARA_APP.ARRAY_STORAGE_ARRAY_ 
(
   ARRAY_OID            varchar(512)                   not null,
   TOTAL_PHYSICAL       numeric(22,9)                  not null default 0,
   V_PHYSICAL_EXTERNAL  numeric(22,9)                  not null default 0,
   LOCAL_LOGICAL        numeric(22,9)                  not null default 0,
   V_VIRTUAL_LOGICAL    numeric(22,9)                  not null default 0,
   LOGICAL_AVAILABLE    numeric(22,9)                  not null default 0,
   ALLOCATED            numeric(22,9)                  not null default 0,
   A_ALLOCATED_TO_HOSTS_DIRECTLY numeric(22,9)                  not null default 0,
   A_ALLOCATED_TO_VIRTUALIZER numeric(22,9)                  not null default 0,
   A_ALLOC_HOST_VIA_VIRTUALIZER numeric(22,9)                  not null default 0,
   V_LOCAL_LOGICAL_ALLOCATED numeric(22,9)                  not null default 0,
   V_VIRTUAL_LOGICAL_ALLOCATED numeric(22,9)                  not null default 0,
   UNALLOCATED          numeric(22,9)                  not null default 0,
   A_UNALLOCATED_VIA_VIRUALIZER numeric(22,9)                  not null default 0,
   V_LOCAL_LOGICAL_UNALLOCATED numeric(22,9)                  not null default 0,
   V_VIRTUAL_LOGICAL_UNALLOCATED numeric(22,9)                  not null default 0,
   CLAIMED_BY_HOSTS     numeric(22,9)                  not null default 0,
   REPLICATION          numeric(22,9)                  not null default 0,
   RAID_OVERHEAD        numeric(22,9)                  not null default 0,
   ADMINISTRATIVE       numeric(22,9)                  not null default 0,
   SNAPSHOT_RESERVE     numeric(22,9)                  not null default 0,
   WAFL_SPARE           numeric(22,9)                  not null default 0,
   DAS_CAPACITY         numeric(22,9)                  not null default 0,
   NAS_USED_CAPACITY    numeric(22,9)                  not null default 0,
   UNCONFIGURED_CAPACITY numeric(22,9)                  not null default 0,
   INTERNAL_PHYSICAL    numeric(22,9)                  not null default 0,
   LOGICAL_UNKNOWN      numeric(22,9)                  not null default 0,
   TOTAL_UNCLAIMED      numeric(22,9)                  not null default 0,
   UNDISCOVERED_ALLOCATED numeric(22,9)                  not null default 0,
   PERCENT_ALLOCATED_HOST_CLAIMED numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED   numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_ALLOCATED numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_AVAILABLE numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_OVERHEAD numeric(7,4)                   not null default 0,
   PERCENT_CONFIGURED_UNDISCOVERD numeric(7,4)                   not null default 0,
   PERCENT_UNCONFIGURED numeric(7,4)                   not null default 0,
   CONSUMPTION_ASSIGNED numeric(22,9)                  not null default 0,
   CONSUMPTION_UNKNOWN  numeric(22,9)                  not null default 0,
   HOST_USAGE_APPS_ON_RAW numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_UN_USED numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_USED   numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_AUSED_APPS numeric(22,9)                  not null default 0,
   HOST_USAGE_FS_NAUSED_NON_APPS numeric(22,9)                  not null default 0,
   HOST_USAGE_OTHER     numeric(22,9)                  not null default 0,
   APPL_USED_CAPACITY   numeric(22,9)                  not null default 0,
   APPL_UN_USED_CAPACITY numeric(22,9)                  not null default 0,
   THINAWARE_CLAIMED_BY_HOST numeric(22,9)                  not null default 0,
   THINAWARE_TOTAL_UNCLAIMED numeric(22,9)                  not null default 0,
   THINAWARE_ALLOCATED_TOHOST numeric(22,9)                  not null default 0,
   THINAWARE_ALC_THGVIRTUALIZER numeric(22,9)                  not null default 0,
   THINAWARE_UNDISC_ALLOCATED numeric(22,9)                  not null default 0,
   NA_VOLUME_CAPACITY   numeric(22,9)                  not null default 0,
   NA_VOLUME_USED_CAPACITY numeric(22,9)                  not null default 0,
   NA_VOLUME_AVAILABLE_CAPACITY numeric(22,9)                  not null default 0,
   NA_QTREES            numeric(10)                    not null default 0,
   NA_SHARES            numeric(10)                    not null default 0,
   NA_VOLUMES           numeric(10)                    not null default 0,
   NA_FILESYSTEMS       numeric(10)                    not null default 0,
   SNAPSHOT_USED        numeric(22,9)                  not null default 0,
   OVERWRITE_RESERVE    numeric(22,9)                  not null default 0,
   NETAPP_REDUNDANT_CAPACITY numeric(22,9)                  not null default 0,
   NUMBER_OF_STORAGE_POOLS numeric(10)                    not null default 0,
   WAFL_SPARE_NEW       numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.ARRAY_STORAGE_ARRAY_ is 
'This is a staging table for loading data into warehouse table SAHARA.ARRAY_STORAGE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for array storage measures.  Each rows represents one sampling of values for an array.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.TOTAL_PHYSICAL is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column TOTAL_PHYSICAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total installed storage capacity of the array in gigabytes.  This is the sum of capacity from all installed physical disks on the array.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.V_PHYSICAL_EXTERNAL is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column V_PHYSICAL_EXTERNAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For virtualizer only.  This is the physical external capacity in gigabytes available to a virtualizer from backend arrays.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.LOCAL_LOGICAL is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column LOCAL_LOGICAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of physical storage, in gigabytes, that has been formatted and is apportioned into LUNs, or is ready to be apportioned into LUNs.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.V_VIRTUAL_LOGICAL is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column V_VIRTUAL_LOGICAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For virtualizer only.  The amount of virtual storage in gigabytes that has been formatted and is apportioned into LUNs, or is ready to be apportioned into LUNs.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.LOGICAL_AVAILABLE is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column LOGICAL_AVAILABLE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of configured storage that has not yet been apportioned into addressable units (LUNs).
This storage is typically part of raw storage volumes such as RAID groups.
Available storage reflects configured storage in arrays and unified storage devices only.
It does not include storage in JBODs, in SCSI disks, or in IDE disks.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs or SCSI disks) claimed by hosts.  "Claimed" means that the operating system has written a device handle for the LUN.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.A_ALLOCATED_TO_HOSTS_DIRECTLY is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column A_ALLOCATED_TO_HOSTS_DIRECTLY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For real array only.  Here LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.

In gigabytes.

TODO:  This needs a better description.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.A_ALLOCATED_TO_VIRTUALIZER is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column A_ALLOCATED_TO_VIRTUALIZER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For real arrays only.  LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.

In gigabytes.

TODO:  This needs a better description.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.A_ALLOC_HOST_VIA_VIRTUALIZER is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column A_ALLOC_HOST_VIA_VIRTUALIZER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For real arrays only.  LUN capacity in gigabytes allocated to hosts through virtualizers, such as TagmaStore.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.V_LOCAL_LOGICAL_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column V_LOCAL_LOGICAL_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For virtualizer only.  LUN capacity in gigabytes that is local to virtualizer arrays, such as TagmaStore, allocated to hosts.  These LUNs have been created from the array''s local physical disks, as opposed to virtual disks.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.V_VIRTUAL_LOGICAL_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column V_VIRTUAL_LOGICAL_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For virtualizer only.  Virtual LUN capacity in gigabytes allocated to hosts from virtualizers, like TagmaStore.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.UNALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column UNALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, for arrays and unified storage devices that is apportioned into LUNs but not designated for use by specific hosts.  These are LUNs that have not been masked to a host.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.A_UNALLOCATED_VIA_VIRUALIZER is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column A_UNALLOCATED_VIA_VIRUALIZER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For real arrays only.  LUN capacity in gigabytes allocated to virtualizers, such as TagmaStore, but NOT allocated to hosts from virtualizers.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.V_LOCAL_LOGICAL_UNALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column V_LOCAL_LOGICAL_UNALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For virtualizer only.  LUN capacity in gigabytes that is local to virtualizer arrays, such as TagmaStore, and is not allocated to hosts.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.V_VIRTUAL_LOGICAL_UNALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column V_VIRTUAL_LOGICAL_UNALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For virtualizer only.  Virtual LUN capacity in gigabytes unallocated to hosts from virtualizers, like TagmaStore.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.CLAIMED_BY_HOSTS is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column CLAIMED_BY_HOSTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total capacity, in gigabytes, of LUNs in arrays and unified storage devices that have been claimed by at least one host.

A LUN is considered claimed when a host''s operating system has written a device handle for it.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.REPLICATION is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column REPLICATION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs that are designated as target LUNs for snapshot or replication techniques.
An example is a business continuance volume (BCV) on a Symmetrix or DMX array, which contains a copy of a
standard device (STD).';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.RAID_OVERHEAD is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column RAID_OVERHEAD.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of configured storage, in gigabytes, that is reserved for overhead -- most commonly for RAID techniques such as mirroring and striping.

Depending on the array type, this value can also include additional storage such as disk-format capacity and internal configuration capacity for the array that is not discovered by CommandCentral Storage.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.ADMINISTRATIVE is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column ADMINISTRATIVE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total capacity, in gigabytes, of LUNs that are designated for administrative uses, not including RAID overhead.

In snapshot or replication scenarios, this value includes target LUNs for storing copied or replicated data.  Examples include:

- For EMC CLARiiON arrays: PSM, snapshot cache, and MirrorView
- For EMC Symmetrix and DMX arrays: BCV, VCMDB, Symmetrix file system device, and save device
- For Hitachi arrays managed by HiCommand: OnDemandDevice and CommandDevice';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.SNAPSHOT_RESERVE is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column SNAPSHOT_RESERVE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For unified storage devices, the total amount of storage, in gigabytes, that is reserved for snapshots in the file system.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.WAFL_SPARE is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column WAFL_SPARE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For unified storage devices, the total amount of storage, in gigabytes, that is devoted to the file system.

This value is calculated as follows: Physical capacity minus the sum of logical capacity, snapshot reserve, and RAID overhead.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.DAS_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column DAS_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Direct attached storage capacity.  This capacity applies to JBODs.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NAS_USED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NAS_USED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity used from NAS.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.UNCONFIGURED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column UNCONFIGURED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of physical storage that has not been formatted.  It includes:

- Space on array disks that has not been converted into raw storage volumes and has not been designated for hot spares
- Disks in NetApp filers on which no file systems have been created and that have not been designated for hot spares

Unconfigured storage does not include storage in SCSI disks, nor does it include storage in JBODs.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.INTERNAL_PHYSICAL is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column INTERNAL_PHYSICAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total amount of storage physically located on the array.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.LOGICAL_UNKNOWN is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column LOGICAL_UNKNOWN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Configured array storage that is not discovered by CommandCentral Storage.  CommandCentral Storage calls this amount "Logical: Unknown".';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.TOTAL_UNCLAIMED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column TOTAL_UNCLAIMED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of array storage allocated to hosts that in which the host''s O/S has not written a device handle to the LUN.  CommandCentral Storage calls this amount "Claimed Status: Unclaimed".';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.UNDISCOVERED_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column UNDISCOVERED_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of array storage allocated to a host that is currently not being discovered by CommandCentral Storage.  CommandCentral Storage calls this amount "Claimed Status: Unknown".';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.PERCENT_ALLOCATED_HOST_CLAIMED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column PERCENT_ALLOCATED_HOST_CLAIMED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of storage allocated to host in which a host''s O/S has written a device handle to the LUN.  This value is calculated by dividing "Claimed Capacity (GB)" by the sum of "Capacity Allocated to Hosts Directly (GB)" and "Capacity Allocated to Hosts Through Virtualizer (GB)".';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.PERCENT_CONFIGURED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column PERCENT_CONFIGURED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of an array''s total storage that has been formatted and usable.  This value is calculated by dividing Configured Capacity (GB) by Total Array Capacity (GB).';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.PERCENT_CONFIGURED_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column PERCENT_CONFIGURED_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of configured array storage that is allocated to either hosts or virtualizers.  This value is calculated by dividing Total Allocated Capacity (GB) by Configured Capacity (GB).';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.PERCENT_CONFIGURED_AVAILABLE is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column PERCENT_CONFIGURED_AVAILABLE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of configured array storage that is currently not yet in use but is available for allocation.  This value is calculated by dividing Capacity Available for Allocation (GB) by Configured Capacity (GB).';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.PERCENT_CONFIGURED_OVERHEAD is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column PERCENT_CONFIGURED_OVERHEAD.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of configured array storage that is reserved for RAID, replication and other administrative uses.  This value is calculated by dividing Capacity Reserved for Overhead (GB) by Configured Capacity (GB).';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.PERCENT_CONFIGURED_UNDISCOVERD is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column PERCENT_CONFIGURED_UNDISCOVERD.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of configured array storage that is not known because CommandCentral is not able to discover it.  This value is calculated by dividing Undiscovered Configured Capacity (GB) by Configured Capacity (GB)';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.PERCENT_UNCONFIGURED is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column PERCENT_UNCONFIGURED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of an array''s total storage that has not been formatted yet.  This value is calculated by dividing Unconfigured Capacity (GB) by Total Array Capacity (GB).';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NA_VOLUME_CAPACITY is 
'The aggregate capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NA_VOLUME_USED_CAPACITY is 
'The aggregate used capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NA_VOLUME_AVAILABLE_CAPACITY is 
'The aggregate free local capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NA_QTREES is 
'The number of qtrees for the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NA_SHARES is 
'The number of shares on the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NA_VOLUMES is 
'The number of volumes on the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.NA_FILESYSTEMS is 
'The number of filesystems on the device.

This is value is only available for Celerra devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.WAFL_SPARE_NEW is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column WAFL_SPARE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For unified storage devices, the total amount of storage, in gigabytes, that is devoted to the file system.

This value is calculated as follows: Physical capacity minus the sum of logical capacity, snapshot reserve, and RAID overhead.';

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter TOTAL_PHYSICAL add constraint C_RRSTRGRR_TOTALPHYSICAL check (TOTAL_PHYSICAL >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter V_PHYSICAL_EXTERNAL add constraint C_RRSTRGRR_VPHYSICALEXTERNAL check (V_PHYSICAL_EXTERNAL >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter LOCAL_LOGICAL add constraint C_RRSTRGRR_LOCALLOGICAL check (LOCAL_LOGICAL >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter V_VIRTUAL_LOGICAL add constraint C_RRSTRGRR_VVIRTUALLOGICAL check (V_VIRTUAL_LOGICAL >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter LOGICAL_AVAILABLE add constraint C_RRSTRGRR_LOGICALAVAILABLE check (LOGICAL_AVAILABLE >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter ALLOCATED add constraint C_RRSTRGRR_ALLOCATED check (ALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter A_ALLOCATED_TO_HOSTS_DIRECTLY add constraint C_RRSTRGRR_LLCTDTHSTSDRCTL check (A_ALLOCATED_TO_HOSTS_DIRECTLY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter A_ALLOCATED_TO_VIRTUALIZER add constraint C_RRSTRGRR_LLCTDTVRTLZR check (A_ALLOCATED_TO_VIRTUALIZER >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter A_ALLOC_HOST_VIA_VIRTUALIZER add constraint C_RRSTRGRR_LLCHSTVVRTLZR check (A_ALLOC_HOST_VIA_VIRTUALIZER >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter V_LOCAL_LOGICAL_ALLOCATED add constraint C_RRSTRGRR_VLCLLGCLLLCTD check (V_LOCAL_LOGICAL_ALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter V_VIRTUAL_LOGICAL_ALLOCATED add constraint C_RRSTRGRR_VVRTLLGCLLLCTD check (V_VIRTUAL_LOGICAL_ALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter UNALLOCATED add constraint C_RRSTRGRR_UNALLOCATED check (UNALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter A_UNALLOCATED_VIA_VIRUALIZER add constraint C_RRSTRGRR_NLLCTDVVRLZR check (A_UNALLOCATED_VIA_VIRUALIZER >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter V_LOCAL_LOGICAL_UNALLOCATED add constraint C_RRSTRGRR_VLCLLGCLNLLCTD check (V_LOCAL_LOGICAL_UNALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter V_VIRTUAL_LOGICAL_UNALLOCATED add constraint C_RRSTRGRR_VVRTLLGCLNLLCTD check (V_VIRTUAL_LOGICAL_UNALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter CLAIMED_BY_HOSTS add constraint C_RRSTRGRR_CLAIMEDBYHOSTS check (CLAIMED_BY_HOSTS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter REPLICATION add constraint C_RRSTRGRR_REPLICATION check (REPLICATION >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter RAID_OVERHEAD add constraint C_RRSTRGRR_RAIDOVERHEAD check (RAID_OVERHEAD >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter ADMINISTRATIVE add constraint C_RRSTRGRR_ADMINISTRATIVE check (ADMINISTRATIVE >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter SNAPSHOT_RESERVE add constraint C_RRSTRGRR_SNAPSHOTRESERVE check (SNAPSHOT_RESERVE >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter WAFL_SPARE add constraint C_RRSTRGRR_WAFLSPARE check (WAFL_SPARE >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter DAS_CAPACITY add constraint C_RRSTRGRR_DASCAPACITY check (DAS_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NAS_USED_CAPACITY add constraint C_RRSTRGRR_NASUSEDCAPACITY check (NAS_USED_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter UNCONFIGURED_CAPACITY add constraint C_RRSTRGRR_NCNFGRDCPCT check (UNCONFIGURED_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter INTERNAL_PHYSICAL add constraint C_RRSTRGRR_INTERNALPHYSICAL check (INTERNAL_PHYSICAL >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter LOGICAL_UNKNOWN add constraint C_RRSTRGRR_LOGICALUNKNOWN check (LOGICAL_UNKNOWN >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter TOTAL_UNCLAIMED add constraint C_RRSTRGRR_TOTALUNCLAIMED check (TOTAL_UNCLAIMED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter UNDISCOVERED_ALLOCATED add constraint C_RRSTRGRR_NDSCVRDLLCTD check (UNDISCOVERED_ALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter CONSUMPTION_ASSIGNED add constraint CKC_CONSUMPTION_ASSIG_ARRAY_ST check (CONSUMPTION_ASSIGNED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter CONSUMPTION_UNKNOWN add constraint CKC_CONSUMPTION_UNKNO_ARRAY_ST check (CONSUMPTION_UNKNOWN >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter HOST_USAGE_APPS_ON_RAW add constraint CKC_HOST_USAGE_APPS_O_ARRAY_ST check (HOST_USAGE_APPS_ON_RAW >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter HOST_USAGE_FS_UN_USED add constraint CKC_HOST_USAGE_FS_UN__ARRAY_ST check (HOST_USAGE_FS_UN_USED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter HOST_USAGE_FS_USED add constraint CKC_HOST_USAGE_FS_USE_ARRAY_ST check (HOST_USAGE_FS_USED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter HOST_USAGE_FS_AUSED_APPS add constraint CKC_HOST_USAGE_FS_AUS_ARRAY_ST check (HOST_USAGE_FS_AUSED_APPS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter HOST_USAGE_FS_NAUSED_NON_APPS add constraint CKC_HOST_USAGE_FS_NAU_ARRAY_ST check (HOST_USAGE_FS_NAUSED_NON_APPS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter HOST_USAGE_OTHER add constraint CKC_HOST_USAGE_OTHER_ARRAY_ST check (HOST_USAGE_OTHER >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter APPL_USED_CAPACITY add constraint CKC_APPL_USED_CAPACIT_ARRAY_ST check (APPL_USED_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter APPL_UN_USED_CAPACITY add constraint CKC_APPL_UN_USED_CAPA_ARRAY_ST check (APPL_UN_USED_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter THINAWARE_CLAIMED_BY_HOST add constraint CKC_THINAWARE_CLAIMED_ARRAY_ST check (THINAWARE_CLAIMED_BY_HOST >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter THINAWARE_TOTAL_UNCLAIMED add constraint CKC_THINAWARE_TOTAL_U_ARRAY_ST check (THINAWARE_TOTAL_UNCLAIMED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter THINAWARE_ALLOCATED_TOHOST add constraint CKC_THINAWARE_ALLOCAT_ARRAY_ST check (THINAWARE_ALLOCATED_TOHOST >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter THINAWARE_ALC_THGVIRTUALIZER add constraint CKC_THINAWARE_ALC_THG_ARRAY_ST check (THINAWARE_ALC_THGVIRTUALIZER >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter THINAWARE_UNDISC_ALLOCATED add constraint CKC_THINAWARE_UNDISC__ARRAY_ST check (THINAWARE_UNDISC_ALLOCATED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUME_CAPACITY add constraint C_ARRAYSTORAGE_NVLMCPCT check (NA_VOLUME_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUME_USED_CAPACITY add constraint C_ARRAYSTORAGE_NVLMSDCPCT check (NA_VOLUME_USED_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUME_AVAILABLE_CAPACITY add constraint C_ARRAYSTORAGE_NVLMVLBLCPCT check (NA_VOLUME_AVAILABLE_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_QTREES add constraint C_ARRAYSTORAGE_NAQTREES check (NA_QTREES >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_SHARES add constraint C_ARRAYSTORAGE_NASHARES check (NA_SHARES >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUMES add constraint C_ARRAYSTORAGE_NAVOLUMES check (NA_VOLUMES >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_FILESYSTEMS add constraint C_ARRAYSTORAGE_NAFILESYSTEMS check (NA_FILESYSTEMS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter SNAPSHOT_USED add constraint CKC_SNAPSHOT_USED_ARRAY_ST check (SNAPSHOT_USED >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter OVERWRITE_RESERVE add constraint CKC_OVERWRITE_RESERVE_ARRAY_ST check (OVERWRITE_RESERVE >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NETAPP_REDUNDANT_CAPACITY add constraint CKC_NETAPP_REDUNDANT__ARRAY_ST check (NETAPP_REDUNDANT_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NUMBER_OF_STORAGE_POOLS add constraint CKC_NUMBER_OF_STORAGE_ARRAY_ST check (NUMBER_OF_STORAGE_POOLS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter WAFL_SPARE_NEW add constraint C_RRSTRGRR_WAFLSPARE_NEW check (WAFL_SPARE_NEW >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   add constraint PK_ARRAY_STORAGE_ARRAY_ primary key (ARRAY_OID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.ARRAY_STORAGE_ARRAY_ to SAHARA;

/*==============================================================*/
/* Table: ARRAY_STORAGE_NETAPP_DM_                              */
/*==============================================================*/
create table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_ 
(
   DM_ID                numeric(10)                    not null,
   DM_BATCH_ID          numeric(10)                    not null,
   NA_CONSUMPTION       numeric(22,9)                  not null default 0,
   NA_FILES             numeric(10)                    not null default 0,
   NA_FOLDERS           numeric(10)                    not null default 0,
   NA_USERS             numeric(10)                    not null default 0
);

comment on table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.ARRAY_STORAGE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for array storage measures.  Each rows represents one sampling of values for an array.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_.NA_CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all of the files on the device, in gigabytes.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_.NA_FILES is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files on the device.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_.NA_FOLDERS is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_FOLDERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of directories on the device.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_.NA_USERS is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_USERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of users that have files on the device.

This is value is only available for NetApp devices and only if the device was included in a DM rule.  It will be null otherwise.';

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_
   alter DM_ID add constraint C_RRSTRGNTPPDM_DMID check (DM_ID >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_
   alter DM_BATCH_ID add constraint C_RRSTRGNTPPDM_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_
   alter NA_CONSUMPTION add constraint C_RRSTRGNTPPDM_NACONSUMPTION check (NA_CONSUMPTION >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_
   alter NA_FILES add constraint C_RRSTRGNTPPDM_NAFILES check (NA_FILES >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_
   alter NA_FOLDERS add constraint C_RRSTRGNTPPDM_NAFOLDERS check (NA_FOLDERS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_
   alter NA_USERS add constraint C_RRSTRGNTPPDM_NAUSERS check (NA_USERS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_
   add constraint PK_ARRAY_STORAGE_NETAPP_DM_ primary key (DM_ID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_ to SAHARA;

/*==============================================================*/
/* Table: ARRAY_STORAGE_NETAPP_HAL_                             */
/*==============================================================*/
create table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_ 
(
   ARRAY_OID            varchar(512)                   not null,
   DM_ID                numeric(10)                    not null,
   NA_VOLUME_CAPACITY   numeric(22,9)                  not null default 0,
   NA_VOLUME_USED_CAPACITY numeric(22,9)                  not null default 0,
   NA_VOLUME_AVAILABLE_CAPACITY numeric(22,9)                  not null default 0,
   NA_QTREES            numeric(10)                    not null default 0,
   NA_SHARES            numeric(10)                    not null default 0,
   NA_VOLUMES           numeric(10)                    not null default 0,
   NA_FILESYSTEMS       numeric(10)                    not null default 0
);

comment on table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_ is 
'This is a staging table for loading data into warehouse table SAHARA.ARRAY_STORAGE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for array storage measures.  Each rows represents one sampling of values for an array.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.NA_VOLUME_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_VOLUME_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.NA_VOLUME_USED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_VOLUME_USED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate used capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.NA_VOLUME_AVAILABLE_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_VOLUME_AVAILABLE_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate free local capacity, in gigabytes, of all volumes associated with the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.NA_QTREES is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_QTREES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of qtrees for the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.NA_SHARES is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_SHARES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of shares on the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.NA_VOLUMES is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_VOLUMES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of volumes on the device.

This is value is only available for NetApp devices and will be null otherwise.';

comment on column SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_.NA_FILESYSTEMS is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column NA_FILESYSTEMS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of filesystems on the device.

This is value is only available for Celerra devices and will be null otherwise.';

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter DM_ID add constraint C_RRSTRGNTPPHL_DMID check (DM_ID >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter NA_VOLUME_CAPACITY add constraint C_RRSTRGNTPPHL_NVLMCPCT check (NA_VOLUME_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter NA_VOLUME_USED_CAPACITY add constraint C_RRSTRGNTPPHL_NVLMSDCPCT check (NA_VOLUME_USED_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter NA_VOLUME_AVAILABLE_CAPACITY add constraint C_RRSTRGNTPPHL_NVLMVLBLCPCT check (NA_VOLUME_AVAILABLE_CAPACITY >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter NA_QTREES add constraint C_RRSTRGNTPPHL_NAQTREES check (NA_QTREES >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter NA_SHARES add constraint C_RRSTRGNTPPHL_NASHARES check (NA_SHARES >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter NA_VOLUMES add constraint C_RRSTRGNTPPHL_NAVOLUMES check (NA_VOLUMES >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   alter NA_FILESYSTEMS add constraint C_RRSTRGNTPPHL_NAFILESYSTEMS check (NA_FILESYSTEMS >= 0);

alter table SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_
   add constraint PK_ARRAY_STORAGE_NETAPP_HAL_ primary key (ARRAY_OID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_ to SAHARA;

/*==============================================================*/
/* Table: BROKER_INFO                                           */
/*==============================================================*/
create table SAHARA.BROKER_INFO 
(
   BROKER_ID            unsigned integer               not null default autoincrement,
   DOMAIN_NAME          varchar(128)                   not null,
   DOMAIN_TYPE          varchar(128)                   not null,
   BROKER_NAME          varchar(128)                   not null,
   BROKER_TYPE          smallint                       not null,
   BROKER_PORT          int                            not null,
   STATUS               int                            null,
   LDAP_SERVER          varchar(256)                   null,
   BIND_USER_NAME       varchar(256)                   null,
   BIND_USER_PASSWORD   varchar(256)                   null,
   SEARCH_USER          varchar(256)                   null,
   SEARCH_GROUP         varchar(256)                   null,
   SEARCH_BASE          varchar(256)                   null,
   SSL_CERT_LOCATION    varchar(256)                   null,
   LDAP_PORT            int                            null
);

comment on table SAHARA.BROKER_INFO is 
'To Store AT Broker Information (LDAP and AD information) ';

alter table SAHARA.BROKER_INFO
   add constraint PK_BROKER_INFO primary key (BROKER_ID);

grant SELECT,ALTER,DELETE,INSERT,REFERENCES,UPDATE on SAHARA.BROKER_INFO to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_APPROVAL_PENDING                                */
/*==============================================================*/
create table SAHARA.BVIEW_APPROVAL_PENDING 
(
   BVIEW_RULES_KEY      numeric(10)                    null,
   BVIEW_APPROVALPENDING_KEY numeric(10)                    not null default autoincrement,
   BUSINESSVIEW_NAME    varchar(64)                    null,
   BUSINESSVIEWNODENAME varchar(64)                    null,
   OBJECTTYPE           varchar(64)                    null,
   APPROVALSTATE        integer                        null,
   NOOFOBJECTS          integer                        null
);

comment on table SAHARA.BVIEW_APPROVAL_PENDING is 
'This table contains user defined Conditions for every business view rules';

comment on column SAHARA.BVIEW_APPROVAL_PENDING.BVIEW_RULES_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension table per business view node';

comment on column SAHARA.BVIEW_APPROVAL_PENDING.BVIEW_APPROVALPENDING_KEY is 
'The unique database generated identifier for the rule.';

alter table SAHARA.BVIEW_APPROVAL_PENDING
   add constraint PK_BVIEW_APPROVAL_PENDING primary key (BVIEW_APPROVALPENDING_KEY);

grant SELECT on SAHARA.BVIEW_APPROVAL_PENDING to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_CONDITIONS                                      */
/*==============================================================*/
create table SAHARA.BVIEW_CONDITIONS 
(
   BVIEW_RULES_KEY      numeric(10)                    null,
   BVIEW_CONDITIONS_KEY numeric(10)                    not null default autoincrement,
   ATTRIBUTE            long varchar                   null,
   OPERATOR             varchar(255)                   null,
   ATTRIBUTEVALUE       long varchar                   null,
   CONJUNCTION          varchar(5)                     null
);

comment on table SAHARA.BVIEW_CONDITIONS is 
'This table contains user defined Conditions for every business view rules';

comment on column SAHARA.BVIEW_CONDITIONS.BVIEW_RULES_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension table per business view node';

comment on column SAHARA.BVIEW_CONDITIONS.BVIEW_CONDITIONS_KEY is 
'The unique database generated identifier for the rule.';

comment on column SAHARA.BVIEW_CONDITIONS.ATTRIBUTE is 
'The column represent the attribute of the object type used for pattern
matching criteria.';

comment on column SAHARA.BVIEW_CONDITIONS.OPERATOR is 
'Operands at this point would be equal, contains and LIKE. Regex could also be used';

comment on column SAHARA.BVIEW_CONDITIONS.ATTRIBUTEVALUE is 
'The Attribute Value that need to be search should be in this column';

alter table SAHARA.BVIEW_CONDITIONS
   add constraint PK_BVIEW_CONDITIONS primary key (BVIEW_CONDITIONS_KEY);

grant SELECT on SAHARA.BVIEW_CONDITIONS to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_DIMENSION                                       */
/*==============================================================*/
create table SAHARA.BVIEW_DIMENSION 
(
   DIMENSION            varchar(26)                    not null,
   ALIAS                varchar(12)                    not null,
   DISPLAY_NAME         varchar(64)                    not null
);

comment on table SAHARA.BVIEW_DIMENSION is 
'This table contains a list of dimensions that can support business views.  This table is populated at install and upgrade.';

comment on column SAHARA.BVIEW_DIMENSION.DIMENSION is 
'The name for the dimension.  This is the table name that represents the dimension.

This column is limited to 26 characters because the table name is expect to allow room for a primary key of the form table_name + "_KEY".';

comment on column SAHARA.BVIEW_DIMENSION.ALIAS is 
'The alias name for the dimension.  This is the name that is used to form table and view names for the dimension.  It has a limited length (12 characters) in order to be able to construct names that fall within the Oracle 30 character identifier limit.  For dimensions with names longer than 12 characters, this is an abbreviation.';

comment on column SAHARA.BVIEW_DIMENSION.DISPLAY_NAME is 
'A display name for the dimension to be used in GUI selections, etc.';

alter table SAHARA.BVIEW_DIMENSION
   add constraint PK_BVIEW_DIMENSION primary key (DIMENSION);

alter table SAHARA.BVIEW_DIMENSION
   add constraint AK_AK_BVIEW_DIMENSION_BVIEW_DI unique (ALIAS);

grant SELECT,REFERENCES on SAHARA.BVIEW_DIMENSION to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_DIMENSION_BVIEW                                 */
/*==============================================================*/
create table SAHARA.BVIEW_DIMENSION_BVIEW 
(
   DIMENSION            varchar(26)                    not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.BVIEW_DIMENSION_BVIEW is 
'This table defines from which dimension(s) rows may be associated with a business view.  It is a join table for the many-to-many relationship between BVIEW_DIMENSION and BVIEW.  To put that in CommandCentral Storage Enterprise Reporter terms: This table defines with which object types objects may be associated with a business view.';

comment on column SAHARA.BVIEW_DIMENSION_BVIEW.DIMENSION is 
'The dimension with which the business view is associated.';

comment on column SAHARA.BVIEW_DIMENSION_BVIEW.BVIEW_KEY is 
'The business view key of the business view with which the dimension is associated.';

alter table SAHARA.BVIEW_DIMENSION_BVIEW
   add constraint PK_BVIEW_DIMENSION_BVIEW primary key (DIMENSION, BVIEW_KEY);

alter table SAHARA.BVIEW_DIMENSION_BVIEW
   add constraint AK_AK_BVIEWDIMENSIONB_BVIEW_DI unique (BVIEW_KEY, DIMENSION);

grant SELECT on SAHARA.BVIEW_DIMENSION_BVIEW to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_IMPL                                            */
/*==============================================================*/
create table SAHARA.BVIEW_IMPL 
(
   BVIEW_KEY            numeric(10)                    not null,
   NAME                 varchar(12)                    not null,
   VIEW_CREATION_REQUIRED char(1)                        not null default 'T',
   APPROVALREQUIRED     bit                            null default 1,
   ENABLERULE           bit                            null default 0
);

comment on table SAHARA.BVIEW_IMPL is 
'A business view (as a whole) is represented largely by its root node, which is special in the BVIEW_NODE table.  This table contains additional attributes for the business view as a whole and supplements the root node.  It is easiest to use the database view BVIEW to visualize business views.

See the comments on table BVIEW_NODE for further explanation.';

comment on column SAHARA.BVIEW_IMPL.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

comment on column SAHARA.BVIEW_IMPL.NAME is 
'This is an internal unique name for the business view.  The name is used to form names for database objects (e.g. views, indexes).

A valid name must begin with an upper case letter, and consist of only alphanumeric characters and the special characters $, _, and #.';

comment on column SAHARA.BVIEW_IMPL.VIEW_CREATION_REQUIRED is 
'This is a boolean flag that indicates if the database views associated with this business view require (re-)creation.  This flag is set by triggers and cleared by BVIEW_API.Publish after the business view is created.';

comment on column SAHARA.BVIEW_IMPL.APPROVALREQUIRED is 
'The bit will be 1 by default, although is valid only if the EnableRule Flag is 
set to 1.';

comment on column SAHARA.BVIEW_IMPL.ENABLERULE is 
'Rule will be disabled or 0 by default. Will be enabled, once user adds the rule';

alter table SAHARA.BVIEW_IMPL
   alter NAME add constraint C_BVIEWIMPL_NAME check (NAME = upper(NAME));

alter table SAHARA.BVIEW_IMPL
   alter VIEW_CREATION_REQUIRED add constraint C_BVIEWIMPL_VWCRTNRQRD check (VIEW_CREATION_REQUIRED in ('T','F'));

alter table SAHARA.BVIEW_IMPL
   add constraint PK_BVIEW_IMPL primary key (BVIEW_KEY);

alter table SAHARA.BVIEW_IMPL
   add constraint AK_BVIEWIMPL_NAME unique (NAME);

comment on index SAHARA.BVIEW_IMPL.AK_BVIEWIMPL_NAME is 
'The name of a business view is a unique identifier for the view (i.e. an alternate key).';

grant SELECT on SAHARA.BVIEW_IMPL to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_LEVEL                                           */
/*==============================================================*/
create table SAHARA.BVIEW_LEVEL 
(
   BVIEW_KEY            numeric(10)                    not null,
   LEVEL                numeric(3)                     not null,
   NAME                 varchar(30)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   NOTE                 long varchar                   null
);

comment on table SAHARA.BVIEW_LEVEL is 
'Metadata for the levels of business views.  Business views are end-user specified hierarchical attributes which can be assigned to objects.  The hierarchy forms a tree.';

comment on column SAHARA.BVIEW_LEVEL.BVIEW_KEY is 
'The business view to which the level belongs.  This is the same as the business view node key for the root node of the view.  It uniquely identified the business view.';

comment on column SAHARA.BVIEW_LEVEL.LEVEL is 
'The level of the business view.';

comment on column SAHARA.BVIEW_LEVEL.NAME is 
'The internal unique name for the level within the business view.  The name is used to form names for database objects (e.g. column names).

A valid name must begin with an upper case letter, and consist of only alphanumeric characters and the special characters $, _, and #.';

comment on column SAHARA.BVIEW_LEVEL.DISPLAY_NAME is 
'The displayable short name for name for the business view level (e.g. for a geography view, Continent, Country, State, City).';

comment on column SAHARA.BVIEW_LEVEL.NOTE is 
'Free form text or notes about the business view level.';

alter table SAHARA.BVIEW_LEVEL
   alter LEVEL add constraint C_BVIEWLEVEL_LEVEL check (LEVEL >= 0);

alter table SAHARA.BVIEW_LEVEL
   alter NAME add constraint C_BVIEWLEVEL_NAME check (NAME = upper(NAME));

alter table SAHARA.BVIEW_LEVEL
   add constraint PK_BVIEW_LEVEL primary key (BVIEW_KEY, LEVEL);

alter table SAHARA.BVIEW_LEVEL
   add constraint AK_BVIEW_LEVEL_NAME unique (BVIEW_KEY, NAME);

comment on index SAHARA.BVIEW_LEVEL.AK_BVIEW_LEVEL_NAME is 
'The name of a business view level is a unique identifier for the level in the business view (i.e. an alternate key).';

grant SELECT on SAHARA.BVIEW_LEVEL to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_NODE                                            */
/*==============================================================*/
create table SAHARA.BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null default autoincrement,
   PARENT_BVIEW_NODE_KEY numeric(10)                    null,
   BVIEW_KEY            numeric(10)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   LEVEL                numeric(3)                     null,
   NOTE                 long varchar                   null,
   "LEFT"               numeric(9)                     not null default 0,
   "RIGHT"              numeric(9)                     not null default 0
);

comment on table SAHARA.BVIEW_NODE is 
'This table contains the nodes of business views.  Business views are end-user specified hierarchical attributes which can be assigned to objects.  The hierarchy forms a tree.  The nodes of the tree are stored in this table.

The root node of a business view tree has null for the parent node and level.  The display name and note for the root node are considered to apply to the business view as a whole, since the root node is never displayed or associated with dimensions.  It also has a corresponding entry in table BVIEW_IMPL.  Business views as a whole are best visualized using the view BVIEW which joins BVIEW_NODE and BVIEW_IMPL.

The nodes of the business view tree can be navigated both recursively and using the special properties of left / right numbers.  Recursively, this is done by using the parent node key for the nodes.  The left / right numbers of a business view have the property that the left and right numbers of any node always have a value between the values of the left and right numbers of its parent node; the left number is also always less than the right number.  Queries may be formed using this property to find all parents and children of nodes without the use of recursion.';

comment on column SAHARA.BVIEW_NODE.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.BVIEW_NODE.PARENT_BVIEW_NODE_KEY is 
'The node key of the parent node for this business view node.  If the parent node key is null, this node is the root of the business view.';

comment on column SAHARA.BVIEW_NODE.BVIEW_KEY is 
'The node key of the root node for the business view.  This is used as an identifier for the business view as a whole.';

comment on column SAHARA.BVIEW_NODE.DISPLAY_NAME is 
'A displayable name for the business view node.  This corresponds to a value inside the hierarchy of the business view.';

comment on column SAHARA.BVIEW_NODE.LEVEL is 
'The level or depth of the node within the business view.  The root of the business view is level 0.  Each subsequent intervening parent-to-child relationship increments the level.';

comment on column SAHARA.BVIEW_NODE.NOTE is 
'Free form text or notes about the business view node.';

comment on column SAHARA.BVIEW_NODE."LEFT" is 
'The left number for hierarchy placement comparisons.';

comment on column SAHARA.BVIEW_NODE."RIGHT" is 
'The right number for hierarchy placement comparisons.';

alter table SAHARA.BVIEW_NODE
   add constraint PK_BVIEW_NODE primary key (BVIEW_NODE_KEY);

/*==============================================================*/
/* Index: I_BVNODE_PARENT_DISPLAYNAME                           */
/*==============================================================*/
create unique index I_BVNODE_PARENT_DISPLAYNAME on SAHARA.BVIEW_NODE (
PARENT_BVIEW_NODE_KEY ASC,
DISPLAY_NAME ASC
);

comment on index SAHARA.BVIEW_NODE.I_BVNODE_PARENT_DISPLAYNAME is 
'The display name of a node must be unique for nodes with the same parent.';

/*==============================================================*/
/* Index: I_BVNODE_BVIEWKEY                                     */
/*==============================================================*/
create index I_BVNODE_BVIEWKEY on SAHARA.BVIEW_NODE (
BVIEW_KEY ASC
);

/*==============================================================*/
/* Index: I_BVNODE_LEVEL                                        */
/*==============================================================*/
create index I_BVNODE_LEVEL on SAHARA.BVIEW_NODE (
LEVEL ASC
);

/*==============================================================*/
/* Index: I_BVNODE_NUMBERS                                      */
/*==============================================================*/
create index I_BVNODE_NUMBERS on SAHARA.BVIEW_NODE (
"LEFT" ASC,
"RIGHT" ASC
);

/*==============================================================*/
/* Index: I_BVNODE_RIGHTNUMBER                                  */
/*==============================================================*/
create index I_BVNODE_RIGHTNUMBER on SAHARA.BVIEW_NODE (
"RIGHT" ASC
);

grant SELECT on SAHARA.BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_RULE                                            */
/*==============================================================*/
create table SAHARA.BVIEW_RULE 
(
   BVIEW_RULE_KEY       numeric(10)                    not null default autoincrement,
   DIMENSION            varchar(26)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   UPPER_DISPLAY_NAME   varchar(64)                    not null,
   DESCRIPTION          long varchar                   null,
   RANK                 numeric(2)                     not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.BVIEW_RULE is 
'This table contains configuration information for user defined business view rules.';

comment on column SAHARA.BVIEW_RULE.BVIEW_RULE_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension.';

comment on column SAHARA.BVIEW_RULE.DIMENSION is 
'The dimension context in which the rule is defined.';

comment on column SAHARA.BVIEW_RULE.DISPLAY_NAME is 
'The display name for the business view rule.  Each display name must be unique (ignoring character case) within a dimension context.';

comment on column SAHARA.BVIEW_RULE.UPPER_DISPLAY_NAME is 
'The upper case form of the display name for the business view rule.  This is used to enforce the constrant that each display name must be unique (ignoring character case) within a dimension context.';

comment on column SAHARA.BVIEW_RULE.DESCRIPTION is 
'A user specified description for the business view rule.';

comment on column SAHARA.BVIEW_RULE.RANK is 
'The rank of the business view rule.  Business view rules are ranked using integers starting at one (1) and increasing by one for each rule.';

comment on column SAHARA.BVIEW_RULE.BVIEW_KEY is 
'The business view key of the business view with which the rule is associated.
Busines View Key is the node key of the root node for the business view.  This is used as an identifier for the business view as a whole.';

alter table SAHARA.BVIEW_RULE
   add constraint PK_BVIEW_RULE primary key (BVIEW_RULE_KEY);

alter table SAHARA.BVIEW_RULE
   add constraint AK_AK_BVIEW_RULE_DIMU_BVIEW_RU unique (UPPER_DISPLAY_NAME);

alter table SAHARA.BVIEW_RULE
   add constraint AK_TIER_CONTEXTRANK unique (RANK);

grant SELECT on SAHARA.BVIEW_RULE to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_RULES                                           */
/*==============================================================*/
create table SAHARA.BVIEW_RULES 
(
   BVIEW_RULES_KEY      numeric(10)                    not null default autoincrement,
   NAME                 varchar(64)                    not null,
   DESCRIPTION          long varchar                   null,
   OBJECTTYPE           varchar(26)                    not null,
   APPROVALREQUIRED     bit                            null,
   ENABLED              bit                            null,
   BVIEW_KEY            numeric(10)                    not null,
   BVIEW_NODE_KEY       numeric(10)                    not null,
   DIMENSION            varchar(26)                    not null,
   APPROVALSTATE        integer                        null,
   SQLQUERY             long varchar                   null
);

comment on table SAHARA.BVIEW_RULES is 
'This table stores all  attributes for the business view rules.';

comment on column SAHARA.BVIEW_RULES.BVIEW_RULES_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension table per business view node';

comment on column SAHARA.BVIEW_RULES.NAME is 
'The display name for the business view rule.  Each display name must be unique (ignoring character case) within a dimension context.';

comment on column SAHARA.BVIEW_RULES.DESCRIPTION is 
'A user specified description for the business view rule.';

comment on column SAHARA.BVIEW_RULES.OBJECTTYPE is 
'The Object Type on which the rule is defined or the dimension table on which the rule is defined.';

comment on column SAHARA.BVIEW_RULES.BVIEW_KEY is 
'The business view key of the business view with which the rule is associated.
Busines View Key is the node key of the root node for the business view.  This is used as an identifier for the business view as a whole.';

comment on column SAHARA.BVIEW_RULES.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.BVIEW_RULES.DIMENSION is 
'The name for the dimension.  This is the table name that represents the dimension.

This column is limited to 26 characters because the table name is expect to allow room for a primary key of the form table_name + "_KEY".';

alter table SAHARA.BVIEW_RULES
   add constraint PK_BVIEW_RULES primary key (BVIEW_RULES_KEY);

alter table SAHARA.BVIEW_RULES
   add constraint AK_C_BV_RULE_BVIEW_RU unique (BVIEW_KEY, BVIEW_NODE_KEY, DIMENSION);

grant SELECT on SAHARA.BVIEW_RULES to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_STATUS                                          */
/*==============================================================*/
create table SAHARA.BVIEW_STATUS 
(
   BVIEW_STATUS_KEY     char(3)                        not null,
   NUMBER_REQUIRED      char(1)                        not null default 'T',
   VIEW_REFRESH_REQUIRED char(1)                        not null default 'T'
);

comment on table SAHARA.BVIEW_STATUS is 
'The status of maintenance tasks related to business views.  Business views are end-user specified hierarchical attributes which can be assigned to objects.  The hierarchy forms a tree.';

comment on column SAHARA.BVIEW_STATUS.BVIEW_STATUS_KEY is 
'A surrogate key that allows only one value.  This forces the table to have only one row.';

comment on column SAHARA.BVIEW_STATUS.NUMBER_REQUIRED is 
'A boolean flag that indicates if left / right numbers need to be assigned to business view nodes.  This flag is set by triggers on the BVIEW_NODE table and cleared by stored procedure BVIEW_API.Publish.';

comment on column SAHARA.BVIEW_STATUS.VIEW_REFRESH_REQUIRED is 
'A boolean flag that indicates if materialized views associated with the business view require refreshing.  This flag is set by triggers on the tables from which the data comes and cleared by BVIEW_API.Publish.';

alter table SAHARA.BVIEW_STATUS
   alter BVIEW_STATUS_KEY add constraint C_BVIEWSTATUS_BVIEWSTATUSKEY check (BVIEW_STATUS_KEY in ('KEY'));

alter table SAHARA.BVIEW_STATUS
   alter NUMBER_REQUIRED add constraint C_BVIEWSTATUS_NUMBERREQUIRED check (NUMBER_REQUIRED in ('T','F'));

alter table SAHARA.BVIEW_STATUS
   alter VIEW_REFRESH_REQUIRED add constraint C_BVIEWSTATUS_VWRFRSHRQRD check (VIEW_REFRESH_REQUIRED in ('T','F'));

alter table SAHARA.BVIEW_STATUS
   add constraint PK_BVIEW_STATUS primary key (BVIEW_STATUS_KEY);

grant SELECT on SAHARA.BVIEW_STATUS to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_SUBRULE                                         */
/*==============================================================*/
create table SAHARA.BVIEW_SUBRULE 
(
   BVIEW_SUBRULE_KEY    numeric(10)                    not null default autoincrement,
   GUI_HINT             long varchar                   null,
   RULE_TEXT            long varchar                   not null,
   SQL_CODE             long varchar                   not null,
   RANK                 numeric(2)                     not null,
   BVIEW_NODE_KEY       numeric(10)                    not null,
   BVIEW_RULE_KEY       numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.BVIEW_SUBRULE is 
'This table contains user defined subrules or conditions for a business view node.';

comment on column SAHARA.BVIEW_SUBRULE.BVIEW_SUBRULE_KEY is 
'The unique database generated identifier for the rule.';

comment on column SAHARA.BVIEW_SUBRULE.GUI_HINT is 
'An optional GUI hint for the rule.  This column is provided for any use the GUI defines.

For example: it may be preferred that the GUI present a specific type of rule in a structured way with specific fields rather than as free form text.  The GUI itself might then generate and store the underlying rule.  To simplify parsing the rule at a later time to populate the GUI when editing an existing rule, the GUI might choose to store the rule in a GUI specific XML format in the GUI hint.';

comment on column SAHARA.BVIEW_SUBRULE.RULE_TEXT is 
'The rule text, before rule elements are substituted.  This is the rule as the user sees it for free form rules.';

comment on column SAHARA.BVIEW_SUBRULE.SQL_CODE is 
'The SQL form of the rule.  This is the rule after rule element substitutions have been performed.';

comment on column SAHARA.BVIEW_SUBRULE.RANK is 
'The rank of the business view sub rule.  Business view sub rules are ranked using integers starting at one (1) and increasing by one for each sub rule.';

comment on column SAHARA.BVIEW_SUBRULE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the subrule is associated.';

comment on column SAHARA.BVIEW_SUBRULE.BVIEW_RULE_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension.';

comment on column SAHARA.BVIEW_SUBRULE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the subrule table record.';

alter table SAHARA.BVIEW_SUBRULE
   add constraint PK_BVIEW_SUBRULE primary key (BVIEW_SUBRULE_KEY);

grant SELECT on SAHARA.BVIEW_SUBRULE to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_SUBRULE_ELEMENT                                 */
/*==============================================================*/
create table SAHARA.BVIEW_SUBRULE_ELEMENT 
(
   DIMENSION            varchar(26)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   DESCRIPTION          long varchar                   null,
   TYPE                 varchar(32)                    not null,
   RULE_TOKEN_UPPER     varchar(128)                   not null,
   RULE_TOKEN           varchar(128)                   not null,
   SQL_CODE             varchar(1024)                  not null
);

comment on table SAHARA.BVIEW_SUBRULE_ELEMENT is 
'Business view subrule elements are items which may be substituted into a business view subrule.  Typically, elements are database columns.

To make it easier for the end-user to understand the meaning of a particular element, the element has a display name and descriptive text.  It also has corresponding SQL, such as a database column name.  The end-user forms a rule using the display name, but the implementation substitutes the actual SQL for the elements when evaluating the expression.

Rule elements are organized in the GUI similar to a two level tree view.  This table contains the information required for the leaf nodes, or elements, of the tree.  Each element belongs to a dimension.

This table is populated at install and upgrade.';

comment on column SAHARA.BVIEW_SUBRULE_ELEMENT.DIMENSION is 
'The dimension to which the rule element belongs.';

comment on column SAHARA.BVIEW_SUBRULE_ELEMENT.DISPLAY_NAME is 
'The display name for the rule element.  The display name must be unique within the rule element''s dimension and is used to name the leaf nodes of the tree view of elements for building rules.';

comment on column SAHARA.BVIEW_SUBRULE_ELEMENT.DESCRIPTION is 
'A description for the rule element.  The description is displayed in the GUI when the rule element is selected.';

comment on column SAHARA.BVIEW_SUBRULE_ELEMENT.TYPE is 
'An enumerated value that identifies the type of rule element.  The type is used in forming and validating the rule.

Valid types:
- COLUMN - the element is a database column';

comment on column SAHARA.BVIEW_SUBRULE_ELEMENT.RULE_TOKEN_UPPER is 
'Rule token names must be unique, independent of character case because a case insensitive replacement is used when converting rule text into SQL code.  This column is populated by a trigger and is part of a unique constraint (alternate key) to enforce this rule.';

comment on column SAHARA.BVIEW_SUBRULE_ELEMENT.RULE_TOKEN is 
'The display text to be inserted into the rule when the user adds the element to a rule.';

comment on column SAHARA.BVIEW_SUBRULE_ELEMENT.SQL_CODE is 
'The actual SQL code substituted for the rule element token when the rule is processed.';

alter table SAHARA.BVIEW_SUBRULE_ELEMENT
   alter TYPE add constraint C_BVWSBRLLMNT_TYPE check (TYPE in ('COLUMN'));

alter table SAHARA.BVIEW_SUBRULE_ELEMENT
   alter RULE_TOKEN_UPPER add constraint C_BVWSBRLLMNT_RULETOKENUPPER check (RULE_TOKEN_UPPER  = upper(RULE_TOKEN_UPPER));

alter table SAHARA.BVIEW_SUBRULE_ELEMENT
   add constraint PK_BVIEW_SUBRULE_ELEMENT primary key (DIMENSION, DISPLAY_NAME);

alter table SAHARA.BVIEW_SUBRULE_ELEMENT
   add constraint AK_RULEELEMENT_RULETOKEN unique (RULE_TOKEN_UPPER);

comment on index SAHARA.BVIEW_SUBRULE_ELEMENT.AK_RULEELEMENT_RULETOKEN is 
'The rule token must be unique within the context regardless of character case.';

grant SELECT on SAHARA.BVIEW_SUBRULE_ELEMENT to SAHARA_APP;

/*==============================================================*/
/* Table: CLUSTER                                               */
/*==============================================================*/
create table SAHARA.CLUSTER 
(
   CLUSTER_KEY          numeric(10)                    not null default autoincrement,
   CLUSTER_OID          varchar(128)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   CLUSTER_TYPE         varchar(1024)                  null,
   CLUSTER_CATEGORY     varchar(1024)                  null,
   VERSION              varchar(32)                    null,
   PLATFORM             varchar(1024)                  null,
   USE_FENCE            varchar(1)                     not null default 'F',
   IS_GLOBAL            varchar(1)                     not null default 'F',
   SECURECLUS           varchar(1)                     null default 'F',
   REMOTECLUSTERS       varchar(2048)                  null default '',
   T2_IS_CURRENT        varchar(1)                     not null default 'T',
   T2_IS_DELETED        varchar(1)                     not null default 'F',
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on table SAHARA.CLUSTER is 
'This is the dimension table for cluster.  It defines the attributes of cluster that may be used to qualify cluster queries.';

comment on column SAHARA.CLUSTER.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.CLUSTER.CLUSTER_TYPE is 
'Cluster types: VCS, MSCS, VMWare HA';

comment on column SAHARA.CLUSTER.CLUSTER_CATEGORY is 
'Category includes: Failover, Oracle RAC, CFS, CVM, Sybase CE.';

comment on column SAHARA.CLUSTER.PLATFORM is 
'details about operating system.';

comment on column SAHARA.CLUSTER.USE_FENCE is 
'Indicates whether the cluster uses SCSI-3 I/O fencing.';

comment on column SAHARA.CLUSTER.IS_GLOBAL is 
'cluster is a global cluster or local cluster';

comment on column SAHARA.CLUSTER.T2_IS_CURRENT is 
'Is a boolean book keeping flag that indicates if this is the "current" or latest record for the object.';

comment on column SAHARA.CLUSTER.T2_IS_DELETED is 
'a boolean book keeping flag that indicates if the dimension row should be considered deleted.';

comment on column SAHARA.CLUSTER.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA.CLUSTER.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA.CLUSTER.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA.CLUSTER.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA.CLUSTER
   alter USE_FENCE add constraint CKC_USE_FENCE_CLUSTER check (USE_FENCE in ('T','F'));

alter table SAHARA.CLUSTER
   alter IS_GLOBAL add constraint CKC_IS_GLOBAL_CLUSTER check (IS_GLOBAL in ('T','F'));

alter table SAHARA.CLUSTER
   alter SECURECLUS add constraint CKC_SECURECLUS_CLUSTER check (SECURECLUS is null or (SECURECLUS in ('T','F')));

alter table SAHARA.CLUSTER
   alter T2_IS_CURRENT add constraint CKC_T2_IS_CURRENT_CLUSTER check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.CLUSTER
   alter T2_IS_DELETED add constraint CKC_T2_IS_DELETED_CLUSTER check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.CLUSTER
   add constraint PK_CLUSTER primary key (CLUSTER_KEY);

/*==============================================================*/
/* Index: I_CLUSTER_IDENTIFIER                                  */
/*==============================================================*/
create index I_CLUSTER_IDENTIFIER on SAHARA.CLUSTER (
CLUSTER_OID ASC
);

comment on index SAHARA.CLUSTER.I_CLUSTER_IDENTIFIER is 
'An index on cluster identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_CLUSTER_DISPLAY_NAME                                */
/*==============================================================*/
create index I_CLUSTER_DISPLAY_NAME on SAHARA.CLUSTER (
DISPLAY_NAME ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_TYPE                                        */
/*==============================================================*/
create index I_CLUSTER_TYPE on SAHARA.CLUSTER (
CLUSTER_TYPE ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_CATEGORY                                    */
/*==============================================================*/
create index I_CLUSTER_CATEGORY on SAHARA.CLUSTER (
CLUSTER_CATEGORY ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_VERSION                                     */
/*==============================================================*/
create index I_CLUSTER_VERSION on SAHARA.CLUSTER (
VERSION ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_PLATFORM                                    */
/*==============================================================*/
create index I_CLUSTER_PLATFORM on SAHARA.CLUSTER (
PLATFORM ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_USE_FENCE                                   */
/*==============================================================*/
create index I_CLUSTER_USE_FENCE on SAHARA.CLUSTER (
USE_FENCE ASC
);

/*==============================================================*/
/* Index: I_IS_GLOBAL_INDEX                                     */
/*==============================================================*/
create index I_IS_GLOBAL_INDEX on SAHARA.CLUSTER (
IS_GLOBAL ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_SECURECLUS                                  */
/*==============================================================*/
create index I_CLUSTER_SECURECLUS on SAHARA.CLUSTER (
USE_FENCE ASC
);

/*==============================================================*/
/* Index: I_REMOTECLUSTERS_INDEX                                */
/*==============================================================*/
create index I_REMOTECLUSTERS_INDEX on SAHARA.CLUSTER (
REMOTECLUSTERS ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_T2ISCUR_A_CKEY                              */
/*==============================================================*/
create index I_CLUSTER_T2ISCUR_A_CKEY on SAHARA.CLUSTER (
CLUSTER_KEY ASC,
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_T2ISCURAT2ISDELACID                         */
/*==============================================================*/
create index I_CLUSTER_T2ISCURAT2ISDELACID on SAHARA.CLUSTER (
CLUSTER_OID ASC,
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_T2ISDELETED                                 */
/*==============================================================*/
create index I_CLUSTER_T2ISDELETED on SAHARA.CLUSTER (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.CLUSTER to SAHARA_APP;

/*==============================================================*/
/* Table: CLUSTER_                                              */
/*==============================================================*/
create table SAHARA_APP.CLUSTER_ 
(
   CLUSTER_OID          varchar(128)                   not null,
   DISPLAY_NAME         varchar(1024)                  null,
   CLUSTER_TYPE         varchar(1024)                  null,
   CLUSTER_CATEGORY     varchar(1024)                  null,
   VERSION              varchar(32)                    null,
   PLATFORM             varchar(1024)                  null,
   USE_FENCE            varchar(1)                     not null default 'F',
   IS_GLOBAL            varchar(1)                     null default 'F',
   SECURECLUS           varchar(1)                     null default 'F',
   REMOTECLUSTERS       varchar(2048)                  not null default '',
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on table SAHARA_APP.CLUSTER_ is 
'This is a staging table for loading data into warehouse table SAHARA.CLUSTER.  Any data in this table is temporary and should not be accessed except by the data loading process';

comment on column SAHARA_APP.CLUSTER_.CLUSTER_TYPE is 
'Cluster types: VCS, MSCS, VMWare HA';

comment on column SAHARA_APP.CLUSTER_.CLUSTER_CATEGORY is 
'Category includes: Failover, Oracle RAC, CFS, CVM, Sybase CE.';

comment on column SAHARA_APP.CLUSTER_.PLATFORM is 
'details about operating system.';

comment on column SAHARA_APP.CLUSTER_.USE_FENCE is 
'Indicates whether the cluster uses SCSI-3 I/O fencing.';

comment on column SAHARA_APP.CLUSTER_.IS_GLOBAL is 
'Cluster is a global or local cluster';

comment on column SAHARA_APP.CLUSTER_.REMOTECLUSTERS is 
'List of remote clusters names.';

comment on column SAHARA_APP.CLUSTER_.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA_APP.CLUSTER_.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA_APP.CLUSTER_.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA_APP.CLUSTER_.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA_APP.CLUSTER_
   alter USE_FENCE add constraint CKC_USE_FENCE_CLUSTER_ check (USE_FENCE in ('T','F'));

alter table SAHARA_APP.CLUSTER_
   alter IS_GLOBAL add constraint CKC_IS_GLOBAL_CLUSTER_ check (IS_GLOBAL is null or (IS_GLOBAL in ('T','F')));

alter table SAHARA_APP.CLUSTER_
   alter SECURECLUS add constraint CKC_SECURECLUS_CLUSTER_ check (SECURECLUS is null or (SECURECLUS in ('T','F')));

alter table SAHARA_APP.CLUSTER_
   add constraint PK_CLUSTER_ primary key (CLUSTER_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_ to SAHARA;

/*==============================================================*/
/* Table: CLUSTER_BVIEW_NODE                                    */
/*==============================================================*/
create table SAHARA.CLUSTER_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   CLUSTER_KEY          numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.CLUSTER_BVIEW_NODE is 
'This is a join table between cluster objects and business view nodes.';

comment on column SAHARA.CLUSTER_BVIEW_NODE.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.CLUSTER_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint PK_CLUSTER_BVIEWNODE primary key (BVIEW_NODE_KEY, CLUSTER_KEY);

/*==============================================================*/
/* Index: I_CLUSTER_BVIEWNODE_BVIEWKEY                          */
/*==============================================================*/
create index I_CLUSTER_BVIEWNODE_BVIEWKEY on SAHARA.CLUSTER_BVIEW_NODE (
BVIEW_NODE_KEY ASC,
CLUSTER_KEY ASC,
BVIEW_KEY ASC
);

comment on index SAHARA.CLUSTER_BVIEW_NODE.I_CLUSTER_BVIEWNODE_BVIEWKEY is 
'An index on Business View Key for dimensional queries.';

grant SELECT on SAHARA.CLUSTER_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: CLUSTER_HOST                                          */
/*==============================================================*/
create table SAHARA.CLUSTER_HOST 
(
   CLUSTER_HOST_KEY     numeric(10)                    not null default autoincrement,
   CLUSTER_KEY          numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   CLUSTER_HOST_OID     varchar(1024)                  not null,
   CLUSTER_OID          varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.CLUSTER_HOST is 
'A join table for forming one to many relationship between cluster and hosts.';

comment on column SAHARA.CLUSTER_HOST.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.CLUSTER_HOST.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.CLUSTER_HOST.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.CLUSTER_HOST
   alter T2_IS_CURRENT add constraint C_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.CLUSTER_HOST
   alter T2_IS_DELETED add constraint C_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.CLUSTER_HOST
   add constraint PK_CLUSTER_HOST primary key clustered (CLUSTER_HOST_KEY);

/*==============================================================*/
/* Index: I_CLUSTERHOST_CLUSTERKEY                              */
/*==============================================================*/
create index I_CLUSTERHOST_CLUSTERKEY on SAHARA.CLUSTER_HOST (
CLUSTER_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERHOST_HOSTKEY                                 */
/*==============================================================*/
create index I_CLUSTERHOST_HOSTKEY on SAHARA.CLUSTER_HOST (
HOST_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERHOST_T2ISCURRENT                             */
/*==============================================================*/
create index I_CLUSTERHOST_T2ISCURRENT on SAHARA.CLUSTER_HOST (
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_HOST_T2ISDELETED                            */
/*==============================================================*/
create index I_CLUSTER_HOST_T2ISDELETED on SAHARA.CLUSTER_HOST (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.CLUSTER_HOST to SAHARA_APP;

/*==============================================================*/
/* Table: CLUSTER_HOST_                                         */
/*==============================================================*/
create table SAHARA_APP.CLUSTER_HOST_ 
(
   CLUSTER_HOST_OID     varchar(1024)                  not null,
   CLUSTER_OID          varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   DISPLAY_NAME         varchar(1024)                  null
);

alter table SAHARA_APP.CLUSTER_HOST_
   add constraint PK_CLUSTERHOST_ primary key clustered (CLUSTER_HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_HOST_ to SAHARA;

/*==============================================================*/
/* Table: CLUSTER_INVENTORY                                     */
/*==============================================================*/
create table SAHARA.CLUSTER_INVENTORY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   CLUSTER_KEY          numeric(10)                    not null,
   NUMBER_OF_NODE       integer                        null default 0,
   NUMBER_OF_SERVICEGROUP integer                        null default 0,
   NUMBER_OF_RESOURCE   integer                        null default 0
);

comment on table SAHARA.CLUSTER_INVENTORY is 
'Contains number of nodes, servicegroups,resources, and allocated capacity';

comment on column SAHARA.CLUSTER_INVENTORY.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.CLUSTER_INVENTORY.DATES_KEY is 
'A unique database generated identifier for the date record used as a surrogate key.';

alter table SAHARA.CLUSTER_INVENTORY
   alter DATES_KEY add constraint CKC_DATES_KEY_CLUSTER_ check (DATES_KEY >= 1);

alter table SAHARA.CLUSTER_INVENTORY
   alter NUMBER_OF_NODE add constraint CKC_NUMBER_OF_NODE_CLUSTER_ check (NUMBER_OF_NODE is null or (NUMBER_OF_NODE >= 0));

alter table SAHARA.CLUSTER_INVENTORY
   alter NUMBER_OF_SERVICEGROUP add constraint CKC_NUMBER_OF_SERVICE_CLUSTER_ check (NUMBER_OF_SERVICEGROUP is null or (NUMBER_OF_SERVICEGROUP >= 0));

alter table SAHARA.CLUSTER_INVENTORY
   alter NUMBER_OF_RESOURCE add constraint CKC_NUMBER_OF_RESOURC_CLUSTER_ check (NUMBER_OF_RESOURCE is null or (NUMBER_OF_RESOURCE >= 0));

/*==============================================================*/
/* Index: I_CLUSTERINV_ROLLUPBATCH                              */
/*==============================================================*/
create index I_CLUSTERINV_ROLLUPBATCH on SAHARA.CLUSTER_INVENTORY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERINV_DATESKEY                                 */
/*==============================================================*/
create index I_CLUSTERINV_DATESKEY on SAHARA.CLUSTER_INVENTORY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERINV_CLUSTERKEY                               */
/*==============================================================*/
create index I_CLUSTERINV_CLUSTERKEY on SAHARA.CLUSTER_INVENTORY (
CLUSTER_KEY ASC
);

grant SELECT on SAHARA.CLUSTER_INVENTORY to SAHARA_APP;

/*==============================================================*/
/* Table: CLUSTER_INVENTORY_                                    */
/*==============================================================*/
create table SAHARA_APP.CLUSTER_INVENTORY_ 
(
   CLUSTER_OID          varchar(128)                   not null,
   NUMBER_OF_NODE       integer                        not null default 0,
   NUMBER_OF_SERVICEGROUP integer                        not null default 0,
   NUMBER_OF_RESOURCE   integer                        not null default 0
);

comment on table SAHARA_APP.CLUSTER_INVENTORY_ is 
'This is a staging table for loading data into warehouse table sahara.cluster_inventory.  Any data in this table is temporary and should not be accessed except by the data loading process.';

alter table SAHARA_APP.CLUSTER_INVENTORY_
   alter NUMBER_OF_NODE add constraint CKC_NUMBER_OF_NODE_CLUSTER_ check (NUMBER_OF_NODE >= 0);

alter table SAHARA_APP.CLUSTER_INVENTORY_
   alter NUMBER_OF_SERVICEGROUP add constraint CKC_NUMBER_OF_SERVICE_CLUSTER_ check (NUMBER_OF_SERVICEGROUP >= 0);

alter table SAHARA_APP.CLUSTER_INVENTORY_
   alter NUMBER_OF_RESOURCE add constraint CKC_NUMBER_OF_RESOURC_CLUSTER_ check (NUMBER_OF_RESOURCE >= 0);

alter table SAHARA_APP.CLUSTER_INVENTORY_
   add constraint PK_CLUSTERINV_CLUSTERID_ primary key (CLUSTER_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_INVENTORY_ to SAHARA;

/*==============================================================*/
/* Table: CLUSTER_REMOTECLUSTER                                 */
/*==============================================================*/
create table SAHARA.CLUSTER_REMOTECLUSTER 
(
   CLUSTER_REMOTECLUSTER_KEY numeric(10)                    not null default autoincrement,
   CLUSTER_KEY          numeric(10)                    not null,
   CLUSTER_REMOTECLUSTER_OID varchar(512)                   not null,
   CLUSTER_OID          varchar(128)                   not null,
   REMOTECLUSTER_OID    varchar(128)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.CLUSTER_REMOTECLUSTER is 
'A join table for forming one to many relationship between local cluster and remote clusters. We treat this as a fact to deal with relationship comfing and going with each batch.';

comment on column SAHARA.CLUSTER_REMOTECLUSTER.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.CLUSTER_REMOTECLUSTER.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.CLUSTER_REMOTECLUSTER.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.CLUSTER_REMOTECLUSTER
   alter T2_IS_CURRENT add constraint C_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.CLUSTER_REMOTECLUSTER
   alter T2_IS_DELETED add constraint C_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.CLUSTER_REMOTECLUSTER
   add constraint PK_CLUSTER_REMOTECLUSTER primary key clustered (CLUSTER_REMOTECLUSTER_KEY);

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_CLUSTERKEY                          */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_CLUSTERKEY on SAHARA.CLUSTER_REMOTECLUSTER (
CLUSTER_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_RCLUSTERID                          */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_RCLUSTERID on SAHARA.CLUSTER_REMOTECLUSTER (
REMOTECLUSTER_OID ASC
);

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_T2ISCURRENT                         */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_T2ISCURRENT on SAHARA.CLUSTER_REMOTECLUSTER (
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_T2ISDELETED                         */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_T2ISDELETED on SAHARA.CLUSTER_REMOTECLUSTER (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.CLUSTER_REMOTECLUSTER to SAHARA_APP;

/*==============================================================*/
/* Table: CLUSTER_REMOTECLUSTER_                                */
/*==============================================================*/
create table SAHARA_APP.CLUSTER_REMOTECLUSTER_ 
(
   CLUSTER_REMOTECLUSTER_OID varchar(512)                   not null,
   CLUSTER_OID          varchar(128)                   not null,
   REMOTECLUSTER_OID    varchar(128)                   not null,
   DISPLAY_NAME         varchar(1024)                  null
);

comment on table SAHARA_APP.CLUSTER_REMOTECLUSTER_ is 
'This is a staging table for loading data into warehouse table sahara.cluster_remotecluster.  Any data in this table is temporary and should be accessed except by the data loading process.';

alter table SAHARA_APP.CLUSTER_REMOTECLUSTER_
   add constraint PK_CLUSTERRCLUSTER_ primary key clustered (CLUSTER_REMOTECLUSTER_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_REMOTECLUSTER_ to SAHARA;

/*==============================================================*/
/* Table: COGNOS_FACT_MAPPING                                   */
/*==============================================================*/
create table SAHARA.COGNOS_FACT_MAPPING 
(
   OBJECT_TYPE          varchar(32)                    not null,
   NAMESPACE            varchar(256)                   not null,
   FACT_DIMENSION_NAME  varchar(256)                   not null,
   FACT_NAME            varchar(256)                   not null
);

comment on table SAHARA.COGNOS_FACT_MAPPING is 
'This table contains a list of all the underlying measures defined in the Cognos metamodel for each of the supported object types.  This information is used during a metamodel update operation to make sure any new dimensions (e.g. business views) that support the Enterprise Reporter Object type are associated properly with all of the measures supported by this type.  If this mapping does not happen then the user will get an error when creating a report indicating there''s no association between the hierarchy/object and the fact they''re trying to report against.

This table is part of configuration stored in the database that is used to drive Cognos metamodel objects.  The idea is to minimize required code changes as the metamodel evolves.  It is populated at installation/upgrade time.

For example, if we add a new business view that supports object types, then the update_metamodel script needs to create actions that will map this new business view dimension to the "File System Capacity (GB)" measure in the File System Analysis namespace.

IMPORTANT: If you make material changes to the metamodel that changes these relationships in any way (e.g. renaming a namespace, adding/removing measures), then you MUST update this configuration information.  Failure to do so will result in possible broken reports.';

comment on column SAHARA.COGNOS_FACT_MAPPING.OBJECT_TYPE is 
'The underlying Enterprise Reporter object type that supports the measure.';

comment on column SAHARA.COGNOS_FACT_MAPPING.NAMESPACE is 
'The namespace to which the measure belongs.';

comment on column SAHARA.COGNOS_FACT_MAPPING.FACT_DIMENSION_NAME is 
'The name of the dimension to which underlying fact belongs.';

comment on column SAHARA.COGNOS_FACT_MAPPING.FACT_NAME is 
'The name of the fact to which map the object type.';

alter table SAHARA.COGNOS_FACT_MAPPING
   add constraint PK_COGNOS_FACT_MAPPING primary key (OBJECT_TYPE, NAMESPACE, FACT_DIMENSION_NAME, FACT_NAME);

grant SELECT on SAHARA.COGNOS_FACT_MAPPING to SAHARA_APP;

/*==============================================================*/
/* Table: COGNOS_HIERARCHY_ATTRIBUTE                            */
/*==============================================================*/
create table SAHARA.COGNOS_HIERARCHY_ATTRIBUTE 
(
   CUSTOM_ATTRIBUTE_CONTEXT varchar(32)                    not null,
   NAMESPACE            varchar(256)                   not null,
   DIMENSION_NAME       varchar(256)                   not null,
   HIERARCHY_NAME       varchar(256)                   not null,
   LEVEL_NAME           varchar(256)                   not null
);

comment on table SAHARA.COGNOS_HIERARCHY_ATTRIBUTE is 
'Location of object attributes based on type.  These values are used to drive the addition and deletion of custom attributes from each hierarchy in the Cognos metamodel.

This table is part of configuration stored in the database that is used to drive Cognos metamodel objects.  The idea is to minimize required code changes as the metamodel evolves.  It is populated at installation/upgrade time.

IMPORTANT: If you make material changes to the metamodel that changes these relationships in any way (e.g. renaming a namespace, adding/removing dimensions), then you MUST update this configuration information.  Failure to do so will result in possible broken reports.';

alter table SAHARA.COGNOS_HIERARCHY_ATTRIBUTE
   add constraint PK_COGNOS_HIERARCHY_ATTRIBUTE primary key (CUSTOM_ATTRIBUTE_CONTEXT, NAMESPACE, DIMENSION_NAME, HIERARCHY_NAME, LEVEL_NAME);

grant SELECT on SAHARA.COGNOS_HIERARCHY_ATTRIBUTE to SAHARA_APP;

/*==============================================================*/
/* Table: COGNOS_MODEL_UPDATE                                   */
/*==============================================================*/
create table SAHARA.COGNOS_MODEL_UPDATE 
(
   COGNOS_MODEL_UPDATE_KEY numeric(10)                    not null default autoincrement,
   TYPE                 varchar(32)                    not null,
   QUEUED_TIME          datetime                       not null,
   ATTEMPTS             numeric(2)                     not null default 0,
   ARG_1                varchar(4000)                  not null,
   ARG_2                varchar(4000)                  not null,
   ARG_3                varchar(4000)                  not null,
   ARG_4                varchar(4000)                  null,
   ARG_5                varchar(4000)                  null,
   ARG_6                varchar(4000)                  null,
   ARG_7                varchar(4000)                  not null,
   ARG_8                varchar(4000)                  null,
   ARG_9                varchar(4000)                  null,
   UPDATE_START_TIME    datetime                       null,
   UPDATE_COMPLETION_TIME datetime                       null,
   STATUS               varchar(32)                    not null default 'Pending',
   STATUS_MESSAGE       long varchar                   null
);

comment on table SAHARA.COGNOS_MODEL_UPDATE is 
'This table contains an ordered list of Cognos metamodel changes resulting from user configuration changes.  The order is defined by the column COGNOS_MODEL_UPDATE_KEY.  The table serves two purposes:

1) When a configuration change that requires a parallel Cognos metamodel change, is processed and applied to the sahara schema, a row is created in this table.  In that capacity, the row represents queued work that needs to be applied to the Cognos metamodel.

2) When the metamodel update is processed, the row is updated.  This provides logging and history for the metamodel updates.

The table may also be used to re-apply changes during upgrade and/or after disaster recovery.

This table is not available for reporting or the SAHARA_READ role because it may contain password.

The table contains a list of "argument" columns whose meaning is overloaded, depending on the type of Cognos metamodel change that the table row represents.  The type is represented by the TYPE column.  Below is an explanation of the meaning of the argument columns for each of the Cognos metamodel update types.

For both types CUSTOM ATTRIBUTE ADD and CUSTOM ATTRIBUTE DELETE, the arguments correspond to the columns for the CUSTOM_ATTRIBUTE/P_CUSTOM_ATTRIBUTE table of the added/deleted attribute as follows:
- ARG_1: CUSTOM_ATTRIBUTE_KEY
- ARG_2: CUSTOM_ATTRIBUTE_CONTEXT
- ARG_3: TYPE
- ARG_4: DISPLAY_NAME
- ARG_5: COLUMN_NAME
- ARG_6: COLUMN_TYPE
- ARG_7: SOURCE_NAME

For type SET SMTP:
- ARG_1: the SMTP server name or IP address
- ARG_2: the SMTP server port number
- ARG_3: the sender email address
- ARG_4: the SMTP user name (or null)
- ARG_5: the SMTP use password (or null)';

comment on column SAHARA.COGNOS_MODEL_UPDATE.COGNOS_MODEL_UPDATE_KEY is 
'A unique database generated identifier for the Cognos metamodel update as well as a value to establish the order in which changes occur.  Each metamodel update gets a key value of increasing numeric value.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.TYPE is 
'The type of Cognos metamodel update.  See the table description for an explanation of the update types and their associated arguments.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.QUEUED_TIME is 
'The date and time, stored in UTC, at which the update was added to the table.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ATTEMPTS is 
'The number of times that an attempt has been made to apply the update to the Cognos metamodel.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_1 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_2 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_3 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_4 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_5 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_6 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_7 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_8 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.ARG_9 is 
'The meaning of this column varies depending on the value of the TYPE column in the row.  See the table description for details.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.UPDATE_START_TIME is 
'The date and time, stored in UTC, at which the update to the Cognos metamodel was begun.  If there are multiple attempts at applying the update, this is the time at which the most recent update attempt was begun.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.UPDATE_COMPLETION_TIME is 
'The date and time, stored in UTC, at which the update to the Cognos metamodel was completed.  If there are multiple attempts at applying the update, this is the time at which the most recent update attempt was completed.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.STATUS is 
'The status of the update to the Cognos metamodel.

Poissble values:
- Pending - the update has been queued and is awaiting processing.
- Processing - processing of the update is occuring now.
- Failed - the update has failed and will not be retried.  If an update attempt fails, but should be retried, it is reset to Pending.
- Succeeded - the update has been successfully applied to the Cognos metamodel.';

comment on column SAHARA.COGNOS_MODEL_UPDATE.STATUS_MESSAGE is 
'An optional text message that accompany the status.  Multiple messages may accumulate in this column throughout the processing for the update.';

alter table SAHARA.COGNOS_MODEL_UPDATE
   alter TYPE add constraint C_CGNSMDLPDT_TYPE check (TYPE in ('CUSTOM ATTRIBUTE ADD','CUSTOM ATTRIBUTE DELETE','SET SMTP'));

alter table SAHARA.COGNOS_MODEL_UPDATE
   alter STATUS add constraint C_CGNSMDLPDT_STATUS check (STATUS in ('Pending','Processing','Failed','Succeeded'));

alter table SAHARA.COGNOS_MODEL_UPDATE
   add constraint PK_COGNOS_MODEL_UPDATE primary key (TYPE, ARG_1, ARG_2, ARG_3, ARG_7);

grant SELECT on SAHARA.COGNOS_MODEL_UPDATE to SAHARA_APP;

/*==============================================================*/
/* Table: COGNOS_PACKAGE                                        */
/*==============================================================*/
create table SAHARA.COGNOS_PACKAGE 
(
   PACKAGE_NAME         varchar(256)                   not null
);

comment on table SAHARA.COGNOS_PACKAGE is 
'This is a complete list of all the packages we publish in the Cognos content store.  This is used during installation to publish the packages needed by reporting.  It is also used during metamodel updates to publish the updated metamodel information.

This table is part of configuration stored in the database that is used to drive Cognos metamodel objects.  The idea is to minimize required code changes as the metamodel evolves.  It is populated at installation/upgrade time.';

comment on column SAHARA.COGNOS_PACKAGE.PACKAGE_NAME is 
'The name of the package.';

alter table SAHARA.COGNOS_PACKAGE
   add constraint PK_COGNOS_PACKAGE primary key (PACKAGE_NAME);

grant SELECT on SAHARA.COGNOS_PACKAGE to SAHARA_APP;

/*==============================================================*/
/* Table: COGNOS_PRESENTATION_SHORTCUT                          */
/*==============================================================*/
create table SAHARA.COGNOS_PRESENTATION_SHORTCUT 
(
   OBJECT_TYPE          varchar(32)                    not null,
   NAMESPACE            varchar(256)                   not null,
   FOLDER_NAME          varchar(256)                   not null
);

comment on table SAHARA.COGNOS_PRESENTATION_SHORTCUT is 
'This table contains information needed to support business views.  The underlying business view dimension itself is created in a type-specific folder in the ''Presentation View'' namespace of the metamodel.  For each Enterprise Reporter object type the business view supports, we''ll create a shortcut to this business view dimensional hierarchy in one or more folders in the metamodel.  This, in effect, makes this dimensional hierarchy available to the end user for reporting.

This table is part of configuration stored in the database that is used to drive Cognos metamodel objects.  The idea is to minimize required code changes as the metamodel evolves.  It is populated at installation/upgrade time.

For example, if we create a new business view that supports HOST objects, we''ll create the business view hierarchy in the "Host Business Views" folder of the "Presentation View" namespace, and we''ll create shortcuts to this hierarchy in a number of folders, including the "Host Consumption Views" folder in the "Host Consumption" namespace of the metamodel.

IMPORTANT: If you make material changes to the metamodel that changes these relationships in any way (e.g. renaming a namespace, adding/removing folders), then you MUST update this configuration information.  Failure to do so will result in possible broken reports.';

alter table SAHARA.COGNOS_PRESENTATION_SHORTCUT
   add constraint PK_COGNOSPRESENTATIONSHORTCUT primary key (OBJECT_TYPE, NAMESPACE, FOLDER_NAME);

grant SELECT on SAHARA.COGNOS_PRESENTATION_SHORTCUT to SAHARA_APP;

/*==============================================================*/
/* Table: COGNOS_RELATIONSHIP                                   */
/*==============================================================*/
create table SAHARA.COGNOS_RELATIONSHIP 
(
   LHS_OBJECT_TYPE      varchar(32)                    not null,
   RHS_TABLE            varchar(30)                    not null,
   LHS_CARDINALITY_A    varchar(32)                    not null,
   LHS_CARDINALITY_B    varchar(32)                    not null,
   RHS_CARDINALITY_A    varchar(32)                    not null,
   RHS_CARDINALITY_B    varchar(32)                    not null
);

comment on table SAHARA.COGNOS_RELATIONSHIP is 
'This table contains configuration information in support of business views.  For each object type supported by the business view, we must map each of the these object types to the underlying database view query subject for that type.  This table describes this relationship and the cardinality on the left hand side (LHS) and right hand side (RHS) of the relationship.

This table is part of configuration stored in the database that is used to drive Cognos metamodel objects.  The idea is to minimize required code changes as the metamodel evolves.  It is populated at installation/upgrade time.

For example, if we create a business view that supports HOST objects, then we create a relationship between this new business view and the HOST table.';

alter table SAHARA.COGNOS_RELATIONSHIP
   alter RHS_TABLE add constraint C_CGNSRLTNSHP_RHSTABLE check (RHS_TABLE = upper(RHS_TABLE));

alter table SAHARA.COGNOS_RELATIONSHIP
   add constraint PK_COGNOS_RELATIONSHIP primary key (LHS_OBJECT_TYPE, RHS_TABLE);

grant SELECT on SAHARA.COGNOS_RELATIONSHIP to SAHARA_APP;

/*==============================================================*/
/* Table: COGNOS_SMTP                                           */
/*==============================================================*/
create table SAHARA.COGNOS_SMTP 
(
   COGNOS_SMTP_KEY      char(3)                        not null,
   SERVER               varchar(256)                   not null,
   PORT                 numeric(5)                     not null,
   SENDER               varchar(256)                   not null,
   "USER"               varchar(256)                   null,
   PASSWORD             varchar(256)                   null
);

comment on table SAHARA.COGNOS_SMTP is 
'This table contains the SMTP configuration used to send email from Cognos.  A separate table is used rather that the generic configuration table because the table contains a password and must be protected.  This table will have exactly one row.

This table should be updated only via COGNOS_MODEL_UPDATE_API.EnQSetSMTP.

This table is not available for reporting or the SAHARA_READ role because it contains a password.';

comment on column SAHARA.COGNOS_SMTP.COGNOS_SMTP_KEY is 
'An artificial primary key that allows only one value.  This is used to limit the table to exactly one row.';

comment on column SAHARA.COGNOS_SMTP.SERVER is 
'The name or IP address of the SMTP server to use for sending mail.';

comment on column SAHARA.COGNOS_SMTP.PORT is 
'The port number of the SMTP server to use for sending mail.';

alter table SAHARA.COGNOS_SMTP
   alter COGNOS_SMTP_KEY add constraint C_COGNOSSMTP_COGNOSSMTPKEY check (COGNOS_SMTP_KEY in ('KEY'));

alter table SAHARA.COGNOS_SMTP
   alter PORT add constraint C_COGNOSSMTP_PORT check (PORT >= 1);

alter table SAHARA.COGNOS_SMTP
   add constraint PK_COGNOS_SMTP primary key (COGNOS_SMTP_KEY);

grant SELECT on SAHARA.COGNOS_SMTP to SAHARA_APP;

/*==============================================================*/
/* Table: COLUMN_METADATA_BASE                                  */
/*==============================================================*/
create table SAHARA.COLUMN_METADATA_BASE 
(
   OWNER                varchar(30)                    not null,
   TABLE_NAME           varchar(30)                    not null,
   COLUMN_NAME          varchar(30)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   IS_FORECAST_ENABLE   char(1)                        null default 'F',
   AGGREGATE_TYPE       varchar(16)                    null default 'summarization'
);

comment on table SAHARA.COLUMN_METADATA_BASE is 
'This table contains metadata for columns of tables and views in the sahara schema.  The contents are generated out of PowerDesigner in which the schema is modeled.

This table should not be queried directly.  Instead, use database view COLUMN_METADATA.

This table is populated at install and upgrade.';

comment on column SAHARA.COLUMN_METADATA_BASE.OWNER is 
'The name of the owner of the table or view in which the column exists.';

comment on column SAHARA.COLUMN_METADATA_BASE.TABLE_NAME is 
'The name of the table or view in which the column exists.';

comment on column SAHARA.COLUMN_METADATA_BASE.COLUMN_NAME is 
'The name of the column.';

comment on column SAHARA.COLUMN_METADATA_BASE.DISPLAY_NAME is 
'A user friendly display name for the column.';

comment on column SAHARA.COLUMN_METADATA_BASE.IS_FORECAST_ENABLE is 
'A boolean flag that indicates if the dimension or fact is enabled for forecasting.

Possible values:
''T'' - the dimension/fact is enabled for forecasting
''F'' - the dimension/fact is not enabled for forecasting';

alter table SAHARA.COLUMN_METADATA_BASE
   alter OWNER add constraint C_CLMNMTDTBS_OWNER check (OWNER = upper(OWNER));

alter table SAHARA.COLUMN_METADATA_BASE
   alter TABLE_NAME add constraint C_CLMNMTDTBS_TABLENAME check (TABLE_NAME = upper(TABLE_NAME));

alter table SAHARA.COLUMN_METADATA_BASE
   alter COLUMN_NAME add constraint C_CLMNMTDTBS_COLUMNNAME check (COLUMN_NAME = upper(COLUMN_NAME));

alter table SAHARA.COLUMN_METADATA_BASE
   alter IS_FORECAST_ENABLE add constraint C_CLMNMTDTBS_ISFORCSTENABLED check (IS_FORECAST_ENABLE is null or (IS_FORECAST_ENABLE in ('T','F')));

alter table SAHARA.COLUMN_METADATA_BASE
   add constraint PK_COLUMN_METADATA_BASE primary key (OWNER, TABLE_NAME, COLUMN_NAME);

grant SELECT on SAHARA.COLUMN_METADATA_BASE to SAHARA_APP;

/*==============================================================*/
/* Table: CONFIGURATION                                         */
/*==============================================================*/
create table SAHARA.CONFIGURATION 
(
   SECTION              varchar(32)                    not null,
   NAME                 varchar(32)                    not null,
   TYPE                 varchar(32)                    not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DESCRIPTION          long varchar                   not null,
   VISIBLE              char(1)                        not null default 'T',
   EDITABLE             char(1)                        not null default 'T',
   STRING_VALUE         varchar(2000)                  null,
   INTEGER_VALUE        numeric(10)                    null,
   BOOLEAN_VALUE        char(1)                        null,
   DATE_VALUE           datetime                       null
);

comment on table SAHARA.CONFIGURATION is 
'This table contains general purpose configuration.  It is modeled after an INI file with each configuration item having a name and belonging to a section.';

comment on column SAHARA.CONFIGURATION.SECTION is 
'The name of the section.  Only upper case characters are allowed.';

comment on column SAHARA.CONFIGURATION.NAME is 
'The name of the configuration item within the section.  Only upper case characters are allowed.';

comment on column SAHARA.CONFIGURATION.TYPE is 
'The type of the configuration item.

Valid values are:
- STRING
- INTEGER
- BOOLEAN
- DATE';

comment on column SAHARA.CONFIGURATION.DISPLAY_NAME is 
'A name for the configuration item used by the GUI for providing editing for configuration.  The display name is used for a summary listing of multiple items.';

comment on column SAHARA.CONFIGURATION.DESCRIPTION is 
'A description for the configuration item used by the GUI for providing editing for configuration.  The description is used for a detailed description when editing a single item''s value.  The description should include a description of acceptable values for the configuration item.';

comment on column SAHARA.CONFIGURATION.VISIBLE is 
'A boolean flag that indicates if the GUI should display this configuration item.';

comment on column SAHARA.CONFIGURATION.EDITABLE is 
'A boolean flag that indicates if the GUI should allow editing of this configuration item.';

comment on column SAHARA.CONFIGURATION.STRING_VALUE is 
'The value of a string configuration item.';

comment on column SAHARA.CONFIGURATION.INTEGER_VALUE is 
'The value of an integer configuration item.';

comment on column SAHARA.CONFIGURATION.BOOLEAN_VALUE is 
'The value of a boolean configuration item.';

comment on column SAHARA.CONFIGURATION.DATE_VALUE is 
'The value of a date configuration item.';

alter table SAHARA.CONFIGURATION
    add constraint C_CONFIGURATION check ((
    ("TYPE" = 'STRING'
        and STRING_VALUE is not null
        and INTEGER_VALUE is null
        and BOOLEAN_VALUE is null
        and DATE_VALUE is null)
    or
    ("TYPE" = 'INTEGER'
        and STRING_VALUE is null
        and INTEGER_VALUE is not null
        and BOOLEAN_VALUE is null
        and DATE_VALUE is null)
    or
    ("TYPE" = 'BOOLEAN'
        and STRING_VALUE is null
        and INTEGER_VALUE is null
        and BOOLEAN_VALUE is not null
        and DATE_VALUE is null)
    or
    ("TYPE" = 'DATE'
        and STRING_VALUE is null
        and INTEGER_VALUE is null
        and BOOLEAN_VALUE is null
        and DATE_VALUE is not null)
 ));

alter table SAHARA.CONFIGURATION
   alter SECTION add constraint C_CONFIGURATION_SECTION check (SECTION = upper(SECTION));

alter table SAHARA.CONFIGURATION
   alter NAME add constraint C_CONFIGURATION_NAME check (NAME = upper(NAME));

alter table SAHARA.CONFIGURATION
   alter TYPE add constraint C_CONFIGURATION_TYPE check (TYPE in ('STRING','INTEGER','BOOLEAN','DATE'));

alter table SAHARA.CONFIGURATION
   alter VISIBLE add constraint C_CONFIGURATION_VISIBLE check (VISIBLE in ('T','F'));

alter table SAHARA.CONFIGURATION
   alter EDITABLE add constraint C_CONFIGURATION_EDITABLE check (EDITABLE in ('T','F'));

alter table SAHARA.CONFIGURATION
   alter BOOLEAN_VALUE add constraint C_CONFIGURATION_BOOLEANVALUE check (BOOLEAN_VALUE is null or (BOOLEAN_VALUE in ('T','F')));

alter table SAHARA.CONFIGURATION
   add constraint PK_CONFIGURATION primary key (SECTION, NAME);

grant SELECT on SAHARA.CONFIGURATION to SAHARA_APP;

/*==============================================================*/
/* Table: CONTROL_OWNER                                         */
/*==============================================================*/
create table SAHARA.CONTROL_OWNER 
(
   NAME                 varchar(32)                    not null,
   SESSION_KEY          numeric(10)                    null,
   ACQUIRED             timestamp                      null,
   RELEASED             timestamp                      null,
   ISLOCKED             bit                            not null default 0
);

comment on table SAHARA.CONTROL_OWNER is 
'This table records what session currently owns (or in some error situations, most recently owned) the exclusive application controls.
The possible modules controlled by this are: 

    - SAHARA_CONFIGURATION_CONTROL
    - SAHARA_IE_CONTROL
    - SAHARA_LOAD_CONTROL

The table is also used to track the lock held on the module through the IsLocked column. The sql file saharaLockAPI.sql implements two stored procedures called LockSession() and UnlockSession() that provide lock/unlock fuctionality to the modules. See the file for more details.
';

comment on column SAHARA.CONTROL_OWNER.NAME is 
'The name of the control.  Each package that utilizes a control makes the control name available as a constant.';

comment on column SAHARA.CONTROL_OWNER.SESSION_KEY is 
'The unique identifier of the session the owns the control.';

comment on column SAHARA.CONTROL_OWNER.ACQUIRED is 
'The timestamp (stored in UTC) at which the control was last acquired.';

comment on column SAHARA.CONTROL_OWNER.RELEASED is 
'The timestamp (stored in UTC) at which the control was last released.';

alter table SAHARA.CONTROL_OWNER
   add constraint PK_CONTROL_OWNER primary key (NAME);

grant SELECT on SAHARA.CONTROL_OWNER to SAHARA_APP;

/*==============================================================*/
/* Table: CUSTOM_ATTRIBUTE                                      */
/*==============================================================*/
create table SAHARA.CUSTOM_ATTRIBUTE 
(
   CUSTOM_ATTRIBUTE_KEY numeric(10)                    not null default autoincrement,
   CUSTOM_ATTRIBUTE_CONTEXT varchar(32)                    not null,
   TYPE                 varchar(32)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   COLUMN_NAME          varchar(30)                    not null,
   COLUMN_TYPE          varchar(64)                    not null,
   SOURCE_NAME          varchar(256)                   null
);

comment on table SAHARA.CUSTOM_ATTRIBUTE is 
'This table contains the editable configuration for custom attributes.

Custom attribute configuration is performed using this edit table, but actual processing of custom attributes for the loading of data is performed from the "published" table, P_CUSTOM_ATTRIBUTE.  We call the act of moving configuration from the edit table to the publish table, and making the associated database schema changes, publishing.

This allows configuration editing to take place free of concurrency constraints and publishing to be postponed until access to all of the required resources is available.';

comment on column SAHARA.CUSTOM_ATTRIBUTE.CUSTOM_ATTRIBUTE_KEY is 
'A unique database generated identifier for an Enterprise Reporter custom attribute.  The value really serves no other purpose other than to make the tier API easier to use by providing a simple way to identify a custom attribute.';

comment on column SAHARA.CUSTOM_ATTRIBUTE.CUSTOM_ATTRIBUTE_CONTEXT is 
'The context in which the source custom attributes are used.';

comment on column SAHARA.CUSTOM_ATTRIBUTE.TYPE is 
'The type of Enterprise Reporter custom attribute.  This defines the mechanism used to calculate the custom attribute value.

Valid values:
- SOURCE COPY - an Enterprise Reporter custom attribute that is a simple copy from a roll-up source custom attribute';

comment on column SAHARA.CUSTOM_ATTRIBUTE.COLUMN_TYPE is 
'The column data type of the Enterprise Reporter custom attribute column.';

comment on column SAHARA.CUSTOM_ATTRIBUTE.SOURCE_NAME is 
'If the custom attribute has type SOURCE COPY, this is the name of the source attribute.';

alter table SAHARA.CUSTOM_ATTRIBUTE
    add constraint C_CUSTOM_ATTRIBUTE check ((TYPE <> 'SOURCE COPY' or TYPE is not null));

alter table SAHARA.CUSTOM_ATTRIBUTE
   alter TYPE add constraint C_CSTMTTRBT_TYPE check (TYPE in ('SOURCE COPY'));

alter table SAHARA.CUSTOM_ATTRIBUTE
   alter COLUMN_NAME add constraint C_CSTMTTRBT_COLUMNNAME check (COLUMN_NAME = upper(COLUMN_NAME));

alter table SAHARA.CUSTOM_ATTRIBUTE
   add constraint PK_CUSTOM_ATTRIBUTE primary key (CUSTOM_ATTRIBUTE_KEY);

alter table SAHARA.CUSTOM_ATTRIBUTE
   add constraint AK_CSTMATTR_CNTEXTDISPLAYNAME unique (CUSTOM_ATTRIBUTE_CONTEXT, DISPLAY_NAME);

comment on index SAHARA.CUSTOM_ATTRIBUTE.AK_CSTMATTR_CNTEXTDISPLAYNAME is 
'The display name must be unique within a context.';

alter table SAHARA.CUSTOM_ATTRIBUTE
   add constraint AK_CSTMATTR_CNTEXTCOLNNAME unique (CUSTOM_ATTRIBUTE_CONTEXT, COLUMN_NAME);

comment on index SAHARA.CUSTOM_ATTRIBUTE.AK_CSTMATTR_CNTEXTCOLNNAME is 
'The column name must be unique within a context.';

grant SELECT on SAHARA.CUSTOM_ATTRIBUTE to SAHARA_APP;

/*==============================================================*/
/* Table: CUSTOM_ATTRIBUTE_                                     */
/*==============================================================*/
create table SAHARA_APP.CUSTOM_ATTRIBUTE_ 
(
   OID                  varchar(512)                   not null,
   NAME                 varchar(256)                   not null,
   VALUE                varchar(4000)                  null,
   VOM_OBJECT_TYPE      varchar(128)                   not null
);

comment on column SAHARA_APP.CUSTOM_ATTRIBUTE_.OID is 
'The unique object identifier of the object from which the custom attribute comes.  This must be unique across all object types for which custom attributes are processed.';

comment on column SAHARA_APP.CUSTOM_ATTRIBUTE_.NAME is 
'The name of the custom attribute.';

comment on column SAHARA_APP.CUSTOM_ATTRIBUTE_.VALUE is 
'The value of the custom attribute.';

comment on column SAHARA_APP.CUSTOM_ATTRIBUTE_.VOM_OBJECT_TYPE is 
'Object type from VOM or this object (Examples: vrts_host, vrts_vxvm_volume etc.)';

alter table SAHARA_APP.CUSTOM_ATTRIBUTE_
   add constraint PK_CUSTOM_ATTRIBUTE_ primary key (OID, NAME, VOM_OBJECT_TYPE);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.CUSTOM_ATTRIBUTE_ to SAHARA;

/*==============================================================*/
/* Table: CUSTOM_ATTRIBUTE_CONTEXT                              */
/*==============================================================*/
create table SAHARA.CUSTOM_ATTRIBUTE_CONTEXT 
(
   CUSTOM_ATTRIBUTE_CONTEXT varchar(32)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   UPDATE_INDICATOR     numeric(10)                    not null default 0
);

comment on table SAHARA.CUSTOM_ATTRIBUTE_CONTEXT is 
'Custom attributes may be used in multiple places in the sahara schema.  This is a look-up table that defines the context in which specific associated custom attributes are used.  It is populated at install and upgrade.';

comment on column SAHARA.CUSTOM_ATTRIBUTE_CONTEXT.CUSTOM_ATTRIBUTE_CONTEXT is 
'The context in which the source custom attributes are used.';

comment on column SAHARA.CUSTOM_ATTRIBUTE_CONTEXT.DISPLAY_NAME is 
'A display name for the context.  Custom attribute contexts are selected in the Enterprise Reporter GUI when configuring custom attributes.';

comment on column SAHARA.CUSTOM_ATTRIBUTE_CONTEXT.UPDATE_INDICATOR is 
'This column contains a simple numeric value that may be used to determine if the (non-published) configuration for a custom attribute context has been updated.  The value itself is meaningless, but it is uniquely changed each time the configuration is modified.  By caching the value when reading custom attribute configuration, the re-fetching and comparing it at a later time, the reader may quickly determine if the cached values are current.

This column is maintained by triggers on table CUSTOM_ATTRIBUTE.';

alter table SAHARA.CUSTOM_ATTRIBUTE_CONTEXT
   add constraint PK_CUSTOM_ATTRIBUTE_CONTEXT primary key (CUSTOM_ATTRIBUTE_CONTEXT);

grant SELECT on SAHARA.CUSTOM_ATTRIBUTE_CONTEXT to SAHARA_APP;

/*==============================================================*/
/* Table: DATABASES                                             */
/*==============================================================*/
create table SAHARA.DATABASES 
(
   DATABASES_KEY        numeric(10)                    not null default autoincrement,
   HOST_KEY             numeric(10)                    not null,
   DATABASES_OID        varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   INSTANCE_DISPLAY_NAME varchar(256)                   null,
   DISPLAY_NAME         varchar(256)                   not null,
   DATABASE_TYPE        varchar(32)                    not null,
   DATABASE_VERSION     varchar(256)                   null,
   DATABASE_STATE       varchar(32)                    null,
   DISCOVERY_STATE      varchar(32)                    null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   IS_CLUSTERED         char(1)                        null,
   IS_RAC_AWARE         char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   IS_BASED_ON_DB_VM    char(1)                        null default 'F'
);

comment on table SAHARA.DATABASES is 
'This is a dimension table for databases.  It contains databases and database attributes.

Following databases are included:
- Oracle
- Sybase ASE
- MS SQL
- DB2

Note: We''d prefer to call this table DATABASE, but the conflict with the DATABASE keyword was too much to overcome.';

comment on column SAHARA.DATABASES.DATABASES_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.DATABASES.CC_STORAGE_URL is 
'The URL for the database object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.DATABASES.INSTANCE_DISPLAY_NAME is 
'The database instance or engine name.  Some database management systems (e.g. Sybase) may run multiple databases using the same instance or engine.  The combination of this value, and database display name, uniquely identifies a database.';

comment on column SAHARA.DATABASES.DISPLAY_NAME is 
'Database display name.';

comment on column SAHARA.DATABASES.DATABASE_TYPE is 
'Database type can be one of the following:

Oracle
Sybase ASE
MS SQL
DB2';

comment on column SAHARA.DATABASES.DATABASE_STATE is 
'Running/Not Running';

comment on column SAHARA.DATABASES.DISCOVERY_STATE is 
'The discovery state of the database:

Possible values:
- Discovered
- Partially Discovered
- Missing

Note: Partially Discovered means that the database was initially discovered by two or more explorers and then one of them stopped discovering the database.

Note: A database that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.DATABASES.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.DATABASES.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.DATABASES.IS_CLUSTERED is 
'In case of oracle database: True, if the database is part of RAC.  
False otherwise.';

comment on column SAHARA.DATABASES.IS_RAC_AWARE is 
'In oracle RAC scenarios, value will be True for one database which is part of RAC setup.  
False otherwise for all the remaining databases which are part of same RAC setup.';

comment on column SAHARA.DATABASES.IS_BASED_ON_THIN is 
'A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the database is based on thin lun
''F'' - the database is not based on thin lun';

comment on column SAHARA.DATABASES.APPLICATION_BE is 
'Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DATABASES.APPLICATION_COMPOSITE_BE is 
'Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DATABASES.ORGANIZATION_BE is 
'Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DATABASES.ORGANIZATION_COMPOSITE_BE is 
'Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DATABASES.IS_BASED_ON_DB_VM is 
'A Boolean flag that indicates whether a database is using database volume manager like Oracle ASM.';

alter table SAHARA.DATABASES
   alter T2_IS_CURRENT add constraint C_DATABASES_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.DATABASES
   alter T2_IS_DELETED add constraint C_DATABASES_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.DATABASES
   alter IS_CLUSTERED add constraint C_DATABASES_ISCLUSTERED check (IS_CLUSTERED is null or (IS_CLUSTERED in ('T','F')));

alter table SAHARA.DATABASES
   alter IS_RAC_AWARE add constraint C_DATABASES_ISRACAWARE check (IS_RAC_AWARE is null or (IS_RAC_AWARE in ('T','F')));

alter table SAHARA.DATABASES
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA.DATABASES
   alter IS_BASED_ON_DB_VM add constraint CKC_IS_BASED_ON_DB_VM_DATABASE check (IS_BASED_ON_DB_VM is null or (IS_BASED_ON_DB_VM in ('T','F')));

alter table SAHARA.DATABASES
   add constraint PK_DATABASES primary key (DATABASES_KEY);

/*==============================================================*/
/* Index: I_DATABASES_HOSTKEY                                   */
/*==============================================================*/
create index I_DATABASES_HOSTKEY on SAHARA.DATABASES (
HOST_KEY ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_HOSTKEY is 
'An index on Host Key for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_DATABASESOID                              */
/*==============================================================*/
create index I_DATABASES_DATABASESOID on SAHARA.DATABASES (
DATABASES_OID ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_DATABASESOID is 
'An index on Database Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_NSTNCDSPLNM                               */
/*==============================================================*/
create index I_DATABASES_NSTNCDSPLNM on SAHARA.DATABASES (
INSTANCE_DISPLAY_NAME ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_NSTNCDSPLNM is 
'An index on Instance Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_DISPLAYNAME                               */
/*==============================================================*/
create index I_DATABASES_DISPLAYNAME on SAHARA.DATABASES (
DISPLAY_NAME ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_DATABASETYPE                              */
/*==============================================================*/
create index I_DATABASES_DATABASETYPE on SAHARA.DATABASES (
DATABASE_TYPE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_DATABASETYPE is 
'An index on Database Type for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_DATABASEVERSION                           */
/*==============================================================*/
create index I_DATABASES_DATABASEVERSION on SAHARA.DATABASES (
DATABASE_VERSION ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_DATABASEVERSION is 
'An index on Database Version for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_DATABASESTATE                             */
/*==============================================================*/
create index I_DATABASES_DATABASESTATE on SAHARA.DATABASES (
DATABASE_STATE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_DATABASESTATE is 
'An index on Database State for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_DISCOVERYSTATE                            */
/*==============================================================*/
create index I_DATABASES_DISCOVERYSTATE on SAHARA.DATABASES (
DISCOVERY_STATE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_T2SCRRNTDTBSSK                            */
/*==============================================================*/
create index I_DATABASES_T2SCRRNTDTBSSK on SAHARA.DATABASES (
T2_IS_CURRENT ASC,
DATABASES_KEY ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_T2SCRRNTDTBSSK is 
'An index on Type 2 - Is Current and Databases Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_DATABASES_T2SCRRNTT2SDLTDD                          */
/*==============================================================*/
create index I_DATABASES_T2SCRRNTT2SDLTDD on SAHARA.DATABASES (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
DATABASES_OID ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Database Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_DATABASES_T2ISDELETED                               */
/*==============================================================*/
create index I_DATABASES_T2ISDELETED on SAHARA.DATABASES (
T2_IS_DELETED ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_ISCLUSTERED                               */
/*==============================================================*/
create index I_DATABASES_ISCLUSTERED on SAHARA.DATABASES (
IS_CLUSTERED ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_ISCLUSTERED is 
'An index on Is Clustered for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_ISRACAWARE                                */
/*==============================================================*/
create index I_DATABASES_ISRACAWARE on SAHARA.DATABASES (
IS_RAC_AWARE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_ISRACAWARE is 
'An index on IS RAC AWARE for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_ISBASEDONTHIN                             */
/*==============================================================*/
create index I_DATABASES_ISBASEDONTHIN on SAHARA.DATABASES (
IS_BASED_ON_THIN ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_ISBASEDONTHIN is 
'An index on Is Based On Thin for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_APPLICATIONBE                             */
/*==============================================================*/
create index I_DATABASES_APPLICATIONBE on SAHARA.DATABASES (
APPLICATION_BE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_APPLICATIONBE is 
'An index on Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_PPLCTNCMPSTB                              */
/*==============================================================*/
create index I_DATABASES_PPLCTNCMPSTB on SAHARA.DATABASES (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_PPLCTNCMPSTB is 
'An index on Composite Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_ORGANIZATIONBE                            */
/*==============================================================*/
create index I_DATABASES_ORGANIZATIONBE on SAHARA.DATABASES (
ORGANIZATION_BE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_ORGANIZATIONBE is 
'An index on Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_RGNZTNCMPSTB                              */
/*==============================================================*/
create index I_DATABASES_RGNZTNCMPSTB on SAHARA.DATABASES (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_RGNZTNCMPSTB is 
'An index on Composite Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DATABASES_ISBASEDONDBVM                             */
/*==============================================================*/
create index I_DATABASES_ISBASEDONDBVM on SAHARA.DATABASES (
IS_BASED_ON_DB_VM ASC
);

comment on index SAHARA.DATABASES.I_DATABASES_ISBASEDONDBVM is 
'An index on Is Based on DB VM for dimensional queries.';

grant SELECT on SAHARA.DATABASES to SAHARA_APP;

/*==============================================================*/
/* Table: DATABASES_                                            */
/*==============================================================*/
create table SAHARA_APP.DATABASES_ 
(
   DATABASES_OID        varchar(512)                   not null,
   HOST_OID             varchar(512)                   not null,
   INSTANCE_DISPLAY_NAME varchar(256)                   null,
   DISPLAY_NAME         varchar(256)                   not null,
   DATABASE_TYPE        varchar(32)                    not null,
   DATABASE_VERSION     varchar(256)                   null,
   DATABASE_STATE       varchar(32)                    null,
   DISCOVERY_STATE      varchar(32)                    null,
   IS_CLUSTERED         char(1)                        null,
   IS_RAC_AWARE         char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   IS_BASED_ON_DB_VM    char(1)                        null default 'F'
);

comment on table SAHARA_APP.DATABASES_ is 
'This is a staging table for loading data into warehouse table SAHARA.DATABASES.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is a dimension table for databases.  It contains databases and database attributes.

Following databases are included:
- Oracle
- Sybase ASE
- MS SQL
- DB2

Note: We''d prefer to call this table DATABASE, but the conflict with the DATABASE keyword was too much to overcome.';

comment on column SAHARA_APP.DATABASES_.DATABASES_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.DATABASES_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.DATABASES_.INSTANCE_DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column INSTANCE_DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The database instance or engine name.  Some database management systems (e.g. Sybase) may run multiple databases using the same instance or engine.  The combination of this value, and database display name, uniquely identifies a database.';

comment on column SAHARA_APP.DATABASES_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Database display name.';

comment on column SAHARA_APP.DATABASES_.DATABASE_TYPE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column DATABASE_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Database type can be one of the following:

Oracle
Sybase ASE
MS SQL
DB2';

comment on column SAHARA_APP.DATABASES_.DATABASE_VERSION is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column DATABASE_VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

';

comment on column SAHARA_APP.DATABASES_.DATABASE_STATE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column DATABASE_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Running/Not Running';

comment on column SAHARA_APP.DATABASES_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the database:

Possible values:
- Discovered
- Partially Discovered
- Missing

Note: Partially Discovered means that the database was initially discovered by two or more explorers and then one of them stopped discovering the database.

Note: A database that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.DATABASES_.IS_CLUSTERED is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column IS_CLUSTERED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

In case of oracle database: True, if the database is part of RAC.  
False otherwise.';

comment on column SAHARA_APP.DATABASES_.IS_RAC_AWARE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column IS_RAC_AWARE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

In oracle RAC scenarios, value will be True for one database which is part of RAC setup.  
False otherwise for all the remaining databases which are part of same RAC setup.';

comment on column SAHARA_APP.DATABASES_.IS_BASED_ON_THIN is 
'This column is for staging data that will end up in table SAHARA.DATABASES column IS_BASED_ON_THIN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the database is based on thin lun
''F'' - the database is not based on thin lun';

comment on column SAHARA_APP.DATABASES_.APPLICATION_BE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column APPLICATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DATABASES_.APPLICATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column APPLICATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DATABASES_.ORGANIZATION_BE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column ORGANIZATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DATABASES_.ORGANIZATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.DATABASES, column ORGANIZATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DATABASES_.IS_BASED_ON_DB_VM is 
'A Boolean flag that indicates whether a database is using database volume manager like Oracle ASM.';

alter table SAHARA_APP.DATABASES_
   alter IS_CLUSTERED add constraint C_DATABASES_ISCLUSTERED check (IS_CLUSTERED is null or (IS_CLUSTERED in ('T','F')));

alter table SAHARA_APP.DATABASES_
   alter IS_RAC_AWARE add constraint C_DATABASES_ISRACAWARE check (IS_RAC_AWARE is null or (IS_RAC_AWARE in ('T','F')));

alter table SAHARA_APP.DATABASES_
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA_APP.DATABASES_
   alter IS_BASED_ON_DB_VM add constraint CKC_IS_BASED_ON_DB_VM_DATABASE check (IS_BASED_ON_DB_VM is null or (IS_BASED_ON_DB_VM in ('T','F')));

alter table SAHARA_APP.DATABASES_
   add constraint PK_DATABASES_ primary key (DATABASES_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.DATABASES_ to SAHARA;

/*==============================================================*/
/* Table: DATABASE_CAPACITY                                     */
/*==============================================================*/
create table SAHARA.DATABASE_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   DATABASES_KEY        numeric(10)                    not null,
   TOTAL_CAPACITY       numeric(22,9)                  not null default 0,
   TOTAL_AVAILABLE      numeric(22,9)                  not null default 0,
   TOTAL_USED           numeric(22,9)                  not null default 0,
   TOTAL_PERCENT_USED   numeric(7,4)                   not null default 0,
   CAPACITY_FROM_RAW    numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_RAW   numeric(22,9)                  not null default 0,
   USED_FROM_RAW        numeric(22,9)                  not null default 0,
   CAPACITY_FROM_FILE_SYSTEM numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_FILE_SYSTEM numeric(22,9)                  not null default 0,
   USED_FROM_FILE_SYSTEM numeric(22,9)                  not null default 0,
   CAPACITY_FROM_VOLUME numeric(22,9)                  not null default 0,
   USED_FROM_VOLUME     numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_VOLUME numeric(22,9)                  not null default 0,
   CAPACITY_FROM_DB_VM  numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_DB_VM numeric(22,9)                  not null default 0,
   USED_FROM_DB_VM      numeric(22,9)                  not null default 0
);

comment on table SAHARA.DATABASE_CAPACITY is 
'This is a fact table for databases that contains database capacities.';

comment on column SAHARA.DATABASE_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.DATABASE_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.DATABASE_CAPACITY.TOTAL_CAPACITY is 
'Total capacity of the database.  This includes all table spaces and files that the database is made up off.';

comment on column SAHARA.DATABASE_CAPACITY.TOTAL_AVAILABLE is 
'Total available capacity of the database.  This includes all table spaces and files that the database is made up off.';

comment on column SAHARA.DATABASE_CAPACITY.TOTAL_USED is 
'Number of ports used on a switch.';

comment on column SAHARA.DATABASE_CAPACITY.TOTAL_PERCENT_USED is 
'Total capacity used of the database, expressed as a percent value';

comment on column SAHARA.DATABASE_CAPACITY.CAPACITY_FROM_RAW is 
'Capacity from database files created on SundiskSlices or OS handles directly without any File System or Volume in between';

comment on column SAHARA.DATABASE_CAPACITY.AVAILABLE_FROM_RAW is 
'This is the capacity available for databases, based on files created from raw device handles (LUNs)';

comment on column SAHARA.DATABASE_CAPACITY.USED_FROM_RAW is 
'Capacity used by database, that has been created from raw devices (LUNs)';

comment on column SAHARA.DATABASE_CAPACITY.CAPACITY_FROM_FILE_SYSTEM is 
'Capacity from database files created on file systems.';

comment on column SAHARA.DATABASE_CAPACITY.AVAILABLE_FROM_FILE_SYSTEM is 
'This is the capacity available for databases, based on files created from file systems.';

comment on column SAHARA.DATABASE_CAPACITY.USED_FROM_FILE_SYSTEM is 
'Capacity used by database, that has been created from file systems.';

comment on column SAHARA.DATABASE_CAPACITY.CAPACITY_FROM_VOLUME is 
'Total capacity from database files that are created directly on volumes, without any file systems';

comment on column SAHARA.DATABASE_CAPACITY.USED_FROM_VOLUME is 
'Used capacity: Capacity from database files that are created directly on volumes, without any file systems';

comment on column SAHARA.DATABASE_CAPACITY.AVAILABLE_FROM_VOLUME is 
'Available capacity: Capacity from database files that are created directly on volumes, without any file systems';

comment on column SAHARA.DATABASE_CAPACITY.CAPACITY_FROM_DB_VM is 
'Capacity from database files deriving storage from database volume manager like Oracle ASM.';

comment on column SAHARA.DATABASE_CAPACITY.AVAILABLE_FROM_DB_VM is 
'Capacity available for database,  based on database volume manager like Oracle ASM.';

comment on column SAHARA.DATABASE_CAPACITY.USED_FROM_DB_VM is 
'Capacity used by database,  based on database volume manager like Oracle ASM.';

alter table SAHARA.DATABASE_CAPACITY
   alter TOTAL_CAPACITY add constraint C_DTBSCPCT_TOTALCAPACITY check (TOTAL_CAPACITY >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter TOTAL_AVAILABLE add constraint C_DTBSCPCT_TOTALAVAILABLE check (TOTAL_AVAILABLE >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter TOTAL_USED add constraint C_DTBSCPCT_TOTALUSED check (TOTAL_USED >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter CAPACITY_FROM_RAW add constraint C_DTBSCPCT_CAPACITYFROMRAW check (CAPACITY_FROM_RAW >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter AVAILABLE_FROM_RAW add constraint C_DTBSCPCT_AVAILABLEFROMRAW check (AVAILABLE_FROM_RAW >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter USED_FROM_RAW add constraint C_DTBSCPCT_USEDFROMRAW check (USED_FROM_RAW >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter CAPACITY_FROM_FILE_SYSTEM add constraint C_DTBSCPCT_CPCTFRMFLSSTM check (CAPACITY_FROM_FILE_SYSTEM >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter AVAILABLE_FROM_FILE_SYSTEM add constraint C_DTBSCPCT_VLBLFRMFLSSTM check (AVAILABLE_FROM_FILE_SYSTEM >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter USED_FROM_FILE_SYSTEM add constraint C_DTBSCPCT_USEDFROMFILESYSTEM check (USED_FROM_FILE_SYSTEM >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter CAPACITY_FROM_VOLUME add constraint C_DTBSCPCT_CAPACITYFROMVOLUME check (CAPACITY_FROM_VOLUME >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter USED_FROM_VOLUME add constraint C_DTBSCPCT_USEDFROMVOLUME check (USED_FROM_VOLUME >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter AVAILABLE_FROM_VOLUME add constraint C_DTBSCPCT_AVAILABLEFROMVOLUME check (AVAILABLE_FROM_VOLUME >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter CAPACITY_FROM_DB_VM add constraint CKC_CAPACITY_FROM_DB__DATABASE check (CAPACITY_FROM_DB_VM >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter AVAILABLE_FROM_DB_VM add constraint CKC_AVAILABLE_FROM_DB_DATABASE check (AVAILABLE_FROM_DB_VM >= 0);

alter table SAHARA.DATABASE_CAPACITY
   alter USED_FROM_DB_VM add constraint CKC_USED_FROM_DB_VM_DATABASE check (USED_FROM_DB_VM >= 0);

/*==============================================================*/
/* Index: I_DBCAPACITY_ROLLUPBATCHKEY                           */
/*==============================================================*/
create index I_DBCAPACITY_ROLLUPBATCHKEY on SAHARA.DATABASE_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_DBCAPACITY_DATESKEY                                 */
/*==============================================================*/
create index I_DBCAPACITY_DATESKEY on SAHARA.DATABASE_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_DBCAPACITY_DATABASESKEY                             */
/*==============================================================*/
create index I_DBCAPACITY_DATABASESKEY on SAHARA.DATABASE_CAPACITY (
DATABASES_KEY ASC
);

grant SELECT on SAHARA.DATABASE_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: DATABASE_CAPACITY_                                    */
/*==============================================================*/
create table SAHARA_APP.DATABASE_CAPACITY_ 
(
   DATABASES_OID        varchar(512)                   not null,
   TOTAL_CAPACITY       numeric(22,9)                  not null default 0,
   TOTAL_AVAILABLE      numeric(22,9)                  not null default 0,
   TOTAL_USED           numeric(22,9)                  not null default 0,
   TOTAL_PERCENT_USED   numeric(7,4)                   not null default 0,
   CAPACITY_FROM_RAW    numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_RAW   numeric(22,9)                  not null default 0,
   USED_FROM_RAW        numeric(22,9)                  not null default 0,
   CAPACITY_FROM_FILE_SYSTEM numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_FILE_SYSTEM numeric(22,9)                  not null default 0,
   USED_FROM_FILE_SYSTEM numeric(22,9)                  not null default 0,
   CAPACITY_FROM_VOLUME numeric(22,9)                  not null default 0,
   USED_FROM_VOLUME     numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_VOLUME numeric(22,9)                  not null default 0,
   CAPACITY_FROM_DB_VM  numeric(22,9)                  not null default 0,
   AVAILABLE_FROM_DB_VM numeric(22,9)                  not null default 0,
   USED_FROM_DB_VM      numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.DATABASE_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.DATABASE_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is a fact table for databases that contains database capacities.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.DATABASES_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.TOTAL_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column TOTAL_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total capacity of the database.  This includes all table spaces and files that the database is made up off.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.TOTAL_AVAILABLE is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column TOTAL_AVAILABLE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total available capacity of the database.  This includes all table spaces and files that the database is made up off.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.TOTAL_USED is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column TOTAL_USED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Number of ports used on a switch.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.TOTAL_PERCENT_USED is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column TOTAL_PERCENT_USED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total capacity used of the database, expressed as a percent value';

comment on column SAHARA_APP.DATABASE_CAPACITY_.CAPACITY_FROM_RAW is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column CAPACITY_FROM_RAW.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity from database files created on SundiskSlices or OS handles directly without any File System or Volume in between';

comment on column SAHARA_APP.DATABASE_CAPACITY_.AVAILABLE_FROM_RAW is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column AVAILABLE_FROM_RAW.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

This is the capacity available for databases, based on files created from raw device handles (LUNs)';

comment on column SAHARA_APP.DATABASE_CAPACITY_.USED_FROM_RAW is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column USED_FROM_RAW.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity used by database, that has been created from raw devices (LUNs)';

comment on column SAHARA_APP.DATABASE_CAPACITY_.CAPACITY_FROM_FILE_SYSTEM is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column CAPACITY_FROM_FILE_SYSTEM.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity from database files created on file systems.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.AVAILABLE_FROM_FILE_SYSTEM is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column AVAILABLE_FROM_FILE_SYSTEM.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

This is the capacity available for databases, based on files created from file systems.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.USED_FROM_FILE_SYSTEM is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column USED_FROM_FILE_SYSTEM.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity used by database, that has been created from file systems.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.CAPACITY_FROM_VOLUME is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column CAPACITY_FROM_VOLUME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total capacity from database files that are created directly on volumes, without any file systems';

comment on column SAHARA_APP.DATABASE_CAPACITY_.USED_FROM_VOLUME is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column USED_FROM_VOLUME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Used capacity: Capacity from database files that are created directly on volumes, without any file systems';

comment on column SAHARA_APP.DATABASE_CAPACITY_.AVAILABLE_FROM_VOLUME is 
'This column is for staging data that will end up in table SAHARA.DATABASE_CAPACITY, column AVAILABLE_FROM_VOLUME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Available capacity: Capacity from database files that are created directly on volumes, without any file systems';

comment on column SAHARA_APP.DATABASE_CAPACITY_.CAPACITY_FROM_DB_VM is 
'Capacity from database files deriving storage from database volume manager like Oracle ASM.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.AVAILABLE_FROM_DB_VM is 
'Capacity available for database,  based on database volume manager like Oracle ASM.';

comment on column SAHARA_APP.DATABASE_CAPACITY_.USED_FROM_DB_VM is 
'Capacity used by database,  based on database volume manager like Oracle ASM.';

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter TOTAL_CAPACITY add constraint C_DTBSCPCT_TOTALCAPACITY check (TOTAL_CAPACITY >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter TOTAL_AVAILABLE add constraint C_DTBSCPCT_TOTALAVAILABLE check (TOTAL_AVAILABLE >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter TOTAL_USED add constraint C_DTBSCPCT_TOTALUSED check (TOTAL_USED >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter CAPACITY_FROM_RAW add constraint C_DTBSCPCT_CAPACITYFROMRAW check (CAPACITY_FROM_RAW >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter AVAILABLE_FROM_RAW add constraint C_DTBSCPCT_AVAILABLEFROMRAW check (AVAILABLE_FROM_RAW >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter USED_FROM_RAW add constraint C_DTBSCPCT_USEDFROMRAW check (USED_FROM_RAW >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter CAPACITY_FROM_FILE_SYSTEM add constraint C_DTBSCPCT_CPCTFRMFLSSTM check (CAPACITY_FROM_FILE_SYSTEM >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter AVAILABLE_FROM_FILE_SYSTEM add constraint C_DTBSCPCT_VLBLFRMFLSSTM check (AVAILABLE_FROM_FILE_SYSTEM >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter USED_FROM_FILE_SYSTEM add constraint C_DTBSCPCT_USEDFROMFILESYSTEM check (USED_FROM_FILE_SYSTEM >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter CAPACITY_FROM_VOLUME add constraint C_DTBSCPCT_CAPACITYFROMVOLUME check (CAPACITY_FROM_VOLUME >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter USED_FROM_VOLUME add constraint C_DTBSCPCT_USEDFROMVOLUME check (USED_FROM_VOLUME >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter AVAILABLE_FROM_VOLUME add constraint C_DTBSCPCT_AVAILABLEFROMVOLUME check (AVAILABLE_FROM_VOLUME >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter CAPACITY_FROM_DB_VM add constraint CKC_CAPACITY_FROM_DB__DATABASE check (CAPACITY_FROM_DB_VM >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter AVAILABLE_FROM_DB_VM add constraint CKC_AVAILABLE_FROM_DB_DATABASE check (AVAILABLE_FROM_DB_VM >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   alter USED_FROM_DB_VM add constraint CKC_USED_FROM_DB_VM_DATABASE check (USED_FROM_DB_VM >= 0);

alter table SAHARA_APP.DATABASE_CAPACITY_
   add constraint PK_DATABASE_CAPACITY_ primary key (DATABASES_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.DATABASE_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: DATES                                                 */
/*==============================================================*/
create table SAHARA.DATES 
(
   DATES_KEY            numeric(10)                    not null default autoincrement,
   ADATE                date                           not null,
   YEAR_NUMBER          numeric(4)                     not null,
   MONTH                varchar(128)                   not null,
   QUARTER              varchar(128)                   not null,
   WEEK_NUMBER_OF_YEAR  numeric(2)                     not null,
   DAY_NUMBER           numeric(2)                     not null,
   DAY_NUMBER_OF_WEEK   numeric(1)                     not null,
   DAY_NUMBER_OF_YEAR   numeric(3)                     not null,
   DAY_NAME             varchar(32)                    not null,
   DAY_NAME_ABBREVIATION varchar(16)                    not null,
   MONTH_NUMBER         numeric(2)                     not null,
   MONTH_NAME           varchar(32)                    not null,
   MONTH_NAME_ABBREVIATION varchar(16)                    not null,
   QUARTER_NUMBER       numeric(1)                     not null,
   IS_WEEKDAY           char(1)                        not null default 'F',
   IS_HOLIDAY           char(1)                        not null default 'F',
   IS_MONTH_END         char(1)                        not null default 'F',
   IS_QUARTER_END       char(1)                        not null default 'F',
   IS_YEAR_END          char(1)                        not null default 'F'
);

comment on table SAHARA.DATES is 
'This table provides a dimension for dates using a single day granularity.

Note: We''d prefer to call this table DATE, but the conflict with the DATE keyword was too much to overcome.';

comment on column SAHARA.DATES.DATES_KEY is 
'A unique database generated identifier for the date record used as a surrogate key.';

comment on column SAHARA.DATES.ADATE is 
'The date for which all other columns of the table apply.

Note: We''d prefer to call this column DATE, but the conflict with the DATE keyword was too much to overcome.';

comment on column SAHARA.DATES.YEAR_NUMBER is 
'The four-digit numeric calendar year of the date.';

comment on column SAHARA.DATES.MONTH is 
'The month and year strings combined for display purposes.

Examples: ''November 2005'', ''January 2006''';

comment on column SAHARA.DATES.QUARTER is 
'The quarter number and year string combined for display purposes.

A quarter begins 1 January, 1 April, 1 July, or 1 October.

Examples: ''Q1 2005'', ''Q2 2006''';

comment on column SAHARA.DATES.WEEK_NUMBER_OF_YEAR is 
'Week number within a year from 1-52.';

comment on column SAHARA.DATES.DAY_NUMBER is 
'The numeric day of the month (1-31) for the date.';

comment on column SAHARA.DATES.DAY_NUMBER_OF_WEEK is 
'The numeric day of the week (1-7) for the date:
- Sun = 1
- Mon = 2
- Tue = 3
- Wed = 4
- Thu = 5
- Fri = 6
- Sat = 7';

comment on column SAHARA.DATES.DAY_NUMBER_OF_YEAR is 
'The day number of the calendar year (1-366) for the date.';

comment on column SAHARA.DATES.DAY_NAME is 
'The name of the day of the week for the date: Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday.';

comment on column SAHARA.DATES.DAY_NAME_ABBREVIATION is 
'The abbreviation for the name of the day of the week for the date: Sun, Mon, Tue, Wed, Thu, Fri, Sat.';

comment on column SAHARA.DATES.MONTH_NUMBER is 
'The numeric month of the year (1-12) for the date.';

comment on column SAHARA.DATES.MONTH_NAME is 
'The name of the month for the date: January, February, March, April, May, June, July, August, September, October, November, December.';

comment on column SAHARA.DATES.MONTH_NAME_ABBREVIATION is 
'The abbreviation for the name of the month for the date: Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec.';

comment on column SAHARA.DATES.QUARTER_NUMBER is 
'The numeric calendar quarter (1-4) for the date.';

comment on column SAHARA.DATES.IS_WEEKDAY is 
'A boolean flag that indicates if the date is a weekday.';

comment on column SAHARA.DATES.IS_HOLIDAY is 
'A boolean flag that indicates if that date is a holiday.';

comment on column SAHARA.DATES.IS_MONTH_END is 
'A boolean flag that indicates if the date is the last day of the month.';

comment on column SAHARA.DATES.IS_QUARTER_END is 
'A boolean flag that indicates if the date is the last day of a quarter.';

comment on column SAHARA.DATES.IS_YEAR_END is 
'A boolean flag that indicates if the date is the last day of the calendar year.';

alter table SAHARA.DATES
   alter DATES_KEY add constraint C_DATES_DATESKEY check (DATES_KEY >= 1);

alter table SAHARA.DATES
   alter ADATE add constraint C_DATES_ADATE check (ADATE = DATE(ADATE));

alter table SAHARA.DATES
   alter DAY_NUMBER add constraint C_DATES_DAYNUMBER check (DAY_NUMBER between 1 and 31);

alter table SAHARA.DATES
   alter DAY_NUMBER_OF_WEEK add constraint C_DATES_DAYNUMBEROFWEEK check (DAY_NUMBER_OF_WEEK between 1 and 7);

alter table SAHARA.DATES
   alter DAY_NUMBER_OF_YEAR add constraint C_DATES_DAYNUMBEROFYEAR check (DAY_NUMBER_OF_YEAR between 1 and 366);

alter table SAHARA.DATES
   alter MONTH_NUMBER add constraint C_DATES_MONTHNUMBER check (MONTH_NUMBER between 1 and 12);

alter table SAHARA.DATES
   alter IS_WEEKDAY add constraint C_DATES_ISWEEKDAY check (IS_WEEKDAY in ('T','F'));

alter table SAHARA.DATES
   alter IS_HOLIDAY add constraint C_DATES_ISHOLIDAY check (IS_HOLIDAY in ('T','F'));

alter table SAHARA.DATES
   alter IS_MONTH_END add constraint C_DATES_ISMONTHEND check (IS_MONTH_END in ('T','F'));

alter table SAHARA.DATES
   alter IS_QUARTER_END add constraint C_DATES_ISQUARTEREND check (IS_QUARTER_END in ('T','F'));

alter table SAHARA.DATES
   alter IS_YEAR_END add constraint C_DATES_ISYEAREND check (IS_YEAR_END in ('T','F'));

alter table SAHARA.DATES
   add constraint PK_DATES primary key (DATES_KEY);

alter table SAHARA.DATES
   add constraint AK_DATES_ADATE unique (ADATE);

/*==============================================================*/
/* Index: I_DATES_YEARNUMBER                                    */
/*==============================================================*/
create index I_DATES_YEARNUMBER on SAHARA.DATES (
YEAR_NUMBER ASC
);

comment on index SAHARA.DATES.I_DATES_YEARNUMBER is 
'An index on Year Number for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_MONTH                                         */
/*==============================================================*/
create index I_DATES_MONTH on SAHARA.DATES (
MONTH ASC
);

comment on index SAHARA.DATES.I_DATES_MONTH is 
'An index on Month for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_QUARTER                                       */
/*==============================================================*/
create index I_DATES_QUARTER on SAHARA.DATES (
QUARTER ASC
);

comment on index SAHARA.DATES.I_DATES_QUARTER is 
'An index on Quarter for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_WEEKNUMBEROFYEAR                              */
/*==============================================================*/
create index I_DATES_WEEKNUMBEROFYEAR on SAHARA.DATES (
WEEK_NUMBER_OF_YEAR ASC
);

comment on index SAHARA.DATES.I_DATES_WEEKNUMBEROFYEAR is 
'An index on Week Number of Year for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_DAYNUMBER                                     */
/*==============================================================*/
create index I_DATES_DAYNUMBER on SAHARA.DATES (
DAY_NUMBER ASC
);

comment on index SAHARA.DATES.I_DATES_DAYNUMBER is 
'An index on Day Number for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_DAYNUMBEROFWEEK                               */
/*==============================================================*/
create index I_DATES_DAYNUMBEROFWEEK on SAHARA.DATES (
DAY_NUMBER_OF_WEEK ASC
);

comment on index SAHARA.DATES.I_DATES_DAYNUMBEROFWEEK is 
'An index on Day Number of Week for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_DAYNUMBEROFYEAR                               */
/*==============================================================*/
create index I_DATES_DAYNUMBEROFYEAR on SAHARA.DATES (
DAY_NUMBER_OF_YEAR ASC
);

comment on index SAHARA.DATES.I_DATES_DAYNUMBEROFYEAR is 
'An index on Day Number of Year for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_DAYNAME                                       */
/*==============================================================*/
create index I_DATES_DAYNAME on SAHARA.DATES (
DAY_NAME ASC
);

comment on index SAHARA.DATES.I_DATES_DAYNAME is 
'An index on Day Name for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_DAYNAMEABBREVIATION                           */
/*==============================================================*/
create index I_DATES_DAYNAMEABBREVIATION on SAHARA.DATES (
DAY_NAME_ABBREVIATION ASC
);

comment on index SAHARA.DATES.I_DATES_DAYNAMEABBREVIATION is 
'An index on Day Name Abbreviation for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_MONTHNUMBER                                   */
/*==============================================================*/
create index I_DATES_MONTHNUMBER on SAHARA.DATES (
MONTH_NUMBER ASC
);

comment on index SAHARA.DATES.I_DATES_MONTHNUMBER is 
'An index on Month Number for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_MONTHNAME                                     */
/*==============================================================*/
create index I_DATES_MONTHNAME on SAHARA.DATES (
MONTH_NAME ASC
);

comment on index SAHARA.DATES.I_DATES_MONTHNAME is 
'An index on Month Name for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_MONTHNAMEABBREVIATION                         */
/*==============================================================*/
create index I_DATES_MONTHNAMEABBREVIATION on SAHARA.DATES (
MONTH_NAME_ABBREVIATION ASC
);

comment on index SAHARA.DATES.I_DATES_MONTHNAMEABBREVIATION is 
'An index on Month Name Abbreviation for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_QUARTERNUMBER                                 */
/*==============================================================*/
create index I_DATES_QUARTERNUMBER on SAHARA.DATES (
QUARTER_NUMBER ASC
);

comment on index SAHARA.DATES.I_DATES_QUARTERNUMBER is 
'An index on Quarter Number for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_ISWEEKDAY                                     */
/*==============================================================*/
create index I_DATES_ISWEEKDAY on SAHARA.DATES (
IS_WEEKDAY ASC
);

comment on index SAHARA.DATES.I_DATES_ISWEEKDAY is 
'An index on Is Weekday for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_ISHOLIDAY                                     */
/*==============================================================*/
create index I_DATES_ISHOLIDAY on SAHARA.DATES (
IS_HOLIDAY ASC
);

comment on index SAHARA.DATES.I_DATES_ISHOLIDAY is 
'An index on Is Holiday for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_ISMONTHEND                                    */
/*==============================================================*/
create index I_DATES_ISMONTHEND on SAHARA.DATES (
IS_MONTH_END ASC
);

comment on index SAHARA.DATES.I_DATES_ISMONTHEND is 
'An index on Is Month End for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_ISQUARTEREND                                  */
/*==============================================================*/
create index I_DATES_ISQUARTEREND on SAHARA.DATES (
IS_QUARTER_END ASC
);

comment on index SAHARA.DATES.I_DATES_ISQUARTEREND is 
'An index on Is Quarter End for dimensional queries.';

/*==============================================================*/
/* Index: I_DATES_ISYEAREND                                     */
/*==============================================================*/
create index I_DATES_ISYEAREND on SAHARA.DATES (
IS_YEAR_END ASC
);

comment on index SAHARA.DATES.I_DATES_ISYEAREND is 
'An index on Is Year End for dimensional queries.';

grant SELECT on SAHARA.DATES to SAHARA_APP;

/*==============================================================*/
/* Table: DB_VERSION                                            */
/*==============================================================*/
create table SAHARA.DB_VERSION 
(
   MAJOR                int                            not null,
   MINOR                int                            not null,
   MP_VERSION           int                            not null
);

alter table SAHARA.DB_VERSION
   add constraint PK_DB_VERSION primary key clustered (MAJOR, MINOR, MP_VERSION);

/*==============================================================*/
/* Table: DISKGROUP                                             */
/*==============================================================*/
create table SAHARA.DISKGROUP 
(
   DISKGROUP_KEY        numeric(10)                    not null default autoincrement,
   HOST_KEY             numeric(10)                    not null,
   DISKGROUP_OID        varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   FAMILY               varchar(256)                   not null,
   VERSION              varchar(256)                   not null,
   TYPE                 varchar(32)                    not null,
   IS_SHARED            char(1)                        null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   IS_BASED_ON_DIRECT_ATTACHED char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   STATE                varchar(256)                   null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   REDUNDANCY_TYPE      varchar(256)                   null,
   IS_USED_BY_CLUSTERED_DB char(1)                        null default 'F',
   USES_VM_VOLUMES      char(1)                        null default 'F'
);

comment on table SAHARA.DISKGROUP is 
'This table contains disk group attributes, like name, type, family etc.';

comment on column SAHARA.DISKGROUP.DISKGROUP_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.DISKGROUP.CC_STORAGE_URL is 
'The URL for the file system object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.DISKGROUP.DISPLAY_NAME is 
'Disk Group display name';

comment on column SAHARA.DISKGROUP.FAMILY is 
'Family of the Disk Group';

comment on column SAHARA.DISKGROUP.VERSION is 
'Version of the Disk Group, Volume Manager';

comment on column SAHARA.DISKGROUP.TYPE is 
'Type of DG. This can be ZFS for ZPools';

comment on column SAHARA.DISKGROUP.IS_SHARED is 
'True, if the disk group is shared.  False otherwise.';

comment on column SAHARA.DISKGROUP.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.DISKGROUP.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.DISKGROUP.IS_BASED_ON_DIRECT_ATTACHED is 
'A boolean flag that indicates if the disk group is based on direct attached.

Possible values:
''T'' - the Diskgroup is comsuming direct attached storage
''F'' - the Diskgroup is not comsuming direct attached storage';

comment on column SAHARA.DISKGROUP.IS_BASED_ON_THIN is 
'A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the diskgroup is based on thin lun
''F'' - the diskgroup is not based on thin lun';

comment on column SAHARA.DISKGROUP.STATE is 
'State of the Disk Group';

comment on column SAHARA.DISKGROUP.APPLICATION_BE is 
'Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DISKGROUP.APPLICATION_COMPOSITE_BE is 
'Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DISKGROUP.ORGANIZATION_BE is 
'Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DISKGROUP.ORGANIZATION_COMPOSITE_BE is 
'Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.DISKGROUP.REDUNDANCY_TYPE is 
'This attribute distinguishes between mirrored disk groups and non-mirrored disk groups.';

comment on column SAHARA.DISKGROUP.IS_USED_BY_CLUSTERED_DB is 
'A Boolean flag that indicates whether a disk group is utilized by a clustered database like Oracle RAC.';

comment on column SAHARA.DISKGROUP.USES_VM_VOLUMES is 
'A Boolean flag that indicates whether a disk group uses volume managers volume.';

alter table SAHARA.DISKGROUP
   alter IS_SHARED add constraint C_FILESYSTEM_ISCLUSTERED check (IS_SHARED is null or (IS_SHARED in ('T','F')));

alter table SAHARA.DISKGROUP
   alter T2_IS_CURRENT add constraint C_FILESYSTEM_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.DISKGROUP
   alter T2_IS_DELETED add constraint C_FILESYSTEM_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.DISKGROUP
   alter IS_BASED_ON_DIRECT_ATTACHED add constraint C_FILESYSTEM_ISDIRECTATTACHED check (IS_BASED_ON_DIRECT_ATTACHED is null or (IS_BASED_ON_DIRECT_ATTACHED in ('T','F')));

alter table SAHARA.DISKGROUP
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA.DISKGROUP
   alter IS_USED_BY_CLUSTERED_DB add constraint CKC_IS_USED_BY_CLUSTE_DISKGROU check (IS_USED_BY_CLUSTERED_DB is null or (IS_USED_BY_CLUSTERED_DB in ('T','F')));

alter table SAHARA.DISKGROUP
   alter USES_VM_VOLUMES add constraint CKC_USES_VM_VOLUMES_DISKGROU check (USES_VM_VOLUMES is null or (USES_VM_VOLUMES in ('T','F')));

alter table SAHARA.DISKGROUP
   add constraint PK_DISKGROUP primary key (DISKGROUP_KEY);

/*==============================================================*/
/* Index: I_DISKGROUP_HOSTKEY                                   */
/*==============================================================*/
create index I_DISKGROUP_HOSTKEY on SAHARA.DISKGROUP (
HOST_KEY ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_HOSTKEY is 
'An index on Host Key for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_DISKGROUPOID                              */
/*==============================================================*/
create index I_DISKGROUP_DISKGROUPOID on SAHARA.DISKGROUP (
DISKGROUP_OID ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_DISKGROUPOID is 
'An index on Diskgroup Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_DISPLAYNAME                               */
/*==============================================================*/
create index I_DISKGROUP_DISPLAYNAME on SAHARA.DISKGROUP (
DISPLAY_NAME ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_FAMILY                                    */
/*==============================================================*/
create index I_DISKGROUP_FAMILY on SAHARA.DISKGROUP (
FAMILY ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_FAMILY is 
'An index on Family for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_VERSION                                   */
/*==============================================================*/
create index I_DISKGROUP_VERSION on SAHARA.DISKGROUP (
VERSION ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_VERSION is 
'An index on Version for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_TYPE                                      */
/*==============================================================*/
create index I_DISKGROUP_TYPE on SAHARA.DISKGROUP (
TYPE ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_ISSHARED                                  */
/*==============================================================*/
create index I_DISKGROUP_ISSHARED on SAHARA.DISKGROUP (
IS_SHARED ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_ISSHARED is 
'An index on Is Shared for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_T2SCRRNTDSKGRPK                           */
/*==============================================================*/
create index I_DISKGROUP_T2SCRRNTDSKGRPK on SAHARA.DISKGROUP (
T2_IS_CURRENT ASC,
DISKGROUP_KEY ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_T2SCRRNTDSKGRPK is 
'An index on Type 2 - Is Current and Diskgroup Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_DISKGROUP_T2SCRRNTT2SDLTDD                          */
/*==============================================================*/
create index I_DISKGROUP_T2SCRRNTT2SDLTDD on SAHARA.DISKGROUP (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
DISKGROUP_OID ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Diskgroup Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_DISKGROUP_T2ISDELETED                               */
/*==============================================================*/
create index I_DISKGROUP_T2ISDELETED on SAHARA.DISKGROUP (
T2_IS_DELETED ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_SBSDNDRCTTTCHD                            */
/*==============================================================*/
create index I_DISKGROUP_SBSDNDRCTTTCHD on SAHARA.DISKGROUP (
IS_BASED_ON_DIRECT_ATTACHED ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_SBSDNDRCTTTCHD is 
'An index on Is Based On Direct Attached for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_ISBASEDONTHIN                             */
/*==============================================================*/
create index I_DISKGROUP_ISBASEDONTHIN on SAHARA.DISKGROUP (
IS_BASED_ON_THIN ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_ISBASEDONTHIN is 
'An index on Is Based On Thin for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_STATE                                     */
/*==============================================================*/
create index I_DISKGROUP_STATE on SAHARA.DISKGROUP (
STATE ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_STATE is 
'An index on State for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_APPLICATIONBE                             */
/*==============================================================*/
create index I_DISKGROUP_APPLICATIONBE on SAHARA.DISKGROUP (
APPLICATION_BE ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_APPLICATIONBE is 
'An index on Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_PPLCTNCMPSTB                              */
/*==============================================================*/
create index I_DISKGROUP_PPLCTNCMPSTB on SAHARA.DISKGROUP (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_PPLCTNCMPSTB is 
'An index on Composite Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_ORGANIZATIONBE                            */
/*==============================================================*/
create index I_DISKGROUP_ORGANIZATIONBE on SAHARA.DISKGROUP (
ORGANIZATION_BE ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_ORGANIZATIONBE is 
'An index on Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_RGNZTNCMPSTB                              */
/*==============================================================*/
create index I_DISKGROUP_RGNZTNCMPSTB on SAHARA.DISKGROUP (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_RGNZTNCMPSTB is 
'An index on Composite Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_REDUNDANCYTYPE                            */
/*==============================================================*/
create index I_DISKGROUP_REDUNDANCYTYPE on SAHARA.DISKGROUP (
REDUNDANCY_TYPE ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_REDUNDANCYTYPE is 
'An index on Redundancy Type for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_SSDBCLSTRDDB                              */
/*==============================================================*/
create index I_DISKGROUP_SSDBCLSTRDDB on SAHARA.DISKGROUP (
IS_USED_BY_CLUSTERED_DB ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_SSDBCLSTRDDB is 
'An index on Is Used By Clustered DB for dimensional queries.';

/*==============================================================*/
/* Index: I_DISKGROUP_USESVMVOLUMES                             */
/*==============================================================*/
create index I_DISKGROUP_USESVMVOLUMES on SAHARA.DISKGROUP (
USES_VM_VOLUMES ASC
);

comment on index SAHARA.DISKGROUP.I_DISKGROUP_USESVMVOLUMES is 
'An index on Uses VM Volumes for dimensional queries.';

grant SELECT on SAHARA.DISKGROUP to SAHARA_APP;

/*==============================================================*/
/* Table: DISKGROUP_                                            */
/*==============================================================*/
create table SAHARA_APP.DISKGROUP_ 
(
   DISKGROUP_OID        varchar(512)                   not null,
   HOST_OID             varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   FAMILY               varchar(256)                   not null,
   VERSION              varchar(256)                   not null,
   TYPE                 varchar(32)                    not null,
   IS_SHARED            char(1)                        null,
   IS_BASED_ON_DIRECT_ATTACHED char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   STATE                varchar(256)                   null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   REDUNDANCY_TYPE      varchar(256)                   null,
   IS_USED_BY_CLUSTERED_DB char(1)                        null default 'F',
   USES_VM_VOLUMES      char(1)                        null default 'F'
);

comment on table SAHARA_APP.DISKGROUP_ is 
'This is a staging table for loading data into warehouse table SAHARA.DISKGROUP.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains file system attributes, like name, vendor, device path etc.';

comment on column SAHARA_APP.DISKGROUP_.DISKGROUP_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.DISKGROUP_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.DISKGROUP_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Disk group display name';

comment on column SAHARA_APP.DISKGROUP_.FAMILY is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column FAMILY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Family of the Disk Group';

comment on column SAHARA_APP.DISKGROUP_.VERSION is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Version of the Disk Group, Volume Manager';

comment on column SAHARA_APP.DISKGROUP_.TYPE is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Type of DG. This can be ZFS for ZPools.';

comment on column SAHARA_APP.DISKGROUP_.IS_SHARED is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column IS_SHARED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

True, if the disk group is shared.  False otherwise.';

comment on column SAHARA_APP.DISKGROUP_.IS_BASED_ON_DIRECT_ATTACHED is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column IS_BASED_ON_DIRECT_ATTACHED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the disk group is based on direct attached.

Possible values:
''T'' - the Diskgroup is comsuming direct attached storage
''F'' - the Diskgroup is not comsuming direct attached storage';

comment on column SAHARA_APP.DISKGROUP_.IS_BASED_ON_THIN is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP column IS_BASED_ON_THIN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the diskgroup is based on thin lun
''F'' - the diskgroup is not based on thin lun';

comment on column SAHARA_APP.DISKGROUP_.STATE is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

State of the Disk Group';

comment on column SAHARA_APP.DISKGROUP_.APPLICATION_BE is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column APPLICATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DISKGROUP_.APPLICATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column APPLICATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DISKGROUP_.ORGANIZATION_BE is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column ORGANIZATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DISKGROUP_.ORGANIZATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP, column ORGANIZATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.DISKGROUP_.REDUNDANCY_TYPE is 
'This attribute distinguishes between mirrored disk groups and non-mirrored disk groups.';

comment on column SAHARA_APP.DISKGROUP_.IS_USED_BY_CLUSTERED_DB is 
'A Boolean flag that indicates whether a disk group is utilized by a clustered database like Oracle RAC.';

comment on column SAHARA_APP.DISKGROUP_.USES_VM_VOLUMES is 
'A Boolean flag that indicates whether a disk group uses volume managers volume.';

alter table SAHARA_APP.DISKGROUP_
   alter IS_SHARED add constraint C_FILESYSTEM_ISCLUSTERED check (IS_SHARED is null or (IS_SHARED in ('T','F')));

alter table SAHARA_APP.DISKGROUP_
   alter IS_BASED_ON_DIRECT_ATTACHED add constraint C_FILESYSTEM_ISDIRECTATTACHED check (IS_BASED_ON_DIRECT_ATTACHED is null or (IS_BASED_ON_DIRECT_ATTACHED in ('T','F')));

alter table SAHARA_APP.DISKGROUP_
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA_APP.DISKGROUP_
   alter IS_USED_BY_CLUSTERED_DB add constraint CKC_IS_USED_BY_CLUSTE_DISKGROU check (IS_USED_BY_CLUSTERED_DB is null or (IS_USED_BY_CLUSTERED_DB in ('T','F')));

alter table SAHARA_APP.DISKGROUP_
   alter USES_VM_VOLUMES add constraint CKC_USES_VM_VOLUMES_DISKGROU check (USES_VM_VOLUMES is null or (USES_VM_VOLUMES in ('T','F')));

alter table SAHARA_APP.DISKGROUP_
   add constraint PK_DISKGROUP_ primary key (DISKGROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.DISKGROUP_ to SAHARA;

/*==============================================================*/
/* Table: DISKGROUP_CAPACITY                                    */
/*==============================================================*/
create table SAHARA.DISKGROUP_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   DISKGROUP_KEY        numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   FREE_CAPACITY        numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   PERCENT_FREE         numeric(7,4)                   not null default 0,
   PERCENT_USED         numeric(7,4)                   not null default 0,
   OVERHEAD_CAPACITY    numeric(22,9)                  not null default 0,
   UNKNOWN_CAPACITY     numeric(22,9)                  not null default 0,
   DB_USED_CAPACITY     numeric(22,9)                  not null default 0,
   USED_CAPACITY_BY_CLUSTERED_DB numeric(22,9)                  not null default 0
);

comment on table SAHARA.DISKGROUP_CAPACITY is 
'This table contains capacity values of file systems.  Use this table in conjunction with the DISKGROUP table.';

comment on column SAHARA.DISKGROUP_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.DISKGROUP_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.DISKGROUP_CAPACITY.CAPACITY is 
'The total capacity available on the disk group. ';

comment on column SAHARA.DISKGROUP_CAPACITY.FREE_CAPACITY is 
'Capacity available to create volumes.';

comment on column SAHARA.DISKGROUP_CAPACITY.USED_CAPACITY is 
'Total disk group capacity on hosts that has volumes created onto.';

comment on column SAHARA.DISKGROUP_CAPACITY.PERCENT_FREE is 
'Disk group capacity available expressed as percentage value.';

comment on column SAHARA.DISKGROUP_CAPACITY.PERCENT_USED is 
'Disk group capacity utilized expressed as percentage value.';

comment on column SAHARA.DISKGROUP_CAPACITY.OVERHEAD_CAPACITY is 
'Capacity of disk groups gone into mirroring.';

comment on column SAHARA.DISKGROUP_CAPACITY.UNKNOWN_CAPACITY is 
'Capacity of disk groups gone into storing meta-data and other book-keeping information.';

comment on column SAHARA.DISKGROUP_CAPACITY.DB_USED_CAPACITY is 
'Capacity of database volume managers disk groups used by databases.';

comment on column SAHARA.DISKGROUP_CAPACITY.USED_CAPACITY_BY_CLUSTERED_DB is 
'Capacity of database volume managers disk groups used by clustered databases like Oracle RAC.';

alter table SAHARA.DISKGROUP_CAPACITY
   alter CAPACITY add constraint C_FLSSTMCPCT_CAPACITY check (CAPACITY >= 0);

alter table SAHARA.DISKGROUP_CAPACITY
   alter FREE_CAPACITY add constraint C_FLSSTMCPCT_FREECAPACITY check (FREE_CAPACITY >= 0);

alter table SAHARA.DISKGROUP_CAPACITY
   alter USED_CAPACITY add constraint C_FLSSTMCPCT_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA.DISKGROUP_CAPACITY
   alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_DISKGROU check (OVERHEAD_CAPACITY >= 0);

alter table SAHARA.DISKGROUP_CAPACITY
   alter UNKNOWN_CAPACITY add constraint CKC_UNKNOWN_CAPACITY_DISKGROU check (UNKNOWN_CAPACITY >= 0);

alter table SAHARA.DISKGROUP_CAPACITY
   alter DB_USED_CAPACITY add constraint CKC_DB_USED_CAPACITY_DISKGROU check (DB_USED_CAPACITY >= 0);

alter table SAHARA.DISKGROUP_CAPACITY
   alter USED_CAPACITY_BY_CLUSTERED_DB add constraint CKC_USED_CAPACITY_BY__DISKGROU check (USED_CAPACITY_BY_CLUSTERED_DB >= 0);

/*==============================================================*/
/* Index: I_DGCAPACITY_ROLLUPBATCHKEY                           */
/*==============================================================*/
create index I_DGCAPACITY_ROLLUPBATCHKEY on SAHARA.DISKGROUP_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_DGCAPACITY_DATESKEY                                 */
/*==============================================================*/
create index I_DGCAPACITY_DATESKEY on SAHARA.DISKGROUP_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_DGCAPACITY_DISKGROUPKEY                             */
/*==============================================================*/
create index I_DGCAPACITY_DISKGROUPKEY on SAHARA.DISKGROUP_CAPACITY (
DISKGROUP_KEY ASC
);

grant SELECT on SAHARA.DISKGROUP_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: DISKGROUP_CAPACITY_                                   */
/*==============================================================*/
create table SAHARA_APP.DISKGROUP_CAPACITY_ 
(
   DISKGROUP_OID        varchar(512)                   not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   FREE_CAPACITY        numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   PERCENT_FREE         numeric(7,4)                   not null default 0,
   PERCENT_USED         numeric(7,4)                   not null default 0,
   OVERHEAD_CAPACITY    numeric(22,9)                  not null default 0,
   UNKNOWN_CAPACITY     numeric(22,9)                  not null default 0,
   DB_USED_CAPACITY     numeric(22,9)                  not null default 0,
   USED_CAPACITY_BY_CLUSTERED_DB numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.DISKGROUP_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.DISKGROUP_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains capacity values of disk groups.  Use this table in conjunction with the DISKGROUP table.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.DISKGROUP_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.CAPACITY is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP_CAPACITY, column CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total capacity available on the disk group. ';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.FREE_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP_CAPACITY, column FREE_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity available to create volumes.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.USED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP_CAPACITY, column USED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total disk group capacity on hosts that has volumes created onto.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.PERCENT_FREE is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP_CAPACITY, column PERCENT_FREE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

File system capacity available expressed as percentage value.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.PERCENT_USED is 
'This column is for staging data that will end up in table SAHARA.DISKGROUP_CAPACITY, column PERCENT_USED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Disk group capacity utilized expressed as percentage value.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.OVERHEAD_CAPACITY is 
'Capacity of disk groups gone into mirroring.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.UNKNOWN_CAPACITY is 
'Capacity of disk groups gone into storing meta-data and other book-keeping information.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.DB_USED_CAPACITY is 
'Capacity of database volume managers disk groups used by databases.';

comment on column SAHARA_APP.DISKGROUP_CAPACITY_.USED_CAPACITY_BY_CLUSTERED_DB is 
'Capacity of database volume managers disk groups used by clustered databases like Oracle RAC.';

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   alter CAPACITY add constraint C_FLSSTMCPCT_CAPACITY check (CAPACITY >= 0);

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   alter FREE_CAPACITY add constraint C_FLSSTMCPCT_FREECAPACITY check (FREE_CAPACITY >= 0);

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   alter USED_CAPACITY add constraint C_FLSSTMCPCT_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_DISKGROU check (OVERHEAD_CAPACITY >= 0);

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   alter UNKNOWN_CAPACITY add constraint CKC_UNKNOWN_CAPACITY_DISKGROU check (UNKNOWN_CAPACITY >= 0);

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   alter DB_USED_CAPACITY add constraint CKC_DB_USED_CAPACITY_DISKGROU check (DB_USED_CAPACITY >= 0);

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   alter USED_CAPACITY_BY_CLUSTERED_DB add constraint CKC_USED_CAPACITY_BY__DISKGROU check (USED_CAPACITY_BY_CLUSTERED_DB >= 0);

alter table SAHARA_APP.DISKGROUP_CAPACITY_
   add constraint PK_DISKGROUP_CAPACITY_ primary key (DISKGROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.DISKGROUP_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: DM_BATCH                                              */
/*==============================================================*/
create table SAHARA.DM_BATCH 
(
   DM_BATCH_KEY         numeric(10)                    not null default autoincrement,
   ROLLUP_SOURCE_KEY    numeric(10)                    not null,
   SOURCE_NAME          varchar(128)                   not null,
   DB_HOST              varchar(128)                   not null,
   DB_PORT              numeric(5)                     not null,
   DM_BATCH_ID          numeric(10)                    not null,
   "TIME"               datetime                       not null,
   TIME_SOURCE          datetime                       not null,
   TIME_ER              datetime                       not null
);

comment on table SAHARA.DM_BATCH is 
'This table is a dimension/look-up table for DM batch information.  It documents that DM batch from which associated DM fact measures have been collected.';

comment on column SAHARA.DM_BATCH.DM_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.DM_BATCH.ROLLUP_SOURCE_KEY is 
'The identifier of the roll-up source from which the DM batch is collected.  This is a "weak link" in that it is not foreign key enforced.  When the roll-up source is physically deleted, this key will no longer point to a value.

All of the relevant roll-up information is copied from the roll-up source to this table when a DM batch record is created.  Reports and other parts of the application should not rely on this link; it exists primarily to be able to identify the specific roll-up source for book-keeping purposes.';

comment on column SAHARA.DM_BATCH.SOURCE_NAME is 
'The external "name" for a roll-up source.  This is not a network name; it is a user-defined name by which the user uniquely identifies the roll-up source and is purely descriptive.  It may differ from the web server host name, database host name, or management server host name.';

comment on column SAHARA.DM_BATCH.DB_HOST is 
'The name or IP address of the host on which the database server of roll-up source is running.';

comment on column SAHARA.DM_BATCH.DB_PORT is 
'The database server port number of the roll-up source.';

comment on column SAHARA.DM_BATCH.DM_BATCH_ID is 
'The unique identifier of the DM batch from within the specific roll-up source.';

comment on column SAHARA.DM_BATCH."TIME" is 
'The date and time (stored in UTC) when collection of the DM batch was begun.';

comment on column SAHARA.DM_BATCH.TIME_SOURCE is 
'The date and time (stored in roll-up source local timezone) when collection of the DM batch was begun.';

comment on column SAHARA.DM_BATCH.TIME_ER is 
'The date and time (stored in Enterprise Reporter local time) when collection of the DM batch was begun.';

alter table SAHARA.DM_BATCH
   alter DB_PORT add constraint C_DMBATCH_DBPORT check (DB_PORT >= 1);

alter table SAHARA.DM_BATCH
   add constraint PK_DM_BATCH primary key (DM_BATCH_KEY);

/*==============================================================*/
/* Index: I_DMBATCH_ROLLUPSOURCEKEY                             */
/*==============================================================*/
create index I_DMBATCH_ROLLUPSOURCEKEY on SAHARA.DM_BATCH (
ROLLUP_SOURCE_KEY ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_ROLLUPSOURCEKEY is 
'An index on Roll-up Source Key for dimensional queries.';

/*==============================================================*/
/* Index: I_DMBATCH_SOURCENAME                                  */
/*==============================================================*/
create index I_DMBATCH_SOURCENAME on SAHARA.DM_BATCH (
SOURCE_NAME ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_SOURCENAME is 
'An index on Source Name for dimensional queries.';

/*==============================================================*/
/* Index: I_DMBATCH_DBPORT                                      */
/*==============================================================*/
create index I_DMBATCH_DBPORT on SAHARA.DM_BATCH (
DB_PORT ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_DBPORT is 
'An index on Database Port for dimensional queries.';

/*==============================================================*/
/* Index: I_DMBATCH_DBHOST                                      */
/*==============================================================*/
create index I_DMBATCH_DBHOST on SAHARA.DM_BATCH (
DB_HOST ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_DBHOST is 
'An index on Database Host for dimensional queries.';

/*==============================================================*/
/* Index: I_DMBATCH_DMBATCHID                                   */
/*==============================================================*/
create index I_DMBATCH_DMBATCHID on SAHARA.DM_BATCH (
DM_BATCH_ID ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_DMBATCHID is 
'An index on DM Batch Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_DMBATCH_TIME                                        */
/*==============================================================*/
create index I_DMBATCH_TIME on SAHARA.DM_BATCH (
"TIME" ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_TIME is 
'An index on Time for dimensional queries.';

/*==============================================================*/
/* Index: I_DMBATCH_TIMESOURCE                                  */
/*==============================================================*/
create index I_DMBATCH_TIMESOURCE on SAHARA.DM_BATCH (
TIME_SOURCE ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_TIMESOURCE is 
'An index on Time - Source for dimensional queries.';

/*==============================================================*/
/* Index: I_DMBATCH_TIMEER                                      */
/*==============================================================*/
create index I_DMBATCH_TIMEER on SAHARA.DM_BATCH (
TIME_ER ASC
);

comment on index SAHARA.DM_BATCH.I_DMBATCH_TIMEER is 
'An index on Time - Enterprise Reporter Time for dimensional queries.';

grant SELECT on SAHARA.DM_BATCH to SAHARA_APP;

/*==============================================================*/
/* Table: DM_BATCH_                                             */
/*==============================================================*/
create table SAHARA_APP.DM_BATCH_ 
(
   DM_BATCH_ID          numeric(10)                    not null,
   "TIME"               datetime                       not null
);

comment on table SAHARA_APP.DM_BATCH_ is 
'This is a staging table for loading data into warehouse table SAHARA.DM_BATCH.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table is a dimension/look-up table for DM batch information.  It documents that DM batch from which associated DM fact measures have been collected.';

comment on column SAHARA_APP.DM_BATCH_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.DM_BATCH_."TIME" is 
'This column is for staging data that will end up in table SAHARA.DM_BATCH, column TIME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The date and time (stored in UTC) when collection of the DM batch was begun.';

alter table SAHARA_APP.DM_BATCH_
   add constraint PK_DM_BATCH_ primary key (DM_BATCH_ID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.DM_BATCH_ to SAHARA;

/*==============================================================*/
/* Table: DOMAIN_TYPE                                           */
/*==============================================================*/
create table SAHARA.DOMAIN_TYPE 
(
   SA_BOOLEAN           char(1)                        not null,
   SA_COLUMN_NAME       varchar(30)                    not null,
   SA_CONTROL_NAME      varchar(32)                    not null,
   SA_DEVICE_DISPLAY_NAME varchar(256)                   not null,
   SA_KEY               numeric(10)                    not null,
   SA_CCS_OID           varchar(512)                   not null,
   SA_TABLE_NAME        varchar(30)                    not null,
   SA_WWN               varchar(128)                   not null
);

comment on table SAHARA.DOMAIN_TYPE is 
'A table of columns that identify the data types of domains used in the sahara schema.  This table exists purely to allow stored procedures and functions to reference the types of the columns using the %type syntax of PL/SQL.';

alter table SAHARA.DOMAIN_TYPE
   alter SA_BOOLEAN add constraint C_DOMAINTYPE_SABOOLEAN check (SA_BOOLEAN in ('T','F'));

alter table SAHARA.DOMAIN_TYPE
   alter SA_COLUMN_NAME add constraint C_DOMAINTYPE_SACOLUMNNAME check (SA_COLUMN_NAME = upper(SA_COLUMN_NAME));

alter table SAHARA.DOMAIN_TYPE
   alter SA_TABLE_NAME add constraint C_DOMAINTYPE_SATABLENAME check (SA_TABLE_NAME = upper(SA_TABLE_NAME));

grant SELECT on SAHARA.DOMAIN_TYPE to SAHARA_APP;

/*==============================================================*/
/* Table: ER_LICENSE                                            */
/*==============================================================*/
create table SAHARA.ER_LICENSE 
(
   LICENSE_KEY          varchar(40)                    not null,
   LICENSE_TYPE         varchar(32)                    not null,
   VERSION              varchar(40)                    not null,
   CLIENT_ACCESS_LICENSES numeric(8)                     not null,
   DURATION             varchar(128)                   not null,
   EXPIRATION           date                           null,
   IS_SITE_KEY          char(1)                        not null,
   IS_EDITION_KEY       char(1)                        not null,
   IS_POINT_PRODUCT_KEY char(1)                        not null
);

comment on table SAHARA.ER_LICENSE is 
'This table contains Enterprise Reporter license records.  Licenses are polled from the Enterprise Reporter server where they are installed and stored and copied into the database from where they are reported.  The snapshot of licenses that this table provides is only as current as the most recent poll.';

comment on column SAHARA.ER_LICENSE.LICENSE_KEY is 
'The actual license key string.';

comment on column SAHARA.ER_LICENSE.LICENSE_TYPE is 
'The license type.';

comment on column SAHARA.ER_LICENSE.VERSION is 
'The license version.';

comment on column SAHARA.ER_LICENSE.CLIENT_ACCESS_LICENSES is 
'The number of client access licenses.';

comment on column SAHARA.ER_LICENSE.DURATION is 
'The duration for which the license is good.  This is a simple string collected during a poll of licenses.  It is not updated or re-calculated until another poll is performed, so its accuracy may be slightly suspect.';

comment on column SAHARA.ER_LICENSE.EXPIRATION is 
'The license expiration date and time.  This is always assumed to be stored in the time zone of the Enterprise Reporter server.  Permanent licenses have no expiration and are stored with the expiration as null.';

comment on column SAHARA.ER_LICENSE.IS_SITE_KEY is 
'A boolean flag that indicates if the license key is a site license key.';

comment on column SAHARA.ER_LICENSE.IS_EDITION_KEY is 
'A boolean flag that indicates if the license key is an editions key.';

comment on column SAHARA.ER_LICENSE.IS_POINT_PRODUCT_KEY is 
'A boolean flag that indicates if the license key is a point product key.';

alter table SAHARA.ER_LICENSE
   alter IS_SITE_KEY add constraint C_ERLICENSE_ISSITEKEY check (IS_SITE_KEY in ('T','F'));

alter table SAHARA.ER_LICENSE
   alter IS_EDITION_KEY add constraint C_ERLICENSE_ISEDITIONKEY check (IS_EDITION_KEY in ('T','F'));

alter table SAHARA.ER_LICENSE
   alter IS_POINT_PRODUCT_KEY add constraint C_ERLICENSE_ISPOINTPRODUCTKEY check (IS_POINT_PRODUCT_KEY in ('T','F'));

alter table SAHARA.ER_LICENSE
   add constraint PK_ER_LICENSE primary key (LICENSE_KEY);

grant SELECT on SAHARA.ER_LICENSE to SAHARA_APP;

/*==============================================================*/
/* Table: ETL_LOG                                               */
/*==============================================================*/
create table SAHARA.ETL_LOG 
(
   ETL_LOG_KEY          numeric(10)                    not null default autoincrement,
   BATCH_KEY            numeric(10)                    not null,
   "TIME"               timestamp                      not null,
   PROCEDURE_NAME       varchar(256)                   null,
   SEVERITY             char(1)                        not null,
   "MESSAGE"            long varchar                   not null
);

comment on table SAHARA.ETL_LOG is 
'ETL log generated on the VOM MS database during extraction. Messages are tagged with a timestamp, but queries should use the BATCH_KEY for ordering to guarantee retreival in the order in which messages were created per data source because multiple messages may have the same timestamp.';

comment on column SAHARA.ETL_LOG.ETL_LOG_KEY is 
'A unique database generated key used to order messages.  Using the timestamp alone for ordering is not good enough because mutliple messages can be created with the same timestamp.';

comment on column SAHARA.ETL_LOG.BATCH_KEY is 
'The identifier of the batch that logged the message.';

comment on column SAHARA.ETL_LOG."TIME" is 
'The time at which the message was logged in VOM MS database.';

comment on column SAHARA.ETL_LOG.PROCEDURE_NAME is 
'The (optional) name of the procedure logging the message.';

comment on column SAHARA.ETL_LOG.SEVERITY is 
'An indication of the type of message being logged.  Valid values:

D - debug
I - informational
W - warning
E - error';

comment on column SAHARA.ETL_LOG."MESSAGE" is 
'The text of the message.';

alter table SAHARA.ETL_LOG
   alter SEVERITY add constraint C_ETLLOG_SEVERITY check (SEVERITY in ('D','I','W','E'));

alter table SAHARA.ETL_LOG
   add constraint PK_ETL_LOG primary key (ETL_LOG_KEY);

/*==============================================================*/
/* Index: I_ETLLOG_BATCHKEY                                     */
/*==============================================================*/
create index I_ETLLOG_BATCHKEY on SAHARA.ETL_LOG (
BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_ETLLOG_PROCEDURENAME                                */
/*==============================================================*/
create index I_ETLLOG_PROCEDURENAME on SAHARA.ETL_LOG (
PROCEDURE_NAME ASC
);

/*==============================================================*/
/* Index: I_ETLLOG_SEVERITY                                     */
/*==============================================================*/
create index I_ETLLOG_SEVERITY on SAHARA.ETL_LOG (
SEVERITY ASC
);

grant SELECT on SAHARA.ETL_LOG to SAHARA_APP;

/*==============================================================*/
/* Table: ETL_LOG_                                              */
/*==============================================================*/
create table SAHARA_APP.ETL_LOG_ 
(
   BATCH_KEY            numeric(10)                    not null,
   "TIME"               timestamp                      not null,
   PROCEDURE_NAME       varchar(256)                   null,
   SEVERITY             char(1)                        not null,
   "MESSAGE"            long varchar                   not null
);

comment on table SAHARA_APP.ETL_LOG_ is 
'This is a staging table for loading data into warehouse table SAHARA.ETL_LOG.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  ETL log generated on the VOM MS database during extraction. Messages are tagged with a timestamp, but queries should use the BATCH_KEY for ordering to guarantee retreival in the order in which messages were created per data source because multiple messages may have the same timestamp.';

comment on column SAHARA_APP.ETL_LOG_.BATCH_KEY is 
'The identifier of the batch that logged the message.';

comment on column SAHARA_APP.ETL_LOG_."TIME" is 
'The time at which the message was logged in VOM MS database.';

comment on column SAHARA_APP.ETL_LOG_.PROCEDURE_NAME is 
'The (optional) name of the procedure logging the message.';

comment on column SAHARA_APP.ETL_LOG_.SEVERITY is 
'An indication of the type of message being logged.  Valid values:

D - debug
I - informational
W - warning
E - error';

comment on column SAHARA_APP.ETL_LOG_."MESSAGE" is 
'The text of the message.';

alter table SAHARA_APP.ETL_LOG_
   alter SEVERITY add constraint C_ETLLOG_SEVERITY check (SEVERITY in ('D','I','W','E'));

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.ETL_LOG_ to SAHARA;

/*==============================================================*/
/* Table: FILESYSTEM                                            */
/*==============================================================*/
create table SAHARA.FILESYSTEM 
(
   FILESYSTEM_KEY       numeric(10)                    not null default autoincrement,
   FILESYSTEM_OID       varchar(512)                   not null,
   VOLUME_KEY           numeric(10)                    null,
   HOST_KEY             numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   MOUNT_POINT          varchar(2048)                  null,
   DEVICE_PATH          varchar(2048)                  null,
   TYPE                 varchar(32)                    not null,
   IS_CREATED_FROM_VOLUME char(1)                        null,
   IS_CLUSTERED         char(1)                        null,
   IS_VCS_CVM           char(1)                        null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   IS_DIRECT_ATTACHED   char(1)                        null,
   IS_CFS_AWARE         char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   IS_EXPORTED          char(1)                        null,
   IS_CHILD             char(1)                        null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   IS_BASED_ON_NAS_SHARE char(1)                        null
);

comment on table SAHARA.FILESYSTEM is 
'This table contains file system attributes, like name, vendor, device path etc.';

comment on column SAHARA.FILESYSTEM.FILESYSTEM_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.FILESYSTEM.CC_STORAGE_URL is 
'The URL for the file system object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.FILESYSTEM.DISPLAY_NAME is 
'File system display name';

comment on column SAHARA.FILESYSTEM.ADVISORY_DISPLAY_NAME is 
'File system name discovered by CommandCentral Storage.';

comment on column SAHARA.FILESYSTEM.MOUNT_POINT is 
'Location where the file system is mounted.';

comment on column SAHARA.FILESYSTEM.DEVICE_PATH is 
'Local path to this file system device.';

comment on column SAHARA.FILESYSTEM.TYPE is 
'This can be raw, ufs, VxFS etc.';

comment on column SAHARA.FILESYSTEM.IS_CREATED_FROM_VOLUME is 
'This is reporting term.  File systems can be created from volumes like VxVM or from Sun disk slices.';

comment on column SAHARA.FILESYSTEM.IS_CLUSTERED is 
'True, if the file system is clustered.  False otherwise.';

comment on column SAHARA.FILESYSTEM.IS_VCS_CVM is 
'True, if the file system is a clustered file system, managed using VCS.  False otherwise.';

comment on column SAHARA.FILESYSTEM.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.FILESYSTEM.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.FILESYSTEM.IS_DIRECT_ATTACHED is 
'A boolean flag that indicates if the device is direct attached.

Possible values:
''T'' - the Filesystem is comsuming direct attached storage
''F'' - the Filesystem is not comsuming direct attached storage';

comment on column SAHARA.FILESYSTEM.IS_CFS_AWARE is 
'In CFS scenarios, value will be True for only one filesystem within the CFS setup.
False otherwise for all the remaining filesystems which are path of same CFS setup.';

comment on column SAHARA.FILESYSTEM.IS_BASED_ON_THIN is 
'A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the filesystem is based on thin lun
''F'' - the filesystem is not based on thin lun';

comment on column SAHARA.FILESYSTEM.IS_EXPORTED is 
'A boolean flag that indicates if the device is exported.

Possible values:
''T'' - the device is exported
''F'' - the device is not exported';

comment on column SAHARA.FILESYSTEM.IS_CHILD is 
'A boolean flag that indicates if the device is child.

Possible values:
''T'' - the device is child
''F'' - the device is not child';

comment on column SAHARA.FILESYSTEM.APPLICATION_BE is 
'Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.FILESYSTEM.APPLICATION_COMPOSITE_BE is 
'Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.FILESYSTEM.ORGANIZATION_BE is 
'Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.FILESYSTEM.ORGANIZATION_COMPOSITE_BE is 
'Composite organization business entity to which this object belongs.

This may be null.';

alter table SAHARA.FILESYSTEM
   alter IS_CREATED_FROM_VOLUME add constraint C_FILESYSTEM_SCRTDFRMVLM check (IS_CREATED_FROM_VOLUME is null or (IS_CREATED_FROM_VOLUME in ('T','F')));

alter table SAHARA.FILESYSTEM
   alter IS_CLUSTERED add constraint C_FILESYSTEM_ISCLUSTERED check (IS_CLUSTERED is null or (IS_CLUSTERED in ('T','F')));

alter table SAHARA.FILESYSTEM
   alter IS_VCS_CVM add constraint C_FILESYSTEM_ISVCSCVM check (IS_VCS_CVM is null or (IS_VCS_CVM in ('T','F')));

alter table SAHARA.FILESYSTEM
   alter T2_IS_CURRENT add constraint C_FILESYSTEM_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.FILESYSTEM
   alter T2_IS_DELETED add constraint C_FILESYSTEM_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.FILESYSTEM
   alter IS_DIRECT_ATTACHED add constraint C_FILESYSTEM_ISDIRECTATTACHED check (IS_DIRECT_ATTACHED is null or (IS_DIRECT_ATTACHED in ('T','F')));

alter table SAHARA.FILESYSTEM
   alter IS_CFS_AWARE add constraint C_FILESYSTEM_ISCFSAWARE check (IS_CFS_AWARE is null or (IS_CFS_AWARE in ('T','F')));

alter table SAHARA.FILESYSTEM
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA.FILESYSTEM
   alter IS_EXPORTED add constraint C_VOLUME_ISEXPORTED check (IS_EXPORTED is null or (IS_EXPORTED in ('T','F')));

alter table SAHARA.FILESYSTEM
   alter IS_CHILD add constraint C_VOLUME_ISCHILD check (IS_CHILD is null or (IS_CHILD in ('T','F')));

alter table SAHARA.FILESYSTEM
   add constraint PK_FILESYSTEM primary key (FILESYSTEM_KEY);

/*==============================================================*/
/* Index: I_FILESYSTEM_FILESYSTEMOID                            */
/*==============================================================*/
create index I_FILESYSTEM_FILESYSTEMOID on SAHARA.FILESYSTEM (
FILESYSTEM_OID ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_FILESYSTEMOID is 
'An index on Filesystem Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_VOLUMEKEY                                */
/*==============================================================*/
create index I_FILESYSTEM_VOLUMEKEY on SAHARA.FILESYSTEM (
VOLUME_KEY ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_VOLUMEKEY is 
'An index on Volume Key for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_HOSTKEY                                  */
/*==============================================================*/
create index I_FILESYSTEM_HOSTKEY on SAHARA.FILESYSTEM (
HOST_KEY ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_HOSTKEY is 
'An index on Host Key for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_DISPLAYNAME                              */
/*==============================================================*/
create index I_FILESYSTEM_DISPLAYNAME on SAHARA.FILESYSTEM (
DISPLAY_NAME ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_DVSRDSPLNM                               */
/*==============================================================*/
create index I_FILESYSTEM_DVSRDSPLNM on SAHARA.FILESYSTEM (
ADVISORY_DISPLAY_NAME ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_DVSRDSPLNM is 
'An index on Advisory Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_MOUNTPOINT                               */
/*==============================================================*/
create index I_FILESYSTEM_MOUNTPOINT on SAHARA.FILESYSTEM (
MOUNT_POINT ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_MOUNTPOINT is 
'An index on Mount Point for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_DEVICEPATH                               */
/*==============================================================*/
create index I_FILESYSTEM_DEVICEPATH on SAHARA.FILESYSTEM (
DEVICE_PATH ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_DEVICEPATH is 
'An index on Device Path for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_TYPE                                     */
/*==============================================================*/
create index I_FILESYSTEM_TYPE on SAHARA.FILESYSTEM (
TYPE ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_SCRTDFRMVLM                              */
/*==============================================================*/
create index I_FILESYSTEM_SCRTDFRMVLM on SAHARA.FILESYSTEM (
IS_CREATED_FROM_VOLUME ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_SCRTDFRMVLM is 
'An index on Is Created From Volume for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_ISCLUSTERED                              */
/*==============================================================*/
create index I_FILESYSTEM_ISCLUSTERED on SAHARA.FILESYSTEM (
IS_CLUSTERED ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_ISCLUSTERED is 
'An index on Is Clustered for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_ISVCSCVM                                 */
/*==============================================================*/
create index I_FILESYSTEM_ISVCSCVM on SAHARA.FILESYSTEM (
IS_VCS_CVM ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_ISVCSCVM is 
'An index on Is VCS CVM for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_T2SCRRNTFLSSTMK                          */
/*==============================================================*/
create index I_FILESYSTEM_T2SCRRNTFLSSTMK on SAHARA.FILESYSTEM (
T2_IS_CURRENT ASC,
FILESYSTEM_KEY ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_T2SCRRNTFLSSTMK is 
'An index on Type 2 - Is Current and Filesystem Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_FILESYSTEM_T2SCRRNTT2SDLTDD                         */
/*==============================================================*/
create index I_FILESYSTEM_T2SCRRNTT2SDLTDD on SAHARA.FILESYSTEM (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
FILESYSTEM_OID ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Filesystem Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_FILESYSTEM_T2ISDELETED                              */
/*==============================================================*/
create index I_FILESYSTEM_T2ISDELETED on SAHARA.FILESYSTEM (
T2_IS_DELETED ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_ISDIRECTATTACHED                         */
/*==============================================================*/
create index I_FILESYSTEM_ISDIRECTATTACHED on SAHARA.FILESYSTEM (
IS_DIRECT_ATTACHED ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_ISDIRECTATTACHED is 
'An index on Is Direct Attached for dimensional queries.';

/*==============================================================*/
/* Index: I_FILESYSTEM_ISCFSAWARE                               */
/*==============================================================*/
create index I_FILESYSTEM_ISCFSAWARE on SAHARA.FILESYSTEM (
IS_CFS_AWARE ASC
);

comment on index SAHARA.FILESYSTEM.I_FILESYSTEM_ISCFSAWARE is 
'An index on IS CFS AWARE for dimensional queries.';

grant SELECT on SAHARA.FILESYSTEM to SAHARA_APP;

/*==============================================================*/
/* Table: FILESYSTEM_                                           */
/*==============================================================*/
create table SAHARA_APP.FILESYSTEM_ 
(
   FILESYSTEM_OID       varchar(512)                   not null,
   VOLUME_OID           varchar(512)                   null,
   HOST_OID             varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   MOUNT_POINT          varchar(2048)                  null,
   DEVICE_PATH          varchar(2048)                  null,
   TYPE                 varchar(32)                    not null,
   IS_CREATED_FROM_VOLUME char(1)                        null,
   IS_CLUSTERED         char(1)                        null,
   IS_VCS_CVM           char(1)                        null,
   IS_DIRECT_ATTACHED   char(1)                        null,
   IS_CFS_AWARE         char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   IS_EXPORTED          char(1)                        null,
   IS_CHILD             char(1)                        null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   IS_BASED_ON_NAS_SHARE char(1)                        null
);

comment on table SAHARA_APP.FILESYSTEM_ is 
'This is a staging table for loading data into warehouse table SAHARA.FILESYSTEM.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains file system attributes, like name, vendor, device path etc.';

comment on column SAHARA_APP.FILESYSTEM_.FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.FILESYSTEM_.VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.FILESYSTEM_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.FILESYSTEM_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

File system display name';

comment on column SAHARA_APP.FILESYSTEM_.ADVISORY_DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column ADVISORY_DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

File system name discovered by CommandCentral Storage.';

comment on column SAHARA_APP.FILESYSTEM_.MOUNT_POINT is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column MOUNT_POINT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Location where the file system is mounted.';

comment on column SAHARA_APP.FILESYSTEM_.DEVICE_PATH is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column DEVICE_PATH.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Local path to this file system device.';

comment on column SAHARA_APP.FILESYSTEM_.TYPE is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

This can be raw, ufs, VxFS etc.';

comment on column SAHARA_APP.FILESYSTEM_.IS_CREATED_FROM_VOLUME is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_CREATED_FROM_VOLUME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

This is reporting term.  File systems can be created from volumes like VxVM or from Sun disk slices.';

comment on column SAHARA_APP.FILESYSTEM_.IS_CLUSTERED is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_CLUSTERED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

True, if the file system is clustered.  False otherwise.';

comment on column SAHARA_APP.FILESYSTEM_.IS_VCS_CVM is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_VCS_CVM.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

True, if the file system is a clustered file system, managed using VCS.  False otherwise.';

comment on column SAHARA_APP.FILESYSTEM_.IS_DIRECT_ATTACHED is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_DIRECT_ATTACHED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is direct attached.

Possible values:
''T'' - the Filesystem is comsuming direct attached storage
''F'' - the Filesystem is not comsuming direct attached storage';

comment on column SAHARA_APP.FILESYSTEM_.IS_CFS_AWARE is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_CFS_AWARE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

In CFS scenarios, value will be True for only one filesystem within the CFS setup.
False otherwise for all the remaining filesystems which are path of same CFS setup.';

comment on column SAHARA_APP.FILESYSTEM_.IS_BASED_ON_THIN is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_BASED_ON_THIN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the filesystem is based on thin lun
''F'' - the filesystem is not based on thin lun';

comment on column SAHARA_APP.FILESYSTEM_.IS_EXPORTED is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_EXPORTED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is exported.

Possible values:
''T'' - the device is exported
''F'' - the device is not exported';

comment on column SAHARA_APP.FILESYSTEM_.IS_CHILD is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column IS_CHILD.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is child.

Possible values:
''T'' - the device is child
''F'' - the device is not child';

comment on column SAHARA_APP.FILESYSTEM_.APPLICATION_BE is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column APPLICATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.FILESYSTEM_.APPLICATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column APPLICATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.FILESYSTEM_.ORGANIZATION_BE is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column ORGANIZATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.FILESYSTEM_.ORGANIZATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM, column ORGANIZATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite organization business entity to which this object belongs.

This may be null.';

alter table SAHARA_APP.FILESYSTEM_
   alter IS_CREATED_FROM_VOLUME add constraint C_FILESYSTEM_SCRTDFRMVLM check (IS_CREATED_FROM_VOLUME is null or (IS_CREATED_FROM_VOLUME in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   alter IS_CLUSTERED add constraint C_FILESYSTEM_ISCLUSTERED check (IS_CLUSTERED is null or (IS_CLUSTERED in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   alter IS_VCS_CVM add constraint C_FILESYSTEM_ISVCSCVM check (IS_VCS_CVM is null or (IS_VCS_CVM in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   alter IS_DIRECT_ATTACHED add constraint C_FILESYSTEM_ISDIRECTATTACHED check (IS_DIRECT_ATTACHED is null or (IS_DIRECT_ATTACHED in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   alter IS_CFS_AWARE add constraint C_FILESYSTEM_ISCFSAWARE check (IS_CFS_AWARE is null or (IS_CFS_AWARE in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   alter IS_EXPORTED add constraint C_VOLUME_ISEXPORTED check (IS_EXPORTED is null or (IS_EXPORTED in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   alter IS_CHILD add constraint C_VOLUME_ISCHILD check (IS_CHILD is null or (IS_CHILD in ('T','F')));

alter table SAHARA_APP.FILESYSTEM_
   add constraint PK_FILESYSTEM_ primary key (FILESYSTEM_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.FILESYSTEM_ to SAHARA;

/*==============================================================*/
/* Table: FILESYSTEM_BVIEW_NODE                                 */
/*==============================================================*/
create table SAHARA.FILESYSTEM_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   FILESYSTEM_KEY       numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.FILESYSTEM_BVIEW_NODE is 
'This is a join table between Filesystem objects and business view nodes.';

comment on column SAHARA.FILESYSTEM_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.FILESYSTEM_BVIEW_NODE.FILESYSTEM_KEY is 
'The unique identifier of the Filesystem object with which the business view node is associated.';

comment on column SAHARA.FILESYSTEM_BVIEW_NODE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the join table record.';

alter table SAHARA.FILESYSTEM_BVIEW_NODE
   add constraint PK_FILESYSTEM_BVIEW_NODE primary key (BVIEW_NODE_KEY, FILESYSTEM_KEY);

/*==============================================================*/
/* Index: I_FLSSTMBVWND_BVIEWKEY                                */
/*==============================================================*/
create index I_FLSSTMBVWND_BVIEWKEY on SAHARA.FILESYSTEM_BVIEW_NODE (
BVIEW_KEY ASC,
FILESYSTEM_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.FILESYSTEM_BVIEW_NODE.I_FLSSTMBVWND_BVIEWKEY is 
'An index on Business View Key for dimensional queries.';

grant SELECT on SAHARA.FILESYSTEM_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: FILESYSTEM_CAPACITY                                   */
/*==============================================================*/
create table SAHARA.FILESYSTEM_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   FILESYSTEM_KEY       numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   NON_SUPER_USER_CAPACITY numeric(22,9)                  not null default 0,
   FREE_CAPACITY        numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   USED_BY_APPS         numeric(22,9)                  not null default 0,
   USED_BY_NON_APPS     numeric(22,9)                  not null default 0,
   RESERVED_CAPACITY    numeric(22,9)                  not null default 0,
   PERCENT_FREE         numeric(7,4)                   not null default 0
);

comment on table SAHARA.FILESYSTEM_CAPACITY is 
'This table contains capacity values of file systems.  Use this table in conjunction with the FILESYSTEM table.';

comment on column SAHARA.FILESYSTEM_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.FILESYSTEM_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.FILESYSTEM_CAPACITY.CAPACITY is 
'The total capacity available on the file system.  This includes capacity for the super user.';

comment on column SAHARA.FILESYSTEM_CAPACITY.NON_SUPER_USER_CAPACITY is 
'The file system capacity available for applications.';

comment on column SAHARA.FILESYSTEM_CAPACITY.FREE_CAPACITY is 
'Capacity available to write data into.';

comment on column SAHARA.FILESYSTEM_CAPACITY.USED_CAPACITY is 
'Total file system capacity on hosts that has actual data written into.';

comment on column SAHARA.FILESYSTEM_CAPACITY.USED_BY_APPS is 
'File system capacity on hosts that is used by applications supported by CommandCentral Storage.
Storage is considered used when a file system has written actual data to the storage.';

comment on column SAHARA.FILESYSTEM_CAPACITY.USED_BY_NON_APPS is 
'File system capacity on hosts that is used by applications not supported by CommandCentral Storage.
Storage is considered used when a file system has written actual data to the storage.';

comment on column SAHARA.FILESYSTEM_CAPACITY.RESERVED_CAPACITY is 
'The amount of storage allocated to the files system that is reserved for use by the host''s operating system for managing the file system itself.  This is the part of the amount CommandCentral Storage 5.0 calls "Capacity" or "File System Capacity".  The equivalent term from CommandCentral Storage 4.3 is "Super User Capacity".';

comment on column SAHARA.FILESYSTEM_CAPACITY.PERCENT_FREE is 
'File system capacity available expressed as percentage value.';

alter table SAHARA.FILESYSTEM_CAPACITY
   alter CAPACITY add constraint C_FLSSTMCPCT_CAPACITY check (CAPACITY >= 0);

alter table SAHARA.FILESYSTEM_CAPACITY
   alter NON_SUPER_USER_CAPACITY add constraint C_FLSSTMCPCT_NNSPRSRCPCT check (NON_SUPER_USER_CAPACITY >= 0);

alter table SAHARA.FILESYSTEM_CAPACITY
   alter FREE_CAPACITY add constraint C_FLSSTMCPCT_FREECAPACITY check (FREE_CAPACITY >= 0);

alter table SAHARA.FILESYSTEM_CAPACITY
   alter USED_CAPACITY add constraint C_FLSSTMCPCT_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA.FILESYSTEM_CAPACITY
   alter USED_BY_APPS add constraint C_FLSSTMCPCT_USEDBYAPPS check (USED_BY_APPS >= 0);

alter table SAHARA.FILESYSTEM_CAPACITY
   alter USED_BY_NON_APPS add constraint C_FLSSTMCPCT_USEDBYNONAPPS check (USED_BY_NON_APPS >= 0);

alter table SAHARA.FILESYSTEM_CAPACITY
   alter RESERVED_CAPACITY add constraint C_FLSSTMCPCT_RESERVEDCAPACITY check (RESERVED_CAPACITY >= 0);

/*==============================================================*/
/* Index: I_FSCAPACITY_ROLLUPBATCHKEY                           */
/*==============================================================*/
create index I_FSCAPACITY_ROLLUPBATCHKEY on SAHARA.FILESYSTEM_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_FSCAPACITY_DATESKEY                                 */
/*==============================================================*/
create index I_FSCAPACITY_DATESKEY on SAHARA.FILESYSTEM_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_FSCAPACITY_FILESYSTEKEY                             */
/*==============================================================*/
create index I_FSCAPACITY_FILESYSTEKEY on SAHARA.FILESYSTEM_CAPACITY (
FILESYSTEM_KEY ASC
);

grant SELECT on SAHARA.FILESYSTEM_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: FILESYSTEM_CAPACITY_                                  */
/*==============================================================*/
create table SAHARA_APP.FILESYSTEM_CAPACITY_ 
(
   FILESYSTEM_OID       varchar(512)                   not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   NON_SUPER_USER_CAPACITY numeric(22,9)                  not null default 0,
   FREE_CAPACITY        numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   USED_BY_APPS         numeric(22,9)                  not null default 0,
   USED_BY_NON_APPS     numeric(22,9)                  not null default 0,
   PERCENT_FREE         numeric(7,4)                   not null default 0
);

comment on table SAHARA_APP.FILESYSTEM_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.FILESYSTEM_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains capacity values of file systems.  Use this table in conjunction with the FILESYSTEM table.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.CAPACITY is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM_CAPACITY, column CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total capacity available on the file system.  This includes capacity for the super user.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.NON_SUPER_USER_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM_CAPACITY, column NON_SUPER_USER_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The file system capacity available for applications.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.FREE_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM_CAPACITY, column FREE_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity available to write data into.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.USED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM_CAPACITY, column USED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total file system capacity on hosts that has actual data written into.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.USED_BY_APPS is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM_CAPACITY, column USED_BY_APPS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

File system capacity on hosts that is used by applications supported by CommandCentral Storage.
Storage is considered used when a file system has written actual data to the storage.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.USED_BY_NON_APPS is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM_CAPACITY, column USED_BY_NON_APPS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

File system capacity on hosts that is used by applications not supported by CommandCentral Storage.
Storage is considered used when a file system has written actual data to the storage.';

comment on column SAHARA_APP.FILESYSTEM_CAPACITY_.PERCENT_FREE is 
'This column is for staging data that will end up in table SAHARA.FILESYSTEM_CAPACITY, column PERCENT_FREE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

File system capacity available expressed as percentage value.';

alter table SAHARA_APP.FILESYSTEM_CAPACITY_
   alter CAPACITY add constraint C_FLSSTMCPCT_CAPACITY check (CAPACITY >= 0);

alter table SAHARA_APP.FILESYSTEM_CAPACITY_
   alter NON_SUPER_USER_CAPACITY add constraint C_FLSSTMCPCT_NNSPRSRCPCT check (NON_SUPER_USER_CAPACITY >= 0);

alter table SAHARA_APP.FILESYSTEM_CAPACITY_
   alter FREE_CAPACITY add constraint C_FLSSTMCPCT_FREECAPACITY check (FREE_CAPACITY >= 0);

alter table SAHARA_APP.FILESYSTEM_CAPACITY_
   alter USED_CAPACITY add constraint C_FLSSTMCPCT_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA_APP.FILESYSTEM_CAPACITY_
   alter USED_BY_APPS add constraint C_FLSSTMCPCT_USEDBYAPPS check (USED_BY_APPS >= 0);

alter table SAHARA_APP.FILESYSTEM_CAPACITY_
   alter USED_BY_NON_APPS add constraint C_FLSSTMCPCT_USEDBYNONAPPS check (USED_BY_NON_APPS >= 0);

alter table SAHARA_APP.FILESYSTEM_CAPACITY_
   add constraint PK_FILESYSTEM_CAPACITY_ primary key (FILESYSTEM_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.FILESYSTEM_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: FILE_CATEGORY                                         */
/*==============================================================*/
create table SAHARA.FILE_CATEGORY 
(
   FILE_CATEGORY_KEY    numeric(10)                    not null default autoincrement,
   FILE_CATEGORY_OID    varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DESCRIPTION          varchar(4000)                  null,
   FILE_EXTENSIONS      varchar(4000)                  null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.FILE_CATEGORY is 
'This is a dimension table for DM file categories.';

comment on column SAHARA.FILE_CATEGORY.FILE_CATEGORY_KEY is 
'A unique database generated identifier for the file category record used as a surrogate key.';

comment on column SAHARA.FILE_CATEGORY.FILE_CATEGORY_OID is 
'The unique identifier for the file category within CommandCentral storage.';

comment on column SAHARA.FILE_CATEGORY.DISPLAY_NAME is 
'The name of the file category.';

comment on column SAHARA.FILE_CATEGORY.DESCRIPTION is 
'The description of the file category.';

comment on column SAHARA.FILE_CATEGORY.FILE_EXTENSIONS is 
'The list of file extensions that define the files in the file category.';

comment on column SAHARA.FILE_CATEGORY.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.FILE_CATEGORY.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.FILE_CATEGORY
   alter T2_IS_CURRENT add constraint C_FILECATEGORY_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.FILE_CATEGORY
   alter T2_IS_DELETED add constraint C_FILECATEGORY_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.FILE_CATEGORY
   add constraint PK_FILE_CATEGORY primary key (FILE_CATEGORY_KEY);

grant SELECT on SAHARA.FILE_CATEGORY to SAHARA_APP;

/*==============================================================*/
/* Table: FILE_CATEGORY_                                        */
/*==============================================================*/
create table SAHARA_APP.FILE_CATEGORY_ 
(
   FILE_CATEGORY_OID    varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DESCRIPTION          varchar(4000)                  null,
   FILE_EXTENSIONS      varchar(4000)                  null
);

comment on table SAHARA_APP.FILE_CATEGORY_ is 
'This is a staging table for loading data into warehouse table SAHARA.FILE_CATEGORY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is a dimension table for DM file categories.';

comment on column SAHARA_APP.FILE_CATEGORY_.FILE_CATEGORY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.FILE_CATEGORY_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.FILE_CATEGORY, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The name of the file category.';

comment on column SAHARA_APP.FILE_CATEGORY_.DESCRIPTION is 
'This column is for staging data that will end up in table SAHARA.FILE_CATEGORY, column DESCRIPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The description of the file category.';

comment on column SAHARA_APP.FILE_CATEGORY_.FILE_EXTENSIONS is 
'This column is for staging data that will end up in table SAHARA.FILE_CATEGORY, column FILE_EXTENSIONS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The list of file extensions that define the files in the file category.';

alter table SAHARA_APP.FILE_CATEGORY_
   add constraint PK_FILE_CATEGORY_ primary key (FILE_CATEGORY_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.FILE_CATEGORY_ to SAHARA;

/*==============================================================*/
/* Table: FORECASTING_DIMENSION                                 */
/*==============================================================*/
create table SAHARA.FORECASTING_DIMENSION 
(
   DIMENSION            varchar(26)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   METAMODEL_PATH       varchar(1024)                  not null,
   FACT_TABLE_NAME      varchar(1024)                  not null,
   METAMODEL_KEY        varchar(1024)                  not null
);

comment on table SAHARA.FORECASTING_DIMENSION is 
'This table contains a list of dimensions that can support forcasting, it will also contain metamodel path for the corresponding dimension. This table is populated at install and upgrade.';

comment on column SAHARA.FORECASTING_DIMENSION.DIMENSION is 
'The name for the dimension.  This is the table name that represents the dimension.

This column is limited to 26 characters because the table name is expect to allow room for a primary key of the form table_name + "_KEY".';

comment on column SAHARA.FORECASTING_DIMENSION.DISPLAY_NAME is 
'A display name for the dimension to be used in GUI selections, etc.';

comment on column SAHARA.FORECASTING_DIMENSION.METAMODEL_PATH is 
'Metamodel path for the dimension to be used in forecasting and report selections.';

comment on column SAHARA.FORECASTING_DIMENSION.FACT_TABLE_NAME is 
'Fact table name for the object type to be used in forecasting and report selections.';

comment on column SAHARA.FORECASTING_DIMENSION.METAMODEL_KEY is 
'Metamodel key for the dimension to be used in forecasting and report selections.';

alter table SAHARA.FORECASTING_DIMENSION
   add constraint PK_FORECASTING_DIMENSION primary key (DIMENSION);

grant SELECT,REFERENCES on SAHARA.FORECASTING_DIMENSION to SAHARA_APP;

/*==============================================================*/
/* Table: HOST                                                  */
/*==============================================================*/
create table SAHARA.HOST 
(
   HOST_KEY             numeric(10)                    not null default autoincrement,
   HOST_OID             varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   OS_DETAILS           varchar(1024)                  null,
   OS_TYPE              varchar(256)                   null,
   IP_ADDRESS           varchar(4000)                  not null,
   VIRTUAL_SERVER       varchar(256)                   null,
   IS_USER_CREATED      char(1)                        null,
   IS_VIRTUAL           char(1)                        not null,
   IS_VIRTUAL_SERVER    char(1)                        not null,
   IS_CLUSTERED         char(1)                        not null,
   CLUSTER_TYPE         varchar(256)                   null,
   CLUSTER_VERSION      varchar(256)                   null,
   IS_ACTIVE            char(1)                        null,
   CLUSTER_NAME         varchar(256)                   null,
   IS_GUEST_INSTALLED   char(1)                        not null,
   GUEST_OS_TYPE        varchar(256)                   null,
   VIRTUALIZATION_TECHNOLOGY varchar(256)                   null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   SF_VERSION           varchar(256)                   null,
   MH_VERSION           varchar(256)                   null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   VOM_LICENSE_REQUIRED char(1)                        null,
   VM_HOST_OID          varchar(512)                   not null default '',
   PARENT_VIRTUAL_SERVER_OID varchar(512)                   null default '',
   CONTROL_GUEST        char(1)                        not null default 'F',
   CONFIGURATION_TYPE   varchar(256)                   null
);

comment on table SAHARA.HOST is 
'This table contains host attributes.';

comment on column SAHARA.HOST.HOST_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.HOST.CC_STORAGE_URL is 
'The URL for the host object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.HOST.DISPLAY_NAME is 
'Host display name';

comment on column SAHARA.HOST.OS_DETAILS is 
'Information about the host''s operating system.  This includes version plus patch or service pack number.';

comment on column SAHARA.HOST.OS_TYPE is 
'OS Type of the host:
Windows
Solaris
AIX
HP
Linux';

comment on column SAHARA.HOST.IP_ADDRESS is 
'One or more IP addresses for the host.  Multiple addresses are stored as a comma separated list.';

comment on column SAHARA.HOST.VIRTUAL_SERVER is 
'The name of the parent virualization server if this is a virtual host.';

comment on column SAHARA.HOST.IS_USER_CREATED is 
'Flag set to TRUE when host has created manually by user in CCStor GUI
Flag set to FALSE when host has been discovered by a CCStor agent

User created host capacity limitations:
CCStor is able to determine capacity allocated, but cannot determine capacity claimed or assigned.';

comment on column SAHARA.HOST.IS_VIRTUAL is 
'A boolean flag that indicates if the host is a virtual host.  This is T if the host is virtual; F otherwise.';

comment on column SAHARA.HOST.IS_VIRTUAL_SERVER is 
'A boolean flag that indicates if the host is a server for one or more virtual host.  This is T if the host is a virtual server; F otherwise.';

comment on column SAHARA.HOST.IS_CLUSTERED is 
'A boolean flag that indicates if the host is part of a cluster.  This is T if the host is part of cluster; F otherwise.';

comment on column SAHARA.HOST.CLUSTER_TYPE is 
'The type of the cluster if this host is part of cluster.';

comment on column SAHARA.HOST.CLUSTER_VERSION is 
'The version of the cluster if this host is part of cluster.';

comment on column SAHARA.HOST.IS_ACTIVE is 
'Flag set to TRUE when CCStor agent discovering the host are not in state stopped or detached.
Flag set to FALSE when CCStor agent discovering the host are in state stopped or detached.';

comment on column SAHARA.HOST.CLUSTER_NAME is 
'Cluster display name';

comment on column SAHARA.HOST.IS_GUEST_INSTALLED is 
'A boolean flag that indicates if the guest OS is installed in the virtual machine.  This is T if the guest OS is installed in vitual machine; F otherwise.';

comment on column SAHARA.HOST.GUEST_OS_TYPE is 
'The OS Type is specified at the time of the virtual machine creation.  This does not necessarily mean that an OS is installed in the machine.

OS Type of the virtual machine:
Windows
Solaris
AIX
HP
Linux';

comment on column SAHARA.HOST.VIRTUALIZATION_TECHNOLOGY is 
'Information about the virtual machine''s virtualization technology.

Virtualization Technology of the virtual machine:
VMWare
LPAR
Zone';

comment on column SAHARA.HOST.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.HOST.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.HOST.SF_VERSION is 
'SF Version of the host.';

comment on column SAHARA.HOST.MH_VERSION is 
'MH Version of the host.';

comment on column SAHARA.HOST.APPLICATION_BE is 
'Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.HOST.APPLICATION_COMPOSITE_BE is 
'Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.HOST.ORGANIZATION_BE is 
'Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.HOST.ORGANIZATION_COMPOSITE_BE is 
'Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.HOST.VOM_LICENSE_REQUIRED is 
'Specifies whether a customer requires a VOM Advanced license to manage this host. A host needs a VOM Advanced license if, for example, has no SF products on it, or has SF non-Enterprise license etc.';

comment on column SAHARA.HOST.VM_HOST_OID is 
'A unique object identifier for all hosts. This column will have HOST_ID for virtual as well as physical machines. This column will be utilized for cross-launh URL. ';

comment on column SAHARA.HOST.PARENT_VIRTUAL_SERVER_OID is 
'A unique object identifier for all parent virtual server. This column will have parent virtual server''s OID for all the virtual physical machines.';

comment on column SAHARA.HOST.CONTROL_GUEST is 
'A Value of T indicates that Guest OS is either VIO Server  for LPAR or 
Control  Domain for LDOM';

comment on column SAHARA.HOST.CONFIGURATION_TYPE is 
'Type of host configuration viz  AGENT, AGENTLESS, VSERVER';

alter table SAHARA.HOST
   alter IS_USER_CREATED add constraint C_HOST_ISUSERCREATED check (IS_USER_CREATED is null or (IS_USER_CREATED in ('T','F')));

alter table SAHARA.HOST
   alter IS_VIRTUAL add constraint C_HOST_ISVIRTUAL check (IS_VIRTUAL in ('T','F'));

alter table SAHARA.HOST
   alter IS_VIRTUAL_SERVER add constraint C_HOST_ISVIRTUALSERVER check (IS_VIRTUAL_SERVER in ('T','F'));

alter table SAHARA.HOST
   alter IS_CLUSTERED add constraint C_HOST_ISCLUSTERED check (IS_CLUSTERED in ('T','F'));

alter table SAHARA.HOST
   alter IS_ACTIVE add constraint CKC_IS_ACTIVE_HOST check (IS_ACTIVE is null or (IS_ACTIVE in ('T','F')));

alter table SAHARA.HOST
   alter IS_GUEST_INSTALLED add constraint C_HOST_ISGUESTINSTALLED check (IS_GUEST_INSTALLED in ('T','F'));

alter table SAHARA.HOST
   alter T2_IS_CURRENT add constraint C_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.HOST
   alter T2_IS_DELETED add constraint C_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.HOST
   alter CONTROL_GUEST add constraint C_HOST_CONTROLGUEST check (CONTROL_GUEST in ('T','F'));

alter table SAHARA.HOST
   add constraint PK_HOST primary key (HOST_KEY);

/*==============================================================*/
/* Index: I_HOST_HOSTOID                                        */
/*==============================================================*/
create index I_HOST_HOSTOID on SAHARA.HOST (
HOST_OID ASC
);

comment on index SAHARA.HOST.I_HOST_HOSTOID is 
'An index on Host Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_DISPLAYNAME                                    */
/*==============================================================*/
create index I_HOST_DISPLAYNAME on SAHARA.HOST (
DISPLAY_NAME ASC
);

comment on index SAHARA.HOST.I_HOST_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_OSDETAILS                                      */
/*==============================================================*/
create index I_HOST_OSDETAILS on SAHARA.HOST (
OS_DETAILS ASC
);

comment on index SAHARA.HOST.I_HOST_OSDETAILS is 
'An index on OS Details for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_OSTYPE                                         */
/*==============================================================*/
create index I_HOST_OSTYPE on SAHARA.HOST (
OS_TYPE ASC
);

comment on index SAHARA.HOST.I_HOST_OSTYPE is 
'An index on OS Type for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_IPADDRESS                                      */
/*==============================================================*/
create index I_HOST_IPADDRESS on SAHARA.HOST (
IP_ADDRESS ASC
);

comment on index SAHARA.HOST.I_HOST_IPADDRESS is 
'An index on IP Address(es) for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_VIRTUALSERVER                                  */
/*==============================================================*/
create index I_HOST_VIRTUALSERVER on SAHARA.HOST (
VIRTUAL_SERVER ASC
);

comment on index SAHARA.HOST.I_HOST_VIRTUALSERVER is 
'An index on Virtual Server for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ISUSERCREATED                                  */
/*==============================================================*/
create index I_HOST_ISUSERCREATED on SAHARA.HOST (
IS_USER_CREATED ASC
);

comment on index SAHARA.HOST.I_HOST_ISUSERCREATED is 
'An index on Is User Created for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ISVIRTUAL                                      */
/*==============================================================*/
create index I_HOST_ISVIRTUAL on SAHARA.HOST (
IS_VIRTUAL ASC
);

comment on index SAHARA.HOST.I_HOST_ISVIRTUAL is 
'An index on Is Virtual for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ISVIRTUALSERVER                                */
/*==============================================================*/
create index I_HOST_ISVIRTUALSERVER on SAHARA.HOST (
IS_VIRTUAL_SERVER ASC
);

comment on index SAHARA.HOST.I_HOST_ISVIRTUALSERVER is 
'An index on Is Virtual Server for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ISCLUSTERED                                    */
/*==============================================================*/
create index I_HOST_ISCLUSTERED on SAHARA.HOST (
IS_CLUSTERED ASC
);

comment on index SAHARA.HOST.I_HOST_ISCLUSTERED is 
'An index on Is Clustered for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_CLUSTERTYPE                                    */
/*==============================================================*/
create index I_HOST_CLUSTERTYPE on SAHARA.HOST (
CLUSTER_TYPE ASC
);

comment on index SAHARA.HOST.I_HOST_CLUSTERTYPE is 
'An index on Cluster Type for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_CLUSTERVERSION                                 */
/*==============================================================*/
create index I_HOST_CLUSTERVERSION on SAHARA.HOST (
CLUSTER_VERSION ASC
);

comment on index SAHARA.HOST.I_HOST_CLUSTERVERSION is 
'An index on Cluster Version for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ISACTIVE                                       */
/*==============================================================*/
create index I_HOST_ISACTIVE on SAHARA.HOST (
IS_ACTIVE ASC
);

comment on index SAHARA.HOST.I_HOST_ISACTIVE is 
'An index on Is Active for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_CLUSTERNAME                                    */
/*==============================================================*/
create index I_HOST_CLUSTERNAME on SAHARA.HOST (
CLUSTER_NAME ASC
);

comment on index SAHARA.HOST.I_HOST_CLUSTERNAME is 
'An index on Cluster Name for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ISGUESTINSTALLED                               */
/*==============================================================*/
create index I_HOST_ISGUESTINSTALLED on SAHARA.HOST (
IS_GUEST_INSTALLED ASC
);

comment on index SAHARA.HOST.I_HOST_ISGUESTINSTALLED is 
'An index on Is Guest Installed for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_GUESTOSTYPE                                    */
/*==============================================================*/
create index I_HOST_GUESTOSTYPE on SAHARA.HOST (
GUEST_OS_TYPE ASC
);

comment on index SAHARA.HOST.I_HOST_GUESTOSTYPE is 
'An index on Guest OS Type for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_VRTLZTNTCHNLG                                  */
/*==============================================================*/
create index I_HOST_VRTLZTNTCHNLG on SAHARA.HOST (
VIRTUALIZATION_TECHNOLOGY ASC
);

comment on index SAHARA.HOST.I_HOST_VRTLZTNTCHNLG is 
'An index on Virtualization Technology for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_T2ISCURRENTHOSTKEY                             */
/*==============================================================*/
create index I_HOST_T2ISCURRENTHOSTKEY on SAHARA.HOST (
T2_IS_CURRENT ASC,
HOST_KEY ASC
);

comment on index SAHARA.HOST.I_HOST_T2ISCURRENTHOSTKEY is 
'An index on Type 2 - Is Current and Host Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_HOST_T2SCRRNTT2SDLTDD                               */
/*==============================================================*/
create index I_HOST_T2SCRRNTT2SDLTDD on SAHARA.HOST (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
HOST_OID ASC
);

comment on index SAHARA.HOST.I_HOST_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Host Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_HOST_T2ISDELETED                                    */
/*==============================================================*/
create index I_HOST_T2ISDELETED on SAHARA.HOST (
T2_IS_DELETED ASC
);

comment on index SAHARA.HOST.I_HOST_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_SFVERSION                                      */
/*==============================================================*/
create index I_HOST_SFVERSION on SAHARA.HOST (
SF_VERSION ASC
);

comment on index SAHARA.HOST.I_HOST_SFVERSION is 
'An index on SF Version for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_MHVERSION                                      */
/*==============================================================*/
create index I_HOST_MHVERSION on SAHARA.HOST (
MH_VERSION ASC
);

comment on index SAHARA.HOST.I_HOST_MHVERSION is 
'An index on MH Version for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_APPLICATIONBE                                  */
/*==============================================================*/
create index I_HOST_APPLICATIONBE on SAHARA.HOST (
APPLICATION_BE ASC
);

comment on index SAHARA.HOST.I_HOST_APPLICATIONBE is 
'An index on Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_APPLICATIONCOMPOSITEBE                         */
/*==============================================================*/
create index I_HOST_APPLICATIONCOMPOSITEBE on SAHARA.HOST (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.HOST.I_HOST_APPLICATIONCOMPOSITEBE is 
'An index on Composite Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ORGANIZATIONBE                                 */
/*==============================================================*/
create index I_HOST_ORGANIZATIONBE on SAHARA.HOST (
ORGANIZATION_BE ASC
);

comment on index SAHARA.HOST.I_HOST_ORGANIZATIONBE is 
'An index on Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_ORGANIZATIONCOMPOSITEBE                        */
/*==============================================================*/
create index I_HOST_ORGANIZATIONCOMPOSITEBE on SAHARA.HOST (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.HOST.I_HOST_ORGANIZATIONCOMPOSITEBE is 
'An index on Composite Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_VOMLICENSEREQUIRED                             */
/*==============================================================*/
create index I_HOST_VOMLICENSEREQUIRED on SAHARA.HOST (
VOM_LICENSE_REQUIRED ASC
);

comment on index SAHARA.HOST.I_HOST_VOMLICENSEREQUIRED is 
'An index on VOM License Required for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_VMHOSTOID                                      */
/*==============================================================*/
create index I_HOST_VMHOSTOID on SAHARA.HOST (
VM_HOST_OID ASC
);

comment on index SAHARA.HOST.I_HOST_VMHOSTOID is 
'An index on VM Host Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_PARENTVIRTUALSERVEROID                         */
/*==============================================================*/
create index I_HOST_PARENTVIRTUALSERVEROID on SAHARA.HOST (
PARENT_VIRTUAL_SERVER_OID ASC
);

comment on index SAHARA.HOST.I_HOST_PARENTVIRTUALSERVEROID is 
'An index on Parent Virtual Server OID for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_CONTROLGUEST                                   */
/*==============================================================*/
create index I_HOST_CONTROLGUEST on SAHARA.HOST (
CONTROL_GUEST ASC
);

comment on index SAHARA.HOST.I_HOST_CONTROLGUEST is 
'An index on Control Guest for dimensional queries.';

/*==============================================================*/
/* Index: I_HOST_CONFIGURATIONTYPE                              */
/*==============================================================*/
create index I_HOST_CONFIGURATIONTYPE on SAHARA.HOST (
CONFIGURATION_TYPE ASC
);

comment on index SAHARA.HOST.I_HOST_CONFIGURATIONTYPE is 
'An index on Configuration Type for dimensional queries.';

grant SELECT on SAHARA.HOST to SAHARA_APP;

/*==============================================================*/
/* Table: HOST_                                                 */
/*==============================================================*/
create table SAHARA_APP.HOST_ 
(
   HOST_OID             varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   OS_DETAILS           varchar(1024)                  null,
   OS_TYPE              varchar(256)                   null,
   IP_ADDRESS           varchar(4000)                  not null,
   VIRTUAL_SERVER       varchar(256)                   null,
   IS_USER_CREATED      char(1)                        null,
   IS_VIRTUAL           char(1)                        not null,
   IS_VIRTUAL_SERVER    char(1)                        not null,
   IS_CLUSTERED         char(1)                        not null,
   CLUSTER_TYPE         varchar(256)                   null,
   CLUSTER_VERSION      varchar(256)                   null,
   IS_ACTIVE            char(1)                        null,
   CLUSTER_NAME         varchar(256)                   null,
   IS_GUEST_INSTALLED   char(1)                        not null,
   GUEST_OS_TYPE        varchar(256)                   null,
   VIRTUALIZATION_TECHNOLOGY varchar(256)                   null,
   MH_VERSION           varchar(256)                   null,
   SF_VERSION           varchar(256)                   null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   VOM_LICENSE_REQUIRED char(1)                        null,
   VM_HOST_OID          varchar(512)                   not null default '',
   PARENT_VIRTUAL_SERVER_OID varchar(512)                   null default '',
   CONTROL_GUEST        char(1)                        not null default 'F',
   CONFIGURATION_TYPE   varchar(256)                   null
);

comment on table SAHARA_APP.HOST_ is 
'This is a staging table for loading data into warehouse table SAHARA.HOST.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains host attributes.';

comment on column SAHARA_APP.HOST_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.HOST_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.HOST, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Host display name';

comment on column SAHARA_APP.HOST_.OS_DETAILS is 
'This column is for staging data that will end up in table SAHARA.HOST, column OS_DETAILS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Information about the host''s operating system.  This includes version plus patch or service pack number.';

comment on column SAHARA_APP.HOST_.OS_TYPE is 
'This column is for staging data that will end up in table SAHARA.HOST, column OS_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

OS Type of the host:
Windows
Solaris
AIX
HP
Linux';

comment on column SAHARA_APP.HOST_.IP_ADDRESS is 
'This column is for staging data that will end up in table SAHARA.HOST, column IP_ADDRESS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

One or more IP addresses for the host.  Multiple addresses are stored as a comma separated list.';

comment on column SAHARA_APP.HOST_.VIRTUAL_SERVER is 
'This column is for staging data that will end up in table SAHARA.HOST, column VIRTUAL_SERVER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The name of the parent virualization server if this is a virtual host.';

comment on column SAHARA_APP.HOST_.IS_USER_CREATED is 
'This column is for staging data that will end up in table SAHARA.HOST, column IS_USER_CREATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Flag set to TRUE when host has created manually by user in CCStor GUI
Flag set to FALSE when host has been discovered by a CCStor agent

User created host capacity limitations:
CCStor is able to determine capacity allocated, but cannot determine capacity claimed or assigned.';

comment on column SAHARA_APP.HOST_.IS_VIRTUAL is 
'This column is for staging data that will end up in table SAHARA.HOST, column IS_VIRTUAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the host is a virtual host.  This is T if the host is virtual; F otherwise.';

comment on column SAHARA_APP.HOST_.IS_VIRTUAL_SERVER is 
'This column is for staging data that will end up in table SAHARA.HOST, column IS_VIRTUAL_SERVER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the host is a server for one or more virtual host.  This is T if the host is a virtual server; F otherwise.';

comment on column SAHARA_APP.HOST_.IS_CLUSTERED is 
'This column is for staging data that will end up in table SAHARA.HOST, column IS_CLUSTERED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the host is part of a cluster.  This is T if the host is part of cluster; F otherwise.';

comment on column SAHARA_APP.HOST_.CLUSTER_TYPE is 
'This column is for staging data that will end up in table SAHARA.HOST, column CLUSTER_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The type of the cluster if this host is part of cluster.';

comment on column SAHARA_APP.HOST_.CLUSTER_VERSION is 
'This column is for staging data that will end up in table SAHARA.HOST, column CLUSTER_VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The version of the cluster if this host is part of cluster.';

comment on column SAHARA_APP.HOST_.IS_ACTIVE is 
'This column is for staging data that will end up in table SAHARA.HOST, column IS_ACTIVE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Flag set to TRUE when CCStor agent discovering the host are not in state stopped or detached.
Flag set to FALSE when CCStor agent discovering the host are in state stopped or detached.';

comment on column SAHARA_APP.HOST_.CLUSTER_NAME is 
'This column is for staging data that will end up in table SAHARA.HOST, column CLUSTER_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Cluster display name';

comment on column SAHARA_APP.HOST_.IS_GUEST_INSTALLED is 
'This column is for staging data that will end up in table SAHARA.HOST, column IS_GUEST_INSTALLED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the guest OS is installed in the virtual machine.  This is T if the guest OS is installed in vitual machine; F otherwise.';

comment on column SAHARA_APP.HOST_.GUEST_OS_TYPE is 
'This column is for staging data that will end up in table SAHARA.HOST, column GUEST_OS_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The OS Type is specified at the time of the virtual machine creation.  This does not necessarily mean that an OS is installed in the machine.

OS Type of the virtual machine:
Windows
Solaris
AIX
HP
Linux';

comment on column SAHARA_APP.HOST_.VIRTUALIZATION_TECHNOLOGY is 
'This column is for staging data that will end up in table SAHARA.HOST, column VIRTUALIZATION_TECHNOLOGY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Information about the virtual machine''s virtualization technology.

Virtualization Technology of the virtual machine:
VMWare
LPAR
Zone';

comment on column SAHARA_APP.HOST_.MH_VERSION is 
'This column is for staging data that will end up in table SAHARA.HOST, column MH_VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

MH Version of the host.';

comment on column SAHARA_APP.HOST_.SF_VERSION is 
'This column is for staging data that will end up in table SAHARA.HOST, column SF_VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

SF Version of the host.';

comment on column SAHARA_APP.HOST_.APPLICATION_BE is 
'This column is for staging data that will end up in table SAHARA.HOST, column APPLICATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.HOST_.APPLICATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.HOST, column APPLICATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.HOST_.ORGANIZATION_BE is 
'This column is for staging data that will end up in table SAHARA.HOST, column ORGANIZATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.HOST_.ORGANIZATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.HOST, column ORGANIZATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.HOST_.VOM_LICENSE_REQUIRED is 
'Specifies whether a customer requires a VOM Advanced license to manage this host. A host needs a VOM Advanced license if, for example, has no SF products on it, or has SF non-Enterprise license etc.';

comment on column SAHARA_APP.HOST_.VM_HOST_OID is 
'A unique object identifier for all hosts. This column will have HOST_ID for virtual as well as physical machines. This column will be utilized for cross-launh URL. 

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.HOST_.PARENT_VIRTUAL_SERVER_OID is 
'A unique object identifier for all parent virtual server. This column will have parent virtual server''s OID for all the virtual physical machines.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.HOST_.CONTROL_GUEST is 
'A Value of T indicates that Guest os is VIO Server or LDOM contorl domain
';

comment on column SAHARA_APP.HOST_.CONFIGURATION_TYPE is 
'Type of host configuration viz  AGENT, AGENTLESS, VSERVER';

alter table SAHARA_APP.HOST_
   alter IS_USER_CREATED add constraint C_HOST_ISUSERCREATED check (IS_USER_CREATED is null or (IS_USER_CREATED in ('T','F')));

alter table SAHARA_APP.HOST_
   alter IS_VIRTUAL add constraint C_HOST_ISVIRTUAL check (IS_VIRTUAL in ('T','F'));

alter table SAHARA_APP.HOST_
   alter IS_VIRTUAL_SERVER add constraint C_HOST_ISVIRTUALSERVER check (IS_VIRTUAL_SERVER in ('T','F'));

alter table SAHARA_APP.HOST_
   alter IS_CLUSTERED add constraint C_HOST_ISCLUSTERED check (IS_CLUSTERED in ('T','F'));

alter table SAHARA_APP.HOST_
   alter IS_ACTIVE add constraint CKC_IS_ACTIVE_HOST_ check (IS_ACTIVE is null or (IS_ACTIVE in ('T','F')));

alter table SAHARA_APP.HOST_
   alter IS_GUEST_INSTALLED add constraint C_HOST_ISGUESTINSTALLED check (IS_GUEST_INSTALLED in ('T','F'));

alter table SAHARA_APP.HOST_
   alter CONTROL_GUEST add constraint C_HOST_CONTROLGUEST check (CONTROL_GUEST in ('T','F'));

alter table SAHARA_APP.HOST_
   add constraint PK_HOST_ primary key (HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.HOST_ to SAHARA;

/*==============================================================*/
/* Table: HOST_ARRAY_CAPACITY                                   */
/*==============================================================*/
create table SAHARA.HOST_ARRAY_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null,
   ALLOCATED_CAPACITY   numeric(22,9)                  not null default 0,
   CLAIMED_CAPACITY     numeric(22,9)                  not null default 0,
   UNKNOWN_CAPACITY     numeric(22,9)                  not null default 0,
   UNCLAIMED_CAPACITY   numeric(22,9)                  not null default 0
);

comment on table SAHARA.HOST_ARRAY_CAPACITY is 
'This table contains capacity consumed, correlated for array and host.
Use this table to determine which host uses capacity from which array.';

comment on column SAHARA.HOST_ARRAY_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.HOST_ARRAY_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.HOST_ARRAY_CAPACITY.ALLOCATED_CAPACITY is 
'Capacity allocated to a host from an array.';

comment on column SAHARA.HOST_ARRAY_CAPACITY.CLAIMED_CAPACITY is 
'Capacity claimed by a host from an array.  User defined hosts will always have 0 for this, because CCStor cannot
determine what the O/S has claimed.';

comment on column SAHARA.HOST_ARRAY_CAPACITY.UNKNOWN_CAPACITY is 
'Allocated - Claimed - Unclaimed.  This value is usually 0 for all auto discovered hosts.
For user created hosts, unknown capacity = Allocated capacity.';

comment on column SAHARA.HOST_ARRAY_CAPACITY.UNCLAIMED_CAPACITY is 
'Allocated - Claimed.  This value is 0 for all user created hosts.';

alter table SAHARA.HOST_ARRAY_CAPACITY
   alter ALLOCATED_CAPACITY add constraint C_HSTRRCPCT_ALLOCATEDCAPACITY check (ALLOCATED_CAPACITY >= 0);

alter table SAHARA.HOST_ARRAY_CAPACITY
   alter CLAIMED_CAPACITY add constraint C_HSTRRCPCT_CLAIMEDCAPACITY check (CLAIMED_CAPACITY >= 0);

alter table SAHARA.HOST_ARRAY_CAPACITY
   alter UNKNOWN_CAPACITY add constraint C_HSTRRCPCT_UNKNOWNCAPACITY check (UNKNOWN_CAPACITY >= 0);

alter table SAHARA.HOST_ARRAY_CAPACITY
   alter UNCLAIMED_CAPACITY add constraint C_HSTRRCPCT_UNCLAIMEDCAPACITY check (UNCLAIMED_CAPACITY >= 0);

/*==============================================================*/
/* Index: I_HOSTARRAYCAP_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_HOSTARRAYCAP_ROLLUPBATCHKEY on SAHARA.HOST_ARRAY_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_HOSTARRCAP_DATESKEY                                 */
/*==============================================================*/
create index I_HOSTARRCAP_DATESKEY on SAHARA.HOST_ARRAY_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_HOSTARRCAP_HOSTKEY                                  */
/*==============================================================*/
create index I_HOSTARRCAP_HOSTKEY on SAHARA.HOST_ARRAY_CAPACITY (
HOST_KEY ASC
);

/*==============================================================*/
/* Index: I_HOSTARRCAP_ARRAYKEY                                 */
/*==============================================================*/
create index I_HOSTARRCAP_ARRAYKEY on SAHARA.HOST_ARRAY_CAPACITY (
ARRAY_KEY ASC
);

grant SELECT on SAHARA.HOST_ARRAY_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: HOST_ARRAY_CAPACITY_                                  */
/*==============================================================*/
create table SAHARA_APP.HOST_ARRAY_CAPACITY_ 
(
   HOST_OID             varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   not null,
   ALLOCATED_CAPACITY   numeric(22,9)                  not null default 0,
   CLAIMED_CAPACITY     numeric(22,9)                  not null default 0,
   UNKNOWN_CAPACITY     numeric(22,9)                  not null default 0,
   UNCLAIMED_CAPACITY   numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.HOST_ARRAY_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.HOST_ARRAY_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains capacity consumed, correlated for array and host.
Use this table to determine which host uses capacity from which array.';

comment on column SAHARA_APP.HOST_ARRAY_CAPACITY_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.HOST_ARRAY_CAPACITY_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.HOST_ARRAY_CAPACITY_.ALLOCATED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.HOST_ARRAY_CAPACITY, column ALLOCATED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity allocated to a host from an array.';

comment on column SAHARA_APP.HOST_ARRAY_CAPACITY_.CLAIMED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.HOST_ARRAY_CAPACITY, column CLAIMED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Capacity claimed by a host from an array.  User defined hosts will always have 0 for this, because CCStor cannot
determine what the O/S has claimed.';

comment on column SAHARA_APP.HOST_ARRAY_CAPACITY_.UNKNOWN_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.HOST_ARRAY_CAPACITY, column UNKNOWN_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Allocated - Claimed - Unclaimed.  This value is usually 0 for all auto discovered hosts.
For user created hosts, unknown capacity = Allocated capacity.';

comment on column SAHARA_APP.HOST_ARRAY_CAPACITY_.UNCLAIMED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.HOST_ARRAY_CAPACITY, column UNCLAIMED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Allocated - Claimed.  This value is 0 for all user created hosts.';

alter table SAHARA_APP.HOST_ARRAY_CAPACITY_
   alter ALLOCATED_CAPACITY add constraint C_HSTRRCPCT_ALLOCATEDCAPACITY check (ALLOCATED_CAPACITY >= 0);

alter table SAHARA_APP.HOST_ARRAY_CAPACITY_
   alter CLAIMED_CAPACITY add constraint C_HSTRRCPCT_CLAIMEDCAPACITY check (CLAIMED_CAPACITY >= 0);

alter table SAHARA_APP.HOST_ARRAY_CAPACITY_
   alter UNKNOWN_CAPACITY add constraint C_HSTRRCPCT_UNKNOWNCAPACITY check (UNKNOWN_CAPACITY >= 0);

alter table SAHARA_APP.HOST_ARRAY_CAPACITY_
   alter UNCLAIMED_CAPACITY add constraint C_HSTRRCPCT_UNCLAIMEDCAPACITY check (UNCLAIMED_CAPACITY >= 0);

alter table SAHARA_APP.HOST_ARRAY_CAPACITY_
   add constraint PK_HOST_ARRAY_CAPACITY_ primary key (HOST_OID, ARRAY_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.HOST_ARRAY_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: HOST_BVIEW_NODE                                       */
/*==============================================================*/
create table SAHARA.HOST_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.HOST_BVIEW_NODE is 
'This is a join table between host objects and business view nodes.';

comment on column SAHARA.HOST_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.HOST_BVIEW_NODE.HOST_KEY is 
'The unique identifier of the host object with which the business view node is associated.';

comment on column SAHARA.HOST_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.HOST_BVIEW_NODE
   add constraint PK_HOST_BVIEW_NODE primary key (BVIEW_NODE_KEY, HOST_KEY);

/*==============================================================*/
/* Index: I_HOSTBVIEWNODE_BVIEWHOST                             */
/*==============================================================*/
create unique index I_HOSTBVIEWNODE_BVIEWHOST on SAHARA.HOST_BVIEW_NODE (
BVIEW_KEY ASC,
HOST_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.HOST_BVIEW_NODE.I_HOSTBVIEWNODE_BVIEWHOST is 
'This index exists to accelerate queries for which hosts are associated with a particular business view.';

grant SELECT on SAHARA.HOST_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: HOST_CAPACITY                                         */
/*==============================================================*/
create table SAHARA.HOST_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   ALLOCATED            numeric(22,9)                  not null default 0,
   CLAIMED              numeric(22,9)                  not null default 0,
   UNCLAIMED            numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED numeric(22,9)                  not null default 0,
   PERCENT_ALLOCATED_CLAIMED numeric(7,4)                   not null default 0,
   PERCENT_ALLOCATED_UNCLAIMED numeric(7,4)                   not null default 0,
   PERCENT_ALLOCATED_UNDISCOVERED numeric(7,4)                   not null default 0,
   ALLOCATED_FCFIBRE    numeric(22,9)                  not null default 0,
   ALLOCATED_NONFCFIBRE numeric(22,9)                  not null default 0,
   ALLOCATED_IDE        numeric(22,9)                  not null default 0,
   ALLOCATED_DAS        numeric(22,9)                  not null default 0,
   ALLOCATED_ISCSI      numeric(22,9)                  not null default 0,
   ALLOCATED_FILESTORAGE numeric(22,9)                  not null default 0,
   CLAIMED_FCFIBRE      numeric(22,9)                  not null default 0,
   CLAIMED_NONFCFIBRE   numeric(22,9)                  not null default 0,
   CLAIMED_IDE          numeric(22,9)                  not null default 0,
   CLAIMED_DAS          numeric(22,9)                  not null default 0,
   CLAIMED_ISCSI        numeric(22,9)                  not null default 0,
   CLAIMED_FILESTORAGE  numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_FCFIBRE numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_N_FC_FB numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_IDE numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_ISCSI numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_FILE_SG numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED     numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_FCFIBRE numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_NONFCFIBRE numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_IDE numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_ISCSI numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_FILESTORAGE numeric(22,9)                  not null default 0,
   VIRTUALPOOL_ALLOCATED numeric(22,9)                  not null default 0,
   TOTAL_ALLOCATED      numeric(22,9)                  not null default 0,
   ALLOCATED_CLAIMED_OTHER_HOSTS numeric(22,9)                  not null default 0,
   ASSIGNED_VM          numeric(22,9)                  not null default 0,
   DTASSIGNED_FS        numeric(22,9)                  not null default 0,
   DTASSIGNED_DB        numeric(22,9)                  not null default 0,
   UNKNOWN_ASSIGNMENT   numeric(22,9)                  not null default 0,
   CLAIMED_ASSIGNED_OTHER_HOSTS numeric(22,9)                  not null default 0,
   DISKGROUP_CAPACITY   numeric(22,9)                  not null default 0,
   DISKGROUP_USED       numeric(22,9)                  not null default 0,
   DISKGROUP_AVAILABLE  numeric(22,9)                  not null default 0,
   VOLUME_CAPACITY      numeric(22,9)                  not null default 0,
   VOLUME_UNUSED        numeric(22,9)                  not null default 0,
   VMGR_OVERHEAD        numeric(22,9)                  not null default 0,
   VMGR_ADMINISTRATIVE  numeric(22,9)                  not null default 0,
   FS_CAPACITY          numeric(22,9)                  not null default 0,
   FS_USED              numeric(22,9)                  not null default 0,
   FS_AVAILABLE         numeric(22,9)                  not null default 0,
   FS_ON_VOLUME         numeric(22,9)                  not null default 0,
   FS_ON_NON_VOLUME     numeric(22,9)                  not null default 0,
   FS_USED_APPS         numeric(22,9)                  not null default 0,
   FS_USED_NON_APPS     numeric(22,9)                  not null default 0,
   DB_CAPACITY          numeric(22,9)                  not null default 0,
   DB_USED              numeric(22,9)                  not null default 0,
   DB_AVAILABLE         numeric(22,9)                  not null default 0,
   APPS_ON_RAW_VOLUME   numeric(22,9)                  not null default 0,
   APPS_ON_FS           numeric(22,9)                  not null default 0,
   APPS_ON_RAW_LUN      numeric(22,9)                  not null default 0,
   PERCENT_HOST_UTILIZATION numeric(7,4)                   not null default 0,
   THIN_SUBSCRIBED      numeric(22,9)                  not null default 0,
   C_THIN               numeric(22,9)                  not null default 0,
   C_ALLOCATED          numeric(22,9)                  not null default 0,
   C_TOTAL_ALLOCATED    numeric(22,9)                  not null default 0,
   C_CLAIMED            numeric(22,9)                  not null default 0,
   C_UNCLAIMED          numeric(22,9)                  not null default 0,
   C_ALLOCATED_CLAIMED_OTHER_HOST numeric(22,9)                  not null default 0,
   C_ASSIGNED_VM        numeric(22,9)                  not null default 0,
   C_DTASSIGNED_FS      numeric(22,9)                  not null default 0,
   C_DTASSIGNED_DB      numeric(22,9)                  not null default 0,
   C_UNKNOWN_ASSIGNMENT numeric(22,9)                  not null default 0,
   C_CLAIMED_ASSIGNED_OTHER_HOST numeric(22,9)                  not null default 0,
   VIRTUALIZATION_CAPACITY numeric(22,9)                  not null default 0,
   VIRTUALIZATION_UNUSED numeric(22,9)                  not null default 0,
   STORAGE_POOL_USED    numeric(22,9)                  not null default 0,
   STORAGE_POOL_UNUSED  numeric(22,9)                  not null default 0,
   VMDK_UNUSED          numeric(22,9)                  not null default 0,
   VM_EXPORTED          numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED   numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_FCFIBRE numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_ISCSI numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_FILESTORAGE numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED     numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED_FCFIBRE numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED_ISCSI numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED_FILESTORAGE numeric(22,9)                  not null default 0,
   ADJUSTED_UNCLAIMED   numeric(22,9)                  not null default 0,
   ADJUSTED_DTMAPED_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_VIRTUALPOOL_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_TOTAL_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_ASSIGNED_VM numeric(22,9)                  not null default 0,
   ADJUSTED_DTASSIGNED_FS numeric(22,9)                  not null default 0,
   ADJUSTED_DTASSIGNED_DB numeric(22,9)                  not null default 0,
   ADJUSTED_UNKNOWN_ASSIGNMENT numeric(22,9)                  not null default 0,
   ADJUSTED_DISKGROUP_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DISKGROUP_USED numeric(22,9)                  not null default 0,
   ADJUSTED_DISKGROUP_AVAILABLE numeric(22,9)                  not null default 0,
   ADJUSTED_VOLUME_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_VOLUME_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_VMGR_OVERHEAD numeric(22,9)                  not null default 0,
   ADJUSTED_VMGR_ADMINISTRATIVE numeric(22,9)                  not null default 0,
   ADJUSTED_FS_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_FS_USED     numeric(22,9)                  not null default 0,
   ADJUSTED_FS_AVAILABLE numeric(22,9)                  not null default 0,
   ADJUSTED_FS_ON_VOLUME numeric(22,9)                  not null default 0,
   ADJUSTED_FS_ON_NON_VOLUME numeric(22,9)                  not null default 0,
   ADJUSTED_FS_USED_APPS numeric(22,9)                  not null default 0,
   ADJUSTED_FS_USED_NON_APPS numeric(22,9)                  not null default 0,
   ADJUSTED_DB_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_USED     numeric(22,9)                  not null default 0,
   ADJUSTED_DB_AVAILABLE numeric(22,9)                  not null default 0,
   ADJUSTED_APPS_ON_RAW_VOLUME numeric(22,9)                  not null default 0,
   ADJUSTED_APPS_ON_FS  numeric(22,9)                  not null default 0,
   ADJUSTED_THIN_SUBSCRIBED numeric(22,9)                  not null default 0,
   C_ADJUSTED_THIN      numeric(22,9)                  not null default 0,
   C_ADJUSTED_ALLOCATED numeric(22,9)                  not null default 0,
   C_ADJUSTED_TOTAL_ALLOCATED numeric(22,9)                  not null default 0,
   C_ADJUSTED_CLAIMED   numeric(22,9)                  not null default 0,
   C_ADJUSTED_UNCLAIMED numeric(22,9)                  not null default 0,
   C_ADJUSTED_ASSIGNED_VM numeric(22,9)                  not null default 0,
   C_ADJUSTED_DTASSIGNED_FS numeric(22,9)                  not null default 0,
   C_ADJUSTED_DTASSIGNED_DB numeric(22,9)                  not null default 0,
   C_ADJUSTED_UNKNOWN_ASSIGNMENT numeric(22,9)                  not null default 0,
   ADJUSTED_VIRTUALIZATION_CAP numeric(22,9)                  not null default 0,
   ADJUSTED_VIRTUALIZATION_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_STORAGE_POOL_USED numeric(22,9)                  not null default 0,
   ADJUSTED_STORAGE_POOL_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_VMDK_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_VM_EXPORTED numeric(22,9)                  not null default 0,
   EXCHANGE_CAPACITY    numeric(22,9)                  not null default 0,
   ASSIGNED_SP          numeric(22,9)                  not null default 0,
   C_ASSIGNED_SP        numeric(22,9)                  not null default 0,
   ADJUSTED_ASSIGNED_SP numeric(22,9)                  not null default 0,
   C_ADJUSTED_ASSIGNED_SP numeric(22,9)                  not null default 0,
   SP_OVERHEAD          numeric(22,9)                  not null default 0,
   ADJUSTED_SP_OVERHEAD numeric(22,9)                  not null default 0,
   NAS_SP               numeric(22,9)                  not null default 0,
   NAS_SP_USED          numeric(22,9)                  not null default 0,
   NAS_SP_UNUSED        numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_SP      numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_SP_USED numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_SP_UNUSED numeric(22,9)                  not null default 0,
   EXPORTED_LUNS        numeric(22,9)                  not null default 0,
   C_EXPORTED_LUNS      numeric(22,9)                  not null default 0,
   EXPORTED_VOLUMES     numeric(22,9)                  not null default 0,
   EXPORTED_FS          numeric(22,9)                  not null default 0,
   ADJUSTED_EXPORTED_LUNS numeric(22,9)                  not null default 0,
   C_ADJUSTED_EXPORTED_LUNS numeric(22,9)                  not null default 0,
   ADJUSTED_EXPORTED_VOLUMES numeric(22,9)                  not null default 0,
   ADJUSTED_EXPORTED_FS numeric(22,9)                  not null default 0,
   UNSUPPORTED_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_UNSUPPORTED_ALLOCATED numeric(22,9)                  not null default 0,
   NAS_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   NAS_CONSUMED_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_ALLOCATE_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_CONSUMED_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_CAPACITY       numeric(22,9)                  not null default 0,
   DB_VM_USED_CAPACITY  numeric(22,9)                  not null default 0,
   DB_VM_FREE_CAPACITY  numeric(22,9)                  not null default 0,
   DB_VM_OVERHEAD_CAPACITY numeric(22,9)                  not null default 0,
   APPS_ON_DB_VM_CAPACITY numeric(22,9)                  not null default 0,
   APPS_ON_DB_VM_OTHER_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_DB_USED_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_DB_USED_OTHER_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_UNKNOWN_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_USED_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_FREE_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_OVERHEAD_CAP numeric(22,9)                  not null default 0,
   ADJUSTED_APPS_ON_DB_VM_CAP numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_DB_USED_CAP numeric(22,9)                  not null default 0
);

comment on table SAHARA.HOST_CAPACITY is 
'Contains capacity values for host';

comment on column SAHARA.HOST_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.HOST_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.';

comment on column SAHARA.HOST_CAPACITY.CLAIMED is 
'LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''s operating system has written a device handle for it.';

comment on column SAHARA.HOST_CAPACITY.UNCLAIMED is 
'LUNs in arrays and unified storage devices that have been masked to hosts whose
operating systems have not yet written device handles for the storage.

Unclaimed storage is usually wasted storage.  The most common causes for unclaimed storage are:
- A zone that is not configured properly
- A host that is not configured to recognize a LUN.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_UNDISCOVERED is 
'The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.';

comment on column SAHARA.HOST_CAPACITY.PERCENT_ALLOCATED_CLAIMED is 
'The percentage of a host''s allocated capacity that has a device handle written to it.  This value is calculated by dividing the Claimed Capacity (GB) by Allocated Capacity (GB).';

comment on column SAHARA.HOST_CAPACITY.PERCENT_ALLOCATED_UNCLAIMED is 
'The percentage of a host''s allocated capacity that does not have a device handle written to it.  This value is calculated by dividing the Unclaimed Capacity (GB) by Allocated Capacity (GB).';

comment on column SAHARA.HOST_CAPACITY.PERCENT_ALLOCATED_UNDISCOVERED is 
'The percentage of a host''s allocated capacity that is not being discovered by CommandCentral Storage.  This value is calculated by dividing the Undiscovered Capacity (GB) by Allocated Capacity (GB).';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_FCFIBRE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the SAN allocated storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_NONFCFIBRE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the allocated storage from the arbirated loop protocol.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_IDE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the IDE allocated storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_DAS is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes direct attached LUNs (typically JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)

The column represents the DAS allocated storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_ISCSI is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the ISCSI allocated storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_FILESTORAGE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the NAS or file base allocated storage.';

comment on column SAHARA.HOST_CAPACITY.CLAIMED_FCFIBRE is 
'LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.  The SAN based claimed storage is represented in the  column.';

comment on column SAHARA.HOST_CAPACITY.CLAIMED_NONFCFIBRE is 
'LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The Arbitrated loop based claimed storage is represented in the  column.';

comment on column SAHARA.HOST_CAPACITY.CLAIMED_IDE is 
'LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The IDE based claimed storage is represented in the  column.';

comment on column SAHARA.HOST_CAPACITY.CLAIMED_DAS is 
'Direct attached storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The DAS based claimed storage is represented in the  column.';

comment on column SAHARA.HOST_CAPACITY.CLAIMED_ISCSI is 
'LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The ISCSI based claimed storage is represented in the  column.';

comment on column SAHARA.HOST_CAPACITY.CLAIMED_FILESTORAGE is 
'LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The NAS or file based claimed storage is represented in the  column.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_UNDISCOVERED_FCFIBRE is 
'The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the SAN allocated undiscovered storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_UNDISCOVERED_N_FC_FB is 
'The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the Arbirated loop allocated undiscovered storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_UNDISCOVERED_IDE is 
'The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the IDE allocated undiscovered storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_UNDISCOVERED_ISCSI is 
'The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the ISCSI allocated undiscovered storage.';

comment on column SAHARA.HOST_CAPACITY.ALLOCATED_UNDISCOVERED_FILE_SG is 
'The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the NAS and or File based allocated undiscovered storage.';

comment on column SAHARA.HOST_CAPACITY.DTMAPD_ALLOCATED is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA.HOST_CAPACITY.DTMAPD_ALLOCATED_FCFIBRE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents FC_Fabric based storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA.HOST_CAPACITY.DTMAPD_ALLOCATED_NONFCFIBRE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents FC_NonFabric storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA.HOST_CAPACITY.DTMAPD_ALLOCATED_IDE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents IDE based storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA.HOST_CAPACITY.DTMAPD_ALLOCATED_ISCSI is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents ISCSI storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA.HOST_CAPACITY.DTMAPD_ALLOCATED_FILESTORAGE is 
'The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents FileStorage storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA.HOST_CAPACITY.VIRTUALPOOL_ALLOCATED is 
'This capacity represents amount of storage allocated to the virtual machine which comes directly from the storage pool.  

Note: This capacity is not applicable for LPARS.';

comment on column SAHARA.HOST_CAPACITY.UNSUPPORTED_ALLOCATED is 
'The amount of unsupported storage that is apportioned into LUNs and designated for use by specific hosts.  This includes direct attached LUNs (typically JBODs, or unsupported arrays) .
';

comment on column SAHARA.HOST_CAPACITY.ADJUSTED_UNSUPPORTED_ALLOCATED is 
'The amount of distributed unsupported storage that is apportioned into LUNs and designated for use by specific hosts.  This includes direct attached LUNs (typically JBODs, or unsupported arrays) . This capacity would be same as Unsupported Allocated unless we could discover associate shared LUN from multiple hosts.
';

comment on column SAHARA.HOST_CAPACITY.DB_VM_CAPACITY is 
'Amount of storage in database specific volume manager like Oracle ASM on the host.';

comment on column SAHARA.HOST_CAPACITY.DB_VM_USED_CAPACITY is 
'Amount of storage used in database specific volume manager on the host.';

comment on column SAHARA.HOST_CAPACITY.DB_VM_FREE_CAPACITY is 
'Amount of storage available for use in database specific volume manager on the host.';

comment on column SAHARA.HOST_CAPACITY.DB_VM_OVERHEAD_CAPACITY is 
'Amount of storage in database specific volume manager, that has gone in overhead like mirroring.';

comment on column SAHARA.HOST_CAPACITY.APPS_ON_DB_VM_CAPACITY is 
'Amount of database file storage, based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';

comment on column SAHARA.HOST_CAPACITY.APPS_ON_DB_VM_OTHER_CAPACITY is 
'For shared storage, amount of storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';

comment on column SAHARA.HOST_CAPACITY.DB_VM_DB_USED_CAPACITY is 
'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';

comment on column SAHARA.HOST_CAPACITY.DB_VM_DB_USED_OTHER_CAPACITY is 
'For shared storage, amount of used storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';

comment on column SAHARA.HOST_CAPACITY.DB_VM_UNKNOWN_CAPACITY is 
'Amount of storage in database specific volume manager, that has gone into storing meta-data and other book-keeping information.';

comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_CAPACITY is 
'Amount of storage in database specific volume manager like Oracle ASM on the host. For shared storage, storage will be distributed across hosts';

comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_USED_CAPACITY is 
'Amount of storage used in database specific volume manager on the host. For shared storage, storage will be distributed across hosts';

comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_FREE_CAPACITY is 
'Amount of storage available for use in database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_OVERHEAD_CAP is 
'Amount of storage in database specific volume manager, that has gone in overhead like mirroring. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA.HOST_CAPACITY.ADJUSTED_APPS_ON_DB_VM_CAP is 
'Amount of database file storage, based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_DB_USED_CAP is 
'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED add constraint C_HOSTCAPACITY_ALLOCATED check (ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED add constraint C_HOSTCAPACITY_CLAIMED check (CLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter UNCLAIMED add constraint C_HOSTCAPACITY_UNCLAIMED check (UNCLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_UNDISCOVERED add constraint C_HOSTCAPACITY_LLCTDNDSCVRD check (ALLOCATED_UNDISCOVERED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_FCFIBRE add constraint C_HOSTCAPACITY_LLCTDFCFBR check (ALLOCATED_FCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_NONFCFIBRE add constraint C_HOSTCAPACITY_LLCTDNNFCFBR check (ALLOCATED_NONFCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_IDE add constraint C_HOSTCAPACITY_ALLOCATEDIDE check (ALLOCATED_IDE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_DAS add constraint C_HOSTCAPACITY_ALLOCATEDDAS check (ALLOCATED_DAS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_ISCSI add constraint C_HOSTCAPACITY_ALLOCATEDISCSI check (ALLOCATED_ISCSI >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_FILESTORAGE add constraint C_HOSTCAPACITY_LLCTDFLSTRG check (ALLOCATED_FILESTORAGE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED_FCFIBRE add constraint C_HOSTCAPACITY_CLAIMEDFCFIBRE check (CLAIMED_FCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED_NONFCFIBRE add constraint C_HOSTCAPACITY_CLMDNNFCFBR check (CLAIMED_NONFCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED_IDE add constraint C_HOSTCAPACITY_CLAIMEDIDE check (CLAIMED_IDE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED_DAS add constraint C_HOSTCAPACITY_CLAIMEDDAS check (CLAIMED_DAS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED_ISCSI add constraint C_HOSTCAPACITY_CLAIMEDISCSI check (CLAIMED_ISCSI >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED_FILESTORAGE add constraint C_HOSTCAPACITY_CLMDFLSTRG check (CLAIMED_FILESTORAGE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_UNDISCOVERED_FCFIBRE add constraint C_HOSTCAPACITY_LLCTDNDSCVRDFCF check (ALLOCATED_UNDISCOVERED_FCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_UNDISCOVERED_N_FC_FB add constraint C_HOSTCAPACITY_LLCTDNDSCVRDNFC check (ALLOCATED_UNDISCOVERED_N_FC_FB >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_UNDISCOVERED_IDE add constraint C_HOSTCAPACITY_LLCTDNDSCVRDD check (ALLOCATED_UNDISCOVERED_IDE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_UNDISCOVERED_ISCSI add constraint C_HOSTCAPACITY_LLCTDNDSCVRDSCS check (ALLOCATED_UNDISCOVERED_ISCSI >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_UNDISCOVERED_FILE_SG add constraint C_HOSTCAPACITY_LLCTDNDSCVRDFLS check (ALLOCATED_UNDISCOVERED_FILE_SG >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTMAPD_ALLOCATED add constraint C_HOSTCAPACITY_DTMAPDALLOCATED check (DTMAPD_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTMAPD_ALLOCATED_FCFIBRE add constraint C_HOSTCAPACITY_DTMPDLLCTDFCFBR check (DTMAPD_ALLOCATED_FCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTMAPD_ALLOCATED_NONFCFIBRE add constraint C_HOSTCAPACITY_DTMPDLLCTDNNFCF check (DTMAPD_ALLOCATED_NONFCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTMAPD_ALLOCATED_IDE add constraint C_HOSTCAPACITY_DTMPDLLCTDD check (DTMAPD_ALLOCATED_IDE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTMAPD_ALLOCATED_ISCSI add constraint C_HOSTCAPACITY_DTMPDLLCTDSCS check (DTMAPD_ALLOCATED_ISCSI >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTMAPD_ALLOCATED_FILESTORAGE add constraint C_HOSTCAPACITY_DTMPDLLCTDFLSTR check (DTMAPD_ALLOCATED_FILESTORAGE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VIRTUALPOOL_ALLOCATED add constraint C_HOSTCAPACITY_VRTLPLLLCTD check (VIRTUALPOOL_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_TOTALALLOCATED check (TOTAL_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ALLOCATED_CLAIMED_OTHER_HOSTS add constraint C_HOSTCAPACITY_LLCTDCLMDTHRHST check (ALLOCATED_CLAIMED_OTHER_HOSTS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ASSIGNED_VM add constraint C_HOSTCAPACITY_ASSIGNEDVM check (ASSIGNED_VM >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTASSIGNED_FS add constraint C_HOSTCAPACITY_DTASSIGNEDFS check (DTASSIGNED_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DTASSIGNED_DB add constraint C_HOSTCAPACITY_DTASSIGNEDDB check (DTASSIGNED_DB >= 0);

alter table SAHARA.HOST_CAPACITY
   alter UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_NKNWNSSGNMNT check (UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA.HOST_CAPACITY
   alter CLAIMED_ASSIGNED_OTHER_HOSTS add constraint C_HOSTCAPACITY_CLMDSSGNDTHRHST check (CLAIMED_ASSIGNED_OTHER_HOSTS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DISKGROUP_CAPACITY add constraint C_HOSTCAPACITY_DSKGRPCPCT check (DISKGROUP_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DISKGROUP_USED add constraint C_HOSTCAPACITY_DISKGROUPUSED check (DISKGROUP_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DISKGROUP_AVAILABLE add constraint C_HOSTCAPACITY_DSKGRPVLBL check (DISKGROUP_AVAILABLE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VOLUME_CAPACITY add constraint C_HOSTCAPACITY_VOLUMECAPACITY check (VOLUME_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VOLUME_UNUSED add constraint C_HOSTCAPACITY_VOLUMEUNUSED check (VOLUME_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VMGR_OVERHEAD add constraint C_HOSTCAPACITY_VMGROVERHEAD check (VMGR_OVERHEAD >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VMGR_ADMINISTRATIVE add constraint C_HOSTCAPACITY_VMGRDMNSTRTV check (VMGR_ADMINISTRATIVE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter FS_CAPACITY add constraint C_HOSTCAPACITY_FSCAPACITY check (FS_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter FS_USED add constraint C_HOSTCAPACITY_FSUSED check (FS_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter FS_AVAILABLE add constraint C_HOSTCAPACITY_FSAVAILABLE check (FS_AVAILABLE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter FS_ON_VOLUME add constraint C_HOSTCAPACITY_FSONVOLUME check (FS_ON_VOLUME >= 0);

alter table SAHARA.HOST_CAPACITY
   alter FS_ON_NON_VOLUME add constraint C_HOSTCAPACITY_FSONNONVOLUME check (FS_ON_NON_VOLUME >= 0);

alter table SAHARA.HOST_CAPACITY
   alter FS_USED_APPS add constraint C_HOSTCAPACITY_FSUSEDAPPS check (FS_USED_APPS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter FS_USED_NON_APPS add constraint C_HOSTCAPACITY_FSUSEDNONAPPS check (FS_USED_NON_APPS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_CAPACITY add constraint C_HOSTCAPACITY_DBCAPACITY check (DB_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_USED add constraint C_HOSTCAPACITY_DBUSED check (DB_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_AVAILABLE add constraint C_HOSTCAPACITY_DBAVAILABLE check (DB_AVAILABLE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter APPS_ON_RAW_VOLUME add constraint C_HOSTCAPACITY_APPSONRAWVOLUME check (APPS_ON_RAW_VOLUME >= 0);

alter table SAHARA.HOST_CAPACITY
   alter APPS_ON_FS add constraint C_HOSTCAPACITY_APPSONFS check (APPS_ON_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter APPS_ON_RAW_LUN add constraint C_HOSTCAPACITY_APPSONRAWLUN check (APPS_ON_RAW_LUN >= 0);

alter table SAHARA.HOST_CAPACITY
   alter THIN_SUBSCRIBED add constraint C_HOSTCAPACITY_THINSUBSCRIBED check (THIN_SUBSCRIBED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_THIN add constraint C_HOSTCAPACITY_CTHIN check (C_THIN >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ALLOCATED add constraint C_HOSTCAPACITY_CALLOCATED check (C_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_CTOTALALLOCATED check (C_TOTAL_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_CLAIMED add constraint C_HOSTCAPACITY_CCLAIMED check (C_CLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_UNCLAIMED add constraint C_HOSTCAPACITY_CUNCLAIMED check (C_UNCLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ALLOCATED_CLAIMED_OTHER_HOST add constraint C_HOSTCAPACITY_CLLCTDCLMDTHRHS check (C_ALLOCATED_CLAIMED_OTHER_HOST >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ASSIGNED_VM add constraint C_HOSTCAPACITY_CASSIGNEDVM check (C_ASSIGNED_VM >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_DTASSIGNED_FS add constraint C_HOSTCAPACITY_CDTASSIGNEDFS check (C_DTASSIGNED_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_DTASSIGNED_DB add constraint C_HOSTCAPACITY_CDTASSIGNEDDB check (C_DTASSIGNED_DB >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_CNKNWNSSGNMNT check (C_UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_CLAIMED_ASSIGNED_OTHER_HOST add constraint C_HOSTCAPACITY_CCLMDSSGNDTHRHS check (C_CLAIMED_ASSIGNED_OTHER_HOST >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VIRTUALIZATION_CAPACITY add constraint C_HOSTCAPACITY_VRTLZTNCPCT check (VIRTUALIZATION_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VIRTUALIZATION_UNUSED add constraint C_HOSTCAPACITY_VRTLZTNNSD check (VIRTUALIZATION_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter STORAGE_POOL_USED add constraint C_HOSTCAPACITY_STORAGEPOOLUSED check (STORAGE_POOL_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter STORAGE_POOL_UNUSED add constraint C_HOSTCAPACITY_STRGPLNSD check (STORAGE_POOL_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VMDK_UNUSED add constraint C_HOSTCAPACITY_VMDKUNUSED check (VMDK_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter VM_EXPORTED add constraint C_HOSTCAPACITY_VMEXPORTED check (VM_EXPORTED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDLLCTD check (ADJUSTED_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_ALLOCATED_FCFIBRE add constraint C_HOSTCAPACITY_DJSTDLLCTDFCFBR check (ADJUSTED_ALLOCATED_FCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_ALLOCATED_ISCSI add constraint C_HOSTCAPACITY_DJSTDLLCTDSCS check (ADJUSTED_ALLOCATED_ISCSI >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_ALLOCATED_FILESTORAGE add constraint C_HOSTCAPACITY_DJSTDLLCTDFLSTR check (ADJUSTED_ALLOCATED_FILESTORAGE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_CLAIMED add constraint C_HOSTCAPACITY_ADJUSTEDCLAIMED check (ADJUSTED_CLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_CLAIMED_FCFIBRE add constraint C_HOSTCAPACITY_DJSTDCLMDFCFBR check (ADJUSTED_CLAIMED_FCFIBRE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_CLAIMED_ISCSI add constraint C_HOSTCAPACITY_DJSTDCLMDSCS check (ADJUSTED_CLAIMED_ISCSI >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_CLAIMED_FILESTORAGE add constraint C_HOSTCAPACITY_DJSTDCLMDFLSTRG check (ADJUSTED_CLAIMED_FILESTORAGE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_UNCLAIMED add constraint C_HOSTCAPACITY_DJSTDNCLMD check (ADJUSTED_UNCLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DTMAPED_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDDTMPDLLCTD check (ADJUSTED_DTMAPED_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VIRTUALPOOL_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDVRTLPLLLCT check (ADJUSTED_VIRTUALPOOL_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDTTLLLCTD check (ADJUSTED_TOTAL_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_ASSIGNED_VM add constraint C_HOSTCAPACITY_DJSTDSSGNDVM check (ADJUSTED_ASSIGNED_VM >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DTASSIGNED_FS add constraint C_HOSTCAPACITY_DJSTDDTSSGNDFS check (ADJUSTED_DTASSIGNED_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DTASSIGNED_DB add constraint C_HOSTCAPACITY_DJSTDDTSSGNDDB check (ADJUSTED_DTASSIGNED_DB >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_DJSTDNKNWNSSGNM check (ADJUSTED_UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DISKGROUP_CAPACITY add constraint C_HOSTCAPACITY_DJSTDDSKGRPCPCT check (ADJUSTED_DISKGROUP_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DISKGROUP_USED add constraint C_HOSTCAPACITY_DJSTDDSKGRPSD check (ADJUSTED_DISKGROUP_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DISKGROUP_AVAILABLE add constraint C_HOSTCAPACITY_DJSTDDSKGRPVLBL check (ADJUSTED_DISKGROUP_AVAILABLE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VOLUME_CAPACITY add constraint C_HOSTCAPACITY_DJSTDVLMCPCT check (ADJUSTED_VOLUME_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VOLUME_UNUSED add constraint C_HOSTCAPACITY_DJSTDVLMNSD check (ADJUSTED_VOLUME_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VMGR_OVERHEAD add constraint C_HOSTCAPACITY_DJSTDVMGRVRHD check (ADJUSTED_VMGR_OVERHEAD >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VMGR_ADMINISTRATIVE add constraint C_HOSTCAPACITY_DJSTDVMGRDMNSTR check (ADJUSTED_VMGR_ADMINISTRATIVE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_FS_CAPACITY add constraint C_HOSTCAPACITY_DJSTDFSCPCT check (ADJUSTED_FS_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_FS_USED add constraint C_HOSTCAPACITY_ADJUSTEDFSUSED check (ADJUSTED_FS_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_FS_AVAILABLE add constraint C_HOSTCAPACITY_DJSTDFSVLBL check (ADJUSTED_FS_AVAILABLE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_FS_ON_VOLUME add constraint C_HOSTCAPACITY_DJSTDFSNVLM check (ADJUSTED_FS_ON_VOLUME >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_FS_ON_NON_VOLUME add constraint C_HOSTCAPACITY_DJSTDFSNNNVLM check (ADJUSTED_FS_ON_NON_VOLUME >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_FS_USED_APPS add constraint C_HOSTCAPACITY_DJSTDFSSDPPS check (ADJUSTED_FS_USED_APPS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_FS_USED_NON_APPS add constraint C_HOSTCAPACITY_DJSTDFSSDNNPPS check (ADJUSTED_FS_USED_NON_APPS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_CAPACITY add constraint C_HOSTCAPACITY_DJSTDDBCPCT check (ADJUSTED_DB_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_USED add constraint C_HOSTCAPACITY_ADJUSTEDDBUSED check (ADJUSTED_DB_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_AVAILABLE add constraint C_HOSTCAPACITY_DJSTDDBVLBL check (ADJUSTED_DB_AVAILABLE >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_APPS_ON_RAW_VOLUME add constraint C_HOSTCAPACITY_DJSTDPPSNRWVLM check (ADJUSTED_APPS_ON_RAW_VOLUME >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_APPS_ON_FS add constraint C_HOSTCAPACITY_DJSTDPPSNFS check (ADJUSTED_APPS_ON_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_THIN_SUBSCRIBED add constraint C_HOSTCAPACITY_DJSTDTHNSBSCRBD check (ADJUSTED_THIN_SUBSCRIBED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_THIN add constraint C_HOSTCAPACITY_CADJUSTEDTHIN check (C_ADJUSTED_THIN >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_ALLOCATED add constraint C_HOSTCAPACITY_CDJSTDLLCTD check (C_ADJUSTED_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_CDJSTDTTLLLCTD check (C_ADJUSTED_TOTAL_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_CLAIMED add constraint C_HOSTCAPACITY_CDJSTDCLMD check (C_ADJUSTED_CLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_UNCLAIMED add constraint C_HOSTCAPACITY_CDJSTDNCLMD check (C_ADJUSTED_UNCLAIMED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_ASSIGNED_VM add constraint C_HOSTCAPACITY_CDJSTDSSGNDVM check (C_ADJUSTED_ASSIGNED_VM >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_DTASSIGNED_FS add constraint C_HOSTCAPACITY_CDJSTDDTSSGNDFS check (C_ADJUSTED_DTASSIGNED_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_DTASSIGNED_DB add constraint C_HOSTCAPACITY_CDJSTDDTSSGNDDB check (C_ADJUSTED_DTASSIGNED_DB >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_CDJSTDNKNWNSSGN check (C_ADJUSTED_UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VIRTUALIZATION_CAP add constraint C_HOSTCAPACITY_DJSTDVRTLZTNCP check (ADJUSTED_VIRTUALIZATION_CAP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VIRTUALIZATION_UNUSED add constraint C_HOSTCAPACITY_DJSTDVRTLZTNNSD check (ADJUSTED_VIRTUALIZATION_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_STORAGE_POOL_USED add constraint C_HOSTCAPACITY_DJSTDSTRGPLSD check (ADJUSTED_STORAGE_POOL_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_STORAGE_POOL_UNUSED add constraint C_HOSTCAPACITY_DJSTDSTRGPLNSD check (ADJUSTED_STORAGE_POOL_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VMDK_UNUSED add constraint C_HOSTCAPACITY_DJSTDVMDKNSD check (ADJUSTED_VMDK_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_VM_EXPORTED add constraint C_HOSTCAPACITY_DJSTDVMXPRTD check (ADJUSTED_VM_EXPORTED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter EXCHANGE_CAPACITY add constraint C_HOSTCAPACITY_XCHNGCPCT check (EXCHANGE_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ASSIGNED_SP add constraint C_HOSTCAPACITY_ASSIGNEDSP check (ASSIGNED_SP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ASSIGNED_SP add constraint C_HOSTCAPACITY_CASSIGNEDSP check (C_ASSIGNED_SP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_ASSIGNED_SP add constraint C_HOSTCAPACITY_DJSTDSSGNDSP check (ADJUSTED_ASSIGNED_SP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_ASSIGNED_SP add constraint C_HOSTCAPACITY_CDJSTDSSGNDSP check (C_ADJUSTED_ASSIGNED_SP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter SP_OVERHEAD add constraint C_HOSTCAPACITY_SPOVERHEAD check (SP_OVERHEAD >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_SP_OVERHEAD add constraint C_HOSTCAPACITY_DJSTDSPVRHD check (ADJUSTED_SP_OVERHEAD >= 0);

alter table SAHARA.HOST_CAPACITY
   alter NAS_SP add constraint C_HOSTCAPACITY_NASSP check (NAS_SP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter NAS_SP_USED add constraint C_HOSTCAPACITY_NASSPUSED check (NAS_SP_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter NAS_SP_UNUSED add constraint C_HOSTCAPACITY_NASSPUNUSED check (NAS_SP_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_NAS_SP add constraint C_HOSTCAPACITY_ADJUSTEDNASSP check (ADJUSTED_NAS_SP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_NAS_SP_USED add constraint C_HOSTCAPACITY_DJSTDNSSPSD check (ADJUSTED_NAS_SP_USED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_NAS_SP_UNUSED add constraint C_HOSTCAPACITY_DJSTDNSSPNSD check (ADJUSTED_NAS_SP_UNUSED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter EXPORTED_LUNS add constraint C_HOSTCAPACITY_EXPORTEDLUNS check (EXPORTED_LUNS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_EXPORTED_LUNS add constraint C_HOSTCAPACITY_CEXPORTEDLUNS check (C_EXPORTED_LUNS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter EXPORTED_VOLUMES add constraint C_HOSTCAPACITY_EXPORTEDVOLUMES check (EXPORTED_VOLUMES >= 0);

alter table SAHARA.HOST_CAPACITY
   alter EXPORTED_FS add constraint C_HOSTCAPACITY_EXPORTEDFS check (EXPORTED_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_EXPORTED_LUNS add constraint C_HOSTCAPACITY_DJSTDXPRTDLNS check (ADJUSTED_EXPORTED_LUNS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter C_ADJUSTED_EXPORTED_LUNS add constraint C_HOSTCAPACITY_CDJSTDXPRTDLNS check (C_ADJUSTED_EXPORTED_LUNS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_EXPORTED_VOLUMES add constraint C_HOSTCAPACITY_DJSTDXPRTDVLMS check (ADJUSTED_EXPORTED_VOLUMES >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_EXPORTED_FS add constraint C_HOSTCAPACITY_DJSTDXPRTDFS check (ADJUSTED_EXPORTED_FS >= 0);

alter table SAHARA.HOST_CAPACITY
   alter UNSUPPORTED_ALLOCATED add constraint CKC_UNSUPPORTED_ALLOC_HOST_CAP check (UNSUPPORTED_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_UNSUPPORTED_ALLOCATED add constraint CKC_ADJUSTED_UNSUPPOR_HOST_CAP check (ADJUSTED_UNSUPPORTED_ALLOCATED >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_VM_CAPACITY add constraint CKC_DB_VM_CAPACITY_HOST_CAP check (DB_VM_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_VM_USED_CAPACITY add constraint CKC_DB_VM_USED_CAPACI_HOST_CAP check (DB_VM_USED_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_VM_FREE_CAPACITY add constraint CKC_DB_VM_FREE_CAPACI_HOST_CAP check (DB_VM_FREE_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_VM_OVERHEAD_CAPACITY add constraint CKC_DB_VM_OVERHEAD_CA_HOST_CAP check (DB_VM_OVERHEAD_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter APPS_ON_DB_VM_CAPACITY add constraint CKC_APPS_ON_DB_VM_CAP_HOST_CAP check (APPS_ON_DB_VM_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter APPS_ON_DB_VM_OTHER_CAPACITY add constraint CKC_APPS_ON_DB_VM_OTH_HOST_CAP check (APPS_ON_DB_VM_OTHER_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_VM_DB_USED_CAPACITY add constraint CKC_DB_VM_DB_USED_CAP_HOST_CAP check (DB_VM_DB_USED_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_VM_DB_USED_OTHER_CAPACITY add constraint CKC_DB_VM_DB_USED_OTH_HOST_CAP check (DB_VM_DB_USED_OTHER_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter DB_VM_UNKNOWN_CAPACITY add constraint CKC_DB_VM_UNKNOWN_CAP_HOST_CAP check (DB_VM_UNKNOWN_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_VM_CAPACITY add constraint CKC_ADJUSTED_DB_VM_CA_HOST_CAP check (ADJUSTED_DB_VM_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_VM_USED_CAPACITY add constraint CKC_ADJUSTED_DB_VM_US_HOST_CAP check (ADJUSTED_DB_VM_USED_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_VM_FREE_CAPACITY add constraint CKC_ADJUSTED_DB_VM_FR_HOST_CAP check (ADJUSTED_DB_VM_FREE_CAPACITY >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_VM_OVERHEAD_CAP add constraint CKC_ADJUSTED_DB_VM_OV_HOST_CAP check (ADJUSTED_DB_VM_OVERHEAD_CAP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_APPS_ON_DB_VM_CAP add constraint CKC_ADJUSTED_APPS_ON__HOST_CAP check (ADJUSTED_APPS_ON_DB_VM_CAP >= 0);

alter table SAHARA.HOST_CAPACITY
   alter ADJUSTED_DB_VM_DB_USED_CAP add constraint CKC_ADJUSTED_DB_VM_DB_HOST_CAP check (ADJUSTED_DB_VM_DB_USED_CAP >= 0);

/*==============================================================*/
/* Index: I_HOSTCAPACITY_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_HOSTCAPACITY_ROLLUPBATCHKEY on SAHARA.HOST_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_HOSTCAPACITY_DATESKEY                               */
/*==============================================================*/
create index I_HOSTCAPACITY_DATESKEY on SAHARA.HOST_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_HOSTCAPACITY_HOSTKEY                                */
/*==============================================================*/
create index I_HOSTCAPACITY_HOSTKEY on SAHARA.HOST_CAPACITY (
HOST_KEY ASC
);

grant SELECT on SAHARA.HOST_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: HOST_CAPACITY_                                        */
/*==============================================================*/
create table SAHARA_APP.HOST_CAPACITY_ 
(
   HOST_OID             varchar(512)                   not null,
   ALLOCATED            numeric(22,9)                  not null default 0,
   CLAIMED              numeric(22,9)                  not null default 0,
   UNCLAIMED            numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED numeric(22,9)                  not null default 0,
   PERCENT_ALLOCATED_CLAIMED numeric(7,4)                   not null default 0,
   PERCENT_ALLOCATED_UNCLAIMED numeric(7,4)                   not null default 0,
   PERCENT_ALLOCATED_UNDISCOVERED numeric(7,4)                   not null default 0,
   ALLOCATED_FCFIBRE    numeric(22,9)                  not null default 0,
   ALLOCATED_NONFCFIBRE numeric(22,9)                  not null default 0,
   ALLOCATED_IDE        numeric(22,9)                  not null default 0,
   ALLOCATED_ISCSI      numeric(22,9)                  not null default 0,
   ALLOCATED_FILESTORAGE numeric(22,9)                  not null default 0,
   CLAIMED_FCFIBRE      numeric(22,9)                  not null default 0,
   CLAIMED_NONFCFIBRE   numeric(22,9)                  not null default 0,
   CLAIMED_ISCSI        numeric(22,9)                  not null default 0,
   CLAIMED_FILESTORAGE  numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED     numeric(22,9)                  not null default 0,
   VIRTUALPOOL_ALLOCATED numeric(22,9)                  not null default 0,
   ALLOCATED_DAS        numeric(22,9)                  not null default 0,
   TOTAL_ALLOCATED      numeric(22,9)                  not null default 0,
   ALLOCATED_CLAIMED_OTHER_HOSTS numeric(22,9)                  not null default 0,
   ASSIGNED_VM          numeric(22,9)                  not null default 0,
   DTASSIGNED_FS        numeric(22,9)                  not null default 0,
   DTASSIGNED_DB        numeric(22,9)                  not null default 0,
   UNKNOWN_ASSIGNMENT   numeric(22,9)                  not null default 0,
   CLAIMED_ASSIGNED_OTHER_HOSTS numeric(22,9)                  not null default 0,
   DISKGROUP_CAPACITY   numeric(22,9)                  not null default 0,
   DISKGROUP_USED       numeric(22,9)                  not null default 0,
   DISKGROUP_AVAILABLE  numeric(22,9)                  not null default 0,
   VOLUME_CAPACITY      numeric(22,9)                  not null default 0,
   VOLUME_UNUSED        numeric(22,9)                  not null default 0,
   VMGR_OVERHEAD        numeric(22,9)                  not null default 0,
   VMGR_ADMINISTRATIVE  numeric(22,9)                  not null default 0,
   FS_CAPACITY          numeric(22,9)                  not null default 0,
   FS_USED              numeric(22,9)                  not null default 0,
   FS_AVAILABLE         numeric(22,9)                  not null default 0,
   FS_ON_VOLUME         numeric(22,9)                  not null default 0,
   FS_ON_NON_VOLUME     numeric(22,9)                  not null default 0,
   FS_USED_APPS         numeric(22,9)                  not null default 0,
   FS_USED_NON_APPS     numeric(22,9)                  not null default 0,
   DB_CAPACITY          numeric(22,9)                  not null default 0,
   DB_USED              numeric(22,9)                  not null default 0,
   DB_AVAILABLE         numeric(22,9)                  not null default 0,
   APPS_ON_RAW_VOLUME   numeric(22,9)                  not null default 0,
   APPS_ON_FS           numeric(22,9)                  not null default 0,
   APPS_ON_RAW_LUN      numeric(22,9)                  not null default 0,
   PERCENT_HOST_UTILIZATION numeric(7,4)                   not null default 0,
   THIN_SUBSCRIBED      numeric(22,9)                  not null default 0,
   C_THIN               numeric(22,9)                  not null default 0,
   C_ALLOCATED          numeric(22,9)                  not null default 0,
   C_TOTAL_ALLOCATED    numeric(22,9)                  not null default 0,
   C_CLAIMED            numeric(22,9)                  not null default 0,
   C_UNCLAIMED          numeric(22,9)                  not null default 0,
   C_ALLOCATED_CLAIMED_OTHER_HOST numeric(22,9)                  not null default 0,
   C_ASSIGNED_VM        numeric(22,9)                  not null default 0,
   C_DTASSIGNED_FS      numeric(22,9)                  not null default 0,
   C_DTASSIGNED_DB      numeric(22,9)                  not null default 0,
   C_UNKNOWN_ASSIGNMENT numeric(22,9)                  not null default 0,
   C_CLAIMED_ASSIGNED_OTHER_HOST numeric(22,9)                  not null default 0,
   VIRTUALIZATION_CAPACITY numeric(22,9)                  not null default 0,
   VIRTUALIZATION_UNUSED numeric(22,9)                  not null default 0,
   STORAGE_POOL_USED    numeric(22,9)                  not null default 0,
   STORAGE_POOL_UNUSED  numeric(22,9)                  not null default 0,
   VMDK_UNUSED          numeric(22,9)                  not null default 0,
   VM_EXPORTED          numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED   numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_FCFIBRE numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_ISCSI numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_FILESTORAGE numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED     numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED_FCFIBRE numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED_ISCSI numeric(22,9)                  not null default 0,
   ADJUSTED_CLAIMED_FILESTORAGE numeric(22,9)                  not null default 0,
   ADJUSTED_UNCLAIMED   numeric(22,9)                  not null default 0,
   ADJUSTED_DTMAPED_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_VIRTUALPOOL_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_TOTAL_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_ASSIGNED_VM numeric(22,9)                  not null default 0,
   ADJUSTED_DTASSIGNED_FS numeric(22,9)                  not null default 0,
   ADJUSTED_DTASSIGNED_DB numeric(22,9)                  not null default 0,
   ADJUSTED_UNKNOWN_ASSIGNMENT numeric(22,9)                  not null default 0,
   ADJUSTED_DISKGROUP_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DISKGROUP_USED numeric(22,9)                  not null default 0,
   ADJUSTED_DISKGROUP_AVAILABLE numeric(22,9)                  not null default 0,
   ADJUSTED_VOLUME_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_VOLUME_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_VMGR_OVERHEAD numeric(22,9)                  not null default 0,
   ADJUSTED_VMGR_ADMINISTRATIVE numeric(22,9)                  not null default 0,
   ADJUSTED_FS_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_FS_USED     numeric(22,9)                  not null default 0,
   ADJUSTED_FS_AVAILABLE numeric(22,9)                  not null default 0,
   ADJUSTED_FS_ON_VOLUME numeric(22,9)                  not null default 0,
   ADJUSTED_FS_ON_NON_VOLUME numeric(22,9)                  not null default 0,
   ADJUSTED_FS_USED_APPS numeric(22,9)                  not null default 0,
   ADJUSTED_FS_USED_NON_APPS numeric(22,9)                  not null default 0,
   ADJUSTED_DB_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_USED     numeric(22,9)                  not null default 0,
   ADJUSTED_DB_AVAILABLE numeric(22,9)                  not null default 0,
   ADJUSTED_APPS_ON_RAW_VOLUME numeric(22,9)                  not null default 0,
   ADJUSTED_APPS_ON_FS  numeric(22,9)                  not null default 0,
   ADJUSTED_THIN_SUBSCRIBED numeric(22,9)                  not null default 0,
   C_ADJUSTED_THIN      numeric(22,9)                  not null default 0,
   C_ADJUSTED_ALLOCATED numeric(22,9)                  not null default 0,
   C_ADJUSTED_TOTAL_ALLOCATED numeric(22,9)                  not null default 0,
   C_ADJUSTED_CLAIMED   numeric(22,9)                  not null default 0,
   C_ADJUSTED_UNCLAIMED numeric(22,9)                  not null default 0,
   C_ADJUSTED_ASSIGNED_VM numeric(22,9)                  not null default 0,
   C_ADJUSTED_DTASSIGNED_FS numeric(22,9)                  not null default 0,
   C_ADJUSTED_DTASSIGNED_DB numeric(22,9)                  not null default 0,
   C_ADJUSTED_UNKNOWN_ASSIGNMENT numeric(22,9)                  not null default 0,
   ADJUSTED_VIRTUALIZATION_CAP numeric(22,9)                  not null default 0,
   ADJUSTED_VIRTUALIZATION_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_STORAGE_POOL_USED numeric(22,9)                  not null default 0,
   ADJUSTED_STORAGE_POOL_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_VMDK_UNUSED numeric(22,9)                  not null default 0,
   ADJUSTED_VM_EXPORTED numeric(22,9)                  not null default 0,
   EXCHANGE_CAPACITY    numeric(22,9)                  not null default 0,
   ASSIGNED_SP          numeric(22,9)                  not null default 0,
   C_ASSIGNED_SP        numeric(22,9)                  not null default 0,
   ADJUSTED_ASSIGNED_SP numeric(22,9)                  not null default 0,
   C_ADJUSTED_ASSIGNED_SP numeric(22,9)                  not null default 0,
   SP_OVERHEAD          numeric(22,9)                  not null default 0,
   ADJUSTED_SP_OVERHEAD numeric(22,9)                  not null default 0,
   NAS_SP               numeric(22,9)                  not null default 0,
   NAS_SP_USED          numeric(22,9)                  not null default 0,
   NAS_SP_UNUSED        numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_SP      numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_SP_USED numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_SP_UNUSED numeric(22,9)                  not null default 0,
   EXPORTED_LUNS        numeric(22,9)                  not null default 0,
   C_EXPORTED_LUNS      numeric(22,9)                  not null default 0,
   EXPORTED_VOLUMES     numeric(22,9)                  not null default 0,
   EXPORTED_FS          numeric(22,9)                  not null default 0,
   ADJUSTED_EXPORTED_LUNS numeric(22,9)                  not null default 0,
   C_ADJUSTED_EXPORTED_LUNS numeric(22,9)                  not null default 0,
   ADJUSTED_EXPORTED_VOLUMES numeric(22,9)                  not null default 0,
   ADJUSTED_EXPORTED_FS numeric(22,9)                  not null default 0,
   UNSUPPORTED_ALLOCATED numeric(22,9)                  not null default 0,
   ADJUSTED_UNSUPPORTED_ALLOCATED numeric(22,9)                  not null default 0,
   CLAIMED_DAS          numeric(22,9)                  not null default 0,
   CLAIMED_IDE          numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_FCFIBRE numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_N_FC_FB numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_IDE numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_ISCSI numeric(22,9)                  not null default 0,
   ALLOCATED_UNDISCOVERED_FILE_SG numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_FCFIBRE numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_NONFCFIBRE numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_IDE numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_ISCSI numeric(22,9)                  not null default 0,
   DTMAPD_ALLOCATED_FILESTORAGE numeric(22,9)                  not null default 0,
   NAS_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   NAS_CONSUMED_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_ALLOCATE_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_NAS_CONSUMED_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_CAPACITY       numeric(22,9)                  not null default 0,
   DB_VM_USED_CAPACITY  numeric(22,9)                  not null default 0,
   DB_VM_FREE_CAPACITY  numeric(22,9)                  not null default 0,
   DB_VM_OVERHEAD_CAPACITY numeric(22,9)                  not null default 0,
   APPS_ON_DB_VM_CAPACITY numeric(22,9)                  not null default 0,
   APPS_ON_DB_VM_OTHER_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_DB_USED_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_DB_USED_OTHER_CAPACITY numeric(22,9)                  not null default 0,
   DB_VM_UNKNOWN_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_USED_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_FREE_CAPACITY numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_OVERHEAD_CAP numeric(22,9)                  not null default 0,
   ADJUSTED_APPS_ON_DB_VM_CAP numeric(22,9)                  not null default 0,
   ADJUSTED_DB_VM_DB_USED_CAP numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.HOST_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.HOST_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  Contains capacity values for host';

comment on column SAHARA_APP.HOST_CAPACITY_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.';

comment on column SAHARA_APP.HOST_CAPACITY_.CLAIMED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column CLAIMED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''s operating system has written a device handle for it.';

comment on column SAHARA_APP.HOST_CAPACITY_.UNCLAIMED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column UNCLAIMED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs in arrays and unified storage devices that have been masked to hosts whose
operating systems have not yet written device handles for the storage.

Unclaimed storage is usually wasted storage.  The most common causes for unclaimed storage are:
- A zone that is not configured properly
- A host that is not configured to recognize a LUN.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_UNDISCOVERED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_UNDISCOVERED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.PERCENT_ALLOCATED_CLAIMED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column PERCENT_ALLOCATED_CLAIMED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of a host''s allocated capacity that has a device handle written to it.  This value is calculated by dividing the Claimed Capacity (GB) by Allocated Capacity (GB).';

comment on column SAHARA_APP.HOST_CAPACITY_.PERCENT_ALLOCATED_UNCLAIMED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column PERCENT_ALLOCATED_UNCLAIMED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of a host''s allocated capacity that does not have a device handle written to it.  This value is calculated by dividing the Unclaimed Capacity (GB) by Allocated Capacity (GB).';

comment on column SAHARA_APP.HOST_CAPACITY_.PERCENT_ALLOCATED_UNDISCOVERED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column PERCENT_ALLOCATED_UNDISCOVERED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of a host''s allocated capacity that is not being discovered by CommandCentral Storage.  This value is calculated by dividing the Undiscovered Capacity (GB) by Allocated Capacity (GB).';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_FCFIBRE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_FCFIBRE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the SAN allocated storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_NONFCFIBRE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_NONFCFIBRE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the allocated storage from the arbirated loop protocol.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_IDE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_IDE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the IDE allocated storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_ISCSI is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_ISCSI.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the ISCSI allocated storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_FILESTORAGE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_FILESTORAGE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes array LUNs masked to hosts and other LUNs (typically on JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)
This value does not include LUNs designated for replication as described in Replication.
LUNs allocated to a virtualization array, such as a TagmaStore, are not counted as allocated if the virtualization array is also discovered by CommandCentral Storage.
In NetApp filers, allocated storage applies only to storage that is configured for block (FC) mode.  The column represents the NAS or file base allocated storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.CLAIMED_FCFIBRE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column CLAIMED_FCFIBRE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.  The SAN based claimed storage is represented in the  column.';

comment on column SAHARA_APP.HOST_CAPACITY_.CLAIMED_NONFCFIBRE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column CLAIMED_NONFCFIBRE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The Arbitrated loop based claimed storage is represented in the  column.';

comment on column SAHARA_APP.HOST_CAPACITY_.CLAIMED_ISCSI is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column CLAIMED_ISCSI.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The ISCSI based claimed storage is represented in the  column.';

comment on column SAHARA_APP.HOST_CAPACITY_.CLAIMED_FILESTORAGE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column CLAIMED_FILESTORAGE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The NAS or file based claimed storage is represented in the  column.';

comment on column SAHARA_APP.HOST_CAPACITY_.DTMAPD_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column DTMAPD_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA_APP.HOST_CAPACITY_.VIRTUALPOOL_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column VIRTUALPOOL_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

This capacity represents amount of storage allocated to the virtual machine which comes directly from the storage pool.  

Note: This capacity is not applicable for LPARS.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_DAS is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_DAS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This includes direct attached LUNs (typically JBODs, SCSI disks, or IDE disks) claimed by hosts.  ("Claimed" means that the operating system has written a device handle for the LUN.)

The column represents the DAS allocated storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.UNSUPPORTED_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column UNSUPPORTED_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of unsupported storage that is apportioned into LUNs and designated for use by specific hosts.  This includes direct attached LUNs (typically JBODs, or unsupported arrays) .';

comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_UNSUPPORTED_ALLOCATED is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ADJUSTED_UNSUPPORTED_ALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of distributed unsupported storage that is apportioned into LUNs and designated for use by specific hosts.  This includes direct attached LUNs (typically JBODs, or unsupported arrays) . This capacity would be same as Unsupported Allocated unless we could discover associate shared LUN from multiple hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.CLAIMED_DAS is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column CLAIMED_DAS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Direct attached storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The DAS based claimed storage is represented in the  column.';

comment on column SAHARA_APP.HOST_CAPACITY_.CLAIMED_IDE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column CLAIMED_IDE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUNs in arrays and unified storage devices that have been claimed by the host.
A LUN is considered claimed when a host''''s operating system has written a device handle for it.The IDE based claimed storage is represented in the  column.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_UNDISCOVERED_FCFIBRE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_UNDISCOVERED_FCFIBRE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the SAN allocated undiscovered storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_UNDISCOVERED_N_FC_FB is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_UNDISCOVERED_N_FC_FB.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the Arbirated loop allocated undiscovered storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_UNDISCOVERED_IDE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_UNDISCOVERED_IDE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the IDE allocated undiscovered storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_UNDISCOVERED_ISCSI is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_UNDISCOVERED_ISCSI.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the ISCSI allocated undiscovered storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.ALLOCATED_UNDISCOVERED_FILE_SG is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column ALLOCATED_UNDISCOVERED_FILE_SG.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of LUNs that have been masked to hosts that are not functioning as CommandCentral Storage managed hosts, or LUNs allocated to virtualization servers that cannot be discovered by CommandCentral Storage.The column represents the NAS and or File based allocated undiscovered storage.';

comment on column SAHARA_APP.HOST_CAPACITY_.DTMAPD_ALLOCATED_FCFIBRE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column DTMAPD_ALLOCATED_FCFIBRE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents FC_Fabric based storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA_APP.HOST_CAPACITY_.DTMAPD_ALLOCATED_NONFCFIBRE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column DTMAPD_ALLOCATED_NONFCFIBRE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents FC_NonFabric storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA_APP.HOST_CAPACITY_.DTMAPD_ALLOCATED_IDE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column DTMAPD_ALLOCATED_IDE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents IDE based storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA_APP.HOST_CAPACITY_.DTMAPD_ALLOCATED_ISCSI is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column DTMAPD_ALLOCATED_ISCSI.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents ISCSI storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA_APP.HOST_CAPACITY_.DTMAPD_ALLOCATED_FILESTORAGE is 
'This column is for staging data that will end up in table SAHARA.HOST_CAPACITY, column DTMAPD_ALLOCATED_FILESTORAGE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage that is apportioned into LUNs and designated for use by specific hosts.  This capacity represents FileStorage storage allocation happening as a result of the case where the luns are directly allocated to the virtual machines or LPARS by VIO servers and ESX servers.';

comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_CAPACITY is 
'Amount of storage in database specific volume manager like Oracle ASM on the host.';

comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_USED_CAPACITY is 
'Amount of storage used in database specific volume manager on the host.';

comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_FREE_CAPACITY is 
'Amount of storage available for use in database specific volume manager on the host.';

comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_OVERHEAD_CAPACITY is 
'Amount of storage in database specific volume manager, that has gone in overhead like mirroring.';

comment on column SAHARA_APP.HOST_CAPACITY_.APPS_ON_DB_VM_CAPACITY is 
'Amount of database file storage, based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.APPS_ON_DB_VM_OTHER_CAPACITY is 
'For shared storage, amount of storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';

comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_DB_USED_CAPACITY is 
'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_DB_USED_OTHER_CAPACITY is 
'For shared storage, amount of used storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';

comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_UNKNOWN_CAPACITY is 
'Amount of storage in database specific volume manager, that has gone into storing meta-data and other book-keeping information.';

comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_CAPACITY is 
'Amount of storage in database specific volume manager like Oracle ASM on the host. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_USED_CAPACITY is 
'Amount of storage used in database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_FREE_CAPACITY is 
'Amount of storage available for use in database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_OVERHEAD_CAP is 
'Amount of storage in database specific volume manager, that has gone in overhead like mirroring. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_APPS_ON_DB_VM_CAP is 
'Amount of database file storage, based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';

comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_DB_USED_CAP is 
'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED add constraint C_HOSTCAPACITY_ALLOCATED check (ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED add constraint C_HOSTCAPACITY_CLAIMED check (CLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter UNCLAIMED add constraint C_HOSTCAPACITY_UNCLAIMED check (UNCLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_UNDISCOVERED add constraint C_HOSTCAPACITY_LLCTDNDSCVRD check (ALLOCATED_UNDISCOVERED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_FCFIBRE add constraint C_HOSTCAPACITY_LLCTDFCFBR check (ALLOCATED_FCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_NONFCFIBRE add constraint C_HOSTCAPACITY_LLCTDNNFCFBR check (ALLOCATED_NONFCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_IDE add constraint C_HOSTCAPACITY_ALLOCATEDIDE check (ALLOCATED_IDE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_ISCSI add constraint C_HOSTCAPACITY_ALLOCATEDISCSI check (ALLOCATED_ISCSI >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_FILESTORAGE add constraint C_HOSTCAPACITY_LLCTDFLSTRG check (ALLOCATED_FILESTORAGE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED_FCFIBRE add constraint C_HOSTCAPACITY_CLAIMEDFCFIBRE check (CLAIMED_FCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED_NONFCFIBRE add constraint C_HOSTCAPACITY_CLMDNNFCFBR check (CLAIMED_NONFCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED_ISCSI add constraint C_HOSTCAPACITY_CLAIMEDISCSI check (CLAIMED_ISCSI >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED_FILESTORAGE add constraint C_HOSTCAPACITY_CLMDFLSTRG check (CLAIMED_FILESTORAGE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTMAPD_ALLOCATED add constraint C_HOSTCAPACITY_DTMAPDALLOCATED check (DTMAPD_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VIRTUALPOOL_ALLOCATED add constraint C_HOSTCAPACITY_VRTLPLLLCTD check (VIRTUALPOOL_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_DAS add constraint C_HOSTCAPACITY_ALLOCATEDDAS check (ALLOCATED_DAS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_TOTALALLOCATED check (TOTAL_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_CLAIMED_OTHER_HOSTS add constraint C_HOSTCAPACITY_LLCTDCLMDTHRHST check (ALLOCATED_CLAIMED_OTHER_HOSTS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ASSIGNED_VM add constraint C_HOSTCAPACITY_ASSIGNEDVM check (ASSIGNED_VM >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTASSIGNED_FS add constraint C_HOSTCAPACITY_DTASSIGNEDFS check (DTASSIGNED_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTASSIGNED_DB add constraint C_HOSTCAPACITY_DTASSIGNEDDB check (DTASSIGNED_DB >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_NKNWNSSGNMNT check (UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED_ASSIGNED_OTHER_HOSTS add constraint C_HOSTCAPACITY_CLMDSSGNDTHRHST check (CLAIMED_ASSIGNED_OTHER_HOSTS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DISKGROUP_CAPACITY add constraint C_HOSTCAPACITY_DSKGRPCPCT check (DISKGROUP_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DISKGROUP_USED add constraint C_HOSTCAPACITY_DISKGROUPUSED check (DISKGROUP_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DISKGROUP_AVAILABLE add constraint C_HOSTCAPACITY_DSKGRPVLBL check (DISKGROUP_AVAILABLE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VOLUME_CAPACITY add constraint C_HOSTCAPACITY_VOLUMECAPACITY check (VOLUME_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VOLUME_UNUSED add constraint C_HOSTCAPACITY_VOLUMEUNUSED check (VOLUME_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VMGR_OVERHEAD add constraint C_HOSTCAPACITY_VMGROVERHEAD check (VMGR_OVERHEAD >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VMGR_ADMINISTRATIVE add constraint C_HOSTCAPACITY_VMGRDMNSTRTV check (VMGR_ADMINISTRATIVE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter FS_CAPACITY add constraint C_HOSTCAPACITY_FSCAPACITY check (FS_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter FS_USED add constraint C_HOSTCAPACITY_FSUSED check (FS_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter FS_AVAILABLE add constraint C_HOSTCAPACITY_FSAVAILABLE check (FS_AVAILABLE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter FS_ON_VOLUME add constraint C_HOSTCAPACITY_FSONVOLUME check (FS_ON_VOLUME >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter FS_ON_NON_VOLUME add constraint C_HOSTCAPACITY_FSONNONVOLUME check (FS_ON_NON_VOLUME >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter FS_USED_APPS add constraint C_HOSTCAPACITY_FSUSEDAPPS check (FS_USED_APPS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter FS_USED_NON_APPS add constraint C_HOSTCAPACITY_FSUSEDNONAPPS check (FS_USED_NON_APPS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_CAPACITY add constraint C_HOSTCAPACITY_DBCAPACITY check (DB_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_USED add constraint C_HOSTCAPACITY_DBUSED check (DB_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_AVAILABLE add constraint C_HOSTCAPACITY_DBAVAILABLE check (DB_AVAILABLE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter APPS_ON_RAW_VOLUME add constraint C_HOSTCAPACITY_APPSONRAWVOLUME check (APPS_ON_RAW_VOLUME >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter APPS_ON_FS add constraint C_HOSTCAPACITY_APPSONFS check (APPS_ON_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter APPS_ON_RAW_LUN add constraint C_HOSTCAPACITY_APPSONRAWLUN check (APPS_ON_RAW_LUN >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter THIN_SUBSCRIBED add constraint C_HOSTCAPACITY_THINSUBSCRIBED check (THIN_SUBSCRIBED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_THIN add constraint C_HOSTCAPACITY_CTHIN check (C_THIN >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ALLOCATED add constraint C_HOSTCAPACITY_CALLOCATED check (C_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_CTOTALALLOCATED check (C_TOTAL_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_CLAIMED add constraint C_HOSTCAPACITY_CCLAIMED check (C_CLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_UNCLAIMED add constraint C_HOSTCAPACITY_CUNCLAIMED check (C_UNCLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ALLOCATED_CLAIMED_OTHER_HOST add constraint C_HOSTCAPACITY_CLLCTDCLMDTHRHS check (C_ALLOCATED_CLAIMED_OTHER_HOST >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ASSIGNED_VM add constraint C_HOSTCAPACITY_CASSIGNEDVM check (C_ASSIGNED_VM >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_DTASSIGNED_FS add constraint C_HOSTCAPACITY_CDTASSIGNEDFS check (C_DTASSIGNED_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_DTASSIGNED_DB add constraint C_HOSTCAPACITY_CDTASSIGNEDDB check (C_DTASSIGNED_DB >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_CNKNWNSSGNMNT check (C_UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_CLAIMED_ASSIGNED_OTHER_HOST add constraint C_HOSTCAPACITY_CCLMDSSGNDTHRHS check (C_CLAIMED_ASSIGNED_OTHER_HOST >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VIRTUALIZATION_CAPACITY add constraint C_HOSTCAPACITY_VRTLZTNCPCT check (VIRTUALIZATION_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VIRTUALIZATION_UNUSED add constraint C_HOSTCAPACITY_VRTLZTNNSD check (VIRTUALIZATION_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter STORAGE_POOL_USED add constraint C_HOSTCAPACITY_STORAGEPOOLUSED check (STORAGE_POOL_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter STORAGE_POOL_UNUSED add constraint C_HOSTCAPACITY_STRGPLNSD check (STORAGE_POOL_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VMDK_UNUSED add constraint C_HOSTCAPACITY_VMDKUNUSED check (VMDK_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter VM_EXPORTED add constraint C_HOSTCAPACITY_VMEXPORTED check (VM_EXPORTED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDLLCTD check (ADJUSTED_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_ALLOCATED_FCFIBRE add constraint C_HOSTCAPACITY_DJSTDLLCTDFCFBR check (ADJUSTED_ALLOCATED_FCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_ALLOCATED_ISCSI add constraint C_HOSTCAPACITY_DJSTDLLCTDSCS check (ADJUSTED_ALLOCATED_ISCSI >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_ALLOCATED_FILESTORAGE add constraint C_HOSTCAPACITY_DJSTDLLCTDFLSTR check (ADJUSTED_ALLOCATED_FILESTORAGE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_CLAIMED add constraint C_HOSTCAPACITY_ADJUSTEDCLAIMED check (ADJUSTED_CLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_CLAIMED_FCFIBRE add constraint C_HOSTCAPACITY_DJSTDCLMDFCFBR check (ADJUSTED_CLAIMED_FCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_CLAIMED_ISCSI add constraint C_HOSTCAPACITY_DJSTDCLMDSCS check (ADJUSTED_CLAIMED_ISCSI >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_CLAIMED_FILESTORAGE add constraint C_HOSTCAPACITY_DJSTDCLMDFLSTRG check (ADJUSTED_CLAIMED_FILESTORAGE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_UNCLAIMED add constraint C_HOSTCAPACITY_DJSTDNCLMD check (ADJUSTED_UNCLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DTMAPED_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDDTMPDLLCTD check (ADJUSTED_DTMAPED_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VIRTUALPOOL_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDVRTLPLLLCT check (ADJUSTED_VIRTUALPOOL_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_DJSTDTTLLLCTD check (ADJUSTED_TOTAL_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_ASSIGNED_VM add constraint C_HOSTCAPACITY_DJSTDSSGNDVM check (ADJUSTED_ASSIGNED_VM >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DTASSIGNED_FS add constraint C_HOSTCAPACITY_DJSTDDTSSGNDFS check (ADJUSTED_DTASSIGNED_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DTASSIGNED_DB add constraint C_HOSTCAPACITY_DJSTDDTSSGNDDB check (ADJUSTED_DTASSIGNED_DB >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_DJSTDNKNWNSSGNM check (ADJUSTED_UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DISKGROUP_CAPACITY add constraint C_HOSTCAPACITY_DJSTDDSKGRPCPCT check (ADJUSTED_DISKGROUP_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DISKGROUP_USED add constraint C_HOSTCAPACITY_DJSTDDSKGRPSD check (ADJUSTED_DISKGROUP_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DISKGROUP_AVAILABLE add constraint C_HOSTCAPACITY_DJSTDDSKGRPVLBL check (ADJUSTED_DISKGROUP_AVAILABLE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VOLUME_CAPACITY add constraint C_HOSTCAPACITY_DJSTDVLMCPCT check (ADJUSTED_VOLUME_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VOLUME_UNUSED add constraint C_HOSTCAPACITY_DJSTDVLMNSD check (ADJUSTED_VOLUME_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VMGR_OVERHEAD add constraint C_HOSTCAPACITY_DJSTDVMGRVRHD check (ADJUSTED_VMGR_OVERHEAD >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VMGR_ADMINISTRATIVE add constraint C_HOSTCAPACITY_DJSTDVMGRDMNSTR check (ADJUSTED_VMGR_ADMINISTRATIVE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_FS_CAPACITY add constraint C_HOSTCAPACITY_DJSTDFSCPCT check (ADJUSTED_FS_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_FS_USED add constraint C_HOSTCAPACITY_ADJUSTEDFSUSED check (ADJUSTED_FS_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_FS_AVAILABLE add constraint C_HOSTCAPACITY_DJSTDFSVLBL check (ADJUSTED_FS_AVAILABLE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_FS_ON_VOLUME add constraint C_HOSTCAPACITY_DJSTDFSNVLM check (ADJUSTED_FS_ON_VOLUME >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_FS_ON_NON_VOLUME add constraint C_HOSTCAPACITY_DJSTDFSNNNVLM check (ADJUSTED_FS_ON_NON_VOLUME >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_FS_USED_APPS add constraint C_HOSTCAPACITY_DJSTDFSSDPPS check (ADJUSTED_FS_USED_APPS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_FS_USED_NON_APPS add constraint C_HOSTCAPACITY_DJSTDFSSDNNPPS check (ADJUSTED_FS_USED_NON_APPS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_CAPACITY add constraint C_HOSTCAPACITY_DJSTDDBCPCT check (ADJUSTED_DB_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_USED add constraint C_HOSTCAPACITY_ADJUSTEDDBUSED check (ADJUSTED_DB_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_AVAILABLE add constraint C_HOSTCAPACITY_DJSTDDBVLBL check (ADJUSTED_DB_AVAILABLE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_APPS_ON_RAW_VOLUME add constraint C_HOSTCAPACITY_DJSTDPPSNRWVLM check (ADJUSTED_APPS_ON_RAW_VOLUME >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_APPS_ON_FS add constraint C_HOSTCAPACITY_DJSTDPPSNFS check (ADJUSTED_APPS_ON_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_THIN_SUBSCRIBED add constraint C_HOSTCAPACITY_DJSTDTHNSBSCRBD check (ADJUSTED_THIN_SUBSCRIBED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_THIN add constraint C_HOSTCAPACITY_CADJUSTEDTHIN check (C_ADJUSTED_THIN >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_ALLOCATED add constraint C_HOSTCAPACITY_CDJSTDLLCTD check (C_ADJUSTED_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_TOTAL_ALLOCATED add constraint C_HOSTCAPACITY_CDJSTDTTLLLCTD check (C_ADJUSTED_TOTAL_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_CLAIMED add constraint C_HOSTCAPACITY_CDJSTDCLMD check (C_ADJUSTED_CLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_UNCLAIMED add constraint C_HOSTCAPACITY_CDJSTDNCLMD check (C_ADJUSTED_UNCLAIMED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_ASSIGNED_VM add constraint C_HOSTCAPACITY_CDJSTDSSGNDVM check (C_ADJUSTED_ASSIGNED_VM >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_DTASSIGNED_FS add constraint C_HOSTCAPACITY_CDJSTDDTSSGNDFS check (C_ADJUSTED_DTASSIGNED_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_DTASSIGNED_DB add constraint C_HOSTCAPACITY_CDJSTDDTSSGNDDB check (C_ADJUSTED_DTASSIGNED_DB >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_UNKNOWN_ASSIGNMENT add constraint C_HOSTCAPACITY_CDJSTDNKNWNSSGN check (C_ADJUSTED_UNKNOWN_ASSIGNMENT >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VIRTUALIZATION_CAP add constraint C_HOSTCAPACITY_DJSTDVRTLZTNCP check (ADJUSTED_VIRTUALIZATION_CAP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VIRTUALIZATION_UNUSED add constraint C_HOSTCAPACITY_DJSTDVRTLZTNNSD check (ADJUSTED_VIRTUALIZATION_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_STORAGE_POOL_USED add constraint C_HOSTCAPACITY_DJSTDSTRGPLSD check (ADJUSTED_STORAGE_POOL_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_STORAGE_POOL_UNUSED add constraint C_HOSTCAPACITY_DJSTDSTRGPLNSD check (ADJUSTED_STORAGE_POOL_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VMDK_UNUSED add constraint C_HOSTCAPACITY_DJSTDVMDKNSD check (ADJUSTED_VMDK_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_VM_EXPORTED add constraint C_HOSTCAPACITY_DJSTDVMXPRTD check (ADJUSTED_VM_EXPORTED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter EXCHANGE_CAPACITY add constraint C_HOSTCAPACITY_XCHNGCPCT check (EXCHANGE_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ASSIGNED_SP add constraint C_HOSTCAPACITY_ASSIGNEDSP check (ASSIGNED_SP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ASSIGNED_SP add constraint C_HOSTCAPACITY_CASSIGNEDSP check (C_ASSIGNED_SP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_ASSIGNED_SP add constraint C_HOSTCAPACITY_DJSTDSSGNDSP check (ADJUSTED_ASSIGNED_SP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_ASSIGNED_SP add constraint C_HOSTCAPACITY_CDJSTDSSGNDSP check (C_ADJUSTED_ASSIGNED_SP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter SP_OVERHEAD add constraint C_HOSTCAPACITY_SPOVERHEAD check (SP_OVERHEAD >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_SP_OVERHEAD add constraint C_HOSTCAPACITY_DJSTDSPVRHD check (ADJUSTED_SP_OVERHEAD >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter NAS_SP add constraint C_HOSTCAPACITY_NASSP check (NAS_SP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter NAS_SP_USED add constraint C_HOSTCAPACITY_NASSPUSED check (NAS_SP_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter NAS_SP_UNUSED add constraint C_HOSTCAPACITY_NASSPUNUSED check (NAS_SP_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_NAS_SP add constraint C_HOSTCAPACITY_ADJUSTEDNASSP check (ADJUSTED_NAS_SP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_NAS_SP_USED add constraint C_HOSTCAPACITY_DJSTDNSSPSD check (ADJUSTED_NAS_SP_USED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_NAS_SP_UNUSED add constraint C_HOSTCAPACITY_DJSTDNSSPNSD check (ADJUSTED_NAS_SP_UNUSED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter EXPORTED_LUNS add constraint C_HOSTCAPACITY_EXPORTEDLUNS check (EXPORTED_LUNS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_EXPORTED_LUNS add constraint C_HOSTCAPACITY_CEXPORTEDLUNS check (C_EXPORTED_LUNS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter EXPORTED_VOLUMES add constraint C_HOSTCAPACITY_EXPORTEDVOLUMES check (EXPORTED_VOLUMES >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter EXPORTED_FS add constraint C_HOSTCAPACITY_EXPORTEDFS check (EXPORTED_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_EXPORTED_LUNS add constraint C_HOSTCAPACITY_DJSTDXPRTDLNS check (ADJUSTED_EXPORTED_LUNS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter C_ADJUSTED_EXPORTED_LUNS add constraint C_HOSTCAPACITY_CDJSTDXPRTDLNS check (C_ADJUSTED_EXPORTED_LUNS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_EXPORTED_VOLUMES add constraint C_HOSTCAPACITY_DJSTDXPRTDVLMS check (ADJUSTED_EXPORTED_VOLUMES >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_EXPORTED_FS add constraint C_HOSTCAPACITY_DJSTDXPRTDFS check (ADJUSTED_EXPORTED_FS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter UNSUPPORTED_ALLOCATED add constraint CKC_UNSUPPORTED_ALLOC_HOST_CAP check (UNSUPPORTED_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_UNSUPPORTED_ALLOCATED add constraint CKC_ADJUSTED_UNSUPPOR_HOST_CAP check (ADJUSTED_UNSUPPORTED_ALLOCATED >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED_DAS add constraint C_HOSTCAPACITY_CLAIMEDDAS check (CLAIMED_DAS >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter CLAIMED_IDE add constraint C_HOSTCAPACITY_CLAIMEDIDE check (CLAIMED_IDE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_UNDISCOVERED_FCFIBRE add constraint C_HOSTCAPACITY_LLCTDNDSCVRDFCF check (ALLOCATED_UNDISCOVERED_FCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_UNDISCOVERED_N_FC_FB add constraint C_HOSTCAPACITY_LLCTDNDSCVRDNFC check (ALLOCATED_UNDISCOVERED_N_FC_FB >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_UNDISCOVERED_IDE add constraint C_HOSTCAPACITY_LLCTDNDSCVRDD check (ALLOCATED_UNDISCOVERED_IDE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_UNDISCOVERED_ISCSI add constraint C_HOSTCAPACITY_LLCTDNDSCVRDSCS check (ALLOCATED_UNDISCOVERED_ISCSI >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ALLOCATED_UNDISCOVERED_FILE_SG add constraint C_HOSTCAPACITY_LLCTDNDSCVRDFLS check (ALLOCATED_UNDISCOVERED_FILE_SG >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTMAPD_ALLOCATED_FCFIBRE add constraint C_HOSTCAPACITY_DTMPDLLCTDFCFBR check (DTMAPD_ALLOCATED_FCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTMAPD_ALLOCATED_NONFCFIBRE add constraint C_HOSTCAPACITY_DTMPDLLCTDNNFCF check (DTMAPD_ALLOCATED_NONFCFIBRE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTMAPD_ALLOCATED_IDE add constraint C_HOSTCAPACITY_DTMPDLLCTDD check (DTMAPD_ALLOCATED_IDE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTMAPD_ALLOCATED_ISCSI add constraint C_HOSTCAPACITY_DTMPDLLCTDSCS check (DTMAPD_ALLOCATED_ISCSI >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DTMAPD_ALLOCATED_FILESTORAGE add constraint C_HOSTCAPACITY_DTMPDLLCTDFLSTR check (DTMAPD_ALLOCATED_FILESTORAGE >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_VM_CAPACITY add constraint CKC_DB_VM_CAPACITY_HOST_CAP check (DB_VM_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_VM_USED_CAPACITY add constraint CKC_DB_VM_USED_CAPACI_HOST_CAP check (DB_VM_USED_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_VM_FREE_CAPACITY add constraint CKC_DB_VM_FREE_CAPACI_HOST_CAP check (DB_VM_FREE_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_VM_OVERHEAD_CAPACITY add constraint CKC_DB_VM_OVERHEAD_CA_HOST_CAP check (DB_VM_OVERHEAD_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter APPS_ON_DB_VM_CAPACITY add constraint CKC_APPS_ON_DB_VM_CAP_HOST_CAP check (APPS_ON_DB_VM_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter APPS_ON_DB_VM_OTHER_CAPACITY add constraint CKC_APPS_ON_DB_VM_OTH_HOST_CAP check (APPS_ON_DB_VM_OTHER_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_VM_DB_USED_CAPACITY add constraint CKC_DB_VM_DB_USED_CAP_HOST_CAP check (DB_VM_DB_USED_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_VM_DB_USED_OTHER_CAPACITY add constraint CKC_DB_VM_DB_USED_OTH_HOST_CAP check (DB_VM_DB_USED_OTHER_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter DB_VM_UNKNOWN_CAPACITY add constraint CKC_DB_VM_UNKNOWN_CAP_HOST_CAP check (DB_VM_UNKNOWN_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_VM_CAPACITY add constraint CKC_ADJUSTED_DB_VM_CA_HOST_CAP check (ADJUSTED_DB_VM_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_VM_USED_CAPACITY add constraint CKC_ADJUSTED_DB_VM_US_HOST_CAP check (ADJUSTED_DB_VM_USED_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_VM_FREE_CAPACITY add constraint CKC_ADJUSTED_DB_VM_FR_HOST_CAP check (ADJUSTED_DB_VM_FREE_CAPACITY >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_VM_OVERHEAD_CAP add constraint CKC_ADJUSTED_DB_VM_OV_HOST_CAP check (ADJUSTED_DB_VM_OVERHEAD_CAP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_APPS_ON_DB_VM_CAP add constraint CKC_ADJUSTED_APPS_ON__HOST_CAP check (ADJUSTED_APPS_ON_DB_VM_CAP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   alter ADJUSTED_DB_VM_DB_USED_CAP add constraint CKC_ADJUSTED_DB_VM_DB_HOST_CAP check (ADJUSTED_DB_VM_DB_USED_CAP >= 0);

alter table SAHARA_APP.HOST_CAPACITY_
   add constraint PK_HOST_CAPACITY_ primary key (HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.HOST_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: LUN                                                   */
/*==============================================================*/
create table SAHARA.LUN 
(
   LUN_KEY              numeric(10)                    not null default autoincrement,
   ARRAY_KEY            numeric(10)                    null,
   NAS_FILESYSTEM_KEY   numeric(10)                    null,
   LUN_OID              varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   DRIVE_TYPE           varchar(32)                    not null,
   RAID_LEVEL           varchar(32)                    null,
   EMULATION_MODE       varchar(32)                    null,
   MASKING_STATUS       varchar(32)                    null,
   CLAIMED_STATUS       varchar(32)                    not null,
   DISCOVERY_STATE      varchar(32)                    null,
   NUMBER_OF_COPIES     numeric(3)                     null,
   PARITY               varchar(32)                    null,
   RESOURCE_TYPE        varchar(32)                    not null,
   REVISION             varchar(256)                   null,
   SERIAL_NUMBER        varchar(256)                   null,
   STRIPE_WIDTH         varchar(256)                   null,
   VENDOR_DEVICE_ID     varchar(256)                   null,
   VENDOR_NAME          varchar(256)                   null,
   VENDOR_STATUS        varchar(256)                   null,
   BINDING_STATE        varchar(32)                    null,
   COLUMNS              varchar(256)                   null,
   TIER                 varchar(64)                    null,
   TIER_COST_PER_GB     numeric(12,6)                  null,
   IS_PRIMARY           char(1)                        not null,
   IS_READ_CACHE_ENABLED char(1)                        not null,
   IS_SECONDARY         char(1)                        not null,
   IS_SEGMENT           char(1)                        not null,
   IS_WRITE_CACHE_ENABLED char(1)                        not null,
   IS_AGGREGATE         char(1)                        not null,
   IS_CONTROL_DEVICE    char(1)                        not null,
   IS_FAULT_TOLERANT    char(1)                        not null,
   IS_ADMINISTRATIVE    char(1)                        not null,
   IS_USED_IN_SNAPSHOT  char(1)                        not null,
   IS_VIRTUALIZER_SOURCE char(1)                        not null,
   IS_DIRECT_ATTACHED   char(1)                        not null,
   IS_BASEDON_EXTERNAL  char(1)                        not null,
   ARRAY_PORTS          numeric(10)                    null default 0,
   DISK_VENDOR          varchar(256)                   null,
   DISK_CAPACITY        numeric(22,9)                  not null default 0,
   VENDOR_LAYOUT        varchar(256)                   null,
   THIN_STATUS          varchar(256)                   null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   VOM_DISCOVER_TYPE    varchar(32)                    null,
   IS_ASSIGNED          char(1)                        null,
   IS_EXPORTED          char(1)                        null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   TYPE                 varchar(32)                    null,
   SUB_TYPE             varchar(64)                    null,
   STORAGE_TECHNOLOGY   varchar(32)                    null,
   STORAGE_POOL_NAME    varchar(256)                   null,
   CONNECTION_PROTOCOL  varchar(32)                    null,
   ARRAY_NAMES          varchar(2560)                  null
);

comment on table SAHARA.LUN is 
'This is the dimension table for LUNs.  It defines the attributes of LUNs that may be used to qualify LUN queries.';

comment on column SAHARA.LUN.ARRAY_KEY is 
'A unique database generated identifier for the array record used as a surrogate key for joins to the ARRAY table.

This may be null.';

comment on column SAHARA.LUN.NAS_FILESYSTEM_KEY is 
'A unique database generated identifier for the NAS filesystem record used as a surrogate key.';

comment on column SAHARA.LUN.LUN_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.LUN.CC_STORAGE_URL is 
'The URL for the LUN object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.LUN.DISPLAY_NAME is 
'LUN display name.';

comment on column SAHARA.LUN.ADVISORY_DISPLAY_NAME is 
'The array name discovered by CommandCentral Storage.

This may be null.';

comment on column SAHARA.LUN.DRIVE_TYPE is 
'The type of the physical disk drive used for creation of the LUN.  It can be either Fibre Channel or ATA.';

comment on column SAHARA.LUN.RAID_LEVEL is 
'The RAID level of the LUN.

Possible values:
- RAID0
- RAID1
- RAID3
- RAID4
- RAID5
- RAID6
- RAID7
- RAID10
- RAID0+1
- RAID053
- Unknown.

This may be null.';

comment on column SAHARA.LUN.EMULATION_MODE is 
'Indicates the emulation mode of the LUN.

This may be null.';

comment on column SAHARA.LUN.MASKING_STATUS is 
'Indicates whether the virtual disk is masked, unmasked or cannot be masked.

This may be null.';

comment on column SAHARA.LUN.CLAIMED_STATUS is 
'Unknown, unclaimed, claimed.';

comment on column SAHARA.LUN.DISCOVERY_STATE is 
'The discovery state of the LUN:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the LUN was initially discovered by two or more explorers and then one of them stopped discovering the LUN.

Note: A LUN that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.LUN.NUMBER_OF_COPIES is 
'The number of copies (mirrors) of the data.  The value one (1) indicates a single copy without mirrors.

This may be null.';

comment on column SAHARA.LUN.PARITY is 
'The number of drives or columns used for parity.  For example: RAID 5.

This may be null.';

comment on column SAHARA.LUN.RESOURCE_TYPE is 
'Indicates the resource type of the addressable unit.  Valid states include: Data Device, System Device, Virtual Copy, and Spare.';

comment on column SAHARA.LUN.REVISION is 
'The object''s vendor-supplied firmware revision number.

This may be null.';

comment on column SAHARA.LUN.SERIAL_NUMBER is 
'Vendor assigned serial number

This may be null.';

comment on column SAHARA.LUN.STRIPE_WIDTH is 
'Indicates the number of bytes in a stripe of data on the disk.

This may be null.';

comment on column SAHARA.LUN.VENDOR_DEVICE_ID is 
'Vendor assigned ID that identifies the device.

This may be null.';

comment on column SAHARA.LUN.VENDOR_NAME is 
'The name of the company that manufactured the physical device.  In the case of a virtual object, such as a volume, this is the name of the company that manufactured the physical device upon which the volume resides.

This may be null.';

comment on column SAHARA.LUN.VENDOR_STATUS is 
'Vendor specific state of the object.

This may be null.';

comment on column SAHARA.LUN.BINDING_STATE is 
'The binding state of the addressable unit.

Possible values:
- Unknown
- Unbound
- Bound
- Unbindable

This may be null.';

comment on column SAHARA.LUN.COLUMNS is 
'Number of data columns used for striping.  A value of 1 indicates striping is not used.

This may be null.';

comment on column SAHARA.LUN.TIER is 
'The storage tier.

This may be null.';

comment on column SAHARA.LUN.TIER_COST_PER_GB is 
'The cost of storage for the associated tier per GB.  This is copied from the tier configuration when the tier value is assigned.

This may be null.';

comment on column SAHARA.LUN.IS_PRIMARY is 
'A boolean flag that indicates that this is a primary virtual disk from which snapshots can be made.  This is only applicable to arrays that use primary/secondary model.

Possible values:
''T'' - the LUN is a primary virtual disk
''F'' - the LUN is not a primary virtual disk';

comment on column SAHARA.LUN.IS_READ_CACHE_ENABLED is 
'A boolean flag that indicates if the LUN is read cache enabled.

Possible values:
''T'' - the LUN is a read cache enabled
''F'' - the LUN is not read cache enabled';

comment on column SAHARA.LUN.IS_SECONDARY is 
'A boolean flag that indicates that this is a secondary virtual disk.  This is only applicable to arrays that use the primary/secondary model.

Possible values:
''T'' - the LUN is a secondary virtual disk
''F'' - the LUN is not a secondary virtual disk';

comment on column SAHARA.LUN.IS_SEGMENT is 
'A boolean flag that indicates that the virtual disk is a segment of a larger virtual disk.

Possible values:
''T'' - the LUN is a segment of a larger virtual disk
''F'' - the LUN is not a segment of a larger virtual disk';

comment on column SAHARA.LUN.IS_WRITE_CACHE_ENABLED is 
'A boolean flag that indicates if write caching is enabled for the LUN.

Possible values:
''T'' - the LUN is write-cache enabled
''F'' - the LUN is not write cache enabled';

comment on column SAHARA.LUN.IS_AGGREGATE is 
'A boolean flag that indicates that the virtual disk is a combination of other virtual disks.  A virtual disk can be both a segment and an aggregate if the virtual disk is in a middle layer with an aggregate virtual disk above and the segment virtual disks below.  In such a case, both segment and aggregate are true.

Possible values:
''T'' - the LUN is an aggregate
''F'' - the LUN is not an aggregate';

comment on column SAHARA.LUN.IS_CONTROL_DEVICE is 
'A boolean flag that indicates if the device is used only for sending control commands to the array.  This tells applications to ignore the device.

Possible values:
''T'' - the LUN is a control device
''F'' - the LUN is not a control device';

comment on column SAHARA.LUN.IS_FAULT_TOLERANT is 
'A boolean flag that indicates whether the virtual disk layout is fault tolerant.

Possible values:
''T'' - the LUN is fault tolerant
''F'' - the LUN is not fault tolerant';

comment on column SAHARA.LUN.IS_ADMINISTRATIVE is 
'A boolean flag that indicates a LUN set aside for a BCV, shadow image, control device, and other administrative task.

Possible values:
''T'' - the LUN is administrative
''F'' - the LUN is not administrative';

comment on column SAHARA.LUN.IS_USED_IN_SNAPSHOT is 
'A boolean flag that indicates that the LUN is secondary and used in snapshot.

Possible values:
''T'' - the LUN is secondary and is used in a snapshot
''F'' - the LUN is not secondary and/or is not used in a snapshot';

comment on column SAHARA.LUN.IS_VIRTUALIZER_SOURCE is 
'A boolean flag that indicates that a LUN that is a virtualizer source.  A virtualizer source is allocated to a virtualizer array such as Tagma or IBM SAN Volume Controller (SVC).

Possible values:
''T'' - the LUN is a virtualizer source
''F'' - the LUN is not a virtualizer source';

comment on column SAHARA.LUN.IS_DIRECT_ATTACHED is 
'A boolean flag that indicates if the device is direct attached.

Possible values:
''T'' - the LUN is direct attached
''F'' - the LUN is not direct attached';

comment on column SAHARA.LUN.IS_BASEDON_EXTERNAL is 
'A boolean flag that indicates if the device is based on external.

Possible values:
''T'' - the LUN is based on external
''F'' - the LUN is not based on external';

comment on column SAHARA.LUN.ARRAY_PORTS is 
'The number of array ports device is connected to.';

comment on column SAHARA.LUN.DISK_VENDOR is 
'The name of the company that manufactured the physical disk from which this lun is made from.

This may be null.';

comment on column SAHARA.LUN.DISK_CAPACITY is 
'Total capacity of the physical disk from which this lun is made from.';

comment on column SAHARA.LUN.VENDOR_LAYOUT is 
'Vendor name for the storage layout used by this device.

This may be null.';

comment on column SAHARA.LUN.THIN_STATUS is 
'Thin provisioning specific state of the lun.  
This attribute tells whether the object is thin or thin pool source.  For non-thin luns this attribute is not applicable.

This may be null.';

comment on column SAHARA.LUN.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.LUN.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.LUN.IS_ASSIGNED is 
'A boolean flag that indicates that the virtual disk is assigned.

Possible values:
''T'' - the LUN is assigned
''F'' - the LUN is not assigned';

comment on column SAHARA.LUN.IS_EXPORTED is 
'A boolean flag that indicates if the device is exported.

Possible values:
''T'' - the device is exported
''F'' - the device is not exported';

comment on column SAHARA.LUN.APPLICATION_BE is 
'Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.LUN.APPLICATION_COMPOSITE_BE is 
'Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.LUN.ORGANIZATION_BE is 
'Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.LUN.ORGANIZATION_COMPOSITE_BE is 
'Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.LUN.TYPE is 
'This attribute differentiate between VMDK and Disk.';

comment on column SAHARA.LUN.SUB_TYPE is 
'For VMDK, this differentiate between RDM, Sparse and Flat VMDKs.';

comment on column SAHARA.LUN.STORAGE_TECHNOLOGY is 
'This attribute represents the storage technology used. Possible values are SAN, DAS and NAS.';

comment on column SAHARA.LUN.STORAGE_POOL_NAME is 
'This attribute represents name of the data store associated to a VMware VMDK';

comment on column SAHARA.LUN.CONNECTION_PROTOCOL is 
'This attribute represents networking protocol used for accessing storage';

comment on column SAHARA.LUN.ARRAY_NAMES is 
'Names of array. Multiple in case of VMDK carved out from SP deriving storage from multiple arrays.';

alter table SAHARA.LUN
   alter IS_PRIMARY add constraint C_LUN_ISPRIMARY check (IS_PRIMARY in ('T','F'));

alter table SAHARA.LUN
   alter IS_READ_CACHE_ENABLED add constraint C_LUN_ISREADCACHEENABLED check (IS_READ_CACHE_ENABLED in ('T','F'));

alter table SAHARA.LUN
   alter IS_SECONDARY add constraint C_LUN_ISSECONDARY check (IS_SECONDARY in ('T','F'));

alter table SAHARA.LUN
   alter IS_SEGMENT add constraint C_LUN_ISSEGMENT check (IS_SEGMENT in ('T','F'));

alter table SAHARA.LUN
   alter IS_WRITE_CACHE_ENABLED add constraint C_LUN_ISWRITECACHEENABLED check (IS_WRITE_CACHE_ENABLED in ('T','F'));

alter table SAHARA.LUN
   alter IS_AGGREGATE add constraint C_LUN_ISAGGREGATE check (IS_AGGREGATE in ('T','F'));

alter table SAHARA.LUN
   alter IS_CONTROL_DEVICE add constraint C_LUN_ISCONTROLDEVICE check (IS_CONTROL_DEVICE in ('T','F'));

alter table SAHARA.LUN
   alter IS_FAULT_TOLERANT add constraint C_LUN_ISFAULTTOLERANT check (IS_FAULT_TOLERANT in ('T','F'));

alter table SAHARA.LUN
   alter IS_ADMINISTRATIVE add constraint C_LUN_ISADMINISTRATIVE check (IS_ADMINISTRATIVE in ('T','F'));

alter table SAHARA.LUN
   alter IS_USED_IN_SNAPSHOT add constraint C_LUN_ISUSEDINSNAPSHOT check (IS_USED_IN_SNAPSHOT in ('T','F'));

alter table SAHARA.LUN
   alter IS_VIRTUALIZER_SOURCE add constraint C_LUN_ISVIRTUALIZERSOURCE check (IS_VIRTUALIZER_SOURCE in ('T','F'));

alter table SAHARA.LUN
   alter IS_DIRECT_ATTACHED add constraint C_LUN_ISDIRECTATTACHED check (IS_DIRECT_ATTACHED in ('T','F'));

alter table SAHARA.LUN
   alter IS_BASEDON_EXTERNAL add constraint C_LUN_ISBASEDONEXTERNAL check (IS_BASEDON_EXTERNAL in ('T','F'));

alter table SAHARA.LUN
   alter ARRAY_PORTS add constraint C_LUN_ARRAYPORTS check (ARRAY_PORTS is null or (ARRAY_PORTS >= 0));

alter table SAHARA.LUN
   alter DISK_CAPACITY add constraint C_LUN_DISKCAPACITY check (DISK_CAPACITY >= 0);

alter table SAHARA.LUN
   alter T2_IS_CURRENT add constraint C_LUN_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.LUN
   alter T2_IS_DELETED add constraint C_LUN_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.LUN
   alter IS_ASSIGNED add constraint C_LUN_ISASSIGNED check (IS_ASSIGNED is null or (IS_ASSIGNED in ('T','F')));

alter table SAHARA.LUN
   alter IS_EXPORTED add constraint C_VOLUME_ISEXPORTED check (IS_EXPORTED is null or (IS_EXPORTED in ('T','F')));

alter table SAHARA.LUN
   add constraint PK_LUN primary key (LUN_KEY);

/*==============================================================*/
/* Index: I_LUN_ARRAYKEY                                        */
/*==============================================================*/
create index I_LUN_ARRAYKEY on SAHARA.LUN (
ARRAY_KEY ASC
);

comment on index SAHARA.LUN.I_LUN_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_NASFILESYSTEMKEY                                */
/*==============================================================*/
create index I_LUN_NASFILESYSTEMKEY on SAHARA.LUN (
NAS_FILESYSTEM_KEY ASC
);

comment on index SAHARA.LUN.I_LUN_NASFILESYSTEMKEY is 
'An index on NAS Filesystem Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_LUNOID                                          */
/*==============================================================*/
create index I_LUN_LUNOID on SAHARA.LUN (
LUN_OID ASC
);

comment on index SAHARA.LUN.I_LUN_LUNOID is 
'An index on LUN Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_DISPLAYNAME                                     */
/*==============================================================*/
create index I_LUN_DISPLAYNAME on SAHARA.LUN (
DISPLAY_NAME ASC
);

comment on index SAHARA.LUN.I_LUN_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ADVISORYDISPLAYNAME                             */
/*==============================================================*/
create index I_LUN_ADVISORYDISPLAYNAME on SAHARA.LUN (
ADVISORY_DISPLAY_NAME ASC
);

comment on index SAHARA.LUN.I_LUN_ADVISORYDISPLAYNAME is 
'An index on Advisory Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_DRIVETYPE                                       */
/*==============================================================*/
create index I_LUN_DRIVETYPE on SAHARA.LUN (
DRIVE_TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_DRIVETYPE is 
'An index on Drive Type for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_RAIDLEVEL                                       */
/*==============================================================*/
create index I_LUN_RAIDLEVEL on SAHARA.LUN (
RAID_LEVEL ASC
);

comment on index SAHARA.LUN.I_LUN_RAIDLEVEL is 
'An index on RAID Level for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_EMULATIONMODE                                   */
/*==============================================================*/
create index I_LUN_EMULATIONMODE on SAHARA.LUN (
EMULATION_MODE ASC
);

comment on index SAHARA.LUN.I_LUN_EMULATIONMODE is 
'An index on Emulation Mode for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_MASKINGSTATUS                                   */
/*==============================================================*/
create index I_LUN_MASKINGSTATUS on SAHARA.LUN (
MASKING_STATUS ASC
);

comment on index SAHARA.LUN.I_LUN_MASKINGSTATUS is 
'An index on Masking Status for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_CLAIMEDSTATUS                                   */
/*==============================================================*/
create index I_LUN_CLAIMEDSTATUS on SAHARA.LUN (
CLAIMED_STATUS ASC
);

comment on index SAHARA.LUN.I_LUN_CLAIMEDSTATUS is 
'An index on Claimed Status for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_DISCOVERYSTATE                                  */
/*==============================================================*/
create index I_LUN_DISCOVERYSTATE on SAHARA.LUN (
DISCOVERY_STATE ASC
);

comment on index SAHARA.LUN.I_LUN_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_NUMBEROFCOPIES                                  */
/*==============================================================*/
create index I_LUN_NUMBEROFCOPIES on SAHARA.LUN (
NUMBER_OF_COPIES ASC
);

comment on index SAHARA.LUN.I_LUN_NUMBEROFCOPIES is 
'An index on Number of Copies for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_PARITY                                          */
/*==============================================================*/
create index I_LUN_PARITY on SAHARA.LUN (
PARITY ASC
);

comment on index SAHARA.LUN.I_LUN_PARITY is 
'An index on Parity for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_RESOURCETYPE                                    */
/*==============================================================*/
create index I_LUN_RESOURCETYPE on SAHARA.LUN (
RESOURCE_TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_RESOURCETYPE is 
'An index on Resource Type for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_REVISION                                        */
/*==============================================================*/
create index I_LUN_REVISION on SAHARA.LUN (
REVISION ASC
);

comment on index SAHARA.LUN.I_LUN_REVISION is 
'An index on Revision for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_SERIALNUMBER                                    */
/*==============================================================*/
create index I_LUN_SERIALNUMBER on SAHARA.LUN (
SERIAL_NUMBER ASC
);

comment on index SAHARA.LUN.I_LUN_SERIALNUMBER is 
'An index on Serial Number for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_STRIPEWIDTH                                     */
/*==============================================================*/
create index I_LUN_STRIPEWIDTH on SAHARA.LUN (
STRIPE_WIDTH ASC
);

comment on index SAHARA.LUN.I_LUN_STRIPEWIDTH is 
'An index on Stripe Width for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_VENDORDEVICEID                                  */
/*==============================================================*/
create index I_LUN_VENDORDEVICEID on SAHARA.LUN (
VENDOR_DEVICE_ID ASC
);

comment on index SAHARA.LUN.I_LUN_VENDORDEVICEID is 
'An index on Vendor Device ID for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_VENDORNAME                                      */
/*==============================================================*/
create index I_LUN_VENDORNAME on SAHARA.LUN (
VENDOR_NAME ASC
);

comment on index SAHARA.LUN.I_LUN_VENDORNAME is 
'An index on Vendor Name for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_VENDORSTATUS                                    */
/*==============================================================*/
create index I_LUN_VENDORSTATUS on SAHARA.LUN (
VENDOR_STATUS ASC
);

comment on index SAHARA.LUN.I_LUN_VENDORSTATUS is 
'An index on Vendor Status for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_BINDINGSTATE                                    */
/*==============================================================*/
create index I_LUN_BINDINGSTATE on SAHARA.LUN (
BINDING_STATE ASC
);

comment on index SAHARA.LUN.I_LUN_BINDINGSTATE is 
'An index on Binding State for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_COLUMNS                                         */
/*==============================================================*/
create index I_LUN_COLUMNS on SAHARA.LUN (
COLUMNS ASC
);

comment on index SAHARA.LUN.I_LUN_COLUMNS is 
'An index on Columns for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_TIER                                            */
/*==============================================================*/
create index I_LUN_TIER on SAHARA.LUN (
TIER ASC
);

comment on index SAHARA.LUN.I_LUN_TIER is 
'An index on Tier for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_TIERCOSTPERGB                                   */
/*==============================================================*/
create index I_LUN_TIERCOSTPERGB on SAHARA.LUN (
TIER_COST_PER_GB ASC
);

comment on index SAHARA.LUN.I_LUN_TIERCOSTPERGB is 
'An index on Tier Cost per GB for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISPRIMARY                                       */
/*==============================================================*/
create index I_LUN_ISPRIMARY on SAHARA.LUN (
IS_PRIMARY ASC
);

comment on index SAHARA.LUN.I_LUN_ISPRIMARY is 
'An index on Is Primary for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISREADCACHEENABLED                              */
/*==============================================================*/
create index I_LUN_ISREADCACHEENABLED on SAHARA.LUN (
IS_READ_CACHE_ENABLED ASC
);

comment on index SAHARA.LUN.I_LUN_ISREADCACHEENABLED is 
'An index on Is Read Cache Enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISSECONDARY                                     */
/*==============================================================*/
create index I_LUN_ISSECONDARY on SAHARA.LUN (
IS_SECONDARY ASC
);

comment on index SAHARA.LUN.I_LUN_ISSECONDARY is 
'An index on Is Secondary for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISSEGMENT                                       */
/*==============================================================*/
create index I_LUN_ISSEGMENT on SAHARA.LUN (
IS_SEGMENT ASC
);

comment on index SAHARA.LUN.I_LUN_ISSEGMENT is 
'An index on Is Segment for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISWRITECACHEENABLED                             */
/*==============================================================*/
create index I_LUN_ISWRITECACHEENABLED on SAHARA.LUN (
IS_WRITE_CACHE_ENABLED ASC
);

comment on index SAHARA.LUN.I_LUN_ISWRITECACHEENABLED is 
'An index on Is Write Cache Enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISAGGREGATE                                     */
/*==============================================================*/
create index I_LUN_ISAGGREGATE on SAHARA.LUN (
IS_AGGREGATE ASC
);

comment on index SAHARA.LUN.I_LUN_ISAGGREGATE is 
'An index on Is Aggregate for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISCONTROLDEVICE                                 */
/*==============================================================*/
create index I_LUN_ISCONTROLDEVICE on SAHARA.LUN (
IS_CONTROL_DEVICE ASC
);

comment on index SAHARA.LUN.I_LUN_ISCONTROLDEVICE is 
'An index on Is Control Device for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISFAULTTOLERANT                                 */
/*==============================================================*/
create index I_LUN_ISFAULTTOLERANT on SAHARA.LUN (
IS_FAULT_TOLERANT ASC
);

comment on index SAHARA.LUN.I_LUN_ISFAULTTOLERANT is 
'An index on Is Fault Tolerant for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISADMINISTRATIVE                                */
/*==============================================================*/
create index I_LUN_ISADMINISTRATIVE on SAHARA.LUN (
IS_ADMINISTRATIVE ASC
);

comment on index SAHARA.LUN.I_LUN_ISADMINISTRATIVE is 
'An index on Is Administrative for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISUSEDINSNAPSHOT                                */
/*==============================================================*/
create index I_LUN_ISUSEDINSNAPSHOT on SAHARA.LUN (
IS_USED_IN_SNAPSHOT ASC
);

comment on index SAHARA.LUN.I_LUN_ISUSEDINSNAPSHOT is 
'An index on Is Used In Snapshot for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISVIRTUALIZERSOURCE                             */
/*==============================================================*/
create index I_LUN_ISVIRTUALIZERSOURCE on SAHARA.LUN (
IS_VIRTUALIZER_SOURCE ASC
);

comment on index SAHARA.LUN.I_LUN_ISVIRTUALIZERSOURCE is 
'An index on Is Virtualizer Source for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISDIRECTATTACHED                                */
/*==============================================================*/
create index I_LUN_ISDIRECTATTACHED on SAHARA.LUN (
IS_DIRECT_ATTACHED ASC
);

comment on index SAHARA.LUN.I_LUN_ISDIRECTATTACHED is 
'An index on Is Direct Attached for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISBASEDONEXTERNAL                               */
/*==============================================================*/
create index I_LUN_ISBASEDONEXTERNAL on SAHARA.LUN (
IS_BASEDON_EXTERNAL ASC
);

comment on index SAHARA.LUN.I_LUN_ISBASEDONEXTERNAL is 
'An index on Is Basedon External for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ARRAYPORTS                                      */
/*==============================================================*/
create index I_LUN_ARRAYPORTS on SAHARA.LUN (
ARRAY_PORTS ASC
);

comment on index SAHARA.LUN.I_LUN_ARRAYPORTS is 
'An index on Array Ports for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_DISKVENDOR                                      */
/*==============================================================*/
create index I_LUN_DISKVENDOR on SAHARA.LUN (
DISK_VENDOR ASC
);

comment on index SAHARA.LUN.I_LUN_DISKVENDOR is 
'An index on Disk Vendor for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_DISKCAPACITY                                    */
/*==============================================================*/
create index I_LUN_DISKCAPACITY on SAHARA.LUN (
DISK_CAPACITY ASC
);

comment on index SAHARA.LUN.I_LUN_DISKCAPACITY is 
'An index on Disk Capacity for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_VENDORLAYOUT                                    */
/*==============================================================*/
create index I_LUN_VENDORLAYOUT on SAHARA.LUN (
VENDOR_LAYOUT ASC
);

comment on index SAHARA.LUN.I_LUN_VENDORLAYOUT is 
'An index on Vendor Layout for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_THINSTATUS                                      */
/*==============================================================*/
create index I_LUN_THINSTATUS on SAHARA.LUN (
THIN_STATUS ASC
);

comment on index SAHARA.LUN.I_LUN_THINSTATUS is 
'An index on Thin Status for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_T2ISCURRENTLUNKEY                               */
/*==============================================================*/
create index I_LUN_T2ISCURRENTLUNKEY on SAHARA.LUN (
T2_IS_CURRENT ASC,
LUN_KEY ASC
);

comment on index SAHARA.LUN.I_LUN_T2ISCURRENTLUNKEY is 
'An index on Type 2 - Is Current and LUN Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_LUN_T2SCRRNTT2SDLTDD                                */
/*==============================================================*/
create index I_LUN_T2SCRRNTT2SDLTDD on SAHARA.LUN (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
LUN_OID ASC
);

comment on index SAHARA.LUN.I_LUN_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and LUN Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_LUN_T2ISDELETED                                     */
/*==============================================================*/
create index I_LUN_T2ISDELETED on SAHARA.LUN (
T2_IS_DELETED ASC
);

comment on index SAHARA.LUN.I_LUN_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_VOMDISCOVERTYPE                                 */
/*==============================================================*/
create index I_LUN_VOMDISCOVERTYPE on SAHARA.LUN (
VOM_DISCOVER_TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_VOMDISCOVERTYPE is 
'An index on VOM Discover type for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISASSIGNED                                      */
/*==============================================================*/
create index I_LUN_ISASSIGNED on SAHARA.LUN (
IS_ASSIGNED ASC
);

comment on index SAHARA.LUN.I_LUN_ISASSIGNED is 
'An index on Is Assigned for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ISEXPORTED                                      */
/*==============================================================*/
create index I_LUN_ISEXPORTED on SAHARA.LUN (
IS_EXPORTED ASC
);

comment on index SAHARA.LUN.I_LUN_ISEXPORTED is 
'An index on Is Exported for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_APPLICATIONBE                                   */
/*==============================================================*/
create index I_LUN_APPLICATIONBE on SAHARA.LUN (
APPLICATION_BE ASC
);

comment on index SAHARA.LUN.I_LUN_APPLICATIONBE is 
'An index on Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_APPLICATIONCOMPOSITEBE                          */
/*==============================================================*/
create index I_LUN_APPLICATIONCOMPOSITEBE on SAHARA.LUN (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.LUN.I_LUN_APPLICATIONCOMPOSITEBE is 
'An index on Composite Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ORGANIZATIONBE                                  */
/*==============================================================*/
create index I_LUN_ORGANIZATIONBE on SAHARA.LUN (
ORGANIZATION_BE ASC
);

comment on index SAHARA.LUN.I_LUN_ORGANIZATIONBE is 
'An index on Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_ORGANIZATIONCOMPOSITEBE                         */
/*==============================================================*/
create index I_LUN_ORGANIZATIONCOMPOSITEBE on SAHARA.LUN (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.LUN.I_LUN_ORGANIZATIONCOMPOSITEBE is 
'An index on Composite Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_TYPE                                            */
/*==============================================================*/
create index I_LUN_TYPE on SAHARA.LUN (
TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_SUBTYPE                                         */
/*==============================================================*/
create index I_LUN_SUBTYPE on SAHARA.LUN (
SUB_TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_SUBTYPE is 
'An index on Sub Type for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_STORAGETECHNOLOGY                               */
/*==============================================================*/
create index I_LUN_STORAGETECHNOLOGY on SAHARA.LUN (
STORAGE_TECHNOLOGY ASC
);

comment on index SAHARA.LUN.I_LUN_STORAGETECHNOLOGY is 
'An index on Storage Technology for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_STORAGEPOOLNAME                                 */
/*==============================================================*/
create index I_LUN_STORAGEPOOLNAME on SAHARA.LUN (
STORAGE_POOL_NAME ASC
);

comment on index SAHARA.LUN.I_LUN_STORAGEPOOLNAME is 
'An index on Storage Pool Name for dimensional queries.';

/*==============================================================*/
/* Index: I_LUN_CONNECTIONPROTOCOL                              */
/*==============================================================*/
create index I_LUN_CONNECTIONPROTOCOL on SAHARA.LUN (
CONNECTION_PROTOCOL ASC
);

comment on index SAHARA.LUN.I_LUN_CONNECTIONPROTOCOL is 
'An index on Connection Protocol for dimensional queries.';

grant SELECT on SAHARA.LUN to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_                                                  */
/*==============================================================*/
create table SAHARA_APP.LUN_ 
(
   LUN_OID              varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   DRIVE_TYPE           varchar(32)                    not null,
   RAID_LEVEL           varchar(32)                    null,
   EMULATION_MODE       varchar(32)                    null,
   MASKING_STATUS       varchar(32)                    null,
   CLAIMED_STATUS       varchar(32)                    not null,
   DISCOVERY_STATE      varchar(32)                    null,
   NUMBER_OF_COPIES     numeric(3)                     null,
   PARITY               varchar(32)                    null,
   RESOURCE_TYPE        varchar(32)                    not null,
   REVISION             varchar(256)                   null,
   SERIAL_NUMBER        varchar(256)                   null,
   STRIPE_WIDTH         varchar(256)                   null,
   VENDOR_DEVICE_ID     varchar(256)                   null,
   VENDOR_NAME          varchar(256)                   null,
   VENDOR_STATUS        varchar(256)                   null,
   BINDING_STATE        varchar(32)                    null,
   COLUMNS              varchar(256)                   null,
   IS_PRIMARY           char(1)                        not null,
   IS_READ_CACHE_ENABLED char(1)                        not null,
   IS_SECONDARY         char(1)                        not null,
   IS_SEGMENT           char(1)                        not null,
   IS_WRITE_CACHE_ENABLED char(1)                        not null,
   IS_AGGREGATE         char(1)                        not null,
   IS_CONTROL_DEVICE    char(1)                        not null,
   IS_FAULT_TOLERANT    char(1)                        not null,
   IS_ADMINISTRATIVE    char(1)                        not null,
   IS_USED_IN_SNAPSHOT  char(1)                        not null,
   IS_VIRTUALIZER_SOURCE char(1)                        not null,
   IS_DIRECT_ATTACHED   char(1)                        not null,
   IS_BASEDON_EXTERNAL  char(1)                        not null,
   ARRAY_PORTS          numeric(10)                    null default 0,
   DISK_VENDOR          varchar(256)                   null,
   DISK_CAPACITY        numeric(22,9)                  not null default 0,
   VENDOR_LAYOUT        varchar(256)                   null,
   THIN_STATUS          varchar(256)                   null,
   TIER                 varchar(64)                    null,
   TIER_COST_PER_GB     numeric(12,6)                  null,
   VOM_DISCOVER_TYPE    varchar(32)                    null,
   IS_ASSIGNED          char(1)                        null,
   IS_EXPORTED          char(1)                        null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   TYPE                 varchar(32)                    null,
   SUB_TYPE             varchar(64)                    null,
   STORAGE_TECHNOLOGY   varchar(32)                    null,
   STORAGE_POOL_NAME    varchar(256)                   null,
   CONNECTION_PROTOCOL  varchar(32)                    null,
   NAS_FILESYSTEM_OID   varchar(512)                   null,
   ARRAY_NAMES          varchar(2560)                  null
);

comment on table SAHARA_APP.LUN_ is 
'This is a staging table for loading data into warehouse table SAHARA.LUN.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for LUNs.  It defines the attributes of LUNs that may be used to qualify LUN queries.';

comment on column SAHARA_APP.LUN_.LUN_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.LUN, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

LUN display name.';

comment on column SAHARA_APP.LUN_.ADVISORY_DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.LUN, column ADVISORY_DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The array name discovered by CommandCentral Storage.

This may be null.';

comment on column SAHARA_APP.LUN_.DRIVE_TYPE is 
'This column is for staging data that will end up in table SAHARA.LUN, column DRIVE_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The type of the physical disk drive used for creation of the LUN.  It can be either Fibre Channel or ATA.';

comment on column SAHARA_APP.LUN_.RAID_LEVEL is 
'This column is for staging data that will end up in table SAHARA.LUN, column RAID_LEVEL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The RAID level of the LUN.

Possible values:
- RAID0
- RAID1
- RAID3
- RAID4
- RAID5
- RAID6
- RAID7
- RAID10
- RAID0+1
- RAID053
- Unknown.

This may be null.';

comment on column SAHARA_APP.LUN_.EMULATION_MODE is 
'This column is for staging data that will end up in table SAHARA.LUN, column EMULATION_MODE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates the emulation mode of the LUN.

This may be null.';

comment on column SAHARA_APP.LUN_.MASKING_STATUS is 
'This column is for staging data that will end up in table SAHARA.LUN, column MASKING_STATUS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates whether the virtual disk is masked, unmasked or cannot be masked.

This may be null.';

comment on column SAHARA_APP.LUN_.CLAIMED_STATUS is 
'This column is for staging data that will end up in table SAHARA.LUN, column CLAIMED_STATUS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Unknown, unclaimed, claimed.';

comment on column SAHARA_APP.LUN_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.LUN, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the LUN:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the LUN was initially discovered by two or more explorers and then one of them stopped discovering the LUN.

Note: A LUN that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.LUN_.NUMBER_OF_COPIES is 
'This column is for staging data that will end up in table SAHARA.LUN, column NUMBER_OF_COPIES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of copies (mirrors) of the data.  The value one (1) indicates a single copy without mirrors.

This may be null.';

comment on column SAHARA_APP.LUN_.PARITY is 
'This column is for staging data that will end up in table SAHARA.LUN, column PARITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of drives or columns used for parity.  For example: RAID 5.

This may be null.';

comment on column SAHARA_APP.LUN_.RESOURCE_TYPE is 
'This column is for staging data that will end up in table SAHARA.LUN, column RESOURCE_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates the resource type of the addressable unit.  Valid states include: Data Device, System Device, Virtual Copy, and Spare.';

comment on column SAHARA_APP.LUN_.REVISION is 
'This column is for staging data that will end up in table SAHARA.LUN, column REVISION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The object''s vendor-supplied firmware revision number.

This may be null.';

comment on column SAHARA_APP.LUN_.SERIAL_NUMBER is 
'This column is for staging data that will end up in table SAHARA.LUN, column SERIAL_NUMBER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Vendor assigned serial number

This may be null.';

comment on column SAHARA_APP.LUN_.STRIPE_WIDTH is 
'This column is for staging data that will end up in table SAHARA.LUN, column STRIPE_WIDTH.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates the number of bytes in a stripe of data on the disk.

This may be null.';

comment on column SAHARA_APP.LUN_.VENDOR_DEVICE_ID is 
'This column is for staging data that will end up in table SAHARA.LUN, column VENDOR_DEVICE_ID.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Vendor assigned ID that identifies the device.

This may be null.';

comment on column SAHARA_APP.LUN_.VENDOR_NAME is 
'This column is for staging data that will end up in table SAHARA.LUN, column VENDOR_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The name of the company that manufactured the physical device.  In the case of a virtual object, such as a volume, this is the name of the company that manufactured the physical device upon which the volume resides.

This may be null.';

comment on column SAHARA_APP.LUN_.VENDOR_STATUS is 
'This column is for staging data that will end up in table SAHARA.LUN, column VENDOR_STATUS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Vendor specific state of the object.

This may be null.';

comment on column SAHARA_APP.LUN_.BINDING_STATE is 
'This column is for staging data that will end up in table SAHARA.LUN, column BINDING_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The binding state of the addressable unit.

Possible values:
- Unknown
- Unbound
- Bound
- Unbindable

This may be null.';

comment on column SAHARA_APP.LUN_.COLUMNS is 
'This column is for staging data that will end up in table SAHARA.LUN, column COLUMNS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Number of data columns used for striping.  A value of 1 indicates striping is not used.

This may be null.';

comment on column SAHARA_APP.LUN_.IS_PRIMARY is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_PRIMARY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates that this is a primary virtual disk from which snapshots can be made.  This is only applicable to arrays that use primary/secondary model.

Possible values:
''T'' - the LUN is a primary virtual disk
''F'' - the LUN is not a primary virtual disk';

comment on column SAHARA_APP.LUN_.IS_READ_CACHE_ENABLED is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_READ_CACHE_ENABLED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the LUN is read cache enabled.

Possible values:
''T'' - the LUN is a read cache enabled
''F'' - the LUN is not read cache enabled';

comment on column SAHARA_APP.LUN_.IS_SECONDARY is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_SECONDARY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates that this is a secondary virtual disk.  This is only applicable to arrays that use the primary/secondary model.

Possible values:
''T'' - the LUN is a secondary virtual disk
''F'' - the LUN is not a secondary virtual disk';

comment on column SAHARA_APP.LUN_.IS_SEGMENT is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_SEGMENT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates that the virtual disk is a segment of a larger virtual disk.

Possible values:
''T'' - the LUN is a segment of a larger virtual disk
''F'' - the LUN is not a segment of a larger virtual disk';

comment on column SAHARA_APP.LUN_.IS_WRITE_CACHE_ENABLED is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_WRITE_CACHE_ENABLED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if write caching is enabled for the LUN.

Possible values:
''T'' - the LUN is write-cache enabled
''F'' - the LUN is not write cache enabled';

comment on column SAHARA_APP.LUN_.IS_AGGREGATE is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_AGGREGATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates that the virtual disk is a combination of other virtual disks.  A virtual disk can be both a segment and an aggregate if the virtual disk is in a middle layer with an aggregate virtual disk above and the segment virtual disks below.  In such a case, both segment and aggregate are true.

Possible values:
''T'' - the LUN is an aggregate
''F'' - the LUN is not an aggregate';

comment on column SAHARA_APP.LUN_.IS_CONTROL_DEVICE is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_CONTROL_DEVICE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is used only for sending control commands to the array.  This tells applications to ignore the device.

Possible values:
''T'' - the LUN is a control device
''F'' - the LUN is not a control device';

comment on column SAHARA_APP.LUN_.IS_FAULT_TOLERANT is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_FAULT_TOLERANT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates whether the virtual disk layout is fault tolerant.

Possible values:
''T'' - the LUN is fault tolerant
''F'' - the LUN is not fault tolerant';

comment on column SAHARA_APP.LUN_.IS_ADMINISTRATIVE is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_ADMINISTRATIVE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates a LUN set aside for a BCV, shadow image, control device, and other administrative task.

Possible values:
''T'' - the LUN is administrative
''F'' - the LUN is not administrative';

comment on column SAHARA_APP.LUN_.IS_USED_IN_SNAPSHOT is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_USED_IN_SNAPSHOT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates that the LUN is secondary and used in snapshot.

Possible values:
''T'' - the LUN is secondary and is used in a snapshot
''F'' - the LUN is not secondary and/or is not used in a snapshot';

comment on column SAHARA_APP.LUN_.IS_VIRTUALIZER_SOURCE is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_VIRTUALIZER_SOURCE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates that a LUN that is a virtualizer source.  A virtualizer source is allocated to a virtualizer array such as Tagma or IBM SAN Volume Controller (SVC).

Possible values:
''T'' - the LUN is a virtualizer source
''F'' - the LUN is not a virtualizer source';

comment on column SAHARA_APP.LUN_.IS_DIRECT_ATTACHED is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_DIRECT_ATTACHED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is direct attached.

Possible values:
''T'' - the LUN is direct attached
''F'' - the LUN is not direct attached';

comment on column SAHARA_APP.LUN_.IS_BASEDON_EXTERNAL is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_BASEDON_EXTERNAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is based on external.

Possible values:
''T'' - the LUN is based on external
''F'' - the LUN is not based on external';

comment on column SAHARA_APP.LUN_.ARRAY_PORTS is 
'This column is for staging data that will end up in table SAHARA.LUN, column ARRAY_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of array ports device is connected to.';

comment on column SAHARA_APP.LUN_.DISK_VENDOR is 
'This column is for staging data that will end up in table SAHARA.LUN, column DISK_VENDOR.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The name of the company that manufactured the physical disk from which this lun is made from.

This may be null.';

comment on column SAHARA_APP.LUN_.DISK_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.LUN, column DISK_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total capacity of the physical disk from which this lun is made from.';

comment on column SAHARA_APP.LUN_.VENDOR_LAYOUT is 
'This column is for staging data that will end up in table SAHARA.LUN, column VENDOR_LAYOUT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Vendor name for the storage layout used by this device.

This may be null.';

comment on column SAHARA_APP.LUN_.THIN_STATUS is 
'This column is for staging data that will end up in table SAHARA.LUN, column THIN_STATUS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Thin provisioning specific state of the lun.  
This attribute tells whether the object is thin or thin pool source.  For non-thin luns this attribute is not applicable.

This may be null.';

comment on column SAHARA_APP.LUN_.TIER is 
'This column is for staging data that will end up in table SAHARA.LUN, column TIER.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The storage tier.

This may be null.';

comment on column SAHARA_APP.LUN_.TIER_COST_PER_GB is 
'This column is for staging data that will end up in table SAHARA.LUN, column TIER_COST_PER_GB.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The cost of storage for the associated tier per GB.  This is copied from the tier configuration when the tier value is assigned.

This may be null.';

comment on column SAHARA_APP.LUN_.IS_ASSIGNED is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_ASSIGNED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates that the virtual disk is assigned.

Possible values:
''T'' - the LUN is assigned
''F'' - the LUN is not assigned';

comment on column SAHARA_APP.LUN_.IS_EXPORTED is 
'This column is for staging data that will end up in table SAHARA.LUN, column IS_EXPORTED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is exported.

Possible values:
''T'' - the device is exported
''F'' - the device is not exported';

comment on column SAHARA_APP.LUN_.APPLICATION_BE is 
'This column is for staging data that will end up in table SAHARA.LUN, column APPLICATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.LUN_.APPLICATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.LUN, column APPLICATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.LUN_.ORGANIZATION_BE is 
'This column is for staging data that will end up in table SAHARA.LUN, column ORGANIZATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.LUN_.ORGANIZATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.LUN, column ORGANIZATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.LUN_.TYPE is 
'This column is for staging data that will end up in table SAHARA.LUN, column TYPE

This attribute differentiate between VMDK and Disk.';

comment on column SAHARA_APP.LUN_.SUB_TYPE is 
'This column is for staging data that will end up in table SAHARA.LUN, column SUBTYPE.

For VMDK, this differentiate between RDM, Sparse and Flat VMDKs.';

comment on column SAHARA_APP.LUN_.STORAGE_TECHNOLOGY is 
'This column is for staging data that will end up in table SAHARA.LUN, column STORAGE_TECHNOLOGY.

This attribute represents the storage technology used. Possible values are SAN, DAS and NAS.';

comment on column SAHARA_APP.LUN_.STORAGE_POOL_NAME is 
'This column is for staging data that will end up in table SAHARA.LUN, column STORAGE_POOL_NAME.

This attribute represents name of the data store associated to a VMware VMDK.';

comment on column SAHARA_APP.LUN_.CONNECTION_PROTOCOL is 
'This column is for staging data that will end up in table SAHARA.LUN, column CONNECTION_PROTOCOL.

This attribute represents networking protocol used for accessing storage.';

comment on column SAHARA_APP.LUN_.NAS_FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_.ARRAY_NAMES is 
'Names of array. Multiple in case of VMDK carved out from SP deriving storage from multiple arrays.';

alter table SAHARA_APP.LUN_
   alter IS_PRIMARY add constraint C_LUN_ISPRIMARY check (IS_PRIMARY in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_READ_CACHE_ENABLED add constraint C_LUN_ISREADCACHEENABLED check (IS_READ_CACHE_ENABLED in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_SECONDARY add constraint C_LUN_ISSECONDARY check (IS_SECONDARY in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_SEGMENT add constraint C_LUN_ISSEGMENT check (IS_SEGMENT in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_WRITE_CACHE_ENABLED add constraint C_LUN_ISWRITECACHEENABLED check (IS_WRITE_CACHE_ENABLED in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_AGGREGATE add constraint C_LUN_ISAGGREGATE check (IS_AGGREGATE in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_CONTROL_DEVICE add constraint C_LUN_ISCONTROLDEVICE check (IS_CONTROL_DEVICE in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_FAULT_TOLERANT add constraint C_LUN_ISFAULTTOLERANT check (IS_FAULT_TOLERANT in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_ADMINISTRATIVE add constraint C_LUN_ISADMINISTRATIVE check (IS_ADMINISTRATIVE in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_USED_IN_SNAPSHOT add constraint C_LUN_ISUSEDINSNAPSHOT check (IS_USED_IN_SNAPSHOT in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_VIRTUALIZER_SOURCE add constraint C_LUN_ISVIRTUALIZERSOURCE check (IS_VIRTUALIZER_SOURCE in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_DIRECT_ATTACHED add constraint C_LUN_ISDIRECTATTACHED check (IS_DIRECT_ATTACHED in ('T','F'));

alter table SAHARA_APP.LUN_
   alter IS_BASEDON_EXTERNAL add constraint C_LUN_ISBASEDONEXTERNAL check (IS_BASEDON_EXTERNAL in ('T','F'));

alter table SAHARA_APP.LUN_
   alter ARRAY_PORTS add constraint C_LUN_ARRAYPORTS check (ARRAY_PORTS is null or (ARRAY_PORTS >= 0));

alter table SAHARA_APP.LUN_
   alter DISK_CAPACITY add constraint C_LUN_DISKCAPACITY check (DISK_CAPACITY >= 0);

alter table SAHARA_APP.LUN_
   alter IS_ASSIGNED add constraint C_LUN_ISASSIGNED check (IS_ASSIGNED is null or (IS_ASSIGNED in ('T','F')));

alter table SAHARA_APP.LUN_
   alter IS_EXPORTED add constraint C_VOLUME_ISEXPORTED check (IS_EXPORTED is null or (IS_EXPORTED in ('T','F')));

alter table SAHARA_APP.LUN_
   add constraint PK_LUN_ primary key (LUN_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.LUN_ to SAHARA;

/*==============================================================*/
/* Table: LUN_BVIEW_NODE                                        */
/*==============================================================*/
create table SAHARA.LUN_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   LUN_KEY              numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.LUN_BVIEW_NODE is 
'This is a join table between LUN objects and business view nodes.';

comment on column SAHARA.LUN_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.LUN_BVIEW_NODE.LUN_KEY is 
'The unique identifier of the LUN object with which the business view node is associated.';

comment on column SAHARA.LUN_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.LUN_BVIEW_NODE
   add constraint PK_LUN_BVIEW_NODE primary key (BVIEW_NODE_KEY, LUN_KEY);

/*==============================================================*/
/* Index: I_LUNBVIEWNODE_BVIEWLUN                               */
/*==============================================================*/
create unique index I_LUNBVIEWNODE_BVIEWLUN on SAHARA.LUN_BVIEW_NODE (
BVIEW_KEY ASC,
LUN_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.LUN_BVIEW_NODE.I_LUNBVIEWNODE_BVIEWLUN is 
'This index exists to accelerate queries for which switches are associated with a particular business view.';

grant SELECT on SAHARA.LUN_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_CAPACITY                                          */
/*==============================================================*/
create table SAHARA.LUN_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   LUN_KEY              numeric(10)                    not null,
   LUN_CAPACITY         numeric(22,9)                  not null default 0,
   ALLOCATED_TO_HOST_DIRECTLY numeric(22,9)                  not null default 0,
   UNALLOCATED          numeric(22,9)                  not null default 0,
   TOTAL_COST           numeric(22,2)                  null,
   ALLOCATED_HOSTS      numeric(4)                     not null default 0,
   CLAIMED_HOSTS        numeric(4)                     not null default 0,
   PHYSICAL_CAPACITY    numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   C_ADJUSTED_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   C_PHYSICAL_CAPACITY  numeric(22,9)                  not null default 0,
   C_LUN_CAPACITY       numeric(22,9)                  not null default 0,
   C_ALLOCATED_TO_HOST_DIRECTLY numeric(22,9)                  not null default 0
);

comment on table SAHARA.LUN_CAPACITY is 
'Contains capacity values for host';

comment on column SAHARA.LUN_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.LUN_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension that identifies the date on which the array storage data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.LUN_CAPACITY.LUN_KEY is 
'The surrogate key from the LUN dimension that identifies the LUN for which the storage data was collected.  This key may be used in joins to the LUN table.';

comment on column SAHARA.LUN_CAPACITY.ALLOCATED_TO_HOST_DIRECTLY is 
'The total amount of LUN storage, in gigabytes, allocated directly to a host.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of LUNs allocated directly to hosts), or zero (in the case of other LUNs such as administrative or unallocated).';

comment on column SAHARA.LUN_CAPACITY.UNALLOCATED is 
'The total amount of LUN storage, in gigabytes, that is not currently allocated to a host and is not being reserved for other purposes.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of unallocated LUNs), or zero (in the case of other LUNs such as allocated or administrative).';

comment on column SAHARA.LUN_CAPACITY.TOTAL_COST is 
'The cost associated with the usable capacity of allocated LUNs, based on storage tiers.';

comment on column SAHARA.LUN_CAPACITY.ALLOCATED_HOSTS is 
'The number of distinct hosts to which the LUN is allocated.';

comment on column SAHARA.LUN_CAPACITY.CLAIMED_HOSTS is 
'The number of distinct hosts which has an O/S Handle for this LUN.';

comment on column SAHARA.LUN_CAPACITY.PHYSICAL_CAPACITY is 
'The total physical capacity used by the lun.';

comment on column SAHARA.LUN_CAPACITY.ADJUSTED_ALLOCATED_CAPACITY is 
'The amount of storage, in gigabytes, comprised of distributed capacity of LUNs that have been masked to multiple hosts within clustering environment and exclusive capacity that is apportioned into LUNs designated for use by specific hosts.';

comment on column SAHARA.LUN_CAPACITY.C_ADJUSTED_ALLOCATED_CAPACITY is 
'The amount of storage, in gigabytes, comprised of distributed capacity of LUNs that have been masked to multiple hosts within clustering environment and exclusive capacity that is apportioned into LUNs designated for use by specific hosts.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''s consumed capacity.';

comment on column SAHARA.LUN_CAPACITY.C_PHYSICAL_CAPACITY is 
'The total physical capacity used by the lun.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''''s consumed capacity.';

comment on column SAHARA.LUN_CAPACITY.C_LUN_CAPACITY is 
'The total amount of LUN storage, in gigabytes.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is LUN''s consumed capacity.';

comment on column SAHARA.LUN_CAPACITY.C_ALLOCATED_TO_HOST_DIRECTLY is 
'The total amount of LUN storage, in gigabytes, allocated directly to a host.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of LUNs allocated directly to hosts), or zero (in the case of other LUNs such as administrative or unallocated).

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''s consumed capacity.';

alter table SAHARA.LUN_CAPACITY
   alter LUN_CAPACITY add constraint C_LUNCAPACITY_LUNCAPACITY check (LUN_CAPACITY >= 0);

alter table SAHARA.LUN_CAPACITY
   alter ALLOCATED_TO_HOST_DIRECTLY add constraint C_LUNCAPACITY_LLCTDTHSTDRCTL check (ALLOCATED_TO_HOST_DIRECTLY >= 0);

alter table SAHARA.LUN_CAPACITY
   alter UNALLOCATED add constraint C_LUNCAPACITY_UNALLOCATED check (UNALLOCATED >= 0);

alter table SAHARA.LUN_CAPACITY
   alter PHYSICAL_CAPACITY add constraint C_LUNCAPACITY_PHYSICALCAPACITY check (PHYSICAL_CAPACITY >= 0);

alter table SAHARA.LUN_CAPACITY
   alter ADJUSTED_ALLOCATED_CAPACITY add constraint C_LUNCAPACITY_DJSTDLLCTDCPCT check (ADJUSTED_ALLOCATED_CAPACITY >= 0);

alter table SAHARA.LUN_CAPACITY
   alter C_ADJUSTED_ALLOCATED_CAPACITY add constraint C_LUNCAPACITY_CDJSTDLLCTDCPCT check (C_ADJUSTED_ALLOCATED_CAPACITY >= 0);

alter table SAHARA.LUN_CAPACITY
   alter C_PHYSICAL_CAPACITY add constraint C_LUNCAPACITY_CPHSCLCPCT check (C_PHYSICAL_CAPACITY >= 0);

alter table SAHARA.LUN_CAPACITY
   alter C_LUN_CAPACITY add constraint C_LUNCAPACITY_CLUNCAPACITY check (C_LUN_CAPACITY >= 0);

alter table SAHARA.LUN_CAPACITY
   alter C_ALLOCATED_TO_HOST_DIRECTLY add constraint C_LUNCAPACITY_CLLCTDTHSTDRCTL check (C_ALLOCATED_TO_HOST_DIRECTLY >= 0);

/*==============================================================*/
/* Index: I_LUNCAPCITY_ROLLUPBATCHKEY                           */
/*==============================================================*/
create index I_LUNCAPCITY_ROLLUPBATCHKEY on SAHARA.LUN_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_LUNCAPCITY_DATESKEY                                 */
/*==============================================================*/
create index I_LUNCAPCITY_DATESKEY on SAHARA.LUN_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_LUNCAPCITY_LUNKEY                                   */
/*==============================================================*/
create index I_LUNCAPCITY_LUNKEY on SAHARA.LUN_CAPACITY (
LUN_KEY ASC
);

grant SELECT on SAHARA.LUN_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_CAPACITY_                                         */
/*==============================================================*/
create table SAHARA_APP.LUN_CAPACITY_ 
(
   LUN_OID              varchar(512)                   not null,
   LUN_CAPACITY         numeric(22,9)                  not null default 0,
   ALLOCATED_TO_HOST_DIRECTLY numeric(22,9)                  not null default 0,
   UNALLOCATED          numeric(22,9)                  not null default 0,
   ALLOCATED_HOSTS      numeric(4)                     not null default 0,
   CLAIMED_HOSTS        numeric(4)                     not null default 0,
   PHYSICAL_CAPACITY    numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   C_ALLOCATED_TO_HOST_DIRECTLY numeric(22,9)                  not null default 0,
   C_LUN_CAPACITY       numeric(22,9)                  not null default 0,
   C_ADJUSTED_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   C_PHYSICAL_CAPACITY  numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.LUN_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.LUN_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  Contains capacity values for host';

comment on column SAHARA_APP.LUN_CAPACITY_.LUN_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_CAPACITY_.LUN_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column LUN_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

';

comment on column SAHARA_APP.LUN_CAPACITY_.ALLOCATED_TO_HOST_DIRECTLY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column ALLOCATED_TO_HOST_DIRECTLY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total amount of LUN storage, in gigabytes, allocated directly to a host.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of LUNs allocated directly to hosts), or zero (in the case of other LUNs such as administrative or unallocated).';

comment on column SAHARA_APP.LUN_CAPACITY_.UNALLOCATED is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column UNALLOCATED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total amount of LUN storage, in gigabytes, that is not currently allocated to a host and is not being reserved for other purposes.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of unallocated LUNs), or zero (in the case of other LUNs such as allocated or administrative).';

comment on column SAHARA_APP.LUN_CAPACITY_.ALLOCATED_HOSTS is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column ALLOCATED_HOSTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of distinct hosts to which the LUN is allocated.';

comment on column SAHARA_APP.LUN_CAPACITY_.CLAIMED_HOSTS is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column CLAIMED_HOSTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of distinct hosts which has an O/S Handle for this LUN.';

comment on column SAHARA_APP.LUN_CAPACITY_.PHYSICAL_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column PHYSICAL_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total physical capacity used by the lun.';

comment on column SAHARA_APP.LUN_CAPACITY_.ADJUSTED_ALLOCATED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column ADJUSTED_ALLOCATED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of distributed capacity of LUNs that have been masked to multiple hosts within clustering environment and exclusive capacity that is apportioned into LUNs designated for use by specific hosts.';

comment on column SAHARA_APP.LUN_CAPACITY_.C_ALLOCATED_TO_HOST_DIRECTLY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column C_ALLOCATED_TO_HOST_DIRECTLY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total amount of LUN storage, in gigabytes, allocated directly to a host.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of LUNs allocated directly to hosts), or zero (in the case of other LUNs such as administrative or unallocated).

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''s consumed capacity.';

comment on column SAHARA_APP.LUN_CAPACITY_.C_LUN_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column C_LUN_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total amount of LUN storage, in gigabytes.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is LUN''s consumed capacity.';

comment on column SAHARA_APP.LUN_CAPACITY_.C_ADJUSTED_ALLOCATED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column C_ADJUSTED_ALLOCATED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The amount of storage, in gigabytes, comprised of distributed capacity of LUNs that have been masked to multiple hosts within clustering environment and exclusive capacity that is apportioned into LUNs designated for use by specific hosts.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''s consumed capacity.';

comment on column SAHARA_APP.LUN_CAPACITY_.C_PHYSICAL_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.LUN_CAPACITY, column C_PHYSICAL_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total physical capacity used by the lun.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''''s consumed capacity.';

alter table SAHARA_APP.LUN_CAPACITY_
   alter LUN_CAPACITY add constraint C_LUNCAPACITY_LUNCAPACITY check (LUN_CAPACITY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter ALLOCATED_TO_HOST_DIRECTLY add constraint C_LUNCAPACITY_LLCTDTHSTDRCTL check (ALLOCATED_TO_HOST_DIRECTLY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter UNALLOCATED add constraint C_LUNCAPACITY_UNALLOCATED check (UNALLOCATED >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter PHYSICAL_CAPACITY add constraint C_LUNCAPACITY_PHYSICALCAPACITY check (PHYSICAL_CAPACITY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter ADJUSTED_ALLOCATED_CAPACITY add constraint C_LUNCAPACITY_DJSTDLLCTDCPCT check (ADJUSTED_ALLOCATED_CAPACITY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter C_ALLOCATED_TO_HOST_DIRECTLY add constraint C_LUNCAPACITY_CLLCTDTHSTDRCTL check (C_ALLOCATED_TO_HOST_DIRECTLY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter C_LUN_CAPACITY add constraint C_LUNCAPACITY_CLUNCAPACITY check (C_LUN_CAPACITY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter C_ADJUSTED_ALLOCATED_CAPACITY add constraint C_LUNCAPACITY_CDJSTDLLCTDCPCT check (C_ADJUSTED_ALLOCATED_CAPACITY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   alter C_PHYSICAL_CAPACITY add constraint C_LUNCAPACITY_CPHSCLCPCT check (C_PHYSICAL_CAPACITY >= 0);

alter table SAHARA_APP.LUN_CAPACITY_
   add constraint PK_LUN_CAPACITY_ primary key (LUN_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.LUN_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: LUN_CLAIMED_STATES                                    */
/*==============================================================*/
create table SAHARA.LUN_CLAIMED_STATES 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   LUN_KEY              numeric(10)                    not null,
   CLAIMED_STATUS       numeric(1)                     null
);

comment on table SAHARA.LUN_CLAIMED_STATES is 
'A join table for forming the many-to-many relationships between duplicate LUNs and its claimed states across multi-site rollup.';

comment on column SAHARA.LUN_CLAIMED_STATES.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.LUN_CLAIMED_STATES.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.LUN_CLAIMED_STATES.LUN_KEY is 
'The surrogate key of the LUN for which claimed status is being stored.  This key may be used in joins to the LUN table.';

comment on column SAHARA.LUN_CLAIMED_STATES.CLAIMED_STATUS is 
'The Claimed Status of the LUN.

Possible values:
- 0
- 1
- 2';

/*==============================================================*/
/* Index: I_LCC_ROLLUPBATCHKEY                                  */
/*==============================================================*/
create index I_LCC_ROLLUPBATCHKEY on SAHARA.LUN_CLAIMED_STATES (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_LCC_DATESKEY                                        */
/*==============================================================*/
create index I_LCC_DATESKEY on SAHARA.LUN_CLAIMED_STATES (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_LCC_ARRAYKEY                                        */
/*==============================================================*/
create index I_LCC_ARRAYKEY on SAHARA.LUN_CLAIMED_STATES (
LUN_KEY ASC
);

grant SELECT on SAHARA.LUN_CLAIMED_STATES to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_DISKGROUP                                         */
/*==============================================================*/
create table SAHARA.LUN_DISKGROUP 
(
   DISKGROUP_KEY        numeric(10)                    null,
   LUN_KEY              numeric(10)                    not null,
   DATES_KEY            numeric(10)                    null,
   ARRAY_KEY            numeric(10)                    null,
   ROLLUP_BATCH_KEY     numeric(10)                    null
);

comment on table SAHARA.LUN_DISKGROUP is 
'A join table for forming the many-to-many relationship between LUNs and diskgroups.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA.LUN_DISKGROUP.DATES_KEY is 
'A unique database generated identifier for the date record used as a surrogate key.';

comment on column SAHARA.LUN_DISKGROUP.ARRAY_KEY is 
'A unique database generated identifier for the array record used as a surrogate key.';

comment on column SAHARA.LUN_DISKGROUP.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

alter table SAHARA.LUN_DISKGROUP
   alter DATES_KEY add constraint CKC_DATES_KEY_LUN_DISK check (DATES_KEY is null or (DATES_KEY >= 1));

/*==============================================================*/
/* Index: I_LUNDISKGROUP_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_LUNDISKGROUP_ROLLUPBATCHKEY on SAHARA.LUN_DISKGROUP (
ROLLUP_BATCH_KEY ASC
);

comment on index SAHARA.LUN_DISKGROUP.I_LUNDISKGROUP_ROLLUPBATCHKEY is 
'An index on Roll-up Batch Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNDISKGROUP_DATESKEY                               */
/*==============================================================*/
create index I_LUNDISKGROUP_DATESKEY on SAHARA.LUN_DISKGROUP (
DATES_KEY ASC
);

comment on index SAHARA.LUN_DISKGROUP.I_LUNDISKGROUP_DATESKEY is 
'An index on Dates Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNDISKGROUP_LUNKEY                                 */
/*==============================================================*/
create index I_LUNDISKGROUP_LUNKEY on SAHARA.LUN_DISKGROUP (
LUN_KEY ASC
);

comment on index SAHARA.LUN_DISKGROUP.I_LUNDISKGROUP_LUNKEY is 
'An index on LUN Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNDISKGROUP_DISKGROUPKEY                           */
/*==============================================================*/
create index I_LUNDISKGROUP_DISKGROUPKEY on SAHARA.LUN_DISKGROUP (
DISKGROUP_KEY ASC
);

comment on index SAHARA.LUN_DISKGROUP.I_LUNDISKGROUP_DISKGROUPKEY is 
'An index on Filesystem Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNDISKGROUP_ARRAYKEY                               */
/*==============================================================*/
create index I_LUNDISKGROUP_ARRAYKEY on SAHARA.LUN_DISKGROUP (
ARRAY_KEY ASC
);

comment on index SAHARA.LUN_DISKGROUP.I_LUNDISKGROUP_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

grant SELECT on SAHARA.LUN_DISKGROUP to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_DISKGROUP_                                        */
/*==============================================================*/
create table SAHARA_APP.LUN_DISKGROUP_ 
(
   DISKGROUP_OID        varchar(512)                   not null,
   LUN_OID              varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   null
);

comment on table SAHARA_APP.LUN_DISKGROUP_ is 
'This is a staging table for loading data into warehouse table SAHARA.LUN_DISKGROUP.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table: A join table for forming the many-to-many relationship between LUNs and diskgroups.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA_APP.LUN_DISKGROUP_.DISKGROUP_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_DISKGROUP_.LUN_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_DISKGROUP_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

alter table SAHARA_APP.LUN_DISKGROUP_
   add constraint PK_LUN_DISKGROUP_ primary key (DISKGROUP_OID, LUN_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.LUN_DISKGROUP_ to SAHARA;

/*==============================================================*/
/* Table: LUN_FILESYSTEM                                        */
/*==============================================================*/
create table SAHARA.LUN_FILESYSTEM 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   LUN_KEY              numeric(10)                    not null,
   FILESYSTEM_KEY       numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    null
);

comment on table SAHARA.LUN_FILESYSTEM is 
'A join table for forming the many-to-many relationship between LUNs and filesystems.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA.LUN_FILESYSTEM.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.LUN_FILESYSTEM.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.LUN_FILESYSTEM.LUN_KEY is 
'The surrogate key of the LUN which is related to a host.  This key may be used in joins to the LUN table.';

comment on column SAHARA.LUN_FILESYSTEM.FILESYSTEM_KEY is 
'The surrogate key of the host which is related to a LUN.  This key may be used in joins to the FILESYSTEM table.';

comment on column SAHARA.LUN_FILESYSTEM.ARRAY_KEY is 
'The surrogate key of the host which is related to a LUN.  This key may be used in joins to the ARRAY table.';

/*==============================================================*/
/* Index: I_LUNFILESYSTEM_ROLLUPBATCHKEY                        */
/*==============================================================*/
create index I_LUNFILESYSTEM_ROLLUPBATCHKEY on SAHARA.LUN_FILESYSTEM (
ROLLUP_BATCH_KEY ASC
);

comment on index SAHARA.LUN_FILESYSTEM.I_LUNFILESYSTEM_ROLLUPBATCHKEY is 
'An index on Roll-up Batch Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNFILESYSTEM_DATESKEY                              */
/*==============================================================*/
create index I_LUNFILESYSTEM_DATESKEY on SAHARA.LUN_FILESYSTEM (
DATES_KEY ASC
);

comment on index SAHARA.LUN_FILESYSTEM.I_LUNFILESYSTEM_DATESKEY is 
'An index on Dates Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNFILESYSTEM_LUNKEY                                */
/*==============================================================*/
create index I_LUNFILESYSTEM_LUNKEY on SAHARA.LUN_FILESYSTEM (
LUN_KEY ASC
);

comment on index SAHARA.LUN_FILESYSTEM.I_LUNFILESYSTEM_LUNKEY is 
'An index on LUN Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNFILESYSTEM_FILESYSTEMKEY                         */
/*==============================================================*/
create index I_LUNFILESYSTEM_FILESYSTEMKEY on SAHARA.LUN_FILESYSTEM (
FILESYSTEM_KEY ASC
);

comment on index SAHARA.LUN_FILESYSTEM.I_LUNFILESYSTEM_FILESYSTEMKEY is 
'An index on Filesystem Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNFILESYSTEM_ARRAYKEY                              */
/*==============================================================*/
create index I_LUNFILESYSTEM_ARRAYKEY on SAHARA.LUN_FILESYSTEM (
ARRAY_KEY ASC
);

comment on index SAHARA.LUN_FILESYSTEM.I_LUNFILESYSTEM_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

grant SELECT on SAHARA.LUN_FILESYSTEM to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_FILESYSTEM_                                       */
/*==============================================================*/
create table SAHARA_APP.LUN_FILESYSTEM_ 
(
   FILESYSTEM_OID       varchar(512)                   not null,
   LUN_OID              varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   null
);

comment on table SAHARA_APP.LUN_FILESYSTEM_ is 
'This is a staging table for loading data into warehouse table SAHARA.LUN_FILESYSTEM.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  A join table for forming the many-to-many relationship between LUNs and filesystems.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA_APP.LUN_FILESYSTEM_.FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_FILESYSTEM_.LUN_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_FILESYSTEM_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

alter table SAHARA_APP.LUN_FILESYSTEM_
   add constraint PK_LUN_FILESYSTEM_ primary key (FILESYSTEM_OID, LUN_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.LUN_FILESYSTEM_ to SAHARA;

/*==============================================================*/
/* Table: LUN_HOST                                              */
/*==============================================================*/
create table SAHARA.LUN_HOST 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   LUN_KEY              numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   LUN_CAPACITY         numeric(22,9)                  not null default 0,
   ALLOCATED_TO_HOST_DIRECTLY numeric(22,9)                  not null default 0,
   TOTAL_COST           numeric(22,2)                  null,
   ALLOCATED_HOSTS      numeric(4)                     not null default 0,
   CLAIMED_HOSTS        numeric(4)                     not null default 0,
   PHYSICAL_CAPACITY    numeric(22,9)                  not null default 0,
   ADJUSTED_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   C_ADJUSTED_ALLOCATED_CAPACITY numeric(22,9)                  not null default 0,
   C_LUN_CAPACITY       numeric(22,9)                  not null default 0,
   C_PHYSICAL_CAPACITY  numeric(22,9)                  not null default 0,
   C_ALLOCATED_TO_HOST_DIRECTLY numeric(22,9)                  not null default 0,
   C_TOTAL_COST         numeric(22,2)                  null
);

comment on table SAHARA.LUN_HOST is 
'A join table for forming the many-to-many relationship between LUNs and hosts.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA.LUN_HOST.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.LUN_HOST.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.LUN_HOST.LUN_KEY is 
'The surrogate key of the LUN which is related to a host.  This key may be used in joins to the LUN table.';

comment on column SAHARA.LUN_HOST.HOST_KEY is 
'The surrogate key of the host which is related to a LUN.  This key may be used in joins to the HOST table.';

comment on column SAHARA.LUN_HOST.ALLOCATED_TO_HOST_DIRECTLY is 
'The total amount of LUN storage, in gigabytes, allocated directly to a host.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of LUNs allocated directly to hosts), or zero (in the case of other LUNs such as administrative or unallocated).

Note: In case of thin luns, this capacity is calaculated based on LUN''s subscribed capacity.';

comment on column SAHARA.LUN_HOST.TOTAL_COST is 
'The cost associated with the usable capacity of allocated LUNs, based on storage tiers.

Note: In case of thin luns, this cost is calaculated based on LUN''s subscribed capacity.';

comment on column SAHARA.LUN_HOST.ALLOCATED_HOSTS is 
'The number of distinct hosts to which the LUN is allocated.';

comment on column SAHARA.LUN_HOST.CLAIMED_HOSTS is 
'The number of distinct hosts which has an O/S Handle for this LUN.';

comment on column SAHARA.LUN_HOST.PHYSICAL_CAPACITY is 
'It stores the physical capacity of the allocated lun.

Note: In case of thin luns, this capacity is calaculated based on LUN''s subscribed capacity.';

comment on column SAHARA.LUN_HOST.ADJUSTED_ALLOCATED_CAPACITY is 
'The amount of storage, in gigabytes, comprised of distributed capacity of LUNs that have been masked to multiple hosts within clustering environment and exclusive capacity that is apportioned into LUNs designated for use by specific hosts.

Note: In case of thin luns, this capacity is calaculated based on LUN''s subscribed capacity.';

comment on column SAHARA.LUN_HOST.C_ADJUSTED_ALLOCATED_CAPACITY is 
'The amount of storage, in gigabytes, comprised of distributed capacity of LUNs that have been masked to multiple hosts within clustering environment and exclusive capacity that is apportioned into LUNs designated for use by specific hosts.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''s consumed capacity.';

comment on column SAHARA.LUN_HOST.C_LUN_CAPACITY is 
'The total amount of LUN storage, in gigabytes.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is LUN''s consumed capacity.';

comment on column SAHARA.LUN_HOST.C_PHYSICAL_CAPACITY is 
'It stores the physical capacity of the allocated lun.

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''''s consumed capacity.';

comment on column SAHARA.LUN_HOST.C_ALLOCATED_TO_HOST_DIRECTLY is 
'The total amount of LUN storage, in gigabytes, allocated directly to a host.  For each individual LUN, this value will either be the same as LUN Capacity (in the case of LUNs allocated directly to hosts), or zero (in the case of other LUNs such as administrative or unallocated).

Note: This capacity is not applicable for non-thin luns.  In case of thin luns, this capacity is based on LUN''s consumed capacity.';

comment on column SAHARA.LUN_HOST.C_TOTAL_COST is 
'The cost associated with the usable capacity of allocated LUNs, based on storage tiers.

Note: This cost is not applicable for non-thin luns.  In case of thin luns, this cost is based on LUN''s consumed capacity.  capacity.';

alter table SAHARA.LUN_HOST
   alter LUN_CAPACITY add constraint C_LUNHOST_LUNCAPACITY check (LUN_CAPACITY >= 0);

alter table SAHARA.LUN_HOST
   alter ALLOCATED_TO_HOST_DIRECTLY add constraint C_LUNHOST_LLCTDTHSTDRCTL check (ALLOCATED_TO_HOST_DIRECTLY >= 0);

alter table SAHARA.LUN_HOST
   alter PHYSICAL_CAPACITY add constraint C_LUNHOST_PHYSICALCAPACITY check (PHYSICAL_CAPACITY >= 0);

alter table SAHARA.LUN_HOST
   alter ADJUSTED_ALLOCATED_CAPACITY add constraint C_LUNHOST_DJSTDLLCTDCPCT check (ADJUSTED_ALLOCATED_CAPACITY >= 0);

alter table SAHARA.LUN_HOST
   alter C_ADJUSTED_ALLOCATED_CAPACITY add constraint C_LUNHOST_CDJSTDLLCTDCPCT check (C_ADJUSTED_ALLOCATED_CAPACITY >= 0);

alter table SAHARA.LUN_HOST
   alter C_LUN_CAPACITY add constraint C_LUNHOST_CLUNCAPACITY check (C_LUN_CAPACITY >= 0);

alter table SAHARA.LUN_HOST
   alter C_PHYSICAL_CAPACITY add constraint C_LUNHOST_CPHYSICALCAPACITY check (C_PHYSICAL_CAPACITY >= 0);

alter table SAHARA.LUN_HOST
   alter C_ALLOCATED_TO_HOST_DIRECTLY add constraint C_LUNHOST_CLLCTDTHSTDRCTL check (C_ALLOCATED_TO_HOST_DIRECTLY >= 0);

/*==============================================================*/
/* Index: I_LUNHOST_ROLLUPBATCHKEY                              */
/*==============================================================*/
create index I_LUNHOST_ROLLUPBATCHKEY on SAHARA.LUN_HOST (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_LUNHOST_DATESKEY                                    */
/*==============================================================*/
create index I_LUNHOST_DATESKEY on SAHARA.LUN_HOST (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_LUNHOST_LUNKEY                                      */
/*==============================================================*/
create index I_LUNHOST_LUNKEY on SAHARA.LUN_HOST (
LUN_KEY ASC
);

/*==============================================================*/
/* Index: I_LUNHOST_HOSTKEY                                     */
/*==============================================================*/
create index I_LUNHOST_HOSTKEY on SAHARA.LUN_HOST (
HOST_KEY ASC
);

grant SELECT on SAHARA.LUN_HOST to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_HOST_                                             */
/*==============================================================*/
create table SAHARA_APP.LUN_HOST_ 
(
   LUN_OID              varchar(512)                   not null,
   HOST_OID             varchar(512)                   not null
);

comment on table SAHARA_APP.LUN_HOST_ is 
'This is a staging table for loading data into warehouse table SAHARA.LUN_HOST.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  A join table for forming the many-to-many relationship between LUNs and hosts.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA_APP.LUN_HOST_.LUN_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_HOST_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

alter table SAHARA_APP.LUN_HOST_
   add constraint PK_LUN_HOST_ primary key (LUN_OID, HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.LUN_HOST_ to SAHARA;

/*==============================================================*/
/* Table: LUN_VOLUME                                            */
/*==============================================================*/
create table SAHARA.LUN_VOLUME 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   LUN_KEY              numeric(10)                    not null,
   VOLUME_KEY           numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    null
);

comment on table SAHARA.LUN_VOLUME is 
'A join table for forming the many-to-many relationship between LUNs and volumes.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA.LUN_VOLUME.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.LUN_VOLUME.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.LUN_VOLUME.LUN_KEY is 
'The surrogate key of the LUN which is related to a host.  This key may be used in joins to the LUN table.';

comment on column SAHARA.LUN_VOLUME.VOLUME_KEY is 
'The surrogate key of the host which is related to a LUN.  This key may be used in joins to the VOLUME table.';

comment on column SAHARA.LUN_VOLUME.ARRAY_KEY is 
'The surrogate key of the host which is related to a LUN.  This key may be used in joins to the ARRAY table.';

/*==============================================================*/
/* Index: I_LUNVOLUME_ROLLUPBATCHKEY                            */
/*==============================================================*/
create index I_LUNVOLUME_ROLLUPBATCHKEY on SAHARA.LUN_VOLUME (
ROLLUP_BATCH_KEY ASC
);

comment on index SAHARA.LUN_VOLUME.I_LUNVOLUME_ROLLUPBATCHKEY is 
'An index on Roll-up Batch Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNVOLUME_DATESKEY                                  */
/*==============================================================*/
create index I_LUNVOLUME_DATESKEY on SAHARA.LUN_VOLUME (
DATES_KEY ASC
);

comment on index SAHARA.LUN_VOLUME.I_LUNVOLUME_DATESKEY is 
'An index on Dates Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNVOLUME_LUNKEY                                    */
/*==============================================================*/
create index I_LUNVOLUME_LUNKEY on SAHARA.LUN_VOLUME (
LUN_KEY ASC
);

comment on index SAHARA.LUN_VOLUME.I_LUNVOLUME_LUNKEY is 
'An index on LUN Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNVOLUME_VOLUMEKEY                                 */
/*==============================================================*/
create index I_LUNVOLUME_VOLUMEKEY on SAHARA.LUN_VOLUME (
VOLUME_KEY ASC
);

comment on index SAHARA.LUN_VOLUME.I_LUNVOLUME_VOLUMEKEY is 
'An index on Volume Key for dimensional queries.';

/*==============================================================*/
/* Index: I_LUNVOLUME_ARRAYKEY                                  */
/*==============================================================*/
create index I_LUNVOLUME_ARRAYKEY on SAHARA.LUN_VOLUME (
ARRAY_KEY ASC
);

comment on index SAHARA.LUN_VOLUME.I_LUNVOLUME_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

grant SELECT on SAHARA.LUN_VOLUME to SAHARA_APP;

/*==============================================================*/
/* Table: LUN_VOLUME_                                           */
/*==============================================================*/
create table SAHARA_APP.LUN_VOLUME_ 
(
   VOLUME_OID           varchar(512)                   not null,
   LUN_OID              varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   null
);

comment on table SAHARA_APP.LUN_VOLUME_ is 
'This is a staging table for loading data into warehouse table SAHARA.LUN_VOLUME.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  A join table for forming the many-to-many relationship between LUNs and volumes.  We treat this as a fact to deal with relationship coming and going with each batch.  This effectively treats the relationship as type 2 updates always.';

comment on column SAHARA_APP.LUN_VOLUME_.VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_VOLUME_.LUN_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.LUN_VOLUME_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

alter table SAHARA_APP.LUN_VOLUME_
   add constraint PK_LUN_VOLUME_ primary key (VOLUME_OID, LUN_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.LUN_VOLUME_ to SAHARA;

/*==============================================================*/
/* Table: MESSAGE_LOG                                           */
/*==============================================================*/
create table SAHARA.MESSAGE_LOG 
(
   MESSAGE_LOG_KEY      numeric(10)                    not null default autoincrement,
   "TIME"               timestamp                      not null,
   SESSION_KEY          numeric(10)                    null,
   FUNCTION_NAME        varchar(256)                   null,
   SEVERITY             char(1)                        not null,
   "MESSAGE"            long varchar                   not null
);

comment on table SAHARA.MESSAGE_LOG is 
'A general purpose message log.  Messages are tagged with a timestamp, but queries should use the MESSAGE_LOG_KEY for ordering to guarantee retreival in the order in which messages were created because multiple messages may have the same timestamp.';

comment on column SAHARA.MESSAGE_LOG.MESSAGE_LOG_KEY is 
'A unique database generated key used to order messages.  Using the timestamp alone for ordering is not good enough because mutliple messages can be created with the same timestamp.';

comment on column SAHARA.MESSAGE_LOG."TIME" is 
'The time at which the message was logged.  All times in the sahara schema are represented in the UTC timezone.

A trigger populates this column if the value if missing from inserts.

Note: The data type for this column is intentionally NOT an SA_DATETIME because for this purpose, we do want to record fractional seconds.  SA_DATETIME has a second granularity.';

comment on column SAHARA.MESSAGE_LOG.SESSION_KEY is 
'The identifier of the session that logged the message.  Most messages will have a session, but some internal processing may not.';

comment on column SAHARA.MESSAGE_LOG.FUNCTION_NAME is 
'The (optional) name of the function logging the message.';

comment on column SAHARA.MESSAGE_LOG.SEVERITY is 
'An indication of the type of message being logged.  Valid values:

D - debug
I - informational
W - warning
E - error';

comment on column SAHARA.MESSAGE_LOG."MESSAGE" is 
'The text of the message.';

alter table SAHARA.MESSAGE_LOG
   alter SEVERITY add constraint C_MESSAGELOG_SEVERITY check (SEVERITY in ('D','I','W','E'));

alter table SAHARA.MESSAGE_LOG
   add constraint PK_MESSAGE_LOG primary key (MESSAGE_LOG_KEY);

/*==============================================================*/
/* Index: I_MESSAGELOG_TIME                                     */
/*==============================================================*/
create index I_MESSAGELOG_TIME on SAHARA.MESSAGE_LOG (
"TIME" ASC
);

/*==============================================================*/
/* Index: I_MESSAGELOG_SESSIONKEY                               */
/*==============================================================*/
create index I_MESSAGELOG_SESSIONKEY on SAHARA.MESSAGE_LOG (
SESSION_KEY ASC
);

/*==============================================================*/
/* Index: I_MESSAGELOG_FUNCTIONNAME                             */
/*==============================================================*/
create index I_MESSAGELOG_FUNCTIONNAME on SAHARA.MESSAGE_LOG (
FUNCTION_NAME ASC
);

/*==============================================================*/
/* Index: I_MESSAGELOG_SEVERITY                                 */
/*==============================================================*/
create index I_MESSAGELOG_SEVERITY on SAHARA.MESSAGE_LOG (
SEVERITY ASC
);

grant SELECT on SAHARA.MESSAGE_LOG to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_FILESYSTEM                                        */
/*==============================================================*/
create table SAHARA.NAS_FILESYSTEM 
(
   NAS_FILESYSTEM_KEY   numeric(10)                    not null default autoincrement,
   NAS_FILESYSTEM_OID   varchar(512)                   not null,
   ARRAY_KEY            numeric(10)                    not null,
   NETAPP_VOLUME_KEY    numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DISCOVERY_STATE      varchar(32)                    null,
   SECURITY_STYLE       varchar(32)                    null,
   FILESYSTEM_ID        numeric(9)                     not null,
   IS_QUOTA_ENABLED     char(1)                        not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   TIER                 varchar(64)                    null,
   TYPE                 varchar(32)                    not null default ' ',
   PATH                 varchar(256)                   not null default ' ',
   AUTO_EXTEND_ENABLED  char(1)                        null default 'F',
   IS_LOCK_ENABLED      char(1)                        null default 'F',
   CWORM_STATE          varchar(32)                    null,
   IS_VIRTUAL_PROVISIONING_ENABLE char(1)                        not null default 'F',
   IS_SOURCE            char(1)                        not null default 'F',
   IS_TARGET            char(1)                        not null default 'F',
   REPLICATION_TECHNOLOGY varchar(32)                    null,
   AUTO_EXTEND_MAX_SIZE numeric(22,9)                  not null default 0,
   HIGH_WATERMARK       numeric(22,9)                  not null default 0
);

comment on table SAHARA.NAS_FILESYSTEM is 
'This is the dimension table for NAS Filesystems.  It defines the attributes of NAS filesystems that may be used to qualify queries.';

comment on column SAHARA.NAS_FILESYSTEM.NAS_FILESYSTEM_KEY is 
'A unique database generated identifier for the NAS filesystem record used as a surrogate key.';

comment on column SAHARA.NAS_FILESYSTEM.NAS_FILESYSTEM_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.NAS_FILESYSTEM.ARRAY_KEY is 
'The unique identifier of the parent filer array object.';

comment on column SAHARA.NAS_FILESYSTEM.NETAPP_VOLUME_KEY is 
'A unique database generated identifier for the NAS volume record used as a surrogate key.';

comment on column SAHARA.NAS_FILESYSTEM.CC_STORAGE_URL is 
'The URL for the NAS filesystem object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.NAS_FILESYSTEM.DISPLAY_NAME is 
'The NAS filesystem display name.';

comment on column SAHARA.NAS_FILESYSTEM.DISCOVERY_STATE is 
'The discovery state of the object:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the NAS filesystem was initially discovered by two or more explorers and then one of them stopped discovering the NAS filesystem.

Note: A NAS filesystem that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.NAS_FILESYSTEM.SECURITY_STYLE is 
'The security style of the NAS filesystem.  Possible values include: UNIX, NTFS, and mixed.';

comment on column SAHARA.NAS_FILESYSTEM.FILESYSTEM_ID is 
'The identifier of the NAS filesystem(unique within the volume).';

comment on column SAHARA.NAS_FILESYSTEM.IS_QUOTA_ENABLED is 
'A boolean value that indicates if a quota is created for the NAS filesystem.';

comment on column SAHARA.NAS_FILESYSTEM.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.NAS_FILESYSTEM.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.NAS_FILESYSTEM
   alter FILESYSTEM_ID add constraint C_NASFILESYSTEM_FILESYSTEMID check (FILESYSTEM_ID >= 0);

alter table SAHARA.NAS_FILESYSTEM
   alter IS_QUOTA_ENABLED add constraint C_NASFILESYSTEM_ISQUOTAENABLED check (IS_QUOTA_ENABLED in ('T','F'));

alter table SAHARA.NAS_FILESYSTEM
   alter T2_IS_CURRENT add constraint C_NASFILESYSTEM_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.NAS_FILESYSTEM
   alter T2_IS_DELETED add constraint C_NASFILESYSTEM_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.NAS_FILESYSTEM
   alter AUTO_EXTEND_ENABLED add constraint CKC_AUTO_EXTEND_ENABL_NAS_FILE check (AUTO_EXTEND_ENABLED is null or (AUTO_EXTEND_ENABLED in ('T','F')));

alter table SAHARA.NAS_FILESYSTEM
   alter IS_LOCK_ENABLED add constraint CKC_IS_LOCK_ENABLED_NAS_FILE check (IS_LOCK_ENABLED is null or (IS_LOCK_ENABLED in ('T','F')));

alter table SAHARA.NAS_FILESYSTEM
   alter IS_VIRTUAL_PROVISIONING_ENABLE add constraint CKC_IS_VIRTUAL_PROVIS_NAS_FILE check (IS_VIRTUAL_PROVISIONING_ENABLE in ('T','F'));

alter table SAHARA.NAS_FILESYSTEM
   alter IS_SOURCE add constraint CKC_IS_SOURCE_NAS_FILE check (IS_SOURCE in ('T','F'));

alter table SAHARA.NAS_FILESYSTEM
   alter IS_TARGET add constraint CKC_IS_TARGET_NAS_FILE check (IS_TARGET in ('T','F'));

alter table SAHARA.NAS_FILESYSTEM
   alter AUTO_EXTEND_MAX_SIZE add constraint CKC_AUTO_EXTEND_MAX_S_NAS_FILE check (AUTO_EXTEND_MAX_SIZE >= 0);

alter table SAHARA.NAS_FILESYSTEM
   alter HIGH_WATERMARK add constraint CKC_HIGH_WATERMARK_NAS_FILE check (HIGH_WATERMARK >= 0);

alter table SAHARA.NAS_FILESYSTEM
   add constraint PK_NAS_FILESYSTEM primary key (NAS_FILESYSTEM_KEY);

/*==============================================================*/
/* Index: I_NASFILESYSTEM_NSFLSSTMD                             */
/*==============================================================*/
create index I_NASFILESYSTEM_NSFLSSTMD on SAHARA.NAS_FILESYSTEM (
NAS_FILESYSTEM_OID ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_NSFLSSTMD is 
'An index on NAS Filesystem Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ARRAYKEY                              */
/*==============================================================*/
create index I_NASFILESYSTEM_ARRAYKEY on SAHARA.NAS_FILESYSTEM (
ARRAY_KEY ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_NTPPVLMK                              */
/*==============================================================*/
create index I_NASFILESYSTEM_NTPPVLMK on SAHARA.NAS_FILESYSTEM (
NETAPP_VOLUME_KEY ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_NTPPVLMK is 
'An index on NetApp Volume Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_DISPLAYNAME                           */
/*==============================================================*/
create index I_NASFILESYSTEM_DISPLAYNAME on SAHARA.NAS_FILESYSTEM (
DISPLAY_NAME ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_DISCOVERYSTATE                        */
/*==============================================================*/
create index I_NASFILESYSTEM_DISCOVERYSTATE on SAHARA.NAS_FILESYSTEM (
DISCOVERY_STATE ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_SECURITYSTYLE                         */
/*==============================================================*/
create index I_NASFILESYSTEM_SECURITYSTYLE on SAHARA.NAS_FILESYSTEM (
SECURITY_STYLE ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_SECURITYSTYLE is 
'An index on Security Style for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_FILESYSTEMID                          */
/*==============================================================*/
create index I_NASFILESYSTEM_FILESYSTEMID on SAHARA.NAS_FILESYSTEM (
FILESYSTEM_ID ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_FILESYSTEMID is 
'An index on Filesystem ID for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ISQUOTAENABLED                        */
/*==============================================================*/
create index I_NASFILESYSTEM_ISQUOTAENABLED on SAHARA.NAS_FILESYSTEM (
IS_QUOTA_ENABLED ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ISQUOTAENABLED is 
'An index on Is Quota Enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_T2SCRRNTNSFLSS                        */
/*==============================================================*/
create index I_NASFILESYSTEM_T2SCRRNTNSFLSS on SAHARA.NAS_FILESYSTEM (
T2_IS_CURRENT ASC,
NAS_FILESYSTEM_KEY ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_T2SCRRNTNSFLSS is 
'An index on Type 2 - Is Current and NAS Filesystem Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_T2SCRRNTT2SDLT                        */
/*==============================================================*/
create index I_NASFILESYSTEM_T2SCRRNTT2SDLT on SAHARA.NAS_FILESYSTEM (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
NAS_FILESYSTEM_OID ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_T2SCRRNTT2SDLT is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and NAS Filesystem Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_T2ISDELETED                           */
/*==============================================================*/
create index I_NASFILESYSTEM_T2ISDELETED on SAHARA.NAS_FILESYSTEM (
T2_IS_DELETED ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TIER                                  */
/*==============================================================*/
create index I_NASFILESYSTEM_TIER on SAHARA.NAS_FILESYSTEM (
TIER ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TIER is 
'An index on Tier for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TYPE                                  */
/*==============================================================*/
create index I_NASFILESYSTEM_TYPE on SAHARA.NAS_FILESYSTEM (
TYPE ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_PATH                                  */
/*==============================================================*/
create index I_NASFILESYSTEM_PATH on SAHARA.NAS_FILESYSTEM (
PATH ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_PATH is 
'An index on Path for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TXTNDNBLD                             */
/*==============================================================*/
create index I_NASFILESYSTEM_TXTNDNBLD on SAHARA.NAS_FILESYSTEM (
AUTO_EXTEND_ENABLED ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TXTNDNBLD is 
'An index on Auto extend enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ISLOCKENABLED                         */
/*==============================================================*/
create index I_NASFILESYSTEM_ISLOCKENABLED on SAHARA.NAS_FILESYSTEM (
IS_LOCK_ENABLED ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ISLOCKENABLED is 
'An index on Is Lock Enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_CWORMSTATE                            */
/*==============================================================*/
create index I_NASFILESYSTEM_CWORMSTATE on SAHARA.NAS_FILESYSTEM (
CWORM_STATE ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_CWORMSTATE is 
'An index on CWORM State for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_SVRTLPRVSNNGNB                        */
/*==============================================================*/
create index I_NASFILESYSTEM_SVRTLPRVSNNGNB on SAHARA.NAS_FILESYSTEM (
IS_VIRTUAL_PROVISIONING_ENABLE ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_SVRTLPRVSNNGNB is 
'An index on Is Virtual Provisioning enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ISSOURCE                              */
/*==============================================================*/
create index I_NASFILESYSTEM_ISSOURCE on SAHARA.NAS_FILESYSTEM (
IS_SOURCE ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ISSOURCE is 
'An index on Is Source for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ISTARGET                              */
/*==============================================================*/
create index I_NASFILESYSTEM_ISTARGET on SAHARA.NAS_FILESYSTEM (
IS_TARGET ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ISTARGET is 
'An index on Is Target for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_RPLCTNTCHNLG                          */
/*==============================================================*/
create index I_NASFILESYSTEM_RPLCTNTCHNLG on SAHARA.NAS_FILESYSTEM (
REPLICATION_TECHNOLOGY ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_RPLCTNTCHNLG is 
'An index on Replication Technology for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TXTNDMXSZ                             */
/*==============================================================*/
create index I_NASFILESYSTEM_TXTNDMXSZ on SAHARA.NAS_FILESYSTEM (
AUTO_EXTEND_MAX_SIZE ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TXTNDMXSZ is 
'An index on Auto extend Max size for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_HIGHWATERMARK                         */
/*==============================================================*/
create index I_NASFILESYSTEM_HIGHWATERMARK on SAHARA.NAS_FILESYSTEM (
HIGH_WATERMARK ASC
);

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_HIGHWATERMARK is 
'An index on High watermark for dimensional queries.';

grant SELECT on SAHARA.NAS_FILESYSTEM to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_FILESYSTEM_                                       */
/*==============================================================*/
create table SAHARA_APP.NAS_FILESYSTEM_ 
(
   NAS_FILESYSTEM_OID   varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   not null,
   NETAPP_VOLUME_OID    varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DISCOVERY_STATE      varchar(32)                    null,
   SECURITY_STYLE       varchar(32)                    null,
   FILESYSTEM_ID        numeric(9)                     not null,
   IS_QUOTA_ENABLED     char(1)                        not null,
   TIER                 varchar(64)                    null,
   TYPE                 varchar(32)                    not null default ' ',
   PATH                 varchar(256)                   not null default ' ',
   AUTO_EXTEND_ENABLED  char(1)                        null default 'F',
   IS_LOCK_ENABLED      char(1)                        null default 'F',
   CWORM_STATE          varchar(32)                    null,
   IS_VIRTUAL_PROVISIONING_ENABLE char(1)                        not null default 'F',
   IS_SOURCE            char(1)                        not null default 'F',
   IS_TARGET            char(1)                        not null default 'F',
   REPLICATION_TECHNOLOGY varchar(32)                    null,
   AUTO_EXTEND_MAX_SIZE numeric(22,9)                  not null default 0,
   HIGH_WATERMARK       numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.NAS_FILESYSTEM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NAS_FILESYSTEM.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for NAS Filesystems.  It defines the attributes of NAS filesystems that may be used to qualify queries.';

comment on column SAHARA_APP.NAS_FILESYSTEM_.NAS_FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_FILESYSTEM_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_FILESYSTEM_.NETAPP_VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_FILESYSTEM_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The NAS filesystem display name.';

comment on column SAHARA_APP.NAS_FILESYSTEM_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the object:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the NAS filesystem was initially discovered by two or more explorers and then one of them stopped discovering the NAS filesystem.

Note: A NAS filesystem that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.NAS_FILESYSTEM_.SECURITY_STYLE is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM, column SECURITY_STYLE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The security style of the NAS filesystem.  Possible values include: UNIX, NTFS, and mixed.';

comment on column SAHARA_APP.NAS_FILESYSTEM_.FILESYSTEM_ID is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM, column FILESYSTEM_ID.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The identifier of the NAS filesystem(unique within the volume).';

comment on column SAHARA_APP.NAS_FILESYSTEM_.IS_QUOTA_ENABLED is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM, column IS_QUOTA_ENABLED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean value that indicates if a quota is created for the NAS filesystem.';

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter FILESYSTEM_ID add constraint C_NASFILESYSTEM_FILESYSTEMID check (FILESYSTEM_ID >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_QUOTA_ENABLED add constraint C_NASFILESYSTEM_ISQUOTAENABLED check (IS_QUOTA_ENABLED in ('T','F'));

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter AUTO_EXTEND_ENABLED add constraint CKC_AUTO_EXTEND_ENABL_NAS_FILE check (AUTO_EXTEND_ENABLED is null or (AUTO_EXTEND_ENABLED in ('T','F')));

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_LOCK_ENABLED add constraint CKC_IS_LOCK_ENABLED_NAS_FILE check (IS_LOCK_ENABLED is null or (IS_LOCK_ENABLED in ('T','F')));

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_VIRTUAL_PROVISIONING_ENABLE add constraint CKC_IS_VIRTUAL_PROVIS_NAS_FILE check (IS_VIRTUAL_PROVISIONING_ENABLE in ('T','F'));

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_SOURCE add constraint CKC_IS_SOURCE_NAS_FILE check (IS_SOURCE in ('T','F'));

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_TARGET add constraint CKC_IS_TARGET_NAS_FILE check (IS_TARGET in ('T','F'));

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter AUTO_EXTEND_MAX_SIZE add constraint CKC_AUTO_EXTEND_MAX_S_NAS_FILE check (AUTO_EXTEND_MAX_SIZE >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_
   alter HIGH_WATERMARK add constraint CKC_HIGH_WATERMARK_NAS_FILE check (HIGH_WATERMARK >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_
   add constraint PK_NAS_FILESYSTEM_ primary key (NAS_FILESYSTEM_OID);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.NAS_FILESYSTEM_ to SAHARA;

/*==============================================================*/
/* Table: NAS_FILESYSTEM_CAPACITY                               */
/*==============================================================*/
create table SAHARA.NAS_FILESYSTEM_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NAS_FILESYSTEM_KEY   numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   FREE_CAPACITY        numeric(22,9)                  not null default 0,
   PHYSICAL_CAPACITY    numeric(22,9)                  not null,
   VOLUME_CAPACITY      numeric(22,9)                  not null default 0,
   TREE_QUOTA_LIMIT     numeric(22,9)                  not null default 0,
   TREE_QUOTA_USED      numeric(22,9)                  not null default 0,
   NUMBER_OF_LUNS       numeric(10)                    not null default 0,
   FILE_COUNT_TOTAL     numeric(10)                    not null default 0,
   FILE_COUNT_USED      numeric(10)                    not null default 0,
   PHYSICAL_USED        numeric(22,9)                  not null default 0,
   PHYSICAL_FREE        numeric(22,9)                  not null default 0,
   PHYSICAL_VOLUME_CAPACITY numeric(22,9)                  not null default 0,
   PHYSICAL_TREE_QUOTA_LIMIT numeric(22,9)                  not null default 0,
   PHYSICAL_TREE_QUOTA_USED numeric(22,9)                  not null default 0,
   NUMBER_OF_TARGETS    numeric(10)                    not null default 0,
   REPLICATION_TARGET_CAPACITY numeric(22,9)                  not null default 0,
   NO_OF_HOSTS_ACCESSING_NAS numeric(10)                    not null default 0
);

comment on table SAHARA.NAS_FILESYSTEM_CAPACITY is 
'This is the fact table for NAS filesystem measures.  Each rows represents one sampling of values for a NAS filesystem.';

comment on column SAHARA.NAS_FILESYSTEM_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NAS_FILESYSTEM_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NAS_FILESYSTEM_CAPACITY.CAPACITY is 
'The total capacity, in gigabytes, of the NAS filesystem.';

comment on column SAHARA.NAS_FILESYSTEM_CAPACITY.USED_CAPACITY is 
'Total file system capacity, in gigabytes, that has actual data written into.';

comment on column SAHARA.NAS_FILESYSTEM_CAPACITY.FREE_CAPACITY is 
'Total available capacity, in gigabytes, of the NAS Filesystem.';

comment on column SAHARA.NAS_FILESYSTEM_CAPACITY.PHYSICAL_CAPACITY is 
'Total physical capacity which can be used for chargeback purpose.';

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter CAPACITY add constraint C_NSFLSSTMCPCT_CAPACITY check (CAPACITY >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter USED_CAPACITY add constraint C_NSFLSSTMCPCT_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter FREE_CAPACITY add constraint C_NSFLSSTMCPCT_FREECAPACITY check (FREE_CAPACITY >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter VOLUME_CAPACITY add constraint CKC_VOLUME_CAPACITY_NAS_FILE check (VOLUME_CAPACITY >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter TREE_QUOTA_LIMIT add constraint CKC_TREE_QUOTA_LIMIT_NAS_FILE check (TREE_QUOTA_LIMIT >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter TREE_QUOTA_USED add constraint CKC_TREE_QUOTA_USED_NAS_FILE check (TREE_QUOTA_USED >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter NUMBER_OF_LUNS add constraint CKC_NUMBER_OF_LUNS_NAS_FILE check (NUMBER_OF_LUNS >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter FILE_COUNT_TOTAL add constraint CKC_FILE_COUNT_TOTAL_NAS_FILE check (FILE_COUNT_TOTAL >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter FILE_COUNT_USED add constraint CKC_FILE_COUNT_USED_NAS_FILE check (FILE_COUNT_USED >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NAS_FILE check (PHYSICAL_USED >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_FREE add constraint CKC_PHYSICAL_FREE_NAS_FILE check (PHYSICAL_FREE >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_VOLUME_CAPACITY add constraint CKC_PHYSICAL_VOLUME_C_NAS_FILE check (PHYSICAL_VOLUME_CAPACITY >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_TREE_QUOTA_LIMIT add constraint CKC_PHYSICAL_TREE_QUO_NAS_FS2 check (PHYSICAL_TREE_QUOTA_LIMIT >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_TREE_QUOTA_USED add constraint CKC_PHYSICAL_TREE_QUO_NAS_FS check (PHYSICAL_TREE_QUOTA_USED >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NAS_FILE check (NUMBER_OF_TARGETS >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter REPLICATION_TARGET_CAPACITY add constraint CKC_REPLICATION_TARGE_NAS_FILE check (REPLICATION_TARGET_CAPACITY >= 0);

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter NO_OF_HOSTS_ACCESSING_NAS add constraint CKC_NO_OF_HOSTS_ACCES_NAS_FILE check (NO_OF_HOSTS_ACCESSING_NAS >= 0);

/*==============================================================*/
/* Index: I_NSFLSSTMCPCT_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_NSFLSSTMCPCT_ROLLUPBATCHKEY on SAHARA.NAS_FILESYSTEM_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

comment on index SAHARA.NAS_FILESYSTEM_CAPACITY.I_NSFLSSTMCPCT_ROLLUPBATCHKEY is 
'An index on Roll-up Batch Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NSFLSSTMCPCT_DATESKEY                               */
/*==============================================================*/
create index I_NSFLSSTMCPCT_DATESKEY on SAHARA.NAS_FILESYSTEM_CAPACITY (
DATES_KEY ASC
);

comment on index SAHARA.NAS_FILESYSTEM_CAPACITY.I_NSFLSSTMCPCT_DATESKEY is 
'An index on Dates Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NSFLSSTMCPCT_NSFLSSTMK                              */
/*==============================================================*/
create index I_NSFLSSTMCPCT_NSFLSSTMK on SAHARA.NAS_FILESYSTEM_CAPACITY (
NAS_FILESYSTEM_KEY ASC
);

comment on index SAHARA.NAS_FILESYSTEM_CAPACITY.I_NSFLSSTMCPCT_NSFLSSTMK is 
'An index on NAS Filesystem Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NSFLSSTMCPCT_CAPACITY                               */
/*==============================================================*/
create index I_NSFLSSTMCPCT_CAPACITY on SAHARA.NAS_FILESYSTEM_CAPACITY (
CAPACITY ASC
);

comment on index SAHARA.NAS_FILESYSTEM_CAPACITY.I_NSFLSSTMCPCT_CAPACITY is 
'An index on Capacity for dimensional queries.';

/*==============================================================*/
/* Index: I_NSFLSSTMCPCT_USEDCAPACITY                           */
/*==============================================================*/
create index I_NSFLSSTMCPCT_USEDCAPACITY on SAHARA.NAS_FILESYSTEM_CAPACITY (
USED_CAPACITY ASC
);

comment on index SAHARA.NAS_FILESYSTEM_CAPACITY.I_NSFLSSTMCPCT_USEDCAPACITY is 
'An index on Used Capacity for dimensional queries.';

/*==============================================================*/
/* Index: I_NSFLSSTMCPCT_FREECAPACITY                           */
/*==============================================================*/
create index I_NSFLSSTMCPCT_FREECAPACITY on SAHARA.NAS_FILESYSTEM_CAPACITY (
FREE_CAPACITY ASC
);

comment on index SAHARA.NAS_FILESYSTEM_CAPACITY.I_NSFLSSTMCPCT_FREECAPACITY is 
'An index on Free Capacity for dimensional queries.';

/*==============================================================*/
/* Index: I_NSFLSSTMCPCT_PHSCLCPCT                              */
/*==============================================================*/
create index I_NSFLSSTMCPCT_PHSCLCPCT on SAHARA.NAS_FILESYSTEM_CAPACITY (
PHYSICAL_CAPACITY ASC
);

comment on index SAHARA.NAS_FILESYSTEM_CAPACITY.I_NSFLSSTMCPCT_PHSCLCPCT is 
'An index on Physical Capacity for dimensional queries.';

grant SELECT on SAHARA.NAS_FILESYSTEM_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_FILESYSTEM_CAPACITY_                              */
/*==============================================================*/
create table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ 
(
   NAS_FILESYSTEM_OID   varchar(512)                   not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   FREE_CAPACITY        numeric(22,9)                  not null default 0,
   PHYSICAL_CAPACITY    numeric(22,9)                  not null,
   VOLUME_CAPACITY      numeric(22,9)                  not null default 0,
   TREE_QUOTA_LIMIT     numeric(22,9)                  not null default 0,
   TREE_QUOTA_USED      numeric(22,9)                  not null default 0,
   NUMBER_OF_LUNS       numeric(10)                    not null default 0,
   FILE_COUNT_TOTAL     numeric(10)                    not null default 0,
   FILE_COUNT_USED      numeric(10)                    not null default 0,
   PHYSICAL_USED        numeric(22,9)                  not null default 0,
   PHYSICAL_FREE        numeric(22,9)                  not null default 0,
   PHYSICAL_VOLUME_CAPACITY numeric(22,9)                  not null default 0,
   PHYSICAL_TREE_QUOTA_LIMIT numeric(22,9)                  not null default 0,
   PHYSICAL_TREE_QUOTA_USED numeric(22,9)                  not null default 0,
   NUMBER_OF_TARGETS    numeric(10)                    not null default 0,
   REPLICATION_TARGET_CAPACITY numeric(22,9)                  not null default 0,
   NO_OF_HOSTS_ACCESSING_NAS numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.NAS_FILESYSTEM_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NAS filesystem measures.  Each rows represents one sampling of values for a NAS filesystem.';

comment on column SAHARA_APP.NAS_FILESYSTEM_CAPACITY_.NAS_FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_FILESYSTEM_CAPACITY_.CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM_CAPACITY, column CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total capacity, in gigabytes, of the NAS filesystem.';

comment on column SAHARA_APP.NAS_FILESYSTEM_CAPACITY_.USED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM_CAPACITY, column USED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total file system capacity, in gigabytes, that has actual data written into.';

comment on column SAHARA_APP.NAS_FILESYSTEM_CAPACITY_.FREE_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM_CAPACITY, column FREE_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total available capacity, in gigabytes, of the NAS Filesystem.';

comment on column SAHARA_APP.NAS_FILESYSTEM_CAPACITY_.PHYSICAL_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NAS_FILESYSTEM_CAPACITY, column PHYSICAL_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total physical capacity which can be used for chargeback purpose.';

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter CAPACITY add constraint C_NSFLSSTMCPCT_CAPACITY check (CAPACITY >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter USED_CAPACITY add constraint C_NSFLSSTMCPCT_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter FREE_CAPACITY add constraint C_NSFLSSTMCPCT_FREECAPACITY check (FREE_CAPACITY >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter VOLUME_CAPACITY add constraint CKC_VOLUME_CAPACITY_NAS_FILE check (VOLUME_CAPACITY >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter TREE_QUOTA_LIMIT add constraint CKC_TREE_QUOTA_LIMIT_NAS_FIL_ check (TREE_QUOTA_LIMIT >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter TREE_QUOTA_USED add constraint CKC_TREE_QUOTA_USED_NAS_FIL_ check (TREE_QUOTA_USED >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter NUMBER_OF_LUNS add constraint CKC_NUMBER_OF_LUNS_NAS_FILE check (NUMBER_OF_LUNS >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter FILE_COUNT_TOTAL add constraint CKC_FILE_COUNT_TOTAL_NAS_FILE check (FILE_COUNT_TOTAL >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter FILE_COUNT_USED add constraint CKC_FILE_COUNT_USED_NAS_FILE check (FILE_COUNT_USED >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NAS_FILE check (PHYSICAL_USED >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_FREE add constraint CKC_PHYSICAL_FREE_NAS_FILE check (PHYSICAL_FREE >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_VOLUME_CAPACITY add constraint CKC_PHYSICAL_VOLUME_C_NAS_FILE check (PHYSICAL_VOLUME_CAPACITY >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_TREE_QUOTA_LIMIT add constraint CKC_PHYSICAL_TREE_QUO_NAS_FILE_2 check (PHYSICAL_TREE_QUOTA_LIMIT >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_TREE_QUOTA_USED add constraint CKC_PHYSICAL_TREE_QUO_NAS_FILE_ check (PHYSICAL_TREE_QUOTA_USED >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NAS_FILE check (NUMBER_OF_TARGETS >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter REPLICATION_TARGET_CAPACITY add constraint CKC_REPLICATION_TARGE_NAS_FILE check (REPLICATION_TARGET_CAPACITY >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter NO_OF_HOSTS_ACCESSING_NAS add constraint CKC_NO_OF_HOSTS_ACCES_NAS_FILE check (NO_OF_HOSTS_ACCESSING_NAS >= 0);

alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   add constraint PK_NAS_FILESYSTEM_CAPACITY_ primary key (NAS_FILESYSTEM_OID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ to SAHARA;

/*==============================================================*/
/* Table: NAS_FS_BVIEW_NODE                                     */
/*==============================================================*/
create table SAHARA.NAS_FS_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   NAS_FILESYSTEM_KEY   numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.NAS_FS_BVIEW_NODE is 
'This is a join table between NAS Filesystem objects and business view nodes.';

comment on column SAHARA.NAS_FS_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.NAS_FS_BVIEW_NODE.NAS_FILESYSTEM_KEY is 
'The unique identifier of the NetApp volume object with which the business view node is associated.';

comment on column SAHARA.NAS_FS_BVIEW_NODE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the join table record.';

alter table SAHARA.NAS_FS_BVIEW_NODE
   add constraint PK_NAS_FS_BVIEW_NODE primary key (BVIEW_NODE_KEY, NAS_FILESYSTEM_KEY);

/*==============================================================*/
/* Index: I_NASFSBVIEWNODE_BVIEWNASFS                           */
/*==============================================================*/
create unique index I_NASFSBVIEWNODE_BVIEWNASFS on SAHARA.NAS_FS_BVIEW_NODE (
BVIEW_KEY ASC,
NAS_FILESYSTEM_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.NAS_FS_BVIEW_NODE.I_NASFSBVIEWNODE_BVIEWNASFS is 
'This index exists to accelerate queries for which NetApp volumes are associated with a particular business view.';

grant SELECT on SAHARA.NAS_FS_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_QUOTA                                             */
/*==============================================================*/
create table SAHARA.NAS_QUOTA 
(
   NAS_QUOTA_KEY        numeric(10)                    not null default autoincrement,
   NAS_FILESYSTEM_KEY   numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null,
   NAS_QUOTA_OID        varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   TYPE                 varchar(32)                    not null,
   TARGET               varchar(256)                   not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.NAS_QUOTA is 
'This is the dimension table for NAS Quotas.  It defines the attributes of arrays that may be used to qualify NAS Quota queries.';

comment on column SAHARA.NAS_QUOTA.NAS_QUOTA_KEY is 
'A unique database generated identifier for the array record used as a surrogate key.';

comment on column SAHARA.NAS_QUOTA.NAS_FILESYSTEM_KEY is 
'A unique database generated identifier for the NAS filesystem record used as a surrogate key.';

comment on column SAHARA.NAS_QUOTA.ARRAY_KEY is 
'A unique database generated identifier for the array record used as a surrogate key.';

comment on column SAHARA.NAS_QUOTA.NAS_QUOTA_OID is 
'Unique identifier for this object in VOM database.';

comment on column SAHARA.NAS_QUOTA.CC_STORAGE_URL is 
'The URL for the array object overview view in the VOM GUI.';

comment on column SAHARA.NAS_QUOTA.DISPLAY_NAME is 
'Quota display name.';

comment on column SAHARA.NAS_QUOTA.TYPE is 
'Quota type.  This can be:
User
Group
Tree';

comment on column SAHARA.NAS_QUOTA.TARGET is 
'Quota target';

comment on column SAHARA.NAS_QUOTA.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.NAS_QUOTA.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.NAS_QUOTA
   alter T2_IS_CURRENT add constraint C_NASFILESYSTEM_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.NAS_QUOTA
   alter T2_IS_DELETED add constraint C_NASFILESYSTEM_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.NAS_QUOTA
   add constraint PK_NAS_QUOTA primary key (NAS_QUOTA_KEY);

/*==============================================================*/
/* Index: I_NASQUOTA_NASFILESYSTEMKEY                           */
/*==============================================================*/
create index I_NASQUOTA_NASFILESYSTEMKEY on SAHARA.NAS_QUOTA (
NAS_FILESYSTEM_KEY ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_NASFILESYSTEMKEY is 
'An index on NAS Filesystem Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_ARRAYKEY                                   */
/*==============================================================*/
create index I_NASQUOTA_ARRAYKEY on SAHARA.NAS_QUOTA (
ARRAY_KEY ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_NASQUOTAOID                                */
/*==============================================================*/
create index I_NASQUOTA_NASQUOTAOID on SAHARA.NAS_QUOTA (
NAS_QUOTA_OID ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_NASQUOTAOID is 
'An index on NAS Quota Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_DISPLAYNAME                                */
/*==============================================================*/
create index I_NASQUOTA_DISPLAYNAME on SAHARA.NAS_QUOTA (
DISPLAY_NAME ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_TYPE                                       */
/*==============================================================*/
create index I_NASQUOTA_TYPE on SAHARA.NAS_QUOTA (
TYPE ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_TARGET                                     */
/*==============================================================*/
create index I_NASQUOTA_TARGET on SAHARA.NAS_QUOTA (
TARGET ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_TARGET is 
'An index on Target for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_T2SCRRNTNSQTK                              */
/*==============================================================*/
create index I_NASQUOTA_T2SCRRNTNSQTK on SAHARA.NAS_QUOTA (
T2_IS_CURRENT ASC,
NAS_QUOTA_KEY ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_T2SCRRNTNSQTK is 
'An index on Type 2 - Is Current and NAS Quota Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_NASQUOTA_T2SCRRNTT2SDLTDD                           */
/*==============================================================*/
create index I_NASQUOTA_T2SCRRNTT2SDLTDD on SAHARA.NAS_QUOTA (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
NAS_QUOTA_OID ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and NAS Quota Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_NASQUOTA_T2ISDELETED                                */
/*==============================================================*/
create index I_NASQUOTA_T2ISDELETED on SAHARA.NAS_QUOTA (
T2_IS_DELETED ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

grant SELECT on SAHARA.NAS_QUOTA to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_QUOTA_                                            */
/*==============================================================*/
create table SAHARA_APP.NAS_QUOTA_ 
(
   NAS_QUOTA_OID        varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   not null,
   NAS_FILESYSTEM_OID   varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   TYPE                 varchar(32)                    not null,
   TARGET               varchar(256)                   not null
);

comment on table SAHARA_APP.NAS_QUOTA_ is 
'This is a staging table for loading data into warehouse table SAHARA.NAS_FILESYSTEM.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for NAS Quota.  It defines the attributes of NAS Quota that may be used to qualify queries.';

comment on column SAHARA_APP.NAS_QUOTA_.NAS_QUOTA_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_.NAS_FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_.DISPLAY_NAME is 
'Array display name.';

comment on column SAHARA_APP.NAS_QUOTA_.TYPE is 
'Array type.  This can be:
Filer
Enclosure
Unenclosed Device';

comment on column SAHARA_APP.NAS_QUOTA_.TARGET is 
'Array display name.';

alter table SAHARA_APP.NAS_QUOTA_
   add constraint PK_NAS_QUOTA_ primary key (NAS_QUOTA_OID);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.NAS_QUOTA_ to SAHARA;

/*==============================================================*/
/* Table: NAS_QUOTA_CAPACITY                                    */
/*==============================================================*/
create table SAHARA.NAS_QUOTA_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NAS_QUOTA_KEY        numeric(10)                    not null,
   DISK_LIMIT_HARD      numeric(22,9)                  not null default 0,
   DISK_LIMIT_SOFT      numeric(22,9)                  not null default 0,
   DISK_USED            numeric(22,9)                  not null default 0,
   FILE_LIMIT_HARD      unsigned bigint                not null default 0,
   FILE_LIMIT_SOFT      unsigned bigint                not null default 0,
   FILES_USED           unsigned bigint                not null default 0,
   THRESHOLD            numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_LIMIT_HARD numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_LIMIT_SOFT numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_USED   numeric(22,9)                  not null default 0
);

comment on table SAHARA.NAS_QUOTA_CAPACITY is 
'This is the fact table for NAS Quota measures.  Each rows represents one sampling of values for a NAS Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DISK_LIMIT_HARD is 
'Hard capacity limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DISK_LIMIT_SOFT is 
'Soft capacity limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DISK_USED is 
'Capacity used on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.FILE_LIMIT_HARD is 
'Hard file limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.FILE_LIMIT_SOFT is 
'Soft file limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.FILES_USED is 
'Number of files used on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.THRESHOLD is 
'Threshold set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.PHYSICAL_DISK_LIMIT_HARD is 
'Physical counterpart of hard capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.PHYSICAL_DISK_LIMIT_SOFT is 
'Physical counterpart of soft capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.PHYSICAL_DISK_USED is 
'Physical counterpart of capacity used on Quota. This involves the RAID overhead as well.';

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter DISK_LIMIT_HARD add constraint CKC_DISK_LIMIT_HARD_NAS_QUOT check (DISK_LIMIT_HARD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter DISK_LIMIT_SOFT add constraint CKC_DISK_LIMIT_SOFT_NAS_QUOT check (DISK_LIMIT_SOFT >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter DISK_USED add constraint CKC_DISK_USED_NAS_QUOT check (DISK_USED >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter FILE_LIMIT_HARD add constraint CKC_FILE_LIMIT_HARD_NAS_QUOT check (FILE_LIMIT_HARD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter FILE_LIMIT_SOFT add constraint CKC_FILE_LIMIT_SOFT_NAS_QUOT check (FILE_LIMIT_SOFT >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter FILES_USED add constraint CKC_FILES_USED_NAS_QUOT check (FILES_USED >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter THRESHOLD add constraint CKC_THRESHOLD_NAS_QUOT check (THRESHOLD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter PHYSICAL_DISK_LIMIT_HARD add constraint CKC_PHYSICAL_DISK_LIM_NAS_Q2 check (PHYSICAL_DISK_LIMIT_HARD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter PHYSICAL_DISK_LIMIT_SOFT add constraint CKC_PHYSICAL_DISK_LIM_NAS_Q check (PHYSICAL_DISK_LIMIT_SOFT >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter PHYSICAL_DISK_USED add constraint CKC_PHYSICAL_DISK_USE_NAS_QUOT check (PHYSICAL_DISK_USED >= 0);

grant SELECT on SAHARA.NAS_QUOTA_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_QUOTA_CAPACITY_                                   */
/*==============================================================*/
create table SAHARA_APP.NAS_QUOTA_CAPACITY_ 
(
   NAS_QUOTA_OID        varchar(512)                   not null,
   DISK_LIMIT_HARD      numeric(22,9)                  not null default 0,
   DISK_LIMIT_SOFT      numeric(22,9)                  not null default 0,
   DISK_USED            numeric(22,9)                  not null default 0,
   FILE_LIMIT_HARD      unsigned bigint                not null default 0,
   FILE_LIMIT_SOFT      unsigned bigint                not null default 0,
   FILES_USED           unsigned bigint                not null default 0,
   THRESHOLD            numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_LIMIT_HARD numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_LIMIT_SOFT numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_USED   numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.NAS_QUOTA_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.NAS_FILESYSTEM_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NAS quota measures.  Each rows represents one sampling of values for a NAS quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.NAS_QUOTA_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.DISK_LIMIT_HARD is 
'Hard capacity limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.DISK_LIMIT_SOFT is 
'Soft capacity limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.DISK_USED is 
'Capacity used on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.FILE_LIMIT_HARD is 
'Hard file limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.FILE_LIMIT_SOFT is 
'Soft file limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.FILES_USED is 
'Number of files used on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.THRESHOLD is 
'Threshold set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.PHYSICAL_DISK_LIMIT_HARD is 
'Physical counterpart of hard capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.PHYSICAL_DISK_LIMIT_SOFT is 
'Physical counterpart of soft capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.PHYSICAL_DISK_USED is 
'Physical counterpart of capacity used on Quota. This involves the RAID overhead as well.';

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter DISK_LIMIT_HARD add constraint CKC_DISK_LIMIT_HARD_NAS_QUO_ check (DISK_LIMIT_HARD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter DISK_LIMIT_SOFT add constraint CKC_DISK_LIMIT_SOFT_NAS_QUO_ check (DISK_LIMIT_SOFT >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter DISK_USED add constraint CKC_DISK_USED_NAS_QUOT check (DISK_USED >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter FILE_LIMIT_HARD add constraint CKC_FILE_LIMIT_HARD_NAS_QUOT check (FILE_LIMIT_HARD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter FILE_LIMIT_SOFT add constraint CKC_FILE_LIMIT_SOFT_NAS_QUOT check (FILE_LIMIT_SOFT >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter FILES_USED add constraint CKC_FILES_USED_NAS_QUOT check (FILES_USED >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter THRESHOLD add constraint CKC_THRESHOLD_NAS_QUOT check (THRESHOLD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter PHYSICAL_DISK_LIMIT_HARD add constraint CKC_PHYSICAL_DISK_LIM_NAS_QUOT_2 check (PHYSICAL_DISK_LIMIT_HARD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter PHYSICAL_DISK_LIMIT_SOFT add constraint CKC_PHYSICAL_DISK_LIM_NAS_QUOT_ check (PHYSICAL_DISK_LIMIT_SOFT >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter PHYSICAL_DISK_USED add constraint CKC_PHYSICAL_DISK_USE_NAS_QUOT check (PHYSICAL_DISK_USED >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   add constraint PK_NAS_QUOTA_CAPACITY_ primary key (NAS_QUOTA_OID);

grant DELETE,ALTER,INSERT,REFERENCES,UPDATE on SAHARA_APP.NAS_QUOTA_CAPACITY_ to SAHARA;

grant SELECT on SAHARA_APP.NAS_QUOTA_CAPACITY_ to SAHARA with grant option;

/*==============================================================*/
/* Table: NAS_SHARE_HOST                                        */
/*==============================================================*/
create table SAHARA.NAS_SHARE_HOST 
(
   NETAPP_SHARE_KEY     numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   ACCESS_TYPE          varchar(32)                    not null
);

comment on table SAHARA.NAS_SHARE_HOST is 
'Table to keep association between NAS Share (Netapp Share) and Host.';

comment on column SAHARA.NAS_SHARE_HOST.NETAPP_SHARE_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NAS_SHARE_HOST.HOST_KEY is 
'The unique identifier of the parent filer for the volume.';

alter table SAHARA.NAS_SHARE_HOST
   add constraint PK_NAS_SHARE_HOST primary key clustered (NETAPP_SHARE_KEY, HOST_KEY);

grant SELECT on SAHARA.NAS_SHARE_HOST to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_SHARE_HOST_                                       */
/*==============================================================*/
create table SAHARA_APP.NAS_SHARE_HOST_ 
(
   NETAPP_SHARE_OID     varchar(512)                   not null,
   HOST_OID             varchar(512)                   not null,
   ACCESS_TYPE          varchar(32)                    not null
);

comment on table SAHARA_APP.NAS_SHARE_HOST_ is 
'Staging table to keep association between NAS Share (Netapp Share) and Host.';

comment on column SAHARA_APP.NAS_SHARE_HOST_.NETAPP_SHARE_OID is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA_APP.NAS_SHARE_HOST_.HOST_OID is 
'The unique identifier of the parent filer for the volume.';

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NAS_SHARE_HOST_ to SAHARA;

/*==============================================================*/
/* Table: NAS_SHARE_HOST_FILESYSTEM                             */
/*==============================================================*/
create table SAHARA.NAS_SHARE_HOST_FILESYSTEM 
(
   NETAPP_SHARE_KEY     numeric(10)                    not null,
   FILESYSTEM_KEY       numeric(10)                    not null
);

comment on table SAHARA.NAS_SHARE_HOST_FILESYSTEM is 
'Table to keep association between NAS Share (Netapp Share) and Host FS.';

comment on column SAHARA.NAS_SHARE_HOST_FILESYSTEM.NETAPP_SHARE_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NAS_SHARE_HOST_FILESYSTEM.FILESYSTEM_KEY is 
'The unique identifier of the parent filer for the volume.';

alter table SAHARA.NAS_SHARE_HOST_FILESYSTEM
   add constraint PK_NAS_SHARE_HOST_FILESYSTEM primary key clustered (NETAPP_SHARE_KEY, FILESYSTEM_KEY);

grant SELECT on SAHARA.NAS_SHARE_HOST_FILESYSTEM to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_SHARE_HOST_FILESYSTEM_                            */
/*==============================================================*/
create table SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_ 
(
   NETAPP_SHARE_OID     varchar(512)                   not null,
   FILESYSTEM_OID       varchar(512)                   not null
);

comment on table SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_ is 
'Staging table to keep association between NAS Share (Netapp Share) and Host Filesystem.';

comment on column SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_.NETAPP_SHARE_OID is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_.FILESYSTEM_OID is 
'The unique identifier of the parent filer for the volume.';

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_ to SAHARA;

/*==============================================================*/
/* Table: NAS_VOLUME_FILER                                      */
/*==============================================================*/
create table SAHARA.NAS_VOLUME_FILER 
(
   NETAPP_VOLUME_KEY    numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null
);

comment on table SAHARA.NAS_VOLUME_FILER is 
'Table to keep association between NAS Volume (Netapp Volume) and filer (Array). As one volume can be associate with multiple filers (virtual filers), this is a many to many association.';

comment on column SAHARA.NAS_VOLUME_FILER.NETAPP_VOLUME_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NAS_VOLUME_FILER.ARRAY_KEY is 
'The unique identifier of the parent filer for the volume.';

alter table SAHARA.NAS_VOLUME_FILER
   add constraint PK_NAS_VOLUME_FILER primary key clustered (NETAPP_VOLUME_KEY, ARRAY_KEY);

grant SELECT on SAHARA.NAS_VOLUME_FILER to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_VOLUME_FILER_                                     */
/*==============================================================*/
create table SAHARA_APP.NAS_VOLUME_FILER_ 
(
   NETAPP_VOLUME_OID    varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   not null
);

comment on table SAHARA_APP.NAS_VOLUME_FILER_ is 
'Staging table to keep association between NAS Volume (Netapp Volume) and filer (Array). As one volume can be associate with multiple filers (virtual filers), this is a many to many association.';

comment on column SAHARA_APP.NAS_VOLUME_FILER_.NETAPP_VOLUME_OID is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA_APP.NAS_VOLUME_FILER_.ARRAY_OID is 
'The unique identifier of the parent filer for the volume.';

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NAS_VOLUME_FILER_ to SAHARA;

/*==============================================================*/
/* Table: NA_VOLUME_BVIEW_NODE                                  */
/*==============================================================*/
create table SAHARA.NA_VOLUME_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   NETAPP_VOLUME_KEY    numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.NA_VOLUME_BVIEW_NODE is 
'This is a join table between NetApp volume objects and business view nodes.';

comment on column SAHARA.NA_VOLUME_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.NA_VOLUME_BVIEW_NODE.NETAPP_VOLUME_KEY is 
'The unique identifier of the NetApp volume object with which the business view node is associated.';

comment on column SAHARA.NA_VOLUME_BVIEW_NODE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the join table record.';

alter table SAHARA.NA_VOLUME_BVIEW_NODE
   add constraint PK_NA_VOLUME_BVIEW_NODE primary key (BVIEW_NODE_KEY, NETAPP_VOLUME_KEY);

/*==============================================================*/
/* Index: I_NAVOLBVIEWNODE_BVIEWNAVOL                           */
/*==============================================================*/
create unique index I_NAVOLBVIEWNODE_BVIEWNAVOL on SAHARA.NA_VOLUME_BVIEW_NODE (
BVIEW_KEY ASC,
NETAPP_VOLUME_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.NA_VOLUME_BVIEW_NODE.I_NAVOLBVIEWNODE_BVIEWNAVOL is 
'This index exists to accelerate queries for which NetApp volumes are associated with a particular business view.';

grant SELECT on SAHARA.NA_VOLUME_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_FILE_FCAT                                      */
/*==============================================================*/
create table SAHARA.NETAPP_FILE_FCAT 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DM_BATCH_KEY         numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null,
   FILE_CATEGORY_KEY    numeric(10)                    not null,
   DM_DATA_AGE          integer                        not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA.NETAPP_FILE_FCAT is 
'This is the fact table for NetApp device (filers/arrays/storage systems) files organized by file category.';

comment on column SAHARA.NETAPP_FILE_FCAT.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NETAPP_FILE_FCAT.DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.';

comment on column SAHARA.NETAPP_FILE_FCAT.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NETAPP_FILE_FCAT.ARRAY_KEY is 
'The surrogate key from the array dimension that identifies the array for which the array storage data was collected.  This key may be used in joins to the ARRAY table.';

comment on column SAHARA.NETAPP_FILE_FCAT.FILE_CATEGORY_KEY is 
'The surrogate key from the file category dimension that identifies the file category for which the file data was collected.  This key may be used in joins to the FILE_STORAGE table.';

comment on column SAHARA.NETAPP_FILE_FCAT.DM_DATA_AGE is 
'The age of the DM data, in seconds,  relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.';

comment on column SAHARA.NETAPP_FILE_FCAT.CONSUMPTION is 
'The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA.NETAPP_FILE_FCAT.FILES is 
'The number of files of the associated category on the device.';

alter table SAHARA.NETAPP_FILE_FCAT
   alter CONSUMPTION add constraint C_NTPPFLFCT_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA.NETAPP_FILE_FCAT
   alter FILES add constraint C_NTPPFLFCT_FILES check (FILES >= 0);

/*==============================================================*/
/* Index: I_NAFILEFCAT_ROLLUPBATCHKEY                           */
/*==============================================================*/
create index I_NAFILEFCAT_ROLLUPBATCHKEY on SAHARA.NETAPP_FILE_FCAT (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_NAFILEFCAT_DATESKEY                                 */
/*==============================================================*/
create index I_NAFILEFCAT_DATESKEY on SAHARA.NETAPP_FILE_FCAT (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_NAFILEFCAT_ARRAYKEY                                 */
/*==============================================================*/
create index I_NAFILEFCAT_ARRAYKEY on SAHARA.NETAPP_FILE_FCAT (
ARRAY_KEY ASC
);

/*==============================================================*/
/* Index: I_NAFILEFCAT_FILECATEGORYKEY                          */
/*==============================================================*/
create index I_NAFILEFCAT_FILECATEGORYKEY on SAHARA.NETAPP_FILE_FCAT (
FILE_CATEGORY_KEY ASC
);

grant SELECT on SAHARA.NETAPP_FILE_FCAT to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_FILE_FCAT_DM_                                  */
/*==============================================================*/
create table SAHARA_APP.NETAPP_FILE_FCAT_DM_ 
(
   DM_ID                numeric(10)                    not null,
   FILE_CATEGORY_OID    varchar(512)                   not null,
   DM_BATCH_ID          numeric(10)                    not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_FILE_FCAT_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_FILE_FCAT.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp device (filers/arrays/storage systems) files organized by file category.';

comment on column SAHARA_APP.NETAPP_FILE_FCAT_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_FILE_FCAT_DM_.FILE_CATEGORY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_FILE_FCAT_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_FILE_FCAT_DM_.CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.NETAPP_FILE_FCAT, column CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA_APP.NETAPP_FILE_FCAT_DM_.FILES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_FILE_FCAT, column FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files of the associated category on the device.';

alter table SAHARA_APP.NETAPP_FILE_FCAT_DM_
   alter DM_ID add constraint C_NTPPFLFCTDM_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_FILE_FCAT_DM_
   alter DM_BATCH_ID add constraint C_NTPPFLFCTDM_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.NETAPP_FILE_FCAT_DM_
   alter CONSUMPTION add constraint C_NTPPFLFCTDM_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA_APP.NETAPP_FILE_FCAT_DM_
   alter FILES add constraint C_NTPPFLFCTDM_FILES check (FILES >= 0);

alter table SAHARA_APP.NETAPP_FILE_FCAT_DM_
   add constraint PK_NETAPP_FILE_FCAT_DM_ primary key (DM_ID, FILE_CATEGORY_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_FILE_FCAT_DM_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_QTREE                                          */
/*==============================================================*/
create table SAHARA.NETAPP_QTREE 
(
   NETAPP_QTREE_KEY     numeric(10)                    not null default autoincrement,
   NETAPP_QTREE_OID     varchar(512)                   not null,
   ARRAY_KEY            numeric(10)                    not null,
   NETAPP_VOLUME_KEY    numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   QTREE_PATH           varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DISCOVERY_STATE      varchar(32)                    null,
   SECURITY_STYLE       varchar(32)                    null,
   FILESYSTEM_ID        numeric(9)                     not null,
   IS_QUOTA_ENABLED     char(1)                        not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.NETAPP_QTREE is 
'This is the dimension table for NetApp qtrees.  It defines the attributes of NetApp qtrees that may be used to qualify queries.';

comment on column SAHARA.NETAPP_QTREE.NETAPP_QTREE_KEY is 
'A unique database generated identifier for the NetApp qtree record used as a surrogate key.';

comment on column SAHARA.NETAPP_QTREE.NETAPP_QTREE_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.NETAPP_QTREE.ARRAY_KEY is 
'The unique identifier of the parent NetApp filer array object.';

comment on column SAHARA.NETAPP_QTREE.NETAPP_VOLUME_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NETAPP_QTREE.CC_STORAGE_URL is 
'The URL for the NetApp qtree object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.NETAPP_QTREE.QTREE_PATH is 
'The complete path of the qtree on the volume.';

comment on column SAHARA.NETAPP_QTREE.DISPLAY_NAME is 
'The NetApp qtree display name.';

comment on column SAHARA.NETAPP_QTREE.DISCOVERY_STATE is 
'The discovery state of the array:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the NetApp qtree was initially discovered by two or more explorers and then one of them stopped discovering the qtree.

Note: A NetApp qtree that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.NETAPP_QTREE.SECURITY_STYLE is 
'The security style of the qtree.  Possible values include: UNIX, NTFS, and mixed.';

comment on column SAHARA.NETAPP_QTREE.FILESYSTEM_ID is 
'The identifier of the qtree (unique within the volume).  This is zero (0) if the qtree is the volume itself.';

comment on column SAHARA.NETAPP_QTREE.IS_QUOTA_ENABLED is 
'A boolean value that indicates if a quota is created for the qtree.';

comment on column SAHARA.NETAPP_QTREE.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.NETAPP_QTREE.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.NETAPP_QTREE
   alter FILESYSTEM_ID add constraint C_NETAPPQTREE_FILESYSTEMID check (FILESYSTEM_ID >= 0);

alter table SAHARA.NETAPP_QTREE
   alter IS_QUOTA_ENABLED add constraint C_NETAPPQTREE_ISQUOTAENABLED check (IS_QUOTA_ENABLED in ('T','F'));

alter table SAHARA.NETAPP_QTREE
   alter T2_IS_CURRENT add constraint C_NETAPPQTREE_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.NETAPP_QTREE
   alter T2_IS_DELETED add constraint C_NETAPPQTREE_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.NETAPP_QTREE
   add constraint PK_NETAPP_QTREE primary key (NETAPP_QTREE_KEY);

/*==============================================================*/
/* Index: I_NETAPPQTREE_NETAPPQTREEOID                          */
/*==============================================================*/
create index I_NETAPPQTREE_NETAPPQTREEOID on SAHARA.NETAPP_QTREE (
NETAPP_QTREE_OID ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_NETAPPQTREEOID is 
'An index on NetApp QTree Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_ARRAYKEY                                */
/*==============================================================*/
create index I_NETAPPQTREE_ARRAYKEY on SAHARA.NETAPP_QTREE (
ARRAY_KEY ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_NETAPPVOLUMEKEY                         */
/*==============================================================*/
create index I_NETAPPQTREE_NETAPPVOLUMEKEY on SAHARA.NETAPP_QTREE (
NETAPP_VOLUME_KEY ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_NETAPPVOLUMEKEY is 
'An index on NetApp Volume Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_QTREEPATH                               */
/*==============================================================*/
create index I_NETAPPQTREE_QTREEPATH on SAHARA.NETAPP_QTREE (
QTREE_PATH ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_QTREEPATH is 
'An index on QTree Path for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_DISPLAYNAME                             */
/*==============================================================*/
create index I_NETAPPQTREE_DISPLAYNAME on SAHARA.NETAPP_QTREE (
DISPLAY_NAME ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_DISCOVERYSTATE                          */
/*==============================================================*/
create index I_NETAPPQTREE_DISCOVERYSTATE on SAHARA.NETAPP_QTREE (
DISCOVERY_STATE ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_SECURITYSTYLE                           */
/*==============================================================*/
create index I_NETAPPQTREE_SECURITYSTYLE on SAHARA.NETAPP_QTREE (
SECURITY_STYLE ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_SECURITYSTYLE is 
'An index on Security Style for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_FILESYSTEMID                            */
/*==============================================================*/
create index I_NETAPPQTREE_FILESYSTEMID on SAHARA.NETAPP_QTREE (
FILESYSTEM_ID ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_FILESYSTEMID is 
'An index on Filesystem ID for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_ISQUOTAENABLED                          */
/*==============================================================*/
create index I_NETAPPQTREE_ISQUOTAENABLED on SAHARA.NETAPP_QTREE (
IS_QUOTA_ENABLED ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_ISQUOTAENABLED is 
'An index on Is Quota Enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_T2SCRRNTNTPPQTRK                        */
/*==============================================================*/
create index I_NETAPPQTREE_T2SCRRNTNTPPQTRK on SAHARA.NETAPP_QTREE (
T2_IS_CURRENT ASC,
NETAPP_QTREE_KEY ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_T2SCRRNTNTPPQTRK is 
'An index on Type 2 - Is Current and NetApp QTree Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_T2SCRRNTT2SDLTDD                        */
/*==============================================================*/
create index I_NETAPPQTREE_T2SCRRNTT2SDLTDD on SAHARA.NETAPP_QTREE (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
NETAPP_QTREE_OID ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and NetApp QTree Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_NETAPPQTREE_T2ISDELETED                             */
/*==============================================================*/
create index I_NETAPPQTREE_T2ISDELETED on SAHARA.NETAPP_QTREE (
T2_IS_DELETED ASC
);

comment on index SAHARA.NETAPP_QTREE.I_NETAPPQTREE_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

grant SELECT on SAHARA.NETAPP_QTREE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_QTREE_                                         */
/*==============================================================*/
create table SAHARA_APP.NETAPP_QTREE_ 
(
   NETAPP_QTREE_OID     varchar(512)                   not null,
   DM_ID                numeric(10)                    not null,
   ARRAY_OID            varchar(512)                   not null,
   NETAPP_VOLUME_OID    varchar(512)                   not null,
   QTREE_PATH           varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DISCOVERY_STATE      varchar(32)                    null,
   SECURITY_STYLE       varchar(32)                    null,
   FILESYSTEM_ID        numeric(9)                     not null,
   IS_QUOTA_ENABLED     char(1)                        not null
);

comment on table SAHARA_APP.NETAPP_QTREE_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_QTREE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for NetApp qtrees.  It defines the attributes of NetApp qtrees that may be used to qualify queries.';

comment on column SAHARA_APP.NETAPP_QTREE_.NETAPP_QTREE_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_.NETAPP_VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_.QTREE_PATH is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE, column QTREE_PATH.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The complete path of the qtree on the volume.';

comment on column SAHARA_APP.NETAPP_QTREE_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The NetApp qtree display name.';

comment on column SAHARA_APP.NETAPP_QTREE_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the array:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the NetApp qtree was initially discovered by two or more explorers and then one of them stopped discovering the qtree.

Note: A NetApp qtree that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.NETAPP_QTREE_.SECURITY_STYLE is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE, column SECURITY_STYLE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The security style of the qtree.  Possible values include: UNIX, NTFS, and mixed.';

comment on column SAHARA_APP.NETAPP_QTREE_.FILESYSTEM_ID is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE, column FILESYSTEM_ID.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The identifier of the qtree (unique within the volume).  This is zero (0) if the qtree is the volume itself.';

comment on column SAHARA_APP.NETAPP_QTREE_.IS_QUOTA_ENABLED is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE, column IS_QUOTA_ENABLED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean value that indicates if a quota is created for the qtree.';

alter table SAHARA_APP.NETAPP_QTREE_
   alter DM_ID add constraint C_NETAPPQTREE_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_QTREE_
   alter FILESYSTEM_ID add constraint C_NETAPPQTREE_FILESYSTEMID check (FILESYSTEM_ID >= 0);

alter table SAHARA_APP.NETAPP_QTREE_
   alter IS_QUOTA_ENABLED add constraint C_NETAPPQTREE_ISQUOTAENABLED check (IS_QUOTA_ENABLED in ('T','F'));

alter table SAHARA_APP.NETAPP_QTREE_
   add constraint PK_NETAPP_QTREE_ primary key (NETAPP_QTREE_OID);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.NETAPP_QTREE_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_QTREE_BVIEW_NODE                               */
/*==============================================================*/
create table SAHARA.NETAPP_QTREE_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   NETAPP_QTREE_KEY     numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.NETAPP_QTREE_BVIEW_NODE is 
'This is a join table between NetApp qtree objects and business view nodes.';

comment on column SAHARA.NETAPP_QTREE_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.NETAPP_QTREE_BVIEW_NODE.NETAPP_QTREE_KEY is 
'The unique identifier of the NetApp volume object with which the business view node is associated.';

comment on column SAHARA.NETAPP_QTREE_BVIEW_NODE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the join table record.';

alter table SAHARA.NETAPP_QTREE_BVIEW_NODE
   add constraint PK_NETAPP_QTREE_BVIEW_NODE primary key (BVIEW_NODE_KEY, NETAPP_QTREE_KEY);

/*==============================================================*/
/* Index: I_NAQTBVIEWNODE_BVIEWNAQT                             */
/*==============================================================*/
create unique index I_NAQTBVIEWNODE_BVIEWNAQT on SAHARA.NETAPP_QTREE_BVIEW_NODE (
BVIEW_KEY ASC,
NETAPP_QTREE_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.NETAPP_QTREE_BVIEW_NODE.I_NAQTBVIEWNODE_BVIEWNAQT is 
'This index exists to accelerate queries for which NetApp qtrees are associated with a particular business view.';

grant SELECT on SAHARA.NETAPP_QTREE_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_QTREE_FILE                                     */
/*==============================================================*/
create table SAHARA.NETAPP_QTREE_FILE 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NETAPP_QTREE_KEY     numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   QUOTA_LIMIT          numeric(22,9)                  not null default 0,
   QUOTA_USED           numeric(22,9)                  not null default 0,
   DM_BATCH_KEY         numeric(10)                    null,
   DM_DATA_AGE          integer                        null,
   TOTAL_CONSUMPTION    numeric(22,9)                  null,
   FILES                numeric(10)                    null,
   FOLDERS              numeric(10)                    null,
   USERS                numeric(10)                    null
);

comment on table SAHARA.NETAPP_QTREE_FILE is 
'This is the fact table for NetApp qtree measures.  Each rows represents one sampling of values for a qtree.';

comment on column SAHARA.NETAPP_QTREE_FILE.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NETAPP_QTREE_FILE.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NETAPP_QTREE_FILE.NETAPP_QTREE_KEY is 
'The surrogate key from the NetApp volume dimension that identifies the volume for which the volume files data was collected.  This key may be used in joins to the NETAPP_VOLUME table.';

comment on column SAHARA.NETAPP_QTREE_FILE.CAPACITY is 
'The total logical capacity, in gigabytes, of the parent volume of the qtree.';

comment on column SAHARA.NETAPP_QTREE_FILE.QUOTA_LIMIT is 
'Maximum amount of disk space, in gigabytes, allowed for the quota of the parent qtree.  (hard disk space limit)';

comment on column SAHARA.NETAPP_QTREE_FILE.QUOTA_USED is 
'Current amount of disk space, in gigabytes, used by the quota of the parent qtree.';

comment on column SAHARA.NETAPP_QTREE_FILE.DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_QTREE_FILE.DM_DATA_AGE is 
'The age of the DM data, in seconds,  relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_QTREE_FILE.TOTAL_CONSUMPTION is 
'The aggregate of the size of all files on the qtree, in gigabytes.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_QTREE_FILE.FILES is 
'The number of files on the qtree.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_QTREE_FILE.FOLDERS is 
'The number of directories on the qtree.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_QTREE_FILE.USERS is 
'The number of users that have files on the qtree.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

alter table SAHARA.NETAPP_QTREE_FILE
   alter CAPACITY add constraint C_NTPPQTRFL_CAPACITY check (CAPACITY >= 0);

alter table SAHARA.NETAPP_QTREE_FILE
   alter QUOTA_LIMIT add constraint C_NTPPQTRFL_QUOTALIMIT check (QUOTA_LIMIT >= 0);

alter table SAHARA.NETAPP_QTREE_FILE
   alter QUOTA_USED add constraint C_NTPPQTRFL_QUOTAUSED check (QUOTA_USED >= 0);

alter table SAHARA.NETAPP_QTREE_FILE
   alter TOTAL_CONSUMPTION add constraint C_NTPPQTRFL_TOTALCONSUMPTION check (TOTAL_CONSUMPTION is null or (TOTAL_CONSUMPTION >= 0));

alter table SAHARA.NETAPP_QTREE_FILE
   alter FILES add constraint C_NTPPQTRFL_FILES check (FILES is null or (FILES >= 0));

alter table SAHARA.NETAPP_QTREE_FILE
   alter FOLDERS add constraint C_NTPPQTRFL_FOLDERS check (FOLDERS is null or (FOLDERS >= 0));

alter table SAHARA.NETAPP_QTREE_FILE
   alter USERS add constraint C_NTPPQTRFL_USERS check (USERS is null or (USERS >= 0));

/*==============================================================*/
/* Index: I_NAQTFILE_ROLLUPBATCHKEY                             */
/*==============================================================*/
create index I_NAQTFILE_ROLLUPBATCHKEY on SAHARA.NETAPP_QTREE_FILE (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_NAQTFILE_DATESKEY                                   */
/*==============================================================*/
create index I_NAQTFILE_DATESKEY on SAHARA.NETAPP_QTREE_FILE (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_NAQTFILE_NAVOLKEY                                   */
/*==============================================================*/
create index I_NAQTFILE_NAVOLKEY on SAHARA.NETAPP_QTREE_FILE (
NETAPP_QTREE_KEY ASC
);

grant SELECT on SAHARA.NETAPP_QTREE_FILE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_QTREE_FILE_DM_                                 */
/*==============================================================*/
create table SAHARA_APP.NETAPP_QTREE_FILE_DM_ 
(
   DM_ID                numeric(10)                    not null,
   DM_BATCH_ID          numeric(10)                    not null,
   TOTAL_CONSUMPTION    numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0,
   FOLDERS              numeric(10)                    not null default 0,
   USERS                numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_QTREE_FILE_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_QTREE_FILE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp qtree measures.  Each rows represents one sampling of values for a qtree.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_DM_.TOTAL_CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE, column TOTAL_CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all files on the qtree, in gigabytes.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_DM_.FILES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE, column FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files on the qtree.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_DM_.FOLDERS is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE, column FOLDERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of directories on the qtree.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_DM_.USERS is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE, column USERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of users that have files on the qtree.

This is value is only available for only if the NetApp file and the qtree was included in a DM rule.  It will be null otherwise.';

alter table SAHARA_APP.NETAPP_QTREE_FILE_DM_
   alter DM_ID add constraint C_NTPPQTRFLDM_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_DM_
   alter DM_BATCH_ID add constraint C_NTPPQTRFLDM_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.NETAPP_QTREE_FILE_DM_
   alter TOTAL_CONSUMPTION add constraint C_NTPPQTRFLDM_TOTALCONSUMPTION check (TOTAL_CONSUMPTION >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_DM_
   alter FILES add constraint C_NTPPQTRFLDM_FILES check (FILES >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_DM_
   alter FOLDERS add constraint C_NTPPQTRFLDM_FOLDERS check (FOLDERS >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_DM_
   alter USERS add constraint C_NTPPQTRFLDM_USERS check (USERS >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_DM_
   add constraint PK_NETAPP_QTREE_FILE_DM_ primary key (DM_ID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_QTREE_FILE_DM_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_QTREE_FILE_FCAT                                */
/*==============================================================*/
create table SAHARA.NETAPP_QTREE_FILE_FCAT 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DM_BATCH_KEY         numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NETAPP_QTREE_KEY     numeric(10)                    not null,
   DM_DATA_AGE          integer                        not null,
   FILE_CATEGORY_KEY    numeric(10)                    not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA.NETAPP_QTREE_FILE_FCAT is 
'This is the fact table for NetApp qtree files organized by file category.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.NETAPP_QTREE_KEY is 
'The surrogate key from the array dimension that identifies the array for which the array storage data was collected.  This key may be used in joins to the ARRAY table.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.DM_DATA_AGE is 
'The age of the DM data, in seconds,  relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.FILE_CATEGORY_KEY is 
'The surrogate key from the file category dimension that identifies the file category for which the file data was collected.  This key may be used in joins to the FILE_STORAGE table.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.CONSUMPTION is 
'The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA.NETAPP_QTREE_FILE_FCAT.FILES is 
'The number of files of the associated category on the device.';

alter table SAHARA.NETAPP_QTREE_FILE_FCAT
   alter CONSUMPTION add constraint C_NTPPQTRFLFCT_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA.NETAPP_QTREE_FILE_FCAT
   alter FILES add constraint C_NTPPQTRFLFCT_FILES check (FILES >= 0);

/*==============================================================*/
/* Index: I_NAQTFILEFCAT_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_NAQTFILEFCAT_ROLLUPBATCHKEY on SAHARA.NETAPP_QTREE_FILE_FCAT (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_NAQTFILEFCAT_DATESKEY                               */
/*==============================================================*/
create index I_NAQTFILEFCAT_DATESKEY on SAHARA.NETAPP_QTREE_FILE_FCAT (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_NAQTFILEFCAT_NAQTKEY                                */
/*==============================================================*/
create index I_NAQTFILEFCAT_NAQTKEY on SAHARA.NETAPP_QTREE_FILE_FCAT (
NETAPP_QTREE_KEY ASC
);

/*==============================================================*/
/* Index: I_NAQFILEFCAT_FILECATEGORYKEY                         */
/*==============================================================*/
create index I_NAQFILEFCAT_FILECATEGORYKEY on SAHARA.NETAPP_QTREE_FILE_FCAT (
FILE_CATEGORY_KEY ASC
);

grant SELECT on SAHARA.NETAPP_QTREE_FILE_FCAT to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_QTREE_FILE_FCAT_DM_                            */
/*==============================================================*/
create table SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_ 
(
   DM_ID                numeric(10)                    not null,
   FILE_CATEGORY_OID    varchar(512)                   not null,
   DM_BATCH_ID          numeric(10)                    not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_QTREE_FILE_FCAT.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp qtree files organized by file category.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_.FILE_CATEGORY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_.CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE_FCAT, column CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_.FILES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE_FCAT, column FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files of the associated category on the device.';

alter table SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_
   alter DM_ID add constraint C_NTPPQTRFLFCTD_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_
   alter DM_BATCH_ID add constraint C_NTPPQTRFLFCTD_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_
   alter CONSUMPTION add constraint C_NTPPQTRFLFCTD_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_
   alter FILES add constraint C_NTPPQTRFLFCTD_FILES check (FILES >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_
   add constraint PK_NETAPP_QTREE_FILE_FCAT_DM_ primary key (DM_ID, FILE_CATEGORY_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_QTREE_FILE_HAL_                                */
/*==============================================================*/
create table SAHARA_APP.NETAPP_QTREE_FILE_HAL_ 
(
   NETAPP_QTREE_OID     varchar(512)                   not null,
   DM_ID                numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   QUOTA_LIMIT          numeric(22,9)                  not null default 0,
   QUOTA_USED           numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.NETAPP_QTREE_FILE_HAL_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_QTREE_FILE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp qtree measures.  Each rows represents one sampling of values for a qtree.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_HAL_.NETAPP_QTREE_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_HAL_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_HAL_.CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE, column CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The total logical capacity, in gigabytes, of the parent volume of the qtree.';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_HAL_.QUOTA_LIMIT is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE, column QUOTA_LIMIT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Maximum amount of disk space, in gigabytes, allowed for the quota of the parent qtree.  (hard disk space limit)';

comment on column SAHARA_APP.NETAPP_QTREE_FILE_HAL_.QUOTA_USED is 
'This column is for staging data that will end up in table SAHARA.NETAPP_QTREE_FILE, column QUOTA_USED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Current amount of disk space, in gigabytes, used by the quota of the parent qtree.';

alter table SAHARA_APP.NETAPP_QTREE_FILE_HAL_
   alter DM_ID add constraint C_NTPPQTRFLHL_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_HAL_
   alter CAPACITY add constraint C_NTPPQTRFLHL_CAPACITY check (CAPACITY >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_HAL_
   alter QUOTA_LIMIT add constraint C_NTPPQTRFLHL_QUOTALIMIT check (QUOTA_LIMIT >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_HAL_
   alter QUOTA_USED add constraint C_NTPPQTRFLHL_QUOTAUSED check (QUOTA_USED >= 0);

alter table SAHARA_APP.NETAPP_QTREE_FILE_HAL_
   add constraint PK_NETAPP_QTREE_FILE_HAL_ primary key (NETAPP_QTREE_OID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_QTREE_FILE_HAL_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_SHARE                                          */
/*==============================================================*/
create table SAHARA.NETAPP_SHARE 
(
   NETAPP_SHARE_KEY     numeric(10)                    not null default autoincrement,
   NETAPP_SHARE_OID     varchar(512)                   not null,
   ARRAY_KEY            numeric(10)                    not null,
   NETAPP_VOLUME_KEY    numeric(10)                    null,
   NETAPP_QTREE_KEY     numeric(10)                    null,
   NAS_FILESYSTEM_KEY   numeric(10)                    null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   SHARE_PATH           varchar(1024)                  null,
   NFS_EXPORT_PATH      varchar(1024)                  null,
   DISPLAY_NAME         varchar(256)                   not null,
   CIFS_MOUNT_POINT     varchar(256)                   null,
   CIFS_SHARE_NAME      varchar(256)                   null,
   MOUNT_POINT_TYPE     varchar(32)                    null,
   DISCOVERY_STATE      varchar(32)                    null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   TIER                 varchar(64)                    null,
   ESX_DATA_STORE_NAMES varchar(1024)                  null,
   HOST_PRIVILEGE       varchar(10240)                 null,
   IS_MAPPED_TO_ANY_UNDISCOV_HOST char(1)                        not null default 'F'
);

comment on table SAHARA.NETAPP_SHARE is 
'This is the dimension table for NetApp shares.  It defines the attributes of NetApp shares that may be used to qualify queries.';

comment on column SAHARA.NETAPP_SHARE.NETAPP_SHARE_KEY is 
'A unique database generated identifier for the NetApp share record used as a surrogate key.';

comment on column SAHARA.NETAPP_SHARE.NETAPP_SHARE_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.NETAPP_SHARE.ARRAY_KEY is 
'The unique identifier of the parent NetApp filer array object.';

comment on column SAHARA.NETAPP_SHARE.NETAPP_VOLUME_KEY is 
'The unique identifier of the parent NetApp volume.  Either the NetApp volume key or the NetApp qtree key will be populate, but the other may be null.';

comment on column SAHARA.NETAPP_SHARE.NETAPP_QTREE_KEY is 
'The unique identifier of the parent NetApp qtree.  Either the NetApp volume key or the NetApp qtree key will be populate, but the other may be null.';

comment on column SAHARA.NETAPP_SHARE.NAS_FILESYSTEM_KEY is 
'The unique identifier of the parent NetApp qtree.  Either the NetApp volume key or the NetApp qtree key will be populate, but the other may be null.';

comment on column SAHARA.NETAPP_SHARE.CC_STORAGE_URL is 
'The URL for the NetApp share object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.NETAPP_SHARE.SHARE_PATH is 
'The complete path of the share on the NetApp volume.';

comment on column SAHARA.NETAPP_SHARE.NFS_EXPORT_PATH is 
'The path of the NFS export.';

comment on column SAHARA.NETAPP_SHARE.DISPLAY_NAME is 
'The NetApp share''s name as displayed in CommandCentral storage.';

comment on column SAHARA.NETAPP_SHARE.CIFS_MOUNT_POINT is 
'The path of the CIFS share.';

comment on column SAHARA.NETAPP_SHARE.CIFS_SHARE_NAME is 
'The name of the CIFS share.';

comment on column SAHARA.NETAPP_SHARE.MOUNT_POINT_TYPE is 
'The type of share/export.  e.g. CIFS/NFS.';

comment on column SAHARA.NETAPP_SHARE.DISCOVERY_STATE is 
'The discovery state of the NetApp share:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the share was initially discovered by two or more explorers and then one of them stopped discovering the share.

Note: A share that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.NETAPP_SHARE.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.NETAPP_SHARE.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.NETAPP_SHARE.HOST_PRIVILEGE is 
'Comma separated list of array side discovered hosts and their privileges';

comment on column SAHARA.NETAPP_SHARE.IS_MAPPED_TO_ANY_UNDISCOV_HOST is 
'True: if any associated array side discovered host is not correlated';

alter table SAHARA.NETAPP_SHARE
   alter T2_IS_CURRENT add constraint C_NETAPPSHARE_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.NETAPP_SHARE
   alter T2_IS_DELETED add constraint C_NETAPPSHARE_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.NETAPP_SHARE
   alter IS_MAPPED_TO_ANY_UNDISCOV_HOST add constraint CKC_IS_MAPPED_TO_ANY__NETAPP_S check (IS_MAPPED_TO_ANY_UNDISCOV_HOST in ('T','F'));

alter table SAHARA.NETAPP_SHARE
   add constraint PK_NETAPP_SHARE primary key (NETAPP_SHARE_KEY);

/*==============================================================*/
/* Index: I_NETAPPSHARE_NETAPPSHAREOID                          */
/*==============================================================*/
create index I_NETAPPSHARE_NETAPPSHAREOID on SAHARA.NETAPP_SHARE (
NETAPP_SHARE_OID ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_NETAPPSHAREOID is 
'An index on NetApp Share Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_ARRAYKEY                                */
/*==============================================================*/
create index I_NETAPPSHARE_ARRAYKEY on SAHARA.NETAPP_SHARE (
ARRAY_KEY ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_NETAPPVOLUMEKEY                         */
/*==============================================================*/
create index I_NETAPPSHARE_NETAPPVOLUMEKEY on SAHARA.NETAPP_SHARE (
NETAPP_VOLUME_KEY ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_NETAPPVOLUMEKEY is 
'An index on NetApp Volume Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_NETAPPQTREEKEY                          */
/*==============================================================*/
create index I_NETAPPSHARE_NETAPPQTREEKEY on SAHARA.NETAPP_SHARE (
NETAPP_QTREE_KEY ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_NETAPPQTREEKEY is 
'An index on NetApp QTree Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_NASFILESYSTEMKEY                        */
/*==============================================================*/
create index I_NETAPPSHARE_NASFILESYSTEMKEY on SAHARA.NETAPP_SHARE (
NAS_FILESYSTEM_KEY ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_NASFILESYSTEMKEY is 
'An index on NAS Filesystem Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_SHAREPATH                               */
/*==============================================================*/
create index I_NETAPPSHARE_SHAREPATH on SAHARA.NETAPP_SHARE (
SHARE_PATH ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_SHAREPATH is 
'An index on Share Path for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_NFSEXPORTPATH                           */
/*==============================================================*/
create index I_NETAPPSHARE_NFSEXPORTPATH on SAHARA.NETAPP_SHARE (
NFS_EXPORT_PATH ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_NFSEXPORTPATH is 
'An index on NFS Export Path for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_DISPLAYNAME                             */
/*==============================================================*/
create index I_NETAPPSHARE_DISPLAYNAME on SAHARA.NETAPP_SHARE (
DISPLAY_NAME ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_CIFSMOUNTPOINT                          */
/*==============================================================*/
create index I_NETAPPSHARE_CIFSMOUNTPOINT on SAHARA.NETAPP_SHARE (
CIFS_MOUNT_POINT ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_CIFSMOUNTPOINT is 
'An index on CIFS Mount Point for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_CIFSSHARENAME                           */
/*==============================================================*/
create index I_NETAPPSHARE_CIFSSHARENAME on SAHARA.NETAPP_SHARE (
CIFS_SHARE_NAME ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_CIFSSHARENAME is 
'An index on CIFS Share Name for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_MOUNTPOINTTYPE                          */
/*==============================================================*/
create index I_NETAPPSHARE_MOUNTPOINTTYPE on SAHARA.NETAPP_SHARE (
MOUNT_POINT_TYPE ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_MOUNTPOINTTYPE is 
'An index on Mount Point Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_DISCOVERYSTATE                          */
/*==============================================================*/
create index I_NETAPPSHARE_DISCOVERYSTATE on SAHARA.NETAPP_SHARE (
DISCOVERY_STATE ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_T2SCRRNTNTPPSHRK                        */
/*==============================================================*/
create index I_NETAPPSHARE_T2SCRRNTNTPPSHRK on SAHARA.NETAPP_SHARE (
T2_IS_CURRENT ASC,
NETAPP_SHARE_KEY ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_T2SCRRNTNTPPSHRK is 
'An index on Type 2 - Is Current and NetApp Share Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_T2SCRRNTT2SDLTDD                        */
/*==============================================================*/
create index I_NETAPPSHARE_T2SCRRNTT2SDLTDD on SAHARA.NETAPP_SHARE (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
NETAPP_SHARE_OID ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and NetApp Share Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_T2ISDELETED                             */
/*==============================================================*/
create index I_NETAPPSHARE_T2ISDELETED on SAHARA.NETAPP_SHARE (
T2_IS_DELETED ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_TIER                                    */
/*==============================================================*/
create index I_NETAPPSHARE_TIER on SAHARA.NETAPP_SHARE (
TIER ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_TIER is 
'An index on Tier for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_SXDTSTRNMS                              */
/*==============================================================*/
create index I_NETAPPSHARE_SXDTSTRNMS on SAHARA.NETAPP_SHARE (
ESX_DATA_STORE_NAMES ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_SXDTSTRNMS is 
'An index on ESX Data Store names for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_HOSTPRIVILEGE                           */
/*==============================================================*/
create index I_NETAPPSHARE_HOSTPRIVILEGE on SAHARA.NETAPP_SHARE (
HOST_PRIVILEGE ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_HOSTPRIVILEGE is 
'An index on Host Privilege for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_SMPPDTNNDSCVHST                         */
/*==============================================================*/
create index I_NETAPPSHARE_SMPPDTNNDSCVHST on SAHARA.NETAPP_SHARE (
IS_MAPPED_TO_ANY_UNDISCOV_HOST ASC
);

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_SMPPDTNNDSCVHST is 
'An index on Is Mapped to Any Undiscov Host for dimensional queries.';

grant SELECT on SAHARA.NETAPP_SHARE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_SHARE_                                         */
/*==============================================================*/
create table SAHARA_APP.NETAPP_SHARE_ 
(
   NETAPP_SHARE_OID     varchar(512)                   not null,
   DM_ID                varchar(1024)                  not null,
   ARRAY_OID            varchar(512)                   not null,
   NETAPP_VOLUME_OID    varchar(512)                   null,
   NETAPP_QTREE_OID     varchar(512)                   null,
   NAS_FILESYSTEM_OID   varchar(512)                   null,
   SHARE_PATH           varchar(1024)                  null,
   NFS_EXPORT_PATH      varchar(1024)                  null,
   DISPLAY_NAME         varchar(256)                   not null,
   CIFS_MOUNT_POINT     varchar(256)                   null,
   CIFS_SHARE_NAME      varchar(256)                   null,
   MOUNT_POINT_TYPE     varchar(32)                    null,
   DISCOVERY_STATE      varchar(32)                    null,
   TIER                 varchar(64)                    null,
   ESX_DATA_STORE_NAMES varchar(1024)                  null,
   HOST_PRIVILEGE       varchar(10240)                 null,
   IS_MAPPED_TO_ANY_UNDISCOV_HOST char(1)                        not null default 'F'
);

comment on table SAHARA_APP.NETAPP_SHARE_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_SHARE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for NetApp shares.  It defines the attributes of NetApp shares that may be used to qualify queries.';

comment on column SAHARA_APP.NETAPP_SHARE_.NETAPP_SHARE_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_.NETAPP_VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_.NETAPP_QTREE_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_.NAS_FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_.SHARE_PATH is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE, column SHARE_PATH.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The complete path of the share on the NetApp volume.';

comment on column SAHARA_APP.NETAPP_SHARE_.NFS_EXPORT_PATH is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE, column NFS_EXPORT_PATH.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The path of the NFS export.';

comment on column SAHARA_APP.NETAPP_SHARE_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The NetApp share''s name as displayed in CommandCentral storage.';

comment on column SAHARA_APP.NETAPP_SHARE_.CIFS_MOUNT_POINT is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE, column CIFS_MOUNT_POINT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The path of the CIFS share.';

comment on column SAHARA_APP.NETAPP_SHARE_.CIFS_SHARE_NAME is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE, column CIFS_SHARE_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The name of the CIFS share.';

comment on column SAHARA_APP.NETAPP_SHARE_.MOUNT_POINT_TYPE is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE, column MOUNT_POINT_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The type of share/export.  e.g. CIFS/NFS.';

comment on column SAHARA_APP.NETAPP_SHARE_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the NetApp share:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the share was initially discovered by two or more explorers and then one of them stopped discovering the share.

Note: A share that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.NETAPP_SHARE_.HOST_PRIVILEGE is 
'Comma separated list of array side discovered hosts and their privileges';

comment on column SAHARA_APP.NETAPP_SHARE_.IS_MAPPED_TO_ANY_UNDISCOV_HOST is 
'True: if any associated array side discovered host is not correlated';

alter table SAHARA_APP.NETAPP_SHARE_
   alter IS_MAPPED_TO_ANY_UNDISCOV_HOST add constraint CKC_IS_MAPPED_TO_ANY__NETAPP_S check (IS_MAPPED_TO_ANY_UNDISCOV_HOST in ('T','F'));

alter table SAHARA_APP.NETAPP_SHARE_
   add constraint PK_NETAPP_SHARE_ primary key (NETAPP_SHARE_OID);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.NETAPP_SHARE_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_SHARE_BVIEW_NODE                               */
/*==============================================================*/
create table SAHARA.NETAPP_SHARE_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   NETAPP_SHARE_KEY     numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.NETAPP_SHARE_BVIEW_NODE is 
'This is a join table between NetApp share objects and business view nodes.';

comment on column SAHARA.NETAPP_SHARE_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.NETAPP_SHARE_BVIEW_NODE.NETAPP_SHARE_KEY is 
'The unique identifier of the NetApp volume object with which the business view node is associated.';

comment on column SAHARA.NETAPP_SHARE_BVIEW_NODE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the join table record.';

alter table SAHARA.NETAPP_SHARE_BVIEW_NODE
   add constraint PK_NETAPP_SHARE_BVIEW_NODE primary key (BVIEW_NODE_KEY, NETAPP_SHARE_KEY);

/*==============================================================*/
/* Index: I_NASHARBVIEWNODE_BVIEWNASHAR                         */
/*==============================================================*/
create unique index I_NASHARBVIEWNODE_BVIEWNASHAR on SAHARA.NETAPP_SHARE_BVIEW_NODE (
BVIEW_KEY ASC,
NETAPP_SHARE_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.NETAPP_SHARE_BVIEW_NODE.I_NASHARBVIEWNODE_BVIEWNASHAR is 
'This index exists to accelerate queries for which NetApp shares are associated with a particular business view.';

grant SELECT on SAHARA.NETAPP_SHARE_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_SHARE_FILE                                     */
/*==============================================================*/
create table SAHARA.NETAPP_SHARE_FILE 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DM_BATCH_KEY         numeric(10)                    null,
   DATES_KEY            numeric(10)                    not null,
   NETAPP_SHARE_KEY     numeric(10)                    not null,
   DM_DATA_AGE          integer                        not null,
   TOTAL_CONSUMPTION    numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0,
   FOLDERS              numeric(10)                    not null default 0,
   USERS                numeric(10)                    not null default 0,
   NO_OF_HOSTS          numeric(10)                    null
);

comment on table SAHARA.NETAPP_SHARE_FILE is 
'This is the fact table for NetApp share measures.  Each rows represents one sampling of values for a volume.';

comment on column SAHARA.NETAPP_SHARE_FILE.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NETAPP_SHARE_FILE.DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.';

comment on column SAHARA.NETAPP_SHARE_FILE.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NETAPP_SHARE_FILE.NETAPP_SHARE_KEY is 
'The surrogate key from the NetApp volume dimension that identifies the volume for which the volume files data was collected.  This key may be used in joins to the NETAPP_VOLUME table.';

comment on column SAHARA.NETAPP_SHARE_FILE.DM_DATA_AGE is 
'The age of the DM data, in seconds, relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.';

comment on column SAHARA.NETAPP_SHARE_FILE.TOTAL_CONSUMPTION is 
'The aggregate of the size of all files on the share, in gigabytes.';

comment on column SAHARA.NETAPP_SHARE_FILE.FILES is 
'The number of files on the share.';

comment on column SAHARA.NETAPP_SHARE_FILE.FOLDERS is 
'The number of directories on the share.';

comment on column SAHARA.NETAPP_SHARE_FILE.USERS is 
'The number of users that have files on the share.';

alter table SAHARA.NETAPP_SHARE_FILE
   alter TOTAL_CONSUMPTION add constraint C_NTPPSHRFL_TOTALCONSUMPTION check (TOTAL_CONSUMPTION >= 0);

alter table SAHARA.NETAPP_SHARE_FILE
   alter FILES add constraint C_NTPPSHRFL_FILES check (FILES >= 0);

alter table SAHARA.NETAPP_SHARE_FILE
   alter FOLDERS add constraint C_NTPPSHRFL_FOLDERS check (FOLDERS >= 0);

alter table SAHARA.NETAPP_SHARE_FILE
   alter USERS add constraint C_NTPPSHRFL_USERS check (USERS >= 0);

/*==============================================================*/
/* Index: I_NASHAREFILE_ROLLUPBATCHKEY                          */
/*==============================================================*/
create index I_NASHAREFILE_ROLLUPBATCHKEY on SAHARA.NETAPP_SHARE_FILE (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_NASHAREFILE_DATESKEY                                */
/*==============================================================*/
create index I_NASHAREFILE_DATESKEY on SAHARA.NETAPP_SHARE_FILE (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_NASHAREFILE_NASHAREKEY                              */
/*==============================================================*/
create index I_NASHAREFILE_NASHAREKEY on SAHARA.NETAPP_SHARE_FILE (
NETAPP_SHARE_KEY ASC
);

grant SELECT on SAHARA.NETAPP_SHARE_FILE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_SHARE_FILE_DM_                                 */
/*==============================================================*/
create table SAHARA_APP.NETAPP_SHARE_FILE_DM_ 
(
   DM_ID                varchar(1024)                  not null,
   DM_BATCH_ID          numeric(10)                    not null,
   TOTAL_CONSUMPTION    numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0,
   FOLDERS              numeric(10)                    not null default 0,
   USERS                numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_SHARE_FILE_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_SHARE_FILE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp share measures.  Each rows represents one sampling of values for a volume.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_DM_.TOTAL_CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE_FILE, column TOTAL_CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all files on the share, in gigabytes.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_DM_.FILES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE_FILE, column FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files on the share.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_DM_.FOLDERS is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE_FILE, column FOLDERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of directories on the share.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_DM_.USERS is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE_FILE, column USERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of users that have files on the share.';

alter table SAHARA_APP.NETAPP_SHARE_FILE_DM_
   alter DM_BATCH_ID add constraint C_NTPPSHRFLDM_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.NETAPP_SHARE_FILE_DM_
   alter TOTAL_CONSUMPTION add constraint C_NTPPSHRFLDM_TOTALCONSUMPTION check (TOTAL_CONSUMPTION >= 0);

alter table SAHARA_APP.NETAPP_SHARE_FILE_DM_
   alter FILES add constraint C_NTPPSHRFLDM_FILES check (FILES >= 0);

alter table SAHARA_APP.NETAPP_SHARE_FILE_DM_
   alter FOLDERS add constraint C_NTPPSHRFLDM_FOLDERS check (FOLDERS >= 0);

alter table SAHARA_APP.NETAPP_SHARE_FILE_DM_
   alter USERS add constraint C_NTPPSHRFLDM_USERS check (USERS >= 0);

alter table SAHARA_APP.NETAPP_SHARE_FILE_DM_
   add constraint PK_NETAPP_SHARE_FILE_DM_ primary key (DM_ID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_SHARE_FILE_DM_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_SHARE_FILE_FCAT                                */
/*==============================================================*/
create table SAHARA.NETAPP_SHARE_FILE_FCAT 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DM_BATCH_KEY         numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NETAPP_SHARE_KEY     numeric(10)                    not null,
   FILE_CATEGORY_KEY    numeric(10)                    not null,
   DM_DATA_AGE          integer                        not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA.NETAPP_SHARE_FILE_FCAT is 
'This is the fact table for NetApp share files organized by file category.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.NETAPP_SHARE_KEY is 
'The surrogate key from the array dimension that identifies the array for which the array storage data was collected.  This key may be used in joins to the ARRAY table.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.FILE_CATEGORY_KEY is 
'The surrogate key from the file category dimension that identifies the file category for which the file data was collected.  This key may be used in joins to the FILE_STORAGE table.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.DM_DATA_AGE is 
'The age of the DM data, in seconds,  relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.CONSUMPTION is 
'The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA.NETAPP_SHARE_FILE_FCAT.FILES is 
'The number of files of the associated category on the device.';

alter table SAHARA.NETAPP_SHARE_FILE_FCAT
   alter CONSUMPTION add constraint C_NTPPSHRFLFCT_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA.NETAPP_SHARE_FILE_FCAT
   alter FILES add constraint C_NTPPSHRFLFCT_FILES check (FILES >= 0);

/*==============================================================*/
/* Index: I_NASHFILEFCAT_ROLLUPBATCHKEY                         */
/*==============================================================*/
create index I_NASHFILEFCAT_ROLLUPBATCHKEY on SAHARA.NETAPP_SHARE_FILE_FCAT (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_NASHFILEFCAT_DATESKEY                               */
/*==============================================================*/
create index I_NASHFILEFCAT_DATESKEY on SAHARA.NETAPP_SHARE_FILE_FCAT (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_NASHFILEFCAT_NASHAREKEY                             */
/*==============================================================*/
create index I_NASHFILEFCAT_NASHAREKEY on SAHARA.NETAPP_SHARE_FILE_FCAT (
NETAPP_SHARE_KEY ASC
);

/*==============================================================*/
/* Index: I_NASHFILEFCAT_FILECATEGORYKEY                        */
/*==============================================================*/
create index I_NASHFILEFCAT_FILECATEGORYKEY on SAHARA.NETAPP_SHARE_FILE_FCAT (
FILE_CATEGORY_KEY ASC
);

grant SELECT on SAHARA.NETAPP_SHARE_FILE_FCAT to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_SHARE_FILE_FCAT_DM_                            */
/*==============================================================*/
create table SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_ 
(
   DM_ID                varchar(1024)                  not null,
   FILE_CATEGORY_OID    varchar(512)                   not null,
   DM_BATCH_ID          numeric(10)                    not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_SHARE_FILE_FCAT.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp share files organized by file category.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_.FILE_CATEGORY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_.CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE_FILE_FCAT, column CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_.FILES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_SHARE_FILE_FCAT, column FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files of the associated category on the device.';

alter table SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_
   alter DM_BATCH_ID add constraint C_NTPPSHRFLFCTD_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_
   alter CONSUMPTION add constraint C_NTPPSHRFLFCTD_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_
   alter FILES add constraint C_NTPPSHRFLFCTD_FILES check (FILES >= 0);

alter table SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_
   add constraint PK_NETAPP_SHARE_FILE_FCAT_DM_ primary key (DM_ID, FILE_CATEGORY_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_SHARE_FILE_HAL_                                */
/*==============================================================*/
create table SAHARA_APP.NETAPP_SHARE_FILE_HAL_ 
(
   NETAPP_SHARE_OID     varchar(512)                   not null,
   DM_ID                varchar(1024)                  not null,
   NO_OF_HOSTS          numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_SHARE_FILE_HAL_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_SHARE_FILE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp share measures.  Each rows represents one sampling of values for a volume.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_HAL_.NETAPP_SHARE_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_SHARE_FILE_HAL_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

alter table SAHARA_APP.NETAPP_SHARE_FILE_HAL_
   alter NO_OF_HOSTS add constraint CKC_NO_OF_HOSTS_NETAPP_S check (NO_OF_HOSTS >= 0);

alter table SAHARA_APP.NETAPP_SHARE_FILE_HAL_
   add constraint PK_NETAPP_SHARE_FILE_HAL_ primary key (NETAPP_SHARE_OID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_SHARE_FILE_HAL_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_VOLUME                                         */
/*==============================================================*/
create table SAHARA.NETAPP_VOLUME 
(
   NETAPP_VOLUME_KEY    numeric(10)                    not null default autoincrement,
   NETAPP_VOLUME_OID    varchar(512)                   not null,
   ARRAY_KEY            numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DISCOVERY_STATE      varchar(32)                    null,
   VENDOR_STATE         varchar(256)                   not null,
   VENDOR_LAYOUT        varchar(256)                   null,
   RAID_LEVEL           varchar(32)                    null,
   IS_AGGREGATE         char(1)                        not null,
   IS_SEGMENT           char(1)                        not null,
   IS_VFILER_VOLUME     char(1)                        not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   TYPE                 varchar(32)                    not null default ' ',
   SUB_TYPE             varchar(32)                    not null default ' ',
   TIER                 varchar(64)                    null,
   RPM                  varchar(32)                    null,
   DRIVE_TYPE           varchar(32)                    null,
   DISK_SIZE            numeric(22,9)                  not null default 0,
   IS_NAS_FS_CREATED    char(1)                        not null default 'F',
   DEDUP_STATE          varchar(32)                    not null default ' ',
   LAST_DEDUP_OP_BEGIN_TIME date                           null,
   LAST_DEDUP_OP_END_TIME date                           null,
   STORAGE_POOL_KEY     numeric(10)                    null,
   STORAGE_POOL_OBJECT_IDENTIFIER varchar(512)                   null
);

comment on table SAHARA.NETAPP_VOLUME is 
'This is the dimension table for NetApp volumes.  It defines the attributes of NetApp volumes that may be used to qualify queries.';

comment on column SAHARA.NETAPP_VOLUME.NETAPP_VOLUME_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NETAPP_VOLUME.NETAPP_VOLUME_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.NETAPP_VOLUME.ARRAY_KEY is 
'The unique identifier of the parent filer for the volume.';

comment on column SAHARA.NETAPP_VOLUME.CC_STORAGE_URL is 
'The URL for the NetApp volume object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.NETAPP_VOLUME.DISPLAY_NAME is 
'The NetApp volume display name.';

comment on column SAHARA.NETAPP_VOLUME.DISCOVERY_STATE is 
'The discovery state of the volume:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the NetApp volume was initially discovered by two or more explorers and then one of them stopped discovering the volume.

Note: A NetApp volume that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.NETAPP_VOLUME.VENDOR_STATE is 
'The vendor specific state of the volume.';

comment on column SAHARA.NETAPP_VOLUME.VENDOR_LAYOUT is 
'The vendor''s name for the storage layout used by the device.';

comment on column SAHARA.NETAPP_VOLUME.RAID_LEVEL is 
'The RAID level of the NetApp volume.

Possible values:
- RAID0
- RAID1
- RAID3
- RAID4
- RAID5
- RAID6
- RAID7
- RAID10
- RAID0+1
- RAID053
- Unknown.

This may be null.';

comment on column SAHARA.NETAPP_VOLUME.IS_AGGREGATE is 
'A boolean value that indicates if the volume is a combination of other volumes.  A volume can be both a segment and an aggregate if the volume is in a middle layer with an aggregate volume above and the segment volumes below.  In such a case, both Segment and Aggregate will be true.';

comment on column SAHARA.NETAPP_VOLUME.IS_SEGMENT is 
'A boolean value that indicates if the volume is flex volume.';

comment on column SAHARA.NETAPP_VOLUME.IS_VFILER_VOLUME is 
'A boolean that indicates if the volume is associated with a vfiler?';

comment on column SAHARA.NETAPP_VOLUME.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.NETAPP_VOLUME.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.NETAPP_VOLUME
   alter IS_AGGREGATE add constraint C_NETAPPVOLUME_ISAGGREGATE check (IS_AGGREGATE in ('T','F'));

alter table SAHARA.NETAPP_VOLUME
   alter IS_SEGMENT add constraint C_NETAPPVOLUME_ISSEGMENT check (IS_SEGMENT in ('T','F'));

alter table SAHARA.NETAPP_VOLUME
   alter IS_VFILER_VOLUME add constraint C_NETAPPVOLUME_ISVFILERVOLUME check (IS_VFILER_VOLUME in ('T','F'));

alter table SAHARA.NETAPP_VOLUME
   alter T2_IS_CURRENT add constraint C_NETAPPVOLUME_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.NETAPP_VOLUME
   alter T2_IS_DELETED add constraint C_NETAPPVOLUME_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.NETAPP_VOLUME
   alter DISK_SIZE add constraint CKC_DISK_SIZE_NETAPP_V check (DISK_SIZE >= 0);

alter table SAHARA.NETAPP_VOLUME
   alter IS_NAS_FS_CREATED add constraint CKC_IS_NAS_FS_CREATED_NETAPP_V check (IS_NAS_FS_CREATED in ('T','F'));

alter table SAHARA.NETAPP_VOLUME
   add constraint PK_NETAPP_VOLUME primary key (NETAPP_VOLUME_KEY);

/*==============================================================*/
/* Index: I_NETAPPVOLUME_NETAPPVOLUMEOID                        */
/*==============================================================*/
create index I_NETAPPVOLUME_NETAPPVOLUMEOID on SAHARA.NETAPP_VOLUME (
NETAPP_VOLUME_OID ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_NETAPPVOLUMEOID is 
'An index on NetApp Volume Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_ARRAYKEY                               */
/*==============================================================*/
create index I_NETAPPVOLUME_ARRAYKEY on SAHARA.NETAPP_VOLUME (
ARRAY_KEY ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DISPLAYNAME                            */
/*==============================================================*/
create index I_NETAPPVOLUME_DISPLAYNAME on SAHARA.NETAPP_VOLUME (
DISPLAY_NAME ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DISCOVERYSTATE                         */
/*==============================================================*/
create index I_NETAPPVOLUME_DISCOVERYSTATE on SAHARA.NETAPP_VOLUME (
DISCOVERY_STATE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_VENDORSTATE                            */
/*==============================================================*/
create index I_NETAPPVOLUME_VENDORSTATE on SAHARA.NETAPP_VOLUME (
VENDOR_STATE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_VENDORSTATE is 
'An index on Vendor State for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_VENDORLAYOUT                           */
/*==============================================================*/
create index I_NETAPPVOLUME_VENDORLAYOUT on SAHARA.NETAPP_VOLUME (
VENDOR_LAYOUT ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_VENDORLAYOUT is 
'An index on Vendor Layout for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_RAIDLEVEL                              */
/*==============================================================*/
create index I_NETAPPVOLUME_RAIDLEVEL on SAHARA.NETAPP_VOLUME (
RAID_LEVEL ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_RAIDLEVEL is 
'An index on RAID Level for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_ISAGGREGATE                            */
/*==============================================================*/
create index I_NETAPPVOLUME_ISAGGREGATE on SAHARA.NETAPP_VOLUME (
IS_AGGREGATE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_ISAGGREGATE is 
'An index on Is Aggregate for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_ISSEGMENT                              */
/*==============================================================*/
create index I_NETAPPVOLUME_ISSEGMENT on SAHARA.NETAPP_VOLUME (
IS_SEGMENT ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_ISSEGMENT is 
'An index on Is Segment for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_ISVFILERVOLUME                         */
/*==============================================================*/
create index I_NETAPPVOLUME_ISVFILERVOLUME on SAHARA.NETAPP_VOLUME (
IS_VFILER_VOLUME ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_ISVFILERVOLUME is 
'An index on Is VFiler Volume for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_T2SCRRNTNTPPVLM                        */
/*==============================================================*/
create index I_NETAPPVOLUME_T2SCRRNTNTPPVLM on SAHARA.NETAPP_VOLUME (
T2_IS_CURRENT ASC,
NETAPP_VOLUME_KEY ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_T2SCRRNTNTPPVLM is 
'An index on Type 2 - Is Current and NetApp Volume Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_T2SCRRNTT2SDLTD                        */
/*==============================================================*/
create index I_NETAPPVOLUME_T2SCRRNTT2SDLTD on SAHARA.NETAPP_VOLUME (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
NETAPP_VOLUME_OID ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_T2SCRRNTT2SDLTD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and NetApp Volume Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_T2ISDELETED                            */
/*==============================================================*/
create index I_NETAPPVOLUME_T2ISDELETED on SAHARA.NETAPP_VOLUME (
T2_IS_DELETED ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_TYPE                                   */
/*==============================================================*/
create index I_NETAPPVOLUME_TYPE on SAHARA.NETAPP_VOLUME (
TYPE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_SUBTYPE                                */
/*==============================================================*/
create index I_NETAPPVOLUME_SUBTYPE on SAHARA.NETAPP_VOLUME (
SUB_TYPE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_SUBTYPE is 
'An index on Sub Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_TIER                                   */
/*==============================================================*/
create index I_NETAPPVOLUME_TIER on SAHARA.NETAPP_VOLUME (
TIER ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_TIER is 
'An index on Tier for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_RPM                                    */
/*==============================================================*/
create index I_NETAPPVOLUME_RPM on SAHARA.NETAPP_VOLUME (
RPM ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_RPM is 
'An index on RPM for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DRIVETYPE                              */
/*==============================================================*/
create index I_NETAPPVOLUME_DRIVETYPE on SAHARA.NETAPP_VOLUME (
DRIVE_TYPE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DRIVETYPE is 
'An index on Drive Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DISKSIZE                               */
/*==============================================================*/
create index I_NETAPPVOLUME_DISKSIZE on SAHARA.NETAPP_VOLUME (
DISK_SIZE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DISKSIZE is 
'An index on Disk Size for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_ISNASFSCREATED                         */
/*==============================================================*/
create index I_NETAPPVOLUME_ISNASFSCREATED on SAHARA.NETAPP_VOLUME (
IS_NAS_FS_CREATED ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_ISNASFSCREATED is 
'An index on Is NAS FS Created for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DEDUPSTATE                             */
/*==============================================================*/
create index I_NETAPPVOLUME_DEDUPSTATE on SAHARA.NETAPP_VOLUME (
DEDUP_STATE ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DEDUPSTATE is 
'An index on Dedup State for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_LSTDDPPBGNTM                           */
/*==============================================================*/
create index I_NETAPPVOLUME_LSTDDPPBGNTM on SAHARA.NETAPP_VOLUME (
LAST_DEDUP_OP_BEGIN_TIME ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_LSTDDPPBGNTM is 
'An index on Last Dedup Op begin time for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_LSTDDPPNDTM                            */
/*==============================================================*/
create index I_NETAPPVOLUME_LSTDDPPNDTM on SAHARA.NETAPP_VOLUME (
LAST_DEDUP_OP_END_TIME ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_LSTDDPPNDTM is 
'An index on Last Dedup Op end time for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_STORAGEPOOLKEY                         */
/*==============================================================*/
create index I_NETAPPVOLUME_STORAGEPOOLKEY on SAHARA.NETAPP_VOLUME (
STORAGE_POOL_KEY ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_STORAGEPOOLKEY is 
'An index on Storage Pool Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_STRGPLBJCTDNTFR                        */
/*==============================================================*/
create index I_NETAPPVOLUME_STRGPLBJCTDNTFR on SAHARA.NETAPP_VOLUME (
STORAGE_POOL_OBJECT_IDENTIFIER ASC
);

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_STRGPLBJCTDNTFR is 
'An index on Storage Pool Object Identifier for dimensional queries.';

grant SELECT on SAHARA.NETAPP_VOLUME to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_VOLUME_                                        */
/*==============================================================*/
create table SAHARA_APP.NETAPP_VOLUME_ 
(
   NETAPP_VOLUME_OID    varchar(512)                   not null,
   DM_ID                numeric(10)                    not null,
   ARRAY_OID            varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   DISCOVERY_STATE      varchar(32)                    null,
   VENDOR_STATE         varchar(256)                   not null,
   VENDOR_LAYOUT        varchar(256)                   null,
   RAID_LEVEL           varchar(32)                    null,
   IS_AGGREGATE         char(1)                        not null,
   IS_SEGMENT           char(1)                        not null,
   IS_VFILER_VOLUME     char(1)                        not null,
   TYPE                 varchar(32)                    not null default ' ',
   SUB_TYPE             varchar(32)                    not null default ' ',
   TIER                 varchar(64)                    null,
   RPM                  varchar(32)                    null,
   DRIVE_TYPE           varchar(32)                    null,
   DISK_SIZE            numeric(22,9)                  not null default 0,
   IS_NAS_FS_CREATED    char(1)                        not null default 'F',
   DEDUP_STATE          varchar(32)                    not null default ' ',
   LAST_DEDUP_OP_BEGIN_TIME date                           null,
   LAST_DEDUP_OP_END_TIME date                           null,
   STORAGE_POOL_OBJECT_IDENTIFIER varchar(512)                   null
);

comment on table SAHARA_APP.NETAPP_VOLUME_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_VOLUME.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for NetApp volumes.  It defines the attributes of NetApp volumes that may be used to qualify queries.';

comment on column SAHARA_APP.NETAPP_VOLUME_.NETAPP_VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The NetApp volume display name.';

comment on column SAHARA_APP.NETAPP_VOLUME_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the volume:

Possible values:
- Discovered
- Partially Discovered
- Missing

This may be null.

Note: Partially Discovered means that the NetApp volume was initially discovered by two or more explorers and then one of them stopped discovering the volume.

Note: A NetApp volume that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.NETAPP_VOLUME_.VENDOR_STATE is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column VENDOR_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The vendor specific state of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_.VENDOR_LAYOUT is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column VENDOR_LAYOUT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The vendor''s name for the storage layout used by the device.';

comment on column SAHARA_APP.NETAPP_VOLUME_.RAID_LEVEL is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column RAID_LEVEL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The RAID level of the NetApp volume.

Possible values:
- RAID0
- RAID1
- RAID3
- RAID4
- RAID5
- RAID6
- RAID7
- RAID10
- RAID0+1
- RAID053
- Unknown.

This may be null.';

comment on column SAHARA_APP.NETAPP_VOLUME_.IS_AGGREGATE is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column IS_AGGREGATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean value that indicates if the volume is a combination of other volumes.  A volume can be both a segment and an aggregate if the volume is in a middle layer with an aggregate volume above and the segment volumes below.  In such a case, both Segment and Aggregate will be true.';

comment on column SAHARA_APP.NETAPP_VOLUME_.IS_SEGMENT is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column IS_SEGMENT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean value that indicates if the volume is flex volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_.IS_VFILER_VOLUME is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME, column IS_VFILER_VOLUME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean that indicates if the volume is associated with a vfiler?';

alter table SAHARA_APP.NETAPP_VOLUME_
   alter DM_ID add constraint C_NETAPPVOLUME_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_
   alter IS_AGGREGATE add constraint C_NETAPPVOLUME_ISAGGREGATE check (IS_AGGREGATE in ('T','F'));

alter table SAHARA_APP.NETAPP_VOLUME_
   alter IS_SEGMENT add constraint C_NETAPPVOLUME_ISSEGMENT check (IS_SEGMENT in ('T','F'));

alter table SAHARA_APP.NETAPP_VOLUME_
   alter IS_VFILER_VOLUME add constraint C_NETAPPVOLUME_ISVFILERVOLUME check (IS_VFILER_VOLUME in ('T','F'));

alter table SAHARA_APP.NETAPP_VOLUME_
   alter DISK_SIZE add constraint CKC_DISK_SIZE_NETAPP_V check (DISK_SIZE >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_
   alter IS_NAS_FS_CREATED add constraint CKC_IS_NAS_FS_CREATED_NETAPP_V check (IS_NAS_FS_CREATED in ('T','F'));

alter table SAHARA_APP.NETAPP_VOLUME_
   add constraint PK_NETAPP_VOLUME_ primary key (NETAPP_VOLUME_OID);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.NETAPP_VOLUME_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_VOLUME_FILE                                    */
/*==============================================================*/
create table SAHARA.NETAPP_VOLUME_FILE 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NETAPP_VOLUME_KEY    numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   AVAILABLE_CAPACITY   numeric(22,9)                  not null default 0,
   QTREES               numeric(10)                    not null default 0,
   DM_BATCH_KEY         numeric(10)                    null,
   DM_DATA_AGE          integer                        null,
   TOTAL_CONSUMPTION    numeric(22,9)                  null default 0,
   FILES                numeric(10)                    null default 0,
   FOLDERS              numeric(10)                    null default 0,
   USERS                numeric(10)                    null default 0,
   PHYSICAL_CAPACITY    numeric(22,9)                  not null default 0,
   OVERHEAD_CAPACITY    numeric(22,9)                  not null default 0,
   PHYSICAL_USED        numeric(22,9)                  not null default 0,
   NUMBER_OF_DATA_DISKS numeric(10)                    not null default 0,
   NUMBER_OF_PARITY_DISKS numeric(10)                    not null default 0,
   NUMBER_OF_VOLS_CREATED numeric(10)                    not null default 0,
   NUMBER_OF_LUNS_CREATED numeric(10)                    not null default 0,
   NUMBER_OF_TARGETS    numeric(10)                    not null default 0,
   SPACE_SHARED         numeric(22,9)                  not null default 0,
   SPACE_SAVED          numeric(22,9)                  not null default 0,
   EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0,
   PHYSICAL_SPACE_SHARED numeric(22,9)                  not null default 0,
   PHYSICAL_SPACE_SAVED numeric(22,9)                  not null default 0,
   PHYSICAL_EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0,
   SUBSCRIBED_PERCENT   numeric(7,4)                   null default 0,
   SPACE_SAVED_PERCENT  numeric(7,4)                   null default 0
);

comment on table SAHARA.NETAPP_VOLUME_FILE is 
'This is the fact table for NetApp volume measures.  Each rows represents one sampling of values for a volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NETAPP_VOLUME_FILE.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NETAPP_VOLUME_FILE.NETAPP_VOLUME_KEY is 
'The surrogate key from the NetApp volume dimension that identifies the volume for which the volume files data was collected.  This key may be used in joins to the NETAPP_VOLUME table.';

comment on column SAHARA.NETAPP_VOLUME_FILE.CAPACITY is 
'The capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.USED_CAPACITY is 
'The used capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.AVAILABLE_CAPACITY is 
'The available capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.QTREES is 
'The number of qtrees for the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_VOLUME_FILE.DM_DATA_AGE is 
'The age of the DM data, in seconds,  relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_VOLUME_FILE.TOTAL_CONSUMPTION is 
'The aggregate of the size of all files on the volume, in gigabytes.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_VOLUME_FILE.FILES is 
'The number of files on the volume.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_VOLUME_FILE.FOLDERS is 
'The number of directories on the volume.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_VOLUME_FILE.USERS is 
'The number of users that have files on the volume.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_CAPACITY is 
'The physical capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.OVERHEAD_CAPACITY is 
'The overhead capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_USED is 
'The physical used capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.NUMBER_OF_VOLS_CREATED is 
'For storage pools, number of volumes created';

comment on column SAHARA.NETAPP_VOLUME_FILE.SPACE_SHARED is 
'deduplication space shared';

comment on column SAHARA.NETAPP_VOLUME_FILE.SPACE_SAVED is 
'deduplication space saved';

comment on column SAHARA.NETAPP_VOLUME_FILE.EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_SPACE_SHARED is 
'deduplication physical space shared';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_SPACE_SAVED is 
'deduplication physical space saved';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA.NETAPP_VOLUME_FILE.SUBSCRIBED_PERCENT is 
'deduplication subscribed percent';

comment on column SAHARA.NETAPP_VOLUME_FILE.SPACE_SAVED_PERCENT is 
'deduplication saved percent';

alter table SAHARA.NETAPP_VOLUME_FILE
   alter CAPACITY add constraint C_NTPPVLMFL_CAPACITY check (CAPACITY >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter USED_CAPACITY add constraint C_NTPPVLMFL_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter AVAILABLE_CAPACITY add constraint C_NTPPVLMFL_AVAILABLECAPACITY check (AVAILABLE_CAPACITY >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter QTREES add constraint C_NTPPVLMFL_QTREES check (QTREES >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter TOTAL_CONSUMPTION add constraint C_NTPPVLMFL_TOTALCONSUMPTION check (TOTAL_CONSUMPTION is null or (TOTAL_CONSUMPTION >= 0));

alter table SAHARA.NETAPP_VOLUME_FILE
   alter FILES add constraint C_NTPPVLMFL_FILES check (FILES is null or (FILES >= 0));

alter table SAHARA.NETAPP_VOLUME_FILE
   alter FOLDERS add constraint C_NTPPVLMFL_FOLDERS check (FOLDERS is null or (FOLDERS >= 0));

alter table SAHARA.NETAPP_VOLUME_FILE
   alter USERS add constraint C_NTPPVLMFL_USERS check (USERS is null or (USERS >= 0));

alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_CAPACITY add constraint CKC_PHYSICAL_CAPACITY_NETAPP_V check (PHYSICAL_CAPACITY >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_NETAPP_V check (OVERHEAD_CAPACITY >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NETAPP_V check (PHYSICAL_USED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_DATA_DISKS add constraint CKC_NUMBER_OF_DATA_DI_NETAPP_V check (NUMBER_OF_DATA_DISKS >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_PARITY_DISKS add constraint CKC_NUMBER_OF_PARITY__NETAPP_V check (NUMBER_OF_PARITY_DISKS >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_VOLS_CREATED add constraint CKC_NUMBER_OF_VOLS_CR_NETAPP_V check (NUMBER_OF_VOLS_CREATED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_LUNS_CREATED add constraint CKC_NUMBER_OF_LUNS_CR_NETAPP_V check (NUMBER_OF_LUNS_CREATED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NETAPP_V check (NUMBER_OF_TARGETS >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter SPACE_SHARED add constraint CKC_SPACE_SHARED_NETAPP_V check (SPACE_SHARED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter SPACE_SAVED add constraint CKC_SPACE_SAVED_NETAPP_V check (SPACE_SAVED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter EFFECTIVE_SPACE_USED add constraint CKC_EFFECTIVE_SPACE_U_NETAPP_V check (EFFECTIVE_SPACE_USED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_SPACE_SHARED add constraint CKC_PHYSICAL_SPACE_SH_NETAPP_V check (PHYSICAL_SPACE_SHARED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_SPACE_SAVED add constraint CKC_PHYSICAL_SPACE_SA_NETAPP_V check (PHYSICAL_SPACE_SAVED >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_EFFECTIVE_SPACE_USED add constraint CKC_PHYSICAL_EFFECTIV_NETAPP_V check (PHYSICAL_EFFECTIVE_SPACE_USED >= 0);

/*==============================================================*/
/* Index: I_NAVOLFILE_ROLLUPBATCHKEY                            */
/*==============================================================*/
create index I_NAVOLFILE_ROLLUPBATCHKEY on SAHARA.NETAPP_VOLUME_FILE (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_NAVOLFILE_DATESKEY                                  */
/*==============================================================*/
create index I_NAVOLFILE_DATESKEY on SAHARA.NETAPP_VOLUME_FILE (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_NAVOLFILE_NAVOLKEY                                  */
/*==============================================================*/
create index I_NAVOLFILE_NAVOLKEY on SAHARA.NETAPP_VOLUME_FILE (
NETAPP_VOLUME_KEY ASC
);

grant SELECT on SAHARA.NETAPP_VOLUME_FILE to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_VOLUME_FILE_DM_                                */
/*==============================================================*/
create table SAHARA_APP.NETAPP_VOLUME_FILE_DM_ 
(
   DM_ID                numeric(10)                    not null,
   DM_BATCH_ID          numeric(10)                    not null,
   TOTAL_CONSUMPTION    numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0,
   FOLDERS              numeric(10)                    not null default 0,
   USERS                numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_VOLUME_FILE_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_VOLUME_FILE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp volume measures.  Each rows represents one sampling of values for a volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_DM_.TOTAL_CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column TOTAL_CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all files on the volume, in gigabytes.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_DM_.FILES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files on the volume.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_DM_.FOLDERS is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column FOLDERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of directories on the volume.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_DM_.USERS is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column USERS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of users that have files on the volume.

This is value is only available for only if the NetApp file and the volume was included in a DM rule.  It will be null otherwise.';

alter table SAHARA_APP.NETAPP_VOLUME_FILE_DM_
   alter DM_ID add constraint C_NTPPVLMFLDM_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_DM_
   alter DM_BATCH_ID add constraint C_NTPPVLMFLDM_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_DM_
   alter TOTAL_CONSUMPTION add constraint C_NTPPVLMFLDM_TOTALCONSUMPTION check (TOTAL_CONSUMPTION >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_DM_
   alter FILES add constraint C_NTPPVLMFLDM_FILES check (FILES >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_DM_
   alter FOLDERS add constraint C_NTPPVLMFLDM_FOLDERS check (FOLDERS >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_DM_
   alter USERS add constraint C_NTPPVLMFLDM_USERS check (USERS >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_DM_
   add constraint PK_NETAPP_VOLUME_FILE_DM_ primary key (DM_ID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_VOLUME_FILE_DM_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_VOLUME_FILE_FCAT                               */
/*==============================================================*/
create table SAHARA.NETAPP_VOLUME_FILE_FCAT 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DM_BATCH_KEY         numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NETAPP_VOLUME_KEY    numeric(10)                    not null,
   FILE_CATEGORY_KEY    numeric(10)                    not null,
   DM_DATA_AGE          integer                        not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA.NETAPP_VOLUME_FILE_FCAT is 
'This is the fact table for NetApp volume files organized by file category.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.DM_BATCH_KEY is 
'The identifier of the DM batch from which the DM fact measure values originated.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.NETAPP_VOLUME_KEY is 
'The surrogate key from the array dimension that identifies the array for which the array storage data was collected.  This key may be used in joins to the ARRAY table.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.FILE_CATEGORY_KEY is 
'The surrogate key from the file category dimension that identifies the file category for which the file data was collected.  This key may be used in joins to the FILE_STORAGE table.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.DM_DATA_AGE is 
'The age of the DM data, in seconds, relative to the Enterprise Reporter batch by which it was collected.  DM data is collected using a schedule managed by each CommandCentral Storage server and saved into the CommandCentral Storage database.  At a later time, Enterprise Reporter gathers this data for the inclusion in the Enterprise Reporter database.  This column reflects that "age" of the DM data at the time it was collected by Enterprise Reporter.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.CONSUMPTION is 
'The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA.NETAPP_VOLUME_FILE_FCAT.FILES is 
'The number of files of the associated category on the device.';

alter table SAHARA.NETAPP_VOLUME_FILE_FCAT
   alter CONSUMPTION add constraint C_NTPPVLMFLFCT_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA.NETAPP_VOLUME_FILE_FCAT
   alter FILES add constraint C_NTPPVLMFLFCT_FILES check (FILES >= 0);

/*==============================================================*/
/* Index: I_NAVOLFILEFCAT_ROLLUPBATCHKEY                        */
/*==============================================================*/
create index I_NAVOLFILEFCAT_ROLLUPBATCHKEY on SAHARA.NETAPP_VOLUME_FILE_FCAT (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_NAVOLFILEFCAT_DATESKEY                              */
/*==============================================================*/
create index I_NAVOLFILEFCAT_DATESKEY on SAHARA.NETAPP_VOLUME_FILE_FCAT (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_NAVOLFILEFCAT_NAVOLKEY                              */
/*==============================================================*/
create index I_NAVOLFILEFCAT_NAVOLKEY on SAHARA.NETAPP_VOLUME_FILE_FCAT (
NETAPP_VOLUME_KEY ASC
);

/*==============================================================*/
/* Index: I_NAVFILEFCAT_FILECATEGORYKEY                         */
/*==============================================================*/
create index I_NAVFILEFCAT_FILECATEGORYKEY on SAHARA.NETAPP_VOLUME_FILE_FCAT (
FILE_CATEGORY_KEY ASC
);

grant SELECT on SAHARA.NETAPP_VOLUME_FILE_FCAT to SAHARA_APP;

/*==============================================================*/
/* Table: NETAPP_VOLUME_FILE_FCAT_DM_                           */
/*==============================================================*/
create table SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_ 
(
   DM_ID                numeric(10)                    not null,
   FILE_CATEGORY_OID    varchar(512)                   not null,
   DM_BATCH_ID          numeric(10)                    not null,
   CONSUMPTION          numeric(22,9)                  not null default 0,
   FILES                numeric(10)                    not null default 0
);

comment on table SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_VOLUME_FILE_FCAT.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp volume files organized by file category.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_.FILE_CATEGORY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_.DM_BATCH_ID is 
'A unique identifier used to correlate data from the DM database with the DM batch from in which it was collected.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_.CONSUMPTION is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE_FCAT, column CONSUMPTION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The aggregate of the size of all of the files of the associated category on the device.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_.FILES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE_FCAT, column FILES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of files of the associated category on the device.';

alter table SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_
   alter DM_ID add constraint C_NTPPVLMFLFCTD_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_
   alter DM_BATCH_ID add constraint C_NTPPVLMFLFCTD_DMBATCHID check (DM_BATCH_ID >= 1);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_
   alter CONSUMPTION add constraint C_NTPPVLMFLFCTD_CONSUMPTION check (CONSUMPTION >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_
   alter FILES add constraint C_NTPPVLMFLFCTD_FILES check (FILES >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_
   add constraint PK_NETAPP_VOLUME_FILE_FCAT_DM_ primary key (DM_ID, FILE_CATEGORY_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_ to SAHARA;

/*==============================================================*/
/* Table: NETAPP_VOLUME_FILE_HAL_                               */
/*==============================================================*/
create table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ 
(
   NETAPP_VOLUME_OID    varchar(512)                   not null,
   DM_ID                numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0,
   USED_CAPACITY        numeric(22,9)                  not null default 0,
   AVAILABLE_CAPACITY   numeric(22,9)                  not null default 0,
   QTREES               numeric(10)                    not null default 0,
   PHYSICAL_CAPACITY    numeric(22,9)                  not null default 0,
   OVERHEAD_CAPACITY    numeric(22,9)                  not null default 0,
   PHYSICAL_USED        numeric(22,9)                  not null default 0,
   NUMBER_OF_DATA_DISKS numeric(10)                    not null default 0,
   NUMBER_OF_PARITY_DISKS numeric(10)                    not null default 0,
   NUMBER_OF_VOLS_CREATED numeric(10)                    not null default 0,
   NUMBER_OF_LUNS_CREATED numeric(10)                    not null default 0,
   NUMBER_OF_TARGETS    numeric(10)                    not null default 0,
   SPACE_SHARED         numeric(22,9)                  not null default 0,
   SPACE_SAVED          numeric(22,9)                  not null default 0,
   EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0,
   PHYSICAL_SPACE_SHARED numeric(22,9)                  not null default 0,
   PHYSICAL_SPACE_SAVED numeric(22,9)                  not null default 0,
   PHYSICAL_EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0,
   SUBSCRIBED_PERCENT   numeric(7,4)                   null default 0,
   SPACE_SAVED_PERCENT  numeric(7,4)                   null default 0
);

comment on table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ is 
'This is a staging table for loading data into warehouse table SAHARA.NETAPP_VOLUME_FILE.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp volume measures.  Each rows represents one sampling of values for a volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.NETAPP_VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.DM_ID is 
'A unique object identifier used to correlate data from the HAL database with data from the DM database while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.USED_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column USED_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The used capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.AVAILABLE_CAPACITY is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column AVAILABLE_CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The available capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.QTREES is 
'This column is for staging data that will end up in table SAHARA.NETAPP_VOLUME_FILE, column QTREES.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of qtrees for the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_CAPACITY is 
'The physical capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.OVERHEAD_CAPACITY is 
'The overhead capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_USED is 
'The physical used capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.NUMBER_OF_VOLS_CREATED is 
'For storage pools, number of volumes created';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SPACE_SHARED is 
'deduplication space shared';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SPACE_SAVED is 
'deduplication space saved';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_SPACE_SHARED is 
'deduplication physical space shared';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_SPACE_SAVED is 
'deduplication physical space saved';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SUBSCRIBED_PERCENT is 
'deduplication subscribed percent';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SPACE_SAVED_PERCENT is 
'deduplication saved percent';

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter DM_ID add constraint C_NTPPVLMFLHL_DMID check (DM_ID >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter CAPACITY add constraint C_NTPPVLMFLHL_CAPACITY check (CAPACITY >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter USED_CAPACITY add constraint C_NTPPVLMFLHL_USEDCAPACITY check (USED_CAPACITY >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter AVAILABLE_CAPACITY add constraint C_NTPPVLMFLHL_VLBLCPCT check (AVAILABLE_CAPACITY >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter QTREES add constraint C_NTPPVLMFLHL_QTREES check (QTREES >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_CAPACITY add constraint CKC_PHYSICAL_CAPACITY_NETAPP_V check (PHYSICAL_CAPACITY >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_NETAPP_V check (OVERHEAD_CAPACITY >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NETAPP_V check (PHYSICAL_USED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_DATA_DISKS add constraint CKC_NUMBER_OF_DATA_DI_NETAPP_V check (NUMBER_OF_DATA_DISKS >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_PARITY_DISKS add constraint CKC_NUMBER_OF_PARITY__NETAPP_V check (NUMBER_OF_PARITY_DISKS >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_VOLS_CREATED add constraint CKC_NUMBER_OF_VOLS_CR_NETAPP_V check (NUMBER_OF_VOLS_CREATED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_LUNS_CREATED add constraint CKC_NUMBER_OF_LUNS_CR_NETAPP_V check (NUMBER_OF_LUNS_CREATED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NETAPP_V check (NUMBER_OF_TARGETS >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter SPACE_SHARED add constraint CKC_SPACE_SHARED_NETAPP_V check (SPACE_SHARED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter SPACE_SAVED add constraint CKC_SPACE_SAVED_NETAPP_V check (SPACE_SAVED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter EFFECTIVE_SPACE_USED add constraint CKC_EFFECTIVE_SPACE_U_NETAPP_V check (EFFECTIVE_SPACE_USED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_SPACE_SHARED add constraint CKC_PHYSICAL_SPACE_SH_NETAPP_V check (PHYSICAL_SPACE_SHARED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_SPACE_SAVED add constraint CKC_PHYSICAL_SPACE_SA_NETAPP_V check (PHYSICAL_SPACE_SAVED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_EFFECTIVE_SPACE_USED add constraint CKC_PHYSICAL_EFFECTIV_NETAPP_V check (PHYSICAL_EFFECTIVE_SPACE_USED >= 0);

alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   add constraint PK_NETAPP_VOLUME_FILE_HAL_ primary key (NETAPP_VOLUME_OID);

grant DELETE,ALTER,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ to SAHARA;

/*==============================================================*/
/* Table: OBJECT_DIM_SOURCE                                     */
/*==============================================================*/
create table SAHARA.OBJECT_DIM_SOURCE 
(
   OID                  varchar(512)                   not null,
   TABLE_NAME           varchar(30)                    not null,
   ROLLUP_SOURCE_KEY    numeric(10)                    not null
);

comment on table SAHARA.OBJECT_DIM_SOURCE is 
'This table tracks the sources of objects stored in dimension tables to implement a reference counting scheme.  When a roll-up source first reports information for an object, a row is added to this table for the object and source.  If the object later fails to be reported from the source, or the roll-up source itself is deleted, the row is deleted.

When the last row for an object is deleted, the "current" instance of the object is marked deleted in the associated dimension table by triggers.';

comment on column SAHARA.OBJECT_DIM_SOURCE.OID is 
'The unique identifier of the object.';

comment on column SAHARA.OBJECT_DIM_SOURCE.TABLE_NAME is 
'The name of the table in which the object''s dimension row(s) are stored.';

comment on column SAHARA.OBJECT_DIM_SOURCE.ROLLUP_SOURCE_KEY is 
'The identifier of the roll-up source that has reported information for the object.  This is a "weak link" in that it is not foreign key enforced.  When the roll-up source is physically deleted, this key will no longer point to a value; such rows are later deleted by stored procedure code executed at the beginning of in the roll-up process.';

alter table SAHARA.OBJECT_DIM_SOURCE
   alter TABLE_NAME add constraint C_BJCTDMSRC_TABLENAME check (TABLE_NAME = upper(TABLE_NAME));

alter table SAHARA.OBJECT_DIM_SOURCE
   add constraint PK_OBJECT_DIM_SOURCE primary key (OID, TABLE_NAME, ROLLUP_SOURCE_KEY);

/*==============================================================*/
/* Index: I_OBJDIMESRC_ROLLUPOSOURCEKEY                         */
/*==============================================================*/
create index I_OBJDIMESRC_ROLLUPOSOURCEKEY on SAHARA.OBJECT_DIM_SOURCE (
ROLLUP_SOURCE_KEY ASC
);

grant SELECT on SAHARA.OBJECT_DIM_SOURCE to SAHARA_APP;

/*==============================================================*/
/* Table: P_CUSTOM_ATTRIBUTE                                    */
/*==============================================================*/
create table SAHARA.P_CUSTOM_ATTRIBUTE 
(
   CUSTOM_ATTRIBUTE_KEY numeric(10)                    not null,
   CUSTOM_ATTRIBUTE_CONTEXT varchar(32)                    not null,
   TYPE                 varchar(32)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   COLUMN_NAME          varchar(30)                    not null,
   COLUMN_TYPE          varchar(64)                    not null,
   SOURCE_NAME          varchar(256)                   null
);

comment on table SAHARA.P_CUSTOM_ATTRIBUTE is 
'This table contains the published configuration for custom attributes.

Custom attribute processing for the loading of data is performed using this edit table, but editing of custom attribute configurations is performed using the "edit" table, CUSTOM_ATTRIBUTE.  We call the act of moving configuration from the edit table to the publish table, and making the associated database schema changes, publishing.

This allows configuration editing to take place free of concurrency constraints and publishing to be postponed until access to all of the required resources is available.';

comment on column SAHARA.P_CUSTOM_ATTRIBUTE.CUSTOM_ATTRIBUTE_KEY is 
'A unique database generated identifier for an Enterprise Reporter custom attribute.  The value really serves no other purpose other than to make the tier API easier to use by providing a simple way to identify a custom attribute.';

comment on column SAHARA.P_CUSTOM_ATTRIBUTE.CUSTOM_ATTRIBUTE_CONTEXT is 
'The context in which the source custom attributes are used.';

comment on column SAHARA.P_CUSTOM_ATTRIBUTE.TYPE is 
'The type of Enterprise Reporter custom attribute.  This defines the mechanism used to calculate the custom attribute value.

Valid values:
- SOURCE COPY - an Enterprise Reporter custom attribute that is a simple copy from a roll-up source custom attribute';

comment on column SAHARA.P_CUSTOM_ATTRIBUTE.COLUMN_TYPE is 
'The column data type of the Enterprise Reporter custom attribute column.';

comment on column SAHARA.P_CUSTOM_ATTRIBUTE.SOURCE_NAME is 
'If the custom attribute has type SOURCE COPY, this is the name of the source attribute.';

alter table SAHARA.P_CUSTOM_ATTRIBUTE
    add constraint C_P_CUSTOM_ATTRIBUTE check ((TYPE <> 'SOURCE COPY' or TYPE is not null));

alter table SAHARA.P_CUSTOM_ATTRIBUTE
   alter TYPE add constraint C_PCSTMTTRBT_TYPE check (TYPE in ('SOURCE COPY'));

alter table SAHARA.P_CUSTOM_ATTRIBUTE
   alter COLUMN_NAME add constraint C_PCSTMTTRBT_COLUMNNAME check (COLUMN_NAME = upper(COLUMN_NAME));

alter table SAHARA.P_CUSTOM_ATTRIBUTE
   add constraint PK_P_CUSTOM_ATTRIBUTE primary key (CUSTOM_ATTRIBUTE_KEY);

alter table SAHARA.P_CUSTOM_ATTRIBUTE
   add constraint AK_PCSTMATTR_CNTEXTDISPLAYNAME unique (CUSTOM_ATTRIBUTE_CONTEXT, DISPLAY_NAME);

comment on index SAHARA.P_CUSTOM_ATTRIBUTE.AK_PCSTMATTR_CNTEXTDISPLAYNAME is 
'The display name must be unique within a context.';

alter table SAHARA.P_CUSTOM_ATTRIBUTE
   add constraint AK_PCSTMATTR_CNTEXTCOLNNAME unique (CUSTOM_ATTRIBUTE_CONTEXT, COLUMN_NAME);

comment on index SAHARA.P_CUSTOM_ATTRIBUTE.AK_PCSTMATTR_CNTEXTCOLNNAME is 
'The column name must be unique within a context.';

grant SELECT on SAHARA.P_CUSTOM_ATTRIBUTE to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_BATCH                                          */
/*==============================================================*/
create table SAHARA.ROLLUP_BATCH 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null default autoincrement,
   ROLLUP_SOURCE_TYPE_KEY numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   SESSION_KEY          numeric(10)                    not null,
   SCHEDULE_HASH        varchar(64)                    null,
   ROLLUP_SOURCE_KEY    numeric(10)                    not null,
   SOURCE_NAME          varchar(128)                   not null,
   WEB_SERVER_HOST      varchar(128)                   not null,
   WEB_SERVER_PORT      numeric(5)                     not null,
   DB_HOST              varchar(128)                   not null,
   DB_PORT              numeric(5)                     not null,
   DB_PASSWORD          varchar(128)                   not null,
   DB_PASSWORD_2        varchar(128)                   null,
   LOG_ARCHIVE          varchar(256)                   not null,
   ROLLUP_DAY           varchar(32)                    not null,
   START_TIME           datetime                       not null,
   START_TIME_ER        datetime                       not null,
   COMPLETION_TIME      datetime                       null,
   COMPLETION_TIME_ER   datetime                       null,
   DURATION             integer                        null,
   UTC_ADJUSTMENT       integer                        not null,
   IS_ROLLUP_NOW        char(1)                        not null default 'F',
   IS_COMPLETE          char(1)                        not null default 'F',
   IS_SUCCESS           char(1)                        not null default 'F',
   IS_LATEST            char(1)                        not null default 'F',
   IS_LATEST_COMPLETE   char(1)                        not null default 'F',
   IS_LATEST_SUCCESS    char(1)                        not null default 'F',
   IS_LAST_OF_MONTH     char(1)                        not null default 'F',
   IS_LAST_OF_QUARTER   char(1)                        not null default 'F',
   IS_LAST_OF_YEAR      char(1)                        not null default 'F',
   STATUS               varchar(32)                    not null,
   STATUS_MESSAGE       long varchar                   null,
   LAST_STATUS_UPDATE   timestamp                      not null,
   IS_LAST_OF_WEEK      char(1)                        null default 'F',
   VALID_CLUSTER_DATA   char(1)                        null default 'F',
   IS_DUMMY             char(1)                        null default 'F',
   SHOW_STATUS          char(1)                        null default 'T',
   PERIOD_TYPE          varchar(16)                    null default 'day'
);

comment on table SAHARA.ROLLUP_BATCH is 
'This table records status information about each roll-up batch.

This table is not available for reporting or the SAHARA_READ role because it contains passwords.  Instead the database view ROLLUP_BATCH_READ, which excludes the passwords, should be used in place of this table.';

comment on column SAHARA.ROLLUP_BATCH.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.ROLLUP_BATCH.ROLLUP_SOURCE_TYPE_KEY is 
'The identifier of the roll-up source type.';

comment on column SAHARA.ROLLUP_BATCH.DATES_KEY is 
'The identifier for the date in the dates dimension (DATES table) to which the roll-up batch is attributed.  This allows the dimension to be used for reporting information about roll-up batches.

The date attributed to a roll-up is based on the start date of the roll-up with respect to the roll-up day of the source for the roll-up.

Refer to Enterprise Reporter documentation for a further explanation of a source''s "roll-up day".';

comment on column SAHARA.ROLLUP_BATCH.SESSION_KEY is 
'The identifier of the session for the roll-up batch.  All logging and audit operations for the roll-up batch will use the same session identifier.';

comment on column SAHARA.ROLLUP_BATCH.SCHEDULE_HASH is 
'A hash of roll-up schedule and roll-up source time information used to prevent the same scheduled roll-up batch from being executed (successfully) more than once.

If the roll-up batch was begun to satisfy the roll-up schedule, the schedule hash is populated; for "roll-up now" roll-ups, this column is null.';

comment on column SAHARA.ROLLUP_BATCH.ROLLUP_SOURCE_KEY is 
'The identifier of the roll-up source from which the roll-up batch is generated.  This is a "weak link" in that it is not foreign key enforced.  When the roll-up source is physically deleted, this key will no longer point to a value.

All of the relevant roll-up information is copied from the roll-up source to this table when a roll-up batch is started.  Reports and other parts of the application should not rely on this link; it exists primarily to be able to identify the specific roll-up source for book-keeping purposes.';

comment on column SAHARA.ROLLUP_BATCH.SOURCE_NAME is 
'The external "name" for a roll-up source.  This is not a network name; it is a user-defined name by which the user uniquely identifies the roll-up source and is purely descriptive.  It may differ from the web server host name, database host name, or management server host name.';

comment on column SAHARA.ROLLUP_BATCH.WEB_SERVER_HOST is 
'The  name or IP address of the web server host through which the objects from the roll-up source may be accessed.';

comment on column SAHARA.ROLLUP_BATCH.WEB_SERVER_PORT is 
'The port number the web server through which the objects from the roll-up source may be accessed.';

comment on column SAHARA.ROLLUP_BATCH.DB_HOST is 
'The name or IP address of the host on which the database server of roll-up source is running.';

comment on column SAHARA.ROLLUP_BATCH.DB_PORT is 
'The database server port number of the roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.DB_PASSWORD is 
'The password with which to connect to the roll-up source database.

For CommandCentral Storage 4.x roll-up sources, this is the password for the sahara user in the vxcc database.
For CommandCentral Storage 5.x roll-up sources, this is the password for the sahara user in the ccstor database.';

comment on column SAHARA.ROLLUP_BATCH.DB_PASSWORD_2 is 
'The optional second password with which to connect to the roll-up source database.

For CommandCentral Storage 4.x roll-up sources, this is is not used and will be null.
For CommandCentral Storage 5.x roll-up sources, this is the password for the sahara user in the ccstordm database.';

comment on column SAHARA.ROLLUP_BATCH.LOG_ARCHIVE is 
'The base name of the file that contains the archive of logs from the processing of the roll-up batch.';

comment on column SAHARA.ROLLUP_BATCH.ROLLUP_DAY is 
'An enumeration that identifies the type of roll-up day applied to the roll-up batch.  This is useful for understanding the detailed meaning of the dates key and how the roll-up was scheduled.

Refer to Enterprise Reporter documentation for a further explanation of a source''s "roll-up day".';

comment on column SAHARA.ROLLUP_BATCH.START_TIME is 
'The date and time (stored in UTC) when roll-up of the batch was begun.';

comment on column SAHARA.ROLLUP_BATCH.START_TIME_ER is 
'The date and time (stored in Enterprise Reporter local time) when roll-up of the batch was begun.';

comment on column SAHARA.ROLLUP_BATCH.COMPLETION_TIME is 
'The date and time (stored in UTC) when roll-up of the batch was completed.';

comment on column SAHARA.ROLLUP_BATCH.COMPLETION_TIME_ER is 
'The date and time (stored in Enterprise Reporter local time) when roll-up of the batch was completed.';

comment on column SAHARA.ROLLUP_BATCH.DURATION is 
'The length of time execution, in seconds,  of the roll-up required.';

comment on column SAHARA.ROLLUP_BATCH.UTC_ADJUSTMENT is 
'The adjustment, in seconds, needed to be added to UTC time to obtain the roll-up source''s local time.  Times in the database are stored in UTC.  This value allows calculation of the roll-up source local time.';

comment on column SAHARA.ROLLUP_BATCH.IS_ROLLUP_NOW is 
'A boolean flag that indicates if the roll-up was started as a result of a "roll-up now" request or via the roll-up schedule.';

comment on column SAHARA.ROLLUP_BATCH.IS_COMPLETE is 
'A boolean flag that indicates if the roll-up batch has completed processing, whether is is successful or not.';

comment on column SAHARA.ROLLUP_BATCH.IS_SUCCESS is 
'A boolean flag that indicates if the roll-up batch has completed processing and has a successful status.';

comment on column SAHARA.ROLLUP_BATCH.IS_LATEST is 
'A boolean flag that indicates if this batch is the latest batch, regardless of status (complete, not complete, success, failure), for the associated roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.IS_LATEST_COMPLETE is 
'A boolean flag that indicates if this batch is the latest complete batch, regardless of success or failure, for the associated roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.IS_LATEST_SUCCESS is 
'A boolean flag that indicates if this batch is the latest complete and successful batch for the associated roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.IS_LAST_OF_MONTH is 
'A boolean flag that indicates if this batch is the latest complete and successful batch of the month for the associated roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.IS_LAST_OF_QUARTER is 
'A boolean flag that indicates if this batch is the latest complete and successful batch for the calendar quarter for the associated roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.IS_LAST_OF_YEAR is 
'A boolean flag that indicates if this batch is the latest complete and successful batch for the calendar quarter for the associated roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.STATUS is 
'A status for the roll-up batch.  Possible values for the status are taken from the look-up table ROLLUP_BATCH_STATUS.';

comment on column SAHARA.ROLLUP_BATCH.STATUS_MESSAGE is 
'An optional text message that accompany the status.  Multiple messages may accumulate in this column throughout the roll-up processing for the batch.  They are generally errors or warnings encountered during roll-up.  Further error investigation may require the user to look at the roll-up logs.';

comment on column SAHARA.ROLLUP_BATCH.LAST_STATUS_UPDATE is 
'The date and time (stored in UTC) of the last change to the status.  This column is maintained by triggers.';

comment on column SAHARA.ROLLUP_BATCH.IS_LAST_OF_WEEK is 
'A boolean flag that indicates if this batch is the latest complete and successful batch of the week for the associated roll-up source.';

comment on column SAHARA.ROLLUP_BATCH.VALID_CLUSTER_DATA is 
'Bit is used by Cluster Meta Model to pick up cluster historical data.  Bit is set to 0 by default.  Loading logic will set this bit to 1.  Other object will ignore failed rollup batches, but cluster object need to look at the bit and determine to pickup historical data for the failed batches';

comment on column SAHARA.ROLLUP_BATCH.IS_DUMMY is 
'Default to False to indicate it is batch created as part of rollup process.  Otherwise, it is batches created manually as we rollup cluster historical dates.';

comment on column SAHARA.ROLLUP_BATCH.SHOW_STATUS is 
'Default as True. Indicates it is real batch created by rollup process. Otherwise, it is dummy batches needed to be created as we rollup  VCS historical dates.';

alter table SAHARA.ROLLUP_BATCH
   alter WEB_SERVER_PORT add constraint C_ROLLUPBATCH_WEBSERVERPORT check (WEB_SERVER_PORT >= 1);

alter table SAHARA.ROLLUP_BATCH
   alter DB_PORT add constraint C_ROLLUPBATCH_DBPORT check (DB_PORT >= 1);

alter table SAHARA.ROLLUP_BATCH
   alter ROLLUP_DAY add constraint C_ROLLUPBATCH_ROLLUPDAY check (ROLLUP_DAY in ('ER','SOURCE'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_ROLLUP_NOW add constraint C_ROLLUPBATCH_ISROLLUPNOW check (IS_ROLLUP_NOW in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_COMPLETE add constraint C_ROLLUPBATCH_ISCOMPLETE check (IS_COMPLETE in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_SUCCESS add constraint C_ROLLUPBATCH_ISSUCCESS check (IS_SUCCESS in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_LATEST add constraint C_ROLLUPBATCH_ISLATEST check (IS_LATEST in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_LATEST_COMPLETE add constraint C_ROLLUPBATCH_ISLATESTCOMPLETE check (IS_LATEST_COMPLETE in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_LATEST_SUCCESS add constraint C_ROLLUPBATCH_ISLATESTSUCCESS check (IS_LATEST_SUCCESS in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_LAST_OF_MONTH add constraint C_ROLLUPBATCH_ISLASTOFMONTH check (IS_LAST_OF_MONTH in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_LAST_OF_QUARTER add constraint C_ROLLUPBATCH_ISLASTOFQUARTER check (IS_LAST_OF_QUARTER in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_LAST_OF_YEAR add constraint C_ROLLUPBATCH_ISLASTOFYEAR check (IS_LAST_OF_YEAR in ('T','F'));

alter table SAHARA.ROLLUP_BATCH
   alter IS_LAST_OF_WEEK add constraint C_ROLLUPBATCH_ISLASTOFWEEK check (IS_LAST_OF_WEEK is null or (IS_LAST_OF_WEEK in ('T','F')));

alter table SAHARA.ROLLUP_BATCH
   alter VALID_CLUSTER_DATA add constraint CKC_VALID_CLUSTER_DAT_ROLLUP_B check (VALID_CLUSTER_DATA is null or (VALID_CLUSTER_DATA in ('T','F')));

alter table SAHARA.ROLLUP_BATCH
   alter IS_DUMMY add constraint CKC_IS_DUMMY_ROLLUP_B check (IS_DUMMY is null or (IS_DUMMY in ('T','F')));

alter table SAHARA.ROLLUP_BATCH
   alter SHOW_STATUS add constraint CKC_SHOW_STATUS_ROLLUP_B check (SHOW_STATUS is null or (SHOW_STATUS in ('T','F')));

alter table SAHARA.ROLLUP_BATCH
   add constraint PK_ROLLUP_BATCH primary key (ROLLUP_BATCH_KEY);

/*==============================================================*/
/* Index: I_ROLLUPBATCH_RLLPSRCTPK                              */
/*==============================================================*/
create index I_ROLLUPBATCH_RLLPSRCTPK on SAHARA.ROLLUP_BATCH (
ROLLUP_SOURCE_TYPE_KEY ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_RLLPSRCTPK is 
'An index on Roll-up Source Type Key for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DATESKEY                                */
/*==============================================================*/
create index I_ROLLUPBATCH_DATESKEY on SAHARA.ROLLUP_BATCH (
DATES_KEY ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DATESKEY is 
'An index on Dates Key for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_SESSIONKEY                              */
/*==============================================================*/
create index I_ROLLUPBATCH_SESSIONKEY on SAHARA.ROLLUP_BATCH (
SESSION_KEY ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_SESSIONKEY is 
'An index on Session Key for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_SCHEDULEHASH                            */
/*==============================================================*/
create index I_ROLLUPBATCH_SCHEDULEHASH on SAHARA.ROLLUP_BATCH (
SCHEDULE_HASH ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_SCHEDULEHASH is 
'An index on Schedule Hash for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ROLLUPSOURCEKEY                         */
/*==============================================================*/
create index I_ROLLUPBATCH_ROLLUPSOURCEKEY on SAHARA.ROLLUP_BATCH (
ROLLUP_SOURCE_KEY ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ROLLUPSOURCEKEY is 
'An index on Roll-up Source Key for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_SOURCENAME                              */
/*==============================================================*/
create index I_ROLLUPBATCH_SOURCENAME on SAHARA.ROLLUP_BATCH (
SOURCE_NAME ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_SOURCENAME is 
'An index on Source Name for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_WEBSERVERHOST                           */
/*==============================================================*/
create index I_ROLLUPBATCH_WEBSERVERHOST on SAHARA.ROLLUP_BATCH (
WEB_SERVER_HOST ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_WEBSERVERHOST is 
'An index on Web Server Host for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_WEBSERVERPORT                           */
/*==============================================================*/
create index I_ROLLUPBATCH_WEBSERVERPORT on SAHARA.ROLLUP_BATCH (
WEB_SERVER_PORT ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_WEBSERVERPORT is 
'An index on Web Server Port for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DBHOST                                  */
/*==============================================================*/
create index I_ROLLUPBATCH_DBHOST on SAHARA.ROLLUP_BATCH (
DB_HOST ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DBHOST is 
'An index on Database Host for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DBPORT                                  */
/*==============================================================*/
create index I_ROLLUPBATCH_DBPORT on SAHARA.ROLLUP_BATCH (
DB_PORT ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DBPORT is 
'An index on Database Port for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DBPASSWORD                              */
/*==============================================================*/
create index I_ROLLUPBATCH_DBPASSWORD on SAHARA.ROLLUP_BATCH (
DB_PASSWORD ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DBPASSWORD is 
'An index on Database Password for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DBPASSWORD2                             */
/*==============================================================*/
create index I_ROLLUPBATCH_DBPASSWORD2 on SAHARA.ROLLUP_BATCH (
DB_PASSWORD_2 ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DBPASSWORD2 is 
'An index on Database Password 2 for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_LOGARCHIVE                              */
/*==============================================================*/
create index I_ROLLUPBATCH_LOGARCHIVE on SAHARA.ROLLUP_BATCH (
LOG_ARCHIVE ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_LOGARCHIVE is 
'An index on Log Archive for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ROLLUPDAY                               */
/*==============================================================*/
create index I_ROLLUPBATCH_ROLLUPDAY on SAHARA.ROLLUP_BATCH (
ROLLUP_DAY ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ROLLUPDAY is 
'An index on Roll-up Day for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_STARTTIME                               */
/*==============================================================*/
create index I_ROLLUPBATCH_STARTTIME on SAHARA.ROLLUP_BATCH (
START_TIME ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_STARTTIME is 
'An index on Start Time for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_STARTTIMEER                             */
/*==============================================================*/
create index I_ROLLUPBATCH_STARTTIMEER on SAHARA.ROLLUP_BATCH (
START_TIME_ER ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_STARTTIMEER is 
'An index on Start Time - Enterprise Reporter Time for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_COMPLETIONTIME                          */
/*==============================================================*/
create index I_ROLLUPBATCH_COMPLETIONTIME on SAHARA.ROLLUP_BATCH (
COMPLETION_TIME ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_COMPLETIONTIME is 
'An index on Completion Time for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_COMPLETIONTIMEER                        */
/*==============================================================*/
create index I_ROLLUPBATCH_COMPLETIONTIMEER on SAHARA.ROLLUP_BATCH (
COMPLETION_TIME_ER ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_COMPLETIONTIMEER is 
'An index on Completion Time - Enterprise Reporter Time for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DURATION                                */
/*==============================================================*/
create index I_ROLLUPBATCH_DURATION on SAHARA.ROLLUP_BATCH (
DURATION ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DURATION is 
'An index on Duration for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_UTCADJUSTMENT                           */
/*==============================================================*/
create index I_ROLLUPBATCH_UTCADJUSTMENT on SAHARA.ROLLUP_BATCH (
UTC_ADJUSTMENT ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_UTCADJUSTMENT is 
'An index on UTC Adjustment for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISROLLUPNOW                             */
/*==============================================================*/
create index I_ROLLUPBATCH_ISROLLUPNOW on SAHARA.ROLLUP_BATCH (
IS_ROLLUP_NOW ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISROLLUPNOW is 
'An index on Is Rollup Now for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISCOMPLETE                              */
/*==============================================================*/
create index I_ROLLUPBATCH_ISCOMPLETE on SAHARA.ROLLUP_BATCH (
IS_COMPLETE ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISCOMPLETE is 
'An index on Is Complete for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISSUCCESS                               */
/*==============================================================*/
create index I_ROLLUPBATCH_ISSUCCESS on SAHARA.ROLLUP_BATCH (
IS_SUCCESS ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISSUCCESS is 
'An index on Is Success for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISLATEST                                */
/*==============================================================*/
create index I_ROLLUPBATCH_ISLATEST on SAHARA.ROLLUP_BATCH (
IS_LATEST ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISLATEST is 
'An index on Is Latest for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISLATESTCOMPLETE                        */
/*==============================================================*/
create index I_ROLLUPBATCH_ISLATESTCOMPLETE on SAHARA.ROLLUP_BATCH (
IS_LATEST_COMPLETE ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISLATESTCOMPLETE is 
'An index on Is Latest Complete for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISLATESTSUCCESS                         */
/*==============================================================*/
create index I_ROLLUPBATCH_ISLATESTSUCCESS on SAHARA.ROLLUP_BATCH (
IS_LATEST_SUCCESS ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISLATESTSUCCESS is 
'An index on Is Latest Success for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISLASTOFMONTH                           */
/*==============================================================*/
create index I_ROLLUPBATCH_ISLASTOFMONTH on SAHARA.ROLLUP_BATCH (
IS_LAST_OF_MONTH ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISLASTOFMONTH is 
'An index on Is Last of Month for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISLASTOFQUARTER                         */
/*==============================================================*/
create index I_ROLLUPBATCH_ISLASTOFQUARTER on SAHARA.ROLLUP_BATCH (
IS_LAST_OF_QUARTER ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISLASTOFQUARTER is 
'An index on Is Last of Quarter for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISLASTOFYEAR                            */
/*==============================================================*/
create index I_ROLLUPBATCH_ISLASTOFYEAR on SAHARA.ROLLUP_BATCH (
IS_LAST_OF_YEAR ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISLASTOFYEAR is 
'An index on Is Last of Year for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_STATUS                                  */
/*==============================================================*/
create index I_ROLLUPBATCH_STATUS on SAHARA.ROLLUP_BATCH (
STATUS ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_STATUS is 
'An index on Status for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_STATUSMESSAGE                           */
/*==============================================================*/
create index I_ROLLUPBATCH_STATUSMESSAGE on SAHARA.ROLLUP_BATCH (
STATUS_MESSAGE ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_STATUSMESSAGE is 
'An index on Status Message for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_LASTSTATUSUPDATE                        */
/*==============================================================*/
create index I_ROLLUPBATCH_LASTSTATUSUPDATE on SAHARA.ROLLUP_BATCH (
LAST_STATUS_UPDATE ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_LASTSTATUSUPDATE is 
'An index on Last Status Update for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_ISLASTOFWEEK                            */
/*==============================================================*/
create index I_ROLLUPBATCH_ISLASTOFWEEK on SAHARA.ROLLUP_BATCH (
IS_LAST_OF_WEEK ASC
);

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_ISLASTOFWEEK is 
'An index on Is_Last_Of_Week for dimensional queries.';

grant SELECT on SAHARA.ROLLUP_BATCH to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_BATCH_STATUS                                   */
/*==============================================================*/
create table SAHARA.ROLLUP_BATCH_STATUS 
(
   STATUS               varchar(32)                    not null,
   DESCRIPTION          long varchar                   not null,
   IS_COMPLETE          char(1)                        not null,
   IS_SUCCESS           char(1)                        not null
);

comment on table SAHARA.ROLLUP_BATCH_STATUS is 
'A look-up table the defines what values are valid for roll-up batch status (ROLLUP_BATCH.STATUS).  The table provides descriptive and meta data about each status value.';

comment on column SAHARA.ROLLUP_BATCH_STATUS.STATUS is 
'The roll-up batch status value.';

comment on column SAHARA.ROLLUP_BATCH_STATUS.DESCRIPTION is 
'A description of the meaning of the roll-up batch status.';

comment on column SAHARA.ROLLUP_BATCH_STATUS.IS_COMPLETE is 
'A boolean flag that indicates if this status is a final status -- i.e.  is the roll-up batch complete? It is used to set the column ROLLUP_STATUS.IS_COMPLETE.';

comment on column SAHARA.ROLLUP_BATCH_STATUS.IS_SUCCESS is 
'A boolean flag that indicates if this status is a successful status.  This flag really only applies to statuses that are also complete.  It is used to set the column ROLLUP_STATUS.IS_LATEST_SUCCESS.';

alter table SAHARA.ROLLUP_BATCH_STATUS
   alter IS_COMPLETE add constraint C_RLLPBTCHSTTS_ISCOMPLETE check (IS_COMPLETE in ('T','F'));

alter table SAHARA.ROLLUP_BATCH_STATUS
   alter IS_SUCCESS add constraint C_RLLPBTCHSTTS_ISSUCCESS check (IS_SUCCESS in ('T','F'));

alter table SAHARA.ROLLUP_BATCH_STATUS
   add constraint PK_ROLLUP_BATCH_STATUS primary key (STATUS);

grant SELECT on SAHARA.ROLLUP_BATCH_STATUS to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_DATA_SET                                       */
/*==============================================================*/
create table SAHARA.ROLLUP_DATA_SET 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   ROLLUP_DATA_SET_TYPE_KEY numeric(10)                    not null,
   STATUS               varchar(32)                    not null,
   STATUS_MESSAGE       long varchar                   null,
   UPLOADING_DURATION   integer                        null,
   STAGING_DURATION     integer                        null,
   LOADING_DURATION     integer                        null,
   ROWS_PROCESSED       numeric(10)                    null,
   LAST_STATUS_UPDATE   timestamp                      not null
);

comment on table SAHARA.ROLLUP_DATA_SET is 
'This table records success or failure for individual data sets within a roll-up batch and various metrics about the data set''s collection.  Each rows represents one data set from a roll-up batch.';

comment on column SAHARA.ROLLUP_DATA_SET.ROLLUP_BATCH_KEY is 
'The identifier of the roll-up batch with which the data set is associated.';

comment on column SAHARA.ROLLUP_DATA_SET.ROLLUP_DATA_SET_TYPE_KEY is 
'The identifier of a roll-up data set type for which status is being recorded.';

comment on column SAHARA.ROLLUP_DATA_SET.STATUS is 
'A status for the roll-up data set.

Possible values:
- Pending
- Remote Processing
- Remote Processing Complete
- Uploading
- Uploading Complete
- Staging
- Staging Complete
- Loading
- Succeeded
- Failed';

comment on column SAHARA.ROLLUP_DATA_SET.STATUS_MESSAGE is 
'An optional textual description of the status.  Further error investigation may require the user to look at the roll-up logs.';

comment on column SAHARA.ROLLUP_DATA_SET.UPLOADING_DURATION is 
'The amount of time, in seconds,  spent uploading the data set from the roll-up source host to the Enterprise Reporter server host.';

comment on column SAHARA.ROLLUP_DATA_SET.STAGING_DURATION is 
'The amount of time, in seconds,  spent inserting the data set into the staging tables of the Enterprise Reporter database.';

comment on column SAHARA.ROLLUP_DATA_SET.LOADING_DURATION is 
'The amount of time, in seconds,  spent processing the data set from the staging tables of the Enterprise Reporter database to the reporting tables.';

comment on column SAHARA.ROLLUP_DATA_SET.ROWS_PROCESSED is 
'The total number of rows inserted or updated while processing the data set.';

comment on column SAHARA.ROLLUP_DATA_SET.LAST_STATUS_UPDATE is 
'The date and time (stored in UTC) of the last change to the status.  This column is maintained by triggers.';

alter table SAHARA.ROLLUP_DATA_SET
   alter STATUS add constraint C_ROLLUPDATASET_STATUS check (STATUS in ('Pending','Remote Processing','Remote Processing Complete','Uploading','Uploading Complete','Staging','Staging Complete','Loading','Succeeded','Failed'));

alter table SAHARA.ROLLUP_DATA_SET
   alter ROWS_PROCESSED add constraint C_ROLLUPDATASET_ROWSPROCESSED check (ROWS_PROCESSED is null or (ROWS_PROCESSED >= 0));

alter table SAHARA.ROLLUP_DATA_SET
   add constraint PK_ROLLUP_DATA_SET primary key (ROLLUP_BATCH_KEY, ROLLUP_DATA_SET_TYPE_KEY);

/*==============================================================*/
/* Index: I_ROLLUPDATASET_STATUS                                */
/*==============================================================*/
create index I_ROLLUPDATASET_STATUS on SAHARA.ROLLUP_DATA_SET (
STATUS ASC
);

grant SELECT on SAHARA.ROLLUP_DATA_SET to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_DATA_SET_COLUMN                                */
/*==============================================================*/
create table SAHARA.ROLLUP_DATA_SET_COLUMN 
(
   ROLLUP_DATA_SET_TYPE_KEY numeric(10)                    not null,
   NAME                 varchar(30)                    not null,
   CUSTOM_ATTRIBUTE_KEY numeric(10)                    null,
   LOAD_PROCESSING      varchar(32)                    not null,
   SOURCE_EXPRESSION    varchar(128)                   null
);

comment on table SAHARA.ROLLUP_DATA_SET_COLUMN is 
'This table contains meta-data for columns involved in loading a data set from staging tables to warehouse tables.

It is a little overloaded in purpose.

Currently, this metadata is used for dimensions only.

This table is populated at install and upgrade.';

comment on column SAHARA.ROLLUP_DATA_SET_COLUMN.ROLLUP_DATA_SET_TYPE_KEY is 
'The identifier of the data set type to which the column information applies.';

comment on column SAHARA.ROLLUP_DATA_SET_COLUMN.NAME is 
'The name of a column in the database in the staging table and/or the warehouse table.';

comment on column SAHARA.ROLLUP_DATA_SET_COLUMN.CUSTOM_ATTRIBUTE_KEY is 
'The unique identifier of a custom attribute if this column is in the data set metadata because it is configured as a custom attribute.  For non-custom attribute columns, this will be null.';

comment on column SAHARA.ROLLUP_DATA_SET_COLUMN.LOAD_PROCESSING is 
'An enumeration that describes the type of processing to be performed on the column while transferring data from the staging table to the warehouse table.  This column identifies what to do with the column on the destination/warehouse end of the load.

Possible values for dimension data sets:
- TYPE 1 - perform Kimball type 1 dimension attribute processing
- TYPE 2 - perform Kimball type 2 dimension processing
- STAGING KEY - the column is a key value used to uniquely identify the row within the staging table and is used to look for the staging table row in the warehouse table to determine in an update or an insert is required
- NONE - ignore the column completely; it is assumed to be populated via triggers or some other means.';

comment on column SAHARA.ROLLUP_DATA_SET_COLUMN.SOURCE_EXPRESSION is 
'An optional expression to be treated as a column on the source/staging end of the load.  All columns in the staging table are automatically considered sources via a straight select from the staging table.  An expression may be used in addition to the staging columns, or to augment or replace a staging column.  The expression will be given the name from the roll-up data set column record.  If the name is the same as a staging column name, the staging column will be replaced by the expression.  The token $COLUMN$ is supported in the expression and is substituted with the name from the roll-up data set column record as though it were a staging table column.';

alter table SAHARA.ROLLUP_DATA_SET_COLUMN
   alter NAME add constraint C_RLLPDTSTCLMN_NAME check (NAME = upper(NAME));

alter table SAHARA.ROLLUP_DATA_SET_COLUMN
   alter LOAD_PROCESSING add constraint C_RLLPDTSTCLMN_LOADPROCESSING check (LOAD_PROCESSING in ('STAGING KEY','TYPE 1','TYPE 2','NONE'));

alter table SAHARA.ROLLUP_DATA_SET_COLUMN
   add constraint PK_ROLLUP_DATA_SET_COLUMN primary key (ROLLUP_DATA_SET_TYPE_KEY, NAME);

grant SELECT on SAHARA.ROLLUP_DATA_SET_COLUMN to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_DATA_SET_STAGE                                 */
/*==============================================================*/
create table SAHARA.ROLLUP_DATA_SET_STAGE 
(
   ROLLUP_DATA_SET_TYPE_KEY numeric(10)                    not null,
   STAGE_TABLE          varchar(30)                    not null
);

comment on table SAHARA.ROLLUP_DATA_SET_STAGE is 
'This table contains the names of staging tables used by a data set.  The tables are assumed to be owned by the SAHARA_APP database user.';

comment on column SAHARA.ROLLUP_DATA_SET_STAGE.ROLLUP_DATA_SET_TYPE_KEY is 
'The unique database identifier of the roll-up data set with which the staging table is associated.';

comment on column SAHARA.ROLLUP_DATA_SET_STAGE.STAGE_TABLE is 
'The name of the staging table.  The table is assumed to be in the SAHARA_APP schema.';

alter table SAHARA.ROLLUP_DATA_SET_STAGE
   alter STAGE_TABLE add constraint C_RLLPDTSTSTG_STAGETABLE check (STAGE_TABLE = upper(STAGE_TABLE));

alter table SAHARA.ROLLUP_DATA_SET_STAGE
   add constraint PK_ROLLUP_DATA_SET_STAGE primary key (ROLLUP_DATA_SET_TYPE_KEY, STAGE_TABLE);

grant SELECT on SAHARA.ROLLUP_DATA_SET_STAGE to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_DATA_SET_TYPE                                  */
/*==============================================================*/
create table SAHARA.ROLLUP_DATA_SET_TYPE 
(
   ROLLUP_DATA_SET_TYPE_KEY numeric(10)                    not null default autoincrement,
   NAME                 varchar(64)                    not null,
   STEREOTYPE           varchar(32)                    not null,
   DEST_TABLE           varchar(30)                    not null,
   DUPLICATE_FACT_PURGE_KEY_COLS long varchar                   null,
   LOAD_SQL             long varchar                   null,
   CSV_NAME             varchar(64)                    null,
   VOM_OBJECT_TYPE      varchar(128)                   null
);

comment on table SAHARA.ROLLUP_DATA_SET_TYPE is 
'This is a look-up and metadata table for information about roll-up data set types.  It is populated at install/upgrade time.

A roll-up data set type is a meta-type that identifies a part of the data rolled-up from a roll-up source.  A data set is loaded into a staging table in which any additional transformations may be calculated.  Finally, the data is merged from the staging table into a real data warehouse table to become available for reporting.';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY is 
'A unique database generated identifier for the roll-up source type used as a surrogate key.';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.NAME is 
'A name for the data set type, used to unique identify the type.';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.STEREOTYPE is 
'An enumerated value that identifies how a data set of this type is used:

Valid values are:
- DIMENSION - the standard model load for dimensions is used
- FACT - the standard model load for facts is used
- CUSTOM - the LOAD_SQL expression is executed without any other expected logic';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.DEST_TABLE is 
'The name of the destination data warehouse table into which the data set data is loaded.

The loading process for some data sets (dimensions, facts) follows very specific naming conventions based on this name.  From this name, the names of tables, key columns, sequences, etc., may be derived to implement the load of data for the data set.';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.DUPLICATE_FACT_PURGE_KEY_COLS is 
'This column contains the names of the columns used for purging duplicate fact rows from a fact table.

When multiple roll-up sources roll-up facts for a dimension row (an object), fact rows previously rolled-up for the same date are purged.';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.LOAD_SQL is 
'A SQL template that is used differently for each stereotype.  It contains substitution tokens that are populated at runtime to generate the actual SQL to populate the warehouse table.

DIMENSION stereotype - a select statement that provides the source columns for the merge statement.

Supported tokens:
- $DEST_TABLE$
- $SRC_COLUMNS$
- $SRC_TABLE$

FACT stereotype - insert statement that selects from the staging table and inserts into the fact table.

Supported tokens:
- $DATES_KEY$
- $DEST_COLUMNS$
- $DEST_TABLE$
- $ROLLUP_BATCH_KEY$
- $SRC_COLUMNS$
- $SRC_TABLE$

CUSTOM stereotype - the load SQL is executed

Supported tokens:
- $DEST_TABLE$
- $ROLLUP_BATCH_KEY$';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.CSV_NAME is 
'The name of the csv file name which stores the data set data. 
    
The staging process for data sets (dimensions, facts) uses this file to load respective staging tables.
';

comment on column SAHARA.ROLLUP_DATA_SET_TYPE.VOM_OBJECT_TYPE is 
'Object type from VOM or this object (Examples: vrts_host, vrts_vxvm_volume etc.)';

alter table SAHARA.ROLLUP_DATA_SET_TYPE
   alter STEREOTYPE add constraint C_RLLPDTSTTP_STEREOTYPE check (STEREOTYPE in ('DIMENSION','FACT','CUSTOM'));

alter table SAHARA.ROLLUP_DATA_SET_TYPE
   alter DEST_TABLE add constraint C_RLLPDTSTTP_DESTTABLE check (DEST_TABLE = upper(DEST_TABLE));

alter table SAHARA.ROLLUP_DATA_SET_TYPE
   add constraint PK_ROLLUP_DATA_SET_TYPE primary key (ROLLUP_DATA_SET_TYPE_KEY);

alter table SAHARA.ROLLUP_DATA_SET_TYPE
   add constraint AK_ROLLUPDATASETTYPE_NAME unique (NAME);

grant SELECT on SAHARA.ROLLUP_DATA_SET_TYPE to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_SCHEDULE                                       */
/*==============================================================*/
create table SAHARA.ROLLUP_SCHEDULE 
(
   ROLLUP_SCHEDULE_KEY  char(3)                        not null,
   REVISION             numeric(10)                    not null default 0,
   TYPE                 varchar(32)                    not null default 'DAILY',
   BY_WEEK_SUNDAY       char(1)                        not null default 'T',
   BY_WEEK_MONDAY       char(1)                        not null default 'T',
   BY_WEEK_TUESDAY      char(1)                        not null default 'T',
   BY_WEEK_WEDNESDAY    char(1)                        not null default 'T',
   BY_WEEK_THURSDAY     char(1)                        not null default 'T',
   BY_WEEK_FRIDAY       char(1)                        not null default 'T',
   BY_WEEK_SATURDAY     char(1)                        not null default 'T',
   BY_WEEK_MULTIPLIER   numeric(2)                     not null default 1,
   BY_WEEK_START_DATE   date                           null,
   BY_MONTH_1_WEEK      varchar(32)                    not null default 'First',
   BY_MONTH_1_DAY       varchar(32)                    not null default 'Sunday',
   BY_MONTH_1_MULTIPLIER numeric(2)                     not null default 1,
   BY_MONTH_1_START_DATE date                           null,
   BY_MONTH_2_DATE      numeric(2)                     not null default 1,
   BY_MONTH_2_MULTIPLIER numeric(2)                     not null default 1,
   BY_MONTH_2_START_DATE date                           null
);

comment on table SAHARA.ROLLUP_SCHEDULE is 
'This table contains the global roll-up schedule that defines on which days roll-up is performed.  The actual time of roll-up is defined by the roll-up time defined for a roll-up source (found in the ROLLUP_SOURCE table).

The roll-up schedule supports multiple types of schedules.  The type of schedule is defined by the TYPE column.  The type then determines what other columns of the table are relevant:

The types are as follows:
- DAILY - roll-up every day
- BY WEEK - roll-up select day(s) (1-7 may be chosen) of the week, every <N> week(s), starting <D>
- BY MONTH 1 - roll-up every <First, Second, Third, Forth, Last> <day of the week> of every <N> month(s), starting <D>
- BY MONTH 2 - roll-up every day <N> (1-31) of every <M> month(s), starting <D>';

comment on column SAHARA.ROLLUP_SCHEDULE.ROLLUP_SCHEDULE_KEY is 
'A surrogate key that allows only one value.  This forces the table to have only one row.';

comment on column SAHARA.ROLLUP_SCHEDULE.REVISION is 
'A revision number used to distinguish between different schedules.  Each time the roll-up schedule is modified, it replaces the previous schedule.  The revision number is automatically incremented by a trigger to reflect the new schedule.

This value is used to generate a hash for each schedule/roll-up source/date instance to and is used to avoid performing duplicate roll-ups.';

comment on column SAHARA.ROLLUP_SCHEDULE.TYPE is 
'The type of roll-up schedule being used.

Valid values:
- DAILY
- BY WEEK
- BY MONTH 1
- BY MONTH 2

See the table description for a more detailed explanation of each type.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_SUNDAY is 
'A boolean flag that indicates of roll-ups should be performed on Sunday when the roll-up schedule type is BY WEEK.

Valid values:
- T - true
- F - false

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_MONDAY is 
'A boolean flag that indicates of roll-ups should be performed on Monday when the roll-up schedule type is BY WEEK.

Valid values:
- T - true
- F - false

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_TUESDAY is 
'A boolean flag that indicates of roll-ups should be performed on Tuesday when the roll-up schedule type is BY WEEK.

Valid values:
- T - true
- F - false

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_WEDNESDAY is 
'A boolean flag that indicates of roll-ups should be performed on Wednesday when the roll-up schedule type is BY WEEK.

Valid values:
- T - true
- F - false

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_THURSDAY is 
'A boolean flag that indicates of roll-ups should be performed on Thursday when the roll-up schedule type is BY WEEK.

Valid values:
- T - true
- F - false

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_FRIDAY is 
'A boolean flag that indicates of roll-ups should be performed on Friday when the roll-up schedule type is BY WEEK.

Valid values:
- T - true
- F - false

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_SATURDAY is 
'A boolean flag that indicates of roll-ups should be performed on Saturday when the roll-up schedule type is BY WEEK.

Valid values:
- T - true
- F - false

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_MULTIPLIER is 
'A multiplier that determines the N of "every <N> weeks" when the roll-up schedule type is BY WEEK.  See the table description for a more detailed explanation of roll-up types.

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_WEEK_START_DATE is 
'A date that identifies the start date for a schedule when the roll-up schedule type is BY WEEK.  See the table description for a more detailed explanation of roll-up types.

If the multiplier (BY_WEEK_MULTIPLIER) is greater than 1, this date must be a scheduled date; it must meet the other criteria established by the schedule.  If the multiplier is 1, this column is ignored and may be null.

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_MONTH_1_WEEK is 
'The week of the month that applies when the roll-up schedule is type BY MONTH 1.  See the table description for a more detailed explanation of roll-up types.

Valid values:
- First
- Second
- Third
- Forth
- Last

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_MONTH_1_DAY is 
'The day of the week that applies when the roll-up schedule is type BY MONTH 1.  See the table description for a more detailed explanation of roll-up types.

Valid values:
- Sunday
- Monday
- Tuesday
- Wednesday
- Thursday
- Friday
- Saturday

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_MONTH_1_MULTIPLIER is 
'A multiplier that determines the N of "every <N> months" when the roll-up schedule type is BY MONTH 1.  See the table description for a more detailed explanation of roll-up types.

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_MONTH_1_START_DATE is 
'A date that identifies the start date for a schedule when the roll-up schedule type is BY MONTH 1.  See the table description for a more detailed explanation of roll-up types.

If the mutiplier (BY_MONTH_1_MULTIPLIER) is greater than 1, this date must be a scheduled date; it must meet the other criteria established by the schedule.  If the multipler is 1, this column is ignored and may be null.

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_MONTH_2_DATE is 
'The date of the month that applies when the roll-up schedule is type BY MONTH 2.  See the table description for a more detailed explanation of roll-up types.

Valid values: 1-31.

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_MONTH_2_MULTIPLIER is 
'A multiplier that determines the M of "every <M> months" when the roll-up schedule type is BY MONTH 2.  See the table description for a more detailed explanation of roll-up types.

This column is ignored for other roll-up types.';

comment on column SAHARA.ROLLUP_SCHEDULE.BY_MONTH_2_START_DATE is 
'A date that identifies the start date for a schedule when the roll-up schedule type is BY MONTH 2.  See the table description for a more detailed explanation of roll-up types.

If the mutiplier (BY_MONTH_2_MULTIPLIER) is greater than 1, this date must be a scheduled date; it must meet the other criteria established by the schedule.  If the multipler is 1, this column is ignored and may be null.

This column is ignored for other roll-up types.';

alter table SAHARA.ROLLUP_SCHEDULE
    add constraint C_ROLLUP_SCHEDULE check (-- at least one day must be enabled
(
    (BY_WEEK_SUNDAY = 'T') or
    (BY_WEEK_MONDAY = 'T') or
    (BY_WEEK_TUESDAY = 'T') or
    (BY_WEEK_WEDNESDAY = 'T') or
    (BY_WEEK_THURSDAY = 'T') or
    (BY_WEEK_FRIDAY = 'T') or
    (BY_WEEK_SATURDAY = 'T')
)
and
(
    -- if the multipler is > 1, a start date is required
    (BY_WEEK_MULTIPLIER = 1) or (BY_WEEK_START_DATE is not null)
)
and
(
    -- if the multipler is > 1, a start date is required
    (BY_MONTH_1_MULTIPLIER = 1) or (BY_MONTH_1_START_DATE is not null)
)
and
(
    -- if the multipler is > 1, a start date is required
    (BY_MONTH_2_MULTIPLIER = 1) or (BY_MONTH_2_START_DATE is not null)
));

alter table SAHARA.ROLLUP_SCHEDULE
   alter ROLLUP_SCHEDULE_KEY add constraint C_RLLPSCHDL_ROLLUPSCHEDULEKEY check (ROLLUP_SCHEDULE_KEY in ('KEY'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter REVISION add constraint C_RLLPSCHDL_REVISION check (REVISION >= 0);

alter table SAHARA.ROLLUP_SCHEDULE
   alter TYPE add constraint C_RLLPSCHDL_TYPE check (TYPE in ('DAILY','BY WEEK','BY MONTH 1','BY MONTH 2'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_SUNDAY add constraint C_RLLPSCHDL_BYWEEKSUNDAY check (BY_WEEK_SUNDAY in ('T','F'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_MONDAY add constraint C_RLLPSCHDL_BYWEEKMONDAY check (BY_WEEK_MONDAY in ('T','F'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_TUESDAY add constraint C_RLLPSCHDL_BYWEEKTUESDAY check (BY_WEEK_TUESDAY in ('T','F'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_WEDNESDAY add constraint C_RLLPSCHDL_BYWEEKWEDNESDAY check (BY_WEEK_WEDNESDAY in ('T','F'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_THURSDAY add constraint C_RLLPSCHDL_BYWEEKTHURSDAY check (BY_WEEK_THURSDAY in ('T','F'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_FRIDAY add constraint C_RLLPSCHDL_BYWEEKFRIDAY check (BY_WEEK_FRIDAY in ('T','F'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_SATURDAY add constraint C_RLLPSCHDL_BYWEEKSATURDAY check (BY_WEEK_SATURDAY in ('T','F'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_MULTIPLIER add constraint C_RLLPSCHDL_BYWEEKMULTIPLIER check (BY_WEEK_MULTIPLIER >= 1);

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_WEEK_START_DATE add constraint C_RLLPSCHDL_BYWEEKSTARTDATE check (BY_WEEK_START_DATE is null or (BY_WEEK_START_DATE = date(BY_WEEK_START_DATE)));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_MONTH_1_WEEK add constraint C_RLLPSCHDL_BYMONTH1WEEK check (BY_MONTH_1_WEEK in ('First','Second','Third','Forth','Last'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_MONTH_1_DAY add constraint C_RLLPSCHDL_BYMONTH1DAY check (BY_MONTH_1_DAY in ('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_MONTH_1_MULTIPLIER add constraint C_RLLPSCHDL_BYMONTH1MULTIPLIER check (BY_MONTH_1_MULTIPLIER >= 1);

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_MONTH_1_START_DATE add constraint C_RLLPSCHDL_BYMONTH1STARTDATE check (BY_MONTH_1_START_DATE is null or (BY_MONTH_1_START_DATE = date(BY_MONTH_1_START_DATE)));

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_MONTH_2_DATE add constraint C_RLLPSCHDL_BYMONTH2DATE check (BY_MONTH_2_DATE between 1 and 31);

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_MONTH_2_MULTIPLIER add constraint C_RLLPSCHDL_BYMONTH2MULTIPLIER check (BY_MONTH_2_MULTIPLIER >= 1);

alter table SAHARA.ROLLUP_SCHEDULE
   alter BY_MONTH_2_START_DATE add constraint C_RLLPSCHDL_BYMONTH2STARTDATE check (BY_MONTH_2_START_DATE is null or (BY_MONTH_2_START_DATE = date(BY_MONTH_2_START_DATE)));

alter table SAHARA.ROLLUP_SCHEDULE
   add constraint PK_ROLLUP_SCHEDULE primary key (ROLLUP_SCHEDULE_KEY);

grant SELECT on SAHARA.ROLLUP_SCHEDULE to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_SCHEDULE_DATE                                  */
/*==============================================================*/
create global temporary table SAHARA.ROLLUP_SCHEDULE_DATE 
(
   SCHEDULE_DATE        date                           not null
);

comment on table SAHARA.ROLLUP_SCHEDULE_DATE is 
'A global temporary table used as a container for roll-up schedule dates.

This table is the basis for the implementation of the roll-up schedule dates/times view (ROLLUP_SCHEDULE_DATETIME).

Before the dates/times view may be used, the dates table (this table) must be populated by calling stored procedure ROLLUP_SCHEDULE_API.PrepareDatesTable.';

comment on column SAHARA.ROLLUP_SCHEDULE_DATE.SCHEDULE_DATE is 
'A date of the roll-up schedule.';

alter table SAHARA.ROLLUP_SCHEDULE_DATE
   alter SCHEDULE_DATE add constraint C_RLLPSCHDLDT_SCHEDULEDATE check (SCHEDULE_DATE = DATE(SCHEDULE_DATE));

alter table SAHARA.ROLLUP_SCHEDULE_DATE
   add constraint PK_ROLLUP_SCHEDULE_DATE primary key (SCHEDULE_DATE);

grant SELECT on SAHARA.ROLLUP_SCHEDULE_DATE to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_SOURCE                                         */
/*==============================================================*/
create table SAHARA.ROLLUP_SOURCE 
(
   ROLLUP_SOURCE_KEY    numeric(10)                    not null default autoincrement,
   ROLLUP_SOURCE_TYPE_KEY numeric(10)                    not null,
   NAME                 varchar(128)                   not null,
   UPPER_NAME           varchar(128)                   not null,
   WEB_SERVER_HOST      varchar(128)                   not null,
   WEB_SERVER_PORT      numeric(5)                     not null,
   DB_HOST              varchar(128)                   not null,
   DB_PORT              numeric(5)                     not null,
   DB_PASSWORD          varchar(128)                   not null,
   DB_PASSWORD_2        varchar(128)                   null,
   UTC_ADJUSTMENT       integer                        not null default 0,
   ROLLUP_HOUR          numeric(2)                     not null default 0,
   ROLLUP_MINUTE        numeric(2)                     not null default 0,
   ROLLUP_TIME_AS_INTERVAL integer                        not null,
   ROLLUP_CLOCK         varchar(32)                    not null default 'SOURCE',
   ROLLUP_TIME_REVISION numeric(10)                    not null default 0,
   ROLLUP_NOW           char(1)                        not null default 'F',
   ENABLED              char(1)                        not null,
   ROLLUP_SOURCE_OID    varchar(512)                   not null,
   ROLLUP_SOURCE_VERSION varchar(32)                    not null,
   CLUSTER_ROLLUP_MARKER date                           null
);

comment on table SAHARA.ROLLUP_SOURCE is 
'This table contains information about source databases from which data is rolled up.

This table is not available for reporting or the SAHARA_READ role because it contains passwords.  Instead the database view ROLLUP_SOURCE_READ, which excludes the passwords, should be used in place of this table.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_SOURCE_KEY is 
'A unique database generated identifier for the roll-up source used as a surrogate key.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_SOURCE_TYPE_KEY is 
'The identifier of the roll-up source type.';

comment on column SAHARA.ROLLUP_SOURCE.NAME is 
'The external "name" for a roll-up source.  This is not a network name; it is a user-defined name by which the user uniquely identifies the roll-up source and is purely descriptive.  It may differ from the web server host name, database host name, or management server host name.

Valid roll-up source names contain only the letters A-Z (upper or lower case), the numbers 0-9, the underscore (_), dash (-), and period (.).  The name must be unique (ignoring character case).';

comment on column SAHARA.ROLLUP_SOURCE.UPPER_NAME is 
'This is the upper case form of the name column.  This column is used to enforce the constraint that the name must be unique (ignoring character case).';

comment on column SAHARA.ROLLUP_SOURCE.WEB_SERVER_HOST is 
'The  name or IP address of the web server host through which the objects from the roll-up source may be accessed.';

comment on column SAHARA.ROLLUP_SOURCE.WEB_SERVER_PORT is 
'The port number the web server through which the objects from the roll-up source may be accessed.';

comment on column SAHARA.ROLLUP_SOURCE.DB_HOST is 
'The name or IP address of the host on which the database server of roll-up source is running.';

comment on column SAHARA.ROLLUP_SOURCE.DB_PORT is 
'The database server port number of the roll-up source.';

comment on column SAHARA.ROLLUP_SOURCE.DB_PASSWORD is 
'The password with which to connect to the roll-up source database.

For CommandCentral Storage 4.x roll-up sources, this is the password for the sahara user in the vxcc database.
For CommandCentral Storage 5.x roll-up sources, this is the password for the sahara user in the ccstor database.';

comment on column SAHARA.ROLLUP_SOURCE.DB_PASSWORD_2 is 
'The optional second password with which to connect to the roll-up source database.

For CommandCentral Storage 4.x roll-up sources, this is is not used and will be null.
For CommandCentral Storage 5.x roll-up sources, this is the password for the sahara user in the ccstordm database.';

comment on column SAHARA.ROLLUP_SOURCE.UTC_ADJUSTMENT is 
'The adjustment, in seconds,  needed to be added to UTC time to obtain the roll-up source''s local time.  
This adjustment is used in calculating the next roll-up time for a roll-up source if the source is configured to use the source''s local time for scheduling roll-up.

This value is the "expected" UTC offset, or rather, the most recently discovered value.  The GUI populates it initially when creating the roll-up source, then it is updated at each roll-up.  When a local time changes, due to daylight saving for example, this value may be incorrect until a roll-up is processed or configuration GUI corrects the value.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_HOUR is 
'The hour of the day (0-23) at which roll-up should be performed.  Also see the comments on column ROLLUP_CLOCK.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_MINUTE is 
'The minute of the hour (0-59) at which roll-up should be performed.  Also see the comments on column ROLLUP_CLOCK.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_TIME_AS_INTERVAL is 
'The roll-up time stored as a time interval, in seconds,  calculated from the previous midnight.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_CLOCK is 
'An enumerated value that indicates which source of time to use when determining a roll-up time.  This is a little deceptive because the database server''s clock is actually used in all cases.  What is actually taken from these sources is a UTC time adjustment (which changes with daylight savings time) to apply to the database server time when calculating times for roll-up.  This gives the appearance of using the ER server clock (which may be on a different host that the database server) or the roll-up source clock.

Valid values:
- ER - the Enterprise Reporter''s server UTC offset is used
- SOURCE - the roll-up sources UTC offset source is used

This column has significant impact in determining the "roll-up day" for the roll-up source.  Refer to Enterprise Reporter documentation for a further explanation of a source''s "roll-up day".';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_TIME_REVISION is 
'A revision number used to distinguish changes in the roll-up time for the roll-up source.  Each time the roll-up time is, the revision is automatically incremented.  This becomes part of the roll-up hash used to distinguish one roll-up from another.

This value is used to generate a hash for each schedule/roll-up source/date instance to and is used to avoid performing duplicate roll-ups.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_NOW is 
'This boolean flag indicates that the roll-up source has been flagged to request a batch at the soonest available time.  It is set by the GUI to implement "roll-up now" functionality.';

comment on column SAHARA.ROLLUP_SOURCE.ENABLED is 
'A boolean flag that indicates if a roll-up source is current enabled for roll-up.  To remove a roll-up source from roll-up, it may be deleted or marked disabled.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_SOURCE_OID is 
'Unique identifier of the roll-up source.';

comment on column SAHARA.ROLLUP_SOURCE.ROLLUP_SOURCE_VERSION is 
'The VOM CS Version of the Rollup Source.';

comment on column SAHARA.ROLLUP_SOURCE.CLUSTER_ROLLUP_MARKER is 
'Use for cluster rollup.  Remember the last rollup historical record and pass the marker to extraction script.';

alter table SAHARA.ROLLUP_SOURCE
    add constraint C_ROLLUP_SOURCE check (-- don't allow the roll-up now flag to be true when enabled is false)
(ENABLED = 'T' or ROLLUP_NOW = 'F'));

alter table SAHARA.ROLLUP_SOURCE
   alter WEB_SERVER_PORT add constraint C_ROLLUPSOURCE_WEBSERVERPORT check (WEB_SERVER_PORT >= 1);

alter table SAHARA.ROLLUP_SOURCE
   alter DB_PORT add constraint C_ROLLUPSOURCE_DBPORT check (DB_PORT >= 1);

alter table SAHARA.ROLLUP_SOURCE
   alter ROLLUP_HOUR add constraint C_ROLLUPSOURCE_ROLLUPHOUR check (ROLLUP_HOUR between 0 and 23);

alter table SAHARA.ROLLUP_SOURCE
   alter ROLLUP_MINUTE add constraint C_ROLLUPSOURCE_ROLLUPMINUTE check (ROLLUP_MINUTE between 0 and 59);

alter table SAHARA.ROLLUP_SOURCE
   alter ROLLUP_CLOCK add constraint C_ROLLUPSOURCE_ROLLUPCLOCK check (ROLLUP_CLOCK in ('ER','SOURCE'));

alter table SAHARA.ROLLUP_SOURCE
   alter ROLLUP_TIME_REVISION add constraint C_ROLLUPSOURCE_RLLPTMRVSN check (ROLLUP_TIME_REVISION >= 0);

alter table SAHARA.ROLLUP_SOURCE
   alter ROLLUP_NOW add constraint C_ROLLUPSOURCE_ROLLUPNOW check (ROLLUP_NOW in ('T','F'));

alter table SAHARA.ROLLUP_SOURCE
   alter ENABLED add constraint C_ROLLUPSOURCE_ENABLED check (ENABLED in ('T','F'));

alter table SAHARA.ROLLUP_SOURCE
   add constraint PK_ROLLUP_SOURCE primary key (ROLLUP_SOURCE_KEY);

alter table SAHARA.ROLLUP_SOURCE
   add constraint AK_ROLLUPSOURCE_NAME unique (UPPER_NAME);

alter table SAHARA.ROLLUP_SOURCE
   add constraint AK_ROLLUPSOURCE_DB unique (DB_HOST, DB_PORT);

/*==============================================================*/
/* Index: I_ROLLUPSOURCE_ENABLED                                */
/*==============================================================*/
create index I_ROLLUPSOURCE_ENABLED on SAHARA.ROLLUP_SOURCE (
ENABLED ASC
);

grant SELECT on SAHARA.ROLLUP_SOURCE to SAHARA_APP;

/*==============================================================*/
/* Table: ROLLUP_SOURCE_TYPE                                    */
/*==============================================================*/
create table SAHARA.ROLLUP_SOURCE_TYPE 
(
   ROLLUP_SOURCE_TYPE_KEY numeric(10)                    not null default autoincrement,
   DISPLAY_NAME         varchar(64)                    not null,
   LOAD_PROCEDURE       varchar(61)                    not null
);

comment on table SAHARA.ROLLUP_SOURCE_TYPE is 
'This is a look-up table for information about roll-up source types.  It is populated at install/upgrade time.

A roll-up source type is a meta-type that defines the data sets that may be obtained from a roll-up source.

Examples from roll-up source types:
- CommandCentral Storage 4.3
- CommandCentral Storage 5.0
- CommandCentral Storage 5.0 MP1';

comment on column SAHARA.ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY is 
'A unique database generated identifier for the roll-up source type used as a surrogate key.';

comment on column SAHARA.ROLLUP_SOURCE_TYPE.DISPLAY_NAME is 
'The display name for the data source type.

Examples from roll-up source types:
- CommandCentral Storage 4.x
- CommandCentral Storage 5.0';

comment on column SAHARA.ROLLUP_SOURCE_TYPE.LOAD_PROCEDURE is 
'The fully qualified name of the stored procedure which implements the overall load from the staging tables to the data warehouse tables for the roll-up source type.';

alter table SAHARA.ROLLUP_SOURCE_TYPE
   add constraint PK_ROLLUP_SOURCE_TYPE primary key (ROLLUP_SOURCE_TYPE_KEY);

alter table SAHARA.ROLLUP_SOURCE_TYPE
   add constraint AK_ROLLUPSOURCETYPE_DNAME unique (DISPLAY_NAME);

grant SELECT on SAHARA.ROLLUP_SOURCE_TYPE to SAHARA_APP;

/*==============================================================*/
/* Table: RSOURCE_TYPE_DATA_SET_TYPE                            */
/*==============================================================*/
create table SAHARA.RSOURCE_TYPE_DATA_SET_TYPE 
(
   ROLLUP_SOURCE_TYPE_KEY numeric(10)                    not null,
   ROLLUP_DATA_SET_TYPE_KEY numeric(10)                    not null
);

comment on table SAHARA.RSOURCE_TYPE_DATA_SET_TYPE is 
'This is a join table between ROLLUP_SOURCE_TYPE and ROLLUP_DATA_SET_TYPE that defines what roll-up data set types are supports by a roll-up source type.  This table is populated at install and upgrade.';

comment on column SAHARA.RSOURCE_TYPE_DATA_SET_TYPE.ROLLUP_SOURCE_TYPE_KEY is 
'The identifier of the roll-up source type to associated with a data set type.';

comment on column SAHARA.RSOURCE_TYPE_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY is 
'The identifier of the roll-up data set type to associated with a roll-up source type.';

alter table SAHARA.RSOURCE_TYPE_DATA_SET_TYPE
   add constraint PK_RSOURCE_TYPE_DATA_SET_TYPE primary key (ROLLUP_SOURCE_TYPE_KEY, ROLLUP_DATA_SET_TYPE_KEY);

/*==============================================================*/
/* Index: I_DSET_TYPE_DATA_SET_TYPE_KEY                         */
/*==============================================================*/
create index I_DSET_TYPE_DATA_SET_TYPE_KEY on SAHARA.RSOURCE_TYPE_DATA_SET_TYPE (
ROLLUP_DATA_SET_TYPE_KEY ASC
);

grant SELECT on SAHARA.RSOURCE_TYPE_DATA_SET_TYPE to SAHARA_APP;

/*==============================================================*/
/* Table: RULE                                                  */
/*==============================================================*/
create table SAHARA.RULE 
(
   RULE_KEY             numeric(10)                    not null default autoincrement,
   RULE_CONTEXT         varchar(32)                    not null,
   GUI_HINT             long varchar                   null,
   RULE_TEXT            long varchar                   not null,
   SQL_CODE             long varchar                   not null
);

comment on table SAHARA.RULE is 
'This table contains user defined rules.';

comment on column SAHARA.RULE.RULE_KEY is 
'The unique database generated identifier for the rule.';

comment on column SAHARA.RULE.RULE_CONTEXT is 
'The context in which the rule is used.

See the schema report model notes: contexts, for further explanation.';

comment on column SAHARA.RULE.GUI_HINT is 
'An optional GUI hint for the rule.  This column is provided for any use the GUI defines.

For example: it may be preferred that the GUI present a specific type of rule in a structured way with specific fields rather than as free form text.  The GUI itself might then generate and store the underlying rule.  To simplify parsing the rule at a later time to populate the GUI when editing an existing rule, the GUI might choose to store the rule in a GUI specific XML format in the GUI hint.';

comment on column SAHARA.RULE.RULE_TEXT is 
'The rule text, before rule elements are substituted.  This is the rule as the user sees it for free form rules.';

comment on column SAHARA.RULE.SQL_CODE is 
'The SQL form of the rule.  This is the rule after rule element substitutions have been performed.';

alter table SAHARA.RULE
   add constraint PK_RULE primary key (RULE_KEY);

grant SELECT on SAHARA.RULE to SAHARA_APP;

/*==============================================================*/
/* Table: RULE_CONTEXT                                          */
/*==============================================================*/
create table SAHARA.RULE_CONTEXT 
(
   RULE_CONTEXT         varchar(32)                    not null,
   HELP_HTML            long varchar                   null
);

comment on table SAHARA.RULE_CONTEXT is 
'Rules may be used in multiple places in the sahara schema.  This is a look-up table that defines the context in which the associated rule configuration may be used.  This table is populated at install and upgrade.';

comment on column SAHARA.RULE_CONTEXT.RULE_CONTEXT is 
'The context in which the rule configuration is used.';

alter table SAHARA.RULE_CONTEXT
   add constraint PK_RULE_CONTEXT primary key (RULE_CONTEXT);

grant SELECT on SAHARA.RULE_CONTEXT to SAHARA_APP;

/*==============================================================*/
/* Table: RULE_ELEMENT                                          */
/*==============================================================*/
create table SAHARA.RULE_ELEMENT 
(
   RULE_CONTEXT         varchar(32)                    not null,
   SECTION_DISPLAY_NAME varchar(64)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   DESCRIPTION          long varchar                   null,
   TYPE                 varchar(32)                    not null,
   RULE_TOKEN_UPPER     varchar(128)                   not null,
   RULE_TOKEN           varchar(128)                   not null,
   SQL_CODE             varchar(1024)                  not null
);

comment on table SAHARA.RULE_ELEMENT is 
'Rule elements are items which may be substituted into a rule.  Typically, elements are database columns, but the implementation is not limited to just database columns.

To make it easier for the end-user to understand the meaning of a particular element, the element has a display name and descriptive text.  It also has corresponding SQL, such as a database column name.  The end-user forms a rule using the display name, but the implementation substitutes the actual SQL for the elements when evaluating the expression.

Rule elements are organized in the GUI in a two level tree view.  This table contains the information required for the leaf nodes, or elements, of the tree.  Each element belongs to a rule element section.

This table is populated at install and upgrade.';

comment on column SAHARA.RULE_ELEMENT.RULE_CONTEXT is 
'The context in which the rule element is used.

See the schema report model notes: contexts, for further explanation.';

comment on column SAHARA.RULE_ELEMENT.SECTION_DISPLAY_NAME is 
'The display name for the rule element section in which the rule element belongs.';

comment on column SAHARA.RULE_ELEMENT.DISPLAY_NAME is 
'The display name for the rule element.  The display name must be unique within the rule element''s section and is used to name the leaf nodes of the tree view of elements for building rules.';

comment on column SAHARA.RULE_ELEMENT.DESCRIPTION is 
'A description for the rule element.  The description is displayed in the GUI when the rule element is selected.';

comment on column SAHARA.RULE_ELEMENT.TYPE is 
'An enumerated value that identifies the type of rule element.  The type is used in forming and validating the rule.

Valid types:
- COLUMN - the element is a database column';

comment on column SAHARA.RULE_ELEMENT.RULE_TOKEN_UPPER is 
'Rule token names must be unique, independent of character case because a case insensitive replacement is used when converting rule text into SQL code.  This column is populated by a trigger and is part of a unique constraint (alternate key) to enforce this rule.';

comment on column SAHARA.RULE_ELEMENT.RULE_TOKEN is 
'The display text to be inserted into the rule when the user adds the element to a rule.';

comment on column SAHARA.RULE_ELEMENT.SQL_CODE is 
'The actual SQL code substituted for the rule element token when the rule is processed.';

alter table SAHARA.RULE_ELEMENT
   alter TYPE add constraint C_RULEELEMENT_TYPE check (TYPE in ('COLUMN'));

alter table SAHARA.RULE_ELEMENT
   alter RULE_TOKEN_UPPER add constraint C_RULEELEMENT_RULETOKENUPPER check (RULE_TOKEN_UPPER  = upper(RULE_TOKEN_UPPER));

alter table SAHARA.RULE_ELEMENT
   add constraint PK_RULE_ELEMENT primary key (RULE_CONTEXT, SECTION_DISPLAY_NAME, DISPLAY_NAME);

alter table SAHARA.RULE_ELEMENT
   add constraint AK_RULEELEMENT_RULETOKEN unique (RULE_CONTEXT, RULE_TOKEN_UPPER);

comment on index SAHARA.RULE_ELEMENT.AK_RULEELEMENT_RULETOKEN is 
'The rule token must be unique within the context regardless of character case.';

grant SELECT on SAHARA.RULE_ELEMENT to SAHARA_APP;

/*==============================================================*/
/* Table: RULE_ELEMENT_SECTION                                  */
/*==============================================================*/
create table SAHARA.RULE_ELEMENT_SECTION 
(
   RULE_CONTEXT         varchar(32)                    not null,
   SECTION_DISPLAY_NAME varchar(64)                    not null,
   DESCRIPTION          long varchar                   null
);

comment on table SAHARA.RULE_ELEMENT_SECTION is 
'Rule elements are items which may be substituted into a rule.  Typically, elements are database columns, but the implementation is not limited to just database columns.

To make it easier for the end-user to understand the meaning of a particular element, the element has a display name and descriptive text.  It also has corresponding SQL, such as a database column name.  The end-user forms a rule using the display name, but the implementation substitutes the actual SQL for the elements when evaluating the expression.

Rule elements are organized in the GUI in a two level tree view.  This table contains the information required for the root nodes, or sections, of the tree.  One or more rule elements go into a section.

This table is populated at install and upgrade.';

comment on column SAHARA.RULE_ELEMENT_SECTION.RULE_CONTEXT is 
'The context in which the rule element section is used.

See the schema report model notes: contexts, for further explanation.';

comment on column SAHARA.RULE_ELEMENT_SECTION.SECTION_DISPLAY_NAME is 
'The display name for the rule element section.  The section display name must be unique within the context and is used to name the upper level nodes of the tree view of elements for building rules.';

comment on column SAHARA.RULE_ELEMENT_SECTION.DESCRIPTION is 
'A description for the rule element section.  The description is displayed in the GUI when the rule element section is selected.';

alter table SAHARA.RULE_ELEMENT_SECTION
   add constraint PK_RULE_ELEMENT_SECTION primary key (RULE_CONTEXT, SECTION_DISPLAY_NAME);

grant SELECT on SAHARA.RULE_ELEMENT_SECTION to SAHARA_APP;

/*==============================================================*/
/* Table: SECURITY_GROUP                                        */
/*==============================================================*/
create table SAHARA.SECURITY_GROUP 
(
   SECURITY_GROUP_ID    unsigned integer               not null default autoincrement,
   ROLE_ID              unsigned integer               null,
   BROKER_ID            unsigned integer               not null,
   PRIVILEGE            bigint                         not null,
   NAME                 varchar(128)                   not null,
   DESCRIPT             varchar(1024)                  null
);

comment on table SAHARA.SECURITY_GROUP is 
'To Store Security Group Information  
Mapping between Security Group , Roles and Broker (combination of Domain name, Broker name and Domain type)
1.	Use of PRIVILEGE bit would be as following 
a.	BusinessView_ADMIN  1
b.	Storage_TIER_ADMIN  2
c.	CUSTOMATTRIBUTE_ADMIN  4';

alter table SAHARA.SECURITY_GROUP
   add constraint PK_SECURITY_GROUP primary key (SECURITY_GROUP_ID);

grant SELECT,ALTER,DELETE,INSERT,REFERENCES,UPDATE on SAHARA.SECURITY_GROUP to SAHARA_APP;

/*==============================================================*/
/* Table: SECURITY_GROUP_REL                                    */
/*==============================================================*/
create table SAHARA.SECURITY_GROUP_REL 
(
   SECURITY_GROUP_ID    unsigned integer               not null,
   BVIEW_ID             varchar(38)                    not null,
   ROLE_ID              unsigned integer               not null,
   BVIEW_KEY            numeric(10)                    null
);

comment on column SAHARA.SECURITY_GROUP_REL.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.SECURITY_GROUP_REL
   add constraint PK_SECURITY_GROUP_REL primary key (SECURITY_GROUP_ID, BVIEW_ID, ROLE_ID);

grant SELECT,ALTER,DELETE,INSERT,REFERENCES,UPDATE on SAHARA.SECURITY_GROUP_REL to SAHARA_APP;

/*==============================================================*/
/* Table: SECURITY_PATTERN                                      */
/*==============================================================*/
create table SAHARA.SECURITY_PATTERN 
(
   PATTERN_ID           varchar(512)                   not null
);

comment on table SAHARA.SECURITY_PATTERN is 
'To  Store Pattern for LDAP and AD  pattern in which they will send the Groups and domains information.';

alter table SAHARA.SECURITY_PATTERN
   add constraint PK_SECURITY_PATTERN primary key (PATTERN_ID);

grant SELECT,ALTER,DELETE,INSERT,REFERENCES,UPDATE on SAHARA.SECURITY_PATTERN to SAHARA_APP;

/*==============================================================*/
/* Table: SECURITY_ROLE                                         */
/*==============================================================*/
create table SAHARA.SECURITY_ROLE 
(
   ROLE_ID              unsigned integer               not null default autoincrement,
   NAME                 varchar(50)                    not null,
   DESCRIPT             varchar(1024)                  null
);

comment on table SAHARA.SECURITY_ROLE is 
'To Store Role Information ';

alter table SAHARA.SECURITY_ROLE
   add constraint PK_SECURITY_ROLE primary key (ROLE_ID);

grant SELECT,ALTER,DELETE,INSERT,REFERENCES,UPDATE on SAHARA.SECURITY_ROLE to SAHARA_APP;

/*==============================================================*/
/* Table: SERVICEGROUP                                          */
/*==============================================================*/
create table SAHARA.SERVICEGROUP 
(
   SERVICEGROUP_KEY     numeric(10)                    not null default autoincrement,
   SERVICEGROUP_OID     varchar(128)                   null,
   CLUSTER_KEY          numeric(10)                    null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   SG_TYPE              varchar(1024)                  null,
   SCOPE                varchar(16)                    null,
   FAILOVERPOLICY       varchar(128)                   null,
   IS_APPHA             varchar(1)                     null default 'F',
   FAMILY               varchar(16)                    null,
   T2_IS_CURRENT        varchar(1)                     not null default 'T',
   T2_IS_DELETED        varchar(1)                     not null default 'F',
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on column SAHARA.SERVICEGROUP.IS_APPHA is 
'Indicates whether a servicegroup belongs to AppHA instance.';

comment on column SAHARA.SERVICEGROUP.T2_IS_CURRENT is 
'Is a boolean book keeping flag that indicates if this is the "current" or latest record for the object.';

comment on column SAHARA.SERVICEGROUP.T2_IS_DELETED is 
'a boolean book keeping flag that indicates if the dimension row should be considered deleted.';

comment on column SAHARA.SERVICEGROUP.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA.SERVICEGROUP.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA.SERVICEGROUP.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA.SERVICEGROUP.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA.SERVICEGROUP
   alter IS_APPHA add constraint CKC_IS_APPHA_SERVICEG check (IS_APPHA is null or (IS_APPHA in ('T','F')));

alter table SAHARA.SERVICEGROUP
   alter T2_IS_CURRENT add constraint CKC_T2_IS_CURRENT_SERVICEG check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SERVICEGROUP
   alter T2_IS_DELETED add constraint CKC_T2_IS_DELETED_SERVICEG check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SERVICEGROUP
   add constraint PK_SERVICEGROUP primary key (SERVICEGROUP_KEY);

/*==============================================================*/
/* Index: I_SG_SGOID                                            */
/*==============================================================*/
create index I_SG_SGOID on SAHARA.SERVICEGROUP (
SERVICEGROUP_OID ASC
);

/*==============================================================*/
/* Index: I_SG_DISPLAYNAME                                      */
/*==============================================================*/
create index I_SG_DISPLAYNAME on SAHARA.SERVICEGROUP (
DISPLAY_NAME ASC
);

/*==============================================================*/
/* Index: I_SG_SGTYPE                                           */
/*==============================================================*/
create index I_SG_SGTYPE on SAHARA.SERVICEGROUP (
SG_TYPE ASC
);

/*==============================================================*/
/* Index: I_SG_SGSCOPE                                          */
/*==============================================================*/
create index I_SG_SGSCOPE on SAHARA.SERVICEGROUP (
SCOPE ASC
);

/*==============================================================*/
/* Index: I_SG_SGFAILOVERPOLICY                                 */
/*==============================================================*/
create index I_SG_SGFAILOVERPOLICY on SAHARA.SERVICEGROUP (
FAILOVERPOLICY ASC
);

/*==============================================================*/
/* Index: I_SG_T2ISCURSGKEY                                     */
/*==============================================================*/
create index I_SG_T2ISCURSGKEY on SAHARA.SERVICEGROUP (
SERVICEGROUP_KEY ASC,
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_SG_T2ISCUR_T2ISDEL_SGID                             */
/*==============================================================*/
create index I_SG_T2ISCUR_T2ISDEL_SGID on SAHARA.SERVICEGROUP (
SERVICEGROUP_OID ASC,
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC
);

/*==============================================================*/
/* Index: I_SG_T2ISDEL                                          */
/*==============================================================*/
create index I_SG_T2ISDEL on SAHARA.SERVICEGROUP (
T2_IS_DELETED ASC
);

/*==============================================================*/
/* Index: I_IS_APPHA                                            */
/*==============================================================*/
create index I_IS_APPHA on SAHARA.SERVICEGROUP (
IS_APPHA ASC
);

comment on index SAHARA.SERVICEGROUP.I_IS_APPHA is 
'Indicates whether a servicegroup belongs to AppHA instance.';

/*==============================================================*/
/* Index: SG_FAMILY                                             */
/*==============================================================*/
create index SG_FAMILY on SAHARA.SERVICEGROUP (
FAMILY ASC
);

grant SELECT on SAHARA.SERVICEGROUP to SAHARA_APP;

/*==============================================================*/
/* Table: SERVICEGROUP_                                         */
/*==============================================================*/
create table SAHARA_APP.SERVICEGROUP_ 
(
   SERVICEGROUP_OID     varchar(128)                   not null,
   CLUSTER_OID          varchar(128)                   null,
   DISPLAY_NAME         varchar(1024)                  null,
   SG_TYPE              varchar(1024)                  null,
   SCOPE                varchar(16)                    null,
   FAILOVERPOLICY       varchar(128)                   null,
   IS_APPHA             varchar(1)                     null default 'F',
   FAMILY               varchar(16)                    null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on table SAHARA_APP.SERVICEGROUP_ is 
'This is a staging table for loading data into warehouse table sahara.servicegroup.  Any data in this table is temporary and should not be accessed except by the data loading process.';

comment on column SAHARA_APP.SERVICEGROUP_.IS_APPHA is 
'Indicate whether servicegroup belongs to an AppHA instance';

comment on column SAHARA_APP.SERVICEGROUP_.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA_APP.SERVICEGROUP_.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA_APP.SERVICEGROUP_.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA_APP.SERVICEGROUP_.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA_APP.SERVICEGROUP_
   alter IS_APPHA add constraint CKC_IS_APPHA_SERVICEG check (IS_APPHA is null or (IS_APPHA in ('T','F')));

alter table SAHARA_APP.SERVICEGROUP_
   add constraint PK_SERVICEGROUP_ primary key (SERVICEGROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SERVICEGROUP_ to SAHARA;

/*==============================================================*/
/* Table: SERVICEGROUP_BVIEW_NODE                               */
/*==============================================================*/
create table SAHARA.SERVICEGROUP_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.SERVICEGROUP_BVIEW_NODE is 
'This is a join between servicegroup objects and business view nodes';

comment on column SAHARA.SERVICEGROUP_BVIEW_NODE.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.SERVICEGROUP_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint PK_SG_BVIEWNODE primary key (BVIEW_NODE_KEY, SERVICEGROUP_KEY);

/*==============================================================*/
/* Index: I_SG_BVIEWNODE_BVIEWKEY                               */
/*==============================================================*/
create index I_SG_BVIEWNODE_BVIEWKEY on SAHARA.SERVICEGROUP_BVIEW_NODE (
BVIEW_NODE_KEY ASC,
SERVICEGROUP_KEY ASC,
BVIEW_KEY ASC
);

comment on index SAHARA.SERVICEGROUP_BVIEW_NODE.I_SG_BVIEWNODE_BVIEWKEY is 
'An index on Business View Key for dimensional queries.';

grant SELECT on SAHARA.SERVICEGROUP_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: SERVICEGROUP_HIST_EVENTS                              */
/*==============================================================*/
create table SAHARA.SERVICEGROUP_HIST_EVENTS 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   PLANNED_FAILOVERS    bigint                         null default 0,
   UNPLANNED_AUTO_FAILOVERS bigint                         null default 0,
   UNPLANNED_MAN_FAILOVERS bigint                         null default 0,
   TOTAL_SUCCESS_FAILOVERS bigint                         null default 0,
   UNRESOLVED_PLANNED_FAILOVERS bigint                         null default 0,
   UNRESOLVED_UNPLANNED_AUTO_FAIL bigint                         null default 0,
   UNRESOLVED_UNPLANNED_MAN_FAILO bigint                         null default 0,
   TOTAL_FAILED_FAILOVERS bigint                         null default 0,
   TIME_ONLINE          bigint                         null default 0,
   TIME_OFFLINE         bigint                         null default 0,
   TIME_PARTIAL         bigint                         null default 0,
   TIME_FAULTED         bigint                         null default 0,
   TIME_TRANSIENTSTATES bigint                         null default 0,
   TIME_NOTKNOWN        bigint                         null default 0
);

comment on table SAHARA.SERVICEGROUP_HIST_EVENTS is 
'Store aggregate historical data for uptime analysis and failover analysis.  The data is stored in 24 hour granularity.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.PLANNED_FAILOVERS is 
'Failover that user planned and executed with online or switch operation';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNPLANNED_AUTO_FAILOVERS is 
'Cluster performed failover operation without user interference.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNPLANNED_MAN_FAILOVERS is 
'VCS requires user interference to clear the faults for performance and failover operation.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.TOTAL_SUCCESS_FAILOVERS is 
'Sum of planned failovers, unplanned auto failovers, unplaned manual failovers.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNRESOLVED_PLANNED_FAILOVERS is 
'Failed planned failovers';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNRESOLVED_UNPLANNED_AUTO_FAIL is 
'Failed unplanned automatic failovers';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.TOTAL_FAILED_FAILOVERS is 
'Sum of unresolved planned failovers, unresolved unplanned automatic failovers, unresolved unplanned manual failovers.';

alter table SAHARA.SERVICEGROUP_HIST_EVENTS
   add constraint PK_SERVICEGROUP_HIST_EVENTS primary key clustered (ROLLUP_BATCH_KEY, DATES_KEY, SERVICEGROUP_KEY);

/*==============================================================*/
/* Index: I_SGHISTEVENT_ROLLUPBATKEY                            */
/*==============================================================*/
create index I_SGHISTEVENT_ROLLUPBATKEY on SAHARA.SERVICEGROUP_HIST_EVENTS (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHISTEVENTS_DATESKEY                               */
/*==============================================================*/
create index I_SGHISTEVENTS_DATESKEY on SAHARA.SERVICEGROUP_HIST_EVENTS (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHISTEVENTS_SGKEY                                  */
/*==============================================================*/
create index I_SGHISTEVENTS_SGKEY on SAHARA.SERVICEGROUP_HIST_EVENTS (
SERVICEGROUP_KEY ASC
);

/*==============================================================*/
/* Table: SERVICEGROUP_HIST_EVENTS_                             */
/*==============================================================*/
create table SAHARA_APP.SERVICEGROUP_HIST_EVENTS_ 
(
   "DATE"               date                           not null,
   SERVICEGROUP_OID     varchar(128)                   not null,
   PLANNED_FAILOVERS    bigint                         null default 0,
   UNPLANNED_AUTO_FAILOVERS bigint                         null default 0,
   UNPLANNED_MAN_FAILOVERS bigint                         null default 0,
   TOTAL_SUCCESS_FAILOVERS bigint                         null default 0,
   UNRESOLVED_PLANNED_FAILOVERS bigint                         null default 0,
   UNRESOLVED_UNPLANNED_AUTO_FAIL bigint                         null default 0,
   UNRESOLVED_UNPLANNED_MAN_FAILO bigint                         null default 0,
   TOTAL_FAILED_FAILOVERS bigint                         null default 0,
   TIME_ONLINE          bigint                         null default 0,
   TIME_OFFLINE         bigint                         null default 0,
   TIME_PARTIAL         bigint                         null default 0,
   TIME_FAULTED         bigint                         null default 0,
   TIME_TRANSIENTSTATES bigint                         null default 0,
   TIME_NOTKNOWN        bigint                         null default 0
);

comment on table SAHARA_APP.SERVICEGROUP_HIST_EVENTS_ is 
'Store aggregate historical data for uptime analysis and failover analysis.  The data is stored in per day basis.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.PLANNED_FAILOVERS is 
'Failover that user planned and executed with online or switch operation';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNPLANNED_AUTO_FAILOVERS is 
'Cluster performed failover operation without user interference.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNPLANNED_MAN_FAILOVERS is 
'VCS requires user interference to clear the faults for performance and failover operation.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.TOTAL_SUCCESS_FAILOVERS is 
'Sum of planned failovers, unplanned auto failovers, unplaned manual failovers.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNRESOLVED_PLANNED_FAILOVERS is 
'Failed planned failovers';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNRESOLVED_UNPLANNED_AUTO_FAIL is 
'Failed unplanned automatic failovers';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.TOTAL_FAILED_FAILOVERS is 
'Sum of unresolved planned failovers, unresolved unplanned automatic failovers, unresolved unplanned manual failovers.';

alter table SAHARA_APP.SERVICEGROUP_HIST_EVENTS_
   add constraint PK_SERVICEGROUP_HIST_EVENTS_ primary key clustered ("DATE", SERVICEGROUP_OID);

/*==============================================================*/
/* Table: SERVICEGROUP_HOST                                     */
/*==============================================================*/
create table SAHARA.SERVICEGROUP_HOST 
(
   SERVICEGROUP_HOST_KEY numeric(10)                    not null default autoincrement,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   SERVICEGROUP_HOST_OID varchar(1024)                  not null,
   SERVICEGROUP_OID     varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on column SAHARA.SERVICEGROUP_HOST.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.SERVICEGROUP_HOST.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.SERVICEGROUP_HOST.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.SERVICEGROUP_HOST
   alter T2_IS_CURRENT add constraint C_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SERVICEGROUP_HOST
   alter T2_IS_DELETED add constraint C_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SERVICEGROUP_HOST
   add constraint PK_SERVICEGROUP_HOST primary key clustered (SERVICEGROUP_HOST_KEY);

/*==============================================================*/
/* Index: I_SGHOST_SGKEY                                        */
/*==============================================================*/
create index I_SGHOST_SGKEY on SAHARA.SERVICEGROUP_HOST (
SERVICEGROUP_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHOST_HOSTKEY                                      */
/*==============================================================*/
create index I_SGHOST_HOSTKEY on SAHARA.SERVICEGROUP_HOST (
HOST_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHOST_T2ISCURRENT                                  */
/*==============================================================*/
create index I_SGHOST_T2ISCURRENT on SAHARA.SERVICEGROUP_HOST (
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_SGHOST_T2ISDELETED                                  */
/*==============================================================*/
create index I_SGHOST_T2ISDELETED on SAHARA.SERVICEGROUP_HOST (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.SERVICEGROUP_HOST to SAHARA_APP;

/*==============================================================*/
/* Table: SERVICEGROUP_HOST_                                    */
/*==============================================================*/
create table SAHARA_APP.SERVICEGROUP_HOST_ 
(
   SERVICEGROUP_HOST_OID varchar(1024)                  not null,
   SERVICEGROUP_OID     varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   DISPLAY_NAME         varchar(1024)                  null
);

alter table SAHARA_APP.SERVICEGROUP_HOST_
   add constraint PK_SGHOST_ primary key (SERVICEGROUP_HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SERVICEGROUP_HOST_ to SAHARA;

/*==============================================================*/
/* Table: SERVICEGROUP_INVENTORY                                */
/*==============================================================*/
create table SAHARA.SERVICEGROUP_INVENTORY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   NUMBER_OF_RESOURCE   integer                        null
);

comment on column SAHARA.SERVICEGROUP_INVENTORY.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.SERVICEGROUP_INVENTORY.DATES_KEY is 
'A unique database generated identifier for the date record used as a surrogate key.';

alter table SAHARA.SERVICEGROUP_INVENTORY
   alter DATES_KEY add constraint CKC_DATES_KEY_SERVICEG check (DATES_KEY >= 1);

/*==============================================================*/
/* Index: I_SGINV_ROLLUPBATCHKEY                                */
/*==============================================================*/
create index I_SGINV_ROLLUPBATCHKEY on SAHARA.SERVICEGROUP_INVENTORY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_SGINV_DATESKEY                                      */
/*==============================================================*/
create index I_SGINV_DATESKEY on SAHARA.SERVICEGROUP_INVENTORY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_SGINV_SGKEY                                         */
/*==============================================================*/
create index I_SGINV_SGKEY on SAHARA.SERVICEGROUP_INVENTORY (
SERVICEGROUP_KEY ASC
);

grant SELECT on SAHARA.SERVICEGROUP_INVENTORY to SAHARA_APP;

/*==============================================================*/
/* Table: SERVICEGROUP_INVENTORY_                               */
/*==============================================================*/
create table SAHARA_APP.SERVICEGROUP_INVENTORY_ 
(
   SERVICEGROUP_OID     varchar(128)                   not null,
   NUMBER_OF_RESOURCE   integer                        null
);

alter table SAHARA_APP.SERVICEGROUP_INVENTORY_
   add constraint PK_SERVICEGROUP_INVENTORY_ primary key clustered (SERVICEGROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SERVICEGROUP_INVENTORY_ to SAHARA;

/*==============================================================*/
/* Table: "SESSION"                                             */
/*==============================================================*/
create table SAHARA."SESSION" 
(
   SESSION_KEY          numeric(10)                    not null default autoincrement,
   TYPE                 varchar(32)                    not null,
   USERS_KEY            numeric(10)                    null,
   START_TIME           datetime                       not null,
   END_TIME             datetime                       null,
   "COMMENT"            varchar(256)                   null
);

comment on table SAHARA."SESSION" is 
'This table provides a record of Enterprise Reporter sessions.  Sessions are periods of time during which logically related database operations are performed.  The provide a sort of auditing as well as a source of information about "who" or "what" is doing something or has done something in the product.';

comment on column SAHARA."SESSION".SESSION_KEY is 
'A unique database generated identifier for the session used as a surrogate key.';

comment on column SAHARA."SESSION".TYPE is 
'The type of session.  Valid values:
- ETL - roll-up processing session
- GUI - end-user GUI session
- IE - import/export session
- SERVER - ER server background processing';

comment on column SAHARA."SESSION".USERS_KEY is 
'The unique identifier of the user associated with the session.  This applies to GUI sessions.';

comment on column SAHARA."SESSION".START_TIME is 
'The timestamp (stored in UTC) of the session''s beginning time.';

comment on column SAHARA."SESSION".END_TIME is 
'The timestamp (stored in UTC) of the session''s ending time or null if the session is still active.';

comment on column SAHARA."SESSION"."COMMENT" is 
'This is used primarily with server session to note the reason for the session.';

alter table SAHARA."SESSION"
   alter TYPE add constraint C_SESSION_TYPE check (TYPE in ('GUI','ETL','IE','SERVER'));

alter table SAHARA."SESSION"
   add constraint PK_SESSION primary key (SESSION_KEY);

/*==============================================================*/
/* Index: I_SESSION_USERKEY                                     */
/*==============================================================*/
create index I_SESSION_USERKEY on SAHARA."SESSION" (
USERS_KEY ASC
);

grant SELECT,REFERENCES on SAHARA."SESSION" to SAHARA_APP;

/*==============================================================*/
/* Table: SF_DEPLOYED_LICENSES                                  */
/*==============================================================*/
create table SAHARA.SF_DEPLOYED_LICENSES 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   SF_LICENSE_KEY       numeric(10)                    not null,
   SF_PRODUCT_KEY       numeric(10)                    not null,
   SF_LICENSE_HOST_KEY  numeric(10)                    not null,
   SF_LICENSE_DEPLOYMENT_KEY numeric(10)                    not null,
   LICENSE_DEPLOYMENTS  numeric(10)                    not null,
   LICENSE_DEPLOYMENTS_EXPIRED numeric(10)                    not null,
   ACCOUNTED_DEPLOYMENTS numeric(10)                    not null,
   ACCOUNTED_DEPLOYMENTS_EXPIRED numeric(10)                    not null,
   KEYLESS_DEPLOYMENTS  numeric(10)                    not null,
   TOTAL_SPVU           numeric(10,2)                  null default 0
);

comment on table SAHARA.SF_DEPLOYED_LICENSES is 
'Fact table that contains number of license deployments across hosts for various SF products.';

comment on column SAHARA.SF_DEPLOYED_LICENSES.LICENSE_DEPLOYMENTS is 
'Total number of license deployments. This includes deployments of:
   - the demo licenses (that have not expired), 
   - expired/non-expired permanent/NFR licenses.
A license deployments is considered in this total irrespective of it''s accountable or not.';

comment on column SAHARA.SF_DEPLOYED_LICENSES.LICENSE_DEPLOYMENTS_EXPIRED is 
'Number of license deployments that are in expired state. Any demo licenses that expired are not counted here.';

comment on column SAHARA.SF_DEPLOYED_LICENSES.ACCOUNTED_DEPLOYMENTS is 
'Number of accountable license deployments. This may be less than or equal to total license deployments.
A license deployment is accountable if it represents the highest product/Edition that is used on a host. For example, if you have SF Basic and SF Enterprise licenses installed on a host, total number of deployments are 2. But, customer is only using Enterprise edition, then only that license is considered accountable.';

comment on column SAHARA.SF_DEPLOYED_LICENSES.ACCOUNTED_DEPLOYMENTS_EXPIRED is 
'Number of accountable license deployments that expired. Expired demo licenses are not counted here.';

comment on column SAHARA.SF_DEPLOYED_LICENSES.TOTAL_SPVU is 
'Total SPVU per product.';

grant SELECT on SAHARA.SF_DEPLOYED_LICENSES to SAHARA_APP;

/*==============================================================*/
/* Table: SF_DEPLOYED_LICENSES_                                 */
/*==============================================================*/
create table SAHARA_APP.SF_DEPLOYED_LICENSES_ 
(
   SF_LICENSE_OID       varchar(512)                   not null,
   SF_PRODUCT_OID       varchar(512)                   not null,
   SF_LICENSE_HOST_OID  varchar(512)                   not null,
   SF_LICENSE_DEPLOYMENT_OID varchar(512)                   null,
   LICENSE_DEPLOYMENTS  numeric(10)                    not null,
   LICENSE_DEPLOYMENTS_EXPIRED numeric(10)                    not null,
   ACCOUNTED_DEPLOYMENTS numeric(10)                    not null,
   ACCOUNTED_DEPLOYMENTS_EXPIRED numeric(10)                    not null,
   KEYLESS_DEPLOYMENTS  numeric(10)                    not null,
   TOTAL_SPVU           numeric(10,2)                  null default 0
);

comment on table SAHARA_APP.SF_DEPLOYED_LICENSES_ is 
'This is a staging table for loading data into warehouse table SAHARA.SF_DEPLOYED_LICENSES fact table. Any data in this table is temporary and should not be accessed except by the data loading process.
';

alter table SAHARA_APP.SF_DEPLOYED_LICENSES_
   add constraint PK_SF_DEPLOYED_LICENSES_ primary key (SF_LICENSE_OID, SF_PRODUCT_OID, SF_LICENSE_HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SF_DEPLOYED_LICENSES_ to SAHARA;

/*==============================================================*/
/* Table: SF_LICENSE                                            */
/*==============================================================*/
create table SAHARA.SF_LICENSE 
(
   SF_LICENSE_KEY       numeric(10)                    not null default autoincrement,
   SF_LICENSE_OID       varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   TYPE                 varchar(128)                   null,
   "KEY"                varchar(128)                   null,
   IS_SITE_LICENSE      char(1)                        null,
   SERIAL_NUMBER        varchar(128)                   null,
   IS_EXPIRED           char(1)                        null,
   IS_KEYLESS           char(1)                        null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.SF_LICENSE is 
'Represents a Storage Foundation license discovered on a Host. A given license may be deployed on many hosts, but this table will only contain one record per unique license key discovered in the datacenter.
';

comment on column SAHARA.SF_LICENSE.TYPE is 
'Type of the license. Values are: "DEMO", "PERMANENT", "NFR".
';

comment on column SAHARA.SF_LICENSE."KEY" is 
'The license key (This is not database key).';

comment on column SAHARA.SF_LICENSE.IS_SITE_LICENSE is 
'Is this a site license?';

comment on column SAHARA.SF_LICENSE.SERIAL_NUMBER is 
'Serial number of the license.';

comment on column SAHARA.SF_LICENSE.IS_EXPIRED is 
'Indicates if the license in is expired state on at least one of the hosts where it is deployed.

We also track the expiry state of a license on per host basis in SF_LICENSE_DEPLOYMENTE table.';

comment on column SAHARA.SF_LICENSE.IS_KEYLESS is 
'Is this a SF keyless license?';

comment on column SAHARA.SF_LICENSE.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.SF_LICENSE.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.SF_LICENSE
   alter IS_SITE_LICENSE add constraint CKC_IS_SITE_LICENSE_SF_LICEN check (IS_SITE_LICENSE is null or (IS_SITE_LICENSE in ('T','F')));

alter table SAHARA.SF_LICENSE
   alter IS_EXPIRED add constraint CKC_IS_EXPIRED_SF_LICEN check (IS_EXPIRED is null or (IS_EXPIRED in ('T','F')));

alter table SAHARA.SF_LICENSE
   alter IS_KEYLESS add constraint CKC_IS_KEYLESS_SF_LICEN check (IS_KEYLESS is null or (IS_KEYLESS in ('T','F')));

alter table SAHARA.SF_LICENSE
   alter T2_IS_CURRENT add constraint C_SF_LICENSE_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SF_LICENSE
   alter T2_IS_DELETED add constraint C_SF_LICENSE_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SF_LICENSE
   add constraint PK_SF_LICENSE primary key (SF_LICENSE_KEY);

/*==============================================================*/
/* Index: I_LICENSE_LICENSEOID                                  */
/*==============================================================*/
create index I_LICENSE_LICENSEOID on SAHARA.SF_LICENSE (
SF_LICENSE_OID ASC
);

comment on index SAHARA.SF_LICENSE.I_LICENSE_LICENSEOID is 
'An index on SF_License Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_LICENSE_T2SCRRNTT2SDLTDD                            */
/*==============================================================*/
create index I_LICENSE_T2SCRRNTT2SDLTDD on SAHARA.SF_LICENSE (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
SF_LICENSE_OID ASC
);

comment on index SAHARA.SF_LICENSE.I_LICENSE_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and SF_LICENSE Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_LICENSE_T2ISCURRENTLICNSEKEY                        */
/*==============================================================*/
create index I_LICENSE_T2ISCURRENTLICNSEKEY on SAHARA.SF_LICENSE (
T2_IS_CURRENT ASC,
SF_LICENSE_KEY ASC
);

comment on index SAHARA.SF_LICENSE.I_LICENSE_T2ISCURRENTLICNSEKEY is 
'An index on Type 2 - Is Current and SF_License Key for use in processing type 2 updates during data loading.';

grant SELECT on SAHARA.SF_LICENSE to SAHARA_APP;

/*==============================================================*/
/* Table: SF_LICENSE_                                           */
/*==============================================================*/
create table SAHARA_APP.SF_LICENSE_ 
(
   SF_LICENSE_OID       varchar(512)                   not null,
   TYPE                 varchar(128)                   null,
   "KEY"                varchar(128)                   null,
   IS_SITE_LICENSE      char(1)                        null,
   SERIAL_NUMBER        varchar(128)                   null,
   IS_EXPIRED           char(1)                        null,
   IS_KEYLESS           char(1)                        null
);

comment on table SAHARA_APP.SF_LICENSE_ is 
'This is a staging table for loading data into warehouse table SAHARA.SF_LICENSE. Any data in this table is temporary and should not be accessed except by the data loading process.
';

comment on column SAHARA_APP.SF_LICENSE_.TYPE is 
'Type of the license. Values are: "DEMO", "PERMANENT", "NFR".';

comment on column SAHARA_APP.SF_LICENSE_."KEY" is 
'The license key (This is not database key).';

comment on column SAHARA_APP.SF_LICENSE_.IS_SITE_LICENSE is 
'Is this a site license?';

comment on column SAHARA_APP.SF_LICENSE_.SERIAL_NUMBER is 
'Serial number of the license.';

comment on column SAHARA_APP.SF_LICENSE_.IS_EXPIRED is 
'Indicates if the license in is expired state on at least one of the hosts where it is deployed.

We also track the expiry state of a license on per host basis in SF_LICENSE_DEPLOYMENTE table.';

comment on column SAHARA_APP.SF_LICENSE_.IS_KEYLESS is 
'Is this a SF keyless license?';

alter table SAHARA_APP.SF_LICENSE_
   add constraint PK_SF_LICENSE_ primary key (SF_LICENSE_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SF_LICENSE_ to SAHARA;

/*==============================================================*/
/* Table: SF_LICENSE_DEPLOYMENT                                 */
/*==============================================================*/
create table SAHARA.SF_LICENSE_DEPLOYMENT 
(
   SF_LICENSE_DEPLOYMENT_KEY numeric(10)                    not null default autoincrement,
   SF_LICENSE_DEPLOYMENT_OID varchar(512)                   not null,
   SF_LICENSE_KEY       numeric(10)                    not null,
   SF_PRODUCT_KEY       numeric(10)                    not null,
   SF_LICENSE_HOST_KEY  numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   EXPIRY_DATE          date                           null,
   IS_EXPIRED           char(1)                        not null,
   IS_ACCOUNTABLE       char(1)                        not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.SF_LICENSE_DEPLOYMENT is 
'This table represents a license, issued for a product, deployed on a specific host. A license may be deployed on multiple hosts. There can be many depolyments of a specific license key on different hosts.

When we a say a license expired, what it really means is that the deployment of a license on a specific host expired. The same license key deployed on a different host may still be valid. This is why we will store ExpiryDate and IsExpired values in this table, instead of SF_LICENSE table.';

comment on column SAHARA.SF_LICENSE_DEPLOYMENT.EXPIRY_DATE is 
'Expiry date of a license on a given host.';

comment on column SAHARA.SF_LICENSE_DEPLOYMENT.IS_EXPIRED is 
'Is the license deployment in expired state? ';

comment on column SAHARA.SF_LICENSE_DEPLOYMENT.IS_ACCOUNTABLE is 
'Set to ''T'' if the license deployent is accountable. A license is considered accountable if it represents the highest edition of the product that is currently used on a given host. For example, you may have licenses deployed for both SF basic and SF Enterprise on a given host, but the customer is only using SF Enterprise then we will ony count one of the license deployments as accountable.
';

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   alter IS_EXPIRED add constraint CKC_IS_EXPIRED_SF_LICEN check (IS_EXPIRED in ('T','F'));

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   alter IS_ACCOUNTABLE add constraint CKC_IS_ACCOUNTABLE_SF_LICEN check (IS_ACCOUNTABLE in ('T','F'));

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   alter T2_IS_CURRENT add constraint CKC_T2_IS_CURRENT_SF_LICEN check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   alter T2_IS_DELETED add constraint CKC_T2_IS_DELETED_SF_LICEN check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   add constraint PK_SF_LICENSE_DEPLOYMENT primary key (SF_LICENSE_DEPLOYMENT_KEY);

/*==============================================================*/
/* Index: I_LICENSEDEPT_LICENSEDEPTOID                          */
/*==============================================================*/
create index I_LICENSEDEPT_LICENSEDEPTOID on SAHARA.SF_LICENSE_DEPLOYMENT (
SF_LICENSE_DEPLOYMENT_OID ASC
);

comment on index SAHARA.SF_LICENSE_DEPLOYMENT.I_LICENSEDEPT_LICENSEDEPTOID is 
'An index on SF_LICENSE_DEPLOYMENT Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_LICENSEDEPT_T2ISCURRENTLCDK                         */
/*==============================================================*/
create index I_LICENSEDEPT_T2ISCURRENTLCDK on SAHARA.SF_LICENSE_DEPLOYMENT (
T2_IS_CURRENT ASC,
SF_LICENSE_DEPLOYMENT_KEY ASC
);

comment on index SAHARA.SF_LICENSE_DEPLOYMENT.I_LICENSEDEPT_T2ISCURRENTLCDK is 
'An index on Type 2 - Is Current and SF_LICENSE_DEPLOYMENT Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_LICENSEDEPT_T2SCRRNTT2SDLTDD                        */
/*==============================================================*/
create index I_LICENSEDEPT_T2SCRRNTT2SDLTDD on SAHARA.SF_LICENSE_DEPLOYMENT (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
SF_LICENSE_DEPLOYMENT_OID ASC
);

comment on index SAHARA.SF_LICENSE_DEPLOYMENT.I_LICENSEDEPT_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and SF_LICENSE_DEPLOYMENT Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

grant SELECT on SAHARA.SF_LICENSE_DEPLOYMENT to SAHARA_APP;

/*==============================================================*/
/* Table: SF_LICENSE_DEPLOYMENT_                                */
/*==============================================================*/
create table SAHARA_APP.SF_LICENSE_DEPLOYMENT_ 
(
   SF_LICENSE_DEPLOYMENT_OID varchar(512)                   not null,
   SF_LICENSE_OID       varchar(512)                   not null,
   SF_PRODUCT_OID       varchar(512)                   not null,
   SF_LICENSE_HOST_OID  varchar(512)                   not null,
   EXPIRY_DATE          date                           null,
   IS_EXPIRED           char(1)                        not null,
   IS_ACCOUNTABLE       char(1)                        not null
);

comment on table SAHARA_APP.SF_LICENSE_DEPLOYMENT_ is 
'This is a staging table for loading data into warehouse table SAHARA.SF_LICENSE_DEPLOYMENT. Any data in this table is temporary and should not be accessed except by the data loading process.
';

comment on column SAHARA_APP.SF_LICENSE_DEPLOYMENT_.IS_ACCOUNTABLE is 
'Set to ''T'' if the license deployent is accountable. A license is considered accountable if it represents the highest edition of the product that is currently used on a given host. For example, you may have licenses deployed for both SF basic and SF Enterprise on a given host, but the customer is only using SF Enterprise then we will ony count one of the license deployments as accountable.
';

alter table SAHARA_APP.SF_LICENSE_DEPLOYMENT_
   add constraint PK_SF_LICENSE_DEPLOYMENT_ primary key (SF_LICENSE_DEPLOYMENT_OID);

/*==============================================================*/
/* Index: I_LICENSEDEPT_LICENSEOID                              */
/*==============================================================*/
create index I_LICENSEDEPT_LICENSEOID on SAHARA_APP.SF_LICENSE_DEPLOYMENT_ (
SF_LICENSE_OID ASC
);

comment on index SAHARA_APP.SF_LICENSE_DEPLOYMENT_.I_LICENSEDEPT_LICENSEOID is 
'An index on SF_License_OID to speed up processing during data loading.';

/*==============================================================*/
/* Index: I_LICENSEDEPT_PRODUCTOID                              */
/*==============================================================*/
create index I_LICENSEDEPT_PRODUCTOID on SAHARA_APP.SF_LICENSE_DEPLOYMENT_ (
SF_PRODUCT_OID ASC
);

comment on index SAHARA_APP.SF_LICENSE_DEPLOYMENT_.I_LICENSEDEPT_PRODUCTOID is 
'An index on SF_PRODUCT_OID to speed up processing during data loading.';

/*==============================================================*/
/* Index: I_LICENSEDEPT_LICENSEHOSTOID                          */
/*==============================================================*/
create index I_LICENSEDEPT_LICENSEHOSTOID on SAHARA_APP.SF_LICENSE_DEPLOYMENT_ (
SF_LICENSE_HOST_OID ASC
);

comment on index SAHARA_APP.SF_LICENSE_DEPLOYMENT_.I_LICENSEDEPT_LICENSEHOSTOID is 
'An index on SF_LICENSE_HOST_OID to speed up processing during data loading.';

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SF_LICENSE_DEPLOYMENT_ to SAHARA;

/*==============================================================*/
/* Table: SF_LICENSE_HOST                                       */
/*==============================================================*/
create table SAHARA.SF_LICENSE_HOST 
(
   SF_LICENSE_HOST_KEY  numeric(10)                    not null default autoincrement,
   SF_LICENSE_HOST_OID  varchar(512)                   not null,
   HOST_KEY             numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   ID                   varchar(38)                    null,
   NAME                 varchar(255)                   null,
   MODEL                varchar(128)                   null,
   PROCESSOR_TYPE       varchar(64)                    null,
   NUM_PROCESSORS       numeric(4)                     null,
   PLATFORM             varchar(16)                    null,
   SERVER_PRICE_TIER    varchar(16)                    null,
   PROCESSOR_PRICE_TIER varchar(16)                    null,
   NUM_PROCESSOR_PRICE_TIER varchar(16)                    null,
   OS_PRICE_TIER        varchar(32)                    null,
   ASSIGNED_PRICE_TIER  varchar(16)                    null,
   OS_EDITION           varchar(128)                   null,
   OS                   varchar(128)                   null,
   HAS_ENTERPRISE_EDITION char(1)                        null,
   SPVU                 numeric(10,2)                  null default 0,
   NCORE                integer                        null default 0,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.SF_LICENSE_HOST is 
'Represents a host that contains one or more SF licenses deployed on it. This host may have zero or more SF products installed on it. Every host represented in this table is expected to have a entry in the HOST table as well.';

comment on column SAHARA.SF_LICENSE_HOST.ID is 
'Object Identifier ofthe Host object discovered. This ID willmatch with the HOST_OID column in the SAHARA.HOST table.';

comment on column SAHARA.SF_LICENSE_HOST.NAME is 
'Name of the host.';

comment on column SAHARA.SF_LICENSE_HOST.MODEL is 
'Model of the host. 
Example: Sun Microsystems sun4v SPARC Enterprise T5220';

comment on column SAHARA.SF_LICENSE_HOST.PROCESSOR_TYPE is 
'Type of the processors in the host.
Example: UltraSPARC-T2 (1167MHz)';

comment on column SAHARA.SF_LICENSE_HOST.NUM_PROCESSORS is 
'Number of processors in the host.';

comment on column SAHARA.SF_LICENSE_HOST.PLATFORM is 
'OS platform of the host. 
Example:SunOS';

comment on column SAHARA.SF_LICENSE_HOST.SERVER_PRICE_TIER is 
'Server price tier.
This is evaluated for each host by VOM MS based on prive tiering information provided. ';

comment on column SAHARA.SF_LICENSE_HOST.PROCESSOR_PRICE_TIER is 
'Processor Tier of the host. Evaluated and set for each host by VOM MS based on tiering information published.';

comment on column SAHARA.SF_LICENSE_HOST.NUM_PROCESSOR_PRICE_TIER is 
'Tier based on number of processors.';

comment on column SAHARA.SF_LICENSE_HOST.OS_PRICE_TIER is 
'Price tier based on OS Edition. This is applicable only to Windows machines.';

comment on column SAHARA.SF_LICENSE_HOST.ASSIGNED_PRICE_TIER is 
'Price tier manually assigned for this Host.';

comment on column SAHARA.SF_LICENSE_HOST.OS_EDITION is 
'OS Edition. Applicable only to Windows.';

comment on column SAHARA.SF_LICENSE_HOST.OS is 
'Operating system version of the host.';

comment on column SAHARA.SF_LICENSE_HOST.SPVU is 
'SPVU';

comment on column SAHARA.SF_LICENSE_HOST.NCORE is 
'Total number of cores in a host.';

comment on column SAHARA.SF_LICENSE_HOST.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.SF_LICENSE_HOST.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.SF_LICENSE_HOST
   alter T2_IS_CURRENT add constraint C_LIC_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SF_LICENSE_HOST
   alter T2_IS_DELETED add constraint C_LIC_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SF_LICENSE_HOST
   add constraint PK_SF_LICENSE_HOST primary key (SF_LICENSE_HOST_KEY);

/*==============================================================*/
/* Index: I_LICENSEHOST_LICENSEHOSTOID                          */
/*==============================================================*/
create index I_LICENSEHOST_LICENSEHOSTOID on SAHARA.SF_LICENSE_HOST (
SF_LICENSE_HOST_OID ASC
);

/*==============================================================*/
/* Index: I_LICENSEHOST_T2SCRRNTT2SDLTDD                        */
/*==============================================================*/
create index I_LICENSEHOST_T2SCRRNTT2SDLTDD on SAHARA.SF_LICENSE_HOST (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
SF_LICENSE_HOST_OID ASC
);

comment on index SAHARA.SF_LICENSE_HOST.I_LICENSEHOST_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and SF_LICENSE_HOST Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_LICENSEHOST_T2SCRRNTLCNSHSTK                        */
/*==============================================================*/
create index I_LICENSEHOST_T2SCRRNTLCNSHSTK on SAHARA.SF_LICENSE_HOST (
T2_IS_CURRENT ASC,
SF_LICENSE_HOST_KEY ASC
);

comment on index SAHARA.SF_LICENSE_HOST.I_LICENSEHOST_T2SCRRNTLCNSHSTK is 
'An index on Type 2 - Is Current and SF_LICENSE_HOST Key for use in processing type 2 updates during data loading.';

grant SELECT on SAHARA.SF_LICENSE_HOST to SAHARA_APP;

/*==============================================================*/
/* Table: SF_LICENSE_HOST_                                      */
/*==============================================================*/
create table SAHARA_APP.SF_LICENSE_HOST_ 
(
   SF_LICENSE_HOST_OID  varchar(512)                   not null,
   HOST_OID             varchar(512)                   not null,
   ID                   varchar(38)                    null,
   NAME                 varchar(255)                   null,
   MODEL                varchar(128)                   null,
   PROCESSOR_TYPE       varchar(64)                    null,
   NUM_PROCESSORS       numeric(4)                     null,
   PLATFORM             varchar(16)                    null,
   SERVER_PRICE_TIER    varchar(16)                    null,
   PROCESSOR_PRICE_TIER varchar(16)                    null,
   NUM_PROCESSOR_PRICE_TIER varchar(16)                    null,
   OS_PRICE_TIER        varchar(32)                    null,
   ASSIGNED_PRICE_TIER  varchar(16)                    null,
   OS_EDITION           varchar(128)                   null,
   OS                   varchar(128)                   null,
   HAS_ENTERPRISE_EDITION char(1)                        null,
   SPVU                 numeric(10,2)                  null default 0,
   NCORE                integer                        null default 0
);

comment on table SAHARA_APP.SF_LICENSE_HOST_ is 
'This is a staging table for loading data into warehouse table SAHARA.SF_LICENSE_HOST. Any data in this table is temporary and should not be accessed except by the data loading process.
';

alter table SAHARA_APP.SF_LICENSE_HOST_
   add constraint PK_SF_LICENSE_HOST_ primary key (SF_LICENSE_HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SF_LICENSE_HOST_ to SAHARA;

/*==============================================================*/
/* Table: SF_PRODUCT                                            */
/*==============================================================*/
create table SAHARA.SF_PRODUCT 
(
   SF_PRODUCT_KEY       numeric(10)                    not null default autoincrement,
   SF_PRODUCT_OID       varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   PRODUCT_NAME         varchar(512)                   not null,
   SHORT_NAME           varchar(512)                   not null,
   PRODUCT_CATEGORY     varchar(128)                   null,
   PRODUCT_EDITION      varchar(512)                   not null,
   PRODUCT_VERSION      varchar(512)                   not null,
   PLATFORM             varchar(16)                    null,
   LICENSE_TYPE         varchar(128)                   null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.SF_PRODUCT is 
'Represents a unique SF product for which a license is deployed on hosts. The table will have one row per <product, version, Edition> combination.
For example, let''s say  you have a product called SF and there are multiple versions of the product out there. For each versin of the produt, a new license key is supplied. SO, you will see license keys for SF 4.0, SF 5.0 etc. We sould like to capture each such combination as a product.
';

comment on column SAHARA.SF_PRODUCT.PRODUCT_NAME is 
'Name of the product. This may repeat in the table, because you may have a product with multiple versions out there.';

comment on column SAHARA.SF_PRODUCT.SHORT_NAME is 
'Short name of the product, like SF for Storage Foundation.';

comment on column SAHARA.SF_PRODUCT.PRODUCT_CATEGORY is 
'Category of the product. Examples: FS/VM/VCS/OTHER.';

comment on column SAHARA.SF_PRODUCT.PRODUCT_EDITION is 
'Edition of SF product. Examples: Basic, Standard, Enterprise.';

comment on column SAHARA.SF_PRODUCT.PRODUCT_VERSION is 
'Version of the product. Examples: 4.0/5.0/5.1 etc.';

comment on column SAHARA.SF_PRODUCT.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.SF_PRODUCT.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.SF_PRODUCT
   alter T2_IS_CURRENT add constraint C_PRODUCT_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SF_PRODUCT
   alter T2_IS_DELETED add constraint C_PRODUCT_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SF_PRODUCT
   add constraint PK_SF_PRODUCT primary key (SF_PRODUCT_KEY);

/*==============================================================*/
/* Index: I_PRODUCT_PRODUCTOID                                  */
/*==============================================================*/
create index I_PRODUCT_PRODUCTOID on SAHARA.SF_PRODUCT (
SF_PRODUCT_OID ASC
);

comment on index SAHARA.SF_PRODUCT.I_PRODUCT_PRODUCTOID is 
'An index on SF_Product Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_PRODUCT_T2SCRRNTT2SDLTDD                            */
/*==============================================================*/
create index I_PRODUCT_T2SCRRNTT2SDLTDD on SAHARA.SF_PRODUCT (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
SF_PRODUCT_OID ASC
);

comment on index SAHARA.SF_PRODUCT.I_PRODUCT_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and SF_PRODUCT Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_PRODUCT_T2ISCURRENTPRDUCTKEY                        */
/*==============================================================*/
create index I_PRODUCT_T2ISCURRENTPRDUCTKEY on SAHARA.SF_PRODUCT (
T2_IS_CURRENT ASC,
SF_PRODUCT_KEY ASC
);

comment on index SAHARA.SF_PRODUCT.I_PRODUCT_T2ISCURRENTPRDUCTKEY is 
'An index on Type 2 - Is Current and SF_License Key for use in processing type 2 updates during data loading.';

grant SELECT on SAHARA.SF_PRODUCT to SAHARA_APP;

/*==============================================================*/
/* Table: SF_PRODUCT_                                           */
/*==============================================================*/
create table SAHARA_APP.SF_PRODUCT_ 
(
   SF_PRODUCT_OID       varchar(512)                   not null,
   PRODUCT_NAME         varchar(512)                   not null,
   SHORT_NAME           varchar(512)                   not null,
   PRODUCT_CATEGORY     varchar(128)                   null,
   PRODUCT_EDITION      varchar(512)                   not null,
   PRODUCT_VERSION      varchar(512)                   not null,
   PLATFORM             varchar(16)                    null,
   LICENSE_TYPE         varchar(128)                   null
);

comment on table SAHARA_APP.SF_PRODUCT_ is 
'This is a staging table for loading data into warehouse table SAHARA.SF_PRODUCT. Any data in this table is temporary and should not be accessed except by the data loading process.';

alter table SAHARA_APP.SF_PRODUCT_
   add constraint PK_SF_PRODUCT_ primary key (SF_PRODUCT_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SF_PRODUCT_ to SAHARA;

/*==============================================================*/
/* Table: SRC_CUSTOM_ATTRIBUTE_BASE                             */
/*==============================================================*/
create table SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE 
(
   CUSTOM_ATTRIBUTE_CONTEXT varchar(32)                    not null,
   ROLLUP_SOURCE_KEY    numeric(10)                    not null,
   NAME                 varchar(256)                   not null
);

comment on table SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE is 
'This table contains the names of custom attributes most recently discovered from each roll-up source.

This table exists primary to implement views that draw data from it.  Consider using view SRC_CUSTOM_ATTRIBUTE or UNUSED_SRC_CUSTOM_ATTRIBUTE rather than querying directly from this table.

This table is used in building and configuring Enterprise Reporter custom attributes; it is not intended in any way to be used for reporting.';

comment on column SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE.CUSTOM_ATTRIBUTE_CONTEXT is 
'The context in which the custom attributes are used.';

comment on column SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE.ROLLUP_SOURCE_KEY is 
'The identifier of the roll-up source from which the custom attribute was collected.  This is a "weak link" in that it is not foreign key enforced.  When the roll-up source is physically deleted, this key will no longer point to a value.

Reports and other parts of the application should not rely on this link; it exists primarily to be able to identify the specific roll-up source for book-keeping purposes.';

comment on column SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE.NAME is 
'The name of the custom attribute.';

alter table SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE
   add constraint PK_SRC_CUSTOM_ATTRIBUTE_BASE primary key (CUSTOM_ATTRIBUTE_CONTEXT, ROLLUP_SOURCE_KEY, NAME);

/*==============================================================*/
/* Index: I_SRCCUSTOMATTRBASE_CTXNAME                           */
/*==============================================================*/
create index I_SRCCUSTOMATTRBASE_CTXNAME on SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE (
CUSTOM_ATTRIBUTE_CONTEXT ASC,
NAME ASC
);

grant SELECT on SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE to SAHARA_APP;

/*==============================================================*/
/* Table: SWITCH                                                */
/*==============================================================*/
create table SAHARA.SWITCH 
(
   SWITCH_KEY           numeric(10)                    not null default autoincrement,
   SWITCH_OID           varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   VENDOR               varchar(256)                   null,
   DISCOVERY_STATE      varchar(32)                    null,
   FIRMWARE_VERSION     varchar(256)                   null,
   MODEL                varchar(256)                   null,
   SNMP_IP_ADDRESS      varchar(4000)                  null,
   SWITCH_ROLE          varchar(256)                   not null,
   STATUS               varchar(256)                   null,
   FABRIC_NAME          varchar(256)                   null,
   WWN                  varchar(128)                   null,
   IS_PHYSICAL          char(1)                        not null,
   HAS_VIRTUAL_SUPPORT  char(1)                        not null,
   HAS_MULTIPLEX_SUPPORT char(1)                        null,
   HAS_BUNDLE_SUPPORT   char(1)                        not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.SWITCH is 
'This table contains FC switch attributes';

comment on column SAHARA.SWITCH.SWITCH_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.SWITCH.CC_STORAGE_URL is 
'The URL for the switch object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.SWITCH.DISPLAY_NAME is 
'Switch display name.';

comment on column SAHARA.SWITCH.ADVISORY_DISPLAY_NAME is 
'Switch name discovered by CommandCentral Storage.';

comment on column SAHARA.SWITCH.VENDOR is 
'The name of the company that manufactured the physical device.  In the case of a virtual object, such as a volume, this is the name of the company that manufactured the physical device upon which the volume resides.';

comment on column SAHARA.SWITCH.DISCOVERY_STATE is 
'The discovery state of the switch:

Possible values:
- Discovered
- Partially Discovered
- Missing

Note: Partially Discovered means that the switch was initially discovered by two or more explorers and then one of them stopped discovering the switch.

Note: A switch that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.SWITCH.FIRMWARE_VERSION is 
'Indicates the version of firmware installed on the object.';

comment on column SAHARA.SWITCH.MODEL is 
'The model of the switch.';

comment on column SAHARA.SWITCH.SNMP_IP_ADDRESS is 
'One or more IP addresses of the SNMP agent providing switch information.  Multiple addresses are stored as a comma separated list.';

comment on column SAHARA.SWITCH.SWITCH_ROLE is 
'The switch''s role in the fabric.  In a single switch fabric, the switch''s role will always be principal.  In a multi-switch fabric, there is one principal switch and there may be n secondaries.';

comment on column SAHARA.SWITCH.STATUS is 
'The switch''s status.  Valid entries include: Online, Offline, Testing, Faulty, or Unknown.';

comment on column SAHARA.SWITCH.FABRIC_NAME is 
'Fabric display name.';

comment on column SAHARA.SWITCH.WWN is 
'The device''s registered, 64-bit, unique identifier.';

comment on column SAHARA.SWITCH.IS_PHYSICAL is 
'Indicates whether the port is part of a physical or virtual switch.  True if physical; False if virtual.';

comment on column SAHARA.SWITCH.HAS_VIRTUAL_SUPPORT is 
'True if the switch supports virtual SANs; false if not.';

comment on column SAHARA.SWITCH.HAS_MULTIPLEX_SUPPORT is 
'True, if the switch allows port multiplexing.  False otherwise.';

comment on column SAHARA.SWITCH.HAS_BUNDLE_SUPPORT is 
'Indicates whether the switch can aggregate traffic across several links.  True if so; false if not.';

comment on column SAHARA.SWITCH.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.SWITCH.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.SWITCH
   alter IS_PHYSICAL add constraint C_SWITCH_ISPHYSICAL check (IS_PHYSICAL in ('T','F'));

alter table SAHARA.SWITCH
   alter HAS_VIRTUAL_SUPPORT add constraint C_SWITCH_HASVIRTUALSUPPORT check (HAS_VIRTUAL_SUPPORT in ('T','F'));

alter table SAHARA.SWITCH
   alter HAS_MULTIPLEX_SUPPORT add constraint C_SWITCH_HASMULTIPLEXSUPPORT check (HAS_MULTIPLEX_SUPPORT is null or (HAS_MULTIPLEX_SUPPORT in ('T','F')));

alter table SAHARA.SWITCH
   alter HAS_BUNDLE_SUPPORT add constraint C_SWITCH_HASBUNDLESUPPORT check (HAS_BUNDLE_SUPPORT in ('T','F'));

alter table SAHARA.SWITCH
   alter T2_IS_CURRENT add constraint C_SWITCH_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SWITCH
   alter T2_IS_DELETED add constraint C_SWITCH_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SWITCH
   add constraint PK_SWITCH primary key (SWITCH_KEY);

/*==============================================================*/
/* Index: I_SWITCH_SWITCHOID                                    */
/*==============================================================*/
create index I_SWITCH_SWITCHOID on SAHARA.SWITCH (
SWITCH_OID ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_SWITCHOID is 
'An index on Switch Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_DISPLAYNAME                                  */
/*==============================================================*/
create index I_SWITCH_DISPLAYNAME on SAHARA.SWITCH (
DISPLAY_NAME ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_ADVISORYDISPLAYNAME                          */
/*==============================================================*/
create index I_SWITCH_ADVISORYDISPLAYNAME on SAHARA.SWITCH (
ADVISORY_DISPLAY_NAME ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_ADVISORYDISPLAYNAME is 
'An index on Advisory Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_VENDOR                                       */
/*==============================================================*/
create index I_SWITCH_VENDOR on SAHARA.SWITCH (
VENDOR ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_VENDOR is 
'An index on Vendor for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_DISCOVERYSTATE                               */
/*==============================================================*/
create index I_SWITCH_DISCOVERYSTATE on SAHARA.SWITCH (
DISCOVERY_STATE ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_DISCOVERYSTATE is 
'An index on Discovery State for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_FIRMWAREVERSION                              */
/*==============================================================*/
create index I_SWITCH_FIRMWAREVERSION on SAHARA.SWITCH (
FIRMWARE_VERSION ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_FIRMWAREVERSION is 
'An index on Firmware Version for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_MODEL                                        */
/*==============================================================*/
create index I_SWITCH_MODEL on SAHARA.SWITCH (
MODEL ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_MODEL is 
'An index on Model for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_SNMPIPADDRESS                                */
/*==============================================================*/
create index I_SWITCH_SNMPIPADDRESS on SAHARA.SWITCH (
SNMP_IP_ADDRESS ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_SNMPIPADDRESS is 
'An index on SNMP IP Address(es) for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_SWITCHROLE                                   */
/*==============================================================*/
create index I_SWITCH_SWITCHROLE on SAHARA.SWITCH (
SWITCH_ROLE ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_SWITCHROLE is 
'An index on Switch Role for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_STATUS                                       */
/*==============================================================*/
create index I_SWITCH_STATUS on SAHARA.SWITCH (
STATUS ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_STATUS is 
'An index on Status for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_FABRICNAME                                   */
/*==============================================================*/
create index I_SWITCH_FABRICNAME on SAHARA.SWITCH (
FABRIC_NAME ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_FABRICNAME is 
'An index on Fabric Name for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_WWN                                          */
/*==============================================================*/
create index I_SWITCH_WWN on SAHARA.SWITCH (
WWN ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_WWN is 
'An index on WWN for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_ISPHYSICAL                                   */
/*==============================================================*/
create index I_SWITCH_ISPHYSICAL on SAHARA.SWITCH (
IS_PHYSICAL ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_ISPHYSICAL is 
'An index on Is Physical for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_HASVIRTUALSUPPORT                            */
/*==============================================================*/
create index I_SWITCH_HASVIRTUALSUPPORT on SAHARA.SWITCH (
HAS_VIRTUAL_SUPPORT ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_HASVIRTUALSUPPORT is 
'An index on Has Virtual Support for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_HASMULTIPLEXSUPPORT                          */
/*==============================================================*/
create index I_SWITCH_HASMULTIPLEXSUPPORT on SAHARA.SWITCH (
HAS_MULTIPLEX_SUPPORT ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_HASMULTIPLEXSUPPORT is 
'An index on Has Multiplex Support for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_HASBUNDLESUPPORT                             */
/*==============================================================*/
create index I_SWITCH_HASBUNDLESUPPORT on SAHARA.SWITCH (
HAS_BUNDLE_SUPPORT ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_HASBUNDLESUPPORT is 
'An index on Has Bundle Support for dimensional queries.';

/*==============================================================*/
/* Index: I_SWITCH_T2ISCURRENTSWITCHKEY                         */
/*==============================================================*/
create index I_SWITCH_T2ISCURRENTSWITCHKEY on SAHARA.SWITCH (
T2_IS_CURRENT ASC,
SWITCH_KEY ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_T2ISCURRENTSWITCHKEY is 
'An index on Type 2 - Is Current and Switch Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_SWITCH_T2SCRRNTT2SDLTDD                             */
/*==============================================================*/
create index I_SWITCH_T2SCRRNTT2SDLTDD on SAHARA.SWITCH (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
SWITCH_OID ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Switch Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_SWITCH_T2ISDELETED                                  */
/*==============================================================*/
create index I_SWITCH_T2ISDELETED on SAHARA.SWITCH (
T2_IS_DELETED ASC
);

comment on index SAHARA.SWITCH.I_SWITCH_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

grant SELECT on SAHARA.SWITCH to SAHARA_APP;

/*==============================================================*/
/* Table: SWITCH_                                               */
/*==============================================================*/
create table SAHARA_APP.SWITCH_ 
(
   SWITCH_OID           varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   ADVISORY_DISPLAY_NAME varchar(256)                   null,
   VENDOR               varchar(256)                   null,
   DISCOVERY_STATE      varchar(32)                    null,
   FIRMWARE_VERSION     varchar(256)                   null,
   MODEL                varchar(256)                   null,
   SNMP_IP_ADDRESS      varchar(4000)                  null,
   SWITCH_ROLE          varchar(256)                   not null,
   STATUS               varchar(256)                   null,
   FABRIC_NAME          varchar(256)                   null,
   WWN                  varchar(128)                   null,
   IS_PHYSICAL          char(1)                        not null,
   HAS_VIRTUAL_SUPPORT  char(1)                        not null,
   HAS_MULTIPLEX_SUPPORT char(1)                        null,
   HAS_BUNDLE_SUPPORT   char(1)                        not null
);

comment on table SAHARA_APP.SWITCH_ is 
'This is a staging table for loading data into warehouse table SAHARA.SWITCH.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains FC switch attributes';

comment on column SAHARA_APP.SWITCH_.SWITCH_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.SWITCH_.DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Switch display name.';

comment on column SAHARA_APP.SWITCH_.ADVISORY_DISPLAY_NAME is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column ADVISORY_DISPLAY_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Switch name discovered by CommandCentral Storage.';

comment on column SAHARA_APP.SWITCH_.VENDOR is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column VENDOR.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The name of the company that manufactured the physical device.  In the case of a virtual object, such as a volume, this is the name of the company that manufactured the physical device upon which the volume resides.';

comment on column SAHARA_APP.SWITCH_.DISCOVERY_STATE is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column DISCOVERY_STATE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the switch:

Possible values:
- Discovered
- Partially Discovered
- Missing

Note: Partially Discovered means that the switch was initially discovered by two or more explorers and then one of them stopped discovering the switch.

Note: A switch that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.SWITCH_.FIRMWARE_VERSION is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column FIRMWARE_VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates the version of firmware installed on the object.';

comment on column SAHARA_APP.SWITCH_.MODEL is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column MODEL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The model of the switch.';

comment on column SAHARA_APP.SWITCH_.SNMP_IP_ADDRESS is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column SNMP_IP_ADDRESS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

One or more IP addresses of the SNMP agent providing switch information.  Multiple addresses are stored as a comma separated list.';

comment on column SAHARA_APP.SWITCH_.SWITCH_ROLE is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column SWITCH_ROLE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The switch''s role in the fabric.  In a single switch fabric, the switch''s role will always be principal.  In a multi-switch fabric, there is one principal switch and there may be n secondaries.';

comment on column SAHARA_APP.SWITCH_.STATUS is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column STATUS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The switch''s status.  Valid entries include: Online, Offline, Testing, Faulty, or Unknown.';

comment on column SAHARA_APP.SWITCH_.FABRIC_NAME is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column FABRIC_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Fabric display name.';

comment on column SAHARA_APP.SWITCH_.WWN is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column WWN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The device''s registered, 64-bit, unique identifier.';

comment on column SAHARA_APP.SWITCH_.IS_PHYSICAL is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column IS_PHYSICAL.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates whether the port is part of a physical or virtual switch.  True if physical; False if virtual.';

comment on column SAHARA_APP.SWITCH_.HAS_VIRTUAL_SUPPORT is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column HAS_VIRTUAL_SUPPORT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

True if the switch supports virtual SANs; false if not.';

comment on column SAHARA_APP.SWITCH_.HAS_MULTIPLEX_SUPPORT is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column HAS_MULTIPLEX_SUPPORT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

True, if the switch allows port multiplexing.  False otherwise.';

comment on column SAHARA_APP.SWITCH_.HAS_BUNDLE_SUPPORT is 
'This column is for staging data that will end up in table SAHARA.SWITCH, column HAS_BUNDLE_SUPPORT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Indicates whether the switch can aggregate traffic across several links.  True if so; false if not.';

alter table SAHARA_APP.SWITCH_
   alter IS_PHYSICAL add constraint C_SWITCH_ISPHYSICAL check (IS_PHYSICAL in ('T','F'));

alter table SAHARA_APP.SWITCH_
   alter HAS_VIRTUAL_SUPPORT add constraint C_SWITCH_HASVIRTUALSUPPORT check (HAS_VIRTUAL_SUPPORT in ('T','F'));

alter table SAHARA_APP.SWITCH_
   alter HAS_MULTIPLEX_SUPPORT add constraint C_SWITCH_HASMULTIPLEXSUPPORT check (HAS_MULTIPLEX_SUPPORT is null or (HAS_MULTIPLEX_SUPPORT in ('T','F')));

alter table SAHARA_APP.SWITCH_
   alter HAS_BUNDLE_SUPPORT add constraint C_SWITCH_HASBUNDLESUPPORT check (HAS_BUNDLE_SUPPORT in ('T','F'));

alter table SAHARA_APP.SWITCH_
   add constraint PK_SWITCH_ primary key (SWITCH_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.SWITCH_ to SAHARA;

/*==============================================================*/
/* Table: SWITCH_BVIEW_NODE                                     */
/*==============================================================*/
create table SAHARA.SWITCH_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   SWITCH_KEY           numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null
);

comment on table SAHARA.SWITCH_BVIEW_NODE is 
'This is a join table between switch objects and business view nodes.';

comment on column SAHARA.SWITCH_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.SWITCH_BVIEW_NODE.SWITCH_KEY is 
'The unique identifier of the switch object with which the business view node is associated.';

comment on column SAHARA.SWITCH_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.SWITCH_BVIEW_NODE
   add constraint PK_SWITCH_BVIEW_NODE primary key (BVIEW_NODE_KEY, SWITCH_KEY);

/*==============================================================*/
/* Index: I_SWITCHBVIEWNODE_BVIEWSWITCH                         */
/*==============================================================*/
create unique index I_SWITCHBVIEWNODE_BVIEWSWITCH on SAHARA.SWITCH_BVIEW_NODE (
BVIEW_KEY ASC,
SWITCH_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.SWITCH_BVIEW_NODE.I_SWITCHBVIEWNODE_BVIEWSWITCH is 
'This index exists to accelerate queries for which switches are associated with a particular business view.';

grant SELECT on SAHARA.SWITCH_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: SWITCH_PORT_COUNTS                                    */
/*==============================================================*/
create table SAHARA.SWITCH_PORT_COUNTS 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   SWITCH_KEY           numeric(10)                    not null,
   TOTAL_PORTS          numeric(10)                    not null default 0,
   USED_PORTS           numeric(10)                    not null default 0,
   UNUSED_PORTS         numeric(10)                    not null default 0,
   ISL_PORTS            numeric(10)                    not null default 0,
   HOST_PORTS           numeric(10)                    not null default 0,
   OTHER_USED_PORTS     numeric(10)                    not null default 0,
   ARRAY_PORTS          numeric(10)                    not null default 0,
   PERCENT_USED         numeric(7,4)                   not null default 0,
   PERCENT_UNUSED       numeric(7,4)                   not null default 0
);

comment on table SAHARA.SWITCH_PORT_COUNTS is 
'This table contains counts of switch ports in the following the configurations:
- Used
- Unused
- Connected to hosts
- Connected to arrays
- ISLs';

comment on column SAHARA.SWITCH_PORT_COUNTS.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.SWITCH_PORT_COUNTS.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.SWITCH_PORT_COUNTS.TOTAL_PORTS is 
'Total number of ports installed on a switch.
Depending on the switch model and firmware version, this number may include ports that are not physically installed.';

comment on column SAHARA.SWITCH_PORT_COUNTS.USED_PORTS is 
'Number of switch ports that are connected to other ports.';

comment on column SAHARA.SWITCH_PORT_COUNTS.UNUSED_PORTS is 
'Number of ports that are not connected to any other port.';

comment on column SAHARA.SWITCH_PORT_COUNTS.ISL_PORTS is 
'Number of Inter-Switch Links (ISLs) or e-ports on this switch.';

comment on column SAHARA.SWITCH_PORT_COUNTS.OTHER_USED_PORTS is 
'The number of ports that are being used to connect to something undiscoverable to CommandCentral Storage.';

comment on column SAHARA.SWITCH_PORT_COUNTS.ARRAY_PORTS is 
'Number of switch ports that are connected to array ports.';

comment on column SAHARA.SWITCH_PORT_COUNTS.PERCENT_USED is 
'The percentage of total switch ports that are being used.  This value is calculated by dividing Used Ports by Total Port Count.';

comment on column SAHARA.SWITCH_PORT_COUNTS.PERCENT_UNUSED is 
'The percentage of total switch ports that are not being used.  This value is calculated by dividing Unused Ports by Total Port Count.';

alter table SAHARA.SWITCH_PORT_COUNTS
   alter TOTAL_PORTS add constraint C_SWTCHPRTCNTS_TOTALPORTS check (TOTAL_PORTS >= 0);

alter table SAHARA.SWITCH_PORT_COUNTS
   alter USED_PORTS add constraint C_SWTCHPRTCNTS_USEDPORTS check (USED_PORTS >= 0);

alter table SAHARA.SWITCH_PORT_COUNTS
   alter UNUSED_PORTS add constraint C_SWTCHPRTCNTS_UNUSEDPORTS check (UNUSED_PORTS >= 0);

alter table SAHARA.SWITCH_PORT_COUNTS
   alter ISL_PORTS add constraint C_SWTCHPRTCNTS_ISLPORTS check (ISL_PORTS >= 0);

alter table SAHARA.SWITCH_PORT_COUNTS
   alter HOST_PORTS add constraint C_SWTCHPRTCNTS_HOSTPORTS check (HOST_PORTS >= 0);

alter table SAHARA.SWITCH_PORT_COUNTS
   alter OTHER_USED_PORTS add constraint C_SWTCHPRTCNTS_OTHERUSEDPORTS check (OTHER_USED_PORTS >= 0);

alter table SAHARA.SWITCH_PORT_COUNTS
   alter ARRAY_PORTS add constraint C_SWTCHPRTCNTS_ARRAYPORTS check (ARRAY_PORTS >= 0);

/*==============================================================*/
/* Index: I_SWITCHPORTCTS_ROLLUPBATCHKEY                        */
/*==============================================================*/
create index I_SWITCHPORTCTS_ROLLUPBATCHKEY on SAHARA.SWITCH_PORT_COUNTS (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_SWITCHPORTCTS_DATESKEY                              */
/*==============================================================*/
create index I_SWITCHPORTCTS_DATESKEY on SAHARA.SWITCH_PORT_COUNTS (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_SWITCHPORTCTS_SWITCHKEY                             */
/*==============================================================*/
create index I_SWITCHPORTCTS_SWITCHKEY on SAHARA.SWITCH_PORT_COUNTS (
SWITCH_KEY ASC
);

grant SELECT on SAHARA.SWITCH_PORT_COUNTS to SAHARA_APP;

/*==============================================================*/
/* Table: SWITCH_PORT_COUNTS_                                   */
/*==============================================================*/
create table SAHARA_APP.SWITCH_PORT_COUNTS_ 
(
   SWITCH_OID           varchar(512)                   not null,
   TOTAL_PORTS          numeric(10)                    not null default 0,
   USED_PORTS           numeric(10)                    not null default 0,
   UNUSED_PORTS         numeric(10)                    not null default 0,
   ISL_PORTS            numeric(10)                    not null default 0,
   HOST_PORTS           numeric(10)                    not null default 0,
   OTHER_USED_PORTS     numeric(10)                    not null default 0,
   ARRAY_PORTS          numeric(10)                    not null default 0,
   PERCENT_USED         numeric(7,4)                   not null default 0,
   PERCENT_UNUSED       numeric(7,4)                   not null default 0
);

comment on table SAHARA_APP.SWITCH_PORT_COUNTS_ is 
'This is a staging table for loading data into warehouse table SAHARA.SWITCH_PORT_COUNTS.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains counts of switch ports in the following the configurations:
- Used
- Unused
- Connected to hosts
- Connected to arrays
- ISLs';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.SWITCH_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.TOTAL_PORTS is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column TOTAL_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total number of ports installed on a switch.
Depending on the switch model and firmware version, this number may include ports that are not physically installed.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.USED_PORTS is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column USED_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Number of switch ports that are connected to other ports.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.UNUSED_PORTS is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column UNUSED_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Number of ports that are not connected to any other port.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.ISL_PORTS is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column ISL_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Number of Inter-Switch Links (ISLs) or e-ports on this switch.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.HOST_PORTS is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column HOST_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.OTHER_USED_PORTS is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column OTHER_USED_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The number of ports that are being used to connect to something undiscoverable to CommandCentral Storage.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.ARRAY_PORTS is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column ARRAY_PORTS.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Number of switch ports that are connected to array ports.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.PERCENT_USED is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column PERCENT_USED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of total switch ports that are being used.  This value is calculated by dividing Used Ports by Total Port Count.';

comment on column SAHARA_APP.SWITCH_PORT_COUNTS_.PERCENT_UNUSED is 
'This column is for staging data that will end up in table SAHARA.SWITCH_PORT_COUNTS, column PERCENT_UNUSED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The percentage of total switch ports that are not being used.  This value is calculated by dividing Unused Ports by Total Port Count.';

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   alter TOTAL_PORTS add constraint C_SWTCHPRTCNTS_TOTALPORTS check (TOTAL_PORTS >= 0);

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   alter USED_PORTS add constraint C_SWTCHPRTCNTS_USEDPORTS check (USED_PORTS >= 0);

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   alter UNUSED_PORTS add constraint C_SWTCHPRTCNTS_UNUSEDPORTS check (UNUSED_PORTS >= 0);

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   alter ISL_PORTS add constraint C_SWTCHPRTCNTS_ISLPORTS check (ISL_PORTS >= 0);

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   alter HOST_PORTS add constraint C_SWTCHPRTCNTS_HOSTPORTS check (HOST_PORTS >= 0);

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   alter OTHER_USED_PORTS add constraint C_SWTCHPRTCNTS_OTHERUSEDPORTS check (OTHER_USED_PORTS >= 0);

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   alter ARRAY_PORTS add constraint C_SWTCHPRTCNTS_ARRAYPORTS check (ARRAY_PORTS >= 0);

alter table SAHARA_APP.SWITCH_PORT_COUNTS_
   add constraint PK_SWITCH_PORT_COUNTS_ primary key (SWITCH_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SWITCH_PORT_COUNTS_ to SAHARA;

/*==============================================================*/
/* Table: T2_INSERT                                             */
/*==============================================================*/
create global temporary table SAHARA.T2_INSERT 
(
   INSERTED_KEY         numeric(10)                    not null,
   INSERTED_OID         varchar(512)                   not null,
   ANCESTOR_ROW         numeric                        null,
   ANCESTOR_KEY         numeric(10)                    null
);

comment on table SAHARA.T2_INSERT is 
'This is a global temporary table used by triggers to track and process inserted rows in dimension tables as part of processing type 2 updates.';

comment on column SAHARA.T2_INSERT.INSERTED_KEY is 
'The unique key of the newly insert dimension row.';

comment on column SAHARA.T2_INSERT.INSERTED_OID is 
'The unique object identifier of the newly insert dimension row.';

comment on column SAHARA.T2_INSERT.ANCESTOR_ROW is 
'The rowid of the former "current" dimension row for the new dimension row.';

comment on column SAHARA.T2_INSERT.ANCESTOR_KEY is 
'The unique key of the former "current" dimension row for the new dimension row.';

alter table SAHARA.T2_INSERT
   add constraint PK_T2_INSERT primary key (INSERTED_KEY);

alter table SAHARA.T2_INSERT
   add constraint AK_T2INSERT_OID unique (INSERTED_OID);

/*==============================================================*/
/* Index: I_T2INSERT_ANCESTORROW                                */
/*==============================================================*/
create unique index I_T2INSERT_ANCESTORROW on SAHARA.T2_INSERT (
ANCESTOR_ROW ASC
);

/*==============================================================*/
/* Index: I_T2INSERT_ANCESTORKEY                                */
/*==============================================================*/
create unique index I_T2INSERT_ANCESTORKEY on SAHARA.T2_INSERT (
ANCESTOR_KEY ASC
);

/*==============================================================*/
/* Table: T2_OBJECT_DIM_SOURCE_DELETE                           */
/*==============================================================*/
create global temporary table SAHARA.T2_OBJECT_DIM_SOURCE_DELETE 
(
   OID                  varchar(512)                   not null,
   TABLE_NAME           varchar(30)                    not null
);

comment on table SAHARA.T2_OBJECT_DIM_SOURCE_DELETE is 
'This is a global temporary table used by triggers to track and process rows delete from OBJECT_DIM_SOURCE part of the implementation of reference counting.';

comment on column SAHARA.T2_OBJECT_DIM_SOURCE_DELETE.OID is 
'The unique identifier of the object whose reference was deleted.';

comment on column SAHARA.T2_OBJECT_DIM_SOURCE_DELETE.TABLE_NAME is 
'The name of the table in which the object''s dimension row(s) are stored.';

alter table SAHARA.T2_OBJECT_DIM_SOURCE_DELETE
   alter TABLE_NAME add constraint C_T2BJCTDMSRCDL_TABLENAME check (TABLE_NAME = upper(TABLE_NAME));

alter table SAHARA.T2_OBJECT_DIM_SOURCE_DELETE
   add constraint PK_T2_OBJECT_DIM_SOURCE_DELETE primary key (OID, TABLE_NAME);

alter table SAHARA.T2_OBJECT_DIM_SOURCE_DELETE
   add constraint AK_AK_T2_OBJECT_DIM_S_T2_OBJEC unique (TABLE_NAME, OID);

/*==============================================================*/
/* Table: TABLE_METADATA_BASE                                   */
/*==============================================================*/
create table SAHARA.TABLE_METADATA_BASE 
(
   OWNER                varchar(30)                    not null,
   TABLE_NAME           varchar(30)                    not null,
   DISPLAY_NAME         varchar(64)                    not null
);

comment on table SAHARA.TABLE_METADATA_BASE is 
'This table contains metadata for tables and views in the sahara schema.  The contents are generated out of PowerDesigner in which the schema is modeled.

This table should not be queried directly.  Instead, use database view TABLE_METADATA.

This table is populated at install and upgrade.';

comment on column SAHARA.TABLE_METADATA_BASE.OWNER is 
'The name of the owner of the table.';

comment on column SAHARA.TABLE_METADATA_BASE.TABLE_NAME is 
'The name of the table.';

comment on column SAHARA.TABLE_METADATA_BASE.DISPLAY_NAME is 
'A user friendly display name for the table.';

alter table SAHARA.TABLE_METADATA_BASE
   alter OWNER add constraint C_TBLMTDTBS_OWNER check (OWNER = upper(OWNER));

alter table SAHARA.TABLE_METADATA_BASE
   alter TABLE_NAME add constraint C_TBLMTDTBS_TABLENAME check (TABLE_NAME = upper(TABLE_NAME));

alter table SAHARA.TABLE_METADATA_BASE
   add constraint PK_TABLE_METADATA_BASE primary key (OWNER, TABLE_NAME);

grant SELECT on SAHARA.TABLE_METADATA_BASE to SAHARA_APP;

/*==============================================================*/
/* Table: TIER                                                  */
/*==============================================================*/
create table SAHARA.TIER 
(
   TIER_KEY             numeric(10)                    not null default autoincrement,
   TIER_CONTEXT         varchar(32)                    not null,
   DISPLAY_NAME         varchar(64)                    not null,
   UPPER_DISPLAY_NAME   varchar(64)                    not null,
   DESCRIPTION          long varchar                   null,
   RANK                 numeric(2)                     not null,
   COST                 numeric(12,6)                  not null,
   IS_DEFAULT           char(1)                        not null default 'F',
   RULE_KEY             numeric(10)                    null
);

comment on table SAHARA.TIER is 
'This table contains configuration information for user defined tiers.';

comment on column SAHARA.TIER.TIER_KEY is 
'A unique database generated identifier for a tier.  The value really serves no other purpose other than to make the tier API easier to use by providing a simple way to identify a tier.';

comment on column SAHARA.TIER.TIER_CONTEXT is 
'The context in which the tier configuration is used.

See the schema report model notes: contexts, for further explanation.';

comment on column SAHARA.TIER.DISPLAY_NAME is 
'The display name for the tier.  Each display name must be unique (ignoring character case) within a context.';

comment on column SAHARA.TIER.UPPER_DISPLAY_NAME is 
'The upper case form of the display name for the tier.  This is used to enforce the constrant that each display name must be unique (ignoring character case) within a context.';

comment on column SAHARA.TIER.DESCRIPTION is 
'A user specified description for the tier.';

comment on column SAHARA.TIER.RANK is 
'The rank of the tier.  Tiers are ranked using integers starting at one (1) and increasing by one for each tier.';

comment on column SAHARA.TIER.COST is 
'The cost per unit for attributes of objects associated with the tier.';

comment on column SAHARA.TIER.IS_DEFAULT is 
'A boolean that indicates which tier is the default for a tier context.  Each tier context has exactly one default.';

comment on column SAHARA.TIER.RULE_KEY is 
'The unique identifier of a a rule associated with the tier.';

alter table SAHARA.TIER
   alter IS_DEFAULT add constraint C_TIER_ISDEFAULT check (IS_DEFAULT in ('T','F'));

alter table SAHARA.TIER
   add constraint PK_TIER primary key (TIER_KEY);

alter table SAHARA.TIER
   add constraint AK_AK_TIER_CONTEXTUDN_TIER unique (TIER_CONTEXT, UPPER_DISPLAY_NAME);

alter table SAHARA.TIER
   add constraint AK_TIER_CONTEXTRANK unique (TIER_CONTEXT, RANK);

grant SELECT on SAHARA.TIER to SAHARA_APP;

/*==============================================================*/
/* Table: TIER_CONTEXT                                          */
/*==============================================================*/
create table SAHARA.TIER_CONTEXT 
(
   TIER_CONTEXT         varchar(32)                    not null,
   COST_UNIT            varchar(64)                    not null,
   RULE_SQL             long varchar                   not null
);

comment on table SAHARA.TIER_CONTEXT is 
'Tiers may be used in multiple places in the sahara schema.  This is a look-up table that defines the context in which specific associated tier configuration is used.

See the schema report model notes: contexts, for further explanation.

This table is populated at install and upgrade.';

comment on column SAHARA.TIER_CONTEXT.TIER_CONTEXT is 
'The context in which the tier configuration is used.';

comment on column SAHARA.TIER_CONTEXT.COST_UNIT is 
'The unit with which the costs in the tiers of the context are associated.  For example: per GB.';

comment on column SAHARA.TIER_CONTEXT.RULE_SQL is 
'This is a SQL statement template which is used to form that actual SQL executed to implement the tier rule for the context.  The token $TIER_RULE_EXPRESSION$ is replaced with the generated tier rule SQL expression; the resultant SQL statement is executed to implement the rule.';

alter table SAHARA.TIER_CONTEXT
   add constraint PK_TIER_CONTEXT primary key (TIER_CONTEXT);

grant SELECT on SAHARA.TIER_CONTEXT to SAHARA_APP;

/*==============================================================*/
/* Table: USERS                                                 */
/*==============================================================*/
create table SAHARA.USERS 
(
   USERS_KEY            numeric(10)                    not null default autoincrement,
   LOGIN_NAME           varchar(80)                    not null,
   DOMAIN               varchar(80)                    not null
);

comment on table SAHARA.USERS is 
'This table contains the users of the Enterprise Reporter GUI.  It provides a single look-up table for users and is used to join to sessions and configuration.

In the future, additional user attributes may be added.

Note: We''d prefer to call this table USER, but the conflict with the USER keyword was too much to overcome.';

comment on column SAHARA.USERS.USERS_KEY is 
'A unique database generated identifier for the user.';

comment on column SAHARA.USERS.LOGIN_NAME is 
'The login name for the user.  This is case sensitive.';

comment on column SAHARA.USERS.DOMAIN is 
'The login domain of the user.  This is case sensitive.';

alter table SAHARA.USERS
   add constraint PK_USERS primary key (USERS_KEY);

alter table SAHARA.USERS
   add constraint AK_AK_USER_LOGINDOMAI_USERS unique (LOGIN_NAME, DOMAIN);

grant SELECT on SAHARA.USERS to SAHARA_APP;

/*==============================================================*/
/* Table: USER_CONFIGURATION                                    */
/*==============================================================*/
create table SAHARA.USER_CONFIGURATION 
(
   USERS_KEY            numeric(10)                    not null,
   NAME                 varchar(128)                   not null,
   VALUE                varchar(2000)                  null
);

comment on table SAHARA.USER_CONFIGURATION is 
'This table contain users specific configuration items.  Each items is stored as a name/value pair along with the identifier of the associated user.';

comment on column SAHARA.USER_CONFIGURATION.USERS_KEY is 
'The unique identifier of the user to which the configuration belongs.';

comment on column SAHARA.USER_CONFIGURATION.NAME is 
'The name of the configuration item.  Configuration items are always stored in upper character case.';

comment on column SAHARA.USER_CONFIGURATION.VALUE is 
'The value of the configuration item.';

alter table SAHARA.USER_CONFIGURATION
   alter NAME add constraint C_SRCNFGRTN_NAME check (NAME = upper(NAME));

alter table SAHARA.USER_CONFIGURATION
   add constraint PK_USER_CONFIGURATION primary key (USERS_KEY, NAME);

grant SELECT on SAHARA.USER_CONFIGURATION to SAHARA_APP;

/*==============================================================*/
/* Table: VOLUME                                                */
/*==============================================================*/
create table SAHARA.VOLUME 
(
   VOLUME_KEY           numeric(10)                    not null default autoincrement,
   VOLUME_OID           varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   HOST_KEY             numeric(10)                    not null,
   DISKGROUP_KEY        numeric(10)                    null,
   VOLUME_NAME          varchar(256)                   null,
   VM_VERSION           varchar(256)                   null,
   VM_TYPE              varchar(32)                    not null,
   DISK_GROUP_NAME      varchar(256)                   null,
   VOLUME_LAYOUT        varchar(32)                    not null,
   IS_VCS_CVM           char(1)                        null default 'F',
   IS_USED_BY_FILESYSTEM char(1)                        null default 'F',
   IS_USED_BY_APPLICATION char(1)                        null default 'F',
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F',
   IS_DIRECT_ATTACHED   char(1)                        null,
   IS_COMPONENT         char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   IS_EXPORTED          char(1)                        null,
   VOLUME_TYPE          varchar(256)                   null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   IS_USED_BY_DB_VM     char(1)                        null default 'F',
   DB_VM_NAME           varchar(256)                   null
);

comment on table SAHARA.VOLUME is 
'This table contains host based O/S volume attributes';

comment on column SAHARA.VOLUME.VOLUME_OID is 
'Unique identifier for this object in CommandCentral Storage database.';

comment on column SAHARA.VOLUME.CC_STORAGE_URL is 
'The URL for the volume object overview view in the CommandCentral Storage GUI.';

comment on column SAHARA.VOLUME.VOLUME_NAME is 
'Name of the volume.';

comment on column SAHARA.VOLUME.VM_VERSION is 
'Volume manager version';

comment on column SAHARA.VOLUME.VM_TYPE is 
'Volume manager type:
SUN VM Disk Suite
SUN VM
VxVM
Microsoft LDM
AIX LVM
Unknown';

comment on column SAHARA.VOLUME.DISK_GROUP_NAME is 
'The discovery state of the volume:

Possible values:
- Discovered
- Partially Discovered
- Missing

Note: Partially Discovered means that the volume was initially discovered by two or more explorers and then one of them stopped discovering the volume.

Note: A volume that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA.VOLUME.VOLUME_LAYOUT is 
'Volume layout, could be striped, mirrored, etc.';

comment on column SAHARA.VOLUME.IS_VCS_CVM is 
'True, if the volume is a clustered volume managed using VCS.  False otherwise.';

comment on column SAHARA.VOLUME.IS_USED_BY_FILESYSTEM is 
'Flag set to true when file system uses the volume';

comment on column SAHARA.VOLUME.IS_USED_BY_APPLICATION is 
'Flag set to true when application uses the raw volume';

comment on column SAHARA.VOLUME.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.VOLUME.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

comment on column SAHARA.VOLUME.IS_DIRECT_ATTACHED is 
'A boolean flag that indicates if the device is direct attached.

Possible values:
''T'' - the Volume is comsuming direct attached storage
''F'' - the Volume is not comsuming direct attached storage';

comment on column SAHARA.VOLUME.IS_COMPONENT is 
'A boolean flag that indicates if the device is a component volume of a VSET.

Possible values:
''T'' - the Volume is component volumes of a VSET
''F'' - the Volume is not component volumes of a VSET';

comment on column SAHARA.VOLUME.IS_BASED_ON_THIN is 
'A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the Volume is based on thin lun
''F'' - the Volume is not based on thin lun';

comment on column SAHARA.VOLUME.IS_EXPORTED is 
'A boolean flag that indicates if the device is exported.

Possible values:
''T'' - the device is exported
''F'' - the device is not exported';

comment on column SAHARA.VOLUME.VOLUME_TYPE is 
'Volume Type:
Data
Snapshot
Clone';

comment on column SAHARA.VOLUME.APPLICATION_BE is 
'Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.VOLUME.APPLICATION_COMPOSITE_BE is 
'Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA.VOLUME.ORGANIZATION_BE is 
'Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.VOLUME.ORGANIZATION_COMPOSITE_BE is 
'Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA.VOLUME.IS_USED_BY_DB_VM is 
'A Boolean flag that indicates whether a volume is utilized by a database volume manager like Oracle ASM.';

comment on column SAHARA.VOLUME.DB_VM_NAME is 
'This attribute represents name of the database volume manager that is using the volume.';

alter table SAHARA.VOLUME
   alter IS_VCS_CVM add constraint C_VOLUME_ISVCSCVM check (IS_VCS_CVM is null or (IS_VCS_CVM in ('T','F')));

alter table SAHARA.VOLUME
   alter IS_USED_BY_FILESYSTEM add constraint C_VOLUME_ISUSEDBYFILESYSTEM check (IS_USED_BY_FILESYSTEM is null or (IS_USED_BY_FILESYSTEM in ('T','F')));

alter table SAHARA.VOLUME
   alter IS_USED_BY_APPLICATION add constraint C_VOLUME_ISUSEDBYAPPLICATION check (IS_USED_BY_APPLICATION is null or (IS_USED_BY_APPLICATION in ('T','F')));

alter table SAHARA.VOLUME
   alter T2_IS_CURRENT add constraint C_VOLUME_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.VOLUME
   alter T2_IS_DELETED add constraint C_VOLUME_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.VOLUME
   alter IS_DIRECT_ATTACHED add constraint C_VOLUME_ISDIRECTATTACHED check (IS_DIRECT_ATTACHED is null or (IS_DIRECT_ATTACHED in ('T','F')));

alter table SAHARA.VOLUME
   alter IS_COMPONENT add constraint C_VOLUME_ISCOMPONENT check (IS_COMPONENT is null or (IS_COMPONENT in ('T','F')));

alter table SAHARA.VOLUME
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA.VOLUME
   alter IS_EXPORTED add constraint C_VOLUME_ISEXPORTED check (IS_EXPORTED is null or (IS_EXPORTED in ('T','F')));

alter table SAHARA.VOLUME
   alter IS_USED_BY_DB_VM add constraint CKC_IS_USED_BY_DB_VM_VOLUME check (IS_USED_BY_DB_VM is null or (IS_USED_BY_DB_VM in ('T','F')));

alter table SAHARA.VOLUME
   add constraint PK_VOLUME primary key (VOLUME_KEY);

/*==============================================================*/
/* Index: I_VOLUME_VOLUMEOID                                    */
/*==============================================================*/
create index I_VOLUME_VOLUMEOID on SAHARA.VOLUME (
VOLUME_OID ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_VOLUMEOID is 
'An index on Volume Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_HOSTKEY                                      */
/*==============================================================*/
create index I_VOLUME_HOSTKEY on SAHARA.VOLUME (
HOST_KEY ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_HOSTKEY is 
'An index on Host Key for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_DISKGROUPKEY                                 */
/*==============================================================*/
create index I_VOLUME_DISKGROUPKEY on SAHARA.VOLUME (
DISKGROUP_KEY ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_DISKGROUPKEY is 
'An index on Diskgroup Key for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_VOLUMENAME                                   */
/*==============================================================*/
create index I_VOLUME_VOLUMENAME on SAHARA.VOLUME (
VOLUME_NAME ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_VOLUMENAME is 
'An index on Volume Name for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_VMVERSION                                    */
/*==============================================================*/
create index I_VOLUME_VMVERSION on SAHARA.VOLUME (
VM_VERSION ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_VMVERSION is 
'An index on VM Version for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_VMTYPE                                       */
/*==============================================================*/
create index I_VOLUME_VMTYPE on SAHARA.VOLUME (
VM_TYPE ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_VMTYPE is 
'An index on VM Type for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_DISKGROUPNAME                                */
/*==============================================================*/
create index I_VOLUME_DISKGROUPNAME on SAHARA.VOLUME (
DISK_GROUP_NAME ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_DISKGROUPNAME is 
'An index on Disk Group Name for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_VOLUMELAYOUT                                 */
/*==============================================================*/
create index I_VOLUME_VOLUMELAYOUT on SAHARA.VOLUME (
VOLUME_LAYOUT ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_VOLUMELAYOUT is 
'An index on Volume Layout for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISVCSCVM                                     */
/*==============================================================*/
create index I_VOLUME_ISVCSCVM on SAHARA.VOLUME (
IS_VCS_CVM ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISVCSCVM is 
'An index on Is VCS CVM for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISUSEDBYFILESYSTEM                           */
/*==============================================================*/
create index I_VOLUME_ISUSEDBYFILESYSTEM on SAHARA.VOLUME (
IS_USED_BY_FILESYSTEM ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISUSEDBYFILESYSTEM is 
'An index on Is Used by Filesystem for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISUSEDBYAPPLICATION                          */
/*==============================================================*/
create index I_VOLUME_ISUSEDBYAPPLICATION on SAHARA.VOLUME (
IS_USED_BY_APPLICATION ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISUSEDBYAPPLICATION is 
'An index on Is Used by Application for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_T2ISCURRENTVOLUMEKEY                         */
/*==============================================================*/
create index I_VOLUME_T2ISCURRENTVOLUMEKEY on SAHARA.VOLUME (
T2_IS_CURRENT ASC,
VOLUME_KEY ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_T2ISCURRENTVOLUMEKEY is 
'An index on Type 2 - Is Current and Volume Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_VOLUME_T2SCRRNTT2SDLTDD                             */
/*==============================================================*/
create index I_VOLUME_T2SCRRNTT2SDLTDD on SAHARA.VOLUME (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
VOLUME_OID ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and Volume Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_VOLUME_T2ISDELETED                                  */
/*==============================================================*/
create index I_VOLUME_T2ISDELETED on SAHARA.VOLUME (
T2_IS_DELETED ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISDIRECTATTACHED                             */
/*==============================================================*/
create index I_VOLUME_ISDIRECTATTACHED on SAHARA.VOLUME (
IS_DIRECT_ATTACHED ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISDIRECTATTACHED is 
'An index on Is Direct Attached for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISCOMPONENT                                  */
/*==============================================================*/
create index I_VOLUME_ISCOMPONENT on SAHARA.VOLUME (
IS_COMPONENT ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISCOMPONENT is 
'An index on Is Component for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISBASEDONTHIN                                */
/*==============================================================*/
create index I_VOLUME_ISBASEDONTHIN on SAHARA.VOLUME (
IS_BASED_ON_THIN ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISBASEDONTHIN is 
'An index on Is Based On Thin for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISEXPORTED                                   */
/*==============================================================*/
create index I_VOLUME_ISEXPORTED on SAHARA.VOLUME (
IS_EXPORTED ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISEXPORTED is 
'An index on Is Exported for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_VOLUMETYPE                                   */
/*==============================================================*/
create index I_VOLUME_VOLUMETYPE on SAHARA.VOLUME (
VOLUME_TYPE ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_VOLUMETYPE is 
'An index on Volume Type for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_APPLICATIONBE                                */
/*==============================================================*/
create index I_VOLUME_APPLICATIONBE on SAHARA.VOLUME (
APPLICATION_BE ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_APPLICATIONBE is 
'An index on Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_PPLCTNCMPSTB                                 */
/*==============================================================*/
create index I_VOLUME_PPLCTNCMPSTB on SAHARA.VOLUME (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_PPLCTNCMPSTB is 
'An index on Composite Application Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ORGANIZATIONBE                               */
/*==============================================================*/
create index I_VOLUME_ORGANIZATIONBE on SAHARA.VOLUME (
ORGANIZATION_BE ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ORGANIZATIONBE is 
'An index on Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_RGNZTNCMPSTB                                 */
/*==============================================================*/
create index I_VOLUME_RGNZTNCMPSTB on SAHARA.VOLUME (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_RGNZTNCMPSTB is 
'An index on Composite Organization Business Entity for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_ISUSEDBYDBVM                                 */
/*==============================================================*/
create index I_VOLUME_ISUSEDBYDBVM on SAHARA.VOLUME (
IS_USED_BY_DB_VM ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_ISUSEDBYDBVM is 
'An index on Is Used By DB VM for dimensional queries.';

/*==============================================================*/
/* Index: I_VOLUME_DBVMNAME                                     */
/*==============================================================*/
create index I_VOLUME_DBVMNAME on SAHARA.VOLUME (
DB_VM_NAME ASC
);

comment on index SAHARA.VOLUME.I_VOLUME_DBVMNAME is 
'An index on DB VM Name for dimensional queries.';

grant SELECT on SAHARA.VOLUME to SAHARA_APP;

/*==============================================================*/
/* Table: VOLUME_                                               */
/*==============================================================*/
create table SAHARA_APP.VOLUME_ 
(
   VOLUME_OID           varchar(512)                   not null,
   DISKGROUP_OID        varchar(512)                   null,
   HOST_OID             varchar(512)                   not null,
   VOLUME_NAME          varchar(256)                   null,
   VM_VERSION           varchar(256)                   null,
   VM_TYPE              varchar(32)                    not null,
   DISK_GROUP_NAME      varchar(256)                   null,
   VOLUME_LAYOUT        varchar(32)                    not null,
   IS_VCS_CVM           char(1)                        null default 'F',
   IS_USED_BY_FILESYSTEM char(1)                        null default 'F',
   IS_USED_BY_APPLICATION char(1)                        null default 'F',
   IS_DIRECT_ATTACHED   char(1)                        null,
   IS_COMPONENT         char(1)                        null,
   IS_BASED_ON_THIN     char(1)                        null,
   IS_EXPORTED          char(1)                        null,
   VOLUME_TYPE          varchar(256)                   null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null,
   IS_USED_BY_DB_VM     char(1)                        null default 'F',
   DB_VM_NAME           varchar(256)                   null
);

comment on table SAHARA_APP.VOLUME_ is 
'This is a staging table for loading data into warehouse table SAHARA.VOLUME.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains host based O/S volume attributes';

comment on column SAHARA_APP.VOLUME_.VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.VOLUME_.HOST_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.VOLUME_.VOLUME_NAME is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column VOLUME_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Name of the volume.';

comment on column SAHARA_APP.VOLUME_.VM_VERSION is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column VM_VERSION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Volume manager version';

comment on column SAHARA_APP.VOLUME_.VM_TYPE is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column VM_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Volume manager type:
SUN VM Disk Suite
SUN VM
VxVM
Microsoft LDM
AIX LVM
Unknown';

comment on column SAHARA_APP.VOLUME_.DISK_GROUP_NAME is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column DISK_GROUP_NAME.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

The discovery state of the volume:

Possible values:
- Discovered
- Partially Discovered
- Missing

Note: Partially Discovered means that the volume was initially discovered by two or more explorers and then one of them stopped discovering the volume.

Note: A volume that has been discovered previously but is not currently discovered will be listed as Missing.';

comment on column SAHARA_APP.VOLUME_.VOLUME_LAYOUT is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column VOLUME_LAYOUT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Volume layout, could be striped, mirrored, etc.';

comment on column SAHARA_APP.VOLUME_.IS_VCS_CVM is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column IS_VCS_CVM.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

True, if the volume is a clustered volume managed using VCS.  False otherwise.';

comment on column SAHARA_APP.VOLUME_.IS_USED_BY_FILESYSTEM is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column IS_USED_BY_FILESYSTEM.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Flag set to true when file system uses the volume';

comment on column SAHARA_APP.VOLUME_.IS_USED_BY_APPLICATION is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column IS_USED_BY_APPLICATION.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Flag set to true when application uses the raw volume';

comment on column SAHARA_APP.VOLUME_.IS_DIRECT_ATTACHED is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column IS_DIRECT_ATTACHED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is direct attached.

Possible values:
''T'' - the Volume is comsuming direct attached storage
''F'' - the Volume is not comsuming direct attached storage';

comment on column SAHARA_APP.VOLUME_.IS_COMPONENT is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column IS_COMPONENT.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is a component volume of a VSET.

Possible values:
''T'' - the Volume is component volumes of a VSET
''F'' - the Volume is not component volumes of a VSET';

comment on column SAHARA_APP.VOLUME_.IS_BASED_ON_THIN is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column IS_BASED_ON_THIN.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is based on thin lun.

Possible values:
''T'' - the Volume is based on thin lun
''F'' - the Volume is not based on thin lun';

comment on column SAHARA_APP.VOLUME_.IS_EXPORTED is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column IS_EXPORTED.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

A boolean flag that indicates if the device is exported.

Possible values:
''T'' - the device is exported
''F'' - the device is not exported';

comment on column SAHARA_APP.VOLUME_.VOLUME_TYPE is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column VOLUME_TYPE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Volume Type:
Data
Snapshot
Clone';

comment on column SAHARA_APP.VOLUME_.APPLICATION_BE is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column APPLICATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.VOLUME_.APPLICATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column APPLICATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite application business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.VOLUME_.ORGANIZATION_BE is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column ORGANIZATION_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.VOLUME_.ORGANIZATION_COMPOSITE_BE is 
'This column is for staging data that will end up in table SAHARA.VOLUME, column ORGANIZATION_COMPOSITE_BE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Composite organization business entity to which this object belongs.

This may be null.';

comment on column SAHARA_APP.VOLUME_.IS_USED_BY_DB_VM is 
'A Boolean flag that indicates whether a volume is utilized by a database volume manager like Oracle ASM.
';

comment on column SAHARA_APP.VOLUME_.DB_VM_NAME is 
'This attribute represents name of the database volume manager that is using the volume';

alter table SAHARA_APP.VOLUME_
   alter IS_VCS_CVM add constraint C_VOLUME_ISVCSCVM check (IS_VCS_CVM is null or (IS_VCS_CVM in ('T','F')));

alter table SAHARA_APP.VOLUME_
   alter IS_USED_BY_FILESYSTEM add constraint C_VOLUME_ISUSEDBYFILESYSTEM check (IS_USED_BY_FILESYSTEM is null or (IS_USED_BY_FILESYSTEM in ('T','F')));

alter table SAHARA_APP.VOLUME_
   alter IS_USED_BY_APPLICATION add constraint C_VOLUME_ISUSEDBYAPPLICATION check (IS_USED_BY_APPLICATION is null or (IS_USED_BY_APPLICATION in ('T','F')));

alter table SAHARA_APP.VOLUME_
   alter IS_DIRECT_ATTACHED add constraint C_VOLUME_ISDIRECTATTACHED check (IS_DIRECT_ATTACHED is null or (IS_DIRECT_ATTACHED in ('T','F')));

alter table SAHARA_APP.VOLUME_
   alter IS_COMPONENT add constraint C_VOLUME_ISCOMPONENT check (IS_COMPONENT is null or (IS_COMPONENT in ('T','F')));

alter table SAHARA_APP.VOLUME_
   alter IS_BASED_ON_THIN add constraint C_VOLUME_ISBASEDONTHIN check (IS_BASED_ON_THIN is null or (IS_BASED_ON_THIN in ('T','F')));

alter table SAHARA_APP.VOLUME_
   alter IS_EXPORTED add constraint C_VOLUME_ISEXPORTED check (IS_EXPORTED is null or (IS_EXPORTED in ('T','F')));

alter table SAHARA_APP.VOLUME_
   alter IS_USED_BY_DB_VM add constraint CKC_IS_USED_BY_DB_VM_VOLUME_ check (IS_USED_BY_DB_VM is null or (IS_USED_BY_DB_VM in ('T','F')));

alter table SAHARA_APP.VOLUME_
   add constraint PK_VOLUME_ primary key (VOLUME_OID);

grant ALTER,DELETE,INSERT,REFERENCES,UPDATE,SELECT on SAHARA_APP.VOLUME_ to SAHARA;

/*==============================================================*/
/* Table: VOLUME_BVIEW_NODE                                     */
/*==============================================================*/
create table SAHARA.VOLUME_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   VOLUME_KEY           numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.VOLUME_BVIEW_NODE is 
'This is a join table between Volume objects and business view nodes.';

comment on column SAHARA.VOLUME_BVIEW_NODE.BVIEW_NODE_KEY is 
'The unique identifier of the business view node with which the object is associated.';

comment on column SAHARA.VOLUME_BVIEW_NODE.VOLUME_KEY is 
'The unique identifier of the Volume object with which the business view node is associated.';

comment on column SAHARA.VOLUME_BVIEW_NODE.BVIEW_KEY is 
'The unique identifier of the business view to which the node belongs.  Although this may be obtained through a join, many operation are made easier by copying the key to the join table record.';

alter table SAHARA.VOLUME_BVIEW_NODE
   add constraint PK_VOLUME_BVIEW_NODE primary key (BVIEW_NODE_KEY, VOLUME_KEY);

/*==============================================================*/
/* Index: I_VLMBVWND_BVIEWKEY                                   */
/*==============================================================*/
create index I_VLMBVWND_BVIEWKEY on SAHARA.VOLUME_BVIEW_NODE (
BVIEW_KEY ASC,
VOLUME_KEY ASC,
BVIEW_NODE_KEY ASC
);

comment on index SAHARA.VOLUME_BVIEW_NODE.I_VLMBVWND_BVIEWKEY is 
'An index on Business View Key for dimensional queries.';

grant SELECT on SAHARA.VOLUME_BVIEW_NODE to SAHARA_APP;

/*==============================================================*/
/* Table: VOLUME_CAPACITY                                       */
/*==============================================================*/
create table SAHARA.VOLUME_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   VOLUME_KEY           numeric(10)                    not null,
   CAPACITY             numeric(22,9)                  not null default 0
);

comment on table SAHARA.VOLUME_CAPACITY is 
'This table contains capacity values for host based O/S volumes.';

comment on column SAHARA.VOLUME_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.VOLUME_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.VOLUME_CAPACITY.CAPACITY is 
'Total capacity of volume.';

alter table SAHARA.VOLUME_CAPACITY
   alter CAPACITY add constraint C_VLMCPCT_CAPACITY check (CAPACITY >= 0);

/*==============================================================*/
/* Index: I_VOLCAPACITY_ROLLUPBATCHKEY                          */
/*==============================================================*/
create index I_VOLCAPACITY_ROLLUPBATCHKEY on SAHARA.VOLUME_CAPACITY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_VOLCAPACITY_DATESKEY                                */
/*==============================================================*/
create index I_VOLCAPACITY_DATESKEY on SAHARA.VOLUME_CAPACITY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_VOLCAPACITY_VOLUMEKEY                               */
/*==============================================================*/
create index I_VOLCAPACITY_VOLUMEKEY on SAHARA.VOLUME_CAPACITY (
VOLUME_KEY ASC
);

grant SELECT on SAHARA.VOLUME_CAPACITY to SAHARA_APP;

/*==============================================================*/
/* Table: VOLUME_CAPACITY_                                      */
/*==============================================================*/
create table SAHARA_APP.VOLUME_CAPACITY_ 
(
   VOLUME_OID           varchar(512)                   not null,
   CAPACITY             numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.VOLUME_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.VOLUME_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This table contains capacity values for host based O/S volumes.';

comment on column SAHARA_APP.VOLUME_CAPACITY_.VOLUME_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.VOLUME_CAPACITY_.CAPACITY is 
'This column is for staging data that will end up in table SAHARA.VOLUME_CAPACITY, column CAPACITY.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

Total capacity of volume.';

alter table SAHARA_APP.VOLUME_CAPACITY_
   alter CAPACITY add constraint C_VLMCPCT_CAPACITY check (CAPACITY >= 0);

alter table SAHARA_APP.VOLUME_CAPACITY_
   add constraint PK_VOLUME_CAPACITY_ primary key (VOLUME_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.VOLUME_CAPACITY_ to SAHARA;

alter table SAHARA.APP_GROUP_BVIEW_NODE
   add constraint FK_PPGRPBVWND_APPGROUP foreign key (APP_GROUP_KEY)
      references SAHARA.APP_GROUP (APP_GROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.APP_GROUP_BVIEW_NODE
   add constraint FK_PPGRPBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.APP_GROUP_BVIEW_NODE
   add constraint FK_PPGRPBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.APP_GROUP_CAPACITY
   add constraint FK_PPGRPCPCT_APPGROUP foreign key (APP_GROUP_KEY)
      references SAHARA.APP_GROUP (APP_GROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.APP_GROUP_CAPACITY
   add constraint FK_PPGRPCPCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.APP_GROUP_CAPACITY
   add constraint FK_PPGRPCPCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.APP_GROUP_HOST
   add constraint FK_APPGROUPHOST_APPGROUP foreign key (APP_GROUP_KEY)
      references SAHARA.APP_GROUP (APP_GROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.APP_GROUP_HOST
   add constraint FK_APPGROUPHOST_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.APP_GROUP_HOST
   add constraint FK_APPGROUPHOST_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.APP_GROUP_HOST
   add constraint FK_APPGROUPHOST_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ARRAY
   add constraint FK_ARRAY_ARRAY foreign key (PARENT_ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ARRAY
   add constraint FK_ARRAY_FK_ARRAY__ARRAY foreign key (CLUSTER_ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete restrict;

comment on foreign key SAHARA.ARRAY.FK_ARRAY_FK_ARRAY__ARRAY is 
'To join array with clustered array';

alter table SAHARA.ARRAY_BVIEW_NODE
   add constraint FK_RRBVWND_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ARRAY_BVIEW_NODE
   add constraint FK_RRBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ARRAY_BVIEW_NODE
   add constraint FK_RRBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ARRAY_STORAGE
   add constraint FK_ARRAYSTORAGE_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ARRAY_STORAGE
   add constraint FK_ARRAYSTORAGE_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.ARRAY_STORAGE
   add constraint FK_ARRAYSTORAGE_DMBATCH foreign key (NA_DM_BATCH_KEY)
      references SAHARA.DM_BATCH (DM_BATCH_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.ARRAY_STORAGE
   add constraint FK_ARRAYSTORAGE_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_APPROVAL_PENDING
   add constraint FK_BVIEW_AP_APFK_BVIEW_RU foreign key (BVIEW_RULES_KEY)
      references SAHARA.BVIEW_RULES (BVIEW_RULES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_CONDITIONS
   add constraint FK_BVIEW_CO_BUSINESS__BVIEW_RU foreign key (BVIEW_RULES_KEY)
      references SAHARA.BVIEW_RULES (BVIEW_RULES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_DIMENSION_BVIEW
   add constraint FK_BVWDMNSNBVW_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_DIMENSION_BVIEW
   add constraint FK_BVWDMNSNBVW_BVWDMNSN foreign key (DIMENSION)
      references SAHARA.BVIEW_DIMENSION (DIMENSION)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_IMPL
   add constraint FK_BVIEWIMPL_BVIEWNODE foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_LEVEL
   add constraint FK_BVIEWLEVEL_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_NODE
   add constraint FK_BVIEWNODE_BVIEWLEVEL foreign key (BVIEW_KEY, LEVEL)
      references SAHARA.BVIEW_LEVEL (BVIEW_KEY, LEVEL)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_NODE
   add constraint FK_BVIEWNODE_BVIEWNODE foreign key (PARENT_BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_NODE
   add constraint FK_BVIEWNODE_BVIEWNODE_2 foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_RULE
   add constraint FK_BVIEWRULE_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_RULE
   add constraint FK_BVIEWRULE_BVWDMNSN foreign key (DIMENSION)
      references SAHARA.BVIEW_DIMENSION (DIMENSION)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_RULES
   add constraint FK_BVIEW_RU_BVIEW_KEY_BVIEW_IM foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_RULES
   add constraint FK_BVIEW_RU_DIMENSION_BVIEW_DI foreign key (DIMENSION)
      references SAHARA.BVIEW_DIMENSION (DIMENSION)
      on update restrict
      on delete restrict;

alter table SAHARA.BVIEW_RULES
   add constraint FK_BVIEW_RU_NODE_KEY_BVIEW_NO foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_SUBRULE
   add constraint FK_BVIEWSUBRULE_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_SUBRULE
   add constraint FK_BVIEW_SU_FK_BVIEWS_BVIEW_RU foreign key (BVIEW_RULE_KEY)
      references SAHARA.BVIEW_RULE (BVIEW_RULE_KEY)
      on update cascade
      on delete restrict;

alter table SAHARA.BVIEW_SUBRULE_ELEMENT
   add constraint FK_BVWSBRLLMNT_BVWDMNSN foreign key (DIMENSION)
      references SAHARA.BVIEW_DIMENSION (DIMENSION)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint FK_CLUSTER__REFERENCE_BVIEW_NO foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint FK_CLUSTER__REFERENCE_BVIEW_IM foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint FK_CLUSTER__REFERENCE_CLUSTER foreign key (CLUSTER_KEY)
      references SAHARA.CLUSTER (CLUSTER_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_HOST
   add constraint FK_CLUSTERHOST_CLUSTER foreign key (CLUSTER_KEY)
      references SAHARA.CLUSTER (CLUSTER_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_HOST
   add constraint FK_CLUSTERHOST_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_INVENTORY
   add constraint FK_CLUSTERINV_CLUSTER foreign key (CLUSTER_KEY)
      references SAHARA.CLUSTER (CLUSTER_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_INVENTORY
   add constraint FK_CLUSTERINV_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_INVENTORY
   add constraint FK_CLUSTERINV_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_REMOTECLUSTER
   add constraint FK_CLUSTERRCLUSTER_CLUSTER foreign key (CLUSTER_KEY)
      references SAHARA.CLUSTER (CLUSTER_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.COGNOS_HIERARCHY_ATTRIBUTE
   add constraint FK_CGNSHRRCHTTR_CSTMTTRBTCNT foreign key (CUSTOM_ATTRIBUTE_CONTEXT)
      references SAHARA.CUSTOM_ATTRIBUTE_CONTEXT (CUSTOM_ATTRIBUTE_CONTEXT)
      on update restrict
      on delete cascade;

alter table SAHARA.COLUMN_METADATA_BASE
   add constraint FK_CLMNMTDTBS_TBLMTDTBS foreign key (OWNER, TABLE_NAME)
      references SAHARA.TABLE_METADATA_BASE (OWNER, TABLE_NAME)
      on update restrict
      on delete cascade;

alter table SAHARA.CONTROL_OWNER
   add constraint FK_CONTROLOWNER_SESSION foreign key (SESSION_KEY)
      references SAHARA."SESSION" (SESSION_KEY)
      on update restrict
      on delete set null;

alter table SAHARA.CUSTOM_ATTRIBUTE
   add constraint FK_CSTMTTRBT_CSTMTTRBTCNT foreign key (CUSTOM_ATTRIBUTE_CONTEXT)
      references SAHARA.CUSTOM_ATTRIBUTE_CONTEXT (CUSTOM_ATTRIBUTE_CONTEXT)
      on update restrict
      on delete cascade;

alter table SAHARA.DATABASES
   add constraint FK_DATABASES_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DATABASE_CAPACITY
   add constraint FK_DTBSCPCT_DATABASES foreign key (DATABASES_KEY)
      references SAHARA.DATABASES (DATABASES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DATABASE_CAPACITY
   add constraint FK_DTBSCPCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.DATABASE_CAPACITY
   add constraint FK_DTBSCPCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DISKGROUP
   add constraint FK_DISKGROUP_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DISKGROUP_CAPACITY
   add constraint FK_DISKGROU_FK_DSKGRP_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DISKGROUP_CAPACITY
   add constraint FK_FLSSTMCPCT_FILESYSTEM foreign key (DISKGROUP_KEY)
      references SAHARA.DISKGROUP (DISKGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.DISKGROUP_CAPACITY
   add constraint FK_DISKGROU_FK_DSKGRP_ROLLUP_B foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.FILESYSTEM
   add constraint FK_FILESYSTEM_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.FILESYSTEM
   add constraint FK_FILESYSTEM_VOLUME foreign key (VOLUME_KEY)
      references SAHARA.VOLUME (VOLUME_KEY)
      on update restrict
      on delete set null;

comment on foreign key SAHARA.FILESYSTEM.FK_FILESYSTEM_VOLUME is 
'This links file systems to volumes that they are created from.  File systems created from volume sets or raw devices, will have null for this key. ';

alter table SAHARA.FILESYSTEM_BVIEW_NODE
   add constraint FK_FLSSTMBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.FILESYSTEM_BVIEW_NODE
   add constraint FK_FLSSTMBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.FILESYSTEM_BVIEW_NODE
   add constraint FK_FLSSTMBVWND_FILESYSTEM foreign key (FILESYSTEM_KEY)
      references SAHARA.FILESYSTEM (FILESYSTEM_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.FILESYSTEM_CAPACITY
   add constraint FK_FLSSTMCPCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.FILESYSTEM_CAPACITY
   add constraint FK_FLSSTMCPCT_FILESYSTEM foreign key (FILESYSTEM_KEY)
      references SAHARA.FILESYSTEM (FILESYSTEM_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.FILESYSTEM_CAPACITY
   add constraint FK_FLSSTMCPCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_ARRAY_CAPACITY
   add constraint FK_HSTRRCPCT_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_ARRAY_CAPACITY
   add constraint FK_HSTRRCPCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.HOST_ARRAY_CAPACITY
   add constraint FK_HSTRRCPCT_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_ARRAY_CAPACITY
   add constraint FK_HSTRRCPCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_BVIEW_NODE
   add constraint FK_HSTBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_BVIEW_NODE
   add constraint FK_HSTBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_BVIEW_NODE
   add constraint FK_HSTBVWND_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_CAPACITY
   add constraint FK_HOSTCAPACITY_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.HOST_CAPACITY
   add constraint FK_HOSTCAPACITY_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.HOST_CAPACITY
   add constraint FK_HOSTCAPACITY_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN
   add constraint FK_LUN_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.LUN
   add constraint FK_LUN_LUN_TO_NA_NAS_FILE foreign key (NAS_FILESYSTEM_KEY)
      references SAHARA.NAS_FILESYSTEM (NAS_FILESYSTEM_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.LUN_BVIEW_NODE
   add constraint FK_LUNBVIEWNODE_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_BVIEW_NODE
   add constraint FK_LUNBVIEWNODE_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_BVIEW_NODE
   add constraint FK_LUNBVIEWNODE_LUN foreign key (LUN_KEY)
      references SAHARA.LUN (LUN_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.LUN_CAPACITY
   add constraint FK_LUNCAPACITY_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.LUN_CAPACITY
   add constraint FK_LUNCAPACITY_LUN foreign key (LUN_KEY)
      references SAHARA.LUN (LUN_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_CAPACITY
   add constraint FK_LUNCAPACITY_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_DISKGROUP
   add constraint FK_LNDSKGRP_LUN foreign key (LUN_KEY)
      references SAHARA.LUN (LUN_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_DISKGROUP
   add constraint FK_LNDSKGRP_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_DISKGROUP
   add constraint FK_LNDSKGRP_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_DISKGROUP
   add constraint FK_LNDSKGRP_DISKGROUP foreign key (DISKGROUP_KEY)
      references SAHARA.DISKGROUP (DISKGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_DISKGROUP
   add constraint FK_LNDSKGRP_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_FILESYSTEM
   add constraint FK_LNFLSSTM_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_FILESYSTEM
   add constraint FK_LNFLSSTM_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_FILESYSTEM
   add constraint FK_LNFLSSTM_FILESYSTEM foreign key (FILESYSTEM_KEY)
      references SAHARA.FILESYSTEM (FILESYSTEM_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_FILESYSTEM
   add constraint FK_LNFLSSTM_LUN foreign key (LUN_KEY)
      references SAHARA.LUN (LUN_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_FILESYSTEM
   add constraint FK_LNFLSSTM_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_HOST
   add constraint FK_LUNHOST_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.LUN_HOST
   add constraint FK_LUNHOST_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_HOST
   add constraint FK_LUNHOST_LUN foreign key (LUN_KEY)
      references SAHARA.LUN (LUN_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_HOST
   add constraint FK_LUNHOST_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_VOLUME
   add constraint FK_LUNVOLUME_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_VOLUME
   add constraint FK_LUNVOLUME_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_VOLUME
   add constraint FK_LUNVOLUME_LUN foreign key (LUN_KEY)
      references SAHARA.LUN (LUN_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_VOLUME
   add constraint FK_LUNVOLUME_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.LUN_VOLUME
   add constraint FK_LUNVOLUME_VOLUME foreign key (VOLUME_KEY)
      references SAHARA.VOLUME (VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.MESSAGE_LOG
   add constraint FK_MESSAGELOG_SESSION foreign key (SESSION_KEY)
      references SAHARA."SESSION" (SESSION_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FILESYSTEM
   add constraint FK_NSFLSSTM_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FILESYSTEM
   add constraint FK_NSFLSSTM_NETAPPVOLUME foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   add constraint FK_NSFLSSTMCPCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   add constraint FK_NSFLSSTMCPCT_NSFLSSTM foreign key (NAS_FILESYSTEM_KEY)
      references SAHARA.NAS_FILESYSTEM (NAS_FILESYSTEM_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   add constraint FK_NSFLSSTMCPCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FS_BVIEW_NODE
   add constraint FK_NSFSBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FS_BVIEW_NODE
   add constraint FK_NSFSBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_FS_BVIEW_NODE
   add constraint FK_NSFSBVWND_NSFLSSTM foreign key (NAS_FILESYSTEM_KEY)
      references SAHARA.NAS_FILESYSTEM (NAS_FILESYSTEM_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_QUOTA
   add constraint FK_NAS_QUOT_FK_NAS_QU_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_QUOTA
   add constraint FK_NAS_QUOT_FK_NAS_QU_NAS_FILE foreign key (NAS_FILESYSTEM_KEY)
      references SAHARA.NAS_FILESYSTEM (NAS_FILESYSTEM_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_QUOTA_CAPACITY
   add constraint FK_NAS_QUOT_FK_NAS_QU_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_QUOTA_CAPACITY
   add constraint FK_NAS_QUOT_FK_NAS_QU_ROLLUP_B foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_QUOTA_CAPACITY
   add constraint FK_NAS_QUOT_NAS_QUOTA_NAS_QUOT foreign key (NAS_QUOTA_KEY)
      references SAHARA.NAS_QUOTA (NAS_QUOTA_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_SHARE_HOST
   add constraint FK_NAS_SHAR_FK_NAS_SH_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_SHARE_HOST
   add constraint FK_NAS_SHAR_FK_NAS_SH_NETAPP_S foreign key (NETAPP_SHARE_KEY)
      references SAHARA.NETAPP_SHARE (NETAPP_SHARE_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_SHARE_HOST_FILESYSTEM
   add constraint FK_NAS_SHAR_FK_NAS_SH_FILESYST foreign key (FILESYSTEM_KEY)
      references SAHARA.FILESYSTEM (FILESYSTEM_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_SHARE_HOST_FILESYSTEM
   add constraint FK_NAS_SHAR_FK_NAS_SH_NETAPP_S foreign key (NETAPP_SHARE_KEY)
      references SAHARA.NETAPP_SHARE (NETAPP_SHARE_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_VOLUME_FILER
   add constraint FK_NAS_VOLU_FK_NAS_VO_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_VOLUME_FILER
   add constraint FK_NAS_VOLU_FK_NAS_VO_NETAPP_V foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NA_VOLUME_BVIEW_NODE
   add constraint FK_NVLMBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NA_VOLUME_BVIEW_NODE
   add constraint FK_NVLMBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NA_VOLUME_BVIEW_NODE
   add constraint FK_NVLMBVWND_NETAPPVOLUME foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_FILE_FCAT
   add constraint FK_NTPPFLFCT_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_FILE_FCAT
   add constraint FK_NTPPFLFCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_FILE_FCAT
   add constraint FK_NTPPFLFCT_DMBATCH foreign key (DM_BATCH_KEY)
      references SAHARA.DM_BATCH (DM_BATCH_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_FILE_FCAT
   add constraint FK_NTPPFLFCT_FILECATEGORY foreign key (FILE_CATEGORY_KEY)
      references SAHARA.FILE_CATEGORY (FILE_CATEGORY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_FILE_FCAT
   add constraint FK_NTPPFLFCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE
   add constraint FK_NETAPPQTREE_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE
   add constraint FK_NETAPPQTREE_NETAPPVOLUME foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_BVIEW_NODE
   add constraint FK_NTPPQTRBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_BVIEW_NODE
   add constraint FK_NTPPQTRBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_BVIEW_NODE
   add constraint FK_NTPPQTRBVWND_NETAPPQTREE foreign key (NETAPP_QTREE_KEY)
      references SAHARA.NETAPP_QTREE (NETAPP_QTREE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_FILE
   add constraint FK_NTPPQTRFL_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_QTREE_FILE
   add constraint FK_NTPPQTRFL_DMBATCH foreign key (DM_BATCH_KEY)
      references SAHARA.DM_BATCH (DM_BATCH_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_QTREE_FILE
   add constraint FK_NTPPQTRFL_NETAPPQTREE foreign key (NETAPP_QTREE_KEY)
      references SAHARA.NETAPP_QTREE (NETAPP_QTREE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_FILE
   add constraint FK_NTPPQTRFL_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_FILE_FCAT
   add constraint FK_NTPPQTRFLFCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_QTREE_FILE_FCAT
   add constraint FK_NTPPQTRFLFCT_DMBATCH foreign key (DM_BATCH_KEY)
      references SAHARA.DM_BATCH (DM_BATCH_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_QTREE_FILE_FCAT
   add constraint FK_NTPPQTRFLFCT_FILECATEGORY foreign key (FILE_CATEGORY_KEY)
      references SAHARA.FILE_CATEGORY (FILE_CATEGORY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_FILE_FCAT
   add constraint FK_NTPPQTRFLFCT_NETAPPQTREE foreign key (NETAPP_QTREE_KEY)
      references SAHARA.NETAPP_QTREE (NETAPP_QTREE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_QTREE_FILE_FCAT
   add constraint FK_NTPPQTRFLFCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE
   add constraint FK_NETAPPSHARE_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE
   add constraint FK_NETAPPSHARE_NETAPPQTREE foreign key (NETAPP_QTREE_KEY)
      references SAHARA.NETAPP_QTREE (NETAPP_QTREE_KEY)
      on update restrict
      on delete set null;

alter table SAHARA.NETAPP_SHARE
   add constraint FK_NETAPPSHARE_NETAPPVOLUME foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE
   add constraint FK_NETAPPSHARE_NSFLSSTM foreign key (NAS_FILESYSTEM_KEY)
      references SAHARA.NAS_FILESYSTEM (NAS_FILESYSTEM_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_BVIEW_NODE
   add constraint FK_NTPPSHRBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_BVIEW_NODE
   add constraint FK_NTPPSHRBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_BVIEW_NODE
   add constraint FK_NTPPSHRBVWND_NETAPPSHARE foreign key (NETAPP_SHARE_KEY)
      references SAHARA.NETAPP_SHARE (NETAPP_SHARE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_FILE
   add constraint FK_NTPPSHRFL_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_SHARE_FILE
   add constraint FK_NTPPSHRFL_NETAPPSHARE foreign key (NETAPP_SHARE_KEY)
      references SAHARA.NETAPP_SHARE (NETAPP_SHARE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_FILE
   add constraint FK_NTPPSHRFL_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_FILE_FCAT
   add constraint FK_NTPPSHRFLFCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_SHARE_FILE_FCAT
   add constraint FK_NTPPSHRFLFCT_DMBATCH foreign key (DM_BATCH_KEY)
      references SAHARA.DM_BATCH (DM_BATCH_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_SHARE_FILE_FCAT
   add constraint FK_NTPPSHRFLFCT_FILECATEGORY foreign key (FILE_CATEGORY_KEY)
      references SAHARA.FILE_CATEGORY (FILE_CATEGORY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_FILE_FCAT
   add constraint FK_NTPPSHRFLFCT_NETAPPSHARE foreign key (NETAPP_SHARE_KEY)
      references SAHARA.NETAPP_SHARE (NETAPP_SHARE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_SHARE_FILE_FCAT
   add constraint FK_NTPPSHRFLFCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_VOLUME
   add constraint FK_NETAPP_V_FK_NAS_VO_NETAPP_V foreign key (STORAGE_POOL_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete restrict;

comment on foreign key SAHARA.NETAPP_VOLUME.FK_NETAPP_V_FK_NAS_VO_NETAPP_V is 
'Volume to Storage Pool';

alter table SAHARA.NETAPP_VOLUME
   add constraint FK_NETAPPVOLUME_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_VOLUME_FILE
   add constraint FK_NTPPVLMFL_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_VOLUME_FILE
   add constraint FK_NTPPVLMFL_NETAPPVOLUME foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_VOLUME_FILE
   add constraint FK_NTPPVLMFL_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_VOLUME_FILE_FCAT
   add constraint FK_NTPPVLMFLFCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_VOLUME_FILE_FCAT
   add constraint FK_NTPPVLMFLFCT_DMBATCH foreign key (DM_BATCH_KEY)
      references SAHARA.DM_BATCH (DM_BATCH_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NETAPP_VOLUME_FILE_FCAT
   add constraint FK_NTPPVLMFLFCT_FILECATEGORY foreign key (FILE_CATEGORY_KEY)
      references SAHARA.FILE_CATEGORY (FILE_CATEGORY_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_VOLUME_FILE_FCAT
   add constraint FK_NTPPVLMFLFCT_NETAPPVOLUME foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NETAPP_VOLUME_FILE_FCAT
   add constraint FK_NTPPVLMFLFCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.P_CUSTOM_ATTRIBUTE
   add constraint FK_PCSTMTTRBT_CSTMTTRBTCNT foreign key (CUSTOM_ATTRIBUTE_CONTEXT)
      references SAHARA.CUSTOM_ATTRIBUTE_CONTEXT (CUSTOM_ATTRIBUTE_CONTEXT)
      on update restrict
      on delete cascade;

alter table SAHARA.ROLLUP_BATCH
   add constraint FK_ROLLUPBATCH_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.ROLLUP_BATCH
   add constraint FK_ROLLUPBATCH_RLLPBTCHSTTS foreign key (STATUS)
      references SAHARA.ROLLUP_BATCH_STATUS (STATUS)
      on update restrict
      on delete restrict;

alter table SAHARA.ROLLUP_BATCH
   add constraint FK_ROLLUPBATCH_RLLPSRCTP foreign key (ROLLUP_SOURCE_TYPE_KEY)
      references SAHARA.ROLLUP_SOURCE_TYPE (ROLLUP_SOURCE_TYPE_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.ROLLUP_BATCH
   add constraint FK_ROLLUPBATCH_SESSION foreign key (SESSION_KEY)
      references SAHARA."SESSION" (SESSION_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.ROLLUP_DATA_SET
   add constraint FK_RLLPDTST_RLLPDTSTTP foreign key (ROLLUP_DATA_SET_TYPE_KEY)
      references SAHARA.ROLLUP_DATA_SET_TYPE (ROLLUP_DATA_SET_TYPE_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.ROLLUP_DATA_SET
   add constraint FK_RLLPDTST_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ROLLUP_DATA_SET_COLUMN
   add constraint FK_RLLPDTSTCLMN_PCSTMTTRBT foreign key (CUSTOM_ATTRIBUTE_KEY)
      references SAHARA.P_CUSTOM_ATTRIBUTE (CUSTOM_ATTRIBUTE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ROLLUP_DATA_SET_COLUMN
   add constraint FK_RLLPDTSTCLMN_RLLPDTSTTP foreign key (ROLLUP_DATA_SET_TYPE_KEY)
      references SAHARA.ROLLUP_DATA_SET_TYPE (ROLLUP_DATA_SET_TYPE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ROLLUP_DATA_SET_STAGE
   add constraint FK_RLLPDTSTSTG_RLLPDTSTTP foreign key (ROLLUP_DATA_SET_TYPE_KEY)
      references SAHARA.ROLLUP_DATA_SET_TYPE (ROLLUP_DATA_SET_TYPE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.ROLLUP_SOURCE
   add constraint FK_ROLLUPSOURCE_RLLPSRCTP foreign key (ROLLUP_SOURCE_TYPE_KEY)
      references SAHARA.ROLLUP_SOURCE_TYPE (ROLLUP_SOURCE_TYPE_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.RSOURCE_TYPE_DATA_SET_TYPE
   add constraint FK_RSRCTPDTSTTP_RLLPDTSTTP foreign key (ROLLUP_DATA_SET_TYPE_KEY)
      references SAHARA.ROLLUP_DATA_SET_TYPE (ROLLUP_DATA_SET_TYPE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.RSOURCE_TYPE_DATA_SET_TYPE
   add constraint FK_RSRCTPDTSTTP_RLLPSRCTP foreign key (ROLLUP_SOURCE_TYPE_KEY)
      references SAHARA.ROLLUP_SOURCE_TYPE (ROLLUP_SOURCE_TYPE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.RULE
   add constraint FK_RULE_RULECONTEXT foreign key (RULE_CONTEXT)
      references SAHARA.RULE_CONTEXT (RULE_CONTEXT)
      on update restrict
      on delete cascade;

alter table SAHARA.RULE_ELEMENT
   add constraint FK_RULEELEMENT_RLLMNTSCTN foreign key (RULE_CONTEXT, SECTION_DISPLAY_NAME)
      references SAHARA.RULE_ELEMENT_SECTION (RULE_CONTEXT, SECTION_DISPLAY_NAME)
      on update restrict
      on delete cascade;

alter table SAHARA.RULE_ELEMENT_SECTION
   add constraint FK_RLLMNTSCTN_RULECONTEXT foreign key (RULE_CONTEXT)
      references SAHARA.RULE_CONTEXT (RULE_CONTEXT)
      on update restrict
      on delete cascade;

alter table SAHARA.SECURITY_GROUP
   add constraint FK_SECURITY_FK_SECURI_BROKER_I foreign key (BROKER_ID)
      references SAHARA.BROKER_INFO (BROKER_ID)
      on update restrict
      on delete cascade;

alter table SAHARA.SECURITY_GROUP
   add constraint FK_SECURITY_FK_SECURI_SECURITY foreign key (ROLE_ID)
      references SAHARA.SECURITY_ROLE (ROLE_ID)
      on update restrict
      on delete cascade;

alter table SAHARA.SECURITY_GROUP_REL
   add constraint FK_SECURITY_SECGROUPR_SECURITY foreign key (ROLE_ID)
      references SAHARA.SECURITY_ROLE (ROLE_ID)
      on update restrict
      on delete cascade;

alter table SAHARA.SECURITY_GROUP_REL
   add constraint FK_SECURITY_SEC_GROUP_SECURITY foreign key (SECURITY_GROUP_ID)
      references SAHARA.SECURITY_GROUP (SECURITY_GROUP_ID)
      on update restrict
      on delete cascade;

alter table SAHARA.SECURITY_GROUP_REL
   add constraint FK_SECURITY_SEC_GROUP_BVIEW_IM foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP
   add constraint FK_SG_CLUSTER foreign key (CLUSTER_KEY)
      references SAHARA.CLUSTER (CLUSTER_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint FK_SERVICEG_REFERENCE_BVIEW_NO foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint FK_SERVICEG_REFERENCE_BVIEW_IM foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint FK_SERVICEG_REFERENCE_SERVICEG foreign key (SERVICEGROUP_KEY)
      references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HIST_EVENTS
   add constraint FK_SERVICEG_FK_SGHIST_SERVICEG foreign key (SERVICEGROUP_KEY)
      references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HIST_EVENTS
   add constraint FK_SERVICEG_FK_SGHIST_ROLLUP_B foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HIST_EVENTS
   add constraint FK_SERVICEG_FK_SGHIST_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HOST
   add constraint FK_SGHOST_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HOST
   add constraint FK_SGHOST_SG foreign key (SERVICEGROUP_KEY)
      references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_INVENTORY
   add constraint FK_SGINV_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_INVENTORY
   add constraint FK_SGINV_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_INVENTORY
   add constraint FK_SGINV_SG foreign key (SERVICEGROUP_KEY)
      references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA."SESSION"
   add constraint FK_SESSION_USERS foreign key (USERS_KEY)
      references SAHARA.USERS (USERS_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.SF_DEPLOYED_LICENSES
   add constraint FK_SF_DEPLO_SF_LICENC_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_DEPLOYED_LICENSES
   add constraint FK_SF_DEPLO_LICS_LIC_HOST foreign key (SF_LICENSE_HOST_KEY)
      references SAHARA.SF_LICENSE_HOST (SF_LICENSE_HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_DEPLOYED_LICENSES
   add constraint FK_SF_DEPLO_LICENS_SF_LDEPL foreign key (SF_LICENSE_DEPLOYMENT_KEY)
      references SAHARA.SF_LICENSE_DEPLOYMENT (SF_LICENSE_DEPLOYMENT_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_DEPLOYED_LICENSES
   add constraint FK_SF_DEPLO_SF_LICENS_SF_LICEN foreign key (SF_LICENSE_KEY)
      references SAHARA.SF_LICENSE (SF_LICENSE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_DEPLOYED_LICENSES
   add constraint FK_SF_DEPLO_SF_LICENS_ROLLUP_B foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_DEPLOYED_LICENSES
   add constraint FK_SF_DEPLO_SF_LICENS_SF_PRODU foreign key (SF_PRODUCT_KEY)
      references SAHARA.SF_PRODUCT (SF_PRODUCT_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   add constraint FK_SF_LIC_DEPLOYMENT_TO_HOST foreign key (SF_LICENSE_HOST_KEY)
      references SAHARA.SF_LICENSE_HOST (SF_LICENSE_HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   add constraint FK_SF_LIC_DEPLOYMENT_LIC foreign key (SF_LICENSE_KEY)
      references SAHARA.SF_LICENSE (SF_LICENSE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_LICENSE_DEPLOYMENT
   add constraint FK_SF_LICEN_SF_DEPLOY_SF_PRODU foreign key (SF_PRODUCT_KEY)
      references SAHARA.SF_PRODUCT (SF_PRODUCT_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SF_LICENSE_HOST
   add constraint FK_SF_LICEN_HOST_TO_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE
   add constraint FK_SRCCSTMTTRBT_CSTMTTRBTCNT foreign key (CUSTOM_ATTRIBUTE_CONTEXT)
      references SAHARA.CUSTOM_ATTRIBUTE_CONTEXT (CUSTOM_ATTRIBUTE_CONTEXT)
      on update restrict
      on delete cascade;

alter table SAHARA.SWITCH_BVIEW_NODE
   add constraint FK_SWTCHBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SWITCH_BVIEW_NODE
   add constraint FK_SWTCHBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SWITCH_BVIEW_NODE
   add constraint FK_SWTCHBVWND_SWITCH foreign key (SWITCH_KEY)
      references SAHARA.SWITCH (SWITCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SWITCH_PORT_COUNTS
   add constraint FK_SWTCHPRTCNTS_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.SWITCH_PORT_COUNTS
   add constraint FK_SWTCHPRTCNTS_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SWITCH_PORT_COUNTS
   add constraint FK_SWTCHPRTCNTS_SWITCH foreign key (SWITCH_KEY)
      references SAHARA.SWITCH (SWITCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.TIER
   add constraint FK_TIER_RULE foreign key (RULE_KEY)
      references SAHARA.RULE (RULE_KEY)
      on update restrict
      on delete restrict;

comment on foreign key SAHARA.TIER.FK_TIER_RULE is 
'It''s tempting to make this a "set null" on delete, but don''t do it!  A trigger on TIER causes tier rules to be deleted as needed.  Making this a "set null"; causes the trigger to become recursive or generate mutation errors (depending on how you arrange the trigger).';

alter table SAHARA.TIER
   add constraint FK_TIER_TIERCONTEXT foreign key (TIER_CONTEXT)
      references SAHARA.TIER_CONTEXT (TIER_CONTEXT)
      on update restrict
      on delete cascade;

alter table SAHARA.USER_CONFIGURATION
   add constraint FK_SRCNFGRTN_USERS foreign key (USERS_KEY)
      references SAHARA.USERS (USERS_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.VOLUME
   add constraint FK_VOLUME_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.VOLUME
   add constraint FK_VOLUME_FK_VOLUME_DISKGROU foreign key (DISKGROUP_KEY)
      references SAHARA.DISKGROUP (DISKGROUP_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.VOLUME_BVIEW_NODE
   add constraint FK_VLMBVWND_BVIEWIMPL foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.VOLUME_BVIEW_NODE
   add constraint FK_VLMBVWND_BVIEWNODE foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.VOLUME_BVIEW_NODE
   add constraint FK_VLMBVWND_VOLUME foreign key (VOLUME_KEY)
      references SAHARA.VOLUME (VOLUME_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.VOLUME_CAPACITY
   add constraint FK_VLMCPCT_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.VOLUME_CAPACITY
   add constraint FK_VLMCPCT_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.VOLUME_CAPACITY
   add constraint FK_VLMCPCT_VOLUME foreign key (VOLUME_KEY)
      references SAHARA.VOLUME (VOLUME_KEY)
      on update restrict
      on delete cascade;

