/* --- */


/* $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $ */
/* $Copyright: Copyright (c) 2012 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $ */

/* --- */

/*==============================================================*/
/* Table: DB_VERSION                                            */
/*==============================================================*/

if not exists(
   select 1 from sys.systable
   where table_name='DB_VERSION'
     and table_type in ('BASE', 'GBL TEMP')
) then

   create table SAHARA.DB_VERSION
   (
      MAJOR                integer                        not null,
      MINOR                integer                        not null,
      MP_VERSION           integer                        not null
   );

   alter table SAHARA.DB_VERSION add constraint PK_DB_VERSION primary key clustered (MAJOR, MINOR, MP_VERSION);

end if;


-- Need to call the custom attribute store procedure to drop the staging table. 
-- By this time the procedure is already loaded.
BEGIN
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vStagingTableName VARCHAR(1024);
  DECLARE @vStagingColExists INTEGER;   


FOR AttributesLOOP as AttributesCUR dynamic scroll cursor for
      select CUSTOM_ATTRIBUTE_KEY as CUSTOM_ATTRIBUTE_KEY_R,
        CUSTOM_ATTRIBUTE_CONTEXT as CUSTOM_ATTRIBUTE_CONTEXT_R,
        TYPE as TYPE_R,
        DISPLAY_NAME as DISPLAY_NAME_R,
        COLUMN_NAME as COLUMN_NAME_R,
        COLUMN_TYPE as COLUMN_TYPE_R,
        SOURCE_NAME as SOURCE_NAME_R    
      from SAHARA.CUSTOM_ATTRIBUTE CONFIGURED
      where CONFIGURED.SOURCE_NAME not in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY')
	    and exists (select 1
                         from SAHARA.P_CUSTOM_ATTRIBUTE PUBLISHED
                        where PUBLISHED.CUSTOM_ATTRIBUTE_KEY =
                              CONFIGURED.CUSTOM_ATTRIBUTE_KEY)
     DO            

            set @vStagingTableName = CUSTOM_ATTRIBUTE_CONTEXT_R || '_';

    
                select SAHARA.fn_HELPER_ColumnExists('SAHARA_APP',@vStagingTableName,COLUMN_NAME_R) into 
                        @vStagingColExists ;

                if @vStagingColExists = 1 then        
                    SET @vSQL = 'lock table ' ||
                    -- source table or view
                    'SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R || '_ ' ||
                            'in exclusive mode';

                    EXECUTE IMMEDIATE @vSQL;
	                SET @vSQL = 'alter table SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R ||
                        '_ drop  "' || COLUMN_NAME_R || '"';
                    EXECUTE IMMEDIATE @vSQL;                    
                end if;            
      COMMIT ;
     END FOR;
 END;



-- Passing of the one will drop the columns related to the custom attribute.

if not exists (select 1 from sys.syscolumns where tname='BROKER_INFO' and cname='LDAP_PORT') then
  ALTER TABLE SAHARA.BROKER_INFO ADD LDAP_PORT int null;
end if;

if not exists (select 1 from sys.syscolumns where tname='LUN_' and cname='TYPE') then
    alter table SAHARA_APP.LUN_
    add
    (TYPE                 varchar(32)                    null,
     SUB_TYPE             varchar(32)                    null,
     STORAGE_TECHNOLOGY   varchar(32)                    null,
     STORAGE_POOL_NAME    varchar(256)                   null,
     CONNECTION_PROTOCOL  varchar(32)                    null
    );
end if;


if not exists (select 1 from sys.syscolumns where tname='LUN' and cname='TYPE') then
    alter table SAHARA.LUN
    add
    (TYPE                 varchar(32)                    null,
     SUB_TYPE             varchar(32)                    null,
     STORAGE_TECHNOLOGY   varchar(32)                    null,
     STORAGE_POOL_NAME    varchar(256)                   null,
     CONNECTION_PROTOCOL  varchar(32)                    null
    );
end if;

if not exists (select 1 from sys.syscolumns where tname='HOST_' and cname='VOM_LICENSE_REQUIRED') then
    alter table SAHARA_APP.HOST_
    add
    (VOM_LICENSE_REQUIRED char(1)                    null
    );
end if;

if not exists (select 1 from sys.syscolumns where tname='HOST' and cname='VOM_LICENSE_REQUIRED') then
    alter table SAHARA.HOST
    add
    (VOM_LICENSE_REQUIRED char(1)                    null
    );
end if;




if not exists (select 1 from sys.syscolumns where tname='HOST_' and cname='VM_HOST_OID') then
    alter table SAHARA_APP.HOST_
    add
    (VM_HOST_OID varchar(512) not null default ''
    );
end if;

if not exists (select 1 from sys.syscolumns where tname='HOST' and cname='VM_HOST_OID') then
    alter table SAHARA.HOST
    add
    (VM_HOST_OID varchar(512) not null default ''
    );
end if;



comment on column SAHARA.HOST.VOM_LICENSE_REQUIRED is
'Specifies whether a customer requires a VOM Advanced license to manage this host. A host needs a VOM Advanced license if, for example, has no SF products on it, or has SF non-Enterprise license etc.';

comment on column SAHARA_APP.HOST_.VOM_LICENSE_REQUIRED is
'Specifies whether a customer requires a VOM Advanced license to manage this host. A host needs a VOM Advanced license if, for example, has no SF products on it, or has SF non-Enterprise license etc.';

comment on column SAHARA_APP.IE_APP_GROUP_.IP is
'IP address - will always be null for app_groups. We include this 
here to make the correlation procedure more readable.';

comment on column SAHARA_APP.IE_APP_GROUP_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_ARRAY_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_FILESYSTEM_.FILESYSTEM_NAME is
'The fully qualified path for this filesystem. This combined with the filer
name forms a unique identifier for the filesystem.';

comment on column SAHARA_APP.IE_FILESYSTEM_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_HOST_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.NAS_FILESYSTEM_NAME is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the filesystem.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.PATH is
'The fully qualified path for this qtree. This combined with the filer
name forms a unique identifier for the qtree.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.PATH is
'The fully qualified path for this share. This combined with the filer
name forms a unique identifier for the share.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.PATH is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the volume.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_NODE_.DEPTH is
'The depth of this node in the hierarchy. The root of the node is at depth 0,
 the next depth is 1, etc.';

comment on column SAHARA_APP.IE_NODE_.PARENT_ID is
'Identifier of this node parent generated by the import script to uniquely identify
the parent of this node. If the node is at the top of the hierarchy this column
will be null.';

comment on column SAHARA_APP.IE_NODE_.BVIEW_NODE_KEY is
'The key assigned by the SAHARA.BVIEW_API.AddNode procedure. This is the id the node
is identified by in ER.';

comment on column SAHARA_APP.IE_SWITCH_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA_APP.IE_TYPES_.TYPE_NAME is
'Each row in this column represents a type supported by this business view.
Supported types are defined in the SAHARA.BVIEW_DIMENSION table.';

comment on column SAHARA_APP.IE_VOLUME_.VOLUME_NAME is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the volume.';

comment on column SAHARA_APP.IE_VOLUME_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

comment on column SAHARA.LUN.TYPE is
'This attribute differentiate between VMDK and Disk.';

comment on column SAHARA.LUN.SUB_TYPE is
'For VMDK, this differentiate between RDM, Sparse and Flat VMDKs.';

comment on column SAHARA.LUN.STORAGE_TECHNOLOGY is
'This attribute represents the storage technology used. Possible values are SAN, DAS and NAS.';

comment on column SAHARA.LUN.STORAGE_POOL_NAME is
'This attribute represents name of the data store associated to a VMware VMDK';

comment on column SAHARA.LUN.CONNECTION_PROTOCOL is
'This attribute represents networking protocol used for accessing storage';

comment on column SAHARA_APP.LUN_.TYPE is
'This column is for staging data that will end up in table SAHARA.LUN, column TYPE

This attribute differentiate between VMDK and Disk.';

comment on column SAHARA_APP.LUN_.SUB_TYPE is
'This column is for staging data that will end up in table SAHARA.LUN, column SUBTYPE.

For VMDK, this differentiate between RDM, Sparse and Flat VMDKs.';

comment on column SAHARA_APP.LUN_.STORAGE_TECHNOLOGY is
'This column is for staging data that will end up in table SAHARA.LUN, column STORAGE_TECHNOLOGY.

This attribute represents the storage technology used. Possible values are SAN, DAS and NAS.';

comment on column SAHARA_APP.LUN_.STORAGE_POOL_NAME is
'This column is for staging data that will end up in table SAHARA.LUN, column STORAGE_POOL_NAME.

This attribute represents name of the data store associated to a VMware VMDK.';

comment on column SAHARA_APP.LUN_.CONNECTION_PROTOCOL is
'This column is for staging data that will end up in table SAHARA.LUN, column CONNECTION_PROTOCOL.

This attribute represents networking protocol used for accessing storage.';



commit;
/*   ---  */


if not exists (select 1 from sys.syscolumns where tname='ROLLUP_BATCH' and cname='IS_LAST_OF_WEEK') then

  ALTER TABLE SAHARA.ROLLUP_BATCH ADD IS_LAST_OF_WEEK char(1) NULL default 'F';

  comment on column SAHARA.ROLLUP_BATCH.IS_LAST_OF_WEEK is
  'A boolean flag that indicates if this batch is the latest complete and successful batch of the week for the associated roll-up source.';

  alter table SAHARA.ROLLUP_BATCH
     alter IS_LAST_OF_WEEK add constraint C_ROLLUPBATCH_ISLASTOFWEEK check (IS_LAST_OF_WEEK is null or (IS_LAST_OF_WEEK in ('T','F')));

  /*==============================================================*/
  /* Index: I_ROLLUPBATCH_ISLASTOFYEAR                            */
  /*==============================================================*/
  create index I_ROLLUPBATCH_ISLASTOFWEEK on SAHARA.ROLLUP_BATCH (
  IS_LAST_OF_WEEK ASC
  );

end if;


if not exists (select 1 from sys.syscolumns where tname='ROLLUP_BATCH' and cname='PERIOD_TYPE') then
  ALTER TABLE SAHARA.ROLLUP_BATCH ADD PERIOD_TYPE varchar(16) NULL default 'day';
end if;

if not exists (select 1 from sys.syscolumns where tname='COLUMN_METADATA_BASE' and cname='AGGREGATE_TYPE') then
  ALTER TABLE SAHARA.COLUMN_METADATA_BASE ADD AGGREGATE_TYPE varchar(16) NULL default 'summarization';
  call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'AGGREGATE_TYPE', 'aggregate function type');
end if;


/*==============================================================*/
/* Table: BVIEW_RULES                                           */
/*==============================================================*/
call "SAHARA"."SP_DROP_OBJECT_SAHARA"('TABLE','BVIEW_RULES');
create table SAHARA.BVIEW_RULES 
(
   BVIEW_RULES_KEY      numeric(10)                    not null default autoincrement,
   NAME                 varchar(64)                    not null,
   DESCRIPTION          long varchar                   null,
   OBJECTTYPE           varchar(26)                    not null,
   APPROVALREQUIRED     bit                            null,
   ENABLED              bit                            null,
   BVIEW_KEY            numeric(10)                    not null,
   BVIEW_NODE_KEY       numeric(10)                    not null,
   DIMENSION            varchar(26)                    not null,
   APPROVALSTATE        integer                        null,
   SQLQUERY             long varchar                   null
);

comment on table SAHARA.BVIEW_RULES is 
'This table stores all  attributes for the business view rules.';

comment on column SAHARA.BVIEW_RULES.BVIEW_RULES_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension table per business view node';

comment on column SAHARA.BVIEW_RULES.NAME is 
'The display name for the business view rule.  Each display name must be unique (ignoring character case) within a dimension context.';

comment on column SAHARA.BVIEW_RULES.DESCRIPTION is 
'A user specified description for the business view rule.';

comment on column SAHARA.BVIEW_RULES.OBJECTTYPE is 
'The Object Type on which the rule is defined or the dimension table on which the rule is defined.';

comment on column SAHARA.BVIEW_RULES.BVIEW_KEY is 
'The business view key of the business view with which the rule is associated.
Busines View Key is the node key of the root node for the business view.  This is used as an identifier for the business view as a whole.';

comment on column SAHARA.BVIEW_RULES.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.BVIEW_RULES.DIMENSION is 
'The name for the dimension.  This is the table name that represents the dimension.

This column is limited to 26 characters because the table name is expect to allow room for a primary key of the form table_name + "_KEY".';

alter table SAHARA.BVIEW_RULES
   add constraint PK_BVIEW_RULES primary key (BVIEW_RULES_KEY);

alter table SAHARA.BVIEW_RULES
   add constraint AK_C_BV_RULE_BVIEW_RU unique (BVIEW_KEY, BVIEW_NODE_KEY, DIMENSION);
   
alter table SAHARA.BVIEW_RULES
   add constraint FK_BVIEW_RU_BVIEW_KEY_BVIEW_IM foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.BVIEW_RULES
   add constraint FK_BVIEW_RU_DIMENSION_BVIEW_DI foreign key (DIMENSION)
      references SAHARA.BVIEW_DIMENSION (DIMENSION)
      on update restrict
      on delete restrict;

alter table SAHARA.BVIEW_RULES
   add constraint FK_BVIEW_RU_NODE_KEY_BVIEW_NO foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;
   

grant SELECT on SAHARA.BVIEW_RULES to SAHARA_APP;


/*==============================================================*/
/* Table: BVIEW_APPROVAL_PENDING                                */
/*==============================================================*/
-- Its a new table drop it and recreate it
call "SAHARA"."SP_DROP_OBJECT_SAHARA"('TABLE','BVIEW_APPROVAL_PENDING');

create table SAHARA.BVIEW_APPROVAL_PENDING 
(
   BVIEW_RULES_KEY      numeric(10)                    null,
   BVIEW_APPROVALPENDING_KEY numeric(10)                    not null default autoincrement,
   BUSINESSVIEW_NAME    varchar(64)                    null,
   BUSINESSVIEWNODENAME varchar(64)                    null,
   OBJECTTYPE           varchar(64)                    null,
   APPROVALSTATE        integer                        null
);

comment on table SAHARA.BVIEW_APPROVAL_PENDING is 
'This table contains user defined Conditions for every business view rules';

comment on column SAHARA.BVIEW_APPROVAL_PENDING.BVIEW_RULES_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension table per business view node';

comment on column SAHARA.BVIEW_APPROVAL_PENDING.BVIEW_APPROVALPENDING_KEY is 
'The unique database generated identifier for the rule.';

alter table SAHARA.BVIEW_APPROVAL_PENDING
   add constraint PK_BVIEW_APPROVAL_PENDING primary key (BVIEW_APPROVALPENDING_KEY);
   
alter table SAHARA.BVIEW_APPROVAL_PENDING
   add constraint FK_BVIEW_AP_APFK_BVIEW_RU foreign key (BVIEW_RULES_KEY)
      references SAHARA.BVIEW_RULES (BVIEW_RULES_KEY)
      on update restrict
      on delete cascade;   

grant SELECT on SAHARA.BVIEW_APPROVAL_PENDING to SAHARA_APP;

/*==============================================================*/
/* Table: BVIEW_CONDITIONS                                      */
/*==============================================================*/
call "SAHARA"."SP_DROP_OBJECT_SAHARA"('TABLE','BVIEW_CONDITIONS');
create table SAHARA.BVIEW_CONDITIONS 
(
   BVIEW_RULES_KEY      numeric(10)                    null,
   BVIEW_CONDITIONS_KEY numeric(10)                    not null default autoincrement,
   ATTRIBUTE            long varchar                   null,
   OPERATOR             varchar(255)                   null,
   ATTRIBUTEVALUE       long varchar                   null,
   CONJUNCTION          varchar(5)                     null
);

comment on table SAHARA.BVIEW_CONDITIONS is 
'This table contains user defined Conditions for every business view rules';

comment on column SAHARA.BVIEW_CONDITIONS.BVIEW_RULES_KEY is 
'A unique database generated identifier for a business view rule.  The value serves use by providing a simple way to identify a business view rule per dimension table per business view node';

comment on column SAHARA.BVIEW_CONDITIONS.BVIEW_CONDITIONS_KEY is 
'The unique database generated identifier for the rule.';

comment on column SAHARA.BVIEW_CONDITIONS.ATTRIBUTE is 
'The column represent the attribute of the object type used for pattern
matching criteria.';

comment on column SAHARA.BVIEW_CONDITIONS.OPERATOR is 
'Operands at this point would be equal, contains and LIKE. Regex could also be used';

comment on column SAHARA.BVIEW_CONDITIONS.ATTRIBUTEVALUE is 
'The Attribute Value that need to be search should be in this column';

alter table SAHARA.BVIEW_CONDITIONS
   add constraint PK_BVIEW_CONDITIONS primary key (BVIEW_CONDITIONS_KEY);
   
alter table SAHARA.BVIEW_CONDITIONS
   add constraint FK_BVIEW_CO_BUSINESS__BVIEW_RU foreign key (BVIEW_RULES_KEY)
      references SAHARA.BVIEW_RULES (BVIEW_RULES_KEY)
      on update restrict
      on delete cascade;
   

grant SELECT on SAHARA.BVIEW_CONDITIONS to SAHARA_APP;


if not exists(select 1 from sys.systable where table_name = 'CLUSTER' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: CLUSTER                                               */
/*==============================================================*/
create table SAHARA.CLUSTER 
(
   CLUSTER_KEY          numeric(10)                    not null default autoincrement,
   CLUSTER_OID          varchar(128)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   CLUSTER_TYPE         varchar(1024)                  null,
   CLUSTER_CATEGORY     varchar(1024)                  null,
   VERSION              varchar(32)                    null,
   PLATFORM             varchar(1024)                  null,
   USE_FENCE            varchar(1)                     not null default 'F',
   IS_GLOBAL            varchar(1)                     not null default 'F',
   SECURECLUS           varchar(1)                     null default 'F',
   REMOTECLUSTERS       varchar(2048)                  null default '',
   T2_IS_CURRENT        varchar(1)                     not null default 'T',
   T2_IS_DELETED        varchar(1)                     not null default 'F',
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on table SAHARA.CLUSTER is 
'This is the dimension table for cluster.  It defines the attributes of cluster that may be used to qualify cluster queries.';

comment on column SAHARA.CLUSTER.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.CLUSTER.CLUSTER_TYPE is 
'Cluster types: VCS, MSCS, VMWare HA';

comment on column SAHARA.CLUSTER.CLUSTER_CATEGORY is 
'Category includes: Failover, Oracle RAC, CFS, CVM, Sybase CE.';

comment on column SAHARA.CLUSTER.PLATFORM is 
'details about operating system.';

comment on column SAHARA.CLUSTER.USE_FENCE is 
'Indicates whether the cluster uses SCSI-3 I/O fencing.';

comment on column SAHARA.CLUSTER.IS_GLOBAL is 
'cluster is a global cluster or local cluster';

comment on column SAHARA.CLUSTER.T2_IS_CURRENT is 
'Is a boolean book keeping flag that indicates if this is the "current" or latest record for the object.';

comment on column SAHARA.CLUSTER.T2_IS_DELETED is 
'a boolean book keeping flag that indicates if the dimension row should be considered deleted.';

comment on column SAHARA.CLUSTER.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA.CLUSTER.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA.CLUSTER.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA.CLUSTER.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA.CLUSTER
   alter USE_FENCE add constraint CKC_USE_FENCE_CLUSTER check (USE_FENCE in ('T','F'));

alter table SAHARA.CLUSTER
   alter IS_GLOBAL add constraint CKC_IS_GLOBAL_CLUSTER check (IS_GLOBAL in ('T','F'));

alter table SAHARA.CLUSTER
   alter SECURECLUS add constraint CKC_SECURECLUS_CLUSTER check (SECURECLUS is null or (SECURECLUS in ('T','F')));

alter table SAHARA.CLUSTER
   alter T2_IS_CURRENT add constraint CKC_T2_IS_CURRENT_CLUSTER check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.CLUSTER
   alter T2_IS_DELETED add constraint CKC_T2_IS_DELETED_CLUSTER check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.CLUSTER
   add constraint PK_CLUSTER primary key (CLUSTER_KEY);

/*==============================================================*/
/* Index: I_CLUSTER_IDENTIFIER                                  */
/*==============================================================*/
create index I_CLUSTER_IDENTIFIER on SAHARA.CLUSTER (
CLUSTER_OID ASC
);

comment on index SAHARA.CLUSTER.I_CLUSTER_IDENTIFIER is 
'An index on cluster identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_CLUSTER_DISPLAY_NAME                                */
/*==============================================================*/
create index I_CLUSTER_DISPLAY_NAME on SAHARA.CLUSTER (
DISPLAY_NAME ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_TYPE                                        */
/*==============================================================*/
create index I_CLUSTER_TYPE on SAHARA.CLUSTER (
CLUSTER_TYPE ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_CATEGORY                                    */
/*==============================================================*/
create index I_CLUSTER_CATEGORY on SAHARA.CLUSTER (
CLUSTER_CATEGORY ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_VERSION                                     */
/*==============================================================*/
create index I_CLUSTER_VERSION on SAHARA.CLUSTER (
VERSION ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_PLATFORM                                    */
/*==============================================================*/
create index I_CLUSTER_PLATFORM on SAHARA.CLUSTER (
PLATFORM ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_USE_FENCE                                   */
/*==============================================================*/
create index I_CLUSTER_USE_FENCE on SAHARA.CLUSTER (
USE_FENCE ASC
);

/*==============================================================*/
/* Index: I_IS_GLOBAL_INDEX                                     */
/*==============================================================*/
create index I_IS_GLOBAL_INDEX on SAHARA.CLUSTER (
IS_GLOBAL ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_SECURECLUS                                  */
/*==============================================================*/
create index I_CLUSTER_SECURECLUS on SAHARA.CLUSTER (
USE_FENCE ASC
);

/*==============================================================*/
/* Index: I_REMOTECLUSTERS_INDEX                                */
/*==============================================================*/
create index I_REMOTECLUSTERS_INDEX on SAHARA.CLUSTER (
REMOTECLUSTERS ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_T2ISCUR_A_CKEY                              */
/*==============================================================*/
create index I_CLUSTER_T2ISCUR_A_CKEY on SAHARA.CLUSTER (
CLUSTER_KEY ASC,
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_T2ISCURAT2ISDELACID                         */
/*==============================================================*/
create index I_CLUSTER_T2ISCURAT2ISDELACID on SAHARA.CLUSTER (
CLUSTER_OID ASC,
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_T2ISDELETED                                 */
/*==============================================================*/
create index I_CLUSTER_T2ISDELETED on SAHARA.CLUSTER (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.CLUSTER to SAHARA_APP;

end if;   



if not exists(select 1 from sys.systable where table_name = 'CLUSTER_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: CLUSTER_                                              */
/*==============================================================*/
create table SAHARA_APP.CLUSTER_ 
(
   CLUSTER_OID          varchar(128)                   not null,
   DISPLAY_NAME         varchar(1024)                  null,
   CLUSTER_TYPE         varchar(1024)                  null,
   CLUSTER_CATEGORY     varchar(1024)                  null,
   VERSION              varchar(32)                    null,
   PLATFORM             varchar(1024)                  null,
   USE_FENCE            varchar(1)                     not null default 'F',
   IS_GLOBAL            varchar(1)                     null default 'F',
   SECURECLUS           varchar(1)                     null default 'F',
   REMOTECLUSTERS       varchar(2048)                  not null default '',
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on table SAHARA_APP.CLUSTER_ is 
'This is a staging table for loading data into warehouse table SAHARA.CLUSTER.  Any data in this table is temporary and should not be accessed except by the data loading process';

comment on column SAHARA_APP.CLUSTER_.CLUSTER_TYPE is 
'Cluster types: VCS, MSCS, VMWare HA';

comment on column SAHARA_APP.CLUSTER_.CLUSTER_CATEGORY is 
'Category includes: Failover, Oracle RAC, CFS, CVM, Sybase CE.';

comment on column SAHARA_APP.CLUSTER_.PLATFORM is 
'details about operating system.';

comment on column SAHARA_APP.CLUSTER_.USE_FENCE is 
'Indicates whether the cluster uses SCSI-3 I/O fencing.';

comment on column SAHARA_APP.CLUSTER_.IS_GLOBAL is 
'Cluster is a global or local cluster';

comment on column SAHARA_APP.CLUSTER_.REMOTECLUSTERS is 
'List of remote clusters names.';

comment on column SAHARA_APP.CLUSTER_.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA_APP.CLUSTER_.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA_APP.CLUSTER_.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA_APP.CLUSTER_.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA_APP.CLUSTER_
   alter USE_FENCE add constraint CKC_USE_FENCE_CLUSTER_ check (USE_FENCE in ('T','F'));

alter table SAHARA_APP.CLUSTER_
   alter IS_GLOBAL add constraint CKC_IS_GLOBAL_CLUSTER_ check (IS_GLOBAL is null or (IS_GLOBAL in ('T','F')));

alter table SAHARA_APP.CLUSTER_
   alter SECURECLUS add constraint CKC_SECURECLUS_CLUSTER_ check (SECURECLUS is null or (SECURECLUS in ('T','F')));

alter table SAHARA_APP.CLUSTER_
   add constraint PK_CLUSTER_ primary key (CLUSTER_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_ to SAHARA;


end if;


/*==============================================================*/
/* Table: CLUSTER_BVIEW_NODE                                    */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'CLUSTER_BVIEW_NODE' and creator=user_id('SAHARA')) then

create table SAHARA.CLUSTER_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   CLUSTER_KEY          numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.CLUSTER_BVIEW_NODE is 
'This is a join table between cluster objects and business view nodes.';

comment on column SAHARA.CLUSTER_BVIEW_NODE.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.CLUSTER_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint PK_CLUSTER_BVIEWNODE primary key (BVIEW_NODE_KEY, CLUSTER_KEY);
   
alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint FK_CLUSTER__REFERENCE_BVIEW_NO foreign key (BVIEW_NODE_KEY)
      references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint FK_CLUSTER__REFERENCE_BVIEW_IM foreign key (BVIEW_KEY)
      references SAHARA.BVIEW_IMPL (BVIEW_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.CLUSTER_BVIEW_NODE
   add constraint FK_CLUSTER__REFERENCE_CLUSTER foreign key (CLUSTER_KEY)
      references SAHARA.CLUSTER (CLUSTER_KEY)
      on update restrict
      on delete cascade;   

/*==============================================================*/
/* Index: I_CLUSTER_BVIEWNODE_BVIEWKEY                          */
/*==============================================================*/
create index I_CLUSTER_BVIEWNODE_BVIEWKEY on SAHARA.CLUSTER_BVIEW_NODE (
BVIEW_NODE_KEY ASC,
CLUSTER_KEY ASC,
BVIEW_KEY ASC
);

comment on index SAHARA.CLUSTER_BVIEW_NODE.I_CLUSTER_BVIEWNODE_BVIEWKEY is 
'An index on Business View Key for dimensional queries.';

grant SELECT on SAHARA.CLUSTER_BVIEW_NODE to SAHARA_APP;

end if;


/*==============================================================*/
/* Table: CLUSTER_HOST                                          */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'CLUSTER_HOST' and creator=user_id('SAHARA')) then

create table SAHARA.CLUSTER_HOST 
(
   CLUSTER_HOST_KEY     numeric(10)                    not null default autoincrement,
   CLUSTER_KEY          numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   CLUSTER_HOST_OID     varchar(1024)                  not null,
   CLUSTER_OID          varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.CLUSTER_HOST is 
'A join table for forming one to many relationship between cluster and hosts.';

comment on column SAHARA.CLUSTER_HOST.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.CLUSTER_HOST.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.CLUSTER_HOST.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.CLUSTER_HOST
   alter T2_IS_CURRENT add constraint C_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.CLUSTER_HOST
   alter T2_IS_DELETED add constraint C_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.CLUSTER_HOST
   add constraint PK_CLUSTER_HOST primary key clustered (CLUSTER_HOST_KEY);
   
alter table SAHARA.CLUSTER_HOST
   add constraint FK_CLUSTERHOST_CLUSTER foreign key (CLUSTER_KEY)
       references SAHARA.CLUSTER (CLUSTER_KEY)
       on update restrict
       on delete cascade;  

alter table SAHARA.CLUSTER_HOST
   add constraint FK_CLUSTERHOST_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;       

/*==============================================================*/
/* Index: I_CLUSTERHOST_CLUSTERKEY                              */
/*==============================================================*/
create index I_CLUSTERHOST_CLUSTERKEY on SAHARA.CLUSTER_HOST (
CLUSTER_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERHOST_HOSTKEY                                 */
/*==============================================================*/
create index I_CLUSTERHOST_HOSTKEY on SAHARA.CLUSTER_HOST (
HOST_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERHOST_T2ISCURRENT                             */
/*==============================================================*/
create index I_CLUSTERHOST_T2ISCURRENT on SAHARA.CLUSTER_HOST (
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_CLUSTER_HOST_T2ISDELETED                            */
/*==============================================================*/
create index I_CLUSTER_HOST_T2ISDELETED on SAHARA.CLUSTER_HOST (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.CLUSTER_HOST to SAHARA_APP;

end if;


/*==============================================================*/
/* Table: CLUSTER_HOST_                                         */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'CLUSTER_HOST_' and creator=user_id('SAHARA_APP')) then

create table SAHARA_APP.CLUSTER_HOST_ 
(
   CLUSTER_HOST_OID     varchar(1024)                  not null,
   CLUSTER_OID          varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   DISPLAY_NAME         varchar(1024)                  null
);

alter table SAHARA_APP.CLUSTER_HOST_
   add constraint PK_CLUSTERHOST_ primary key clustered (CLUSTER_HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_HOST_ to SAHARA;

end if;


if not exists(select 1 from sys.systable where table_name = 'CLUSTER_INVENTORY' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: CLUSTER_INVENTORY                                     */
/*==============================================================*/
create table SAHARA.CLUSTER_INVENTORY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   CLUSTER_KEY          numeric(10)                    not null,
   NUMBER_OF_NODE       integer                        null default 0,
   NUMBER_OF_SERVICEGROUP integer                        null default 0,
   NUMBER_OF_RESOURCE   integer                        null default 0
);

comment on table SAHARA.CLUSTER_INVENTORY is 
'Contains number of nodes, servicegroups,resources, and allocated capacity';

comment on column SAHARA.CLUSTER_INVENTORY.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.CLUSTER_INVENTORY.DATES_KEY is 
'A unique database generated identifier for the date record used as a surrogate key.';

alter table SAHARA.CLUSTER_INVENTORY
   alter DATES_KEY add constraint CKC_DATES_KEY_CLUSTER_ check (DATES_KEY >= 1);

alter table SAHARA.CLUSTER_INVENTORY
   alter NUMBER_OF_NODE add constraint CKC_NUMBER_OF_NODE_CLUSTER_ check (NUMBER_OF_NODE is null or (NUMBER_OF_NODE >= 0));

alter table SAHARA.CLUSTER_INVENTORY
   alter NUMBER_OF_SERVICEGROUP add constraint CKC_NUMBER_OF_SERVICE_CLUSTER_ check (NUMBER_OF_SERVICEGROUP is null or (NUMBER_OF_SERVICEGROUP >= 0));

alter table SAHARA.CLUSTER_INVENTORY
   alter NUMBER_OF_RESOURCE add constraint CKC_NUMBER_OF_RESOURC_CLUSTER_ check (NUMBER_OF_RESOURCE is null or (NUMBER_OF_RESOURCE >= 0));
   
alter table SAHARA.CLUSTER_INVENTORY
   add constraint FK_CLUSTERINV_CLUSTER foreign key (CLUSTER_KEY)
       references SAHARA.CLUSTER (CLUSTER_KEY)
       on update restrict
       on delete cascade;

alter table SAHARA.CLUSTER_INVENTORY
   add constraint FK_CLUSTERINV_DATES foreign key (DATES_KEY)
       references SAHARA.DATES (DATES_KEY)
       on update restrict
       on delete cascade;

alter table SAHARA.CLUSTER_INVENTORY
   add constraint FK_CLUSTERINV_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
       references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
       on update restrict
       on delete cascade;   

/*==============================================================*/
/* Index: I_CLUSTERINV_ROLLUPBATCH                              */
/*==============================================================*/
create index I_CLUSTERINV_ROLLUPBATCH on SAHARA.CLUSTER_INVENTORY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERINV_DATESKEY                                 */
/*==============================================================*/
create index I_CLUSTERINV_DATESKEY on SAHARA.CLUSTER_INVENTORY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERINV_CLUSTERKEY                               */
/*==============================================================*/
create index I_CLUSTERINV_CLUSTERKEY on SAHARA.CLUSTER_INVENTORY (
CLUSTER_KEY ASC
);

grant SELECT on SAHARA.CLUSTER_INVENTORY to SAHARA_APP;


end if;

/*==============================================================*/
/* Table: CLUSTER_INVENTORY_                                    */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'CLUSTER_INVENTORY_' and creator=user_id('SAHARA_APP')) then    

/*==============================================================*/
/* Table: CLUSTER_INVENTORY_                                    */
/*==============================================================*/
create table SAHARA_APP.CLUSTER_INVENTORY_ 
(
   CLUSTER_OID          varchar(128)                   not null,
   NUMBER_OF_NODE       integer                        not null default 0,
   NUMBER_OF_SERVICEGROUP integer                        not null default 0,
   NUMBER_OF_RESOURCE   integer                        not null default 0
);

comment on table SAHARA_APP.CLUSTER_INVENTORY_ is 
'This is a staging table for loading data into warehouse table sahara.cluster_inventory.  Any data in this table is temporary and should not be accessed except by the data loading process.';

alter table SAHARA_APP.CLUSTER_INVENTORY_
   alter NUMBER_OF_NODE add constraint CKC_NUMBER_OF_NODE_CLUSTER_ check (NUMBER_OF_NODE >= 0);

alter table SAHARA_APP.CLUSTER_INVENTORY_
   alter NUMBER_OF_SERVICEGROUP add constraint CKC_NUMBER_OF_SERVICE_CLUSTER_ check (NUMBER_OF_SERVICEGROUP >= 0);

alter table SAHARA_APP.CLUSTER_INVENTORY_
   alter NUMBER_OF_RESOURCE add constraint CKC_NUMBER_OF_RESOURC_CLUSTER_ check (NUMBER_OF_RESOURCE >= 0);

alter table SAHARA_APP.CLUSTER_INVENTORY_
   add constraint PK_CLUSTERINV_CLUSTERID_ primary key (CLUSTER_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_INVENTORY_ to SAHARA;



end if;


/*==============================================================*/
/* Table: CLUSTER_REMOTECLUSTER                                 */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'CLUSTER_REMOTECLUSTER' and creator=user_id('SAHARA')) then

create table SAHARA.CLUSTER_REMOTECLUSTER 
(
   CLUSTER_REMOTECLUSTER_KEY numeric(10)                    not null default autoincrement,
   CLUSTER_KEY          numeric(10)                    not null,
   CLUSTER_REMOTECLUSTER_OID varchar(512)                   not null,
   CLUSTER_OID          varchar(128)                   not null,
   REMOTECLUSTER_OID    varchar(128)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.CLUSTER_REMOTECLUSTER is 
'A join table for forming one to many relationship between local cluster and remote clusters. We treat this as a fact to deal with relationship comfing and going with each batch.';

comment on column SAHARA.CLUSTER_REMOTECLUSTER.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.CLUSTER_REMOTECLUSTER.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.CLUSTER_REMOTECLUSTER.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.CLUSTER_REMOTECLUSTER
   alter T2_IS_CURRENT add constraint C_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.CLUSTER_REMOTECLUSTER
   alter T2_IS_DELETED add constraint C_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.CLUSTER_REMOTECLUSTER
   add constraint PK_CLUSTER_REMOTECLUSTER primary key clustered (CLUSTER_REMOTECLUSTER_KEY);
   
alter table SAHARA.CLUSTER_REMOTECLUSTER
   add constraint FK_CLUSTERRCLUSTER_CLUSTER foreign key (CLUSTER_KEY)
       references SAHARA.CLUSTER (CLUSTER_KEY)
       on update restrict
       on delete cascade;   

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_CLUSTERKEY                          */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_CLUSTERKEY on SAHARA.CLUSTER_REMOTECLUSTER (
CLUSTER_KEY ASC
);

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_RCLUSTERID                          */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_RCLUSTERID on SAHARA.CLUSTER_REMOTECLUSTER (
REMOTECLUSTER_OID ASC
);

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_T2ISCURRENT                         */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_T2ISCURRENT on SAHARA.CLUSTER_REMOTECLUSTER (
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_CLUSTERRCLUSTER_T2ISDELETED                         */
/*==============================================================*/
create index I_CLUSTERRCLUSTER_T2ISDELETED on SAHARA.CLUSTER_REMOTECLUSTER (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.CLUSTER_REMOTECLUSTER to SAHARA_APP;

end if;


/*==============================================================*/
/* Table: CLUSTER_REMOTECLUSTER_                                */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'CLUSTER_REMOTECLUSTER_' and creator=user_id('SAHARA_APP')) then

create table SAHARA_APP.CLUSTER_REMOTECLUSTER_ 
(
   CLUSTER_REMOTECLUSTER_OID varchar(512)                   not null,
   CLUSTER_OID          varchar(128)                   not null,
   REMOTECLUSTER_OID    varchar(128)                   not null,
   DISPLAY_NAME         varchar(1024)                  null
);

comment on table SAHARA_APP.CLUSTER_REMOTECLUSTER_ is 
'This is a staging table for loading data into warehouse table sahara.cluster_remotecluster.  Any data in this table is temporary and should be accessed except by the data loading process.';

alter table SAHARA_APP.CLUSTER_REMOTECLUSTER_
   add constraint PK_CLUSTERRCLUSTER_ primary key clustered (CLUSTER_REMOTECLUSTER_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.CLUSTER_REMOTECLUSTER_ to SAHARA;

end if;


if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: SERVICEGROUP                                          */
/*==============================================================*/

create table SAHARA.SERVICEGROUP 
(
   SERVICEGROUP_KEY     numeric(10)                    not null default autoincrement,
   SERVICEGROUP_OID     varchar(128)                   null,
   CLUSTER_KEY          numeric(10)                    null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   SG_TYPE              varchar(1024)                  null,
   SCOPE                varchar(16)                    null,
   FAILOVERPOLICY       varchar(128)                   null,
   IS_APPHA             varchar(1)                     null default 'F',
   FAMILY               varchar(16)                    null,
   T2_IS_CURRENT        varchar(1)                     not null default 'T',
   T2_IS_DELETED        varchar(1)                     not null default 'F',
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on column SAHARA.SERVICEGROUP.IS_APPHA is 
'Indicates whether a servicegroup belongs to AppHA instance.';

comment on column SAHARA.SERVICEGROUP.T2_IS_CURRENT is 
'Is a boolean book keeping flag that indicates if this is the "current" or latest record for the object.';

comment on column SAHARA.SERVICEGROUP.T2_IS_DELETED is 
'a boolean book keeping flag that indicates if the dimension row should be considered deleted.';

comment on column SAHARA.SERVICEGROUP.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA.SERVICEGROUP.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA.SERVICEGROUP.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA.SERVICEGROUP.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA.SERVICEGROUP
   alter IS_APPHA add constraint CKC_IS_APPHA_SERVICEG check (IS_APPHA is null or (IS_APPHA in ('T','F')));

alter table SAHARA.SERVICEGROUP
   alter T2_IS_CURRENT add constraint CKC_T2_IS_CURRENT_SERVICEG check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SERVICEGROUP
   alter T2_IS_DELETED add constraint CKC_T2_IS_DELETED_SERVICEG check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SERVICEGROUP
   add constraint PK_SERVICEGROUP primary key (SERVICEGROUP_KEY);
   
alter table SAHARA.SERVICEGROUP
   add constraint FK_SG_CLUSTER foreign key (CLUSTER_KEY)
       references SAHARA.CLUSTER (CLUSTER_KEY)
       on update restrict
       on delete cascade;   
 
/*==============================================================*/
/* Index: I_SG_SGOID                                            */
/*==============================================================*/
create index I_SG_SGOID on SAHARA.SERVICEGROUP (
SERVICEGROUP_OID ASC
);

/*==============================================================*/
/* Index: I_SG_DISPLAYNAME                                      */
/*==============================================================*/
create index I_SG_DISPLAYNAME on SAHARA.SERVICEGROUP (
DISPLAY_NAME ASC
);

/*==============================================================*/
/* Index: I_SG_SGTYPE                                           */
/*==============================================================*/
create index I_SG_SGTYPE on SAHARA.SERVICEGROUP (
SG_TYPE ASC
);

/*==============================================================*/
/* Index: I_SG_SGSCOPE                                          */
/*==============================================================*/
create index I_SG_SGSCOPE on SAHARA.SERVICEGROUP (
SCOPE ASC
);

/*==============================================================*/
/* Index: I_SG_SGFAILOVERPOLICY                                 */
/*==============================================================*/
create index I_SG_SGFAILOVERPOLICY on SAHARA.SERVICEGROUP (
FAILOVERPOLICY ASC
);

/*==============================================================*/
/* Index: I_SG_T2ISCURSGKEY                                     */
/*==============================================================*/
create index I_SG_T2ISCURSGKEY on SAHARA.SERVICEGROUP (
SERVICEGROUP_KEY ASC,
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_SG_T2ISCUR_T2ISDEL_SGID                             */
/*==============================================================*/
create index I_SG_T2ISCUR_T2ISDEL_SGID on SAHARA.SERVICEGROUP (
SERVICEGROUP_OID ASC,
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC
);

/*==============================================================*/
/* Index: I_SG_T2ISDEL                                          */
/*==============================================================*/
create index I_SG_T2ISDEL on SAHARA.SERVICEGROUP (
T2_IS_DELETED ASC
);

/*==============================================================*/
/* Index: I_IS_APPHA                                            */
/*==============================================================*/
create index I_IS_APPHA on SAHARA.SERVICEGROUP (
IS_APPHA ASC
);

comment on index SAHARA.SERVICEGROUP.I_IS_APPHA is 
'Indicates whether a servicegroup belongs to AppHA instance.';

/*==============================================================*/
/* Index: SG_FAMILY                                             */
/*==============================================================*/
create index SG_FAMILY on SAHARA.SERVICEGROUP (
FAMILY ASC
);

grant SELECT on SAHARA.SERVICEGROUP to SAHARA_APP;


end if;


if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: SERVICEGROUP_                                         */
/*==============================================================*/
create table SAHARA_APP.SERVICEGROUP_ 
(
   SERVICEGROUP_OID     varchar(128)                   not null,
   CLUSTER_OID          varchar(128)                   null,
   DISPLAY_NAME         varchar(1024)                  null,
   SG_TYPE              varchar(1024)                  null,
   SCOPE                varchar(16)                    null,
   FAILOVERPOLICY       varchar(128)                   null,
   IS_APPHA             varchar(1)                     null default 'F',
   FAMILY               varchar(16)                    null,
   APPLICATION_BE       varchar(512)                   null,
   APPLICATION_COMPOSITE_BE varchar(512)                   null,
   ORGANIZATION_BE      varchar(512)                   null,
   ORGANIZATION_COMPOSITE_BE varchar(512)                   null
);

comment on table SAHARA_APP.SERVICEGROUP_ is 
'This is a staging table for loading data into warehouse table sahara.servicegroup.  Any data in this table is temporary and should not be accessed except by the data loading process.';

comment on column SAHARA_APP.SERVICEGROUP_.IS_APPHA is 
'Indicate whether servicegroup belongs to an AppHA instance';

comment on column SAHARA_APP.SERVICEGROUP_.APPLICATION_BE is 
'Application Business Entity to which this object belongs.';

comment on column SAHARA_APP.SERVICEGROUP_.APPLICATION_COMPOSITE_BE is 
'Composite Application Business Entity for which this object belongs';

comment on column SAHARA_APP.SERVICEGROUP_.ORGANIZATION_BE is 
'Organization Business entity for which this object belongs to.';

comment on column SAHARA_APP.SERVICEGROUP_.ORGANIZATION_COMPOSITE_BE is 
'Composite Organization Business entity for which this objext belongs.';

alter table SAHARA_APP.SERVICEGROUP_
   alter IS_APPHA add constraint CKC_IS_APPHA_SERVICEG check (IS_APPHA is null or (IS_APPHA in ('T','F')));

alter table SAHARA_APP.SERVICEGROUP_
   add constraint PK_SERVICEGROUP_ primary key (SERVICEGROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SERVICEGROUP_ to SAHARA;

end if;


if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_BVIEW_NODE' and creator=user_id('SAHARA')) then
/*==============================================================*/
/* Table: SERVICEGROUP_BVIEW_NODE                               */
/*==============================================================*/
create table SAHARA.SERVICEGROUP_BVIEW_NODE 
(
   BVIEW_NODE_KEY       numeric(10)                    not null,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   BVIEW_KEY            numeric(10)                    not null,
   STATUS               integer                        null default 0
);

comment on table SAHARA.SERVICEGROUP_BVIEW_NODE is 
'This is a join between servicegroup objects and business view nodes';

comment on column SAHARA.SERVICEGROUP_BVIEW_NODE.BVIEW_NODE_KEY is 
'A unique database generated key for the business view node.  This key is used as a surrogate key in joins.';

comment on column SAHARA.SERVICEGROUP_BVIEW_NODE.BVIEW_KEY is 
'The business view key.  This is the same as the node key for the root node of the business view.  It uniquely identifies the business view.';

alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint PK_SG_BVIEWNODE primary key (BVIEW_NODE_KEY, SERVICEGROUP_KEY);
   
alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint FK_SERVICEG_REFERENCE_BVIEW_NO foreign key (BVIEW_NODE_KEY)
       references SAHARA.BVIEW_NODE (BVIEW_NODE_KEY)
       on update restrict
       on delete cascade;

alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint FK_SERVICEG_REFERENCE_BVIEW_IM foreign key (BVIEW_KEY)
       references SAHARA.BVIEW_IMPL (BVIEW_KEY)
       on update restrict
       on delete cascade;

alter table SAHARA.SERVICEGROUP_BVIEW_NODE
   add constraint FK_SERVICEG_REFERENCE_SERVICEG foreign key (SERVICEGROUP_KEY)
       references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
       on update restrict
       on delete cascade;   

/*==============================================================*/
/* Index: I_SG_BVIEWNODE_BVIEWKEY                               */
/*==============================================================*/
create index I_SG_BVIEWNODE_BVIEWKEY on SAHARA.SERVICEGROUP_BVIEW_NODE (
BVIEW_NODE_KEY ASC,
SERVICEGROUP_KEY ASC,
BVIEW_KEY ASC
);

comment on index SAHARA.SERVICEGROUP_BVIEW_NODE.I_SG_BVIEWNODE_BVIEWKEY is 
'An index on Business View Key for dimensional queries.';

grant SELECT on SAHARA.SERVICEGROUP_BVIEW_NODE to SAHARA_APP;

end if;

if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_HOST' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: SERVICEGROUP_HOST                                     */
/*==============================================================*/
create table SAHARA.SERVICEGROUP_HOST 
(
   SERVICEGROUP_HOST_KEY numeric(10)                    not null default autoincrement,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   SERVICEGROUP_HOST_OID varchar(1024)                  not null,
   SERVICEGROUP_OID     varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  null,
   DISPLAY_NAME         varchar(1024)                  null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on column SAHARA.SERVICEGROUP_HOST.CC_STORAGE_URL is 
'The URL for the cluster object overview view in the VOM GUI';

comment on column SAHARA.SERVICEGROUP_HOST.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.SERVICEGROUP_HOST.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.SERVICEGROUP_HOST
   alter T2_IS_CURRENT add constraint C_HOST_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.SERVICEGROUP_HOST
   alter T2_IS_DELETED add constraint C_HOST_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.SERVICEGROUP_HOST
   add constraint PK_SERVICEGROUP_HOST primary key clustered (SERVICEGROUP_HOST_KEY);
   
alter table SAHARA.SERVICEGROUP_HOST
   add constraint FK_SGHOST_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HOST
   add constraint FK_SGHOST_SG foreign key (SERVICEGROUP_KEY)
      references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
      on update restrict
      on delete cascade;   

/*==============================================================*/
/* Index: I_SGHOST_SGKEY                                        */
/*==============================================================*/
create index I_SGHOST_SGKEY on SAHARA.SERVICEGROUP_HOST (
SERVICEGROUP_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHOST_HOSTKEY                                      */
/*==============================================================*/
create index I_SGHOST_HOSTKEY on SAHARA.SERVICEGROUP_HOST (
HOST_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHOST_T2ISCURRENT                                  */
/*==============================================================*/
create index I_SGHOST_T2ISCURRENT on SAHARA.SERVICEGROUP_HOST (
T2_IS_CURRENT ASC
);

/*==============================================================*/
/* Index: I_SGHOST_T2ISDELETED                                  */
/*==============================================================*/
create index I_SGHOST_T2ISDELETED on SAHARA.SERVICEGROUP_HOST (
T2_IS_DELETED ASC
);

grant SELECT on SAHARA.SERVICEGROUP_HOST to SAHARA_APP;

end if;


if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_HOST_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: SERVICEGROUP_HOST_                                    */
/*==============================================================*/
create table SAHARA_APP.SERVICEGROUP_HOST_ 
(
   SERVICEGROUP_HOST_OID varchar(1024)                  not null,
   SERVICEGROUP_OID     varchar(128)                   not null,
   HOST_OID             varchar(512)                   not null,
   DISPLAY_NAME         varchar(1024)                  null
);

alter table SAHARA_APP.SERVICEGROUP_HOST_
   add constraint PK_SGHOST_ primary key (SERVICEGROUP_HOST_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SERVICEGROUP_HOST_ to SAHARA;

end if;


if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_INVENTORY' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: SERVICEGROUP_INVENTORY                                */
/*==============================================================*/
create table SAHARA.SERVICEGROUP_INVENTORY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   NUMBER_OF_RESOURCE   integer                        null
);

comment on column SAHARA.SERVICEGROUP_INVENTORY.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.SERVICEGROUP_INVENTORY.DATES_KEY is 
'A unique database generated identifier for the date record used as a surrogate key.';

alter table SAHARA.SERVICEGROUP_INVENTORY
   alter DATES_KEY add constraint CKC_DATES_KEY_SERVICEG check (DATES_KEY >= 1);

alter table SAHARA.SERVICEGROUP_INVENTORY
   add constraint FK_SGINV_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_INVENTORY
   add constraint FK_SGINV_ROLLUPBATCH foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_INVENTORY
   add constraint FK_SGINV_SG foreign key (SERVICEGROUP_KEY)
      references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
      on update restrict
      on delete cascade;   


/*==============================================================*/
/* Index: I_SGINV_ROLLUPBATCHKEY                                */
/*==============================================================*/
create index I_SGINV_ROLLUPBATCHKEY on SAHARA.SERVICEGROUP_INVENTORY (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_SGINV_DATESKEY                                      */
/*==============================================================*/
create index I_SGINV_DATESKEY on SAHARA.SERVICEGROUP_INVENTORY (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_SGINV_SGKEY                                         */
/*==============================================================*/
create index I_SGINV_SGKEY on SAHARA.SERVICEGROUP_INVENTORY (
SERVICEGROUP_KEY ASC
);

grant SELECT on SAHARA.SERVICEGROUP_INVENTORY to SAHARA_APP;

end if;


if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_INVENTORY_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: SERVICEGROUP_INVENTORY_                               */
/*==============================================================*/
create table SAHARA_APP.SERVICEGROUP_INVENTORY_ 
(
   SERVICEGROUP_OID     varchar(128)                   not null,
   NUMBER_OF_RESOURCE   integer                        null
);

alter table SAHARA_APP.SERVICEGROUP_INVENTORY_
   add constraint PK_SERVICEGROUP_INVENTORY_ primary key clustered (SERVICEGROUP_OID);

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.SERVICEGROUP_INVENTORY_ to SAHARA;

end if;

if not exists(select 1 from sys.systable where table_name='CLUSTER_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* View: CLUSTER_CURRENT                                        */
/*==============================================================*/
create view SAHARA.CLUSTER_CURRENT  as
select
   CLUSTER_KEY,
   CLUSTER_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   CLUSTER_TYPE,
   CLUSTER_CATEGORY,
   SEVERITY,
   PLATFORM,
   USE_FENCE,
   IS_GLOBAL,
   REMOTECLUSTERS,
   APPLICATION_BE,
   APPLICATION_COMPOSITE_BE,
   ORGANIZATION_BE,
   ORGANIZATION_COMPOSITE_BE
from
   SAHARA.CLUSTER
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.CLUSTER_CURRENT is 
'This view filters the CLUSTER table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.CLUSTER_CURRENT to SAHARA_APP;

end if;

if not exists(select 1 from sys.systable where table_name='CLUSTER_HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* View: CLUSTER_HOST_CURRENT                                   */
/*==============================================================*/
create view SAHARA.CLUSTER_HOST_CURRENT (CLUSTER_HOST_KEY, CLUSTER_KEY, HOST_KEY, CLUSTER_OID, HOST_OID, CC_STORAGE_URL, DISPLAY_NAME) as
select
   CLUSTER_HOST_KEY,
   CLUSTER_KEY,
   HOST_KEY,
   CLUSTER_OID,
   HOST_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME
from
   SAHARA.CLUSTER_HOST
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

grant SELECT on SAHARA.CLUSTER_HOST_CURRENT to SAHARA_APP;

end if;

if not exists(select 1 from sys.systable where table_name='CLUSTER_RCLUSTER_RELATION' and table_type='VIEW' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* View: CLUSTER_RCLUSTER_RELATION                              */
/*==============================================================*/
create view SAHARA.CLUSTER_RCLUSTER_RELATION  as
select 
    cobj_rcobj.c1_cluster_key as cluster_key, 
    cobj_rcobj.c2_cluster_key as remotecluster_key
from
(
select 
       c1.cluster_key as c1_cluster_key, 
       substr(c1.cluster_oid, 0, charindex('-', c1.cluster_oid)) as c1_cluster_oid, 
       c1.remoteclusters as c1_remotecluster_list ,
       c2.cluster_key as c2_cluster_key, 
       substr(c2.cluster_oid, 0, charindex('-', c2.cluster_oid)) as c2_cluster_oid, 
       c2.remoteclusters as c2_remotecluster_list
from 
    sahara.cluster_remotecluster as c2rc
    join sahara.cluster as c1
    on c2rc.cluster_oid = c1.cluster_oid 
    join sahara.cluster as c2
    on substr(c2rc.remotecluster_oid, 0, charindex('@', c2rc.remotecluster_oid)) = substr(c2.cluster_oid, 0, charindex('-', c2.cluster_oid))
) as cobj_rcobj
where charindex(c1_cluster_oid, c2_remotecluster_list) <> 0
and charindex(c2_cluster_oid, c1_remotecluster_list) <> 0;

comment on view SAHARA.CLUSTER_RCLUSTER_RELATION is 
'This view show valid relationship of cluster to remote cluster only if 
- it finds both cluster and remotecluster objects rolled up in the Cluster Dimension Table.
and
- It successfully cross checking  c1 exists in list of c2''s remotecluster list.
 and c2 exists in the list of c1''s remotecluster lists.

';

grant SELECT on SAHARA.CLUSTER_RCLUSTER_RELATION to SAHARA;

grant SELECT on SAHARA.CLUSTER_RCLUSTER_RELATION to SAHARA_APP;

end if;

if not exists(select 1 from sys.systable where table_name='CLUSTER_REMOTECLUSTER_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* View: CLUSTER_REMOTECLUSTER_CURRENT                          */
/*==============================================================*/
create view SAHARA.CLUSTER_REMOTECLUSTER_CURRENT  as
select
   CLUSTER_REMOTECLUSTER_KEY,
   CLUSTER_KEY,
   CLUSTER_OID,
   REMOTECLUSTER_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME
from
   SAHARA.CLUSTER_REMOTECLUSTER
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

grant SELECT on SAHARA.CLUSTER_REMOTECLUSTER_CURRENT to SAHARA_APP;

end if;

if not exists(select 1 from sys.systable where table_name='SERVICEGROUP_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* View: SERVICEGROUP_CURRENT                                   */
/*==============================================================*/
create view SAHARA.SERVICEGROUP_CURRENT  as
select
   SERVICEGROUP_KEY,
   SERVICEGROUP_OID,
   CLUSTER_KEY,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   SG_TYPE,
   SCOPE,
   FAILOVERPOLICY,
   IS_APPHA,
   FAMILY,
   APPLICATION_BE,
   APPLICATION_COMPOSITE_BE,
   ORGANIZATION_BE,
   ORGANIZATION_COMPOSITE_BE
from
   SAHARA.SERVICEGROUP
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SERVICEGROUP_CURRENT is 
'This view filters the SERVICEGROUP table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.SERVICEGROUP_CURRENT to SAHARA_APP;

end if;

if not exists(select 1 from sys.systable where table_name='SERVICEGROUP_HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* View: SERVICEGROUP_HOST_CURRENT                              */
/*==============================================================*/
create view SAHARA.SERVICEGROUP_HOST_CURRENT  as
select
   SERVICEGROUP_HOST_KEY,
   SERVICEGROUP_KEY,
   HOST_KEY,
   SERVICEGROUP_OID,
   HOST_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME
from
   SAHARA.SERVICEGROUP_HOST
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

grant SELECT on SAHARA.SERVICEGROUP_HOST_CURRENT to SAHARA_APP;

end if;

/******************************* NAS ************************************/

/*==============================================================*/
/* Table: NAS_QUOTA                                             */
/*==============================================================*/
if not exists(select 1 from sys.systable where table_name='NAS_QUOTA' and creator=user_id('SAHARA')) then

create table SAHARA.NAS_QUOTA 
(
   NAS_QUOTA_KEY        numeric(10)                    not null default autoincrement,
   NAS_FILESYSTEM_KEY   numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null,
   NAS_QUOTA_OID        varchar(512)                   not null,
   CC_STORAGE_URL       varchar(1024)                  not null,
   DISPLAY_NAME         varchar(256)                   not null,
   TYPE                 varchar(32)                    not null,
   TARGET               varchar(256)                   not null,
   T2_IS_CURRENT        char(1)                        not null default 'T',
   T2_IS_DELETED        char(1)                        not null default 'F'
);

comment on table SAHARA.NAS_QUOTA is 
'This is the dimension table for NAS Quotas.  It defines the attributes of arrays that may be used to qualify NAS Quota queries.';

comment on column SAHARA.NAS_QUOTA.NAS_QUOTA_KEY is 
'A unique database generated identifier for the array record used as a surrogate key.';

comment on column SAHARA.NAS_QUOTA.NAS_FILESYSTEM_KEY is 
'A unique database generated identifier for the NAS filesystem record used as a surrogate key.';

comment on column SAHARA.NAS_QUOTA.ARRAY_KEY is 
'A unique database generated identifier for the array record used as a surrogate key.';

comment on column SAHARA.NAS_QUOTA.NAS_QUOTA_OID is 
'Unique identifier for this object in VOM database.';

comment on column SAHARA.NAS_QUOTA.CC_STORAGE_URL is 
'The URL for the array object overview view in the VOM GUI.';

comment on column SAHARA.NAS_QUOTA.DISPLAY_NAME is 
'Quota display name.';

comment on column SAHARA.NAS_QUOTA.TYPE is 
'Quota type.  This can be:
User
Group
Tree';

comment on column SAHARA.NAS_QUOTA.TARGET is 
'Quota target';

comment on column SAHARA.NAS_QUOTA.T2_IS_CURRENT is 
'A boolean book keeping flag that indicates if this is the "current" or latest record for the object.  An object on which type 2 attribute changes have been applied may have multiple rows representing different time periods in the object''s lifetime.  Only the most recent row for an object will have this column set to true.';

comment on column SAHARA.NAS_QUOTA.T2_IS_DELETED is 
'A boolean book keeping flag that indicates if the dimension row should be considered deleted.  This flag is maintained using a reference counting schema.  When a roll-up source rolls up values for the row, a reference is add for the row from the source.  When the latest successful batch for a source no longer includes data for the row, the references is removed.  If the roll-up source itself is deleted, the reference is removed.

When the reference count for a row reaches zero sources, the row is marked deleted.';

alter table SAHARA.NAS_QUOTA
   alter T2_IS_CURRENT add constraint C_NASFILESYSTEM_T2ISCURRENT check (T2_IS_CURRENT in ('T','F'));

alter table SAHARA.NAS_QUOTA
   alter T2_IS_DELETED add constraint C_NASFILESYSTEM_T2ISDELETED check (T2_IS_DELETED in ('T','F'));

alter table SAHARA.NAS_QUOTA
   add constraint PK_NAS_QUOTA primary key (NAS_QUOTA_KEY);

/*==============================================================*/
/* Index: I_NASQUOTA_NASFILESYSTEMKEY                           */
/*==============================================================*/
create index I_NASQUOTA_NASFILESYSTEMKEY on SAHARA.NAS_QUOTA (
NAS_FILESYSTEM_KEY ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_NASFILESYSTEMKEY is 
'An index on NAS Filesystem Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_ARRAYKEY                                   */
/*==============================================================*/
create index I_NASQUOTA_ARRAYKEY on SAHARA.NAS_QUOTA (
ARRAY_KEY ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_ARRAYKEY is 
'An index on Array Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_NASQUOTAOID                                */
/*==============================================================*/
create index I_NASQUOTA_NASQUOTAOID on SAHARA.NAS_QUOTA (
NAS_QUOTA_OID ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_NASQUOTAOID is 
'An index on NAS Quota Object Identifier for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_DISPLAYNAME                                */
/*==============================================================*/
create index I_NASQUOTA_DISPLAYNAME on SAHARA.NAS_QUOTA (
DISPLAY_NAME ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_DISPLAYNAME is 
'An index on Display Name for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_TYPE                                       */
/*==============================================================*/
create index I_NASQUOTA_TYPE on SAHARA.NAS_QUOTA (
TYPE ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_TARGET                                     */
/*==============================================================*/
create index I_NASQUOTA_TARGET on SAHARA.NAS_QUOTA (
TARGET ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_TARGET is 
'An index on Target for dimensional queries.';

/*==============================================================*/
/* Index: I_NASQUOTA_T2SCRRNTNSQTK                              */
/*==============================================================*/
create index I_NASQUOTA_T2SCRRNTNSQTK on SAHARA.NAS_QUOTA (
T2_IS_CURRENT ASC,
NAS_QUOTA_KEY ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_T2SCRRNTNSQTK is 
'An index on Type 2 - Is Current and NAS Quota Key for use in processing type 2 updates during data loading.';

/*==============================================================*/
/* Index: I_NASQUOTA_T2SCRRNTT2SDLTDD                           */
/*==============================================================*/
create index I_NASQUOTA_T2SCRRNTT2SDLTDD on SAHARA.NAS_QUOTA (
T2_IS_CURRENT ASC,
T2_IS_DELETED ASC,
NAS_QUOTA_OID ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_T2SCRRNTT2SDLTDD is 
'An index on Type 2 - Is Current, Type 2 - Is Deleted and NAS Quota Object Identifier for dimensional queries, to help implement the *_CURRENT used view, and to support the merge statement when data is loaded.';

/*==============================================================*/
/* Index: I_NASQUOTA_T2ISDELETED                                */
/*==============================================================*/
create index I_NASQUOTA_T2ISDELETED on SAHARA.NAS_QUOTA (
T2_IS_DELETED ASC
);

comment on index SAHARA.NAS_QUOTA.I_NASQUOTA_T2ISDELETED is 
'An index on Type 2 - Is Deleted for dimensional queries.';

grant SELECT on SAHARA.NAS_QUOTA to SAHARA_APP;

alter table SAHARA.NAS_QUOTA
   add constraint FK_NAS_QUOT_FK_NAS_QU_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_QUOTA
   add constraint FK_NAS_QUOT_FK_NAS_QU_NAS_FILE foreign key (NAS_FILESYSTEM_KEY)
      references SAHARA.NAS_FILESYSTEM (NAS_FILESYSTEM_KEY)
      on update restrict
      on delete restrict;

end if; /*Quota if*/

if not exists(select 1 from sys.systable where table_name='NAS_QUOTA_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: NAS_QUOTA_                                            */
/*==============================================================*/
create table SAHARA_APP.NAS_QUOTA_ 
(
   NAS_QUOTA_OID        varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   not null,
   NAS_FILESYSTEM_OID   varchar(512)                   not null,
   DISPLAY_NAME         varchar(256)                   not null,
   TYPE                 varchar(32)                    not null,
   TARGET               varchar(256)                   not null
);

comment on table SAHARA_APP.NAS_QUOTA_ is 
'This is a staging table for loading data into warehouse table SAHARA.NAS_FILESYSTEM.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the dimension table for NAS Quota.  It defines the attributes of NAS Quota that may be used to qualify queries.';

comment on column SAHARA_APP.NAS_QUOTA_.NAS_QUOTA_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_.ARRAY_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_.NAS_FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_.DISPLAY_NAME is 
'Array display name.';

comment on column SAHARA_APP.NAS_QUOTA_.TYPE is 
'Array type.  This can be:
Filer
Enclosure
Unenclosed Device';

comment on column SAHARA_APP.NAS_QUOTA_.TARGET is 
'Array display name.';

alter table SAHARA_APP.NAS_QUOTA_
   add constraint PK_NAS_QUOTA_ primary key (NAS_QUOTA_OID);

grant DELETE,INSERT,UPDATE,SELECT,ALTER,REFERENCES on SAHARA_APP.NAS_QUOTA_ to SAHARA;

end if; /*Quota_ if*/

if not exists(select 1 from sys.systable where table_name='NAS_QUOTA_CAPACITY' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: NAS_QUOTA_CAPACITY                                    */
/*==============================================================*/
create table SAHARA.NAS_QUOTA_CAPACITY 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   NAS_QUOTA_KEY        numeric(10)                    not null,
   DISK_LIMIT_HARD      numeric(22,9)                  not null default 0,
   DISK_LIMIT_SOFT      numeric(22,9)                  not null default 0,
   DISK_USED            numeric(22,9)                  not null default 0,
   FILE_LIMIT_HARD      unsigned bigint                not null default 0,
   FILE_LIMIT_SOFT      unsigned bigint                not null default 0,
   FILES_USED           unsigned bigint                not null default 0,
   THRESHOLD            numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_LIMIT_HARD numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_LIMIT_SOFT numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_USED   numeric(22,9)                  not null default 0
);

comment on table SAHARA.NAS_QUOTA_CAPACITY is 
'This is the fact table for NAS Quota measures.  Each rows represents one sampling of values for a NAS Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.ROLLUP_BATCH_KEY is 
'The surrogate key from the roll-up batch that identifies the roll-up batch in which the fact data was collected and details about the roll-up source.  This key may be used in joins to the ROLLUP_BATCH table.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DATES_KEY is 
'The surrogate key from the dates dimension (DATES table) that identifies the date on which the data was collected.  This key may be used in joins to the DATES table.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DISK_LIMIT_HARD is 
'Hard capacity limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DISK_LIMIT_SOFT is 
'Soft capacity limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.DISK_USED is 
'Capacity used on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.FILE_LIMIT_HARD is 
'Hard file limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.FILE_LIMIT_SOFT is 
'Soft file limit set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.FILES_USED is 
'Number of files used on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.THRESHOLD is 
'Threshold set on Quota.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.PHYSICAL_DISK_LIMIT_HARD is 
'Physical counterpart of hard capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.PHYSICAL_DISK_LIMIT_SOFT is 
'Physical counterpart of soft capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA.NAS_QUOTA_CAPACITY.PHYSICAL_DISK_USED is 
'Physical counterpart of capacity used on Quota. This involves the RAID overhead as well.';

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter DISK_LIMIT_HARD add constraint CKC_DISK_LIMIT_HARD_NAS_QUOT check (DISK_LIMIT_HARD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter DISK_LIMIT_SOFT add constraint CKC_DISK_LIMIT_SOFT_NAS_QUOT check (DISK_LIMIT_SOFT >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter DISK_USED add constraint CKC_DISK_USED_NAS_QUOT check (DISK_USED >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter FILE_LIMIT_HARD add constraint CKC_FILE_LIMIT_HARD_NAS_QUOT check (FILE_LIMIT_HARD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter FILE_LIMIT_SOFT add constraint CKC_FILE_LIMIT_SOFT_NAS_QUOT check (FILE_LIMIT_SOFT >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter FILES_USED add constraint CKC_FILES_USED_NAS_QUOT check (FILES_USED >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter THRESHOLD add constraint CKC_THRESHOLD_NAS_QUOT check (THRESHOLD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter PHYSICAL_DISK_LIMIT_HARD add constraint CKC_PHYSICAL_DISK_LIM_NAS_Q2 check (PHYSICAL_DISK_LIMIT_HARD >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter PHYSICAL_DISK_LIMIT_SOFT add constraint CKC_PHYSICAL_DISK_LIM_NAS_Q check (PHYSICAL_DISK_LIMIT_SOFT >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   alter PHYSICAL_DISK_USED add constraint CKC_PHYSICAL_DISK_USE_NAS_QUOT check (PHYSICAL_DISK_USED >= 0);

alter table SAHARA.NAS_QUOTA_CAPACITY
   add constraint FK_NAS_QUOT_FK_NAS_QU_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_QUOTA_CAPACITY
   add constraint FK_NAS_QUOT_FK_NAS_QU_ROLLUP_B foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.NAS_QUOTA_CAPACITY
   add constraint FK_NAS_QUOT_NAS_QUOTA_NAS_QUOT foreign key (NAS_QUOTA_KEY)
      references SAHARA.NAS_QUOTA (NAS_QUOTA_KEY)
      on update restrict
      on delete restrict;

grant SELECT on SAHARA.NAS_QUOTA_CAPACITY to SAHARA_APP;

end if; /*NAS_QUOTA_CAPACITY if*/

if not exists(select 1 from sys.systable where table_name='NAS_QUOTA_CAPACITY_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: NAS_QUOTA_CAPACITY_                                   */
/*==============================================================*/
create table SAHARA_APP.NAS_QUOTA_CAPACITY_ 
(
   NAS_QUOTA_OID        varchar(512)                   not null,
   DISK_LIMIT_HARD      numeric(22,9)                  not null default 0,
   DISK_LIMIT_SOFT      numeric(22,9)                  not null default 0,
   DISK_USED            numeric(22,9)                  not null default 0,
   FILE_LIMIT_HARD      unsigned bigint                not null default 0,
   FILE_LIMIT_SOFT      unsigned bigint                not null default 0,
   FILES_USED           unsigned bigint                not null default 0,
   THRESHOLD            numeric(22,9)                    not null default 0,
   PHYSICAL_DISK_LIMIT_HARD numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_LIMIT_SOFT numeric(22,9)                  not null default 0,
   PHYSICAL_DISK_USED   numeric(22,9)                  not null default 0
);

comment on table SAHARA_APP.NAS_QUOTA_CAPACITY_ is 
'This is a staging table for loading data into warehouse table SAHARA.NAS_FILESYSTEM_CAPACITY.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NAS quota measures.  Each rows represents one sampling of values for a NAS quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.NAS_QUOTA_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.

See the table comments for more information about the staging table.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.DISK_LIMIT_HARD is 
'Hard capacity limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.DISK_LIMIT_SOFT is 
'Soft capacity limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.DISK_USED is 
'Capacity used on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.FILE_LIMIT_HARD is 
'Hard file limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.FILE_LIMIT_SOFT is 
'Soft file limit set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.FILES_USED is 
'Number of files used on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.THRESHOLD is 
'Threshold set on Quota.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.PHYSICAL_DISK_LIMIT_HARD is 
'Physical counterpart of hard capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.PHYSICAL_DISK_LIMIT_SOFT is 
'Physical counterpart of soft capacity limit set on Quota. This involves the RAID overhead as well.';

comment on column SAHARA_APP.NAS_QUOTA_CAPACITY_.PHYSICAL_DISK_USED is 
'Physical counterpart of capacity used on Quota. This involves the RAID overhead as well.';

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter DISK_LIMIT_HARD add constraint CKC_DISK_LIMIT_HARD_NAS_QUO_ check (DISK_LIMIT_HARD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter DISK_LIMIT_SOFT add constraint CKC_DISK_LIMIT_SOFT_NAS_QUO_ check (DISK_LIMIT_SOFT >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter DISK_USED add constraint CKC_DISK_USED_NAS_QUOT check (DISK_USED >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter FILE_LIMIT_HARD add constraint CKC_FILE_LIMIT_HARD_NAS_QUOT check (FILE_LIMIT_HARD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter FILE_LIMIT_SOFT add constraint CKC_FILE_LIMIT_SOFT_NAS_QUOT check (FILE_LIMIT_SOFT >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter FILES_USED add constraint CKC_FILES_USED_NAS_QUOT check (FILES_USED >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter THRESHOLD add constraint CKC_THRESHOLD_NAS_QUOT check (THRESHOLD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter PHYSICAL_DISK_LIMIT_HARD add constraint CKC_PHYSICAL_DISK_LIM_NAS_QUOT_2 check (PHYSICAL_DISK_LIMIT_HARD >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter PHYSICAL_DISK_LIMIT_SOFT add constraint CKC_PHYSICAL_DISK_LIM_NAS_QUOT_ check (PHYSICAL_DISK_LIMIT_SOFT >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   alter PHYSICAL_DISK_USED add constraint CKC_PHYSICAL_DISK_USE_NAS_QUOT check (PHYSICAL_DISK_USED >= 0);

alter table SAHARA_APP.NAS_QUOTA_CAPACITY_
   add constraint PK_NAS_QUOTA_CAPACITY_ primary key (NAS_QUOTA_OID);

grant DELETE,ALTER,INSERT,REFERENCES,UPDATE on SAHARA_APP.NAS_QUOTA_CAPACITY_ to SAHARA;

grant SELECT on SAHARA_APP.NAS_QUOTA_CAPACITY_ to SAHARA with grant option;

end if; /*NAS_QUOTA_CAPACITY_ if*/

if not exists(select 1 from sys.systable where table_name='NAS_SHARE_HOST' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: NAS_SHARE_HOST                                        */
/*==============================================================*/
create table SAHARA.NAS_SHARE_HOST 
(
   NETAPP_SHARE_KEY     numeric(10)                    not null,
   HOST_KEY             numeric(10)                    not null,
   ACCESS_TYPE          varchar(32)                    not null
);

comment on table SAHARA.NAS_SHARE_HOST is 
'Table to keep association between NAS Share (Netapp Share) and Host.';

comment on column SAHARA.NAS_SHARE_HOST.NETAPP_SHARE_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NAS_SHARE_HOST.HOST_KEY is 
'The unique identifier of the parent filer for the volume.';

alter table SAHARA.NAS_SHARE_HOST
   add constraint PK_NAS_SHARE_HOST primary key clustered (NETAPP_SHARE_KEY, HOST_KEY);

alter table SAHARA.NAS_SHARE_HOST
   add constraint FK_NAS_SHAR_FK_NAS_SH_HOST foreign key (HOST_KEY)
      references SAHARA.HOST (HOST_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_SHARE_HOST
   add constraint FK_NAS_SHAR_FK_NAS_SH_NETAPP_S foreign key (NETAPP_SHARE_KEY)
      references SAHARA.NETAPP_SHARE (NETAPP_SHARE_KEY)
      on update restrict
      on delete restrict;

grant SELECT on SAHARA.NAS_SHARE_HOST to SAHARA_APP;

end if; /*NAS_SHARE_HOST if*/

if not exists(select 1 from sys.systable where table_name='NAS_SHARE_HOST_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: NAS_SHARE_HOST_                                       */
/*==============================================================*/
create table SAHARA_APP.NAS_SHARE_HOST_ 
(
   NETAPP_SHARE_OID     varchar(512)                   not null,
   HOST_OID             varchar(512)                   not null,
   ACCESS_TYPE          varchar(32)                    not null
);

comment on table SAHARA_APP.NAS_SHARE_HOST_ is 
'Staging table to keep association between NAS Share (Netapp Share) and Host.';

comment on column SAHARA_APP.NAS_SHARE_HOST_.NETAPP_SHARE_OID is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA_APP.NAS_SHARE_HOST_.HOST_OID is 
'The unique identifier of the parent filer for the volume.';

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NAS_SHARE_HOST_ to SAHARA;

end if; /*NAS_SHARE_HOST_ if*/

if not exists(select 1 from sys.systable where table_name='NAS_SHARE_HOST_FILESYSTEM' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: NAS_SHARE_HOST_FILESYSTEM                             */
/*==============================================================*/
create table SAHARA.NAS_SHARE_HOST_FILESYSTEM 
(
   NETAPP_SHARE_KEY     numeric(10)                    not null,
   FILESYSTEM_KEY       numeric(10)                    not null
);

comment on table SAHARA.NAS_SHARE_HOST_FILESYSTEM is 
'Table to keep association between NAS Share (Netapp Share) and Host FS.';

comment on column SAHARA.NAS_SHARE_HOST_FILESYSTEM.NETAPP_SHARE_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NAS_SHARE_HOST_FILESYSTEM.FILESYSTEM_KEY is 
'The unique identifier of the parent filer for the volume.';

alter table SAHARA.NAS_SHARE_HOST_FILESYSTEM
   add constraint PK_NAS_SHARE_HOST_FILESYSTEM primary key clustered (NETAPP_SHARE_KEY, FILESYSTEM_KEY);

alter table SAHARA.NAS_SHARE_HOST_FILESYSTEM
   add constraint FK_NAS_SHAR_FK_NAS_SH_FILESYST foreign key (FILESYSTEM_KEY)
      references SAHARA.FILESYSTEM (FILESYSTEM_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_SHARE_HOST_FILESYSTEM
   add constraint FK_NAS_SHAR_FK_NAS_SH_NETAPP_S foreign key (NETAPP_SHARE_KEY)
      references SAHARA.NETAPP_SHARE (NETAPP_SHARE_KEY)
      on update restrict
      on delete restrict;

grant SELECT on SAHARA.NAS_SHARE_HOST_FILESYSTEM to SAHARA_APP;

end if; /*NAS_SHARE_HOST_FILESYSTEM if*/

if not exists(select 1 from sys.systable where table_name='NAS_SHARE_HOST_FILESYSTEM_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: NAS_SHARE_HOST_FILESYSTEM_                            */
/*==============================================================*/
create table SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_ 
(
   NETAPP_SHARE_OID     varchar(512)                   not null,
   FILESYSTEM_OID       varchar(512)                   not null
);

comment on table SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_ is 
'Staging table to keep association between NAS Share (Netapp Share) and Host Filesystem.';

comment on column SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_.NETAPP_SHARE_OID is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_.FILESYSTEM_OID is 
'The unique identifier of the parent filer for the volume.';

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NAS_SHARE_HOST_FILESYSTEM_ to SAHARA;

end if; /*NAS_SHARE_HOST_FILESYSTEM_ if*/

if not exists(select 1 from sys.systable where table_name='NAS_VOLUME_FILER' and creator=user_id('SAHARA')) then

/*==============================================================*/
/* Table: NAS_VOLUME_FILER                                      */
/*==============================================================*/
create table SAHARA.NAS_VOLUME_FILER 
(
   NETAPP_VOLUME_KEY    numeric(10)                    not null,
   ARRAY_KEY            numeric(10)                    not null
);

comment on table SAHARA.NAS_VOLUME_FILER is 
'Table to keep association between NAS Volume (Netapp Volume) and filer (Array). As one volume can be associate with multiple filers (virtual filers), this is a many to many association.';

comment on column SAHARA.NAS_VOLUME_FILER.NETAPP_VOLUME_KEY is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA.NAS_VOLUME_FILER.ARRAY_KEY is 
'The unique identifier of the parent filer for the volume.';

alter table SAHARA.NAS_VOLUME_FILER
   add constraint PK_NAS_VOLUME_FILER primary key clustered (NETAPP_VOLUME_KEY, ARRAY_KEY);

alter table SAHARA.NAS_VOLUME_FILER
   add constraint FK_NAS_VOLU_FK_NAS_VO_ARRAY foreign key (ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete restrict;

alter table SAHARA.NAS_VOLUME_FILER
   add constraint FK_NAS_VOLU_FK_NAS_VO_NETAPP_V foreign key (NETAPP_VOLUME_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete restrict;

grant SELECT on SAHARA.NAS_VOLUME_FILER to SAHARA_APP;

end if; /*NAS_VOLUME_FILER if */

if not exists(select 1 from sys.systable where table_name='NAS_VOLUME_FILER_' and creator=user_id('SAHARA_APP')) then

/*==============================================================*/
/* Table: NAS_VOLUME_FILER_                                     */
/*==============================================================*/
create table SAHARA_APP.NAS_VOLUME_FILER_ 
(
   NETAPP_VOLUME_OID    varchar(512)                   not null,
   ARRAY_OID            varchar(512)                   not null
);

comment on table SAHARA_APP.NAS_VOLUME_FILER_ is 
'Staging table to keep association between NAS Volume (Netapp Volume) and filer (Array). As one volume can be associate with multiple filers (virtual filers), this is a many to many association.';

comment on column SAHARA_APP.NAS_VOLUME_FILER_.NETAPP_VOLUME_OID is 
'A unique database generated identifier for the NetApp volume record used as a surrogate key.';

comment on column SAHARA_APP.NAS_VOLUME_FILER_.ARRAY_OID is 
'The unique identifier of the parent filer for the volume.';

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.NAS_VOLUME_FILER_ to SAHARA;

end if; /*NAS_VOLUME_FILER_ if*/

/*==============================================================*/
/* Table: ARRAY                                                 */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='ARRAY' and cname='SUB_TYPE') then
ALTER TABLE SAHARA.ARRAY ADD SUB_TYPE             varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY' and cname='IP_ADDRESSES') then
ALTER TABLE SAHARA.ARRAY ADD    IP_ADDRESSES         varchar(4000)                  null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY' and cname='CLUSTER_ARRAY_KEY') then
ALTER TABLE SAHARA.ARRAY ADD    CLUSTER_ARRAY_KEY    numeric(10)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY' and cname='CLUSTER_ARRAY_NAME') then
ALTER TABLE SAHARA.ARRAY ADD    CLUSTER_ARRAY_NAME   varchar(256)                   null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY' and cname='CLUSTER_ARRAY_OID') then
ALTER TABLE SAHARA.ARRAY ADD    CLUSTER_ARRAY_OID    varchar(512)                   null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY' and cname='FILER_TYPE') then
ALTER TABLE SAHARA.ARRAY ADD    FILER_TYPE           varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY' and cname='PARENT_ARRAY_NAME') then
ALTER TABLE SAHARA.ARRAY ADD    PARENT_ARRAY_NAME    varchar(256)                   null;
end if;

grant SELECT on SAHARA.ARRAY to SAHARA_APP;

comment on column SAHARA.ARRAY.SUB_TYPE is 
'sub-type of array. Example: PFiler, Vfiler, PDM, VDM etc.';

comment on column SAHARA.ARRAY.CLUSTER_ARRAY_OID is 
'This unique identifier of the host pfiler array in CommandCentral Storage for a NetApp vfiler.  This will be null for all other arrays and NetApp filers.';

comment on column SAHARA.ARRAY.FILER_TYPE is 
'Type of Filer: Physical Filer, Virtual Filer, PDM, VDM';

comment on column SAHARA.ARRAY.PARENT_ARRAY_NAME is 
'Name of parent array of component';


if exists (select 1 from sys.sysindexes
           where tname = 'ARRAY' and iname = 'I_ARRAY_DISCOVERTYPE' and indextype = 'Non-unique' )
then
    DROP INDEX SAHARA.ARRAY.I_ARRAY_DISCOVERTYPE;
end if;

/*==============================================================*/
/* Index: I_ARRAY_SUBTYPE                                       */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_SUBTYPE'
) then
create index I_ARRAY_SUBTYPE on SAHARA.ARRAY (
SUB_TYPE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_SUBTYPE is 
'An index on Sub Type for dimensional queries.';

end if;
/*==============================================================*/
/* Index: I_ARRAY_IPADDRESSES                                   */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_IPADDRESSES'
) then
create index I_ARRAY_IPADDRESSES on SAHARA.ARRAY (
IP_ADDRESSES ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_IPADDRESSES is 
'An index on IP Addresses for dimensional queries.';

end if;
/*==============================================================*/
/* Index: I_ARRAY_CLUSTERARRAYKEY                               */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_CLUSTERARRAYKEY'
) then
create index I_ARRAY_CLUSTERARRAYKEY on SAHARA.ARRAY (
CLUSTER_ARRAY_KEY ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_CLUSTERARRAYKEY is 
'An index on Cluster Array Key for dimensional queries.';
end if;
/*==============================================================*/
/* Index: I_ARRAY_CLUSTERARRAYNAME                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_CLUSTERARRAYNAME'
) then
create index I_ARRAY_CLUSTERARRAYNAME on SAHARA.ARRAY (
CLUSTER_ARRAY_NAME ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_CLUSTERARRAYNAME is 
'An index on Cluster Array Name for dimensional queries.';
end if;
/*==============================================================*/
/* Index: I_ARRAY_CLUSTERARRAYOID                               */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_CLUSTERARRAYOID'
) then
create index I_ARRAY_CLUSTERARRAYOID on SAHARA.ARRAY (
CLUSTER_ARRAY_OID ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_CLUSTERARRAYOID is 
'An index on Cluster Array Object Identifier for dimensional queries.';

end if;

/*==============================================================*/
/* Index: I_ARRAY_APPLICATIONBE                                 */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_APPLICATIONBE'
) then
create index I_ARRAY_APPLICATIONBE on SAHARA.ARRAY (
APPLICATION_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_APPLICATIONBE is
'An index on Application Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_ARRAY_APPLICATIONCOMPOSITEBE                        */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_APPLICATIONCOMPOSITEBE'
) then
create index I_ARRAY_APPLICATIONCOMPOSITEBE on SAHARA.ARRAY (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_APPLICATIONCOMPOSITEBE is
'An index on Composite Application Business Entity for dimensional queries.';

end if;

/*==============================================================*/
/* Index: I_ARRAY_ORGANIZATIONBE                                */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_ORGANIZATIONBE'
) then
create index I_ARRAY_ORGANIZATIONBE on SAHARA.ARRAY (
ORGANIZATION_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_ORGANIZATIONBE is
'An index on Organization Business Entity for dimensional queries.';

end if;

/*==============================================================*/
/* Index: I_ARRAY_RGNZTNCMPSTB                                  */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_RGNZTNCMPSTB'
) then
create index I_ARRAY_RGNZTNCMPSTB on SAHARA.ARRAY (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_RGNZTNCMPSTB is
'An index on Composite Organization Business Entity for dimensional queries.';

end if;

/*==============================================================*/
/* Index: I_ARRAY_VOMDISCOVERTYPE                               */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_VOMDISCOVERTYPE'
) then
create index I_ARRAY_VOMDISCOVERTYPE on SAHARA.ARRAY (
VOM_DISCOVER_TYPE ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_VOMDISCOVERTYPE is
'An index on VOM Discover type for dimensional queries.';

end if;

if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'FK_ARRAY_FK_ARRAY__ARRAY'
) then
alter table SAHARA.ARRAY
   add constraint FK_ARRAY_FK_ARRAY__ARRAY foreign key (CLUSTER_ARRAY_KEY)
      references SAHARA.ARRAY (ARRAY_KEY)
      on update restrict
      on delete restrict;
end if;

/*==============================================================*/
/* Index: I_ARRAY_FILERTYPE                                     */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_FILERTYPE'
) then
create index I_ARRAY_FILERTYPE on SAHARA.ARRAY (
FILER_TYPE ASC
);
end if;

/*==============================================================*/
/* Index: I_ARRAY_PARENTARRAYNAME                               */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ARRAY' and
index_name = 'I_ARRAY_PARENTARRAYNAME'
) then
create index I_ARRAY_PARENTARRAYNAME on SAHARA.ARRAY (
PARENT_ARRAY_NAME ASC
);

comment on index SAHARA.ARRAY.I_ARRAY_PARENTARRAYNAME is 
'An index on Parent Array Name for dimensional queries.';
end if;

/*==============================================================*/
/* Table: ARRAY_                                                */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='ARRAY_' and cname='SUB_TYPE') then
ALTER TABLE SAHARA_APP.ARRAY_ ADD SUB_TYPE             varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_' and cname='IP_ADDRESSES') then
ALTER TABLE SAHARA_APP.ARRAY_ ADD    IP_ADDRESSES         varchar(4000)                  null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_' and cname='CLUSTER_ARRAY_NAME') then
ALTER TABLE SAHARA_APP.ARRAY_ ADD    CLUSTER_ARRAY_NAME   varchar(256)                   null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_' and cname='CLUSTER_ARRAY_OID') then
ALTER TABLE SAHARA_APP.ARRAY_ ADD    CLUSTER_ARRAY_OID    varchar(512)                   null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_' and cname='FILER_TYPE') then
ALTER TABLE SAHARA_APP.ARRAY_ ADD    FILER_TYPE           varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_' and cname='PARENT_ARRAY_NAME') then
ALTER TABLE SAHARA_APP.ARRAY_ ADD    PARENT_ARRAY_NAME    varchar(256)                   null;
end if;

comment on column SAHARA_APP.ARRAY_.SUB_TYPE is 
'sub-type of array. Example: PFiler, Vfiler, PDM, VDM etc.';

comment on column SAHARA_APP.ARRAY_.CLUSTER_ARRAY_OID is 
'This unique identifier of the host pfiler array in CommandCentral Storage for a NetApp vfiler.  This will be null for all other arrays and NetApp filers.';

comment on column SAHARA_APP.ARRAY_.FILER_TYPE is 
'Type of filer';

comment on column SAHARA_APP.ARRAY_.PARENT_ARRAY_NAME is 
'Name of parent array of component';


if exists(select 1 from sys.systable where table_name='ARRAY_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ARRAY_CURRENT
end if;

/*==============================================================*/
/* View: ARRAY_CURRENT                                          */
/*==============================================================*/
create view SAHARA.ARRAY_CURRENT  as
select ARRAY_KEY,
       PARENT_ARRAY_KEY,
       ARRAY_OID,
       PARENT_ARRAY_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       ADVISORY_DISPLAY_NAME,
       VENDOR,
       FAMILY,
       TYPE,
       DISCOVERY_STATE,
       FIRMWARE_VERSION,
       MODEL,
       SERIAL_NUMBER,
       SNMP_IP_ADDRESS,
       NIS_DOMAIN,
       NIS_SERVER,     
       IS_AUTO_DISCOVERED,
       IS_VIRTUALIZER,
       IS_NIS_ENABLED,
       VOM_DISCOVER_TYPE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       SUB_TYPE,	
       IP_ADDRESSES,
       CLUSTER_ARRAY_KEY,
       CLUSTER_ARRAY_NAME,
       CLUSTER_ARRAY_OID,
       FILER_TYPE,
       PARENT_ARRAY_NAME
  from SAHARA.ARRAY
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.ARRAY_CURRENT is 
'This view filters the ARRAY table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

grant SELECT on SAHARA.ARRAY_CURRENT to SAHARA_APP;

/*==============================================================*/
/* Table: ARRAY_STORAGE                                         */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='SNAPSHOT_USED') then
ALTER TABLE SAHARA.ARRAY_STORAGE ADD SNAPSHOT_USED        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='OVERWRITE_RESERVE') then
ALTER TABLE SAHARA.ARRAY_STORAGE ADD    OVERWRITE_RESERVE    numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NETAPP_REDUNDANT_CAPACITY') then
ALTER TABLE SAHARA.ARRAY_STORAGE ADD    NETAPP_REDUNDANT_CAPACITY numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NUMBER_OF_STORAGE_POOLS') then
ALTER TABLE SAHARA.ARRAY_STORAGE ADD    NUMBER_OF_STORAGE_POOLS numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='WAFL_SPARE_NEW') then
ALTER TABLE SAHARA.ARRAY_STORAGE ADD    WAFL_SPARE_NEW       numeric(22,9) not null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_FILES') then
update SAHARA.ARRAY_STORAGE set NA_FILES = 0 where NA_FILES is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_FILES       numeric(10) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_FOLDERS') then
update SAHARA.ARRAY_STORAGE set NA_FOLDERS = 0 where NA_FOLDERS is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_FOLDERS       numeric(10) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_QTREES') then
update SAHARA.ARRAY_STORAGE set NA_QTREES = 0 where NA_QTREES is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_QTREES       numeric(10) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_SHARES') then
update SAHARA.ARRAY_STORAGE set NA_SHARES = 0 where NA_SHARES is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_SHARES       numeric(10) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_USERS') then
update SAHARA.ARRAY_STORAGE set NA_USERS = 0 where NA_USERS is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_USERS       numeric(10) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_VOLUMES') then
update SAHARA.ARRAY_STORAGE set NA_VOLUMES = 0 where NA_VOLUMES is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_VOLUMES       numeric(10) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_VOLUME_AVAILABLE_CAPACITY') then
update SAHARA.ARRAY_STORAGE set NA_VOLUME_AVAILABLE_CAPACITY = 0 where NA_VOLUME_AVAILABLE_CAPACITY is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_VOLUME_AVAILABLE_CAPACITY       numeric(22,9) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_VOLUME_CAPACITY') then
update SAHARA.ARRAY_STORAGE set NA_VOLUME_CAPACITY = 0 where NA_VOLUME_CAPACITY is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_VOLUME_CAPACITY       numeric(22,9) null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE' and cname='NA_VOLUME_USED_CAPACITY') then
update SAHARA.ARRAY_STORAGE set NA_VOLUME_USED_CAPACITY = 0 where NA_VOLUME_USED_CAPACITY is null; commit;
ALTER TABLE SAHARA.ARRAY_STORAGE ALTER    NA_VOLUME_USED_CAPACITY       numeric(22,9) null default 0;
end if;

comment on column SAHARA.ARRAY_STORAGE.WAFL_SPARE_NEW is 
'For unified storage devices, the total amount of storage, in gigabytes, that is devoted to the file system.

This value is calculated as follows: Physical capacity minus the sum of logical capacity, snapshot reserve, and RAID overhead.';

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_SNAPSHOT_USED_ARRAY_ST' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE
alter SNAPSHOT_USED add constraint CKC_SNAPSHOT_USED_ARRAY_ST check (SNAPSHOT_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERWRITE_RESERVE_ARRAY_ST' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE
   alter OVERWRITE_RESERVE add constraint CKC_OVERWRITE_RESERVE_ARRAY_ST check (OVERWRITE_RESERVE >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NETAPP_REDUNDANT__ARRAY_ST' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE
   alter NETAPP_REDUNDANT_CAPACITY add constraint CKC_NETAPP_REDUNDANT__ARRAY_ST check (NETAPP_REDUNDANT_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_STORAGE_ARRAY_ST' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE
   alter NUMBER_OF_STORAGE_POOLS add constraint CKC_NUMBER_OF_STORAGE_ARRAY_ST check (NUMBER_OF_STORAGE_POOLS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_WAFLSPARE_NEW' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE
   alter WAFL_SPARE_NEW add constraint C_ARRAYSTORAGE_WAFLSPARE_NEW check (WAFL_SPARE_NEW >= 0);
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAFILES' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NAFILES check (NA_FILES is null or(NA_FILES >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAFOLDERS' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NAFOLDERS check (NA_FOLDERS is null or(NA_FOLDERS >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAQTREES' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NAQTREES check (NA_QTREES is null or(NA_QTREES >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NASHARES' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NASHARES check (NA_SHARES is null or(NA_SHARES >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAUSERS' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NAUSERS check (NA_USERS is null or(NA_USERS >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAVOLUMES' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NAVOLUMES check (NA_VOLUMES is null or(NA_VOLUMES >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NVLMVLBLCPCT' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NVLMVLBLCPCT check (NA_VOLUME_AVAILABLE_CAPACITY is null or(NA_VOLUME_AVAILABLE_CAPACITY >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NVLMCPCT' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NVLMCPCT check (NA_VOLUME_CAPACITY is null or(NA_VOLUME_CAPACITY >= 0));
end if;

if exists (select 1 from sysconstraint c join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NVLMSDCPCT' and table_name='ARRAY_STORAGE')
then
alter table SAHARA.ARRAY_STORAGE alter constraint C_ARRAYSTORAGE_NVLMSDCPCT check (NA_VOLUME_USED_CAPACITY is null or(NA_VOLUME_USED_CAPACITY >= 0));
end if;

/*==============================================================*/
/* Table: ARRAY_STORAGE_ARRAY_                                  */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NA_VOLUME_CAPACITY') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NA_VOLUME_CAPACITY       numeric(22,9)                  not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NA_VOLUME_USED_CAPACITY') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NA_VOLUME_USED_CAPACITY       numeric(22,9)                  not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NA_VOLUME_AVAILABLE_CAPACITY') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NA_VOLUME_AVAILABLE_CAPACITY       numeric(22,9)                  not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NA_QTREES') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NA_QTREES       numeric(10)                  not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NA_SHARES') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NA_SHARES       numeric(10)                  not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NA_VOLUMES') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NA_VOLUMES       numeric(10)                  not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NA_FILESYSTEMS') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NA_FILESYSTEMS       numeric(10)                  not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='SNAPSHOT_USED') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD SNAPSHOT_USED        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='OVERWRITE_RESERVE') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD    OVERWRITE_RESERVE    numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NETAPP_REDUNDANT_CAPACITY') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD    NETAPP_REDUNDANT_CAPACITY numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='NUMBER_OF_STORAGE_POOLS') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD NUMBER_OF_STORAGE_POOLS numeric(10)                    not null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ARRAY_STORAGE_ARRAY_' and cname='WAFL_SPARE_NEW') then
ALTER TABLE SAHARA_APP.ARRAY_STORAGE_ARRAY_ ADD WAFL_SPARE_NEW       numeric(22,9)                  not null default 0;
end if;


comment on column SAHARA_APP.ARRAY_STORAGE_ARRAY_.WAFL_SPARE_NEW is 
'This column is for staging data that will end up in table SAHARA.ARRAY_STORAGE, column WAFL_SPARE.

See the table comments for more information about the staging table.  The following description is copied from the warehouse table column:

For unified storage devices, the total amount of storage, in gigabytes, that is devoted to the file system.

This value is calculated as follows: Physical capacity minus the sum of logical capacity, snapshot reserve, and RAID overhead.';

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_SNAPSHOT_USED_ARRAY_ST' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter SNAPSHOT_USED add constraint CKC_SNAPSHOT_USED_ARRAY_ST check (SNAPSHOT_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERWRITE_RESERVE_ARRAY_ST' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter OVERWRITE_RESERVE add constraint CKC_OVERWRITE_RESERVE_ARRAY_ST check (OVERWRITE_RESERVE >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NETAPP_REDUNDANT__ARRAY_ST' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NETAPP_REDUNDANT_CAPACITY add constraint CKC_NETAPP_REDUNDANT__ARRAY_ST check (NETAPP_REDUNDANT_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_STORAGE_ARRAY_ST' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NUMBER_OF_STORAGE_POOLS add constraint CKC_NUMBER_OF_STORAGE_ARRAY_ST check (NUMBER_OF_STORAGE_POOLS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_RRSTRGRR_WAFLSPARE_NEW' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter WAFL_SPARE_NEW add constraint C_RRSTRGRR_WAFLSPARE_NEW check (WAFL_SPARE_NEW >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NVLMCPCT' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUME_CAPACITY add constraint C_ARRAYSTORAGE_NVLMCPCT check (NA_VOLUME_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NVLMSDCPCT' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUME_USED_CAPACITY add constraint C_ARRAYSTORAGE_NVLMSDCPCT check (NA_VOLUME_USED_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NVLMVLBLCPCT' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUME_AVAILABLE_CAPACITY add constraint C_ARRAYSTORAGE_NVLMVLBLCPCT check (NA_VOLUME_AVAILABLE_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAQTREES' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_QTREES add constraint C_ARRAYSTORAGE_NAQTREES check (NA_QTREES >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NASHARES' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_SHARES add constraint C_ARRAYSTORAGE_NASHARES check (NA_SHARES >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAVOLUMES' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_VOLUMES add constraint C_ARRAYSTORAGE_NAVOLUMES check (NA_VOLUMES >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_ARRAYSTORAGE_NAFILESYSTEMS' and table_name='ARRAY_STORAGE_ARRAY_')
then
alter table SAHARA_APP.ARRAY_STORAGE_ARRAY_
   alter NA_FILESYSTEMS add constraint C_ARRAYSTORAGE_NAFILESYSTEMS check (NA_FILESYSTEMS >= 0);
end if;


/*==============================================================*/
/* Table: LUN                                                   */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='LUN' and cname='NAS_FILESYSTEM_KEY') then
ALTER TABLE SAHARA.LUN ADD NAS_FILESYSTEM_KEY   numeric(10)                    null;
end if;

if not exists (select 1 from sys.syscolumns where tname='LUN' and cname='ARRAY_NAMES') then
ALTER TABLE SAHARA.LUN ADD ARRAY_NAMES          varchar(2560)                  null;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'FK_LUN_LUN_TO_NA_NAS_FILE' and table_name='LUN')
then
alter table SAHARA.LUN
   add constraint FK_LUN_LUN_TO_NA_NAS_FILE foreign key (NAS_FILESYSTEM_KEY)
      references SAHARA.NAS_FILESYSTEM (NAS_FILESYSTEM_KEY)
      on update restrict
      on delete restrict;
end if;

comment on column SAHARA.LUN.NAS_FILESYSTEM_KEY is 
'A unique database generated identifier for the NAS filesystem record used as a surrogate key.';

if exists (select 1 from sys.sysindexes
           where tname = 'LUN' and iname = 'I_LUN_DISCOVERTYPE' and indextype = 'Non-unique' )
then
    DROP INDEX SAHARA.LUN.I_LUN_DISCOVERTYPE;
end if;

/*==============================================================*/
/* Index: I_LUN_NASFILESYSTEMKEY                                */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_NASFILESYSTEMKEY'
) then
create index I_LUN_NASFILESYSTEMKEY on SAHARA.LUN (
NAS_FILESYSTEM_KEY ASC
);
end if;

/*==============================================================*/
/* Index: I_LUN_APPLICATIONBE                                   */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_APPLICATIONBE'
) then
create index I_LUN_APPLICATIONBE on SAHARA.LUN (
APPLICATION_BE ASC
);

comment on index SAHARA.LUN.I_LUN_APPLICATIONBE is
'An index on Application Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_APPLICATIONCOMPOSITEBE                          */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_APPLICATIONCOMPOSITEBE'
) then
create index I_LUN_APPLICATIONCOMPOSITEBE on SAHARA.LUN (
APPLICATION_COMPOSITE_BE ASC
);

comment on index SAHARA.LUN.I_LUN_APPLICATIONCOMPOSITEBE is
'An index on Composite Application Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_ORGANIZATIONBE                                  */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_ORGANIZATIONBE'
) then
create index I_LUN_ORGANIZATIONBE on SAHARA.LUN (
ORGANIZATION_BE ASC
);

comment on index SAHARA.LUN.I_LUN_ORGANIZATIONBE is
'An index on Organization Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_ORGANIZATIONCOMPOSITEBE                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_ORGANIZATIONCOMPOSITEBE'
) then
create index I_LUN_ORGANIZATIONCOMPOSITEBE on SAHARA.LUN (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.LUN.I_LUN_ORGANIZATIONCOMPOSITEBE is
'An index on Composite Organization Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_STORAGETECHNOLOGY                               */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_STORAGETECHNOLOGY'
) then
create index I_LUN_STORAGETECHNOLOGY on SAHARA.LUN (
STORAGE_TECHNOLOGY ASC
);

comment on index SAHARA.LUN.I_LUN_STORAGETECHNOLOGY is
'An index on Storage Technology for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_STORAGEPOOLNAME                                 */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_STORAGEPOOLNAME'
) then
create index I_LUN_STORAGEPOOLNAME on SAHARA.LUN (
STORAGE_POOL_NAME ASC
);

comment on index SAHARA.LUN.I_LUN_STORAGEPOOLNAME is
'An index on Storage Pool Name for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_CONNECTIONPROTOCOL                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_CONNECTIONPROTOCOL'
) then
create index I_LUN_CONNECTIONPROTOCOL on SAHARA.LUN (
CONNECTION_PROTOCOL ASC
);

comment on index SAHARA.LUN.I_LUN_CONNECTIONPROTOCOL is
'An index on Connection Protocol for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_VOMDISCOVERTYPE                                 */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_VOMDISCOVERTYPE'
) then
create index I_LUN_VOMDISCOVERTYPE on SAHARA.LUN (
VOM_DISCOVER_TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_VOMDISCOVERTYPE is
'An index on VOM Discover type for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_ISASSIGNED                                      */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_ISASSIGNED'
) then
create index I_LUN_ISASSIGNED on SAHARA.LUN (
IS_ASSIGNED ASC
);

comment on index SAHARA.LUN.I_LUN_ISASSIGNED is
'An index on Is Assigned for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_ISEXPORTED                                      */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_ISEXPORTED'
) then
create index I_LUN_ISEXPORTED on SAHARA.LUN (
IS_EXPORTED ASC
);

comment on index SAHARA.LUN.I_LUN_ISEXPORTED is
'An index on Is Exported for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_TYPE                                            */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_TYPE'
) then
create index I_LUN_TYPE on SAHARA.LUN (
TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_TYPE is
'An index on Type for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_LUN_SUBTYPE                                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='LUN' and
index_name = 'I_LUN_SUBTYPE'
) then
create index I_LUN_SUBTYPE on SAHARA.LUN (
SUB_TYPE ASC
);

comment on index SAHARA.LUN.I_LUN_SUBTYPE is
'An index on Sub Type for dimensional queries.';
end if;


/*==============================================================*/
/* Table: LUN_                                                  */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='LUN_' and cname='NAS_FILESYSTEM_OID') then
ALTER TABLE SAHARA_APP.LUN_ ADD NAS_FILESYSTEM_OID   varchar(512)                   null;
end if;

if not exists (select 1 from sys.syscolumns where tname='LUN_' and cname='ARRAY_NAMES') then
ALTER TABLE SAHARA_APP.LUN_ ADD ARRAY_NAMES          varchar(2560)                  null;
end if;


comment on column SAHARA_APP.LUN_.NAS_FILESYSTEM_OID is 
'A unique object identifier used to correlate dimension and fact records with each other and with the warehouse records while loading the staged data.';

/*==============================================================*/
/* Table: NETAPP_VOLUME                                         */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='TYPE') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD    TYPE                 varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='SUB_TYPE') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       SUB_TYPE             varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='TIER') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       TIER                 varchar(64)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='RPM') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       RPM                  varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='DRIVE_TYPE') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       DRIVE_TYPE           varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='DISK_SIZE') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       DISK_SIZE            numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='IS_NAS_FS_CREATED') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       IS_NAS_FS_CREATED    char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='DEDUP_STATE') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       DEDUP_STATE          varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='LAST_DEDUP_OP_BEGIN_TIME') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       LAST_DEDUP_OP_BEGIN_TIME date                           null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='LAST_DEDUP_OP_END_TIME') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       LAST_DEDUP_OP_END_TIME date                           null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='STORAGE_POOL_KEY') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       STORAGE_POOL_KEY     numeric(10)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME' and cname='STORAGE_POOL_OBJECT_IDENTIFIER') then
ALTER TABLE SAHARA.NETAPP_VOLUME ADD       STORAGE_POOL_OBJECT_IDENTIFIER varchar(512)                   null;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DISK_SIZE_NETAPP_V' and table_name='NETAPP_VOLUME')
then
alter table SAHARA.NETAPP_VOLUME
   alter DISK_SIZE add constraint CKC_DISK_SIZE_NETAPP_V check (DISK_SIZE >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_NAS_FS_CREATED_NETAPP_V' and table_name='NETAPP_VOLUME')
then
alter table SAHARA.NETAPP_VOLUME
   alter IS_NAS_FS_CREATED add constraint CKC_IS_NAS_FS_CREATED_NETAPP_V check (IS_NAS_FS_CREATED in ('T','F'));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'FK_NETAPP_V_FK_NAS_VO_NETAPP_V' and table_name='NETAPP_VOLUME')
then
alter table SAHARA.NETAPP_VOLUME
   add constraint FK_NETAPP_V_FK_NAS_VO_NETAPP_V foreign key (STORAGE_POOL_KEY)
      references SAHARA.NETAPP_VOLUME (NETAPP_VOLUME_KEY)
      on update restrict
      on delete restrict;
end if;

comment on foreign key SAHARA.NETAPP_VOLUME.FK_NETAPP_V_FK_NAS_VO_NETAPP_V is 
'Volume to Storage Pool';


/*==============================================================*/
/* Index: I_NETAPPVOLUME_TYPE                                   */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_TYPE'
) then
create index I_NETAPPVOLUME_TYPE on SAHARA.NETAPP_VOLUME (
TYPE ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_SUBTYPE                                */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_SUBTYPE'
) then
create index I_NETAPPVOLUME_SUBTYPE on SAHARA.NETAPP_VOLUME (
SUB_TYPE ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_SUBTYPE is 
'An index on Sub Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_TIER                                   */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_TIER'
) then
create index I_NETAPPVOLUME_TIER on SAHARA.NETAPP_VOLUME (
TIER ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_TIER is 
'An index on Tier for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_RPM                                    */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_RPM'
) then
create index I_NETAPPVOLUME_RPM on SAHARA.NETAPP_VOLUME (
RPM ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_RPM is 
'An index on RPM for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DRIVETYPE                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_DRIVETYPE'
) then
create index I_NETAPPVOLUME_DRIVETYPE on SAHARA.NETAPP_VOLUME (
DRIVE_TYPE ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DRIVETYPE is 
'An index on Drive Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DISKSIZE                               */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_DISKSIZE'
) then
create index I_NETAPPVOLUME_DISKSIZE on SAHARA.NETAPP_VOLUME (
DISK_SIZE ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DISKSIZE is 
'An index on Disk Size for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_ISNASFSCREATED                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_ISNASFSCREATED'
) then
create index I_NETAPPVOLUME_ISNASFSCREATED on SAHARA.NETAPP_VOLUME (
IS_NAS_FS_CREATED ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_ISNASFSCREATED is 
'An index on Is NAS FS Created for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_DEDUPSTATE                             */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_DEDUPSTATE'
) then
create index I_NETAPPVOLUME_DEDUPSTATE on SAHARA.NETAPP_VOLUME (
DEDUP_STATE ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_DEDUPSTATE is 
'An index on Dedup State for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_LSTDDPPBGNTM                           */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_LSTDDPPBGNTM'
) then
create index I_NETAPPVOLUME_LSTDDPPBGNTM on SAHARA.NETAPP_VOLUME (
LAST_DEDUP_OP_BEGIN_TIME ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_LSTDDPPBGNTM is 
'An index on Last Dedup Op begin time for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_LSTDDPPNDTM                            */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_LSTDDPPNDTM'
) then
create index I_NETAPPVOLUME_LSTDDPPNDTM on SAHARA.NETAPP_VOLUME (
LAST_DEDUP_OP_END_TIME ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_LSTDDPPNDTM is 
'An index on Last Dedup Op end time for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_STORAGEPOOLKEY                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_STORAGEPOOLKEY'
) then
create index I_NETAPPVOLUME_STORAGEPOOLKEY on SAHARA.NETAPP_VOLUME (
STORAGE_POOL_KEY ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_STORAGEPOOLKEY is 
'An index on Storage Pool Key for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPVOLUME_STRGPLBJCTDNTFR                        */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_VOLUME' and
index_name = 'I_NETAPPVOLUME_STRGPLBJCTDNTFR'
) then
create index I_NETAPPVOLUME_STRGPLBJCTDNTFR on SAHARA.NETAPP_VOLUME (
STORAGE_POOL_OBJECT_IDENTIFIER ASC
);
end if;

comment on index SAHARA.NETAPP_VOLUME.I_NETAPPVOLUME_STRGPLBJCTDNTFR is 
'An index on Storage Pool Object Identifier for dimensional queries.';


if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='TYPE') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD TYPE                 varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='SUB_TYPE') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    SUB_TYPE             varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='TIER') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    TIER                 varchar(64)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='RPM') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    RPM                  varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='DRIVE_TYPE') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    DRIVE_TYPE           varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='DISK_SIZE') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    DISK_SIZE            numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='IS_NAS_FS_CREATED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    IS_NAS_FS_CREATED    char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='DEDUP_STATE') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    DEDUP_STATE          varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='LAST_DEDUP_OP_BEGIN_TIME') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    LAST_DEDUP_OP_BEGIN_TIME date                           null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='LAST_DEDUP_OP_END_TIME') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    LAST_DEDUP_OP_END_TIME date                           null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_' and cname='STORAGE_POOL_OBJECT_IDENTIFIER') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_ ADD    STORAGE_POOL_OBJECT_IDENTIFIER varchar(512)                   null;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DISK_SIZE_NETAPP_V' and table_name='NETAPP_VOLUME_')
then
alter table SAHARA_APP.NETAPP_VOLUME_
   alter DISK_SIZE add constraint CKC_DISK_SIZE_NETAPP_V check (DISK_SIZE >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_NAS_FS_CREATED_NETAPP_V' and table_name='NETAPP_VOLUME_')
then
alter table SAHARA_APP.NETAPP_VOLUME_
   alter IS_NAS_FS_CREATED add constraint CKC_IS_NAS_FS_CREATED_NETAPP_V check (IS_NAS_FS_CREATED in ('T','F'));
end if;

/*==============================================================*/
/* Table: NETAPP_VOLUME_FILE                                    */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='PHYSICAL_CAPACITY') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD    PHYSICAL_CAPACITY    numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='OVERHEAD_CAPACITY') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       OVERHEAD_CAPACITY    numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='PHYSICAL_USED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       PHYSICAL_USED        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='NUMBER_OF_DATA_DISKS') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       NUMBER_OF_DATA_DISKS numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='NUMBER_OF_PARITY_DISKS') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       NUMBER_OF_PARITY_DISKS numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='NUMBER_OF_VOLS_CREATED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       NUMBER_OF_VOLS_CREATED numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='NUMBER_OF_LUNS_CREATED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       NUMBER_OF_LUNS_CREATED numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='NUMBER_OF_TARGETS') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       NUMBER_OF_TARGETS    numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='SPACE_SHARED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       SPACE_SHARED         numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='SPACE_SAVED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       SPACE_SAVED          numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='EFFECTIVE_SPACE_USED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='PHYSICAL_SPACE_SHARED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       PHYSICAL_SPACE_SHARED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='PHYSICAL_SPACE_SAVED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       PHYSICAL_SPACE_SAVED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='PHYSICAL_EFFECTIVE_SPACE_USED') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       PHYSICAL_EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='SUBSCRIBED_PERCENT') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       SUBSCRIBED_PERCENT   numeric(7,4)                   null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='SPACE_SAVED_PERCENT') then
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ADD       SPACE_SAVED_PERCENT  numeric(7,4)                   null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='FILES') then
update SAHARA.NETAPP_VOLUME_FILE set FILES = 0 where FILES is null; commit;
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ALTER       FILES  numeric(10)                   null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='FOLDERS') then
update SAHARA.NETAPP_VOLUME_FILE set FOLDERS = 0 where FOLDERS is null; commit;
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ALTER       FOLDERS  numeric(10)                   null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='TOTAL_CONSUMPTION') then
update SAHARA.NETAPP_VOLUME_FILE set TOTAL_CONSUMPTION = 0 where TOTAL_CONSUMPTION is null; commit;
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ALTER       TOTAL_CONSUMPTION  numeric(22,9)                   null default 0;
end if;

if exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE' and cname='USERS') then
update SAHARA.NETAPP_VOLUME_FILE set USERS = 0 where USERS is null; commit;
ALTER TABLE SAHARA.NETAPP_VOLUME_FILE ALTER       USERS  numeric(10)                   null default 0;
end if;

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_CAPACITY is 
'The physical capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.OVERHEAD_CAPACITY is 
'The overhead capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_USED is 
'The physical used capacity, in gigabytes, of the volume.';

comment on column SAHARA.NETAPP_VOLUME_FILE.NUMBER_OF_VOLS_CREATED is 
'For storage pools, number of volumes created';

comment on column SAHARA.NETAPP_VOLUME_FILE.SPACE_SHARED is 
'deduplication space shared';

comment on column SAHARA.NETAPP_VOLUME_FILE.SPACE_SAVED is 
'deduplication space saved';

comment on column SAHARA.NETAPP_VOLUME_FILE.EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_SPACE_SHARED is 
'deduplication physical space shared';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_SPACE_SAVED is 
'deduplication physical space saved';

comment on column SAHARA.NETAPP_VOLUME_FILE.PHYSICAL_EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA.NETAPP_VOLUME_FILE.SUBSCRIBED_PERCENT is 
'deduplication subscribed percent';

comment on column SAHARA.NETAPP_VOLUME_FILE.SPACE_SAVED_PERCENT is 
'deduplication saved percent';

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_CAPACITY_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_CAPACITY add constraint CKC_PHYSICAL_CAPACITY_NETAPP_V check (PHYSICAL_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERHEAD_CAPACITY_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_NETAPP_V check (OVERHEAD_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_USED_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NETAPP_V check (PHYSICAL_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_DATA_DI_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_DATA_DISKS add constraint CKC_NUMBER_OF_DATA_DI_NETAPP_V check (NUMBER_OF_DATA_DISKS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_PARITY__NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_PARITY_DISKS add constraint CKC_NUMBER_OF_PARITY__NETAPP_V check (NUMBER_OF_PARITY_DISKS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_VOLS_CR_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_VOLS_CREATED add constraint CKC_NUMBER_OF_VOLS_CR_NETAPP_V check (NUMBER_OF_VOLS_CREATED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_LUNS_CR_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_LUNS_CREATED add constraint CKC_NUMBER_OF_LUNS_CR_NETAPP_V check (NUMBER_OF_LUNS_CREATED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_TARGETS_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NETAPP_V check (NUMBER_OF_TARGETS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_SPACE_SHARED_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter SPACE_SHARED add constraint CKC_SPACE_SHARED_NETAPP_V check (SPACE_SHARED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_SPACE_SAVED_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter SPACE_SAVED add constraint CKC_SPACE_SAVED_NETAPP_V check (SPACE_SAVED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_EFFECTIVE_SPACE_U_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter EFFECTIVE_SPACE_USED add constraint CKC_EFFECTIVE_SPACE_U_NETAPP_V check (EFFECTIVE_SPACE_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_SPACE_SH_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_SPACE_SHARED add constraint CKC_PHYSICAL_SPACE_SH_NETAPP_V check (PHYSICAL_SPACE_SHARED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_SPACE_SA_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_SPACE_SAVED add constraint CKC_PHYSICAL_SPACE_SA_NETAPP_V check (PHYSICAL_SPACE_SAVED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_EFFECTIV_NETAPP_V' and table_name='NETAPP_VOLUME_FILE')
then
alter table SAHARA.NETAPP_VOLUME_FILE
   alter PHYSICAL_EFFECTIVE_SPACE_USED add constraint CKC_PHYSICAL_EFFECTIV_NETAPP_V check (PHYSICAL_EFFECTIVE_SPACE_USED >= 0);
end if;


if exists (select 1 from sys.sysindexes
          where tname = 'NETAPP_VOLUME_FILE' and iname = 'FK_NTPPVLMFL_DMBATCH' and indextype = 'Foreign Key' )
then
   ALTER TABLE SAHARA.NETAPP_VOLUME_FILE DROP FOREIGN KEY FK_NTPPVLMFL_DMBATCH;
end if;

/*==============================================================*/
/* Table: NETAPP_VOLUME_FILE_HAL_                               */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='PHYSICAL_CAPACITY') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    PHYSICAL_CAPACITY    numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='OVERHEAD_CAPACITY') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    OVERHEAD_CAPACITY    numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='PHYSICAL_USED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    PHYSICAL_USED        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='NUMBER_OF_DATA_DISKS') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    NUMBER_OF_DATA_DISKS numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='NUMBER_OF_PARITY_DISKS') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    NUMBER_OF_PARITY_DISKS numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='NUMBER_OF_VOLS_CREATED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    NUMBER_OF_VOLS_CREATED numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='NUMBER_OF_LUNS_CREATED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    NUMBER_OF_LUNS_CREATED numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='NUMBER_OF_TARGETS') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    NUMBER_OF_TARGETS    numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='SPACE_SHARED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    SPACE_SHARED         numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='SPACE_SAVED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    SPACE_SAVED          numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='EFFECTIVE_SPACE_USED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='PHYSICAL_SPACE_SHARED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    PHYSICAL_SPACE_SHARED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='PHYSICAL_SPACE_SAVED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    PHYSICAL_SPACE_SAVED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='PHYSICAL_EFFECTIVE_SPACE_USED') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    PHYSICAL_EFFECTIVE_SPACE_USED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='SUBSCRIBED_PERCENT') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    SUBSCRIBED_PERCENT   numeric(7,4)                   null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_VOLUME_FILE_HAL_' and cname='SPACE_SAVED_PERCENT') then
ALTER TABLE SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ ADD    SPACE_SAVED_PERCENT  numeric(7,4)                   null default 0;
end if;

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_CAPACITY is 
'The physical capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.OVERHEAD_CAPACITY is 
'The overhead capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_USED is 
'The physical used capacity, in gigabytes, of the volume.';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.NUMBER_OF_VOLS_CREATED is 
'For storage pools, number of volumes created';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SPACE_SHARED is 
'deduplication space shared';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SPACE_SAVED is 
'deduplication space saved';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_SPACE_SHARED is 
'deduplication physical space shared';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_SPACE_SAVED is 
'deduplication physical space saved';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.PHYSICAL_EFFECTIVE_SPACE_USED is 
'deduplication effective space used';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SUBSCRIBED_PERCENT is 
'deduplication subscribed percent';

comment on column SAHARA_APP.NETAPP_VOLUME_FILE_HAL_.SPACE_SAVED_PERCENT is 
'deduplication saved percent';

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_CAPACITY_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_CAPACITY add constraint CKC_PHYSICAL_CAPACITY_NETAPP_V check (PHYSICAL_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERHEAD_CAPACITY_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_NETAPP_V check (OVERHEAD_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_USED_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NETAPP_V check (PHYSICAL_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_DATA_DI_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_DATA_DISKS add constraint CKC_NUMBER_OF_DATA_DI_NETAPP_V check (NUMBER_OF_DATA_DISKS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_PARITY__NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_PARITY_DISKS add constraint CKC_NUMBER_OF_PARITY__NETAPP_V check (NUMBER_OF_PARITY_DISKS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_VOLS_CR_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_VOLS_CREATED add constraint CKC_NUMBER_OF_VOLS_CR_NETAPP_V check (NUMBER_OF_VOLS_CREATED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_LUNS_CR_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_LUNS_CREATED add constraint CKC_NUMBER_OF_LUNS_CR_NETAPP_V check (NUMBER_OF_LUNS_CREATED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_TARGETS_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NETAPP_V check (NUMBER_OF_TARGETS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_SPACE_SHARED_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter SPACE_SHARED add constraint CKC_SPACE_SHARED_NETAPP_V check (SPACE_SHARED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_SPACE_SAVED_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter SPACE_SAVED add constraint CKC_SPACE_SAVED_NETAPP_V check (SPACE_SAVED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_EFFECTIVE_SPACE_U_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter EFFECTIVE_SPACE_USED add constraint CKC_EFFECTIVE_SPACE_U_NETAPP_V check (EFFECTIVE_SPACE_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_SPACE_SH_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_SPACE_SHARED add constraint CKC_PHYSICAL_SPACE_SH_NETAPP_V check (PHYSICAL_SPACE_SHARED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_SPACE_SA_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_SPACE_SAVED add constraint CKC_PHYSICAL_SPACE_SA_NETAPP_V check (PHYSICAL_SPACE_SAVED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_EFFECTIV_NETAPP_V' and table_name='NETAPP_VOLUME_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_VOLUME_FILE_HAL_
   alter PHYSICAL_EFFECTIVE_SPACE_USED add constraint CKC_PHYSICAL_EFFECTIV_NETAPP_V check (PHYSICAL_EFFECTIVE_SPACE_USED >= 0);
end if;


/*==============================================================*/
/* Table: NAS_FILESYSTEM                                        */
/*==============================================================*/


if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='TIER') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD TIER                 varchar(64)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='TYPE') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    TYPE                 varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='PATH') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    PATH                 varchar(256)                   not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='AUTO_EXTEND_ENABLED') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    AUTO_EXTEND_ENABLED  char(1)                        null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='IS_LOCK_ENABLED') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    IS_LOCK_ENABLED      char(1)                        null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='CWORM_STATE') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    CWORM_STATE          varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='IS_VIRTUAL_PROVISIONING_ENABLE') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    IS_VIRTUAL_PROVISIONING_ENABLE char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='IS_SOURCE') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    IS_SOURCE            char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='IS_TARGET') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    IS_TARGET            char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='REPLICATION_TECHNOLOGY') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    REPLICATION_TECHNOLOGY varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='AUTO_EXTEND_MAX_SIZE') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    AUTO_EXTEND_MAX_SIZE numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM' and cname='HIGH_WATERMARK') then
ALTER TABLE SAHARA.NAS_FILESYSTEM ADD    HIGH_WATERMARK       numeric(22,9)                  not null default 0;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AUTO_EXTEND_ENABL_NAS_FILE' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   alter AUTO_EXTEND_ENABLED add constraint CKC_AUTO_EXTEND_ENABL_NAS_FILE check (AUTO_EXTEND_ENABLED is null or (AUTO_EXTEND_ENABLED in ('T','F')));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_LOCK_ENABLED_NAS_FILE' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   alter IS_LOCK_ENABLED add constraint CKC_IS_LOCK_ENABLED_NAS_FILE check (IS_LOCK_ENABLED is null or (IS_LOCK_ENABLED in ('T','F')));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_VIRTUAL_PROVIS_NAS_FILE' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   alter IS_VIRTUAL_PROVISIONING_ENABLE add constraint CKC_IS_VIRTUAL_PROVIS_NAS_FILE check (IS_VIRTUAL_PROVISIONING_ENABLE in ('T','F'));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_SOURCE_NAS_FILE' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   alter IS_SOURCE add constraint CKC_IS_SOURCE_NAS_FILE check (IS_SOURCE in ('T','F'));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_TARGET_NAS_FILE' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   alter IS_TARGET add constraint CKC_IS_TARGET_NAS_FILE check (IS_TARGET in ('T','F'));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AUTO_EXTEND_MAX_S_NAS_FILE' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   alter AUTO_EXTEND_MAX_SIZE add constraint CKC_AUTO_EXTEND_MAX_S_NAS_FILE check (AUTO_EXTEND_MAX_SIZE >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_HIGH_WATERMARK_NAS_FILE' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   alter HIGH_WATERMARK add constraint CKC_HIGH_WATERMARK_NAS_FILE check (HIGH_WATERMARK >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'PK_NAS_FILESYSTEM' and table_name='NAS_FILESYSTEM')
then
alter table SAHARA.NAS_FILESYSTEM
   add constraint PK_NAS_FILESYSTEM primary key (NAS_FILESYSTEM_KEY);
end if;

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TIER                                  */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_TIER'
) then
create index I_NASFILESYSTEM_TIER on SAHARA.NAS_FILESYSTEM (
TIER ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TIER is 
'An index on Tier for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TYPE                                  */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_TYPE'
) then
create index I_NASFILESYSTEM_TYPE on SAHARA.NAS_FILESYSTEM (
TYPE ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TYPE is 
'An index on Type for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_PATH                                  */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_PATH'
) then
create index I_NASFILESYSTEM_PATH on SAHARA.NAS_FILESYSTEM (
PATH ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_PATH is 
'An index on Path for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TXTNDNBLD                             */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_TXTNDNBLD'
) then
create index I_NASFILESYSTEM_TXTNDNBLD on SAHARA.NAS_FILESYSTEM (
AUTO_EXTEND_ENABLED ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TXTNDNBLD is 
'An index on Auto extend enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ISLOCKENABLED                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_ISLOCKENABLED'
) then
create index I_NASFILESYSTEM_ISLOCKENABLED on SAHARA.NAS_FILESYSTEM (
IS_LOCK_ENABLED ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ISLOCKENABLED is 
'An index on Is Lock Enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_CWORMSTATE                            */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_CWORMSTATE'
) then
create index I_NASFILESYSTEM_CWORMSTATE on SAHARA.NAS_FILESYSTEM (
CWORM_STATE ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_CWORMSTATE is 
'An index on CWORM State for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_SVRTLPRVSNNGNB                        */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_SVRTLPRVSNNGNB'
) then
create index I_NASFILESYSTEM_SVRTLPRVSNNGNB on SAHARA.NAS_FILESYSTEM (
IS_VIRTUAL_PROVISIONING_ENABLE ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_SVRTLPRVSNNGNB is 
'An index on Is Virtual Provisioning enabled for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ISSOURCE                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_ISSOURCE'
) then
create index I_NASFILESYSTEM_ISSOURCE on SAHARA.NAS_FILESYSTEM (
IS_SOURCE ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ISSOURCE is 
'An index on Is Source for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_ISTARGET                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_ISTARGET'
) then
create index I_NASFILESYSTEM_ISTARGET on SAHARA.NAS_FILESYSTEM (
IS_TARGET ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_ISTARGET is 
'An index on Is Target for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_RPLCTNTCHNLG                          */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_RPLCTNTCHNLG'
) then
create index I_NASFILESYSTEM_RPLCTNTCHNLG on SAHARA.NAS_FILESYSTEM (
REPLICATION_TECHNOLOGY ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_RPLCTNTCHNLG is 
'An index on Replication Technology for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_TXTNDMXSZ                             */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_TXTNDMXSZ'
) then
create index I_NASFILESYSTEM_TXTNDMXSZ on SAHARA.NAS_FILESYSTEM (
AUTO_EXTEND_MAX_SIZE ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_TXTNDMXSZ is 
'An index on Auto extend Max size for dimensional queries.';

/*==============================================================*/
/* Index: I_NASFILESYSTEM_HIGHWATERMARK                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NAS_FILESYSTEM' and
index_name = 'I_NASFILESYSTEM_HIGHWATERMARK'
) then
create index I_NASFILESYSTEM_HIGHWATERMARK on SAHARA.NAS_FILESYSTEM (
HIGH_WATERMARK ASC
);
end if;

comment on index SAHARA.NAS_FILESYSTEM.I_NASFILESYSTEM_HIGHWATERMARK is 
'An index on High watermark for dimensional queries.';

grant SELECT on SAHARA.NAS_FILESYSTEM to SAHARA_APP;

/*==============================================================*/
/* Table: NAS_FILESYSTEM_                                       */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='TIER') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD TIER                 varchar(64)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='TYPE') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    TYPE                 varchar(32)                    not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='PATH') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    PATH                 varchar(256)                   not null default ' ';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='AUTO_EXTEND_ENABLED') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    AUTO_EXTEND_ENABLED  char(1)                        null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='IS_LOCK_ENABLED') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    IS_LOCK_ENABLED      char(1)                        null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='CWORM_STATE') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    CWORM_STATE          varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='IS_VIRTUAL_PROVISIONING_ENABLE') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    IS_VIRTUAL_PROVISIONING_ENABLE char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='IS_SOURCE') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    IS_SOURCE            char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='IS_TARGET') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    IS_TARGET            char(1)                        not null default 'F';
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='REPLICATION_TECHNOLOGY') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    REPLICATION_TECHNOLOGY varchar(32)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='AUTO_EXTEND_MAX_SIZE') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    AUTO_EXTEND_MAX_SIZE numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_' and cname='HIGH_WATERMARK') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_ ADD    HIGH_WATERMARK       numeric(22,9)                  not null default 0;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AUTO_EXTEND_ENABL_NAS_FILE' and table_name='NAS_FILESYSTEM_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_
   alter AUTO_EXTEND_ENABLED add constraint CKC_AUTO_EXTEND_ENABL_NAS_FILE check (AUTO_EXTEND_ENABLED is null or (AUTO_EXTEND_ENABLED in ('T','F')));
end if;  

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_LOCK_ENABLED_NAS_FILE' and table_name='NAS_FILESYSTEM_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_LOCK_ENABLED add constraint CKC_IS_LOCK_ENABLED_NAS_FILE check (IS_LOCK_ENABLED is null or (IS_LOCK_ENABLED in ('T','F')));
end if;  

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_VIRTUAL_PROVIS_NAS_FILE' and table_name='NAS_FILESYSTEM_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_VIRTUAL_PROVISIONING_ENABLE add constraint CKC_IS_VIRTUAL_PROVIS_NAS_FILE check (IS_VIRTUAL_PROVISIONING_ENABLE in ('T','F'));
end if;  

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_SOURCE_NAS_FILE' and table_name='NAS_FILESYSTEM_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_SOURCE add constraint CKC_IS_SOURCE_NAS_FILE check (IS_SOURCE in ('T','F'));
end if;  

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_TARGET_NAS_FILE' and table_name='NAS_FILESYSTEM_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_
   alter IS_TARGET add constraint CKC_IS_TARGET_NAS_FILE check (IS_TARGET in ('T','F'));
end if;  

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AUTO_EXTEND_MAX_S_NAS_FILE' and table_name='NAS_FILESYSTEM_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_
   alter AUTO_EXTEND_MAX_SIZE add constraint CKC_AUTO_EXTEND_MAX_S_NAS_FILE check (AUTO_EXTEND_MAX_SIZE >= 0);
end if;  

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_HIGH_WATERMARK_NAS_FILE' and table_name='NAS_FILESYSTEM_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_
   alter HIGH_WATERMARK add constraint CKC_HIGH_WATERMARK_NAS_FILE check (HIGH_WATERMARK >= 0);
end if;  
/*==============================================================*/
/* Table: NAS_FILESYSTEM_CAPACITY                               */
/*==============================================================*/


if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='VOLUME_CAPACITY') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD VOLUME_CAPACITY      numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='TREE_QUOTA_LIMIT') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    TREE_QUOTA_LIMIT     numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='TREE_QUOTA_USED') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    TREE_QUOTA_USED      numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='NUMBER_OF_LUNS') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    NUMBER_OF_LUNS       numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='FILE_COUNT_TOTAL') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    FILE_COUNT_TOTAL     numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='FILE_COUNT_USED') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    FILE_COUNT_USED      numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='PHYSICAL_USED') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    PHYSICAL_USED        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='PHYSICAL_FREE') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    PHYSICAL_FREE        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='PHYSICAL_VOLUME_CAPACITY') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    PHYSICAL_VOLUME_CAPACITY numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='PHYSICAL_TREE_QUOTA_LIMIT') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    PHYSICAL_TREE_QUOTA_LIMIT numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='PHYSICAL_TREE_QUOTA_USED') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    PHYSICAL_TREE_QUOTA_USED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='NUMBER_OF_TARGETS') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    NUMBER_OF_TARGETS    numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='REPLICATION_TARGET_CAPACITY') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    REPLICATION_TARGET_CAPACITY numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY' and cname='NO_OF_HOSTS_ACCESSING_NAS') then
ALTER TABLE SAHARA.NAS_FILESYSTEM_CAPACITY ADD    NO_OF_HOSTS_ACCESSING_NAS numeric(10)                    not null default 0;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_VOLUME_CAPACITY_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter VOLUME_CAPACITY add constraint CKC_VOLUME_CAPACITY_NAS_FILE check (VOLUME_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_TREE_QUOTA_LIMIT_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter TREE_QUOTA_LIMIT add constraint CKC_TREE_QUOTA_LIMIT_NAS_FILE check (TREE_QUOTA_LIMIT >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_TREE_QUOTA_USED_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter TREE_QUOTA_USED add constraint CKC_TREE_QUOTA_USED_NAS_FILE check (TREE_QUOTA_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_LUNS_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter NUMBER_OF_LUNS add constraint CKC_NUMBER_OF_LUNS_NAS_FILE check (NUMBER_OF_LUNS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_FILE_COUNT_TOTAL_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter FILE_COUNT_TOTAL add constraint CKC_FILE_COUNT_TOTAL_NAS_FILE check (FILE_COUNT_TOTAL >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_FILE_COUNT_USED_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter FILE_COUNT_USED add constraint CKC_FILE_COUNT_USED_NAS_FILE check (FILE_COUNT_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_USED_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NAS_FILE check (PHYSICAL_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_FREE_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_FREE add constraint CKC_PHYSICAL_FREE_NAS_FILE check (PHYSICAL_FREE >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_VOLUME_C_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_VOLUME_CAPACITY add constraint CKC_PHYSICAL_VOLUME_C_NAS_FILE check (PHYSICAL_VOLUME_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_TREE_QUO_NAS_FS2' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_TREE_QUOTA_LIMIT add constraint CKC_PHYSICAL_TREE_QUO_NAS_FS2 check (PHYSICAL_TREE_QUOTA_LIMIT >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_TREE_QUO_NAS_FS' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter PHYSICAL_TREE_QUOTA_USED add constraint CKC_PHYSICAL_TREE_QUO_NAS_FS check (PHYSICAL_TREE_QUOTA_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_TARGETS_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NAS_FILE check (NUMBER_OF_TARGETS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_REPLICATION_TARGE_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter REPLICATION_TARGET_CAPACITY add constraint CKC_REPLICATION_TARGE_NAS_FILE check (REPLICATION_TARGET_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NO_OF_HOSTS_ACCES_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY')
then
alter table SAHARA.NAS_FILESYSTEM_CAPACITY
   alter NO_OF_HOSTS_ACCESSING_NAS add constraint CKC_NO_OF_HOSTS_ACCES_NAS_FILE check (NO_OF_HOSTS_ACCESSING_NAS >= 0);
end if;
/*==============================================================*/
/* Table: NAS_FILESYSTEM_CAPACITY_                              */
/*==============================================================*/


if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='VOLUME_CAPACITY') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD    VOLUME_CAPACITY      numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='TREE_QUOTA_LIMIT') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       TREE_QUOTA_LIMIT     numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='TREE_QUOTA_USED') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       TREE_QUOTA_USED      numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='NUMBER_OF_LUNS') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       NUMBER_OF_LUNS       numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='FILE_COUNT_TOTAL') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       FILE_COUNT_TOTAL     numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='FILE_COUNT_USED') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       FILE_COUNT_USED      numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='PHYSICAL_USED') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       PHYSICAL_USED        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='PHYSICAL_FREE') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       PHYSICAL_FREE        numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='PHYSICAL_VOLUME_CAPACITY') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       PHYSICAL_VOLUME_CAPACITY numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='PHYSICAL_TREE_QUOTA_LIMIT') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       PHYSICAL_TREE_QUOTA_LIMIT numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='PHYSICAL_TREE_QUOTA_USED') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       PHYSICAL_TREE_QUOTA_USED numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='NUMBER_OF_TARGETS') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       NUMBER_OF_TARGETS    numeric(10)                    not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='REPLICATION_TARGET_CAPACITY') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       REPLICATION_TARGET_CAPACITY numeric(22,9)                  not null default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='NAS_FILESYSTEM_CAPACITY_' and cname='NO_OF_HOSTS_ACCESSING_NAS') then
ALTER TABLE SAHARA_APP.NAS_FILESYSTEM_CAPACITY_ ADD       NO_OF_HOSTS_ACCESSING_NAS numeric(10)                    not null default 0;
end if;


if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_VOLUME_CAPACITY_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter VOLUME_CAPACITY add constraint CKC_VOLUME_CAPACITY_NAS_FILE check (VOLUME_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_TREE_QUOTA_LIMIT_NAS_FIL_' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter TREE_QUOTA_LIMIT add constraint CKC_TREE_QUOTA_LIMIT_NAS_FIL_ check (TREE_QUOTA_LIMIT >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_TREE_QUOTA_USED_NAS_FIL_' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter TREE_QUOTA_USED add constraint CKC_TREE_QUOTA_USED_NAS_FIL_ check (TREE_QUOTA_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_LUNS_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter NUMBER_OF_LUNS add constraint CKC_NUMBER_OF_LUNS_NAS_FILE check (NUMBER_OF_LUNS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_FILE_COUNT_TOTAL_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter FILE_COUNT_TOTAL add constraint CKC_FILE_COUNT_TOTAL_NAS_FILE check (FILE_COUNT_TOTAL >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_FILE_COUNT_USED_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter FILE_COUNT_USED add constraint CKC_FILE_COUNT_USED_NAS_FILE check (FILE_COUNT_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_USED_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_USED add constraint CKC_PHYSICAL_USED_NAS_FILE check (PHYSICAL_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_FREE_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_FREE add constraint CKC_PHYSICAL_FREE_NAS_FILE check (PHYSICAL_FREE >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_VOLUME_C_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_VOLUME_CAPACITY add constraint CKC_PHYSICAL_VOLUME_C_NAS_FILE check (PHYSICAL_VOLUME_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_TREE_QUO_NAS_FILE_2' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_TREE_QUOTA_LIMIT add constraint CKC_PHYSICAL_TREE_QUO_NAS_FILE_2 check (PHYSICAL_TREE_QUOTA_LIMIT >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_PHYSICAL_TREE_QUO_NAS_FILE_' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter PHYSICAL_TREE_QUOTA_USED add constraint CKC_PHYSICAL_TREE_QUO_NAS_FILE_ check (PHYSICAL_TREE_QUOTA_USED >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NUMBER_OF_TARGETS_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter NUMBER_OF_TARGETS add constraint CKC_NUMBER_OF_TARGETS_NAS_FILE check (NUMBER_OF_TARGETS >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_REPLICATION_TARGE_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter REPLICATION_TARGET_CAPACITY add constraint CKC_REPLICATION_TARGE_NAS_FILE check (REPLICATION_TARGET_CAPACITY >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NO_OF_HOSTS_ACCES_NAS_FILE' and table_name='NAS_FILESYSTEM_CAPACITY_')
then
alter table SAHARA_APP.NAS_FILESYSTEM_CAPACITY_
   alter NO_OF_HOSTS_ACCESSING_NAS add constraint CKC_NO_OF_HOSTS_ACCES_NAS_FILE check (NO_OF_HOSTS_ACCESSING_NAS >= 0);
end if;

/*==============================================================*/
/* Table: NETAPP_SHARE                                          */
/*==============================================================*/


if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE' and cname='TIER') then
ALTER TABLE SAHARA.NETAPP_SHARE ADD TIER                 varchar(64)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE' and cname='ESX_DATA_STORE_NAMES') then
ALTER TABLE SAHARA.NETAPP_SHARE ADD    ESX_DATA_STORE_NAMES varchar(1024)                  null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE' and cname='HOST_PRIVILEGE') then
ALTER TABLE SAHARA.NETAPP_SHARE ADD    HOST_PRIVILEGE       varchar(10240)                 null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE' and cname='IS_MAPPED_TO_ANY_UNDISCOV_HOST') then
ALTER TABLE SAHARA.NETAPP_SHARE ADD    IS_MAPPED_TO_ANY_UNDISCOV_HOST char(1)                        not null default 'F';
end if;

   comment on column SAHARA.NETAPP_SHARE.HOST_PRIVILEGE is 
'Comma separated list of array side discovered hosts and their privileges';

comment on column SAHARA.NETAPP_SHARE.IS_MAPPED_TO_ANY_UNDISCOV_HOST is 
'True: if any associated array side discovered host is not correlated';

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_MAPPED_TO_ANY__NETAPP_S' and table_name='NETAPP_SHARE')
then
alter table SAHARA.NETAPP_SHARE
   alter IS_MAPPED_TO_ANY_UNDISCOV_HOST add constraint CKC_IS_MAPPED_TO_ANY__NETAPP_S check (IS_MAPPED_TO_ANY_UNDISCOV_HOST in ('T','F'));
end if;

/*==============================================================*/
/* Index: I_NETAPPSHARE_TIER                                    */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_SHARE' and
index_name = 'I_NETAPPSHARE_TIER'
) then
create index I_NETAPPSHARE_TIER on SAHARA.NETAPP_SHARE (
TIER ASC
);
end if;

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_TIER is 
'An index on Tier for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_SXDTSTRNMS                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_SHARE' and
index_name = 'I_NETAPPSHARE_SXDTSTRNMS'
) then
create index I_NETAPPSHARE_SXDTSTRNMS on SAHARA.NETAPP_SHARE (
ESX_DATA_STORE_NAMES ASC
);
end if;

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_SXDTSTRNMS is 
'An index on ESX Data Store names for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_HOSTPRIVILEGE                           */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_SHARE' and
index_name = 'I_NETAPPSHARE_HOSTPRIVILEGE'
) then
create index I_NETAPPSHARE_HOSTPRIVILEGE on SAHARA.NETAPP_SHARE (
HOST_PRIVILEGE ASC
);
end if;

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_HOSTPRIVILEGE is 
'An index on Host Privilege for dimensional queries.';

/*==============================================================*/
/* Index: I_NETAPPSHARE_SMPPDTNNDSCVHST                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='NETAPP_SHARE' and
index_name = 'I_NETAPPSHARE_SMPPDTNNDSCVHST'
) then
create index I_NETAPPSHARE_SMPPDTNNDSCVHST on SAHARA.NETAPP_SHARE (
IS_MAPPED_TO_ANY_UNDISCOV_HOST ASC
);
end if;

comment on index SAHARA.NETAPP_SHARE.I_NETAPPSHARE_SMPPDTNNDSCVHST is 
'An index on Is Mapped to Any Undiscov Host for dimensional queries.';

/*==============================================================*/
/* Table: NETAPP_SHARE_                                         */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE_' and cname='TIER') then
ALTER TABLE SAHARA_APP.NETAPP_SHARE_ ADD    TIER                 varchar(64)                    null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE_' and cname='ESX_DATA_STORE_NAMES') then
ALTER TABLE SAHARA_APP.NETAPP_SHARE_ ADD    ESX_DATA_STORE_NAMES varchar(1024)                  null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE_' and cname='HOST_PRIVILEGE') then
ALTER TABLE SAHARA_APP.NETAPP_SHARE_ ADD    HOST_PRIVILEGE       varchar(10240)                 null;
end if;
if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE_' and cname='IS_MAPPED_TO_ANY_UNDISCOV_HOST') then
ALTER TABLE SAHARA_APP.NETAPP_SHARE_ ADD    IS_MAPPED_TO_ANY_UNDISCOV_HOST char(1)                        not null default 'F';
end if;

comment on column SAHARA_APP.NETAPP_SHARE_.HOST_PRIVILEGE is 
'Comma separated list of array side discovered hosts and their privileges';

comment on column SAHARA_APP.NETAPP_SHARE_.IS_MAPPED_TO_ANY_UNDISCOV_HOST is 
'True: if any associated array side discovered host is not correlated';

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_MAPPED_TO_ANY__NETAPP_S' and table_name='NETAPP_SHARE_')
then
alter table SAHARA_APP.NETAPP_SHARE_
   alter IS_MAPPED_TO_ANY_UNDISCOV_HOST add constraint CKC_IS_MAPPED_TO_ANY__NETAPP_S check (IS_MAPPED_TO_ANY_UNDISCOV_HOST in ('T','F'));
end if;

/*==============================================================*/
/* Table: NETAPP_SHARE_FILE                                     */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE_FILE' and cname='NO_OF_HOSTS') then
ALTER TABLE SAHARA.NETAPP_SHARE_FILE ADD    NO_OF_HOSTS          numeric(10)                    null;
end if;

if exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE_FILE' and cname='DM_BATCH_KEY') then
-- DCC: drop foreign key, alter column and add (foreign key?).
   if exists (select 1 from sys.sysindexes
              where tname = 'NETAPP_SHARE_FILE' and iname = 'FK_NTPPSHRFL_DMBATCH' and indextype = 'Foreign Key' )
   then
       ALTER TABLE SAHARA.NETAPP_SHARE_FILE DROP FOREIGN KEY FK_NTPPSHRFL_DMBATCH;
   end if;
ALTER TABLE SAHARA.NETAPP_SHARE_FILE ALTER    DM_BATCH_KEY       numeric(10)                    null;
end if;

/*==============================================================*/
/* Table: NETAPP_SHARE_FILE_HAL_                                */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='NETAPP_SHARE_FILE_HAL_' and cname='NO_OF_HOSTS') then
ALTER TABLE SAHARA_APP.NETAPP_SHARE_FILE_HAL_ ADD    NO_OF_HOSTS          numeric(10)                    not null default 0;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_NO_OF_HOSTS_NETAPP_S' and table_name='NETAPP_SHARE_FILE_HAL_')
then
alter table SAHARA_APP.NETAPP_SHARE_FILE_HAL_
   alter NO_OF_HOSTS add constraint CKC_NO_OF_HOSTS_NETAPP_S check (NO_OF_HOSTS >= 0);
end if;

/*==============================================================*/
/* Table: HOST_CAPACITY                                         */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='NAS_ALLOCATED_CAPACITY') then
ALTER TABLE SAHARA.HOST_CAPACITY ADD    NAS_ALLOCATED_CAPACITY numeric(22,9)                  not null  default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='NAS_CONSUMED_CAPACITY') then
ALTER TABLE SAHARA.HOST_CAPACITY ADD       NAS_CONSUMED_CAPACITY numeric(22,9)                  not null  default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_NAS_ALLOCATE_CAPACITY') then
ALTER TABLE SAHARA.HOST_CAPACITY ADD       ADJUSTED_NAS_ALLOCATE_CAPACITY numeric(22,9)                  not null  default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_NAS_CONSUMED_CAPACITY') then
ALTER TABLE SAHARA.HOST_CAPACITY ADD       ADJUSTED_NAS_CONSUMED_CAPACITY numeric(22,9)                  not null  default 0;
end if;

/*==============================================================*/
/* Table: HOST_CAPACITY_                                        */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='NAS_ALLOCATED_CAPACITY') then
ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD    NAS_ALLOCATED_CAPACITY numeric(22,9)                  not null  default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='NAS_CONSUMED_CAPACITY') then
ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD       NAS_CONSUMED_CAPACITY numeric(22,9)                  not null  default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_NAS_ALLOCATE_CAPACITY') then
ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD       ADJUSTED_NAS_ALLOCATE_CAPACITY numeric(22,9)                  not null  default 0;
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_NAS_CONSUMED_CAPACITY') then
ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD       ADJUSTED_NAS_CONSUMED_CAPACITY numeric(22,9)                  not null  default 0;
end if;


/*==============================================================*/
/* Table: FILESYSTEM                                            */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='FILESYSTEM' and cname='IS_BASED_ON_NAS_SHARE') then
ALTER TABLE SAHARA.FILESYSTEM ADD    IS_BASED_ON_NAS_SHARE char(1)                        null;
end if;

/*==============================================================*/
/* Table: FILESYSTEM_                                           */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='FILESYSTEM_' and cname='IS_BASED_ON_NAS_SHARE') then
ALTER TABLE SAHARA_APP.FILESYSTEM_ ADD    IS_BASED_ON_NAS_SHARE char(1)                        null;
end if;

/*==============================================================*/
/* View: ARRAY_STORAGE_                                         */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='ARRAY_STORAGE_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.ARRAY_STORAGE_
end if;

create view SAHARA_APP.ARRAY_STORAGE_ (ARRAY_OID, TOTAL_PHYSICAL, V_PHYSICAL_EXTERNAL, LOCAL_LOGICAL, V_VIRTUAL_LOGICAL, LOGICAL_AVAILABLE, ALLOCATED, A_ALLOCATED_TO_HOSTS_DIRECTLY, A_ALLOCATED_TO_VIRTUALIZER, A_ALLOC_HOST_VIA_VIRTUALIZER, V_LOCAL_LOGICAL_ALLOCATED, V_VIRTUAL_LOGICAL_ALLOCATED, UNALLOCATED, A_UNALLOCATED_VIA_VIRUALIZER, V_LOCAL_LOGICAL_UNALLOCATED, V_VIRTUAL_LOGICAL_UNALLOCATED, CLAIMED_BY_HOSTS, REPLICATION, RAID_OVERHEAD, ADMINISTRATIVE, SNAPSHOT_RESERVE, WAFL_SPARE, DAS_CAPACITY, NAS_USED_CAPACITY, UNCONFIGURED_CAPACITY, INTERNAL_PHYSICAL, LOGICAL_UNKNOWN, TOTAL_UNCLAIMED, UNDISCOVERED_ALLOCATED, PERCENT_ALLOCATED_HOST_CLAIMED, PERCENT_CONFIGURED, PERCENT_CONFIGURED_ALLOCATED, PERCENT_CONFIGURED_AVAILABLE, PERCENT_CONFIGURED_OVERHEAD, PERCENT_CONFIGURED_UNDISCOVERD, PERCENT_UNCONFIGURED, CONSUMPTION_ASSIGNED, CONSUMPTION_UNKNOWN, HOST_USAGE_APPS_ON_RAW, HOST_USAGE_FS_UN_USED, HOST_USAGE_FS_USED, HOST_USAGE_FS_AUSED_APPS, HOST_USAGE_FS_NAUSED_NON_APPS, HOST_USAGE_OTHER, APPL_USED_CAPACITY, APPL_UN_USED_CAPACITY, THINAWARE_CLAIMED_BY_HOST, THINAWARE_TOTAL_UNCLAIMED, THINAWARE_ALLOCATED_TOHOST, THINAWARE_ALC_THGVIRTUALIZER, THINAWARE_UNDISC_ALLOCATED, NA_VOLUME_CAPACITY, NA_VOLUME_USED_CAPACITY, NA_VOLUME_AVAILABLE_CAPACITY, NA_QTREES, NA_SHARES, NA_VOLUMES, NA_FILESYSTEMS, DM_BATCH_ID, NA_CONSUMPTION, NA_FILES, NA_FOLDERS, NA_USERS, SNAPSHOT_USED, OVERWRITE_RESERVE, NETAPP_REDUNDANT_CAPACITY, NUMBER_OF_STORAGE_POOLS, WAFL_SPARE_NEW) as
select ARRAY.ARRAY_OID,
       ARRAY.TOTAL_PHYSICAL,
       ARRAY.V_PHYSICAL_EXTERNAL,
       ARRAY.LOCAL_LOGICAL,
       ARRAY.V_VIRTUAL_LOGICAL,
       ARRAY.LOGICAL_AVAILABLE,
       ARRAY.ALLOCATED,
       ARRAY.A_ALLOCATED_TO_HOSTS_DIRECTLY,
       ARRAY.A_ALLOCATED_TO_VIRTUALIZER,
       ARRAY.A_ALLOC_HOST_VIA_VIRTUALIZER,
       ARRAY.V_LOCAL_LOGICAL_ALLOCATED,
       ARRAY.V_VIRTUAL_LOGICAL_ALLOCATED,
       ARRAY.UNALLOCATED,
       ARRAY.A_UNALLOCATED_VIA_VIRUALIZER,
       ARRAY.V_LOCAL_LOGICAL_UNALLOCATED,
       ARRAY.V_VIRTUAL_LOGICAL_UNALLOCATED,
       ARRAY.CLAIMED_BY_HOSTS,
       ARRAY.REPLICATION,
       ARRAY.RAID_OVERHEAD,
       ARRAY.ADMINISTRATIVE,
       ARRAY.SNAPSHOT_RESERVE,
       ARRAY.WAFL_SPARE,
       ARRAY.DAS_CAPACITY,
       ARRAY.NAS_USED_CAPACITY,
       ARRAY.UNCONFIGURED_CAPACITY,
       ARRAY.INTERNAL_PHYSICAL,
       ARRAY.LOGICAL_UNKNOWN,
       ARRAY.TOTAL_UNCLAIMED,
       ARRAY.UNDISCOVERED_ALLOCATED,
       ARRAY.PERCENT_ALLOCATED_HOST_CLAIMED,
       ARRAY.PERCENT_CONFIGURED,
       ARRAY.PERCENT_CONFIGURED_ALLOCATED,
       ARRAY.PERCENT_CONFIGURED_AVAILABLE,
       ARRAY.PERCENT_CONFIGURED_OVERHEAD,
       ARRAY.PERCENT_CONFIGURED_UNDISCOVERD,
       ARRAY.PERCENT_UNCONFIGURED,
       ARRAY.CONSUMPTION_ASSIGNED,
  	   ARRAY.CONSUMPTION_UNKNOWN,
  	   ARRAY.HOST_USAGE_APPS_ON_RAW,
  	   ARRAY.HOST_USAGE_FS_UN_USED,
  	   ARRAY.HOST_USAGE_FS_USED,
  	   ARRAY.HOST_USAGE_FS_AUSED_APPS,
  	   ARRAY.HOST_USAGE_FS_NAUSED_NON_APPS,
  	   ARRAY.HOST_USAGE_OTHER,
  	   ARRAY.APPL_USED_CAPACITY,
  	   ARRAY.APPL_UN_USED_CAPACITY,
       ARRAY.THINAWARE_CLAIMED_BY_HOST,	
       ARRAY.THINAWARE_TOTAL_UNCLAIMED,	
       ARRAY.THINAWARE_ALLOCATED_TOHOST,
       ARRAY.THINAWARE_ALC_THGVIRTUALIZER,
       ARRAY.THINAWARE_UNDISC_ALLOCATED,
       ARRAY.NA_VOLUME_CAPACITY,
       ARRAY.NA_VOLUME_USED_CAPACITY,
       ARRAY.NA_VOLUME_AVAILABLE_CAPACITY,
       ARRAY.NA_QTREES,
       ARRAY.NA_SHARES,
       ARRAY.NA_VOLUMES,
       ARRAY.NA_FILESYSTEMS,
       isnull(NADM.DM_BATCH_ID,0) as DM_BATCH_ID,
       isnull(NADM.NA_CONSUMPTION,0) as NA_CONSUMPTION,
       isnull(NADM.NA_FILES,0) as NA_FILES,
       isnull(NADM.NA_FOLDERS,0) as NA_FOLDERS,
       isnull(NADM.NA_USERS,0) as NA_USERS,
       ARRAY.SNAPSHOT_USED,
       ARRAY.OVERWRITE_RESERVE,
       ARRAY.NETAPP_REDUNDANT_CAPACITY,
       ARRAY.NUMBER_OF_STORAGE_POOLS,
       ARRAY.WAFL_SPARE_NEW
  from SAHARA_APP.ARRAY_STORAGE_ARRAY_ ARRAY
  left outer join SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_ NAHAL on NAHAL.ARRAY_OID = ARRAY.ARRAY_OID
  left outer join SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_ NADM on NADM.DM_ID = NAHAL.DM_ID;

comment on view SAHARA_APP.ARRAY_STORAGE_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.ARRAY_STORAGE.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for array storage measures.  Each rows represents one sampling of values for an array.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.ARRAY_STORAGE_ to SAHARA;

/*==============================================================*/
/* View: FILESYSTEM_CURRENT                                     */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='FILESYSTEM_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.FILESYSTEM_CURRENT
end if;

create view SAHARA.FILESYSTEM_CURRENT  as
select FILESYSTEM_KEY,
       FILESYSTEM_OID,
       VOLUME_KEY,
       HOST_KEY,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       ADVISORY_DISPLAY_NAME,
       MOUNT_POINT,
       DEVICE_PATH,
       TYPE,
       IS_CREATED_FROM_VOLUME,
       IS_CLUSTERED,
       IS_VCS_CVM,
       IS_DIRECT_ATTACHED,
       IS_CFS_AWARE,
       IS_BASED_ON_THIN,
       IS_EXPORTED,
       IS_CHILD,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_BASED_ON_NAS_SHARE
  from SAHARA.FILESYSTEM 
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.FILESYSTEM_CURRENT is 
'This view filters the FILESYSTEM table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.FILESYSTEM_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: LUN_CURRENT                                            */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='LUN_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.LUN_CURRENT
end if;

create view SAHARA.LUN_CURRENT  as
select
   LUN_KEY,
   ARRAY_KEY,
   LUN_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   ADVISORY_DISPLAY_NAME,
   DRIVE_TYPE,
   RAID_LEVEL,
   EMULATION_MODE,
   MASKING_STATUS,
   CLAIMED_STATUS,
   DISCOVERY_STATE,
   NUMBER_OF_COPIES,
   PARITY,
   RESOURCE_TYPE,
   REVISION,
   SERIAL_NUMBER,
   STRIPE_WIDTH,
   VENDOR_DEVICE_ID,
   VENDOR_NAME,
   VENDOR_STATUS,
   BINDING_STATE,
   COLUMNS,
   TIER,
   TIER_COST_PER_GB,
   IS_PRIMARY,
   IS_READ_CACHE_ENABLED,
   IS_SECONDARY,
   IS_SEGMENT,
   IS_WRITE_CACHE_ENABLED,
   IS_AGGREGATE,
   IS_CONTROL_DEVICE,
   IS_FAULT_TOLERANT,
   IS_ADMINISTRATIVE,
   IS_USED_IN_SNAPSHOT,
   IS_VIRTUALIZER_SOURCE,
   IS_DIRECT_ATTACHED,
   IS_BASEDON_EXTERNAL,
   ARRAY_PORTS,
   DISK_VENDOR,
   DISK_CAPACITY,
   VENDOR_LAYOUT,
   THIN_STATUS,
   VOM_DISCOVER_TYPE,
   IS_ASSIGNED,
   IS_EXPORTED,
   APPLICATION_BE,
   APPLICATION_COMPOSITE_BE,
   ORGANIZATION_BE,
   ORGANIZATION_COMPOSITE_BE,
   TYPE,
   SUB_TYPE,
   STORAGE_TECHNOLOGY,
   STORAGE_POOL_NAME,
   CONNECTION_PROTOCOL,
   NAS_FILESYSTEM_KEY,
   ARRAY_NAMES
from
   SAHARA.LUN
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.LUN_CURRENT is 
'This view filters the LUN table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.LUN_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NAS_FILESYSTEM_CURRENT                                 */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='NAS_FILESYSTEM_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NAS_FILESYSTEM_CURRENT
end if;

create view SAHARA.NAS_FILESYSTEM_CURRENT  as
select NAS_FILESYSTEM_KEY,
   NAS_FILESYSTEM_OID,
   ARRAY_KEY,
   NETAPP_VOLUME_KEY,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   DISCOVERY_STATE,
   SECURITY_STYLE,
   FILESYSTEM_ID,
   IS_QUOTA_ENABLED,
   TIER,
   TYPE,
   PATH,
   AUTO_EXTEND_ENABLED,
   IS_LOCK_ENABLED,
   CWORM_STATE,
   IS_VIRTUAL_PROVISIONING_ENABLE,
   IS_SOURCE,
   IS_TARGET,
   REPLICATION_TECHNOLOGY,
   AUTO_EXTEND_MAX_SIZE,
   HIGH_WATERMARK
  from SAHARA.NAS_FILESYSTEM
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NAS_FILESYSTEM_CURRENT is 
'This view filters the NAS_FILESYSTEM table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

grant SELECT on SAHARA.NAS_FILESYSTEM_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NAS_QUOTA_CURRENT                                      */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='NAS_QUOTA_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NAS_QUOTA_CURRENT
end if;

create view SAHARA.NAS_QUOTA_CURRENT  as
select NAS_QUOTA_KEY,
   NAS_QUOTA_OID,
   ARRAY_KEY,
   NAS_FILESYSTEM_KEY,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   TYPE,
   TARGET
  from SAHARA.NAS_QUOTA
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NAS_QUOTA_CURRENT is 
'This view filters the NAS_FILESYSTEM table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

grant SELECT on SAHARA.NAS_QUOTA_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NETAPP_SHARE_CURRENT                                   */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='NETAPP_SHARE_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NETAPP_SHARE_CURRENT
end if;

create view SAHARA.NETAPP_SHARE_CURRENT  as
select NETAPP_SHARE_KEY,
       NETAPP_SHARE_OID,
       ARRAY_KEY,
       NETAPP_VOLUME_KEY,
       NETAPP_QTREE_KEY,
       NAS_FILESYSTEM_KEY,
       CC_STORAGE_URL,
       SHARE_PATH,
       NFS_EXPORT_PATH,
       DISPLAY_NAME,
       CIFS_MOUNT_POINT,
       CIFS_SHARE_NAME,
       MOUNT_POINT_TYPE,
       DISCOVERY_STATE,
       TIER,
       ESX_DATA_STORE_NAMES,
       HOST_PRIVILEGE,
       IS_MAPPED_TO_ANY_UNDISCOV_HOST
  from SAHARA.NETAPP_SHARE
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NETAPP_SHARE_CURRENT is 
'This view filters the NETAPP_SHARE table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.NETAPP_SHARE_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NETAPP_SHARE_FILE_                                     */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='NETAPP_SHARE_FILE_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_SHARE_FILE_
end if;

create view SAHARA_APP.NETAPP_SHARE_FILE_  as
select HAL.NETAPP_SHARE_OID,
       isnull(DM.DM_BATCH_ID,0) as DM_BATCH_ID,
       isnull(DM.TOTAL_CONSUMPTION,0) as TOTAL_CONSUMPTION,
       isnull(DM.FILES,0) as FILES,
       isnull(DM.FOLDERS,0) as FOLDERS,
       isnull(DM.USERS,0) as USERS,
       HAL.NO_OF_HOSTS
  from SAHARA_APP.NETAPP_SHARE_FILE_HAL_ HAL
  left join SAHARA_APP.NETAPP_SHARE_FILE_DM_ DM on DM.DM_ID = HAL.DM_ID;

comment on view SAHARA_APP.NETAPP_SHARE_FILE_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_SHARE_FILE.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp share measures.  Each rows represents one sampling of values for a volume.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_SHARE_FILE_ to SAHARA;

/*==============================================================*/
/* View: NETAPP_VOLUME_CURRENT                                  */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='NETAPP_VOLUME_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NETAPP_VOLUME_CURRENT
end if;

create view SAHARA.NETAPP_VOLUME_CURRENT  as
select NETAPP_VOLUME_KEY,
       NETAPP_VOLUME_OID,
       ARRAY_KEY,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       DISCOVERY_STATE,
       VENDOR_STATE,
       VENDOR_LAYOUT,
       RAID_LEVEL,
       IS_AGGREGATE,
       IS_SEGMENT,
       IS_VFILER_VOLUME,
       TYPE,
       SUB_TYPE,
       TIER,
       RPM,
       DRIVE_TYPE,
       DISK_SIZE,
       IS_NAS_FS_CREATED,
       DEDUP_STATE,
       LAST_DEDUP_OP_BEGIN_TIME,
       LAST_DEDUP_OP_END_TIME,
       STORAGE_POOL_KEY,
       STORAGE_POOL_OBJECT_IDENTIFIER
  from SAHARA.NETAPP_VOLUME
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NETAPP_VOLUME_CURRENT is 
'This view filters the NETAPP_VOLUME table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.NETAPP_VOLUME_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NETAPP_VOLUME_FILE_                                    */
/*==============================================================*/
if exists(select 1 from sys.systable where table_name='NETAPP_VOLUME_FILE_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_VOLUME_FILE_
end if;

create view SAHARA_APP.NETAPP_VOLUME_FILE_  as
select HAL.NETAPP_VOLUME_OID,
       HAL.CAPACITY,
       HAL.USED_CAPACITY,
       HAL.AVAILABLE_CAPACITY,
       HAL.QTREES,
       isnull(DM.DM_BATCH_ID,0) as DM_BATCH_ID,
       isnull(DM.TOTAL_CONSUMPTION,0) as TOTAL_CONSUMPTION,
       isnull(DM.FILES,0) as FILES,
       isnull(DM.FOLDERS,0) as FOLDERS,
       isnull(DM.USERS,0) as USERS,
       HAL.PHYSICAL_CAPACITY,
       HAL.OVERHEAD_CAPACITY,
       HAL.PHYSICAL_USED,
       HAL.NUMBER_OF_DATA_DISKS,
       HAL.NUMBER_OF_PARITY_DISKS,
       HAL.NUMBER_OF_VOLS_CREATED,
       HAL.NUMBER_OF_LUNS_CREATED,
       HAL.NUMBER_OF_TARGETS,
       HAL.SPACE_SHARED,
       HAL.SPACE_SAVED,
       HAL.EFFECTIVE_SPACE_USED,
       HAL.PHYSICAL_SPACE_SHARED,
       HAL.PHYSICAL_SPACE_SAVED,
       HAL.PHYSICAL_EFFECTIVE_SPACE_USED,
       HAL.SUBSCRIBED_PERCENT,
       HAL.SPACE_SAVED_PERCENT
  from SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ HAL
  left join SAHARA_APP.NETAPP_VOLUME_FILE_DM_ DM on DM.DM_ID = HAL.DM_ID;

comment on view SAHARA_APP.NETAPP_VOLUME_FILE_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_VOLUME_FILE.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp volume measures.  Each rows represents one sampling of values for a volume.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_VOLUME_FILE_ to SAHARA;


/*==============================================================*/
/* Table: FORECASTING_DIMENSION                                 */
/*==============================================================*/
if not exists(select 1 from sys.systable where table_name='FORECASTING_DIMENSION' and creator=user_id('SAHARA')) then

	create table SAHARA.FORECASTING_DIMENSION 
	(
	   DIMENSION            varchar(26)                    not null,
	   DISPLAY_NAME         varchar(64)                    not null,
	   METAMODEL_PATH       varchar(1024)                  not null,
	   FACT_TABLE_NAME      varchar(1024)                  not null,
	   METAMODEL_KEY        varchar(1024)                  not null
	);

	comment on table SAHARA.FORECASTING_DIMENSION is 
	'This table contains a list of dimensions that can support forcasting, it will also contain metamodel path for the corresponding dimension. This table is populated at install and upgrade.';

	comment on column SAHARA.FORECASTING_DIMENSION.DIMENSION is 
	'The name for the dimension.  This is the table name that represents the dimension.

	This column is limited to 26 characters because the table name is expect to allow room for a primary key of the form table_name + "_KEY".';

	comment on column SAHARA.FORECASTING_DIMENSION.DISPLAY_NAME is 
	'A display name for the dimension to be used in GUI selections, etc.';

	comment on column SAHARA.FORECASTING_DIMENSION.METAMODEL_PATH is 
	'Metamodel path for the dimension to be used in forecasting and report selections.';

	comment on column SAHARA.FORECASTING_DIMENSION.FACT_TABLE_NAME is 
	'Fact table name for the object type to be used in forecasting and report selections.';

	comment on column SAHARA.FORECASTING_DIMENSION.METAMODEL_KEY is 
	'Metamodel key for the dimension to be used in forecasting and report selections.';

	alter table SAHARA.FORECASTING_DIMENSION
	   add constraint PK_FORECASTING_DIMENSION primary key (DIMENSION);

	grant SELECT,REFERENCES on SAHARA.FORECASTING_DIMENSION to SAHARA_APP;

end if;

/*==============================================================*/
/* COLUMN_METADATA_BASE TABLE FORECAST Changes                                 */
/*==============================================================*/
if not exists (select 1 from sys.syscolumns where tname='COLUMN_METADATA_BASE' and cname='IS_FORECAST_ENABLE') then
ALTER TABLE SAHARA.COLUMN_METADATA_BASE ADD       IS_FORECAST_ENABLE char(1)                  null default 'F';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_CLMNMTDTBS_ISFORCSTENABLED' and table_name='COLUMN_METADATA_BASE')
then
alter table SAHARA.COLUMN_METADATA_BASE
   alter IS_FORECAST_ENABLE add constraint C_CLMNMTDTBS_ISFORCSTENABLED check (IS_FORECAST_ENABLE is null or (IS_FORECAST_ENABLE in ('T','F')));
end if;


/*==============================================================*/
/* Table: SERVICEGROUP_HIST_EVENTS_                             */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_HIST_EVENTS_' and creator=user_id('SAHARA_APP')) then

create table SAHARA_APP.SERVICEGROUP_HIST_EVENTS_ 
(
   "DATE"               date                           not null,
   SERVICEGROUP_OID     varchar(128)                   not null,
   PLANNED_FAILOVERS    bigint                         null default 0,
   UNPLANNED_AUTO_FAILOVERS bigint                         null default 0,
   UNPLANNED_MAN_FAILOVERS bigint                         null default 0,
   TOTAL_SUCCESS_FAILOVERS bigint                         null default 0,
   UNRESOLVED_PLANNED_FAILOVERS bigint                         null default 0,
   UNRESOLVED_UNPLANNED_AUTO_FAIL bigint                         null default 0,
   UNRESOLVED_UNPLANNED_MAN_FAILO bigint                         null default 0,
   TOTAL_FAILED_FAILOVERS bigint                         null default 0,
   TIME_ONLINE          bigint                         null default 0,
   TIME_OFFLINE         bigint                         null default 0,
   TIME_PARTIAL         bigint                         null default 0,
   TIME_FAULTED         bigint                         null default 0,
   TIME_TRANSIENTSTATES bigint                         null default 0,
   TIME_NOTKNOWN        bigint                         null default 0
);

comment on table SAHARA_APP.SERVICEGROUP_HIST_EVENTS_ is 
'Store aggregate historical data for uptime analysis and failover analysis.  The data is stored in per day basis.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.PLANNED_FAILOVERS is 
'Failover that user planned and executed with online or switch operation';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNPLANNED_AUTO_FAILOVERS is 
'Cluster performed failover operation without user interference.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNPLANNED_MAN_FAILOVERS is 
'VCS requires user interference to clear the faults for performance and failover operation.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.TOTAL_SUCCESS_FAILOVERS is 
'Sum of planned failovers, unplanned auto failovers, unplaned manual failovers.';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNRESOLVED_PLANNED_FAILOVERS is 
'Failed planned failovers';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.UNRESOLVED_UNPLANNED_AUTO_FAIL is 
'Failed unplanned automatic failovers';

comment on column SAHARA_APP.SERVICEGROUP_HIST_EVENTS_.TOTAL_FAILED_FAILOVERS is 
'Sum of unresolved planned failovers, unresolved unplanned automatic failovers, unresolved unplanned manual failovers.';

alter table SAHARA_APP.SERVICEGROUP_HIST_EVENTS_
   add constraint PK_SERVICEGROUP_HIST_EVENTS_ primary key clustered ("DATE", SERVICEGROUP_OID);
      
end if;


/*==============================================================*/
/* Table: SERVICEGROUP_HIST_EVENTS                              */
/*==============================================================*/

if not exists(select 1 from sys.systable where table_name = 'SERVICEGROUP_HIST_EVENTS' and creator=user_id('SAHARA')) then

create table SAHARA.SERVICEGROUP_HIST_EVENTS 
(
   ROLLUP_BATCH_KEY     numeric(10)                    not null,
   DATES_KEY            numeric(10)                    not null,
   SERVICEGROUP_KEY     numeric(10)                    not null,
   PLANNED_FAILOVERS    bigint                         null default 0,
   UNPLANNED_AUTO_FAILOVERS bigint                         null default 0,
   UNPLANNED_MAN_FAILOVERS bigint                         null default 0,
   TOTAL_SUCCESS_FAILOVERS bigint                         null default 0,
   UNRESOLVED_PLANNED_FAILOVERS bigint                         null default 0,
   UNRESOLVED_UNPLANNED_AUTO_FAIL bigint                         null default 0,
   UNRESOLVED_UNPLANNED_MAN_FAILO bigint                         null default 0,
   TOTAL_FAILED_FAILOVERS bigint                         null default 0,
   TIME_ONLINE          bigint                         null default 0,
   TIME_OFFLINE         bigint                         null default 0,
   TIME_PARTIAL         bigint                         null default 0,
   TIME_FAULTED         bigint                         null default 0,
   TIME_TRANSIENTSTATES bigint                         null default 0,
   TIME_NOTKNOWN        bigint                         null default 0
);

comment on table SAHARA.SERVICEGROUP_HIST_EVENTS is 
'Store aggregate historical data for uptime analysis and failover analysis.  The data is stored in 24 hour granularity.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.ROLLUP_BATCH_KEY is 
'A unique database generated identifier for the roll-up batch used as a surrogate key.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.PLANNED_FAILOVERS is 
'Failover that user planned and executed with online or switch operation';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNPLANNED_AUTO_FAILOVERS is 
'Cluster performed failover operation without user interference.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNPLANNED_MAN_FAILOVERS is 
'VCS requires user interference to clear the faults for performance and failover operation.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.TOTAL_SUCCESS_FAILOVERS is 
'Sum of planned failovers, unplanned auto failovers, unplaned manual failovers.';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNRESOLVED_PLANNED_FAILOVERS is 
'Failed planned failovers';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.UNRESOLVED_UNPLANNED_AUTO_FAIL is 
'Failed unplanned automatic failovers';

comment on column SAHARA.SERVICEGROUP_HIST_EVENTS.TOTAL_FAILED_FAILOVERS is 
'Sum of unresolved planned failovers, unresolved unplanned automatic failovers, unresolved unplanned manual failovers.';

alter table SAHARA.SERVICEGROUP_HIST_EVENTS   
   add constraint PK_SERVICEGROUP_HIST_EVENTS primary key clustered (ROLLUP_BATCH_KEY,DATES_KEY, SERVICEGROUP_KEY);
   
alter table SAHARA.SERVICEGROUP_HIST_EVENTS
   add constraint FK_SERVICEG_FK_SGHIST_SERVICEG foreign key (SERVICEGROUP_KEY)
      references SAHARA.SERVICEGROUP (SERVICEGROUP_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HIST_EVENTS
   add constraint FK_SERVICEG_FK_SGHIST_ROLLUP_B foreign key (ROLLUP_BATCH_KEY)
      references SAHARA.ROLLUP_BATCH (ROLLUP_BATCH_KEY)
      on update restrict
      on delete cascade;

alter table SAHARA.SERVICEGROUP_HIST_EVENTS
   add constraint FK_SERVICEG_FK_SGHIST_DATES foreign key (DATES_KEY)
      references SAHARA.DATES (DATES_KEY)
      on update restrict
      on delete cascade;
      
/*==============================================================*/
/* Index: I_SGHISTEVENT_ROLLUPBATKEY                            */
/*==============================================================*/
create index I_SGHISTEVENT_ROLLUPBATKEY on SAHARA.SERVICEGROUP_HIST_EVENTS (
ROLLUP_BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHISTEVENTS_DATESKEY                               */
/*==============================================================*/
create index I_SGHISTEVENTS_DATESKEY on SAHARA.SERVICEGROUP_HIST_EVENTS (
DATES_KEY ASC
);

/*==============================================================*/
/* Index: I_SGHISTEVENTS_SGKEY                                  */
/*==============================================================*/
create index I_SGHISTEVENTS_SGKEY on SAHARA.SERVICEGROUP_HIST_EVENTS (
SERVICEGROUP_KEY ASC
);

end if;


if not exists (select 1 from sys.syscolumns where tname='BVIEW_IMPL' and cname='APPROVALREQUIRED') then
    ALTER TABLE SAHARA.BVIEW_IMPL ADD APPROVALREQUIRED bit null default 1;
end if;

if not exists (select 1 from sys.syscolumns where tname='BVIEW_IMPL' and cname='ENABLERULE') then
    ALTER TABLE SAHARA.BVIEW_IMPL ADD ENABLERULE bit null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='ROLLUP_BATCH' and cname='IS_DUMMY') then
   ALTER TABLE SAHARA.ROLLUP_BATCH ADD IS_DUMMY char(1) NULL default 'F';
end if;

if not exists (select 1 from sys.syscolumns where tname='ROLLUP_BATCH' and cname='SHOW_STATUS') then
   ALTER TABLE SAHARA.ROLLUP_BATCH ADD SHOW_STATUS char(1) NULL default 'T';
end if;

if not exists (select 1 from sys.syscolumns where tname='ROLLUP_BATCH' and cname='VALID_CLUSTER_DATA') then
   ALTER TABLE SAHARA.ROLLUP_BATCH ADD VALID_CLUSTER_DATA char(1) NULL default 'F';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_VALID_CLUSTER_DAT_ROLLUP_B' and table_name='ROLLUP_BATCH')
then
alter table SAHARA.ROLLUP_BATCH
   alter VALID_CLUSTER_DATA add constraint CKC_VALID_CLUSTER_DAT_ROLLUP_B check (VALID_CLUSTER_DATA is null or (VALID_CLUSTER_DATA in ('T','F')));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_DUMMY_ROLLUP_B' and table_name='ROLLUP_BATCH')
then
alter table SAHARA.ROLLUP_BATCH
   alter IS_DUMMY add constraint CKC_IS_DUMMY_ROLLUP_B check (IS_DUMMY is null or (IS_DUMMY in ('T','F')));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_SHOW_STATUS_ROLLUP_B' and table_name='ROLLUP_BATCH')
then
alter table SAHARA.ROLLUP_BATCH
   alter SHOW_STATUS add constraint CKC_SHOW_STATUS_ROLLUP_B check (SHOW_STATUS is null or (SHOW_STATUS in ('T','F')));
end if;

if not exists (select 1 from sys.syscolumns where tname='BVIEW_APPROVAL_PENDING' and cname='NOOFOBJECTS') then
   ALTER TABLE SAHARA.BVIEW_APPROVAL_PENDING ADD NOOFOBJECTS integer NULL;
end if;

if not exists (select 1 from sys.syscolumns where tname='ROLLUP_SOURCE' and cname='CLUSTER_ROLLUP_MARKER') then
   ALTER TABLE SAHARA.ROLLUP_SOURCE ADD CLUSTER_ROLLUP_MARKER date NULL; 
end if;


/*==============================================================*/
/* Index: I_ROLLUPBATCH_WEBSERVERPORT                           */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ROLLUP_BATCH' and
index_name = 'I_ROLLUPBATCH_WEBSERVERPORT'
) then
create index I_ROLLUPBATCH_WEBSERVERPORT on SAHARA.ROLLUP_BATCH (
WEB_SERVER_PORT ASC
);
end if;

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DBPORT                                  */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ROLLUP_BATCH' and
index_name = 'I_ROLLUPBATCH_DBPORT'
) then
create index I_ROLLUPBATCH_DBPORT on SAHARA.ROLLUP_BATCH (
DB_PORT ASC
);
end if;

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DBPORT is 
'An index on Database Port for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DBPASSWORD                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ROLLUP_BATCH' and
index_name = 'I_ROLLUPBATCH_DBPASSWORD'
) then
create index I_ROLLUPBATCH_DBPASSWORD on SAHARA.ROLLUP_BATCH (
DB_PASSWORD ASC
);
end if;

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DBPASSWORD is 
'An index on Database Password for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_DBPASSWORD2                             */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ROLLUP_BATCH' and
index_name = 'I_ROLLUPBATCH_DBPASSWORD2'
) then
create index I_ROLLUPBATCH_DBPASSWORD2 on SAHARA.ROLLUP_BATCH (
DB_PASSWORD_2 ASC
);
end if;

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_DBPASSWORD2 is 
'An index on Database Password 2 for dimensional queries.';


/*==============================================================*/
/* Index: I_ROLLUPBATCH_LOGARCHIVE                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ROLLUP_BATCH' and
index_name = 'I_ROLLUPBATCH_LOGARCHIVE'
) then
create index I_ROLLUPBATCH_LOGARCHIVE on SAHARA.ROLLUP_BATCH (
LOG_ARCHIVE ASC
);
end if;

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_LOGARCHIVE is 
'An index on Log Archive for dimensional queries.';

/*==============================================================*/
/* Index: I_ROLLUPBATCH_STATUSMESSAGE                           */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='ROLLUP_BATCH' and
index_name = 'I_ROLLUPBATCH_STATUSMESSAGE'
) then
create index I_ROLLUPBATCH_STATUSMESSAGE on SAHARA.ROLLUP_BATCH (
STATUS_MESSAGE ASC
);
end if;

comment on index SAHARA.ROLLUP_BATCH.I_ROLLUPBATCH_STATUSMESSAGE is 
'An index on Status Message for dimensional queries.';


/*==============================================================*/
/* SPVU LICENSING CHANGES		                                */
/*==============================================================*/

if not exists (select 1 from sys.syscolumns where tname='SF_LICENSE_HOST' and cname='SPVU') then
ALTER TABLE SAHARA.SF_LICENSE_HOST ADD       SPVU numeric(10,2)                   null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='SF_LICENSE_HOST' and cname='NCORE') then
ALTER TABLE SAHARA.SF_LICENSE_HOST ADD       NCORE integer                   null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='SF_LICENSE_HOST_' and cname='SPVU') then
ALTER TABLE SAHARA_APP.SF_LICENSE_HOST_ ADD       SPVU numeric(10,2)                   null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='SF_LICENSE_HOST_' and cname='NCORE') then
ALTER TABLE SAHARA_APP.SF_LICENSE_HOST_ ADD       NCORE integer                   null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='SF_DEPLOYED_LICENSES' and cname='TOTAL_SPVU') then
ALTER TABLE SAHARA.SF_DEPLOYED_LICENSES ADD       TOTAL_SPVU numeric(10,2)                   null default 0;
end if;

if not exists (select 1 from sys.syscolumns where tname='SF_DEPLOYED_LICENSES_' and cname='TOTAL_SPVU') then
ALTER TABLE SAHARA_APP.SF_DEPLOYED_LICENSES_ ADD       TOTAL_SPVU numeric(10,2)                   null default 0;
end if;

if exists(select 1 from sys.systable where table_name='SF_LICENSE_HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SF_LICENSE_HOST_CURRENT
end if;

/*==============================================================*/
/* View: SF_LICENSE_HOST_CURRENT                                */
/*==============================================================*/
create view SAHARA.SF_LICENSE_HOST_CURRENT  as
select
   SF_LICENSE_HOST_KEY,
   SF_LICENSE_HOST_OID,
   HOST_KEY,
   CC_STORAGE_URL,
   ID,
   NAME,
   MODEL,
   PROCESSOR_TYPE,
   NUM_PROCESSORS,
   PLATFORM,
   SERVER_PRICE_TIER,
   PROCESSOR_PRICE_TIER,
   NUM_PROCESSOR_PRICE_TIER,
   OS_PRICE_TIER,
   ASSIGNED_PRICE_TIER,
   OS_EDITION,
   OS,
   HAS_ENTERPRISE_EDITION,
   SPVU,
   NCORE
from
   sahara.SF_LICENSE_HOST
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SF_LICENSE_HOST_CURRENT is 
'This view filters the SF_LICENSE_HOST table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

if not exists (select 1 from sys.syscolumns where tname='HOST' and cname='PARENT_VIRTUAL_SERVER_OID') then
ALTER TABLE SAHARA.HOST ADD       PARENT_VIRTUAL_SERVER_OID varchar(512)                   null default '';
end if;

if not exists (select 1 from sys.syscolumns where tname='HOST_' and cname='PARENT_VIRTUAL_SERVER_OID') then
ALTER TABLE SAHARA_APP.HOST_ ADD       PARENT_VIRTUAL_SERVER_OID varchar(512)                   null default '';
end if;


/*==============================================================*/
/* View: HOST_CURRENT                                           */
/*==============================================================*/

if exists(select 1 from sys.systable where table_name='HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.HOST_CURRENT
end if;

create view SAHARA.HOST_CURRENT  as
select HOST_KEY,
       HOST_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       OS_DETAILS,
       OS_TYPE,
       IP_ADDRESS,
       VIRTUAL_SERVER,
       IS_USER_CREATED,
       IS_VIRTUAL,
       IS_VIRTUAL_SERVER,
       IS_CLUSTERED,
       CLUSTER_TYPE,
       CLUSTER_VERSION,
       IS_GUEST_INSTALLED,
       IS_ACTIVE,
       CLUSTER_NAME,
       GUEST_OS_TYPE,
       VIRTUALIZATION_TECHNOLOGY,
       SF_VERSION,
       MH_VERSION,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       VOM_LICENSE_REQUIRED,
       VM_HOST_OID,
	   PARENT_VIRTUAL_SERVER_OID,
	   CONFIGURATION_TYPE
  from SAHARA.HOST
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.HOST_CURRENT is 
'This view filters the HOST table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.HOST_CURRENT to SAHARA_APP;


/*
Drop the obselete constraint which restricts NAS SHARE FACT table to have DM_BATCH foreign key
*/
if exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'FK_NTPPSHRFL_DMBATCH' and table_name='NETAPP_SHARE_FILE')
then
alter table SAHARA.NETAPP_SHARE_FILE drop constraint FK_NTPPSHRFL_DMBATCH;
end if;


--Virtualization Changes and config type changes


if not exists (select 1 from sys.syscolumns where tname='HOST' and cname='CONTROL_GUEST') then
ALTER TABLE SAHARA.HOST ADD       CONTROL_GUEST char(1)     not null default 'F';
end if;

if not exists (select 1 from sys.syscolumns where tname='HOST' and cname='CONFIGURATION_TYPE') then
ALTER TABLE SAHARA.HOST ADD       CONFIGURATION_TYPE   varchar(256)                   null;
end if;

if not exists (select 1 from sys.syscolumns where tname='HOST_' and cname='CONTROL_GUEST') then
ALTER TABLE SAHARA_APP.HOST_ ADD       CONTROL_GUEST char(1)  not null default 'F';
end if;

if not exists (select 1 from sys.syscolumns where tname='HOST_' and cname='CONFIGURATION_TYPE') then
ALTER TABLE SAHARA_APP.HOST_ ADD       CONFIGURATION_TYPE   varchar(256)                   null;
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_HOST_CONTROLGUEST' and table_name='HOST')
then
alter table SAHARA.HOST
   alter CONTROL_GUEST add constraint C_HOST_CONTROLGUEST check (CONTROL_GUEST in ('T','F'));
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'C_HOST_CONTROLGUEST' and table_name='HOST_')
then
alter table SAHARA_APP.HOST_
   alter CONTROL_GUEST add constraint C_HOST_CONTROLGUEST check (CONTROL_GUEST in ('T','F'));
end if;

if exists (select 1 from sys.syscolumns where tname='LUN_' and cname='SUB_TYPE') then
alter table  "SAHARA_APP".LUN_ modify sub_type varchar(64);
end if;

if exists (select 1 from sys.syscolumns where tname='LUN' and cname='SUB_TYPE') then
alter table  "SAHARA".LUN modify sub_type varchar(64);
end if;

commit;

/*
Reset the partial load flag to TRUE to avoid partial load rollup failures
*/
if exists (select 1 from SAHARA.CONFIGURATION where section = 'ROLL-UP' and name = 'ALLOW_PARTIAL_LOADS' )
then 
    call SAHARA.fn_Configuration_InstallBool
    (
        'ROLL-UP',                              -- section
        'ALLOW_PARTIAL_LOADS',                  -- name
        1,										-- value
        'Allow partial data loads to succeed',  -- display name
        -- description
        'A flag that directs the data loading process to retain data sets ' ||
        'in the database that may not be complete.  Normally, such data sets ' ||
        'are discarded and the load is failed.  This usually occurs as the ' ||
        'result of an error in the roll-up collection logic.  ' ||
        'The default value is false.'
    );
end if;

-- insert entries for proxy settings for analytics
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_SERVER' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_SERVER',                           -- name
    -- value
    '',
    'Proxy server for analytics',                           -- display name
    -- description
    'Hostname/ip address of server to be used as proxy ' ||
    ' while sending analytics data ',
	1,
    1
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_USER' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_USER',                           -- name
    -- value
    '',
    'Username for Proxy server ',                           -- display name
    -- description
    'username to be used for proxy server',
	1,
    1
);
end if;
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_PASSWD' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_PASSWD',                           -- name
    -- value
    '',
    'password for proxy server ',                           -- display name
    -- description
    'password for username to be used ' ||
    ' while sending analytics data via proxy server ',
	1,
    1
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_SERVER_PORT' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_SERVER_PORT',                           -- name
    -- value
    '',
    'Port for proxy server ',                           -- display name
    -- description
    'port number on proxy server which should be used' ||
    ' while sending analytics data via proxy server ',
	1,
    1
);
end if;

-- insert entries for telemetry settings 
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TELE' and name = 'VOMES_VERSION' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'TELE',                             -- section
    'VOMES_VERSION',                    -- name
    '',					-- value
    'VOMES Version for Telemetry',      -- display name
    'Stores VOMES version to identify if VOMES install/config logs needs to be collected ', -- description
    1,
    1
);
end if;

/* Resilient Rollup Support */

/*==============================================================*/
/* Table: ETL_LOG                                               */
/*==============================================================*/
create table SAHARA.ETL_LOG 
(
   ETL_LOG_KEY          numeric(10)                    not null default autoincrement,
   BATCH_KEY            numeric(10)                    not null,
   "TIME"               timestamp                      not null,
   PROCEDURE_NAME       varchar(256)                   null,
   SEVERITY             char(1)                        not null,
   "MESSAGE"            long varchar                   not null
);

comment on table SAHARA.ETL_LOG is 
'ETL log generated on the VOM MS database during extraction. Messages are tagged with a timestamp, but queries should use the BATCH_KEY for ordering to guarantee retreival in the order in which messages were created per data source because multiple messages may have the same timestamp.';

comment on column SAHARA.ETL_LOG.ETL_LOG_KEY is 
'A unique database generated key used to order messages.  Using the timestamp alone for ordering is not good enough because mutliple messages can be created with the same timestamp.';

comment on column SAHARA.ETL_LOG.BATCH_KEY is 
'The identifier of the batch that logged the message.';

comment on column SAHARA.ETL_LOG."TIME" is 
'The time at which the message was logged in VOM MS database.';

comment on column SAHARA.ETL_LOG.PROCEDURE_NAME is 
'The (optional) name of the procedure logging the message.';

comment on column SAHARA.ETL_LOG.SEVERITY is 
'An indication of the type of message being logged.  Valid values:

D - debug
I - informational
W - warning
E - error';

comment on column SAHARA.ETL_LOG."MESSAGE" is 
'The text of the message.';

alter table SAHARA.ETL_LOG
   alter SEVERITY add constraint C_ETLLOG_SEVERITY check (SEVERITY in ('D','I','W','E'));

alter table SAHARA.ETL_LOG
   add constraint PK_ETL_LOG primary key (ETL_LOG_KEY);

/*==============================================================*/
/* Index: I_ETLLOG_BATCHKEY                                     */
/*==============================================================*/
create index I_ETLLOG_BATCHKEY on SAHARA.ETL_LOG (
BATCH_KEY ASC
);

/*==============================================================*/
/* Index: I_ETLLOG_PROCEDURENAME                                */
/*==============================================================*/
create index I_ETLLOG_PROCEDURENAME on SAHARA.ETL_LOG (
PROCEDURE_NAME ASC
);

/*==============================================================*/
/* Index: I_ETLLOG_SEVERITY                                     */
/*==============================================================*/
create index I_ETLLOG_SEVERITY on SAHARA.ETL_LOG (
SEVERITY ASC
);

grant SELECT on SAHARA.ETL_LOG to SAHARA_APP;

/*==============================================================*/
/* Table: ETL_LOG_                                              */
/*==============================================================*/
create table SAHARA_APP.ETL_LOG_ 
(
   BATCH_KEY            numeric(10)                    not null,
   "TIME"               timestamp                      not null,
   PROCEDURE_NAME       varchar(256)                   null,
   SEVERITY             char(1)                        not null,
   "MESSAGE"            long varchar                   not null
);

comment on table SAHARA_APP.ETL_LOG_ is 
'This is a staging table for loading data into warehouse table SAHARA.ETL_LOG.  Any data in this table is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  ETL log generated on the VOM MS database during extraction. Messages are tagged with a timestamp, but queries should use the BATCH_KEY for ordering to guarantee retreival in the order in which messages were created per data source because multiple messages may have the same timestamp.';

comment on column SAHARA_APP.ETL_LOG_.BATCH_KEY is 
'The identifier of the batch that logged the message.';

comment on column SAHARA_APP.ETL_LOG_."TIME" is 
'The time at which the message was logged in VOM MS database.';

comment on column SAHARA_APP.ETL_LOG_.PROCEDURE_NAME is 
'The (optional) name of the procedure logging the message.';

comment on column SAHARA_APP.ETL_LOG_.SEVERITY is 
'An indication of the type of message being logged.  Valid values:

D - debug
I - informational
W - warning
E - error';

comment on column SAHARA_APP.ETL_LOG_."MESSAGE" is 
'The text of the message.';

alter table SAHARA_APP.ETL_LOG_
   alter SEVERITY add constraint C_ETLLOG_SEVERITY check (SEVERITY in ('D','I','W','E'));

grant ALTER,DELETE,INSERT,REFERENCES,SELECT,UPDATE on SAHARA_APP.ETL_LOG_ to SAHARA;


/*==============================================================*/
/* Index: I_HOST_APPLICATIONBE                                  */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_APPLICATIONBE'
) then
create index I_HOST_APPLICATIONBE on SAHARA.HOST (
APPLICATION_BE ASC
);

comment on index SAHARA.HOST.I_HOST_APPLICATIONBE is 
'An index on Application Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_APPLICATIONCOMPOSITEBE                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_APPLICATIONCOMPOSITEBE'
) then
create index I_HOST_APPLICATIONCOMPOSITEBE on SAHARA.HOST (
APPLICATION_COMPOSITE_BE ASC
);
end if;

/*==============================================================*/
/* Index: I_HOST_CLUSTERNAME                                    */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_CLUSTERNAME'
) then
create index I_HOST_CLUSTERNAME on SAHARA.HOST (
CLUSTER_NAME ASC
);

comment on index SAHARA.HOST.I_HOST_CLUSTERNAME is 
'An index on Cluster Name for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_CONFIGURATIONTYPE                              */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_CONFIGURATIONTYPE'
) then
create index I_HOST_CONFIGURATIONTYPE on SAHARA.HOST (
CONFIGURATION_TYPE ASC
);

comment on index SAHARA.HOST.I_HOST_CONFIGURATIONTYPE is 
'An index on Configuration Type for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_CONTROLGUEST                                   */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_CONTROLGUEST'
) then
create index I_HOST_CONTROLGUEST on SAHARA.HOST (
CONTROL_GUEST ASC
);

comment on index SAHARA.HOST.I_HOST_CONTROLGUEST is 
'An index on Control Guest for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_ISACTIVE                                       */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_ISACTIVE'
) then
create index I_HOST_ISACTIVE on SAHARA.HOST (
IS_ACTIVE ASC
);

comment on index SAHARA.HOST.I_HOST_ISACTIVE is 
'An index on Is Active for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_MHVERSION                                      */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_MHVERSION'
) then
create index I_HOST_MHVERSION on SAHARA.HOST (
MH_VERSION ASC
);

comment on index SAHARA.HOST.I_HOST_MHVERSION is 
'An index on MH Version for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_ORGANIZATIONBE                                 */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_ORGANIZATIONBE'
) then
create index I_HOST_ORGANIZATIONBE on SAHARA.HOST (
ORGANIZATION_BE ASC
);

comment on index SAHARA.HOST.I_HOST_ORGANIZATIONBE is 
'An index on Organization Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_ORGANIZATIONCOMPOSITEBE                        */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_ORGANIZATIONCOMPOSITEBE'
) then
create index I_HOST_ORGANIZATIONCOMPOSITEBE on SAHARA.HOST (
ORGANIZATION_COMPOSITE_BE ASC
);

comment on index SAHARA.HOST.I_HOST_ORGANIZATIONCOMPOSITEBE is 
'An index on Composite Organization Business Entity for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_PARENTVIRTUALSERVEROID                         */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_PARENTVIRTUALSERVEROID'
) then
create index I_HOST_PARENTVIRTUALSERVEROID on SAHARA.HOST (
PARENT_VIRTUAL_SERVER_OID ASC
);

comment on index SAHARA.HOST.I_HOST_PARENTVIRTUALSERVEROID is 
'An index on Parent Virtual Server OID for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_SFVERSION                                      */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_SFVERSION'
) then
create index I_HOST_SFVERSION on SAHARA.HOST (
SF_VERSION ASC
);

comment on index SAHARA.HOST.I_HOST_SFVERSION is 
'An index on SF Version for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_VMHOSTOID                                      */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_VMHOSTOID'
) then
create index I_HOST_VMHOSTOID on SAHARA.HOST (
VM_HOST_OID ASC
);

comment on index SAHARA.HOST.I_HOST_VMHOSTOID is 
'An index on VM Host Object Identifier for dimensional queries.';
end if;

/*==============================================================*/
/* Index: I_HOST_VOMLICENSEREQUIRED                             */
/*==============================================================*/
if not exists (select 1 from sys.sysindex i
join systable t on i.table_id = t.table_id 
where 
table_name='HOST' and
index_name = 'I_HOST_VOMLICENSEREQUIRED'
) then
create index I_HOST_VOMLICENSEREQUIRED on SAHARA.HOST (
VOM_LICENSE_REQUIRED ASC
);

comment on index SAHARA.HOST.I_HOST_VOMLICENSEREQUIRED is 
'An index on VOM License Required for dimensional queries.';
end if;/*   ---  */

/* Sahara Tables*/

/* Table: DATABASE */
if not exists (select 1 from sys.syscolumns where tname='DATABASES' and cname='IS_BASED_ON_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASES ADD IS_BASED_ON_DB_VM	char(1)	null default 'F';
	comment on column SAHARA.DATABASES.IS_BASED_ON_DB_VM is 
	'A Boolean flag that indicates whether a database is using database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_BASED_ON_DB_VM_DATABASE' and table_name='DATABASES')
then
	alter table SAHARA.DATABASES
	   alter IS_BASED_ON_DB_VM add constraint CKC_IS_BASED_ON_DB_VM_DATABASE check (IS_BASED_ON_DB_VM is null or (IS_BASED_ON_DB_VM in ('T','F')));
end if;

/* Index: I_DATABASES_ISBASEDONTHIN */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_ISBASEDONTHIN')
then
	create index I_DATABASES_ISBASEDONTHIN on SAHARA.DATABASES (
	IS_BASED_ON_THIN ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_ISBASEDONTHIN is 
	'An index on Is Based On Thin for dimensional queries.';   
end if;

/* Index: I_DATABASES_APPLICATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_APPLICATIONBE')
then
	create index I_DATABASES_APPLICATIONBE on SAHARA.DATABASES (
	APPLICATION_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_APPLICATIONBE is 
	'An index on Application Business Entity for dimensional queries.';
end if;

/* Index: I_DATABASES_PPLCTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_PPLCTNCMPSTB')
then
	create index I_DATABASES_PPLCTNCMPSTB on SAHARA.DATABASES (
	APPLICATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_PPLCTNCMPSTB is 
	'An index on Composite Application Business Entity for dimensional queries.'; 
end if;

/* Index: I_DATABASES_ORGANIZATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_ORGANIZATIONBE')
then
	create index I_DATABASES_ORGANIZATIONBE on SAHARA.DATABASES (
	ORGANIZATION_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_ORGANIZATIONBE is 
	'An index on Organization Business Entity for dimensional queries.';  
end if;

/* Index: I_DATABASES_RGNZTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_RGNZTNCMPSTB')
then
	create index I_DATABASES_RGNZTNCMPSTB on SAHARA.DATABASES (
	ORGANIZATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_RGNZTNCMPSTB is 
	'An index on Composite Organization Business Entity for dimensional queries.';
end if;

/* Index: I_DATABASES_ISBASEDONDBVM */
if not exists (select 1 from sys.sysindexes where tname = 'DATABASES' and iname = 'I_DATABASES_ISBASEDONDBVM')
then
	create index I_DATABASES_ISBASEDONDBVM on SAHARA.DATABASES (
	IS_BASED_ON_DB_VM ASC
	);

	comment on index SAHARA.DATABASES.I_DATABASES_ISBASEDONDBVM is 
	'An index on Is Based on DB VM for dimensional queries.';   
end if;

/* Table: DATABASE_ */

if not exists (select 1 from sys.syscolumns where tname='DATABASES_' and cname='IS_BASED_ON_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASES_ ADD IS_BASED_ON_DB_VM	char(1)	null default 'F';
	comment on column SAHARA_APP.DATABASES_.IS_BASED_ON_DB_VM is 
	'A Boolean flag that indicates whether a database is using database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_BASED_ON_DB_VM_DATABASE' and table_name='DATABASES_')
then
	alter table SAHARA_APP.DATABASES_
	alter IS_BASED_ON_DB_VM add constraint CKC_IS_BASED_ON_DB_VM_DATABASE check (IS_BASED_ON_DB_VM is null or (IS_BASED_ON_DB_VM in ('T','F')));
end if;


/* Table: SAHARA.DATABASE_CAPACITY  */
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY' and cname='CAPACITY_FROM_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASE_CAPACITY ADD CAPACITY_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA.DATABASE_CAPACITY.CAPACITY_FROM_DB_VM is 
	'Capacity from database files deriving storage from database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY' and cname='AVAILABLE_FROM_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASE_CAPACITY ADD AVAILABLE_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA.DATABASE_CAPACITY.AVAILABLE_FROM_DB_VM is 
	'Capacity available for database,  based on database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY' and cname='USED_FROM_DB_VM') 
then
	ALTER TABLE SAHARA.DATABASE_CAPACITY ADD USED_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA.DATABASE_CAPACITY.USED_FROM_DB_VM is 
	'Capacity used by database,  based on database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_CAPACITY_FROM_DB__DATABASE' and table_name='DATABASE_CAPACITY')
then
	alter table SAHARA.DATABASE_CAPACITY
	alter CAPACITY_FROM_DB_VM add constraint CKC_CAPACITY_FROM_DB__DATABASE check (CAPACITY_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AVAILABLE_FROM_DB_DATABASE' and table_name='DATABASE_CAPACITY')
then
	alter table SAHARA.DATABASE_CAPACITY
	alter AVAILABLE_FROM_DB_VM add constraint CKC_AVAILABLE_FROM_DB_DATABASE check (AVAILABLE_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_FROM_DB_VM_DATABASE' and table_name='DATABASE_CAPACITY')
then
	alter table SAHARA.DATABASE_CAPACITY
	alter USED_FROM_DB_VM add constraint CKC_USED_FROM_DB_VM_DATABASE check (USED_FROM_DB_VM >= 0);
end if;

/* Table: DATABASE_CAPACITY_  */

if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY_' and cname='CAPACITY_FROM_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASE_CAPACITY_ ADD CAPACITY_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DATABASE_CAPACITY_.CAPACITY_FROM_DB_VM is 
	'Capacity from database files deriving storage from database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY_' and cname='AVAILABLE_FROM_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASE_CAPACITY_ ADD AVAILABLE_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DATABASE_CAPACITY_.AVAILABLE_FROM_DB_VM is 
	'Capacity available for database,  based on database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DATABASE_CAPACITY_' and cname='USED_FROM_DB_VM') 
then
	ALTER TABLE SAHARA_APP.DATABASE_CAPACITY_ ADD USED_FROM_DB_VM numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DATABASE_CAPACITY_.USED_FROM_DB_VM is 
	'Capacity used by database,  based on database volume manager like Oracle ASM.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_CAPACITY_FROM_DB__DATABASE' and table_name='DATABASE_CAPACITY_')
then
	alter table SAHARA_APP.DATABASE_CAPACITY_
	alter CAPACITY_FROM_DB_VM add constraint CKC_CAPACITY_FROM_DB__DATABASE check (CAPACITY_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_AVAILABLE_FROM_DB_DATABASE' and table_name='DATABASE_CAPACITY_')
then
	alter table SAHARA_APP.DATABASE_CAPACITY_
	alter AVAILABLE_FROM_DB_VM add constraint CKC_AVAILABLE_FROM_DB_DATABASE check (AVAILABLE_FROM_DB_VM >= 0);
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_FROM_DB_VM_DATABASE' and table_name='DATABASE_CAPACITY_')
then
	alter table SAHARA_APP.DATABASE_CAPACITY_
	alter USED_FROM_DB_VM add constraint CKC_USED_FROM_DB_VM_DATABASE check (USED_FROM_DB_VM >= 0);
end if;

/* Table: DISKGROUP  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP' and cname='REDUNDANCY_TYPE') 
then
	ALTER TABLE SAHARA.DISKGROUP ADD REDUNDANCY_TYPE varchar(256) null;
	comment on column SAHARA.DISKGROUP.REDUNDANCY_TYPE is 
	'This attribute distinguishes between mirrored disk groups and non-mirrored disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP' and cname='IS_USED_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA.DISKGROUP ADD IS_USED_BY_CLUSTERED_DB char(1) null default 'F';
	comment on column SAHARA.DISKGROUP.IS_USED_BY_CLUSTERED_DB is 
	'A Boolean flag that indicates whether a disk group is utilized by a clustered database like Oracle RAC.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP' and cname='USES_VM_VOLUMES') 
then
	ALTER TABLE SAHARA.DISKGROUP ADD USES_VM_VOLUMES char(1) null default 'F';
	comment on column SAHARA.DISKGROUP.USES_VM_VOLUMES is 
	'A Boolean flag that indicates whether a disk group uses volume managers volume.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_CLUSTE_DISKGROU' and table_name='DISKGROUP')
then
	alter table SAHARA.DISKGROUP
	alter IS_USED_BY_CLUSTERED_DB add constraint CKC_IS_USED_BY_CLUSTE_DISKGROU check (IS_USED_BY_CLUSTERED_DB is null or (IS_USED_BY_CLUSTERED_DB in ('T','F')));
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USES_VM_VOLUMES_DISKGROU' and table_name='DISKGROUP')
then
	alter table SAHARA.DISKGROUP
	alter USES_VM_VOLUMES add constraint CKC_USES_VM_VOLUMES_DISKGROU check (USES_VM_VOLUMES is null or (USES_VM_VOLUMES in ('T','F')));
end if;

/* Index: I_DISKGROUP_DISKGROUPOID */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_DISKGROUPOID')
then
	create index I_DISKGROUP_DISKGROUPOID on SAHARA.DISKGROUP (
	DISKGROUP_OID ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_DISKGROUPOID is 
	'An index on Diskgroup Object Identifier for dimensional queries.';
end if;

/* Index: I_DISKGROUP_FAMILY */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_FAMILY')
then
	create index I_DISKGROUP_FAMILY on SAHARA.DISKGROUP (
	FAMILY ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_FAMILY is 
	'An index on Family for dimensional queries.';
end if;

/* Index: I_DISKGROUP_VERSION */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_VERSION')
then
	create index I_DISKGROUP_VERSION on SAHARA.DISKGROUP (
	VERSION ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_VERSION is 
	'An index on Version for dimensional queries.';
end if;

/* Index: I_DISKGROUP_REDUNDANCYTYPE */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_REDUNDANCYTYPE')
then
	create index I_DISKGROUP_REDUNDANCYTYPE on SAHARA.DISKGROUP (
	REDUNDANCY_TYPE ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_REDUNDANCYTYPE is 
	'An index on Redundancy Type for dimensional queries.';
end if;

/* Index: I_DISKGROUP_SSDBCLSTRDDB */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_SSDBCLSTRDDB')
then
	create index I_DISKGROUP_SSDBCLSTRDDB on SAHARA.DISKGROUP (
	IS_USED_BY_CLUSTERED_DB ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_SSDBCLSTRDDB is 
	'An index on Is Used By Clustered DB for dimensional queries.';
end if;

/* Index: I_DISKGROUP_USESVMVOLUMES */
if not exists (select 1 from sys.sysindexes where tname = 'DISKGROUP' and iname = 'I_DISKGROUP_USESVMVOLUMES')
then
	create index I_DISKGROUP_USESVMVOLUMES on SAHARA.DISKGROUP (
	USES_VM_VOLUMES ASC
	);

	comment on index SAHARA.DISKGROUP.I_DISKGROUP_USESVMVOLUMES is 
	'An index on Uses VM Volumes for dimensional queries.';
end if;

/* Table: SAHARA_APP.DISKGROUP_  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_' and cname='REDUNDANCY_TYPE') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_ ADD REDUNDANCY_TYPE varchar(256) null;
	comment on column SAHARA_APP.DISKGROUP_.REDUNDANCY_TYPE is 
	'This attribute distinguishes between mirrored disk groups and non-mirrored disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_' and cname='IS_USED_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_ ADD IS_USED_BY_CLUSTERED_DB char(1) null default 'F';
	comment on column SAHARA_APP.DISKGROUP_.IS_USED_BY_CLUSTERED_DB is 
	'A Boolean flag that indicates whether a disk group is utilized by a clustered database like Oracle RAC.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_' and cname='USES_VM_VOLUMES') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_ ADD USES_VM_VOLUMES char(1) null default 'F';
	comment on column SAHARA_APP.DISKGROUP_.USES_VM_VOLUMES is 
	'A Boolean flag that indicates whether a disk group uses volume managers volume.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_CLUSTE_DISKGROU' and table_name='DISKGROUP_')
then
	alter table SAHARA_APP.DISKGROUP_
	alter IS_USED_BY_CLUSTERED_DB add constraint CKC_IS_USED_BY_CLUSTE_DISKGROU check (IS_USED_BY_CLUSTERED_DB is null or (IS_USED_BY_CLUSTERED_DB in ('T','F')));
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USES_VM_VOLUMES_DISKGROU' and table_name='DISKGROUP_')
then
	alter table SAHARA_APP.DISKGROUP_
	alter USES_VM_VOLUMES add constraint CKC_USES_VM_VOLUMES_DISKGROU check (USES_VM_VOLUMES is null or (USES_VM_VOLUMES in ('T','F')));
end if;

/* Table: SAHARA.DISKGROUP_CAPACITY  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.OVERHEAD_CAPACITY is 
	'Capacity of disk groups gone into mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.UNKNOWN_CAPACITY is 
	'Capacity of disk groups gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.DB_USED_CAPACITY is 
	'Capacity of database volume managers disk groups used by databases.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY' and cname='USED_CAPACITY_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA.DISKGROUP_CAPACITY ADD USED_CAPACITY_BY_CLUSTERED_DB numeric(22,9) not null default 0;
	comment on column SAHARA.DISKGROUP_CAPACITY.USED_CAPACITY_BY_CLUSTERED_DB is 
	'Capacity of database volume managers disk groups used by clustered databases like Oracle RAC.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERHEAD_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_DISKGROU check (OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_UNKNOWN_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter UNKNOWN_CAPACITY add constraint CKC_UNKNOWN_CAPACITY_DISKGROU check (UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_USED_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter DB_USED_CAPACITY add constraint CKC_DB_USED_CAPACITY_DISKGROU check (DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_CAPACITY_BY__DISKGROU' and table_name='DISKGROUP_CAPACITY')
then
	alter table SAHARA.DISKGROUP_CAPACITY
	alter USED_CAPACITY_BY_CLUSTERED_DB add constraint CKC_USED_CAPACITY_BY__DISKGROU check (USED_CAPACITY_BY_CLUSTERED_DB >= 0);
end if;

/* Table: SAHARA_APP.DISKGROUP_CAPACITY_  */
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.OVERHEAD_CAPACITY is 
	'Capacity of disk groups gone into mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.UNKNOWN_CAPACITY is 
	'Capacity of disk groups gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.DB_USED_CAPACITY is 
	'Capacity of database volume managers disk groups used by databases.';
end if;
if not exists (select 1 from sys.syscolumns where tname='DISKGROUP_CAPACITY_' and cname='USED_CAPACITY_BY_CLUSTERED_DB') 
then
	ALTER TABLE SAHARA_APP.DISKGROUP_CAPACITY_ ADD USED_CAPACITY_BY_CLUSTERED_DB numeric(22,9) not null default 0;
	comment on column SAHARA_APP.DISKGROUP_CAPACITY_.USED_CAPACITY_BY_CLUSTERED_DB is 
	'Capacity of database volume managers disk groups used by clustered databases like Oracle RAC.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_OVERHEAD_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter OVERHEAD_CAPACITY add constraint CKC_OVERHEAD_CAPACITY_DISKGROU check (OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_UNKNOWN_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter UNKNOWN_CAPACITY add constraint CKC_UNKNOWN_CAPACITY_DISKGROU check (UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_USED_CAPACITY_DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter DB_USED_CAPACITY add constraint CKC_DB_USED_CAPACITY_DISKGROU check (DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_USED_CAPACITY_BY__DISKGROU' and table_name='DISKGROUP_CAPACITY_')
then
	alter table SAHARA_APP.DISKGROUP_CAPACITY_
	alter USED_CAPACITY_BY_CLUSTERED_DB add constraint CKC_USED_CAPACITY_BY__DISKGROU check (USED_CAPACITY_BY_CLUSTERED_DB >= 0);
end if;

/* Table: SAHARA.HOST_CAPACITY  */
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_OVERHEAD_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='APPS_ON_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD APPS_ON_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.APPS_ON_DB_VM_CAPACITY is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='APPS_ON_DB_VM_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD APPS_ON_DB_VM_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.APPS_ON_DB_VM_OTHER_CAPACITY is 
	'For shared storage, amount of storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_DB_USED_CAPACITY is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_DB_USED_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_DB_USED_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_DB_USED_OTHER_CAPACITY is 
	'For shared storage, amount of used storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='DB_VM_UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD DB_VM_UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.DB_VM_UNKNOWN_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_OVERHEAD_CAP') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_OVERHEAD_CAP numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_OVERHEAD_CAP is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_APPS_ON_DB_VM_CAP') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_APPS_ON_DB_VM_CAP numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_APPS_ON_DB_VM_CAP is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY' and cname='ADJUSTED_DB_VM_DB_USED_CAP') 
then
	ALTER TABLE SAHARA.HOST_CAPACITY ADD ADJUSTED_DB_VM_DB_USED_CAP numeric(22,9) not null default 0;
	comment on column SAHARA.HOST_CAPACITY.ADJUSTED_DB_VM_DB_USED_CAP is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_CAPACITY_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_CAPACITY add constraint CKC_DB_VM_CAPACITY_HOST_CAP check (DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_USED_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_USED_CAPACITY add constraint CKC_DB_VM_USED_CAPACI_HOST_CAP check (DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_FREE_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_FREE_CAPACITY add constraint CKC_DB_VM_FREE_CAPACI_HOST_CAP check (DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_OVERHEAD_CA_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_OVERHEAD_CAPACITY add constraint CKC_DB_VM_OVERHEAD_CA_HOST_CAP check (DB_VM_OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_CAP_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter APPS_ON_DB_VM_CAPACITY add constraint CKC_APPS_ON_DB_VM_CAP_HOST_CAP check (APPS_ON_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_OTH_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter APPS_ON_DB_VM_OTHER_CAPACITY add constraint CKC_APPS_ON_DB_VM_OTH_HOST_CAP check (APPS_ON_DB_VM_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_CAP_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_DB_USED_CAPACITY add constraint CKC_DB_VM_DB_USED_CAP_HOST_CAP check (DB_VM_DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_OTH_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_DB_USED_OTHER_CAPACITY add constraint CKC_DB_VM_DB_USED_OTH_HOST_CAP check (DB_VM_DB_USED_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_UNKNOWN_CAP_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter DB_VM_UNKNOWN_CAPACITY add constraint CKC_DB_VM_UNKNOWN_CAP_HOST_CAP check (DB_VM_UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_CA_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_CAPACITY add constraint CKC_ADJUSTED_DB_VM_CA_HOST_CAP check (ADJUSTED_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_US_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_USED_CAPACITY add constraint CKC_ADJUSTED_DB_VM_US_HOST_CAP check (ADJUSTED_DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_FR_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_FREE_CAPACITY add constraint CKC_ADJUSTED_DB_VM_FR_HOST_CAP check (ADJUSTED_DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_OV_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_OVERHEAD_CAP add constraint CKC_ADJUSTED_DB_VM_OV_HOST_CAP check (ADJUSTED_DB_VM_OVERHEAD_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_APPS_ON__HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_APPS_ON_DB_VM_CAP add constraint CKC_ADJUSTED_APPS_ON__HOST_CAP check (ADJUSTED_APPS_ON_DB_VM_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_DB_HOST_CAP' and table_name='HOST_CAPACITY')
then
	alter table SAHARA.HOST_CAPACITY
	alter ADJUSTED_DB_VM_DB_USED_CAP add constraint CKC_ADJUSTED_DB_VM_DB_HOST_CAP check (ADJUSTED_DB_VM_DB_USED_CAP >= 0);
end if;

/* Table SAHARA_APP.HOST_CAPACITY_ */

if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_OVERHEAD_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_OVERHEAD_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_OVERHEAD_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='APPS_ON_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD APPS_ON_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.APPS_ON_DB_VM_CAPACITY is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='APPS_ON_DB_VM_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD APPS_ON_DB_VM_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.APPS_ON_DB_VM_OTHER_CAPACITY is 
	'For shared storage, amount of storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_DB_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_DB_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_DB_USED_CAPACITY is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, same storage will be considered across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_DB_USED_OTHER_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_DB_USED_OTHER_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_DB_USED_OTHER_CAPACITY is 
	'For shared storage, amount of used storage in database files on other hosts, when storage is based on database specific volume managers disk groups.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='DB_VM_UNKNOWN_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD DB_VM_UNKNOWN_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.DB_VM_UNKNOWN_CAPACITY is 
	'Amount of storage in database specific volume manager, that has gone into storing meta-data and other book-keeping information.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_CAPACITY is 
	'Amount of storage in database specific volume manager like Oracle ASM on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_USED_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_USED_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_USED_CAPACITY is 
	'Amount of storage used in database specific volume manager on the host. For shared storage, storage will be distributed across hosts';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_FREE_CAPACITY') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_FREE_CAPACITY numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_FREE_CAPACITY is 
	'Amount of storage available for use in database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_OVERHEAD_CAP') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_OVERHEAD_CAP numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_OVERHEAD_CAP is 
	'Amount of storage in database specific volume manager, that has gone in overhead like mirroring. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_APPS_ON_DB_VM_CAP') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_APPS_ON_DB_VM_CAP numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_APPS_ON_DB_VM_CAP is 
	'Amount of database file storage, based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;
if not exists (select 1 from sys.syscolumns where tname='HOST_CAPACITY_' and cname='ADJUSTED_DB_VM_DB_USED_CAP') 
then
	ALTER TABLE SAHARA_APP.HOST_CAPACITY_ ADD ADJUSTED_DB_VM_DB_USED_CAP numeric(22,9) not null default 0;
	comment on column SAHARA_APP.HOST_CAPACITY_.ADJUSTED_DB_VM_DB_USED_CAP is 
	'Amount of used storage in database files that are based on database specific volume manager on the host. For shared storage, storage will be distributed across hosts.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_CAPACITY_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_CAPACITY add constraint CKC_DB_VM_CAPACITY_HOST_CAP check (DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_USED_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_USED_CAPACITY add constraint CKC_DB_VM_USED_CAPACI_HOST_CAP check (DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_FREE_CAPACI_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_FREE_CAPACITY add constraint CKC_DB_VM_FREE_CAPACI_HOST_CAP check (DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_OVERHEAD_CA_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_OVERHEAD_CAPACITY add constraint CKC_DB_VM_OVERHEAD_CA_HOST_CAP check (DB_VM_OVERHEAD_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_CAP_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter APPS_ON_DB_VM_CAPACITY add constraint CKC_APPS_ON_DB_VM_CAP_HOST_CAP check (APPS_ON_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_APPS_ON_DB_VM_OTH_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter APPS_ON_DB_VM_OTHER_CAPACITY add constraint CKC_APPS_ON_DB_VM_OTH_HOST_CAP check (APPS_ON_DB_VM_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_CAP_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_DB_USED_CAPACITY add constraint CKC_DB_VM_DB_USED_CAP_HOST_CAP check (DB_VM_DB_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_DB_USED_OTH_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_DB_USED_OTHER_CAPACITY add constraint CKC_DB_VM_DB_USED_OTH_HOST_CAP check (DB_VM_DB_USED_OTHER_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_DB_VM_UNKNOWN_CAP_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter DB_VM_UNKNOWN_CAPACITY add constraint CKC_DB_VM_UNKNOWN_CAP_HOST_CAP check (DB_VM_UNKNOWN_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_CA_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_CAPACITY add constraint CKC_ADJUSTED_DB_VM_CA_HOST_CAP check (ADJUSTED_DB_VM_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_US_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_USED_CAPACITY add constraint CKC_ADJUSTED_DB_VM_US_HOST_CAP check (ADJUSTED_DB_VM_USED_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_FR_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_FREE_CAPACITY add constraint CKC_ADJUSTED_DB_VM_FR_HOST_CAP check (ADJUSTED_DB_VM_FREE_CAPACITY >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_OV_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_OVERHEAD_CAP add constraint CKC_ADJUSTED_DB_VM_OV_HOST_CAP check (ADJUSTED_DB_VM_OVERHEAD_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_APPS_ON__HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_APPS_ON_DB_VM_CAP add constraint CKC_ADJUSTED_APPS_ON__HOST_CAP check (ADJUSTED_APPS_ON_DB_VM_CAP >= 0);
end if;
if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_ADJUSTED_DB_VM_DB_HOST_CAP' and table_name='HOST_CAPACITY_')
then
	alter table SAHARA_APP.HOST_CAPACITY_
	alter ADJUSTED_DB_VM_DB_USED_CAP add constraint CKC_ADJUSTED_DB_VM_DB_HOST_CAP check (ADJUSTED_DB_VM_DB_USED_CAP >= 0);
end if;

/* table SAHARA.VOLUME */
if not exists (select 1 from sys.syscolumns where tname='VOLUME' and cname='IS_USED_BY_DB_VM') 
then
	ALTER TABLE SAHARA.VOLUME ADD IS_USED_BY_DB_VM char(1) null default 'F';
	comment on column SAHARA.VOLUME.IS_USED_BY_DB_VM is 
	'A Boolean flag that indicates whether a volume is utilized by a database volume manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='VOLUME' and cname='DB_VM_NAME') 
then
	ALTER TABLE SAHARA.VOLUME ADD DB_VM_NAME varchar(256) null;
	comment on column SAHARA.VOLUME.DB_VM_NAME is 
	'This attribute represents name of the database volume manager that is using the volume.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_DB_VM_VOLUME' and table_name='VOLUME')
then
	alter table SAHARA.VOLUME
	alter IS_USED_BY_DB_VM add constraint CKC_IS_USED_BY_DB_VM_VOLUME check (IS_USED_BY_DB_VM is null or (IS_USED_BY_DB_VM in ('T','F')));
end if;

/* Index: I_VOLUME_ISCOMPONENT */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISCOMPONENT')
then
	create index I_VOLUME_ISCOMPONENT on SAHARA.VOLUME (
	IS_COMPONENT ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISCOMPONENT is 
	'An index on Is Component for dimensional queries.';
end if;

/* Index: I_VOLUME_ISBASEDONTHIN */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISBASEDONTHIN')
then
	create index I_VOLUME_ISBASEDONTHIN on SAHARA.VOLUME (
	IS_BASED_ON_THIN ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISBASEDONTHIN is 
	'An index on Is Based On Thin for dimensional queries.';
end if;
/* Index: I_VOLUME_ISEXPORTED */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISEXPORTED')
then
	create index I_VOLUME_ISEXPORTED on SAHARA.VOLUME (
	IS_EXPORTED ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISEXPORTED is 
	'An index on Is Exported for dimensional queries.';
end if;
/* Index: I_VOLUME_VOLUMETYPE */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_VOLUMETYPE')
then
	create index I_VOLUME_VOLUMETYPE on SAHARA.VOLUME (
	VOLUME_TYPE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_VOLUMETYPE is 
	'An index on Volume Type for dimensional queries.';
end if;
/* Index: I_VOLUME_APPLICATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_APPLICATIONBE')
then
	create index I_VOLUME_APPLICATIONBE on SAHARA.VOLUME (
	APPLICATION_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_APPLICATIONBE is 
	'An index on Application Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_PPLCTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_PPLCTNCMPSTB')
then
	create index I_VOLUME_PPLCTNCMPSTB on SAHARA.VOLUME (
	APPLICATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_PPLCTNCMPSTB is 
	'An index on Composite Application Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_ORGANIZATIONBE */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ORGANIZATIONBE')
then
	create index I_VOLUME_ORGANIZATIONBE on SAHARA.VOLUME (
	ORGANIZATION_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ORGANIZATIONBE is 
	'An index on Organization Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_RGNZTNCMPSTB */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_RGNZTNCMPSTB')
then
	create index I_VOLUME_RGNZTNCMPSTB on SAHARA.VOLUME (
	ORGANIZATION_COMPOSITE_BE ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_RGNZTNCMPSTB is 
	'An index on Composite Organization Business Entity for dimensional queries.';
end if;
/* Index: I_VOLUME_ISUSEDBYDBVM */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_ISUSEDBYDBVM')
then
	create index I_VOLUME_ISUSEDBYDBVM on SAHARA.VOLUME (
	IS_USED_BY_DB_VM ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_ISUSEDBYDBVM is 
	'An index on Is Used By DB VM for dimensional queries.';
end if;

/* Index: I_DATABASES_ISBASEDONTHIN */
if not exists (select 1 from sys.sysindexes where tname = 'VOLUME' and iname = 'I_VOLUME_DBVMNAME')
then
	create index I_VOLUME_DBVMNAME on SAHARA.VOLUME (
	DB_VM_NAME ASC
	);

	comment on index SAHARA.VOLUME.I_VOLUME_DBVMNAME is 
	'An index on DB VM Name for dimensional queries.';   
end if;

/* TABLE SAHARA_APP.VOLUME_  */

if not exists (select 1 from sys.syscolumns where tname='VOLUME_' and cname='IS_USED_BY_DB_VM') 
then
	ALTER TABLE SAHARA_APP.VOLUME_ ADD IS_USED_BY_DB_VM char(1) null default 'F';
	comment on column SAHARA_APP.VOLUME_.IS_USED_BY_DB_VM is 
	'A Boolean flag that indicates whether a VOLUME_ is utilized by a database VOLUME_ manager like Oracle ASM.';
end if;
if not exists (select 1 from sys.syscolumns where tname='VOLUME_' and cname='DB_VM_NAME') 
then
	ALTER TABLE SAHARA_APP.VOLUME_ ADD DB_VM_NAME varchar(256) null;
	comment on column SAHARA_APP.VOLUME_.DB_VM_NAME is 
	'This attribute represents name of the database VOLUME_ manager that is using the VOLUME_.';
end if;

if not exists (select 1 from sysconstraint c
join systable t on c.table_object_id = t.object_id where constraint_name = 'CKC_IS_USED_BY_DB_VM_VOLUME_' and table_name='VOLUME_')
then
	alter table SAHARA_APP.VOLUME_
	alter IS_USED_BY_DB_VM add constraint CKC_IS_USED_BY_DB_VM_VOLUME_ check (IS_USED_BY_DB_VM is null or (IS_USED_BY_DB_VM in ('T','F')));
end if;

commit;

/* Sahara PD Dump*/

/* HOST_CAPACITY*/
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_CAPACITY', 'DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_USED_CAPACITY', 'DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_FREE_CAPACITY', 'DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_OVERHEAD_CAPACITY', 'DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_DB_VM_CAPACITY', 'Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_DB_VM_OTHER_CAPACITY', 'Apps on DB VM Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_DB_USED_CAPACITY', 'DB VM DB Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_DB_USED_OTHER_CAPACITY', 'DB VM DB Used Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_UNKNOWN_CAPACITY', 'DB VM Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_CAPACITY', 'Adjusted DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_USED_CAPACITY', 'Adjusted DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_FREE_CAPACITY', 'Adjusted DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_OVERHEAD_CAP', 'Adjusted DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_APPS_ON_DB_VM_CAP', 'Adjusted Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_DB_USED_CAP', 'Adjusted DB VM DB Used Capacity');

/* HOST_CAPACITY_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_CAPACITY', 'DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_USED_CAPACITY', 'DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_FREE_CAPACITY', 'DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_OVERHEAD_CAPACITY', 'DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_DB_VM_CAPACITY', 'Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_DB_VM_OTHER_CAPACITY', 'Apps on DB VM Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_DB_USED_CAPACITY', 'DB VM DB Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_DB_USED_OTHER_CAPACITY', 'DB VM DB Used Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_UNKNOWN_CAPACITY', 'DB VM Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_CAPACITY', 'Adjusted DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_USED_CAPACITY', 'Adjusted DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_FREE_CAPACITY', 'Adjusted DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_OVERHEAD_CAP', 'Adjusted DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_APPS_ON_DB_VM_CAP', 'Adjusted Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_DB_USED_CAP', 'Adjusted DB VM: DB Used Capacity');

/* DATABASE_CAPACITY */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'CAPACITY_FROM_DB_VM', 'Capacity From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'AVAILABLE_FROM_DB_VM', 'Available From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'USED_FROM_DB_VM', 'Used From DB VM');

/* DATABASE_CAPACITY_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'CAPACITY_FROM_DB_VM', 'Capacity From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'AVAILABLE_FROM_DB_VM', 'Available From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'USED_FROM_DB_VM', 'Used From DB VM');

/* DATABASES */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');
/* DATABASES_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');

/* VOLUME */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_USED_BY_DB_VM', 'Is Used By DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'DB_VM_NAME', 'DB VM Name');

/* VOLUME_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_USED_BY_DB_VM', 'Is Used by DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'DB_VM_NAME', 'DB VM Name');

/* DISKGROUP */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'IS_USED_BY_CLUSTERED_DB', 'Is Used By Clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'USES_VM_VOLUMES', 'Uses VM Volumes');
/* DISKGROUP_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'IS_USED_BY_CLUSTERED_DB', 'Is Used by clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'USES_VM_VOLUMES', 'Uses VM Volumes');

/* DISKGROUP_CAPACITY */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'OVERHEAD_CAPACITY', 'Overhead capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'DB_USED_CAPACITY', 'DB Used capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'USED_CAPACITY_BY_CLUSTERED_DB', 'Used Capacity By Clustered DB');
/* DISKGROUP_CAPACITY_ */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'OVERHEAD_CAPACITY', 'Overhead capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'DB_USED_CAPACITY', 'DB Used capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'USED_CAPACITY_BY_CLUSTERED_DB', 'Used Capacity By Clustered DB');

/* DATABASES_CURRENT */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');

/* VOLUME_CURRENT */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_USED_BY_DB_VM', 'Is Used By DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'DB_VM_NAME', 'DB VM Name');

/* DISKGROUP_CURRENT */
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'IS_USED_BY_CLUSTERED_DB', 'Is Used By Clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'USES_VM_VOLUMES', 'Uses VM Volumes');

commit;

/* Sahara views */

/* View: DATABASE_CURRENT*/
if exists(select 1 from sys.systable where table_name='DATABASES_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.DATABASES_CURRENT
end if;

create view SAHARA.DATABASES_CURRENT  as
select DATABASES_KEY,
       HOST_KEY,
       DATABASES_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       INSTANCE_DISPLAY_NAME,
       DATABASE_TYPE,
       DATABASE_VERSION,
       DATABASE_STATE,
       DISCOVERY_STATE,
       IS_CLUSTERED,
       IS_RAC_AWARE,
       IS_BASED_ON_THIN,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_BASED_ON_DB_VM
  from SAHARA.DATABASES
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.DATABASES_CURRENT is 
'This view filters the DATABASES table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.DATABASES_CURRENT to SAHARA_APP;

/* View: DISKGROUP_CURRENT */
if exists(select 1 from sys.systable where table_name='DISKGROUP_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.DISKGROUP_CURRENT
end if;

create view SAHARA.DISKGROUP_CURRENT (DISKGROUP_KEY, DISKGROUP_OID, HOST_KEY, CC_STORAGE_URL, DISPLAY_NAME, FAMILY, VERSION, TYPE, IS_SHARED, IS_BASED_ON_DIRECT_ATTACHED, IS_BASED_ON_THIN, STATE, APPLICATION_BE, APPLICATION_COMPOSITE_BE, ORGANIZATION_BE, ORGANIZATION_COMPOSITE_BE, REDUNDANCY_TYPE, IS_USED_BY_CLUSTERED_DB, USES_VM_VOLUMES) as
select DISKGROUP_KEY,
       DISKGROUP_OID,       
       HOST_KEY,
       CC_STORAGE_URL,
       DISPLAY_NAME,       
       FAMILY,
       VERSION,
       TYPE,
       IS_SHARED,
       IS_BASED_ON_DIRECT_ATTACHED,
       IS_BASED_ON_THIN,
       STATE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       REDUNDANCY_TYPE,
       IS_USED_BY_CLUSTERED_DB,
       USES_VM_VOLUMES
  from SAHARA.DISKGROUP 
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.DISKGROUP_CURRENT is 
'This view filters the DISKGROUP table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.DISKGROUP_CURRENT to SAHARA_APP;

/* View: VOLUME_CURRENT */
if exists(select 1 from sys.systable where table_name='VOLUME_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.VOLUME_CURRENT
end if;

create view SAHARA.VOLUME_CURRENT  as
select VOLUME_KEY,
       VOLUME_OID,
       CC_STORAGE_URL,
       DISKGROUP_KEY,
       HOST_KEY,
       VOLUME_NAME,
       VOLUME_NAME AS DISPLAY_NAME,
       VM_VERSION,
       VM_TYPE,
       DISK_GROUP_NAME,
       VOLUME_LAYOUT,
       IS_VCS_CVM,
       IS_USED_BY_FILESYSTEM,
       IS_USED_BY_APPLICATION,
       IS_DIRECT_ATTACHED,
       IS_COMPONENT,
       IS_BASED_ON_THIN,
       IS_EXPORTED,
       VOLUME_TYPE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_USED_BY_DB_VM,
       DB_VM_NAME
  from SAHARA.VOLUME
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.VOLUME_CURRENT is 
'This view filters the VOLUME table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.VOLUME_CURRENT to SAHARA_APP;

commit;

begin
                message '-- configuring data set DISKGROUP_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DG_DIM',                       -- name
        'DIMENSION',                        -- stereotype
        'DISKGROUP',                           -- destination table
        'DISKGROUP_',                          -- staging table
                                null,


        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
                                'diskgroup_dim.csv',
                                'vrts_vxvm_diskgroup'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'DISKGROUP_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'DISKGROUP_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.DISKGROUP_OID || @URL2');
end;

commit;

begin
                message '-- configuring data set VOLUME_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'VOLUME_DIM',                       -- name
        'DIMENSION',                        -- stereotype
        'VOLUME',                           -- destination table
        'VOLUME_',                          -- staging table
                                null,


--        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
--          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
--          'join SAHARA.HOST_CURRENT ' ||
--            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID'

        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY, DISKGROUP_CURRENT.DISKGROUP_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID ' ||
          'left outer join SAHARA.DISKGROUP_CURRENT ' ||
            'on SRC.DISKGROUP_OID = DISKGROUP_CURRENT.DISKGROUP_OID',
                                'volume_dim.csv',
                                'vrts_vxvm_volume'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'VOLUME_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'VOLUME_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.VOLUME_OID || @URL2');
end;

commit;

begin
                message '-- configuring data set DB_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DB_DIM',                           -- name
        'DIMENSION',                        -- stereotype
        'DATABASES',                        -- destination table
        'DATABASES_',                       -- staging table
                                null,

        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
                                'database_dim.csv',
                                'vrts_dbed_basic'
    );
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'DATABASES_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'DATABASES_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.DATABASES_OID || @URL2');
end;

commit;

call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DG_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'VOLUME_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DB_DIM');

commit;
/*==============================================================*/
/* DBMS name:      Sybase SQL Anywhere 11                       */
/* Created on:     25/09/2012 4:13:22 PM                        */
/*==============================================================*/


if exists(select 1 from sys.systable where table_name='APP_GROUP_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.APP_GROUP_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='ARRAY_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ARRAY_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='ARRAY_STORAGE_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.ARRAY_STORAGE_
end if;

if exists(select 1 from sys.systable where table_name='BVIEW' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.BVIEW
end if;

if exists(select 1 from sys.systable where table_name='BVIEW_HIERARCHY' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.BVIEW_HIERARCHY
end if;

if exists(select 1 from sys.systable where table_name='BVIEW_SUPPORTED_DIMENSION' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.BVIEW_SUPPORTED_DIMENSION
end if;

if exists(select 1 from sys.systable where table_name='CLUSTER_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.CLUSTER_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='CLUSTER_HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.CLUSTER_HOST_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='CLUSTER_RCLUSTER_RELATION' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.CLUSTER_RCLUSTER_RELATION
end if;

if exists(select 1 from sys.systable where table_name='CLUSTER_REMOTECLUSTER_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.CLUSTER_REMOTECLUSTER_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='COLUMN_METADATA' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.COLUMN_METADATA
end if;

if exists(select 1 from sys.systable where table_name='DATABASES_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.DATABASES_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='DISKGROUP_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.DISKGROUP_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='ER_LICENSE_DISPLAY' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ER_LICENSE_DISPLAY
end if;

if exists(select 1 from sys.systable where table_name='ER_LICENSE_USAGE' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ER_LICENSE_USAGE
end if;

if exists(select 1 from sys.systable where table_name='FILESYSTEM_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.FILESYSTEM_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='FILE_CATEGORY_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.FILE_CATEGORY_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.HOST_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='LUN_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.LUN_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='MESSAGE_LOG_SUMMARY' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.MESSAGE_LOG_SUMMARY
end if;

if exists(select 1 from sys.systable where table_name='NAS_FILESYSTEM_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NAS_FILESYSTEM_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='NAS_QUOTA_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NAS_QUOTA_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_FILE_FCAT_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_FILE_FCAT_
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_QTREE_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NETAPP_QTREE_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_QTREE_FILE_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_QTREE_FILE_
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_QTREE_FILE_FCAT_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_QTREE_FILE_FCAT_
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_SHARE_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NETAPP_SHARE_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_SHARE_FILE_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_SHARE_FILE_
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_SHARE_FILE_FCAT_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_SHARE_FILE_FCAT_
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_VOLUME_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.NETAPP_VOLUME_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_VOLUME_FILE_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_VOLUME_FILE_
end if;

if exists(select 1 from sys.systable where table_name='NETAPP_VOLUME_FILE_FCAT_' and table_type='VIEW' and creator=user_id('SAHARA_APP')) then
   drop view SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_BATCH_READ' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_BATCH_READ
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_DATA_SET_METADATA' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_DATA_SET_METADATA
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_DATA_SET_STATUS' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_DATA_SET_STATUS
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_SCHEDULE_DATETIME' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_SCHEDULE_DATETIME
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_SOURCE_LATEST_BATCH' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_SOURCE_LATEST_BATCH
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_SOURCE_LATEST_SUCCESS' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_SOURCE_LATEST_SUCCESS
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_SOURCE_READ' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_SOURCE_READ
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_SOURCE_TIME' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_SOURCE_TIME
end if;

if exists(select 1 from sys.systable where table_name='ROLLUP_STATUS' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.ROLLUP_STATUS
end if;

if exists(select 1 from sys.systable where table_name='RSOURCE_TYPE_SUPPORTED_DS' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.RSOURCE_TYPE_SUPPORTED_DS
end if;

if exists(select 1 from sys.systable where table_name='SERVICEGROUP_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SERVICEGROUP_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='SERVICEGROUP_HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SERVICEGROUP_HOST_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='SF_LICENSE_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SF_LICENSE_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='SF_LICENSE_DEPLOYMENT_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SF_LICENSE_DEPLOYMENT_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='SF_LICENSE_HOST_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SF_LICENSE_HOST_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='SF_PRODUCT_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SF_PRODUCT_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='SRC_CUSTOM_ATTRIBUTE' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SRC_CUSTOM_ATTRIBUTE
end if;

if exists(select 1 from sys.systable where table_name='SWITCH_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.SWITCH_CURRENT
end if;

if exists(select 1 from sys.systable where table_name='TABLE_METADATA' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.TABLE_METADATA
end if;

if exists(select 1 from sys.systable where table_name='UNUSED_SRC_CUSTOM_ATTRIBUTE' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.UNUSED_SRC_CUSTOM_ATTRIBUTE
end if;

if exists(select 1 from sys.systable where table_name='VIEW_BROKER_INFO' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.VIEW_BROKER_INFO
end if;

if exists(select 1 from sys.systable where table_name='VIEW_SECURITY_GROUP' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.VIEW_SECURITY_GROUP
end if;

if exists(select 1 from sys.systable where table_name='VIEW_SECURITY_ROLE' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.VIEW_SECURITY_ROLE
end if;

if exists(select 1 from sys.systable where table_name='VOLUME_CURRENT' and table_type='VIEW' and creator=user_id('SAHARA')) then
   drop view SAHARA.VOLUME_CURRENT
end if;

/*==============================================================*/
/* View: APP_GROUP_CURRENT                                      */
/*==============================================================*/
create view SAHARA.APP_GROUP_CURRENT  as
select APP_GROUP_KEY,
       APP_GROUP_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       DESCRIPTION
  from SAHARA.APP_GROUP
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.APP_GROUP_CURRENT is 
'This view filters the APP_GROUP table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

grant SELECT on SAHARA.APP_GROUP_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: ARRAY_CURRENT                                          */
/*==============================================================*/
create view SAHARA.ARRAY_CURRENT  as
select ARRAY_KEY,
       PARENT_ARRAY_KEY,
       ARRAY_OID,
       PARENT_ARRAY_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       ADVISORY_DISPLAY_NAME,
       VENDOR,
       FAMILY,
       TYPE,
       DISCOVERY_STATE,
       FIRMWARE_VERSION,
       MODEL,
       SERIAL_NUMBER,
       SNMP_IP_ADDRESS,
       NIS_DOMAIN,
       NIS_SERVER,     
       IS_AUTO_DISCOVERED,
       IS_VIRTUALIZER,
       IS_NIS_ENABLED,
       VOM_DISCOVER_TYPE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       SUB_TYPE,	
       IP_ADDRESSES,
       CLUSTER_ARRAY_KEY,
       CLUSTER_ARRAY_NAME,
       CLUSTER_ARRAY_OID,
       FILER_TYPE,
       PARENT_ARRAY_NAME
  from SAHARA.ARRAY
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.ARRAY_CURRENT is 
'This view filters the ARRAY table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

grant SELECT on SAHARA.ARRAY_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: ARRAY_STORAGE_                                         */
/*==============================================================*/
create view SAHARA_APP.ARRAY_STORAGE_ (ARRAY_OID, TOTAL_PHYSICAL, V_PHYSICAL_EXTERNAL, LOCAL_LOGICAL, V_VIRTUAL_LOGICAL, LOGICAL_AVAILABLE, ALLOCATED, A_ALLOCATED_TO_HOSTS_DIRECTLY, A_ALLOCATED_TO_VIRTUALIZER, A_ALLOC_HOST_VIA_VIRTUALIZER, V_LOCAL_LOGICAL_ALLOCATED, V_VIRTUAL_LOGICAL_ALLOCATED, UNALLOCATED, A_UNALLOCATED_VIA_VIRUALIZER, V_LOCAL_LOGICAL_UNALLOCATED, V_VIRTUAL_LOGICAL_UNALLOCATED, CLAIMED_BY_HOSTS, REPLICATION, RAID_OVERHEAD, ADMINISTRATIVE, SNAPSHOT_RESERVE, WAFL_SPARE, DAS_CAPACITY, NAS_USED_CAPACITY, UNCONFIGURED_CAPACITY, INTERNAL_PHYSICAL, LOGICAL_UNKNOWN, TOTAL_UNCLAIMED, UNDISCOVERED_ALLOCATED, PERCENT_ALLOCATED_HOST_CLAIMED, PERCENT_CONFIGURED, PERCENT_CONFIGURED_ALLOCATED, PERCENT_CONFIGURED_AVAILABLE, PERCENT_CONFIGURED_OVERHEAD, PERCENT_CONFIGURED_UNDISCOVERD, PERCENT_UNCONFIGURED, CONSUMPTION_ASSIGNED, CONSUMPTION_UNKNOWN, HOST_USAGE_APPS_ON_RAW, HOST_USAGE_FS_UN_USED, HOST_USAGE_FS_USED, HOST_USAGE_FS_AUSED_APPS, HOST_USAGE_FS_NAUSED_NON_APPS, HOST_USAGE_OTHER, APPL_USED_CAPACITY, APPL_UN_USED_CAPACITY, THINAWARE_CLAIMED_BY_HOST, THINAWARE_TOTAL_UNCLAIMED, THINAWARE_ALLOCATED_TOHOST, THINAWARE_ALC_THGVIRTUALIZER, THINAWARE_UNDISC_ALLOCATED, NA_VOLUME_CAPACITY, NA_VOLUME_USED_CAPACITY, NA_VOLUME_AVAILABLE_CAPACITY, NA_QTREES, NA_SHARES, NA_VOLUMES, NA_FILESYSTEMS, DM_BATCH_ID, NA_CONSUMPTION, NA_FILES, NA_FOLDERS, NA_USERS, SNAPSHOT_USED, OVERWRITE_RESERVE, NETAPP_REDUNDANT_CAPACITY, NUMBER_OF_STORAGE_POOLS, WAFL_SPARE_NEW) as
select ARRAY.ARRAY_OID,
       ARRAY.TOTAL_PHYSICAL,
       ARRAY.V_PHYSICAL_EXTERNAL,
       ARRAY.LOCAL_LOGICAL,
       ARRAY.V_VIRTUAL_LOGICAL,
       ARRAY.LOGICAL_AVAILABLE,
       ARRAY.ALLOCATED,
       ARRAY.A_ALLOCATED_TO_HOSTS_DIRECTLY,
       ARRAY.A_ALLOCATED_TO_VIRTUALIZER,
       ARRAY.A_ALLOC_HOST_VIA_VIRTUALIZER,
       ARRAY.V_LOCAL_LOGICAL_ALLOCATED,
       ARRAY.V_VIRTUAL_LOGICAL_ALLOCATED,
       ARRAY.UNALLOCATED,
       ARRAY.A_UNALLOCATED_VIA_VIRUALIZER,
       ARRAY.V_LOCAL_LOGICAL_UNALLOCATED,
       ARRAY.V_VIRTUAL_LOGICAL_UNALLOCATED,
       ARRAY.CLAIMED_BY_HOSTS,
       ARRAY.REPLICATION,
       ARRAY.RAID_OVERHEAD,
       ARRAY.ADMINISTRATIVE,
       ARRAY.SNAPSHOT_RESERVE,
       ARRAY.WAFL_SPARE,
       ARRAY.DAS_CAPACITY,
       ARRAY.NAS_USED_CAPACITY,
       ARRAY.UNCONFIGURED_CAPACITY,
       ARRAY.INTERNAL_PHYSICAL,
       ARRAY.LOGICAL_UNKNOWN,
       ARRAY.TOTAL_UNCLAIMED,
       ARRAY.UNDISCOVERED_ALLOCATED,
       ARRAY.PERCENT_ALLOCATED_HOST_CLAIMED,
       ARRAY.PERCENT_CONFIGURED,
       ARRAY.PERCENT_CONFIGURED_ALLOCATED,
       ARRAY.PERCENT_CONFIGURED_AVAILABLE,
       ARRAY.PERCENT_CONFIGURED_OVERHEAD,
       ARRAY.PERCENT_CONFIGURED_UNDISCOVERD,
       ARRAY.PERCENT_UNCONFIGURED,
       ARRAY.CONSUMPTION_ASSIGNED,
  	   ARRAY.CONSUMPTION_UNKNOWN,
  	   ARRAY.HOST_USAGE_APPS_ON_RAW,
  	   ARRAY.HOST_USAGE_FS_UN_USED,
  	   ARRAY.HOST_USAGE_FS_USED,
  	   ARRAY.HOST_USAGE_FS_AUSED_APPS,
  	   ARRAY.HOST_USAGE_FS_NAUSED_NON_APPS,
  	   ARRAY.HOST_USAGE_OTHER,
  	   ARRAY.APPL_USED_CAPACITY,
  	   ARRAY.APPL_UN_USED_CAPACITY,
       ARRAY.THINAWARE_CLAIMED_BY_HOST,	
       ARRAY.THINAWARE_TOTAL_UNCLAIMED,	
       ARRAY.THINAWARE_ALLOCATED_TOHOST,
       ARRAY.THINAWARE_ALC_THGVIRTUALIZER,
       ARRAY.THINAWARE_UNDISC_ALLOCATED,
       ARRAY.NA_VOLUME_CAPACITY,
       ARRAY.NA_VOLUME_USED_CAPACITY,
       ARRAY.NA_VOLUME_AVAILABLE_CAPACITY,
       ARRAY.NA_QTREES,
       ARRAY.NA_SHARES,
       ARRAY.NA_VOLUMES,
       ARRAY.NA_FILESYSTEMS,
       isnull(NADM.DM_BATCH_ID,0) as DM_BATCH_ID,
       isnull(NADM.NA_CONSUMPTION,0) as NA_CONSUMPTION,
       isnull(NADM.NA_FILES,0) as NA_FILES,
       isnull(NADM.NA_FOLDERS,0) as NA_FOLDERS,
       isnull(NADM.NA_USERS,0) as NA_USERS,
       ARRAY.SNAPSHOT_USED,
       ARRAY.OVERWRITE_RESERVE,
       ARRAY.NETAPP_REDUNDANT_CAPACITY,
       ARRAY.NUMBER_OF_STORAGE_POOLS,
       ARRAY.WAFL_SPARE_NEW
  from SAHARA_APP.ARRAY_STORAGE_ARRAY_ ARRAY
  left outer join SAHARA_APP.ARRAY_STORAGE_NETAPP_HAL_ NAHAL on NAHAL.ARRAY_OID = ARRAY.ARRAY_OID
  left outer join SAHARA_APP.ARRAY_STORAGE_NETAPP_DM_ NADM on NADM.DM_ID = NAHAL.DM_ID;

comment on view SAHARA_APP.ARRAY_STORAGE_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.ARRAY_STORAGE.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for array storage measures.  Each rows represents one sampling of values for an array.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.ARRAY_STORAGE_ to SAHARA;

/*==============================================================*/
/* View: BVIEW                                                  */
/*==============================================================*/
create view SAHARA.BVIEW  as
select BVIEW_IMPL.BVIEW_KEY,
       BVIEW_IMPL.NAME,
       BVIEW_NODE.DISPLAY_NAME,
       BVIEW_NODE.NOTE,
       BVIEW_IMPL.VIEW_CREATION_REQUIRED
  from SAHARA.BVIEW_IMPL
  join SAHARA.BVIEW_NODE
    on BVIEW_IMPL.BVIEW_KEY = BVIEW_NODE.BVIEW_NODE_KEY;

comment on view SAHARA.BVIEW is 
'This view simplifies access to the attributes of a business view as a whole.  A business view as a whole is represented largely by its root node, which is special in the BVIEW_NODE table.  The table BVIEW_IMPL adds additional attributes for the business view as a whole and supplements the root node.  This view combines the two.

See the comments on table BVIEW_NODE for further explanation.';

grant SELECT on SAHARA.BVIEW to SAHARA_APP;

/*==============================================================*/
/* View: BVIEW_HIERARCHY                                        */
/*==============================================================*/
create view SAHARA.BVIEW_HIERARCHY  as
select A.BVIEW_NODE_KEY as ANCESTOR_KEY,
       A.PARENT_BVIEW_NODE_KEY as ANCESTOR_PARENT_KEY,
       A.BVIEW_KEY as ANCESTOR_BVIEW_KEY,
       A.DISPLAY_NAME as ANCESTOR_DISPLAY_NAME,
       A."LEVEL" as ANCESTOR_LEVEL,
       D.BVIEW_NODE_KEY as DESCENDENT_KEY,
       D.PARENT_BVIEW_NODE_KEY as DESCENDENT_PARENT_KEY,
       D.BVIEW_KEY as DESCENDENT_BVIEW_KEY,
       D.DISPLAY_NAME as DESCENDENT_DISPLAY_NAME,
       D."LEVEL" as DESCENDENT_LEVEL
  from SAHARA.BVIEW_NODE A
  join SAHARA.BVIEW_NODE D
    on D."LEFT" between A."LEFT" and A."RIGHT"
   and A.BVIEW_NODE_KEY <> D.BVIEW_NODE_KEY;

comment on view SAHARA.BVIEW_HIERARCHY is 
'This view provides a simple way to visualize the relationships between nodes in the business view trees.';

grant SELECT on SAHARA.BVIEW_HIERARCHY to SAHARA_APP;

/*==============================================================*/
/* View: BVIEW_SUPPORTED_DIMENSION                              */
/*==============================================================*/
create view SAHARA.BVIEW_SUPPORTED_DIMENSION  as
select BVIEW_DIMENSION.DIMENSION,
       BVIEW_DIMENSION.ALIAS,
       BVIEW_DIMENSION.DISPLAY_NAME as DIMENSION_DISPLAY_NAME,
       BVIEW.BVIEW_KEY,
       BVIEW.NAME as BVIEW_NAME,
       BVIEW.DISPLAY_NAME BVIEW_DISPLAY_NAME,
       BVIEW.NOTE as BVIEW_NOTE
  from SAHARA.BVIEW_DIMENSION_BVIEW
  join SAHARA.BVIEW_DIMENSION
    on BVIEW_DIMENSION_BVIEW.DIMENSION = BVIEW_DIMENSION.DIMENSION
  join SAHARA.BVIEW
    on BVIEW_DIMENSION_BVIEW.BVIEW_KEY = BVIEW.BVIEW_KEY;

comment on view SAHARA.BVIEW_SUPPORTED_DIMENSION is 
'This view provides a simplified way to visualize with which dimensions a business view may have object associations.';

grant SELECT on SAHARA.BVIEW_SUPPORTED_DIMENSION to SAHARA_APP;

/*==============================================================*/
/* View: CLUSTER_CURRENT                                        */
/*==============================================================*/
create view SAHARA.CLUSTER_CURRENT  as
select
   CLUSTER_KEY,
   CLUSTER_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   CLUSTER_TYPE,
   CLUSTER_CATEGORY,
   VERSION,
   PLATFORM,
   USE_FENCE,
   IS_GLOBAL,
   SECURECLUS,
   REMOTECLUSTERS,
   APPLICATION_BE,
   APPLICATION_COMPOSITE_BE,
   ORGANIZATION_BE,
   ORGANIZATION_COMPOSITE_BE
from
   SAHARA.CLUSTER
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.CLUSTER_CURRENT is 
'This view filters the CLUSTER table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.CLUSTER_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: CLUSTER_HOST_CURRENT                                   */
/*==============================================================*/
create view SAHARA.CLUSTER_HOST_CURRENT (CLUSTER_HOST_KEY, CLUSTER_KEY, HOST_KEY, CLUSTER_OID, HOST_OID, CC_STORAGE_URL, DISPLAY_NAME) as
select
   CLUSTER_HOST_KEY,
   CLUSTER_KEY,
   HOST_KEY,
   CLUSTER_OID,
   HOST_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME
from
   SAHARA.CLUSTER_HOST
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

grant SELECT on SAHARA.CLUSTER_HOST_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: CLUSTER_RCLUSTER_RELATION                              */
/*==============================================================*/
create view SAHARA.CLUSTER_RCLUSTER_RELATION  as
select 
    cobj_rcobj.c1_cluster_key as cluster_key, 
    cobj_rcobj.c2_cluster_key as remotecluster_key
from
(
select 
       c1.cluster_key as c1_cluster_key, 
       substr(c1.cluster_oid, 0, charindex('-', c1.cluster_oid)) as c1_cluster_oid, 
       c1.remoteclusters as c1_remotecluster_list ,
       c2.cluster_key as c2_cluster_key, 
       substr(c2.cluster_oid, 0, charindex('-', c2.cluster_oid)) as c2_cluster_oid, 
       c2.remoteclusters as c2_remotecluster_list
from 
    sahara.cluster_remotecluster as c2rc
    join sahara.cluster as c1
    on c2rc.cluster_oid = c1.cluster_oid 
    join sahara.cluster as c2
    on substr(c2rc.remotecluster_oid, 0, charindex('@', c2rc.remotecluster_oid)) = substr(c2.cluster_oid, 0, charindex('-', c2.cluster_oid))
) as cobj_rcobj
where charindex(c1_cluster_oid, c2_remotecluster_list) <> 0
and charindex(c2_cluster_oid, c1_remotecluster_list) <> 0;

comment on view SAHARA.CLUSTER_RCLUSTER_RELATION is 
'This view show valid relationship of cluster to remote cluster only if 
- it finds both cluster and remotecluster objects rolled up in the Cluster Dimension Table.
and
- It successfully cross checking  c1 exists in list of c2''s remotecluster list.
 and c2 exists in the list of c1''s remotecluster lists.

';

grant SELECT on SAHARA.CLUSTER_RCLUSTER_RELATION to SAHARA;

grant SELECT on SAHARA.CLUSTER_RCLUSTER_RELATION to SAHARA_APP;

/*==============================================================*/
/* View: CLUSTER_REMOTECLUSTER_CURRENT                          */
/*==============================================================*/
create view SAHARA.CLUSTER_REMOTECLUSTER_CURRENT  as
select
   CLUSTER_REMOTECLUSTER_KEY,
   CLUSTER_KEY,
   CLUSTER_OID,
   REMOTECLUSTER_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME
from
   SAHARA.CLUSTER_REMOTECLUSTER
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

grant SELECT on SAHARA.CLUSTER_REMOTECLUSTER_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: COLUMN_METADATA                                        */
/*==============================================================*/
create view SAHARA.COLUMN_METADATA  as
select COLUMN_METADATA_BASE.OWNER,
       COLUMN_METADATA_BASE.TABLE_NAME,
       COLUMN_METADATA_BASE.OWNER || '.' || COLUMN_METADATA_BASE.TABLE_NAME as FULL_TABLE_NAME,
       COLUMN_METADATA_BASE.COLUMN_NAME,
       COLUMN_METADATA_BASE.DISPLAY_NAME,
       SC.REMARKS
  from SAHARA.COLUMN_METADATA_BASE
  join SYS.SYSCOLUMN SC on SC.COLUMN_NAME = COLUMN_METADATA_BASE.COLUMN_NAME
  join SYS.SYSTABLE ST on ST.TABLE_ID = SC.TABLE_ID and ST.TABLE_NAME = COLUMN_METADATA_BASE.TABLE_NAME
  join SYS.SYSUSER SU on SU.USER_ID = ST.CREATOR and SU.USER_NAME = COLUMN_METADATA_BASE.OWNER;

grant SELECT on SAHARA.COLUMN_METADATA to SAHARA_APP;

/*==============================================================*/
/* View: DATABASES_CURRENT                                      */
/*==============================================================*/
create view SAHARA.DATABASES_CURRENT  as
select DATABASES_KEY,
       HOST_KEY,
       DATABASES_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       INSTANCE_DISPLAY_NAME,
       DATABASE_TYPE,
       DATABASE_VERSION,
       DATABASE_STATE,
       DISCOVERY_STATE,
       IS_CLUSTERED,
       IS_RAC_AWARE,
       IS_BASED_ON_THIN,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_BASED_ON_DB_VM
  from SAHARA.DATABASES
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.DATABASES_CURRENT is 
'This view filters the DATABASES table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.DATABASES_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: DISKGROUP_CURRENT                                      */
/*==============================================================*/
create view SAHARA.DISKGROUP_CURRENT (DISKGROUP_KEY, DISKGROUP_OID, HOST_KEY, CC_STORAGE_URL, DISPLAY_NAME, FAMILY, VERSION, TYPE, IS_SHARED, IS_BASED_ON_DIRECT_ATTACHED, IS_BASED_ON_THIN, STATE, APPLICATION_BE, APPLICATION_COMPOSITE_BE, ORGANIZATION_BE, ORGANIZATION_COMPOSITE_BE, REDUNDANCY_TYPE, IS_USED_BY_CLUSTERED_DB, USES_VM_VOLUMES) as
select DISKGROUP_KEY,
       DISKGROUP_OID,       
       HOST_KEY,
       CC_STORAGE_URL,
       DISPLAY_NAME,       
       FAMILY,
       VERSION,
       TYPE,
       IS_SHARED,
       IS_BASED_ON_DIRECT_ATTACHED,
       IS_BASED_ON_THIN,
       STATE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       REDUNDANCY_TYPE,
       IS_USED_BY_CLUSTERED_DB,
       USES_VM_VOLUMES
  from SAHARA.DISKGROUP 
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.DISKGROUP_CURRENT is 
'This view filters the DISKGROUP table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.DISKGROUP_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: ER_LICENSE_DISPLAY                                     */
/*==============================================================*/
create view SAHARA.ER_LICENSE_DISPLAY  as
select
    LICENSE_KEY,
    LICENSE_TYPE,
    VERSION,
    CLIENT_ACCESS_LICENSES,
    DURATION,
    coalesce(DATEFORMAT(EXPIRATION, 'YYYY-Mmm-DD hh:nn aa'), 'Never') as EXPIRATION,
    case IS_SITE_KEY when 'T' then 'Yes' else 'No' end as IS_SITE_KEY,
    case IS_EDITION_KEY when 'T' then 'Yes' else 'No' end as IS_EDITION_KEY,
    case IS_POINT_PRODUCT_KEY when 'T' then 'Yes' else 'No' end as IS_POINT_PRODUCT_KEY
from SAHARA.ER_LICENSE;

comment on view SAHARA.ER_LICENSE_DISPLAY is 
'This view provides a list of installed Enterprise Reporter licenses for display in the GUI.  It performs some minor translations of values found in the ER_LICENSE table.

Licenses are polled from the Enterprise Reporter server where they are installed and stored and copied into the database from where they are reported.  The snapshot of licenses that this view provides is only as current as the most recent poll.';

grant SELECT on SAHARA.ER_LICENSE_DISPLAY to SAHARA_APP;

/*==============================================================*/
/* View: HOST_CURRENT                                           */
/*==============================================================*/
create view SAHARA.HOST_CURRENT  as
select HOST_KEY,
       HOST_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       OS_DETAILS,
       OS_TYPE,
       IP_ADDRESS,
       VIRTUAL_SERVER,
       IS_USER_CREATED,
       IS_VIRTUAL,
       IS_VIRTUAL_SERVER,
       IS_CLUSTERED,
       CLUSTER_TYPE,
       CLUSTER_VERSION,
       IS_GUEST_INSTALLED,
       IS_ACTIVE,
       CLUSTER_NAME,
       GUEST_OS_TYPE,
       VIRTUALIZATION_TECHNOLOGY,
       SF_VERSION,
       MH_VERSION,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       VOM_LICENSE_REQUIRED,
       VM_HOST_OID,
       PARENT_VIRTUAL_SERVER_OID,
       CONFIGURATION_TYPE
  from SAHARA.HOST
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.HOST_CURRENT is 
'This view filters the HOST table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.HOST_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: ER_LICENSE_USAGE                                       */
/*==============================================================*/
create view SAHARA.ER_LICENSE_USAGE  as
with
    -- get the ER server time for use in comparisons
    ER_TIME as
        (select cast(DATEADD( second, INTEGER_VALUE, current utc timestamp) as date) as ER_NOW
           from SAHARA.CONFIGURATION
          where SECTION = 'SERVER'
            and NAME = 'UTC_ADJUSTMENT')

select VALID_CALS.COUNT                             as VALID_CALS,
       EXPIRED_CALS.COUNT                           as EXPIRED_CALS,
       (CASE WHEN (HOSTS.COUNT < VALID_CALS.COUNT) 
            THEN HOSTS.COUNT
            ELSE VALID_CALS.COUNT
        END)                                        as USED_CALS,
       (CASE WHEN (VALID_CALS.COUNT - HOSTS.COUNT) < 0 
            THEN 0
            ELSE (VALID_CALS.COUNT - HOSTS.COUNT) 
        END)                                        as UNUSED_CALS,
       (CASE WHEN (HOSTS.COUNT - VALID_CALS.COUNT) < 0 
            THEN 0
            ELSE (HOSTS.COUNT - VALID_CALS.COUNT) 
        END)                                        as EXCEEDED_CALS


  from
  -- get the number of valid (non-expired) client access licenses
    (select sum(CLIENT_ACCESS_LICENSES) as COUNT
       from SAHARA.ER_LICENSE
      cross join (select ER_NOW from ER_TIME) ET
      where EXPIRATION is null       -- permanent licenses
         or ER_NOW < EXPIRATION) VALID_CALS

  -- get the number of expired client access licenses
  cross join
    (select sum(CLIENT_ACCESS_LICENSES) as COUNT
       from SAHARA.ER_LICENSE
      cross join (select ER_NOW from ER_TIME) ET
      where EXPIRATION is not null   -- non-permanent licenses
        and ER_NOW >= EXPIRATION) EXPIRED_CALS

  -- get the number of non-user created managed hosts
  cross join
    (select count(*) as COUNT
       from SAHARA.HOST_CURRENT
      where IS_USER_CREATED = 'F') HOSTS;

comment on view SAHARA.ER_LICENSE_USAGE is 
'This view provides a summary of Enterprise Reporter license usage for display in the GUI.';

grant SELECT on SAHARA.ER_LICENSE_USAGE to SAHARA_APP;

/*==============================================================*/
/* View: FILESYSTEM_CURRENT                                     */
/*==============================================================*/
create view SAHARA.FILESYSTEM_CURRENT  as
select FILESYSTEM_KEY,
       FILESYSTEM_OID,
       VOLUME_KEY,
       HOST_KEY,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       ADVISORY_DISPLAY_NAME,
       MOUNT_POINT,
       DEVICE_PATH,
       TYPE,
       IS_CREATED_FROM_VOLUME,
       IS_CLUSTERED,
       IS_VCS_CVM,
       IS_DIRECT_ATTACHED,
       IS_CFS_AWARE,
       IS_BASED_ON_THIN,
       IS_EXPORTED,
       IS_CHILD,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_BASED_ON_NAS_SHARE
  from SAHARA.FILESYSTEM 
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.FILESYSTEM_CURRENT is 
'This view filters the FILESYSTEM table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.FILESYSTEM_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: FILE_CATEGORY_CURRENT                                  */
/*==============================================================*/
create view SAHARA.FILE_CATEGORY_CURRENT  as
select FILE_CATEGORY_KEY,
       FILE_CATEGORY_OID,
       DISPLAY_NAME,
       DESCRIPTION,
       FILE_EXTENSIONS
  from SAHARA.FILE_CATEGORY
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.FILE_CATEGORY_CURRENT is 
'This view filters the FILE_CATEGORY table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

grant SELECT on SAHARA.FILE_CATEGORY_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: LUN_CURRENT                                            */
/*==============================================================*/
create view SAHARA.LUN_CURRENT  as
select
   LUN_KEY,
   ARRAY_KEY,
   LUN_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   ADVISORY_DISPLAY_NAME,
   DRIVE_TYPE,
   RAID_LEVEL,
   EMULATION_MODE,
   MASKING_STATUS,
   CLAIMED_STATUS,
   DISCOVERY_STATE,
   NUMBER_OF_COPIES,
   PARITY,
   RESOURCE_TYPE,
   REVISION,
   SERIAL_NUMBER,
   STRIPE_WIDTH,
   VENDOR_DEVICE_ID,
   VENDOR_NAME,
   VENDOR_STATUS,
   BINDING_STATE,
   COLUMNS,
   TIER,
   TIER_COST_PER_GB,
   IS_PRIMARY,
   IS_READ_CACHE_ENABLED,
   IS_SECONDARY,
   IS_SEGMENT,
   IS_WRITE_CACHE_ENABLED,
   IS_AGGREGATE,
   IS_CONTROL_DEVICE,
   IS_FAULT_TOLERANT,
   IS_ADMINISTRATIVE,
   IS_USED_IN_SNAPSHOT,
   IS_VIRTUALIZER_SOURCE,
   IS_DIRECT_ATTACHED,
   IS_BASEDON_EXTERNAL,
   ARRAY_PORTS,
   DISK_VENDOR,
   DISK_CAPACITY,
   VENDOR_LAYOUT,
   THIN_STATUS,
   VOM_DISCOVER_TYPE,
   IS_ASSIGNED,
   IS_EXPORTED,
   APPLICATION_BE,
   APPLICATION_COMPOSITE_BE,
   ORGANIZATION_BE,
   ORGANIZATION_COMPOSITE_BE,
   TYPE,
   SUB_TYPE,
   STORAGE_TECHNOLOGY,
   STORAGE_POOL_NAME,
   CONNECTION_PROTOCOL,
   NAS_FILESYSTEM_KEY,
   ARRAY_NAMES
from
   SAHARA.LUN
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.LUN_CURRENT is 
'This view filters the LUN table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.LUN_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: MESSAGE_LOG_SUMMARY                                    */
/*==============================================================*/
create view SAHARA.MESSAGE_LOG_SUMMARY  as
select MESSAGE_LOG_KEY,
       "TIME",
       cast(current utc timestamp as timestamp) - "TIME" as DURATION_SINCE,
       SESSION_KEY,
       FUNCTION_NAME,
       SEVERITY,
       cast(substr("MESSAGE", 1, 4000) as long varchar) as "MESSAGE"
  from SAHARA.MESSAGE_LOG;

comment on view SAHARA.MESSAGE_LOG_SUMMARY is 
'A view for simplifying access to the message log by converting the message text from a clob to a varchar2(4000).  Warning: messages longer than 4000 bytes will be truncated.';

grant SELECT on SAHARA.MESSAGE_LOG_SUMMARY to SAHARA_APP;

/*==============================================================*/
/* View: NAS_FILESYSTEM_CURRENT                                 */
/*==============================================================*/
create view SAHARA.NAS_FILESYSTEM_CURRENT  as
select NAS_FILESYSTEM_KEY,
   NAS_FILESYSTEM_OID,
   ARRAY_KEY,
   NETAPP_VOLUME_KEY,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   DISCOVERY_STATE,
   SECURITY_STYLE,
   FILESYSTEM_ID,
   IS_QUOTA_ENABLED,
   TIER,
   TYPE,
   PATH,
   AUTO_EXTEND_ENABLED,
   IS_LOCK_ENABLED,
   CWORM_STATE,
   IS_VIRTUAL_PROVISIONING_ENABLE,
   IS_SOURCE,
   IS_TARGET,
   REPLICATION_TECHNOLOGY,
   AUTO_EXTEND_MAX_SIZE,
   HIGH_WATERMARK
  from SAHARA.NAS_FILESYSTEM
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NAS_FILESYSTEM_CURRENT is 
'This view filters the NAS_FILESYSTEM table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

/*==============================================================*/
/* View: NAS_QUOTA_CURRENT                                      */
/*==============================================================*/
create view SAHARA.NAS_QUOTA_CURRENT  as
select NAS_QUOTA_KEY,
   NAS_QUOTA_OID,
   ARRAY_KEY,
   NAS_FILESYSTEM_KEY,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   TYPE,
   TARGET
  from SAHARA.NAS_QUOTA
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NAS_QUOTA_CURRENT is 
'This view filters the NAS_FILESYSTEM table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

grant SELECT on SAHARA.NAS_QUOTA_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NETAPP_FILE_FCAT_                                      */
/*==============================================================*/
create view SAHARA_APP.NETAPP_FILE_FCAT_  as
select HAL.ARRAY_OID,
       DM.FILE_CATEGORY_OID,
       DM.DM_BATCH_ID,
       DM.CONSUMPTION,
       DM.FILES
  from SAHARA_APP.NETAPP_FILE_FCAT_DM_ DM
  join SAHARA_APP.ARRAY_ HAL on HAL.DM_ID = DM.DM_ID;

comment on view SAHARA_APP.NETAPP_FILE_FCAT_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_FILE_FCAT.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp device (filers/arrays/storage systems) files organized by file category.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_FILE_FCAT_ to SAHARA;

/*==============================================================*/
/* View: NETAPP_QTREE_CURRENT                                   */
/*==============================================================*/
create view SAHARA.NETAPP_QTREE_CURRENT  as
select
   NETAPP_QTREE_KEY,
   NETAPP_QTREE_OID,
   ARRAY_KEY,
   NETAPP_VOLUME_KEY,
   CC_STORAGE_URL,
   QTREE_PATH,
   DISPLAY_NAME,
   DISCOVERY_STATE,
   SECURITY_STYLE,
   FILESYSTEM_ID,
   IS_QUOTA_ENABLED
from
   SAHARA.NETAPP_QTREE
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NETAPP_QTREE_CURRENT is 
'This view filters the NETAPP_QUTREE table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.NETAPP_QTREE_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NETAPP_QTREE_FILE_                                     */
/*==============================================================*/
create view SAHARA_APP.NETAPP_QTREE_FILE_  as
select HAL.NETAPP_QTREE_OID,
       HAL.CAPACITY,
       HAL.QUOTA_LIMIT,
       HAL.QUOTA_USED,
       DM.DM_BATCH_ID,
       DM.TOTAL_CONSUMPTION,
       DM.FILES,
       DM.FOLDERS,
       DM.USERS
  from SAHARA_APP.NETAPP_QTREE_FILE_HAL_ HAL
  left outer join SAHARA_APP.NETAPP_QTREE_FILE_DM_ DM on DM.DM_ID = HAL.DM_ID;

comment on view SAHARA_APP.NETAPP_QTREE_FILE_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_QTREE_FILE.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp qtree measures.  Each rows represents one sampling of values for a qtree.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_QTREE_FILE_ to SAHARA;

/*==============================================================*/
/* View: NETAPP_QTREE_FILE_FCAT_                                */
/*==============================================================*/
create view SAHARA_APP.NETAPP_QTREE_FILE_FCAT_  as
select HAL.NETAPP_QTREE_OID,
       DM.FILE_CATEGORY_OID,
       DM.DM_BATCH_ID,
       DM.CONSUMPTION,
       DM.FILES
  from SAHARA_APP.NETAPP_QTREE_FILE_FCAT_DM_ DM
  join SAHARA_APP.NETAPP_QTREE_ HAL on HAL.DM_ID = DM.DM_ID;

comment on view SAHARA_APP.NETAPP_QTREE_FILE_FCAT_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_QTREE_FILE_FCAT.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp qtree files organized by file category.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_QTREE_FILE_FCAT_ to SAHARA;

/*==============================================================*/
/* View: NETAPP_SHARE_CURRENT                                   */
/*==============================================================*/
create view SAHARA.NETAPP_SHARE_CURRENT  as
select NETAPP_SHARE_KEY,
       NETAPP_SHARE_OID,
       ARRAY_KEY,
       NETAPP_VOLUME_KEY,
       NETAPP_QTREE_KEY,
       NAS_FILESYSTEM_KEY,
       CC_STORAGE_URL,
       SHARE_PATH,
       NFS_EXPORT_PATH,
       DISPLAY_NAME,
       CIFS_MOUNT_POINT,
       CIFS_SHARE_NAME,
       MOUNT_POINT_TYPE,
       DISCOVERY_STATE,
       TIER,
       ESX_DATA_STORE_NAMES,
       HOST_PRIVILEGE,
       IS_MAPPED_TO_ANY_UNDISCOV_HOST
  from SAHARA.NETAPP_SHARE
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NETAPP_SHARE_CURRENT is 
'This view filters the NETAPP_SHARE table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.NETAPP_SHARE_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NETAPP_SHARE_FILE_                                     */
/*==============================================================*/
create view SAHARA_APP.NETAPP_SHARE_FILE_  as
select HAL.NETAPP_SHARE_OID,
       isnull(DM.DM_BATCH_ID,0) as DM_BATCH_ID,
       isnull(DM.TOTAL_CONSUMPTION,0) as TOTAL_CONSUMPTION,
       isnull(DM.FILES,0) as FILES,
       isnull(DM.FOLDERS,0) as FOLDERS,
       isnull(DM.USERS,0) as USERS,
       HAL.NO_OF_HOSTS
  from SAHARA_APP.NETAPP_SHARE_FILE_HAL_ HAL
  left join SAHARA_APP.NETAPP_SHARE_FILE_DM_ DM on DM.DM_ID = HAL.DM_ID;

comment on view SAHARA_APP.NETAPP_SHARE_FILE_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_SHARE_FILE.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp share measures.  Each rows represents one sampling of values for a volume.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_SHARE_FILE_ to SAHARA;

/*==============================================================*/
/* View: NETAPP_SHARE_FILE_FCAT_                                */
/*==============================================================*/
create view SAHARA_APP.NETAPP_SHARE_FILE_FCAT_  as
select HAL.NETAPP_SHARE_OID,
       DM.FILE_CATEGORY_OID,
       DM.DM_BATCH_ID,
       DM.CONSUMPTION,
       DM.FILES
  from SAHARA_APP.NETAPP_SHARE_FILE_FCAT_DM_ DM
  join SAHARA_APP.NETAPP_SHARE_ HAL on HAL.DM_ID = DM.DM_ID;

comment on view SAHARA_APP.NETAPP_SHARE_FILE_FCAT_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_SHARE_FILE_FCAT.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp share files organized by file category.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_SHARE_FILE_FCAT_ to SAHARA;

/*==============================================================*/
/* View: NETAPP_VOLUME_CURRENT                                  */
/*==============================================================*/
create view SAHARA.NETAPP_VOLUME_CURRENT  as
select NETAPP_VOLUME_KEY,
       NETAPP_VOLUME_OID,
       ARRAY_KEY,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       DISCOVERY_STATE,
       VENDOR_STATE,
       VENDOR_LAYOUT,
       RAID_LEVEL,
       IS_AGGREGATE,
       IS_SEGMENT,
       IS_VFILER_VOLUME,
       TYPE,
       SUB_TYPE,
       TIER,
       RPM,
       DRIVE_TYPE,
       DISK_SIZE,
       IS_NAS_FS_CREATED,
       DEDUP_STATE,
       LAST_DEDUP_OP_BEGIN_TIME,
       LAST_DEDUP_OP_END_TIME,
       STORAGE_POOL_KEY,
       STORAGE_POOL_OBJECT_IDENTIFIER
  from SAHARA.NETAPP_VOLUME
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.NETAPP_VOLUME_CURRENT is 
'This view filters the NETAPP_VOLUME table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.NETAPP_VOLUME_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: NETAPP_VOLUME_FILE_                                    */
/*==============================================================*/
create view SAHARA_APP.NETAPP_VOLUME_FILE_  as
select HAL.NETAPP_VOLUME_OID,
       HAL.CAPACITY,
       HAL.USED_CAPACITY,
       HAL.AVAILABLE_CAPACITY,
       HAL.QTREES,
       isnull(DM.DM_BATCH_ID,0) as DM_BATCH_ID,
       isnull(DM.TOTAL_CONSUMPTION,0) as TOTAL_CONSUMPTION,
       isnull(DM.FILES,0) as FILES,
       isnull(DM.FOLDERS,0) as FOLDERS,
       isnull(DM.USERS,0) as USERS,
       HAL.PHYSICAL_CAPACITY,
       HAL.OVERHEAD_CAPACITY,
       HAL.PHYSICAL_USED,
       HAL.NUMBER_OF_DATA_DISKS,
       HAL.NUMBER_OF_PARITY_DISKS,
       HAL.NUMBER_OF_VOLS_CREATED,
       HAL.NUMBER_OF_LUNS_CREATED,
       HAL.NUMBER_OF_TARGETS,
       HAL.SPACE_SHARED,
       HAL.SPACE_SAVED,
       HAL.EFFECTIVE_SPACE_USED,
       HAL.PHYSICAL_SPACE_SHARED,
       HAL.PHYSICAL_SPACE_SAVED,
       HAL.PHYSICAL_EFFECTIVE_SPACE_USED,
       HAL.SUBSCRIBED_PERCENT,
       HAL.SPACE_SAVED_PERCENT
  from SAHARA_APP.NETAPP_VOLUME_FILE_HAL_ HAL
  left join SAHARA_APP.NETAPP_VOLUME_FILE_DM_ DM on DM.DM_ID = HAL.DM_ID;

comment on view SAHARA_APP.NETAPP_VOLUME_FILE_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_VOLUME_FILE.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp volume measures.  Each rows represents one sampling of values for a volume.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_VOLUME_FILE_ to SAHARA;

/*==============================================================*/
/* View: NETAPP_VOLUME_FILE_FCAT_                               */
/*==============================================================*/
create view SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_  as
select HAL.NETAPP_VOLUME_OID,
       DM.FILE_CATEGORY_OID,
       DM.DM_BATCH_ID,
       DM.CONSUMPTION,
       DM.FILES
  from SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_DM_ DM
  join SAHARA_APP.NETAPP_VOLUME_ HAL on HAL.DM_ID = DM.DM_ID;

comment on view SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_ is 
'This is a view that joins staging tables for loading data into warehouse table SAHARA.NETAPP_VOLUME_FILE_FCAT.  Any data in this view is temporary and should not be accessed except by the data loading process.

The following description is copied from the warehouse table:  This is the fact table for NetApp volume files organized by file category.';

grant DELETE,INSERT,SELECT,UPDATE on SAHARA_APP.NETAPP_VOLUME_FILE_FCAT_ to SAHARA;

/*==============================================================*/
/* View: ROLLUP_BATCH_READ                                      */
/*==============================================================*/
create view SAHARA.ROLLUP_BATCH_READ  as
select
   ROLLUP_BATCH.ROLLUP_BATCH_KEY,
   ROLLUP_BATCH.ROLLUP_SOURCE_TYPE_KEY,
   ROLLUP_BATCH.DATES_KEY,
   ROLLUP_BATCH.SESSION_KEY,
   ROLLUP_BATCH.SCHEDULE_HASH,
   ROLLUP_BATCH.ROLLUP_SOURCE_KEY,
   ROLLUP_BATCH.SOURCE_NAME,
   ROLLUP_BATCH.WEB_SERVER_HOST,
   ROLLUP_BATCH.WEB_SERVER_PORT,
   ROLLUP_BATCH.DB_HOST,
   ROLLUP_BATCH.DB_PORT,
   ROLLUP_BATCH.LOG_ARCHIVE,
   ROLLUP_BATCH.ROLLUP_DAY,
   ROLLUP_BATCH.START_TIME,
   ROLLUP_BATCH.START_TIME_ER,
   ROLLUP_BATCH.COMPLETION_TIME,
   ROLLUP_BATCH.COMPLETION_TIME_ER,
   ROLLUP_BATCH.DURATION,
   ROLLUP_BATCH.UTC_ADJUSTMENT,
   ROLLUP_BATCH.IS_ROLLUP_NOW,
   ROLLUP_BATCH.IS_COMPLETE,
   ROLLUP_BATCH.IS_SUCCESS,
   ROLLUP_BATCH.IS_LATEST,
   ROLLUP_BATCH.IS_LATEST_COMPLETE,
   ROLLUP_BATCH.IS_LATEST_SUCCESS,
   ROLLUP_BATCH.IS_LAST_OF_MONTH,
   ROLLUP_BATCH.IS_LAST_OF_QUARTER,
   ROLLUP_BATCH.IS_LAST_OF_YEAR,
   ROLLUP_BATCH.STATUS,
   ROLLUP_BATCH.STATUS_MESSAGE,
   ROLLUP_BATCH.LAST_STATUS_UPDATE,
   ROLLUP_BATCH.IS_LAST_OF_WEEK,
   ROLLUP_BATCH.PERIOD_TYPE,
   ROLLUP_BATCH.VALID_CLUSTER_DATA,
   ROLLUP_BATCH.IS_DUMMY,
   ROLLUP_BATCH.SHOW_STATUS
from
   SAHARA.ROLLUP_BATCH;

comment on view SAHARA.ROLLUP_BATCH_READ is 
'This view has the same structure as the table ROLLUP_BATCH except that it omits the database password column.  Select priviledge is granted on ROLLUP_BATCH to the SAHARA_APP database user only (the user used by the Enterprise Reporter application).  Select priviledge is granted to SAHARA_READ for this view.  This allows access for reports and general queries to see all columns except the database passwords.';

grant SELECT on SAHARA.ROLLUP_BATCH_READ to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_DATA_SET_METADATA                               */
/*==============================================================*/
create view SAHARA.ROLLUP_DATA_SET_METADATA  as
select ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY,
       ROLLUP_DATA_SET_TYPE.NAME as TYPE_NAME,
       ROLLUP_DATA_SET_TYPE.STEREOTYPE,
       ROLLUP_DATA_SET_TYPE.DEST_TABLE,
       ROLLUP_DATA_SET_TYPE.LOAD_SQL,
       DSC.NAME as COLUMN_NAME,
       DSC.LOAD_PROCESSING,
       DSC.SOURCE_EXPRESSION
  from SAHARA.ROLLUP_DATA_SET_TYPE
  left outer join SAHARA.ROLLUP_DATA_SET_COLUMN DSC on ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY = DSC.ROLLUP_DATA_SET_TYPE_KEY;

comment on view SAHARA.ROLLUP_DATA_SET_METADATA is 
'This view joins the roll-up data set metadata tables to provide simplified access.';

grant SELECT on SAHARA.ROLLUP_DATA_SET_METADATA to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_STATUS                                          */
/*==============================================================*/
create view SAHARA.ROLLUP_STATUS  as
select ROLLUP_BATCH.ROLLUP_BATCH_KEY,
       ROLLUP_BATCH.START_TIME as BATCH_START_TIME,
       ROLLUP_BATCH.START_TIME_ER as BATCH_START_TIME_ER,
       ROLLUP_BATCH.COMPLETION_TIME as BATCH_COMPLETION_TIME,
       ROLLUP_BATCH.COMPLETION_TIME_ER as BATCH_COMPLETION_TIME_ER,
       ROLLUP_BATCH.IS_ROLLUP_NOW as BATCH_IS_ROLLUP_NOW,
       ROLLUP_BATCH.IS_COMPLETE as BATCH_IS_COMPLETE,
       ROLLUP_BATCH.IS_SUCCESS as BATCH_IS_SUCCESS,
       ROLLUP_BATCH.IS_LATEST as BATCH_IS_LATEST,
       ROLLUP_BATCH.IS_LATEST_COMPLETE as BATCH_IS_LATEST_COMPLETE,
       ROLLUP_BATCH.IS_LATEST_SUCCESS as BATCH_IS_LATEST_SUCCESS,
       ROLLUP_BATCH.STATUS as BATCH_STATUS,
       ROLLUP_BATCH.STATUS_MESSAGE as BATCH_STATUS_MESSAGE,
       ROLLUP_BATCH.LAST_STATUS_UPDATE as BATCH_LAST_STATUS_UPDATE,
       ROLLUP_BATCH.SOURCE_NAME,
       ROLLUP_SOURCE_TYPE.DISPLAY_NAME as SOURCE_TYPE_DISPLAY_NAME,
       ROLLUP_BATCH.LOG_ARCHIVE
  from SAHARA.ROLLUP_BATCH
  join SAHARA.ROLLUP_SOURCE_TYPE on ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY = ROLLUP_BATCH.ROLLUP_SOURCE_TYPE_KEY;

comment on view SAHARA.ROLLUP_STATUS is 
'This view provides a summary of roll-up batch status information.';

grant SELECT on SAHARA.ROLLUP_STATUS to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_DATA_SET_STATUS                                 */
/*==============================================================*/
create view SAHARA.ROLLUP_DATA_SET_STATUS  as
select ROLLUP_STATUS.ROLLUP_BATCH_KEY,
       ROLLUP_STATUS.BATCH_START_TIME,
       ROLLUP_STATUS.BATCH_START_TIME_ER,
       ROLLUP_STATUS.BATCH_COMPLETION_TIME,
       ROLLUP_STATUS.BATCH_COMPLETION_TIME_ER,
       ROLLUP_STATUS.BATCH_IS_ROLLUP_NOW,
       ROLLUP_STATUS.BATCH_IS_COMPLETE,
       ROLLUP_STATUS.BATCH_IS_SUCCESS,
       ROLLUP_STATUS.BATCH_IS_LATEST,
       ROLLUP_STATUS.BATCH_IS_LATEST_COMPLETE,
       ROLLUP_STATUS.BATCH_IS_LATEST_SUCCESS,
       ROLLUP_STATUS.BATCH_STATUS,
       ROLLUP_STATUS.BATCH_STATUS_MESSAGE,
       ROLLUP_STATUS.BATCH_LAST_STATUS_UPDATE,
       ROLLUP_STATUS.SOURCE_NAME,
       ROLLUP_STATUS.SOURCE_TYPE_DISPLAY_NAME,
       ROLLUP_STATUS.LOG_ARCHIVE,
       ROLLUP_DATA_SET_TYPE.NAME as DS_TYPE_NAME,
       ROLLUP_DATA_SET.STATUS as DS_STATUS,
       ROLLUP_DATA_SET.STATUS_MESSAGE as DS_STATUS_MESSAGE,
       ROLLUP_DATA_SET.UPLOADING_DURATION as DS_UPLOADING_DURATION,
       ROLLUP_DATA_SET.STAGING_DURATION as DS_STAGING_DURATION,
       ROLLUP_DATA_SET.LOADING_DURATION as DS_LOADING_DURATION,

       coalesce(ROLLUP_DATA_SET.UPLOADING_DURATION, 0) +
       coalesce(ROLLUP_DATA_SET.STAGING_DURATION, 0) +
       coalesce(ROLLUP_DATA_SET.LOADING_DURATION, 0) as DS_TOTAL_DURATION,

       ROLLUP_DATA_SET.ROWS_PROCESSED as DS_ROWS_PROCESSED,
       ROLLUP_DATA_SET.LAST_STATUS_UPDATE as DS_LAST_STATUS_UPDATE
  from SAHARA.ROLLUP_STATUS
  join SAHARA.ROLLUP_DATA_SET on ROLLUP_DATA_SET.ROLLUP_BATCH_KEY = ROLLUP_STATUS.ROLLUP_BATCH_KEY
  join SAHARA.ROLLUP_DATA_SET_TYPE on ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY = ROLLUP_DATA_SET.ROLLUP_DATA_SET_TYPE_KEY;

comment on view SAHARA.ROLLUP_DATA_SET_STATUS is 
'This view provides a summary of roll-up batch status information including the status of batch details (the status of roll-up from each roll-up source for the batch) and the status of roll-up data sets for those details.';

grant SELECT on SAHARA.ROLLUP_DATA_SET_STATUS to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_SCHEDULE_DATETIME                               */
/*==============================================================*/
create view SAHARA.ROLLUP_SCHEDULE_DATETIME  as
select ROLLUP_SOURCE.ROLLUP_SOURCE_KEY,
       ROLLUP_SOURCE.NAME,
       ROLLUP_SOURCE.UTC_ADJUSTMENT,
       ROLLUP_SOURCE.ROLLUP_HOUR,
       ROLLUP_SOURCE.ROLLUP_MINUTE,
       ROLLUP_SOURCE.ROLLUP_TIME_AS_INTERVAL,
       ROLLUP_SOURCE.ROLLUP_CLOCK,
      ROLLUP_SOURCE.ROLLUP_TIME_REVISION,

       -- the scheduled date without factoring in time
       ROLLUP_SCHEDULE_DATE.SCHEDULE_DATE,
       
       -- the scheduled date and time without factoring in the timezone adjustement
      DATEADD(second, ROLLUP_SOURCE.ROLLUP_TIME_AS_INTERVAL,ROLLUP_SCHEDULE_DATE.SCHEDULE_DATE) as SCHEDULE_DATETIME,

       -- the scheduled date and time adjusted to UTC based on the roll-up clock
       DATEADD(second, (ROLLUP_SOURCE.ROLLUP_TIME_AS_INTERVAL -
            (CASE ROLLUP_SOURCE.ROLLUP_CLOCK
                WHEN 'ER'       THEN ER_CONFIG.UTC_ADJUSTMENT
                WHEN 'SOURCE'   THEN ROLLUP_SOURCE.UTC_ADJUSTMENT
              END
            )
        ),ROLLUP_SCHEDULE_DATE.SCHEDULE_DATE) as SCHEDULE_UTC_DATETIME,

       -- what will ER time be then?
       DATEADD(second, (ROLLUP_SOURCE.ROLLUP_TIME_AS_INTERVAL -
        (CASE ROLLUP_SOURCE.ROLLUP_CLOCK
                WHEN 'ER'       THEN ER_CONFIG.UTC_ADJUSTMENT
                WHEN 'SOURCE'   THEN ROLLUP_SOURCE.UTC_ADJUSTMENT
              END
         ) +
         ER_CONFIG.UTC_ADJUSTMENT),ROLLUP_SCHEDULE_DATE.SCHEDULE_DATE) as SCHEDULE_ER_DATETIME,

       -- what will the source time be then?
       DATEADD(second, (ROLLUP_SOURCE.ROLLUP_TIME_AS_INTERVAL -
        (CASE ROLLUP_SOURCE.ROLLUP_CLOCK
                WHEN 'ER'       THEN ER_CONFIG.UTC_ADJUSTMENT
                WHEN 'SOURCE'   THEN ROLLUP_SOURCE.UTC_ADJUSTMENT
              END
         ) +
         ROLLUP_SOURCE.UTC_ADJUSTMENT),ROLLUP_SCHEDULE_DATE.SCHEDULE_DATE) as SCHEDULE_SOURCE_DATETIME,

       -- add the roll-up hash
       DATEFORMAT(ROLLUP_SCHEDULE_DATE.SCHEDULE_DATE, 'YYYYMMDD') || '-' ||
       ROLLUP_SCHEDULE.REVISION || '-' ||
       ROLLUP_SOURCE.ROLLUP_SOURCE_KEY || '-' ||
       ROLLUP_SOURCE.ROLLUP_TIME_REVISION as SCHEDULE_HASH

  from SAHARA.ROLLUP_SOURCE

       -- join the roll-up schedule dates with each source
cross join SAHARA.ROLLUP_SCHEDULE_DATE

       -- get the roll-up schedule to use the revision in the hash
cross join SAHARA.ROLLUP_SCHEDULE

       -- get ER server UTC adjustment to use in calculations
cross join
       (select INTEGER_VALUE as UTC_ADJUSTMENT
          from SAHARA.CONFIGURATION
         where CONFIGURATION.SECTION = 'SERVER'
           and CONFIGURATION.NAME = 'UTC_ADJUSTMENT') ER_CONFIG

       -- only enabled sources
where ROLLUP_SOURCE.ENABLED = 'T';

comment on view SAHARA.ROLLUP_SCHEDULE_DATETIME is 
'A view that joins the roll-up schedule dates with the roll-up sources and their roll-up time.  Only enabled roll-up sources are included.

This view provides the most clear representation of the actual schedule of roll-ups to be performed.

This view is based on data in the roll-up schedule dates table (ROLLUP_SCHEDULE_DATE) which is a global temporary table.  Before the dates/times view (this view) may be used, the dates table must be populated by calling stored procedure ROLLUP_SCHEDULE_API.PrepareDatesTable.';

grant SELECT on SAHARA.ROLLUP_SCHEDULE_DATETIME to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_SOURCE_LATEST_BATCH                             */
/*==============================================================*/
create view SAHARA.ROLLUP_SOURCE_LATEST_BATCH  as
select ROLLUP_SOURCE.ROLLUP_SOURCE_KEY,
       ROLLUP_SOURCE.NAME as SOURCE_NAME,
       ROLLUP_SOURCE.ENABLED as SOURCE_ENABLED,
       ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY,
       ROLLUP_SOURCE_TYPE.DISPLAY_NAME as SOURCE_TYPE_DISPLAY_NAME,
       ROLLUP_BATCH.ROLLUP_BATCH_KEY,
       ROLLUP_BATCH.START_TIME,
       ROLLUP_BATCH.START_TIME_ER,
       ROLLUP_BATCH.COMPLETION_TIME,
       ROLLUP_BATCH.COMPLETION_TIME_ER,
       ROLLUP_BATCH.DURATION,
       ROLLUP_BATCH.IS_ROLLUP_NOW,
       ROLLUP_BATCH.IS_COMPLETE,
       ROLLUP_BATCH.IS_SUCCESS,
       ROLLUP_BATCH.IS_LATEST_COMPLETE,
       ROLLUP_BATCH.IS_LATEST_SUCCESS,
       ROLLUP_BATCH.STATUS,
       ROLLUP_BATCH.STATUS_MESSAGE,
       ROLLUP_BATCH.LAST_STATUS_UPDATE,
       ROLLUP_BATCH.LOG_ARCHIVE

    -- only non-deleted roll-up sources
  from SAHARA.ROLLUP_SOURCE

  join SAHARA.ROLLUP_SOURCE_TYPE on ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY = ROLLUP_SOURCE.ROLLUP_SOURCE_TYPE_KEY

  left outer join SAHARA.ROLLUP_BATCH
    on ROLLUP_SOURCE.ROLLUP_SOURCE_KEY = ROLLUP_BATCH.ROLLUP_SOURCE_KEY
   and ROLLUP_BATCH.IS_LATEST = 'T';

comment on view SAHARA.ROLLUP_SOURCE_LATEST_BATCH is 
'This view provides summary information for the latest batch for each non-deleted roll-source.  The latest batch may be in process or complete and a complete roll-up for the batch may have succeeded for failed.

This view exists primarily to support the GUI in providng a summary of roll-up status for each roll-up source..';

grant SELECT on SAHARA.ROLLUP_SOURCE_LATEST_BATCH to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_SOURCE_LATEST_SUCCESS                           */
/*==============================================================*/
create view SAHARA.ROLLUP_SOURCE_LATEST_SUCCESS  as
select ROLLUP_SOURCE.ROLLUP_SOURCE_KEY,
       ROLLUP_SOURCE.NAME as SOURCE_NAME,
       ROLLUP_SOURCE.ENABLED as SOURCE_ENABLED,
       ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY,
       ROLLUP_SOURCE_TYPE.DISPLAY_NAME as SOURCE_TYPE_DISPLAY_NAME,
       ROLLUP_BATCH.ROLLUP_BATCH_KEY,
       ROLLUP_BATCH.START_TIME,
       ROLLUP_BATCH.START_TIME_ER,
       ROLLUP_BATCH.COMPLETION_TIME,
       ROLLUP_BATCH.COMPLETION_TIME_ER,
       ROLLUP_BATCH.DURATION,
       ROLLUP_BATCH.IS_ROLLUP_NOW,
       ROLLUP_BATCH.IS_COMPLETE,
       ROLLUP_BATCH.IS_SUCCESS,
       ROLLUP_BATCH.IS_LATEST_COMPLETE,
       ROLLUP_BATCH.IS_LATEST_SUCCESS,
       ROLLUP_BATCH.STATUS,
       ROLLUP_BATCH.STATUS_MESSAGE,
       ROLLUP_BATCH.LAST_STATUS_UPDATE,
       ROLLUP_BATCH.LOG_ARCHIVE

    -- only non-deleted roll-up sources
  from SAHARA.ROLLUP_SOURCE

  join SAHARA.ROLLUP_SOURCE_TYPE on ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY = ROLLUP_SOURCE.ROLLUP_SOURCE_TYPE_KEY

  left outer join SAHARA.ROLLUP_BATCH
    on ROLLUP_SOURCE.ROLLUP_SOURCE_KEY = ROLLUP_BATCH.ROLLUP_SOURCE_KEY
   and ROLLUP_BATCH.IS_LATEST_SUCCESS = 'T';

comment on view SAHARA.ROLLUP_SOURCE_LATEST_SUCCESS is 
'This view provides summary information for the latest successful roll-up batch batch for each non-deleted roll-source.';

grant SELECT on SAHARA.ROLLUP_SOURCE_LATEST_SUCCESS to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_SOURCE_READ                                     */
/*==============================================================*/
create view SAHARA.ROLLUP_SOURCE_READ  as
select
   ROLLUP_SOURCE_KEY,
   ROLLUP_SOURCE_TYPE_KEY,
   NAME,
   UPPER_NAME,
   WEB_SERVER_HOST,
   WEB_SERVER_PORT,
   DB_HOST,
   DB_PORT,
   UTC_ADJUSTMENT,
   ROLLUP_HOUR,
   ROLLUP_MINUTE,
   ROLLUP_TIME_AS_INTERVAL,
   ROLLUP_CLOCK,
   ROLLUP_TIME_REVISION,
   ROLLUP_NOW,
   ENABLED
from
   SAHARA.ROLLUP_SOURCE;

comment on view SAHARA.ROLLUP_SOURCE_READ is 
'This view has the same structure as the table ROLLUP_SOURCE except that it omits the database password column.  Select priviledge is granted on ROLLUP_SOURCE to the SAHARA_APP database user only (the user used by the Enterprise Reporter application).  Select priviledge is granted to SAHARA_READ for this view.  This allows access for reports and general queries to see all columns except the database passwords.';

grant SELECT on SAHARA.ROLLUP_SOURCE_READ to SAHARA_APP;

/*==============================================================*/
/* View: ROLLUP_SOURCE_TIME                                     */
/*==============================================================*/
create view SAHARA.ROLLUP_SOURCE_TIME  as
select ROLLUP_SOURCE.ROLLUP_SOURCE_KEY,
       ROLLUP_SOURCE.NAME,
       ROLLUP_SOURCE.UTC_ADJUSTMENT,
       ROLLUP_SOURCE.ROLLUP_HOUR,
       ROLLUP_SOURCE.ROLLUP_MINUTE,
       ROLLUP_SOURCE.ROLLUP_TIME_AS_INTERVAL,
       ROLLUP_SOURCE.ROLLUP_CLOCK,
       ROLLUP_SOURCE.ROLLUP_TIME_REVISION,
       ROLLUP_SOURCE.ROLLUP_NOW,
       ROLLUP_SOURCE.ENABLED,
       cast(UTC."TIME" as date) as CURRENT_UTC_DATETIME,
       cast(DATEADD(second, ER_CONFIG.UTC_ADJUSTMENT, UTC."TIME") as date) as CURRENT_ER_DATETIME,
       cast(DATEADD(second, ROLLUP_SOURCE.UTC_ADJUSTMENT, UTC."TIME") as date) as CURRENT_SOURCE_DATETIME
  from SAHARA.ROLLUP_SOURCE

       -- join UTC time with each row
 cross join
       (select current utc timestamp as "TIME" from dummy) UTC

       -- join the ER server UTC adjustment with each row
 cross join
       (select INTEGER_VALUE as UTC_ADJUSTMENT
          from SAHARA.CONFIGURATION
         where CONFIGURATION.SECTION = 'SERVER'
           and CONFIGURATION.NAME = 'UTC_ADJUSTMENT') ER_CONFIG;

comment on view SAHARA.ROLLUP_SOURCE_TIME is 
'A view that brings together the various date and time elements of the roll-up sources and roll-up schedule from the roll-up source point of view.  It includes the current date and time in the various appropriate time locations for each source.';

grant SELECT on SAHARA.ROLLUP_SOURCE_TIME to SAHARA_APP;

/*==============================================================*/
/* View: RSOURCE_TYPE_SUPPORTED_DS                              */
/*==============================================================*/
create view SAHARA.RSOURCE_TYPE_SUPPORTED_DS  as
select ROLLUP_SOURCE_TYPE.DISPLAY_NAME as ROLLUP_SOURCE_DISPLAY_NAME,
       ROLLUP_DATA_SET_TYPE.NAME as ROLLUP_DATA_SET_TYPE_NAME,
       ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY,
       ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY
  from SAHARA.ROLLUP_SOURCE_TYPE
  join SAHARA.RSOURCE_TYPE_DATA_SET_TYPE on RSOURCE_TYPE_DATA_SET_TYPE.ROLLUP_SOURCE_TYPE_KEY = ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY
  join SAHARA.ROLLUP_DATA_SET_TYPE on ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY = RSOURCE_TYPE_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY;

comment on view SAHARA.RSOURCE_TYPE_SUPPORTED_DS is 
'This view provide a summary of data sets supported by each roll-up source type.  It simplifies queries that use the join table between roll-up source types and data set types.';

grant SELECT on SAHARA.RSOURCE_TYPE_SUPPORTED_DS to SAHARA_APP;

/*==============================================================*/
/* View: SERVICEGROUP_CURRENT                                   */
/*==============================================================*/
create view SAHARA.SERVICEGROUP_CURRENT  as
select
   SERVICEGROUP_KEY,
   SERVICEGROUP_OID,
   CLUSTER_KEY,
   CC_STORAGE_URL,
   DISPLAY_NAME,
   SG_TYPE,
   SCOPE,
   FAILOVERPOLICY,
   IS_APPHA,
   FAMILY,
   APPLICATION_BE,
   APPLICATION_COMPOSITE_BE,
   ORGANIZATION_BE,
   ORGANIZATION_COMPOSITE_BE
from
   SAHARA.SERVICEGROUP
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SERVICEGROUP_CURRENT is 
'This view filters the SERVICEGROUP table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.SERVICEGROUP_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: SERVICEGROUP_HOST_CURRENT                              */
/*==============================================================*/
create view SAHARA.SERVICEGROUP_HOST_CURRENT  as
select
   SERVICEGROUP_HOST_KEY,
   SERVICEGROUP_KEY,
   HOST_KEY,
   SERVICEGROUP_OID,
   HOST_OID,
   CC_STORAGE_URL,
   DISPLAY_NAME
from
   SAHARA.SERVICEGROUP_HOST
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

grant SELECT on SAHARA.SERVICEGROUP_HOST_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: SF_LICENSE_CURRENT                                     */
/*==============================================================*/
create view SAHARA.SF_LICENSE_CURRENT  as
select
   SF_LICENSE_KEY,
   SF_LICENSE_OID,
   CC_STORAGE_URL,
   TYPE,
   "KEY",
   IS_SITE_LICENSE,
   SERIAL_NUMBER,
   IS_EXPIRED,
   IS_KEYLESS
from
   sahara.SF_LICENSE
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SF_LICENSE_CURRENT is 
'This view filters the SF_LICENSE table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

/*==============================================================*/
/* View: SF_LICENSE_DEPLOYMENT_CURRENT                          */
/*==============================================================*/
create view SAHARA.SF_LICENSE_DEPLOYMENT_CURRENT  as
select
   SF_LICENSE_DEPLOYMENT_KEY,
   SF_LICENSE_DEPLOYMENT_OID,
   SF_LICENSE_KEY,
   SF_PRODUCT_KEY,
   SF_LICENSE_HOST_KEY,
   CC_STORAGE_URL,
   EXPIRY_DATE,
   IS_EXPIRED,
   IS_ACCOUNTABLE
from
   sahara.SF_LICENSE_DEPLOYMENT
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SF_LICENSE_DEPLOYMENT_CURRENT is 
'This view filters the SF_LICENSE_DEPLOYMENT table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

/*==============================================================*/
/* View: SF_LICENSE_HOST_CURRENT                                */
/*==============================================================*/
create view SAHARA.SF_LICENSE_HOST_CURRENT  as
select
   SF_LICENSE_HOST_KEY,
   SF_LICENSE_HOST_OID,
   HOST_KEY,
   CC_STORAGE_URL,
   ID,
   NAME,
   MODEL,
   PROCESSOR_TYPE,
   NUM_PROCESSORS,
   PLATFORM,
   SERVER_PRICE_TIER,
   PROCESSOR_PRICE_TIER,
   NUM_PROCESSOR_PRICE_TIER,
   OS_PRICE_TIER,
   ASSIGNED_PRICE_TIER,
   OS_EDITION,
   OS,
   HAS_ENTERPRISE_EDITION,
   SPVU,
   NCORE
from
   sahara.SF_LICENSE_HOST
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SF_LICENSE_HOST_CURRENT is 
'This view filters the SF_LICENSE_HOST table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

/*==============================================================*/
/* View: SF_PRODUCT_CURRENT                                     */
/*==============================================================*/
create view SAHARA.SF_PRODUCT_CURRENT  as
select
   SF_PRODUCT_KEY,
   SF_PRODUCT_OID,
   CC_STORAGE_URL,
   PRODUCT_NAME,
   SHORT_NAME,
   PRODUCT_CATEGORY,
   PRODUCT_EDITION,
   PRODUCT_VERSION,
   PLATFORM,
   LICENSE_TYPE
from
   sahara.SF_PRODUCT
where
   T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SF_PRODUCT_CURRENT is 
'This view filters the SF_PRODUCT table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplest access to a current inventory of objects.';

/*==============================================================*/
/* View: SRC_CUSTOM_ATTRIBUTE                                   */
/*==============================================================*/
create view SAHARA.SRC_CUSTOM_ATTRIBUTE  as
select distinct SRC_CUSTOM_ATTRIBUTE_BASE.CUSTOM_ATTRIBUTE_CONTEXT, SRC_CUSTOM_ATTRIBUTE_BASE.NAME
  from SAHARA.SRC_CUSTOM_ATTRIBUTE_BASE
    join SAHARA.ROLLUP_SOURCE on SRC_CUSTOM_ATTRIBUTE_BASE.rollup_source_key=ROLLUP_SOURCE.rollup_source_key;

comment on view SAHARA.SRC_CUSTOM_ATTRIBUTE is 
'This view provides the names of all custom attributes discovered from roll-up sources.

This view is used in building and configuring Enterprise Reporter custom attributes; it is not intended in any way to be used for reporting.';

grant SELECT on SAHARA.SRC_CUSTOM_ATTRIBUTE to SAHARA_APP;

/*==============================================================*/
/* View: SWITCH_CURRENT                                         */
/*==============================================================*/
create view SAHARA.SWITCH_CURRENT  as
select SWITCH_KEY,
       SWITCH_OID,
       CC_STORAGE_URL,
       DISPLAY_NAME,
       ADVISORY_DISPLAY_NAME,
       VENDOR,
       DISCOVERY_STATE,
       FIRMWARE_VERSION,
       MODEL,
       SNMP_IP_ADDRESS,
       SWITCH_ROLE,
       STATUS,
       FABRIC_NAME,
       WWN,
       IS_PHYSICAL,
       HAS_VIRTUAL_SUPPORT,
       HAS_MULTIPLEX_SUPPORT,
       HAS_BUNDLE_SUPPORT
  from SAHARA.SWITCH
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.SWITCH_CURRENT is 
'This view filters the SWITCH table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.SWITCH_CURRENT to SAHARA_APP;

/*==============================================================*/
/* View: TABLE_METADATA                                         */
/*==============================================================*/
create view SAHARA.TABLE_METADATA  as
select TABLE_METADATA_BASE.OWNER,
       TABLE_METADATA_BASE.TABLE_NAME,
       TABLE_METADATA_BASE.OWNER || '.' || TABLE_METADATA_BASE.TABLE_NAME as FULL_TABLE_NAME,
       TABLE_METADATA_BASE.DISPLAY_NAME,
       (CASE ST.TABLE_TYPE
            WHEN 'BASE' THEN 'TABLE'
            WHEN 'VIEW' THEN 'VIEW'
            WHEN 'MAT VIEW' THEN 'VIEW'
            ELSE 'UNDEFINED'
        END) as TABLE_TYPE,
       ST.REMARKS
  from SYS.SYSTABLE ST
  join SYS.SYSUSER SU on ST.creator = SU.user_id
  join SAHARA.TABLE_METADATA_BASE 
    on TABLE_METADATA_BASE.OWNER = SU.user_name
   and TABLE_METADATA_BASE.TABLE_NAME = ST.TABLE_NAME;

grant SELECT on SAHARA.TABLE_METADATA to SAHARA_APP;

/*==============================================================*/
/* View: UNUSED_SRC_CUSTOM_ATTRIBUTE                            */
/*==============================================================*/
create view SAHARA.UNUSED_SRC_CUSTOM_ATTRIBUTE  as
select distinct CUSTOM_ATTRIBUTE_CONTEXT, NAME
  from SAHARA.SRC_CUSTOM_ATTRIBUTE
 where NAME not in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY')
   and not exists (select 1
                     from SAHARA.CUSTOM_ATTRIBUTE
                    where SRC_CUSTOM_ATTRIBUTE.CUSTOM_ATTRIBUTE_CONTEXT =
                          CUSTOM_ATTRIBUTE.CUSTOM_ATTRIBUTE_CONTEXT
                      and CUSTOM_ATTRIBUTE.TYPE = 'SOURCE COPY'
                      and CUSTOM_ATTRIBUTE.SOURCE_NAME = SRC_CUSTOM_ATTRIBUTE.NAME);

comment on view SAHARA.UNUSED_SRC_CUSTOM_ATTRIBUTE is 
'This view provides the names of all custom attributes discovered from roll-up sources that have not yet been configured as Enterprise Reporter custom attributes.

This view is used in building and configuring Enterprise Reporter custom attributes; it is not intended in any way to be used for reporting.';

grant SELECT on SAHARA.UNUSED_SRC_CUSTOM_ATTRIBUTE to SAHARA_APP;

/*==============================================================*/
/* View: VIEW_BROKER_INFO                                       */
/*==============================================================*/
create view SAHARA.VIEW_BROKER_INFO  as
select
   SAHARA.BROKER_INFO.BROKER_ID,
   SAHARA.BROKER_INFO.DOMAIN_NAME,
   SAHARA.BROKER_INFO.DOMAIN_TYPE,
   SAHARA.BROKER_INFO.BROKER_NAME,
   SAHARA.BROKER_INFO.BROKER_TYPE,
   SAHARA.BROKER_INFO.BROKER_PORT,
   SAHARA.BROKER_INFO.STATUS,
   SAHARA.BROKER_INFO.LDAP_SERVER,
   SAHARA.BROKER_INFO.BIND_USER_NAME,
   SAHARA.BROKER_INFO.BIND_USER_PASSWORD,
   SAHARA.BROKER_INFO.SEARCH_USER,
   SAHARA.BROKER_INFO.SEARCH_GROUP,
   SAHARA.BROKER_INFO.SEARCH_BASE,
   SAHARA.BROKER_INFO.SSL_CERT_LOCATION
from
   SAHARA.BROKER_INFO;

comment on view SAHARA.VIEW_BROKER_INFO is 
'View directly mapped to the SAHARA.BROKER_INFO table.

This view provides AT Broker information (LDAP and AD information) ';

grant SELECT on SAHARA.VIEW_BROKER_INFO to SAHARA_APP;

/*==============================================================*/
/* View: VIEW_SECURITY_GROUP                                    */
/*==============================================================*/
create view SAHARA.VIEW_SECURITY_GROUP  as
select SECURITY_GROUP.SECURITY_GROUP_ID, 
       SECURITY_GROUP.NAME as SG_NAME, 
       SECURITY_GROUP.PRIVILEGE, 
       SECURITY_GROUP.DESCRIPT as SG_DESCRIPT,
       SECURITY_GROUP.BROKER_ID, 
       SECURITY_GROUP.ROLE_ID,
       BROKER_INFO.BROKER_NAME, 
       BROKER_INFO.DOMAIN_TYPE, 
       BROKER_INFO.DOMAIN_NAME, 
       SECURITY_ROLE.NAME as SR_NAME, 
       SECURITY_ROLE.DESCRIPT as SR_DESCRIPT   
  from SAHARA.SECURITY_GROUP
  left outer join SAHARA.BROKER_INFO 
    on BROKER_INFO.BROKER_ID = SECURITY_GROUP.BROKER_ID
  left outer join SAHARA.SECURITY_ROLE 
    on SECURITY_ROLE.ROLE_ID = SECURITY_GROUP.ROLE_ID;

comment on view SAHARA.VIEW_SECURITY_GROUP is 
'This view directly mapped to the SAHARA.SECURITY_GROUP table.

This view provides a simple way to visualize the relationships between security groups, their roles and broker information.';

grant SELECT on SAHARA.VIEW_SECURITY_GROUP to SAHARA_APP;

/*==============================================================*/
/* View: VIEW_SECURITY_ROLE                                     */
/*==============================================================*/
create view SAHARA.VIEW_SECURITY_ROLE  as
select
   SAHARA.SECURITY_ROLE.ROLE_ID,
   SAHARA.SECURITY_ROLE.NAME,
   SAHARA.SECURITY_ROLE.DESCRIPT
from
   SAHARA.SECURITY_ROLE;

comment on view SAHARA.VIEW_SECURITY_ROLE is 
'View directly mapped to the SAHARA.SECURITY_ROLE table.

This view provides Security Role Information.';

grant SELECT on SAHARA.VIEW_SECURITY_ROLE to SAHARA_APP;

/*==============================================================*/
/* View: VOLUME_CURRENT                                         */
/*==============================================================*/
create view SAHARA.VOLUME_CURRENT  as
select VOLUME_KEY,
       VOLUME_OID,
       CC_STORAGE_URL,
       DISKGROUP_KEY,
       HOST_KEY,
       VOLUME_NAME,
       VOLUME_NAME AS DISPLAY_NAME,
       VM_VERSION,
       VM_TYPE,
       DISK_GROUP_NAME,
       VOLUME_LAYOUT,
       IS_VCS_CVM,
       IS_USED_BY_FILESYSTEM,
       IS_USED_BY_APPLICATION,
       IS_DIRECT_ATTACHED,
       IS_COMPONENT,
       IS_BASED_ON_THIN,
       IS_EXPORTED,
       VOLUME_TYPE,
       APPLICATION_BE,
       APPLICATION_COMPOSITE_BE,
       ORGANIZATION_BE,
       ORGANIZATION_COMPOSITE_BE,
       IS_USED_BY_DB_VM,
       DB_VM_NAME
  from SAHARA.VOLUME
 where T2_IS_CURRENT = 'T'
   and T2_IS_DELETED = 'F';

comment on view SAHARA.VOLUME_CURRENT is 
'This view filters the VOLUME table to just those rows which represent "current" active objects.  Obsolete "duplicate" rows from "type 2" updates and rows marked deleted are excluded.  The *_CURRENT views provide the simplist access to a current inventory of objects.';

grant SELECT on SAHARA.VOLUME_CURRENT to SAHARA_APP;

--
-- $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
-- $Revision: 1.119.4.1 $
--
-- $Copyright: Copyright (c) 2012 Symantec Corporation.
-- All rights reserved.
--
-- THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
-- SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
-- WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
--
-- The Licensed Software and Documentation are deemed to be commercial
-- computer software as defined in FAR 12.212 and subject to restricted
-- rights as defined in FAR Section 52.227-19 "Commercial Computer
-- Software - Restricted Rights" and DFARS 227.7202, "Rights in
-- Commercial Computer Software or Commercial Computer Software
-- Documentation", as applicable, and any successor regulations. Any use,
-- modification, reproduction release, performance, display or disclosure
-- of the Licensed Software and Documentation by the U.S. Government
-- shall be solely in accordance with the terms of this Agreement.  $

--
-- The stored proc SP_saharaAPIInit() is provided to create and initialize all
-- required connectionvariables used by many modules, like LOAD/IE/BV/TIER etc.
--
-- NOTE: All clients are required to call this stored proc immediately after
-- connecting to sahara DB, before calling any other stored proc from those
-- modules. Typical clients are GUI, kettle jobs, import/export BV CLIs etc.
-- 
--

IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_saharaAPIInit')THEN
      DROP PROCEDURE "SAHARA"."SP_saharaAPIInit";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_saharaAPIInit" ()
BEGIN

   	-- 
    -- *******************************************************************
    -- Global variable to make SP_saharaAPIInit reentrant 
    -- If not present then create it , set it and then go ahead with rest 
	-- variable creation. If present, just return.
    -- *******************************************************************
    --
	if  ( VAREXISTS('@gIsSaharaAPIInitialized') = 0 ) then
		create variable @gIsSaharaAPIInitialized bit;
	    set @gIsSaharaAPIInitialized = 0;
	else
		return;
	end if;
	
	-- 
    -- *******************************************************************
    -- Connection veriables, used as constsnts, to represent error codes.
    -- These error codes are returned by sahara stored procs to callers.
    -- Any new errors codes to be returned by various modules should be defined
    -- here.
    -- *******************************************************************
    --
    create variable @EInvalidArgument numeric(10);
    create variable @ERollupSourceNotFound numeric(10);
    create variable @ERollupBatchNotFound numeric(10);
    create variable @ERollupDataSetTypeNotFound numeric(10);
    create variable @ERUDSTypeStereotype numeric(10);
    create variable @ERUDSCLoadProcessing numeric(10);
    create variable @ERollupSourceNotEnabled numeric(10);
    create variable @ETableNotFound numeric(10);
    create variable @ENoLoadControl numeric(10);
    create variable @ENoIEControl numeric(10);
    create variable @ENoInitializeLoad numeric(10);
    create variable @EDBMSLockRequestFailed numeric(10);
    create variable @EDBMSLockReleaseFailed numeric(10);
    create variable @EDataSetLoadFailed numeric(10);
    create variable @ETimeout numeric(10);
    create variable @ENoConfigurationControl numeric(10);
    create variable @ERollupNowSourceNotEnabled numeric(10);
    create variable @ERollupNowRunningBatch numeric(10);
    create variable @ERollupNowAlreadyFlagged numeric(10);
    create variable @EBViewNotFound numeric(10);
    create variable @EBViewDimensionNotFound numeric(10);
    create variable @EBViewRootNodeUse numeric(10);
    create variable @EBViewNodeNotFound numeric(10);
    create variable @ERollupSchedule numeric(10);
    create variable @EConfigWrongType numeric(10);
    create variable @EConfigNotFound numeric(10);
    create variable @ERollupScheduleStartDate numeric(10);
    create variable @EActiveRollup numeric(10);
    create variable @EScheduleHash numeric(10);
    create variable @EPartialLoad numeric(10);
    create variable @ETierContextNotFound numeric(10);
    create variable @ETierNotFound numeric(10);
    create variable @ERuleContextNotFound numeric(10);
    create variable @ETierRuleValidateFailed numeric(10);
    create variable @ECustomAttrContextNotFound numeric(10);
    create variable @ECustomAttrNotFound numeric(10);
    create variable @ECADisplayNameNotUnique numeric(10);
    create variable @ECAColumnNameNotUnique numeric(10);
    create variable @EBViewInvalidName numeric(10);
    create variable @ERollupSourceInvalidName numeric(10);
    create variable @ESessionNotFound numeric(10);
    create variable @ENoUserForSession numeric(10);
    create variable @ECognosUpdateNotFound numeric(10);
    create variable @ENoCognosModelControl numeric(10);
    create variable @ELicenseKeyNotFound numeric(10);
    create variable @EBViewError numeric(10);
    create variable @ENoImoprtExportControl numeric(10);
    create variable @ETierError numeric(10);
    create variable @EStageError numeric(10);
    create variable @ERuleError numeric(10);
    create variable @ERollupSourceExists numeric(10);
   
    set @EInvalidArgument =20000;
    set @ERollupSourceNotFound =20004;
    set @ERollupBatchNotFound =20001;
    set @ERollupDataSetTypeNotFound =20002;
    set @ERUDSTypeStereotype =20007;
    set @ERUDSCLoadProcessing =20006;
    set @ERollupSourceNotEnabled =20003;
    set @ETableNotFound =20005;
    set @ENoLoadControl =20008;
    set @ENoIEControl =20009;
    set @ENoInitializeLoad =20010;
    set @EDBMSLockRequestFailed =20011;
    set @EDBMSLockReleaseFailed =20012;
    set @EDataSetLoadFailed =20013;
    set @ETimeout =20014;
    set @ENoConfigurationControl =20015;
    set @ERollupNowSourceNotEnabled =20016;
    set @ERollupNowRunningBatch =20017;
    set @ERollupNowAlreadyFlagged =20018;
    set @EBViewNotFound =20019;
    set @EBViewDimensionNotFound =20020;
    set @EBViewRootNodeUse =20021;
    set @EBViewNodeNotFound =20022;
    set @ERollupSchedule =20023;
    set @EConfigWrongType =20024;
    set @EConfigNotFound =20025;
    set @ERollupScheduleStartDate =20026;
    set @EActiveRollup =20027;
    set @EScheduleHash =20028;
    set @EPartialLoad =20029;
    set @ETierContextNotFound =20030;
    set @ETierNotFound =20031;
    set @ERuleContextNotFound =20032;
    set @ETierRuleValidateFailed =20033;
    set @ECustomAttrContextNotFound =20034;
    set @ECustomAttrNotFound =20035;
    set @ECADisplayNameNotUnique =20036;
    set @ECAColumnNameNotUnique =20037;
    set @EBViewInvalidName =20038;
    set @ERollupSourceInvalidName =20039;
    set @ESessionNotFound =20040;
    set @ENoUserForSession =20041;
    set @ECognosUpdateNotFound =20042;
    set @ENoCognosModelControl =20043;
    set @ELicenseKeyNotFound =20044;
    set @EBViewError =20045;
    set @ENoImoprtExportControl = 20046;
	set @ETierError = 20047;
	set @EStageError = 20048;
	set @ERuleError = 20049;
	set @ERollupSourceExists = 20050;


    -- 
    -- ***************************************************
    -- ERROR Module connection variables
    -- ***************************************************
    -- 
    create variable @gLastErrorCode  numeric(10);
    create variable @gLastErrorMessage varchar(255);


    -- 
    -- ***************************************************
    -- LOG Module connection variables
    -- ***************************************************
    -- 

    -- A constant that defines the section in which configuration
    -- for message logging is found
    create variable @gLog_CONFIG_SECTION varchar(15);
    set @gLog_CONFIG_SECTION = 'MESSAGE_LOGGING';

    --
    -- A boolean flag that determines if debug messages are logged.
    -- This value is set in this stored proc, which is supposed to called i
    -- only once per connection, so changing the configuration requires 
    -- connections to be re-created to see the change.
    --
    create variable @gLogDebugEnabled bit;

    -- TODO
    -- Need to pcik the value from the configuration table
    -- the old code need is below the new code need to  pick up from the below command.    
    --@gLogDebugEnabled := GetConfigBool(gCONFIG_SECTION, 'DEBUG_ENABLED');
    set @gLogDebugEnabled = 1;

    -- Set this option to OFF, so that when an exception is raised, the
    -- execution flow will stop there and go back to the caller.
    -- We rely on this to return the exception to caller from many of our
    -- stored procs.
    set temporary option CONTINUE_AFTER_RAISERROR = OFF;


    -- 
    -- ******************************************
    -- LOAD module specifc connection variables:
    -- ******************************************
    -- 
    create variable @gLoadHaveControl bit;
    create variable @gLoadActiveSessionKey numeric(10);
    create variable @gLoadActiveBatchKey numeric(10);

    set @gLoadHaveControl = 0;
    
    -- 
    -- ***********************************************************
    -- CONFIGURATION CONTROL module specifc connection variables:
    -- ***********************************************************
    -- 
    create variable @gConfigHaveControl bit;
    create variable @gConfigActiveSessionKey numeric(10);

    set @gConfigHaveControl = 0;
    
	-- 
    -- ***********************************************************
    -- COGNOS METAMODEL UPDATE module specifc connection variables:
    -- ***********************************************************
    -- 
    create variable @gCMUpdHaveControl bit;
    set @gCMUpdHaveControl = 0;

    --**************************************************
    -- Business View Module specific variable 
    --**************************************************
    --  Column Size Limitation 

    Create variable @gVIEW_NAME_LEN integer;
    Create variable @gLEVEL_NAME_LEN integer ;

    set @gVIEW_NAME_LEN = 10;
    set @gLEVEL_NAME_LEN = 30;

    -- 
    -- ***********************************************************
    -- Import Export module specifc connection variables:
    -- ***********************************************************
    -- 
    create variable @gIEHaveControl bit;
    create variable @gIEActiveSessionKey numeric(10);

    set @gIEHaveControl = 0;


    -- ***********************************************************
    -- Custom Attribute module specifc global variables:
    -- ***********************************************************
    -- 

    Create variable @gCOLUMN_NAME_LEN integer;
    Create variable @gDISPLAY_NAME_LEN integer ;

    set @gCOLUMN_NAME_LEN = 30;
    set @gDISPLAY_NAME_LEN = 64;


END;

grant execute on sahara.SP_saharaAPIInit to sahara_app;

/* global temp table creation */

IF EXISTS (SELECT 1 FROM sys.systable WHERE table_name = 'tmp_BViewKeyTable'  
			AND  table_type = 'GBL TEMP')
THEN
    DROP TABLE SAHARA.tmp_BViewKeyTable;
END IF;

create global temporary table SAHARA.tmp_BViewKeyTable(
BVIEW_KEY INTEGER,
SessionKey INTEGER default 0 ,
primary key (BVIEW_KEY,SessionKey)) on commit preserve rows;
------------------------------------------------------------------------------------------------------------------
--- no need to give permissions to othere user as these procs / functions will always be called 
--- with rights of calling users, will not be called directly . 

--- helper to drop SAHARA table , procedure and view

IF EXISTS (select * from sys.sysprocedure sp key join sysusers u
 where sp.proc_name = 'SP_DROP_OBJECT_SAHARA' and u.name = 'SAHARA' ) THEN
  DROP PROCEDURE SAHARA.SP_DROP_OBJECT_SAHARA;
END IF;


CREATE PROCEDURE SAHARA.SP_DROP_OBJECT_SAHARA( IN @pObjectType LONG VARCHAR,
                                      IN @pObjectName LONG VARCHAR)
BEGIN
  DECLARE @ct UNSIGNED INTEGER;
  DECLARE OBJECT_TYPE_NOTEXISTS EXCEPTION FOR
  SQLSTATE '99001'; 
  BEGIN
    SET @ct = 0;
    IF @pObjectType NOT IN ('TABLE','PROCEDURE', 'FUNCTION','VIEW','EVENT') THEN
     message 'Invalid Object Type '||@pObjectType to client;
     SIGNAL OBJECT_TYPE_NOTEXISTS;
    END IF;

    IF @pObjectType = 'TABLE' THEN 

        select count(1) into @ct 
        from systable t 
        key join sysusers u
        where t.table_name = @pObjectName
        and u.name = 'SAHARA';
       
    END IF;
    
    IF @pObjectType in ( 'PROCEDURE', 'FUNCTION') THEN 
        
        select count(1) into @ct  
        from sys.sysprocedure sp 
        key join sysusers u
        where sp.proc_name = @pObjectName and u.name = 'SAHARA'

    END IF;

    IF @pObjectType = 'VIEW' THEN 
       
        select count(1) into @ct 
        from systable t key join sysusers u
        where 
            t.table_name = @pObjectName and 
            t.table_type = 'VIEW' and 
            u.name = 'SAHARA';
       
    END IF;
    
    IF @pObjectType = 'EVENT' THEN 

    select count(1) into @ct  from sysevent e , sysuser o 
                where e.creator = o.user_id 
                and user_name = 'SAHARA'
                and event_name = @pObjectName;

   END IF;
    IF @ct > 0 THEN
       if @pObjectType = 'EVENT' THEN
          execute immediate 'BEGIN alter ' || @pObjectType || ' SAHARA."' || @pObjectName || '"'||' disable; COMMIT; END;';
       end if;
       execute immediate 'BEGIN drop ' || @pObjectType || ' SAHARA."' || @pObjectName || '"'||'; COMMIT; END;';
    END IF;
  END;
END;



CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_HELPER_ColumnExists');

CREATE FUNCTION SAHARA.fn_HELPER_ColumnExists(@pOwner varchar(30)  , @pTable VARCHAR(255),
                      @pColumn VARCHAR(255))
    RETURNS BIT 

BEGIN
  DECLARE @vCt integer;

  SET @vCt = 0;

   select  count(1)
      into @vCt from systabcol col, systable tab , sysuser u
   where col.table_id = tab.table_id and 
   tab.creator = u.user_id 
  and u.user_name = upper(@pOwner)
  and tab.table_name = upper(@pTable)
  and col.column_name = upper(@pColumn);

 
   IF @vCt > 0 THEN 
   --- say it exits 
     RETURN 1;
   ELSE 
   --- it does not exits 
     RETURN 0;
   END IF;
END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_HELPER_IndexExists');

CREATE FUNCTION SAHARA.fn_HELPER_IndexExists
(
    @pIndex long varchar,
    @pOwner varchar(1024) default 'SAHARA'
)
    returns bit
BEGIN 
  DECLARE @vCt integer;
  BEGIN

    select count(1) into @vCt from sysindex ind, sysuser u
    where ind.creator = u.user_id
    and u.user_name = @pOwner
    and ind.index_name = @pIndex;
    
    IF @vCt > 0 THEN 
      RETURN 1;
    ELSE 
      RETURN 0;
    END IF;
  END;
END;

message 'Creating procedure SP_DataDict_TableMetadata...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_TableMetadata' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_TableMetadata
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_TableMetadata" (@pOwner varchar(32), @pTableName varchar(128), @pDisplayName varchar(128))
BEGIN
		insert into sahara.TABLE_METADATA_BASE(OWNER, TABLE_NAME, DISPLAY_NAME) 
		on existing update values (@pOwner, @pTableName, @pDisplayName);
END;

grant execute on sahara.SP_DataDict_TableMetadata to sahara;



message 'Creating procedure SP_DataDict_ColumnMetadata...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_ColumnMetadata' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_ColumnMetadata
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_ColumnMetadata" (@pOwner varchar(32), @pTableName varchar(128), @pColumnName varchar(128), @pDisplayName varchar(128))
BEGIN
		insert into sahara.COLUMN_METADATA_BASE(OWNER, TABLE_NAME, COLUMN_NAME, DISPLAY_NAME)
		on existing update values (@pOwner, @pTableName, @pColumnName, @pDisplayName);
END;

grant execute on sahara.SP_DataDict_ColumnMetadata to sahara;

------------------------------------------------------------------------------------------------------------------

--read saharaPDDump.sql;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_BindRDSTypeToRSType...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_BindRDSTypeToRSType' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_BindRDSTypeToRSType
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_BindRDSTypeToRSType" (@pRollupSourceType varchar(64), @pRollupDataSetType varchar(128))

BEGIN
DECLARE @error integer;
DECLARE @sqlstate varchar(5);
DECLARE @serror varchar(512);
DECLARE DUP_VAL_ON_INDEX EXCEPTION FOR SQLSTATE '23W01';

	insert into SAHARA.RSOURCE_TYPE_DATA_SET_TYPE
        (ROLLUP_SOURCE_TYPE_KEY, ROLLUP_DATA_SET_TYPE_KEY)
    values
    (
        (select ROLLUP_SOURCE_TYPE_KEY
           from SAHARA.ROLLUP_SOURCE_TYPE
          where DISPLAY_NAME = @pRollupSourceType),
        (select ROLLUP_DATA_SET_TYPE_KEY
           from ROLLUP_DATA_SET_TYPE
          where NAME = @pRollupDataSetType)
    );
			
	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN
		-- the binding already exists, so there's nothing to do
			return ;
		WHEN others THEN
			SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
			message 'Error ' || @serror to client;
END;

grant execute on sahara.SP_DataDict_BindRDSTypeToRSType to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_BViewDimension...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_BViewDimension' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_BViewDimension
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_BViewDimension" (@pDimension varchar(32), @pDisplayName varchar(32), @pAlias varchar(32) default null) 
BEGIN
DECLARE @valias varchar(32);
SET @valias = @pAlias;

    -- provide a default alias value
    if @vAlias is null then
        SET @vAlias = @pDimension;
    end if;

	insert into sahara.BVIEW_DIMENSION(DIMENSION, ALIAS, DISPLAY_NAME)
	on existing update values (@pDimension, @vAlias, @pDisplayName);
END;

grant execute on sahara.SP_DataDict_BViewDimension to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_CustomAttributeContext...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_CustomAttributeContext' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_CustomAttributeContext
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_CustomAttributeContext" (@pContext varchar(32), @pDisplayName varchar(32)) 
BEGIN
		insert into sahara.CUSTOM_ATTRIBUTE_CONTEXT(CUSTOM_ATTRIBUTE_CONTEXT, DISPLAY_NAME)
		on existing update values (@pContext, @pDisplayName);
END;

grant execute on sahara.SP_DataDict_CustomAttributeContext to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_FCastDimension...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_FCastDimension' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_FCastDimension
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_FCastDimension" (@pDimension varchar(32), @pDisplayName varchar(32), @pModelPath varchar(1024),@pDBFctTable varchar(1024), @pModelKey varchar(1024)) 
BEGIN

	insert into sahara.FORECASTING_DIMENSION(DIMENSION, DISPLAY_NAME,METAMODEL_PATH,FACT_TABLE_NAME,METAMODEL_KEY)
	on existing update values (@pDimension, @pDisplayName, @pModelPath, @pDBFctTable, @pModelKey);
END;

grant execute on sahara.SP_DataDict_FCastDimension to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_FCAST_GetObjAttrValues...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_FCAST_GetObjAttrValues' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_FCAST_GetObjAttrValues
end if;

CREATE PROCEDURE SAHARA.SP_FCAST_GetObjAttrValues
(
 IN @tableName varchar(128), 
 IN @colname varchar(128)
  )
RESULT(val varchar(512))
BEGIN
    DECLARE @SQLQuery long varchar;
    set @SQLQuery = 'select distinct(' + @colname + ') from sahara.' + @tableName + '_current;';
   execute immediate @SQLQuery;
END;
grant execute on sahara.SP_FCAST_GetObjAttrValues to sahara;
grant execute on sahara.SP_FCAST_GetObjAttrValues to sahara_reports;
------------------------------------------------------------------------------------------------------------------------
message 'Creating procedure SP_DataDict_EnableFCastSchema...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_EnableFCastSchema' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_EnableFCastSchema
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_EnableFCastSchema" (@pTableName varchar(32), @pColumnName varchar(32)) 
BEGIN

	update sahara.COLUMN_METADATA_BASE
		set IS_FORECAST_ENABLE = 'T'
	where TABLE_NAME=@pTableName
		 and COLUMN_NAME=@pColumnName;

END;

grant execute on sahara.SP_DataDict_EnableFCastSchema to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating function fn_BoolToChar...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_BoolToChar' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_BoolToChar
end if;

CREATE FUNCTION "SAHARA"."fn_BoolToChar" (@value bit) 
returns char(1)
BEGIN

	case 
		when @value is null then return
		when @value = 1 then return 'T'
		else return 'F'
    end

END;

grant execute on sahara.fn_BoolToChar to sahara;

------------------------------------------------------------------------------------------------------------------

/*
 **********************************************************************
 * fn_CharToBool():
 * 
 * Converts a character into a bit.
 * 
 * If the argument is 'T' or 't', the function returns true.
 * If the argument is 'F' or 'f', the function returns false.
 * If the argument is null, the function returns null.
 * All other values result in an exception.
 **********************************************************************
 */
message 'Creating function fn_CharToBool...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_CharToBool' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_CharToBool
end if;

CREATE FUNCTION "SAHARA"."fn_CharToBool" (@value char(1)) 
returns bit
BEGIN

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	if @value in ('T', 't') then
		return 1;
	else 
		if @value in ('F', 'f') then
			return 0;
		else 
			if @value is null then
				return null;
			else
				CALL SP_Error_RaiseApplicationError
				(
				@EInvalidArgument,
				'invalid value specified for boolean value: ' || @value
				);
				
			end if;
		end if;
	end if;

END;

grant execute on sahara.fn_CharToBool to sahara_app;

------------------------------------------------------------------------------------------------------------------

message 'Creating function fn_TableOrViewExists...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_TableOrViewExists' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_TableOrViewExists
end if;

CREATE FUNCTION "SAHARA"."fn_TableOrViewExists" (@pName varchar(32), @pOwner varchar(16) default 'SAHARA') 
returns bit
BEGIN
DECLARE @vCt int;

	select count(*)
      into @vCt
      from SYS.systable
     where user_name(creator) = @pOwner
       and TABLE_NAME = @pName;

    if @vCt = 1 then
        return 1;
    end if;

    select count(*)
      into @vCt
      from SYS.sysviews
     where vcreator = @pOwner
       and viewname = @pName;

	if @vCt = 1 then
        return 1;
	else
		return 0;
    end if;

END;

grant execute on sahara.fn_TableOrViewExists to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating function fn_TableExists...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_TableExists' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_TableExists
end if;

CREATE FUNCTION "SAHARA"."fn_TableExists" (@pName varchar(32), @pOwner varchar(16) default 'SAHARA') 
returns bit
BEGIN
DECLARE @vCt int;

	select count(*)
      into @vCt
      from SYS.systable
     where user_name(creator) = @pOwner
       and TABLE_NAME = @pName;
 
	if @vCt = 1 then
        return 1;
	else
		return 0;
    end if;

END;

grant execute on sahara.fn_TableExists to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating function fn_ViewExists...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_ViewExists' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_ViewExists
end if;

CREATE FUNCTION "SAHARA"."fn_ViewExists" (@pName varchar(32), @pOwner varchar(16) default 'SAHARA') 
returns bit
BEGIN
DECLARE @vCt int;

	select count(*)
      into @vCt
      from SYS.sysviews
     where vcreator = @pOwner
       and viewname = @pName;

	if @vCt = 1 then
        return 1;
	else
		return 0;
    end if;

END;

grant execute on sahara.fn_ViewExists to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RollupBatchStatus...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RollupBatchStatus' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RollupBatchStatus
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RollupBatchStatus" (@pStatus varchar(32), @pDescription long varchar, @pIsComplete bit default 0, @pIsSuccess bit default 0) 
BEGIN
DECLARE @vIsComplete char(1);
DECLARE @vIsSuccess char(1);

SET @vIsComplete = fn_BoolToChar(@pIsComplete) ;
SET @vIsSuccess = fn_BoolToChar(@pIsSuccess) ;

	insert into sahara.ROLLUP_BATCH_STATUS(STATUS, DESCRIPTION, IS_COMPLETE, IS_SUCCESS)
	on existing update values (@pStatus, @pDescription,@vIsComplete,@vIsSuccess);

END;

grant execute on sahara.SP_DataDict_RollupBatchStatus to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RollupDataSetColumn...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RollupDataSetColumn' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RollupDataSetColumn
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RollupDataSetColumn" (@pDataSetType varchar(32), @pName varchar(30), @pLoadProcessing varchar(32), @pSourceExpression varchar(128) default null) 
BEGIN
DECLARE @dataSetTypeKey numeric(10);
DECLARE @sqlstate varchar(5);

	-- validate the data set type and get its key
	select ROLLUP_DATA_SET_TYPE_KEY
		into @dataSetTypeKey
    from sahara.ROLLUP_DATA_SET_TYPE
    where NAME = @pDataSetType;

	SELECT SQLSTATE INTO @sqlstate;

	if @sqlstate=02000 then
		raiserror 30001 'non-existent roll-up data set type: ' || @pDataSetType;
		return;
	end if;


	insert into sahara.ROLLUP_DATA_SET_COLUMN(ROLLUP_DATA_SET_TYPE_KEY, NAME, LOAD_PROCESSING, SOURCE_EXPRESSION) on existing update values (@dataSetTypeKey, @pName, @pLoadProcessing, @pSourceExpression)


END;

grant execute on sahara.SP_DataDict_RollupDataSetColumn to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RollupSourceType...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RollupSourceType' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RollupSourceType
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RollupSourceType" (@pDisplayName varchar(64), @pLoadProcedure varchar(32)) 
BEGIN
 	if exists(select 1 from sahara.ROLLUP_SOURCE_TYPE where DISPLAY_NAME = @pDisplayName) then
		-- update first
		update sahara.ROLLUP_SOURCE_TYPE set LOAD_PROCEDURE = @pLoadProcedure where DISPLAY_NAME = @pDisplayName
	else
		-- if nothing was found to update, do an insert
		insert into sahara.ROLLUP_SOURCE_TYPE(DISPLAY_NAME, LOAD_PROCEDURE) values (@pDisplayName, @pLoadProcedure)
	end if;
END;

grant execute on sahara.SP_DataDict_RollupSourceType to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RuleContext...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RuleContext' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RuleContext
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RuleContext" (@pContext varchar(32), @pHelpHTML long varchar) 
BEGIN
	insert into sahara.RULE_CONTEXT(RULE_CONTEXT, HELP_HTML) 
	on existing update values (@pContext, @pHelpHTML)
END;

grant execute on sahara.SP_DataDict_RuleContext to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RuleElement...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RuleElement' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RuleElement
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RuleElement" (@pContext varchar(32), @pSectionName varchar(64),@pDisplayName varchar(64),@pDescription long varchar,@pType varchar(32),@pRuleToken varchar(128),@pSQL varchar(1024)) 
BEGIN
	insert into sahara.RULE_ELEMENT(RULE_CONTEXT, SECTION_DISPLAY_NAME, DISPLAY_NAME, DESCRIPTION,
		 TYPE, RULE_TOKEN, SQL_CODE) 
	 on existing update values (@pContext, @pSectionName, @pDisplayName, @pDescription,
	 @pType, @pRuleToken, @pSQL)

END;

grant execute on sahara.SP_DataDict_RuleElement to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RuleElement...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RuleElementSection' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RuleElementSection
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RuleElementSection" (@pContext varchar(32), @pDisplayName varchar(64),@pDescription long varchar default null) 
BEGIN
	insert into sahara.RULE_ELEMENT_SECTION(RULE_CONTEXT, SECTION_DISPLAY_NAME, DESCRIPTION)
	on existing update values (@pContext, @pDisplayName, @pDescription)
END;

grant execute on sahara.SP_DataDict_RuleElementSection to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_TierContext...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_TierContext' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_TierContext
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_TierContext" (@pContext varchar(32), @pCostUnit varchar(64),@pRuleSQL long varchar) 
BEGIN
DECLARE @vRuleSQL long varchar;
SET @vRuleSQL = trim(@pRuleSQL);

	insert into sahara.TIER_CONTEXT(TIER_CONTEXT, COST_UNIT, RULE_SQL)
	on existing update values (@pContext, @pCostUnit, @vRuleSQL)

END;

grant execute on sahara.SP_DataDict_TierContext to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RollupDataSetType...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RollupDataSetType' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RollupDataSetType
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RollupDataSetType" (@pName varchar(64), @pStereotype varchar(64),@pDestTable varchar(32), @pStageTables varchar(32), @pDuplicateFactPurgeKeyCols varchar(128) default null, @pLoadSQL long varchar default null,@pCsvName varchar(64) default null,@pVOMType varchar(128) default null) 
BEGIN

DECLARE @vATable varchar(30);
DECLARE @vCurrentView varchar(30);
DECLARE @vLoadSQL long varchar;
DECLARE @vRollupDataSetTypeKey numeric(10);
DECLARE @vColumn varchar(30);
DECLARE @vDuplicateFactPurgeKeyCols varchar(128);

SET @vLoadSQL = trim(@pLoadSQL);
SET @vDuplicateFactPurgeKeyCols = trim(@pDuplicateFactPurgeKeyCols);

	-- we want all or nothing from this call
    SAVEPOINT DDRollupDataSetTypeSP;

    case 
        when @pStereotype='DIMENSION' then
            set @vATable = @pDestTable || '_';
            set @vCurrentView = @pDestTable || '_CURRENT';

        when @pStereotype='FACT' then
            set @vATable = @pDestTable || '_'

        when @pStereotype='CUSTOM' then
            set @vATable = null

        else
			raiserror 30002 'unknown stereotype for roll-up data set type: ' || @pStereotype;
			return;
    end;
	
	-- validate that the tables exist
    --
    -- dimension and facts require a loading source
    if @vATable is not null and fn_TableOrViewExists(@vATable, 'SAHARA_APP')=0 then
		raiserror 30003 'source table or view SAHARA_APP.' || @vATable || ' does not exist or cannot be accessed';
		return;
    end if;

    if @pDestTable is not null and fn_TableExists(@pDestTable)=0 then
		raiserror 30004 'destination table SAHARA.' || @pDestTable || ' does not exist or cannot be accessed';
		return;
    end if;

    if @vCurrentView is not null and fn_ViewExists(@vCurrentView)=0 then
		raiserror 30005 'current view SAHARA.' || @vCurrentView || ' does not exist or cannot be accessed';
		return;
    end if;
	
    -- provide a default load SQL for dimensions
    if @pStereotype = 'DIMENSION' and @vLoadSQL is null then
        set @vLoadSQL = 'select $SRC_COLUMNS$ from SAHARA_APP."$SRC_TABLE$" SRC';
    end if;

    -- tbd - validate that the duplicate fact purge key columns exist

   	if exists(select 1 from sahara.ROLLUP_DATA_SET_TYPE where NAME = @pName) then
		-- update first
		update sahara.ROLLUP_DATA_SET_TYPE set STEREOTYPE = @pStereotype, DEST_TABLE = @pDestTable, DUPLICATE_FACT_PURGE_KEY_COLS = @vDuplicateFactPurgeKeyCols, LOAD_SQL = @vLoadSQL, CSV_NAME = @pCsvName, VOM_OBJECT_TYPE = @pVOMType where NAME = @pName
	else
		-- if nothing was found to update, do an insert
		insert into sahara.ROLLUP_DATA_SET_TYPE(NAME, STEREOTYPE, DEST_TABLE, DUPLICATE_FACT_PURGE_KEY_COLS, LOAD_SQL,CSV_NAME,VOM_OBJECT_TYPE) values (@pName, @pStereotype, @pDestTable, @vDuplicateFactPurgeKeyCols, @vLoadSQL,@pCsvName,@pVOMType)
	end if;

	-- get the key to make the rest easier
    select ROLLUP_DATA_SET_TYPE_KEY
      into @vRollupDataSetTypeKey
      from SAHARA.ROLLUP_DATA_SET_TYPE
     where NAME = @pName;

    -- In addition to a loading source (which may be a view), we have staging
    -- table(s) which provide the data to load (views may draw their data
    -- from these).  The staging tables are purged at the start of a batch,
    -- so we need to know what they are.
    --
    -- clear any existing staging table entries.
    delete from SAHARA.ROLLUP_DATA_SET_STAGE
     where ROLLUP_DATA_SET_TYPE_KEY = @vRollupDataSetTypeKey;

    -- add the staging table records
    if @pStageTables is not null then

            SET @vATable = @pStageTables;

            -- validate that the table exist
            if fn_TableExists(@vATable, 'SAHARA_APP')=0 then
				raiserror 30006 'staging table SAHARA_APP.' || @vATable || ' does not exist or cannot be accessed';
				return;
            end if;

            insert into SAHARA.ROLLUP_DATA_SET_STAGE(ROLLUP_DATA_SET_TYPE_KEY, STAGE_TABLE)
            values(@vRollupDataSetTypeKey, @vATable);

    end if;

	-- For dimensions, make populating the data dictionary easier by adding
    -- all of the dimension columns as TYPE 1 if they aren't there yet.
    -- The data dictionary script can override ones that should be different.
    if @pStereotype = 'DIMENSION' then
        -- start with all of the columns in the dimension table except
        -- the surrogate key

		FOR @ColumnTypeLoop AS @ColumnTypeCursor CURSOR FOR (   
			select cname as vColumn
			from sys.syscolumns
			where creator = 'SAHARA' and tname = @pDestTable
			-- ignore custom attribute columns
			and cname not like 'CA\_%' escape '\'
			-- ignore special book-keeping columns
			and cname not like 'T2\_%' escape '\'

			and cname not in
			-- substract those that we already have metadata for
	        (select ROLLUP_DATA_SET_COLUMN.NAME
		      from sahara.ROLLUP_DATA_SET_COLUMN
			 where ROLLUP_DATA_SET_TYPE_KEY = @vRollupDataSetTypeKey)

			)

		DO
			-- add the unknown columns
			if vColumn is not null and vColumn!='' then
				call SP_DataDict_RollupDataSetColumn(@pName, vColumn, 'TYPE 1');
			end if;

		END FOR;

    end if;

EXCEPTION
	WHEN others THEN
		ROLLBACK TO SAVEPOINT DDRollupDataSetTypeSP;

END;

grant execute on sahara.SP_DataDict_RollupDataSetType to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_RuleElementsFromDimensionSrc...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_RuleElementsFromDimensionSrc' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_RuleElementsFromDimensionSrc
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_RuleElementsFromDimensionSrc" (@pContext varchar(32), @pSectionName varchar(64),@pSrcTable varchar(32), @pDimensionTable varchar(32), @pRuleTokenPrefix varchar(128) default null, @pExcludedColumns varchar(256) default null) 
BEGIN

DECLARE @vRuleTokenPrefix varchar(128);
DECLARE @vExcludedColumns varchar(256);

DECLARE @columnName varchar(30);
DECLARE @displayName varchar(64);
DECLARE @description long varchar;
DECLARE @ruleToken varchar(128);
DECLARE @sql_code varchar(1024);

SET @vRuleTokenPrefix = @pRuleTokenPrefix;
SET @vExcludedColumns = replace(@pExcludedColumns, ' ', '');

	-- we want all or nothing from this call
    SAVEPOINT DDRuleElementsFromDimSrcSP;

    -- validate that the tables exist
	if fn_TableOrViewExists(@pSrcTable, 'SAHARA_APP')=0 then
		raiserror 30004 'source table or view SAHARA_APP.' || @pSrcTable || ' does not exist or cannot be accessed';
		return;
    end if;

	if fn_TableOrViewExists(@pDimensionTable)=0 then
		raiserror 30005 'dimension table or view SAHARA.' || @pDimensionTable || ' does not exist or cannot be accessed';
		return;
    end if;

    -- implement a default for the rule token prefix
    if @vRuleTokenPrefix is null then
        SET @vRuleTokenPrefix = @pSectionName;
    end if;

	if @vExcludedColumns is not null then
		SET @vExcludedColumns = ',' || @vExcludedColumns || ',';
	else
		SET @vExcludedColumns = '** none **';
	end if;
	
	-- get the columns for which to create rule elements
	FOR @RuleElementLoop AS @RuleElementCursor CURSOR FOR (   
		select COLUMN_NAME AS columnName,
		DISPLAY_NAME AS displayName,
		REMARKS AS description,
		'[' || @vRuleTokenPrefix || '].[' || DISPLAY_NAME || ']' AS ruleToken,
		'"' || @pSrcTable || '"."' || COLUMN_NAME || '"' AS sql_code
		from COLUMN_METADATA
		where OWNER = 'SAHARA'
			and TABLE_NAME = @pDimensionTable
			and COLUMN_NAME in (select cname
									from sys.syscolumns
									where creator = 'SAHARA_APP'
									and tname = @pSrcTable)
		)
	DO

		if columnName is not null and columnName!='' then
			-- create each rule element, skipping the excluded ones
			if 0 = charindex(',' || columnName || ',',@vExcludedColumns) then
				call SP_DataDict_RuleElement(@pContext, @pSectionName,
							displayName,
							description,
							'COLUMN',                       
							ruleToken,
							sql_code);
			end if;

		end if;

	END FOR;


EXCEPTION
	WHEN others THEN
		ROLLBACK TO SAVEPOINT DDRuleElementsFromDimSrcSP;

END;

grant execute on sahara.SP_DataDict_RuleElementsFromDimensionSrc to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating procedure SP_DataDict_CONFIG...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataDict_CONFIG' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataDict_CONFIG
end if;

CREATE PROCEDURE "SAHARA"."SP_DataDict_CONFIG" (@pInsert long varchar, @pValues long varchar) 
BEGIN
	DECLARE @pSQL long varchar;
	SET @pSQL = @pInsert || ' ( ' || @pValues || ' ) ';

	execute immediate @pSQL;

END;

grant execute on sahara.SP_DataDict_CONFIG to sahara;

------------------------------------------------------------------------------------------------------------------

message 'Creating function fn_Conf_UTCTimestamp...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Conf_UTCTimestamp' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_Conf_UTCTimestamp
end if;

CREATE FUNCTION "SAHARA"."fn_Conf_UTCTimestamp" () 
returns datetime
BEGIN
	DECLARE @timestamp datetime;

	SET @timestamp=DATETIME(current utc timestamp);

	return @timestamp;

END;


grant execute on sahara.fn_Conf_UTCTimestamp to sahara;







/*
 **********************************************************************
 * fn_Conf_GetConfigDayInterval():
 * 
 * Gets a day interval configuration item value. 
 **********************************************************************
 */
message 'Creating function fn_Conf_GetConfigDayInterval...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Conf_GetConfigDayInterval' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_Conf_GetConfigDayInterval
end if;

CREATE FUNCTION "SAHARA"."fn_Conf_GetConfigDayInterval" 
(
    @pSection varchar(32),
    @pName varchar(32)
) 
returns numeric(10)
BEGIN

    DECLARE @vType varchar(32);
    DECLARE @vValue numeric(10);
	DECLARE @sqlstate varchar(5);

	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';

    select TYPE, INTEGER_VALUE
      into @vType, @vValue
      from sahara.CONFIGURATION
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigNotFound,
            'Failed getting a configuration item value because the item ' ||
            'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.'
		);
		return;
	end if;

    -- make sure the data type is correct
    if 'INTEGER' <> @vType then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigWrongType,
			'Failed getting an existing configuration item value of ' ||
			'type ' || @vType || ' as type DAY INTERVAL.  Section: ' || @pSection ||
			', Name: ' || @pName || '.'
		);
    end if;

    return @vValue;
end;

grant execute on sahara.fn_Conf_GetConfigDayInterval to SAHARA_APP;


/*
 **********************************************************************
 * fn_Conf_DatesKey():
 * 
 * Gets the dates key from the dates dimension (DATES table) for a date. 
 * The dates table row is inserted if it does not already exist.
 **********************************************************************
 */
message 'Creating function fn_Conf_DatesKey...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Conf_DatesKey' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_Conf_DatesKey
end if;

CREATE FUNCTION "SAHARA"."fn_Conf_DatesKey" 
(
    @pDate date
) 
returns numeric(10)
BEGIN
    DECLARE @vDate date;
	DECLARE @retry bit;
	DECLARE @rc numeric(10);
	DECLARE DUP_VAL_ON_INDEX EXCEPTION FOR SQLSTATE '23505';
	
	set @vdate = @pDate;
    set @retry = 1;
    
    -- allow for one retry
    loop
        -- most of the time, the row will already exist, so just select it
		select DATES_KEY
		  into @rc
		  from DATES
		 where ADATE = @vDate;

		if sqlstate=0 then
			return @rc;
		end if;


        -- if we got here, we didn't find it, so try to insert it
        begin
            insert into DATES(ADATE)
            values(@vDate);

			select max(DATES_KEY) into @rc from sahara.DATES; 
            return @rc;

        EXCEPTION
            when DUP_VAL_ON_INDEX then
                -- There is a slight chance of a concurrency issue here if
                -- another connection tries to add the date at the same time.
                if @retry=0 then
					CALL SP_Error_RaiseApplicationError(20050,'raise');
                end if;

                -- allow only one retry
                set @retry = 0;
        end;
    end loop;

EXCEPTION
    when others then
        rollback;
		CALL SP_Error_RaiseApplicationError(20050,'raiseME');
end;
grant execute on sahara.fn_Conf_DatesKey to SAHARA;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'fn_GetConfigBool' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.fn_GetConfigBool;
END IF;
create function sahara.fn_GetConfigBool(
     @pSection varchar(32),
    @pName varchar(32)
)
    returns bit
BEGIN
    declare @vType varchar(32);
    declare @vBoolean char(1);

    select TYPE, BOOLEAN_VALUE
      into @vType, @vBoolean
      from CONFIGURATION
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

    -- make sure the data type is correct
    if 'BOOLEAN' <> @vType then
        call SAHARA.SP_Error_RaiseApplicationError
        (
            @EConfigWrongType,
            'Failed getting an existing configuration item value of ' ||
            'type ' || @vType || ' as type BOOLEAN.  Section: ' || @pSection ||
            ', Name: ' || @pName || '.'
        );
    end if;

    return sahara.fn_CharToBool(@vBoolean);

exception
    when others then
        call SAHARA.SP_Error_RaiseApplicationError
        (
            @EConfigNotFound,
            'Failed getting a configuration item value because the item ' ||
            'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.'
        );
end;

/*
 **********************************************************************
 * fn_Conf_GetConfigInt():
 * 
 * Gets the integer configuration item value. 
 **********************************************************************
 */
message 'Creating function fn_Conf_GetConfigInt...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Conf_GetConfigInt' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_Conf_GetConfigInt
end if;

CREATE FUNCTION "SAHARA"."fn_Conf_GetConfigInt" 
(
    @pSection varchar(32),
    @pName varchar(32)
) 
returns integer
BEGIN

    DECLARE @vType varchar(32);
    DECLARE @vValue numeric(10);
	DECLARE @sqlstate varchar(5);

	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';

    -- initialize global variables
	call sahara.SP_saharaAPIInit();
	
	select TYPE, INTEGER_VALUE
      into @vType, @vValue
      from sahara.CONFIGURATION
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigNotFound,
            'Failed getting a configuration item value because the item ' ||
            'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.'
		);
		return;
	end if;

    -- make sure the data type is correct
    if 'INTEGER' <> @vType then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigWrongType,
			'Failed getting an existing configuration item value of ' ||
			'type ' || @vType || ' as type INTEGER.  Section: ' || @pSection ||
			', Name: ' || @pName || '.'
		);
    end if;

    return @vValue;
end;

grant execute on sahara.fn_Conf_GetConfigInt to SAHARA_APP;


/*
 **********************************************************************
 * fn_Conf_GetConfigStr():
 * 
 * Gets the string configuration item value. 
 **********************************************************************
 */
message 'Creating function fn_Conf_GetConfigStr...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Conf_GetConfigStr' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_Conf_GetConfigStr
end if;

CREATE FUNCTION "SAHARA"."fn_Conf_GetConfigStr" 
(
    @pSection varchar(32),
    @pName varchar(32)
) 
returns varchar(2000)
BEGIN

    DECLARE @vType varchar(32);
    DECLARE @vValue varchar(2000);
	DECLARE @sqlstate varchar(5);

	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';

    -- initialize global variables
	call sahara.SP_saharaAPIInit();
	
	select TYPE, STRING_VALUE
      into @vType, @vValue
      from sahara.CONFIGURATION
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigNotFound,
            'Failed getting a configuration item value because the item ' ||
            'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.'
		);
		return;
	end if;

    -- make sure the data type is correct
    if 'STRING' <> @vType then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigWrongType,
			'Failed getting an existing configuration item value of ' ||
			'type ' || @vType || ' as type STRING.  Section: ' || @pSection ||
			', Name: ' || @pName || '.'
		);
    end if;

    return @vValue;
end;

grant execute on sahara.fn_Conf_GetConfigStr to SAHARA_APP;

/*
 **********************************************************************
 * fn_Conf_GetConfigDate():
 * 
 * Gets the Date configuration item value. 
 **********************************************************************
 */
message 'Creating function fn_Conf_GetConfigDate...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Conf_GetConfigDate' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_Conf_GetConfigDate
end if;

CREATE FUNCTION "SAHARA"."fn_Conf_GetConfigDate" 
(
    @pSection varchar(32),
    @pName varchar(32)
) 
returns DATETIME
BEGIN

    DECLARE @vType varchar(32);
    DECLARE @vValue datetime;
	DECLARE @sqlstate varchar(5);

	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';

    -- initialize global variables
	call sahara.SP_saharaAPIInit();
	
	select TYPE, DATE_VALUE
      into @vType, @vValue
      from sahara.CONFIGURATION
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigNotFound,
            'Failed getting a configuration item value because the item ' ||
            'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.'
		);
		return;
	end if;

    -- make sure the data type is correct
    if 'DATE' <> @vType then
		CALL SP_Error_RaiseApplicationError 
		(
			@EConfigWrongType,
			'Failed getting an existing configuration item value of ' ||
			'type ' || @vType || ' as type DATE.  Section: ' || @pSection ||
			', Name: ' || @pName || '.'
		);
    end if;

    return @vValue;
end;

grant execute on sahara.fn_Conf_GetConfigDate to SAHARA_APP;

/*
 **********************************************************************
 * fn_License_isTimetoPoll():
 * 
 * Reports if it is time for licenses to be polled and updated into the database.
 * 
 **********************************************************************
 */
message 'Creating function fn_License_isTimetoPoll...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_License_isTimetoPoll' and user_name(creator) = 'SAHARA') then
   drop FUNCTION sahara.fn_License_isTimetoPoll
end if;

CREATE FUNCTION "SAHARA"."fn_License_isTimetoPoll" () 
returns bit
BEGIN
	DECLARE @FUNC varchar(64);
	DECLARE @vCt integer;

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @FUNC = 'fn_License_isTimetoPoll'; 

	CALL SP_Log_LogDebug(null, @FUNC, 'entry');

	set @vCt = DATEDIFF(second,sahara.fn_conf_GetConfigDate('LICENSE', 'LAST_POLL'),sahara.fn_Conf_UTCTimestamp());	

	if @vCt >= fn_conf_GetConfigDayInterval('LICENSE', 'POLL_INTERVAL') then 
		-- say it is the time to poll 
		return 1;
	else 
		-- not the time to poll, wait for next cycle 
		return 0;
	end if;

END;

grant execute on sahara.fn_License_isTimetoPoll to sahara_app;

/*
 **********************************************************************
 * sp_License_RecordPoll():
 * 
 * Records the current time as the most recent successful license poll.
 * 
 **********************************************************************
 */
message 'Creating function sp_License_RecordPoll...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'sp_License_RecordPoll' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_License_RecordPoll
end if;

CREATE PROCEDURE "SAHARA"."sp_License_RecordPoll" () 
BEGIN
    DECLARE @func VARCHAR(64);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.sp_License_RecordPoll';
	CALL SP_Log_LogDebug(null, @func, 'entry');

	CALL sahara.fn_Configuration_SetDate(null, 'LICENSE', 'LAST_POLL', sahara.fn_Conf_UTCTimestamp());

END;

grant execute on sahara.sp_License_RecordPoll to sahara_app;

if exists(select 1 from sys.sysprocedure where proc_name = 'sp_recompile_views' and user_name(creator) = 'SAHARA')
then
    drop procedure sahara.sp_recompile_views
end if;

CREATE procedure sahara.sp_recompile_views()
BEGIN

DECLARE @error integer;
DECLARE @serror varchar(512);
DECLARE @sqlstate varchar(5);
DECLARE @view_name varchar(128);
DECLARE @sql_stmts long varchar;
DECLARE @sql_stmt long varchar;
DECLARE @outer_loop_count int;
DECLARE @inner_loop_count int;

DECLARE vom_views NO SCROLL CURSOR FOR
select tbl.table_name
from sys.systable tbl, sys.sysuser usr, sys.sysobject obj
where usr.user_id = tbl.creator and
      usr.user_name in ('SAHARA', 'SAHARA_APP') and
      tbl.object_id = obj.object_id and
      obj.status = 2 and
      tbl.table_type = 'VIEW'
FOR READ ONLY;


  set @outer_loop_count = 0;
  set @inner_loop_count = 1;

  WHILE ( @inner_loop_count > 0 and @outer_loop_count < 20 ) LOOP
     set @sql_stmts = '';
     set @inner_loop_count = 0;

     OPEN vom_views WITH HOLD isolation level 0;

     FETCH vom_views INTO @view_name;

    LOOP1:

     WHILE(@@sqlstatus = 0) LOOP
         set @inner_loop_count = @inner_loop_count + 1;
         set @sql_stmt = 'alter view ' + @view_name + ' recompile;';

         execute(@sql_stmt);

         FETCH vom_views INTO @view_name;
     END LOOP LOOP1;

     CLOSE vom_views;

     set @outer_loop_count = @outer_loop_count + 1;
  END LOOP;

EXCEPTION

    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 )
        THEN
            BEGIN
                message string (' Error failed at = ', @event ) to LOG;
                message string ('          @error = ', @error )    to LOG;
                message string ('      @serrir()  = ', @serror )   to LOG;
                message string ('     @sqlstate() = ', @sqlstate ) to LOG;
            END;
            RESIGNAL;
        END IF;


END;

GRANT EXECUTE ON sahara.sp_recompile_views TO PUBLIC;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_metadata' and user_name(creator) = 'SAHARA')
then
    drop procedure sahara.SP_metadata
end if;

CREATE procedure sahara.SP_metadata()
BEGIN

DECLARE @error integer;
DECLARE @serror varchar(512);
DECLARE @sqlstate varchar(5);


select type, owner, name1, name2, coltype, c_length, c_default_value, c_nullability
from (
   select 'table' as type, u.name as owner, tbl.table_name as name1, col.cname as name2,
          col.coltype as coltype, col.length as c_length, col.default_value as c_default_value, col.nulls as c_nullability
   from sysusers u key join sys.systable tbl inner join sys.syscolumns col on tbl.table_name = col.tname and tbl.table_type in ('BASE')
   where owner in ( 'SAHARA', 'SAHARA_APP', 'SAHARA_REPORTS', 'SAHARA_COGNOS' )

   union all

   select 'view' as type, u.name as owner, tbl.table_name as name1, col.cname as name2,
          col.coltype as c_coltype, col.length as c_length, col.default_value as c_default_value, null as c_nullability
   from sysusers u key join sys.systable tbl inner join sys.syscolumns col on tbl.table_name = col.tname and tbl.table_type in ('VIEW')
   where owner in ( 'SAHARA', 'SAHARA_APP', 'SAHARA_REPORTS', 'SAHARA_COGNOS' )

   union all

   select 'sp/fn' as type, u.name as owner, sp.proc_name as name1, null as name2,
          null as coltype, null as c_length, null as c_default_value, null as c_nullability
   from sys.sysprocedure sp key join sysusers u
   where owner in ( 'SAHARA', 'SAHARA_APP', 'SAHARA_REPORTS', 'SAHARA_COGNOS' )

   union all

   select 'trigger' as type, u.name as owner, tbl.table_name as name1, trig.trigger_name as name2,
          null as coltype, null as c_length, null as c_default_value, null as c_nullability
   from sys.systrigger trig key join sys.systable tbl key join sysusers u
   where name2 is not null and owner in ( 'SAHARA', 'SAHARA_APP', 'SAHARA_REPORTS', 'SAHARA_COGNOS' )

   union all

   select 'event' as type, u.name as owner, ev.event_name as name1, null as name2,
          null as coltype, null as c_length, null as c_default_value, null as c_nullability
   from sysevent ev key join sysusers u
   where owner in ( 'SAHARA', 'SAHARA_APP', 'SAHARA_REPORTS', 'SAHARA_COGNOS' )

   union all

   select 'column_constraint' as type, u.name as owner, tbl.table_name as name1, col.cname+'.'+cr.constraint_name as name2,
          ch.check_defn as coltype, null as c_length, null as c_default_value, null as c_nullability
   from sysusers u key join sys.systable tbl
        inner join sys.syscolumns col on tbl.table_name = col.tname
        inner join sys.syscolumn col1 on col.cname = col1.column_name and tbl.table_id = col1.table_id
        inner join sys.sysconstraint cr on tbl.object_id = cr.table_object_id and cr.ref_object_id = col1.object_id
        inner join sys.syscheck ch on ch.check_id = cr.constraint_id
   where owner in ( 'SAHARA', 'SAHARA_APP', 'SAHARA_REPORTS', 'SAHARA_COGNOS' )

   union all

   select 'table_index' as type, creator as owner, tname as name1, iname as name2,
          '('+colnames+')'+indextype as coltype, null as c_length, null as c_default_value, null as c_nullability
   from sys.sysindexes
   where creator in ( 'SAHARA', 'SAHARA_APP', 'SAHARA_REPORTS', 'SAHARA_COGNOS' )
   order by type, owner, name1, name2, coltype asc ) as v;

EXCEPTION

    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 )
        THEN
            BEGIN
                message string (' SP_dump_metadata: Error failed at = ', @error ) to LOG;
                message string ('                        @serrir()  = ', @serror )   to LOG;
                message string ('                       @sqlstate() = ', @sqlstate ) to LOG;
            END;
            RESIGNAL;
        END IF;

END;

GRANT EXECUTE ON sahara.SP_metadata TO PUBLIC;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_dump_metadata' and user_name(creator) = 'SAHARA')
then
    drop procedure sahara.SP_dump_metadata
end if;

CREATE procedure sahara.SP_dump_metadata(IN @filename varchar(512))
BEGIN

DECLARE @error integer;
DECLARE @serror varchar(512);
DECLARE @sqlstate varchar(5);


unload select type, owner, name1, name2, coltype, c_length, c_default_value
from sahara.sp_metadata()
   to  @filename FORMAT 'ASCII' QUOTES ON ORDER OFF ESCAPES ON DELIMITED BY ',';


EXCEPTION

    when others then
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 )
        THEN
            BEGIN
                message string (' SP_dump_metadata: Error failed at = ', @error ) to LOG;
                message string ('                        @serrir()  = ', @serror )   to LOG;
                message string ('                       @sqlstate() = ', @sqlstate ) to LOG;
            END;
            RESIGNAL;
        END IF;

END;

GRANT EXECUTE ON sahara.SP_dump_metadata TO PUBLIC;



message 'Creating procedure SP_SericeGroupHistoricalEvents...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_SericeGroupHistoricalEvents' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_SericeGroupHistoricalEvents
end if;

CREATE PROCEDURE "SAHARA"."SP_SericeGroupHistoricalEvents" ()

BEGIN
DECLARE @error integer;
DECLARE @sqlstate varchar(5);
DECLARE @func varchar(100);
DECLARE @serror varchar(512);
declare @vDatesKey numeric(10);
declare @vSourceKey numeric(10);
declare @rollup_source_type_key numeric(10);
declare @source_name varchar(128);
declare @web_server_host varchar(128);
declare @web_server_port numeric(5);
declare @db_password varchar(128);
declare @db_password_2 varchar(128);
declare @db_host varchar(128);
declare @db_port numeric(5);
declare @pLogArchiveExt varchar(16) ;
declare @pScheduleHash varchar(64);
declare @is_rollup_now char(1);
declare @rollup_day varchar(10);
declare @sgcolumns long varchar;
DECLARE @startTime datetime;
DECLARE @utcadj integer;
DECLARE @SQLQuery long varchar;
DECLARE @vRowCount integer;
DECLARE @cluster_rollup_marker date;
DECLARE @min_cluster_rollup_marker date;
DECLARE @r_count integer;
DECLARE local temporary table sg_rb_count
(            
       servicegroup_oid varchar(128),
       adate date,
       servicegroup_key numeric(10,0),
       dates_key numeric(10,0), 
       rb_count integer,
       rb_keys long varchar, 
       primary key (servicegroup_key, dates_key)
) on commit preserve rows; 

--DECLARE DUP_VAL_ON_INDEX EXCEPTION FOR SQLSTATE '23W01';

   -- insert only if its not present.
    --set @pLogArchiveExt = '.tar.gz';
     set @func = 'SP_SericeGroupHistoricalEvents';
    set @pLogArchiveExt = '.tar.gz';
     	CALL SP_Log_LogDebug(@ActiveSessionKey_, @func, 'entry');

	insert into SAHARA.DATES(ADATE)
    select distinct "DATE" from sahara_app.servicegroup_hist_events_ where "DATE" not in (select ADATE from sahara.DATES);
			

select DATES_KEY, ROLLUP_SOURCE_KEY,IS_ROLLUP_NOW,ROLLUP_DAY,START_TIME,UTC_ADJUSTMENT
      into @vDatesKey, @vSourceKey,@is_rollup_now,@rollup_day,@startTime,@utcadj
      from ROLLUP_BATCH
      where ROLLUP_BATCH_KEY = @ActiveBatchKey_;

 message 'Source key ' || @vSourceKey to client;
-- Get the source information
select ROLLUP_SOURCE_TYPE_KEY,
         "NAME",
         WEB_SERVER_HOST, 
        WEB_SERVER_PORT, 
       DB_HOST, 
         DB_PORT,
         DB_PASSWORD, 
        DB_PASSWORD_2
         into
            @rollup_source_type_key, 
            @source_name,
            @web_server_host,
            @web_server_port,
            @db_host,
            @db_port,
            @db_password,
            @db_password_2    
      from ROLLUP_SOURCE
     where ROLLUP_SOURCE_KEY = @vSourceKey;


 message @rollup_source_type_key to client;
insert into sahara.ROLLUP_BATCH
        (ROLLUP_SOURCE_TYPE_KEY,
         DATES_KEY, SESSION_KEY,
         ROLLUP_SOURCE_KEY, SOURCE_NAME,
         WEB_SERVER_HOST, WEB_SERVER_PORT, DB_HOST, DB_PORT,
         DB_PASSWORD, DB_PASSWORD_2,
         LOG_ARCHIVE,
          STATUS,IS_ROLLUP_NOW,IS_DUMMY,ROLLUP_DAY,START_TIME,UTC_ADJUSTMENT,SHOW_STATUS)
    select distinct 
         @rollup_source_type_key,
         d.DATES_KEY, @ActiveSessionKey_, 
         @vSourceKey, @source_name,
         @web_server_host, @web_server_port, @db_host, @db_port,
         @db_password, @db_password_2,
     
         DATEFORMAT(sahara.fn_conf_utctimestamp(), 'YYYY_MM_DD_HH_MM') || '_log' || @pLogArchiveExt,
          'Succeeded','F','T',@rollup_day,
                        DATEADD(second,0,d.ADATE),@utcadj,'F'        
        from sahara_app.servicegroup_hist_events_  ev,SAHARA.DATES d where d.ADATE = ev."DATE" and d.DATES_KEY not in (select DATES_KEY from sahara.ROLLUP_BATCH);
      
     select list(c.cname) into @sgcolumns from sys.syscolumns c where tname = 'SERVICEGROUP_HIST_EVENTS' and c.cname not in ('ROLLUP_BATCH_KEY','DATES_KEY','SERVICEGROUP_KEY');

     
    --select * from sahara.ROLLUP_BATCH
    /* set @SQLQuery = 'insert into sahara.servicegroup_hist_events( ROLLUP_BATCH_KEY,DATES_KEY,SERVICEGROUP_KEY , ' || @sgcolumns 
                    || ') on existing update select b.rollup_batch_key,b.dates_key,sg.servicegroup_key, ' || @sgcolumns ||
                      ' from sahara.servicegroup sg, sahara_app.servicegroup_hist_events_  ev, SAHARA.DATES d, sahara.ROLLUP_BATCH b 
                         where ev.SERVICEGROUP_OID = sg.SERVICEGROUP_OID and d.ADATE= ev."DATE" and d.DATES_KEY = b.DATES_KEY ;';
    */

    insert into sg_rb_count(servicegroup_oid, adate, servicegroup_key, dates_key, rb_count, rb_keys)
    select distinct ev.servicegroup_oid, d.adate as adate, sg.servicegroup_key, d.dates_key, 
           count(rb.rollup_batch_key) as rb_count, 
           list(distinct rb.rollup_batch_key) as rb_keys
    from sahara_app.servicegroup_hist_events_ ev
    join sahara.servicegroup as sg 
    on sg.servicegroup_oid = ev.servicegroup_oid
    join sahara.dates as d on d.adate = ev."DATE"
    join
    (  
      select * from sahara.rollup_batch where period_type = 'day'
    ) as rb 
    on rb.dates_key = d.dates_key
    group by ev.servicegroup_oid, d.adate, sg.servicegroup_key, d.dates_key;

   set @SQLQuery = 'insert into sahara.servicegroup_hist_events(ROLLUP_BATCH_KEY,DATES_KEY,SERVICEGROUP_KEY, ' || @sgcolumns
         || ') on existing update select rb.rollup_batch_key, rb_count.dates_key, rb_count.servicegroup_key, ' || @sgcolumns
         || ' from (select servicegroup_oid, adate, servicegroup_key, dates_key from sg_rb_count where rb_count <= 1) as rb_count
join sahara_app.servicegroup_hist_events_ as ev_ on ev_.servicegroup_oid = rb_count.servicegroup_oid and
ev_."DATE" = rb_count.adate join (select * from sahara.rollup_batch where period_type = ''day'') as rb on rb.dates_key = rb_count.dates_key;';

    execute immediate @SQLQuery;
    commit work;

  set @r_count = 0;
  for EventUpLoadLoop as EventUpLoadCUR dynamic scroll cursor for
    select servicegroup_oid as @sg_oid_R, adate as @adate_R, servicegroup_key as @sg_key_R, dates_key as @dates_key_R, rb_keys as @rb_keys_R
    from sg_rb_count where rb_count > 1
  do begin

      CALL SP_Log_LogDebug(@ActiveSessionKey_, @func, 'Process sg_key::dates_key::multiple rb_keys: @sg_key_R=' || @sg_key_R || '; @dates_key_R=' || @dates_key_R || ';@adate_R=' || @adate_R || ';@sg_oid_R=' || @sg_oid_R ||  '; @rb_keys_R=' || @rb_keys_R);
    
      set @SQLQuery = 'delete from sahara.servicegroup_hist_events
      from ( select ev.rollup_batch_key, ev.servicegroup_key, ev.dates_key
             from sahara.servicegroup_hist_events ev
             join sahara.rollup_batch as rb
             on rb.dates_key = ev.dates_key  
             where ev.servicegroup_key = ' || @sg_key_R || ' and ev.dates_key = ' || @dates_key_R ||
             ' and ev.rollup_batch_key in(' || replace(@rb_keys_R, '''', '') ||') ) as del_ev;';    
      execute immediate @SQLQuery;

      set @SQLQuery = 'insert into sahara.servicegroup_hist_events(ROLLUP_BATCH_KEY,DATES_KEY,SERVICEGROUP_KEY, ' || @sgcolumns
             || ') on existing update 
                select max_rb.rollup_batch_key, staged.dates_key, staged.servicegroup_key, ' || @sgcolumns
             || ' from 
             (
                select servicegroup_oid, adate, servicegroup_key, dates_key 
                from sg_rb_count where servicegroup_key = ' || @sg_key_R || ' and dates_key = ' || @dates_key_R ||
             ' ) as staged
             join sahara_app.servicegroup_hist_events_ as ev_ 
             on ev_.servicegroup_oid = staged.servicegroup_oid and ev_."DATE" = staged.adate
             join 
             (
                select max(rb.rollup_batch_key) as rollup_batch_key, rb.dates_key
                from sahara.rollup_batch as rb             
                where rb.rollup_batch_key in (' || replace(@rb_keys_R, '''', '') || ')
                group by rb.dates_key  
             ) as max_rb
             on max_rb.dates_key = staged.dates_key;';
       execute immediate @SQLQuery;

       -- CALL SP_Log_LogDebug(@ActiveSessionKey_, @func, 'insert @sg_key_R=' || @sg_key_R || '; @dates_key_R=' || @dates_key_R || ';@adate_R=' || @adate_R || ';@sg_oid_R=' || @sg_oid_R);  

       commit work;
       set @r_count = @r_count + 1;
  end;
  end for;
  
    set @cluster_rollup_marker = null;
    select  MAX(d.ADATE) into @cluster_rollup_marker from sahara_app.servicegroup_hist_events_ ev,sahara.dates d where d.ADATE = ev."DATE";
    
    CALL SP_Log_LogDebug(@ActiveSessionKey_, @func, @SQLQuery);
    execute immediate @SQLQuery;
    set @vRowCount = @@rowcount;
    call SP_Log_LogDebug(@ActiveSessionKey_, @func, 'Total Rows inserted :SERVICEGROUP_HIST_EVENTS ' || @vRowCount + @r_count);
    
	-- update the valid cluster data for all the rollup batches
     update sahara.rollup_batch rb set rb.valid_cluster_data = 'T' where  rb.valid_cluster_data = 'F' and rb.ROLLUP_BATCH_KEY  in ( select distinct pev.rollup_batch_key from 
                                                                                                                    sahara.servicegroup_hist_events pev,sahara.rollup_batch rbr 
                                                                                                                        where rbr.rollup_batch_key = pev.rollup_batch_key 
                                                                                                                            and rbr.valid_cluster_data = 'F');

 
      -- If the marker is the marker for the source.
      -- check what is the rollup marker for the source.
       select cluster_rollup_marker into @min_cluster_rollup_marker from sahara.rollup_source rs where rs.rollup_source_key =  @vSourceKey;    

     if @cluster_rollup_marker is not null then
        update sahara.rollup_source  rs set rs.cluster_rollup_marker = @cluster_rollup_marker where rs.rollup_source_key =  @vSourceKey;    
        call "SAHARA"."SP_AdjustServiceGroupSummarization"(@min_cluster_rollup_marker,@cluster_rollup_marker,'servicegroup_hist_events',@vSourceKey)
     end if;

       

	EXCEPTION	
		WHEN others THEN
			SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
            call SP_Log_LogError(@ActiveSessionKey_, @func, 'SQLCODE: ' || @error || 'SQLSTATE: ' || @sqlstate|| 'SQLERRORMESSAGE :' || @serror );
            -- Don't fail			
--message 'Error ' || @serror to client;
END;
grant execute on sahara.SP_SericeGroupHistoricalEvents to sahara;
                 
/*
 **************************************************************************
 * SP_AdjustServiceGroupSummarization()
 * Iterate and summarize SERVICEGROUP_HIST_EVENTS based on the given min,max date.
 **************************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_AdjustServiceGroupSummarization' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_AdjustServiceGroupSummarization
end if;

CREATE PROCEDURE SAHARA.SP_AdjustServiceGroupSummarization(@pMinADATE date, @pMaxADATE date, @pTableName varchar(256), @pRollupSourceKey numeric(10))

BEGIN

   DECLARE @FUNC                       varchar(256);

   DECLARE @pRollupBatch               numeric(10);
   DECLARE @pDates_key                 numeric(10);
   DECLARE @pYear_Period_key           numeric(10);
   DECLARE @pRollup_source_key         numeric(10);

   DECLARE @pADATE_Yesterday           date;
   DECLARE @pADATE_PrevSuccRollup      date;
   DECLARE @pYearCount                 int;
   DECLARE @pQuarterCount              int;
   DECLARE @pMonthCount                int;
   DECLARE @pWeekCount                 int;
   DECLARE @pQuarterNumber             int;
   DECLARE @pMonthNumber               int;
   DECLARE @pYearNumber                int;
   DECLARE @pWeekNumber                int;

   DECLARE @pMigrateSummaryData        char(1);

   DECLARE @error                      integer;
   DECLARE @sqlstate                   varchar(5);
   DECLARE @serror                     varchar(512);
   DECLARE @section                    varchar(512);
   DECLARE @TableName                  varchar(512);
   DECLARE @migrate_summary_data       int;

   set @FUNC = 'SP_AdjustServiceGroupSummarization';
   set @TableName = UPPER(@pTableName);

   message 'SAHARA.SP_table_summarization' to client;

   call SAHARA.SP_saharaAPIInit();
   call SP_Log_LogDebug(null, @FUNC, 'enter');

   -- TODO: populate Missing data treatment.

   set @pYearCount = 0;
   set @pQuarterCount = 0;
   set @pMonthCount = 0;
   set @pWeekCount = 0;

   set @pYearNumber = 0;

   if @pMinADATE is null
   then
       select min(dates.adate) into @pMinADATE
       from dates 
            inner join rollup_batch rb on rb.dates_key = dates.dates_key
       where rb.rollup_source_key = @pRollupSourceKey;
   end if;

   for date_entry as date_entry_loop cursor for
      select da.adate as adate
      from sahara.DATES da
      where da.adate >= @pMinADATE and da.adate <= @pMaxADATE
      order by adate asc
   do
   begin

      savepoint SP_AdjustServiceGroupSummarization;

      set @section = string('rollup_source_key=',@pRollupSourceKey, ',adate=',adate,',period=','week',',TableName=',@TableName);
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS(@pRollupSourceKey, adate, 'week','', @TableName);

      set @section = string('rollup_source_key=',@pRollupSourceKey, ',adate=',adate,',period=','month',',TableName=',@TableName);
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS(@pRollupSourceKey, adate, 'month','', @TableName);

      set @section = string('rollup_source_key=',@pRollupSourceKey, ',adate=',adate,',period=','quarter',',TableName=',@TableName);
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS(@pRollupSourceKey, adate, 'quarter','', @TableName);

      set @section = string('rollup_source_key=',@pRollupSourceKey, ',adate=',adate,',period=','year',',TableName=',@TableName);
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS(@pRollupSourceKey, adate, 'year','', @TableName);

      commit;

      -- EXCEPTION
      --       WHEN OTHERS THEN
      --      SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
      --      IF ( @error <> 0 )
      --      THEN
      --          BEGIN
      --              call SP_Log_LogError(null, @FUNC,
      --                                         string ('          @error    = ', @error ) +
      --                                         string ('         @serror    = ', @serror ) +
      --                                         string ('       @sqlstate    = ', @sqlstate ) +
      --                                         string (' Error failed at    = ', @FUNC+':'+@section ) );
      --              rollback to savepoint SP_AdjustServiceGroupSummarization;
      --              call SP_Log_LogDebug(null, @FUNC, 'Rollback, exit');
      --          END;
      --      END IF;

   end;
   end for;

   call SP_Log_LogDebug(null, @FUNC, 'exit');

END;

grant execute on sahara.SP_AdjustServiceGroupSummarization to sahara;


/*
 **********************************************************************
 * SP_COGNOS_MOD_UPD_AcquireExclusiveLock():
 * Acquires exclusive lock on the COGNOS METAMODEL UPDATE module.
 * Any module that does metamodel update must acquire this lock 
 * to ensure * that only one connection doing the operations.
 * The stored proc uses underlying Lock API stored proc.
 * The stored proc does call commit inside, so there may be side effects
 * if the caller hamust be aware of this.
 * The passed in session is remembered with the lock.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_COGNOS_MOD_UPD_AcquireExclusiveLock' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_COGNOS_MOD_UPD_AcquireExclusiveLock;
END IF;

create procedure sahara.SP_COGNOS_MOD_UPD_AcquireExclusiveLock()
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @SessionKey NUMERIC(10);

    -- initialize global variables
    call sahara.SP_saharaAPIInit();
 
    SET @func = 'SP_COGNOS_MOD_UPD_AcquireExclusiveLock';
    SET @SessionKey = NULL;

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call LockSession() for the specific activity. If fails, it will raise
    -- an error, which will throw the exception which will return the
    -- control to the caller.
    CALL SP_Lock_LockSession('SAHARA_METAMODEL_UPDATE', @SessionKey);
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully acquired the lock');

    -- If we are here, then the lock was acquired, remember the session etc.
    SET @gCMUpdHaveControl = 1;

	call SP_COGNOS_MOD_UPD_ResetProcessingUpdates();

END;
grant execute on sahara.SP_COGNOS_MOD_UPD_AcquireExclusiveLock to sahara_app;

/*
 **********************************************************************
 * SP_COGNOS_MOD_UPD_ReleaseExclusiveLock():
 * Release the exclusive lock on the COGNOS METAMODEL UPDATE module previously 
 * acquired for the given session.
 * The stored proc uses underlying unlock API stored proc.
 * The stored proc does call commit inside, so there may be side effects
 * if the caller hamust be aware of this.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_COGNOS_MOD_UPD_ReleaseExclusiveLock' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_COGNOS_MOD_UPD_ReleaseExclusiveLock;
END IF;

create procedure sahara.SP_COGNOS_MOD_UPD_ReleaseExclusiveLock()
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @SessionKey NUMERIC(10);

    -- initialize global variables
    call sahara.SP_saharaAPIInit();

    SET @func = 'SP_COGNOS_MOD_UPD_ReleaseExclusiveLock';
    SET @SessionKey = NULL;

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call UnlockSession() for the given activity.
    -- If this fails, it will raise an error, which will throw the 
    -- exception which will return the control to the caller.
    CALL SP_Lock_UnlockSession('SAHARA_METAMODEL_UPDATE', @SessionKey);

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully release the lock');

	SET @gCMUpdHaveControl = 0;
END;
grant execute on sahara.SP_COGNOS_MOD_UPD_ReleaseExclusiveLock to sahara_app;

/*
 **********************************************************************
 * SP_COGNOS_MOD_UPD_RequireControl():
 * Check that the Cognos metamodel update control is held by the
 * connection and throw an exception if not
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_COGNOS_MOD_UPD_RequireControl' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_COGNOS_MOD_UPD_RequireControl;
END IF;

CREATE PROCEDURE sahara.SP_COGNOS_MOD_UPD_RequireControl(
    IN @Func long varchar)
BEGIN
    IF ( @gCMUpdHaveControl = 0) THEN
      BEGIN
        CALL SP_Error_RaiseApplicationError(
            @ENoCognosModelControl,
            @Func || ': ' || 'caller does not have the Cognos metamodel update control'
        );
       END
    END IF;
END;


/*
 **********************************************************************
 * SP_COGNOS_MOD_UPD_Status():
 * Sets the status of a Cognos metamodel update. 
 * This is the primary procedure used by the external code that updates the metamodel. 
 * Every time the status is set to "Processing", the update start time is 
 * set and the number of attempts is incremented. 
 * Every time the status is set to another status, the update completion time is set.
 * 
 * Before calling this procedure, the calling database session must previously 
 * have acquired the Cognos metamodel update control by successfully calling 
 * procedure SP_COGNOS_MOD_UPD_AcquireExclusiveLock. 
 **********************************************************************
 */
 if exists(select 1 from sys.sysprocedure where proc_name = 'SP_COGNOS_MOD_UPD_Status' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_COGNOS_MOD_UPD_Status
end if;

CREATE PROCEDURE SAHARA.SP_COGNOS_MOD_UPD_Status 
(
    @pUpdateKey numeric(10),
    @pStatus varchar(32),
    @pMessage long varchar default null
) 
BEGIN
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);
    DECLARE @vOldMessage varchar(2);
	DECLARE @NL char(2);

    DECLARE @updateCursor CURSOR FOR
        select substr(STATUS_MESSAGE, 1, 2)
          from COGNOS_MODEL_UPDATE
         where COGNOS_MODEL_UPDATE_KEY = @pUpdateKey
           for update;

	SET @NL = char(10);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_COGNOS_MOD_UPD_Status';
	CALL SP_Log_LogDebug(null, @func, 'entry');
	
	CALL SP_COGNOS_MOD_UPD_RequireControl(@func);

    -- select the existing record for validation and to set the
    -- message properly
	open @updateCursor;
    fetch next @updateCursor into @vOldMessage;
 
    -- we expect to find it
	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
        -- else - we didn't find an existing record
		CALL SP_Error_RaiseApplicationError 
		(
			@ECognosUpdateNotFound,
			@func || ': ' || 'failed finding Cognos model update with key ' || @pUpdateKey
		);
	end if;    	
    -- else, we found the record

    -- update the status and the message if one is provided
    if trim(@pMessage) is not null then
        if @vOldMessage is null then
            -- this is the first message
            update COGNOS_MODEL_UPDATE
               set STATUS = @pStatus,
                   STATUS_MESSAGE = trim(@pMessage)
             where current of @updateCursor;
        else if '* ' <> @vOldMessage then
            -- this is the second message
            update COGNOS_MODEL_UPDATE
               set STATUS = @pStatus,
                   STATUS_MESSAGE = '* ' || STATUS_MESSAGE || @NL ||
                                    '* ' || trim(@pMessage)
             where current of @updateCursor;
			else
				-- this is the third or greater message
				update COGNOS_MODEL_UPDATE
				   set STATUS = @pStatus,
					   STATUS_MESSAGE = STATUS_MESSAGE || @NL ||
										'* ' || trim(@pMessage)
	             where current of @updateCursor;
		     end if;
        end if;
    else
        -- just update the status
        update COGNOS_MODEL_UPDATE
           set STATUS = @pStatus
         where current of @updateCursor;
    end if;

    close @updateCursor;
end;
grant execute on sahara.SP_COGNOS_MOD_UPD_Status to sahara_app;


/*
 **********************************************************************
 * SP_COGNOS_MOD_UPD_ResetProcessingUpdates():
 * Resets any Cognos metamodel updates that are in the processing
 * state to the pending state
 * TODO: pragma autonomous_transaction;
 **********************************************************************
 */
 if exists(select 1 from sys.sysprocedure where proc_name = 'SP_COGNOS_MOD_UPD_ResetProcessingUpdates' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_COGNOS_MOD_UPD_ResetProcessingUpdates
end if;

CREATE PROCEDURE SAHARA.SP_COGNOS_MOD_UPD_ResetProcessingUpdates() 
BEGIN
	DECLARE @vProcessingUpdates numeric(10);
	    
	FOR @ProcessingUpdatesLoop AS @ProcessingUpdatesCursor CURSOR FOR (   
		select COGNOS_MODEL_UPDATE_KEY as vProcessingUpdates
		  from COGNOS_MODEL_UPDATE
		 where STATUS = 'Processing'
	)
	DO
	    if vProcessingUpdates is not null and vProcessingUpdates > 0 then
			call SP_COGNOS_MOD_UPD_Status(vProcessingUpdates, 'Pending',
                   'A previous processing attempt was found incomplete and was reset.');
		end if;
	END FOR;    

    commit;

exception
    when others then
        -- commit what we can
        commit;
        CALL SP_Error_RaiseApplicationError 
		(
			@ECognosUpdateNotFound,
			'Failed in ResetProcessingUpdates'
		);
end;

/*
 **********************************************************************
 * SP_COGNOS_MOD_UPD_StatusMessage():
 * Adds a status message to a Cognos metamodel update. 
 * Each message that is added is appended to any previous messages.
 * 
 * Before calling this procedure, the calling database session must previously 
 * have acquired the Cognos metamodel update control by successfully calling 
 * procedure SP_COGNOS_MOD_UPD_AcquireExclusiveLock. 
 **********************************************************************
 */
 if exists(select 1 from sys.sysprocedure where proc_name = 'SP_COGNOS_MOD_UPD_StatusMessage' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_COGNOS_MOD_UPD_StatusMessage
end if;

CREATE PROCEDURE SAHARA.SP_COGNOS_MOD_UPD_StatusMessage 
(
    @pUpdateKey numeric(10),
    @pMessage long varchar default null
) 
BEGIN
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);
    DECLARE @vOldMessage varchar(2);
	DECLARE @NL char(2);

    DECLARE @updateCursor CURSOR FOR
        select substr(STATUS_MESSAGE, 1, 2)
          from COGNOS_MODEL_UPDATE
         where COGNOS_MODEL_UPDATE_KEY = @pUpdateKey
           for update;

	SET @NL = char(10);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_COGNOS_MOD_UPD_StatusMessage';
	CALL SP_Log_LogDebug(null, @func, 'entry');
	
	CALL SP_COGNOS_MOD_UPD_RequireControl(@func);

    -- select the existing record for validation and to set the
    -- message properly
	open @updateCursor;
    fetch next @updateCursor into @vOldMessage;
 
    -- we expect to find it
	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
        -- else - we didn't find an existing record
		CALL SP_Error_RaiseApplicationError 
		(
			@ECognosUpdateNotFound,
			@func || ': ' || 'failed finding Cognos model update with key ' || @pUpdateKey
		);
	end if;    	
    -- else, we found the record

    -- update the message if one is provided
    if trim(@pMessage) is not null then
        if @vOldMessage is null then
            -- this is the first message
            update COGNOS_MODEL_UPDATE
               set STATUS_MESSAGE = trim(@pMessage)
             where current of @updateCursor;
        else if '* ' <> @vOldMessage then
            -- this is the second message
            update COGNOS_MODEL_UPDATE
               set STATUS_MESSAGE = '* ' || STATUS_MESSAGE || @NL ||
                                    '* ' || trim(@pMessage)
             where current of @updateCursor;
			else
				-- this is the third or greater message
				update COGNOS_MODEL_UPDATE
				   set STATUS_MESSAGE = STATUS_MESSAGE || @NL ||
										'* ' || trim(@pMessage)
				 where current of @updateCursor;
			end if;
        end if;
    end if;

    close @updateCursor;
end;

grant execute on sahara.SP_COGNOS_MOD_UPD_StatusMessage to sahara_app;



CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_COGNOS_MOD_UPD_EnQCustomAttrDelete');

CREATE PROCEDURE SAHARA.SP_COGNOS_MOD_UPD_EnQCustomAttrDelete()
BEGIN 
  DECLARE @FUNC VARCHAR(264);
  BEGIN 
    
    ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_COGNOS_MOD_UPD_EnQCustomAttrDelete';

    insert into COGNOS_MODEL_UPDATE
        (TYPE, ARG_1,
         ARG_2, ARG_3,
         ARG_4, ARG_5,
         ARG_6, ARG_7,
	 QUEUED_TIME) on existing update
    select 
    'CUSTOM ATTRIBUTE DELETE', CUSTOM_ATTRIBUTE_KEY,
         CUSTOM_ATTRIBUTE_CONTEXT,TYPE,
         DISPLAY_NAME, COLUMN_NAME,
         COLUMN_TYPE, SOURCE_NAME,
	 CURRENT UTC TIMESTAMP
    from SAHARA.tmp_CADeleted;
  END;
END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_COGNOS_MOD_UPD_EnQCustomAttrAdd');

CREATE PROCEDURE SAHARA.SP_COGNOS_MOD_UPD_EnQCustomAttrAdd()

BEGIN 
  DECLARE @FUNC VARCHAR(264);

  BEGIN 

  ---- Intialize connection variables 

  call sahara.SP_saharaAPIInit();

  SET @FUNC = 'SP_COGNOS_MOD_UPD_EnQCustomAttrAdd';
  
  insert into COGNOS_MODEL_UPDATE
        (TYPE, ARG_1,
         ARG_2, ARG_3,
         ARG_4, ARG_5,
         ARG_6, ARG_7,
         QUEUED_TIME) on existing update
  select 
    'CUSTOM ATTRIBUTE ADD', CUSTOM_ATTRIBUTE_KEY,
         CUSTOM_ATTRIBUTE_CONTEXT,TYPE,
         DISPLAY_NAME, COLUMN_NAME,
         COLUMN_TYPE, SOURCE_NAME, CURRENT UTC TIMESTAMP
  from SAHARA.tmp_CAAdded ;

  END;
END;

grant execute on sahara.SP_COGNOS_MOD_UPD_EnQCustomAttrDelete to sahara_app;
grant execute on sahara.SP_COGNOS_MOD_UPD_EnQCustomAttrAdd to sahara_app;
CALL SAHARA.SP_DROP_OBJECT_SAHARA('TABLE','tmp_CADeleted');

CREATE GLOBAL TEMPORARY TABLE SAHARA.tmp_CADeleted
(
   CUSTOM_ATTRIBUTE_KEY      NUMERIC(10)                    NOT NULL,
   CUSTOM_ATTRIBUTE_CONTEXT  VARCHAR(32)                    NOT NULL,
   TYPE                      VARCHAR(32)                    NOT NULL,
   DISPLAY_NAME              VARCHAR(64)                    NOT NULL,
   COLUMN_NAME               VARCHAR(64)                    NOT NULL,
   COLUMN_TYPE               VARCHAR(64)                    NOT NULL,
   SOURCE_NAME               VARCHAR(256)                   NULL
)
ON COMMIT PRESERVE ROWS;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('TABLE','tmp_CAAdded');

CREATE GLOBAL TEMPORARY TABLE SAHARA.tmp_CAAdded
(
   CUSTOM_ATTRIBUTE_KEY       NUMERIC(10)                    NOT NULL,
   CUSTOM_ATTRIBUTE_CONTEXT   VARCHAR(32)                    NOT NULL,
   TYPE                       VARCHAR(32)                    NOT NULL,
   DISPLAY_NAME               VARCHAR(64)                    NOT NULL,
   COLUMN_NAME                VARCHAR(64)                    NOT NULL,
   COLUMN_TYPE                VARCHAR(64)                    NOT NULL,
   SOURCE_NAME                VARCHAR(256)                   NULL
)
ON COMMIT PRESERVE ROWS;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('TABLE','tmp_Attributes');

CREATE GLOBAL TEMPORARY TABLE SAHARA.tmp_Attributes 
(
   CUSTOM_ATTRIBUTE_KEY       NUMERIC(10)                    NOT NULL,
   CUSTOM_ATTRIBUTE_CONTEXT   VARCHAR(32)                    NOT NULL,
   TYPE                       VARCHAR(32)                    NOT NULL,
   DISPLAY_NAME               VARCHAR(64)                    NOT NULL,
   COLUMN_NAME                VARCHAR(30)                    NOT NULL,
   COLUMN_TYPE                VARCHAR(64)                    NOT NULL,
   SOURCE_NAME                VARCHAR(256)                   NULL
)
ON COMMIT PRESERVE ROWS;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('TABLE','tmp_columns');

CREATE GLOBAL TEMPORARY TABLE SAHARA.tmp_columns 
(
column_name VARCHAR(1024)
) ON COMMIT PRESERVE ROWS;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_CUST_ATTR_RemoveInvalidColumnCharacters');

CREATE FUNCTION SAHARA.fn_CUST_ATTR_RemoveInvalidColumnCharacters( @pStr LONG VARCHAR)
    RETURNS VARCHAR(2048)

BEGIN 
  DECLARE @vBad VARCHAR(2048);
  DECLARE @vCh CHAR;
  DECLARE @vRc VARCHAR(2048);
  DECLARE @incr INTEGER;
 
  BEGIN
    -- if there's nothing to work with, return an empty string
    SET @incr = 0;

    IF @pStr is null THEN

        RETURN '';
    
    END IF;

    -- make a list of the invalid characters
 
    WHILE @incr <= length(@pStr) 
    LOOP

      SET @vCh = substr(@pStr, @incr, 1);
    
      IF  @vCh NOT REGEXP '[[:upper:][:digit:]_#$]+' THEN
      
	SET @vBad = @vBad || @vCh;
      
      END IF;
      
      SET @incr = @incr + 1 ;
    
    END LOOP;

    -- if there are no invalid characters, we're done
    
    IF @vBad IS NULL THEN

        RETURN @pStr;
    
    END IF;

    -- remove the invalid characters
    SET @incr = 0;
    SET @vRc = @pStr;
    WHILE @incr <=LENGTH(@vBad)
    LOOP
      SET @vCh = substr(@vBad, @incr, 1);
      SET @vRc = replace(@vRc, @vCh, '');
      SET @incr = @incr + 1 ;
    END LOOP;

    RETURN @vRc;
 
  END;
END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_CUST_ATTR_NameUsed');

CREATE FUNCTION SAHARA.fn_CUST_ATTR_NameUsed(@pContext VARCHAR(32) , @pName VARCHAR(64))
  RETURNS BIT

BEGIN
  DECLARE @vCount INTEGER;
  
  BEGIN
  
    SET @vCount = 0;
  
      select isnull(count(*),0)
      into @vCount
      from CUSTOM_ATTRIBUTE
      where CUSTOM_ATTRIBUTE_CONTEXT = @pContext
      and upper(DISPLAY_NAME) = upper(@pName);
  
    RETURN @vCount;

  END;
END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_CUST_ATTR_PurifyDisplayName');

CREATE FUNCTION SAHARA.fn_CUST_ATTR_PurifyDisplayName
(
  IN @pSessionKey NUMERIC(10),
  IN @pContext VARCHAR(32),
  IN @pSourceName VARCHAR(256),
  IN @pDisplayName VARCHAR(64)
)
  RETURNS VARCHAR(64)

BEGIN
  DECLARE @FUNC VARCHAR(35);
  DECLARE @vDisplayName VARCHAR(64); 
  DECLARE @vDisplayNameBase VARCHAR(64); 
  DECLARE @vSuffix INTEGER;
  DECLARE @vPREFIX VARCHAR(32);

  BEGIN 
    
    SET @vPREFIX = 'Extended Attribute: ';
    
    SET @FUNC = 'fn_CUST_ATTR_PurifyDisplayName';

    SET @vDisplayName = SUBSTR(TRIM(@pDisplayName), 1, @gDISPLAY_NAME_LEN);

     -- check if a display name is already in use (case insensitive)
    CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');
   
    IF @vDisplayName IS NOT NULL THEN

        -- add the prefix if it is not present
    
      IF @vPREFIX <> substr(@vDisplayName, 1, length(@vPREFIX)) THEN
   
	SET @vDisplayName = substr(@vPREFIX || @vDisplayName, 1, @gDISPLAY_NAME_LEN);
      
      END IF;

        -- make sure the display name is unique
      IF SAHARA.fn_CUST_ATTR_NameUsed(@pContext, @vDisplayName) = 1 THEN
      
	CALL SP_Error_RaiseApplicationErrorDetail
         (
           @ECADisplayNameNotUnique,
           @FUNC,
           'the custom attribute display name is already used: ' || @vDisplayName,
           @pSessionKey
         );

      END IF;
    
    ELSE
		if @pSourceName in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY') then
			SET @vPREFIX = 'Business Entity: ';
		end if;
		-- create a display name
        SET @vDisplayName = SUBSTR(@vPREFIX || @pSourceName, 1, @gDISPLAY_NAME_LEN);
        
	SET @vDisplayNameBase = SUBSTR(@vDisplayName, 1, @gDISPLAY_NAME_LEN - 5);

        -- if it's in use, append digits until it is not
        SET @vSuffix = 1;

lbl1:
        LOOP
            -- when we find an unused name, we're done
          
	  IF SAHARA.fn_CUST_ATTR_NameUsed(@pContext, @vDisplayName) = 0 THEN
          
		leave lbl1;     -- break out of the loop
          
	  END IF;
        
	  SET @vSuffix = @vSuffix + 1;

	  IF @vSuffix > 99 then
	  
	    CALL   SP_Error_RaiseApplicationErrorDetail
	      (
               @ECADisplayNameNotUnique,
               @FUNC,
               'unable to generate a unique display name for the custom attribute: ' || @vDisplayName,
               @pSessionKey
               );
          
	  END IF;
          
	  SET @vDisplayName = @vDisplayNameBase || ' (' || @vSuffix || ')';
        
	END LOOP lbl1;

    END IF;
    
    RETURN @vDisplayName;

  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_CUST_ATTR_ReduceColumnName');

CREATE function SAHARA.fn_CUST_ATTR_ReduceColumnName(@pStr LONG VARCHAR, @pMaxLength integer)
  RETURNS VARCHAR(64)
  
  BEGIN
    DECLARE @vRc varchar(2048);
    
    BEGIN
    
      IF @pStr IS NULL OR length(@pStr) <= @pMaxLength THEN
        return @pStr;
      END IF;

      -- try removing underscores
      
      SET @vRc = REPLACE(@pStr, '_', '');

      IF @vRc is null or length(@vRc) <= @pMaxLength THEN
        RETURN @vRc;
      END IF;

      -- try removing vowels

      SET @vRc = REPLACE(@vRc, 'A', '');
      SET @vRc = REPLACE(@vRc, 'E', '');
      SET @vRc = REPLACE(@vRc, 'I', '');
      SET @vRc = REPLACE(@vRc, 'O', '');
      SET @vRc = REPLACE(@vRc, 'U', '');
    
      IF @vRc is null or length(@vRc) <= @pMaxLength THEN
        
	RETURN @vRc;

      END IF;

    -- truncate the string

      RETURN SUBSTR(@vRc, 1, @pMaxLength);
 
  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_CUST_ATTR_PurifyColumnName');

CREATE FUNCTION SAHARA.fn_CUST_ATTR_PurifyColumnName
(
   @pSessionKey NUMERIC(10),
   @pContext  VARCHAR(32),
   @pSourceName LONG VARCHAR,
   @pColumnName VARCHAR(64)
)
   RETURNS VARCHAR(64)

BEGIN
  DECLARE @FUNC VARCHAR(32);
  DECLARE @vPREFIX VARCHAR(10);
  DECLARE @vColumnName VARCHAR(64);
  DECLARE @vColumnNameBase VARCHAR(30);
  DECLARE @vSuffix UNSIGNED INT;

  BEGIN
    SET @FUNC = 'fn_CUST_ATTR_PurifyColumnName';
    SET @vPREFIX = 'CA_';
    SET @vColumnName = UPPER(trim(@pColumnName));
  
    BEGIN
    
      CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');
      
      IF @vColumnName IS NOT NULL THEN

	-- make sure the column name contains only legal characters
        
	  IF  SUBSTR(@vColumnName,1,1) NOT REGEXP '[A-Z]+' THEN

          CALL SP_Error_RaiseApplicationErrorDetail
            (
             @EInvalidArgument,
             @FUNC,
             'the custom attribute column name first character us invalid: ' || @vColumnName,
             @pSessionKey
            );
        END IF;
        
	IF  @vColumnName NOT REGEXP '[[:upper:][:digit:]_#$]+' THEN
          CALL SP_Error_RaiseApplicationErrorDetail
             (
             @EInvalidArgument,
             @FUNC,
             'the custom attribute column name contains an invalid character: ' || @vColumnName,
             @pSessionKey
             );
        END IF;
              
       -- add the prefix if it is not present

	IF @vPREFIX <> SUBSTR(@vColumnName, 1, LENGTH(@vPREFIX)) THEN
            SET @vColumnName = @vPREFIX || @vColumnName;
        END IF;

        IF 30 < length(@vColumnName) THEN
          CALL SP_Error_RaiseApplicationErrorDetail
           (
             @EInvalidArgument,
             @FUNC,
             'the custom attribute column name must not be more ' ||
             'than 30 characters long: ' || @vColumnName,
             @pSessionKey
           );

        END IF;

        -- make sure the column name is unique for the context

        IF sahara.fn_CUST_ATTR_NameUsed(@pContext, @vColumnName) = 1  THEN
          CALL SP_Error_RaiseApplicationErrorDetail
           (
             @ECAColumnNameNotUnique,
             @FUNC,
             'the custom attribute column name is already used: ' || @vColumnName,
             @pSessionKey
           );

        END IF;
      ELSE

        ---- create a column name
        SET @vColumnName = sahara.fn_CUST_ATTR_ReduceColumnName(
    
	    sahara.fn_CUST_ATTR_RemoveInvalidColumnCharacters(upper(@pSourceName)),
                                          @gCOLUMN_NAME_LEN - length(@vPREFIX));

        IF @vColumnName IS NULL THEN

          SET @vColumnName = 'X';
        
	END IF;

        SET @vColumnName = @vPREFIX || @vColumnName;
        
	SET @vColumnNameBase = SUBSTR(@vColumnName, 1, @gCOLUMN_NAME_LEN - 2);

        -- if the column name is being used, add a numeric suffix and try again

        SET @vSuffix = 1;
lbl1:
        LOOP
          IF sahara.fn_CUST_ATTR_NameUsed(@pContext, @vColumnName) = 0  then
          
	    LEAVE lbl1;       -- break out of the loop
          
	  END IF;

          SET @vSuffix = @vSuffix + 1;

          IF @vSuffix > 99 THEN

            CALL SP_Error_RaiseApplicationErrorDetail
            (
              @ECAColumnNameNotUnique,
              @FUNC,
              'unable to generate a unique column name for the custom attribute: ' || @vColumnName,
              @pSessionKey
            );

          END IF;
         
	  SET @vColumnName = @vColumnNameBase || @vSuffix;
        
	END LOOP;
    END IF;
    
    RETURN @vColumnName;
  
    END;

  END;

END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','fn_CUST_ATTR_CreateFromSource');

CREATE FUNCTION SAHARA.fn_CUST_ATTR_CreateFromSource
(
  @pSessionKey NUMERIC(10),
  @pContext VARCHAR(32),
  @pSourceName LONG VARCHAR,
  @pDisplayName LONG VARCHAR default null,
  @pColumnName LONG VARCHAR DEFAULT NULL 
)
  RETURNS INTEGER

BEGIN
  DECLARE @FUNC VARCHAR(32);
  DECLARE @vCt UNSIGNED INTEGER;
  DECLARE @vDisplayName VARCHAR(64);
  DECLARE @vCustomAttrKey NUMERIC(10);
  DECLARE @vColumnName VARCHAR(30);

  BEGIN
    ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();

    SET @FUNC = 'fn_CUST_ATTR_CreateFromSource';

    CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');
    -- validate the context
    SET @vCt = 0;
    
    select count(*)
      into @vCt
      from CUSTOM_ATTRIBUTE_CONTEXT
     where CUSTOM_ATTRIBUTE_CONTEXT = @pContext;

    IF @vCt = 0 THEN

      CALL SP_Error_RaiseApplicationErrorDetail
            (
             @ECustomAttrContextNotFound,
             @FUNC,
             'non-existent custom attribute context: ' || @pContext,
             @pSessionKey
            );
    END IF;

    -- validate the source name - we dont' care much if it is an
    -- existing source attribute, but the name must be usable

    IF @pSourceName IS NULL THEN

      CALL SP_Error_RaiseApplicationErrorDetail
            (
             @EInvalidArgument,
             @FUNC,
             'the source custom attribute name must not be null or an empty string',
             @pSessionKey
            );

    END IF;

    -- valid or assign the display name
    SET @vDisplayName = SAHARA.fn_CUST_ATTR_PurifyDisplayName(@pSessionKey, @pContext, @pSourceName, @pDisplayName);

    -- validate or assign the column name
    SET @vColumnName = SAHARA.fn_CUST_ATTR_PurifyColumnName(@pSessionKey, @pContext, @pSourceName, @pColumnName);

    -- create the new custom attribute
    SET @vCustomAttrKey = 0 ;


    insert into CUSTOM_ATTRIBUTE
        ( CUSTOM_ATTRIBUTE_CONTEXT, TYPE, DISPLAY_NAME,
         COLUMN_NAME, COLUMN_TYPE, SOURCE_NAME)
    values
        ( @pContext, 'SOURCE COPY', @vDisplayName,
         @vColumnName, 'varchar(4000)', @pSourceName);

    RETURN @vCustomAttrKey;

  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_Delete1');

CREATE PROCEDURE SAHARA.SP_CUST_ATTR_Delete1
(
  @pSessionKey NUMERIC(10),
  @pCustomAttrKey NUMERIC(10)
)
BEGIN
  DECLARE @FUNC VARCHAR(32); 
  DECLARE @vRowCnt INTEGER;
  DECLARE @vName varchar(256);
  
  BEGIN 

    ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_CUST_ATTR_Delete1';
    CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

    select SOURCE_NAME
      into @vName
      from CUSTOM_ATTRIBUTE
     where CUSTOM_ATTRIBUTE_KEY = @pCustomAttrKey;

	delete from CUSTOM_ATTRIBUTE
    where CUSTOM_ATTRIBUTE_KEY = @pCustomAttrKey;

    SET @vRowCnt = @@rowcount;
    
    IF @vRowCnt = 0 THEN 
      CALL SP_Error_RaiseApplicationErrorDetail
        (
        @ECustomAttrNotFound,
        @FUNC,
        'custom attribute not found to delete - key: ' || @pCustomAttrKey,
        @pSessionKey
        );
    END IF;
  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('EVENT','EV_PurgeSrcAttrsForDeletedSources');

CREATE  EVENT SAHARA.EV_PurgeSrcAttrsForDeletedSources
handler

BEGIN
  DECLARE @vCount INTEGER;

  BEGIN 
     delete from SRC_CUSTOM_ATTRIBUTE_BASE
     where not exists (select 1
                         from ROLLUP_SOURCE
                        where ROLLUP_SOURCE.ROLLUP_SOURCE_KEY =
                          SRC_CUSTOM_ATTRIBUTE_BASE.ROLLUP_SOURCE_KEY);
    SET @vCount = @@rowcount;
    IF @vCount > 0 THEN
      call SP_Log_LogInfo(eActiveSessionKey, @FUNC,'deleted ' || @vCount || ' source custom attributes ' || '(metadata) from delete sources');
    END IF;
    commit;
  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_PurgeSrcAttrsForDeletedSources');

CREATE procedure SAHARA.SP_CUST_ATTR_PurgeSrcAttrsForDeletedSources()

BEGIN
  DECLARE @FUNC VARCHAR(80);
  
  BEGIN 
    SET  @FUNC = 'SP_CUST_ATTR_PurgeSrcAttrsForDeletedSources';
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'Entry');
    --- open a new trasaction and execute it 
    trigger event EV_PurgeSrcAttrsForDeletedSources( "eActiveSessionKey" = @gConfigActiveSessionKey);
  END;
END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_InitializeLoad');

CREATE PROCEDURE SAHARA.SP_CUST_ATTR_InitializeLoad ()
BEGIN
  DECLARE @FUNC VARCHAR(40);
  DECLARE @ActiveRollupSourceKey_ NUMERIC(10);
  BEGIN 
    ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_CUST_ATTR_InitializeLoad';
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'Entry');
    
    SET @ActiveRollupSourceKey_ = 0;

    -- get the roll-up source key
    select ROLLUP_SOURCE_KEY
      into @ActiveRollupSourceKey_
      from ROLLUP_BATCH
     where ROLLUP_BATCH_KEY = @gLoadActiveBatchKey;

    IF @ActiveRollupSourceKey_ > 0 THEN 
    -- delete any custom attribute names for deleted sources
      CALL sahara.SP_CUST_ATTR_PurgeSrcAttrsForDeletedSources()
    END IF;
  END;
END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_LoadNames');

CREATE PROCEDURE SAHARA.SP_CUST_ATTR_LoadNames
(@pContext VARCHAR(32))

BEGIN 
  DECLARE @FUNC VARCHAR(30);
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @ActiveRollupSourceKey_ NUMERIC(10);
  DECLARE @rowcnt INTEGER;

  BEGIN
      ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();
    SET @FUNC = 'SP_CUST_ATTR_LoadNames';
    
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'Entry');
    -- assumes that caller holds the load control
    call SP_Log_LogInfo(@gLoadActiveSessionKey, @FUNC,'processing custom attribute names for ' ||'context: ' || @pContext);
    -- tbd - maybe this should be a merge statement

    -- delete any previously collected custom attribute names for the source
    -- in preparation for inserting the latest ones
    
    SET @ActiveRollupSourceKey_ = 0;

    -- get the roll-up source key
    select ROLLUP_SOURCE_KEY
      into @ActiveRollupSourceKey_
      from ROLLUP_BATCH
     where ROLLUP_BATCH_KEY = @gLoadActiveBatchKey;

    delete from SRC_CUSTOM_ATTRIBUTE_BASE
     where CUSTOM_ATTRIBUTE_CONTEXT = @pContext
       and ROLLUP_SOURCE_KEY = @ActiveRollupSourceKey_;

    -- insert the names rolled-up in this batch for the source
    SET @vSQL = 
      'insert into SRC_CUSTOM_ATTRIBUTE_BASE ' ||
      '(CUSTOM_ATTRIBUTE_CONTEXT, ROLLUP_SOURCE_KEY, NAME) ' ||
      'select distinct '''||@pContext||''','|| @ActiveRollupSourceKey_ ||', CUSTOM_ATTRIBUTE_.NAME ' ||
      'from SAHARA_APP.CUSTOM_ATTRIBUTE_ ' ||
      'join SAHARA_APP.' || @pContext || '_ ' ||
      'on CUSTOM_ATTRIBUTE_.OID = ' || @pContext || '_.' || @pContext || '_OID ' ||
      'join SAHARA.ROLLUP_DATA_SET_TYPE ' ||
      'on ROLLUP_DATA_SET_TYPE.VOM_OBJECT_TYPE = ' ||
      '   CUSTOM_ATTRIBUTE_.VOM_OBJECT_TYPE ' ||
      '   AND ROLLUP_DATA_SET_TYPE.DEST_TABLE = ''' || 
      @pContext || ''' ';
    
    EXECUTE IMMEDIATE @vSQL;
    
    SET @rowcnt = @@rowcount; 
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
    
    CALL SP_Log_LogInfo(@gLoadActiveSessionKey, @FUNC,'context: ' || @pContext ||
                    ', source custom attributes names inserted: ' || @rowcnt);
 	    
 END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_LoadValues');

CREATE PROCEDURE SAHARA.SP_CUST_ATTR_LoadValues
(@pContext VARCHAR(32)) 

BEGIN
  DECLARE @FUNC VARCHAR(30);
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vrowcount INTEGER;

  BEGIN 
    SET @FUNC = 'SP_CUST_ATTR_LoadValues';
    
    TRUNCATE TABLE SAHARA.tmp_Attributes;    
    
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'Entry');

    -- assumes that caller holds the load control
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'processing custom attribute values for context: ' || @pContext);

    -- get the names of the attributes to process
    
    INSERT INTO SAHARA.tmp_Attributes
    select *
      from P_CUSTOM_ATTRIBUTE
     where CUSTOM_ATTRIBUTE_CONTEXT = @pContext
       and TYPE = 'SOURCE COPY';

    -- if there are no attributes to process for the context, we're done
    IF NOT EXISTS ( SELECT 1 FROM SAHARA.tmp_Attributes) THEN
      --- don't do anything 
      RETURN;
    
    END IF;
    
    FOR AttributesLOOP AS AttributesCUR CURSOR FOR 
      select 
        CUSTOM_ATTRIBUTE_KEY AS CUSTOM_ATTRIBUTE_KEY_R,
        CUSTOM_ATTRIBUTE_CONTEXT AS CUSTOM_ATTRIBUTE_CONTEXT_R,
        TYPE AS TYPE_R,
        DISPLAY_NAME AS DISPLAY_NAME_R,
        COLUMN_NAME AS COLUMN_NAME_R,
        COLUMN_TYPE AS COLUMN_TYPE_R,
        SOURCE_NAME AS SOURCE_NAME_R
      from 
        SAHARA.tmp_Attributes
    DO 
      SET @vSQL = 'update SAHARA_APP.' || @pContext || '_ DEST ' ||
                   'set "' || COLUMN_NAME_R || '" = ' ||
                        '(select SRC.VALUE ' ||
                           'from SAHARA_APP.CUSTOM_ATTRIBUTE_ SRC ' ||
                           'join SAHARA.ROLLUP_DATA_SET_TYPE ' ||
                           '  on ROLLUP_DATA_SET_TYPE.VOM_OBJECT_TYPE = ' ||
                           '     SRC.VOM_OBJECT_TYPE ' ||
                           '     AND ROLLUP_DATA_SET_TYPE.DEST_TABLE = ''' || 
                           @pContext || ''' ' ||
                          'where DEST.' || @pContext || '_OID = SRC.OID ' ||
                            'and SRC.NAME = '''|| SOURCE_NAME_R ||''' )';

      CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL:' || @vSQL);
	  EXECUTE IMMEDIATE @vSQL;

      SET @vrowcount = @@rowcount;
      
      CALL SP_Log_LogInfo(@gLoadActiveSessionKey, @FUNC,
      'context: ' || @pContext ||', 
      column: ' || COLUMN_NAME_R ||
      ', values: ' || @vrowcount);
    
    END FOR;
  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_Load');

CREATE procedure SAHARA.SP_CUST_ATTR_Load(@pContext varchar(32)) 

BEGIN
  DECLARE @FUNC VARCHAR(30);

  BEGIN 
    SET @FUNC = 'SP_CUST_ATTR_Load';
    -- we want all or nothing from this call
   
    SAVEPOINT CustomAttributeImplLoadSP;
    
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'Entry');

    CALL SAHARA.SP_CUST_ATTR_LoadNames (@pContext);
    CALL SAHARA.SP_CUST_ATTR_LoadValues(@pContext);

  EXCEPTION
    WHEN OTHERS THEN
      ROLLBACK TO SAVEPOINT CustomAttributeImplLoadSP;
    
  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_GetTableLocks');

CREATE procedure SAHARA.SP_CUST_ATTR_GetTableLocks(@pSessionKey NUMERIC(10),
@pTable VARCHAR(1024) )

BEGIN
  DECLARE @FUNC VARCHAR(40);
  DECLARE @vSQL LONG VARCHAR;
  
  BEGIN 
    SET @FUNC = 'SP_CUST_ATTR_GetTableLocks';
    CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'Entry');
   
   -- gain locks on everything we need to modify in the hope that will
    -- ensure success
    --
    -- don't get a lock on the current view because it's redundant with
    -- getting one on the destination table and if the view is invalid,
    -- it will fail

    SET TEMPORARY OPTION BLOCKING_TIMEOUT = 5000;

    SET @vSQL = 'lock table ' ||
                    -- source table or view
                    'SAHARA_APP.' || @pTable || '_ ' ||
            'in exclusive mode';

    CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'SQL: ' || @vSQL);

    EXECUTE IMMEDIATE @vSQL;

    SET @vSQL = 'lock table ' ||
                    -- destination table
                    'SAHARA."' || @pTable || '" ' ||
            'in exclusive mode';

    CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'SQL: ' || @vSQL);

    EXECUTE IMMEDIATE @vSQL;

  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_UpdateView');

CREATE PROCEDURE SAHARA.SP_CUST_ATTR_UpdateView(
  @pSessionKey NUMERIC(10),
  @pTable  VARCHAR(1024),
  @pDBMSOutput BIT  DEFAULT 0)

  BEGIN 
    DECLARE @FUNC VARCHAR(35);
    DECLARE @vMakeList LONG VARCHAR;
    DECLARE @vSQL LONG VARCHAR;
    DECLARE @vViewName LONG VARCHAR;

    BEGIN 
      
      ---- Intialize connection variables 

      call sahara.SP_saharaAPIInit();

      TRUNCATE TABLE sahara.tmp_columns;

      SET @FUNC = 'SP_CUST_ATTR_UpdateView';
  
      CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'Entering:');
     
      IF @pDBMSOutput = 1 THEN
      /* no idea why why we need this  */ 
        MESSAGE 'Replacing view SAHARA.' || @pTable || '_CURRENT...' to client;
      
      END IF;

      insert into sahara.tmp_columns (column_name)
      select COLUMN_NAME from syscolumn col, systable tab, sysuser u 
      where 
        tab.table_id = col.table_id
        and  COLUMN_NAME not like 'T2\_%' escape '\'
        and tab.creator = u.user_id
        and user_name = 'SAHARA'
        and table_name =@pTable;

      IF NOT EXISTS (select 1 from sahara.tmp_columns)   THEN 

        CALL   SP_Error_RaiseApplicationErrorDetail
        (
          @ETableNotFound,
          @FUNC,
          'table ' || @pTable || ' not found or contains no usable columns'
        );
       
       ELSE 

         SET @vViewName = @pTable || '_CURRENT';

         select List(column_name) into @vMakeList from sahara.tmp_columns ; 

         SET @vSQL =  'CALL SAHARA.SP_DROP_OBJECT_SAHARA(''VIEW'','''|| @vViewName ||'''); '; 
         
	 SET @vSQL = @vSQL ||' create view SAHARA.' || @vViewName ||' as ' ||
            ' select  '|| @vMakeList ||
            ' from SAHARA."' || @pTable || '"' ||
            ' where T2_IS_CURRENT = ''T''' ||
            ' and T2_IS_DELETED = ''F''';

         CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'SQL: ' || @vSQL);

        EXECUTE IMMEDIATE @vSQL;
  
      END IF;
   END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_PublishDelete');

CREATE PROCEDURE SAHARA.SP_CUST_ATTR_PublishDelete
(
@pSessionKey NUMERIC(10))

BEGIN
  DECLARE @FUNC VARCHAR(40);
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vStagingTableName VARCHAR(1024);
  DECLARE @vStagingColExists INTEGER;

  BEGIN 
    SET @FUNC = 'SP_CUST_ATTR_PublishDelete';
    CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'Entry');

    -- gain locks on everything we need to modify in the hope that will
    -- ensure success
    
    FOR AttributesLOOP AS AttributesCUR CURSOR FOR 
    select 
      CUSTOM_ATTRIBUTE_KEY AS CUSTOM_ATTRIBUTE_KEY_R,
      CUSTOM_ATTRIBUTE_CONTEXT AS CUSTOM_ATTRIBUTE_CONTEXT_R,
      TYPE AS TYPE_R,
      DISPLAY_NAME AS DISPLAY_NAME_R,
      COLUMN_NAME AS COLUMN_NAME_R,
      COLUMN_TYPE AS COLUMN_TYPE_R,
      SOURCE_NAME AS SOURCE_NAME_R
    from 
      SAHARA.tmp_CADeleted

    DO
    
      -- We process the production table and the current first because
      -- they are the most likely to fail due to external interference
      -- (e.g. a report run).  We gained an exclusive lock on the tables
      -- before executing the alter table, but since alter table has an
      -- implicit commit associated with it, the locks are lost after the
      -- first alter.  This presents an unavoidable race condition which
      -- can only be correct by a retry later.

    
    
      CALL SAHARA.SP_CUST_ATTR_GetTableLocks(@pSessionKey, CUSTOM_ATTRIBUTE_CONTEXT_R);

      -- drop the column from the production (warehouse) table
      -- dropping the column also drops the index on the column
      
      IF sahara.fn_HELPER_IndexExists('I_CA_' || CUSTOM_ATTRIBUTE_KEY_R) = 1 THEN

        SET @vSQL = 'drop index I_CA_' || CUSTOM_ATTRIBUTE_KEY_R;
      
	CALL SP_Log_LogDebug(@pSessionKey,@FUNC,'SQL: ' || @vSQL);
      
	EXECUTE IMMEDIATE @vSQL


      END IF;

      IF SAHARA.fn_HELPER_ColumnExists('SAHARA',CUSTOM_ATTRIBUTE_CONTEXT_R, COLUMN_NAME_R) = 1 THEN
        SET @vSQL = 'alter table SAHARA."' || CUSTOM_ATTRIBUTE_CONTEXT_R ||
                '" drop  "' || COLUMN_NAME_R || '"';

        CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'SQL: ' || @vSQL);

        EXECUTE IMMEDIATE @vSQL;

      END IF;

      CALL SAHARA.SP_CUST_ATTR_UpdateView(@pSessionKey,CUSTOM_ATTRIBUTE_CONTEXT_R);

      -- drop the column from the staging table
      set @vStagingTableName = CUSTOM_ATTRIBUTE_CONTEXT_R || '_';

      select SAHARA.fn_HELPER_ColumnExists('SAHARA_APP',@vStagingTableName,COLUMN_NAME_R) into 
      @vStagingColExists ;


      
      IF @vStagingColExists = 1 then
        
	SET @vSQL = 'alter table SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R ||
                '_ drop  "' || COLUMN_NAME_R || '"';
        
	CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'SQL: ' || @vSQL);

       
        EXECUTE IMMEDIATE @vSQL;
      
      END IF;

      -- we want the following two statements to complete atomically
      
      BEGIN
      
	SAVEPOINT CustomAttributePublishDeleteSP;

        -- delete the row from the metadata
        -- a foreign key with cascade delete will delete the data set loading
        -- metadata
      
        DELETE FROM P_CUSTOM_ATTRIBUTE
        WHERE CUSTOM_ATTRIBUTE_KEY = CUSTOM_ATTRIBUTE_KEY_R;

        -- create a queue entry for Cognos metamodel update
        

        CALL SAHARA.SP_COGNOS_MOD_UPD_EnQCustomAttrDelete;


      EXCEPTION
        WHEN OTHERS THEN
          ROLLBACK TO SAVEPOINT CustomAttributePublishDeleteSP;

      END;

      -- release the table locks

      COMMIT;
      
    END FOR;

  EXCEPTION
  
  WHEN OTHERS THEN
  -- release the table locks
    COMMIT;

  END;

END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_PublishAdd');

CREATE PROCEDURE sahara.SP_CUST_ATTR_PublishAdd( 
  @pSessionKey numeric(10) ) 

BEGIN
  DECLARE @FUNC varchar(30);
  DECLARE @vSQL long varchar;

  BEGIN
    SET @FUNC = 'SP_CUST_ATTR_PublishAdd';
    CALL SP_Log_LogDebug(@pSessionKey,@FUNC,'Entry');
    -- gain locks on everything we need to modify in the hope that will
    -- ensure success

    FOR AttributesLOOP as AttributesCUR dynamic scroll cursor for
      select CUSTOM_ATTRIBUTE_KEY as CUSTOM_ATTRIBUTE_KEY_R,
        CUSTOM_ATTRIBUTE_CONTEXT as CUSTOM_ATTRIBUTE_CONTEXT_R,
        TYPE as TYPE_R,
        DISPLAY_NAME as DISPLAY_NAME_R,
        COLUMN_NAME as COLUMN_NAME_R,
        COLUMN_TYPE as COLUMN_TYPE_R,
        SOURCE_NAME as SOURCE_NAME_R
        from SAHARA.tmp_CAAdded
    DO
      call sahara.SP_CUST_ATTR_GetTableLocks(@pSessionKey,CUSTOM_ATTRIBUTE_CONTEXT_R);
      -- We process the production table and the current first because
      -- they are the most likely to fail due to external interference
      -- (e.g. a report run).  We gained an exclusive lock on the tables
      -- before executing the alter table, but since alter table has an
      -- implicit commit associated with it, the locks are lost after the
      -- first alter.  This presents an unavoidable race condition which
      -- can only be correct by a retry later.
      -- add the column to the production (warehouse) table

      IF SAHARA.fn_HELPER_ColumnExists('SAHARA',CUSTOM_ATTRIBUTE_CONTEXT_R,COLUMN_NAME_R) = 0 THEN

        set @vSQL = 'alter table SAHARA."' || CUSTOM_ATTRIBUTE_CONTEXT_R
           || '" add ("' || COLUMN_NAME_R || '" ' || COLUMN_TYPE_R || ')';

        CALL SP_Log_LogDebug(@pSessionKey,@FUNC,'SQL: ' || @vSQL);
        
	EXECUTE IMMEDIATE @vSQL
      
      END IF;

      -- add an index on the column
      
      IF sahara.fn_HELPER_IndexExists('I_CA_' || CUSTOM_ATTRIBUTE_KEY_R) = 1 THEN

        SET @vSQL = 'create index I_CA_' || CUSTOM_ATTRIBUTE_KEY_R
           || ' on SAHARA."' || CUSTOM_ATTRIBUTE_CONTEXT_R
           || '" ("' || COLUMN_NAME_R || '") ;';
      
	CALL SP_Log_LogDebug(@pSessionKey,@FUNC,'SQL: ' || @vSQL);
        EXECUTE IMMEDIATE @vSQL

      END IF;

      -- add the column to the current view
      
      CALL sahara.SP_CUST_ATTR_UpdateView(@pSessionKey,CUSTOM_ATTRIBUTE_CONTEXT_R);

      -- add the column to the staging table
      
      IF SAHARA.fn_HELPER_ColumnExists('SAHARA_APP',CUSTOM_ATTRIBUTE_CONTEXT_R || '_',
        COLUMN_NAME_R) = 0 THEN

        SET @vSQL = 'alter table SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R
           || '_ add ("' || COLUMN_NAME_R || '" ' || COLUMN_TYPE_R || ')';
        
	CALL SP_Log_LogDebug(@pSessionKey,@FUNC,'SQL: ' || @vSQL);
        EXECUTE IMMEDIATE @vSQL

      END IF;

      -- we want the following two statements to complete atomically
      BEGIN
        
	SAVEPOINT CustomAttributePublishAddSP;
        -- add the row the metadata

        insert into P_CUSTOM_ATTRIBUTE
          ( CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,
          TYPE,DISPLAY_NAME,
          COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME ) 
	  values (CUSTOM_ATTRIBUTE_KEY_R,CUSTOM_ATTRIBUTE_CONTEXT_R,
          TYPE_R,DISPLAY_NAME_R,
          COLUMN_NAME_R,COLUMN_TYPE_R,SOURCE_NAME_R);

--          values( CUSTOM_ATTRIBUTE_KEY_R,CUSTOM_ATTRIBUTE_CONTEXT_R,
 --        TYPE_R,DISPLAY_NAME_R,
--          COLUMN_NAME_R,COLUMN_TYPE_R,SOURCE_NAME_R ) ;
        -- create the data set loading metadata
        
	insert into ROLLUP_DATA_SET_COLUMN
          ( ROLLUP_DATA_SET_TYPE_KEY,NAME,CUSTOM_ATTRIBUTE_KEY,LOAD_PROCESSING ) 
          select ROLLUP_DATA_SET_TYPE_KEY,COLUMN_NAME_R,CUSTOM_ATTRIBUTE_KEY_R,'TYPE 1'
            from ROLLUP_DATA_SET_TYPE
            where DEST_TABLE = CUSTOM_ATTRIBUTE_CONTEXT_R;
        -- create a queue entry for Cognos metamodel update
        
	CALL SAHARA.SP_COGNOS_MOD_UPD_EnQCustomAttrAdd()
      
      EXCEPTION

        WHEN OTHERS THEN

          ROLLBACK TO SAVEPOINT CustomAttributePublishAddSP;
          
      END;
      -- release the table locks
      COMMIT ;
    END FOR;

  EXCEPTION
    WHEN OTHERS THEN

      -- release the table locks
      COMMIT ;

  END;

END;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_CUST_ATTR_Publish');
create procedure sahara.SP_CUST_ATTR_Publish()
BEGIN 
    DECLARE @FUNC VARCHAR(40);
    DECLARE @vSessionKey NUMERIC(10);
    DECLARE @HaveDeletedWork BIT ;
    DECLARE @HaveAddedWork BIT ;
    DECLARE @vrwcnt INTEGER ;
    DECLARE @vIsolation integer;
    DECLARE @vsql LONG VARCHAR;

    BEGIN
      
      ---- Intialize connection variables 

      call sahara.SP_saharaAPIInit();
      --- clear these two tables first
      
      delete from tmp_CADeleted;
      delete from tmp_CAAdded; 
       
      set @HaveDeletedWork =0;
      set @HaveAddedWork =0;
      set @vrwcnt = null;
     

      select @@isolation into @vIsolation from dummy; /* expects it works everytime  */

      set temporary option isolation_level = 3; /* read commited records and also open it for read only mode */

      commit; /* just to make sure rows are not locked  */

      -- look for deleted custom attributes

      insert into tmp_CADeleted
      select *
      from P_CUSTOM_ATTRIBUTE PUBLISHED
      where PUBLISHED.SOURCE_NAME not in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY')
	    and not exists (select 1
                         from CUSTOM_ATTRIBUTE CONFIGURED
                        where CONFIGURED.CUSTOM_ATTRIBUTE_KEY =
                              PUBLISHED.CUSTOM_ATTRIBUTE_KEY);
     
      set @vrwcnt = @@rowcount;

      if isnull(@vrwcnt,0) = 0 then 
        set @HaveDeletedWork = 0; 
      else 
        set @HaveDeletedWork = 1;
      end if;


     


--- look for added attribute 
      set @vrwcnt = null; 

      insert into tmp_CAAdded
      select *
      from CUSTOM_ATTRIBUTE CONFIGURED
      where CONFIGURED.SOURCE_NAME not in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY')
	    and not exists (select 1
                         from P_CUSTOM_ATTRIBUTE PUBLISHED
                        where PUBLISHED.CUSTOM_ATTRIBUTE_KEY =
                              CONFIGURED.CUSTOM_ATTRIBUTE_KEY);

      set @vrwcnt = @@rowcount;  

      if isnull(@vrwcnt,0) = 0 then 
        set @HaveAddedWork = 0; 
      else 
        set @HaveAddedWork = 1;
      end if;



      commit; /* dont hold tables as read is done */
      set @vsql = 'set temporary option isolation_level = '||@vIsolation ;
      execute immediate @vsql; /* setiing it back to old isolation lkevel  */

    END;
    
    if @HaveDeletedWork = 0 and @HaveAddedWork = 0 then
        return; /* just return nothing to do anything */
    else 
      --- got something to do   
      begin 
        --- get a session key for below work 
      
        set @vSessionKey = fn_Session_BeginServerSession(@FUNC);

        --- get control againsts this session 

        call sahara.SP_Load_AcquireExclusiveLock(@vSessionKey);      

        IF @HaveDeletedWork = 1 THEN 
          CALL SAHARA.SP_CUST_ATTR_PublishDelete(@vSessionKey);
        END IF;
        
        IF @HaveAddedWork = 1 THEN 
          CALL SAHARA.SP_CUST_ATTR_PublishAdd(@vSessionKey);
        END IF;

      commit;
      exception
        when others then

          -- save off the error message so we can report it
            
          CALL SP_Log_LogError(@vSessionKey, @FUNC,
                         'exception caught: ' || @@sqlcode);
          
      end;
      -- we're done with the staging table

      call sahara.SP_Load_ReleaseExclusiveLock(@vSessionKey);
      commit;

      -- we're done with the session
      call fn_Session_EndSession(@vSessionKey);
      
    end if;
end;


/*
 **********************************************************************
 * SP_CUST_ATTR_BELoad():
 * Auto Configure Business Entity Custom Attributes 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_CUST_ATTR_BELoad' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_CUST_ATTR_BELoad
end if;

CREATE PROCEDURE SAHARA.SP_CUST_ATTR_BELoad ()
BEGIN
	DECLARE @func varchar(64);
	DECLARE @vContext varchar(32);
	DECLARE @vName varchar(256);
	DECLARE @sqlstate varchar(5);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();    
	
	Set @FUNC = 'sahara.SP_CUST_ATTR_BELoad';
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @func, 'Entry');

	-- iterate thru all custom attributes and only auto configure BE type ones
	FOR @BEAttributeLoop AS @BEAttributeCursor CURSOR FOR (   
		select CUSTOM_ATTRIBUTE_CONTEXT as vContext, NAME as vName
		  from UNUSED_SRC_CUSTOM_ATTRIBUTE
	)
	DO
		if vName in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY') then
			    if fn_GetConfigBool('BUSINESS_ENTITY', vName)=1 then
					call fn_CUST_ATTR_CreateFromSource(@gLoadActiveSessionKey,vContext,vName);
			end if;
		end if;
	END FOR;    

exception
    when others then
		CALL SP_Error_RaiseApplicationError(20051,'raise');
end;


---  giving execute permission to following sp/fuctions to sahara_app
---  as these 2 functions/procs are public( exposed to other users and SAHARA ) , assuming 
---- these are consumed by GUI or some other user  
---- also initializing global variables only from these procs/functions 

grant execute on sahara.fn_CUST_ATTR_CreateFromSource to sahara_app;
grant execute on sahara.SP_CUST_ATTR_Delete1 to sahara_app;
grant execute on sahara.SP_CUST_ATTR_Load to sahara_app;
grant execute on sahara.SP_CUST_ATTR_Publish to sahara_app;
grant execute on sahara.SP_CUST_ATTR_InitializeLoad to sahara_app;
grant execute on sahara.SP_CUST_ATTR_UpdateView to sahara_app;
message 'Creating/replacing CONFIGURATION module stored procedures...' to client;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_InstallBool' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_InstallBool
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_InstallBool" (@pSection varchar(32),
    @pName varchar(32),
    @pValue bit,
    @pDisplayName varchar(256),
    @pDescription long varchar,
    @pVisible bit default 1,
    @pEditable bit default 1) 
BEGIN
    DECLARE @vDisplayName varchar(256);
    DECLARE @vDescription long varchar;
    DECLARE @vValue char(1);
    DECLARE @vVisible char(1);
    DECLARE @vEditable char(1);

    SET @vDisplayName = trim(@pDisplayName);
    SET @vDescription = trim(@pDescription);
    SET @vValue = fn_BoolToChar(@pValue);
    SET @vVisible = fn_BoolToChar(@pVisible);
    SET @vEditable = fn_BoolToChar(@pEditable);

    insert into CONFIGURATION
        (SECTION, NAME, TYPE, DISPLAY_NAME, DESCRIPTION,
         BOOLEAN_VALUE, VISIBLE, EDITABLE)
	on existing update values
        (upper(@pSection), upper(@pName), 'BOOLEAN', @vDisplayName, @vDescription,
         @vValue, @vVisible, @vEditable);

END;

grant execute on sahara.fn_Configuration_InstallBool to sahara;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_InstallDate' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_InstallDate
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_InstallDate" (@pSection varchar(32),
    @pName varchar(32),
    @pValue date,
    @pDisplayName varchar(256),
    @pDescription long varchar,
    @pVisible bit default 1,
    @pEditable bit default 1) 
BEGIN
    DECLARE @vDisplayName varchar(256);
    DECLARE @vDescription long varchar;
    DECLARE @vVisible char(1);
    DECLARE @vEditable char(1);

    SET @vDisplayName = trim(@pDisplayName);
    SET @vDescription = trim(@pDescription);
    SET @vVisible = fn_BoolToChar(@pVisible);
    SET @vEditable = fn_BoolToChar(@pEditable);

    insert into CONFIGURATION
        (SECTION, NAME, TYPE, DISPLAY_NAME, DESCRIPTION,
         DATE_VALUE, VISIBLE, EDITABLE)
	on existing update values
        (upper(@pSection), upper(@pName), 'DATE', @vDisplayName, @vDescription,
         @pValue, @vVisible, @vEditable);

END;

grant execute on sahara.fn_Configuration_InstallDate to sahara;

-------------------------------------------------------------------------------
/*
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_InstallDayInterval' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_InstallDayInterval
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_InstallDayInterval" (@pSection varchar(32),
    @pName varchar(32),
    @pValue integer,
    @pDisplayName varchar(256),
    @pDescription long varchar,
    @pVisible bit default 1,
    @pEditable bit default 1) 
BEGIN
    DECLARE @vDisplayName varchar(256);
    DECLARE @vDescription long varchar;
    DECLARE @vVisible char(1);
    DECLARE @vEditable char(1);

    SET @vDisplayName = trim(@pDisplayName);
    SET @vDescription = trim(@pDescription);
    SET @vVisible = fn_BoolToChar(@pVisible);
    SET @vEditable = fn_BoolToChar(@pEditable);

    insert into CONFIGURATION
        (SECTION, NAME, TYPE, DISPLAY_NAME, DESCRIPTION,
         DAY_INTERVAL_VALUE, VISIBLE, EDITABLE)
	on existing update values
        (upper(@pSection), upper(@pName), 'DAY INTERVAL', @vDisplayName, @vDescription,
         @pValue, @vVisible, @vEditable);

END;

grant execute on sahara.fn_Configuration_InstallDayInterval to sahara;
*/
-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_InstallInt' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_InstallInt
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_InstallInt" (@pSection varchar(32),
    @pName varchar(32),
    @pValue numeric(10),
    @pDisplayName varchar(256),
    @pDescription long varchar,
    @pVisible bit default 1,
    @pEditable bit default 1) 
BEGIN
    DECLARE @vDisplayName varchar(256);
    DECLARE @vDescription long varchar;
    DECLARE @vVisible char(1);
    DECLARE @vEditable char(1);

    SET @vDisplayName = trim(@pDisplayName);
    SET @vDescription = trim(@pDescription);
    SET @vVisible = fn_BoolToChar(@pVisible);
    SET @vEditable = fn_BoolToChar(@pEditable);

    insert into CONFIGURATION
        (SECTION, NAME, TYPE, DISPLAY_NAME, DESCRIPTION,
         INTEGER_VALUE, VISIBLE, EDITABLE)
	on existing update values
        (upper(@pSection), upper(@pName), 'INTEGER', @vDisplayName, @vDescription,
         @pValue, @vVisible, @vEditable);

END;

grant execute on sahara.fn_Configuration_InstallInt to sahara;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_InstallStr' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_InstallStr
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_InstallStr" (@pSection varchar(32),
    @pName varchar(32),
    @pValue varchar(2000),
    @pDisplayName varchar(256),
    @pDescription long varchar,
    @pVisible bit default 1,
    @pEditable bit default 1) 
BEGIN
    DECLARE @vDisplayName varchar(256);
    DECLARE @vDescription long varchar;
    DECLARE @vVisible char(1);
    DECLARE @vEditable char(1);

    SET @vDisplayName = trim(@pDisplayName);
    SET @vDescription = trim(@pDescription);
    SET @vVisible = fn_BoolToChar(@pVisible);
    SET @vEditable = fn_BoolToChar(@pEditable);

    insert into CONFIGURATION
        (SECTION, NAME, TYPE, DISPLAY_NAME, DESCRIPTION,
         STRING_VALUE, VISIBLE, EDITABLE)
	on existing update values
        (upper(@pSection), upper(@pName), 'STRING', @vDisplayName, @vDescription,
         @pValue, @vVisible, @vEditable);

END;

grant execute on sahara.fn_Configuration_InstallStr to sahara;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_SetBool' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_SetBool
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_SetBool" 
(
	@pSessionKey numeric(10),
    @pSection varchar(32),
    @pName varchar(32),
    @pValue bit
) 
BEGIN
	DECLARE @vType varchar(256);
    DECLARE @vValue char(1);
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_Configuration_SetBool';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    SET @vValue = fn_BoolToChar(@pValue);

    -- check the data type and existence of the configuration item

	select TYPE
	  into @vType
	  from sahara.CONFIGURATION
	 where SECTION = upper(@pSection)
	   and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigNotFound,
			@func,
			'Failed setting a configuration item value because the item ' ||
			'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.',
			@pSessionKey
		);
	end if;

    -- make sure the data type is correct
    if 'BOOLEAN' <> @vType then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigWrongType,
			@func,
			'Failed setting an existing configuration item value of ' ||
            'type ' || @vType || ' as type BOOLEAN.  Section: ' || @pSection ||
            ', Name: ' || @pName || '.',
			@pSessionKey
		);
    end if;

    -- update the existing item
    update sahara.CONFIGURATION
       set BOOLEAN_VALUE = @vValue
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

END;

grant execute on sahara.fn_Configuration_SetBool to sahara_app;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_SetDate' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_SetDate
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_SetDate" 
(
	@pSessionKey numeric(10),
    @pSection varchar(32),
    @pName varchar(32),
    @pValue datetime
) 
BEGIN
	DECLARE @vType varchar(256);
	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_Configuration_SetDate';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    -- check the data type and existence of the configuration item

	select TYPE
	  into @vType
	  from sahara.CONFIGURATION
	 where SECTION = upper(@pSection)
	   and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigNotFound,
			@func,
			'Failed setting a configuration item value because the item ' ||
			'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.',
			@pSessionKey
		);
	end if;

    -- make sure the data type is correct
    if 'DATE' <> @vType then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigWrongType,
			@func,
			'Failed setting an existing configuration item value of ' ||
            'type ' || @vType || ' as type DATE.  Section: ' || @pSection ||
            ', Name: ' || @pName || '.',
			@pSessionKey
		);
    end if;

 
    if @pValue is null then
		set @pValue = CURRENT TIMESTAMP;
	end if;

	-- update the existing item
    update sahara.CONFIGURATION
       set DATE_VALUE =  @pValue 
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

END;

grant execute on sahara.fn_Configuration_SetDate to sahara_app;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_SetDayInterval' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_SetDayInterval
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_SetDayInterval" 
(
	@pSessionKey numeric(10),
    @pSection varchar(32),
    @pName varchar(32),
    @pValue integer
) 
BEGIN
	DECLARE @vType varchar(256);
	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_Configuration_SetDayInterval';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    -- check the data type and existence of the configuration item

	select TYPE
	  into @vType
	  from sahara.CONFIGURATION
	 where SECTION = upper(@pSection)
	   and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigNotFound,
			@func,
			'Failed setting a configuration item value because the item ' ||
			'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.',
			@pSessionKey
		);
	end if;

    -- make sure the data type is correct
    if 'DAY INTERVAL' <> @vType then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigWrongType,
			@func,
			'Failed setting an existing configuration item value of ' ||
            'type ' || @vType || ' as type DAY INTERVAL.  Section: ' || @pSection ||
            ', Name: ' || @pName || '.',
			@pSessionKey
		);
    end if;

    -- update the existing item
    update sahara.CONFIGURATION
       set DAY_INTERVAL_VALUE = @pValue
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

END;

grant execute on sahara.fn_Configuration_SetDayInterval to sahara;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_SetInt' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_SetInt
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_SetInt" 
(
	@pSessionKey numeric(10),
    @pSection varchar(32),
    @pName varchar(32),
    @pValue numeric(10)
) 
BEGIN
	DECLARE @vType varchar(256);
	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_Configuration_SetInt';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    -- check the data type and existence of the configuration item
	select TYPE
	  into @vType
	  from sahara.CONFIGURATION
	 where SECTION = upper(@pSection)
	   and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigNotFound,
			@func,
			'Failed setting a configuration item value because the item ' ||
			'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.',
			@pSessionKey
		);
	end if;

    -- make sure the data type is correct
    if 'INTEGER' <> @vType then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigWrongType,
			@func,
			'Failed setting an existing configuration item value of ' ||
            'type ' || @vType || ' as type INTEGER.  Section: ' || @pSection ||
            ', Name: ' || @pName || '.',
			@pSessionKey
		);
    end if;

    -- update the existing item
    update sahara.CONFIGURATION
       set INTEGER_VALUE = @pValue
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

END;

grant execute on sahara.fn_Configuration_SetInt to sahara_app;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Configuration_SetStr' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Configuration_SetStr
end if;

CREATE PROCEDURE "SAHARA"."fn_Configuration_SetStr" 
(
	@pSessionKey numeric(10),
    @pSection varchar(32),
    @pName varchar(32),
    @pValue varchar(2000)
) 
BEGIN
	DECLARE @vType varchar(256);
	DECLARE NO_DATA_FOUND EXCEPTION FOR SQLSTATE '02000';
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_Configuration_SetStr';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    -- check the data type and existence of the configuration item
	select TYPE
	  into @vType
	  from sahara.CONFIGURATION
	 where SECTION = upper(@pSection)
	   and NAME = upper(@pName);

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigNotFound,
			@func,
			'Failed setting a configuration item value because the item ' ||
			'was not found.  Section: ' || @pSection || ', Name: ' || @pName || '.',
			@pSessionKey
		);
	end if;
    -- make sure the data type is correct
    if 'STRING' <> @vType then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@EConfigWrongType,
			@func,
			'Failed setting an existing configuration item value of ' ||
            'type ' || @vType || ' as type STRING.  Section: ' || @pSection ||
            ', Name: ' || @pName || '.',
			@pSessionKey
		);
    end if;

    -- update the existing item
    update sahara.CONFIGURATION
       set STRING_VALUE = @pValue
     where SECTION = upper(@pSection)
       and NAME = upper(@pName);

END;

grant execute on sahara.fn_Configuration_SetStr to sahara_app;

if exists(select 1 from sys.sysevent where event_name = 'EV_RESET_CONTROL_OWNER' and user_name(creator) = 'SAHARA') then
   drop event sahara.EV_RESET_CONTROL_OWNER
end if;

create event sahara.EV_RESET_CONTROL_OWNER
TYPE DatabaseStart
handler
begin
    UPDATE sahara.control_owner SET IsLocked=0, session_key=null;
    commit;
end;

--
-- there needs to be one row in the business view status table
--
insert into SAHARA.BVIEW_STATUS(BVIEW_STATUS_KEY) on existing update values('KEY');
commit;

--
-- there needs to be one row for each control lock in the control owner table
--
insert into SAHARA.CONTROL_OWNER(NAME) on existing update values('SAHARA_CONFIGURATION_CONTROL');
insert into SAHARA.CONTROL_OWNER(NAME) on existing update values('SAHARA_IE_CONTROL');
insert into SAHARA.CONTROL_OWNER(NAME) on existing update values('SAHARA_LOAD_CONTROL');
insert into SAHARA.CONTROL_OWNER(NAME) on existing update values('SAHARA_ETL_SCHEDULER');
insert into SAHARA.CONTROL_OWNER(NAME) on existing update values('SAHARA_METAMODEL_UPDATE');
insert into SAHARA.CONTROL_OWNER(NAME) on existing update values('SAHARA_ETL_METAMODEL_UPDATE');
commit;

--
-- there needs to be one row in the roll-up schedule table
--
insert into SAHARA.ROLLUP_SCHEDULE(ROLLUP_SCHEDULE_KEY) on existing update values('KEY');

commit;

-------------------------------------------------------------------------------
--
-- general configuration items
--
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'ROLL-UP' and name = 'ALLOW_PARTIAL_LOADS' )
then 
    call SAHARA.fn_Configuration_InstallBool
    (
        'ROLL-UP',                              -- section
        'ALLOW_PARTIAL_LOADS',                  -- name
        1,										-- value
        'Allow partial data loads to succeed',  -- display name
        -- description
        'A flag that directs the data loading process to retain data sets ' ||
        'in the database that may not be complete.  Normally, such data sets ' ||
        'are discarded and the load is failed.  This usually occurs as the ' ||
        'result of an error in the roll-up collection logic.  ' ||
        'The default value is false.'
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'ROLL-UP' and name = 'ARCHIVE_RETENTION' )
then 
    call SAHARA.fn_Configuration_InstallInt
    (
        'ROLL-UP',                              -- section
        'ARCHIVE_RETENTION',                    -- name
        10,                                     -- value
        'Roll-up archives to retain',           -- display name
        -- description
        'The number of roll-up archives to retain for each roll-up ' ||
        'source.  Each time a roll-up is begun, any logs and CSV files ' ||
        'from the previous roll-up are archived.  This value defines how ' ||
        'many such obsolete archives to retain for debugging purposes.  ' ||
        'The value for this configuration item must be zero (0) or ' ||
        'larger.  The default value is ten (10).'
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'UTC_ADJUSTMENT' )
then 
    call SAHARA.fn_Configuration_InstallInt
    (
        'SERVER',                               -- section
        'UTC_ADJUSTMENT',                       -- name
        0,		                                -- value
        'UTC adjustment',                       -- display name
        -- description
        'The interval that must be added to UTC time to obtain the ' ||
        'Enterprise Reporter local time.  This is maintained automatically ' ||
        'by the Enterprise Reporter server processes.',
        1,
        0
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'COGNOS' and name = 'COGNOS_URL' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'COGNOS',                               -- section
    'COGNOS_URL',                           -- name
    -- value
    'http://localhost:9300/p2pd/servlet/dispatch',
    'Cognos URL',                           -- display name
    -- description
    'The Cognos URL for CommandCentral Enterprise Reporter.  This is ' ||
    'used by the Enterprise Reporter server to communicate with ' ||
    'the Cognos server.',
	1,
    0
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'COGNOS' and name = 'HOME_REPORT_PATH' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'COGNOS',                               -- section
    'HOME_REPORT_PATH',                     -- name
    -- value
     '/content/folder[@name=''Sample Reports'']/folder[@name=''Storage Allocation and Chargeback'']/report[@name=''Enterprise Storage Visibility Report'']',
    'Default home report',                  -- display name
    -- description
    'The Cognos URL for the default home report.',
	1,
    0
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'COGNOS' and name = 'ROLLUP_STATUS_REPORT_PATH' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'COGNOS',                               -- section
    'ROLLUP_STATUS_REPORT_PATH',            -- name
    -- value
    'content/folder[@name=''Sample Reports'']/folder[@name=''Administrative'']/report[@name=''Rollup status'']',
    'Rollup status report URL',             -- display name
    -- description
    'The Cognos URL for the roll-up status report.  This is ' ||
    'used to display roll-up status in the GUI.',
	1,
	0
);
end if;


if not exists (select 1 from SAHARA.CONFIGURATION where section = 'ORACLE' and name = 'PATCH_5169247_INSTALLED' )
then 
call SAHARA.fn_Configuration_InstallBool
(
    'ORACLE',                               -- section
    'PATCH_5169247_INSTALLED',              -- name
    1,		                                -- value
    -- display name
    'Is Oracle patch 5169247 for 10.2.0.3 installed',
    -- description
    'Oracle patch 5169247 for Oracle 10.2.0.3 database server is required to ' ||
    'correct an Oracle bug that causes the database server connection ' ||
    'process to fail during certain Enterprise Reporter operations.  It ' ||
    'is not possible to determine if the patch is installed from within ' ||
    'Oracle, so this configuration item is utilized to provide the ' ||
    'information to Enterprise Reporter.  If the patch is not installed, ' ||
    'Enterprise Reporter executes additional, sub-optimum code to avoid ' ||
    'the Oracle bug.  After changing this configuration item, all ' ||
    'Enterprise Reporter processes must be restarted before the change ' ||
    'will take effect.  The default value for this configuration item ' ||
    'is false.'
);
end if;

/*
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'ER_LICENSE' and name = 'LAST_POLL' )
then 
call SAHARA.fn_Configuration_InstallDate
(
    'ER_LICENSE',                           -- section
    'LAST_POLL',                            -- name
    -- value
    date('2011/02/09'),
    -- display name
    'Enterprise Reporter last license poll date and time',
    -- description
    'The last date and time, stored in UTC, at which Enterprise Reporter ' ||
    'licenses were polled from the Enterprise Reporter server and copied ' ||
    'into the database.',
	1,
    0
);
end if;


if not exists (select 1 from SAHARA.CONFIGURATION where section = 'ER_LICENSE' and name = 'POLL_INTERVAL' )
then 
call SAHARA.fn_Configuration_InstallInt
(
    'ER_LICENSE',                           -- section
    'POLL_INTERVAL',                        -- name
    21600,                              -- value
    -- display name
    'Enterprise Reporter license poll interval',
    -- description
    'The interval at which Enterprise Reporter licenses are polled ' ||
    'from the Enterprise Reporter server and copied into the ' ||
    'database.  The default value is every 6 hours.'
);
end if;
*/

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TIER' and name = 'DEFAULT_CURRENCY' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'TIER',                                 -- section
    'DEFAULT_CURRENCY',                     -- name
    'USD',                                  -- value
    'Currency for storage tiers',           -- display name
    -- description
    'The currency for tier and storage costs in the storage tier ' ||
    'configuration and charge back reports.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'MESSAGE_LOGGING' and name = 'DEBUG_ENABLED' )
then 
call SAHARA.fn_Configuration_InstallBool
(
    'MESSAGE_LOGGING',                      -- section
    'DEBUG_ENABLED',                        -- name
    0,                                  -- value
    'Debug logging enabled',                -- display name
    -- description
    'A boolean value that determines if debug log messages are written to ' ||
    'the message log table in the database.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'MESSAGE_LOGGING' and name = 'AGING_ENABLED' )
then 
call SAHARA.fn_Configuration_InstallBool
(
    'MESSAGE_LOGGING',                      -- section
    'AGING_ENABLED',                        -- name
    1,                                   -- value
    'Message log aging enabled',            -- display name
    -- description
    'A boolean value that determines if message log records in the database ' ||
    'are aged to prevent unlimited growth of the message log table.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'META-MODEL' and name = 'ACTIVE_UPDATE' )
then 
call SAHARA.fn_Configuration_InstallInt
(
    'META-MODEL',                      -- section
    'ACTIVE_UPDATE',                   -- name
    0,								   -- value
    'Active Metamodel Update',         -- display name
    -- description
    'A interger value that determines state and progress of meta-model update. ' ||
    'Each time a meta-model update is begun, either the business view ADD/MODIFY or ' ||
    'DELETE operation is being performed. This value defines which operation ' ||
    'is in progress like ADD/MODIFY or Delete. Also result of these operations is defined in this value. ' ||
    'The value for this configuration item is one (1) in case of ADD, two (2) in case of DELETE, ' ||
	'zero (0) if update was successful and negative (-1) if update failed.  The default value is zero (0).'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'MESSAGE_LOGGING' and name = 'AGING_RETENTION' )
then 
call SAHARA.fn_Configuration_InstallInt
(
    'MESSAGE_LOGGING',                      -- section
    'AGING_RETENTION',                      -- name
    2419200,                              -- value
    'Message log aging retention',          -- display name
    -- description
    'If message log aging is enabled, this value determines the amount of ' ||
    'time that messages that are retained before they are eligible for aging.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'MESSAGE_LOGGING' and name = 'AGING_INTERVAL' )
then 
call SAHARA.fn_Configuration_InstallInt
(
    'MESSAGE_LOGGING',                      -- section
    'AGING_INTERVAL',                       -- name
    43200,                              -- value
    'Message log aging interval',           -- display name
    -- description
    'If message log aging is enabled, this value determines the amount of ' ||
    'time between each check for records to delete.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'MESSAGE_LOGGING' and name = 'LAST_AGED' )
then 
call SAHARA.fn_Configuration_InstallDate
(
    'MESSAGE_LOGGING',                      -- section
    'LAST_AGED',                            -- name
    -- value
    date('2011/02/09'),
    'Message log last aged',                -- display name
    -- description
    'The date and time, stored in UTC, at which the message log table ' ||
    'was last aged.',
	1,
    0
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'LICENSE' and name = 'LICENSE_TYPE' )
then 
call SAHARA.fn_Configuration_InstallInt
(
    'LICENSE',                           -- section
    'LICENSE_TYPE',                        -- name
    0,                              -- value
    -- display name
    'VOM Advanced license type',
    -- description
    'The type of VOM Adavced License. ' ||
	'This will be set to either of 3 possible values 0  demo, 1- permanent, 2  NFR.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'LICENSE' and name = 'LICENSE_DATE_OF_INSTALL' )
then 
call SAHARA.fn_Configuration_InstallDate
(
    'LICENSE',                           -- section
    'LICENSE_DATE_OF_INSTALL',           -- name
    -- value
    cast(current timestamp as date),
    -- display name
    'VOM Advanced license date',
    -- description
    'This will store the date when the VOM Advanced product was installed ' ||
	'to be able to calculate the number of remaining days for demo license. '||
	'Once the license is changed to permanent then this will no longer be ' ||
	'used to calculate the remaining days.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'LICENSE' and name = 'DAYS_REMAINING' )
then 
call SAHARA.fn_Configuration_InstallInt
(
    'LICENSE',                           -- section
    'DAYS_REMAINING',                    -- name
    0,                                   -- value
    -- display name
    'License days remaining',
    -- description
    'The number of days remaining of VOM Advanced License.' ||
	'This will be populated by GUI for demo licenses and will be populated '||
	'by the license discovery script for NFR.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'LICENSE' and name = 'POLL_INTERVAL' )
then 
call SAHARA.fn_Configuration_InstallInt
(
    'LICENSE',                      -- section
    'POLL_INTERVAL',                -- name
    86400,                          -- value
    -- display name
    'VOM Advanced license poll interval',
    -- description
    'The interval at which VOM Advanced Enterprise Server licenses are polled ' ||
    'from the server and copied into the ' ||
    'database.  The default value is every 24 hours.'
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'LICENSE' and name = 'LAST_POLL' )
then 
call SAHARA.fn_Configuration_InstallDate
(
    'LICENSE',								-- section
    'LAST_POLL',                            -- name
    -- value
    date('2011/03/09'),
    'VOM Advanced last license poll date and time',                -- display name
    -- description
    'The last date and time, stored in UTC, at which VOM Advanced ' ||
    'licenses were polled from the server and copied ' ||
    'into the database.',
	1,
    0
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'ANALYTICS' )
then 
call SAHARA.fn_Configuration_InstallBool
(
    'SERVER',                      -- section
    'ANALYTICS',                        -- name
    0,                                  -- value
    'Web Analytics enabled',                -- display name
    -- description
    'A boolean value that determines if web analytics is enabled ' ||
    'for this ES.'
);
end if;

-- Added in 5.0
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'AUTO_PURGE_ENABLED' )
then 
    call SAHARA.fn_Configuration_InstallBool
    (
        'TREND_DATA_RETENTION',                 -- section
        'AUTO_PURGE_ENABLED',                   -- name
        1,		                        -- value
        'Auto Purge Enabled',                   -- display name
        -- description
        'A boolean value that determine if FACT tables purging is enabled for this ES',
        1,
        1
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'PURGE_DATA_OLDER_THAN_MONTHS' )
then 
    call SAHARA.fn_Configuration_InstallInt
    (
        'TREND_DATA_RETENTION',                 -- section
        'PURGE_DATA_OLDER_THAN_MONTHS',         -- name
        36,		                        -- value
        'Purge Data After Months',              -- display name
        -- description
        'If FACT tables purging is enabled, this value determines how many months the FACT tables ' ||
        'will be kept.',
        1,
        1
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'SUMMARIZE_DATA_AFTER_DAYS' )
then 
    call SAHARA.fn_Configuration_InstallInt
    (
        'TREND_DATA_RETENTION',                 -- section
        'SUMMARIZE_DATA_AFTER_DAYS',            -- name
        360,		                        -- value
        'Summarized Data After Days',           -- display name
        -- description
        'If FACT tables purging is enabled, this value determines when should the FACT table entries' ||
        'be summarized.',
        1,
        0
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'SUMMARIZATION_PURGING_FREQ' )
then 
    call SAHARA.fn_Configuration_InstallInt
    (
        'TREND_DATA_RETENTION',                     -- section
        'SUMMARIZATION_PURGING_FREQ',               -- name
        1,		                            -- value
        'Day Frequency of Summarization/Purge',     -- display name
        -- description
        'If FACT tables purging/Summarization is enabled, ' ||
        'this value determines how soon the next purging/summarization process should run',
        1,
        1
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'LAST_SUMMARIZATION_PURGING' )
then 
    call SAHARA.fn_Configuration_InstallDate
    (
        'TREND_DATA_RETENTION',                     -- section
        'LAST_SUMMARIZATION_PURGING',               -- name
        date(dateadd( day, -1, now(*))),            -- value
        'Date of the last Summarization/Purging',     -- display name
        -- description
        'If FACT tables purging/Summarization is enabled, ' ||
        'this value stores the last purging/summarization execution time.',
        0,
        1
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'AUTO_SUMMARIZATION_ENABLED' )
then 
    call SAHARA.fn_Configuration_InstallBool
    (
        'TREND_DATA_RETENTION',                     -- section
        'AUTO_SUMMARIZATION_ENABLED',               -- name
        1,		                            -- value
        'Auto Summarization Enabled',               -- display name
        -- description
        'A boolean value that determine if FACT tables summarization is enabled for this ES',
        1,
        0
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'KEEP_DAILY_SAMPLES_FOR_DAYS_MIN' )
then 
    call SAHARA.fn_Configuration_InstallInt
    (
        'TREND_DATA_RETENTION',                     -- section
        'KEEP_DAILY_SAMPLES_FOR_DAYS_MIN',          -- name
        180,		                            -- value
        'Keep Min days of Data for summarization',  -- display name
        -- description
        'If FACT tables Summarization is enabled, this value determines ' || 
        'the minimum days the FACT tables ' ||
        'will be kept before summarization mechanism kick in. ' || 
        'This value should always be smaller than SUMMARIZE_DATA_AFTER_DAYS.',
        1,
        0
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'PURGE_DATA_OLDER_THAN_MONTHS_MIN' )
then 
    call SAHARA.fn_Configuration_InstallInt
    (
        'TREND_DATA_RETENTION',                     -- section
        'PURGE_DATA_OLDER_THAN_MONTHS_MIN',         -- name
        36,		                            -- value
        'Purge Data after Minimum of months',       -- display name
        -- description
        'If FACT tables purging is enabled, this value determines ' || 
        'the minimum months the FACT tables ' ||
        'will be kept. This value should always be smaller than ' || 
        'PURGE_DATA_OLDER_THAN_MONTHS.',
        1,
        0
    );
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'MIGRATE_SUMMARIZED_DATA' )
then
      call SAHARA.fn_Configuration_InstallBool (
           'TREND_DATA_RETENTION',                     -- section
           'MIGRATE_SUMMARIZED_DATA',                  -- name
           0,                                          -- value
           'Indicate Simmarization process is done or not.', -- display name
           -- description
           '',
           0,
           1
       );
end if;


if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TREND_DATA_RETENTION' and name = 'LAST_SUMMARIZATION_TIME' )
then
    call SAHARA.fn_Configuration_InstallDate
    (
        'TREND_DATA_RETENTION',                     -- section
        'LAST_SUMMARIZATION_TIME',                  -- name
        date(dateadd( day, -1, now(*))),            -- value
        'Last Data Summarization processed.',       -- display name
        -- description
        '',
        0,
        1
    );
end if;

-- insert entries for proxy settings for analytics
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_SERVER' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_SERVER',                           -- name
    -- value
    '',
    'Proxy server for analytics',                           -- display name
    -- description
    'Hostname/ip address of server to be used as proxy ' ||
    ' while sending analytics data ',
	1,
    1
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_USER' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_USER',                           -- name
    -- value
    '',
    'Username for Proxy server ',                           -- display name
    -- description
    'username to be used for proxy server',
	1,
    1
);
end if;
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_PASSWD' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_PASSWD',                           -- name
    -- value
    '',
    'password for proxy server ',                           -- display name
    -- description
    'password for username to be used ' ||
    ' while sending analytics data via proxy server ',
	1,
    1
);
end if;

if not exists (select 1 from SAHARA.CONFIGURATION where section = 'SERVER' and name = 'PROXY_SERVER_PORT' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'SERVER',                               -- section
    'PROXY_SERVER_PORT',                           -- name
    -- value
    '',
    'Port for proxy server ',                           -- display name
    -- description
    'port number on proxy server which should be used' ||
    ' while sending analytics data via proxy server ',
	1,
    1
);
end if;

-- insert entries for telemetry settings 
if not exists (select 1 from SAHARA.CONFIGURATION where section = 'TELE' and name = 'VOMES_VERSION' )
then 
call SAHARA.fn_Configuration_InstallStr
(
    'TELE',                             -- section
    'VOMES_VERSION',                    -- name
    '',					-- value
    'VOMES Version for Telemetry',      -- display name
    'Stores VOMES version to identify if VOMES install/config logs needs to be collected ', -- description
    1,
    1
);
end if;


if exists (select 1 from sys.sysprocedure where proc_name = 'SP_Error_LastErrorCode') Then
    Drop Function sahara.SP_Error_LastErrorCode;
end if;

CREATE FUNCTION  "sahara"."SP_Error_LastErrorCode"  ()
returns integer
BEGIN
    return @gLastErrorCode;
END;


if exists (select 1 from sys.sysprocedure where proc_name = 'SP_Error_LastErrorMessage')Then
    Drop Function sahara.SP_Error_LastErrorMessage;
end if;

CREATE FUNCTION  "sahara"."SP_Error_LastErrorMessage"  ()
returns long varchar
BEGIN
    return @gLastErrorMessage;
END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Error_RaiseApplicationError')THEN
      DROP PROCEDURE "sahara"."SP_Error_RaiseApplicationError";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Error_RaiseApplicationError" (IN @pCode numeric(10),
                                                    IN @pMessage                                VARCHAR(255),
                                                    IN @pPushOnErrorStack  BIT default 0)
                                                   
BEGIN

declare @messg long varchar;

     --set @messg = 'none';
     set @gLastErrorCode = @pCode;
    set @gLastErrorMessage = @pMessage;
   call SP_Log_Log(null, null,  @pMessage,'E');
   -- message @pMessage to client;
-- Get the Message and add if only non null
   call sp_getmessage(@pcode ,@messg);
   
    --message @messg to client;
   if (@messg is null) then
       call sp_addmessage(@pcode ,@pMessage);
    else
      call sp_dropmessage(@pcode);
      call sp_addmessage(@pcode ,@pMessage);
    end if;
     commit;
     raiserror @pcode,@pMessage;
    --execute immediate 'raiserror ' || @pCode||','|| @pMessage||';';

END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Error_RaiseApplicationErrorDetail')THEN
      DROP PROCEDURE "SAHARA"."SP_Error_RaiseApplicationErrorDetail";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Error_RaiseApplicationErrorDetail" (IN @pCode integer,
    IN @pFunction  VARCHAR(255),
    IN @pMessage varchar(255),
    IN @pSessionKey numeric(10) default null,
    IN @pPushOnErrorStack  BIT default 0)      
BEGIN
declare @messg long varchar;

    
     set @gLastErrorCode = @pCode;
    set @gLastErrorMessage = @pMessage;
   call SP_Log_Log(@pSessionKey, @pFunction,  @pMessage,'E');
    --message @pMessage to client;
-- Get the Message and add if only non null
      call sp_getmessage(@pcode ,@messg);
   
    --message @messg to client;
   if (@messg is null) then
       call sp_addmessage(@pcode ,@pMessage);
    else
      call sp_dropmessage(@pcode);
      call sp_addmessage(@pcode ,@pMessage);
    end if;
     commit;
     raiserror @pcode,@pMessage;

    --execute immediate 'raiserror ' || @pCode||','|| @pMessage||';';

END;










message 'Creating/replacing LOAD module stored procedures...' to client;


/*
 **********************************************************************
 * SP_Load_AcquireExclusiveLock():
 * Acquires exclusive lock on the LOAD module.
 * Any rollup script must acquire this lock to ensure
 * that only one rollup is staging/loding data into sahara tables.
 * The stored proc uses underlying Lock API stored proc.
 * The stored proc does call commit inside, so there may be side effects
 * if the caller hamust be aware of this.
 * The passed in session is remembered with the lock.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_AcquireExclusiveLock')
THEN
        DROP PROCEDURE sahara.SP_Load_AcquireExclusiveLock;
END IF;

create procedure sahara.SP_Load_AcquireExclusiveLock(
     IN @SessionKey NUMERIC(10))
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SSP_Load_AcquireExclusiveLock';
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call LockSession() for the LOAD module. If this fails, it will raise
    -- an error, which will throw the exception which will return the
    -- control to the caller.
    CALL SP_Lock_LockSession('SAHARA_LOAD_CONTROL', @SessionKey);
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully acquired the lock');

    -- If we are here, then the lock was acquired, remember the session etc.
    SET @gLoadHaveControl = 1;
    SET @gLoadActiveSessionKey = @SessionKey;

END;

grant execute on sahara.SP_Load_AcquireExclusiveLock to sahara_app;
/*
 **********************************************************************
 * SP_Load_ReleaseExclusiveLock():
 * Release the exclusive lock on the LOAD module previously acquired for
 * a given session.
 * The stored proc uses underlying unlock API stored proc.
 * The stored proc does call commit inside, so there may be side effects
 * if the caller hamust be aware of this.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_ReleaseExclusiveLock')
THEN
        DROP PROCEDURE sahara.SP_Load_ReleaseExclusiveLock;
END IF;

create procedure sahara.SP_Load_ReleaseExclusiveLock(
     IN @SessionKey NUMERIC(10))
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SP_Load_ReleaseExclusiveLock';
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Clear the state.
    SET @gLoadHaveControl = 0;
    SET @gLoadActiveSessionKey = null;
    SET @gLoadActiveBatchKey = null;

    -- Call UnlockSession() for the LOAD module. If this fails, it will raise
    -- an error, which will throw the exception which will return the
    -- control to the caller.
    CALL SP_Lock_UnlockSession('SAHARA_LOAD_CONTROL', @SessionKey);

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully release the lock');
END;

grant execute on sahara.SP_Load_ReleaseExclusiveLock to sahara_app;

-------------------------------------------------------------------------------
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_GetActiveBatchKey')
THEN
        DROP FUNCTION sahara.Fn_Load_GetActiveBatchKey;
END IF;

CREATE FUNCTION sahara.Fn_Load_GetActiveBatchKey()
RETURNS NUMERIC(10)
NOT DETERMINISTIC
BEGIN
    RETURN @gLoadActiveBatchKey;
END;


-------------------------------------------------------------------------------
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_GetActiveSessionKey')
THEN
        DROP PROCEDURE sahara.Fn_Load_GetActiveSessionKey;
END IF;

CREATE FUNCTION sahara.Fn_Load_GetActiveSessionKey()
RETURNS NUMERIC(10)
NOT DETERMINISTIC
BEGIN
    RETURN @gLoadActiveSessionKey;
END;


-------------------------------------------------------------------------------
-- TODO: procedure InitializeLoad

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_InitializeLoad' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_InitializeLoad;
END IF;

CREATE PROCEDURE sahara.SP_Load_InitializeLoad
(
    IN @pBatchKey numeric(10),
    IN @pTruncateStagingTables  bit default 1
)
BEGIN
   DECLARE @FUNC varchar(64);
   DECLARE @vRollupSourceTypeKey numeric(10);
   
   set @FUNC = 'SP_Load_InitializeLoad';
   call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');
 

    -- the caller must have load control
    if (@gLoadHaveControl = 0) then
         CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ENoLoadControl,
            @FUNC,
            'caller does not have load control',
            @gLoadActiveSessionKey
        );
    end if;

    if  (@pTruncateStagingTables = 0) then
        call SP_Log_LogWarn(@gLoadActiveSessionKey, @FUNC, 'staging table truncation disabled');
    end if;

    -- validate the batch
    set @vRollupSourceTypeKey  = 0;
    
        select rb.ROLLUP_SOURCE_TYPE_KEY
          into @vRollupSourceTypeKey
          from ROLLUP_BATCH as rb
          join ROLLUP_SOURCE_TYPE as rst on rst.ROLLUP_SOURCE_TYPE_KEY = rb.ROLLUP_SOURCE_TYPE_KEY
         where rb.ROLLUP_BATCH_KEY = @pBatchKey;

       
       if (@vRollupSourceTypeKey  = 0) then
            call SAHARA.SP_Error_RaiseApplicationErrorDetail
            (
                @ERollupBatchNotFound,
                @FUNC,
                'roll-up batch ' || @pBatchKey || ' not found',
                @gLoadActiveSessionKey
            );
        end if;
 
          
    -- walk all of the staging tables for the source type and get
    -- them ready for loading
  for StagingCursor AS StagingLoop cursor for select distinct STAGE_TABLE as stgtablename
                from ROLLUP_DATA_SET_STAGE as rds
                join RSOURCE_TYPE_DATA_SET_TYPE  as rtdt on rtdt.ROLLUP_DATA_SET_TYPE_KEY = rds.ROLLUP_DATA_SET_TYPE_KEY
                join ROLLUP_SOURCE_TYPE rst on rst.ROLLUP_SOURCE_TYPE_KEY = rtdt.ROLLUP_SOURCE_TYPE_KEY
               where rst.ROLLUP_SOURCE_TYPE_KEY = @vRollupSourceTypeKey
    do
        call SP_Log_LogInfo(@gLoadActiveSessionKey, @FUNC,
                        'preparing staging table ' || stgtablename);
        call sahara_app.SP_Load_PrepareStageForLoad(stgtablename, @pTruncateStagingTables);
    end for;
     
    -- set the overall batch status to staging THIS FAILS NEED DEBUGGING
  -- call fn_Batch_Status(@gLoadActiveSessionKey, @pBatchKey,'staging', 'Staging completed', 0);
   call SP_Load_BatchStatus( @pBatchKey, 'Staging');

    -- we're ready to rock!
    set @gLoadActiveBatchKey = @pBatchKey;
    commit;
end;

grant execute on sahara.SP_Load_InitializeLoad to sahara_app;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_PrepareStageForLoad' )
THEN
        DROP PROCEDURE sahara_app.SP_Load_PrepareStageForLoad;
END IF;
create  procedure sahara_app.SP_Load_PrepareStageForLoad(in @pTableName varchar(30), in @pTruncate bit)
begin
    if (@pTruncate = 1) then
        execute immediate 'truncate table SAHARA_APP.' || @pTableName || ';';
    end if;
end;

grant execute on sahara_app.SP_Load_PrepareStageForLoad to sahara;

-------------------------------------------------------------------------------
-- TODO: procedure PerformLoad

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_PerformLoad_1' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_PerformLoad_1;
END IF;
create procedure sahara.SP_Load_PerformLoad_1()
BEGIN

DECLARE @FUNC varchar(64);
   DECLARE @vRollupSourceTypeKey numeric(10);
    set @FUNC = 'SP_Load_PerformLoad_1';  
   call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');
 

    -- the caller must have load control
    -- the caller must have load control
    if (@gLoadHaveControl = 0) then
         CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ENoLoadControl,
            @FUNC,
            'caller does not have load control',
            @gLoadActiveSessionKey
        );
    end if;

    -- make sure the load has been initialized
    if (@gloadActiveBatchKey is null) then
        call SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ENoInitializeLoad,
            @FUNC,
            'InitializeLoad must be successfully called before PerformLoad',
            @gLoadActiveSessionKey
        );
    end if;

    -- call the implementation procedure that does the real work
         
    call sahara.SP_Load_PerformLoad();
end;




IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_BatchStatus' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_BatchStatus;
END IF;

create procedure sahara.SP_Load_BatchStatus(
    IN @pBatchKey numeric(10),
    IN @pStatus varchar(32),
    IN  @pMessage long varchar default null,
IN @pOverwriteComplete bit default 0
)
--as
  --  pragma autonomous_transaction;

    
begin
    DECLARE @FUNC varchar(32);
    set @FUNC = 'SP_Load_BatchStatus';
    -- TBD
    
   call fn_Batch_Status(@gLoadActiveSessionKey, @pBatchKey,@pStatus, @pMessage, @pOverwriteComplete);
    commit;
end;

-------------------------------------------------------------------------------
--
-- private: appends a message to the roll-up batch status
--          message autonomously
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_BatchStatusMessage' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_BatchStatusMessage;
END IF;

create procedure sahara.SP_Load_BatchStatusMessage(in @pMessage long varchar)
--as
  --  pragma autonomous_transaction;

    --FUNC constant API.FunctionName := PKGPFX || 'BatchStatusMessage';
begin

    call fn_Batch_StatusMessage(@gLoadActiveSessionKey, @ActiveBatchKey_, @pMessage);
    commit;
end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_Load' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_Load;
END IF;
create procedure sahara.SP_Load_Load()
BEGIN
  message ' Finally into the load' to client;
END;

-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--
-- Loads a single data set of the specified type from the source table or
-- view into to the destination (warehouse) table.--

-------------------------------------------------------------------------------
--
-- private: Calls the load procedure that does the real work and wrap it
--          with error handling.
--

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_LoadBatch' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_LoadBatch;
END IF;
create procedure sahara.SP_Load_LoadBatch()
begin
    DECLARE @FUNC varchar(32);
    DECLARE @vLoadProcedure varchar(61);
    DECLARE @vStatus varchar(32);
    DECLARE @vMessage  long varchar;
    DECLARE @rslt bit;
    
    set @FUNC= 'SP_Load_LoadBatch';

    
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

    -- set the overall batch status to loading
    call SP_Load_BatchStatus(@ActiveBatchKey_, 'Loading');
    commit;

    -- purge any object dimension references for roll-up sources that
    -- have been deleted
    call SP_Log_LogInfo(@ActiveSessionKey_, @FUNC,
                    'purging object dimension source references for deleted sources');
    

    delete sahara.OBJECT_DIM_SOURCE
     where ROLLUP_SOURCE_KEY in (select ROLLUP_SOURCE_KEY
                                   from sahara.OBJECT_DIM_SOURCE
                                  except
                                 select ROLLUP_SOURCE_KEY
                                   from sahara.ROLLUP_SOURCE);
    commit;

    -- get the name of the load procedure
    select ROLLUP_SOURCE_TYPE.LOAD_PROCEDURE
      into @vLoadProcedure
      from ROLLUP_BATCH
      join ROLLUP_SOURCE_TYPE
        on ROLLUP_BATCH.ROLLUP_SOURCE_TYPE_KEY =
           ROLLUP_SOURCE_TYPE.ROLLUP_SOURCE_TYPE_KEY
     where ROLLUP_BATCH.ROLLUP_BATCH_KEY = @ActiveBatchKey_;

    -- call it
    call SP_Log_LogInfo(@ActiveSessionKey_, @FUNC,
                    'calling load procedure ' || @vLoadProcedure || ' for batch ' ||
                    @ActiveBatchKey_);
    --message @vLoadProcedure to client;
    execute immediate 'begin  call ' || @vLoadProcedure || '(); end;';
    
    call SP_Log_LogInfo(@ActiveSessionKey_, @FUNC,
                    'load procedure ' || @vLoadProcedure || ' for batch ' ||
                    @ActiveBatchKey_ || ' completed without exception');

    -- allowing partial loads is configurable
    if (@BatchHasPartialLoads_ = 1) then
            set @rslt = sahara.fn_GetConfigBool('ROLL-UP', 'ALLOW_PARTIAL_LOADS');
        if ( @rslt = 0) then
            set @vMessage =
                'The roll-up batch has partially loaded data sets.  ' ||
                'The current configuration does not allow batches ' ||
                'with partial data to be retained.';
            call sahara.SP_Load_BatchStatusMessage(@vMessage);

            set @UseExceptionMessage_ = 0;
            call SAHARA.SP_Error_RaiseApplicationErrorDetail
            (
                @EPartialLoad,
                @FUNC,
                @vMessage,
                @ActiveSessionKey_
            );
        
        else 
            set @BatchHasWarnings_ = 1;
            set @vMessage =
                'The roll-up batch has partially loaded data sets.  ' ||
                'The current configuration allows batches with partial ' ||
                'data to be retained.  This may afect the accuracy of ' ||
                'reported data.';
            call sahara.SP_Load_BatchStatusMessage(@vMessage);
        end if;
    end if;
    message 'completed  load loop' to client;
    -- purge facts from previous batches for the same source and date 
    -- TOBE PORTED BY DEV
    call sp_Load_PurgeFactsByDatesKey();
	
	-- load ETL logs extracted during VOM MS extraction into MESSAGE_LOG table
    call sp_Load_ResilientCheck();
	
	-- if we got here without an exception, it looks like success - check
    -- for warnings
    if (@BatchHasWarnings_  = 1 )then
        set @vStatus = 'Succeeded with Warnings';
    else
        set @vStatus = 'Succeeded';
    end if;
    call SP_Load_BatchStatus(@ActiveBatchKey_, @vStatus);
    commit;

exception
    when others then
        set @vMessage = ERRORMSG( SQLCODE );;
        rollback; 

       CALL SP_Log_LogError(@ActiveSessionKey_, @FUNC,
                         'Load failed for batch ' || @ActiveBatchKey_ ||
                         '.  Rolling back updates.  Message: ' || @vMessage);

        -- record the status
        if (@UseExceptionMessage_ = 1) then
            call SP_Load_BatchStatus(@ActiveBatchKey_, 'Failed', @vMessage);
        else
            call SP_Load_BatchStatus(@ActiveBatchKey_, 'Failed');
        end if;

 
        -- rollback the load
       

        -- re-raise the error
        CALL SP_Error_RaiseApplicationError(20050,'raise');
end;

-------------------------------------------------------------------------------
--
-- private: initializes package variables in preparation for loading a batch
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_InitBatch' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_InitBatch;
END IF;
create procedure sahara.SP_Load_InitBatch()
begin
   DECLARE @func VARCHAR(32);
   set @func =  'SP_Load_InitBatch';

   call SP_Log_LogDebug(@gLoadActiveSessionKey, @func, 'entry');
   if  ( VAREXISTS('@ActiveBatchKey_') = 0 ) then
      create variable @ActiveBatchKey_ numeric(10);
   end if;
      if  ( VAREXISTS('@ActiveDataSetType_') = 0 ) then
        create variable @ActiveDataSetType_ varchar(64);
    end if;
   if  ( VAREXISTS('@ActiveSessionKey_') = 0 ) then
        create variable @ActiveSessionKey_ numeric(10);
   end if;
   if  ( VAREXISTS('@BatchHasWarnings_') = 0 ) then
    create variable @BatchHasWarnings_ bit;
   end if;
   if  ( VAREXISTS('@BatchHasPartialLoads_') = 0 ) then
        create variable @BatchHasPartialLoads_ bit;
   end if;
   if  ( VAREXISTS('@UseExceptionMessage_') = 0 ) then
    create variable @UseExceptionMessage_  bit;
   end if;
   if  ( VAREXISTS('@MaxKeyBeforeInsert') = 0 ) then
        create VARIABLE @MaxKeyBeforeInsert integer;
   end if;
    
   set @ActiveBatchKey_ = @gLoadActiveBatchKey;
   set @ActiveDataSetType_ = null;
   set @ActiveSessionKey_ =@gLoadActiveSessionKey; 
   set @BatchHasWarnings_  = 0;
   set @BatchHasPartialLoads_ = 0;
   set @UseExceptionMessage_ = 1;

   call SP_Load_CreateTempTable();

end;

-------------------------------------------------------------------------------
--
-- private: frees resources after completion of loading a batch
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_FiniBatch' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_FiniBatch;
END IF;
create procedure sahara.SP_Load_FiniBatch()
BEGIN
    DECLARE @func VARCHAR(32);
    set @FUNC =  'SP_Load_FiniBatch';

   call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');


if  ( VAREXISTS('@ActiveBatchKey_') = 1 ) then
      drop variable @ActiveBatchKey_ ;
   end if;
      if  ( VAREXISTS('@ActiveDataSetType_') = 1 ) then
        drop variable @ActiveDataSetType_ ;
    end if;
   if  ( VAREXISTS('@ActiveSessionKey_') = 1 ) then
        drop variable @ActiveSessionKey_ ;
   end if;
   if  ( VAREXISTS('@BatchHasWarnings_') = 1 ) then
    drop variable @BatchHasWarnings_ ;
   end if;
   if  ( VAREXISTS('@BatchHasPartialLoads_') = 1 ) then
        drop variable @BatchHasPartialLoads_ ;
   end if;
   if  ( VAREXISTS('@UseExceptionMessage_') = 1 ) then
    drop variable @UseExceptionMessage_  ;
   end if;

if  ( VAREXISTS('@MaxKeyBeforeInsert') = 0 ) then
        drop variable @MaxKeyBeforeInsert ;
   end if;
   
    
end;

-------------------------------------------------------------------------------
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_PerformLoad' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_PerformLoad;
END IF;
create procedure sahara.SP_Load_PerformLoad()
BEGIN
DECLARE @FUNC varchar(64);
   

    set @FUNC =  'SP_Load_PerformLoad';

   call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

if (@gLoadHaveControl = 0) then
         CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ENoLoadControl,
            @FUNC,
            'caller does not have load control',
            @gLoadActiveSessionKey
        );
    end if;

    -- make sure the load has been initialized
    if (@gloadActiveBatchKey is null) then
        call SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ENoInitializeLoad,
            @FUNC,
            'InitializeLoad must be successfully called before PerformLoad',
            @gLoadActiveSessionKey
        );
    end if;


    call SP_Log_LogInfo(@gLoadActiveSessionKey, @FUNC,
                    'starting load for batch ' || @gLoadActiveBatchKey);

    -- initialize things before the load
    call SP_Load_InitBatch();
    

    -- make sure we clean-up regardless how things end
    begin
        -- load the batch's data and perform data related error handling
    
       call  SP_Load_LoadBatch();
    
        -- clean-up after the load
        call SP_Load_FiniBatch();
    exception
      when others then
            -- clean-up after the load
           call SP_Load_FiniBatch();
            --raise;
    end;
end;
grant execute on sahara.SP_Load_PerformLoad to sahara_app;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_LoadDataSet' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_LoadDataSet;
END IF;
create procedure sahara.SP_Load_LoadDataSet(in @pDataSetType varchar(64))
begin
    declare @FUNC varchar(32);
    declare @vMessage long varchar;
    declare @stereotype varchar(32);
    set @FUNC = 'SP_Load_LoadDataSet';
    
               call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');
    

    -- tbd - validate the batch key
    -- tbd - validate that the batch's type supports the data set type

    -- get the metadata for the data set type
    begin
        select stereotype
          into @stereotype
          from sahara.ROLLUP_DATA_SET_TYPE
         where NAME = @pDataSetType;

    if @stereotype is null  then
        call SAHARA.SP_Error_RaiseApplicationErrorDetail
            (
                @ERollupDataSetTypeNotFound,
                @FUNC,
                'invalid roll-up data set type requested for loading: ' ||
                @pDataSetType,
                @ActiveSessionKey_
            );        
    end if;

  end;
    set @ActiveDataSetType_ = @pDataSetType;

    --select md.stereotype into @stereotype from #vMetadata as md;
    -- set the status to loading (autonomously committed)
    call SP_Load_DataSetStatus('Loading');

    
    begin
        -- each stereotype has its own rules
        case @stereotype
            when 'DIMENSION' then
                call SP_Load_LoadDimension(@pDataSetType);

            when 'FACT' then
                call SP_Load_LoadFact(@pDataSetType);

            when 'CUSTOM' then
                call SP_Load_LoadCustom(@pDataSetType);

            else
                call SAHARA.SP_Error_RaiseApplicationErrorDetail
                (
                    @ERUDSTypeStereotype,
                    @FUNC,
                    'invalid or unknown roll-up data set - stereotype: ' ||
                    @stereotype || ', data set type: ' || @ActiveDataSetType_,
                    @ActiveSessionKey_
                );
        end case;

        -- if we got here without an exception, it looks like success!
        -- (autonomously committed)
        call SP_Load_DataSetStatus('Succeeded');

    exception
        -- oh no!
        when others then
            -- save off the error message so we can report it
            set @vMessage = ERRORMSG( SQLCODE );;
             call sahara.SP_Log_LogError(@ActiveSessionKey_, @FUNC, @vMessage);

            -- tbd - message from exception?
            -- (autonomously committed)
            call sahara.SP_Load_DataSetStatus('Failed', @vMessage);

            -- we'll raise a different error exception to the overall load
            call SAHARA.SP_Error_RaiseApplicationErrorDetail
            (
                @EDataSetLoadFailed,
                @FUNC,
                'failed loading roll-up data set - stereotype: ' ||
                @stereotype || ', data set type: ' || @ActiveDataSetType_ ||
                ', error: ' || @vMessage,
                @ActiveSessionKey_
            );
    end;
end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_DataSetStatus' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_DataSetStatus;
END IF;

create procedure sahara.SP_Load_DataSetStatus
(
    IN @pStatus varchar(32),
    IN  @pStatusMessage long varchar default null
)
--as
  --  pragma autonomous_transaction;
begin
     call fn_Batch_DataSetStatus(@ActiveSessionKey_, @ActiveBatchKey_,@ActiveDataSetType_, @pStatus, @pStatusMessage);
    
    commit;
end;



IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_LoadDimension' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_LoadDimension;
END IF;
create procedure sahara.SP_Load_LoadDimension(in @pDataSetType varchar(64))
begin
declare @FUNC varchar(32);
declare @vSrcTable varchar(32);
declare @vDstTable varchar(32);
declare @vCurrentView varchar(32);
declare @vSurrogateKey varchar(32);
declare @vOID varchar(30);
declare @vSrcRowCount integer;
declare @vRowCount integer;
declare @vMessage long varchar;
declare @vLoadSQL long varchar;
    
    
    set @FUNC = 'SP_Load_LoadDimension';
    
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

truncate table SrcColumnRecord;
truncate table vDimColumns;
truncate table vType1Columns;
truncate table vType2Columns;
truncate table vKeyColumns;


      select rds.DEST_TABLE,rds.DEST_TABLE||'_',rds.DEST_TABLE|| '_CURRENT',rds.DEST_TABLE||'_KEY ',rds.LOAD_SQL
          into @vDstTable,@vSrcTable,@vCurrentView,@vSurrogateKey,@vLoadSQL
          from sahara.ROLLUP_DATA_SET_TYPE as rds
         where NAME = @pDataSetType;

        set @ActiveDataSetType_ = @pDataSetType;


    --vSrcTable constant API.CSQLIdentifier := vMetadata.DEST_TABLE || '_';
    --vCurrentView constant API.CSQLIdentifier := vMetadata.DEST_TABLE || '_CURRENT';
    --vSurrogateKey constant API.CSQLIdentifier := vMetadata.DEST_TABLE || '_KEY';
    --vSurrogateKeySequence constant API.CSQLIdentifier := 'S_' || vSurrogateKey;

    --vSrcRowCount pls_integer;
    --vSrcColumns SrcColumnTable;
    --vDimColumns ColumnTable := ColumnTable();
    --vKeyColumns ColumnTable := ColumnTable();
    --vType1Columns ColumnTable := ColumnTable();
    --vType2Columns ColumnTable := ColumnTable();

    --vLoadSQL varchar2(32767) := trim(vMetadata.LOAD_SQL);
    --vRowCount pls_integer;
    --vMessage clob;

    -- makes the column list for the source (select) portion of the merge
    -- statement that is based on staging table columns merged with
    -- any configured source expressions


    
--begin
    

    -- get the staging table count for logging, auditing, and error checking
    execute immediate        'select count(*) into @vSrcRowCount from SAHARA_APP.' || @vSrcTable || ';';
         
    call SP_Log_LogInfo(@ActiveSessionKey_, @FUNC, 'starting dimension load: ' ||
                    @vSrcTable || ' (' || @vSrcRowCount || ' rows) -> ' ||
                    @vDstTable);

    set @vLoadSQL = TRIM(@vLoadSQL);

    -- get all of the source columns
    --with srcExprColumns as
        -- a factored sub-query for columns with source expressions
        select NAME, SOURCE_EXPRESSION
            into #srcExprColumns
           from ROLLUP_DATA_SET_COLUMN
          where ROLLUP_DATA_SET_TYPE_KEY = (select ROLLUP_DATA_SET_TYPE_KEY
                                              from ROLLUP_DATA_SET_TYPE
                                             where NAME = @ActiveDataSetType_
                                                )
            and SOURCE_EXPRESSION is not null
          order by NAME ;
     
    -- get all the staging table columns exception those spelled out
    -- as having source expressions
        select col.cname as name, null as expression
       into #vSrcColumns
      from SYS.syscolumns as col
     where col.creator = 'SAHARA_APP'
       and col.tname = @vSrcTable
       and col.cname not in (select NAME
                                 from #srcExprColumns)
     union all
    -- add in the columns that have source expressions
    select NAME as  name, SOURCE_EXPRESSION as expression
      from #srcExprColumns;

    -- THE ASSERT IS NOT REQUIRED
 --   $if $$SAHARA_ASSERT $then
   --     TEST_API.Assert(vSrcColumns is not null and vSrcColumns.count > 0,
     --                   FUNC || ': source columns should be populated');
    --$end

    -- categorize the dimension columns from the data dictionary to
    -- make building the statement easy

    for LoadCursor as LoadLoop cursor for select NAME as colname, LOAD_PROCESSING as ldprocessing
                from ROLLUP_DATA_SET_COLUMN
               where ROLLUP_DATA_SET_TYPE_KEY = (select ROLLUP_DATA_SET_TYPE_KEY
                                                   from ROLLUP_DATA_SET_TYPE
                                                  where NAME = @ActiveDataSetType_)
                 and LOAD_PROCESSING <> 'NONE'
               order by NAME
    do
        insert into vDimColumns(Name) Values(colname);
        --vDimColumns.extend;
        --vDimColumns(vDimColumns.last) := c.NAME;

        case  ldprocessing
            when 'STAGING KEY' then
                  insert into vKeyColumns(Name) Values(colname);
          --      vKeyColumns.extend;
            --    vKeyColumns(vKeyColumns.last) := c.NAME;

            when 'TYPE 1' then
                 insert into vType1Columns(Name) Values(colname);
                --vType1Columns.extend;
                --vType1Columns(vType1Columns.last) := c.NAME;

            when 'TYPE 2' then
                insert into vType2Columns(Name) Values(colname);
                --vType2Columns.extend;
                --vType2Columns(vType2Columns.last) := c.NAME;

            else
                 call SAHARA.SP_Error_RaiseApplicationErrorDetail
            (
             
                    @ERUDSCLoadProcessing,
                    @FUNC,
                    'invalid or unknown roll-up data set column ' ||
                    'load processing type (' || ldprocessing||
                    ') for dimension',
                    @ActiveSessionKey_
                );
        end case;
    end for;

    -- tbd - vSrcColumns must not be empty - validate
    -- tbd - vDimColumns must not be empty - validate
    -- tbd - vKeyColumns must not be empty - validate
    message @vLoadSQL to client;
    call SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL0: ' || @vLoadSQL);
    set @vLoadSQL = replace(@vLoadSQL, '$SRC_COLUMNS$', fn_Load_MakeSrcColumns(@pDataSetType));
    set @vLoadSQL = replace(@vLoadSQL, '$SRC_TABLE$', @vSrcTable);
    set @vLoadSQL = replace(@vLoadSQL, '$DEST_TABLE$', @vDstTable);
    message @vLoadSQL to client;
    call SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL1: ' || @vLoadSQL);
    -- create the merge statement
    if (@vLoadSQL is null) then
       -- Get only the staging table columns.

	 select col.cname as name
       into #vSrcStagingColumns
      from SYS.syscolumns as col
     where col.creator = 'SAHARA_APP'
       and col.tname = @vSrcTable;
     select distinct name into @vOID from vKeyColumns;
          set @vLoadSQL = 'select '||Fn_Load_MakeListStaging('SRC.') || ',(@URL1|| @vOID||@URL2) as "CC_STORAGE_URL" from sahara_app.' || @vSrcTable || ' as SRC'
    end if;

   set @vLoadSQL =
        'begin ' ||
        'merge into "' || @vDstTable || '" DEST ' ||
        'using (' || @vLoadSQL || ') SRC ' ||
        'on (' || fn_Load_MakeOnKey() || ' and  DEST.T2_IS_CURRENT = ''T''   and T2_IS_DELETED = ''F'''  ;
    if exists(select 1 from vType2Columns) then    
        set @vLoadSQL = @vLoadSQL || ' and ' || fn_Load_MakeOnType2();
    end if;

    set @vLoadSQL = @vLoadSQL || ')';

    message @vLoadSQL to client;
    call SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL2: ' || @vLoadSQL);
    
    if exists(select 1 from vType1Columns) then    
        set @vLoadSQL = @vLoadSQL ||
           ' when matched then ' ||
            'update set ' || fn_Load_MakeSet();
    end if;
    message @vLoadSQL to client;
    call SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL: ' || @vLoadSQL);
    set @vLoadSQL = @vLoadSQL ||
       ' when not matched then ' ||
        --'insert (' || vSurrogateKey || ', ' || fn_Load_MakeListDim( 'DEST.') || ') ' ||
        --'values (' || vSurrogateKeySequence || '.nextval, ' || MakeList(vDimColumns, 'SRC.') || '); ' ||
         'insert (' || fn_Load_MakeListDim( 'DEST.') || ') ' ||
        'values ('  || fn_Load_MakeListDim('SRC.') || '); ' ||
        --'LOAD_IMPL.RowCount := sql%rowcount; ' ||
        'end;';
   --message 'before execute immediate' to client;
    -- execute it
    call SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL: ' || @vLoadSQL);
    call SP_Load_BeforeDimensionInsert(@vDstTable); -- This is only for insert, although we don't know if its an insert or update.
    execute immediate @vLoadSQL;
    --message 'after execute immediate' to client;
    set @vRowCount = @@rowcount;
	call sahara.SP_Load_DataSetRowCount(@vRowCount);
    
    -- make it obvious if we seem to be losing rows
    if @vRowCount <> @vSrcRowCount then
        set @BatchHasPartialLoads_ = 1;
        set @vMessage = 'For dimension data set ' || @ActiveDataSetType_ ||
                    ', the number of source rows in ' || @vSrcTable ||
                    ' (' || @vSrcRowCount || ') ' ||
                    'differs from the number of rows inserted/updated in ' ||
                    @vDstTable || ' (' || @vRowCount || ').';
        call SP_Load_BatchStatusMessage(@vMessage);
        call SP_Log_LogWarn(@ActiveSessionKey_, @FUNC, @vMessage);
    end if;
    --message 'Before the adjustobjectdimensionsource' to client;
    -- perform source reference counting
    call sahara.SP_Load_AdjustObjectDimensionSource(@vDstTable);

    --message 'After the adjustobjectdimensionsource' to client;
    call SP_Log_LogInfo(@ActiveSessionKey_, @FUNC, 'completed dimension load: ' ||
                    @vSrcTable || ' -> ' || @vDstTable ||
                    ': ' || @vRowCount || ' rows inserted/updated');

exception
  when others then
    set @vMessage = ERRORMSG( SQLCODE );
        --message ERRORMSG( SQLCODE ) to client;

     call SAHARA.SP_Error_RaiseApplicationErrorDetail
           (
             @ERUDSCLoadProcessing,
           @FUNC,
         'Loading of dimension failed for  DataSetType: ' ||@pDataSetType|| 'SQL Error:'||@vMessage,
               @ActiveSessionKey_
            );

end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_BeforeDimensionInsert' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_BeforeDimensionInsert;
END IF;

create procedure sahara.SP_Load_BeforeDimensionInsert(in @pTable varchar(32))
BEGIN
declare @FUNC varchar(64);
declare @vSQL long varchar;

    set @FUNC = 'SP_Load_BeforeDimensionInsert';

    -- tbd - Right now we waste a sequence value and get the
    --       max value from the sequence to avoid mutation errors.
    --       This should be fixed.
    set @vSQL = 'select MAX($TABLE$_KEY) into @MaxKeyBeforeInsert from ' || @pTable||';';
   

  set @vSQL = replace(@vSQL, '$TABLE$', @pTable);
--begin
 -- message @vSQL to client;   
 call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL ;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC,
                     'max key before insert: '|| @MaxKeyBeforeInsert);
   -- message @vSQL to client;
    --message @MaxKeyBeforeInsert to client;
end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_AfterDimensionInsert' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_AfterDimensionInsert;
END IF;

create procedure sahara.SP_Load_AfterDimensionInsert(in @pTable varchar(32))
BEGIN
declare @vSQL long varchar;
declare @vAlias varchar(12);
declare @vRows integer;
declare @FUNC varchar(64);
declare @COUNT_INSERTED long varchar;
declare @COUNT_POSSIBLE_ANCESTORS long varchar;
declare @FIND_ANCESTORS long varchar;
declare @UPDATE_ANCESTORS long varchar;
declare @INSERT_BVIEW_LINKS long varchar;
declare @vMessage long varchar;

    set @FUNC = 'sahara.SP_Load_AfterDimensionInsert';
    --message 'Into the function' to client;

    set @COUNT_INSERTED =
        'select count(*) into @vRows' ||
          ' from "$TABLE$" ' ||
         'where $TABLE$_KEY > @MaxKeyBeforeInsert';

    set @COUNT_POSSIBLE_ANCESTORS =
        'select count(*) into @vRows' ||
          ' from "$TABLE$" ' ||
         'where T2_IS_CURRENT = ''T'' ' ||
           'and $TABLE$_KEY <= @MaxKeyBeforeInsert';

    set @FIND_ANCESTORS =
        'insert into T2_INSERT ' ||
            '(INSERTED_KEY, INSERTED_OID, ' ||
             'ANCESTOR_ROW, ANCESTOR_KEY) ' ||
        'select ' ||
             'INSERTED.$TABLE$_KEY, INSERTED.$TABLE$_OID, ' ||
             'ANCESTOR.rowid, ANCESTOR.$TABLE$_KEY ' ||
          'from (select $TABLE$_KEY, $TABLE$_OID ' ||
                  'from "$TABLE$" ' ||
                 'where $TABLE$_KEY > @MaxKeyBeforeInsert) INSERTED ' ||
          'join (select rowid($TABLE$) as rowid, $TABLE$_KEY, $TABLE$_OID ' ||
                  'from "$TABLE$" ' ||
                 'where T2_IS_CURRENT = ''T'' ' ||
                   'and $TABLE$_KEY <= @MaxKeyBeforeInsert) ANCESTOR ' ||
            'on INSERTED.$TABLE$_OID = ANCESTOR.$TABLE$_OID';

    set @UPDATE_ANCESTORS =
        'update "$TABLE$" ' ||
           'set T2_IS_CURRENT = ''F'' ' ||
         'where rowid($TABLE$) in (select ANCESTOR_ROW ' ||
                           'from T2_INSERT)';

    set @INSERT_BVIEW_LINKS =
        'insert into $ALIAS$_BVIEW_NODE ' ||
            '(BVIEW_NODE_KEY, $TABLE$_KEY, BVIEW_KEY) ' ||
        'select ' ||
            'BVN.BVIEW_NODE_KEY, T2_INSERT.INSERTED_KEY, BVN.BVIEW_KEY ' ||
          'from $ALIAS$_BVIEW_NODE BVN ' ||
          'join T2_INSERT ' ||
            'on BVN.$TABLE$_KEY = T2_INSERT.ANCESTOR_KEY';

    

    -- see if it is possible to have ancestors
    -- if there are no ancestors, there's nothing more to do
    set @vSQL = replace(@COUNT_POSSIBLE_ANCESTORS, '$TABLE$', @pTable);
    message @vSQL to client;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL ;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'possible ancestors: '|| @vRows);
    if @vRows = 0 then
        return;
    end if;

    -- see if any rows were actually inserted
    -- if nothing new was inserted, there's nothing more to do
    set @vSQL = replace(@COUNT_INSERTED, '$TABLE$', @pTable);
    message @vSQL to client;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL ;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'rows inserted: '|| @vRows);
    if @vRows = 0 then
        return;
    end if;

    -- populate a temporary table with the inserted row-to-ancestor
    -- relationship for any inserted rows that are type 2 inserts
    delete from T2_INSERT;
    set @vSQL = replace(@FIND_ANCESTORS, '$TABLE$', @pTable);
    message @vSQL to client;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL;

   --message 'Before update' to client;
    -- mark the ancestors as no longer current
    set @vSQL = replace(@UPDATE_ANCESTORS, '$TABLE$', @pTable);
    message @vSQL to client;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL;

    -- get the dimension alias used to form the business view
    -- node table name
    
        --set @vAlias = null;
      --   select ALIAS into @vAlias
        --  from BVIEW_DIMENSION
         --where DIMENSION = @pTable ;
   
        set @vAlias = null;
         select ALIAS into @vAlias
        from BVIEW_DIMENSION
      where DIMENSION = @pTable ;
   
        if (@vAlias is null) then  
            call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC,'no dimension alias found');
        end if;
    

    -- insert business view node assignments for any inserts whose ancestors
    -- had node assignments
    -- To be done.
    -- insert business view node assignments for any inserts whose ancestors
    -- had node assignments
    -- To be done.
    if @vAlias is not null and fn_TableExists(@vAlias || '_BVIEW_NODE') = 1 then
        set @vSQL = replace(@INSERT_BVIEW_LINKS, '$TABLE$', @pTable);
        set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
        call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
        execute immediate @vSQL;
    end if;
    --if vAlias is not null and TableExists(vAlias || '_BVIEW_NODE') then
      --  vSQL := replace(INSERT_BVIEW_LINKS, '$TABLE$', pTable);
        --vSQL := replace(vSQL, '$ALIAS$', vAlias);
        --LOG_API.LogDebug(LOAD_API.ActiveSessionKey, FUNC, 'SQL: ' || vSQL);
       -- execute immediate vSQL;
    --end if;

    -- free the temporary
    delete from T2_INSERT;

exception
    when others then
        --set @vMessage = ERRORMSG( SQLCODE );
        message ERRORMSG( SQLCODE ) to client;
        --CALL SP_Log_LogError(@gLoadActiveSessionKey, @FUNC,
          --               'Error in after insert trigger  Message: ' || @vMessage);

end;




create or replace trigger SAHARA.T_ARRAY_3AI
    after insert
    on SAHARA.ARRAY
begin
    call SP_Load_AfterDimensionInsert('ARRAY');
end;

create or replace trigger SAHARA.T_VOLUME_3AI
    after insert
    on SAHARA.VOLUME
begin
    call SP_Load_AfterDimensionInsert('VOLUME');
end;


create or replace trigger SAHARA.T_APP_GROUP_3AI
    after insert
    on SAHARA.APP_GROUP
begin
    call SP_Load_AfterDimensionInsert('APP_GROUP');
end;


create or replace trigger SAHARA.T_DATABASES_3AI
    after insert
    on SAHARA."DATABASES"
begin
    call SP_Load_AfterDimensionInsert('DATABASES');
end;


create or replace trigger SAHARA.T_FILESYSTEM_3AI
    after insert
    on SAHARA.FILESYSTEM
begin
    call SP_Load_AfterDimensionInsert('FILESYSTEM');
end;
create or replace trigger SAHARA.T_FILE_CATEGORY_3AI
    after insert
    on SAHARA.FILE_CATEGORY
begin
    call SP_Load_AfterDimensionInsert('FILE_CATEGORY');
end;
create or replace trigger SAHARA.T_HOST_3AI
    after insert
    on SAHARA.HOST
begin
    call SP_Load_AfterDimensionInsert('HOST');
end;
create or replace trigger SAHARA.T_LUN_3AI
    after insert
    on SAHARA.LUN
begin
    call SP_Load_AfterDimensionInsert('LUN');
end;

create or replace trigger SAHARA.T_DISKGROUP_3AI
    after insert
    on SAHARA.DISKGROUP
begin
    call SP_Load_AfterDimensionInsert('DISKGROUP');
end;


create or replace trigger SAHARA.T_SF_LICENSE_3AI
    after insert
    on SAHARA.SF_LICENSE
begin
    call SP_Load_AfterDimensionInsert('SF_LICENSE');
end;


create or replace trigger SAHARA.T_SF_LICENSE_DEPLOYMENT_3AI
    after insert
    on SAHARA.SF_LICENSE_DEPLOYMENT
begin
    call SP_Load_AfterDimensionInsert('SF_LICENSE_DEPLOYMENT');
end;

create or replace trigger SAHARA.T_SF_LICENSE_HOST_3AI
    after insert
    on SAHARA.SF_LICENSE_HOST
begin
    call SP_Load_AfterDimensionInsert('SF_LICENSE_HOST');
end;


create or replace trigger SAHARA.T_SF_PRODUCT_3AI
    after insert
    on SAHARA.SF_PRODUCT
begin
    call SP_Load_AfterDimensionInsert('SF_PRODUCT');
end;


create or replace trigger SAHARA.T_CLUSTER_3AI
    after insert
    on SAHARA.CLUSTER
begin
    call SP_Load_AfterDimensionInsert('CLUSTER');
end;

create or replace trigger SAHARA.T_SERVICEGROUP_3AI
    after insert
    on SAHARA.SERVICEGROUP
begin
    call SP_Load_AfterDimensionInsert('SERVICEGROUP');
end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_AdjustObjectDimensionSource' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_AdjustObjectDimensionSource;
END IF;


create procedure sahara.SP_Load_AdjustObjectDimensionSource(in @pTableName varchar(32))
begin
declare @FUNC varchar(64);
declare @vSourceKey numeric(10);
declare @vRowCount integer;
declare @DeleteMissingSQL long varchar;
declare @InsertNewSQL long varchar;
declare @vSQL long varchar;

    set @FUNC =  'SP_Load_AdjustDimensionReferences';
   call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

    message 'Into the Load AdjustObjectDimension' to client;
    
    


    select ROLLUP_SOURCE_KEY
        into @vSourceKey    
      from ROLLUP_BATCH
     where ROLLUP_BATCH_KEY = @gLoadActiveBatchKey;

    --message ' before  the select' to client;
    -- delete any references for objects that have gone missing
   -- Also delete the entires from object dim source delete table.
    delete from T2_OBJECT_DIM_SOURCE_DELETE;
    set  @DeleteMissingSQL = 
        'delete from OBJECT_DIM_SOURCE ' ||
         'where not exists (select 1 ' ||
                             'from SAHARA_APP.$TABLE$_ ' ||
                            'where $TABLE$_.$TABLE$_OID = OBJECT_DIM_SOURCE.OID) ' ||
           'and TABLE_NAME = ''' || @pTableName  ||
           ''' and  ROLLUP_SOURCE_KEY = ' || @vSourceKey||';';

    --message ' before ' ||@DeleteMissingSQL  to client;
    set @vSQL = replace(@DeleteMissingSQL, '$TABLE$', @pTableName);
    message @vSQL to client; 
   call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);
 
    execute immediate @vSQL;

    set @vRowCount = @@rowcount;;
    if (@vRowCount > 0)  then
        call SP_Log_LogInfo(@gLoadActiveSessionKey, @FUNC,
                        'object dimension source references dropped for ' ||
                        @pTableName || ': ' || @vRowCount);
    end if;

    set @InsertNewSQL =
        'insert into OBJECT_DIM_SOURCE(OID, TABLE_NAME, ROLLUP_SOURCE_KEY) ' ||
        'select $TABLE$_OID, '''|| @pTableName ||''', ' || @vSourceKey  ||
          ' from SAHARA_APP.$TABLE$_ ' ||
         'where not exists (select 1 ' ||
                             'from OBJECT_DIM_SOURCE ' ||
                            'where OBJECT_DIM_SOURCE.OID = $TABLE$_.$TABLE$_OID ' ||
                              'and TABLE_NAME = '''||@pTableName  ||
                              '''and ROLLUP_SOURCE_KEY = '|| @vSourceKey||' )';
      --message ' before:'||@InsertNewSQL to client;
    -- add references for any objects for which we did not have one
   set  @vSQL = replace(@InsertNewSQL , '$TABLE$', @pTableName);
   message @vSQL to client;
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vSQL);

    execute immediate @vSQL ;;

     set @vRowCount = @@rowcount;;
    if (@vRowCount > 0)  then
        call SP_Log_LogInfo(@gLoadActiveSessionKey, @FUNC,
                        'object dimension source references added for ' ||
                        @pTableName || ': ' || @vRowCount);
    end if;
end; 

commit;

IF  EXISTS ( SELECT 1 FROM SYS.SYSTRIGGER WHERE TRIGGER_NAME = 'T_OBJECT_DIM_SOURCE_2ADFER' ) THEN
  DROP TRIGGER T_OBJECT_DIM_SOURCE_2ADFER;
END IF;

CREATE TRIGGER "T_OBJECT_DIM_SOURCE_2ADFER" AFTER DELETE 
ORDER 1 ON SAHARA.OBJECT_DIM_SOURCE 
REFERENCING OLD AS old_row
FOR EACH ROW
BEGIN
    insert into T2_OBJECT_DIM_SOURCE_DELETE(OID, TABLE_NAME)
    on existing skip values(old_row.OID, old_row.TABLE_NAME);

--exception
  --  when dup_val_on_index then
        -- ignore rows that are already inserted
    --    null;
END;

IF  EXISTS ( SELECT 1 FROM SYS.SYSTRIGGER WHERE TRIGGER_NAME = 'T_OBJECT_DIM_SOURCE_3AD' ) THEN
  DROP TRIGGER T_OBJECT_DIM_SOURCE_3AD;
END IF;

CREATE OR REPLACE TRIGGER "SAHARA"."T_OBJECT_DIM_SOURCE_3AD"  AFTER DELETE
    ORDER 4 ON  SAHARA.OBJECT_DIM_SOURCE     
begin
    CALL SP_Load_AfterObjectDimSourceDelete();
end;

 

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_AfterObjectDimSourceDelete' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_AfterObjectDimSourceDelete;
END IF;
create procedure sahara.SP_Load_AfterObjectDimSourceDelete()
BEGIN
declare @vSQL long varchar;
declare @UPDATE_IS_DELETED long varchar;

    -- mark deleted the objects whose last reference has been deleted
    set @UPDATE_IS_DELETED =
        'update "$TABLE$" ' ||
           'set T2_IS_DELETED = ''T'' ' ||
         'where T2_IS_DELETED = ''F'' ' ||
           'and T2_IS_CURRENT = ''T'' ' ||
           'and $TABLE$_OID in ' ||
            -- select the OIDs that have been delete in this update
            -- and for which no more remain in the reference count table
            '(select OID ' ||
               'from T2_OBJECT_DIM_SOURCE_DELETE ' ||
              'where not exists (select 1 ' ||
                                  'from OBJECT_DIM_SOURCE ' ||
                                 'where OBJECT_DIM_SOURCE.OID = T2_OBJECT_DIM_SOURCE_DELETE.OID ' ||
                                   'and TABLE_NAME = vTable)' ||
                'and TABLE_NAME = vTable)';

    

    -- figure out which tables we're processing
    select distinct TABLE_NAME
       into #vTables
      from T2_OBJECT_DIM_SOURCE_DELETE;
    
    if not exists(select 1 from #vTables) then
        -- there nothing to do
        return;
    end if;

    -- mark all the objects as deleted if all the sources have
    -- stopped reporting them
   
    for TableCursor as cursorloop cursor for select TABLE_NAME as vTable from #vTables
        do
        set @vSQL = replace(@UPDATE_IS_DELETED, '$TABLE$', vTable);
        execute immediate @vSQL ;
    end for;

    -- free the temporary space
    delete from T2_OBJECT_DIM_SOURCE_DELETE;
end;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_Load_LoadCustom');
create procedure sahara.SP_Load_LoadCustom(in @pDataSetType varchar(64))
begin
 declare @FUNC varchar(32);
 declare @vSrcTable varchar(32);
 declare @vDstTable varchar(32);
 declare @vCurrentView varchar(64); 
 declare @vSurrogateKey varchar(32);
 declare @vRowCount integer;
 declare @vLoadSQL long varchar;
 declare @vDatesKey numeric(10);
 declare @vSourceKey numeric(10);
 declare @vSQL long varchar;
    
  begin
    set @FUNC = 'SP_Load_LoadCustom';
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

    select   rds.DEST_TABLE,rds.DEST_TABLE||'_',rds.DEST_TABLE|| '_CURRENT',rds.DEST_TABLE||'_KEY ',rds.LOAD_SQL
          into @vDstTable,@vSrcTable,@vCurrentView,@vSurrogateKey,@vLoadSQL
          from sahara.ROLLUP_DATA_SET_TYPE as rds
         where NAME = @pDataSetType;



    if @vLoadSQL is not null then
		
		-- the dates key was assigned to the batch when it was created
      select DATES_KEY, ROLLUP_SOURCE_KEY
      into @vDatesKey, @vSourceKey
      from ROLLUP_BATCH
      where ROLLUP_BATCH_KEY = @ActiveBatchKey_;

      set @vLoadSQL = replace(@vLoadSQL, '$DEST_TABLE$', @vDstTable);
      set @vLoadSQL = replace(@vLoadSQL, '$SRC_TABLE$', @vSrcTable);
--      set @vLoadSQL = replace(@vLoadSQL, '$ROLLUP_BATCH_KEY$', @ActiveBatchKey_);
--	  set @vLoadSQL = replace(@vLoadSQL, '$DATES_KEY$', @vDatesKey);

      set @vLoadSQL = 'begin ' || @vLoadSQL || '; end;';

        
      call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vLoadSQL);

      set @vRowCount = 0;
      execute immediate @vLoadSQL;
      
      set @vRowCount = @@rowcount;
        if @vRowCount > 0  then
            call sahara.SP_Load_DataSetRowCount(@vRowCount);
        end if;
    end if;

	-- merge the dimensions for the same day and same objects from
    -- different sources

--    if @vDstTable = 'LUN_CLAIMED_STATES' then

      --  call sahara.SP_Load_MergeDuplicateDims(@pDataSetType);
--	end if;

	-- merge the facts for the same day and same objects from
    -- different sources

--    if @vDstTable  = 'ARRAY_CLAIMED_CAPACITY' then

   ---     call sahara.SP_Load_MergeDuplicateFacts(@pDataSetType);
		
--	end if;
  end;	
end;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_Load_MergeDuplicateFacts');

create procedure sahara.SP_Load_MergeDuplicateFacts( in @pDataSetType varchar(64) )
begin 
  declare @FUNC varchar(60);
  declare @vPURGE_SQL long varchar;
  declare @vMERGE_SQL long varchar;
  declare @vUPDATE_SQL long varchar;
  declare @vSrcTable varchar(32);
  declare @vDstTable varchar(32);
  declare @vCurrentView varchar(32); 
  declare @vSurrogateKey varchar(32);
  declare @vDatesKey  numeric(10);
  declare @vSourceKey numeric(10);
  begin 

    set @FUNC = 'SP_Load_MergeDuplicateFacts';
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

	-- get some information from the batch to make the SQL a little simpler
    select DATES_KEY, ROLLUP_SOURCE_KEY
      into @vDatesKey, @vSourceKey
      from ROLLUP_BATCH
     where ROLLUP_BATCH_KEY = @ActiveBatchKey_;

    select rds.DEST_TABLE,rds.DEST_TABLE||'_',rds.DEST_TABLE|| '_CURRENT',rds.DEST_TABLE||'_KEY ',rds.LOAD_SQL
    into @vDstTable,@vSrcTable,@vCurrentView,@vSurrogateKey,@vLoadSQL
    from sahara.ROLLUP_DATA_SET_TYPE as rds
    where NAME = @pDataSetType;

  

     set @vPURGE_SQL = 
    'delete from SAHARA.ARRAY_CLAIMED_CAPACITY ' ||
         'where DATES_KEY <>'|| @vDatesKey;

     call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vPURGE_SQL);

	 execute immediate @vPURGE_SQL;
	 execute immediate 'truncate table SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY';

    -- form the MERGE_SQL
    set @vMERGE_SQL = 
    'insert into ACTUAL_ARRAY_CLAIMED_CAPACITY(ARRAY_KEY, PERCENT_ALLOCATED_HOST_CLAIMED, CLAIMED_BY_HOSTS) ' ||
		'SELECT ARRAY_KEY,sum(PERCENT_ALLOCATED_HOST_CLAIMED),sum(CLAIMED_BY_HOSTS) ' ||
		'from SAHARA.'||@vDstTable ||
		' WHERE ARRAY_KEY IN ' ||
			'(select ARRAY_KEY from SAHARA.'||@vDstTable ||
			 '  where DATES_KEY = '||@vDatesKey ||
			   '  and ROLLUP_BATCH_KEY in (select ROLLUP_BATCH_KEY ' ||
										  'from SAHARA.ROLLUP_BATCH ' ||
										 'where DATES_KEY = '||@vDatesKey ||
										   ' and ROLLUP_BATCH_KEY <> '||@ActiveBatchKey_  ||
										   '  and ROLLUP_SOURCE_KEY <> '||@vSourceKey ||') ' ||
			   'and (ARRAY_KEY) in (select ARRAY_KEY ' ||
									  'from SAHARA.'||@vDstTable ||
									 ' where ROLLUP_BATCH_KEY = '||@ActiveBatchKey_||')' ||
		') GROUP BY ARRAY_KEY';

    
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vMERGE_SQL);
   
    execute immediate @vMERGE_SQL;

    -- execute the merge facts sql

	
    set @vUPDATE_SQL =
		'MERGE INTO SAHARA.ARRAY_STORAGE DEST USING ' ||
			'SAHARA.ACTUAL_ARRAY_CLAIMED_CAPACITY SRC ' ||
				'ON (SRC.ARRAY_KEY = DEST.ARRAY_KEY and DEST.ROLLUP_BATCH_KEY = '||ActiveBatchKey_||') ' ||
			'WHEN MATCHED THEN UPDATE SET ' ||
				'DEST.PERCENT_ALLOCATED_HOST_CLAIMED = SRC.PERCENT_ALLOCATED_HOST_CLAIMED, ' ||
				'DEST.CLAIMED_BY_HOSTS = SRC.CLAIMED_BY_HOSTS';

    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vUPDATE_SQL);

 	execute immediate @vUPDATE_SQL;
end;
end;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_Load_MergeDuplicateDims');
create procedure sahara.SP_Load_MergeDuplicateDims(in @pDataSetType varchar(64))
begin 
  declare @FUNC varchar(60);
  declare @vPURGE_SQL long varchar;
  declare @vMERGE_SQL long varchar;
  declare @vUPDATE_SQL long varchar;
  declare @vSrcTable varchar(32);
  declare @vDstTable varchar(32);
  declare @vCurrentView varchar(32); 
  declare @vSurrogateKey varchar(32);
  declare @vDatesKey  numeric(10);
  declare @vSourceKey numeric(10);
begin

  set @FUNC = 'SP_Load_MergeDuplicateDims';
  call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

  select rds.DEST_TABLE,rds.DEST_TABLE||'_',rds.DEST_TABLE|| '_CURRENT',rds.DEST_TABLE||'_KEY ',rds.LOAD_SQL
  into @vDstTable,@vSrcTable,@vCurrentView,@vSurrogateKey,@vLoadSQL
  from sahara.ROLLUP_DATA_SET_TYPE as rds
  where NAME = @pDataSetType;

  select DATES_KEY, ROLLUP_SOURCE_KEY
  into @vDatesKey, @vSourceKey
  from ROLLUP_BATCH
  where ROLLUP_BATCH_KEY = @ActiveBatchKey_;
   
  set @vPURGE_SQL= 
  'delete from SAHARA.LUN_CLAIMED_STATES ' ||
         'where DATES_KEY <> '||@vDatesKey ;

  execute immediate @vPURGE_SQL;
  execute immediate 'truncate table SAHARA.ACTUAL_LUN_CLAIMED_STATES';

    -- form the MERGE_SQL

  set @vMERGE_SQL =
		'insert into ACTUAL_LUN_CLAIMED_STATES(LUN_KEY, Claimed_Status) ' ||
		'SELECT lun_key,Max(claimed_status) ' ||
		'from SAHARA.'||@vDstTable ||
		' WHERE LUN_KEY IN ' ||
			'(select LUN_KEY from SAHARA.'||@vDstTable||
			 ' where DATES_KEY = '||@vDatesKey ||
			   ' and ROLLUP_BATCH_KEY in (select ROLLUP_BATCH_KEY ' ||
										  'from SAHARA.ROLLUP_BATCH ' ||
										 'where DATES_KEY = '||@vDatesKey  ||
										   ' and ROLLUP_BATCH_KEY <> '||@ActiveBatchKey_  ||
										   ' and ROLLUP_SOURCE_KEY <> '||@vSourceKey||') ' ||
			   'and (LUN_KEY) in (select LUN_KEY ' ||
									  'from SAHARA.'||@vDstTable ||
									 ' where ROLLUP_BATCH_KEY = '||@ActiveBatchKey_||' )' ||
		') GROUP BY LUN_KEY';

  call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'SQL: ' || @vMERGE_SQL);

  execute immediate @vMERGE_SQL;
    
	
  set @vUPDATE_SQL=
        'update SAHARA.LUN ' ||
			'set LUN.CLAIMED_STATUS = ' ||
			'(SELECT ' ||
				'(CASE WHEN ACTUAL_LUN_CLAIMED_STATES.CLAIMED_STATUS = 0 THEN ''UNKNOWN'' ' ||
					   'WHEN ACTUAL_LUN_CLAIMED_STATES.CLAIMED_STATUS = 1 THEN ''UNCLAIMED'' '||
					   'ELSE ''CLAIMED'' END) AS ENUM_CLAIMED_STATE ' ||
			'FROM ACTUAL_LUN_CLAIMED_STATES WHERE LUN.LUN_KEY=ACTUAL_LUN_CLAIMED_STATES.LUN_KEY) ' ||			 
		'WHERE EXISTS ' ||
		'( SELECT ACTUAL_LUN_CLAIMED_STATES.LUN_KEY ' ||
		'FROM ACTUAL_LUN_CLAIMED_STATES ' ||
		'WHERE LUN.LUN_KEY=ACTUAL_LUN_CLAIMED_STATES.LUN_KEY) ';
 	execute immediate @vUPDATE_SQL;
end;
end;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('EVENT','ev_DataSetRowCount');

CREATE  EVENT SAHARA.ev_DataSetRowCount
handler
BEGIN
	commit;
    update ROLLUP_DATA_SET
       set ROWS_PROCESSED = event_parameter('eRowCount')
     where ROLLUP_BATCH_KEY =  event_parameter('eActiveBatchKey_')
       and ROLLUP_DATA_SET_TYPE_KEY = (select ROLLUP_DATA_SET_TYPE_KEY
                                         from ROLLUP_DATA_SET_TYPE
                                        where NAME = event_parameter('eActiveDataSetType_'));
 
       commit;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_Load_DataSetRowCount');
create procedure sahara.SP_Load_DataSetRowCount(in @vRowCount integer)
begin 
  declare @FUNC varchar(65);
  begin 
    set @FUNC = 'SP_Load_DataSetRowCount';
    call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');
   trigger event ev_DataSetRowCount( "eActiveDataSetType_" = @ActiveDataSetType_, "eRowCount" = @vRowCount,"eActiveBatchKey_"= @ActiveBatchKey_);
  end;
end;
 



IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_ActiveBatchSupportsDataSet' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Load_ActiveBatchSupportsDataSet;
END IF;

CREATE FUNCTION sahara.Fn_Load_ActiveBatchSupportsDataSet(@pDataSetName varchar(64))
RETURNS bit
begin
declare @vct integer;

    select count(*)
      into @vCt
      from ROLLUP_DATA_SET
      join ROLLUP_DATA_SET_TYPE
        on ROLLUP_DATA_SET.ROLLUP_DATA_SET_TYPE_KEY =
           ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY
       and ROLLUP_DATA_SET_TYPE.NAME = @pDataSetName
     where ROLLUP_DATA_SET.ROLLUP_BATCH_KEY = @gLoadActiveBatchKey;

    if (@vCt <> 0) then
        return 1;
    else
        return 0;
    end if;
    
end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_MakeSrcColumns' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Load_MakeSrcColumns;
END IF;

create function sahara.Fn_Load_MakeSrcColumns(in @pDataSetType varchar(64))
        returns  long varchar
    begin
      declare @vRC long varchar;
      declare @vColumnName varchar(64);
      declare @i integer;
	  declare @vVomObjType varchar(128);
	  declare @vAppendurl varchar(160);

        if not exists(select 1 from #vSrcColumns)  then
            return null;
        end if;

        set @vRC = '';
		set @vAppendurl = '';
	     
		 select VOM_OBJECT_TYPE into @vVomObjType
          from sahara.ROLLUP_DATA_SET_TYPE
         where NAME = @pDataSetType;

		 if @pDataSetType not in ('SF_LICENSE_DIM','SF_PRODUCT_DIM', 'SF_LICENSE_HOST_DIM','SF_LICENSE_DEPLOYMENT_DIM') then
				set @vAppendurl = '&ObjectType=' || @vVomObjType || '&ObjectName=';
 	     end if;

 		 if @pDataSetType = 'DB_DIM' then
				set @vAppendurl = '&ObjectType=vrts_dbed_''' || '||lower(SRC.DATABASE_TYPE)||' || '''_database&ObjectName=';
 	     end if;


        for ColumnCursor as cursorloop cursor for select name,expression, row_number() over (order by name) as rownum from #vSrcColumns
        do

            if rownum > 1 then
                set @vRC = @vRC || ', ';
            end if;

            set @vColumnName = 'SRC."' || name || '"';

            if expression is null then
                set @vRC = @vRC || @vColumnName;
            else
				if @vAppendurl <> '' then
					if @pDataSetType = 'VOLUME_DIM' then
						 set @vRC = @vRC || '(' ||  replace(expression,'$COLUMN$', @vColumnName) || ' || ''' || @vAppendurl || ''' || SRC.VOLUME_NAME' ||
										  ') as "' || name || '"';
					else if @pDataSetType = 'LUN_DIM' then
							 set @vRC = @vRC || '(case when SRC.type=''VMDK'' and SRC.vom_discover_type=''discovered by VMware VC only'' then '''' when SRC.type=''VMDK'' OR (SRC.type=''Disk'' AND SRC.ARRAY_OID is null) then (' || replace(expression,'$COLUMN$', @vColumnName) ||  ' || ''&ObjectType=vrts_vxvm_disk&ObjectName=' || ''' || SRC.DISPLAY_NAME) else ' ||
							 '(' ||  replace(expression,'$COLUMN$', @vColumnName) || ' || ''' || @vAppendurl || ''' || SRC.DISPLAY_NAME' ||
											  ') end) as "' || name || '"';
					else if @pDataSetType = 'NETAPP_VOLUME_DIM' then
							 set @vRC = @vRC || '(case when SRC.type=''Storage Pool'' then (' || replace(expression,'$COLUMN$', @vColumnName) ||  ' || ''&ObjectType=vrts_array_raid_group&ObjectName=' || ''' || SRC.DISPLAY_NAME) when SRC.type=''Volume'' then (' || replace(expression,'$COLUMN$', @vColumnName) ||  ' || ''&ObjectType=vrts_array_storevol&ObjectName=' || ''' || SRC.DISPLAY_NAME) else ' ||
							 '(' ||  replace(expression,'$COLUMN$', @vColumnName) || ' || ''' || @vAppendurl || ''' || SRC.DISPLAY_NAME' ||
											  ') end) as "' || name || '"';
					else if @pDataSetType = 'QUOTA_DIM' then
							 set @vRC = @vRC || ''''' ) as "' || name || '"';
						else
							 set @vRC = @vRC || '(' ||  replace(expression,'$COLUMN$', @vColumnName) || ' || ''' || @vAppendurl || ''' || SRC.DISPLAY_NAME' ||
											  ') as "' || name || '"';
						end if;
					end if;
                    end if;
                    end if;
				else
					set @vRC = @vRC || '(' ||  replace(expression,'$COLUMN$', @vColumnName) ||
								  ') as "' || name || '"';				
				end if;
            end if;
        end for;

        return @vRC;
    end;


-- makes the "on" portion of the merge statement that is based on
    -- staging key columns - no nulls allowed
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_MakeOnKey' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Load_MakeOnKey;
END IF;

create function sahara.Fn_Load_MakeOnKey()
returns  long varchar
begin
declare @vRC long varchar;

        
         


    for ColumnCursor as cursorloop cursor for select name as colname , row_number() over (order by name) as rownum from vKeyColumns
        do

            if rownum = 1 then
                set @vRC = 'SRC."' || colname || '" = DEST."' || colname || '"';
            else 
                   set @vRC = @vRC || ' and SRC."' || colname || '" = DEST."' || colname || '"';
            end if;
         end for;
        return @vRC;
    end;

    -- makes the "on" portion of the merge statement that is based on type 2
    -- dimension columns - nulls allowed

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_MakeOnType2' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Load_MakeOnType2;
END IF;

create function sahara.Fn_Load_MakeOnType2()
returns  long varchar
begin
declare @vRC long varchar;


    
     
for ColumnCursor as cursorloop cursor for select name as colname , row_number() over (order by name) as rownum from vType2Columns
        do

            if rownum = 1 then                
                SET @vRC = '(SRC."' || colname || '" = DEST."' || colname || '" or ' ||
                   '(SRC."' || colname || '" is null and DEST."' || colname || '" is null))';

            else 
                  SET @vRC =@vRC|| 'and (SRC."' || colname || '" = DEST."' || colname || '" or ' ||
                   '(SRC."' || colname || '" is null and DEST."' || colname || '" is null))';
                   
            end if;
         end for;
        return @vRC;
        

    end;

    -- makes the update "set" portion of the merge statement

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_MakeSet' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Load_MakeSet;
END IF;

create function sahara.Fn_Load_MakeSet()
returns  long varchar
begin
declare @vRC long varchar;

        
         


    for ColumnCursor as cursorloop cursor for select name as colname , row_number() over (order by name) as rownum from vType1Columns
        do

            if rownum = 1 then
                set @vRC = 'DEST."' || colname || '" = SRC."' || colname || '"';
            else 
                   set @vRC = @vRC || ', DEST."' || colname || '" = SRC."' || colname || '"';
            end if;
         end for;
        return @vRC;
    end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_MakeListDim' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Load_MakeListDim;
END IF;

create function sahara.Fn_Load_MakeListDim( in @pColumnPrefix varchar(32) default '')
returns  long varchar
begin
declare @vRC long varchar;


for ColumnCursor as cursorloop cursor for select name as colname , row_number() over (order by name) as rownum from vDimColumns
        do

            if rownum = 1 then
                set @vRC = @pColumnPrefix || '"' || colname || '"';    
            else 
                   set @vRC = @vRC || ', ' || @pColumnPrefix || '"' || colname || '"';                   
            end if;
         end for;
        return @vRC;    
end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Load_MakeListStaging' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Load_MakeListStaging;
END IF;

create function sahara.Fn_Load_MakeListStaging( in @pColumnPrefix varchar(32) default '')
returns  long varchar
begin
declare @vRC long varchar;


for ColumnCursor as cursorloop cursor for select name as colname , row_number() over (order by name) as rownum from #vSrcStagingColumns
        do

            if rownum = 1 then
                set @vRC = @pColumnPrefix || '"' || colname || '"';    
            else 
                   set @vRC = @vRC || ', ' || @pColumnPrefix || '"' || colname || '"';                   
            end if;
         end for;
        return @vRC;    
end;



IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Load_CreateTempTable' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Load_CreateTempTable;
END IF;
create PROCEDURE sahara.SP_Load_CreateTempTable()
BEGIN
IF EXISTS (SELECT * FROM sys.systable
        WHERE table_name = 'SrcColumnRecord'
        AND table_type = 'GBL TEMP')
THEN
        DROP TABLE sahara.SrcColumnRecord;
END IF;
create global temporary table sahara.SrcColumnRecord 
    (
       NAME                 varchar(30)                    not null,
        SOURCE_EXPRESSION    varchar(128)                   null
    )on commit preserve rows;


IF EXISTS (SELECT * FROM sys.systable
        WHERE table_name = 'vDimColumns'
        AND table_type = 'GBL TEMP')
THEN
        DROP TABLE sahara.vDimColumns;
END IF;
create global temporary table sahara.vDimColumns 
(
       NAME                 varchar(30)                    not null
)on commit preserve rows;


IF EXISTS (SELECT * FROM sys.systable
        WHERE table_name = 'vType1Columns'
        AND table_type = 'GBL TEMP')
THEN
        DROP TABLE sahara.vType1Columns;
END IF;

create global temporary table sahara.vType1Columns 
(
       NAME                 varchar(30)                    not null
)on commit preserve rows;

IF EXISTS (SELECT * FROM sys.systable
        WHERE table_name = 'vType2Columns'
        AND table_type = 'GBL TEMP')
THEN
        DROP TABLE sahara.vType2Columns;
END IF;

    create global temporary table sahara.vType2Columns 
    (
       NAME                 varchar(30)                    not null
    )on commit preserve rows;

IF EXISTS (SELECT * FROM sys.systable
        WHERE table_name = 'vKeyColumns'
        AND table_type = 'GBL TEMP')
THEN
        DROP TABLE sahara.vKeyColumns;
END IF;
create global temporary table sahara.vKeyColumns 
(
       NAME                 varchar(30)                    not null
)on commit preserve rows;
END;
        

-------------------------------------------------------------------------------


IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_Log')THEN
      DROP PROCEDURE "SAHARA"."SP_Log_Log";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_Log" (IN @pSessionKey numeric(10),
    IN @pFunction  VARCHAR(256),
    IN @pMessage long varchar,
    IN @pSeverity char(1))      
BEGIN
	DECLARE @l_CurrentTimeStamp TIMESTAMP;
	set @l_CurrentTimeStamp = CURRENT UTC TIMESTAMP;
	
	/*
	 **********************************************************************	 
	 * Execute the logging thru an event to make it autonomous transation
	 * Logging should be DONE in case of failure also.
	 **********************************************************************
	 */
	trigger event ER_Log_LOG( "esessionkey" = @pSessionKey,"etime" =@l_CurrentTimeStamp,"efunction" =@pFunction, "eseverity" =@pSeverity, "emsg" =@pMessage );

END;
    
-------------------------------------------------------------------------------
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_LogDebug')THEN
      DROP PROCEDURE "SAHARA"."SP_Log_LogDebug";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_LogDebug" (IN @pSessionKey numeric(10),
    IN @pFunction  VARCHAR(256),
    IN @pMessage long varchar )  
BEGIN
    if (@gLogDebugEnabled = 1) then
       call  SP_Log_Log(@pSessionKey, @pFunction, @pMessage,'D');
    end if;       
END;




IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_LogError')THEN
      DROP PROCEDURE "SAHARA"."SP_Log_LogError";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_LogError" (IN @pSessionKey numeric(10),
    IN @pFunction  VARCHAR(256),
    IN @pMessage long varchar )  
BEGIN

     call  SP_Log_Log(@pSessionKey, @pFunction, @pMessage,'E');
END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_LogInfo')THEN
      DROP PROCEDURE "SAHARA"."SP_Log_LogInfo";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_LogInfo" (IN @pSessionKey numeric(10),
    IN @pFunction  VARCHAR(256),
    IN @pMessage long varchar )  
BEGIN

     call  SP_Log_Log(@pSessionKey, @pFunction, @pMessage,'I');
       
END;



IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_LogWarn')THEN
      DROP PROCEDURE "SAHARA"."SP_Log_LogWarn";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_LogWarn" (IN @pSessionKey numeric(10),
    IN @pFunction  VARCHAR(256),
    IN @pMessage long varchar )  
BEGIN

     call  SP_Log_Log(@pSessionKey, @pFunction, @pMessage,'W');
   
END;





IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_Purge' and user_name(creator) = 'SAHARA' )THEN
      DROP PROCEDURE "SAHARA"."SP_Log_Purge";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_Purge" (IN @pPurgetime integer)      
BEGIN
	 message @pPurgetime to client;
	trigger event Ev_Log_DeleteLog( "vAgeCutoffInSeconds" = @pPurgetime);

END;

-------------------------------------------------------------------------------
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_PurgeMessageLogTable' and user_name(creator) = 'SAHARA' )THEN
      DROP PROCEDURE "SAHARA"."SP_Log_PurgeMessageLogTable";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_PurgeMessageLogTable" ()
BEGIN

    DECLARE @FUNC varchar(50);
    DECLARE @vEnabled bit;
    DECLARE @vNow TIMESTAMP;
    DECLARE @vAgeCutoff TIMESTAMP;
    DECLARE @vRowCount integer;

   set  @FUNC =  'SP_Log_PurgeMessageLogTable';
    call SP_Log_LogDebug(null, @FUNC, 'entry');
    set @vEnabled = fn_GetConfigBool(@glog_CONFIG_SECTION, 'AGING_ENABLED');
    if (@vEnabled = 0) then
        call SP_Log_LogDebug(null, @FUNC, 'message log purging is disabled');
        return;
    end if;

    
    -- delete records older than the aging retention
    set @vNow = CURRENT UTC TIMESTAMP;
    set @vAgeCutoff = DATEADD(second, 
                            fn_conf_GetConfigDayInterval(@gLog_CONFIG_SECTION, 'AGING_RETENTION')*(-1)
                            ,@vNow);
    CALL SP_Log_LogDebug(null, @FUNC, 'purging at: ' ||
                         DATEFORMAT(@vNow, 'MM/dd/yyyy HH:NN:SS') ||
                         ', retention cut-off: ' || DATEFORMAT(@vAgeCutoff, 'MM/dd/yyyy HH:NN:SS'));

    
    --call LogDebug(null, @FUNC, 'purging at: ' ||
      --                   to_char(vNow, 'mm/dd/yyyy hh24:mi:ss') ||
        --                 ', retention cut-off: ' ||
          --               to_char(vAgeCutoff, 'mm/dd/yyyy hh24:mi:ss'));

    -- do the actual delete as an autonomous operation to avoid deadlock
   call SP_Log_Purge(fn_conf_GetConfigDayInterval(@gLog_CONFIG_SECTION, 'AGING_RETENTION'));
    
    -- save the time we aged last for future calculation
    call fn_Configuration_SetDate(null, @gLog_CONFIG_SECTION, 'LAST_AGED', @vNow);
    commit;
end;

-------------------------------------------------------------------------------
if exists (select 1 from sys.sysprocedure where proc_name = 'fn_Log_TimeToNextMessageLogTablePurge' and user_name(creator) = 'SAHARA') then
    DROP FUNCTION sahara.fn_Log_TimeToNextMessageLogTablePurge;
end if;
CREATE FUNCTION  "sahara"."fn_Log_TimeToNextMessageLogTablePurge"  ()
returns integer
BEGIN
DECLARE @FUNC varchar(64);   
DECLARE @vEnabled bit;
DECLARE @vNextAge DATETIME;
DECLARE @vRC integer;

--function TimeToNextMessageLogTablePurge
  --  return CONFIGURATION.DAY_INTERVAL_VALUE%type
--as
  set  @FUNC =  'fn_Log_TimeToNextMessageLogTablePurge';

    set @vEnabled = fn_GetConfigBool(@gLog_CONFIG_SECTION, 'AGING_ENABLED');
      --  GetConfigBool(gLog_CONFIG_SECTION, 'AGING_ENABLED');
    --vNextAge date;
    --vRC CONFIGURATION.DAY_INTERVAL_VALUE%type;
--begin
    CALL SP_Log_LogDebug(null, @FUNC, 'entry');

    -- if purge is not enabled, the next purge time is "forever" away
    if (@vEnabled = 0) then
      CALL SP_Log_LogDebug(null, @FUNC, 'message log purging is disabled');
        return 31536000;        -- Keep it one year
    end if;

    -- figure out when the next aging time is
    set @vNextAge = DATEADD(second, 
                            fn_conf_GetConfigDayInterval(@gLog_CONFIG_SECTION, 'AGING_INTERVAL')
                            ,fn_conf_GetConfigDate(@gLog_CONFIG_SECTION, 'LAST_AGED'));
                ;
    set @vRC = DATEDIFF(second,CURRENT UTC TIMESTAMP,@vNextAge);

    CALL SP_Log_LogDebug(null, @FUNC, 'Last time aging occured : ' || DATEFORMAT(fn_conf_GetConfigDate(@gLog_CONFIG_SECTION, 'LAST_AGED'),'MM/dd/yyyy HH:NN:SS') || 'next age time: ' ||
                         DATEFORMAT(@vNextAge, 'MM/dd/yyyy HH:NN:SS') ||
                         ', wait interval: ' || @vRC || ' seconds' );

    return @vRC;
end;

-------------------------------------------------------------------------------

IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Log_RefreshCachedConfiguration'  and user_name(creator) = 'SAHARA')THEN
      DROP PROCEDURE "SAHARA"."SP_Log_RefreshCachedConfiguration" ;  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_Log_RefreshCachedConfiguration" ()
   begin
    -- to set the debug flag
    set @gLogDebugEnabled = fn_GetConfigBool(@glog_CONFIG_SECTION, 'DEBUG_ENABLED');

exception
    when others then
    set @gLogDebugEnabled = 1;
end;


/*
 **********************************************************************
 * EVENT: Ev_Log_DeleteLog
 * 
 * Creating the event to purge the message log.
 *
 **********************************************************************
 */

if exists(select 1 from sys.sysevent where event_name = 'Ev_Log_DeleteLog' and user_name(creator) = 'SAHARA') then
   drop event sahara.Ev_Log_DeleteLog
end if;

create event sahara.Ev_Log_DeleteLog
handler
begin  
   
   --   message 'In the delete log event' to client;
     --call SP_Log_LogInfo(null, 'Ev_Log_DeleteLog', ' Entry : time'|| event_parameter('vAgeCutoffInSeconds'));    
    delete SAHARA.MESSAGE_LOG
         where DATEDIFF(second,"TIME",CURRENT UTC TIMESTAMP)  > event_parameter('vAgeCutoffInSeconds');
    commit;  
   call SP_Log_LogInfo(null, 'Ev_Log_DeleteLog', @@rowcount || ': message log rows deleted');    
   
end;

/*
 **********************************************************************
 * EVENT: ER_Log_LOG
 * 
 * Execute the logging thru an event to make it autonomous transation.
 * When an event is executed/triggered, it gets a separate connection, 
 * which is why it can commit separately. 
 * It also means the event runs asynchronously, unlike a procedure call 
 * which runs synchronously.
 **********************************************************************
 */
if exists(select 1 from sys.sysevent where event_name = 'ER_Log_LOG' and user_name(creator) = 'SAHARA') then
   drop event sahara.ER_Log_LOG
end if;

create event sahara.ER_Log_LOG
handler
begin  
    insert into sahara.MESSAGE_LOG(SESSION_KEY,"TIME", FUNCTION_NAME, SEVERITY,"MESSAGE")
        values(event_parameter( 'esessionkey' ),event_parameter( 'etime' ),event_parameter( 'efunction' ),event_parameter( 'eseverity' ),event_parameter( 'emsg' ));
 
    commit;
end;




/*
 * $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
 * $Revision: 1.119.4.1 $
 * $Copyright: Copyright (c) 2012 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

/*
 **********************************************************************
 * SP_Lock_LockSession():
 * Acquires lock on the given control module(CONFIGURATION/IE/LOAD.
 * The passed in session is remembered with the lock.
 * Each module is required to acquire the lock in order to seralize 
 * access to it. For example, any rollup must acquire this lock to ensure
 * that only one rollup is staging/loding data into tables.
 * The stored proc uses isloation level 3 to ensure that test/set of lock
 * is atomic. It does resets the isolation level to old value before 
 * returning, so the parent transaction is not impacted.
 * The stored proc does call commit inside, so there may be side effects 
 * if the caller hamust be aware of this.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Lock_LockSession')
THEN
        DROP PROCEDURE sahara.SP_Lock_LockSession;
END IF;

create procedure sahara.SP_Lock_LockSession(
     IN @ControlName VARCHAR(32),
     IN @SessionKey NUMERIC(10))
BEGIN
   DECLARE @func VARCHAR(64);
   DECLARE @name_val VARCHAR(32);
   DECLARE @sess_key NUMERIC(10);
   DECLARE @islocked BIT;
   DECLARE @gotlock BIT;
   DECLARE @old_level NUMERIC(1);
   DECLARE record_not_found EXCEPTION FOR SQLSTATE '02000';

   SET @func = 'sahara.SP_Lock_LockSession';
   CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:' || @ControlName || ',' 
		|| @SessionKey);

   -- Check if the given control name exists. If so, the session key passed 
   -- matches with the one in the table
   SELECT name INTO @name_val FROM sahara.control_owner 
	WHERE name = @ControlName;
   if( @name_val is null) THEN
   BEGIN
        CALL SP_Log_LogDebug(@SessionKey, @func, 'Error: Invalid Control Name ' || 
		@ControlName);
        CALL SP_Error_RaiseApplicationErrorDetail (
                @EDBMSLockRequestFailed,
                @FUNC,
		'failed acquiring lock: Invalid Control name ' || @ControlName,
                @SessionKey);
   END
   END IF;

   -- Get the current isolation level and remember it
   -- and then set isolation level to 3
   SELECT @@isolation INTO @old_level;
   CALL SP_Log_LogDebug(@SessionKey, @func, 'Old isolation level = ' || @old_level);
   SET TEMPORARY OPTION ISOLATION_LEVEL = 3;
   
   SET @gotlock = 0;
   WHILE (@gotlock = 0)
   LOOP
	CALL SP_Log_LogDebug(@SessionKey, @func, 'About to check ER lock for ' || 
		@ControlName );
    SELECT IsLocked, session_key INTO @islocked, @sess_key 
	FROM sahara.control_owner WHERE name=@ControlName;
   
    -- IF lock is already obtained by the same session, return error
    IF @sess_key is not null AND @islocked = 1 AND @sess_key = @SessionKey THEN
    BEGIN
        CALL SP_Log_LogDebug(@SessionKey, @func,  
		'Lock already held by the session' || @SessionKey );
        commit;
        --reset the isolation level to old value
        EXECUTE IMMEDIATE 'SET TEMPORARY OPTION ISOLATION_LEVEL =' ||  
		@old_level;

        CALL SP_Error_RaiseApplicationErrorDetail (
                @EDBMSLockRequestFailed,
                @FUNC,'Failed to acquire lock: Lock already held for ' || 
			@ControlName || ' by the session ' || @SessionKey,
                @SessionKey);
    END
    ENDIF;

	IF (@islocked = 0) THEN
	  BEGIN
	   UPDATE sahara.control_owner set IsLocked=1, session_key=@SessionKey 
		WHERE name=@ControlName;
           CALL SP_Log_LogDebug(@SessionKey, @func, 'GOT the lock!');
           COMMIT;
           SET @gotlock = 1;
	  END
	else
	  BEGIN
		CALL SP_Log_LogDebug(@SessionKey, @func, 
			'ER lock already held, sleeping...' );
        	COMMIT;
		WAITFOR DELAY '00:00:05';
	  END
    END IF
   END LOOP;

   --reset the isolation level to old value
   EXECUTE IMMEDIATE 'SET TEMPORARY OPTION ISOLATION_LEVEL =' ||  @old_level;
END;




/*
 **********************************************************************
 * SP_Lock_UnlockSession():
 * Release lock on the given control module(CONFIGURATION/IE/LOAD.
 * The passed in session is used to validate that the lock was acquired by it.
 *
 * The stored proc uses isloation level 3 to ensure that test/set of lock
 * is atomic. It does resets the isolation level to old value before 
 * returning, so the parent transaction is not impacted.
 * The stored proc does call commit inside, so there may be side effects 
 * if the caller hamust be aware of this.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Lock_UnlockSession')
THEN
        DROP PROCEDURE sahara.SP_Lock_UnlockSession;
END IF;

create procedure sahara.SP_Lock_UnlockSession(
     IN @ControlName VARCHAR(32),
     IN @SessionKey numeric(10))
BEGIN
   DECLARE @func VARCHAR(64);
   DECLARE @sess_key NUMERIC(10);
   DECLARE @islocked BIT;
   DECLARE @name_val VARCHAR(32);
   DECLARE @old_level NUMERIC(1);

   SET @func = 'sahara.SP_Lock_UnlockSession';
   CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:' || @ControlName || ',' || @SessionKey);

   -- Check if the given control name exists.
   SELECT name INTO @name_val FROM sahara.control_owner WHERE name = @ControlName;
   IF( @name_val is null) THEN
   BEGIN
        CALL SP_Error_RaiseApplicationErrorDetail (
                @EDBMSLockReleaseFailed,
                @FUNC, 'Failed to release lock: Invalid Control name ' || @ControlName,
                @SessionKey);
   END
   END IF;

   -- Get the current isolation level and remember it
   -- and then set isolation level to 3
   SELECT @@isolation INTO @old_level;
   CALL SP_Log_LogDebug(@SessionKey, @func,  'Old isolation level = ' || @old_level );
   SET TEMPORARY OPTION ISOLATION_LEVEL = 3;
   
    -- Validate that the it is currently locked, and the lock is held by the session passed in. If not,fail.
    SELECT IsLocked, session_key INTO @islocked, @sess_key 
	FROM sahara.control_owner 
	WHERE name=@ControlName;
    IF (@islocked = 0) OR (@islocked = 1 AND @sess_key <> @SessionKey) THEN
    BEGIN
        CALL SP_Log_LogDebug(@SessionKey, @func,  
	  'Lock not held, or held by a different session!!' || @SessionKey );
        commit;

        --reset the isolation level to old value
        EXECUTE IMMEDIATE 'SET TEMPORARY OPTION ISOLATION_LEVEL =' ||  
		@old_level;

        CALL SP_Error_RaiseApplicationErrorDetail (
                @EDBMSLockReleaseFailed,
                @FUNC,'Failed to release lock: Either not locked, or not held my the requested session ' || @SessionKey,
                @SessionKey);
    END
    ENDIF;

   -- All good, let's clear the lock!
   UPDATE sahara.control_owner SET IsLocked=0, session_key=null 
	WHERE name=@ControlName;
   commit;

   --reset the isolation level to old value
   EXECUTE IMMEDIATE 'SET TEMPORARY OPTION ISOLATION_LEVEL =' ||  @old_level;
END;


-----------------------------------------------------------------------
-- $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
-- $Revision: 1.119.4.1 $
-- $Copyright: Copyright (c) 2012 Symantec Corporation.
-- All rights reserved.
--
-- THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
-- SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
-- WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
--
-- The Licensed Software and Documentation are deemed to be commercial
-- computer software as defined in FAR 12.212 and subject to restricted
-- rights as defined in FAR Section 52.227-19 "Commercial Computer
-- Software - Restricted Rights" and DFARS 227.7202, "Rights in
-- Commercial Computer Software or Commercial Computer Software
-- Documentation", as applicable, and any successor regulations. Any use,
-- modification, reproduction release, performance, display or disclosure
-- of the Licensed Software and Documentation by the U.S. Government
-- shall be solely in accordance with the terms of this Agreement.  $
-----------------------------------------------------------------------

message 'Creating/replacing CONFIGURATION LOCK module stored procedures...' to client;


/*
 **********************************************************************
 * SP_Config_AcquireExclusiveLock():
 * Acquires exclusive lock on the CONFIGUATION module.
 * Any module that does business view / Tier management must acquire this lock 
 * to ensure * that only one connection doing the operations.
 * The stored proc uses underlying Lock API stored proc.
 * The stored proc does call commit inside, so there may be side effects
 * if the caller hamust be aware of this.
 * The passed in session is remembered with the lock.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Config_AcquireExclusiveLock' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Config_AcquireExclusiveLock;
END IF;

create procedure sahara.SP_Config_AcquireExclusiveLock(
     IN @SessionKey NUMERIC(10),
     IN @Timeout    integer default null)
BEGIN
    DECLARE @func VARCHAR(64);
    -- initialize global variables
    call sahara.SP_saharaAPIInit();
 
    SET @func = 'SP_Config_AcquireExclusiveLock';
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call LockSession() for the CONFIGURATION module. If fails, it will raise
    -- an error, which will throw the exception which will return the
    -- control to the caller.
    CALL SP_Lock_LockSession('SAHARA_CONFIGURATION_CONTROL', @SessionKey);
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully acquired the lock');

    -- If we are here, then the lock was acquired, remember the session etc.
    SET @gConfigHaveControl = 1;
    SET @gConfigActiveSessionKey = @SessionKey;

END;
grant execute on sahara.SP_Config_AcquireExclusiveLock to sahara_app;

/*
 **********************************************************************
 * SP_Config_ReleaseExclusiveLock():
 * Release the exclusive lock on the CONFIGURATION CONTROL module previously 
 * acquired for the given session.
 * The stored proc uses underlying unlock API stored proc.
 * The stored proc does call commit inside, so there may be side effects
 * if the caller hamust be aware of this.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Config_ReleaseExclusiveLock' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Config_ReleaseExclusiveLock;
END IF;

create procedure sahara.SP_Config_ReleaseExclusiveLock()
BEGIN
    DECLARE @func VARCHAR(64);

    -- initialize global variables
    call sahara.SP_saharaAPIInit();

    SET @func = 'SP_Config_ReleaseExclusiveLock';
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @func, 'Entering:');


    -- Call UnlockSession() for the CONFIGURATION CONTROL module. 
    -- If this fails, it will raise an error, which will throw the 
    -- exception which will return the control to the caller.
    CALL SP_Lock_UnlockSession('SAHARA_CONFIGURATION_CONTROL', @gConfigActiveSessionKey);

    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @func, 'Successfully release the lock');
    -- Clear the state.
    SET @gConfigHaveControl = 0;
    SET @gConfigActiveSessionKey = null;
END;
grant execute on sahara.SP_Config_ReleaseExclusiveLock to sahara_app;


-------------------------------------------------------------------------------
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_Config_GetActiveSessionKey' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_Config_GetActiveSessionKey;
END IF;

CREATE FUNCTION sahara.Fn_Config_GetActiveSessionKey()
RETURNS integer
NOT DETERMINISTIC
BEGIN
	DECLARE @vPersistedActiveSessionKey numeric(10);

	-- if this database session has the control, we have the
	-- session key cached
	if ( VAREXISTS('@gConfigHaveControl') = 1 ) then
		if ( @gConfigHaveControl = 1) then
	        return @gConfigActiveSessionKey;
		end if;	
	end if;

	-- otherwise, get it from the control_owner table
	select SESSION_KEY
	  into @vPersistedActiveSessionKey
	  from SAHARA.CONTROL_OWNER
	 where NAME = 'SAHARA_CONFIGURATION_CONTROL';
	
	return @vPersistedActiveSessionKey;
    
END;

grant execute on sahara.Fn_Config_GetActiveSessionKey to sahara_app;


----------------------------------------------------------------------------
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_Config_RequireControl' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_Config_RequireControl;
END IF;

CREATE PROCEDURE sahara.SP_Config_RequireControl(
    IN @Func long varchar)
BEGIN
    IF ( @gConfigHaveControl = 0) THEN
      BEGIN
        CALL SP_Error_RaiseApplicationError(
            @ENoConfigurationControl,
            @Func || ': ' || 'caller does not have configuration control lock'
        );
       END
    END IF;
END;
-----------------------------------------------------------------------
-- $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
-- $Revision: 1.119.4.1 $
-- $Copyright: Copyright (c) 2012 Symantec Corporation.
-- All rights reserved.
--
-- THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
-- SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
-- WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
--
-- The Licensed Software and Documentation are deemed to be commercial
-- computer software as defined in FAR 12.212 and subject to restricted
-- rights as defined in FAR Section 52.227-19 "Commercial Computer
-- Software - Restricted Rights" and DFARS 227.7202, "Rights in
-- Commercial Computer Software or Commercial Computer Software
-- Documentation", as applicable, and any successor regulations. Any use,
-- modification, reproduction release, performance, display or disclosure
-- of the Licensed Software and Documentation by the U.S. Government
-- shall be solely in accordance with the terms of this Agreement.  $
--
-- Business View management APIS
-----------------------------------------------------------------------

message 'Creating/replacing BUSINESS VIEW module TMP TABLES ...' to client;

IF EXISTS (SELECT 1 FROM sys.systable WHERE table_name = 'tmp_BViewKeyTable'  
			AND  table_type = 'GBL TEMP')
THEN
    DROP TABLE SAHARA.tmp_BViewKeyTable;
END IF;

create global temporary table SAHARA.tmp_BViewKeyTable(
BVIEW_KEY INTEGER,
SessionKey INTEGER default 0,primary key (BVIEW_KEY,SessionKey) ) on commit preserve rows;


IF EXISTS (SELECT 1 FROM sys.systable WHERE table_name = 'DimensionRowKeyTable'  
			AND  table_type = 'GBL TEMP')
THEN
    DROP TABLE SAHARA.DimensionRowKeyTable;
END IF;

create global temporary table SAHARA.DimensionRowKeyTable(
KEYS numeric(10)) on commit preserve rows;

create index ind_DimKeyTable on SAHARA.DimensionRowKeyTable(KEYS);

IF EXISTS (SELECT 1 FROM sys.systable WHERE table_name = 'tmp_BViewDimensionKeyTable'  
			AND  table_type = 'GBL TEMP')
THEN
    DROP TABLE SAHARA.tmp_BViewDimensionKeyTable;
END IF;

CREATE GLOBAL TEMPORARY TABLE SAHARA.tmp_BViewDimensionKeyTable(
BVIEW_KEY INTEGER,
SessionKey INTEGER default 0,primary key (BVIEW_KEY,SessionKey) ) 
ON COMMIT PRESERVE ROWS;



message 'Creating/replacing BUSINESS VIEW module stored procedures...' to client;


/*
 **********************************************************************
 * SP_BView_Create
 * Creates a business view object.
 **********************************************************************
 */


-----

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BView_Create')
THEN
        DROP PROCEDURE sahara.SP_BView_Create;
END IF;

create procedure sahara.SP_BView_Create(
    IN @DisplayName VARCHAR(64),
    IN @Name        VARCHAR(12),
    IN @Note        long varchar default null
)
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SP_BView_Create';
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @func, 'Entering:');

    -- Check to make sure CONFIGURATION LOCK is held
    CALL SP_Config_RequireControl(@func);


    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @func, 'Leaving:');
END;


/*
 **********************************************************************
 * SP_BView_Delete
 * Deletes a business view object.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BView_Delete')
THEN
        DROP PROCEDURE sahara.SP_BView_Delete;
END IF;

create procedure sahara.SP_BView_Delete(
    IN @BViewKey NUMERIC(10)
)
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SP_BView_Delete';
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @func, 'Entering:');

    -- Check to make sure CONFIGURATION LOCK is held
    CALL SP_Config_RequireControl(@func);


    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @func, 'Leaving:');
END;


IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_NameUsed') THEN
  DROP FUNCTION SAHARA.FN_BVIEW_NameUsed;
END IF;

CREATE FUNCTION SAHARA.FN_BVIEW_NameUsed(in @pName LONG VARCHAR)
  RETURNS BIT
BEGIN
  
  DECLARE @vCount int;
  DECLARE @vNameUsed BIT;
  
  BEGIN
    SET @vNameUsed = 0;	
    SET @vCount = 0;
      
    SELECT 
      count(*)into @vCount
    FROM 
      BVIEW_IMPL
    WHERE 
      NAME = @pName;

    IF coalesce(@vCount,0) > 0 THEN 
      SET @vNameUsed = 1;	
    END IF;
  
    RETURN @vNameUsed ;
  
  END;

END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure  WHERE proc_name = 'SP_BVIEW_ValidateName')THEN
  DROP PROCEDURE "sahara"."SP_BVIEW_ValidateName";  
END IF; 

CREATE PROCEDURE SAHARA.SP_BVIEW_ValidateName(IN @pName LONG VARCHAR)

BEGIN
  
  IF @pName is null THEN
  
    CALL   SP_Error_RaiseApplicationErrorDetail
      (
            @EBViewInvalidName,
            'SP_BVIEW_ValidateName',
            'the business view or level name is null - a valid column ' ||
            'name is required'
      );
    
    END IF;

    IF  SUBSTRING(@pName, 1, 1) NOT REGEXP '[A-Z]+' THEN
    
      CALL  SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewInvalidName,
            'SP_BVIEW_ValidateName',
            'the first characters of the business view or level name ' ||
            'is invalid - a valid column name is required - name: ' || @pName
        );
    
    END IF;

    IF  @pName NOT REGEXP '[[:upper:][:digit:]_#$]+' THEN
    
      CALL  SP_Error_RaiseApplicationErrorDetail
        (
                 @EBViewInvalidName,
                'SP_BVIEW_ValidateName',
                'the business view or level name is invalid - a valid column ' ||
                'name is required - name: ' || @pName
        );
    
    END IF;
END;


IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_RemoveInvalidCharacters') THEN
  DROP FUNCTION "SAHARA"."FN_BVIEW_RemoveInvalidCharacters";
END IF;

CREATE FUNCTION "SAHARA"."FN_BVIEW_RemoveInvalidCharacters"(in @pStr LONG VARCHAR)
    RETURNS VARCHAR(255)
BEGIN
  
  DECLARE @vCh VARCHAR(1);
  DECLARE @incr INTEGER;
  DECLARE @newstr VARCHAR(2048);
  
  BEGIN
    
    SET @incr = 1;

-- if there's nothing to work with, return an empty string
    IF @pStr IS NULL THEN
      RETURN '';
    END IF;

--- check if string is valid 
   
   IF  @pStr NOT REGEXP   '[[:upper:][:digit:]_#$]+' THEN 

---- remove invalid character and return valid string 

     SET  @newstr = @pStr;
     
     WHILE @incr <= length(@pStr)
     LOOP
       
       SET @vCh = substring(@pStr, @incr, 1);  
     
       IF @vCh NOT REGEXP   '[[:upper:][:digit:]_#$]+' THEN
         SET @newstr = replace(@newstr,@vCh,'')
       END IF;
       
       SET @incr = @incr + 1;
    
    END LOOP;
    
    SET @pStr = @newstr;
   
    END IF;

    RETURN @pStr;
  
  END;

END;


IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_ReduceViewName') THEN
  DROP FUNCTION SAHARA.FN_BVIEW_ReduceViewName;
END IF;

CREATE FUNCTION SAHARA.FN_BVIEW_ReduceViewName(IN @pStr LONG VARCHAR, IN @pMaxLength INTEGER)
  RETURNS  LONG VARCHAR
BEGIN 
  DECLARE @vRc varchar(2048);
  
  BEGIN
    IF @pStr IS NULL OR length(@pStr) <= @pMaxLength THEN
      RETURN @pStr;
    END IF;

    -- try removing underscores
    
    SET @vRc = replace(@pStr, '_', '');
    
    IF @vRc IS NULL OR length(@vRc) <= @pMaxLength THEN
      return @vRc;
    END IF;

    -- try removing vowels
    SET @vRc = replace(@vRc, 'A', '');
    SET @vRc = replace(@vRc, 'E', '');
    SET @vRc = replace(@vRc, 'I', '');
    SET @vRc = replace(@vRc, 'O', '');
    SET @vRc = replace(@vRc, 'U', '');



    IF @vRc IS NULL OR length(@vRc) <= @pMaxLength THEN
        RETURN @vRc;
    END IF;

    -- truncate the string
    SET @vRc = substr(@vRc, 1, @pMaxLength);
  
    RETURN @vRc;

  END;

END;


IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_PurifyDisplayName') THEN
   DROP FUNCTION SAHARA.FN_BVIEW_PurifyDisplayName;
END IF;

CREATE FUNCTION SAHARA.FN_BVIEW_PurifyDisplayName( 
  IN @pName LONG VARCHAR,
  IN @pDisplayName LONG VARCHAR,
  IN @pMaxLength INTEGER ) 

RETURNS LONG VARCHAR
BEGIN
  DECLARE @vDisplayName VARCHAR(30);
  DECLARE @vDisplayNameBase VARCHAR(30);
  DECLARE @vSuffix INTEGER;
  DECLARE @FUNC VARCHAR(255);

  BEGIN

    SET @FUNC = 'FN_BVIEW_PurifyDisplayName';
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');

    IF @pName is not null THEN
      
      CALL SAHARA.SP_BVIEW_ValidateName(@pName);
      
      -- make sure the view name contains only legal characters
      
      SET @vDisplayName = @pName
    
    ELSE
      -- create a view name, first replace spaces with underscores and further remove any invalid characters 
      
      SET @vDisplayName = SAHARA.FN_BVIEW_ReduceViewName(
        SAHARA.FN_BVIEW_RemoveInvalidCharacters(upper(replace(@pDisplayName,' ','_'))),@pMaxLength);
            
	  CALL SAHARA.SP_BVIEW_ValidateName(@vDisplayName);

	  SET @vDisplayNameBase = @vDisplayName;


      -- if the name is being used, add a numeric suffix and try again

      SET @vSuffix = 1;
      
      WHILE SAHARA.FN_BVIEW_NameUsed(@vDisplayName) = 1 
      
      LOOP
        
	-- make sure the name isn't being used
        
        SET @vSuffix = @vSuffix+1;
        
	IF @vSuffix > 99 THEN
         
	  CALL SP_Error_RaiseApplicationErrorDetail(
          @ECAColumnNameNotUnique,
          @FUNC,
          'unable to generate a unique name for business view: ' || @vDisplayName,
          @gLoadActiveSessionKey);

        END IF;

        SET @vDisplayName = @vDisplayNameBase || @vSuffix
      
      END LOOP

    END IF;
    RETURN @vDisplayName

  END;

END;

IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_Create1') Then
  DROP FUNCTION SAHARA.FN_BVIEW_Create1;
END IF;

CREATE FUNCTION SAHARA.FN_BVIEW_Create1
(
  IN @pDisplayName  LONG VARCHAR,
  IN @pName LONG VARCHAR DEFAULT NULL,
  IN @pNote LONG VARCHAR DEFAULT NULL
)
  RETURNS INTEGER
BEGIN
  DECLARE @vName VARCHAR(12);
  DECLARE @vNote LONG VARCHAR;
  DECLARE @vBViewKey INTEGER;
  DECLARE @FUNC VARCHAR(255);

  BEGIN
  
  ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();
  
    SET @FUNC ='Fn_Create';
    -- we want all or nothing from this call
    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');


    
    CALL SP_Config_RequireControl(@FUNC);	
    
    SET @vName = SAHARA.FN_BVIEW_PurifyDisplayName(@pName, trim(@pDisplayName), @gVIEW_NAME_LEN);

    SET @vNote = trim(@pNote);

    -- create the new tree root node

    SELECT isnull(max(BVIEW_NODE_KEY),0)+1 into @vBViewKey FROM  sahara.BVIEW_NODE;
    
    BEGIN 
        savepoint BViewAPICreate1SP;
    
	insert into BVIEW_NODE (BVIEW_NODE_KEY, PARENT_BVIEW_NODE_KEY, BVIEW_KEY, DISPLAY_NAME, NOTE)
        values (@vBViewKey, null, @vBViewKey, trim(@pDisplayName), @vNote);

    -- create the tree metadata
    
        insert into BVIEW_IMPL(BVIEW_KEY, VIEW_CREATION_REQUIRED, NAME)
        values(@vBViewKey, 'T', @vName);

    EXCEPTION WHEN OTHERS THEN 
        ROLLBACK TO savepoint BViewAPICreate1SP;
    END;

    RETURN @vBViewKey;
              
  END;

END;

IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_BVIEW_Delete1')THEN
      DROP PROCEDURE "SAHARA"."SP_BVIEW_Delete1";  
END IF; 

CREATE PROCEDURE "SAHARA"."SP_BVIEW_Delete1"
(
  IN @pViewKey INTEGER
)
BEGIN 
 
  DECLARE @vDummy INTEGER;
  DECLARE @FUNC VARCHAR(255);

  BEGIN

    SET @FUNC = 'SP_Delete1';
          
  ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();


    CALL SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');
    
    CALL SAHARA.SP_Config_RequireControl(@FUNC);

----- Validate DB view 
    
    select count(1) 
          into @vDummy
          from BVIEW_IMPL
          join BVIEW_NODE
            on BVIEW_IMPL.BVIEW_KEY = BVIEW_NODE.BVIEW_NODE_KEY
    where BVIEW_IMPL.BVIEW_KEY = @pViewKey;

    IF @vDummy > 0 THEN 
      
      delete from BVIEW_NODE where BVIEW_NODE_KEY = @pViewKey;

    ELSE

--- Raise Exception if View does not exists         
    
       call SP_Error_RaiseApplicationErrorDetail(
          @EBViewNotFound,
          @FUNC,
          'the specific business view was not found - key: ' || @pViewKey,
          @gLoadActiveSessionKey);
  
    END IF;

  END; 

END;



---------------------------------------------------------
IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_AddLevel') Then
  DROP FUNCTION SAHARA.FN_BVIEW_AddLevel;
END IF;

CREATE FUNCTION SAHARA.FN_BVIEW_AddLevel
(
  IN @pViewKey INTEGER,
  IN @pDisplayName LONG VARCHAR,
  IN @pName LONG VARCHAR DEFAULT NULL,
  IN @pNote LONG VARCHAR DEFAULT NULL
)
  RETURNS INTEGER

BEGIN
  
  DECLARE @FUNC VARCHAR(255);
  DECLARE @vLevel INTEGER;
  DECLARE @vName VARCHAR(30);
  DECLARE @vNote LONG VARCHAR;

  BEGIN
  --- Initialize connections variables
    call sahara.SP_saharaAPIInit();
    SET @FUNC = 'SP_BVIEW_AddLevel'; 
    SET @vLevel = null;

    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');

    CALL SP_Config_RequireControl(@FUNC);
    
    CALL  SAHARA.SP_BVIEW_BViewKeyTable();

    FOR ValidateBViewLOOP AS ValidateBViewCUR CURSOR FOR 
      select 
        1
      from 
        BVIEW_IMPL
      join BVIEW_NODE
      on BVIEW_IMPL.BVIEW_KEY = BVIEW_NODE.BVIEW_NODE_KEY
      where 
        BVIEW_IMPL.BVIEW_KEY = @pViewKey for update of BVIEW_KEY

    DO 

      SET @vName = SAHARA.FN_BVIEW_PurifyDisplayName(@pName, trim(@pDisplayName), @gLEVEL_NAME_LEN);

      SET @vNote = trim(@pNote);

    -- figure out the level number
  
        select 
	  max("LEVEL") + 1 into @vLevel
        from 
	  BVIEW_LEVEL
        where 
	  BVIEW_KEY = @pViewKey;

    -- aggregate functions don't throw NO_DATA_FOUND exception - instead,
    -- level will just be null

      IF @vLevel IS NULL THEN
        SET @vLevel = 1;
      END IF;

    -- create the level
      
      

      insert into BVIEW_LEVEL
        (BVIEW_KEY, "LEVEL", NAME, DISPLAY_NAME, NOTE)
      values
        (@pViewKey, @vLevel, @vName, trim(@pDisplayName), @vNote);

      

    END FOR;

    RETURN @vLevel;
  
  END;

END;

IF EXISTS (SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_BVIEW_DeleteLevel')THEN
      DROP PROCEDURE SAHARA.SP_BVIEW_DeleteLevel;
END IF; 

CREATE PROCEDURE SAHARA.SP_BVIEW_DeleteLevel
(
    IN @pViewKey  INTEGER,
    IN @pLevel INTEGER
)
BEGIN
  DECLARE @vDummy INTEGER;
  DECLARE @FUNC VARCHAR(255);
  
  BEGIN 

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_BVIEW_DeleteLevel';

    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');

    CALL SP_Config_RequireControl(@FUNC);

     select count(1) 
          into @vDummy
          from BVIEW_IMPL
          join BVIEW_NODE
            on BVIEW_IMPL.BVIEW_KEY = BVIEW_NODE.BVIEW_NODE_KEY
    where BVIEW_IMPL.BVIEW_KEY = @pViewKey;

    IF @vDummy > 0 THEN 

      CALL  SAHARA.SP_BVIEW_BViewKeyTable();
    
       -- trigger will remove the correspoding nodes for the levels
       -- and also any object assignments for the deleted nodes will be removed
      
      delete from BVIEW_LEVEL where "LEVEL" = @pLevel and BVIEW_KEY = @pViewKey;

    ELSE

--- Raise Exception if View does not exists         
    
       call SP_Error_RaiseApplicationErrorDetail(
          @EBViewNotFound,
          @FUNC,
          'the specific business view was not found - key: ' || @pViewKey,
          @gLoadActiveSessionKey);
  
    END IF;

  END;

END;

------------------------------------------------------------------------------------------------------------------------

IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_ValidateNode') Then
  DROP FUNCTION SAHARA.FN_BVIEW_ValidateNode;
END IF;


CREATE FUNCTION SAHARA.FN_BVIEW_ValidateNode
  (
  IN @FUNC VARCHAR(255),
  IN @pNodeKey INTEGER,
  IN @pAllowRoot BIT
  )
  RETURNS INTEGER
BEGIN
  DECLARE @vLevel INTEGER;
  DECLARE @vBViewKey INTEGER;
  
  BEGIN
      select BVIEW_KEY, "LEVEL"
      into @vBViewKey, @vLevel
      from BVIEW_NODE
     where BVIEW_NODE_KEY = @pNodeKey;

    IF @vBViewKey = 0 THEN

      call SP_Error_RaiseApplicationErrorDetail(
          @EBViewNodeNotFound,
          @FUNC,
          'the specified business view node was not found - node: ' || @pNodeKey,
	  @gLoadActiveSessionKey);

    END IF;

    IF @pAllowRoot = 0  AND @vLevel = NULL THEN
      
      call SP_Error_RaiseApplicationErrorDetail(
          @EBViewRootNodeUse,
          @FUNC,
          'the root node of the business view was specified where it ' ||
          'is not allowed - node: ' || pNodeKey,
          @gLoadActiveSessionKey);

    END IF;

    RETURN @vBViewKey;

  END;

END;



IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'FN_BVIEW_AddNode') Then
  DROP FUNCTION SAHARA.FN_BVIEW_AddNode;
END IF;

CREATE FUNCTION SAHARA.FN_BVIEW_AddNode
(
    IN @pParentKey INTEGER,
    IN @pDisplayName LONG VARCHAR,
    IN @pNote LONG VARCHAR DEFAULT NULL
)
    RETURNS INTEGER
BEGIN
  DECLARE @FUNC VARCHAR(255);
  DECLARE @vBViewKey INTEGER;
  DECLARE @vNote LONG VARCHAR;
  DECLARE @vNodeKey INTEGER;
  
  BEGIN 
    
    -- initialize global variables
    call sahara.SP_saharaAPIInit();

    SET @FUNC = 'FN_BVIEW_AddNode';

    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');

    CALL SP_Config_RequireControl(@FUNC);

    -- validate the parent node

    SET @vBViewKey = FN_BVIEW_ValidateNode(@FUNC, @pParentKey, 1);

    -- tbd - the display name must be not null, not empty and unique
    --       amongst the siblings

    SET @vNote = trim(@pNote);

    -- get a key for the new node
  
    insert into BVIEW_NODE
        (PARENT_BVIEW_NODE_KEY, DISPLAY_NAME, NOTE)
    values
        ( @pParentKey, @pDisplayName, @vNote);

    RETURN @@identity;
  
  END;

END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_BVIEW_AddNodes')THEN
      DROP PROCEDURE SAHARA.SP_BVIEW_AddNodes;
END IF; 

CREATE PROCEDURE SAHARA.SP_BVIEW_AddNodes
(
  IN  @pParentKey INTEGER,
  IN  @pDisplayName LONG VARCHAR
)
BEGIN
  DECLARE @FUNC VARCHAR(255);
  DECLARE @vBViewKey INTEGER;  
  DECLARE @vKey INTEGER;  
  DECLARE @vDisplayName VARCHAR(32767);
  DECLARE @vdelim VARCHAR(1);
  DECLARE @vCounter INTEGER;
  DECLARE @vPosition INTEGER;
  DECLARE @vnodekeys INTEGER;

  -----vnodekeys NodeKeyArray := NodeKeyArray();

  BEGIN
        -- initialize global variables
	call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_BVIEW_AddNodes';
    
    SET @vDisplayName = trim(@pDisplayName);
    SET @vdelim = ',';
    SET @vCounter =0;
    SET @vPosition =0;	

    BEGIN
    
      CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');
      CALL SP_Config_RequireControl(@FUNC);

-- validate the parent node
      
      SET @vBViewKey = SAHARA.FN_BVIEW_ValidateNode(@FUNC, @pParentKey,1);
	
-- determine first token of string  

      SET @vPosition = CHARINDEX(@vdelim,@vDisplayName); 

	-- while there are tokens left, loop 

      WHILE ( @vPosition <> 0) LOOP
         
       -- increment counter
       
        SET @vCounter = @vCounter + 1;
       
       -- create array element for token of string

        SET @vnodekeys = SAHARA.FN_BVIEW_AddNode(@pParentKey, trim(substr(@vDisplayName,1,@vPosition-1)));
	
      -- remove token from string 

        SET @vDisplayName = substr(@vDisplayName,@vPosition+1,length(@vDisplayName));
         
       -- determine next token 

        SET @vPosition = CHARINDEX( @vdelim,@vDisplayName);  

      END LOOP; 
	
	-- last token, add to array 

      IF @vPosition = 0 THEN        
	   
	   SET @vnodekeys = SAHARA.FN_BVIEW_AddNode(@pParentKey, @vDisplayName);

      END IF;
  
    END;  
  END;
END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_BVIEW_DeleteNode')THEN
      DROP PROCEDURE SAHARA.SP_BVIEW_DeleteNode;
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_DeleteNode
(
  IN @pNodeKey INTEGER
)
BEGIN 
  DECLARE @FUNC VARCHAR(255);
  DECLARE @vBViewKey INTEGER;
  
  BEGIN 
   -- initialize global variables
call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_BVIEW_DeleteNode';
    BEGIN
      CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');
      CALL SP_Config_RequireControl(@FUNC);
 
    -- validate the node
    
      SET @vBViewKey = SAHARA.FN_BVIEW_ValidateNode(@FUNC, @pNodeKey,1);

    -- trigger will remove the correspoding nodes for the levels
	-- and also any object assignments for the deleted nodes will be removed
      delete from BVIEW_NODE where BVIEW_NODE_KEY = @pNodeKey;
    END;
  END;
END;
 


IF EXISTS (SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_BVIEW_UpdateDisplayName')THEN
      DROP PROCEDURE SAHARA.SP_BVIEW_UpdateDisplayName;
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_UpdateDisplayName
  (
  IN @pNodeKey INTEGER,
  IN @pDisplayName LONG VARCHAR
  )
BEGIN

  DECLARE @FUNC VARCHAR(255);
  DECLARE @vBViewKey INTEGER;
  
  BEGIN 
    -- initialize global variables
	call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_BVIEW_UpdateDisplayName';
  
    BEGIN
    
      CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');
      CALL SP_Config_RequireControl(@FUNC);

	-- validate the node
      SET @vBViewKey = SAHARA.FN_BVIEW_ValidateNode(@FUNC, @pNodeKey, 1);

	-- update the display name

      update BVIEW_NODE
       set DISPLAY_NAME = @pDisplayName
       where BVIEW_NODE_KEY = @pNodeKey;

    END;
  
  END;

END;

IF EXISTS (SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_BVIEW_UpdateLevel')THEN
      DROP PROCEDURE SAHARA.SP_BVIEW_UpdateLevel;
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_UpdateLevel
  (
    IN @pViewKey INTEGER,
    IN @pLevel INTEGER,
    IN @pDisplayName LONG VARCHAR,
    IN @pName LONG VARCHAR,		
    IN @pNote LONG VARCHAR DEFAULT NULL
)
BEGIN 
  
  DECLARE @FUNC VARCHAR(255);
  DECLARE @vName VARCHAR(30);
  DECLARE @vNote LONG VARCHAR;

  BEGIN 
        -- initialize global variables
	call sahara.SP_saharaAPIInit();

    SET @FUNC = SP_BVIEW_UpdateLevel;
    
    BEGIN 
      
      CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');
      CALL SP_Config_RequireControl(@FUNC);

      CALL  SAHARA.SP_BVIEW_BViewKeyTable();

    -- validate the business view
      
      FOR ValidateBViewLOOP AS ValidateBViewCUR CURSOR FOR 
        select 
          1
        from 
           BVIEW_IMPL
           join BVIEW_NODE
           on BVIEW_IMPL.BVIEW_KEY = BVIEW_NODE.BVIEW_NODE_KEY
        where 
           BVIEW_IMPL.BVIEW_KEY = @pViewKey for update of BVIEW_KEY

      DO 
        SET @vName = SAHARA.FN_BVIEW_PurifyDisplayName(@pName, trim(@pDisplayName), @gVIEW_NAME_LEN);

        IF @pNote IS NULL THEN
		-- figure out the note
	      SELECT note INTO @vNote
	      FROM 
	        BVIEW_LEVEL
	      WHERE 
	        BVIEW_KEY = @pViewKey
            AND LEVEL=@pLevel;
	
	    ELSE
	
	      SET @vNote = trim(@pNote);
        
	    END IF;

    -- update the display name

          update 
            BVIEW_LEVEL
          set 
            NAME = @vName,DISPLAY_NAME = @pDisplayName,NOTE = @vNote 
	      WHERE 
	        BVIEW_KEY = @pViewKey
	        AND "LEVEL"=@pLevel;
      END FOR ;
    END;

  END;

END;



IF EXISTS (SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_BVIEW_UpdateNote')THEN
      DROP PROCEDURE SAHARA.SP_BVIEW_UpdateNote;
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_UpdateNote
  (
    IN @pNodeKey INTEGER,
    IN @pNote LONG VARCHAR
  )

BEGIN
  DECLARE @FUNC VARCHAR(255);
  DECLARE @vBViewKey INTEGER;
  DECLARE @vNote LONG VARCHAR;
  
  BEGIN 
        -- initialize global variables
	call sahara.SP_saharaAPIInit();

    SET @FUNC = 'SP_BVIEW_UpdateNote';
    
    BEGIN
    
      CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');

      CALL SP_Config_RequireControl(@FUNC);
    
    -- validate the node

      SET @vBViewKey = FN_BVIEW_ValidateNode(@FUNC, @pNodeKey, 1);


      SET @vNote = trim(@pNote);

    -- update the node

      update BVIEW_NODE
      set NOTE = @vNote
      where BVIEW_NODE_KEY = @pNodeKey;

    END;
  
  END;

END;

IF EXISTS (SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_BVIEW_BViewKeyTable')THEN
      DROP PROCEDURE SAHARA.SP_BVIEW_BViewKeyTable;
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_BViewKeyTable()
BEGIN
truncate table sahara.SAHARA.tmp_BViewKeyTable;
END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_CreateDimensionViews' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE sahara.SP_BVIEW_CreateDimensionViews;
END IF;
CREATE  PROCEDURE SAHARA.SP_BVIEW_CreateDimensionViews(in @pViewKey numeric(10))
begin
    -- which dimensions need a view
    select DIMENSION, ALIAS
       into #vDimensions
      from BVIEW_SUPPORTED_DIMENSION
     where BVIEW_KEY = @pViewKey;

    if not exists (select 1 from #vDimensions) then
        return;
    end if;
    -- else

    for DimensionCursor as DimensionLoop cursor for select dt.DIMENSION as dimension, dt.ALIAS as alias from #vDimensions as dt
    do
        call SP_BVIEW_CreateDimensionView(@pViewKey, dimension,alias);
    end for;
end;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_CreateDimensionView' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE sahara.SP_BVIEW_CreateDimensionView;
END IF;
CREATE  PROCEDURE sahara.SP_BVIEW_CreateDimensionView
(
    in @pViewKey numeric(10),
    in @DIMENSION varchar(26),
    in @ALIAS varchar(12)
    
)
begin
declare @FUNC varchar(64);
declare @vSessionKey numeric(10);
declare @vJoinTable varchar(32);
declare @vViewName varchar(30);
declare @vSupportViewName varchar(30);
declare @vSQL long varchar;
declare @Levels integer;

    set @FUNC = 'SP_BVIEW_CreateDimensionView';

    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');
    set @vSessionKey = @gConfigActiveSessionKey;



    set @vJoinTable = @ALIAS || '_BVIEW_NODE';
    
    
    

    -- get the business view related names
    select 'BV_' || @ALIAS || '_' || NAME, 'BVS_' || NAME
      into @vViewName, @vSupportViewName
      from BVIEW
     where BVIEW_KEY = @pViewKey;

    -- we're creating a view
       
    set @vSQL = 'CALL SAHARA.SP_DROP_OBJECT_SAHARA(''VIEW'','''|| @vViewName ||'''); '||' create view SAHARA.' || @vViewName || ' as ' ||
            'select ' || @vJoinTable || '.' || @DIMENSION || '_KEY';

    -- get the level information
    select "LEVEL", NAME, DISPLAY_NAME
       into #vLevels
      from BVIEW_LEVEL
     where BVIEW_KEY = @pViewKey
     order by "LEVEL";
    
    select count(*) into @Levels from #vLevels;

    if (@Levels > 0) then
    -- add a column for each level
    
        
        for LevelCursor as LevelLoop cursor for select NAME, row_number() over (order by "LEVEL") as rownum from #vLevels  order by "LEVEL" 
        do
            set @vSQL = @vSQL || ', ' || @vSupportViewName || '."' || NAME || '"'
                         || ', ' || @vSupportViewName || '.BVIEW_NODE_KEY_' || rownum;
        end for;


		set @vSQL = @vSQL || ' ' ||
			'from SAHARA.' || @vJoinTable || ' ' ||
			'join SAHARA.' || @vSupportViewName || ' ' ||
			'on ' || @vJoinTable || '.BVIEW_NODE_KEY = ' || @vSupportViewName || '.BVIEW_NODE_KEY';

		-- create the view
		call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
		execute immediate @vSQL;

		-- add required grants
		set @vSQL = 'grant select on SAHARA.' || @vViewName || ' to SAHARA_APP';
		call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
		execute immediate @vSQL;

		set @vSQL = 'grant select on SAHARA.' || @vViewName || ' to SAHARA_REPORTS';
		call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
		execute immediate @vSQL;
    end if;
end;



IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_Recurse' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE sahara.SP_BVIEW_Recurse;
END IF;
CREATE PROCEDURE sahara.SP_BVIEW_Recurse
    (
        in @pNodeKey numeric(10)
  --,
    --    in @vNextNum integer
    )
BEGIN
        -- find the child nodes
        --message 'In Recurse' to client;
        select  rowid(BVIEW_NODE) as rowid, BVIEW_NODE_KEY
           into #vNodes
          from BVIEW_NODE
         where PARENT_BVIEW_NODE_KEY = @pNodeKey
         order by DISPLAY_NAME;

        -- if nothing was returned, there's nothing more to do
        if not exists (select 1 from #vNodes)  then
            return;
        end if;

        -- tbd - assert nodes is not null and nodes.count > 0

        -- set the left and right numbers for nodes of the trees
       for BVNodesCursor as BVNodesLoop cursor for select rowid,BVIEW_NODE_KEY  as nodekey from #vNodes
        do
            -- set the left number
         --  MESSAGE 'Assigning left number' || @vNextNum || 'key:' || nodekey to client;
            update BVIEW_NODE
               set "LEFT" = @vNextNum
             where rowid(BVIEW_NODE) = rowid;
            set @vNextNum = @vNextNum + 1;

            -- set the child numbers
             
            call SP_BVIEW_Recurse(nodekey);

       --      MESSAGE 'Assigning right number '||@vNextNum || 'key:' || nodekey to client;
            -- set the right number
            update BVIEW_NODE
               set "RIGHT" = @vNextNum
             where rowid(BVIEW_NODE) = rowid;
            set @vNextNum = @vNextNum + 1;
        end for;
END;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_AssignLeftRightNumbers' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE sahara.SP_BVIEW_AssignLeftRightNumbers;
END IF;
CREATE PROCEDURE sahara.SP_BVIEW_AssignLeftRightNumbers()
BEGIN

if  ( VAREXISTS('@vNextNum') = 0 ) then
        create VARIABLE @vNextNum integer;
end if;

    --
    -- a helper to implement the recursion after finding the root nodes
    --
--    message ' into the function1' to client;
    set @vNextNum = 1;
    -- tbd - lock table?

    -- get the node that are roots
    select rowid(BVIEW_NODE) as rowid, BVIEW_NODE_KEY 
       into #rootvNodes
      from BVIEW_NODE
     where PARENT_BVIEW_NODE_KEY is null
     order by DISPLAY_NAME;

    -- if nothing was returned, there's nothing more to do
    if not exists (select 1 from  #rootvNodes ) then
        return;
    end if;
  -- message ' into the function' to client;

    -- tbd - assert nodes is not null and nodes.count > 0

    -- set the left and right numbers for nodes of the trees
    --for i in vNodes.first..vNodes.last
    for BVRootNodeCursor as BVrootNodeLoop cursor for select rowid,BVIEW_NODE_KEY as rootkey from #rootvNodes
    do
        -- set the left number
        update BVIEW_NODE
           set "LEFT" = @vNextNum
         where rowid(BVIEW_NODE) = rowid;
        set @vNextNum = @vNextNum + 1;

        -- set the child numbers
        call SP_BVIEW_Recurse(rootkey);

        -- set the right number
        update BVIEW_NODE
           set "RIGHT" = @vNextNum
         where rowid(BVIEW_NODE) = rowid;
        set @vNextNum = @vNextNum + 1;

    end for;
if  ( VAREXISTS('@vNextNum') = 0 ) then
   drop VARIABLE @vNextNum;
end if;
end;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_PublishBV' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE sahara.SP_BVIEW_PublishBV;
END IF;
CREATE  PROCEDURE sahara.SP_BVIEW_PublishBV()
BEGIN
declare @FUNC varchar(64);
declare @vNumberRequired char(1);

        -- initialize global variables
	call sahara.SP_saharaAPIInit();

    set @FUNC = 'SP_BVIEW_PublishBV';
    call SP_Log_LogInfo(@gConfigActiveSessionKey,@FUNC, 'begin Publish');
--   message 'before assignlefRightNumbers1' to client;
   CALL SP_Config_RequireControl(@FUNC);
    

    -- figure out what work needs doing
    select NUMBER_REQUIRED
      into @vNumberRequired
      from BVIEW_STATUS;
     
  --  message @vNumberRequired to client;
    -- re-number the business view nodes
    --if (fn_CharToBool(@vNumberRequired) is not null)  then
     if (@vNumberRequired = 'T' )  then
        begin
            savepoint BViewPublishSP1;
    --       message 'before assignlefRightNumbers' to client;
            call SP_BVIEW_AssignLeftRightNumbers();
      --                 message 'After assignlefRightNumbers' to client;
            update BVIEW_STATUS
               set NUMBER_REQUIRED = 'F';

            commit;
        exception
            when others then
                rollback to savepoint BViewPublishSP1;
             CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewError,
            @FUNC,
            'Error while re-numbering the business view',
            @gConfigActiveSessionKey
        );
;
        end;
    end if;

    -- (re-)create the views that need it
    select BVIEW_KEY
      into #vBViewKeys
      from BVIEW
     where VIEW_CREATION_REQUIRED = 'T'
     order by NAME;

     message @vNumberRequired to client;
        for BViewKeysCursor as BViewKeysLoop cursor for select BVIEW_KEY as vBViewKeys from BVIEW
        do  
        
         message 'Create Support View' to client;
            -- (re-)create a supporting view for the dimension data
            call SP_BVIEW_CreateSupportView(vBViewKeys);

         message 'Create dimension View' to client;
            -- (re-)create a dimension view for each of the
            -- supported dimensions
            call SP_BVIEW_CreateDimensionViews(vBViewKeys);

            -- record that the views are current
            update BVIEW_IMPL
               set VIEW_CREATION_REQUIRED = 'F'
             where BVIEW_KEY = vBViewKeys;

            commit;
        end for;
    

    -- tbd - removed obsolete views
    call SP_Log_LogInfo(@gConfigActiveSessionKey,@FUNC, 'complete');
end;



IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_CreateSupportView' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_CreateSupportView;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_CreateSupportView(in @pViewKey numeric(10))
BEGIN
declare @FUNC varchar(64);
declare @vSessionKey numeric(10);
declare @vViewName varchar(30);
declare @vSQL long varchar;

 set @FUNC = 'SP_BVIEW_CreateSupportView';

    -- get the business view related names
    set @vSessionKey = @gConfigActiveSessionKey;

    select 'BVS_' || NAME
      into @vViewName
      from BVIEW
     where BVIEW_KEY = @pViewKey;

    -- get the level information
    select "LEVEL", "NAME", DISPLAY_NAME
       into #vLevels
      from BVIEW_LEVEL
     where BVIEW_KEY = @pViewKey
     order by "LEVEL";

    -- tbd - check that rows were returned

    -- we're creating a view
    set @vSQL = 'CALL SAHARA.SP_DROP_OBJECT_SAHARA(''VIEW'','''|| @vViewName ||'''); '||'create  view SAHARA.' || @vViewName || ' as';

    message '  before the main loop' to client;

    -- because objects can be attached to nodes at any level of the tree
    -- the query consists of a union of a query for each level
    if exists (select 1 from #vLevels) then
        message 'Inside the if statement' to client;
		for CursorcurrentLevel as  CurrsorLevelloop cursor for select row_number() over (order by "LEVEL") as currentlevel, "LEVEL", "NAME", DISPLAY_NAME from #vLevels
		do
			-- if this isn't the first, select, add the union all separator
			if (currentLevel > 1) then
				set @vSQL = @vSQL || ' union all';
			end if;

			-- start the select with the key
			set @vSQL = @vSQL || ' select L' || currentLevel || '.BVIEW_NODE_KEY';

			-- add the dimension columns
			--for l in vLevels.first..vLevels.last

          message 'Before the dimension level' to client;
            for CursorDimLevel as  CurrsorDimloop cursor for select row_number() over (order by "LEVEL") as l,"NAME" as vname from #vLevels
			do
				if (l <= currentLevel) then
					set @vSQL = @vSQL || ', L' || l || '.DISPLAY_NAME as "' || vname || '"'
								 || ', L' || l || '.BVIEW_NODE_KEY as BVIEW_NODE_KEY_' || l;
				else
					set @vSQL = @vSQL || ', null as "' || vname || '"'
								 || ', null as BVIEW_NODE_KEY_' || l;
				end if;
			end for;

			-- add the from and join clauses
           message 'Before the label' to client;
           lbl: 
            for Cursor1 as  Currsor1loop cursor for select row_number() over (order by "LEVEL") as ll from #vLevels
			do
				 if ( ll > currentLevel) then 
                    leave lbl; 
                  end if;

				if (ll = 1) then
					set @vSQL = @vSQL || ' from SAHARA.BVIEW_NODE L' || ll;
				else
					set @vSQL = @vSQL || ' join SAHARA.BVIEW_NODE L' || ll ||
						' on L' || ll || '."LEVEL" = ' || ll ||
						' and L' || ll || '."LEFT" ' || 'between' ||
						' L' || (ll - 1) || '."LEFT" and L' || (ll - 1) || '."RIGHT"';
				end if;
			end for lbl;

			-- the where clause
			set @vSQL = @vSQL || ' where L1."LEVEL" = 1 and L1.BVIEW_KEY = ' || @pViewKey;
		end for;

		-- create the view
		call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
		execute immediate @vSQL;

		-- add required grants
		set @vSQL = 'grant select on SAHARA.' || @vViewName || ' to SAHARA_REPORTS';
		call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
		execute immediate @vSQL;

		set @vSQL = 'grant select on SAHARA.' || @vViewName || ' to SAHARA_REPORTS';
		call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
		execute immediate @vSQL;
	end if;
end;

IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'SP_BVIEW_ValidateDimension') THEN
  DROP PROCEDURE SAHARA.SP_BVIEW_ValidateDimension;
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_ValidateDimension(IN @FUNC VARCHAR(255), IN @pDimension VARCHAR(26))
BEGIN
  DECLARE @vDummy UNSIGNED INTEGER;
  BEGIN
    SET @vDummy = 0 ;
    
    select 
      1 into @vDummy
    from 
      BVIEW_DIMENSION
    where 
      DIMENSION = @pDimension;

    IF @vDummy = 0 THEN 
      
      CALL SP_Error_RaiseApplicationErrorDetail
      (
        @EBViewDimensionNotFound,
        'SP_BVIEW_ValidateDimension',
        'the specifed business view dimension was not found - dimension: ' || @pDimension,
	    @gLoadActiveSessionKey
      );

    END IF;
  END;
END;


IF EXISTS (select 1 from sys.sysprocedure where proc_name = 'SP_BVIEW_AddDimensionSupport') THEN
  DROP PROCEDURE SAHARA.SP_BVIEW_AddDimensionSupport;
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_AddDimensionSupport
(
  IN @pViewKey NUMERIC(10),
  IN @pDimension VARCHAR(26)
)

BEGIN
  DECLARE @FUNC VARCHAR(255);
  DECLARE @cnt INT;
  
  BEGIN
   -- initialize global variables
    call sahara.SP_saharaAPIInit();
    SET @FUNC = 'SP_BVIEW_AddDimensionSupport' ;
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');
    CALL SP_Config_RequireControl(@FUNC);;

    FOR ValidateBViewLOOP AS ValidateBViewCUR CURSOR FOR 
      select 
        1
      from 
        BVIEW_IMPL
        join BVIEW_NODE
        on BVIEW_IMPL.BVIEW_KEY = BVIEW_NODE.BVIEW_NODE_KEY
      where 
        BVIEW_IMPL.BVIEW_KEY = @pViewKey for update of BVIEW_KEY

    DO 

      CALL SAHARA.SP_BVIEW_ValidateDimension(@FUNC, @pDimension)
    
    END FOR ;
    
    SET @cnt  = 0;

    SELECT 
      1 INTO @cnt 
    FROM 
      SAHARA.BVIEW_DIMENSION_BVIEW
    WHERE 
      DIMENSION = @pDimension AND 
      BVIEW_KEY = @pViewKey;
    
    IF @cnt = 0 THEN  

      DELETE FROM SAHARA.tmp_BViewDimensionKeyTable ; 
        
      insert into SAHARA.BVIEW_DIMENSION_BVIEW(DIMENSION, BVIEW_KEY)
      values(@pDimension, @pViewKey);

    END IF;

    -- tbd - there will be database views to create, etc.
  END;

END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'FN_BVIEW_ValidateDimension' and user_name(creator) = 'SAHARA' )
THEN
        DROP FUNCTION SAHARA.FN_BVIEW_ValidateDimension;
END IF;
CREATE FUNCTION SAHARA.FN_BVIEW_ValidateDimension(in @FUNC varchar(64), in @pDimension varchar(26))
    returns varchar(12)
begin
 declare @vAlias varchar(12);
     set @vAlias = null;

    select ALIAS
      into @vAlias
      from BVIEW_DIMENSION
     where DIMENSION = @pDimension;
     message @vAlias to client;
    if (@vAlias is null)  then

        CALL   SP_Error_RaiseApplicationErrorDetail
            (
                @EBViewDimensionNotFound,
                @FUNC,
                'the specifed business view dimension was not found - dimension: ' || @pDimension,
                @gConfigActiveSessionKey
        );            
    end if;

    return @vAlias;       
end;



IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_AssociateDimensionRow' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_AssociateDimensionRow;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_AssociateDimensionRow(in @pNodeKey numeric(10),
                                                in @pDimRowKey numeric(10),
                                                in @pDimension varchar(26),
                                                in @punique bit default 1)
                                                --in @pAssState integer default 0)
BEGIN
declare @FUNC varchar(64);
declare @vSessionKey numeric(10);
declare @vAlias varchar(12);
declare @pBViewKey numeric(10);
declare @vSQL long varchar;
declare @ASSOC_DIM_ROW_DELETE_DUPS long varchar;
declare @ASSOC_DIM_ROW_UNIQUE long varchar;
declare @ASSOC_DIM_ROW long varchar;

    
    set @FUNC = 'SP_BVIEW_AssociateDimensionRow';

    set @vSessionKey = @gConfigActiveSessionKey;

    message 'After the Validate Dimension' to client;
     set @ASSOC_DIM_ROW_DELETE_DUPS =
    'delete from SAHARA.$ALIAS$_BVIEW_NODE from ' ||
     ' (select $DIM$_KEY as dimkey, BVIEW_KEY as ibvkey ' ||
                                        'from SAHARA.$ALIAS$_BVIEW_NODE ' ||
                                       'where $DIM$_KEY = $pDimRowKey$ '||
                                         ' and BVIEW_KEY = $pBViewKey$  '||
                                    ' group by $DIM$_KEY, BVIEW_KEY ' ||
                                      'having count(*) > 1) as bvn where $DIM$_KEY = bvn.dimkey and BVIEW_KEY = bvn.ibvkey';

 set @ASSOC_DIM_ROW_UNIQUE =
    'merge into SAHARA.$ALIAS$_BVIEW_NODE dest ' ||
    'using (select $pNodeKey$  as BVIEW_NODE_KEY, ' ||
                  ' $pDimRowKey$  as $DIM$_KEY, ' ||
                  ' $pBViewKey$  as BVIEW_KEY from dummy) src ' ||
       'on (dest.$DIM$_KEY = src.$DIM$_KEY and ' ||
           'dest.BVIEW_KEY = src.BVIEW_KEY) ' ||
    'when matched then ' ||
        'update set dest.BVIEW_NODE_KEY = src.BVIEW_NODE_KEY ' ||
    'when not matched then ' ||
        'insert (BVIEW_NODE_KEY, $DIM$_KEY, BVIEW_KEY) ' ||
        'values (src.BVIEW_NODE_KEY, src.$DIM$_KEY, src.BVIEW_KEY)';
set @ASSOC_DIM_ROW=
    'merge into SAHARA.$ALIAS$_BVIEW_NODE dest ' ||
    'using (select  $pNodeKey$  as BVIEW_NODE_KEY, ' ||
                   ' $pDimRowKey$  as $DIM$_KEY, ' ||
                  ' $pBViewKey$   BVIEW_KEY from dummy) src ' ||
       'on (dest.BVIEW_NODE_KEY = src.BVIEW_NODE_KEY and
            dest.$DIM$_KEY = src.$DIM$_KEY and ' ||
           'dest.BVIEW_KEY = src.BVIEW_KEY) ' ||
    'when not matched then ' ||
        'insert (BVIEW_NODE_KEY, $DIM$_KEY, BVIEW_KEY) ' ||
        'values (src.BVIEW_NODE_KEY, src.$DIM$_KEY, src.BVIEW_KEY)';

    call SP_Log_LogDebug(@vSessionKey, @FUNC, 'entry');
    set @vAlias = FN_BVIEW_ValidateDimension(@FUNC, @pDimension);

    --message 'After the Validate Dimension' to client;
        -- we want all or nothing from this call
    savepoint BViewAPIAssDimRowSP;
    
    

    CALL SP_Config_RequireControl(@FUNC);
    --CONFIGURATION_CONTROL_API.RequireControl(FUNC);

--message 'Before the validate node' to client;
    -- validate the node and get the business view key
   set  @pBViewKey = FN_BVIEW_ValidateNode(@FUNC, @pNodeKey, 0);

   message 'after the validate node' to client;
    -- tbd - validate that this viewsupports this dimension

    -- Assemble one of two merge statements that do the work depending on
    -- whether we want to keep dimension rows unique per business view or
    -- not.  We'll count on Oracle's cache of statements to be smart
    -- about dealing with repeated calls.
    if (@pUnique = 1)  then
        -- Remove associations for an dimension row to multiple nodes.  If
        -- we didn't do this, moving mutiple associations to one, would fail
        -- because each association row must be unique.
        set @vSQL = replace(@ASSOC_DIM_ROW_DELETE_DUPS, '$DIM$', @pDimension);
        set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
        set @vSQL = replace(@vSQL, '$pDimRowKey$', @pDimRowKey);
        set @vSQL = replace(@vSQL, '$pBViewKey$', @pBViewKey);

        call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
        execute immediate @vSQL ;

        set @vSQL = replace(@ASSOC_DIM_ROW_UNIQUE, '$DIM$', @pDimension);
        set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
       
    else
        set @vSQL = replace(@ASSOC_DIM_ROW, '$DIM$', @pDimension);
        set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
    end if;
        set @vSQL = replace(@vSQL, '$pDimRowKey$', @pDimRowKey);
         set @vSQL = replace(@vSQL, '$pBViewKey$', @pBViewKey);        
         set @vSQL = replace(@vSQL, '$pNodeKey$', @pNodeKey);          

    call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL ;

exception
    when others then
        rollback to savepoint BViewAPIAssDimRowsSP;
        CALL   SP_Error_RaiseApplicationErrorDetail
            (
                 @EBViewError,
                @FUNC,
                'Associating the dimension rows failed: ' || @pDimension,
                @gConfigActiveSessionKey
        );            
        
end;

-------------------------------------------------------------------------------

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_AssociateDimensionRows' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_AssociateDimensionRows;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_AssociateDimensionRows(
                                                in @pNodeKey numeric(10),
                                                in @pDimension varchar(26),
                                                in @punique bit default 1,
                                                in @pAssStatus integer default 0,
                                                in @pRuleKey numeric(10) default null  
)
BEGIN
declare @FUNC varchar(64);
declare @vSessionKey numeric(10);
declare @vAlias varchar(12);
declare @pBViewKey numeric(10);
declare @pDimRowKey numeric(10);
declare @vSQL long varchar;
declare @vSQLOrignal long varchar;
declare @ASSOC_DIM_ROW_DELETE_DUPS long varchar;
declare @ASSOC_DIM_ROW_UNIQUE long varchar;
declare @ASSOC_DIM_ROW long varchar;

    
    set @FUNC = 'SP_BVIEW_AssociateDimensionRows';

    set @vSessionKey = @gConfigActiveSessionKey;

     set @ASSOC_DIM_ROW_DELETE_DUPS =
    'delete from SAHARA.$ALIAS$_BVIEW_NODE from ' ||
     ' (select $DIM$_KEY as dimkey, BVIEW_KEY as ibvkey ' ||
                                        'from SAHARA.$ALIAS$_BVIEW_NODE ' ||
                                       'where $DIM$_KEY = $pDimRowKey$ '||
                                         ' and BVIEW_KEY = $pBViewKey$  '||
                                    ' group by $DIM$_KEY, BVIEW_KEY ' ||
                                      'having count(*) > 1) as bvn where $DIM$_KEY = bvn.dimkey and BVIEW_KEY = bvn.ibvkey';

 set @ASSOC_DIM_ROW_UNIQUE =
    'merge into SAHARA.$ALIAS$_BVIEW_NODE dest ' ||
    'using (select $pNodeKey$  as BVIEW_NODE_KEY, ' ||
                  ' $pDimRowKey$  as $DIM$_KEY, ' ||                   
                  ' $pBViewKey$  as BVIEW_KEY from dummy) src ' ||
       'on (dest.$DIM$_KEY = src.$DIM$_KEY and ' ||
           'dest.BVIEW_KEY = src.BVIEW_KEY) ' ||
    'when matched then ' ||
        'update set dest.BVIEW_NODE_KEY = src.BVIEW_NODE_KEY ' ||
    'when not matched then ' ||
        'insert (BVIEW_NODE_KEY, $DIM$_KEY, BVIEW_KEY,STATUS,BVIEW_RULES_KEY) ' ||
        'values (src.BVIEW_NODE_KEY, src.$DIM$_KEY, src.BVIEW_KEY,@pAssStatus,@pRuleKey)';
set @ASSOC_DIM_ROW=
    'merge into SAHARA.$ALIAS$_BVIEW_NODE dest ' ||
    'using (select  $pNodeKey$  as BVIEW_NODE_KEY, ' ||
                   ' $pDimRowKey$  as $DIM$_KEY, ' ||                  
                  ' $pBViewKey$   BVIEW_KEY from dummy) src ' ||
       'on (dest.BVIEW_NODE_KEY = src.BVIEW_NODE_KEY and
            dest.$DIM$_KEY = src.$DIM$_KEY and ' ||
           'dest.BVIEW_KEY = src.BVIEW_KEY) ' ||
    'when not matched then ' ||
        'insert (BVIEW_NODE_KEY, $DIM$_KEY, BVIEW_KEY,STATUS,BVIEW_RULES_KEY) ' ||
        'values (src.BVIEW_NODE_KEY, src.$DIM$_KEY, src.BVIEW_KEY,@pAssStatus,@pRuleKey)';

    call SP_Log_LogDebug(@vSessionKey, @FUNC, 'entry');
    set @vAlias = FN_BVIEW_ValidateDimension(@FUNC, @pDimension);
     
    -- we want all or nothing from this call
    savepoint BViewAPIAssDimRowsSP;
              


    CALL SP_Config_RequireControl(@FUNC);
    --CONFIGURATION_CONTROL_API.RequireControl(FUNC);

    -- validate the node and get the business view key
   set  @pBViewKey = FN_BVIEW_ValidateNode(@FUNC, @pNodeKey, 0);

    -- validate the node and get the business view key
    -- tbd - validate that this view supports this dimension

    -- if no dimension rows are provided, end of show
    if not exists (select 1 from DimensionRowKeyTable)    then
        -- tbd - error?
        return;
    end if;

    -- Assemble one of two merge statements that do the work depending on
    -- whether we want to keep dimension rows unique per business view or
    -- not.  We'll count on Oracle's cache of statements to be smart
    -- about dealing with repeated calls.
    if (@pUnique = 1) then
        -- Remove associations for an dimension row to multiple nodes.  If
        -- we didn't do this, moving mutiple associations to one, would fail
        -- because each association row must be unique.
        set @vSQL = @ASSOC_DIM_ROW_DELETE_DUPS;

                 set @vSQLOrignal =  @vSQL;
                call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
        for dimRowCursor as dimrowloop cursor for select keys as dimkey from DimensionRowKeyTable 
        do
        --forall i in pDimRowKeys.first..pDimRowKeys.last
            --execute immediate vSQL using pDimRowKeys(i), vBViewKey;
              set @vSQL = replace(@vSQL, '$DIM$', @pDimension);
        set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
              set @pDimRowKey = dimkey;              
              set @vSQL = replace(@vSQL, '$pDimRowKey$', @pDimRowKey);
              set @vSQL = replace(@vSQL, '$pBViewKey$', @pBViewKey);
        --message 'dimension key:'||@pDimRowKey || 'Business view key:'||@pBViewKey   to client;
          --    message @vSQL to client;
             execute immediate @vSQL ;
             set @vSQL = @vSQLOrignal;
        end for;
       set @vSQL = @ASSOC_DIM_ROW_UNIQUE;
        
    else
        set @vSQL = @ASSOC_DIM_ROW;        
    end if;
         set @vSQLOrignal =  @vSQL;

    call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
    --forall i in pDimRowKeys.first..pDimRowKeys.last
      for dimRowCursor1 as dimrowloop1 cursor for select keys as dimkey1 from DimensionRowKeyTable 
        do
        --execute immediate vSQL using pNodeKey, pDimRowKeys(i), vBViewKey;
         set @vSQL = replace(@vSQL, '$DIM$', @pDimension);
        set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
         set @pDimRowKey = dimkey1;      
         message 'dimension key:'||@pDimRowKey to client;
         set @vSQL = replace(@vSQL, '$pDimRowKey$', @pDimRowKey);
         set @vSQL = replace(@vSQL, '$pBViewKey$', @pBViewKey);        
         set @vSQL = replace(@vSQL, '$pNodeKey$', @pNodeKey);          
         message 'dimension SQL:'||@vSQL to client;
         execute immediate @vSQL ;
         set @vSQL = @vSQLOrignal;
        end for;
exception
when others then
        rollback to savepoint BViewAPIAssDimRowsSP;
      CALL   SP_Error_RaiseApplicationErrorDetail
        (
           @EBViewError,
        @FUNC,
      'Trouble Associating  the dimension rows: ' || @pDimension,
               @gConfigActiveSessionKey
        ); ;
end;

-------------------------------------------------------------------------------

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_AssociateDimensionRows_string' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_AssociateDimensionRows_string;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_AssociateDimensionRows_string(
                                                in @pNodeKey numeric(10),
                                                in @pDimKeys long varchar,
                                                in @pDimension varchar(26),
                                                in @punique bit default 1
                                                ,in @pAssStatus integer default 0
                                                , in @pRuleKey numeric(10) default null  
)
BEGIN

declare @FUNC varchar(64);
declare @vSessionKey numeric(10);
declare @vDimKeys long varchar;
declare @vAlias varchar(12);
declare @vBViewKey numeric(10);
declare @vSQL long varchar;
declare @vdelim varchar(1);
declare @vCounter integer;
declare @vPosition integer;
declare @keysnum numeric(10);
declare @string long varchar;
    
    -- initialize global variables

    call sahara.SP_saharaAPIInit();

    set @FUNC = 'SP_BVIEW_AssociateDimensionRows_string';

    set @vSessionKey = @gConfigActiveSessionKey;

    
	

	set @vDimKeys = trim(@pDimKeys);
	set @vdelim = ',';
	set @vCounter =0;
    set @vPosition =0;	

    

    call SP_Log_LogDebug(@vSessionKey, @FUNC, 'entry');

	-- 
	-- construct DimensionRowKeyTable from the GUI provided comma separated dimension keys
	-- 
	-- determine first dimension key token of string  
     




    set @vPosition = charindex(@vdelim,@vDimKeys); 
    delete from DimensionRowKeyTable;
	-- while there are dim key tokens left, loop 
    WHILE ( @vPosition != 0) LOOP
         
       -- increment counter 
       set @vCounter = @vCounter + 1;
      set @string = TRIM(substring(@vDimKeys,1,@vPosition-1));
     set @keysnum =  CAST(@string as numeric(10));
       -- create array element for token of string
      --message @keysnum to client;
       insert into DimensionRowKeyTable(keys) values(@keysnum);
	   --vDimRowKeys.extend;
       --vDimRowKeys(vDimRowKeys.last) := to_number(trim(substr(vDimKeys,1,vPosition-1)));
       
       -- remove token from string 
       set @vDimKeys = substring(@vDimKeys,@vPosition+1);
       
       -- determine next token 
       set @vPosition = charindex(@vdelim,@vDimKeys);      

  
	END LOOP; 
	
	-- last token, add to array 
    IF (@vPosition = 0) THEN        
     
     set @string = TRIM(@vDimKeys);
     set @keysnum =  CAST(@string as numeric(10));
     
      message @keysnum to client;
	 insert into DimensionRowKeyTable(keys) Values(@keysnum);  
	   --vDimRowKeys(vDimRowKeys.last) := to_number(trim(vDimKeys));
    END IF;
	
	-- if no dimension rows are provided, end of show
    if not exists (select 1 from DimensionRowKeyTable) then
        -- tbd - error?
        return;
    end if;

	-- call multiassignment API
	call SP_BVIEW_AssociateDimensionRows(@pNodeKey,@pDimension,@pUnique,@pAssStatus,@pRuleKey);
--,@pAssState);

end;

-------------------------------------------------------------------------------

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_RemoveAssignment' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_RemoveAssignment;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_RemoveAssignment(
                                                in @pViewKey numeric(10),
                                                in @pDimRowKey numeric,
                                                in @pDimension varchar(26) 
)

BEGIN
declare @FUNC varchar(64);
declare @REM_ASSOC_DIM_ROW_DELETE long varchar;
declare @vSQL long varchar;
declare @vRowCount integer;
declare @vSessionKey numeric(10);
declare @vAlias varchar(12);
declare @vBViewKey numeric(10);

    set @FUNC =  'SP_BVIEW_RemoveAssignment(single)';

	set @REM_ASSOC_DIM_ROW_DELETE = 'delete from SAHARA.$ALIAS$_BVIEW_NODE ' || 
									' where bview_key = ' ||@pViewKey||' and $DIM$_key= ' || @pDimRowKey ;

    set @vSessionKey = @gConfigActiveSessionKey;  
	    

    call SP_Log_LogDebug(@vSessionKey, @FUNC, 'entry');

    set @vAlias = FN_BVIEW_ValidateDimension(@FUNC, @pDimension);
    
	
	-- we want all or nothing from this call
    savepoint BViewAPIRmDimRowSP;

    CALL SP_Config_RequireControl(@FUNC);
    


    -- delete assignment data from dimension business view node table 
	set @vSQL = replace(@REM_ASSOC_DIM_ROW_DELETE, '$DIM$', @pDimension);
	set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
	call SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL;

	set @vRowCount = @@rowcount;
    if (@vRowCount > 0) then
      CALL SP_Log_LogInfo(@vSessionKey, @FUNC,
                        'business view node key #' || @pViewKey || ' and dimension row key #' || @pDimRowKey || ', assignment data rows dropped for dimension ' ||
                       @pDimension || ': ' || @vRowCount);
    end if;

exception
    when others then
        rollback to savepoint BViewAPIRmDimRowSP;
        CALL   SP_Error_RaiseApplicationErrorDetail
            (
                 @EBViewError,
                @FUNC,
                'Remove assigment failed- dimension: ' || @pDimension,
                @gConfigActiveSessionKey
        ); ;
end;

-------------------------------------------------------------------------------

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_RemoveAssignments' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_RemoveAssignments;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_RemoveAssignments(
                                                in @pViewKey numeric(10),
                                                in @pDimKeys long varchar,
                                                in @pDimension varchar(26) 
)
begin
	-- we want all or nothing from this call

declare @FUNC varchar(64);
declare @REM_ASSOC_DIM_ROWS_DELETE long varchar;
declare @vSQL long varchar;
declare @vRowCount integer;
declare @vSessionKey numeric(10);
declare @vAlias varchar(12);
declare @vBViewKey numeric(10);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

    set @FUNC =  'SP_BVIEW_RemoveAssignment(multiple)';

	set @REM_ASSOC_DIM_ROWS_DELETE = 'delete from SAHARA.$ALIAS$_BVIEW_NODE ' || 
									' where bview_key = ' ||@pViewKey||' and $DIM$_key in ($TBRMKEYS$)';

    set @vSessionKey = @gConfigActiveSessionKey;  
	set @vAlias = FN_BVIEW_ValidateDimension(@FUNC, @pDimension);
    
    
    call SP_Log_LogDebug(@vSessionKey, @FUNC, 'entry');

    savepoint BViewAPIRmDimRowsSP;
       CALL SP_Config_RequireControl(@FUNC); 
       

    



    -- delete assignment data from dimension business view node table 
	set @vSQL = replace(@REM_ASSOC_DIM_ROWS_DELETE, '$DIM$', @pDimension);
	set @vSQL = replace(@vSQL, '$ALIAS$', @vAlias);
	set @vSQL = replace(@vSQL, '$TBRMKEYS$', @pDimKeys);
	
    CALL SP_Log_LogDebug(@vSessionKey, @FUNC, 'SQL: ' || @vSQL);
	CALL SP_Log_LogDebug(@vSessionKey, @FUNC, 'pDimKeys: ' || @pDimKeys);
    execute immediate @vSQL ;
	set @vRowCount = @@rowcount;;
    if (@vRowCount > 0) then
      CALL SP_Log_LogInfo(@vSessionKey, @FUNC,
                        'business view node key #' || @pViewKey || ' and dimension row key #' || @pDimKeys || ', assignment data rows dropped for dimension ' ||
                       @pDimension || ': ' || @vRowCount);
    end if;

exception
    when others then
        rollback to savepoint BViewAPIRmDimRowsSP;
        CALL   SP_Error_RaiseApplicationErrorDetail
            (
                 @EBViewError,
                @FUNC,
                'Remove assigment(s) failed- dimension: ' || @pDimension,
                @gConfigActiveSessionKey
        ); ;
end;

IF EXISTS (SELECT 1 FROM sys.sysprocedure  WHERE proc_name = 'SP_BVIEW_RemoveDimensionSupport')THEN
  DROP PROCEDURE "SAHARA"."SP_BVIEW_RemoveDimensionSupport";  
END IF;

CREATE PROCEDURE SAHARA.SP_BVIEW_RemoveDimensionSupport
(
  IN @pViewKey NUMERIC(10),
  in @pDimension VARCHAR(26)
)
BEGIN
  
  DECLARE @FUNC VARCHAR(255);
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vAlias VARCHAR(12); 
  DECLARE @vRowCount INTEGER;
  DECLARE @vSQLDIM LONG VARCHAR;
  DECLARE @IfExists BIT;
  
  BEGIN 
    -- initialize global variables
	call sahara.SP_saharaAPIInit();


    SET @vSQL = 'delete from SAHARA.$ALIAS$_BVIEW_NODE where BVIEW_KEY = '|| @pViewKey;
    
    SET @vAlias = SAHARA.FN_BVIEW_ValidateDimension(@FUNC, @pDimension);
    
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'entry');
    
    CALL SP_Config_RequireControl(@FUNC);
  
    BEGIN 
    
      SET @IfExists = 0;

      SELECT 
        1 INTO @IfExists
      FROM 
        BVIEW_IMPL
        JOIN BVIEW_NODE ON BVIEW_IMPL.BVIEW_KEY = BVIEW_NODE.BVIEW_NODE_KEY
      WHERE BVIEW_IMPL.BVIEW_KEY = @pViewKey;

      IF @IfExists = 1 THEN 

        -- trigger asynchronous schema changes on successful commit

	DELETE FROM SAHARA.tmp_BViewDimensionKeyTable;
	
	DELETE FROM BVIEW_DIMENSION_BVIEW WHERE DIMENSION = @pDimension AND BVIEW_KEY = @pViewKey;

        -- if the binding doesn't exist, we'll have the result we wanted
        -- so don't throw an error

        -- delete assignment data from dimension business view node table 
    
        SET @vSQLDIM = replace(@vSQL, '$ALIAS$', @vAlias);
        
	CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, @vSQLDIM); 

        EXECUTE IMMEDIATE @vSQLDIM;

	SET @vRowCount = @@rowcount;

        IF @vRowCount > 0 then
        
	  CALL SP_Log_LogInfo(@gConfigActiveSessionKey,@FUNC, 'business view key #' || @pViewKey || ', assignment data rows dropped for dimension ' || @pDimension || ': ' || @vRowCount);

        END IF;
      
      ELSE 

	CALL SP_Error_RaiseApplicationErrorDetail(@EBViewNotFound,@FUNC,'the specific business view was not found - key: ' || @pViewKey,@gConfigActiveSessionKey ); 
      
      END IF;
   
    END;
  END;
END;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_BVIEW_SetProcessingUpdate');

CREATE PROCEDURE SAHARA.SP_BVIEW_SetProcessingUpdate
  (
  IN @pValue INTEGER DEFAULT 0
  )
BEGIN
  DECLARE @FUNC VARCHAR(255);
  BEGIN 
    SET @FUNC= 'SP_BVIEW_SetProcessingUpdate';
    CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @FUNC, 'Entry');

   -- update the configuration table entry
    UPDATE 
      CONFIGURATION
    SET 
      INTEGER_VALUE = @pValue
    WHERE 
      SECTION = 'META-MODEL'
    AND NAME = 'ACTIVE_UPDATE';

    COMMIT;
  END;
END;

message 'Creating/replacing BUSINESS VIEW module EVENTS ...' to client;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('EVENT','EV_DROP_BVIEW');

CREATE  EVENT SAHARA.EV_DROP_BVIEW
handler
BEGIN
  DECLARE @vAlias VARCHAR(12);
  DECLARE @vName  VARCHAR(12);
  DECLARE @vViewName VARCHAR(255);
  BEGIN 

	commit;

        select BVIEW_DIMENSION.ALIAS into @vAlias
        FROM SAHARA.BVIEW_DIMENSION
        WHERE BVIEW_DIMENSION.DIMENSION = event_parameter('eDimension');

        select  BVIEW_IMPL.NAME into  @vName
        FROM SAHARA.BVIEW_IMPL
        WHERE BVIEW_IMPL.BVIEW_KEY = event_parameter('eBviewKey');
       
    	SET @vViewName = 'BV_' || @vAlias || '_' || @vName;

        CALL SP_DROP_OBJECT_SAHARA('VIEW',@vViewName) ;
 
       commit;
  END;
END;


message 'Creating/replacing BUSINESS VIEW module TRIGGERS ...' to client;


--- Triggers on table  BVIEW_IMPL
------------------

CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_IMPL_AUFER
AFTER UPDATE OF NAME ORDER 1 ON SAHARA.BVIEW_IMPL 
REFERENCING  OLD AS before_update NEW AS after_update
FOR EACH ROW
begin
    CALL SAHARA.SP_BVIEW_ValidateName(after_update.NAME);
end;

CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_IMPL_AIFER
AFTER INSERT ORDER 2 ON SAHARA.BVIEW_IMPL 
REFERENCING  OLD AS before_update NEW AS after_update
FOR EACH ROW
begin
    CALL SAHARA.SP_BVIEW_ValidateName(after_update.NAME);
end;

CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_IMPL_AIU 
AFTER INSERT, UPDATE ORDER 3 ON SAHARA.BVIEW_IMPL
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
    set VIEW_REFRESH_REQUIRED = 'T'
end;


---- Triggers on table BVIEW_LEVEL


CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_LEVEL_2AIUDFER
AFTER INSERT ,UPDATE, DELETE ORDER 1 ON SAHARA.BVIEW_LEVEL
REFERENCING NEW AS new_row OLD AS old_row
FOR EACH ROW
BEGIN

    IF INSERTING THEN
   
        CALL SP_BVIEW_ValidateName(new_row.NAME);

	    INSERT INTO SAHARA.tmp_BViewKeyTable(BVIEW_KEY,SessionKey ) ON EXISTING SKIP VALUES (new_row.BVIEW_KEY, isnull(@gConfigActiveSessionKey,0)) ;


    ELSE
    
       IF UPDATING THEN

        CALL SP_BVIEW_ValidateName(new_row.NAME);

        -- only if the name or level had changed, has the business view really changed
        
    IF old_row.NAME <> new_row.NAME OR newrow.LEVEL <> old_row.LEVEL  THEN
          
            INSERT INTO SAHARA.tmp_BViewKeyTable(BVIEW_KEY,SessionKey ) ON EXISTING SKIP VALUES (new_row.BVIEW_KEY, isnull(@gConfigActiveSessionKey,0));
           
    END IF;
       
 ELSE
        -- must be deleting
        
            INSERT INTO SAHARA.tmp_BViewKeyTable(BVIEW_KEY,SessionKey )ON EXISTING SKIP VALUES (old_row.BVIEW_KEY, isnull(@gConfigActiveSessionKey,0)) ;
        
    END IF;
END IF;

END;


CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_LEVEL_3AIUD
AFTER INSERT,UPDATE,DELETE ORDER 2 ON SAHARA.BVIEW_LEVEL
begin

 FOR BVIEWLOOP AS BVIEWCUR CURSOR FOR 
      SELECT 
        BVIEW_KEY AS BVIEW_KEY_R 
      from 
        SAHARA.tmp_BViewKeyTable
      where 
        isnull(@gConfigActiveSessionKey,0) in (SessionKey,0) 

    DO 
         update BVIEW
           set VIEW_CREATION_REQUIRED = 'T'
         where BVIEW_KEY = BVIEW_KEY_R ;

    END FOR;

END;


---------------Triggers on table VOLUME_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_VOLUME_BVIEW_NODE_AIUD
AFTER INSERT,UPDATE,DELETE ON SAHARA.VOLUME_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table NA_VOLUME_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_NA_VOLUME_BVIEW_NODE_AIUD
AFTER INSERT, UPDATE ,DELETE ON SAHARA.NA_VOLUME_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;


---------------Triggers on table SWITCH_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_SWITCH_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.SWITCH_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;


---------------Triggers on table LUN_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_LUN_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.LUN_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table HOST_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_HOST_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.HOST_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table FILESYSTEM_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_FS_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.FILESYSTEM_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;


---------------Triggers on table ARRAY_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_ARRAY_BVIEW_NODE_AIUD
AFTER INSERT ,UPDATE , DELETE ON SAHARA.ARRAY_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table NETAPP_SHARE_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_NETAPP_SHARE_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.NETAPP_SHARE_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table NAS_FS_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_NAS_FS_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.NAS_FS_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table APP_GROUP_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_APP_GROUP_BVIEW_NODE_AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.APP_GROUP_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table NETAPP_QTREE_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_NETAPP_QTREE_BVIEW_NODE_AIUD
AFTER INSERT ,UPDATE ,DELETE ON SAHARA.NETAPP_QTREE_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table CLUSTER_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_CLUSTER_BVIEW_NODE_AIUD
AFTER INSERT ,UPDATE , DELETE ON SAHARA.CLUSTER_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table SERVICEGROUP_BVIEW_NODE

CREATE OR REPLACE TRIGGER SAHARA.T_SERVICEGROUP_BVIEW_NODE_AIUD
AFTER INSERT ,UPDATE , DELETE ON SAHARA.SERVICEGROUP_BVIEW_NODE
FOR EACH STATEMENT
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

---------------Triggers on table BVIEW_DIMENSION_BVIEW

CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_DIM_BVIEW_2AIUDFER
AFTER INSERT , UPDATE , DELETE ON SAHARA.BVIEW_DIMENSION_BVIEW
REFERENCING  OLD AS OLDROW NEW AS NEWROW
FOR EACH ROW
begin
  DECLARE @vBviewKey NUMERIC(10);
  DECLARE @vDimension VARCHAR(26);
  BEGIN 
    if inserting then
              

	    INSERT INTO SAHARA.tmp_BViewDimensionKeyTable(BVIEW_KEY,SessionKey ) ON EXISTING SKIP VALUES (NEWROW.BVIEW_KEY, isnull(@gConfigActiveSessionKey,0)) ;

    else 
        if updating then
        -- only if something changed, has the business view really changed
           if (OLDROW.DIMENSION <> NEWROW.DIMENSION) or (NEWROW.BVIEW_KEY <> OLDROW.BVIEW_KEY)  then

             INSERT INTO SAHARA.tmp_BViewDimensionKeyTable(BVIEW_KEY,SessionKey ) ON EXISTING SKIP VALUES (NEWROW.BVIEW_KEY, isnull(@gConfigActiveSessionKey,0)) ;

            
           end if;
       else
        -- must be deleting
         INSERT INTO SAHARA.tmp_BViewDimensionKeyTable(BVIEW_KEY,SessionKey ) ON EXISTING SKIP VALUES (OLDROW.BVIEW_KEY, isnull(@gConfigActiveSessionKey,0)) ;


        -- get the buisness view name and alais name for the deleted object_type
        SET @vBviewKey =  OLDROW.BVIEW_KEY;
        SET @vDimension = OLDROW.DIMENSION;

        trigger event EV_DROP_BVIEW( "eBviewKey" = @vBviewKey, "eDimension" = @vDimension );

      END IF;

    end if;
  END;
END;

CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_DIMB_VIEW_3AIUD
AFTER INSERT , UPDATE , DELETE ON SAHARA.BVIEW_DIMENSION_BVIEW
FOR EACH STATEMENT
begin
    for BViewDimensionKeyTableCUR as BViewDimensionKeyTableLoop cursor for 
    SELECT BVIEW_KEY FROM 
      tmp_BViewDimensionKeyTable
    WHERE SessionKey IN (@gConfigActiveSessionKey,0)
    DO 
        update BVIEW
           set VIEW_CREATION_REQUIRED = 'T';
    END FOR;
END;

---------------Triggers on table BVIEW_NODE

CREATE OR REPLACE TRIGGER "SAHARA"."T_BVIEW_NODE_AIFES" 
    after insert    on SAHARA.BVIEW_NODE
begin
    update SAHARA.BVIEW_STATUS
       set NUMBER_REQUIRED = 'T',
           VIEW_REFRESH_REQUIRED = 'T';
end;


CREATE OR REPLACE TRIGGER "SAHARA"."T_BVIEW_NODE_AUFES" 
    after  update of PARENT_BVIEW_NODE_KEY, BVIEW_KEY, DISPLAY_NAME, "LEVEL"
    on SAHARA.BVIEW_NODE
begin
    update SAHARA.BVIEW_STATUS
       set NUMBER_REQUIRED = 'T',
           VIEW_REFRESH_REQUIRED = 'T';
end;

CREATE OR REPLACE TRIGGER "SAHARA"."T_BVIEW_NODE_AD" 
    after delete
    on SAHARA.BVIEW_NODE
begin
    update SAHARA.BVIEW_STATUS
       set VIEW_REFRESH_REQUIRED = 'T';
end;

CREATE OR REPLACE TRIGGER SAHARA.T_BVIEW_NODE_BIFER
BEFORE INSERT ON SAHARA.BVIEW_NODE
REFERENCING  NEW AS new_row 
FOR EACH ROW

BEGIN
  DECLARE @vBVKey INTEGER;
  DECLARE @vLevel INTEGER;
  BEGIN 

    SET @vBVKey = 0;
    SET @vLevel = 0;
  
    -- set the level and tree key

    IF new_row.PARENT_BVIEW_NODE_KEY is null THEN

      SET new_row.LEVEL = null;
   
    ELSE

      -- this is a new node in an existing tree -- we set columns
      -- based on the parent
    
	
	select ISNULL(BVIEW_KEY,0),ISNULL(LEVEL, 0) + 1
          into @vBVKey,@vLevel
          from SAHARA.BVIEW_NODE
         where BVIEW_NODE_KEY = new_row.PARENT_BVIEW_NODE_KEY;
    
      
        SET new_row.BVIEW_KEY = @vBVKey;
        SET new_row.LEVEL = @vLevel;

    
    END IF;

  END;

END;

-- The temporary table will be populated by the GUI.
IF EXISTS (SELECT 1 FROM sys.systable t key join sysusers u WHERE t.table_name = 'tmp_BVIEW_CONDITIONS'
                        AND  t.table_type = 'GBL TEMP' and     u.name = 'SAHARA')
THEN
    DROP TABLE SAHARA.tmp_BVIEW_CONDITIONS;
END IF;

create global temporary table SAHARA.tmp_BVIEW_CONDITIONS 
(
   ATTRIBUTE            long varchar                   null,
   OPERATOR             varchar(255)                   null,
   ATTRIBUTEVALUE       long varchar                   null,
   CONJUNCTION          varchar(5)                     null
) on commit preserve rows;

-- Function to  add the rule
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'FN_BVIEW_AddRule' and user_name(creator) = 'SAHARA' )
THEN
        DROP FUNCTION SAHARA.FN_BVIEW_AddRule;
END IF;
CREATE FUNCTION SAHARA.FN_BVIEW_AddRule
(
 IN @RuleName varchar(64),
 IN @RuleDescription long varchar,
 IN @Enabled bit,
 IN @ApprovalRequired bit,
 IN @ObjectType varchar(26),
 IN @pNodeKey numeric(10),
 IN @pViewKey numeric(10)
)
RETURNS INTEGER
BEGIN

DECLARE @func VARCHAR(64);
	DECLARE @ruleKey numeric(10);
    DECLARE @vMessage long varchar;
    DECLARE @errormsg long varchar;
	
  
    -- initialize global variables
	
  call sahara.SP_saharaAPIInit();

	SET @func = 'FN_BVIEW_AddRule';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    --Get the object type key.

    savepoint BViewCreateRules;
    
    
    -- create the SQL text from the rule text
    insert into SAHARA.BVIEW_RULES(
        NAME   ,
        DESCRIPTION,       
        objecttype,
        APPROVALREQUIRED,
        ENABLED,
        BVIEW_KEY,
        BVIEW_NODE_KEY,
        DIMENSION,
        APPROVALSTATE)
        values(@RuleName,@RuleDescription,@objecttype, @ApprovalRequired, @Enabled, @pViewKey,@pNodeKey,@ObjectType,0);

	set @rulekey =  @@identity;

    -- insert the condtion
   insert into SAHARA.BVIEW_CONDITIONS 
    (
     BVIEW_RULES_KEY,   
   ATTRIBUTE         ,
   OPERATOR       ,
   ATTRIBUTEVALUE ,
   CONJUNCTION )
   select @rulekey,attribute,operator,attributevalue,conjunction from SAHARA.tmp_BVIEW_CONDITIONS as tmp;

 // GUI re-uses the connection, hence deleting it.
   delete sahara.tmp_BVIEW_CONDITIONS;
     return @rulekey;

  -- opeation has to be automic
    EXCEPTION 
       WHEN OTHERS THEN         
           ROLLBACK TO savepoint BViewCreateRules;        
        delete sahara.tmp_BVIEW_CONDITIONS;
      CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'Add rule failed' || @RuleName ,
             @gConfigActiveSessionKey
        );   

   
   
END;


-- Function to modify  the rule
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'FN_BVIEW_ModifyRule' and user_name(creator) = 'SAHARA' )
THEN
        DROP FUNCTION SAHARA.FN_BVIEW_ModifyRule;
END IF;
CREATE FUNCTION SAHARA.FN_BVIEW_ModifyRule
(
 IN @RuleNum numeric(10),
 IN @RuleName varchar(64),
 IN @RuleDescription long varchar,
 IN @Enabled bit,
 IN @ApprovalRequired bit,
 IN @ObjectType varchar(26),
 IN @pNodeKey numeric(10),
 IN @pViewKey numeric(10)
)
RETURNS INTEGER
BEGIN
DECLARE @func VARCHAR(64);
call sahara.SP_saharaAPIInit();

	SET @func = 'FN_BVIEW_ModifyRule';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    --Get the object type key.

    savepoint BViewModifyRules;
    
    
    -- create the SQL text from the rule text
    insert into SAHARA.BVIEW_RULES(
        BVIEW_RULES_KEY,
        NAME   ,
        DESCRIPTION,       
        objecttype,
        APPROVALREQUIRED,
        ENABLED,
        BVIEW_KEY,
        BVIEW_NODE_KEY,
        DIMENSION,
        APPROVALSTATE)
        on existing update values(@RuleNum,@RuleName,@RuleDescription,@objecttype, @ApprovalRequired, @Enabled, @pViewKey,@pNodeKey,@ObjectType,1);

	
--- Blindly delet the old conditons, there can be new condition added.

    DELETE BVIEW_CONDITIONS where BVIEW_RULES_KEY = @ruleNum;


-- Insert back any thing that is newly added as a part of the BVIEW_CONDTION
    insert  SAHARA.BVIEW_CONDITIONS 
    (
     BVIEW_RULES_KEY,   
   ATTRIBUTE         ,
   OPERATOR       ,
   ATTRIBUTEVALUE ,
   CONJUNCTION )
   select @RuleNum,attribute,operator,attributevalue,conjunction from SAHARA.tmp_BVIEW_CONDITIONS  ;

  // GUI re-uses the connection, hence deleting it.
   delete sahara.tmp_BVIEW_CONDITIONS;

return @RuleNum;

  -- opeation has to be automic
   EXCEPTION 
      WHEN OTHERS THEN         
           ROLLBACK TO savepoint BViewModifyRules;        

       delete sahara.tmp_BVIEW_CONDITIONS;
     CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'Modify rule failed' || @RuleName ,
             @gConfigActiveSessionKey
        );   

END;


IF EXISTS (SELECT 1 FROM sys.systable t key join sysusers u WHERE t.table_name = 'BVIEW_OPERATORS'  and     u.name = 'SAHARA')
THEN
    DROP TABLE SAHARA.BVIEW_OPERATORS;
END IF;

create  table SAHARA.BVIEW_OPERATORS
(
   DisplayName            varchar(255),
   Name             varchar(255) 
);

insert into SAHARA.BVIEW_OPERATORS(DisplayName,Name)
   values('equals','=');
insert into SAHARA.BVIEW_OPERATORS(DisplayName,Name)
   values('starting','LIKE');
insert into SAHARA.BVIEW_OPERATORS(DisplayName,Name)
   values('ending','LIKE');
insert into SAHARA.BVIEW_OPERATORS(DisplayName,Name)
   values('contains','LIKE');
--insert into SAHARA.BVIEW_OPERATORS(DisplayName,Name)
  -- values('REGEX','REGEX');
commit;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_AutoRunRules' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_AutoRunRules;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_AutoRunRules
( 
 )
BEGIN
DECLARE @func varchar(255);
DECLARE @bvrulekey numeric(10);
    SET @func = 'SP_BVIEW_AutoRunRules';

    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry' );

for LoadBVIEWCursor as LoadBVIEWLoop cursor for select BVIEW_KEY as bvkey from BVIEW_IMPL  where ApprovalRequired = 0
do
       message  'The bv rule key ' || bvkey to client;
       SET @bvrulekey = bvkey;
       CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'Executing rules for business view : ' || bvkey );
       call sahara.SP_BVIEW_RunRules(bvkey);
end for;

EXCEPTION 
       WHEN OTHERS THEN         
       CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'ERROR  Executing the rules for business view : ' || @bvrulekey );
      return;   

END;
-- Procedure  to Run the rule on a business view
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_RunRules' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_RunRules;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_RunRules
(
 IN @pViewKey numeric(10),
 IN @ApprovalRequired bit default 1
 )
BEGIN
    DECLARE @func varchar(255);
    SET @func = 'SP_BVIEW_RunRules';
    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry' );


for LoadRulesCursor as LoadRuleLoop cursor for select BVIEW_RULES_KEY as rulekey
                         from sahara.BVIEW_RULES as bvr
                                where bvr.bview_key = @pViewKey and bvr.ENABLED = 1
do
       message  'The rule key ' || rulekey to client;
       call sahara.SP_BVIEW_RunRule(rulekey,@ApprovalRequired);
end for;
    
END;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_RunRule' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_RunRule;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_RunRule
(
 IN @pRuleKey numeric(10),
 IN @ApprovalRequired bit
 )
BEGIN
    DECLARE @l_WHERECLAUSE long varchar;
    DECLARE @operand varchar(255);
    DECLARE @func varchar(255);
    DECLARE @objecttype varchar(255);
    DECLARE @dimension varchar(255);
    DECLARE @nodekey numeric(10);
    DECLARE @dimcurrent varchar(255);
    DECLARE @dimnode varchar(255);
    DECLARE @dimkey varchar(255);
    DECLARE @bvkey varchar(255);
    DECLARE @attrvalue varchar(255);
    DECLARE @operatorvalue varchar(255);
    DECLARE @SQLQuery long varchar;
    DECLARE @ObjectList long varchar;
    DECLARE @lockheld bit;
    DECLARE @dimAlias varchar(255);


    SET @func = 'SP_BVIEW_RunRule';
    set @lockheld = 0;
    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
    
    select objecttype,DIMENSION,BVIEW_NODE_KEY,BVIEW_KEY into @objecttype,@dimension,@nodekey,@bvkey from sahara.BVIEW_RULES br where br.BVIEW_RULES_KEY = @pRuleKey;
    
    select ALIAS into @dimAlias from SAHARA.BVIEW_DIMENSION where DIMENSION = @dimension;

    set @dimcurrent = @dimension ||'_CURRENT';
    set @dimnode  = @dimAlias ||'_BVIEW_NODE';
    set @dimkey  = '.'||@dimension ||'_KEY';

        
    for LoadConditionCursor as LoadConditionLoop cursor for select attribute as attrnm,attributevalue as attrval, bop.Name as oper,bop.DisplayName as operdisp, (if conjunction = 'NA' then '' else conjunction end if) as conj
                         from sahara.BVIEW_CONDITIONS as bvc, sahara.BVIEW_OPERATORS as bop
                                where bvc.BVIEW_RULES_KEY =  @pRuleKey 
                        and bop.DisplayName =  bvc.operator 
                                order by bvc.BVIEW_CONDITIONS_KEY ASC
    do
             CASE operdisp
                WHEN 'starting' THEN
                        set @attrvalue =  '''' || attrval || '%''';
                WHEN 'ending' THEN
                    set @attrvalue =  '''%' || attrval ||'''';
                WHEN 'equals' THEN
                    set @attrvalue =  ''''||attrval ||'''';
                WHEN 'contains' THEN
                    set @attrvalue = '''%'|| attrval||'%''';
               ELSE
                set @attrvalue = attrval;
               END CASE;

                
             
        set @l_WHERECLAUSE = @l_WHERECLAUSE ||  conj || ' ( ' || @dimcurrent ||'.'|| attrnm ||' '|| oper||' ' || @attrvalue || ') '  ;
        message @l_WHERECLAUSE to client;
    end for;

    set @SQLQuery = 'select  list( distinct ' || @dimcurrent|| @dimkey || ' )  into @objectlist from sahara.' 
                                                                            ||@dimcurrent || ' where  ' || @dimcurrent || @dimkey || 
                                                                '  not in (select ' || @dimnode || @dimkey || ' from sahara.' || @dimnode || ' where ' 
                                                                ||  @dimnode|| '.BVIEW_KEY = ' || @bvkey 
                                                                || ' ) AND (' ||  @l_WHERECLAUSE || ')';

--select Count(*) from sahara."+dimensionTableCurrent + " WHERE "+joinWithObejctTable+" NOT IN (SELECT " +joinWithObejctTable +" FROM sahara."+bviewNodeTable+" WHERE bview_key='"+bviewId+"')"+whereCond);
  CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, @SQLQuery); 
  message @SQLQuery to client;
   execute immediate @SQLQuery;
    message @objectlist to client;
    
    if (length(@objectlist) =  0 ) then
        return;
        --nothing to run go back.
    end if;

    message 'Acquiring the connection' to client;
    -- Acquire the connection
    call SAHARA.SP_Config_AcquireExclusiveLock( sahara.Fn_Config_GetActiveSessionKey()) ;
   
   set @lockheld = 1;
    -- do the assignment of the objects
    call SAHARA.SP_BVIEW_AssociateDimensionRows_String(@nodekey,@objectlist,@dimension,1,1,@pRuleKey);
   
    -- Release the lock
   call "SAHARA"."SP_Config_ReleaseExclusiveLock"();
   
   
EXCEPTION 
       WHEN OTHERS THEN         
   if (@lockheld = 1) then
         call "SAHARA"."SP_Config_ReleaseExclusiveLock"();
       end if;
      CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'Failed to run the rule  ' ,
             sahara.Fn_Config_GetActiveSessionKey()
        );   

   
END;

-- The Store procedure to Test  the rule --
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_TestRule' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_TestRule;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_TestRule
(
 IN @dimension varchar(26), 
 IN @bvkey numeric(10)
  )
RESULT(asset_key numeric(10),
        display_name long varchar)

BEGIN
    DECLARE @l_WHERECLAUSE long varchar;
    DECLARE @func varchar(255);
    DECLARE @dimcurrent varchar(255);
    DECLARE @dimnode varchar(255);
    DECLARE @dimkey varchar(255);    
    DECLARE @attrvalue varchar(255);
    DECLARE @operatorvalue varchar(255);
    DECLARE @SQLQuery long varchar;
    DECLARE @ObjectList long varchar;
    DECLARE @errmsg long varchar;
	DECLARE @dimAlias varchar(255);
    SET @func = 'SP_BVIEW_TestRule';

    set @errmsg = 'Test Rule failed';
    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
    
    select ALIAS into @dimAlias from SAHARA.BVIEW_DIMENSION d where d.DIMENSION = @dimension;

    set @dimcurrent = @dimension ||'_CURRENT';
    set @dimnode  = @dimAlias ||'_BVIEW_NODE';
    set @dimkey  = '.'||@dimension ||'_KEY';

    
    if not exists(select 1 from sahara.tmp_BVIEW_CONDITIONS) then
     set @errmsg = 'No Condition Supplied';
      CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'No conditions supplied  ' ,
             sahara.Fn_Config_GetActiveSessionKey()
        );
    end if;
    for LoadConditionCursor as LoadConditionLoop cursor for select attribute as attrnm,attributevalue as attrval, bop.Name as oper,bop.DisplayName as operdisp, (if conjunction = 'NA' then '' else conjunction end if) as conj
                         from sahara.tmp_BVIEW_CONDITIONS as bvc, sahara.BVIEW_OPERATORS as bop
                                where  bop.DisplayName =  bvc.operator                                 
    do
             CASE operdisp
                WHEN 'starting' THEN
                        set @attrvalue =  '''' || attrval || '%''';
                WHEN 'ending' THEN
                    set @attrvalue =  '''%' || attrval ||'''';
                WHEN 'equals' THEN
                    set @attrvalue =  ''''||attrval ||'''';
                WHEN 'contains' THEN
                    set @attrvalue = '''%'|| attrval||'%''';
               ELSE
                set @attrvalue = attrval;
               END CASE;

                
        set @l_WHERECLAUSE = @l_WHERECLAUSE ||  conj || ' ( ' || @dimcurrent ||'.'|| attrnm ||' '|| oper||' ' || @attrvalue || ') '  ;     
        //set @l_WHERECLAUSE = @l_WHERECLAUSE ||  ' ( ' || @dimcurrent ||'.'|| attrnm ||' '|| oper||' ' || @attrvalue || ') ' || conj ;
        message @l_WHERECLAUSE to client;
    end for;

    set @SQLQuery = 'select  distinct top 500 ' || @dimcurrent|| @dimkey|| ',' || @dimcurrent||'.display_name' || ' from sahara.' 
                                                                            ||@dimcurrent || ' where  ' || @dimcurrent || @dimkey || 
                                                                '  not in (select ' || @dimnode || @dimkey || ' from sahara.' || @dimnode || ' where ' 
                                                                ||  @dimnode|| '.BVIEW_KEY = ' || @bvkey 
                                                                || ' ) AND (' ||  @l_WHERECLAUSE || ')';

--select Count(*) from sahara."+dimensionTableCurrent + " WHERE "+joinWithObejctTable+" NOT IN (SELECT " +joinWithObejctTable +" FROM sahara."+bviewNodeTable+" WHERE bview_key='"+bviewId+"')"+whereCond);
  -- Delete before executing it 

  delete sahara.tmp_BVIEW_CONDITIONS;
CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, @SQLQuery); 
   message @SQLQuery to client;
   execute immediate @SQLQuery;
    message @objectlist to client;
    
    
    
EXCEPTION 
     WHEN OTHERS THEN         
      delete sahara.tmp_BVIEW_CONDITIONS;
 CALL   SP_Error_RaiseApplicationErrorDetail
    (
            @EBViewError,
             @func,
       'test rule failed' ,
       sahara.Fn_Config_GetActiveSessionKey()
       );   

   
END;


commit;
-- Procedure to delete the rule
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_DeleteRule' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_DeleteRule;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_DeleteRule
(
 IN @pRuleKey numeric(10)
 )
BEGIN

DECLARE @l_WHERECLAUSE long varchar;
    DECLARE @operand varchar(255);
    DECLARE @func varchar(255);
    DECLARE @objecttype varchar(255);
    DECLARE @dimension varchar(255);
    DECLARE @nodekey numeric(10);
    DECLARE @dimcurrent varchar(255);
    DECLARE @dimnode varchar(255);
    DECLARE @dimkey varchar(255);
    DECLARE @bvkey varchar(255);
    DECLARE @attrvalue varchar(255);
    DECLARE @operatorvalue varchar(255);
    DECLARE @SQLQuery long varchar;
    DECLARE @ObjectList long varchar;
    DECLARE @lockheld  bit;
    DECLARE @dimAlias varchar(255);


    SET @func = 'SP_BVIEW_DeleteRule';
    set @lockheld = 0;
    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
    
    select objecttype,DIMENSION,BVIEW_NODE_KEY,BVIEW_KEY into @objecttype,@dimension,@nodekey,@bvkey from sahara.BVIEW_RULES br where br.BVIEW_RULES_KEY = @pRuleKey;
    select ALIAS into @dimAlias from SAHARA.BVIEW_DIMENSION where DIMENSION = @dimension;

    set @dimcurrent = @dimension ||'_CURRENT';
    set @dimnode  = @dimAlias ||'_BVIEW_NODE';
    set @dimkey  = '.'||@dimension ||'_KEY';

        
    for LoadConditionCursor as LoadConditionLoop cursor for select attribute as attrnm,attributevalue as attrval, bop.Name as oper,bop.DisplayName as operdisp, (if conjunction = 'NA' then '' else conjunction end if) as conj
                         from sahara.BVIEW_CONDITIONS as bvc, sahara.BVIEW_OPERATORS as bop
                                where bvc.BVIEW_RULES_KEY =  @pRuleKey 
                        and bop.DisplayName =  bvc.operator 
                                order by bvc.BVIEW_CONDITIONS_KEY ASC
    do
             CASE operdisp
                WHEN 'starting' THEN
                        set @attrvalue =  '''' || attrval || '%''';
                WHEN 'ending' THEN
                    set @attrvalue =  '''%' || attrval ||'''';
                WHEN 'equals' THEN
                    set @attrvalue =  ''''||attrval ||'''';
                WHEN 'contains' THEN
                    set @attrvalue = '''%'|| attrval||'%''';
               ELSE
                set @attrvalue = attrval;
               END CASE;

                
             
        --set @l_WHERECLAUSE = @l_WHERECLAUSE ||  conj || ' ( ' || @dimcurrent ||'.'|| attrnm ||' '|| oper||' ' || @attrvalue || ') '  ;
        set @l_WHERECLAUSE = @dimnode ||'.BVIEW_RULES_KEY = ' || @pRuleKey;
        message @l_WHERECLAUSE to client;
    end for;

    set @SQLQuery = 'select  list( distinct ' || @dimcurrent|| @dimkey || ' )  into @objectlist from sahara.' 
                                                                            ||@dimcurrent || ' where  ' || @dimcurrent || @dimkey || 
                                                                '  in (select ' || @dimnode || @dimkey || ' from sahara.' || @dimnode || ' where ' 
                                                                ||  @dimnode|| '.BVIEW_KEY = ' || @bvkey 
                                                                || ' AND ' || @dimnode || '.STATUS = 1   AND ' ||  @l_WHERECLAUSE || ');';

--select Count(*) from sahara."+dimensionTableCurrent + " WHERE "+joinWithObejctTable+" NOT IN (SELECT " +joinWithObejctTable +" FROM sahara."+bviewNodeTable+" WHERE bview_key='"+bviewId+"')"+whereCond);
  CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, @SQLQuery); 
  message @SQLQuery to client;
   execute immediate @SQLQuery;
    message @objectlist to client;
    // Remove the assignment if there are no objects to be deleted
    if (length(@objectlist) !=  0 ) then

    message 'Acquiring the connection' to client;
    -- Acquire the connection
    call SAHARA.SP_Config_AcquireExclusiveLock( sahara.Fn_Config_GetActiveSessionKey()) ;
   
   set @lockheld = 1;
   -- do the assignment of the objects
   call SAHARA.SP_BVIEW_RemoveAssignments(@bvkey,@objectlist,@dimension);    
   
    -- Release the lock
   call "SAHARA"."SP_Config_ReleaseExclusiveLock"();

   -- Since assignment is done time to delete the rule

    //    return;
        --nothing to run go back.
    end if;

    
   DELETE sahara.BVIEW_RULES where BVIEW_RULES_KEY = @pRuleKey;

EXCEPTION 
       WHEN OTHERS THEN         
       if (@lockheld = 1) then
         call "SAHARA"."SP_Config_ReleaseExclusiveLock"();
       end if;
   
      CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'Failed to delete the rule  ' ,
             sahara.Fn_Config_GetActiveSessionKey()
        );   


END;


-- Procedure to delete the rule
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_DeleteTestResult' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_DeleteTestResult;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_DeleteTestResult
(
 IN @pRuleKey numeric(10)
 )
RESULT(asset_key numeric(10),
        display_name long varchar)

BEGIN

DECLARE @l_WHERECLAUSE long varchar;
    DECLARE @operand varchar(255);
    DECLARE @func varchar(255);
    DECLARE @objecttype varchar(255);
    DECLARE @dimension varchar(255);
    DECLARE @nodekey numeric(10);
    DECLARE @dimcurrent varchar(255);
    DECLARE @dimnode varchar(255);
    DECLARE @dimkey varchar(255);
    DECLARE @bvkey varchar(255);
    DECLARE @attrvalue varchar(255);
    DECLARE @operatorvalue varchar(255);
    DECLARE @SQLQuery long varchar;
    DECLARE @ObjectList long varchar;
    DECLARE @lockheld  bit;
	DECLARE @dimAlias varchar(255);

    SET @func = 'SP_BVIEW_DeleteTestResult';
    set @lockheld = 0;
    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
    
    select objecttype,DIMENSION,BVIEW_NODE_KEY,BVIEW_KEY into @objecttype,@dimension,@nodekey,@bvkey from sahara.BVIEW_RULES br where br.BVIEW_RULES_KEY = @pRuleKey;
    
    select ALIAS into @dimAlias from sahara.BVIEW_DIMENSION where DIMENSION = @dimension;

    set @dimcurrent = @dimension ||'_CURRENT';
    set @dimnode  = @dimAlias ||'_BVIEW_NODE';
    set @dimkey  = '.'||@dimension ||'_KEY';

        
    for LoadConditionCursor as LoadConditionLoop cursor for select attribute as attrnm,attributevalue as attrval, bop.Name as oper,bop.DisplayName as operdisp, (if conjunction = 'NA' then '' else conjunction end if) as conj
                         from sahara.BVIEW_CONDITIONS as bvc, sahara.BVIEW_OPERATORS as bop
                                where bvc.BVIEW_RULES_KEY =  @pRuleKey 
                        and bop.DisplayName =  bvc.operator 
                                order by bvc.BVIEW_CONDITIONS_KEY ASC
    do
             CASE operdisp
                WHEN 'starting' THEN
                        set @attrvalue =  '''' || attrval || '%''';
                WHEN 'ending' THEN
                    set @attrvalue =  '''%' || attrval ||'''';
                WHEN 'equals' THEN
                    set @attrvalue =  ''''||attrval ||'''';
                WHEN 'contains' THEN
                    set @attrvalue = '''%'|| attrval||'%''';
               ELSE
                set @attrvalue = attrval;
               END CASE;

                
             
        --set @l_WHERECLAUSE = @l_WHERECLAUSE ||  conj || ' ( ' || @dimcurrent ||'.'|| attrnm ||' '|| oper||' ' || @attrvalue || ') '  ;
        set @l_WHERECLAUSE = @dimnode ||'.BVIEW_RULES_KEY = ' || @pRuleKey;
        message @l_WHERECLAUSE to client;
    end for;

    set @SQLQuery = 'select  distinct ' || @dimcurrent|| @dimkey|| ' , ' || @dimcurrent||'.display_name' || '  from sahara.' 
                                                                            ||@dimcurrent || ' where  ' || @dimcurrent || @dimkey || 
                                                                '  in (select ' || @dimnode || @dimkey || ' from sahara.' || @dimnode || ' where ' 
                                                                ||  @dimnode|| '.BVIEW_KEY = ' || @bvkey 
                                                                || ' AND ' || @dimnode || '.STATUS = 1   AND  ' || @l_WHERECLAUSE || ');';

--select Count(*) from sahara."+dimensionTableCurrent + " WHERE "+joinWithObejctTable+" NOT IN (SELECT " +joinWithObejctTable +" FROM sahara."+bviewNodeTable+" WHERE bview_key='"+bviewId+"')"+whereCond);
  CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, @SQLQuery); 
  message @SQLQuery to client;
   execute immediate @SQLQuery;
    
    
EXCEPTION 
       WHEN OTHERS THEN         
    
      CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'Failed to execute the Delete Test rule  ' ,
             sahara.Fn_Config_GetActiveSessionKey()
        );   


END;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_ChangeApproval' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_ChangeApproval;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_ChangeApproval
(
 IN @BviewKey long varchar,
 IN @ApprovalRequired bit default 1
 )
BEGIN
    DECLARE @func varchar(255);
    

    SET @func = 'SP_BVIEW_ChangeApproval';
    
    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
    
    update SAHARA.BVIEW_IMPL set ApprovalRequired = @ApprovalRequired where BVIEW_KEY = @BviewKey  ;
   

EXCEPTION 
       WHEN OTHERS THEN         
    
      CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'Failed to Change the Approval   ' ,
             sahara.Fn_Config_GetActiveSessionKey()
        );   

END;




IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_EnableRules' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_EnableRules;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_EnableRules
(
 IN @pRuleKeys long varchar
 )
BEGIN
DECLARE @SQLString long varchar;
DECLARE @func varchar(255);
SET @func = 'SP_BVIEW_EnableRules';

    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
    

 set @SQLString = 'UPDATE SAHARA.BVIEW_RULES br set  ENABLED = (if enabled =1 then 0 else 1 endif)  where br.BVIEW_RULES_KEY  in (' || @pRuleKeys || ');';
 message @SQLString to client;
CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, @SQLString); 
 execute immediate @SQLString; 
 

EXCEPTION 
       WHEN OTHERS THEN         
    
      CALL   SP_Error_RaiseApplicationErrorDetail
         (
             @EBViewError,
                @func,
              'Failed to execute the enable rules  ' ,
             sahara.Fn_Config_GetActiveSessionKey()
        );   

END;


-- Store Procedure to approve, reject or delete the pending rules
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_ApprovalPendingRule' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_ApprovalPendingRule;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_ApprovalPendingRule
(
 IN @pViewKey numeric(10)
 )
BEGIN
END;

-- Procedure to add the BVIEW_RULES_KEY to the business view tables
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_AddRuleKey' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_AddRuleKey;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_AddRuleKey( )
BEGIN
declare @SQLQuery long varchar;
declare @tablename varchar(255);


for BviewTablesCursor as BviewTablesLoop cursor for select ALIAS as aliasname
                         from sahara.BVIEW_DIMENSION
    do
            
            set @tablename = aliasname ||'_BVIEW_NODE' ;
            set @SQLQuery = '  if not exists ( select 1 from systable t,syscolumn c
                         where t.table_name = '''|| @tablename ||''' and c.table_id = t.table_id and  c.column_name = ''BVIEW_RULES_KEY'' ) then
                            ALTER TABLE SAHARA.'|| @tablename || ' ADD BVIEW_RULES_KEY numeric(10) default null ; END IF;';
            message @SQLQuery to client;
            execute immediate @SQLQuery;

end for;
END;

call SAHARA.SP_BVIEW_AddRuleKey();


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_BVIEW_GetColumnValues' and user_name(creator) = 'SAHARA' )
THEN
        DROP PROCEDURE SAHARA.SP_BVIEW_GetColumnValues;
END IF;
CREATE PROCEDURE SAHARA.SP_BVIEW_GetColumnValues
(
 IN @pObjectType varchar(255),
 IN @pColumnName varchar(255)
 )
RESULT(column_values varchar(255))

BEGIN
    DECLARE @l_SQLString long varchar;
    DECLARE @func varchar(255);
    DECLARE @lockheld bit;


    SET @func = 'SP_BVIEW_GetColumnValues';
    set @lockheld = 0;
    call sahara.SP_saharaAPIInit();
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
    
    set @l_SQLString = 'select distinct ' ||@pColumnName|| ' from sahara.'||@pObjectType ||';';
 
  CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, @l_SQLString); 
  message @l_SQLString to client;
   execute immediate @l_SQLString;
    
    
   
   
EXCEPTION 
       WHEN OTHERS THEN         
      CALL   SP_Error_RaiseApplicationErrorDetail
          (
             @EBViewError,
                @func,
              'Failed to Get the column values ' ,
             sahara.Fn_Config_GetActiveSessionKey()
        );   

   
END;

-- VIEW will be used by the GUI to display the column names.
CREATE OR REPLACE VIEW SAHARA.BVIEW_TABLE_COLUMNS 
 AS 
select DIMENSION as tablename,rdsc.name as columnname,LOWER(rdsc.name) as displayname,col.coltype  from SAHARA.BVIEW_DIMENSION bvd
            join SAHARA.ROLLUP_DATA_SET_TYPE rds on  rds.STEREOTYPE = 'DIMENSION' and rds.DEST_TABLE  = bvd.DIMENSION
            join SAHARA.ROLLUP_DATA_SET_COLUMN rdsc on rdsc.ROLLUP_DATA_SET_TYPE_KEY = rds.ROLLUP_DATA_SET_TYPE_KEY             
            join sys.syscolumns as col on col.cname = rdsc.name and col.tname = tablename
                and rdsc.LOAD_PROCESSING = 'TYPE 1' and rdsc.name NOT LIKE '%_KEY' and rdsc.name NOT LIKE  '%_OID'
                       and rdsc.name NOT LIKE '%_URL'  order by tablename;




message 'start - grant on SAHARA objects to SAHARA_APP  ...' to client;

grant execute on SAHARA.SP_BVIEW_RemoveDimensionSupport to sahara_app;
grant execute on SAHARA.SP_BVIEW_PublishBV to sahara_app;
grant execute on SAHARA.SP_BVIEW_AssociateDimensionRows to sahara_app;
grant execute on SAHARA.SP_BVIEW_Delete1 to sahara_app;
grant execute on SAHARA.FN_BVIEW_AddLevel to sahara_app;
grant execute on SAHARA.SP_BVIEW_AddDimensionSupport to sahara_app;
grant execute on SAHARA.FN_BVIEW_Create1 to sahara_app;
grant execute on SAHARA.FN_BVIEW_AddNode to sahara_app;
--grant execute on sahara.SP_BVIEW_PublishBV to sahara_app;
grant select , insert,update,delete on SAHARA.DimensionRowKeyTable to sahara_app
grant select,insert, update,delete on SAHARA.tmp_BViewDimensionKeyTable to sahara_app;
grant execute on sahara.SP_Error_RaiseApplicationErrorDetail to sahara_app;
--grant execute on SAHARA.SP_DROP_OBJECT_SAHARA to sahara_app;
grant execute on sahara.SP_Log_LogInfo to sahara_app;
grant execute on sahara.SP_Log_LogDebug to sahara_app;
grant execute on sahara.SP_BVIEW_UpdateNote to sahara_app;
grant execute on sahara.SP_BVIEW_DeleteLevel to sahara_app;
grant execute on sahara.SP_BVIEW_UpdateLevel to sahara_app;
grant execute on sahara.SP_BVIEW_UpdateDisplayName to sahara_app;
grant execute on sahara.SP_BVIEW_AddNodes to sahara_app;
grant execute on sahara.SP_BVIEW_DeleteNode to sahara_app;
grant execute on sahara.SP_BVIEW_RemoveAssignments to sahara_app;
grant execute on sahara.SP_BVIEW_AssociateDimensionRows_string to sahara_app;
grant execute on sahara.SP_BVIEW_SetProcessingUpdate to sahara_app;
grant execute on sahara.SP_BVIEW_ApprovalPendingRule to sahara_app;
grant execute on sahara.SP_BVIEW_DeleteRule to sahara_app;
grant execute on sahara.SP_BVIEW_RunRule to sahara_app;
grant execute on sahara.SP_BVIEW_RunRules to sahara_app;
grant execute on sahara.FN_BVIEW_ModifyRule to sahara_app;
grant execute on sahara.FN_BVIEW_AddRule to sahara_app;
grant execute on sahara.SP_BVIEW_TestRule to sahara_app;
grant execute on sahara.SP_BVIEW_EnableRules to sahara_app;
grant execute on sahara.SP_BVIEW_DeleteTestResult to sahara_app;
grant execute on sahara.SP_BVIEW_ChangeApproval to sahara_app;
grant execute on SAHARA.SP_BVIEW_GetColumnValues to sahara_app;
grant select  on SAHARA.BVIEW_OPERATORS to sahara_app;
grant select,insert,delete,update on SAHARA.tmp_BVIEW_CONDITIONS to sahara_app;
grant select,insert,delete,update on SAHARA.BVIEW_TABLE_COLUMNS  to sahara_reports;
grant select,insert,delete,update on SAHARA.BVIEW_TABLE_COLUMNS  to sahara_app;

/*

 * $Source: /project/sahara/asadb/scripts/voma_upgrade.sql,v $

 *

 * $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $

 *

 * $Copyright: Copyright (c) 2012 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $

 *

 */

	
/*
 **********************************************************************
 * This package provides procedures and functions for working with 
 * roll-up batches. It is used by the ETL tools and the loading code.  
 **********************************************************************
*/
message 'Creating/replacing Batch module stored procedures...' to client;

/*
 **********************************************************************
 * fn_Batch_Create1():
 * Creates a new roll-up batch for a roll-up source.
 * Return:
 * The unique identifier of the newly created roll-up batch. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Batch_Create1' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Batch_Create1
end if;

CREATE FUNCTION SAHARA.fn_Batch_Create1 
(
	@pSessionKey numeric(10),
	@pSourceKey numeric(10),
	@pScheduleHash varchar(64),
	@pLogArchiveExt varchar(16) default '.tar.gz'
) 
	returns numeric(10)
BEGIN
	-- times are stored in UTC
    DECLARE @func VARCHAR(64);
	DECLARE @startTime datetime;
    DECLARE @sourceTypeKey numeric(10);
    DECLARE @vEnabled char(1);
    DECLARE @UTCAdjustment integer;
    DECLARE @rollupClock varchar(32);
    DECLARE @running integer;
    DECLARE @scheduleHash varchar(64);
    DECLARE @rollupDate date;
    DECLARE @newUTCTimestamp datetime;
    DECLARE @vDatesKey numeric(10);
    DECLARE @isRollupNow char(1);
    DECLARE @batchKey numeric(10);

	-- select the roll-up source so we can update it when the batch is created
	DECLARE @sourceCursor CURSOR FOR
        select ROLLUP_SOURCE_TYPE_KEY, ENABLED, UTC_ADJUSTMENT, ROLLUP_CLOCK
          from sahara.ROLLUP_SOURCE
         where ROLLUP_SOURCE_KEY = @pSourceKey
           FOR UPDATE; -- BY LOCK CLAUSE

    Set @startTime = sahara.fn_Conf_UTCTimestamp();
	set @scheduleHash = trim(@pScheduleHash);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	-- we want all or nothing from this call
    savepoint RollupBatchAPICreate1SP;

	SET @func = 'sahara.fn_Batch_Create1';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');
	
    -- validate the roll-up source
    open @sourceCursor;	

    FETCH NEXT @sourceCursor INTO @sourceTypeKey, @vEnabled, @UTCAdjustment, @rollupClock;

    if @@fetch_status = -2 then
        close @sourceCursor;

		CALL SP_Error_RaiseApplicationErrorDetail (
		@ERollupSourceNotFound,
		@FUNC, 'no roll-up source found with key ' || @pSourceKey,
		@pSessionKey);
    end if;

    if @vEnabled <> 'T' then
		close @sourceCursor;

		CALL SP_Error_RaiseApplicationErrorDetail (
		@ERollupSourceNotEnabled,
		@FUNC, 
		'roll-up batches prohibited for disabled sources - source key: ' || @pSourceKey,
		@pSessionKey);
    end if;

    -- check for running batches
    select count(*)
      into @running
      from sahara.ROLLUP_BATCH
     where IS_DUMMY = 'F' and  ROLLUP_SOURCE_KEY = @pSourceKey and PERIOD_TYPE = 'day'
       and IS_COMPLETE = 'F';

	if @running > 0 then
        close @sourceCursor;

		CALL SP_Error_RaiseApplicationErrorDetail (
		@EActiveRollup,
		@FUNC, 
		'An active roll-up batch exists for the roll-up source (' ||
         @pSourceKey || ') .  Only one batch may be active at a ' ||
         'time.',
		@pSessionKey);
    end if;

    -- to make it easier for Perl and Kettle, we'll allow some
    -- alternatives to null for the schedule hash
    if @scheduleHash = '0' or @scheduleHash like '0-%' then
        SET @scheduleHash = null;
    end if;

    if @scheduleHash is not null then
        -- this is a scheduled roll-up, not a "roll-up now"
        SET @isRollupNow = 'F';

        -- the hash must be at least 9 characters long and have a dash (-)
        -- in the nineth position
        if length(@scheduleHash) < 9 or '-' <> substr(@scheduleHash, 9, 1) then
            close @sourceCursor;

			CALL SP_Error_RaiseApplicationErrorDetail (
			@EScheduleHash,
			@FUNC, 
            'The schedule hash (' || @scheduleHash || ') for the ' ||
            'batch appears to be formatted incorrectly.',
			@pSessionKey);
        end if;

        -- extract the date from the schedule hash
        begin
            begin
                set @rollupDate = date(substr(@scheduleHash, 1, 8));
            EXCEPTION
                WHEN OTHERS THEN
                    close @sourceCursor;

					CALL SP_Error_RaiseApplicationErrorDetail (
					@EScheduleHash,
					@FUNC, 
					'Failed extracting the date from the schedule ' ||
					'hash (' || @scheduleHash || '): ' ||
					sqlcode || ': ' || ERRORMSG(),
					@pSessionKey);
           end;
        end;
    else
        -- if there's no schedule hash, we have to assume we're doing
        -- a "roll-up now"
        set @isRollupNow = 'T';
		SELECT sahara.fn_Conf_UTCTimestamp() into @newUTCTimestamp FROM dummy;

        -- take our best guess at the roll-up date
        case @rollupClock
            when 'ER' then                
				set @rollupDate = DATEADD(second, 
								  sahara.fn_Conf_GetConfigDayInterval('SERVER', 'UTC_ADJUSTMENT'),
								  @newUTCTimestamp);

            when 'SOURCE' then
                set @rollupDate = DATEADD(second, 
								  @UTCAdjustment,
								  @newUTCTimestamp);
        end case;
    end if;

	-- convert the roll-up date into a dates key
    set @vDatesKey = fn_Conf_DatesKey(@rollupDate);

    -- create the new roll-up batch
    insert into sahara.ROLLUP_BATCH
        (ROLLUP_SOURCE_TYPE_KEY,
         DATES_KEY, SESSION_KEY, SCHEDULE_HASH,
         ROLLUP_SOURCE_KEY, SOURCE_NAME,
         WEB_SERVER_HOST, WEB_SERVER_PORT, DB_HOST, DB_PORT,
         DB_PASSWORD, DB_PASSWORD_2,
         LOG_ARCHIVE,
         ROLLUP_DAY, START_TIME, UTC_ADJUSTMENT, IS_ROLLUP_NOW, STATUS)
    select
         ROLLUP_SOURCE_TYPE_KEY,
         @vDatesKey, @pSessionKey, @scheduleHash,
         @pSourceKey, NAME,
         WEB_SERVER_HOST, WEB_SERVER_PORT, DB_HOST, DB_PORT,
         DB_PASSWORD, DB_PASSWORD_2,
         DATEFORMAT(sahara.fn_conf_utctimestamp(), 'YYYY_MM_DD_HH_MM') || '_log' || @pLogArchiveExt,
         ROLLUP_CLOCK, @startTime, UTC_ADJUSTMENT, @isRollupNow, 'Initializing'
      from ROLLUP_SOURCE
     where ROLLUP_SOURCE_KEY = @pSourceKey;

	select max(ROLLUP_BATCH_KEY) into @batchKey from sahara.ROLLUP_BATCH where  IS_DUMMY = 'F' and PERIOD_TYPE = 'day'; 

	-- create the roll-up data set records
    insert into ROLLUP_DATA_SET
        (ROLLUP_BATCH_KEY, ROLLUP_DATA_SET_TYPE_KEY, STATUS)
    select
        @batchKey, ROLLUP_DATA_SET_TYPE_KEY, 'Pending'
      from RSOURCE_TYPE_SUPPORTED_DS
     where ROLLUP_SOURCE_TYPE_KEY = @sourceTypeKey;

    -- we've created a batch successfully, so make sure we clear the
    -- roll-up now flag
    update ROLLUP_SOURCE
       set ROLLUP_NOW = 'F'
     where current of @sourceCursor;

    close @sourceCursor;

    -- return the new batch key
    return @batchKey;

EXCEPTION
    when others then
        ROLLBACK TO SAVEPOINT RollupBatchAPICreate1SP;
		CALL SP_Error_RaiseApplicationError(20050,'raise');
END;

grant execute on sahara.fn_Batch_Create1 to sahara_app;

/*
 **********************************************************************
 * fn_Batch_DataSetStatus():
 * Sets the status for a roll-up data set which is one part of a roll-up batch. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Batch_DataSetStatus' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Batch_DataSetStatus
end if;

CREATE PROCEDURE SAHARA.fn_Batch_DataSetStatus 
(
    @pSessionKey numeric(10),
    @pBatchKey numeric(10),
    @pDataSetType varchar(64),
    @pStatus varchar(64),
    @pMessage long varchar default null
)
BEGIN
	DECLARE @FUNC varchar(64);
    DECLARE @dataSetTypeKey numeric(10);
	DECLARE @sqlstate varchar(5);
	DECLARE @count integer;

	
	DECLARE @dataSetCursor cursor FOR
        select STATUS, LAST_STATUS_UPDATE
          from ROLLUP_DATA_SET
         where ROLLUP_BATCH_KEY = @pBatchKey
           and ROLLUP_DATA_SET_TYPE_KEY = @dataSetTypeKey 
           FOR UPDATE;

    DECLARE @vOldStatus varchar(32);
    DECLARE @vOldUpdateTime timestamp;
    DECLARE @vElapsed integer;

    -- initialize global variables
	call sahara.SP_saharaAPIInit();    
	
	Set @FUNC = 'sahara.fn_Batch_DataSetStatus';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');


    -- validate the data set type and get its key
    begin
        select ROLLUP_DATA_SET_TYPE_KEY
          into @dataSetTypeKey
          from ROLLUP_DATA_SET_TYPE
         where NAME = @pDataSetType;

		SELECT SQLSTATE INTO @sqlstate;

		if @sqlstate=02000 then
			CALL SP_Error_RaiseApplicationErrorDetail 
			(
				@EInvalidArgument,
				@FUNC,
                'non-existent roll-up data set type: ' || @pDataSetType,
                @pSessionKey
			);
		end if;
    end;

    -- validate the batch key

	select 1
	  into @count
	  from ROLLUP_BATCH
	 where ROLLUP_BATCH_KEY = @pBatchKey and PERIOD_TYPE = 'day';

	 SELECT SQLSTATE INTO @sqlstate;

	 if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ERollupBatchNotFound,
			@FUNC,
			'invalid roll-up batch specified: ' || @pBatchKey,
			@pSessionKey
		);
	end if;

    -- select the existing record so we can calculate the time spent
    -- at the current status (for logging)
    open @dataSetCursor;
    fetch next @dataSetCursor into @vOldStatus, @vOldUpdateTime;

    
	SELECT SQLSTATE INTO @sqlstate;

	-- we expect to find it
	-- if @sqlstate=02000 then
	if @@fetch_status = -2 then
        close @dataSetCursor;

		-- we didn't find an existing record
		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@EInvalidArgument,
			@func,
            'there is no roll-up data set of type ' || @pDataSetType ||
            ' to update for roll-up batch ' || @pBatchKey,
			@pSessionKey
		);	
	end if;

    -- we found the record, so update it if something has changed
    if @vOldStatus <> @pStatus then
        -- log the time elapsed since the previous status
        set @vElapsed = DATEDIFF(second,@vOldUpdateTime,sahara.fn_conf_utctimestamp());

        call sahara.SP_Log_LogInfo
        (
            @pSessionKey,
            @FUNC,
            'roll-up batch: ' || @pBatchKey || ', data set: ' || @pDataSetType ||
            ' -- ' || @vOldStatus || ' -> ' || @pStatus ||
            ': ' || @vElapsed || ' secs'
        );

        -- update the record
        if @pMessage is null then
            update ROLLUP_DATA_SET
               set STATUS = @pStatus
             where current of @dataSetCursor;
        else
            update ROLLUP_DATA_SET
               set STATUS = @pStatus,
                   STATUS_MESSAGE = trim(@pMessage)
             where current of @dataSetCursor;
        end if;
    end if;

    close @dataSetCursor;

exception
    when others then
		CALL SP_Error_RaiseApplicationError(20050,'raise');
end;

grant execute on sahara.fn_Batch_DataSetStatus to sahara_app;

/*
 **********************************************************************
 * fn_Batch_DataSetStatus():
 * Sets the status for a roll-up data set which is one part of a roll-up batch. 
 * updates the status of a roll-up batch
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Batch_Status' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Batch_Status
end if;

CREATE PROCEDURE SAHARA.fn_Batch_Status 
(
    @pSessionKey numeric(10),
    @pBatchKey numeric(10),
    @pStatus varchar(64),
    @pMessage long varchar default null,
    @pOverwriteComplete bit default 0
)
BEGIN
    DECLARE @FUNC varchar(64);

    DECLARE @batchCursor CURSOR FOR
        select STATUS, substr(STATUS_MESSAGE, 1, 2),
               IS_COMPLETE, LAST_STATUS_UPDATE
          from ROLLUP_BATCH
         where ROLLUP_BATCH_KEY = @pBatchKey and IS_DUMMY = 'F'
           FOR UPDATE;

    DECLARE @vOldStatus varchar(32);
    DECLARE @vOldMessage varchar(2);
    DECLARE @vOldIsComplete char(1);
    DECLARE @vOldUpdateTime timestamp;
    DECLARE @vElapsed integer;
	DECLARE @sqlstate varchar(5);
	DECLARE @NL char(2);

	SET @NL = char(10);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();	
	
	-- we want all or nothing from this call
    savepoint RollupBatchAPIStatusSP;

	SET @func = 'sahara.fn_Batch_Status';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    -- select the existing record so we can calculate the time spent
    -- at the current status (for logging)
    open @batchCursor;
    FETCH NEXT @batchCursor into @vOldStatus, @vOldMessage,
                           @vOldIsComplete, @vOldUpdateTime;


	-- we expect to find it
	SELECT SQLSTATE INTO @sqlstate;

	-- if @sqlstate=02000 then
	if @@fetch_status = -2 then
		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@EInvalidArgument,
			@FUNC,
            'failed finding roll-up batch with key ' || @pBatchKey,
			@pSessionKey
		);
	end if;

    -- update the message if one is provided
    if trim(@pMessage) is not null then
        if @vOldMessage is null then
            -- this is the first message
            update ROLLUP_BATCH
               set STATUS_MESSAGE = trim(@pMessage)
             where current of @batchCursor;
        else if '* ' <> @vOldMessage then
				-- this is the second message
				update ROLLUP_BATCH
				   set STATUS_MESSAGE = '* ' || STATUS_MESSAGE || @NL ||
										'* ' || trim(@pMessage)
				 where current of @batchCursor;
			else
				-- this is the third or greater message
				update ROLLUP_BATCH
				   set STATUS_MESSAGE = STATUS_MESSAGE || @NL ||
										'* ' || trim(@pMessage)
				 where current of @batchCursor;
			end if;	
        end if;
    end if;

    -- update further only if something has changed
    if @vOldStatus = @pStatus then
        close @batchCursor;
        return;
    end if;
    -- else

    -- Don't overwrite complete statuses unless specifically told to.
    -- This is a hack to get around confusion between ETL and
    -- loading code.
    if fn_CharToBool(@vOldIsComplete)=1 and @pOverwriteComplete=0 then
        close @batchCursor;

        -- We'll quietly ignore this status setting.  We'd rather raise
        -- an exception, but Kettle probably isn't expecting one.
        call sahara.SP_Log_LogWarn
        (
            @pSessionKey,
            @FUNC,
            'ignoring set of status for roll-up batch ' ||
            @pBatchKey || ' from ' || @vOldStatus || ' to ' || @pStatus ||
            ' -- the batch is already complete'
        );

        return;
    end if;
    -- else

    -- log the time elapsed since the previous status
    SET @vElapsed = DATEDIFF(second,@vOldUpdateTime,sahara.fn_conf_utctimestamp());
    call sahara.SP_Log_LogInfo
    (
        @pSessionKey,
        @FUNC,
        'roll-up batch: ' || @pBatchKey || ' -- ' ||
        @vOldStatus || ' -> ' || @pStatus ||
        ': ' || @vElapsed || ' secs'
    );

	-- update the status
    update ROLLUP_BATCH
       set STATUS = @pStatus
     where current of @batchCursor;

    close @batchCursor;

EXCEPTION
    WHEN OTHERS THEN
        ROLLBACK TO SAVEPOINT RollupBatchAPIStatusSP;
		CALL SP_Error_RaiseApplicationError(20050,'raise');
end;

grant execute on sahara.fn_Batch_Status to sahara_app;

/*
 **********************************************************************
 * fn_Batch_StatusMessage():
 * 
 * Adds a status message to the roll-up batch record.
 * Each call to Status or StatusMessage that supplies a non-null message
 * results in the message being added to any existing message(s).
 * Append to the status message for a batch
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Batch_StatusMessage' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Batch_StatusMessage
end if;

CREATE PROCEDURE SAHARA.fn_Batch_StatusMessage 
(
    @pSessionKey numeric(10),
    @pBatchKey numeric(10),
    @pMessage long varchar
)
BEGIN
	DECLARE @func VARCHAR(64);
	DECLARE @vOldMessage varchar(2);
	DECLARE @sqlstate varchar(5);
	DECLARE @NL char(2);

    DECLARE @batchCursor CURSOR FOR
        select substr(STATUS_MESSAGE, 1, 2)
          from ROLLUP_BATCH
         where ROLLUP_BATCH_KEY = @pBatchKey and IS_DUMMY = 'F'
           FOR UPDATE;
	
	SET @NL = char(10);

	SET @func = 'sahara.fn_Batch_StatusMessage';
	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    if trim(@pMessage) is null then
        return;
    end if;

    -- select the existing record so we can calculate the time spent
    -- at the current status (for logging)
    open @batchCursor;
    fetch next @batchCursor into @vOldMessage;

	SELECT SQLSTATE INTO @sqlstate;

	-- we expect to find it
	-- if @sqlstate=02000 then
	if @@fetch_status = -2 then
		-- we didn't find an existing record
		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ERollupBatchNotFound,
			@func,
            'failed finding roll-up batch with key ' || @pBatchKey,
			@pSessionKey
		);	
	end if;
    -- else, we found the record

    -- update the message
    if @vOldMessage is null then
        -- this is the first message
        update ROLLUP_BATCH
           set STATUS_MESSAGE = trim(@pMessage)
         where current of @batchCursor;
    else 
		if '* ' <> @vOldMessage then
			-- this is the second message
			update ROLLUP_BATCH
			   set STATUS_MESSAGE = '* ' || STATUS_MESSAGE || @NL ||
									'* ' || trim(@pMessage)
			 where current of @batchCursor;
		else
			-- this is the third or greater message
			update ROLLUP_BATCH
			   set STATUS_MESSAGE = STATUS_MESSAGE || @NL ||
									'* ' || trim(@pMessage)
			 where current of @batchCursor;
		end if;
    end if;

    close @batchCursor;

EXCEPTION
    WHEN OTHERS THEN
		CALL SP_Error_RaiseApplicationError(20050,'raise');
end;

/*
 **********************************************************************
 * fn_Batch_UTCAdjustment():
 * 
 * Sets the UTC time adjustment value for a roll-up batch and propagates the value back to the roll-up source. 
 * This value is collected during roll-up.
 * Times in the database are stored in UTC. This value allows calculation of the roll-up source local time. 
 * Creates a new roll-up batch for a roll-up source.
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Batch_UTCAdjustment' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Batch_UTCAdjustment
end if;

CREATE PROCEDURE SAHARA.fn_Batch_UTCAdjustment 
(
	@pSessionKey numeric(10),
	@pBatchKey numeric(10),
	@pUTCAdjustment integer
) 
BEGIN
	DECLARE @UTCAdjustment integer;
	DECLARE @vOldUTCAdjustment integer;
	DECLARE @sourceKey numeric(10);
	DECLARE @sqlstate varchar(5);
	DECLARE @func varchar(64);

    DECLARE @batchCursor CURSOR FOR
        select ROLLUP_SOURCE_KEY,
               UTC_ADJUSTMENT
          from ROLLUP_BATCH
         where ROLLUP_BATCH_KEY = @pBatchKey
        FOR UPDATE;

    -- initialize global variables
	call sahara.SP_saharaAPIInit();    
	
	-- we want all or nothing from this call
    savepoint RollupBatchAPIUTCAdjustmentSP;

	SET @func = 'sahara.fn_Batch_UTCAdjustment';

	-- convert rollup source TimeZoneAdjustment minutes to seconds
    set @UTCAdjustment = @pUTCAdjustment;

	CALL SP_Log_LogDebug(@pSessionKey, @func, 'entry');

    -- select the existing record to validate the batch and get
    -- some info we need later
    open @batchCursor;

    fetch next @batchCursor into @sourceKey, @vOldUTCAdjustment;

    -- we expect to find it
    if @@fetch_status = -2 then
        -- else - we didn't find an existing record
        close @batchCursor;

		CALL SP_Error_RaiseApplicationErrorDetail (
		@ERollupBatchNotFound,
		@FUNC,
		'failed finding roll-up batch with key ' || @pBatchKey,
		@pSessionKey);
    end if;

    -- update the UTC adjustment in the batch
    if @UTCAdjustment <> @vOldUTCAdjustment then
        update ROLLUP_BATCH
           set UTC_ADJUSTMENT = @UTCAdjustment
         where current of @batchCursor;
    end if;

    close @batchCursor;

    -- propagate the UTC adjustment value back to the source
    begin
        call sahara.SP_RollupSource_UTCAdjustment(@pSessionKey, @sourceKey, @pUTCAdjustment);

    exception
        -- ignore: roll-up source not found exception - if the source
        -- is deleted after the batch is started, this may happen
        -- and is ok
        when others then
            if sqlcode <> @ERollupSourceNotFound then
				CALL SP_Error_RaiseApplicationError(20050,'raise');
            end if;
    end;

exception
    when others then
	    ROLLBACK TO SAVEPOINT RollupBatchAPIUTCAdjustmentSP;
		CALL SP_Error_RaiseApplicationError(20050,'raise');
end;

grant execute on sahara.fn_Batch_UTCAdjustment to sahara_app;

/*
 **********************************************************************
 * T_DATES_BIFER
 * 
 * Trigger to update the SAHARA.DATES table with values relative to date
 **********************************************************************
 */
if exists(select 1 from sys.systrigger where trigger_name = 'T_DATES_BIFER') then
   drop TRIGGER T_DATES_BIFER
end if;

create or replace trigger SAHARA.T_DATES_BIFER
    before insert
    on SAHARA.DATES
	REFERENCING NEW AS newrow
	FOR EACH ROW
BEGIN
    DECLARE @year varchar(4);   -- year in 4 digit format like 2011 or 2012
	DECLARE @fist_day_curr_month date;
	DECLARE @fist_day_next_month date;
	DECLARE @last_day date;

    -- make sure the date is the start of the day (truncate the time portion)
	SET newrow.ADATE = DATE(newrow.ADATE);

    -- first date of the month
    set newrow.DAY_NUMBER = datepart(DAY, newrow.ADATE);

    -- get the year in 4 digit format
	set @year = datename(YY,	newrow.ADATE);

    -- MONTH is a combination of month and year. For example 'November 2006'
    set newrow.MONTH = datename (mm, newrow.ADATE) || ' ' || @year;
    set newrow.QUARTER = 'Q' || datepart(quarter, newrow.ADATE) || ' ' || @year;

    -- first date of the year
    set newrow.DAY_NUMBER_OF_YEAR = datepart(dayofyear, newrow.ADATE);

    -- first date of the quarter
    set newrow.MONTH_NUMBER = datepart(month, newrow.ADATE);

    -- first date of the week
    set newrow.DAY_NUMBER_OF_WEEK = datepart(weekday, newrow.ADATE);

    -- day name
    set newrow.DAY_NAME = DAYNAME(newrow.ADATE);
    set newrow.DAY_NAME_ABBREVIATION = substr(DAYNAME (newrow.ADATE), 1,3);
    set newrow.WEEK_NUMBER_OF_YEAR = datepart(week, newrow.ADATE);

    -- month name
    set newrow.MONTH_NAME = datename (mm, newrow.ADATE);
    set newrow.MONTH_NAME_ABBREVIATION = substr(datename (mm, newrow.ADATE), 1, 3);

    -- quarter
    set newrow.QUARTER_NUMBER = datepart(quarter, newrow.ADATE);

    -- year number
    set newrow.YEAR_NUMBER = datepart(YY, newrow.ADATE);

    -- "is" flags - all default to false
    if newrow.DAY_NUMBER_OF_WEEK between 2 and 6 then
        set newrow.IS_WEEKDAY = 'T';
    end if;
    
	--last_day of month funda
	/* This is the first day of the current month */
	set @fist_day_curr_month = dateadd(dd,-(datepart(dd,newrow.ADATE))+1,newrow.ADATE);
	/* This is the first day of the next month */
	set @fist_day_next_month = dateadd(mm,1,@fist_day_curr_month);
	/* Finally, this is the 1ast day of the current month */
	set @last_day = dateadd(dd,-1,@fist_day_next_month);
	
	if newrow.ADATE = @last_day then
        set newrow.IS_MONTH_END = 'T';
    end if;
    if (newrow.MONTH_NUMBER in (3, 6, 9, 12)) and (newrow.IS_MONTH_END = 'T') then
        set newrow.IS_QUARTER_END = 'T';
    end if;
    if (newrow.MONTH_NUMBER = 12) and (newrow.IS_MONTH_END = 'T') then
        set newrow.IS_YEAR_END = 'T';
    end if;
end;


/*
 **********************************************************************
 * T_ROLLUP_BATCH_BIUFER
 * 
 * TRIGGER to punch various state's timestamps in ROLLUP_BATCH
 * - START_TIME_ER
 * - COMPLETION_TIME
 * - COMPLETION_TIME_ER
 * - DURATION
 * - IS_COMPLETE
 * - LAST_STATUS_UPDATE';
 **********************************************************************
 */
if exists(select 1 from sys.systrigger where trigger_name = 'T_ROLLUP_BATCH_BIUFER') then
   drop TRIGGER T_ROLLUP_BATCH_BIUFER
end if;

create or replace trigger SAHARA.T_ROLLUP_BATCH_BIUFER
    before insert,update --of "STATUS"
    order 4 on SAHARA.ROLLUP_BATCH
	REFERENCING NEW AS newrow
	            OLD AS oldrow
    FOR EACH ROW
BEGIN
    DECLARE @vUTCAdjustment integer;
    DECLARE @vISCOMPLETE char(1);
    DECLARE @vISSUCCESS char (1);
	
	Set @vUTCAdjustment = sahara.fn_Conf_GetConfigInt('SERVER', 'UTC_ADJUSTMENT');

	if inserting then
        set newrow.LAST_STATUS_UPDATE = sahara.fn_Conf_UTCTimestamp();
    else
        -- must be updating

        if update(status) then
			-- if the status didn't change, there's nothing else to do
			if oldrow.STATUS <> newrow.STATUS then
				set newrow.LAST_STATUS_UPDATE = sahara.fn_Conf_UTCTimestamp();

				-- If the batch status is already complete, we don't want to change
				-- anything that we calculated when it became complete.  If it
				-- isn't already complete, see if this update completed it.
				if oldrow.IS_COMPLETE = 'F' then
					-- decide if this is a complete status, and successful status
					select IS_COMPLETE, IS_SUCCESS
					  into @vISCOMPLETE, @vISSUCCESS
					  from sahara.ROLLUP_BATCH_STATUS
					 where STATUS = newrow.STATUS;

					Set newrow.IS_COMPLETE = @vISCOMPLETE;
					Set newrow.IS_SUCCESS = @vISSUCCESS;

					-- if this update completes the batch...
					if 'T' = newrow.IS_COMPLETE then
						set newrow.COMPLETION_TIME = sahara.fn_Conf_UTCTimestamp();
					end if;
				end if;
			end if;
		end if;
    end if;

    -- alway set these based on other columns
    set newrow.START_TIME_ER = dateadd(second,@vUTCAdjustment,newrow.START_TIME);
    set newrow.COMPLETION_TIME_ER = dateadd(second,@vUTCAdjustment,newrow.COMPLETION_TIME);
    set newrow.DURATION = datediff(second,newrow.START_TIME,newrow.COMPLETION_TIME); 
end;


/*
 **********************************************************************
 * T_ROLLUP_DATA_SET_BIUFER
 * 
 * TRIGGER to punch various state's timestamps per ROLLUP_DATA_SET.
 **********************************************************************
 */
if exists(select 1 from sys.systrigger where trigger_name = 'T_ROLLUP_DATA_SET_BIUFER') then
   drop TRIGGER T_ROLLUP_DATA_SET_BIUFER
end if;

create or replace trigger SAHARA.T_ROLLUP_DATA_SET_BIUFER
    before insert,update
    on SAHARA.ROLLUP_DATA_SET
	REFERENCING NEW AS newrow
	            OLD AS oldrow
    FOR EACH ROW
BEGIN
    DECLARE @vUTCAdjustment integer;
    DECLARE @vDuration integer;

	if inserting then
        set newrow.LAST_STATUS_UPDATE = sahara.fn_Conf_UTCTimestamp();
    else
        -- must be updating
        if oldrow.STATUS <> newrow.STATUS then
            set newrow.LAST_STATUS_UPDATE = sahara.fn_Conf_UTCTimestamp();
            set @vDuration = datediff(second, oldrow.LAST_STATUS_UPDATE, newrow.LAST_STATUS_UPDATE );

            case oldrow.STATUS
                when 'Uploading' then
                    set newrow.UPLOADING_DURATION = @vDuration;

                when 'Staging' then
                    set newrow.STAGING_DURATION = @vDuration;

                when 'Loading' then
                    set newrow.LOADING_DURATION = @vDuration;

                else
                    begin
                    end;
            end case;
        end if;
    end if;
end;


/*
 **********************************************************************
 * T_ROLLUP_BATCH_AIU
 * 
 * AFTER INSERT, UPDATE TRIGGER which internally calls corresponding 
 * procedures to update various flags of ROLLUP_BATCH table
 **********************************************************************
 */
if exists(select 1 from sys.systrigger where trigger_name = 'T_ROLLUP_BATCH_AIU') then
   drop TRIGGER T_ROLLUP_BATCH_AIU
end if;

create or replace trigger SAHARA.T_ROLLUP_BATCH_AIU
    after insert, update -- of STATUS
    order 3 on SAHARA.ROLLUP_BATCH
	REFERENCING NEW AS newrow
	            OLD AS oldrow
    FOR EACH ROW
BEGIN
	if inserting then
		call sp_Batch_Trigger_AfterInsert(newrow.ROLLUP_BATCH_KEY);
    else
		if update(status) then
			call sp_Batch_Trigger_AfterUpdateStatus(oldrow.ROLLUP_BATCH_KEY);
		end if;
    end if;

end;

/*
 **********************************************************************
 * sp_Batch_Trigger_AfterInsert
 * 
 * This procedure gets called from AFTER INSERT trigger on ROLLUP_BATCH
 * It marks the value for IS_LATEST flag T, if not marked and it should be
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Batch_Trigger_AfterInsert' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Batch_Trigger_AfterInsert
end if;

CREATE PROCEDURE SAHARA.sp_Batch_Trigger_AfterInsert (@pBatchkey numeric(10)) 
BEGIN
    DECLARE @vRollupBatchCursor CURSOR FOR
        select ROLLUP_BATCH.ROLLUP_SOURCE_KEY, ROLLUP_BATCH.START_TIME,
               ROLLUP_BATCH.IS_LATEST
          from ROLLUP_BATCH
         where ROLLUP_BATCH_KEY = @pBatchkey and IS_DUMMY = 'F'
           for update of IS_LATEST;

    DECLARE @vCount integer;
	DECLARE @vSourceKey numeric(10);
	DECLARE @vStartTime datetime;
	DECLARE @vIsLatest char(1);

	-- get information about the new batch
    open @vRollupBatchCursor;
    fetch next @vRollupBatchCursor into @vSourceKey, @vStartTime, @vIsLatest;

    -- if it's not marked as the latest, see if it should be
    if 'T' <> @vIsLatest then
        -- see if there are any roll-up batches that better qualify
        -- as the latest
        select count(*)
          into @vCount
          from ROLLUP_BATCH
         where ROLLUP_SOURCE_KEY = @vSourceKey
           and START_TIME > @vStartTime
           and PERIOD_TYPE = 'day';

        -- if there's none better, make this the latest
        if 0 = @vCount then
            -- first un-mark the current latest
            update ROLLUP_BATCH
               set IS_LATEST = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and 'T' = IS_LATEST
               and PERIOD_TYPE = 'day';

            update ROLLUP_BATCH
               set IS_LATEST = 'T'
             where current of @vRollupBatchCursor;
        end if;
    end if;

    close @vRollupBatchCursor;
end;

/*
 **********************************************************************
 * sp_Batch_Trigger_AfterUpdateStatus
 * 
 * This procedure gets called from AFTER UPDATE trigger on ROLLUP_BATCH
 * It updates ROLLUP_BATCH table columns as follows:
 * 	IS_LATEST_COMPLETE,
 * 	IS_LATEST_SUCCESS,
 * 	IS_LAST_OF_MONTH,
 * 	IS_LAST_OF_QUARTER,
 * 	IS_LAST_OF_YEAR;
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Batch_Trigger_AfterUpdateStatus' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Batch_Trigger_AfterUpdateStatus
end if;

CREATE PROCEDURE SAHARA.sp_Batch_Trigger_AfterUpdateStatus (@pBatchkey numeric(10)) 
BEGIN

	DECLARE  @vRollupBatchCursor cursor FOR
		 select ROLLUP_BATCH.ROLLUP_SOURCE_KEY, ROLLUP_BATCH.START_TIME,
			ROLLUP_BATCH.IS_SUCCESS,
			ROLLUP_BATCH.IS_LATEST_COMPLETE, ROLLUP_BATCH.IS_LATEST_SUCCESS,
			DATES.WEEK_NUMBER_OF_YEAR as WEEK, ROLLUP_BATCH.IS_LAST_OF_WEEK,
			DATES.MONTH, ROLLUP_BATCH.IS_LAST_OF_MONTH,
			DATES.QUARTER, ROLLUP_BATCH.IS_LAST_OF_QUARTER,
			DATES.YEAR_NUMBER, ROLLUP_BATCH.IS_LAST_OF_YEAR,
                        DATES.ADATE
		  from ROLLUP_BATCH
		  join DATES on ROLLUP_BATCH.DATES_KEY=DATES.DATES_KEY
		  where ROLLUP_BATCH.ROLLUP_BATCH_KEY = @pBatchkey and ROLLUP_BATCH.PERIOD_TYPE = 'day' 
		  for update of IS_LATEST_COMPLETE,
                                IS_LATEST_SUCCESS,
                                IS_LAST_OF_WEEK,
                                IS_LAST_OF_MONTH,
                                IS_LAST_OF_QUARTER,
                                IS_LAST_OF_YEAR;

	DECLARE @vCount integer;
	DECLARE @vSourceKey numeric(10);
	DECLARE @vStartTime datetime;
	DECLARE @vIsSuccess char(1);
	DECLARE @vIsLatestComplete char(1);
	DECLARE @vIsLatestSuccess char(1);
	DECLARE @vWeek numeric(2);
	DECLARE @vIsLastOfWeek char(1);
	DECLARE @vMonth varchar(128);
	DECLARE @vIsLastOfMonth char(1);
	DECLARE @vQuarter varchar(128);
	DECLARE @vIsLastOfQuarter char(1);
	DECLARE @vYear numeric(4); 
	DECLARE @vIsLastOfYear char(1);
        DECLARE @pPeriodStart date;
        DECLARE @pPeriodEnd date;
        DECLARE @pADATE date;


	-- get information about the completed batch
    open @vRollupBatchCursor;
    fetch @vRollupBatchCursor into @vSourceKey,@vStartTime,@vIsSuccess,@vIsLatestComplete,@vIsLatestSuccess,
		                   @vWeek,@vIsLastOfWeek,@vMonth,@vIsLastOfMonth,@vQuarter,@vIsLastOfQuarter,
                                   @vYear,@vIsLastOfYear, @pADATE;

    -- if it's not marked as the latest complete, see if it should be
    if 'T' <> @vIsLatestComplete then
        -- see if there are any roll-up batches that better qualify
        -- as the latest complete
        select count(*)
          into @vCount
          from ROLLUP_BATCH
         where ROLLUP_SOURCE_KEY = @vSourceKey
           and 'T' = IS_COMPLETE
           and PERIOD_TYPE = 'day'
           and START_TIME > @vStartTime;

        -- if there's none better, make this the latest complete
        if 0 = @vCount then
            -- first un-mark the current latest complete
            update ROLLUP_BATCH
               set IS_LATEST_COMPLETE = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and PERIOD_TYPE = 'day'
               and 'T' = IS_LATEST_COMPLETE;

            update ROLLUP_BATCH
               set IS_LATEST_COMPLETE = 'T'
			 where ROLLUP_BATCH_KEY = @pBatchkey
             --where current of @vRollupBatchCursor;
        end if;
    end if;

    -- we only process the remainder of the logic if the roll-up batch is
    -- a success
    if 'T' <> @vIsSuccess then
        close @vRollupBatchCursor;
        return;
    end if;

    -- if it's not marked as the latest success, see if it should be
    if 'T' <> @vIsLatestSuccess then
        -- see if there are any roll-up batches that better qualify
        -- as the latest success
        select count(*)
          into @vCount
          from ROLLUP_BATCH
         where ROLLUP_SOURCE_KEY = @vSourceKey
           and 'T' = IS_COMPLETE and 'T' = IS_SUCCESS
           and PERIOD_TYPE = 'day'
           and START_TIME > @vStartTime;

        -- if there's none better, make this the latest success
        if 0 = @vCount then
            -- first un-mark the current latest success
            update ROLLUP_BATCH
               set IS_LATEST_SUCCESS = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and PERIOD_TYPE = 'day'
               and 'T' = IS_LATEST_SUCCESS;

            update ROLLUP_BATCH
               set IS_LATEST_SUCCESS = 'T'
			 where ROLLUP_BATCH_KEY = @pBatchkey
             --where current of @vRollupBatchCursor;
        end if;
    end if;

    -- if it's not marked as the latest success for the week, see if it should be
    if 'T' <> @vIsLastOfWeek then
        -- see if there are any roll-up batches that better qualify
        -- as the latest success for the week

        set @pPeriodStart = dateadd(dd,-(datepart(weekday,@pADATE))+1,@pADATE);
        set @pPeriodEnd   = dateadd(week,1, @pPeriodStart);

        select count(*)
          into @vCount
          from ROLLUP_BATCH
          join DATES on ROLLUP_BATCH.DATES_KEY = DATES.DATES_KEY and
                        DATES.ADATE >= @pPeriodStart and DATES.ADATE < @pPeriodEnd
         where ROLLUP_SOURCE_KEY = @vSourceKey and PERIOD_TYPE = 'day'
           and 'T' = IS_COMPLETE and 'T' = IS_SUCCESS
           and START_TIME > @vStartTime;

        -- if there's none better, make this the latest success for the week
        if 0 = @vCount then
            update ROLLUP_BATCH
               set IS_LAST_OF_WEEK = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and 'T' = IS_LAST_OF_WEEK
               and PERIOD_TYPE = 'day'
               and DATES_KEY in (select DATES_KEY
                                 from DATES
                                 where ADATE >= @pPeriodStart and ADATE < @pPeriodEnd);

            update ROLLUP_BATCH
               set IS_LAST_OF_WEEK = 'T'
			 where ROLLUP_BATCH_KEY = @pBatchkey
             --where current of @vRollupBatchCursor;
        end if;
    end if;

    -- if it's not marked as the latest success for the month, see if it should be
    if 'T' <> @vIsLastOfMonth then
        -- see if there are any roll-up batches that better qualify
        -- as the latest success for the month
        select count(*)
          into @vCount
          from ROLLUP_BATCH
          join DATES on ROLLUP_BATCH.DATES_KEY = DATES.DATES_KEY
           and DATES.MONTH = @vMonth
           and DATES.YEAR_NUMBER = @vYear
         where ROLLUP_SOURCE_KEY = @vSourceKey
           and 'T' = IS_COMPLETE and 'T' = IS_SUCCESS
           and PERIOD_TYPE = 'day'
           and START_TIME > @vStartTime;

        -- if there's none better, make this the latest success for the month
        if 0 = @vCount then
            update ROLLUP_BATCH
               set IS_LAST_OF_MONTH = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and 'T' = IS_LAST_OF_MONTH
               and PERIOD_TYPE = 'day'
               and DATES_KEY in (select DATES_KEY
                                   from DATES
                                  where MONTH = @vMonth and
                                        YEAR_NUMBER = @vYear);

            update ROLLUP_BATCH
               set IS_LAST_OF_MONTH = 'T'
			 where ROLLUP_BATCH_KEY = @pBatchkey
             --where current of @vRollupBatchCursor;
        end if;
    end if;

    -- if it's not marked as the latest success for the quarter, see if it should be
    if 'T' <> @vIsLastOfQuarter then
        -- see if there are any roll-up batches that better qualify
        -- as the latest success for the quarter
        select count(*)
          into @vCount
          from ROLLUP_BATCH
          join DATES on ROLLUP_BATCH.DATES_KEY = DATES.DATES_KEY
           and DATES.QUARTER = @vQuarter
           and DATES.YEAR_NUMBER = @vYear
         where ROLLUP_SOURCE_KEY = @vSourceKey
           and 'T' = IS_COMPLETE and 'T' = IS_SUCCESS
           and PERIOD_TYPE = 'day'
           and START_TIME > @vStartTime;

        -- if there's none better, make this the latest success for the quarter
        if 0 = @vCount then
            update ROLLUP_BATCH
               set IS_LAST_OF_QUARTER = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and 'T' = IS_LAST_OF_QUARTER
               and PERIOD_TYPE = 'day'
               and DATES_KEY in (select DATES_KEY
                                   from DATES
                                  where QUARTER = @vQuarter and
                                        YEAR_NUMBER = @vYear);

            update ROLLUP_BATCH
               set IS_LAST_OF_QUARTER = 'T'
			 where ROLLUP_BATCH_KEY = @pBatchkey
             --where current of @vRollupBatchCursor;
        end if;
    end if;

    -- if it's not marked as the latest success for the year, see if it should be
    if 'T' <> @vIsLastOfYear then
        -- see if there are any roll-up batches that better qualify
        -- as the latest success for the year
        select count(*)
          into @vCount
          from ROLLUP_BATCH
          join DATES on ROLLUP_BATCH.DATES_KEY = DATES.DATES_KEY
           and DATES.YEAR_NUMBER = @vYear
         where ROLLUP_SOURCE_KEY = @vSourceKey
           and 'T' = IS_COMPLETE and 'T' = IS_SUCCESS
           and PERIOD_TYPE = 'day'
           and START_TIME > @vStartTime;

        -- if there's none better, make this the latest success for the year
        if 0 = @vCount then
            update ROLLUP_BATCH
               set IS_LAST_OF_YEAR = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and 'T' = IS_LAST_OF_YEAR
               and PERIOD_TYPE = 'day'
               and DATES_KEY in (select DATES_KEY
                                   from DATES
                                  where YEAR_NUMBER = @vYear);

            update ROLLUP_BATCH
               set IS_LAST_OF_YEAR = 'T'
			 where ROLLUP_BATCH_KEY = @pBatchkey
             --where current of @vRollupBatchCursor;
        end if;
    end if;

    close @vRollupBatchCursor;
end;

/*
 **********************************************************************
 * sp_Update_Rollup_Batch_IsLastOfWeek_column
 * 
 * This procedure gets called at the end of VOM-ES 5.0 upgrade script
 * It updates each ROLLUP_BATCH:IS_LAST_OF_WEEK column with batchkey
 * = @pBatchkey
 **********************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Update_Rollup_Batch_IsLastOfWeek_column' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Update_Rollup_Batch_IsLastOfWeek_column
end if;

CREATE PROCEDURE SAHARA.sp_Update_Rollup_Batch_IsLastOfWeek_column (@pBatchkey numeric(10)) 
BEGIN

	DECLARE  @vRollupBatchCursor cursor FOR
		 select ROLLUP_BATCH.ROLLUP_SOURCE_KEY, 
                        ROLLUP_BATCH.START_TIME,ROLLUP_BATCH.IS_SUCCESS,
		        DATES.YEAR_NUMBER as YEAR,
		        DATES.ADATE as ADATE,
		        DATES.WEEK_NUMBER_OF_YEAR as WEEK,
                        ROLLUP_BATCH.IS_LAST_OF_WEEK
		 from ROLLUP_BATCH
		 join DATES on ROLLUP_BATCH.DATES_KEY=DATES.DATES_KEY
		 where ROLLUP_BATCH.ROLLUP_BATCH_KEY = @pBatchkey
                       and PERIOD_TYPE = 'day'
		 for update of IS_LAST_OF_WEEK;

	DECLARE @vCount integer;
	DECLARE @vSourceKey numeric(10);
	DECLARE @vStartTime datetime;
	DECLARE @vIsSuccess char(1);
	DECLARE @vWeek numeric(2);
	DECLARE @vYear numeric(4);
	DECLARE @vIsLastOfWeek char(1);
	DECLARE @vAdate varchar(16);
        DECLARE @pPeriodStart date;
        DECLARE @pPeriodEnd date;

	-- get information about the completed batch
    open @vRollupBatchCursor;
    fetch @vRollupBatchCursor into @vSourceKey,@vStartTime,@vIsSuccess,@vYear,@vAdate,@vWeek,@vIsLastOfWeek;

    -- we only process the remainder of the logic if the roll-up batch is
    -- a success
    if 'T' <> @vIsSuccess then
        close @vRollupBatchCursor;
        return;
    end if;

    -- if it's not marked as the latest success for the week, see if it should be
    if 'T' <> @vIsLastOfWeek then
        -- see if there are any roll-up batches that better qualify
        -- as the latest success for the week

        set @pPeriodStart = dateadd(dd,-(datepart(weekday,@vAdate))+1,@vAdate);
        set @pPeriodEnd   = dateadd(week,1, @pPeriodStart);

        select count(*)
          into @vCount
          from ROLLUP_BATCH
          join DATES on ROLLUP_BATCH.DATES_KEY = DATES.DATES_KEY and
                        DATES.ADATE >= @pPeriodStart and DATES.ADATE < @pPeriodEnd
         where ROLLUP_SOURCE_KEY = @vSourceKey
           and 'T' = IS_COMPLETE and 'T' = IS_SUCCESS
           and PERIOD_TYPE = 'day'
           and START_TIME > @vStartTime;

        -- if there's none better, make this the latest success for the week
        if 0 = @vCount then
            update ROLLUP_BATCH
               set IS_LAST_OF_WEEK = 'F'
             where ROLLUP_SOURCE_KEY = @vSourceKey
               and 'T' = IS_LAST_OF_WEEK
               and PERIOD_TYPE = 'day'
               and DATES_KEY in (select DATES_KEY
                                   from DATES
                                  where ADATE >= @pPeriodStart and ADATE < @pPeriodEnd);

            update ROLLUP_BATCH
               set IS_LAST_OF_WEEK = 'T'
			 where ROLLUP_BATCH_KEY = @pBatchkey;
             --where current of @vRollupBatchCursor;
            -- call SP_Log_LogDebug(null, 'SAHARA.sp_Update_Rollup_Batch_IsLastOfWeek', @vAdate );
        end if;
    end if;

    close @vRollupBatchCursor;
end;


/*
 **********************************************************************
 * sp_Update_Rollup_Batch_IsLastOfWeek_columns
 * 
 * This procedure gets called at the end of VOM-ES 5.0 upgrade script
 * It updates each ROLLUP_BATCH:IS_LAST_OF_WEEK column with batchkey
 * = @pBatchkey
 **********************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Update_Rollup_Batch_IsLastOfWeek_columns' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Update_Rollup_Batch_IsLastOfWeek_columns
end if;

CREATE PROCEDURE SAHARA.sp_Update_Rollup_Batch_IsLastOfWeek_columns () 
BEGIN

	DECLARE @vCount integer;
	DECLARE @vSourceKey numeric(10);
	DECLARE @vStartTime datetime;
	DECLARE @vIsSuccess char(1);
	DECLARE @vWeek varchar(128);
	DECLARE @vIsLastOfWeek char(1);

    for rollup_batch_entries as rollup_batch_entries_loop cursor for
        select ROLLUP_BATCH_KEY as batchkey
        from ROLLUP_BATCH
        join DATES on ROLLUP_BATCH.DATES_KEY=DATES.DATES_KEY
        where PERIOD_TYPE = 'day'
        order by adate, batchkey asc
    do
        call SAHARA.sp_Update_Rollup_Batch_IsLastOfWeek_column(batchkey);
    end for;
    commit;

end;


/*
 **********************************************************************
 * sp_summarization_purge_tables
 * 
 * This procedure gets called daily from db event.
 * It checks last_summarization_purge time in configuration.
 * If current date and the last update time is greater than (TREND_DATA_RETENTION,SUMMARIZATION_PURGING_FREQ)
 * It will start the purg/summarization process.
 **********************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'sp_summarization_purge_tables' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_summarization_purge_tables
end if;

-- @remove_old_sample_data is by default on. 
-- But for very first time, we don't want to do it.
CREATE PROCEDURE SAHARA.sp_summarization_purge_tables (@remove_old_sample_data int default 1) 
BEGIN

   DECLARE @purge_older_than_months       int;
   DECLARE @purge_enabled                 int;
   DECLARE @summarization_enabaled        int;
   DECLARE @summarization_older_than_days int;
   DECLARE @summarization_purge_freq      int;
   DECLARE @keep_daily_min                int;
   DECLARE @last_summarization_purge_time date;
   DECLARE @dates_key                     int;
   DECLARE @adate                         date;
   DECLARE @error                         integer;
   DECLARE @sqlstate                      varchar(5);
   DECLARE @serror                        varchar(512);
   DECLARE @section                       varchar(128);

   call SAHARA.SP_saharaAPIInit();
   call SP_Log_LogDebug(null, 'SAHARA.sp_summarization_purge_tables', 'enter');

   set @last_summarization_purge_time = 
       sahara.fn_conf_GetConfigDate('TREND_DATA_RETENTION', 'LAST_SUMMARIZATION_PURGING');
   set @summarization_purge_freq      = 
       sahara.fn_conf_GetConfigInt('TREND_DATA_RETENTION', 'SUMMARIZATION_PURGING_FREQ');
   set @purge_older_than_months       = 
       sahara.fn_conf_GetConfigInt('TREND_DATA_RETENTION', 'PURGE_DATA_OLDER_THAN_MONTHS');
   set @purge_enabled                 = 
       sahara.fn_GetConfigBool('TREND_DATA_RETENTION', 'AUTO_PURGE_ENABLED');
   set @summarization_enabaled        = 
       sahara.fn_GetConfigBool('TREND_DATA_RETENTION', 'AUTO_SUMMARIZATION_ENABLED');
   set @summarization_older_than_days = 
       sahara.fn_conf_GetConfigInt('TREND_DATA_RETENTION', 'SUMMARIZE_DATA_AFTER_DAYS');
   set @keep_daily_min                = 
       sahara.fn_conf_GetConfigInt('TREND_DATA_RETENTION', 'KEEP_DAILY_SAMPLES_FOR_DAYS_MIN');

   -- Determine whether it is too short to process
   if datediff(day, @last_summarization_purge_time, date(now(*))) < @summarization_purge_freq
   then
      -- message 'do nothing' to log;
      return;
   end if;

   for DateEntries as date_cursor cursor for 
                      select dates_key, adate from sahara.dates 
                      where datediff(day, adate, date(now(*))) > @keep_daily_min
                      order by adate asc
   do
   BEGIN

         set @adate = adate;
         set @dates_key = dates_key;

         call SP_Log_LogDebug(null, 'SAHARA.sp_summarization_purge_tables', 'Working on ADATE = "' + convert(varchar, @adate) + '"' );
         savepoint purge_summarization_sp;

         if @purge_enabled = 1 and datediff (day, @adate, date(now(*))) > @purge_older_than_months * 30
         then
                for rollup_batch_entries as rollup_batch_entries_loop1 cursor for
                  select ROLLUP_BATCH_KEY as batchkey
                  from SAHARA.ROLLUP_BATCH
                  where SAHARA.ROLLUP_BATCH.DATES_KEY=@dates_key 
                  order by SAHARA.ROLLUP_BATCH.ROLLUP_BATCH_KEY asc
                do
                        set @section = 'purge rollup_batch entry';
                        call SP_Log_LogDebug(null, 'SAHARA.sp_summarization_purge_tables', 
                                                   'Purge data on batchkey = ' + convert(varchar, batchkey) );
                        
                        DELETE FROM SAHARA.ROLLUP_BATCH WHERE SAHARA.ROLLUP_BATCH.ROLLUP_BATCH_KEY=batchkey;
                end for;

                set @section = 'purge dates entry';
                call SP_Log_LogDebug(null, 'SAHARA.sp_summarization_purge_tables', 'Date purged' );
                DELETE FROM SAHARA.DATES WHERE SAHARA.DATES.DATES_KEY = @dates_key; 
                
         end if;

         if ( @remove_old_sample_data = 1 )
         then
            if @summarization_enabaled = 1 and datediff (day, @adate, date(now(*))) > @summarization_older_than_days
            then

                 for rollup_batch_entries as rollup_batch_entries_loop cursor for
                     select ROLLUP_BATCH_KEY as batchkey
                     from SAHARA.ROLLUP_BATCH
                     where SAHARA.ROLLUP_BATCH.DATES_KEY=@dates_key     and
                           SAHARA.ROLLUP_BATCH.PERIOD_TYPE = 'day'
                     order by SAHARA.ROLLUP_BATCH.ROLLUP_BATCH_KEY asc
                 do
                           call SP_Log_LogDebug(null, 'SAHARA.sp_summarization_purge_tables', 
                                                      'Summarization data on batchkey = ' + convert(varchar, batchkey) );
                           set @section = 'summarization rollup_batch entry';
                           DELETE FROM SAHARA.ROLLUP_BATCH WHERE SAHARA.ROLLUP_BATCH.ROLLUP_BATCH_KEY=batchkey;
                           -- This will FK cascade delete all associate records
                 end for;

            end if;
         end if;

         commit;
   EXCEPTION
         WHEN OTHERS THEN
        SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
        IF ( @error <> 0 )
        THEN
            BEGIN
                call SP_Log_LogError(null, 'SAHARA.sp_summarization_purge_tables', 
                                           string (' Error failed at    = ', 'sahara.sp_summarization_purge_tables:'+@section ) +
                                           string ('          @error    = ', @error ) +
                                           string ('         @serrir    = ', @serror ) +
                                           string ('       @sqlstate    = ', @sqlstate ));
                rollback to savepoint purge_summarization_sp;
            END;
        END IF;
   END;
   end for;

   -- Update configuration(TREND_DATA_RETENTION, LAST_SUMMARIZATION_PURGING)
   call SAHARA.fn_Configuration_InstallDate (
        'TREND_DATA_RETENTION',                     -- section
        'LAST_SUMMARIZATION_PURGING',               -- name
        date(now(*)),                               -- value
        'Date of the last Summarization/Purging',     -- display name
        -- description
        '',
        0,
        1
    );

   call SP_Log_LogDebug(null, 'SAHARA.sp_summarization_purge_tables', 'exit');

END;

if exists(select 1 from sys.sysevent where event_name = 'EV_PURGE_SUMMARIZATION' and user_name(creator) = 'SAHARA') then
   drop event SAHARA.EV_PURGE_SUMMARIZATION
end if;

create event SAHARA.EV_PURGE_SUMMARIZATION
schedule
        start time '01:00:00'
        on ( 'Monday', 'Tuesday', 'Wednesday', 'Thursday',
                'Friday','Saturday', 'Sunday' )
handler
begin
           -- TODO: data migration, missing data treatment
           call SAHARA.SP_Missing_And_Summarization_after_Db_Upgrade(); -- executes only once.
           -- TODO: daily missing data treatment
           call SAHARA.SP_DAILY_SUMMARIZATION(date(dateadd(dd, -1, date(now(*)))));
           call SAHARA.sp_summarization_purge_tables();
end;

-- upgrade case, disable event now. Will enable the event at the end of database upgrade.
if exists (select 1 from sahara.rollup_batch)
then
   alter event EV_PURGE_SUMMARIZATION disable;
end if;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_YEAR_PERIOD_UPDATE_VIEW' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_YEAR_PERIOD_UPDATE_VIEW
end if;

CREATE PROCEDURE SAHARA.SP_YEAR_PERIOD_UPDATE_VIEW()

BEGIN
    DECLARE @FUNC VARCHAR(35);
    DECLARE @vMakeList LONG VARCHAR;
    DECLARE @vSQL LONG VARCHAR;
    DECLARE @vViewName LONG VARCHAR;

    BEGIN

      ---- Intialize connection variables

      call sahara.SP_saharaAPIInit();

      set @FUNC = 'SP_YEAR_PERIOD_UPDATE_VIEW';

      CALL SP_Log_LogDebug(NULL, @FUNC, 'Entering:');

      SET @vViewName = 'fact_table_cols_view';

      SET @vSQL =  'CALL SAHARA.SP_DROP_OBJECT_SAHARA(''VIEW'','''|| @vViewName ||'''); ';

      SET @vSQL = @vSQL ||' create view SAHARA.' || @vViewName ||' as ' ||
            ' Select tab.DEST_TABLE as table_name,
                     Col.column_name as column_name,
                     Case when col.column_name like ''%_KEY'' then ''key''
                          else col.aggregate_type end as column_type
              From ROLLUP_DATA_SET_TYPE tab
                  inner join COLUMN_METADATA_BASE col on
                        tab.STEREOTYPE = ''FACT'' and
                        tab.DEST_TABLE = col.TABLE_NAME
              order by table_name, column_type, column_name';

        CALL SP_Log_LogDebug(NULL, @FUNC, 'SQL: ' || @vSQL);

        EXECUTE IMMEDIATE @vSQL;

   END;
END;

IF EXISTS(SELECT 1 FROM SYS.SYSTABLE WHERE TABLE_NAME='fact_table_cols_view' AND TABLE_TYPE='VIEW') THEN
   DROP VIEW fact_table_cols_view
END IF;

call SAHARA.SP_YEAR_PERIOD_UPDATE_VIEW();

/*
 **************************************************************************
 * SP_DAILY_SUMMARIZATION()
 * Summarize sampled data daily.
 * It iterates through rollup_sources.
 **************************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DAILY_SUMMARIZATION' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DAILY_SUMMARIZATION
end if;

CREATE PROCEDURE SAHARA.SP_DAILY_SUMMARIZATION(@pADATE date)

BEGIN

   DECLARE @FUNC                       varchar(256);

   DECLARE @pRollupBatch               numeric(10);
   DECLARE @pDates_key                 numeric(10);
   DECLARE @pYear_Period_key           numeric(10);
   DECLARE @pRollup_source_key         numeric(10);

   DECLARE @pADATE_Yesterday           date;
   DECLARE @pADATE_PrevSuccRollup      date;

   DECLARE @error                      integer;
   DECLARE @sqlstate                   varchar(5);
   DECLARE @serror                     varchar(512);
   DECLARE @section                    varchar(512);

   DECLARE @pQuarterNumber             int;
   DECLARE @pMonthNumber               int;
   DECLARE @pYearNumber                int;
   DECLARE @pWeekNumber                int;
   DECLARE @last_summarization_time    date;

   set @FUNC = 'SP_DAILY_SUMMARIZATION';

   call SAHARA.SP_saharaAPIInit();
   call SP_Log_LogDebug(null, @FUNC, 'enter');

   set @last_summarization_time =
       sahara.fn_conf_GetConfigDate('TREND_DATA_RETENTION', 'LAST_SUMMARIZATION_TIME');

   set @pADATE_Yesterday = @pADATE;

   if @pADATE_Yesterday = @last_summarization_time
   then
       return; -- summarization has been executed
   end if;

   for rollup_source_entry as rollup_source_loop cursor for
          select rs.rollup_source_key rsk
          from sahara.rollup_source rs
            inner join sahara.rollup_batch rb on rs.rollup_source_key = rb.rollup_source_key
            inner join sahara.dates dates     on rb.dates_key = dates.dates_key
          where dates.adate = @pADATE_Yesterday and
                rb.rollup_batch_key in (select max(rollup_batch_key) from sahara.rollup_batch where is_success = 'T' group by dates_key)
          order by rsk asc
   do

       select max(dates.adate) into @pADATE_PrevSuccRollup
       from sahara.rollup_batch rb
            inner join sahara.dates dates on rb.dates_key = dates.dates_key
       where dates.adate < @pADATE_Yesterday and
             rb.rollup_source_key = rsk      and
             rb.rollup_batch_key in
               (select max(rollup_batch_key) from sahara.rollup_batch where is_success = 'T' and rollup_source_key = rsk group by dates_key);

       if @pADATE_PrevSuccRollup is null
       then
           set @pADATE_PrevSuccRollup = @pADATE_Yesterday;
       end if;

       -- This is to adjust the case that rollup is fine, but unable to process DAILY_SUMMARIZATION.
       if @pADATE_PrevSuccRollup > @last_summarization_time
       then
           set @pADATE_PrevSuccRollup = @last_summarization_time;
       end if;

       set @pYearNumber = 0;

       for adate_entry as adate_loop cursor for
           select adate as adate
           from sahara.dates da
           where adate >= @pADATE_PrevSuccRollup and adate <= @pADATE_Yesterday
           order by adate asc
       do

       begin

           savepoint SP_DAILY_SUMMARIZATION;

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','week');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'week');

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','quarter');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'month');

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','month');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'quarter');

           set @section = string('rollup_source_key=',rsk, 'adate=',adate, ',period=','year');
           call sahara.SP_SUMMARIZE_PERIOD(rsk, adate, 'year');

           commit;
       EXCEPTION
             WHEN OTHERS THEN
            SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
            IF ( @error <> 0 )
            THEN
                BEGIN
                    call SP_Log_LogError(null, @FUNC,
                                               string ('          @error    = ', @error ) +
                                               string ('         @serror    = ', @serror ) +
                                               string ('       @sqlstate    = ', @sqlstate ) +
                                               string (' Error failed at    = ', @FUNC+':'+@section ) );
                    rollback to savepoint SP_DAILY_SUMMARIZATION;
                END;
            END IF;

       end;

       end for;

   end for;

   call SAHARA.fn_Configuration_InstallDate (
        'TREND_DATA_RETENTION',                     -- section
        'LAST_SUMMARIZATION_TIME',                  -- name
        date(dateadd( day, -1, now(*))),            -- value
        'Last Time the Data Summarization is processed.',       -- display name
        -- description
        '',
        0,
        1
    );

   call SP_Log_LogDebug(null, @FUNC, 'exit');


END;

/*
 **************************************************************************
 * SP_Missing_And_Summarization_after_Db_Upgrade()
 * Migrate all the existing samples.
 * It iterates through dates.
 **************************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_Missing_And_Summarization_after_Db_Upgrade' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_Missing_And_Summarization_after_Db_Upgrade
end if;

CREATE PROCEDURE SAHARA.SP_Missing_And_Summarization_after_Db_Upgrade(@pMinADATE date default null, @pMaxADATE date default null)

BEGIN

   DECLARE @FUNC                       varchar(256);

   DECLARE @pRollupBatch               numeric(10);
   DECLARE @pDates_key                 numeric(10);
   DECLARE @pYear_Period_key           numeric(10);
   DECLARE @pRollup_source_key         numeric(10);

   DECLARE @pADATE_Yesterday           date;
   DECLARE @pADATE_PrevSuccRollup      date;
   DECLARE @pYearCount                 int;
   DECLARE @pQuarterCount              int;
   DECLARE @pMonthCount                int;
   DECLARE @pWeekCount                 int;
   DECLARE @pQuarterNumber             int;
   DECLARE @pMonthNumber               int;
   DECLARE @pYearNumber                int;
   DECLARE @pWeekNumber                int;

   DECLARE @pMigrateSummaryData        char(1);

   DECLARE @error                      integer;
   DECLARE @sqlstate                   varchar(5);
   DECLARE @serror                     varchar(512);
   DECLARE @section                    varchar(512);
   DECLARE @migrate_summary_data       int;

   set @FUNC = 'SP_Missing_And_Summarization_after_Db_Upgrade';
   message 'SAHARA.SP_Missing_And_Summarization_after_Db_Upgrade' to client;

   call SAHARA.SP_saharaAPIInit();
   call SP_Log_LogDebug(null, @FUNC, 'enter');

   -- TODO: populate Missing data treatment.

   set @migrate_summary_data =
       sahara.fn_GetConfigBool('TREND_DATA_RETENTION', 'MIGRATE_SUMMARIZED_DATA');

   if @migrate_summary_data = 1
   then
       -- migration had done before, skip.
       call SP_Log_LogDebug(null, @FUNC, 'Already migrated, exit');
       return;
   end if;

   set @pYearCount = 0;
   set @pQuarterCount = 0;
   set @pMonthCount = 0;
   set @pWeekCount = 0;

   set @pYearNumber = 0;

   if @pMinADATE is null
   then
      select min(adate) into @pMinADATE from sahara.dates;
   end if;

   if @pMaxADATE is null
   then
      select max(adate) into @pMaxADATE from sahara.dates;
      if @pMaxADATE > date(dateadd( day, -1, now(*)))
      then
          set @pMaxADATE = date(dateadd( day, -1, now(*)));
      end if;
   end if;

   for date_entry as date_entry_loop cursor for
      select distinct rs.rollup_source_key rollup_source_key,
             da.adate as adate
      from sahara.rollup_source rs
          inner join sahara.rollup_batch rb on rb.rollup_source_key = rs.rollup_source_key and rb.period_type = 'day'
          inner join sahara.DATES da on da.dates_key = rb.dates_key
      where da.adate >= @pMinADATE and da.adate <= @pMaxADATE
      order by rollup_source_key asc, adate asc
   do
   begin

      savepoint SP_Missing_And_Summarization_after_Db_Upgrade;

      set @section = string('rollup_source_key=',rollup_source_key, ',adate=',adate,',period=','week');
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD(rollup_source_key, adate, 'week');

      set @section = string('rollup_source_key=',rollup_source_key, ',adate=',adate,',period=','month');
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD(rollup_source_key, adate, 'month');

      set @section = string('rollup_source_key=',rollup_source_key, ',adate=',adate,',period=','quarter');
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD(rollup_source_key, adate, 'quarter');

      set @section = string('rollup_source_key=',rollup_source_key, ',adate=',adate,',period=','year');
      CALL SP_Log_LogDebug(NULL, @FUNC, 'Processing ' || @section );
      call sahara.SP_SUMMARIZE_PERIOD(rollup_source_key, adate, 'year');

      call SAHARA.fn_Configuration_InstallDate (
           'TREND_DATA_RETENTION',                     -- section
           'LAST_SUMMARIZATION_TIME',                  -- name
           adate,                                      -- value
           'Last Data Summarization processed.',       -- display name
           -- description
           '',
           0,
           1
       );
   
      commit;

      EXCEPTION
            WHEN OTHERS THEN
           SELECT SQLCODE, SQLSTATE, ERRORMSG() INTO @error, @sqlstate, @serror;
           IF ( @error <> 0 )
           THEN
               BEGIN
                   call SP_Log_LogError(null, @FUNC,
                                              string ('          @error    = ', @error ) +
                                              string ('         @serror    = ', @serror ) +
                                              string ('       @sqlstate    = ', @sqlstate ) +
                                              string (' Error failed at    = ', @FUNC+':'+@section ) );
                   rollback to savepoint SP_Missing_And_Summarization_after_Db_Upgrade;
                   call SP_Log_LogDebug(null, @FUNC, 'Rollback, exit');
               END;
           END IF;

   end;
   end for;

   -- Update configuration(TREND_DATA_RETENTION, LAST_SUMMARIZATION_PURGING)

   if @migrate_summary_data = 0
   then
      call SAHARA.fn_Configuration_InstallBool (
           'TREND_DATA_RETENTION',                     -- section
           'MIGRATE_SUMMARIZED_DATA',                  -- name
           1,                                          -- value
           'Indicate One Time Simmarization process is done or not.', -- display name
           -- description
           '',
           0,
           1
       );
   end if;
   
   call SP_Log_LogDebug(null, @FUNC, 'exit');

END;


/*
 **************************************************************************
 * SP_SUMMARIZE_PERIOD()
 * Summarize sampled data within <rollup_source>,<adate>,<period>
 **************************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_SUMMARIZE_PERIOD' and user_name(creator) = 'SAHARA') then
   drop procedure SAHARA.SP_SUMMARIZE_PERIOD
end if;

CREATE PROCEDURE SAHARA.SP_SUMMARIZE_PERIOD( @pRollup_Source_Key numeric(10), @pADATE date, @pPeriod varchar(16), @pSubType varchar(16) default '')
BEGIN

  DECLARE @pVSQL_stmt                          long varchar;

  DECLARE @pSubstr                             varchar(32767);
  DECLARE @pProcessed_Summ_Columns             varchar(32767);
  DECLARE @pVSQL_stmt_insert                   varchar(32767); 
  DECLARE @pVSQL_stmt_select                   varchar(32767);
  DECLARE @pVSQL_stmt_from_t1                  varchar(32767);
  DECLARE @pVSQL_stmt_from_t2                  varchar(32767);

  DECLARE @pKey_Columns                        varchar(32767);
  DECLARE @pSel_Key_Columns                    varchar(32767);
  DECLARE @pSumm_Columns                       varchar(32767);
  DECLARE @pAggr_Columns                       varchar(32767);
  DECLARE @pSum_Aggr_Columns                   varchar(32767);
  DECLARE @pAvg_Summ_Columns                   varchar(32767);
  DECLARE @pRollup_Batch_key_list              varchar(32767);
  DECLARE @pSet_Clause                         varchar(32767);

  DECLARE @pJoin_Conditions                    varchar(32767);
  DECLARE @pColumnDiff                         varchar(32767);

  DECLARE @FUNC                                varchar(256);

  DECLARE @pDates_key                          numeric(10);
  DECLARE @pYear_Period_key                    numeric(10);
  DECLARE @pPeriod_End_Dates_Key               numeric(10);
  DECLARE @pPeriod_End_Rollup_Batch_Key        numeric(10);
  DECLARE @pPeriod_Start_Rollup_Batch_Key      numeric(10);
  DECLARE @pNew_Rollup_Batch_Key               numeric(10);

  DECLARE @pYear                               int;
  DECLARE @pQuarter                            int;
  DECLARE @pMonth                              int;
  DECLARE @pWeek                               int;
  DECLARE @pCount                              int;
  DECLARE @pCount1                             int;
  DECLARE @aggr_count                          int;

  DECLARE @pIs_Week_End                        char(1);
  DECLARE @pIs_Month_End                       char(1);
  DECLARE @pIs_Quarter_End                     char(1);
  DECLARE @pIs_Year_End                        char(1);

  DECLARE @pIs_Last_Of_Week                    char(1);
  DECLARE @pIs_Last_Of_Month                   char(1);
  DECLARE @pIs_Last_Of_Quarter                 char(1);
  DECLARE @pIs_Last_Of_Year                    char(1);

  DECLARE @pCol_Name                           varchar(128);
  DECLARE @pT1_Col_Name                        varchar(128);
  DECLARE @pT2_Col_Name                        varchar(128);
  DECLARE @pComponentPeriod                    varchar(16);
  DECLARE @section                             varchar(256);
  DECLARE @quote                               varchar(16);
  DECLARE @pPeriodStart                        date;
  DECLARE @pPeriodEnd                          date;

  DECLARE @error                               integer;
  DECLARE @sqlstate                            varchar(5);
  DECLARE @serror                              varchar(512);

  call sahara.SP_saharaAPIInit();

  set @FUNC = 'SP_SUMMARIZE_PERIOD';
  set @quote = '''';

  CALL SP_Log_LogDebug(NULL, @FUNC, 'Entering:');

  if @pADATE = date(now(*))
  then
     CALL SP_Log_LogDebug(NULL, @FUNC, 'No Ops on current date, Exit:');
     return;
  end if;

  set @section = string('rollup_source_key=',@pRollup_Source_Key, ',adate=',@pADATE,',period=',@pPeriod,',subtype=',@pSubType,'time=',now(*));

  message @section to client;

  if not exists (select 1 
                 from sahara.dates da
                   inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                                        rb.is_success = 'T'         and
                                                        rb.period_type = 'day'
                 where da.adate = @pADATE and 
                       rb.rollup_source_key = @pRollup_Source_Key and
                       ( ( @pPeriod = 'week' and rb.is_last_of_week = 'T' )       or 
                         ( @pPeriod = 'month' and rb.is_last_of_month = 'T' )     or 
                         ( @pPeriod = 'quarter' and rb.is_last_of_quarter = 'T' ) or 
                         ( @pPeriod = 'year' and rb.is_last_of_year = 'T' ) ) )
  then
     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ': Rollup Source has no successful rollup on this day or not last successful rollup of the period, Exit:');
     return;
  end if;

  select da.year_number, da.quarter_number, da.month_number, da.week_number_of_year, da.dates_key, da.is_month_end, da.is_quarter_end, da.is_year_end,
         rb.is_last_of_week,rb.is_last_of_month, rb.is_last_of_quarter, rb.is_last_of_year
    into @pYear, @pQuarter, @pMonth, @pWeek, @pDates_key, @pIs_Month_End, @pIs_Quarter_End, @pIs_Year_End,
         @pIs_Last_Of_Week, @pIs_Last_Of_Month, @pIs_Last_Of_Quarter, @pIs_Last_Of_Year
  from sahara.dates da
          inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                               da.adate = @pADATE and
                                               rb.rollup_source_key = @pRollup_Source_Key and
                                               rb.rollup_batch_key in
                                               (select max(rollup_batch_key) 
                                                from sahara.rollup_batch 
                                                where is_success = 'T' and 
                                                      rollup_source_key = @pRollup_Source_Key and
                                                      period_type = 'day'
                                                group by dates_key);

  -- Calculate whether @pDATE is end of week or not.
  if @pADATE = dateadd(dd,-1,dateadd(week,1,dateadd(dd,-(datepart(weekday,@pADATE))+1,@pADATE)))
  then 
     set @pIs_Week_End = 'T';
  else
     set @pIs_Week_End = 'F';
  end if;

  -- counts of successful rollup for this rollup source where its adate > @pDATE
  select count(*) into @pCount
  from sahara.dates da
     inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                          rb.rollup_source_key = @pRollup_Source_Key and
                                          rb.is_success = 'T' and
                                          rb.period_type = 'day'
  where da.adate > @pADATE;

  if @pPeriod = 'week'
  then 

    set @pPeriodStart = dateadd(dd,-(datepart(weekday,@pADATE))+1,@pADATE); -- first date of the week
    set @pPeriodEnd   = dateadd(week,1, @pPeriodStart);                     -- first date of coming week

    -- @pCount1 is zero if last successful rollup of the period
    select count(*) into @pCount1
    from sahara.dates da
       inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                            rb.rollup_source_key = @pRollup_Source_Key and
                                            rb.is_success = 'T' and
                                            rb.period_type = 'day'
    where da.adate > @pADATE and da.adate < @pPeriodEnd;

    -- if @pIs_Week_End = 'T' or (@pIs_Week_End = 'F' and @pIs_Last_Of_Week = 'T' and @pCount > 0)
    if @pIs_Week_End = 'T' or (@pCount > 0 and @pCount1 = 0)
    then
    else
        CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', Not week end or Missing value , Exit:');
        return;
    end if;

    set @pComponentPeriod = 'day';

  else if @pPeriod = 'month'
       then 

          set @pPeriodStart = dateadd(dd,-(datepart(dd,@pADATE))+1,@pADATE); -- first date of the month
          set @pPeriodEnd   = dateadd(mm,1, @pPeriodStart);                  -- first date of coming month

          -- @pCount1 is zero if last successful rollup of the period
          select count(*) into @pCount1
          from sahara.dates da
             inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                                  rb.rollup_source_key = @pRollup_Source_Key and
                                                  rb.is_success = 'T' and
                                                  rb.period_type = 'day'
          where da.adate > @pADATE and da.adate < @pPeriodEnd;

          -- if @pIs_Month_End = 'T' or (@pIs_Month_End = 'F' and @pIs_Last_Of_Month = 'T' and @pCount > 0)
          if @pIs_Month_End = 'T' or (@pCount > 0 and @pCount1 = 0)
          then
          else
              CALL SP_Log_LogDebug(NULL, @FUNC, @section || ',Not month end or Missing value , Exit:');
              return;
          end if;

          set @pComponentPeriod = 'day';

       else if @pPeriod = 'quarter' 
            then

                 set @pPeriodStart = dateadd(dd,-(datepart(dd,@pADATE))+1,@pADATE);              -- first date of the month.
                 set @pPeriodStart = dateadd(mm,-(datepart(mm,@pPeriodStart)-1), @pPeriodStart); -- first date of the year
                 set @pPeriodStart = dateadd(qq, @pQuarter - 1, @pPeriodStart);                  -- first date of the quarter
                 set @pPeriodEnd   = dateadd(qq,1, @pPeriodStart);                               -- first date of the coming quarter

                 -- @pCount1 is zero if last successful rollup of the period
                 select count(*) into @pCount1
                 from sahara.dates da
                    inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                                         rb.rollup_source_key = @pRollup_Source_Key and
                                                         rb.is_success = 'T' and
                                                         rb.period_type = 'day'
                 where da.adate > @pADATE and da.adate < @pPeriodEnd;

                 -- if @pIs_Quarter_End = 'T' or (@pIs_Quarter_End = 'F' and @pIs_Last_Of_Quarter = 'T' and @pCount > 0)
                 if @pIs_Quarter_End = 'T' or (@pCount > 0 and @pCount1 = 0)
                 then
                 else
                     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', Not quarter end or Missing value , Exit:');
                     return;
                 end if;

                 set @pComponentPeriod = 'month';
            else

                 set @pPeriodStart = dateadd(dd,-(datepart(dd,@pADATE))+1,@pADATE);              -- first date of the month.
                 set @pPeriodStart = dateadd(mm,-(datepart(mm,@pPeriodStart)-1), @pPeriodStart); -- first date of the year
                 set @pPeriodEnd   = dateadd(yy,1, @pPeriodStart);                               -- first date of the coming year

                 -- @pCount1 is zero if last successful rollup of the period
                 select count(*) into @pCount1
                 from sahara.dates da
                    inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                                         rb.rollup_source_key = @pRollup_Source_Key and
                                                         rb.is_success = 'T' and
                                                         rb.period_type = 'day'
                 where da.adate > @pADATE and da.adate < @pPeriodEnd;

                 -- if @pIs_Year_End = 'T' or (@pIs_Year_End = 'F' and @pIs_Last_Of_Year = 'T' and @pCount > 0)
                 if @pIs_Year_End = 'T' or (@pCount > 0 and @pCount1 = 0)
                 then
                 else
                     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ',Not year end or Missing value , Exit:');
                     return;
                 end if;

                 set @pComponentPeriod = 'quarter';
            end if;
       end if;
  end if;

  -- Obtain the last dates_key of the period and uses it as the dates_key of new aggregate records.
  -- TODO: When dealing with missing records, make sure the ascending order requirements between ADATE and dates_keys still valid.day
  --       Otherwise, we can not use max function.
  -- select max(distinct da.dates_key) into @pPeriod_End_Dates_Key
  -- from sahara.dates da
  --      inner join sahara.rollup_batch rb on rb.dates_key = da.dates_key and
  --                                              rb.rollup_source_key = @pRollup_Source_Key and
  --                                              rb.period_type = 'day'
  -- where rb.is_success = 'T'       and
  --       da.adate >= @pPeriodStart and 
  --       da.adate < @pPeriodEnd;

  select da.dates_key into @pPeriod_End_Dates_Key
  from sahara.dates da
  where da.adate = dateadd(dd,-1,@pPeriodEnd);

  CALL SP_Log_LogDebug(NULL, @FUNC, '@pPeriod_End_Dates_Key: ' || convert(varchar, @pPeriod_End_Dates_Key));

  select rb.rollup_batch_key into @pPeriod_Start_Rollup_Batch_Key
  from sahara.rollup_batch rb
  where rollup_batch_key in
        (select top 1 max(rb.rollup_batch_key) as rollup_batch_key
         from sahara.rollup_batch rb
              inner join sahara.dates da on rb.dates_key = da.dates_key
         where is_success = 'T' and
               rollup_source_key = @pRollup_Source_Key and
               period_type = 'day' and da.adate >= @pPeriodStart and da.adate < @pPeriodEnd
         group by adate
         order by adate asc);

  CALL SP_Log_LogDebug(NULL, @FUNC, '@pPeriod_Start_Rollup_Batch_Key: ' || convert(varchar, @pPeriod_Start_Rollup_Batch_Key));

  select rb.rollup_batch_key into @pPeriod_End_Rollup_Batch_Key
  from sahara.rollup_batch rb
  where rollup_batch_key in
        (select top 1 max(rb.rollup_batch_key) as rollup_batch_key
         from sahara.rollup_batch rb
              inner join sahara.dates da on rb.dates_key = da.dates_key
         where is_success = 'T' and
               rollup_source_key = @pRollup_Source_Key and
               period_type = 'day' and da.adate >= @pPeriodStart and da.adate < @pPeriodEnd
         group by adate
         order by adate desc);

  CALL SP_Log_LogDebug(NULL, @FUNC, '@pPeriod_End_Rollup_Batch_Key: ' || convert(varchar, @pPeriod_End_Rollup_Batch_Key));

  -- Encode all the max rollup_batch_keys within period into a list
  -- Will be used in the update construction of the sql, to simply the query construct.
  -- (Unless there is a performance issue)

  select list(rb.rollup_batch_key) into @pRollup_Batch_key_list
  from sahara.rollup_batch rb
  where rollup_batch_key in
        (select max(rb.rollup_batch_key) as rollup_batch_key
         from sahara.rollup_batch rb
              inner join sahara.dates da on rb.dates_key = da.dates_key
         where is_success = 'T' and
               rollup_source_key = @pRollup_Source_Key and
               period_type = @pComponentPeriod and da.adate >= @pPeriodStart and da.adate < @pPeriodEnd
         group by adate
         order by adate desc);

  CALL SP_Log_LogDebug(NULL, @FUNC, '@pRollup_Batch_key_list: ' || convert(varchar, @pRollup_Batch_key_list));

  -- Check if rollup_batch of same (period_type, rollup_source and dates_key) has been created. If so, use it.
  -- Otherwise, create new rollup_batch entry
  select top 1 rollup_batch_key into @pNew_Rollup_Batch_Key
  from ROLLUP_BATCH
  where DATES_KEY = @pPeriod_End_Dates_Key and 
        ROLLUP_SOURCE_KEY = @pRollup_Source_Key and
        ((@pSubType = '' and period_type = @pPeriod) or (period_type = @pPeriod || '_' || @pSubType));
  
  if ( @pNew_Rollup_Batch_Key is null )
  then
     -- create new rollup_batch
     INSERT INTO ROLLUP_BATCH(ROLLUP_SOURCE_TYPE_KEY, DATES_KEY, SESSION_KEY, SCHEDULE_HASH, ROLLUP_SOURCE_KEY,
                 SOURCE_NAME,WEB_SERVER_HOST,WEB_SERVER_PORT,
                 DB_HOST, DB_PORT, DB_PASSWORD, DB_PASSWORD_2, LOG_ARCHIVE, ROLLUP_DAY,
                 START_TIME,
                 START_TIME_ER,
                 COMPLETION_TIME,
                 COMPLETION_TIME_ER,
                 DURATION, UTC_ADJUSTMENT, IS_ROLLUP_NOW, IS_COMPLETE, IS_SUCCESS, IS_LATEST, IS_LATEST_COMPLETE, IS_LATEST_SUCCESS,
                 IS_LAST_OF_MONTH, IS_LAST_OF_QUARTER, IS_LAST_OF_YEAR, IS_LAST_OF_WEEK, STATUS, STATUS_MESSAGE,
                 LAST_STATUS_UPDATE, PERIOD_TYPE, SHOW_STATUS, IS_DUMMY
     )
     select top 1 ROLLUP_SOURCE_TYPE_KEY, @pPeriod_End_Dates_Key, SESSION_KEY, SCHEDULE_HASH, ROLLUP_SOURCE_KEY,
                  SOURCE_NAME,WEB_SERVER_HOST,WEB_SERVER_PORT,
                  DB_HOST, DB_PORT, DB_PASSWORD, DB_PASSWORD_2, LOG_ARCHIVE, ROLLUP_DAY,
                  START_TIME, START_TIME_ER, COMPLETION_TIME, COMPLETION_TIME_ER,
                  DURATION, UTC_ADJUSTMENT, IS_ROLLUP_NOW, IS_COMPLETE, 'F' as IS_SUCCESS, IS_LATEST, IS_LATEST_COMPLETE, 
                  'F' as IS_LATEST_SUCCESS, IS_LAST_OF_MONTH, IS_LAST_OF_QUARTER, IS_LAST_OF_YEAR, IS_LAST_OF_WEEK, STATUS, 
                  STATUS_MESSAGE, LAST_STATUS_UPDATE, 
                  case when @pSubType = '' then @pPeriod else @pPeriod || '_' || @pSubType end as period_type,
                  'F' as SHOW_STATUS, 'T' as IS_DUMMY
     from ROLLUP_BATCH RB
     where RB.ROLLUP_BATCH_KEY = @pPeriod_End_Rollup_Batch_Key;

     select max(ROLLUP_BATCH_KEY) into @pNew_Rollup_Batch_Key from ROLLUP_BATCH;
  end if;

  CALL SP_Log_LogDebug(NULL, @FUNC, '@pPeriod_End_Rollup_Batch_Key: ' || convert(varchar, @pPeriod_End_Rollup_Batch_Key));

  -- Process each FACT table
  for fact_tables as fact_tables_loop cursor for
      select table_name as fact_tbl, 
      sum (case when column_type not in ('key', 'summarization') then 1 else 0 end) as aggr_count
      from sahara.fact_table_cols_view 
      -- TODO: remove the where clause
      -- where fact_tbl = 'HOST_CAPACITY'
      group by fact_tbl
  do
      -- Getting Fact table key columns

      set @pCount = 0;

      if fact_tbl not in ( 'CLUSTER_REMOTECLUSTER_SUMMARY', 'CLUSTER_REMOTECLUSTER' )
      then

         set @pVSQL_stmt = '';
         set @pKey_Columns = '';
         set @pSel_Key_Columns = '';
         set @pSumm_Columns = '';
         set @pProcessed_Summ_Columns = '';
         set @pAggr_columns = '';
         set @pJoin_Conditions = '';
         set @pSet_Clause = '';
         set @pSum_Aggr_Columns = '';
         set @pCount = 0;
         
         for fact_tbl_key_cols as fact_tbl_key_cols_loop cursor for
             select column_name as key_col_name
             from sahara.fact_table_cols_view
             where table_name = fact_tbl and column_type = 'key' and key_col_name not in ( 'DATES_KEY', 'ROLLUP_BATCH_KEY' )
         do

             set @pCol_Name = '"' || key_col_name || '"';
             if @pCount = 0
             then
                set @pKey_Columns     = @pKey_Columns     || @pCol_Name;
                set @pSel_Key_Columns = @pSel_Key_Columns || 't1.'  || @pCol_Name;
                set @pJoin_Conditions = @pJoin_Conditions || ' t1.' || @pCol_Name || '=t2.' || @pCol_Name;
             else
                set @pKey_Columns     = @pKey_Columns     || ','    || @pCol_Name;
                set @pSel_Key_Columns = @pSel_Key_Columns || ',t1.' || @pCol_Name;
                set @pJoin_Conditions = @pJoin_Conditions || ' and t1.' || @pCol_Name || '=t2.' || @pCol_Name || ' ' ;
             end if;
             set @pCount = @pCount + 1;
         end for;

         set @pCount = 0;

         -- Getting Fact table Summarization columns
         for fact_tbl_summ_cols as fact_tbl_summ_cols_loop cursor for
             select column_name as summ_col_name
             from sahara.fact_table_cols_view
             where table_name = fact_tbl and column_type = 'summarization'
         do
             set @pCol_Name = '"' || summ_col_name || '"';
             set @pSumm_Columns = @pSumm_Columns || ',' || @pCol_Name;
             set @pProcessed_Summ_Columns = @pProcessed_Summ_Columns || fn_gen_grow_sql_str(@pCol_Name, @pSubType);


             set @pCount = @pCount + 1;
         end for;

         if @pSubType = ''
         then
             set @pVSQL_stmt = 'insert into sahara.' || fact_tbl || '(DATES_KEY,ROLLUP_BATCH_KEY,' || @pKey_Columns || @pSumm_Columns || ') '
                            || ' select ' || @pPeriod_End_Dates_Key || ' as dates_key,'
                            || '        ' || @pNew_Rollup_Batch_Key || ' as rollup_batch_key,'
                            ||               @pKey_Columns 
                            ||               @pSumm_Columns
                            || ' from sahara.' || fact_tbl
                            || ' where sahara.' || fact_tbl || '.rollup_batch_key  = ' || @pPeriod_End_Rollup_Batch_Key || ';';

            CALL SP_Log_LogDebug(NULL, @FUNC, 'Summarization SQL: ' || @pVSQL_stmt);
            execute immediate @pVSQL_stmt;
         else 

             if @pSubType in ('grow', 'grow_perc')
             then
                 set @pVSQL_stmt_insert = 'insert into sahara.' || fact_tbl || '(DATES_KEY,ROLLUP_BATCH_KEY,' || @pKey_Columns || @pSumm_Columns || ') ';
                 set @pVSQL_stmt_select = ' select ' || @pPeriod_End_Dates_Key || ' as dates_key,' 
                                                     || @pNew_Rollup_Batch_Key || ' as rollup_batch_key,' 
                                                     || @pSel_Key_Columns 
                                                     || @pProcessed_Summ_Columns;
                 set @pVSQL_stmt_from_t2     = '(select ' || @pKey_Columns || @pSumm_Columns
                                            || ' from sahara.' || fact_tbl 
                                            || ' where rollup_batch_key = ' || @pPeriod_Start_Rollup_Batch_Key || ') t2 ' ;


                 set @pVSQL_stmt_from_t1     = '(select ' || @pKey_Columns || @pSumm_Columns 
                                            || ' from sahara.' || fact_tbl 
                                            || ' where rollup_batch_key = ' || @pPeriod_End_Rollup_Batch_Key   || ') t1 ' ;
                                            
                 set @pVSQL_stmt = @pVSQL_stmt_insert 
                               ||  @pVSQL_stmt_select 
                               || ' from '          || @pVSQL_stmt_from_t2 
                               || '    inner join ' || @pVSQL_stmt_from_t1 || ' on ' || @pJoin_Conditions;

                 if @pSubType in ('grow', 'grow_perc') and @pCount = 0
                 then
                    CALL SP_Log_LogDebug(NULL, @FUNC, fact_tbl || ' does not have summarization column:');
                 else

                    CALL SP_Log_LogDebug(NULL, @FUNC, 'Summarization SQL: ' || @pVSQL_stmt_select);
                    CALL SP_Log_LogDebug(NULL, @FUNC, 'Summarization SQL: ' || @pVSQL_stmt_from_t1);
                    CALL SP_Log_LogDebug(NULL, @FUNC, 'Summarization SQL: ' || @pVSQL_stmt_from_t2);
                    execute immediate @pVSQL_stmt_insert 
                                  ||  @pVSQL_stmt_select 
                                  || ' from '          || @pVSQL_stmt_from_t2 
                                  || '    inner join ' || @pVSQL_stmt_from_t1 || ' on ' || @pJoin_Conditions;
                 end if;

             end if;
         end if;


         set @pCount = 0;

         -- Getting Fact table Aggregation columns
         for fact_tbl_aggr_cols as fact_tbl_aggr_cols_loop cursor for
             select column_name as aggr_col_name,
                    column_type as col_aggr_fn
             from sahara.fact_table_cols_view
             where @pSubType = '' and table_name = fact_tbl and column_type not in ('key', 'summarization')
         do
             set @pAggr_Columns     = @pAggr_Columns || ',"' || aggr_col_name || '"';
             set @pSum_Aggr_Columns = @pSum_Aggr_Columns || ',' || col_aggr_fn || '(coalesce("' || aggr_col_name || '",0)) as ' || aggr_col_name ;
             if @pCount = 0
             then
               set @pSet_Clause       = '"' || aggr_col_name || '"=t2."' || aggr_col_name || '" '; 
             else
               set @pSet_Clause       = @pSet_Clause   || ',"' || aggr_col_name || '"=t2."' || aggr_col_name || '" ';
             end if;
             set @pCount = @pCount + 1;
         end for;

         if @pCount = 0 
         then
                     CALL SP_Log_LogDebug(NULL, @FUNC, fact_tbl || ' does not have aggregate column:');
         else
 
            set @pVSQL_stmt = ' update sahara.' || fact_tbl || ' set ' || @pSet_Clause ||
                              ' from sahara.' || fact_tbl || ' as t1 ' ||
                              '      inner join ' ||
                              '   (select ' || @pPeriod_End_Dates_Key || ' as dates_key,' || 
                              '           ' || @pNew_Rollup_Batch_Key || ' as rollup_batch_key,' || @pKey_Columns || @pSum_Aggr_Columns || 
                              '    from sahara.' || fact_tbl || 
                              '    where sahara.' || fact_tbl || '.rollup_batch_key in (select row_value from sa_split_list(' || @quote || @pRollup_Batch_key_list || @quote || ')) '  ||
                              '    group by ' || @pKey_Columns || ') as t2 ' ||
                              ' on t1.rollup_batch_key = ' || @pNew_Rollup_Batch_Key || ' and t2.rollup_batch_key = ' || @pNew_Rollup_Batch_Key || ' and ' ||
                                      @pJoin_Conditions || ';';

            CALL SP_Log_LogDebug(NULL, @FUNC, 'Aggregation SQL: ' || @pVSQL_stmt);
            -- message @pVSQL_stmt to log;
            execute immediate @pVSQL_stmt;
         end if;
      end if;
  end for;
  
  CALL SP_Log_LogDebug(NULL, @FUNC, 'Exit:');

END;

/*
 **********************************************************************
 * fn_summ_col():
 *    retrieve, calculate the arguments depending on the subtype
 *    use it to simply the coding of the dynamic sql code.
 * Return:
 * the retrieved, calculated value.
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_summ_col' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_summ_col
end if;

CREATE FUNCTION SAHARA.fn_summ_col(@pEndValue numeric(22,9), @pStartValue numeric(22,9), @pSubType varchar(16))
       returns numeric(22,9)
BEGIN
   DECLARE @pDiffValue   numeric(22,9);
   DECLARE @pReturnValue numeric(22,9);
   DECLARE @pStart       numeric(22,9);
   DECLARE @pEnd         numeric(22,9);

   set @pStart = coalesce(@pStartValue, 0);
   set @pEnd   = coalesce(@pEndValue, 0);

   set @pDiffValue = @pEnd - @pStart;

   select 
           case when @pSubType = ''          then @pEndValue
                when @pSubType = 'grow'      then @pDiffValue
                when @pSubType = 'grow_perc' and  @pDiffValue  = 0.0 then 0.0
                when @pSubType = 'grow_perc' and  @pStart      = 0.0 then 100.0
                when @pSubType = 'grow_perc'                         then @pDiffValue / @pStart
                else 0.0 end 
   into @pReturnValue;

   return abs(@pReturnValue);

END;

/*
 **********************************************************************
 * fn_gen_grow_sql_str():
 *    retrieve, calculate the arguments depending on the subtype
 *    use it to simply the coding of the dynamic sql code.
 * Return:
 * the retrieved, calculated value.
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_gen_grow_sql_str' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_gen_grow_sql_str
end if;

CREATE FUNCTION SAHARA.fn_gen_grow_sql_str(@pCol_Name varchar(128), @pSubType varchar(16))
       returns varchar(1024)
BEGIN

   DECLARE @pReturnValue varchar(1024);
   DECLARE @quote        varchar(16);
   set @quote = '''';

   set @pReturnValue = ',fn_summ_col(' || string('t1.',@pCol_Name) || ',' 
                                       || string('t2.',@pCol_Name) || ',' 
                                       || @quote || @pSubType || @quote || ') as ' || @pCol_Name ;
   return @pReturnValue;

END;

/*
 **************************************************************************
 * SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS()
 * Summarize sampled data within <rollup_source>,<adate>,<period>
 **************************************************************************
 */

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS' and user_name(creator) = 'SAHARA') then
   drop procedure SAHARA.SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS
end if;

-- @pTableName should exist in column_metadata_base table
CREATE PROCEDURE SAHARA.SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS( @pRollup_Source_Key numeric(10), @pADATE date, @pPeriod varchar(16), @pSubType varchar(16) default '', @pTableName varchar(64) default '')
BEGIN

  DECLARE @pVSQL_stmt                          long varchar;

  DECLARE @pSubstr                             varchar(32767);
  DECLARE @pProcessed_Summ_Columns             varchar(32767);
  DECLARE @pVSQL_stmt_insert                   varchar(32767); 
  DECLARE @pVSQL_stmt_select                   varchar(32767);
  DECLARE @pVSQL_stmt_from_t1                  varchar(32767);
  DECLARE @pVSQL_stmt_from_t2                  varchar(32767);

  DECLARE @pKey_Columns                        varchar(32767);
  DECLARE @pSel_Key_Columns                    varchar(32767);
  DECLARE @pSumm_Columns                       varchar(32767);
  DECLARE @pAggr_Columns                       varchar(32767);
  DECLARE @pSum_Aggr_Columns                   varchar(32767);
  DECLARE @pAvg_Summ_Columns                   varchar(32767);
  DECLARE @pRollup_Batch_key_list              varchar(32767);
  DECLARE @pSet_Clause                         varchar(32767);

  DECLARE @pJoin_Conditions                    varchar(32767);
  DECLARE @pColumnDiff                         varchar(32767);

  DECLARE @FUNC                                varchar(256);

  DECLARE @pDates_key                          numeric(10);
  DECLARE @pYear_Period_key                    numeric(10);
  DECLARE @pPeriod_End_Dates_Key               numeric(10);
  DECLARE @pPeriod_End_Rollup_Batch_Key        numeric(10);
  DECLARE @pPeriod_Start_Rollup_Batch_Key      numeric(10);
  DECLARE @pNew_Rollup_Batch_Key               numeric(10);

  DECLARE @pYear                               int;
  DECLARE @pQuarter                            int;
  DECLARE @pMonth                              int;
  DECLARE @pWeek                               int;
  DECLARE @pCount                              int;
  DECLARE @pCount1                             int;
  DECLARE @aggr_count                          int;

  DECLARE @pIs_Week_End                        char(1);
  DECLARE @pIs_Month_End                       char(1);
  DECLARE @pIs_Quarter_End                     char(1);
  DECLARE @pIs_Year_End                        char(1);

  DECLARE @pIs_Last_Of_Week                    char(1);
  DECLARE @pIs_Last_Of_Month                   char(1);
  DECLARE @pIs_Last_Of_Quarter                 char(1);
  DECLARE @pIs_Last_Of_Year                    char(1);

  DECLARE @pCol_Name                           varchar(128);
  DECLARE @pT1_Col_Name                        varchar(128);
  DECLARE @pT2_Col_Name                        varchar(128);
  DECLARE @pComponentPeriod                    varchar(16);
  DECLARE @section                             varchar(256);
  DECLARE @quote                               varchar(16);
  DECLARE @pPeriodStart                        date;
  DECLARE @pPeriodEnd                          date;

  DECLARE @error                               integer;
  DECLARE @sqlstate                            varchar(5);
  DECLARE @serror                              varchar(512);

  call sahara.SP_saharaAPIInit();

  set @FUNC = 'SP_SUMMARIZE_PERIOD_VCS_HIST_STATISTICS';
  set @quote = '''';

  CALL SP_Log_LogDebug(NULL, @FUNC, 'Entering:');

  if @pADATE = date(now(*))
  then
     CALL SP_Log_LogDebug(NULL, @FUNC, 'No Ops on current date, Exit:');
     return;
  end if;

  select da.year_number, da.quarter_number, da.month_number, da.week_number_of_year, da.dates_key, da.is_month_end, da.is_quarter_end, da.is_year_end
    into @pYear, @pQuarter, @pMonth, @pWeek, @pDates_key, @pIs_Month_End, @pIs_Quarter_End, @pIs_Year_End
  from sahara.dates da
  where da.adate = @pADATE;

  -- Calculate whether @pDATE is end of week or not.
  if @pADATE = dateadd(dd,-1,dateadd(week,1,dateadd(dd,-(datepart(weekday,@pADATE))+1,@pADATE)))
  then 
     set @pIs_Week_End = 'T';
  else
     set @pIs_Week_End = 'F';
  end if;

  set @section = string('rollup_source_key=(',@pRollup_Source_Key, '),adate=(',@pADATE,'),period=(',@pPeriod,'),subtype=(',@pSubType,
	'),Month_End=(',@pIs_Month_End,'),Quarter_End=(',@pIs_Quarter_End,'),Year_End=(',@pIs_Year_End,'),Week_End=(',@pIs_Week_End,')');
  message @section to client;

  if @pPeriod = 'week'
  then 

    set @pPeriodStart = dateadd(dd,-(datepart(weekday,@pADATE))+1,@pADATE); -- first date of the week
    set @pPeriodEnd   = dateadd(week,1, @pPeriodStart);                     -- first date of coming week

    -- @pCount1 is zero if last successful rollup of the period
    select count(*) into @pCount1
    from sahara.dates da
       inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                            rb.rollup_source_key = @pRollup_Source_Key and
                                            rb.valid_cluster_data = 'T' and
                                            rb.period_type = 'day'
    where da.adate >= @pPeriodStart and da.adate < @pPeriodEnd;

    if @pIs_Week_End = 'T' and @pCount1 > 0
    then
    else
        CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', No data in the week or not end of week, Exit:');
        return;
    end if;

    set @pComponentPeriod = 'day';

  else if @pPeriod = 'month'
       then 

          set @pPeriodStart = dateadd(dd,-(datepart(dd,@pADATE))+1,@pADATE); -- first date of the month
          set @pPeriodEnd   = dateadd(mm,1, @pPeriodStart);                  -- first date of coming month

          -- @pCount1 is zero if last successful rollup of the period
          select count(*) into @pCount1
          from sahara.dates da
             inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                                  rb.rollup_source_key = @pRollup_Source_Key and
                                                  rb.valid_cluster_data = 'T' and
                                                  rb.period_type = 'day'
          where da.adate >= @pPeriodStart and da.adate < @pPeriodEnd;

          if @pIs_Month_End = 'T' and @pCount1 > 0
          then
          else
              CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', No data in the month period or not end of month, Exit:');
              return;
          end if;

          set @pComponentPeriod = 'day';

       else if @pPeriod = 'quarter' 
            then

                 set @pPeriodStart = dateadd(dd,-(datepart(dd,@pADATE))+1,@pADATE);              -- first date of the month.
                 set @pPeriodStart = dateadd(mm,-(datepart(mm,@pPeriodStart)-1), @pPeriodStart); -- first date of the year
                 set @pPeriodStart = dateadd(qq, @pQuarter - 1, @pPeriodStart);                  -- first date of the quarter
                 set @pPeriodEnd   = dateadd(qq,1, @pPeriodStart);                               -- first date of the coming quarter

                 -- @pCount1 is zero if last successful rollup of the period
                 select count(*) into @pCount1
                 from sahara.dates da
                    inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                                         rb.rollup_source_key = @pRollup_Source_Key and
                                                         rb.valid_cluster_data = 'T' and
                                                         rb.period_type = 'day'
                 where da.adate >= @pPeriodStart and da.adate < @pPeriodEnd;

                 if @pIs_Quarter_End = 'T' and @pCount1 > 0
                 then
                 else
                     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', No data in the quarter period or not end of quarter, Exit:');
                     return;
                 end if;

                 set @pComponentPeriod = 'month';
            else

                 set @pPeriodStart = dateadd(dd,-(datepart(dd,@pADATE))+1,@pADATE);              -- first date of the month.
                 set @pPeriodStart = dateadd(mm,-(datepart(mm,@pPeriodStart)-1), @pPeriodStart); -- first date of the year
                 set @pPeriodEnd   = dateadd(yy,1, @pPeriodStart);                               -- first date of the coming year

                 -- @pCount1 is zero if last successful rollup of the period
                 select count(*) into @pCount1
                 from sahara.dates da
                    inner join sahara.rollup_batch rb on da.dates_key = rb.dates_key and
                                                         rb.rollup_source_key = @pRollup_Source_Key and
                                                         rb.valid_cluster_data = 'T' and
                                                         rb.period_type = 'day'
                 where da.adate >= @pPeriodStart and da.adate < @pPeriodEnd;

                 if @pIs_Year_End = 'T' and @pCount1 > 0
                 then
                 else
                     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', No data in the year period or not end of year, Exit:');
                     return;
                 end if;

                 set @pComponentPeriod = 'month';
            end if;
       end if;
  end if;

  select da.dates_key into @pPeriod_End_Dates_Key
  from sahara.dates da
  where da.adate = dateadd(dd,-1,@pPeriodEnd);

  CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', @pPeriod_End_Dates_Key: ' || convert(varchar, @pPeriod_End_Dates_Key));

  select rb.rollup_batch_key into @pPeriod_Start_Rollup_Batch_Key
  from sahara.rollup_batch rb
  where rollup_batch_key in
        (select top 1 max(rb.rollup_batch_key) as rollup_batch_key
         from sahara.rollup_batch rb
              inner join sahara.dates da on rb.dates_key = da.dates_key
         where valid_cluster_data = 'T' and
               rollup_source_key = @pRollup_Source_Key and
               period_type = 'day' and da.adate >= @pPeriodStart and da.adate < @pPeriodEnd
         group by adate
         order by adate asc);

  if @pPeriod_Start_Rollup_Batch_Key is null
  then
     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', @pPeriod_Start_Rollup_Batch_Key is null, exit.');
     return;
  else
     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', @pPeriod_Start_Rollup_Batch_Key: ' || convert(varchar, @pPeriod_Start_Rollup_Batch_Key));
  end if;

  select rb.rollup_batch_key into @pPeriod_End_Rollup_Batch_Key
  from sahara.rollup_batch rb
  where rollup_batch_key in
        (select top 1 max(rb.rollup_batch_key) as rollup_batch_key
         from sahara.rollup_batch rb
              inner join sahara.dates da on rb.dates_key = da.dates_key
         where valid_cluster_data = 'T' and
               rollup_source_key = @pRollup_Source_Key and
               period_type = 'day' and da.adate >= @pPeriodStart and da.adate < @pPeriodEnd
         group by adate
         order by adate desc);

  if @pPeriod_End_Rollup_Batch_Key is null
  then
       CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', @pPeriod_End_Rollup_Batch_Key is null. Exit¸');
       return;
  else
       CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', @pPeriod_End_Rollup_Batch_Key: ' || convert(varchar, @pPeriod_End_Rollup_Batch_Key));
  end if;

  -- Encode all the max rollup_batch_keys within period into a list
  -- Will be used in the update construction of the sql, to simply the query construct.
  -- (Unless there is a performance issue)

  select list(rb.rollup_batch_key) into @pRollup_Batch_key_list
  from sahara.rollup_batch rb
  where rollup_batch_key in
        (select max(rb.rollup_batch_key) as rollup_batch_key
         from sahara.rollup_batch rb
              inner join sahara.dates da on rb.dates_key = da.dates_key
               -- When @pComponentPeriod = 'day', use the is_dummy data
	       -- When @pComponentPeriod <> 'day', use the summarized data
         where ((@pComponentPeriod = 'day' and valid_cluster_data = 'T') or (@pComponentPeriod <> 'day' and is_success = 'T')) and
               rollup_source_key = @pRollup_Source_Key and
               period_type = @pComponentPeriod and da.adate >= @pPeriodStart and da.adate < @pPeriodEnd
         group by adate
         order by adate desc);

  CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', @pRollup_Batch_key_list: ' || convert(varchar, @pRollup_Batch_key_list));

  -- Check if rollup_batch of same (period_type, rollup_source and dates_key) has been created. If so, use it.
  -- Otherwise, create new rollup_batch entry
  select top 1 rollup_batch_key into @pNew_Rollup_Batch_Key
  from ROLLUP_BATCH
  where DATES_KEY = @pPeriod_End_Dates_Key and 
        ROLLUP_SOURCE_KEY = @pRollup_Source_Key and
        ((@pSubType = '' and period_type = @pPeriod) or (period_type = @pPeriod || '_' || @pSubType))
  order by rollup_batch_key asc ;
  
  CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', @pNew_Rollup_Batch_Key: ' || '(' || convert(varchar, @pNew_Rollup_Batch_Key) || ')');

  if @pNew_Rollup_Batch_Key is null
  then
     -- create new rollup_batch
     INSERT INTO ROLLUP_BATCH(ROLLUP_SOURCE_TYPE_KEY, DATES_KEY, SESSION_KEY, SCHEDULE_HASH, ROLLUP_SOURCE_KEY,
                 SOURCE_NAME,WEB_SERVER_HOST,WEB_SERVER_PORT,
                 DB_HOST, DB_PORT, DB_PASSWORD, DB_PASSWORD_2, LOG_ARCHIVE, ROLLUP_DAY,
                 START_TIME,
                 START_TIME_ER,
                 COMPLETION_TIME,
                 COMPLETION_TIME_ER,
                 DURATION, UTC_ADJUSTMENT, IS_ROLLUP_NOW, IS_COMPLETE, IS_SUCCESS, IS_LATEST, IS_LATEST_COMPLETE, IS_LATEST_SUCCESS,
                 IS_LAST_OF_MONTH, IS_LAST_OF_QUARTER, IS_LAST_OF_YEAR, IS_LAST_OF_WEEK, STATUS, STATUS_MESSAGE,
                 LAST_STATUS_UPDATE, PERIOD_TYPE, SHOW_STATUS, IS_DUMMY
     )
     select top 1 ROLLUP_SOURCE_TYPE_KEY, @pPeriod_End_Dates_Key, SESSION_KEY, SCHEDULE_HASH, ROLLUP_SOURCE_KEY,
                  SOURCE_NAME,WEB_SERVER_HOST,WEB_SERVER_PORT,
                  DB_HOST, DB_PORT, DB_PASSWORD, DB_PASSWORD_2, LOG_ARCHIVE, ROLLUP_DAY,
                  START_TIME, START_TIME_ER, COMPLETION_TIME, COMPLETION_TIME_ER,
                  DURATION, UTC_ADJUSTMENT, IS_ROLLUP_NOW, IS_COMPLETE, 'F' as IS_SUCCESS, IS_LATEST, IS_LATEST_COMPLETE, 
                  'F' as IS_LATEST_SUCCESS, IS_LAST_OF_MONTH, IS_LAST_OF_QUARTER, IS_LAST_OF_YEAR, IS_LAST_OF_WEEK, STATUS, 
                  STATUS_MESSAGE, LAST_STATUS_UPDATE, 
                  case when @pSubType = '' then @pPeriod else @pPeriod || '_' || @pSubType end as period_type,
                  'F' as SHOW_STATUS, 'T' as IS_DUMMY
     from ROLLUP_BATCH RB
     where RB.ROLLUP_BATCH_KEY = @pPeriod_End_Rollup_Batch_Key;

     select max(ROLLUP_BATCH_KEY) into @pNew_Rollup_Batch_Key from ROLLUP_BATCH;
     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', new Rollup_Batch_Key created: ' || convert(varchar, @pNew_Rollup_Batch_Key));
  end if;


  -- TODO: populate @aggr_count.
  if ( @pTableName = '' )
  then
     set @aggr_count = 0;
  else
     set @aggr_count = 1;
  end if;

  -- Process each FACT table
  for fact_tables as fact_tables_loop cursor for
      select row_value as fact_tbl,
             @aggr_count as aggr_count
      from sa_split_list(@pTableName,',')
  do
      -- Getting Fact table key columns

      set @pCount = 0;

      if fact_tbl not in ( 'CLUSTER_REMOTECLUSTER_SUMMARY', 'CLUSTER_REMOTECLUSTER' )
      then

         set @pVSQL_stmt = '';
         set @pKey_Columns = '';
         set @pSel_Key_Columns = '';
         set @pSumm_Columns = '';
         set @pProcessed_Summ_Columns = '';
         set @pAggr_columns = '';
         set @pJoin_Conditions = '';
         set @pSet_Clause = '';
         set @pSum_Aggr_Columns = '';
         set @pCount = 0;
         
         for fact_tbl_key_cols as fact_tbl_key_cols_loop cursor for
             select column_name as key_col_name
             from column_metadata_base
             where table_name = fact_tbl and key_col_name like '%_KEY' and key_col_name not in ( 'DATES_KEY', 'ROLLUP_BATCH_KEY' )
         do

             set @pCol_Name = '"' || key_col_name || '"';
             if @pCount = 0
             then
                set @pKey_Columns     = @pKey_Columns     || @pCol_Name;
                set @pSel_Key_Columns = @pSel_Key_Columns || 't1.'  || @pCol_Name;
                set @pJoin_Conditions = @pJoin_Conditions || ' t1.' || @pCol_Name || '=t2.' || @pCol_Name;
             else
                set @pKey_Columns     = @pKey_Columns     || ','    || @pCol_Name;
                set @pSel_Key_Columns = @pSel_Key_Columns || ',t1.' || @pCol_Name;
                set @pJoin_Conditions = @pJoin_Conditions || ' and t1.' || @pCol_Name || '=t2.' || @pCol_Name || ' ' ;
             end if;
             set @pCount = @pCount + 1;
         end for;

         set @pCount = 0;

         -- Getting Fact table Aggregation columns
         for fact_tbl_aggr_cols as fact_tbl_aggr_cols_loop cursor for
             select column_name as aggr_col_name,
                    aggregate_type as col_aggr_fn
             from column_metadata_base
             where @pSubType = '' and
                   table_name = fact_tbl and
                   aggr_col_name not like '%_KEY' and
                   aggregate_type <> 'summarization' 
         do
             set @pAggr_Columns     = @pAggr_Columns || ',"' || aggr_col_name || '"';
             set @pSum_Aggr_Columns = @pSum_Aggr_Columns || ',' || col_aggr_fn || '(coalesce("' || aggr_col_name || '",0)) as ' || aggr_col_name ;
             if @pCount = 0
             then
               set @pSet_Clause       = '"' || aggr_col_name || '"=t2."' || aggr_col_name || '" '; 
             else
               set @pSet_Clause       = @pSet_Clause   || ',"' || aggr_col_name || '"=t2."' || aggr_col_name || '" ';
             end if;
             set @pCount = @pCount + 1;
         end for;

         if @pCount = 0 
         then
                     CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', ' || fact_tbl || ' does not have aggregate column:');
		     return;
         else
 
             set @pVSQL_stmt = 'insert into sahara.' || fact_tbl || '(DATES_KEY,ROLLUP_BATCH_KEY,' || @pKey_Columns || @pAggr_Columns || ') on existing update ' ||
                              ' select ' || @pPeriod_End_Dates_Key || ' as dates_key,' || 
                              '        ' || @pNew_Rollup_Batch_Key || ' as rollup_batch_key,' || @pKey_Columns || @pSum_Aggr_Columns || 
                              ' from sahara.' || fact_tbl || 
                              ' where sahara.' || fact_tbl || '.rollup_batch_key in (select row_value from sa_split_list(' || @quote || @pRollup_Batch_key_list || @quote || ')) '  ||
                              ' group by ' || @pKey_Columns ;

         end if;

         begin
            CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', Aggregation SQL: ' || @pVSQL_stmt);
            -- message @pVSQL_stmt to log;
            execute immediate @pVSQL_stmt;

            set @pCount = 0;
            select count(*) into @pCount
	    from SERVICEGROUP_HIST_EVENTS sg
		 inner join rollup_batch rb on rb.rollup_batch_key = sg.rollup_batch_key
	    where rb.rollup_batch_key = @pNew_Rollup_Batch_Key and rb.period_type = @pPeriod;

            call SP_Log_LogDebug(null, @FUNC, @section || ', inserted(' || convert(varchar, @pCount) || ')');

	 end
      end if;
  end for;
  
  CALL SP_Log_LogDebug(NULL, @FUNC, @section || ', Exit:');

END;

-----------------------------------------------------------------------
-- $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
-- $Revision: 1.119.4.1 $
-- $Copyright: Copyright (c) 2012 Symantec Corporation.
-- All rights reserved.
--
-- THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
-- SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
-- WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
--
-- The Licensed Software and Documentation are deemed to be commercial
-- computer software as defined in FAR 12.212 and subject to restricted
-- rights as defined in FAR Section 52.227-19 "Commercial Computer
-- Software - Restricted Rights" and DFARS 227.7202, "Rights in
-- Commercial Computer Software or Commercial Computer Software
-- Documentation", as applicable, and any successor regulations. Any use,
-- modification, reproduction release, performance, display or disclosure
-- of the Licensed Software and Documentation by the U.S. Government
-- shall be solely in accordance with the terms of this Agreement.  $
--
-- ROllup schedule APIs
-----------------------------------------------------------------------

message 'Creating/replacing ROLLUP SCHEDULE API module stored procedures...' to client;


/*
 **********************************************************************
 * SP_RollupSched_AcquireScriptControl():
 * Acquires exclusive lock on running Rollup schedule activities.
 * There are teo types of activities:
 *           - rollup scheduler ( to check and start rollups)
 *           - doing metamodel updates
 * The caller is expected to pass the activity type to get the necessary
 * lock:
 *	- ETL_SCHEDULER
 *	- METAMODEL_UPDATE
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_RollupSched_AcquireScriptControl')
THEN
        DROP PROCEDURE sahara.SP_RollupSched_AcquireScriptControl;
END IF;

create procedure sahara.SP_RollupSched_AcquireScriptControl(
     IN @pActivityName varchar(64),
     IN @Timeout    integer default 0)
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @SessionKey NUMERIC(10);

    -- initialize global variables
    call sahara.SP_saharaAPIInit();
 
    SET @func = 'SP_RollupSched_AcquireScriptControl';
    SET @SessionKey = NULL;

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call LockSession() for the specific activity. If fails, it will raise
    -- an error, which will throw the exception which will return the
    -- control to the caller.
    CALL SP_Lock_LockSession(@pActivityName, @SessionKey);
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully acquired the lock');

END;


/*
 **********************************************************************
 * SP_RollupSched_ReleaseScriptControl():
 * Release the exclusive lock on a rollup schedle activity.
 * There are teo types of activities:
 *           - rollup scheduler ( to check and start rollups)
 *           - doing metamodel updates
 * The caller is expected to pass the activity type to get the necessary
 * lock:
 *	- ETL_SCHEDULER
 *	- METAMODEL_UPDATE
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_RollupSched_ReleaseScriptControl')
THEN
        DROP PROCEDURE sahara.SP_RollupSched_ReleaseScriptControl;
END IF;

create procedure sahara.SP_RollupSched_ReleaseScriptControl(
     IN @pActivityName varchar(64))
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @SessionKey NUMERIC(10);

    -- initialize global variables
    call sahara.SP_saharaAPIInit();

    SET @func = 'SP_RollupSched_ReleaseScriptControl';
    SET @SessionKey = NULL;

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call UnlockSession() for the given activity.
    -- If this fails, it will raise an error, which will throw the 
    -- exception which will return the control to the caller.
    CALL SP_Lock_UnlockSession(@pActivityName, @SessionKey);

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully release the lock');
END;




if exists(select 1 from sys.sysprocedure where proc_name = 'SP_RollupSchedule_SetServerUTCAdjustment' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_RollupSchedule_SetServerUTCAdjustment
end if;
create procedure sahara.SP_RollupSchedule_SetServerUTCAdjustment(in @newUTCAdjustment integer)
    --as
      --  pragma autonomous_transaction;  -- THIS IS NOT AT  required. Put an exception in this call.
    begin
    DECLARE @FUNC varchar(64);
    
        set @FUNC = 'SP_RollupSchedule_SetServerUTCAdjustment';
        -- tbd - passing null session key - is that what we want?
        --fn_Configuration_SetInt
        call fn_Configuration_SetInt(null, 'SERVER', 'UTC_ADJUSTMENT', @newUTCAdjustment);
        commit;
    exception when others then
      cALL SAHARA.SP_Error_RaiseApplicationError
        (   @EInvalidArgument,
            @FUNC,
            'Could not set Day interval time for UTC Adjustment'
        );
end;




if exists(select 1 from sys.sysprocedure where proc_name = 'SP_RollupSchedule_PrepareDatesTable' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_RollupSchedule_PrepareDatesTable
end if;

create procedure sahara.SP_RollupSchedule_PrepareDatesTable()
BEGIN
    DECLARE @FUNC varchar(64) ;
    


    CALL SP_Log_LogDebug(null, @FUNC, 'entry');
    

    call SP_RollupSchedule_CalculateDates(NOW(*) - 1, NOW(*) + 1);

    delete from ROLLUP_SCHEDULE_DATE;

    if exists (select * from vDatesTable) then
        --forall i in dates.first .. dates.last
            insert into ROLLUP_SCHEDULE_DATE(SCHEDULE_DATE) select datevalue from vDatesTable;
            --values(dates(i));
    end if;
end;


if exists(select 1 from sys.sysprocedure where proc_name = 'SP_RollupSchedule_CalculateDates' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_RollupSchedule_CalculateDates
end if;
create procedure sahara.SP_RollupSchedule_CalculateDates
(    
    IN @pStartDate DATETIME,
    IN @pLastDate DATETIME
)
BEGIN    
    DECLARE @FUNC varchar(64);
    DECLARE @pSchedule varchar(32);

    SET @FUNC =  'fn_RollupSchedule_CalculateDates(schedule)';

    CALL SP_Log_LogDebug(null, @FUNC, 'entry');

   

    if @pStartDate is null then
        CALL SP_Error_RaiseApplicationErrorDetail
        (
            @EInvalidArgument,
            @FUNC,
            'The start date of the period of interest must not be null.',
            null
        );
    end if;

    if @pLastDate is null then
        CALL SP_Error_RaiseApplicationErrorDetail
        (
            @EInvalidArgument,
            @FUNC,
            'The last date of the period of interest must not be null.',
            null
        );
    end if;

-- TODO: IDEALLY NOT REQUIRED
    --if trunc(@pStartDate) > trunc(@pLastDate) then
      --  CALL SP_Error_RaiseApplicationErrorDetail
        --(
          --  @EInvalidArgument,
            --@FUNC,
            --'The start date of the period of interest must be less than or equal to the last date.',
            --null
        --);
    --end if;

   select Type
      into @pschedule     
      from ROLLUP_SCHEDULE;

    message @pSchedule to client;
    call  sahara.SP_RollupSchedule_DatesCalculator(@pSchedule, @pStartDate, @pLastDate, null);
end;




if exists(select 1 from sys.sysprocedure where proc_name = 'SP_RollupSchedule_ValidateImpl' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_RollupSchedule_ValidateImpl
end if;
create procedure sahara.SP_RollupSchedule_ValidateImpl(in @pSchedule varchar(32),
                       in @pValidateStartDate bit,
                       in @FUNC varchar(64),
                       in @pSessionKey numeric(10))
BEGIN

    if @pSchedule is null then
       CALL SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSchedule,
            @FUNC,
            'The schedule type is null.',
            @pSessionKey
        );       
    end if;

    case pSchedule
        -- DAILY - every day
        when 'DAILY' then
            -- it's hard to mess up a daily schedule
            return;

        -- BY WEEK - select day(s) (1-7 may be chosen) of the week,
        --           every <N> week(s), starting <D>
        when 'BY WEEK' then
        
        -- BY MONTH 1 - every <First, Second, Third, Forth, Last>
        --              <day of the week> of every <N> month(s), starting <D>
        when 'BY MONTH 1' then
        -- BY MONTH 2 - every day <N> (1-31) of every <M> month(s),
        --              starting <D>
        when 'BY MONTH 2' then
        
    end case;
end;


if exists(select 1 from sys.sysprocedure where proc_name = 'SP_RollupSchedule_DatesCalculator' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_RollupSchedule_DatesCalculator
end if;
create procedure sahara.SP_RollupSchedule_DatesCalculator
(
    in @pSchedule varchar(32),
    in @pFirstDate datetime,
    in @pLastDate datetime,
    in @pNumDates integer
)
BEGIN
    DECLARE @FUNC varchar(64);
    
    set @FUNC =  'SP_RollupSchedule_DatesCalculator';

    --$if $$SAHARA_DEBUG $then
      --  ValidateImpl(pSchedule, true, FUNC, null);
    --$end
    --$if $$SAHARA_ASSERT $then
        --TEST_API.Assert(pFirstDate is not null,
          --              FUNC || ': pFirstDate must not be null');
        --TEST_API.Assert((pLastDate is null and pNumDates is not null) or
            --            (pLastDate is not null and pNumDates is null),
              --          FUNC || ': either pLastDate or pNumDates must be supplied');
        --TEST_API.Assert(pLastDate is null or trunc(pFirstDate) <= trunc(pLastDate),
          --              FUNC || ': pFirstDate must be <= pLastDate');
        --TEST_API.Assert(pNumDates is null or pNumDates >= 1,
          --              FUNC || ': pNumDates must be >= 1');
    --$end

    case @pSchedule
        -- DAILY - every day
        when 'DAILY' then
            CALL  sahara.SP_RollupSchedule_DatesCalculatorDaily(@pSchedule, @pFirstDate,
                                        @pLastDate, @pNumDates);

        -- BY WEEK - select day(s) (1-7 may be chosen) of the week,
        --           every <N> week(s) starting <D>
--        when 'BY WEEK' then
  --          return DatesCalculatorByWeek(pSchedule, trunc(pFirstDate),
      --                                   trunc(pLastDate), pNumDates);

        -- BY MONTH 1 - every <First, Second, Third, Forth, Last>
        --              <day of the week> of every <N> month(s), starting <D>
    --    when 'BY MONTH 1' then
      --      return DatesCalculatorByMonth1(pSchedule, trunc(pFirstDate),
        --                                   trunc(pLastDate), pNumDates);

        -- BY MONTH 2 - every day <N> (1-31) of every <M> month(s), starting <D>
        --when 'BY MONTH 2' then
          --  return DatesCalculatorByMonth2(pSchedule, trunc(pFirstDate),
          --                                 trunc(pLastDate), pNumDates);
    end case;

    -- should never get here
    --return null;
end;


IF EXISTS (SELECT * FROM sys.systable
        WHERE table_name = 'vDatesTable'
        AND table_type = 'GBL TEMP')
THEN
        DROP TABLE sahara.vDatesTable;
END IF;
create global temporary table sahara.vDatesTable
(
       DateValue                 DATETIME
)on commit preserve rows;

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_RollupSchedule_DatesCalculatorDaily' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_RollupSchedule_DatesCalculatorDaily
end if;
create procedure sahara.SP_RollupSchedule_DatesCalculatorDaily
(
    in @pSchedule varchar(32),
    in @pFirstDate datetime,
    in @pLastDate datetime,
    in @pNumDates integer
)
BEGIN   
DECLARE @FUNC varchar(64);
DECLARE @numDates integer;
DECLARE @i integer;
DECLARE @tmpDates DATETIME;

    set @FUNC =  'fn_RollupSchedule_DatesCalculatorDaily';

    if @pLastDate is not null then
        set @numDates =  DATEDIFF(day,@pFirstDate,@pLastDate) + 1;
    else
        set @numDates = @pNumDates;
    end if;

    set @i = 0;
   truncate table   vDatesTable;   
    WHILE @i < @numDates
    loop
        set @tmpDates = DATEADD(day,@i,@pFirstDate);
        insert into vDatesTable(DateValue) VALUES(@tmpDates);
        set @i = @i + 1;
    end loop;

 end;

commit;



if exists(select 1 from sys.sysprocedure where proc_name = 'SP_RollupSchedule_DueRollupSources' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_RollupSchedule_DueRollupSources
end if;
create PROCEDURE sahara.SP_RollupSchedule_DueRollupSources
(
    in @pServerUTCAdjustment integer,
    in @pPollDelayTolerance integer
)
RESULT(ROLLUP_SOURCE_KEY numeric(10),
        SCHEDULE_HASH varchar(64))
BEGIN
    DECLARE @FUNC varchar(64) ;
    DECLARE @UTCAdjustment integer;
    DECLARE @newUTCAdjustment integer;
    DECLARE @nowUTC DATETIME;
    DECLARE @v24Hours numeric(10);
    DECLARE @pollTolerance numeric(10);

    set @FUNC = 'SP_RollupSchedule_DueRollupSources';

    call SP_saharaAPIInit();
    
    --dueCursor DueRollupSourcesCursor;

    --
    -- use an autonomous transaction to make sure resetting the UTC
    -- adjustment succeeds
    --
    
    CALL SP_Log_LogDebug(null, @FUNC, 'entry');

CALL SP_Log_LogDebug(null, @FUNC, 'Adjustement' || @pServerUTCAdjustment);
    -- validate arguments
    if @pServerUTCAdjustment is null then
        CALL SAHARA.SP_Error_RaiseApplicationError
        (   @EInvalidArgument,
            @FUNC,
            'Null server UTC adjustment not allowed.'
        );
    end if;

    if @pPollDelayTolerance is null then
        CALL SAHARA.SP_Error_RaiseApplicationError
        (  @EInvalidArgument,
            @FUNC,
            'Null poll delay tolerance not allowed.');
    end if;

    if @pPollDelayTolerance <= 0 then
        CALL SP_Error_RaiseApplicationError
        (
            @EInvalidArgument,
            @FUNC,
            'The poll delay tolerance must be greater than zero (0).'
        );
    end if;

   
  -- update the server UTC adjustment if needed
   set @UTCAdjustment =        sahara.fn_Conf_GetConfigDayInterval('SERVER', 'UTC_ADJUSTMENT');
    set @newUTCAdjustment  =        @pServerUTCAdjustment*60; -- Convert it into seconds

   if @newUTCAdjustment <> @UTCAdjustment then
       -- tbd - passing null session key - is that what we want?
       CALL SP_Log_LogWarn(null, @FUNC,
                       'the server UTC adjustment has changed from ' ||
                       @UTCAdjustment || ' to ' || @newUTCAdjustment);
     ---TODO Need more testing 
      call SP_RollupSchedule_SetServerUTCAdjustment(@newUTCAdjustment);
       set @UTCAdjustment = @newUTCAdjustment;
   end if;


    set @nowUTC = CURRENT UTC TIMESTAMP;
    set @v24Hours = 86400; --Seconds for 1 day
    set @pollTolerance =    @pPollDelayTolerance*60*(-1);  --  MinutesToIntervalDS(pPollDelayTolerance);

   -- Due to timezones, the date/time for a roll-up source or even the ER
   -- server could differ by as much 24 hours from the current UTC date/time
   -- in either direction.  Add to that, the poll tolerance to come up with
   -- the range of possible roll-up dates to consider.
   call SP_RollupSchedule_PrepareDatesTable();

   --open dueCursor for
     --   with rollups as
            -- a factored sub-query for scheduled roll-ups to return
           message 'nowUTC :' || @nowUTC to client;
           message '@pollTolerance :' || @pollTolerance to client;
           message 'difference :' || DATEADD(second,@pollTolerance,@nowUTC) to client;
            select ROLLUP_SOURCE_KEY,
                    min(SCHEDULE_UTC_DATETIME) as SCHEDULE_UTC_DATETIME  into  #rollups
               from ROLLUP_SCHEDULE_DATETIME schedule
                -- look for scheduled roll-ups in the period of interest
              where schedule.SCHEDULE_UTC_DATETIME between DATEADD(second,@pollTolerance,@nowUTC) and @nowUTC 
                -- they can't have an active batch
                and              not exists (select 1
                                  from ROLLUP_BATCH batches
                                 where batches.IS_COMPLETE = 'F' and batches.IS_DUMMY = 'F'
                                   -- correlated sub-query
                                   and batches.ROLLUP_SOURCE_KEY = schedule.ROLLUP_SOURCE_KEY)
                -- don't repeat successful roll-ups
                and not exists (select 1
                                  from ROLLUP_BATCH successes
                                 where successes.STATUS = 'Succeeded' and successes.IS_DUMMY = 'F'
                                   -- correlated sub-query
                                   and successes.SCHEDULE_HASH = schedule.SCHEDULE_HASH)
              group by schedule.ROLLUP_SOURCE_KEY;

        -- select the information for the scheduled roll-ups
        select ROLLUP_SCHEDULE_DATETIME.ROLLUP_SOURCE_KEY,
               ROLLUP_SCHEDULE_DATETIME.SCHEDULE_HASH
          from ROLLUP_SCHEDULE_DATETIME
          join #rollups
            on ROLLUP_SCHEDULE_DATETIME.ROLLUP_SOURCE_KEY = #rollups.ROLLUP_SOURCE_KEY
           and ROLLUP_SCHEDULE_DATETIME.SCHEDULE_UTC_DATETIME = #rollups.SCHEDULE_UTC_DATETIME

         union all

        -- add any roll-up now sources thay haven't already
        -- been included
        select sources.ROLLUP_SOURCE_KEY,
               '0-0-0-0'
          from ROLLUP_SOURCE sources
         where sources.ENABLED = 'T'
           and sources.ROLLUP_NOW = 'T'
           and not exists (select 1
                             from #rollups
                            -- correlated sub-query
                            where sources.ROLLUP_SOURCE_KEY = #rollups.ROLLUP_SOURCE_KEY);

end;

grant execute on sahara.SP_RollupSchedule_DueRollupSources to sahara_app;
grant execute on sahara.SP_RollupSched_AcquireScriptControl to sahara_app;
grant execute on sahara.SP_RollupSched_ReleaseScriptControl to sahara_app; 
if exists (select 1 from sys.sysprocedure where proc_name = 'SP_RollupSource_FindRollupSourceType') Then
    Drop Function SAHARA.SP_RollupSource_FindRollupSourceType;
end if;

create function sahara.SP_RollupSource_FindRollupSourceType
(
    in @pSessionKey numeric(10),
    in @FUNC  varchar(32),
    in @pType varchar(64)
)
    returns varchar(32)
BEGIN
    declare @typekey numeric(10);
    select ROLLUP_SOURCE_TYPE_KEY
      into @typeKey
      from sahara.ROLLUP_SOURCE_TYPE
     where DISPLAY_NAME = @pType;
    
    if (@typekey is null) then
       call SAHARA.SP_Error_RaiseApplicationErrorDetail( @EInvalidArgument,@FUNC,'Invalid roll-up source type specified  :'||@pType,@pSessionKey);
    else
      return @typeKey;
    end if;    

END;



-------------------------------------------------------------------------------
--
-- updates all fields of a roll-up source
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_RollupSource_Update1')THEN
      DROP PROCEDURE "sahara"."SP_RollupSource_Update1";  
END IF; 

create procedure sahara.SP_RollupSource_Update1
(
    in @pSessionKey numeric(10),
    in @pSourceKey numeric(10),
    in @pName varchar(128),
    in @pType varchar(64),
    in @pWebServerHost varchar(128),
    in @pWebServerPort numeric(5),
    in @pDBHost varchar(128),
    in @pDBPort numeric(5),
    in @pDBPassword varchar(128),
    in @pDBPassword2 varchar(128),
    in @pIsEnabled bit


)
BEGIN
declare @rollupSrcTypeKey numeric(10);
declare @FUNC varchar(32);
  
    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	  set @FUNC= 'SP_RollupSource_Update1';
      set   @rollupSrcTypeKey =  sahara.SP_RollupSource_FindRollupSourceType(@pSessionKey, @FUNC, @pType);

    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

    if (@pIsEnabled  = 1) then
        update SAHARA.ROLLUP_SOURCE
           set NAME = @pName,
               ROLLUP_SOURCE_TYPE_KEY = @rollupSrcTypeKey ,
               WEB_SERVER_HOST = @pWebServerHost,
               WEB_SERVER_PORT = @pWebServerPort,
               DB_HOST = @pDBHost,
               DB_PORT = @pDBPort,
               DB_PASSWORD = @pDBPassword,
               DB_PASSWORD_2 = @pDBPassword2,
               ENABLED = 'T'
         where ROLLUP_SOURCE_KEY = @pSourceKey;
    else
        -- if not enabled, make sure we also clear the roll-up now flag
        update SAHARA.ROLLUP_SOURCE
           set NAME = @pName,
               ROLLUP_SOURCE_TYPE_KEY = @rollupSrcTypeKey,
               WEB_SERVER_HOST = @pWebServerHost,
               WEB_SERVER_PORT = @pWebServerPort,
               DB_HOST = @pDBHost,
               DB_PORT = @pDBPort,
               DB_PASSWORD = @pDBPassword,
               DB_PASSWORD_2 = @pDBPassword2,
               ROLLUP_NOW = 'F',
               ENABLED = 'F'
         where ROLLUP_SOURCE_KEY = @pSourceKey;
    end if;

    -- check that we really update something
   -- if nothing got updated there is problem raise the error.
    if (@@rowcount) = 0 then
        CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceNotFound,
            @FUNC,
            'failed finding roll-up source ' || @pSourceKey || ' to update',
            @pSessionKey
        );
    end if;
end;

grant execute on sahara.SP_RollupSource_Update1 to sahara_app;


-------------------------------------------------------------------------------
--
-- updates type field of roll-up source
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_Source_VersionCheck')THEN
      DROP PROCEDURE "sahara"."SP_Source_VersionCheck";  
END IF; 

create procedure sahara.SP_Source_VersionCheck
(
   	@pSessionKey numeric(10),
	@pBatchKey numeric(10),
	@pVersion varchar(10),
	@pMHVersion varchar(10)
)
BEGIN
declare @oldrollupSrcTypeKey numeric(10);
declare @newrollupSrcTypeKey numeric(10);
declare @sourceKey numeric(10);
declare @FUNC varchar(32);
declare @pType varchar(64);
  
    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	set @FUNC= 'SP_Source_VersionCheck';

    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry' || @pMHVersion);

	select ROLLUP_SOURCE_TYPE_KEY, ROLLUP_SOURCE_KEY into @oldrollupSrcTypeKey, @sourceKey
		from SAHARA.ROLLUP_BATCH
	 where ROLLUP_BATCH_KEY = @pBatchKey;


	if @pVersion = '41' then
		set @pType = 'Veritas Operations Manager 4.0 RU1';	
	else if @pVersion = '42' then
			set @pType = 'Veritas Operations Manager 4.1';
		else if @pVersion = '50' then
			set @pType = 'Veritas Operations Manager 5.0';
			else		
				CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
				(
					@EInvalidArgument,
					@FUNC,				
					'Invalid roll-up source. VOM Advanced does not support version of VOM Management Server.',
					@pSessionKey
				);
			end if;
		end if;
	end if;

    set @newrollupSrcTypeKey =  sahara.SP_RollupSource_FindRollupSourceType(@pSessionKey, @FUNC, @pType);

	 if @oldrollupSrcTypeKey <> @newrollupSrcTypeKey then
			-- update rollup source table with new type key
			update SAHARA.ROLLUP_SOURCE
				set ROLLUP_SOURCE_TYPE_KEY = @newrollupSrcTypeKey,
					ROLLUP_SOURCE_VERSION = @pMHVersion
	         where ROLLUP_SOURCE_KEY = @sourceKey;

			-- update rollup batch table with new type key
			update SAHARA.ROLLUP_BATCH
				set ROLLUP_SOURCE_TYPE_KEY = @newrollupSrcTypeKey
	         where ROLLUP_BATCH_KEY = @pBatchKey;
	 end if;
	
end;

grant execute on sahara.SP_Source_VersionCheck to sahara_app;

-------------------------------------------------------------------------------
--
-- creates a new roll-up source
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'fn_RollupSource_Create1')THEN
      DROP function "sahara"."fn_RollupSource_Create1";  
END IF; 


create function sahara.fn_RollupSource_Create1
(
    in @pSessionKey numeric(10),
    in @pName varchar(128),
    in @pType varchar(64),
    in @pWebServerHost varchar(128),
    in @pWebServerPort numeric(5),
    in @pDBHost varchar(128),
    in @pDBPort numeric(5),
    in @pDBPassword varchar(128),
    in @pDBPassword2 varchar(128),
	in @pSourceOID varchar(512) default 'SourceOID',
	in @pSourceVersion varchar(32) default '4.0.101.0',
    in @pIsEnabled bit default 1
)
    returns integer
BEGIN

declare @rollupSrcTypeKey numeric(10);
declare @sourceKey numeric(10);
declare @FUNC varchar(32);
declare @enabledCh char(1);
declare DUP_VAL_ON_INDEX EXCEPTION FOR SQLSTATE '23505';

	-- initialize global variables
	call sahara.SP_saharaAPIInit();
  
      set @FUNC= 'fn_RollupSource_Create1';
                                        --SP_RollupSource_FindRollupSourceType
      set   @rollupSrcTypeKey =  sahara.SP_RollupSource_FindRollupSourceType(@pSessionKey, @FUNC, @pType);

    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');


    -- we can't use this function directly the statement below because of
    -- the boolean (I think)
    set @enabledCh  = fn_BoolToChar(@pIsEnabled);

    --sourceKey ROLLUP_SOURCE.ROLLUP_SOURCE_KEY%type;
      
    --select S_ROLLUP_SOURCE_KEY.nextval
     -- into @sourceKey
      --from dual;
     -- This is an autoincremented number that need to be added

    insert into ROLLUP_SOURCE( NAME, ROLLUP_SOURCE_TYPE_KEY,
         WEB_SERVER_HOST, WEB_SERVER_PORT, DB_HOST, DB_PORT,
         DB_PASSWORD, DB_PASSWORD_2, ROLLUP_SOURCE_OID, ROLLUP_SOURCE_VERSION, ENABLED)
    values(@pName,@rollupSrcTypeKey,@pWebServerHost, @pWebServerPort, @pDBHost, @pDBPort,
         @pDBPassword, @pDBPassword2, @pSourceOID, @pSourceVersion, @enabledCh);
    -- Last Inserted column
    set @sourcekey = @@identity; 

    return @sourceKey;
exception
    when DUP_VAL_ON_INDEX then
        CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceExists,
            @FUNC,
            'roll-up source already configured.',
            @pSessionKey
        );        
end;

grant execute on sahara.fn_RollupSource_Create1 to sahara_app;

-------------------------------------------------------------------------------
--
-- deletes a roll-up source
--

IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_RollupSource_Delete1')THEN
      DROP procedure "sahara"."SP_RollupSource_Delete1";  
END IF; 

create procedure sahara.SP_RollupSource_Delete1
(
    in @pSessionKey numeric(10),
    in @pSourceKey numeric(10)
)
begin
declare @FUNC varchar(32);
  
    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	  set @FUNC= 'SP_RollupSource_Delete1';
      

    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

    
    
    delete from ROLLUP_SOURCE
     where ROLLUP_SOURCE_KEY = @pSourceKey;

    -- check that we really deleted somethingexception 
exception
   when others then
    
    CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceNotFound,
            @FUNC,
            'failed finding roll-up source ' || @pSourceKey || ' for deletion',
            @pSessionKey
        );
end;

grant execute on sahara.SP_RollupSource_Delete1 to sahara_app;

-------------------------------------------------------------------------------
--
-- sets the enabled flag of a roll-up source
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'fn_RollupSource_Enabled')THEN
      DROP procedure "sahara"."fn_RollupSource_Enabled";  
END IF; 

create procedure sahara.fn_RollupSource_Enabled
(
    in @pSessionKey numeric(10),
    in @pSourceKey numeric(10),
    in @pIsEnabled bit
)
begin
declare @FUNC varchar(32);

      -- initialize global variables
	call sahara.SP_saharaAPIInit();

      set @FUNC= 'fn_RollupSource_Enabled';
      

    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

    

    if (@pIsEnabled = 1) then
        update ROLLUP_SOURCE
           set ENABLED = 'T'
         where ROLLUP_SOURCE_KEY = @pSourceKey;
    else
        -- if not enabled, make sure we also clear the roll-up now flag
        update ROLLUP_SOURCE
           set ROLLUP_NOW = 'F',
               ENABLED = 'F'
         where ROLLUP_SOURCE_KEY = @pSourceKey;
    end if;

if (@@rowcount) = 0 then
        CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceNotFound,
            @FUNC,
            'failed finding roll-up source ' || @pSourceKey || ' to enable/disable',
            @pSessionKey
        );
    end if;

end;

grant execute on sahara.fn_RollupSource_Enabled to sahara_app;
-------------------------------------------------------------------------------
--
-- sets the roll-up now flag
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_RollupSource_RollupNow')THEN
      DROP PROCEDURE "sahara"."SP_RollupSource_RollupNow";  
END IF; 

create procedure sahara.SP_RollupSource_RollupNow
(
    in @pSessionKey numeric(10),
    in @pSourceKey numeric(10)
)
BEGIN
declare @FUNC varchar(32);
declare @vRollupNow char(1);
declare @vEnabled char(1);
declare @rslt bit;
declare @activeBatches int;
--DECLARE sourceCursor CURSOR  for select ROLLUP_NOW as @RollupNow, ENABLED as @Enabled
--from SAHARA.ROLLUP_SOURCE where ROLLUP_SOURCE_KEY = @pSourceKey for update by lock;   

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

set @FUNC = 'SP_RollupSource_RollupNow';
   call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

-- we want all or nothing from this call
--    savepoint RollupSourceRollupNowSP;
    -- tbd - audit based on session key


    -- select the roll-up source so we can update it after validation
   -- begin of handle input xml data
 
 
    


    -- validate the roll-up source
--    open sourceCursor;


select s.ROLLUP_NOW,s.ENABLED  into @vRollupNow, @vEnabled
          from SAHARA.ROLLUP_SOURCE  as s
                       where s.ROLLUP_SOURCE_KEY = @pSourceKey;

    --LBL1: LOOP
--    fetch next sourceCursor into @vRollupNow, @vEnabled;
--   message 'after the fetch' to client;
   
 if (SQLCODE <> 0) then
        -- This will include the row not found error.
   --     close sourceCursor;
--           rollback to savepoint RollupSourceRollupNowSP;
        CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceNotFound,
            @FUNC,
            'no rollup source found with key ' || @pSourceKey ,
            @pSessionKey
        );

   end if;

-- message  'fetch successful' to client;    
    -- don't set the flag if it's already set
--        set @rslt  = fn_BoolToChar(@vRollupNow);
    if (@vRollupNow = 'T') then
--        close sourceCursor;

        CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupNowAlreadyFlagged,
            @FUNC,
            'rollupNow already set to true for the  key ' || @pSourceKey ,
            @pSessionKey
        );

    end if;

    -- we can't request roll-up now for a source that is not enabled
-- message  'Checking Enabled Flag' to client;      
--   set @rslt  = fn_BoolToChar(@vEnabled);
    if (@vEnabled = 'F') then
--        close sourceCursor;
--          rollback to savepoint RollupSourceRollupNowSP;
    CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupNowSourceNotEnabled,
            @FUNC,
            'roll-up now is not allowed for sources that are not ' ||
            'enabled - source key: ' || @pSourceKey,
            @pSessionKey
        );
        
    end if;

    -- make sure there isn't a batch running for the source
    select count(*)
      into @activeBatches
      from SAHARA.ROLLUP_BATCH 
     where ROLLUP_SOURCE_KEY = @pSourceKey
       and IS_DUMMY = 'F' and IS_COMPLETE = 'F';
-- message  'Checking Active Batches' to client;      
    if (@activeBatches > 0) then
--        close sourceCursor;
--         rollback to savepoint RollupSourceRollupNowSP;

       CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupNowRunningBatch,
            @FUNC,
            'roll-up now is not allowed for sources that have ' ||
            'active batches - source key: ' || @pSourceKey,
            @pSessionKey
        );

        
    end if;
-- message 'Finally ending it' to client;
    -- all looks well, so do the update
    update SAHARA.ROLLUP_SOURCE
       set ROLLUP_NOW = 'T'
      where ROLLUP_SOURCE_KEY = @pSourceKey;  ;
   
    if (@@rowcount) = 0 then
             CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceNotFound,
            @FUNC,
            'failed to set the rollup_now flag for the source key ' || @pSourceKey ,
            @pSessionKey
        );
    end if;   




--   close sourceerror;
   commit;

end;

grant execute on sahara.SP_RollupSource_RollupNow to sahara_app;
-------------------------------------------------------------------------------
--
-- set the roll-up time for a roll-up source

-------------------------------------------------------------------------------
--
-- set the roll-up time for a roll-up source
--

-- sets the roll-up now flag
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_RollupSource_RollupTime')THEN
      DROP PROCEDURE "sahara"."SP_RollupSource_RollupTime";  
END IF; 

create procedure sahara.SP_RollupSource_RollupTime
(
    IN @pSessionKey numeric(10),
    IN @pSourceKey numeric(10),
    IN @pHour numeric(2),
    IN @pMinute numeric(2),
    IN @pClock varchar(32)
)
BEGIN
declare @FUNC varchar(32);
declare @vRollupNow char(1);
declare @vEnabled bit;
declare @rslt bit;
declare @activeBatches int;


    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	set @FUNC = 'SP_RollupSource_RollupTime';
   call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

   

    update SAHARA.ROLLUP_SOURCE
       set ROLLUP_HOUR = @pHour,
           ROLLUP_MINUTE = @pMinute,
           ROLLUP_CLOCK = @pClock
     where ROLLUP_SOURCE_KEY = @pSourceKey;

    -- check that we really updated something
    
    if (@@rowcount) = 0 then
        CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceNotFound,
            @FUNC,
            'failed finding roll-up source ' || @pSourceKey || ' to set the roll-up time',
            @pSessionKey
        );
    end if;

end;

grant execute on sahara.SP_RollupSource_RollupTime to sahara_app;
-------------------------------------------------------------------------------
--
-- set the UTC adjustment for a roll-up source
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_RollupSource_UTCAdjustment')THEN
      DROP PROCEDURE "sahara"."SP_RollupSource_UTCAdjustment";  
END IF; 

create procedure sahara.SP_RollupSource_UTCAdjustment
(
    @pSessionKey numeric(10),
    @pSourceKey numeric(10),
    @pUTCAdjustment integer
)
BEGIN
declare @FUNC varchar(32);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	set @FUNC = 'SP_RollupSource_UTCAdjustment';

    --utcAdjustment constant interval day to second :=
      --  MinutesToIntervalDS(@pUTCAdjustment);

    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

    update SAHARA.ROLLUP_SOURCE
       set UTC_ADJUSTMENT = @pUTCAdjustment
     where ROLLUP_SOURCE_KEY = @pSourceKey;

    -- check that we really updated something
    if (@@rowcount) = 0 then
             CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ERollupSourceNotFound,
            @FUNC,
            'failed finding roll-up source ' || @pSourceKey || ' to set the  UTCAdjustment',
            @pSessionKey
        );
    end if;   
end;

grant execute on sahara.SP_RollupSource_UTCAdjustment to sahara_app;

IF EXISTS (SELECT 1 FROM sys.sysprocedure  
        WHERE proc_name = 'SP_RollupSource_ValidateName')THEN
      DROP PROCEDURE "sahara"."SP_RollupSource_ValidateName";  
END IF; 


create procedure sahara.SP_RollupSource_ValidateName(in @pName  varchar(32))
BEGIN
declare @FUNC varchar(32);
declare @VALID_CHARS varchar(128);


    set @FUNC = 'ValidateName';
    set @VALID_CHARS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-.';


    if (@pName is null) then
       CALL SAHARA.SP_Error_RaiseApplicationError
        (            @ERollupSourceInvalidName,
            'the roll-up source name is null ' 
        );        
    end if;

    if (LENGTH(@pName) > 128) then
        CALL SAHARA.SP_Error_RaiseApplicationError
        (
            @ERollupSourceInvalidName,            
            'the roll-up source name may not exceed 128 characters  in length' || @PName
        );        
    end if;

    --for i in 1 .. length(pName) loop
      --  if 0 = instr(VALID_CHARS, upper(substr(pName, i, 1))) then
       
       -- Any thing other than alphanumeric will be discarded completly
        IF   @pName  REGEXP   '[[:upper:][:digit:]-_.]+'   then
          --message 'Every thing okay ' to client;
        ELSE
            CALL SAHARA.SP_Error_RaiseApplicationError
        (
            @ERollupSourceInvalidName,
            'the roll-up source name contains an invalid Character' || @PName
        );
       end if;
    
end;

/*
 ********************************************************************************
 * fn_GetSource_Pwd():
 * Decrypts the encrypted password stored for rollup source based on batch input. 
 ********************************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_GetSource_Pwd' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_GetSource_Pwd
end if;

CREATE FUNCTION SAHARA.fn_GetSource_Pwd (@pSourceKey numeric(10) default 0, @pBatchKey numeric(10) default 0) 
returns varchar(128)
BEGIN
	DECLARE @value varchar(128);

	if @pSourceKey = 0 or @pSourceKey is null then
		if @pBatchKey <> 0 then
			SELECT CAST(DECRYPT(ROLLUP_SOURCE.DB_PASSWORD, ROLLUP_SOURCE.ROLLUP_SOURCE_OID) AS varchar(128)) into @value
			FROM sahara.ROLLUP_BATCH
				join sahara.ROLLUP_SOURCE on ROLLUP_BATCH.ROLLUP_SOURCE_KEY=ROLLUP_SOURCE.ROLLUP_SOURCE_KEY
			 where ROLLUP_BATCH_KEY=@pBatchKey;
		end if;
	else
		SELECT CAST(DECRYPT(ROLLUP_SOURCE.DB_PASSWORD, ROLLUP_SOURCE.ROLLUP_SOURCE_OID) AS varchar(128)) into @value
		FROM sahara.ROLLUP_SOURCE				
		 where ROLLUP_SOURCE_KEY=@pSourceKey;
	end if;

       return @value;

END;

ALTER FUNCTION sahara.fn_GetSource_Pwd SET HIDDEN;

grant execute on sahara.fn_GetSource_Pwd to sahara_app;

IF  EXISTS ( SELECT 1 FROM SYS.SYSTRIGGER WHERE TRIGGER_NAME = 'T_ROLLUP_SOURCE_BIUFER' ) THEN
  DROP TRIGGER T_ROLLUP_SOURCE_BIUFER;
END IF;

CREATE TRIGGER "T_ROLLUP_SOURCE_BIUFER" BEFORE INSERT,UPDATE
ORDER 8 ON SAHARA.ROLLUP_SOURCE
REFERENCING NEW AS new_row
            OLD AS old_row
FOR EACH ROW
BEGIN
declare @ptime integer;

if inserting THEN
	-- set @ptime = '0' || new_row.ROLLUP_HOUR || ':' || new_row.ROLLUP_MINUTE || ':00';
	set new_row.UPPER_NAME = upper(new_row.NAME);
    --  set new_row.ROLLUP_TIME_AS_INTERVAL =  SECONDS(@ptime);
	-- For insertion it ought to be 0 seconds.
	set  new_row.ROLLUP_TIME_AS_INTERVAL =  0;

	-- encrypt password here
	set new_row.DB_PASSWORD=ENCRYPT(new_row.DB_PASSWORD, new_row.ROLLUP_SOURCE_OID);
	set new_row.DB_PASSWORD_2=ENCRYPT(new_row.DB_PASSWORD_2, new_row.ROLLUP_SOURCE_OID);

elseif updating then
	set @ptime =  (new_row.ROLLUP_HOUR*60*60)  +  (new_row.ROLLUP_MINUTE*60) ;
	set new_row.UPPER_NAME = upper(new_row.NAME);
	set new_row.ROLLUP_TIME_AS_INTERVAL =  @ptime;
	--set new_row.ROLLUP_TIME_AS_INTERVAL =  0;
	if old_row.ROLLUP_TIME_AS_INTERVAL <> new_row.ROLLUP_TIME_AS_INTERVAL or
		old_row.ROLLUP_CLOCK <> new_row.ROLLUP_CLOCK then
		set new_row.ROLLUP_TIME_REVISION = old_row.ROLLUP_TIME_REVISION + 1;
	end if;

	if update(DB_PASSWORD) then
		-- encrypt password here
		set new_row.DB_PASSWORD=ENCRYPT(new_row.DB_PASSWORD, new_row.ROLLUP_SOURCE_OID);
		set new_row.DB_PASSWORD_2=ENCRYPT(new_row.DB_PASSWORD_2, new_row.ROLLUP_SOURCE_OID);
	end if;
  end if;

END;

ALTER TRIGGER T_ROLLUP_SOURCE_BIUFER on SAHARA.ROLLUP_SOURCE SET HIDDEN;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_GetSource_PwdFromName' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_GetSource_PwdFromName
end if;

CREATE FUNCTION SAHARA.fn_GetSource_PwdFromName (@pSourceName varchar(256))
returns varchar(128)
BEGIN
        DECLARE @rollupkey numeric(10);
        select rollup_source_key into @rollupkey from sahara.rollup_source rs where rs.upper_name  = upper(@pSourceName); 
        return SAHARA.fn_GetSource_Pwd(@rollupkey);
END;
ALTER FUNCTION sahara.fn_GetSource_PwdFromName SET HIDDEN;

grant execute on sahara.fn_GetSource_PwdFromName to sahara_app;
/*
 * $Source: /project/sahara/asadb/scripts/voma_upgrade.sql,v $
 *
 * $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
 *
 * $Copyright: Copyright (c) 2012 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 *
 */


/*
 **********************************************************************
 * This module provide stored procedures and functions for configuring rules and expressions.
 * Editing of rules is considered editing of Enterprise Reporter configuration. 
 * Only one session may edit configuration at a time. Before any other function or procedure in this 
 * package may be used, the caller must first successfully call SP_Config_AcquireExclusiveLock. 
**********************************************************************
*/

message 'Creating/replacing Tier Rule module stored procedures and functions...' to client;

/*
 **********************************************************************
 * fn_TierRule_Create1():
 * Creates a new rule.

 * Return value:
 * The unique identifier for the new rule.
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_TierRule_Create1' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_TierRule_Create1
end if;

CREATE FUNCTION SAHARA.fn_TierRule_Create1 
(
    @pContext varchar(32),
	@pRuleText long varchar,
    @pGUIHint long varchar default null
) 
	returns integer
BEGIN
	DECLARE @func VARCHAR(64);
	DECLARE @vGUIHint long varchar; 
    DECLARE @vRuleText long varchar;
    DECLARE @vSQLCode long varchar;
    DECLARE @ruleKey numeric(10);

	SET @vGUIHint = trim(@pGUIHint);
	SET @vRuleText = trim(@pRuleText);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_TierRule_Create1';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    -- create the SQL text from the rule text
    Set @vSQLCode = fn_TierRule_RuleTextToSQLCode(@pContext, @vRuleText);

    insert into RULE(RULE_KEY, RULE_CONTEXT, GUI_HINT, RULE_TEXT, SQL_CODE)
    values(@ruleKey, @pContext, @vGUIHint, @vRuleText, @vSQLCode);

	return @@identity;
end;

/*
 **********************************************************************
 * SP_TierRule_Delete1():
 * Deletes a rule. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_TierRule_Delete1' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_TierRule_Delete1
end if;

CREATE PROCEDURE SAHARA.SP_TierRule_Delete1 
(
    @pRuleKey numeric(10)
) 
BEGIN
	DECLARE @func VARCHAR(64);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_TierRule_Delete1';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    delete from RULE
     where RULE_KEY = @pRuleKey;
end;


/*
 **********************************************************************
 * SP_TierRule_Update1():
 * Updates an existing rule. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_TierRule_Update1' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_TierRule_Update1
end if;

CREATE PROCEDURE SAHARA.SP_TierRule_Update1 
(
    @pRuleKey numeric(10),
	@pRuleText long varchar,
    @pGUIHint long varchar default null
) 
BEGIN
	DECLARE @func VARCHAR(64);
	DECLARE @vGUIHint long varchar; 
    DECLARE @vRuleText long varchar;
    DECLARE @vSQLCode long varchar;
    DECLARE @vContext varchar(32);

    DECLARE @ruleCursor CURSOR FOR
        select RULE_CONTEXT
          from RULE
         where RULE_KEY = @pRuleKey
           for update;

	SET @vGUIHint = trim(@pGUIHint);
	SET @vRuleText = trim(@pRuleText);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_TierRule_Update1';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    -- get the context and validate the rule's existence as long as we
    -- have to execute a query anyway
    open @ruleCursor;
    fetch next @ruleCursor into @vContext;
	
    -- create the SQL text from the rule text
    set @vSQLCode = fn_TierRule_RuleTextToSQLCode(@vContext, @vRuleText);

    -- update the rule
    update RULE
       set GUI_HINT = @vGUIHint,
           RULE_TEXT = @vRuleText,
           SQL_CODE = @vSQLCode
     where current of @ruleCursor;

    close @ruleCursor;

exception
    when others then

		close @ruleCursor;

		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@ERuleError,
			@FUNC,
			'Error while updating tier rule: ' || @pRuleKey,
			sahara.Fn_Config_GetActiveSessionKey()
		); 

end;

/*
 **********************************************************************
 * fn_TierRule_RuleTextToSQLCode():
 * The function uses a rule context's available tokens to convert a rule 
 * from the user-friends rule text form into the executable SQL text form. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_TierRule_RuleTextToSQLCode' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_TierRule_RuleTextToSQLCode
end if;

CREATE FUNCTION SAHARA.fn_TierRule_RuleTextToSQLCode 
(
    @pContext varchar(32),
	@pRuleText long varchar
) 
	returns long varchar
BEGIN
	DECLARE @func VARCHAR(64);
    DECLARE @vSQLCode long varchar;
	DECLARE @ruleToken varchar(30);
	DECLARE @ruleSqlcode varchar(30);

	SET @vSQLCode = trim(@pRuleText);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_TierRule_RuleTextToSQLCode';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    -- do the token substitution as case insensitive
	FOR @RuleElementLoop AS @RuleElementCursor CURSOR FOR (   
		select RULE_TOKEN as ruleToken, SQL_CODE as ruleSqlcode
		  from RULE_ELEMENT
		 where RULE_CONTEXT = @pContext
	)
	DO
		if ruleToken is not null and ruleToken!='' then
			set @vSQLCode = replace(@vSQLCode, ruleToken, ruleSqlcode);
		end if;
	END FOR;    

    return @vSQLCode;
end;

/*
 **********************************************************************
 * fn_TierRule_Expression():
 * Generate the tier rule expression for substitution into the rule 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_TierRule_Expression' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_TierRule_Expression
end if;

CREATE FUNCTION SAHARA.fn_TierRule_Expression 
(
    @pContext varchar(32)
) 
	returns long varchar
BEGIN
	DECLARE @func VARCHAR(64);
    DECLARE @vSQLCode long varchar;
	DECLARE @vName varchar(64);
	DECLARE @vCost numeric(12,6);
	DECLARE @ct integer;
	DECLARE @vDefaultTierName varchar(64);
	DECLARE @vDefaultTierCost numeric(12,6);	
	DECLARE @vExpression long varchar;

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_TierRule_Expression';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    -- We should not ever get here if there are no tiers at all.  Their
    -- must be at least a default tier.

	-- get the default tier
	select TIER.DISPLAY_NAME, TIER.COST
	  into @vDefaultTierName, @vDefaultTierCost
	  from TIER
	 where TIER_CONTEXT = @pContext
	   and IS_DEFAULT = 'T';

    -- get the tiers with rules
	select count(*)
	  into @ct
	  from TIER
	  join RULE
		on TIER.RULE_KEY = RULE.RULE_KEY
	 where TIER_CONTEXT = @pContext;


	if @ct = 0 then
        -- There is a default tier, but no rule tiers, so we assign
        -- the default tier to everything.  There's no need for a complex
        -- expression.
        SET @vExpression = '''' || @vDefaultTierName|| '''';
    else
        -- there are rule tiers and a default tier
        set @vExpression = '(case';

		FOR @TierRuleLoop AS @TierRuleCursor CURSOR FOR (   
			select TIER.DISPLAY_NAME as vName, TIER.COST as vCost, RULE.SQL_CODE as vSQLCode
			  from TIER
			  join RULE
				on TIER.RULE_KEY = RULE.RULE_KEY
			 where TIER_CONTEXT = @pContext
			 order by RANK
		)
		DO
			set @vExpression = @vExpression || ' when ' || vSQLCode || ' then '''|| vName || '''';
		END FOR;    

        -- add the default tier and end the expression
        set @vExpression = @vExpression || ' else '''||@vDefaultTierName || ''' end)';
    end if;

    return @vExpression;
end;


/*
 **********************************************************************
 * SP_TierRule_Execute():
 * Executes a tier rule for a tier context.  
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_TierRule_Execute' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_TierRule_Execute
end if;

CREATE PROCEDURE SAHARA.SP_TierRule_Execute 
(
   @pContext varchar(32)
) 
BEGIN
	DECLARE @func VARCHAR(64);
	DECLARE @vRuleSQL long varchar; 
	DECLARE @sqlstate varchar(5);
	DECLARE @vDefaultTierName varchar(64);

	SET @func = 'sahara.SP_TierRule_Execute';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

    -- validate the context and get the rule template
	select RULE_SQL
	 into @vRuleSQL
	from TIER_CONTEXT
	 where TIER_CONTEXT.TIER_CONTEXT = @pContext;

    -- see if we really updated anything
	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@ETierContextNotFound,
			@func,
			'non-existent tier context: ' || @pContext,
			sahara.Fn_Config_GetActiveSessionKey()
		);
	end if;

	call sahara.SP_Log_LogInfo
	(
		sahara.Fn_Config_GetActiveSessionKey(),
		@func,
		'evaluating tier rules for context: ' || @pContext
	);

    -- figure out if there is anything to do, and initialize the "global"
    -- package variables that we use in implementing the tier expression
    -- if there is
	-- get the default tier
	select TIER.DISPLAY_NAME
	  into @vDefaultTierName
	  from TIER
	 where TIER_CONTEXT = @pContext
	   and IS_DEFAULT = 'T';

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
        call sahara.SP_Log_LogWarn
        (
            sahara.Fn_Config_GetActiveSessionKey(),
            @func,
			'not tier rules configured for: ' || @pContext        
		);

		return;
    end if;

    -- there us something to do - contruct the full rule SQL
    set @vRuleSQL = 'begin ' ||
                replace(@vRuleSQL, '$TIER_RULE_EXPRESSION$', fn_TierRule_Expression(@pContext)) ||
                '; end;';

	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'SQL: ' || @vRuleSQL);

    execute immediate @vRuleSQL;

	call sahara.SP_Log_LogInfo
	(
		sahara.Fn_Config_GetActiveSessionKey(),
		@func,
		'successfully executed tier rules for: ' || @pContext
	);
end;


/*
 **********************************************************************
 * SP_TierRule_Validate():
 * Validates a tier rule  
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_TierRule_Validate' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_TierRule_Validate
end if;

CREATE PROCEDURE SAHARA.SP_TierRule_Validate 
(
   @pContext varchar(32),
   @pRuleText long varchar
) 
BEGIN
	DECLARE @func VARCHAR(64);
    DECLARE @vExpression long varchar;
    DECLARE @vRuleSQL long varchar;
    DECLARE @vSQLErrm long varchar;
	DECLARE @sqlstate varchar(5);
	DECLARE @serror long varchar;
	DECLARE @result long varchar;
	DECLARE @NL char(2);

	SET @func = 'sahara.SP_TierRule_Validate';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');

	SET @NL = char(10);

    -- validate the context and get the rule template
	select RULE_SQL
	  into @vRuleSQL
	  from TIER_CONTEXT
	 where TIER_CONTEXT.TIER_CONTEXT = @pContext;

	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail
		(
			@ETierContextNotFound,
			@func,
			'non-existent tier context: ' || @pContext,
			sahara.Fn_Config_GetActiveSessionKey()
        );
	end if;

    -- convert the rule text from the user form to the SQL form
    CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'rule text: ' || @pRuleText);
    SET @vExpression = fn_TierRule_RuleTextToSQLCode(@pContext || ' TIER', @pRuleText);
    CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'rule SQL: ' || @vExpression);

    -- create an expression similar to what the larger rule would have when
    -- all tiers rule are included
    set @vExpression =
        'case when ' || @vExpression || ' then ''Tier Value'' else null end';

    -- construct the full rule SQL by inserting the expression
    set @vRuleSQL = 'begin ' ||
                replace(@vRuleSQL, '$TIER_RULE_EXPRESSION$', @vExpression) ||
                '; end;';

    CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func,
                    'validation SQL: ' || @vRuleSQL);

	-- parse the SQL to validate it
	set @result = SQLDIALECT (@vRuleSQL);

	-- if there was an error, add some information, and raise it
	if @result not in ( 'Watcom-SQL', 'Transact-SQL' ) then
			CALL SP_Log_LogError
			(
				sahara.Fn_Config_GetActiveSessionKey(), 
				@func,
				'Tier rule validation failed -- error message: ' || @NL || @result
			);
			
			CALL SP_Error_RaiseApplicationErrorDetail
			(
				@ETierRuleValidateFailed,
				@func,
				'Tier rule validation failed at compile time.',
				sahara.Fn_Config_GetActiveSessionKey()
			);
	end if;

	begin
		--truncate the staging table for faster query execution
		execute immediate 'truncate table SAHARA_APP.LUN_';

		--since SQLDIALECT just validates query syntactically
		--so execute the rule query in real to see whether the query is valid or not		
		execute immediate @vRuleSQL;
	exception        
        when others then
            select ERRORMSG(SQLCODE) into @serror;

			CALL SP_Log_LogError
			(
				sahara.Fn_Config_GetActiveSessionKey(), 
				@func,
				'Tier rule validation failed -- error message: ' || @NL || @serror
			);
			
			CALL SP_Error_RaiseApplicationErrorDetail
			(
				@ETierRuleValidateFailed,
				@func,
				'Tier rule validation failed at execution time.',
				sahara.Fn_Config_GetActiveSessionKey()
			);
	end;
end;


/*
 **********************************************************************
 * T_RULELEMENT_BIUFER
 * 
 * TRIGGER to set RULE_TOKEN_UPPER per RULE_ELEMENT.
 **********************************************************************
 */
if exists(select 1 from sys.systrigger where trigger_name = 'T_RULELEMENT_BIUFER') then
   drop TRIGGER T_RULELEMENT_BIUFER
end if;

create or replace trigger SAHARA.T_RULELEMENT_BIUFER
    before insert,update
    on SAHARA.RULE_ELEMENT
	REFERENCING NEW AS newrow
	            OLD AS oldrow
    FOR EACH ROW
BEGIN
	set newrow.RULE_TOKEN_UPPER = upper(newrow.RULE_TOKEN);
END;
message 'Creating/replacing SESSION module stored procedures...' to client;

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Session_BeginETLSession' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Session_BeginETLSession
end if;

CREATE FUNCTION "SAHARA"."fn_Session_BeginETLSession" () 
returns integer
BEGIN
	DECLARE @sessionKey numeric(10);

	insert into sahara."SESSION"
        (TYPE,START_TIME)
    values
        ('ETL',sahara.fn_Conf_UTCTimestamp());

	select max(SESSION_KEY) into @sessionKey from sahara."SESSION" where TYPE='ETL'; 

	return @sessionKey;

END;

grant execute on sahara.fn_Session_BeginETLSession to sahara_app;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Session_BeginGUISession' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Session_BeginGUISession
end if;

CREATE FUNCTION "SAHARA"."fn_Session_BeginGUISession" (@pLoginName varchar(80), @pDomain varchar(80)) 
returns integer
BEGIN
	DECLARE @sessionKey numeric(10);
	DECLARE @usersKey numeric(10);

 call sahara.SP_saharaAPIInit();

    -- we want all or nothing from this call
    savepoint SessionBeginGUISessionSP;

	set @usersKey = sahara.fn_USER_FindOrCreate(null, @pLoginName, @pDomain);

    insert into "SESSION"
        (TYPE, START_TIME, USERS_KEY)
    values
        ('GUI', sahara.fn_Conf_UTCTimestamp(), @usersKey);

	select max(SESSION_KEY) into @sessionKey from sahara."SESSION" where TYPE='GUI'; 

	return @sessionKey;

EXCEPTION
	WHEN others THEN
		ROLLBACK TO SAVEPOINT SessionBeginGUISessionSP;
		raiserror 30007 'Not able to create GUI session for ' || @pLoginName || ' User';
END;

grant execute on sahara.fn_Session_BeginGUISession to sahara_app;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Session_BeginIESession' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Session_BeginIESession
end if;

CREATE FUNCTION "SAHARA"."fn_Session_BeginIESession" () 
returns integer
BEGIN
	DECLARE @sessionKey numeric(10);

	insert into sahara."SESSION"
        (TYPE,START_TIME)
    values
        ('IE',sahara.fn_Conf_UTCTimestamp());

	select max(SESSION_KEY) into @sessionKey from sahara."SESSION" where TYPE='IE'; 

	return @sessionKey;

END;

grant execute on sahara.fn_Session_BeginIESession to sahara_app;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Session_BeginServerSession' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Session_BeginServerSession
end if;

CREATE FUNCTION "SAHARA"."fn_Session_BeginServerSession" (@pComment varchar(256) default null) 
returns integer
BEGIN
	DECLARE @sessionKey numeric(10);

	insert into sahara."SESSION"
        (TYPE,START_TIME,"COMMENT")
    values
        ('SERVER',sahara.fn_Conf_UTCTimestamp(),@pComment);

	select max(SESSION_KEY) into @sessionKey from sahara."SESSION" where TYPE='SERVER'; 

	return @sessionKey;

END;

grant execute on sahara.fn_Session_BeginServerSession to sahara;

-------------------------------------------------------------------------------

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Session_EndSession' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Session_EndSession
end if;

CREATE PROCEDURE "SAHARA"."fn_Session_EndSession" (@pSessionKey numeric(10)) 
BEGIN

       call sahara.SP_saharaAPIInit();

	update sahara."SESSION"
       set END_TIME = sahara.fn_Conf_UTCTimestamp()
     where SESSION_KEY = @pSessionKey;

END;

grant execute on sahara.fn_Session_EndSession to sahara_app;

-------------------------------------------------------------------------------
/*
 * $Source: /project/sahara/asadb/scripts/voma_upgrade.sql,v $
 *
 * $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
 *
 * $Copyright: Copyright (c) 2012 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 *
 */

/*
 **********************************************************************
 * This module provides functions and procedures for editing tiers and rules for tiers.

 * The module enforces the following business rules:
 * - there must be exactly one default tier for a tier context
 * - all tiers of a tier context must have consecutive ranks starting from one (1)
 * - a tier may have exactly zero (0) or one (1) rule
 * - if a tier has a rule, the rule context is "tier context" + " TIER" and is unique for the rule (rules are not shared)
 * - when a tier is deleted, so is its rule

 * Editing of tiers is considered editing of VOM Advanced configuration. 
 * Only one session may edit configuration at a time. Before any other function or procedure in this package may be used, the caller must first successfully call SP_Config_AcquireExclusiveLock. 
 **********************************************************************
*/

message 'Creating/replacing Tier module stored procedures and functions...' to client;

/*
 **********************************************************************
 * fn_Tier_Create1():
 * Creates a tier with no rule. The new tier is assigned the next available rank for the context. The first tier created for a context is made the default tier; subsequent tiers are not.
 * Return:
 * The unique identifier for the new tier. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'fn_Tier_Create1' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_Tier_Create1
end if;

CREATE FUNCTION SAHARA.fn_Tier_Create1 
(
	@pContext varchar(32),
	@pDisplayName varchar(64),
	@pDescription long varchar,
	@pCost numeric(12,6)
) 
	returns integer
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @ct integer;
    DECLARE @vRank numeric(2);
    DECLARE @vIsDefault char(1);
    DECLARE @tierKey numeric(10);

    -- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.fn_Tier_Create1';
	CALL SP_Log_LogDebug(@gConfigActiveSessionKey, @func, 'entry');

	-- 2413008 BV Update and other configuration activity blocks Tier operations.
	-- TODO: Revisit this behaviour and introduce one more Tier level locking to be modular.
	-- CALL SP_Config_RequireControl(@func);

    -- validate the context
    select count(*)
      into @ct
      from TIER_CONTEXT
     where TIER_CONTEXT = @pContext;

    if @ct = 0 then
		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ETierContextNotFound,
			@FUNC, 
			'non-existent tier context: ' || @pContext,
			sahara.Fn_Config_GetActiveSessionKey()
		);
    end if;

    -- see if this is the first tier in the context
    select count(*)
      into @ct
      from TIER
     where TIER_CONTEXT = @pContext;

	if @ct = 0 then
		set @vIsDefault = fn_BoolToChar(1);
	else
		set @vIsDefault = fn_BoolToChar(0);
    end if;
    

    set @vRank = @ct + 1;

    -- create the new tier
    insert into TIER(TIER_CONTEXT, DISPLAY_NAME,
                     DESCRIPTION, RANK, COST, IS_DEFAULT)
    values(@pContext, @pDisplayName,
           @pDescription, @vRank, @pCost, @vIsDefault);

	return @@identity;
end;

grant execute on sahara.fn_Tier_Create1 to sahara_app;

/*
 **********************************************************************
 * SP_Tier_Delete1():
 * Deletes a tier. The rank for all remaining tiers of the tier context 
 * are adjusted to close any gap left by the deletion. If the deleted tier 
 * is the default, the highest ranked remaining tier becomes the default. 
 * Any rule associated with the tier is also deleted. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_Tier_Delete1' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_Tier_Delete1
end if;

CREATE PROCEDURE SAHARA.SP_Tier_Delete1 
(
	@pTierKey  numeric(10)
) 
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @ct integer;
    DECLARE @vRank numeric(2);
    DECLARE @vIsDefault char(1);
    DECLARE @vTierContext varchar(32);

    DECLARE @deleteCursor CURSOR FOR
        select TIER_CONTEXT, RANK, IS_DEFAULT
          from TIER
         where TIER_KEY = @pTierKey
           for update;

    -- we want all or nothing from this call
    savepoint TierAPIDelete1SP;

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_Tier_Delete1';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
	
	-- 2413008 BV Update and other configuration activity blocks Tier operations.
	-- TODO: Revisit this behaviour and introduce one more Tier level locking to be modular.
	-- CALL SP_Config_RequireControl(@func);

    -- use a cursor to validate the existence of the tier and get
    -- the rank and "is default" flags to adjust other tiers in the
    -- context
    open @deleteCursor;
    fetch next @deleteCursor into @vTierContext, @vRank, @vIsDefault;

	-- we expect to find it
    if @@fetch_status = -2 then
        -- else - we didn't find an existing record
        close @deleteCursor;

		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ETierNotFound,
			@FUNC,
			'failed finding tier ' || @pTierKey || ' for deletion',
			sahara.Fn_Config_GetActiveSessionKey()
		);
    end if;
	
    -- delete the tier
    delete TIER
     where current of @deleteCursor;

    close @deleteCursor;
    -- a trigger will delete any associated rule

    -- adjust the ranks of the remaining tiers
    update TIER
       set RANK = RANK - 1
     where TIER_CONTEXT = @vTierContext
       and RANK >= @vRank;

    -- if the deleted tier was the default, pick a new one
    if @vIsDefault = 'T' then
        update TIER
           set IS_DEFAULT = 'T'
         where TIER_CONTEXT = @vTierContext
           and RANK = (select max(RANK)
                         from TIER
                        where TIER_CONTEXT = @vTierContext);
    end if;

exception
    when others then
			begin
		        rollback to savepoint TierAPIDelete1SP;
				close @deleteCursor;
			exception
			    when others then
				begin
				end;
			end;

			CALL SP_Error_RaiseApplicationErrorDetail
			(
				@ETierError,
				@FUNC,
				'Error while deleting tier: ' || @pTierKey,
				sahara.Fn_Config_GetActiveSessionKey()
			); 
end;

grant execute on sahara.SP_Tier_Delete1 to sahara_app;

/*
 **********************************************************************
 * SP_Tier_Update1():
 * Updates attributes of a tier. 
 * This procedure has no effect on: the default tier flag, tier rank, 
 * or any rule associated with the tier.
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_Tier_Update1' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_Tier_Update1
end if;

CREATE PROCEDURE SAHARA.SP_Tier_Update1 
(
	@pTierKey  numeric(10),
	@pDisplayName varchar(32),
	@pDescription long varchar,
	@pCost numeric(12,6)
) 
BEGIN
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_Tier_Update1';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
	
	-- 2413008 BV Update and other configuration activity blocks Tier operations.
	-- TODO: Revisit this behaviour and introduce one more Tier level locking to be modular.	
	-- CALL SP_Config_RequireControl(@func);

    -- update the tier
    update TIER
       set DISPLAY_NAME = @pDisplayName,
           DESCRIPTION = @pDescription,
           COST = @pCost
     where TIER_KEY = @pTierKey;

    -- see if we really updated anything
	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ETierNotFound,
			@FUNC,
			'failed finding tier ' || @pTierKey || ' to update',
			sahara.Fn_Config_GetActiveSessionKey()
		);
	end if;
end;

grant execute on sahara.SP_Tier_Update1 to sahara_app;

/*
 **********************************************************************
 * SP_Tier_UpdateDefault():
 * Sets a tier as the default for its tier context. There can be only one 
 * default tier for a tier context; any previous default is cleared. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_Tier_UpdateDefault' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_Tier_UpdateDefault
end if;

CREATE PROCEDURE SAHARA.SP_Tier_UpdateDefault 
(
	@pTierKey  numeric(10)
) 
BEGIN
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);
	DECLARE @vContext varchar(32);
    DECLARE @vIsDefault char(1);

    DECLARE @tierCursor CURSOR FOR
        select TIER_CONTEXT, IS_DEFAULT
          from TIER
         where TIER_KEY = @pTierKey
           for update;

	-- we want all or nothing from this call
    savepoint TierAPIUpdateDefaultSP;

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_Tier_UpdateDefault';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
	
	-- 2413008 BV Update and other configuration activity blocks Tier operations.
	-- TODO: Revisit this behaviour and introduce one more Tier level locking to be modular.	
	-- CALL SP_Config_RequireControl(@func);

    -- validate the tier and get what we need to update thingds
	open @tierCursor;
    fetch next @tierCursor into @vContext, @vIsDefault;

    -- make sure we found something
	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
	    close @tierCursor;

		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ETierNotFound,
			@func,
            'failed finding tier ' || @pTierKey || ' to update as the default',
			sahara.Fn_Config_GetActiveSessionKey()
		);
	end if;

    -- see if it is already the default
    if 'T' = @vIsDefault then
        close @tierCursor;

        return;
    end if;

    -- update the default flags
    update TIER
       set IS_DEFAULT = 'T'
     where current of @tierCursor;

    close @tierCursor;

    update TIER
       set IS_DEFAULT = 'F'
     where TIER_CONTEXT = @vContext
       and TIER_KEY <> @pTierKey;

exception
    when others then
			begin
		        rollback to savepoint TierAPIUpdateDefaultSP;
				close @tierCursor;
			exception
			    when others then
				begin
				end;
			end;

			CALL SP_Error_RaiseApplicationErrorDetail
			(
				@ETierError,
				@FUNC,
				'Error while updating default tier: ' || @pTierKey,
				sahara.Fn_Config_GetActiveSessionKey()
			); 
end;
grant execute on sahara.SP_Tier_UpdateDefault to sahara_app;

/*
 **********************************************************************
 * SP_Tier_UpdateRank():
 * Sets the rank for a tier. The rank for all remaining tiers of the 
 * tier context are adjusted accordingly. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_Tier_UpdateRank' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_Tier_UpdateRank
end if;

CREATE PROCEDURE SAHARA.SP_Tier_UpdateRank 
(
	@pTierKey  numeric(10),
	@pRank numeric(2)
) 
BEGIN
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);
	DECLARE @vTierContext varchar(32);
	DECLARE @vOldRank numeric(2);
	DECLARE @vMaxRank numeric(2);

    DECLARE @tierCursor CURSOR FOR
        select TIER_CONTEXT, RANK
          from TIER
         where TIER_KEY = @pTierKey
           for update;

    -- we want all or nothing from this call
    savepoint TierAPIUpdateRankSP;

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_Tier_UpdateRank';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
	
	-- 2413008 BV Update and other configuration activity blocks Tier operations.
	-- TODO: Revisit this behaviour and introduce one more Tier level locking to be modular.	
	-- CALL SP_Config_RequireControl(@func);

    if @pRank < 1 then
		CALL SP_Error_RaiseApplicationErrorDetail
        (
            @EInvalidArgument,
            @func,
            'rank values must be greater than one (1) - value: ' || @pRank,
            sahara.Fn_Config_GetActiveSessionKey()
        );
    end if;

    -- use a cursor to validate the existence of the tier and get
    -- the rank to adjust other tiers in the context
	open @tierCursor;
    fetch next @tierCursor into @vTierContext, @vOldRank;

    -- make sure we found something
	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
	    close @tierCursor;

		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ETierNotFound,
			@func,
            'failed finding tier ' || @pTierKey || ' to update its rank',
			sahara.Fn_Config_GetActiveSessionKey()
		);
	end if;
  
    -- if the rank is not changing, there's nothing to do
    if @vOldRank = @pRank then
        close @tierCursor;

        return;
    end if;

    -- get the maximum rank so we can validate the arguments and shuffle
    -- ranks without violating constraints
    select max(RANK)
      into @vMaxRank
      from TIER
     where TIER_CONTEXT = @vTierContext;

    if @pRank > @vMaxRank then
        close tierCursor;

		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@EInvalidArgument,
			@func,
			'rank values must be less than or equal to the ' ||
            'number of tiers - value: ' || @pRank,
			sahara.Fn_Config_GetActiveSessionKey()
		);
    end if;

    -- get the rank value for the tier we're going to update out of the way
    update TIER
       set RANK = @vMaxRank + 1
     where current of @tierCursor;

    -- shift the ranks for all of the tiers between the old and new ranks
    if @vOldRank < @pRank then
        update TIER
           set RANK = RANK - 1
         where TIER_CONTEXT = @vTierContext
           and RANK between @vOldRank + 1 and @pRank;
    else
        update TIER
           set RANK = RANK + 1
         where TIER_CONTEXT = @vTierContext
           and RANK between @pRank and @vOldRank - 1;
    end if;

    -- finally, put the tier at the desired rank
    update TIER
       set RANK = @pRank
     where current of @tierCursor;

    close @tierCursor;

exception
    when others then
			begin
		        rollback to savepoint TierAPIUpdateRankSP;
				close @tierCursor;
			exception
			    when others then
				begin
				end;
			end;

			CALL SP_Error_RaiseApplicationErrorDetail
			(
				@ETierError,
				@FUNC,
				'Error while updating rank for tier: ' || @pTierKey,
				sahara.Fn_Config_GetActiveSessionKey()
			); 
end;
grant execute on sahara.SP_Tier_UpdateRank to sahara_app;

/*
 **********************************************************************
 * SP_Tier_UpdateRule():
 * Creates, updates, or clears a rule for a tier. 
 * The rule is validated before it is updated.
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_Tier_UpdateRule' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_Tier_UpdateRule
end if;

CREATE PROCEDURE SAHARA.SP_Tier_UpdateRule 
(
	@pTierKey  numeric(10),
    @pRuleText long varchar,
    @pGUIHint long varchar default null
) 
BEGIN
    DECLARE @func VARCHAR(64);
	DECLARE @sqlstate varchar(5);
	DECLARE @vContext varchar(32);
    DECLARE @vRuleContext varchar(32);
    DECLARE @vRuleKey numeric(10);

    DECLARE @tierCursor CURSOR FOR
        select TIER_CONTEXT, TIER_CONTEXT || ' TIER', RULE_KEY
          from TIER
         where TIER_KEY = @pTierKey
           for update;

    -- we want all or nothing from this call
    savepoint TierAPIUpdateRuleSP;

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_Tier_UpdateRule';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
	
	-- 2413008 BV Update and other configuration activity blocks Tier operations.
	-- TODO: Revisit this behaviour and introduce one more Tier level locking to be modular.	
	-- CALL SP_Config_RequireControl(@func);

    -- get any existing rule key and validate the tier
	open @tierCursor;
    fetch next @tierCursor into @vContext, @vRuleContext, @vRuleKey;

    -- make sure we found something
	SELECT SQLSTATE INTO @sqlstate;
	if @sqlstate=02000 then
	    close @tierCursor;

		CALL SP_Error_RaiseApplicationErrorDetail 
		(
			@ETierNotFound,
			@func,
            'failed finding tier ' || @pTierKey || ' to update rule',
			sahara.Fn_Config_GetActiveSessionKey()
		);
	end if;

    -- figure out which action we're performing on the rule
    if @pRuleText is null then
        -- we're deleting the rule
        if @vRuleKey is not null then
            update TIER
               set RULE_KEY = null
             where current of @tierCursor;

            -- a trigger will delete the rule
            set @vRuleKey = null;
        end if;
    else
        -- TODO: validate the rule
        --CALL SP_TIERRULE_Validate(@vContext, @pRuleText);

        if @vRuleKey is null then
            -- we're creating a new rule
            set @vRuleKey = fn_TierRule_Create1(@vRuleContext, @pRuleText, @pGUIHint);

            -- set the new rule
            update TIER
               set RULE_KEY = @vRuleKey
             where current of @tierCursor;
        else
            -- we're updating a rule
            CALL SP_TierRule_Update1(@vRuleKey, @pRuleText, @pGUIHint);
        end if;
    end if;

    -- success
    close @tierCursor;

exception
    when others then
			begin
		        rollback to savepoint TierAPIUpdateRuleSP;
				close @tierCursor;
			exception
			    when others then
				begin
				end;
			end;

			CALL SP_Error_RaiseApplicationErrorDetail
			(
				@ETierError,
				@FUNC,
				'Error while updating rule for tier: ' || @pTierKey,
				sahara.Fn_Config_GetActiveSessionKey()
			); 
end;
grant execute on sahara.SP_Tier_UpdateRule to sahara_app;

/*
 **********************************************************************
 * SP_Tier_ValidateRule():
 * Validates a tier rule. If the validation succeeds, the procedure returns quietly. 
 * On failure, an exception is raised with error code @ETierRuleValidateFailed.

 * This procedure is a proxy to procedure SP_TIERRULE_Validate. TIERRULE module is private to the implementation. 
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_Tier_ValidateRule' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_Tier_ValidateRule
end if;

CREATE PROCEDURE SAHARA.SP_Tier_ValidateRule 
(
	@pContext varchar(32),
    @pRuleText long varchar
) 
BEGIN
    DECLARE @func VARCHAR(64);

	-- initialize global variables
	call sahara.SP_saharaAPIInit();

	SET @func = 'sahara.SP_Tier_ValidateRule';
	CALL SP_Log_LogDebug(sahara.Fn_Config_GetActiveSessionKey(), @func, 'entry');
	
	-- 2413008 BV Update and other configuration activity blocks Tier operations.
	-- TODO: Revisit this behaviour and introduce one more Tier level locking to be modular.	
	-- CALL SP_Config_RequireControl(@func);

    CALL SP_TierRule_Validate(@pContext, @pRuleText);
end;
grant execute on sahara.SP_Tier_ValidateRule to sahara_app;

/*
 **********************************************************************
 * T_TIER_BIUFER
 * 
 * TRIGGER to set UPPER_DISPLAY_NAME per Tier.
 **********************************************************************
 */
if exists(select 1 from sys.systrigger where trigger_name = 'T_TIER_BIUFER') then
   drop TRIGGER T_TIER_BIUFER
end if;

create or replace trigger SAHARA.T_TIER_BIUFER
    before insert,update
    on SAHARA.TIER
	REFERENCING NEW AS newrow
	            OLD AS oldrow
    FOR EACH ROW
BEGIN
	set newrow.UPPER_DISPLAY_NAME = upper(newrow.DISPLAY_NAME);
END;

/*
 **********************************************************************
 * T_TIER_AUDFER
 * 
 * AFTER DELETE, UPDATE TRIGGER which internally deletes corresponding 
 * tier rule from RULE table
 **********************************************************************
 */
if exists(select 1 from sys.systrigger where trigger_name = 'T_TIER_AUDFER') then
   drop TRIGGER T_TIER_AUDFER
end if;

create or replace trigger SAHARA.T_TIER_AUDFER
    after delete, update -- of RULE_KEY
	on SAHARA.TIER
	REFERENCING NEW AS newrow
	            OLD AS oldrow
    FOR EACH ROW
BEGIN
	if oldrow.RULE_KEY is null then
        return;
    end if;

	if update(RULE_KEY) then
        if oldrow.RULE_KEY <> coalesce(newrow.RULE_KEY, 0) then
            delete from RULE
			  where RULE_KEY = oldrow.RULE_KEY;
        end if;
    elseif deleting then
        -- must be deleting
         delete from RULE
		   where RULE_KEY = oldrow.RULE_KEY;
    end if;
end;
message 'Creating/replacing USER module stored procedures...' to client;

--
-- Find or create a user record and return its key.
--

if exists(select 1 from sys.sysprocedure where proc_name = 'fn_User_FindOrCreate' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.fn_User_FindOrCreate
end if;

CREATE FUNCTION "SAHARA"."fn_User_FindOrCreate" (@pSessionKey numeric(10),
                      @pLoginName varchar(80),
                      @pDomain varchar(80)) 
returns numeric(10)
BEGIN
	DECLARE @vUsersKey numeric(10);
	DECLARE DUP_VAL_ON_INDEX EXCEPTION FOR SQLSTATE '23505';

  -- try an insert first to make the row exists
    begin
         insert into USERS(LOGIN_NAME, DOMAIN)
         values(@pLoginName, @pDomain);
	EXCEPTION
         -- if the row already exists, ignore the error
		WHEN DUP_VAL_ON_INDEX THEN
			message 'no worry';
    end;

	-- get the key value and return it
    select USERS_KEY
      into @vUsersKey
      from sahara.USERS
     where LOGIN_NAME = @pLoginName
       and DOMAIN = @pDomain;

    return @vUsersKey;

END;

grant execute on sahara.fn_User_FindOrCreate to sahara;


commit;
-----------------------------------------------------------------------
-- $Id: voma_upgrade.sql,v 1.119.4.1 2012/09/26 11:25:51 ajain8 Exp $
-- $Revision: 1.119.4.1 $
-- $Copyright: Copyright (c) 2012 Symantec Corporation.
-- All rights reserved.
--
-- THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
-- SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
-- WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
--
-- The Licensed Software and Documentation are deemed to be commercial
-- computer software as defined in FAR 12.212 and subject to restricted
-- rights as defined in FAR Section 52.227-19 "Commercial Computer
-- Software - Restricted Rights" and DFARS 227.7202, "Rights in
-- Commercial Computer Software or Commercial Computer Software
-- Documentation", as applicable, and any successor regulations. Any use,
-- modification, reproduction release, performance, display or disclosure
-- of the Licensed Software and Documentation by the U.S. Government
-- shall be solely in accordance with the terms of this Agreement.  $
--
-- User configuration Mgmt APIS
-- Used by GUI to store/fetch per-user configuration information.
-----------------------------------------------------------------------

message 'Creating/replacing USER CONFIGURATION stored procedures...' to client;


/*
 **********************************************************************
 * Fn_UserConfig_GetUsersKey():
 * Looks up the user-key for a given session key
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_UserConfig_GetUsersKey' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_UserConfig_GetUsersKey;
END IF;

create function sahara.Fn_UserConfig_GetUsersKey(
     IN @SessionKey NUMERIC(10))
  returns NUMERIC(10)
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @usersKey NUMERIC(10);

    -- initialize global variables
    call sahara.SP_saharaAPIInit();
 
    set @func = 'sahara.Fn_UserConfig_GetUsersKey';

    select USERS_KEY
      into @usersKey
      from "SESSION"
     where SESSION_KEY = @SessionKey;

    if @usersKey is null then
        call SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @ENoUserForSession,
            @func,
            'the specified session (' || @SessionKey || ') does not have an ' ||
            'associated user',
            @SessionKey
        );
    end if;

    return @usersKey;
END;


/*
 **********************************************************************
 * SP_UserConfig_Set():
 * Sets <Name,Value> for a user that belongs to given session
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_UserConfig_Set' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_UserConfig_Set;
END IF;

create procedure sahara.SP_UserConfig_Set(
     IN @SessionKey NUMERIC(10),
     IN @aName VARCHAR(128),
     IN @aValue VARCHAR(2000))
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @usersKey NUMERIC(10);

    set @func = 'sahara.SP_UserConfig_Set';

    -- initialize global variables
    call sahara.SP_saharaAPIInit();

    call SP_Log_LogDebug(@SessionKey, @func, 'entry');

    -- Get UsersKey from session table
    set @usersKey = Fn_UserConfig_GetUsersKey(@SessionKey);

    merge into USER_CONFIGURATION D
    using (select @usersKey as USERS_KEY,
                  upper(@aName) as NAME,
                  @aValue as VALUE
             from dummy) S
       on (D.USERS_KEY = S.USERS_KEY and
           D.NAME = S.NAME)
    when matched then
        update set D.VALUE = S.VALUE
    when not matched then
        insert (D.USERS_KEY, D.NAME, D.VALUE)
        values (S.USERS_KEY, S.NAME, S.VALUE);
END;

grant execute on sahara.SP_UserConfig_Set to sahara_app;

/*
 **********************************************************************
 * SP_UserConfig_Delete1():
 * Deletes a <Name,value> row from USER_CONFIGURATION table
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_UserConfig_Delete1' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_UserConfig_Delete1;
END IF;

create procedure sahara.SP_UserConfig_Delete1(
     IN @SessionKey NUMERIC(10),
     IN @aName VARCHAR(128))
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @usersKey NUMERIC(10);

    set @func = 'sahara.SP_UserConfig_Delete1';

    -- initialize global variables
    call sahara.SP_saharaAPIInit();

    call SP_Log_LogDebug(@SessionKey, @func, 'entry');

    -- Get UsersKey from session table
    set @usersKey = Fn_UserConfig_GetUsersKey(@SessionKey);

    delete from USER_CONFIGURATION
     where USERS_KEY = @usersKey
       and NAME = upper(@aName);

    -- if we didn't find it, exit quietly - it is gone which is
    -- what we wanted
END;

grant execute on sahara.SP_UserConfig_Delete1 to sahara_app;

/*
 **********************************************************************
 * Fn_UserConfig_Get():
 * Looks up the value f a given attr name for a user session
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_UserConfig_Get' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_UserConfig_Get;
END IF;

create function sahara.Fn_UserConfig_Get(
     IN @SessionKey NUMERIC(10),
     in @aName varchar(128))
  returns varchar(2000)
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @usersKey NUMERIC(10);
    DECLARE @aValue VARCHAR(2000);

    set @func = 'sahara.Fn_UserConfig_Get';

    -- initialize global variables
    call sahara.SP_saharaAPIInit();
 
    call SP_Log_LogDebug(@SessionKey, @func, 'entry');

    -- Get UsersKey from session table
    set @usersKey = Fn_UserConfig_GetUsersKey(@SessionKey);

    select VALUE
      into @aValue
      from USER_CONFIGURATION
     where USERS_KEY = @usersKey
       and NAME = upper(@aName);

    return @aValue;
END;
grant execute on sahara.Fn_UserConfig_Get to sahara_app;



/*
 **********************************************************************
 * Fn_UserConfig_Exists():
 * Checks if a value is already stored fora given attr name.
 **********************************************************************
 */
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_UserConfig_Exists' and user_name(creator) = 'SAHARA')
THEN
        DROP FUNCTION sahara.Fn_UserConfig_Exists;
END IF;

create function sahara.Fn_UserConfig_Exists(
     IN @SessionKey NUMERIC(10),
     in @aName varchar(128))
  returns bit
BEGIN
    DECLARE @func VARCHAR(64);
    DECLARE @usersKey NUMERIC(10);
    DECLARE @aValue VARCHAR(2000);

    set @func = 'sahara.Fn_UserConfig_Exists';

    -- initialize global variables
    call sahara.SP_saharaAPIInit();
 
    call SP_Log_LogDebug(@SessionKey, @func, 'entry');

    -- Get UsersKey from session table
    set @usersKey = Fn_UserConfig_GetUsersKey(@SessionKey);

    if exists( 
        select 1
        from USER_CONFIGURATION
        where USERS_KEY = @usersKey and NAME = upper(@aName)
    )
    then
	return 1;
    else
	return 0;
    end if;

END;
grant execute on sahara.Fn_UserConfig_Exists to sahara_app;


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','InstallWorkerEvent');
create procedure sahara.InstallWorkerEvent ( @vInterval integer )
begin
declare @FUNC varchar(30);
begin 
 set @FUNC = 'InstallWorkerEvent';
       ---- Intialize connection variables 

    call sahara.SP_saharaAPIInit();
 CALL SP_Log_LogDebug(null, @FUNC, 'Entry');
 
-- check if event handler alreday exists , just disbale it as we are going to create event and trigger event

    if IsEventHandlerExists('OccursEvery30Sec') = 1 then
         alter event ev_sahara_worker disable;   
    end if;
--- since there is no event handler for event ev_shara_worker , now create the event and let event do its job  

    call sahara.SubmitEvent(@vInterval);
 end;
end;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SubmitEvent');
create procedure sahara.SubmitEvent
( @vInterval integer )
begin 
  declare @vSql long varchar ;
  declare @IntervalInSec integer;

  begin 
--- lets create event ev_shara_worker for the specified interval

    set @vSql = 'CALL SAHARA.SP_DROP_OBJECT_SAHARA(''EVENT'',''ev_sahara_worker'');  '||
              'CREATE EVENT ev_sahara_worker
               SCHEDULE OccursEvery30Sec 
               between ''12:00 AM'' and ''11:59 PM'' EVERY '||@vInterval ||' seconds 
               HANDLER 
               BEGIN 
                   call Sahara.WorkerMain();
               END';

    execute immediate @vSql;
  
    trigger event ev_sahara_worker;

  end;
end;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','IsEventHandlerExists');
create function sahara.IsEventHandlerExists( @vHandlerName varchar(1024))
returns bit
 begin 
  declare @vcnt integer;
  begin 
--- check whether given schedule is alive in database 
    SELECT 
      count(1) into @vcnt
    FROM SYS.SYSSCHEDULE
    WHERE sched_name = @vHandlerName;
    if @vcnt > 0 then 
      return 1;
    else 
      return 0;
    end if;
  end;    
end;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','IsEventExists');
create function sahara.IsEventExists( @vEventName varchar(1024))
returns bit
 begin 
  declare @vcnt integer;
  begin 
--- check whether given event is there in db 

    SELECT 
      count(1) into @vcnt
    FROM SYS.sysevent
    WHERE event_name = @vEventName;

    if @vcnt > 0 then 
      return 1;
    else 
      return 0;
    end if;
  end;    

end;



CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','WorkerMain');
create procedure Sahara.WorkerMain()
begin 
  declare @FUNC varchar (30);
  declare @vMessage long varchar;
  begin 
    set @FUNC = 'WorkerMain';

     -- initialize global variables
     call sahara.SP_saharaAPIInit();
    
    CALL SP_Log_LogDebug(null, @FUNC, 'Entry');

   
    if IsEventHandlerExists('OccursEvery30Sec') = 1 then 
      call SP_CUST_ATTR_Publish();  
      commit;
    end if; 
    
     if fn_Log_TimeToNextMessageLogTablePurge() <= 0 then
    --if LOG_API.TimeToNextMessageLogTablePurge <= interval '0' second then
        call  SP_Log_PurgeMessageLogTable();
    end if;


        -- since this runs forever, the LOG API has no opportunity to
        -- refresh it's configuration unless we tell it to
      
        call SP_Log_RefreshCachedConfiguration()

  exception
    when others then
        -- save off the error message so we can report it
       set @vMessage = ERRORMSG( SQLCODE );
       CALL SP_Log_LogError(null, @FUNC,'exception caught: ' ||@vMessage);

  end; 
end;--- helper to drop SAHARA_APP table and Procedure

IF EXISTS (select * from sys.sysprocedure sp key join sysusers u
 where sp.proc_name = 'SP_DROP_' and u.name = 'SAHARA_APP' ) THEN
  DROP PROCEDURE SAHARA_APP.SP_DROP_;
END IF;


CREATE PROCEDURE SAHARA_APP.SP_DROP_( IN @pObjectType LONG VARCHAR,
                                      IN @pObjectName LONG VARCHAR)
BEGIN
  DECLARE @ct UNSIGNED INTEGER;

  BEGIN

    SET @ct = 0;
    
    IF @pObjectType = 'TABLE' THEN 

        select count(1) into @ct from systable t key join sysusers u
        where t.table_name = @pObjectName
        and u.name = 'SAHARA_APP';
       
    END IF;
    
    IF @pObjectType = 'PROCEDURE' THEN 
          select count(1) into @ct  from sys.sysprocedure sp key join sysusers u
          where sp.proc_name = @pObjectName and u.name = 'SAHARA_APP'

    END IF;

    IF @ct > 0 THEN
        execute immediate 'drop ' || @pObjectType || ' SAHARA_APP."' || @pObjectName || '"';
    END IF;

  END;
END;


------------------------------ Staging table for business view overview information

CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_OVERVIEW_');
CREATE TABLE SAHARA_APP.IE_OVERVIEW_
(
    BVIEW_NAME varchar(12) NOT NULL,
    DISPLAY_NAME varchar(64) NOT NULL,
    NOTE LONG VARCHAR
);


comment on table SAHARA_APP.IE_OVERVIEW_ is
'Import/Export staging table for view overview information. Contains the
Oracle friendly view name, a user-friendly display name, and any comments.
Views are imported one at a time. Values in this table represent the
view that is currently being imported.';

comment on column SAHARA_APP.IE_OVERVIEW_.BVIEW_NAME is
'Oracle friendly name for this business view, constrained to 12 characters.';

comment on column SAHARA_APP.IE_OVERVIEW_.DISPLAY_NAME is
'User friendly display name for this business view.';

comment on column SAHARA_APP.IE_OVERVIEW_.NOTE is
'Any comments or notes for this business view.';

--
-- staging table for types supported by the business view.
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_TYPES_');
CREATE TABLE SAHARA_APP.IE_TYPES_
(
    TYPE_NAME VARCHAR(26) NOT NULL
);

comment on table SAHARA_APP.IE_TYPES_ is
'Import/Export staging table containing the types supported by this view. ';

comment on column SAHARA_APP.IE_TYPES_.TYPE_NAME is
'Each row in this column represents a type supported by this business view.
Supported types are defined in the SAHARA.BVIEW_DIMENSION table.';

-- support only types defined in the SAHARA.BVIEW_DIMENSION table.
alter table SAHARA_APP.IE_TYPES_
   add constraint FK_IETYPES_DIMESION
   foreign key (TYPE_NAME)
   references SAHARA.BVIEW_DIMENSION(DIMENSION);

--
-- staging table for the business view levels.
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_LEVELS_');
CREATE TABLE SAHARA_APP.IE_LEVELS_
(
    DEPTH NUMERIC NOT NULL,
    LEVEL_NAME VARCHAR(30) NOT NULL,
    LEVEL_DISPLAY_NAME VARCHAR(256) NOT NULL,
    NOTE LONG VARCHAR DEFAULT NULL
);

comment on table SAHARA_APP.IE_LEVELS_ is
'Import/Export staging table containing the business view level hierarchy.
The top of the hierarchy is the first row, the bottom is the last row.
This hierarchy imposes an ordering relationship on the objects in this business view.';

comment on column SAHARA_APP.IE_LEVELS_.DEPTH is
'Depth of this level. The root of the hierarchy is at 0, the first child of root is 1, etc';

comment on column SAHARA_APP.IE_LEVELS_.LEVEL_NAME is
'Oracle friendly name for this level, constrained to 30 characters.';

comment on column SAHARA_APP.IE_LEVELS_.LEVEL_DISPLAY_NAME is
'User friendly display name for this level.';

comment on column SAHARA_APP.IE_LEVELS_.NOTE is
'Any comments or notes for this level.';

--
-- Import status table to record the status of each import.
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_STATUS');
CREATE TABLE SAHARA_APP.IE_STATUS
(
    IMPORT_TIME timestamp default CURRENT UTC TIMESTAMP,
    SESSION_KEY NUMERIC(10) NOT NULL,
    VIEWNAME VARCHAR(256) NOT NULL,
    STATUS_CODE VARCHAR(256) NOT NULL
);

grant SELECT,INSERT,DELETE,update on SAHARA_APP.IE_STATUS to SAHARA;

alter table SAHARA_APP.IE_STATUS
   add constraint FK_IE_STATUS_SESSION foreign key (SESSION_KEY)
      references SAHARA."SESSION" (SESSION_KEY)
      on delete cascade;

comment on table SAHARA_APP.IE_STATUS is
'Import status table to record the status of each import. Every import
attempt is given a session key. Use this key to query for the results
of a specific import/export request.';

comment on column SAHARA_APP.IE_STATUS.IMPORT_TIME is
'Time the import request completed.';

comment on column SAHARA_APP.IE_STATUS.SESSION_KEY is
'The session key that identifies each import request.';

comment on column SAHARA_APP.IE_STATUS.VIEWNAME is
'The basename for all the files in the import set. e.g. Geography.';

comment on column SAHARA_APP.IE_STATUS.STATUS_CODE is
'A free form status code indicating the final status of the import.';

--
-- staging table for host objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_HOST_');

CREATE TABLE SAHARA_APP.IE_HOST_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    HOST_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);
comment on table SAHARA_APP.IE_HOST_ is
'Import/Export staging table for host objects.';

comment on column SAHARA_APP.IE_HOST_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_HOST_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_HOST_.HOST_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_HOST_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_HOST_.IP is
'The IP address for the object';

comment on column SAHARA_APP.IE_HOST_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_HOST_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_HOST_
    add constraint C_IEHOST_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for array objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_ARRAY_');
CREATE TABLE SAHARA_APP.IE_ARRAY_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    ARRAY_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_ARRAY_ is
'Import/Export staging table for array objects.';

comment on column SAHARA_APP.IE_ARRAY_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_ARRAY_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_ARRAY_.ARRAY_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_ARRAY_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_ARRAY_.IP is
'The IP address for the object';

comment on column SAHARA_APP.IE_ARRAY_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_ARRAY_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_ARRAY_
    add constraint C_IEARRAY_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for switch objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_SWITCH_');
CREATE TABLE SAHARA_APP.IE_SWITCH_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    SWITCH_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_SWITCH_ is
'Import/Export staging table for switch objects.';

comment on column SAHARA_APP.IE_SWITCH_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_SWITCH_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_SWITCH_.SWITCH_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_SWITCH_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_SWITCH_.IP is
'The IP address for the object';

comment on column SAHARA_APP.IE_SWITCH_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_SWITCH_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_SWITCH_
    add constraint C_IESWITCH_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for app_group objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_APP_GROUP_');
CREATE TABLE SAHARA_APP.IE_APP_GROUP_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    APP_GROUP_KEY NUMERIC(10),
    NAME VARCHAR(256),
    IP VARCHAR(256) DEFAULT NULL,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_APP_GROUP_ is
'Import/Export staging table for application group objects.';

comment on column SAHARA_APP.IE_APP_GROUP_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_APP_GROUP_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_APP_GROUP_.APP_GROUP_KEY is
'The shara.host object unique key for this object.';

comment on column SAHARA_APP.IE_APP_GROUP_.NAME is
'The hostname for the object';

comment on column SAHARA_APP.IE_APP_GROUP_.IP is
'IP address - will always be null for app_groups. We include this
here to make the correlation procedure more readable.';

comment on column SAHARA_APP.IE_APP_GROUP_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_APP_GROUP_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_APP_GROUP_
    add constraint C_IEAPPGROUP_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp volume objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NETAPP_VOLUME_');
create table SAHARA_APP.IE_NETAPP_VOLUME_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NETAPP_VOLUME_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    PATH VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);

comment on table SAHARA_APP.IE_NETAPP_VOLUME_ is
'Import/Export staging table for NetApp volume objects.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.NETAPP_VOLUME_KEY is
'The shara.netapp_volume object unique key for this object.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.FILER_NAME is
'The name of the owner of this volume';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.PATH is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the volume.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NETAPP_VOLUME_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NETAPP_VOLUME_
    add constraint C_IE_NAVOLUME_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp share types
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NETAPP_SHARE_');
create table SAHARA_APP.IE_NETAPP_SHARE_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NETAPP_SHARE_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    PATH VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_NETAPP_SHARE_ is
'Import/Export staging table for NetApp share objects.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.NETAPP_SHARE_KEY is
'The shara.netapp_share object unique key for this object.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.FILER_NAME is
'The name of the owner of this share.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.PATH is
'The fully qualified path for this share. This combined with the filer
name forms a unique identifier for the share.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NETAPP_SHARE_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NETAPP_SHARE_
    add constraint C_IE_NASHARE_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp qtree objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NETAPP_QTREE_');
create table SAHARA_APP.IE_NETAPP_QTREE_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NETAPP_QTREE_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    PATH VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_NETAPP_QTREE_ is
'Import/Export staging table for NetApp qtree objects.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.NETAPP_QTREE_KEY is
'The shara.netapp_qtree object unique key for this object.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.FILER_NAME is
'The name of the owner of this qtree.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.PATH is
'The fully qualified path for this qtree. This combined with the filer
name forms a unique identifier for the qtree.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NETAPP_QTREE_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NETAPP_QTREE_
    add constraint C_IE_NAQTREE_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for NetApp volume objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NAS_FILESYSTEM_');
create table SAHARA_APP.IE_NAS_FILESYSTEM_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    NAS_FILESYSTEM_KEY NUMERIC(10),
    FILER_NAME VARCHAR(256),
    NAS_FILESYSTEM_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_NAS_FILESYSTEM_ is
'Import/Export staging table for NAS filesystem objects.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.NAS_FILESYSTEM_KEY is
'The shara.netapp_volume object unique key for this object.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.FILER_NAME is
'The name of the owner of this filesystem';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.NAS_FILESYSTEM_NAME is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the filesystem.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_NAS_FILESYSTEM_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_NAS_FILESYSTEM_
    add constraint C_IE_NASFS_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for Fileystem objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_FILESYSTEM_');
create table SAHARA_APP.IE_FILESYSTEM_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    FILESYSTEM_KEY NUMERIC(10),
    HOST_NAME VARCHAR(256),
    FILESYSTEM_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_FILESYSTEM_ is
'Import/Export staging table for filesystem objects.';

comment on column SAHARA_APP.IE_FILESYSTEM_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_FILESYSTEM_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_FILESYSTEM_.FILESYSTEM_KEY is
'The shara.filesystem object unique key for this object.';

comment on column SAHARA_APP.IE_FILESYSTEM_.HOST_NAME is
'The name of the owner of this filesystem';

comment on column SAHARA_APP.IE_FILESYSTEM_.FILESYSTEM_NAME is
'The fully qualified path for this filesystem. This combined with the filer
name forms a unique identifier for the filesystem.';

comment on column SAHARA_APP.IE_FILESYSTEM_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_FILESYSTEM_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_FILESYSTEM_
    add constraint C_IE_FS_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for Volume objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_VOLUME_');
create table SAHARA_APP.IE_VOLUME_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    VOLUME_KEY NUMERIC(10),
    HOST_NAME VARCHAR(256),
    VOLUME_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_VOLUME_ is
'Import/Export staging table for volume objects.';

comment on column SAHARA_APP.IE_VOLUME_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_VOLUME_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_VOLUME_.VOLUME_KEY is
'The shara.volume object unique key for this object.';

comment on column SAHARA_APP.IE_VOLUME_.HOST_NAME is
'The name of the owner of this volume';

comment on column SAHARA_APP.IE_VOLUME_.VOLUME_NAME is
'The fully qualified path for this volume. This combined with the filer
name forms a unique identifier for the volume.';

comment on column SAHARA_APP.IE_VOLUME_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_VOLUME_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_VOLUME_
    add constraint C_IE_VOLUME_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for ServiceGroup objects
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_SERVICEGROUP_');
create table SAHARA_APP.IE_SERVICEGROUP_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    SERVICEGROUP_KEY NUMERIC(10),
    CLUSTER_NAME VARCHAR(256),
    SERVICEGROUP_NAME VARCHAR(256) default null,
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED char default 'T'
);

comment on table SAHARA_APP.IE_SERVICEGROUP_ is
'Import/Export staging table for service group objects.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.SERVICEGROUP_KEY is
'The shara.servicegroup object unique key for this object.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.CLUSTER_NAME is
'The name of the cluster of this service group';

comment on column SAHARA_APP.IE_SERVICEGROUP_.SERVICEGROUP_NAME is
'The name of service group.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.OBJECT_UNIQUE_KEY is
'The CC OID for this object.';

comment on column SAHARA_APP.IE_SERVICEGROUP_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_SERVICEGROUP_
    add constraint C_IE_SG_REJECTED check(REJECTED in ('T', 'F'));

--
-- staging table for cluster objects
--
CALL SAHARA_APP.SP_DROP_('TABLE', 'IE_CLUSTER_');

CREATE TABLE SAHARA_APP.IE_CLUSTER_
(
    NODE_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    CLUSTER_KEY NUMERIC(10),
    NAME VARCHAR(256),
    OBJECT_UNIQUE_KEY VARCHAR(256),
    REJECTED CHAR DEFAULT 'T'
);
comment on table SAHARA_APP.IE_CLUSTER_ is
'Import/Export staging table for cluster objects.';

comment on column SAHARA_APP.IE_CLUSTER_.NODE_ID is
'The business view node identifier as calculated by the import script.';

comment on column SAHARA_APP.IE_CLUSTER_.BVIEW_NODE_KEY is
'The SAHARA.BVIEW_NODE key to which the object is attached.';

comment on column SAHARA_APP.IE_CLUSTER_.CLUSTER_KEY is
'The shara.cluster object unique key for this object.';

comment on column SAHARA_APP.IE_CLUSTER_.NAME is
'The cluster name for the object';

comment on column SAHARA_APP.IE_CLUSTER_.OBJECT_UNIQUE_KEY is
'The CC OID for thie object.';

comment on column SAHARA_APP.IE_CLUSTER_.REJECTED is
'Indicates this row was rejected because the object did not correlate.
to an existing ER object. Only objects known to ER are imported.';

alter table SAHARA_APP.IE_CLUSTER_
    add constraint C_IECLUSTER_REJECTED check(REJECTED in ('T', 'F'));

--
-- Staging table for business view node information.  This holds an entry for
-- each node in a business view tree and a NODE_ID generated by the import
-- script.  It also generates a PARENT_ID for each row.  These IDs are used
-- to populate the BVIEW_NODE_KEY and PARENT_BVIEW_NODE_KEY after the rows
-- are staged and the nodes added to the production table.  These same ids
-- are then used to associate the node with each assigned object.
--
call SAHARA_APP.SP_DROP_('TABLE', 'IE_NODE_');
create table SAHARA_APP.IE_NODE_
(
    DISPLAY_NAME VARCHAR(254) not null,
    DEPTH NUMERIC(3,0) not null,
    NODE_ID NUMERIC(10),
    PARENT_ID NUMERIC(10),
    BVIEW_NODE_KEY NUMERIC(10),
    PARENT_BVIEW_NODE_KEY NUMERIC(10)
);

comment on column SAHARA_APP.IE_NODE_.DISPLAY_NAME is
'The display name for the business view node.';

comment on column SAHARA_APP.IE_NODE_.DEPTH is
'The depth of this node in the hierarchy. The root of the node is at depth 0,
 the next depth is 1, etc.';

comment on column SAHARA_APP.IE_NODE_.NODE_ID is
'Identifier generated by the import script to uniquely identify this node.';

comment on column SAHARA_APP.IE_NODE_.PARENT_ID is
'Identifier of this node parent generated by the import script to uniquely identify
the parent of this node. If the node is at the top of the hierarchy this column
will be null.';

comment on column SAHARA_APP.IE_NODE_.BVIEW_NODE_KEY is
'The key assigned by the SAHARA.BVIEW_API.AddNode procedure. This is the id the node
is identified by in ER.';

comment on column SAHARA_APP.IE_NODE_.PARENT_BVIEW_NODE_KEY is
'The BVIEW_NODE_KEY for the parent node if this node.';





CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_StringToSet');

CREATE PROCEDURE SAHARA_APP.SP_IE_StringToSet
(IN @pString LONG VARCHAR)
RESULT (StringSet VARCHAR(255))
BEGIN
  DECLARE @vStart INTEGER;
  DECLARE @vValue VARCHAR(255);
  DECLARE @incr INTEGER;
  
  BEGIN   
    
    SET @vStart = 1;
    SET @incr = 1;
    
    BEGIN
      
      WHILE (@vStart <> 0 )
      LOOP
      
	SET @pstring = SUBSTRING(@pstring,@vStart);
        SET @vStart = CHARINDEX(',',@pstring);
        IF @vStart <> 0 THEN 
          SET @vValue = SUBSTRING(trim(SUBSTRING(@pstring, 1, @vStart - 1)), 1, 256);
          SET @vStart = @vStart + 1;
        ELSE 
          SET @vValue = trim(@pstring);

        END IF; 
        IF @incr = 1 THEN 
          SELECT @vValue as vValue INTO #vset ;
        ELSE 
	  INSERT INTO #vset(vValue) VALUES (@vValue);
	END IF;   
        SET @incr = 0;                  
      
      END LOOP;
      
      SELECT vValue AS StringSet FROM #vset;

      END;
  
  END;

END;

CALL SAHARA_APP.SP_DROP_('PROCEDURE','FN_IE_StringSetsOverlap');

CREATE FUNCTION SAHARA_APP.FN_IE_StringSetsOverlap
(
  IN @pString1 LONG VARCHAR,
  IN @pString2 LONG VARCHAR
)
  RETURNS BIT
BEGIN
   DECLARE @IsOverLap BIT;

   BEGIN 

    -- convert the strings to sets
    
     SELECT StringSet INTO #vSet1 FROM SAHARA_APP.SP_IE_StringToSet(@pString1);
     SELECT StringSet INTO #vSet2 FROM SAHARA_APP.SP_IE_StringToSet(@pString2);
    
    -- if either set is empty, there's no overlap

     IF ( SELECT ISNULL(COUNT(1),0) FROM #vSet1 ) = 0  THEN
       RETURN 0;
     END IF;

     IF  ( SELECT ISNULL(COUNT(1),0) FROM #vSet2 ) = 0  THEN
        RETURN 0;
     END IF;

    -- iterate the smaller set and test each of its members for inclusion
    -- in the larger set

     SET @IsOverLap = 0; 
   
     SELECT ISNULL(MAX(1),0) INTO @IsOverLap FROM #vSet1 Set1,  #vSet2 Set2
     WHERE Set1.StringSet = Set2.StringSet;
   
     RETURN @IsOverLap;
  
  END;
      
END;

CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateImportedViews');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateImportedViews
(IN @pOverWrite BIT)
BEGIN
  DECLARE @vViewName VARCHAR(12);
  DECLARE @vViewDisplayName VARCHAR(64);
  DECLARE @vViewNote LONG VARCHAR;
  DECLARE @vViewKey INTEGER;
  DECLARE @vLevelKey INTEGER;
  DECLARE VIEW_EXISTS EXCEPTION FOR
        SQLSTATE '42000'; 

  BEGIN

    
    -- extract the view information from the staging tables

    select 
      BVIEW_NAME, DISPLAY_NAME, NOTE
      into @vViewName, @vViewDisplayName, @vViewNote
    from 
      SAHARA_APP.IE_OVERVIEW_;

    -- See if the view already exists
    BEGIN
       -- raise an exception if we're not over-writing

       
      SET @vViewKey = 0;
      
      select 
        ISNULL(BVIEW_KEY,0) into @vViewKey
      from 
        SAHARA.BVIEW_IMPL
      where 
        NAME = @vViewName;

      IF  @vViewKey <> 0 THEN
        --- If exixsts and overwrite is not set then send it back.
            IF @pOverWrite = 0  THEN
                   SIGNAL VIEW_EXISTS;
            END IF;


        CALL SP_BVIEW_Delete1(@vViewKey);

      END IF;
      
      SET @vViewKey = SAHARA.FN_BVIEW_Create1(@vViewDisplayName, @vViewName, @vViewNote);
    
    END;

    -- cursor through the level staging table in depth order and create our levels
    
    FOR LevelRecLOOP AS LevelRecCUR CURSOR FOR 
      SELECT 
        LEVEL_NAME AS LEVEL_NAME_R, 
        LEVEL_DISPLAY_NAME AS LEVEL_DISPLAY_NAME_R, 
	NOTE AS NOTE_R
      FROM 
        SAHARA_APP.IE_LEVELS_
      ORDER BY DEPTH
    DO
       -- insert view level rows.

       SET @vLevelKey = SAHARA.FN_BVIEW_AddLevel(@vViewKey,
                                          LEVEL_DISPLAY_NAME_R,
                                          LEVEL_NAME_R,
                                          NOTE_R);
    END FOR;

    -- add dimension support for each
    
    FOR IETypesLOOP AS IETypesCUR CURSOR FOR 
      select 
        TYPE_NAME AS TYPE_NAME_R
      from 
        SAHARA_APP.IE_TYPES_
    DO
    
	CALL SAHARA.SP_BVIEW_AddDimensionSupport(@vViewKey, TYPE_NAME_R);

    END FOR;

  END;

END;


CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateNodesAtLevel');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateNodesAtLevel(IN @pViewName LONG VARCHAR,
                                                    IN @pLevelDepth INTEGER,
                                                    IN @pSessionKey INTEGER)
BEGIN
  DECLARE @vRootKey INTEGER;
  DECLARE @v_BVIEW_NODE_KEY numeric(10,0);
  DECLARE @v_parent_BVIEW_NODE_KEY numeric(10,0);
  DECLARE @FUNC varchar(64);
  BEGIN

    SET @vRootKey = 0;


   set @FUNC = 'SP_IE_CorrelateNodesAtLevel';
    
    call SAHARA.SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');


   -- get the node key for the root of the view
   
    select 
      ISNULL(BVIEW_KEY,0) into @vRootKey
    from 
      SAHARA.BVIEW
    where 
      NAME = @pViewName;

    IF @vRootKey = 0 THEN

      CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewNotFound,
            'SP_IE_CorrelateNodesAtLevel',
            'the business view not found ' || @pViewName
        );

    END IF; 

  END;
   
  FOR IETypesLOOP AS IETypesCUR CURSOR FOR 
    select 
       DISPLAY_NAME AS DISPLAY_NAME_R,
       NODE_ID AS NODE_ID_R,
       PARENT_ID AS PARENT_ID_R,
       BVIEW_NODE_KEY AS BVIEW_NODE_KEY_R,
       PARENT_BVIEW_NODE_KEY AS PARENT_BVIEW_NODE_KEY_R
    from 
      SAHARA_APP.IE_NODE_
    where 
      DEPTH = @pLevelDepth
    for update of BVIEW_NODE_KEY,PARENT_BVIEW_NODE_KEY

    DO

      IF parent_BVIEW_NODE_KEY_R IS NULL THEN

        SET  @v_parent_BVIEW_NODE_KEY = @vRootKey;
      
      ELSE
      
	SET  @v_parent_BVIEW_NODE_KEY = parent_BVIEW_NODE_KEY_R;
      
      END IF;

      SET @v_BVIEW_NODE_KEY = BVIEW_NODE_KEY_R;

      IF @v_BVIEW_NODE_KEY is NULL THEN
      
        BEGIN

          SET @v_BVIEW_NODE_KEY = SAHARA.FN_BVIEW_AddNode(@v_parent_BVIEW_NODE_KEY,
                                                          DISPLAY_NAME_R,
                                                          NULL);
         EXCEPTION WHEN OTHERS THEN 
          
		IF SQLCODE = -196 THEN 

                   select BVIEW_NODE_KEY into @v_BVIEW_NODE_KEY
                      from SAHARA.BVIEW_NODE
                     where PARENT_BVIEW_NODE_KEY =@v_parent_BVIEW_NODE_KEY
                       and DISPLAY_NAME = DISPLAY_NAME_R;

                    -- TODO : discard or update based on options

               END IF;
       END;
     
      END IF;

   -- update the current node
      
      update 
        SAHARA_APP.IE_NODE_
      set 
        BVIEW_NODE_KEY= @v_BVIEW_NODE_KEY,
        PARENT_BVIEW_NODE_KEY= @v_parent_BVIEW_NODE_KEY
      where 
       DISPLAY_NAME = DISPLAY_NAME_R AND 
       NODE_ID = NODE_ID_R AND 
       isnull(PARENT_ID_R,0)  in (PARENT_ID,0)   AND
       isnull(BVIEW_NODE_KEY_R,0) in (BVIEW_NODE_KEY,0) AND
       isnull(PARENT_BVIEW_NODE_KEY_R,0) in (PARENT_BVIEW_NODE_KEY,0);

      
   -- set the parent node key for all the direct children of this node

      update 
        SAHARA_APP.IE_NODE_
      set 
        PARENT_BVIEW_NODE_KEY= @v_BVIEW_NODE_KEY
      where 
        PARENT_ID= NODE_ID_R;
    
    END FOR;
   
END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjectByObjectName');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjectByObjectName(IN @pType  VARCHAR(255))
BEGIN
  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  BEGIN 

    SET @vSQL_TEMPLATE ='begin FOR StageCursorLOOP AS StageCursor CURSOR FOR select HOST_NAME AS HOST_NAME_R, $TYPE$_NAME AS $TYPE$_NAME_R '||
                        ' from SAHARA_APP.IE_$TYPE$_ where HOST_NAME is not null ' ||
                   'and $TYPE$_NAME is not null and $TYPE$_KEY is null and BVIEW_NODE_KEY is not null for update of $TYPE$_KEY ' ||
                   'DO ' ||
                'update SAHARA_APP.IE_$TYPE$_ ' ||
                    'set $TYPE$_KEY = ' ||
                        '(select FIRST $TYPE$_KEY ' ||
                           'from SAHARA.$TYPE$_CURRENT OT ' ||
                           'join SAHARA.HOST_CURRENT ON HOST_CURRENT.HOST_KEY = OT.HOST_KEY ' ||
                          'where OT.$PATH_COLUMN$ = $PATH_VALUE$ ' ||
                            'and HOST_CURRENT.DISPLAY_NAME = HOST_NAME_R ' ||
                            ') '|| 
	          'WHERE HOST_NAME = HOST_NAME_R '||
		  ' AND $TYPE$_NAME = $TYPE$_NAME_R ; ' ||
            'end for; ' ||
        'end;';
    SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);
   
    IF @pType = 'FILESYSTEM' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'DISPLAY_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'FILESYSTEM_NAME_R');
    END IF;
    
    IF @pType = 'VOLUME' THEN 
      SET @vSQL = REPLACE(@vSQL, '$PATH_COLUMN$', 'VOLUME_NAME');
      SET @vSQL = REPLACE(@vSQL, '$PATH_VALUE$', 'VOLUME_NAME_R');
    END IF;

 EXECUTE IMMEDIATE @vSQL;
  END;
END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjects');
CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjects
(
IN @viewname LONG VARCHAR,
IN @session_key numeric(10)
)
BEGIN
    DECLARE @vRootKey NUMERIC(10);
    DECLARE @vStagingTable VARCHAR(61);
    DECLARE @vType VARCHAR(30);
    DECLARE @vSQL LONG VARCHAR;
   DECLARE @FUNC varchar(64);
    
    BEGIN
    -- get the node key for the root of the view
     SET @vRootKey = 0 ; 

    set @FUNC = 'SP_IE_CorrelateObjects';
    
    call SAHARA.SP_Log_LogDebug(@session_key, @FUNC, 'entry');


     select BVIEW_KEY
      into @vRootKey
      from SAHARA.BVIEW
     where NAME = @viewname;

    if @vRootKey =0  then
      
      CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewNotFound,
            'SP_IE_CorrelateObjects',
            'the business view not found ' || @viewname
        );

    end if;

    -- Iterate over the supported types.  For each, join the node and object
    -- staging tables and set the BVIEW_NODE_KEY column in the object staging
    -- table.  Use this join to call AssociateDimensionRows.  Select all the
    -- objects that don't have BVIEW_NODE_KEY values and mark them
    -- as rejected.




    FOR IETypesLOOP AS IETypesCUR CURSOR FOR 
    select TYPE_NAME from SAHARA_APP.IE_TYPES_
    
    DO
      SET @vType = UPPER(TYPE_NAME);
      SET @vStagingTable = 'SAHARA_APP.IE_' || @vType || '_';

        -- Set the business view node keys for the objects based on
        -- the IE node IDs previously populated by CorrelateNodesAtLevel.

        SET @vSQL =
            'update ' || @vStagingTable || ' STAGE ' ||
               'set BVIEW_NODE_KEY = ' ||
                    '(select BVIEW_NODE_KEY ' ||
                       'from SAHARA_APP.IE_NODE_ ' ||
                      'where STAGE.NODE_ID = IE_NODE_.NODE_ID)';
        execute immediate @vSQL;

        -- Now do the object correlation by filling in the <type>_KEY column
        -- of the object staging table.  If multiple object records match
        -- a criteria, associate the node with first object one we find.

        -- First we'll try based on the object's OID.
        SET @vSQL =
            'update ' || @vStagingTable || ' STAGE ' ||
               'set ' || @vType || '_KEY = ' ||
                    '(select ' || @vType || '_KEY ' ||
                       'from SAHARA.' || @vType || '_CURRENT OBJECT_TYPE ' ||
                      'where OBJECT_TYPE.' || @vType || '_OID = STAGE.OBJECT_UNIQUE_KEY) ' ||
             'where ' || @vType || '_KEY is null ' ||
               'and BVIEW_NODE_KEY is not null';
        execute immediate @vSQL;

        -- try matching the IP address for object types that have them
  
	if @vType in ('HOST', 'ARRAY', 'SWITCH') then
            CALL SAHARA_APP.SP_IE_CorrelateObjectsByIPAddress(@vType);
        end if;

        -- try matching the filer name and fully qualified path
        -- for netapp objects

        if @vType in ('NETAPP_VOLUME', 'NETAPP_SHARE', 'NETAPP_QTREE') then
            CALL SP_IE_CorrelateObjectByNetAppPath(@vType);
        end if;

		-- try matching the host name and object display name
        -- for filesystem and volume objects
        if @vType in ('FILESYSTEM', 'VOLUME') then
            CALL SP_IE_CorrelateObjectByObjectName(@vType);
        end if;

        -- match against the object name

        if @vType in ('APP_GROUP', 'HOST', 'ARRAY', 'SWITCH') then
            SET @vSQL =
                'update ' || @vStagingTable || ' STAGE ' ||
                    'set ' || @vType || '_KEY = ' ||
                        '(select FIRST ' || @vType || '_KEY ' ||
                            'from SAHARA.' || @vType || '_CURRENT OT ' ||
                           'where OT.DISPLAY_NAME = STAGE."NAME" ' ||
                             ') ' ||
                    'where ' || @vType || '_KEY is null ' ||
                      'and BVIEW_NODE_KEY is not null';
            execute immediate @vSQL;
        end if;

        -- Object corellation is complete and recorded in the staging tables.
        -- Create the node to object associations.

        CALL SP_IE_CreateNodeObjectAssociations(@vType);

        -- Anything that correlated is NOT rejected.  Everything else is
        -- already marked rejected.

        SET @vSQL = 'update ' || @vStagingTable || ' ' ||
                   'set REJECTED = ''F'' ' ||
                 'where ' || @vType || '_KEY is not null ' ||
                   'and BVIEW_NODE_KEY is not null';
        execute immediate @vSQL;

	-- Anything that is NOT correlated is rejected.  Everything else is
        -- already marked NOT rejected.

        SET @vSQL = 'update ' || @vStagingTable || ' ' ||
                   'set REJECTED = ''T'' ' ||
                 'where ' || @vType || '_KEY is null ' ||
                   'and BVIEW_NODE_KEY is not null';
   
	execute immediate @vSQL;

        CALL SAHARA.SP_BVIEW_AddDimensionSupport(@vRootKey, @vType);

        
    end for;

    call SAHARA.SP_BVIEW_PublishBV();
  END ;

END;


CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjectsByIPAddress');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjectsByIPAddress(IN @pType  varchar(255))
BEGIN
  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  BEGIN
    
    SET @vSQL_TEMPLATE = 'begin ' ||
            'FOR StageCursorLOOP AS StageCursor CURSOR FOR ' ||
	    'select IP AS IP_R ' ||
                  'from SAHARA_APP.IE_$TYPE$_ ' ||
                 'where IP is not null ' ||
                   'and $TYPE$_KEY is null ' ||
                   'and BVIEW_NODE_KEY is not null ' ||
                'for update OF $TYPE$_KEY ' ||
            'DO ' ||
                'update SAHARA_APP.IE_$TYPE$_ ' ||
                    'set $TYPE$_KEY = ' ||
                        '(select FIRST $TYPE$_KEY ' ||
                           'from SAHARA.$TYPE$_CURRENT ' ||
                          'where 1 = SAHARA_APP.'||'FN_IE_StringSetsOverlap($IP_COLUMN$, IP_R) ' ||
                            'and $IP_COLUMN$ <> ''UNKNOWN'' ' ||
                            ') ' ||
                 'where IP = IP_R; ' ||
            'END FOR; ' ||
        'end; ';

    SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);

    if @pType in ('ARRAY', 'SWITCH') then
        SET @vSQL = replace(@vSQL, '$IP_COLUMN$', 'SNMP_IP_ADDRESS');
    else
        SET @vSQL = replace(@vSQL, '$IP_COLUMN$', 'IP_ADDRESS');
    end if;

    EXECUTE IMMEDIATE @vSQL;

  END;
END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CreateNodeObjectAssociations');

CREATE PROCEDURE SAHARA_APP.SP_IE_CreateNodeObjectAssociations(IN @pType LONG VARCHAR)
BEGIN

  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vKeys LONG VARCHAR;

  BEGIN 
    SET @vSQL_TEMPLATE = 'begin ' ||
                         'FOR StageCursorLOOP AS StageCursor CURSOR FOR ' ||
                         'select distinct BVIEW_NODE_KEY as BVIEW_NODE_KEY_R  ' ||
                         'from SAHARA_APP.IE_$TYPE$_ ' ||
                         'where BVIEW_NODE_KEY is not null ' ||
            'do ' ||
                'delete from sahara.DimensionRowKeyTable; INSERT INTO SAHARA.DimensionRowKeyTable select $TYPE$_KEY ' ||
                  ' from SAHARA_APP.IE_$TYPE$_ ' ||
                 'where BVIEW_NODE_KEY = BVIEW_NODE_KEY_R ' ||
                   'and $TYPE$_KEY is not null; ' ||
               
                  ' CALL SAHARA.SP_BVIEW_AssociateDimensionRows(BVIEW_NODE_KEY_R, ''$TYPE$''); ' ||

            'end for; ' ||
        'end;';
   

    BEGIN
      SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);
      EXECUTE IMMEDIATE @vSQL;
    end;
  END;

END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateObjectByNetAppPath');

CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateObjectByNetAppPath(IN @pType LONG VARCHAR)
BEGIN
  DECLARE @vSQL_TEMPLATE LONG VARCHAR;
  DECLARE @vSQL LONG VARCHAR;
  
  BEGIN
    SET @vSQL_TEMPLATE ='begin ' ||
                        'FOR StageCursorLOOP AS StageCursor CURSOR FOR ' ||
                'select FILER_NAME AS FILER_NAME_R, PATH AS PATH_R ' ||
                  'from SAHARA_APP.IE_$TYPE$_ ' ||
                 'where FILER_NAME is not null ' ||
                   'and PATH is not null ' ||
                   'and $TYPE$_KEY is null ' ||
                   'and BVIEW_NODE_KEY is not null ' ||
                'for update ' || ' DO ' ||
                'update SAHARA_APP.IE_$TYPE$_ ' ||
                    'set $TYPE$_KEY = ' ||
                        '(select FIRST $TYPE$_KEY ' ||
                           'from SAHARA.$TYPE$_CURRENT OT ' ||
                           'join SAHARA.ARRAY_CURRENT ON  ARRAY_CURRENT.ARRAY_KEY =  OT.ARRAY_KEY ' ||
                          'where OT.$PATH_COLUMN$ = $PATH_VALUE$ ' ||
                            'and ARRAY_CURRENT.DISPLAY_NAME = FILER_NAME_R ' ||
                            ') ' ||
                 ' WHERE  FILER_NAME = FILER_NAME_R AND PATH = PATH_R;' ||
            'end FOR; ' ||
        'end;';

    SET @vSQL = replace(@vSQL_TEMPLATE, '$TYPE$', @pType);

    case @pType

        when 'NETAPP_VOLUME' then
          SET @vSQL = replace(@vSQL, '$PATH_COLUMN$', 'DISPLAY_NAME');
            -- Volume paths are prefixed with '/vol/'.  We need to strip these
            -- out when actually comparing against the _CURRENT table.
          SET @vSQL = replace(@vSQL, '$PATH_VALUE$', 'substr(PATH_R, 6)');

        when 'NETAPP_SHARE' then

            SET @vSQL = replace(@vSQL, '$PATH_COLUMN$', 'SHARE_PATH');
            SET @vSQL = replace(@vSQL, '$PATH_VALUE$', 'PATH_R');


        when 'NETAPP_QTREE' then

            SET @vSQL = replace(@vSQL, '$PATH_COLUMN$', 'QTREE_PATH');
            SET @vSQL = replace(@vSQL, '$PATH_VALUE$', 'PATH_R');
    end case;

    EXECUTE IMMEDIATE @vSQL;
END;

END;



CALL SAHARA_APP.SP_DROP_('PROCEDURE','SP_IE_CorrelateNodesAndObjects');
CREATE PROCEDURE SAHARA_APP.SP_IE_CorrelateNodesAndObjects
(
IN @pViewName VARCHAR(12),
IN @pSessionKey NUMERIC(10)
)
BEGIN
  DECLARE @vRootKey NUMERIC(10);
  DECLARE @FUNC varchar(64);
  BEGIN
    SET @vRootKey = 0 ;

    set @FUNC = 'SP_IE_CorrelateNodesAndObjects';
    
    call sahara.SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

      select BVIEW_KEY into @vRootKey
        from SAHARA.BVIEW
       where name = @pViewName;

    IF @vRootKey = 0 THEN 
         CALL SAHARA.SP_Error_RaiseApplicationErrorDetail
        (
            @EBViewNotFound,
            'CorrelateNodesAndObjects',
            'the business view not found ' || @pViewName
        );

    END IF;
   
    
    FOR BViewLevelLOOP AS BViewLevelCUR CURSOR FOR 
    select LEVEL AS LEVEL_R
                 from SAHARA.BVIEW_LEVEL
                where BVIEW_KEY = @vRootKey
                order by "LEVEL"
    DO

      CALL SP_IE_CorrelateNodesAtLevel(@pViewName, LEVEL_R, @pSessionKey);

    END FOR;

    CALL SP_IE_CorrelateObjects (@pViewName, @pSessionKey);
  END;
END;

grant execute on sahara_app.SP_IE_CorrelateNodesAndObjects to sahara_app;

IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_IE_LoadingBVStagingNode' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_IE_LoadingBVStagingNode;
END IF;


CREATE PROCEDURE SAHARA.SP_IE_LoadingBVStagingNode(IN @SQLString Long varchar)
BEGIN
declare @vSQL long varchar;
declare @vMessage long varchar;     
declare @FUNC varchar(64);

set @FUNC = 'SP_IE_LoadingBVStagingNode';

    set @vSQL = 'LOAD INTO TABLE SAHARA_APP.IE_NODE_
(
    "DISPLAY_NAME" ,
    "NODE_ID" ,
    "PARENT_ID" ,
    "DEPTH"
) USING VALUE ' || @SQLString  ||' WITH ROW LOGGING ;';
    
    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vSQL);

    execute immediate  @vSQL;

exception
  when others then
    set @vMessage = ERRORMSG( SQLCODE );
        --message ERRORMSG( SQLCODE ) to client;

    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vMessage);

END;
grant execute on sahara.SP_IE_LoadingBVStagingNode to sahara_app;


IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_IE_LoadingBVStagingTypes' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_IE_LoadingBVStagingTypes;
END IF;


CREATE PROCEDURE SAHARA.SP_IE_LoadingBVStagingTypes(IN @TableName varchar(64),IN @ColumnNames long varchar,IN @SQLString Long varchar)
BEGIN
declare @vSQL long varchar;
declare @vMessage long varchar;     
declare @FUNC varchar(64);

set @FUNC = 'SP_IE_LoadingBVStagingTypes';

    set @vSQL = 'LOAD INTO TABLE '|| @TableName || ' ( ' || @ColumnNames || ' ) USING  VALUE ' || @SQLString  ||' ESCAPES OFF WITH ROW LOGGING ;';
    
    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vSQL);

    execute immediate  @vSQL;

exception
  when others then
    set @vMessage = ERRORMSG( SQLCODE );
        --message ERRORMSG( SQLCODE ) to client;

    call SP_Log_LogInfo(1, @FUNC,
                        'SQL: ' || @vMessage);

END;
grant execute on sahara.SP_IE_LoadingBVStagingTypes to sahara_app;





/*  Lock sps  specific to Import Export module */


CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_IE_AcquireExclusiveLock');

create procedure sahara.SP_IE_AcquireExclusiveLock(
     IN @SessionKey NUMERIC(10),
     IN @Timeout    integer default 0)
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SP_IE_AcquireExclusiveLock';
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Call LockSession() for the CONFIGURATION module. If fails, it will raise
    -- an error, which will throw the exception which will return the
    -- control to the caller.

    CALL SP_Lock_LockSession('SAHARA_IE_CONTROL', @SessionKey);
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully acquired the lock');

    -- If we are here, then the lock was acquired, remember the session etc.
    SET @gIEHaveControl = 1;
    SET @gIEActiveSessionKey = @SessionKey;

END;

grant execute on sahara.SP_IE_AcquireExclusiveLock to sahara_app;

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_IE_ReleaseExclusiveLock');
create procedure sahara.SP_IE_ReleaseExclusiveLock(
     IN @SessionKey NUMERIC(10))
BEGIN
    DECLARE @func VARCHAR(64);

    SET @func = 'SP_IE_ReleaseExclusiveLock';
    CALL SP_Log_LogDebug(@SessionKey, @func, 'Entering:');

    -- Clear the state.

    SET @gIEHaveControl = 0;
    SET @gIEActiveSessionKey = null;

    -- Call UnlockSession() for the CONFIGURATION CONTROL module. 
    -- If this fails, it will raise an error, which will throw the 
    -- exception which will return the control to the caller.

    CALL SP_Lock_UnlockSession('SAHARA_IE_CONTROL', @SessionKey);

    CALL SP_Log_LogDebug(@SessionKey, @func, 'Successfully release the lock');
END;

grant execute on sahara.SP_IE_ReleaseExclusiveLock to sahara_app;

-------------------------------------------------------------------------------
CALL SAHARA.SP_DROP_OBJECT_SAHARA('FUNCTION','Fn_IE_GetActiveSessionKey');
CREATE FUNCTION sahara.Fn_IE_GetActiveSessionKey()
RETURNS NUMERIC(10)
NOT DETERMINISTIC
BEGIN
    RETURN @gIEActiveSessionKey;
END;

----------------------------------------------------------------------------

CALL SAHARA.SP_DROP_OBJECT_SAHARA('PROCEDURE','SP_IE_RequireControl');
CREATE PROCEDURE sahara.SP_IE_RequireControl(
    IN @Func long varchar)
BEGIN
    IF ( @gIEHaveControl = 0) THEN
      BEGIN
        CALL SP_Error_RaiseApplicationError(
            @ENoIEControl,
            @Func || ': ' || 'caller does not have IE control lock'
        );
       END
    END IF;
END;




-- Function used for the export CLI
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'Fn_IE_CONCAT' and user_name(creator) = 'SAHARA_APP')
THEN
        DROP FUNCTION sahara_app.Fn_IE_CONCAT;
END IF;

CREATE FUNCTION sahara_app.Fn_IE_CONCAT(in @str1 long varchar , in @str2 long varchar)
RETURNS long varchar
BEGIN
    declare @string long varchar;
    set @string = (@str1 + @str2);
    RETURN @string;
END;




--
-- Copyright (c) 2007, 2008 Symantec Corporation.
-- All rights reserved.
--
-- THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
-- SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
-- WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
--
-- The Licensed Software and Documentation are deemed to be commercial
-- computer software as defined in FAR 12.212 and subject to restricted
-- rights as defined in FAR Section 52.227-19 "Commercial Computer Software -
-- Restricted Rights" and DFARS 227.7202, Rights in Commercial Computer
-- Software or Commercial Computer Software Documentation, as applicable,
-- and any successor regulations.  Any use, modification, reproduction
-- release, performance, display or disclosure of the Licensed Software and
-- Documentation by the U.S. Government shall be solely in accordance with
-- the terms of this Agreement.
--
-- $Revision: 1.119.4.1 $
--
--
-- NOTE: DO NOT MODIFY THIS FILE.  This file is generated automatically.
--       Any changes will be lost when the file is re-generated.
--

call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ARRAY_STORAGE', 'Array Storage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'TOTAL_PHYSICAL', 'Total Physical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'V_PHYSICAL_EXTERNAL', 'Virtual Physical External');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'LOCAL_LOGICAL', 'Local Logical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'V_VIRTUAL_LOGICAL', 'Virtual Logical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'LOGICAL_AVAILABLE', 'Logical Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'ALLOCATED', 'Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'A_ALLOCATED_TO_HOSTS_DIRECTLY', 'Allocated to Hosts Directly');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'A_ALLOCATED_TO_VIRTUALIZER', 'Allocated to Virtualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'A_ALLOC_HOST_VIA_VIRTUALIZER', 'Allocated to Hosts Through Virtualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'V_LOCAL_LOGICAL_ALLOCATED', 'Local Logical Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'V_VIRTUAL_LOGICAL_ALLOCATED', 'Virtual Logical Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'UNALLOCATED', 'Unallocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'A_UNALLOCATED_VIA_VIRUALIZER', 'Unallocated Through Virualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'V_LOCAL_LOGICAL_UNALLOCATED', 'Local Logical Unallocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'V_VIRTUAL_LOGICAL_UNALLOCATED', 'Virtual Logical Unallocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'CLAIMED_BY_HOSTS', 'Claimed by Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'REPLICATION', 'Replication Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'RAID_OVERHEAD', 'RAID Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'ADMINISTRATIVE', 'Administrative Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'SNAPSHOT_RESERVE', 'Snapshot Reserve');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'WAFL_SPARE', 'WAFL Spare');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'DAS_CAPACITY', 'DAS Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NAS_USED_CAPACITY', 'NAS Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'UNCONFIGURED_CAPACITY', 'Unconfigured Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'INTERNAL_PHYSICAL', 'Internal Physical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'LOGICAL_UNKNOWN', 'Logical Unknown');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'TOTAL_UNCLAIMED', 'Total Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'UNDISCOVERED_ALLOCATED', 'Undiscovered Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'PERCENT_ALLOCATED_HOST_CLAIMED', 'Percent Allocated Host Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'PERCENT_CONFIGURED', 'Percent Configured');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'PERCENT_CONFIGURED_ALLOCATED', 'Percent Configured Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'PERCENT_CONFIGURED_AVAILABLE', 'Percent Configured Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'PERCENT_CONFIGURED_OVERHEAD', 'Percent Configured Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'PERCENT_CONFIGURED_UNDISCOVERD', 'Percent Configured Undiscovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'CONSUMPTION_ASSIGNED', 'CONSUMPTION_ASSIGNED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'CONSUMPTION_UNKNOWN', 'CONSUMPTION_UNKNOWN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'HOST_USAGE_APPS_ON_RAW', 'HOST_USAGE_APPS_ON_RAW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'HOST_USAGE_FS_UN_USED', 'HOST_USAGE_FS_UN_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'HOST_USAGE_FS_USED', 'HOST_USAGE_FS_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'HOST_USAGE_FS_AUSED_APPS', 'HOST_USAGE_FS_AUSED_APPS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'HOST_USAGE_FS_NAUSED_NON_APPS', 'HOST_USAGE_FS_NAUSED_NON_APPS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'HOST_USAGE_OTHER', 'HOST_USAGE_OTHER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'APPL_USED_CAPACITY', 'APPL_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'APPL_UN_USED_CAPACITY', 'APPL_UN_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'PERCENT_UNCONFIGURED', 'Percent Unconfigured');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'THINAWARE_CLAIMED_BY_HOST', 'THINAWARE_CLAIMED_BY_HOST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'THINAWARE_TOTAL_UNCLAIMED', 'THINAWARE_TOTAL_UNCLAIMED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'THINAWARE_ALLOCATED_TOHOST', 'THINAWARE_ALLOCATED_TOHOST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'THINAWARE_ALC_THGVIRTUALIZER', 'THINAWARE_ALC_THGVIRTUALIZER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'THINAWARE_UNDISC_ALLOCATED', 'THINAWARE_UNDISC_ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_VOLUME_CAPACITY', 'NetApp Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_VOLUME_USED_CAPACITY', 'NetApp Volume Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_VOLUME_AVAILABLE_CAPACITY', 'NetApp Volume Available Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_QTREES', 'NetApp QTrees');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_SHARES', 'NetApp Shares');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_VOLUMES', 'NetApp Volumes');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_FILESYSTEMS', 'NAS Filesystems');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_DM_BATCH_KEY', 'NetApp DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_DM_DATA_AGE', 'NetApp DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_CONSUMPTION', 'NetApp Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_FILES', 'NetApp Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_FOLDERS', 'NetApp Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NA_USERS', 'NetApp Users');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'SNAPSHOT_USED', 'Snapshot Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'OVERWRITE_RESERVE', 'Overwrite Reserve');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NETAPP_REDUNDANT_CAPACITY', 'Netapp Redundant Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'NUMBER_OF_STORAGE_POOLS', 'Number of Storage Pools');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_STORAGE', 'WAFL_SPARE_NEW', 'WAFL Spare New');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ARRAY', 'Array');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'PARENT_ARRAY_KEY', 'Parent Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'PARENT_ARRAY_OID', 'Parent Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'VENDOR', 'Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'FAMILY', 'Family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'FIRMWARE_VERSION', 'Firmware Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'SNMP_IP_ADDRESS', 'SNMP IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'NIS_DOMAIN', 'NIS Domain');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'NIS_SERVER', 'NIS Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'IS_AUTO_DISCOVERED', 'Is Auto Discovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'IS_VIRTUALIZER', 'Is Virtualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'IS_NIS_ENABLED', 'Is NAS NIS Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'VOM_DISCOVER_TYPE', 'VOM Discover type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'IP_ADDRESSES', 'IP Addresses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'CLUSTER_ARRAY_KEY', 'Cluster Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'CLUSTER_ARRAY_NAME', 'Cluster Array Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'CLUSTER_ARRAY_OID', 'Cluster Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'FILER_TYPE', 'Filer Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY', 'PARENT_ARRAY_NAME', 'Parent Array Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DATES', 'Dates');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'ADATE', 'Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'YEAR_NUMBER', 'Year Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'MONTH', 'Month');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'QUARTER', 'Quarter');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'WEEK_NUMBER_OF_YEAR', 'Week Number of Year');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'DAY_NUMBER', 'Day Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'DAY_NUMBER_OF_WEEK', 'Day Number of Week');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'DAY_NUMBER_OF_YEAR', 'Day Number of Year');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'DAY_NAME', 'Day Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'DAY_NAME_ABBREVIATION', 'Day Name Abbreviation');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'MONTH_NUMBER', 'Month Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'MONTH_NAME', 'Month Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'MONTH_NAME_ABBREVIATION', 'Month Name Abbreviation');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'QUARTER_NUMBER', 'Quarter Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'IS_WEEKDAY', 'Is Weekday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'IS_HOLIDAY', 'Is Holiday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'IS_MONTH_END', 'Is Month End');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'IS_QUARTER_END', 'Is Quarter End');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATES', 'IS_YEAR_END', 'Is Year End');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_NODE', 'Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'PARENT_BVIEW_NODE_KEY', 'Parent Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'LEVEL', 'Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'NOTE', 'Note');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'LEFT', 'Left');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_NODE', 'RIGHT', 'Right');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ARRAY_BVIEW_NODE', 'Array - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_BVIEW_NODE', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_IMPL', 'Business View Implementation');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_IMPL', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_IMPL', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_IMPL', 'VIEW_CREATION_REQUIRED', 'View Creation Required');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_IMPL', 'APPROVALREQUIRED', 'ApprovalRequired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_IMPL', 'ENABLERULE', 'EnableRule');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_LEVEL', 'Business View Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_LEVEL', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_LEVEL', 'LEVEL', 'Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_LEVEL', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_LEVEL', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_LEVEL', 'NOTE', 'Note');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_STATUS', 'Business View Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_STATUS', 'BVIEW_STATUS_KEY', 'Business View Status Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_STATUS', 'NUMBER_REQUIRED', 'Number Required');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_STATUS', 'VIEW_REFRESH_REQUIRED', 'View Refresh Required');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_BATCH', 'Roll-up Batch');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'ROLLUP_SOURCE_TYPE_KEY', 'Roll-up Source Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'SESSION_KEY', 'Session Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'SCHEDULE_HASH', 'Schedule Hash');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'ROLLUP_SOURCE_KEY', 'Roll-up Source Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'SOURCE_NAME', 'Source Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'WEB_SERVER_HOST', 'Web Server Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'WEB_SERVER_PORT', 'Web Server Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'DB_HOST', 'Database Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'DB_PORT', 'Database Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'DB_PASSWORD', 'Database Password');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'DB_PASSWORD_2', 'Database Password 2');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'LOG_ARCHIVE', 'Log Archive');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'ROLLUP_DAY', 'Roll-up Day');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'START_TIME', 'Start Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'START_TIME_ER', 'Start Time - Enterprise Reporter Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'COMPLETION_TIME', 'Completion Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'COMPLETION_TIME_ER', 'Completion Time - Enterprise Reporter Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'DURATION', 'Duration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'UTC_ADJUSTMENT', 'UTC Adjustment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_ROLLUP_NOW', 'Is Rollup Now');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_COMPLETE', 'Is Complete');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_SUCCESS', 'Is Success');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_LATEST', 'Is Latest');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_LATEST_COMPLETE', 'Is Latest Complete');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_LATEST_SUCCESS', 'Is Latest Success');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_LAST_OF_MONTH', 'Is Last of Month');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_LAST_OF_QUARTER', 'Is Last of Quarter');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_LAST_OF_YEAR', 'Is Last of Year');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'STATUS_MESSAGE', 'Status Message');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'LAST_STATUS_UPDATE', 'Last Status Update');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_LAST_OF_WEEK', 'Is_Last_Of_Week');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'VALID_CLUSTER_DATA', 'Valid_Cluster_Data');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'IS_DUMMY', 'Is_Dummy');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'SHOW_STATUS', 'Show_Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH', 'PERIOD_TYPE', 'PERIOD_TYPE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SOURCE', 'Roll-up Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_SOURCE_KEY', 'Roll-up Source Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_SOURCE_TYPE_KEY', 'Roll-up Source Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'UPPER_NAME', 'Upper Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'WEB_SERVER_HOST', 'Web Server Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'WEB_SERVER_PORT', 'Web Server Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'DB_HOST', 'Database Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'DB_PORT', 'Database Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'DB_PASSWORD', 'Database Password');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'DB_PASSWORD_2', 'Database Password 2');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'UTC_ADJUSTMENT', 'UTC Adjustment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_HOUR', 'Roll-up Hour');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_MINUTE', 'Roll-up Minute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_TIME_AS_INTERVAL', 'Roll-up Time as Interval');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_CLOCK', 'Roll-up Clock');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_TIME_REVISION', 'Roll-up Time Revision');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_NOW', 'Roll-up Now');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ENABLED', 'Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_SOURCE_OID', 'Rollup-up Source Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'ROLLUP_SOURCE_VERSION', 'Rollup-up Source Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE', 'CLUSTER_ROLLUP_MARKER', 'Cluster Rollup Marker');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'HOST', 'Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'OS_DETAILS', 'OS Details');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'OS_TYPE', 'OS Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'IP_ADDRESS', 'IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'VIRTUAL_SERVER', 'Virtual Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'IS_USER_CREATED', 'Is User Created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'IS_VIRTUAL', 'Is Virtual');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'IS_VIRTUAL_SERVER', 'Is Virtual Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'CLUSTER_TYPE', 'Cluster Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'CLUSTER_VERSION', 'Cluster Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'IS_ACTIVE', 'Is Active');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'CLUSTER_NAME', 'Cluster Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'IS_GUEST_INSTALLED', 'Is Guest Installed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'GUEST_OS_TYPE', 'Guest OS Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'VIRTUALIZATION_TECHNOLOGY', 'Virtualization Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'SF_VERSION', 'SF Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'MH_VERSION', 'MH Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'VOM_LICENSE_REQUIRED', 'VOM License Required');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'VM_HOST_OID', 'VM Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'PARENT_VIRTUAL_SERVER_OID', 'Parent Virtual Server OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'CONTROL_GUEST', 'Control Guest');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST', 'CONFIGURATION_TYPE', 'Configuration Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'Filesystem Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'FILESYSTEM_KEY', 'Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'NON_SUPER_USER_CAPACITY', 'Non Super User Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'FREE_CAPACITY', 'Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'USED_BY_APPS', 'Used by Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'USED_BY_NON_APPS', 'Used by non-Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'RESERVED_CAPACITY', 'Reserved Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CAPACITY', 'PERCENT_FREE', 'Percent Free');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'HOST_CAPACITY', 'Host Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED', 'Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED', 'Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'UNCLAIMED', 'Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_UNDISCOVERED', 'Allocated Undiscovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'PERCENT_ALLOCATED_CLAIMED', 'Percent Allocated Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'PERCENT_ALLOCATED_UNCLAIMED', 'Percent Allocated Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'PERCENT_ALLOCATED_UNDISCOVERED', 'Percent Allocated Undiscovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_FCFIBRE', 'Allocated FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_NONFCFIBRE', 'Allocated NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_IDE', 'Allocated IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_DAS', 'Allocated DAS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_ISCSI', 'Allocated ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_FILESTORAGE', 'Allocated Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED_FCFIBRE', 'Claimed FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED_NONFCFIBRE', 'Claimed NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED_IDE', 'Claimed IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED_DAS', 'Claimed DAS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED_ISCSI', 'Claimed ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED_FILESTORAGE', 'Claimed Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_UNDISCOVERED_FCFIBRE', 'Allocated Undiscovered FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_UNDISCOVERED_N_FC_FB', 'Allocated Undiscovered NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_UNDISCOVERED_IDE', 'Allocated Undiscovered IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_UNDISCOVERED_ISCSI', 'Allocated Undiscovered ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_UNDISCOVERED_FILE_SG', 'Allocated Undiscovered Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTMAPD_ALLOCATED', 'Directly Mapped Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTMAPD_ALLOCATED_FCFIBRE', 'Directly Mapped Allocated FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTMAPD_ALLOCATED_NONFCFIBRE', 'Directly Mapped Allocated NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTMAPD_ALLOCATED_IDE', 'Directly Mapped Allocated IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTMAPD_ALLOCATED_ISCSI', 'Directly Mapped Allocated ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTMAPD_ALLOCATED_FILESTORAGE', 'Directly Mapped Allocated Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VIRTUALPOOL_ALLOCATED', 'VirtualPool Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'TOTAL_ALLOCATED', 'Total Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ALLOCATED_CLAIMED_OTHER_HOSTS', 'Allocated Claimed Other Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ASSIGNED_VM', 'Assigned VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTASSIGNED_FS', 'Directly Assigned FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DTASSIGNED_DB', 'Directly Assigned DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'UNKNOWN_ASSIGNMENT', 'Unknown Assignment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'CLAIMED_ASSIGNED_OTHER_HOSTS', 'Claimed Assigned Other Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DISKGROUP_CAPACITY', 'Diskgroup Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DISKGROUP_USED', 'Diskgroup Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DISKGROUP_AVAILABLE', 'Diskgroup Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VOLUME_CAPACITY', 'Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VOLUME_UNUSED', 'Volume Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VMGR_OVERHEAD', 'Volume Manager Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VMGR_ADMINISTRATIVE', 'Volume Manager Administrative');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'FS_CAPACITY', 'FS Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'FS_USED', 'FS Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'FS_AVAILABLE', 'FS Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'FS_ON_VOLUME', 'FS On Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'FS_ON_NON_VOLUME', 'FS On Non Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'FS_USED_APPS', 'FS Used Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'FS_USED_NON_APPS', 'FS Used Non Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_CAPACITY', 'DB Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_USED', 'DB Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_AVAILABLE', 'DB Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_RAW_VOLUME', 'Apps On Raw Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_FS', 'Apps on FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_RAW_LUN', 'Apps On Raw Lun');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'PERCENT_HOST_UTILIZATION', 'Percent Host Utilization');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'THIN_SUBSCRIBED', 'Thin Subscribed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_THIN', 'Thin Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ALLOCATED', 'Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_TOTAL_ALLOCATED', 'Total Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_CLAIMED', 'Claimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_UNCLAIMED', 'Unclaimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ALLOCATED_CLAIMED_OTHER_HOST', 'Allocated Claimed By Other Hosts Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ASSIGNED_VM', 'Assigned VM Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_DTASSIGNED_FS', 'Directly Assigned FS Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_DTASSIGNED_DB', 'Directly Assigned DB Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_UNKNOWN_ASSIGNMENT', 'Unknown Assignment Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_CLAIMED_ASSIGNED_OTHER_HOST', 'Claimed Assigned To Other Hosts Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VIRTUALIZATION_CAPACITY', 'Virtualization Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VIRTUALIZATION_UNUSED', 'Virtualization Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'STORAGE_POOL_USED', 'Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'STORAGE_POOL_UNUSED', 'Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VMDK_UNUSED', 'VMDK Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'VM_EXPORTED', 'VM Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_ALLOCATED', 'Adjusted Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_ALLOCATED_FCFIBRE', 'Adjusted Allocated FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_ALLOCATED_ISCSI', 'Adjusted Allocated ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_ALLOCATED_FILESTORAGE', 'Adjusted Allocated Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_CLAIMED', 'Adjusted Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_CLAIMED_FCFIBRE', 'Adjusted Claimed FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_CLAIMED_ISCSI', 'Adjusted Claimed ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_CLAIMED_FILESTORAGE', 'Adjusted Claimed Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_UNCLAIMED', 'Adjusted Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DTMAPED_ALLOCATED', 'Adjusted Directly Mapped Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VIRTUALPOOL_ALLOCATED', 'Adjusted VirtualPool Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_TOTAL_ALLOCATED', 'Adjusted Total Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_ASSIGNED_VM', 'Adjusted Assigned VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DTASSIGNED_FS', 'Adjusted Directly Assigned FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DTASSIGNED_DB', 'Adjusted Directly Assigned DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_UNKNOWN_ASSIGNMENT', 'Adjusted Unknown Assignment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DISKGROUP_CAPACITY', 'Adjusted Diskgroup Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DISKGROUP_USED', 'Adjusted Diskgroup Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DISKGROUP_AVAILABLE', 'Adjusted Diskgroup Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VOLUME_CAPACITY', 'Adjusted Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VOLUME_UNUSED', 'Adjusted Volume Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VMGR_OVERHEAD', 'Adjusted Volume Manager Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VMGR_ADMINISTRATIVE', 'Adjusted Volume Manager Administrative');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_FS_CAPACITY', 'Adjusted FS Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_FS_USED', 'Adjusted FS Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_FS_AVAILABLE', 'Adjusted FS Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_FS_ON_VOLUME', 'Adjusted FS On Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_FS_ON_NON_VOLUME', 'Adjusted FS On Non Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_FS_USED_APPS', 'Adjusted FS Used Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_FS_USED_NON_APPS', 'Adjusted FS Used Non Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_CAPACITY', 'Adjusted DB Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_USED', 'Adjusted DB Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_AVAILABLE', 'Adjusted DB Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_APPS_ON_RAW_VOLUME', 'Adjusted Apps On Raw Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_APPS_ON_FS', 'Adjusted Apps on FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_THIN_SUBSCRIBED', 'Adjusted Thin Subscribed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_THIN', 'Adjusted Thin Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_ALLOCATED', 'Adjusted Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_TOTAL_ALLOCATED', 'Adjusted Total Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_CLAIMED', 'Adjusted Claimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_UNCLAIMED', 'Adjusted Unclaimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_ASSIGNED_VM', 'Adjusted Assigned VM Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_DTASSIGNED_FS', 'Adjusted Directly Assigned FS Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_DTASSIGNED_DB', 'Adjusted Directly Assigned DB Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_UNKNOWN_ASSIGNMENT', 'Adjusted Unknown Assignment Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VIRTUALIZATION_CAP', 'Adjusted Virtualization Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VIRTUALIZATION_UNUSED', 'Adjusted Virtualization Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_STORAGE_POOL_USED', 'Adjusted Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_STORAGE_POOL_UNUSED', 'Adjusted Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VMDK_UNUSED', 'Adjusted VMDK Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_VM_EXPORTED', 'Adjusted VM Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'EXCHANGE_CAPACITY', 'Exchange Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ASSIGNED_SP', 'Assigned SP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ASSIGNED_SP', 'Consumed Assigned SP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_ASSIGNED_SP', 'Adjusted Assigned SP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_ASSIGNED_SP', 'Adjusted Assigned SP Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'SP_OVERHEAD', 'SP Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_SP_OVERHEAD', 'Adjusted SP Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'NAS_SP', 'NAS Storage Pool');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'NAS_SP_USED', 'NAS Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'NAS_SP_UNUSED', 'NAS Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_NAS_SP', 'Adjusted NAS Storage Pool');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_NAS_SP_USED', 'Adjusted NAS Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_NAS_SP_UNUSED', 'Adjusted NAS Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'EXPORTED_LUNS', 'Exported LUNs');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_EXPORTED_LUNS', 'Exported LUNs Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'EXPORTED_VOLUMES', 'Exported Volumes');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'EXPORTED_FS', 'Exported FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_EXPORTED_LUNS', 'Adjusted Exported LUNs');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'C_ADJUSTED_EXPORTED_LUNS', 'Adjusted Exported LUNs Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_EXPORTED_VOLUMES', 'Adjusted Exported Volumes');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_EXPORTED_FS', 'Adjusted Exported FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'UNSUPPORTED_ALLOCATED', 'Unsupported Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_UNSUPPORTED_ALLOCATED', 'Adjusted Unsupported Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'NAS_ALLOCATED_CAPACITY', 'NAS Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'NAS_CONSUMED_CAPACITY', 'NAS Consumed Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_NAS_ALLOCATE_CAPACITY', 'Adjusted NAS Allocate Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_NAS_CONSUMED_CAPACITY', 'Adjusted NAS Consumed Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_CAPACITY', 'DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_USED_CAPACITY', 'DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_FREE_CAPACITY', 'DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_OVERHEAD_CAPACITY', 'DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_DB_VM_CAPACITY', 'Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'APPS_ON_DB_VM_OTHER_CAPACITY', 'Apps on DB VM Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_DB_USED_CAPACITY', 'DB VM DB Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_DB_USED_OTHER_CAPACITY', 'DB VM DB Used Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'DB_VM_UNKNOWN_CAPACITY', 'DB VM Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_CAPACITY', 'Adjusted DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_USED_CAPACITY', 'Adjusted DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_FREE_CAPACITY', 'Adjusted DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_OVERHEAD_CAP', 'Adjusted DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_APPS_ON_DB_VM_CAP', 'Adjusted Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CAPACITY', 'ADJUSTED_DB_VM_DB_USED_CAP', 'Adjusted DB VM DB Used Capacity');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'FILESYSTEM', 'Filesystem');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'FILESYSTEM_KEY', 'Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'FILESYSTEM_OID', 'Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'VOLUME_KEY', 'Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'MOUNT_POINT', 'Mount Point');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'DEVICE_PATH', 'Device Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_CREATED_FROM_VOLUME', 'Is Created From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_VCS_CVM', 'Is VCS CVM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_CFS_AWARE', 'IS CFS AWARE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_CHILD', 'Is Child');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM', 'IS_BASED_ON_NAS_SHARE', 'Is Based on NAS share');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DATABASE_CAPACITY', 'Database Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'DATABASES_KEY', 'Databases Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'TOTAL_CAPACITY', 'Total Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'TOTAL_AVAILABLE', 'Total Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'TOTAL_USED', 'Total Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'TOTAL_PERCENT_USED', 'Total Percent Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'CAPACITY_FROM_RAW', 'Capacity From Raw');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'AVAILABLE_FROM_RAW', 'Available From Raw');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'USED_FROM_RAW', 'Used From Raw');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'CAPACITY_FROM_FILE_SYSTEM', 'Capacity From File System');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'AVAILABLE_FROM_FILE_SYSTEM', 'Available From File System');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'USED_FROM_FILE_SYSTEM', 'Used From File System');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'CAPACITY_FROM_VOLUME', 'Capacity From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'USED_FROM_VOLUME', 'Used From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'AVAILABLE_FROM_VOLUME', 'Available From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'CAPACITY_FROM_DB_VM', 'Capacity From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'AVAILABLE_FROM_DB_VM', 'Available From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASE_CAPACITY', 'USED_FROM_DB_VM', 'Used From DB VM');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DATABASES', 'Databases');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'DATABASES_KEY', 'Databases Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'DATABASES_OID', 'Database Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'INSTANCE_DISPLAY_NAME', 'Instance Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'DATABASE_TYPE', 'Database Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'DATABASE_VERSION', 'Database Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'DATABASE_STATE', 'Database State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'IS_RAC_AWARE', 'IS RAC AWARE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'Application Group Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'APP_GROUP_KEY', 'Application Group Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'ALLOCATED', 'Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'USED', 'Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'UNUSED', 'Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CAPACITY', 'UNKNOWN', 'Unknown');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'APP_GROUP', 'Application Group');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP', 'APP_GROUP_KEY', 'Application Group Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP', 'APP_GROUP_OID', 'Application Group Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'APP_GROUP_HOST', 'Application Group Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_HOST', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_HOST', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_HOST', 'APP_GROUP_KEY', 'Application Group Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_HOST', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SWITCH', 'Switch');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'SWITCH_KEY', 'Switch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'SWITCH_OID', 'Switch Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'VENDOR', 'Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'FIRMWARE_VERSION', 'Firmware Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'SNMP_IP_ADDRESS', 'SNMP IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'SWITCH_ROLE', 'Switch Role');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'FABRIC_NAME', 'Fabric Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'WWN', 'WWN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'IS_PHYSICAL', 'Is Physical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'HAS_VIRTUAL_SUPPORT', 'Has Virtual Support');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'HAS_MULTIPLEX_SUPPORT', 'Has Multiplex Support');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'HAS_BUNDLE_SUPPORT', 'Has Bundle Support');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'Switch Port Counts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'SWITCH_KEY', 'Switch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'TOTAL_PORTS', 'Total Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'USED_PORTS', 'Used Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'UNUSED_PORTS', 'Unused Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'ISL_PORTS', 'ISL Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'HOST_PORTS', 'Host Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'OTHER_USED_PORTS', 'Other Used Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'ARRAY_PORTS', 'Array Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'PERCENT_USED', 'Percent Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_PORT_COUNTS', 'PERCENT_UNUSED', 'Percent Unused');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SWITCH_BVIEW_NODE', 'Switch - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_BVIEW_NODE', 'SWITCH_KEY', 'Switch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_DIMENSION', 'Business View Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_DIMENSION', 'DIMENSION', 'Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_DIMENSION', 'ALIAS', 'Alias');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_DIMENSION', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'VOLUME_CAPACITY', 'Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CAPACITY', 'VOLUME_KEY', 'Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CAPACITY', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'VOLUME', 'Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'VOLUME_KEY', 'Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'VOLUME_OID', 'Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'DISKGROUP_KEY', 'Diskgroup Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'VOLUME_NAME', 'Volume Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'VM_VERSION', 'VM Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'VM_TYPE', 'VM Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'DISK_GROUP_NAME', 'Disk Group Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'VOLUME_LAYOUT', 'Volume Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_VCS_CVM', 'Is VCS CVM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_USED_BY_FILESYSTEM', 'Is Used by Filesystem');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_USED_BY_APPLICATION', 'Is Used by Application');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_COMPONENT', 'Is Component');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'VOLUME_TYPE', 'Volume Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'IS_USED_BY_DB_VM', 'Is Used By DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME', 'DB_VM_NAME', 'DB VM Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'HOST_BVIEW_NODE', 'Host - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_BVIEW_NODE', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'APP_GROUP_BVIEW_NODE', 'Application Group - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_BVIEW_NODE', 'APP_GROUP_KEY', 'Application Group Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'Host Array Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'ALLOCATED_CAPACITY', 'Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'CLAIMED_CAPACITY', 'Claimed Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_ARRAY_CAPACITY', 'UNCLAIMED_CAPACITY', 'Unclaimed Capacity');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_CAPACITY', 'LUN Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'LUN_CAPACITY', 'LUN Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'ALLOCATED_TO_HOST_DIRECTLY', 'Allocated to Host Directly');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'UNALLOCATED', 'Unallocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'TOTAL_COST', 'Total Cost');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'ALLOCATED_HOSTS', 'Allocated Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'CLAIMED_HOSTS', 'Claimed Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'PHYSICAL_CAPACITY', 'Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'ADJUSTED_ALLOCATED_CAPACITY', 'Adjusted Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'C_ADJUSTED_ALLOCATED_CAPACITY', 'Consumed Adjusted Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'C_PHYSICAL_CAPACITY', 'Consumed Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'C_LUN_CAPACITY', 'Consumed LUN Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CAPACITY', 'C_ALLOCATED_TO_HOST_DIRECTLY', 'Consumed Allocated to Host Directly');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN', 'LUN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'LUN_OID', 'LUN Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'DRIVE_TYPE', 'Drive Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'RAID_LEVEL', 'RAID Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'EMULATION_MODE', 'Emulation Mode');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'MASKING_STATUS', 'Masking Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'CLAIMED_STATUS', 'Claimed Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'NUMBER_OF_COPIES', 'Number of Copies');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'PARITY', 'Parity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'RESOURCE_TYPE', 'Resource Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'REVISION', 'Revision');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'STRIPE_WIDTH', 'Stripe Width');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'VENDOR_DEVICE_ID', 'Vendor Device ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'VENDOR_NAME', 'Vendor Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'VENDOR_STATUS', 'Vendor Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'BINDING_STATE', 'Binding State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'COLUMNS', 'Columns');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'TIER_COST_PER_GB', 'Tier Cost per GB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_PRIMARY', 'Is Primary');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_READ_CACHE_ENABLED', 'Is Read Cache Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_SECONDARY', 'Is Secondary');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_SEGMENT', 'Is Segment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_WRITE_CACHE_ENABLED', 'Is Write Cache Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_AGGREGATE', 'Is Aggregate');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_CONTROL_DEVICE', 'Is Control Device');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_FAULT_TOLERANT', 'Is Fault Tolerant');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_ADMINISTRATIVE', 'Is Administrative');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_USED_IN_SNAPSHOT', 'Is Used In Snapshot');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_VIRTUALIZER_SOURCE', 'Is Virtualizer Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_BASEDON_EXTERNAL', 'Is Basedon External');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'ARRAY_PORTS', 'Array Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'DISK_VENDOR', 'Disk Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'DISK_CAPACITY', 'Disk Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'VENDOR_LAYOUT', 'Vendor Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'THIN_STATUS', 'Thin Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'VOM_DISCOVER_TYPE', 'VOM Discover type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_ASSIGNED', 'Is Assigned');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'STORAGE_TECHNOLOGY', 'Storage Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'STORAGE_POOL_NAME', 'Storage Pool Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'CONNECTION_PROTOCOL', 'Connection Protocol');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN', 'ARRAY_NAMES', 'Array Names');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SESSION', 'Session');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SESSION', 'SESSION_KEY', 'Session Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SESSION', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SESSION', 'USERS_KEY', 'Users Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SESSION', 'START_TIME', 'Start Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SESSION', 'END_TIME', 'End Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SESSION', 'COMMENT', 'Comment');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_DATA_SET', 'Roll-up Data Set');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'ROLLUP_DATA_SET_TYPE_KEY', 'Roll-up Data Set Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'STATUS_MESSAGE', 'Status Message');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'UPLOADING_DURATION', 'Uploading Duration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'STAGING_DURATION', 'Staging Duration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'LOADING_DURATION', 'Loading Duration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'ROWS_PROCESSED', 'Rows Processed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET', 'LAST_STATUS_UPDATE', 'Last Status Update');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'Roll-up Data Set Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'ROLLUP_DATA_SET_TYPE_KEY', 'Roll-up Data Set Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'STEREOTYPE', 'Stereotype');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'DEST_TABLE', 'Destination Table');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'DUPLICATE_FACT_PURGE_KEY_COLS', 'Duplicate Fact Purge Key Columns');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'LOAD_SQL', 'Load SQL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'CSV_NAME', 'CSV Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_TYPE', 'VOM_OBJECT_TYPE', 'VOM Object Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SOURCE_TYPE', 'Roll-up Source Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TYPE', 'ROLLUP_SOURCE_TYPE_KEY', 'Roll-up Source Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TYPE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TYPE', 'LOAD_PROCEDURE', 'Load Procedure');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'RSOURCE_TYPE_DATA_SET_TYPE', 'Roll-up Source Type Data Set Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RSOURCE_TYPE_DATA_SET_TYPE', 'ROLLUP_SOURCE_TYPE_KEY', 'Roll-up Source Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RSOURCE_TYPE_DATA_SET_TYPE', 'ROLLUP_DATA_SET_TYPE_KEY', 'Roll-up Data Set Type Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'ARRAY_', 'Array Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'PARENT_ARRAY_OID', 'Parent Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'VENDOR', 'Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'FAMILY', 'Family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'FIRMWARE_VERSION', 'Firmware Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'SNMP_IP_ADDRESS', 'SNMP IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'NIS_DOMAIN', 'NIS Domain');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'NIS_SERVER', 'NIS Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'IS_AUTO_DISCOVERED', 'Is Auto Discovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'IS_VIRTUALIZER', 'Is Virtualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'IS_NIS_ENABLED', 'Is NAS NIS Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'VOM_DISCOVER_TYPE', 'VOM Discover type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'IP_ADDRESSES', 'IP Addresses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'CLUSTER_ARRAY_NAME', 'Cluster Array Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'CLUSTER_ARRAY_OID', 'Cluster Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'FILER_TYPE', 'Filer Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_', 'PARENT_ARRAY_NAME', 'Parent Array Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_DATA_SET_COLUMN', 'Roll-up Data Set Column');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_COLUMN', 'ROLLUP_DATA_SET_TYPE_KEY', 'Roll-up Data Set Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_COLUMN', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_COLUMN', 'CUSTOM_ATTRIBUTE_KEY', 'Custom Attribute Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_COLUMN', 'LOAD_PROCESSING', 'Load Processing');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_COLUMN', 'SOURCE_EXPRESSION', 'Source Expression');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'Array Storage Stage Array Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'TOTAL_PHYSICAL', 'Total Physical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'V_PHYSICAL_EXTERNAL', 'Virtual Physical External');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'LOCAL_LOGICAL', 'Local Logical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'V_VIRTUAL_LOGICAL', 'Virtual Logical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'LOGICAL_AVAILABLE', 'Logical Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'ALLOCATED', 'Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'A_ALLOCATED_TO_HOSTS_DIRECTLY', 'Allocated to Hosts Directly');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'A_ALLOCATED_TO_VIRTUALIZER', 'Allocated to Virtualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'A_ALLOC_HOST_VIA_VIRTUALIZER', 'Allocated to Hosts Through Virtualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'V_LOCAL_LOGICAL_ALLOCATED', 'Local Logical Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'V_VIRTUAL_LOGICAL_ALLOCATED', 'Virtual Logical Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'UNALLOCATED', 'Unallocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'A_UNALLOCATED_VIA_VIRUALIZER', 'Unallocated Through Virualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'V_LOCAL_LOGICAL_UNALLOCATED', 'Local Logical Unallocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'V_VIRTUAL_LOGICAL_UNALLOCATED', 'Virtual Logical Unallocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'CLAIMED_BY_HOSTS', 'Claimed by Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'REPLICATION', 'Replication Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'RAID_OVERHEAD', 'RAID Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'ADMINISTRATIVE', 'Administrative Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'SNAPSHOT_RESERVE', 'Snapshot Reserve');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'WAFL_SPARE', 'WAFL Spare');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'DAS_CAPACITY', 'DAS Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NAS_USED_CAPACITY', 'NAS Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'UNCONFIGURED_CAPACITY', 'Unconfigured Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'INTERNAL_PHYSICAL', 'Internal Physical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'LOGICAL_UNKNOWN', 'Logical Unknown');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'TOTAL_UNCLAIMED', 'Total Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'UNDISCOVERED_ALLOCATED', 'Undiscovered Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'PERCENT_ALLOCATED_HOST_CLAIMED', 'Percent Allocated Host Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'PERCENT_CONFIGURED', 'Percent Configured');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'PERCENT_CONFIGURED_ALLOCATED', 'Percent Configured Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'PERCENT_CONFIGURED_AVAILABLE', 'Percent Configured Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'PERCENT_CONFIGURED_OVERHEAD', 'Percent Configured Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'PERCENT_CONFIGURED_UNDISCOVERD', 'Percent Configured Undiscovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'PERCENT_UNCONFIGURED', 'Percent Unconfigured');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'CONSUMPTION_ASSIGNED', 'CONSUMPTION_ASSIGNED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'CONSUMPTION_UNKNOWN', 'CONSUMPTION_UNKNOWN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'HOST_USAGE_APPS_ON_RAW', 'HOST_USAGE_APPS_ON_RAW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'HOST_USAGE_FS_UN_USED', 'HOST_USAGE_FS_UN_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'HOST_USAGE_FS_USED', 'HOST_USAGE_FS_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'HOST_USAGE_FS_AUSED_APPS', 'HOST_USAGE_FS_AUSED_APPS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'HOST_USAGE_FS_NAUSED_NON_APPS', 'HOST_USAGE_FS_NAUSED_NON_APPS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'HOST_USAGE_OTHER', 'HOST_USAGE_OTHER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'APPL_USED_CAPACITY', 'APPL_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'APPL_UN_USED_CAPACITY', 'APPL_UN_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'THINAWARE_CLAIMED_BY_HOST', 'THINAWARE_CLAIMED_BY_HOST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'THINAWARE_TOTAL_UNCLAIMED', 'THINAWARE_TOTAL_UNCLAIMED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'THINAWARE_ALLOCATED_TOHOST', 'THINAWARE_ALLOCATED_TOHOST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'THINAWARE_ALC_THGVIRTUALIZER', 'THINAWARE_ALC_THGVIRTUALIZER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'THINAWARE_UNDISC_ALLOCATED', 'THINAWARE_UNDISC_ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NA_VOLUME_CAPACITY', 'NetApp Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NA_VOLUME_USED_CAPACITY', 'NetApp Volume Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NA_VOLUME_AVAILABLE_CAPACITY', 'NetApp Volume Available Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NA_QTREES', 'NetApp QTrees');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NA_SHARES', 'NetApp Shares');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NA_VOLUMES', 'NetApp Volumes');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NA_FILESYSTEMS', 'NAS Filesystems');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'SNAPSHOT_USED', 'Snapshot Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'OVERWRITE_RESERVE', 'Overwrite Reserve');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NETAPP_REDUNDANT_CAPACITY', 'Netapp Redundant Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'NUMBER_OF_STORAGE_POOLS', 'Number of Storage Pools');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_ARRAY_', 'WAFL_SPARE_NEW', 'WAFL Spare New');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'MESSAGE_LOG', 'Message Log');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG', 'MESSAGE_LOG_KEY', 'Message Log Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG', 'TIME', 'Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG', 'SESSION_KEY', 'Session Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG', 'FUNCTION_NAME', 'Function Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG', 'SEVERITY', 'Severity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG', 'MESSAGE', 'Message');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'Database Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'DATABASES_OID', 'Database Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'TOTAL_CAPACITY', 'Total Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'TOTAL_AVAILABLE', 'Total Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'TOTAL_USED', 'Total Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'TOTAL_PERCENT_USED', 'Total Percent Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'CAPACITY_FROM_RAW', 'Capacity From Raw');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'AVAILABLE_FROM_RAW', 'Available From Raw');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'USED_FROM_RAW', 'Used From Raw');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'CAPACITY_FROM_FILE_SYSTEM', 'Capacity From File System');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'AVAILABLE_FROM_FILE_SYSTEM', 'Available From File System');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'USED_FROM_FILE_SYSTEM', 'Used From File System');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'CAPACITY_FROM_VOLUME', 'Capacity From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'USED_FROM_VOLUME', 'Used From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'AVAILABLE_FROM_VOLUME', 'Available From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'CAPACITY_FROM_DB_VM', 'Capacity From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'AVAILABLE_FROM_DB_VM', 'Available From DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASE_CAPACITY_', 'USED_FROM_DB_VM', 'Used From DB VM');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'DATABASES_', 'Databases Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'DATABASES_OID', 'Database Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'INSTANCE_DISPLAY_NAME', 'Instance Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'DATABASE_TYPE', 'Database Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'DATABASE_VERSION', 'Database Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'DATABASE_STATE', 'Database State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'IS_RAC_AWARE', 'IS RAC AWARE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DATABASES_', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'Host_Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED', 'Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED', 'Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'UNCLAIMED', 'Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_UNDISCOVERED', 'Allocated Undiscovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'PERCENT_ALLOCATED_CLAIMED', 'Percent Allocated Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'PERCENT_ALLOCATED_UNCLAIMED', 'Percent Allocated Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'PERCENT_ALLOCATED_UNDISCOVERED', 'Percent Allocated Undiscovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_FCFIBRE', 'Allocated FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_NONFCFIBRE', 'Allocated NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_IDE', 'Allocated IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_ISCSI', 'Allocated ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_FILESTORAGE', 'Allocated Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED_FCFIBRE', 'Claimed FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED_NONFCFIBRE', 'Claimed NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED_ISCSI', 'Claimed ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED_FILESTORAGE', 'Claimed Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTMAPD_ALLOCATED', 'Directly Mapped Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VIRTUALPOOL_ALLOCATED', 'VirtualPool Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_DAS', 'Allocated DAS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'TOTAL_ALLOCATED', 'Total Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_CLAIMED_OTHER_HOSTS', 'Allocated Claimed Other Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ASSIGNED_VM', 'Assigned VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTASSIGNED_FS', 'Directly Assigned FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTASSIGNED_DB', 'Directly Assigned DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'UNKNOWN_ASSIGNMENT', 'Unknown Assignment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED_ASSIGNED_OTHER_HOSTS', 'Claimed Assigned Other Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DISKGROUP_CAPACITY', 'Diskgroup Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DISKGROUP_USED', 'Diskgroup Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DISKGROUP_AVAILABLE', 'Diskgroup Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VOLUME_CAPACITY', 'Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VOLUME_UNUSED', 'Volume Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VMGR_OVERHEAD', 'Volume Manager Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VMGR_ADMINISTRATIVE', 'Volume Manager Administrative');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'FS_CAPACITY', 'FS Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'FS_USED', 'FS Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'FS_AVAILABLE', 'FS Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'FS_ON_VOLUME', 'FS On Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'FS_ON_NON_VOLUME', 'FS On Non Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'FS_USED_APPS', 'FS Used Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'FS_USED_NON_APPS', 'FS Used Non Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_CAPACITY', 'DB Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_USED', 'DB Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_AVAILABLE', 'DB Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_RAW_VOLUME', 'Apps On Raw Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_FS', 'Apps on FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_RAW_LUN', 'Apps On Raw Lun');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'PERCENT_HOST_UTILIZATION', 'Percent Host Utilization');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'THIN_SUBSCRIBED', 'Thin Subscribed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_THIN', 'Thin Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ALLOCATED', 'Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_TOTAL_ALLOCATED', 'Total Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_CLAIMED', 'Claimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_UNCLAIMED', 'Unclaimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ALLOCATED_CLAIMED_OTHER_HOST', 'Allocated Claimed By Other Hosts Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ASSIGNED_VM', 'Assigned VM Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_DTASSIGNED_FS', 'Directly Assigned FS Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_DTASSIGNED_DB', 'Directly Assigned DB Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_UNKNOWN_ASSIGNMENT', 'Unknown Assignment Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_CLAIMED_ASSIGNED_OTHER_HOST', 'Claimed Assigned To Other Hosts Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VIRTUALIZATION_CAPACITY', 'Virtualization Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VIRTUALIZATION_UNUSED', 'Virtualization Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'STORAGE_POOL_USED', 'Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'STORAGE_POOL_UNUSED', 'Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VMDK_UNUSED', 'VMDK Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'VM_EXPORTED', 'VM Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_ALLOCATED', 'Adjusted Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_ALLOCATED_FCFIBRE', 'Adjusted Allocated FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_ALLOCATED_ISCSI', 'Adjusted Allocated ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_ALLOCATED_FILESTORAGE', 'Adjusted Allocated Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_CLAIMED', 'Adjusted Claimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_CLAIMED_FCFIBRE', 'Adjusted Claimed FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_CLAIMED_ISCSI', 'Adjusted Claimed ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_CLAIMED_FILESTORAGE', 'Adjusted Claimed Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_UNCLAIMED', 'Adjusted Unclaimed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DTMAPED_ALLOCATED', 'Adjusted Directly Mapped Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VIRTUALPOOL_ALLOCATED', 'Adjusted VirtualPool Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_TOTAL_ALLOCATED', 'Adjusted Total Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_ASSIGNED_VM', 'Adjusted Assigned VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DTASSIGNED_FS', 'Adjusted Directly Assigned FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DTASSIGNED_DB', 'Adjusted Directly Assigned DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_UNKNOWN_ASSIGNMENT', 'Adjusted Unknown Assignment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DISKGROUP_CAPACITY', 'Adjusted Diskgroup Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DISKGROUP_USED', 'Adjusted Diskgroup Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DISKGROUP_AVAILABLE', 'Adjusted Diskgroup Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VOLUME_CAPACITY', 'Adjusted Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VOLUME_UNUSED', 'Adjusted Volume Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VMGR_OVERHEAD', 'Adjusted Volume Manager Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VMGR_ADMINISTRATIVE', 'Adjusted Volume Manager Administrative');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_FS_CAPACITY', 'Adjusted FS Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_FS_USED', 'Adjusted FS Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_FS_AVAILABLE', 'Adjusted FS Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_FS_ON_VOLUME', 'Adjusted FS On Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_FS_ON_NON_VOLUME', 'Adjusted FS On Non Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_FS_USED_APPS', 'Adjusted FS Used Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_FS_USED_NON_APPS', 'Adjusted FS Used Non Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_CAPACITY', 'Adjusted DB Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_USED', 'Adjusted DB Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_AVAILABLE', 'Adjusted DB Available');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_APPS_ON_RAW_VOLUME', 'Adjusted Apps On Raw Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_APPS_ON_FS', 'Adjusted Apps on FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_THIN_SUBSCRIBED', 'Adjusted Thin Subscribed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_THIN', 'Adjusted Thin Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_ALLOCATED', 'Adjusted Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_TOTAL_ALLOCATED', 'Adjusted Total Allocated Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_CLAIMED', 'Adjusted Claimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_UNCLAIMED', 'Adjusted Unclaimed Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_ASSIGNED_VM', 'Adjusted Assigned VM Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_DTASSIGNED_FS', 'Adjusted Directly Assigned FS Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_DTASSIGNED_DB', 'Adjusted Directly Assigned DB Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_UNKNOWN_ASSIGNMENT', 'Adjusted Unknown Assignment Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VIRTUALIZATION_CAP', 'Adjusted Virtualization Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VIRTUALIZATION_UNUSED', 'Adjusted Virtualization Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_STORAGE_POOL_USED', 'Adjusted Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_STORAGE_POOL_UNUSED', 'Adjusted Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VMDK_UNUSED', 'Adjusted VMDK Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_VM_EXPORTED', 'Adjusted VM Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'EXCHANGE_CAPACITY', 'Exchange Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ASSIGNED_SP', 'Assigned SP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ASSIGNED_SP', 'Consumed Assigned SP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_ASSIGNED_SP', 'Adjusted Assigned SP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_ASSIGNED_SP', 'Adjusted Assigned SP Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'SP_OVERHEAD', 'SP Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_SP_OVERHEAD', 'Adjusted SP Overhead');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'NAS_SP', 'NAS Storage Pool');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'NAS_SP_USED', 'NAS Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'NAS_SP_UNUSED', 'NAS Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_NAS_SP', 'Adjusted NAS Storage Pool');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_NAS_SP_USED', 'Adjusted NAS Storage Pool Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_NAS_SP_UNUSED', 'Adjusted NAS Storage Pool Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'EXPORTED_LUNS', 'Exported LUNs');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_EXPORTED_LUNS', 'Exported LUNs Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'EXPORTED_VOLUMES', 'Exported Volumes');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'EXPORTED_FS', 'Exported FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_EXPORTED_LUNS', 'Adjusted Exported LUNs');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'C_ADJUSTED_EXPORTED_LUNS', 'Adjusted Exported LUNs Consumed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_EXPORTED_VOLUMES', 'Adjusted Exported Volumes');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_EXPORTED_FS', 'Adjusted Exported FS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'UNSUPPORTED_ALLOCATED', 'Unsupported Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_UNSUPPORTED_ALLOCATED', 'Adjusted Unsupported Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED_DAS', 'Claimed DAS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'CLAIMED_IDE', 'Claimed IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_UNDISCOVERED_FCFIBRE', 'Allocated Undiscovered FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_UNDISCOVERED_N_FC_FB', 'Allocated Undiscovered NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_UNDISCOVERED_IDE', 'Allocated Undiscovered IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_UNDISCOVERED_ISCSI', 'Allocated Undiscovered ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ALLOCATED_UNDISCOVERED_FILE_SG', 'Allocated Undiscovered Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTMAPD_ALLOCATED_FCFIBRE', 'Directly Mapped Allocated FCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTMAPD_ALLOCATED_NONFCFIBRE', 'Directly Mapped Allocated NonFCFibre');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTMAPD_ALLOCATED_IDE', 'Directly Mapped Allocated IDE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTMAPD_ALLOCATED_ISCSI', 'Directly Mapped Allocated ISCSI');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DTMAPD_ALLOCATED_FILESTORAGE', 'Directly Mapped Allocated Filestorage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'NAS_ALLOCATED_CAPACITY', 'NAS Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'NAS_CONSUMED_CAPACITY', 'NAS Consumed Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_NAS_ALLOCATE_CAPACITY', 'Adjusted NAS Allocate Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_NAS_CONSUMED_CAPACITY', 'Adjusted NAS Consumed Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_CAPACITY', 'DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_USED_CAPACITY', 'DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_FREE_CAPACITY', 'DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_OVERHEAD_CAPACITY', 'DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_DB_VM_CAPACITY', 'Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'APPS_ON_DB_VM_OTHER_CAPACITY', 'Apps on DB VM Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_DB_USED_CAPACITY', 'DB VM DB Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_DB_USED_OTHER_CAPACITY', 'DB VM DB Used Other Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'DB_VM_UNKNOWN_CAPACITY', 'DB VM Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_CAPACITY', 'Adjusted DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_USED_CAPACITY', 'Adjusted DB VM Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_FREE_CAPACITY', 'Adjusted DB VM Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_OVERHEAD_CAP', 'Adjusted DB VM Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_APPS_ON_DB_VM_CAP', 'Adjusted Apps on DB VM Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_CAPACITY_', 'ADJUSTED_DB_VM_DB_USED_CAP', 'Adjusted DB VM: DB Used Capacity');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'HOST_ARRAY_CAPACITY_', 'Host Array Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_ARRAY_CAPACITY_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_ARRAY_CAPACITY_', 'ARRAY_OID', 'Array Unique Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_ARRAY_CAPACITY_', 'ALLOCATED_CAPACITY', 'Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_ARRAY_CAPACITY_', 'CLAIMED_CAPACITY', 'Claimed Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_ARRAY_CAPACITY_', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_ARRAY_CAPACITY_', 'UNCLAIMED_CAPACITY', 'Unclaimed Capacity');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'HOST_', 'Host Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'OS_DETAILS', 'OS Details');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'OS_TYPE', 'OS Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'IP_ADDRESS', 'IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'VIRTUAL_SERVER', 'Virtual Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'IS_USER_CREATED', 'Is User Created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'IS_VIRTUAL', 'Is Virtual');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'IS_VIRTUAL_SERVER', 'Is Virtual Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'CLUSTER_TYPE', 'Cluster Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'CLUSTER_VERSION', 'Cluster Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'IS_ACTIVE', 'Is Active');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'CLUSTER_NAME', 'Cluster Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'IS_GUEST_INSTALLED', 'Is Guest Installed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'GUEST_OS_TYPE', 'Guest OS Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'VIRTUALIZATION_TECHNOLOGY', 'Virtualization Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'MH_VERSION', 'MH Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'SF_VERSION', 'SF Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'VOM_LICENSE_REQUIRED', 'VOM License Required');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'VM_HOST_OID', 'VM Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'PARENT_VIRTUAL_SERVER_OID', 'Parent Virtual Server OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'CONTROL_GUEST', 'Control Guest');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'HOST_', 'CONFIGURATION_TYPE', 'Configuration Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'VOLUME_', 'Volume Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'VOLUME_OID', 'Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'DISKGROUP_OID', 'Diskgroup Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'VOLUME_NAME', 'Volume Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'VM_VERSION', 'VM Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'VM_TYPE', 'VM Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'DISK_GROUP_NAME', 'Disk Group Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'VOLUME_LAYOUT', 'Volume Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_VCS_CVM', 'Is VCS CVM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_USED_BY_FILESYSTEM', 'Is Used by Filesystem');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_USED_BY_APPLICATION', 'Is Used by Application');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_COMPONENT', 'Is Component');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'VOLUME_TYPE', 'Volume Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'IS_USED_BY_DB_VM', 'Is Used by DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_', 'DB_VM_NAME', 'DB VM Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'VOLUME_CAPACITY_', 'Volume Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_CAPACITY_', 'VOLUME_OID', 'Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'VOLUME_CAPACITY_', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'DISKGROUP_', 'Diskgroup Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'DISKGROUP_OID', 'Diskgroup Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'FAMILY', 'Family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'VERSION', 'Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'IS_SHARED', 'Is Shared');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'IS_BASED_ON_DIRECT_ATTACHED', 'Is Based On Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'STATE', 'State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'IS_USED_BY_CLUSTERED_DB', 'Is Used by clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_', 'USES_VM_VOLUMES', 'Uses VM Volumes');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'Filesystem Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'FILESYSTEM_OID', 'Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'NON_SUPER_USER_CAPACITY', 'Non Super User Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'FREE_CAPACITY', 'Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'USED_BY_APPS', 'Used by Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'USED_BY_NON_APPS', 'Used by non-Apps');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_CAPACITY_', 'PERCENT_FREE', 'Percent Free');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'APP_GROUP_CAPACITY_', 'Application Group Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_CAPACITY_', 'APP_GROUP_OID', 'Application Group Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_CAPACITY_', 'ALLOCATED', 'Allocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_CAPACITY_', 'USED', 'Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_CAPACITY_', 'UNUSED', 'Unused');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_CAPACITY_', 'UNKNOWN', 'Unknown');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'APP_GROUP_', 'Application Group Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_', 'APP_GROUP_OID', 'Application Group Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'APP_GROUP_HOST_', 'Application Group Host Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_HOST_', 'APP_GROUP_OID', 'Application Group Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'APP_GROUP_HOST_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SWITCH_', 'Switch Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'SWITCH_OID', 'Switch Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'VENDOR', 'Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'FIRMWARE_VERSION', 'Firmware Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'SNMP_IP_ADDRESS', 'SNMP IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'SWITCH_ROLE', 'Switch Role');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'FABRIC_NAME', 'Fabric Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'WWN', 'WWN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'IS_PHYSICAL', 'Is Physical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'HAS_VIRTUAL_SUPPORT', 'Has Virtual Support');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'HAS_MULTIPLEX_SUPPORT', 'Has Multiplex Support');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_', 'HAS_BUNDLE_SUPPORT', 'Has Bundle Support');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'Switch Port Counts Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'SWITCH_OID', 'Switch Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'TOTAL_PORTS', 'Total Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'USED_PORTS', 'Used Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'UNUSED_PORTS', 'Unused Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'ISL_PORTS', 'ISL Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'HOST_PORTS', 'Host Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'OTHER_USED_PORTS', 'Other Used Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'ARRAY_PORTS', 'Array Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'PERCENT_USED', 'Percent Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SWITCH_PORT_COUNTS_', 'PERCENT_UNUSED', 'Percent Unused');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'Roll-up Schedule');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'ROLLUP_SCHEDULE_KEY', 'Roll-up Schedule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'REVISION', 'Revision');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_SUNDAY', 'By Week Sunday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_MONDAY', 'By Week Monday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_TUESDAY', 'By Week Tuesday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_WEDNESDAY', 'By Week Wednesday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_THURSDAY', 'By Week Thursday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_FRIDAY', 'By Week Friday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_SATURDAY', 'By Week Saturday');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_MULTIPLIER', 'By Week Multiplier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_WEEK_START_DATE', 'By Week Start Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_MONTH_1_WEEK', 'By Month 1 Week');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_MONTH_1_DAY', 'By Month 1 Day');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_MONTH_1_MULTIPLIER', 'By Month 1 Multiplier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_MONTH_1_START_DATE', 'By Month 1 Start Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_MONTH_2_DATE', 'By Month 2 Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_MONTH_2_MULTIPLIER', 'By Month 2 Multiplier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE', 'BY_MONTH_2_START_DATE', 'By Month 2 Start Date');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CONTROL_OWNER', 'Control Owner');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONTROL_OWNER', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONTROL_OWNER', 'SESSION_KEY', 'Session Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONTROL_OWNER', 'ACQUIRED', 'Acquired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONTROL_OWNER', 'RELEASED', 'Released');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONTROL_OWNER', 'ISLOCKED', 'Is Locked');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_DIMENSION_BVIEW', 'Business View Dimension Business View');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_DIMENSION_BVIEW', 'DIMENSION', 'Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_DIMENSION_BVIEW', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DOMAIN_TYPE', 'Domain Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_BOOLEAN', 'Boolean');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_COLUMN_NAME', 'Column Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_CONTROL_NAME', 'Control Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_DEVICE_DISPLAY_NAME', 'Device Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_KEY', 'Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_CCS_OID', 'OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_TABLE_NAME', 'Table Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DOMAIN_TYPE', 'SA_WWN', 'World Wide Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_BVIEW_NODE', 'LUN - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_BVIEW_NODE', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_HOST', 'LUN - HOST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'LUN_CAPACITY', 'LUN Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'ALLOCATED_TO_HOST_DIRECTLY', 'Allocated to Host Directly');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'TOTAL_COST', 'Total Cost');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'ALLOCATED_HOSTS', 'Allocated Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'CLAIMED_HOSTS', 'Claimed Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'PHYSICAL_CAPACITY', 'Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'ADJUSTED_ALLOCATED_CAPACITY', 'Adjusted Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'C_ADJUSTED_ALLOCATED_CAPACITY', 'Consumed Adjusted Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'C_LUN_CAPACITY', 'Consumed LUN Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'C_PHYSICAL_CAPACITY', 'Consumed Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'C_ALLOCATED_TO_HOST_DIRECTLY', 'Consumed Allocated to Host Directly');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_HOST', 'C_TOTAL_COST', 'Consumed Total Cost');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'LUN_', 'LUN Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'LUN_OID', 'LUN Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'DRIVE_TYPE', 'Drive Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'RAID_LEVEL', 'RAID Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'EMULATION_MODE', 'Emulation Mode');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'MASKING_STATUS', 'Masking Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'CLAIMED_STATUS', 'Claimed Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'NUMBER_OF_COPIES', 'Number of Copies');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'PARITY', 'Parity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'RESOURCE_TYPE', 'Resource Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'REVISION', 'Revision');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'STRIPE_WIDTH', 'Stripe Width');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'VENDOR_DEVICE_ID', 'Vendor Device ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'VENDOR_NAME', 'Vendor Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'VENDOR_STATUS', 'Vendor Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'BINDING_STATE', 'Binding State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'COLUMNS', 'Columns');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_PRIMARY', 'Is Primary');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_READ_CACHE_ENABLED', 'Is Read Cache Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_SECONDARY', 'Is Secondary');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_SEGMENT', 'Is Segment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_WRITE_CACHE_ENABLED', 'Is Write Cache Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_AGGREGATE', 'Is Aggregate');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_CONTROL_DEVICE', 'Is Control Device');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_FAULT_TOLERANT', 'Is Fault Tolerant');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_ADMINISTRATIVE', 'Is Administrative');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_USED_IN_SNAPSHOT', 'Is Used In Snapshot');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_VIRTUALIZER_SOURCE', 'Is Virtualizer Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_BASEDON_EXTERNAL', 'Is Basedon External');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'ARRAY_PORTS', 'Array Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'DISK_VENDOR', 'Disk Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'DISK_CAPACITY', 'Disk Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'VENDOR_LAYOUT', 'Vendor Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'THIN_STATUS', 'Thin Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'TIER_COST_PER_GB', 'Tier Cost per GB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'VOM_DISCOVER_TYPE', 'VOM Discover type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_ASSIGNED', 'Is Assigned');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'STORAGE_TECHNOLOGY', 'Storage Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'STORAGE_POOL_NAME', 'Storage Pool Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'CONNECTION_PROTOCOL', 'Connection Protocol');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'NAS_FILESYSTEM_OID', 'NAS FS Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_', 'ARRAY_NAMES', 'Array Names');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'LUN_HOST_', 'LUN - HOST Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_HOST_', 'LUN_OID', 'LUN Object Identifer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_HOST_', 'HOST_OID', 'Host Object Identifer');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'LUN Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'LUN_OID', 'LUN Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'LUN_CAPACITY', 'LUN Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'ALLOCATED_TO_HOST_DIRECTLY', 'Allocated to Host Directly');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'UNALLOCATED', 'Unallocated');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'ALLOCATED_HOSTS', 'Allocated Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'CLAIMED_HOSTS', 'Claimed Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'PHYSICAL_CAPACITY', 'Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'ADJUSTED_ALLOCATED_CAPACITY', 'Adjusted Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'C_ALLOCATED_TO_HOST_DIRECTLY', 'Consumed Allocated to Host Directly');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'C_LUN_CAPACITY', 'Consumed LUN Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'C_ADJUSTED_ALLOCATED_CAPACITY', 'Consumed Adjusted Allocated Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_CAPACITY_', 'C_PHYSICAL_CAPACITY', 'Consumed Physical Capacity');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CONFIGURATION', 'Configuration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'SECTION', 'Section');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'VISIBLE', 'Visible');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'EDITABLE', 'Editable');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'STRING_VALUE', 'String Value');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'INTEGER_VALUE', 'Integer Value');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'BOOLEAN_VALUE', 'Boolean Value');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CONFIGURATION', 'DATE_VALUE', 'Date Value');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATE', 'Roll-up Schedule Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATE', 'SCHEDULE_DATE', 'Schedule Date');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'TIER_CONTEXT', 'Tier Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER_CONTEXT', 'TIER_CONTEXT', 'Tier Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER_CONTEXT', 'COST_UNIT', 'Cost Unit');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER_CONTEXT', 'RULE_SQL', 'Rule SQL');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'TIER_KEY', 'Tier Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'TIER_CONTEXT', 'Tier Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'UPPER_DISPLAY_NAME', 'Upper Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'RANK', 'Rank');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'COST', 'Cost');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'IS_DEFAULT', 'Is Default');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TIER', 'RULE_KEY', 'Rule Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'RULE', 'Rule');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE', 'RULE_KEY', 'Rule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE', 'RULE_CONTEXT', 'Rule Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE', 'GUI_HINT', 'GUI Hint');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE', 'RULE_TEXT', 'Rule Text');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE', 'SQL_CODE', 'SQL Code');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'RULE_CONTEXT', 'Rule Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_CONTEXT', 'RULE_CONTEXT', 'Rule Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_CONTEXT', 'HELP_HTML', 'Help HTML');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'RULE_ELEMENT_SECTION', 'Rule Element Section');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT_SECTION', 'RULE_CONTEXT', 'Rule Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT_SECTION', 'SECTION_DISPLAY_NAME', 'Section Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT_SECTION', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'RULE_ELEMENT', 'Rule Element');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'RULE_CONTEXT', 'Rule Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'SECTION_DISPLAY_NAME', 'Section Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'RULE_TOKEN_UPPER', 'Rule Token Upper');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'RULE_TOKEN', 'Rule Token');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RULE_ELEMENT', 'SQL_CODE', 'SQL Code');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'CUSTOM_ATTRIBUTE_', 'Custom Attribute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CUSTOM_ATTRIBUTE_', 'OID', 'Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CUSTOM_ATTRIBUTE_', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CUSTOM_ATTRIBUTE_', 'VALUE', 'Value');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CUSTOM_ATTRIBUTE_', 'VOM_OBJECT_TYPE', 'VOM Object Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SRC_CUSTOM_ATTRIBUTE_BASE', 'Source Custom Attribute Base');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SRC_CUSTOM_ATTRIBUTE_BASE', 'CUSTOM_ATTRIBUTE_CONTEXT', 'Custom Attribute Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SRC_CUSTOM_ATTRIBUTE_BASE', 'ROLLUP_SOURCE_KEY', 'Roll-up Source Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SRC_CUSTOM_ATTRIBUTE_BASE', 'NAME', 'Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CUSTOM_ATTRIBUTE_CONTEXT', 'Custom Attribute Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE_CONTEXT', 'CUSTOM_ATTRIBUTE_CONTEXT', 'Custom Attribute Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE_CONTEXT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE_CONTEXT', 'UPDATE_INDICATOR', 'Update Indicator');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'Custom Attribute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'CUSTOM_ATTRIBUTE_KEY', 'Custom Attribute Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'CUSTOM_ATTRIBUTE_CONTEXT', 'Custom Attribute Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'COLUMN_NAME', 'Column Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'COLUMN_TYPE', 'Column Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CUSTOM_ATTRIBUTE', 'SOURCE_NAME', 'Source Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ER_LICENSE', 'Enterprise Reporter License');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'LICENSE_KEY', 'License Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'LICENSE_TYPE', 'License Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'VERSION', 'Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'CLIENT_ACCESS_LICENSES', 'Client Access Licenses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'DURATION', 'Duration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'EXPIRATION', 'Expiration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'IS_SITE_KEY', 'Is Site Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'IS_EDITION_KEY', 'Is Edition Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE', 'IS_POINT_PRODUCT_KEY', 'Is Point Product Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'TABLE_METADATA_BASE', 'Table Metadata Base');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA_BASE', 'OWNER', 'Owner');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA_BASE', 'TABLE_NAME', 'Table Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA_BASE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'Column Metadata Base');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'OWNER', 'Owner');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'TABLE_NAME', 'Table Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'COLUMN_NAME', 'Column Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'IS_FORECAST_ENABLE', 'Is Forecast Enable');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA_BASE', 'AGGREGATE_TYPE', 'AGGREGATE_TYPE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COGNOS_PACKAGE', 'Cognos Package');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_PACKAGE', 'PACKAGE_NAME', 'Package Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COGNOS_PRESENTATION_SHORTCUT', 'Cognos Presentation Shortcut');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_PRESENTATION_SHORTCUT', 'OBJECT_TYPE', 'Object Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_PRESENTATION_SHORTCUT', 'NAMESPACE', 'Namespace');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_PRESENTATION_SHORTCUT', 'FOLDER_NAME', 'Folder Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COGNOS_RELATIONSHIP', 'Cognos Relationship');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_RELATIONSHIP', 'LHS_OBJECT_TYPE', 'Left Hand Side Object Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_RELATIONSHIP', 'RHS_TABLE', 'Right Hand Side Table');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_RELATIONSHIP', 'LHS_CARDINALITY_A', 'Left Hand Side Object Cardinality A');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_RELATIONSHIP', 'LHS_CARDINALITY_B', 'Left Hand Side Object Cardinality B');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_RELATIONSHIP', 'RHS_CARDINALITY_A', 'Right Hand Side Object Cardinality A');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_RELATIONSHIP', 'RHS_CARDINALITY_B', 'Right Hand Side Object Cardinality B');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COGNOS_FACT_MAPPING', 'Cognos Fact Mapping');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_FACT_MAPPING', 'OBJECT_TYPE', 'Object Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_FACT_MAPPING', 'NAMESPACE', 'Namespace');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_FACT_MAPPING', 'FACT_DIMENSION_NAME', 'Fact Dimension Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_FACT_MAPPING', 'FACT_NAME', 'Fact Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COGNOS_HIERARCHY_ATTRIBUTE', 'Cognos Hierarchy Attribute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_HIERARCHY_ATTRIBUTE', 'CUSTOM_ATTRIBUTE_CONTEXT', 'Custom Attribute Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_HIERARCHY_ATTRIBUTE', 'NAMESPACE', 'Namespace');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_HIERARCHY_ATTRIBUTE', 'DIMENSION_NAME', 'Dimension Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_HIERARCHY_ATTRIBUTE', 'HIERARCHY_NAME', 'Hierarchy Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_HIERARCHY_ATTRIBUTE', 'LEVEL_NAME', 'Level Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_BATCH_STATUS', 'Roll-up Batch Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_STATUS', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_STATUS', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_STATUS', 'IS_COMPLETE', 'Is Complete');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_STATUS', 'IS_SUCCESS', 'Is Success');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'Published Custom Attribute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'CUSTOM_ATTRIBUTE_KEY', 'Custom Attribute Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'CUSTOM_ATTRIBUTE_CONTEXT', 'Custom Attribute Context');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'COLUMN_NAME', 'Column Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'COLUMN_TYPE', 'Column Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'P_CUSTOM_ATTRIBUTE', 'SOURCE_NAME', 'Source Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'T2_INSERT', 'Type 2 Support - Insert');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'T2_INSERT', 'INSERTED_KEY', 'Inserted Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'T2_INSERT', 'INSERTED_OID', 'Inserted OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'T2_INSERT', 'ANCESTOR_ROW', 'Ancestor Row');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'T2_INSERT', 'ANCESTOR_KEY', 'Ancestor Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'OBJECT_DIM_SOURCE', 'Object Dimension Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'OBJECT_DIM_SOURCE', 'OID', 'OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'OBJECT_DIM_SOURCE', 'TABLE_NAME', 'Table Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'OBJECT_DIM_SOURCE', 'ROLLUP_SOURCE_KEY', 'Roll-up Source Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'T2_OBJECT_DIM_SOURCE_DELETE', 'Type 2 Support - Object Dimension Source Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'T2_OBJECT_DIM_SOURCE_DELETE', 'OID', 'OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'T2_OBJECT_DIM_SOURCE_DELETE', 'TABLE_NAME', 'Table Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'Cognos Model Update');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'COGNOS_MODEL_UPDATE_KEY', 'Cognos Model Update Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'QUEUED_TIME', 'Queued Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ATTEMPTS', 'Attempts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_1', 'Argument 1');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_2', 'Argument 2');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_3', 'Argument 3');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_4', 'Argument 4');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_5', 'Argument 5');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_6', 'Argument 6');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_7', 'Argument 7');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_8', 'Argument 8');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'ARG_9', 'Argument 9');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'UPDATE_START_TIME', 'Update Start Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'UPDATE_COMPLETION_TIME', 'Update Completion Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_MODEL_UPDATE', 'STATUS_MESSAGE', 'Status Message');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'USERS', 'Users');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'USERS', 'USERS_KEY', 'Users Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'USERS', 'LOGIN_NAME', 'Login Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'USERS', 'DOMAIN', 'Domain');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'USER_CONFIGURATION', 'User Configuration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'USER_CONFIGURATION', 'USERS_KEY', 'Users Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'USER_CONFIGURATION', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'USER_CONFIGURATION', 'VALUE', 'Value');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'Array Storage NetApp HAL Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'NA_VOLUME_CAPACITY', 'NetApp Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'NA_VOLUME_USED_CAPACITY', 'NetApp Volume Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'NA_VOLUME_AVAILABLE_CAPACITY', 'NetApp Volume Available Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'NA_QTREES', 'NetApp QTrees');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'NA_SHARES', 'NetApp Shares');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'NA_VOLUMES', 'NetApp Volumes');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_HAL_', 'NA_FILESYSTEMS', 'NAS Filesystems');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NA_VOLUME_BVIEW_NODE', 'NetApp Volume - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NA_VOLUME_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NA_VOLUME_BVIEW_NODE', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NA_VOLUME_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_VOLUME', 'NetApp Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'VENDOR_STATE', 'Vendor State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'VENDOR_LAYOUT', 'Vendor Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'RAID_LEVEL', 'RAID Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'IS_AGGREGATE', 'Is Aggregate');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'IS_SEGMENT', 'Is Segment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'IS_VFILER_VOLUME', 'Is VFiler Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'RPM', 'RPM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'DRIVE_TYPE', 'Drive Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'DISK_SIZE', 'Disk Size');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'IS_NAS_FS_CREATED', 'Is NAS FS Created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'DEDUP_STATE', 'Dedup State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'LAST_DEDUP_OP_BEGIN_TIME', 'Last Dedup Op begin time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'LAST_DEDUP_OP_END_TIME', 'Last Dedup Op end time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'STORAGE_POOL_KEY', 'Storage Pool Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME', 'STORAGE_POOL_OBJECT_IDENTIFIER', 'Storage Pool Object Identifier');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'NetApp Volume Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'AVAILABLE_CAPACITY', 'Available Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'QTREES', 'QTrees');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'DM_DATA_AGE', 'DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'TOTAL_CONSUMPTION', 'Total Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'FILES', 'Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'FOLDERS', 'Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'USERS', 'Users');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'PHYSICAL_CAPACITY', 'Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'OVERHEAD_CAPACITY', 'Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'PHYSICAL_USED', 'Physical Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'NUMBER_OF_DATA_DISKS', 'Number of data disks');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'NUMBER_OF_PARITY_DISKS', 'Number of parity disks');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'NUMBER_OF_VOLS_CREATED', 'Number of vols created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'NUMBER_OF_LUNS_CREATED', 'Number of LUNs created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'NUMBER_OF_TARGETS', 'Number of targets');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'SPACE_SHARED', 'Space Shared');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'SPACE_SAVED', 'Space Saved');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'EFFECTIVE_SPACE_USED', 'Effective Space Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'PHYSICAL_SPACE_SHARED', 'Physical Space Shared');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'PHYSICAL_SPACE_SAVED', 'Physical Space Saved');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'PHYSICAL_EFFECTIVE_SPACE_USED', 'Physical Effective Space Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'SUBSCRIBED_PERCENT', 'Subscribed Percent');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE', 'SPACE_SAVED_PERCENT', 'Space Saved Percent');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'FILE_CATEGORY', 'File Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY', 'FILE_CATEGORY_KEY', 'File Category Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY', 'FILE_CATEGORY_OID', 'File Category Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY', 'FILE_EXTENSIONS', 'File Extensions');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'NetApp File By File Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'FILE_CATEGORY_KEY', 'File Category Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'DM_DATA_AGE', 'DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_FILE_FCAT', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_DM_', 'Array Storage NetApp DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_DM_', 'NA_CONSUMPTION', 'NetApp Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_DM_', 'NA_FILES', 'NetApp Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_DM_', 'NA_FOLDERS', 'NetApp Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_NETAPP_DM_', 'NA_USERS', 'NetApp Users');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_SHARE_BVIEW_NODE', 'NetApp Share - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_BVIEW_NODE', 'NETAPP_SHARE_KEY', 'NetApp Share Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_SHARE', 'NetApp Share');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'NETAPP_SHARE_KEY', 'NetApp Share Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'NETAPP_SHARE_OID', 'NetApp Share Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'NETAPP_QTREE_KEY', 'NetApp QTree Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'SHARE_PATH', 'Share Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'NFS_EXPORT_PATH', 'NFS Export Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'CIFS_MOUNT_POINT', 'CIFS Mount Point');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'CIFS_SHARE_NAME', 'CIFS Share Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'MOUNT_POINT_TYPE', 'Mount Point Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'ESX_DATA_STORE_NAMES', 'ESX Data Store names');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'HOST_PRIVILEGE', 'Host Privilege');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE', 'IS_MAPPED_TO_ANY_UNDISCOV_HOST', 'Is Mapped to Any Undiscov Host');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'NetApp Share Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'NETAPP_SHARE_KEY', 'NetApp Share Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'DM_DATA_AGE', 'DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'TOTAL_CONSUMPTION', 'Total Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'FILES', 'Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'FOLDERS', 'Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'USERS', 'Users');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE', 'NO_OF_HOSTS', 'No of hosts');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_QTREE_BVIEW_NODE', 'NetApp QTree - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_BVIEW_NODE', 'NETAPP_QTREE_KEY', 'NetApp QTree Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_QTREE', 'NetApp Qtree');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'NETAPP_QTREE_KEY', 'NetApp QTree Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'NETAPP_QTREE_OID', 'NetApp QTree Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'QTREE_PATH', 'QTree Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'SECURITY_STYLE', 'Security Style');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'FILESYSTEM_ID', 'Filesystem ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'IS_QUOTA_ENABLED', 'Is Quota Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'NetApp QTree File');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'NETAPP_QTREE_KEY', 'NetApp QTree Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'QUOTA_LIMIT', 'Quota Limit');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'QUOTA_USED', 'Quota Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'DM_DATA_AGE', 'DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'TOTAL_CONSUMPTION', 'Total Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'FILES', 'Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'FOLDERS', 'Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE', 'USERS', 'Users');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'NetApp QTree File By File Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'NETAPP_QTREE_KEY', 'NetApp QTree Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'DM_DATA_AGE', 'DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'FILE_CATEGORY_KEY', 'File Category Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_FILE_FCAT', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'NetApp Volume File By File Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'FILE_CATEGORY_KEY', 'File Category Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'DM_DATA_AGE', 'DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_FILE_FCAT', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'NetApp Share File By File Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'NETAPP_SHARE_KEY', 'NetApp Share Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'FILE_CATEGORY_KEY', 'File Category Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'DM_DATA_AGE', 'DM Data Age');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_FILE_FCAT', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'FILE_CATEGORY_', 'File Category Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILE_CATEGORY_', 'FILE_CATEGORY_OID', 'File Category Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILE_CATEGORY_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILE_CATEGORY_', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILE_CATEGORY_', 'FILE_EXTENSIONS', 'File Extensions');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_DM_', 'NetApp File By File Category DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_DM_', 'FILE_CATEGORY_OID', 'File Category Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_DM_', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_DM_', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'NetApp Volume Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'VENDOR_STATE', 'Vendor State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'VENDOR_LAYOUT', 'Vendor Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'RAID_LEVEL', 'RAID Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'IS_AGGREGATE', 'Is Aggregate');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'IS_SEGMENT', 'Is Segment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'IS_VFILER_VOLUME', 'Is VFiler Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'RPM', 'RPM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'DRIVE_TYPE', 'Drive Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'DISK_SIZE', 'Disk Size');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'IS_NAS_FS_CREATED', 'Is NAS FS Created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'DEDUP_STATE', 'Dedup State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'LAST_DEDUP_OP_BEGIN_TIME', 'Last Dedup Op begin time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'LAST_DEDUP_OP_END_TIME', 'Last Dedup Op end time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_', 'STORAGE_POOL_OBJECT_IDENTIFIER', 'Storage Pool Object Identifier');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'NetApp Volume File HAL Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'AVAILABLE_CAPACITY', 'Available Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'QTREES', 'QTrees');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'PHYSICAL_CAPACITY', 'Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'OVERHEAD_CAPACITY', 'Overhead Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'PHYSICAL_USED', 'Physical Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'NUMBER_OF_DATA_DISKS', 'Number of data disks');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'NUMBER_OF_PARITY_DISKS', 'Number of parity disks');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'NUMBER_OF_VOLS_CREATED', 'Number of vols created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'NUMBER_OF_LUNS_CREATED', 'Number of LUNs created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'NUMBER_OF_TARGETS', 'Number of targets');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'SPACE_SHARED', 'Space Shared');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'SPACE_SAVED', 'Space Saved');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'EFFECTIVE_SPACE_USED', 'Effective Space Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'PHYSICAL_SPACE_SHARED', 'Physical Space Shared');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'PHYSICAL_SPACE_SAVED', 'Physical Space Saved');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'PHYSICAL_EFFECTIVE_SPACE_USED', 'Physical Effective Space Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'SUBSCRIBED_PERCENT', 'Subscribed Percent');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_HAL_', 'SPACE_SAVED_PERCENT', 'Space Saved Percent');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_DM_', 'Netapp Volume File DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_DM_', 'TOTAL_CONSUMPTION', 'Total Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_DM_', 'FILES', 'Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_DM_', 'FOLDERS', 'Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_DM_', 'USERS', 'Users');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_DM_', 'NetApp Volume File By File Category DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_DM_', 'FILE_CATEGORY_OID', 'File Category Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_DM_', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_DM_', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'NetApp QTree Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'NETAPP_QTREE_OID', 'NetApp QTree Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'QTREE_PATH', 'QTree Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'SECURITY_STYLE', 'Security Style');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'FILESYSTEM_ID', 'Filesystem ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_', 'IS_QUOTA_ENABLED', 'Is Quota Enabled');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_HAL_', 'NetApp QTree File HAL Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_HAL_', 'NETAPP_QTREE_OID', 'NetApp QTree Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_HAL_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_HAL_', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_HAL_', 'QUOTA_LIMIT', 'Quota Limit');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_HAL_', 'QUOTA_USED', 'Quota Used');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_DM_', 'Netapp QTree File DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_DM_', 'TOTAL_CONSUMPTION', 'Total Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_DM_', 'FILES', 'Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_DM_', 'FOLDERS', 'Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_DM_', 'USERS', 'Users');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_DM_', 'NetApp QTree File By File Category DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_DM_', 'FILE_CATEGORY_OID', 'File Category Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_DM_', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_DM_', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'NetApp Share Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'NETAPP_SHARE_OID', 'NetApp Share Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'NETAPP_QTREE_OID', 'NetApp QTree Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'NAS_FILESYSTEM_OID', 'NAS Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'SHARE_PATH', 'Share Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'NFS_EXPORT_PATH', 'NFS Export Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'CIFS_MOUNT_POINT', 'CIFS Mount Point');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'CIFS_SHARE_NAME', 'CIFS Share Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'MOUNT_POINT_TYPE', 'Mount Point Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'ESX_DATA_STORE_NAMES', 'ESX Data Store names');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'HOST_PRIVILEGE', 'Host Privilege');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_', 'IS_MAPPED_TO_ANY_UNDISCOV_HOST', 'Is Mapped to Any Undiscov Host');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_HAL_', 'NetApp Share File HAL Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_HAL_', 'NETAPP_SHARE_OID', 'NetApp Share Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_HAL_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_HAL_', 'NO_OF_HOSTS', 'No of hosts');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_DM_', 'Netapp Share File DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_DM_', 'TOTAL_CONSUMPTION', 'Total Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_DM_', 'FILES', 'Files');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_DM_', 'FOLDERS', 'Folders');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_DM_', 'USERS', 'Users');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_DM_', 'NetApp Share File By File Category DM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_DM_', 'DM_ID', 'DM Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_DM_', 'FILE_CATEGORY_OID', 'File Category Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_DM_', 'DM_BATCH_ID', 'DM Batch ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_DM_', 'CONSUMPTION', 'Consumption');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_DM_', 'FILES', 'Files');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COGNOS_SMTP', 'Cognos SMTP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_SMTP', 'COGNOS_SMTP_KEY', 'Cognos SMTP Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_SMTP', 'SERVER', 'Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_SMTP', 'PORT', 'Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_SMTP', 'SENDER', 'Sender');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_SMTP', 'USER', 'User');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COGNOS_SMTP', 'PASSWORD', 'Password');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DM_BATCH', 'DM Batch');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'DM_BATCH_KEY', 'DM Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'ROLLUP_SOURCE_KEY', 'Roll-up Source Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'SOURCE_NAME', 'Source Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'DB_HOST', 'Database Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'DB_PORT', 'Database Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'DM_BATCH_ID', 'DM Batch Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'TIME', 'Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'TIME_SOURCE', 'Time - Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DM_BATCH', 'TIME_ER', 'Time - Enterprise Reporter Time');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'DM_BATCH_', 'DM Batch Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DM_BATCH_', 'DM_BATCH_ID', 'DM Batch Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DM_BATCH_', 'TIME', 'Time');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_DATA_SET_STAGE', 'Roll-up Data Set Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STAGE', 'ROLLUP_DATA_SET_TYPE_KEY', 'Roll-up Data Set Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STAGE', 'STAGE_TABLE', 'Stage Table');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_FILESYSTEM', 'NAS Filesystem');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'NAS_FILESYSTEM_OID', 'NAS Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'SECURITY_STYLE', 'Security Style');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'FILESYSTEM_ID', 'Filesystem ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'IS_QUOTA_ENABLED', 'Is Quota Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'PATH', 'Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'AUTO_EXTEND_ENABLED', 'Auto extend enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'IS_LOCK_ENABLED', 'Is Lock Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'CWORM_STATE', 'CWORM State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'IS_VIRTUAL_PROVISIONING_ENABLE', 'Is Virtual Provisioning enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'IS_SOURCE', 'Is Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'IS_TARGET', 'Is Target');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'REPLICATION_TECHNOLOGY', 'Replication Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'AUTO_EXTEND_MAX_SIZE', 'Auto extend Max size');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM', 'HIGH_WATERMARK', 'High watermark');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'NAS Filesystems Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'NAS_FILESYSTEM_OID', 'NAS Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'SECURITY_STYLE', 'Security Style');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'FILESYSTEM_ID', 'Filesystem ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'IS_QUOTA_ENABLED', 'Is Quota Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'PATH', 'Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'AUTO_EXTEND_ENABLED', 'Auto extend enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'IS_LOCK_ENABLED', 'Is Lock Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'CWORM_STATE', 'CWORM State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'IS_VIRTUAL_PROVISIONING_ENABLE', 'Is Virtual Provisioning enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'IS_SOURCE', 'Is Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'IS_TARGET', 'Is Target');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'REPLICATION_TECHNOLOGY', 'Replication Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'AUTO_EXTEND_MAX_SIZE', 'Auto extend Max size');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_', 'HIGH_WATERMARK', 'High watermark');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'NAS Filesystem Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'FREE_CAPACITY', 'Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'PHYSICAL_CAPACITY', 'Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'VOLUME_CAPACITY', 'Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'TREE_QUOTA_LIMIT', 'Tree Quota Limit');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'TREE_QUOTA_USED', 'Tree Quota used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'NUMBER_OF_LUNS', 'Number Of LUNs');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'FILE_COUNT_TOTAL', 'File Count Total');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'FILE_COUNT_USED', 'File Count Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'PHYSICAL_USED', 'Physical Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'PHYSICAL_FREE', 'Physical Free');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'PHYSICAL_VOLUME_CAPACITY', 'Physical Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'PHYSICAL_TREE_QUOTA_LIMIT', 'Physical Tree Quota Limit');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'PHYSICAL_TREE_QUOTA_USED', 'Physical Tree Quota Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'NUMBER_OF_TARGETS', 'Number of targets');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'REPLICATION_TARGET_CAPACITY', 'Replication Target Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CAPACITY', 'NO_OF_HOSTS_ACCESSING_NAS', 'No of hosts accessing NAS');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'NAS Filesystem Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'NAS_FILESYSTEM_OID', 'NAS Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'FREE_CAPACITY', 'Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'PHYSICAL_CAPACITY', 'Physical Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'VOLUME_CAPACITY', 'Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'TREE_QUOTA_LIMIT', 'Tree Quota Limit');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'TREE_QUOTA_USED', 'Tree Quota used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'NUMBER_OF_LUNS', 'Number Of LUNs');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'FILE_COUNT_TOTAL', 'File Count Total');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'FILE_COUNT_USED', 'File Count Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'PHYSICAL_USED', 'Physical Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'PHYSICAL_FREE', 'Physical Free');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'PHYSICAL_VOLUME_CAPACITY', 'Physical Volume Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'PHYSICAL_TREE_QUOTA_LIMIT', 'Physical Tree Quota Limit');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'PHYSICAL_TREE_QUOTA_USED', 'Physical Tree Quota Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'NUMBER_OF_TARGETS', 'Number of targets');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'REPLICATION_TARGET_CAPACITY', 'Replication Target Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_FILESYSTEM_CAPACITY_', 'NO_OF_HOSTS_ACCESSING_NAS', 'No of hosts accessing NAS');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_FS_BVIEW_NODE', 'NAS Filesystem - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FS_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FS_BVIEW_NODE', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FS_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FS_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'LUN_FILESYSTEM_', 'LUN - FILESYSTEM Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_FILESYSTEM_', 'FILESYSTEM_OID', 'Filesystem Object Identifer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_FILESYSTEM_', 'LUN_OID', 'LUN Object Identifer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_FILESYSTEM_', 'ARRAY_OID', 'Array Object Identifer');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_FILESYSTEM', 'LUN_FILESYSTEM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_FILESYSTEM', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_FILESYSTEM', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_FILESYSTEM', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_FILESYSTEM', 'FILESYSTEM_KEY', 'Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_FILESYSTEM', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'LUN_VOLUME_', 'LUN - VOLUME Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_VOLUME_', 'VOLUME_OID', 'Volume Object Identifer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_VOLUME_', 'LUN_OID', 'LUN Object Identifer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_VOLUME_', 'ARRAY_OID', 'Array Object Identifer');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_VOLUME', 'LUN_VOLUME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_VOLUME', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_VOLUME', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_VOLUME', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_VOLUME', 'VOLUME_KEY', 'Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_VOLUME', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'FILESYSTEM_BVIEW_NODE', 'Filesystem - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_BVIEW_NODE', 'FILESYSTEM_KEY', 'Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'VOLUME_BVIEW_NODE', 'Volume - Business View Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_BVIEW_NODE', 'VOLUME_KEY', 'Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_RULE', 'BVIEW_RULE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULE', 'BVIEW_RULE_KEY', 'Business View Rule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULE', 'DIMENSION', 'Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULE', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULE', 'UPPER_DISPLAY_NAME', 'Upper Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULE', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULE', 'RANK', 'Rank');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULE', 'BVIEW_KEY', 'Busines View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_SUBRULE', 'Business View  Subrule');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'BVIEW_SUBRULE_KEY', 'Business View Subrule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'GUI_HINT', 'GUI Hint');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'RULE_TEXT', 'Rule Text');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'SQL_CODE', 'SQL Code');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'RANK', 'Rank');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'BVIEW_RULE_KEY', 'Business View Rule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'Business View Subrule Element');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'DIMENSION', 'Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'RULE_TOKEN_UPPER', 'Rule Token Upper');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'RULE_TOKEN', 'Rule Token');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUBRULE_ELEMENT', 'SQL_CODE', 'SQL Code');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_CLAIMED_STATES', 'LUN_CLAIMED_STATES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CLAIMED_STATES', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CLAIMED_STATES', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CLAIMED_STATES', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CLAIMED_STATES', 'CLAIMED_STATUS', 'Claimed Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ACTUAL_LUN_CLAIMED_STATES', 'ACTUAL_LUN_CLAIMED_STATES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ACTUAL_LUN_CLAIMED_STATES', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ACTUAL_LUN_CLAIMED_STATES', 'CLAIMED_STATUS', 'Claimed Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ARRAY_CLAIMED_CAPACITY', 'ARRAY_CLAIMED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CLAIMED_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CLAIMED_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CLAIMED_CAPACITY', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CLAIMED_CAPACITY', 'ROLLUP_SOURCE_KEY', 'Rollup Source Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CLAIMED_CAPACITY', 'CLAIMED_BY_HOSTS', 'Claimed by Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CLAIMED_CAPACITY', 'PERCENT_ALLOCATED_HOST_CLAIMED', 'Percent Allocated Host Claimed');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ACTUAL_ARRAY_CLAIMED_CAPACITY', 'ACTUAL_ARRAY_CLAIMED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ACTUAL_ARRAY_CLAIMED_CAPACITY', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ACTUAL_ARRAY_CLAIMED_CAPACITY', 'CLAIMED_BY_HOSTS', 'Claimed by Hosts');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ACTUAL_ARRAY_CLAIMED_CAPACITY', 'PERCENT_ALLOCATED_HOST_CLAIMED', 'Percent Allocated Host Claimed');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BROKER_INFO', 'Broker Info');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'BROKER_ID', 'Broker ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'DOMAIN_NAME', 'Domain Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'DOMAIN_TYPE', 'Domain Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'BROKER_NAME', 'Broker Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'BROKER_TYPE', 'Broker Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'BROKER_PORT', 'Broker Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'LDAP_SERVER', 'LDAP Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'BIND_USER_NAME', 'Bind User Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'BIND_USER_PASSWORD', 'Bind User Password');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'SEARCH_USER', 'Search User');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'SEARCH_GROUP', 'Search Group');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'SEARCH_BASE', 'Search Base');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'SSL_CERT_LOCATION', 'SSL Cert Location');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BROKER_INFO', 'LDAP_PORT', 'LDAP_PORT');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SECURITY_GROUP', 'Security Group');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP', 'SECURITY_GROUP_ID', 'Security Group Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP', 'ROLE_ID', 'Role Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP', 'BROKER_ID', 'Broker Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP', 'PRIVILEGE', 'Privilege');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP', 'DESCRIPT', 'Descript');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SECURITY_GROUP_REL', 'Security Group Rel');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP_REL', 'SECURITY_GROUP_ID', 'Security Group Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP_REL', 'BVIEW_ID', 'BView Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP_REL', 'ROLE_ID', 'Role Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_GROUP_REL', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SECURITY_ROLE', 'Security Role');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_ROLE', 'ROLE_ID', 'Role Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_ROLE', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_ROLE', 'DESCRIPT', 'Descript');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SECURITY_PATTERN', 'Security Pattern');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SECURITY_PATTERN', 'PATTERN_ID', 'Pattern Id');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_LICENSE', 'SF License');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'SF_LICENSE_KEY', 'SF License Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'SF_LICENSE_OID', 'SF License OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'KEY', 'Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'IS_SITE_LICENSE', 'Is Site License');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'IS_EXPIRED', 'Is Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'IS_KEYLESS', 'Is Keyless');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_PRODUCT', 'SF Product');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'SF_PRODUCT_KEY', 'SF Product Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'SF_PRODUCT_OID', 'SF Product OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'PRODUCT_NAME', 'Product Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'SHORT_NAME', 'Short Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'PRODUCT_CATEGORY', 'Product Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'PRODUCT_EDITION', 'Product Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'PRODUCT_VERSION', 'Product Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'PLATFORM', 'Platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'LICENSE_TYPE', 'License Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'SF Deployed Licenses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'ROLLUP_BATCH_KEY', 'Rollup Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'SF_LICENSE_KEY', 'SF License Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'SF_PRODUCT_KEY', 'SF Product Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'SF_LICENSE_HOST_KEY', 'SF License Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'SF_LICENSE_DEPLOYMENT_KEY', 'SF License Deployment Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'LICENSE_DEPLOYMENTS', 'License Deployments');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'LICENSE_DEPLOYMENTS_EXPIRED', 'License Deployments Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'ACCOUNTED_DEPLOYMENTS', 'Accounted Deployments');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'ACCOUNTED_DEPLOYMENTS_EXPIRED', 'Accounted Deployments Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'KEYLESS_DEPLOYMENTS', 'Keyless Deployments');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_DEPLOYED_LICENSES', 'TOTAL_SPVU', 'Total SPVU');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_LICENSE_HOST', 'SF License Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'SF_LICENSE_HOST_KEY', 'SF License Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'SF_LICENSE_HOST_OID', 'SF License Host OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'ID', 'Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'PROCESSOR_TYPE', 'Processor Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'NUM_PROCESSORS', 'Num Processors');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'PLATFORM', 'Platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'SERVER_PRICE_TIER', 'Server Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'PROCESSOR_PRICE_TIER', 'Processor Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'NUM_PROCESSOR_PRICE_TIER', 'Num Processor Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'OS_PRICE_TIER', 'OS Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'ASSIGNED_PRICE_TIER', 'Assigned Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'OS_EDITION', 'OS Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'OS', 'OS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'HAS_ENTERPRISE_EDITION', 'Has Enterprise Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'SPVU', 'SPVU');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'NCORE', 'NCORE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SF_LICENSE_', 'SF License Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_', 'SF_LICENSE_OID', 'SF License Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_', 'KEY', 'Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_', 'IS_SITE_LICENSE', 'Is Site License');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_', 'IS_EXPIRED', 'Is Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_', 'IS_KEYLESS', 'Is Keyless');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'SF License Host Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'SF_LICENSE_HOST_OID', 'SF License Host OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'HOST_OID', 'Host OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'ID', 'Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'PROCESSOR_TYPE', 'Processor Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'NUM_PROCESSORS', 'Num Processors');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'PLATFORM', 'Platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'SERVER_PRICE_TIER', 'Server Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'PROCESSOR_PRICE_TIER', 'Processor Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'NUM_PROCESSOR_PRICE_TIER', 'Num Processor Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'OS_PRICE_TIER', 'OS Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'ASSIGNED_PRICE_TIER', 'Assigned Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'OS_EDITION', 'OS Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'OS', 'OS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'HAS_ENTERPRISE_EDITION', 'Has Enterprise Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'SPVU', 'SPVU');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_HOST_', 'NCORE', 'NCORE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SF_PRODUCT_', 'SF Product Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'SF_PRODUCT_OID', 'SF Product OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'PRODUCT_NAME', 'Product Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'SHORT_NAME', 'Short Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'PRODUCT_CATEGORY', 'Product Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'PRODUCT_EDITION', 'Product Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'PRODUCT_VERSION', 'Product Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'PLATFORM', 'Platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_PRODUCT_', 'LICENSE_TYPE', 'License Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'SF License Deployment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'SF_LICENSE_DEPLOYMENT_KEY', 'SF License Deployment Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'SF_LICENSE_DEPLOYMENT_OID', 'SF License Deployment OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'SF_LICENSE_KEY', 'SF License Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'SF_PRODUCT_KEY', 'SF Product Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'SF_LICENSE_HOST_KEY', 'SF License Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'EXPIRY_DATE', 'Expiry Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'IS_EXPIRED', 'Is Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'IS_ACCOUNTABLE', 'Is Accountable');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'SF License Deployment Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'SF_LICENSE_DEPLOYMENT_OID', 'SF License Deployment OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'SF_LICENSE_OID', 'SF License OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'SF_PRODUCT_OID', 'SF Product OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'SF_LICENSE_HOST_OID', 'SF License Host OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'EXPIRY_DATE', 'Expiry Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'IS_EXPIRED', 'Is Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_LICENSE_DEPLOYMENT_', 'IS_ACCOUNTABLE', 'Is Accountable');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'SF Deployed Licenses Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'SF_LICENSE_OID', 'SF License OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'SF_PRODUCT_OID', 'SF Product OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'SF_LICENSE_HOST_OID', 'SF License Host OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'SF_LICENSE_DEPLOYMENT_OID', 'SF License Deployment OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'LICENSE_DEPLOYMENTS', 'License Deployments');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'LICENSE_DEPLOYMENTS_EXPIRED', 'License Deployments Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'ACCOUNTED_DEPLOYMENTS', 'Accounted Deployments');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'ACCOUNTED_DEPLOYMENTS_EXPIRED', 'Accounted Deployments Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'KEYLESS_DEPLOYMENTS', 'Keyless Deployments');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SF_DEPLOYED_LICENSES_', 'TOTAL_SPVU', 'Total SPVU');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DISKGROUP', 'Diskgroup');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'DISKGROUP_KEY', 'Diskgroup Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'DISKGROUP_OID', 'Diskgroup Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'FAMILY', 'Family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'VERSION', 'Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'IS_SHARED', 'Is Shared');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'IS_BASED_ON_DIRECT_ATTACHED', 'Is Based On Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'STATE', 'State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'IS_USED_BY_CLUSTERED_DB', 'Is Used By Clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP', 'USES_VM_VOLUMES', 'Uses VM Volumes');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'Diskgroup Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'DISKGROUP_KEY', 'Diskgroup Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'FREE_CAPACITY', 'Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'PERCENT_FREE', 'Percent Free');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'PERCENT_USED', 'Percent Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'OVERHEAD_CAPACITY', 'Overhead capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'DB_USED_CAPACITY', 'DB Used capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CAPACITY', 'USED_CAPACITY_BY_CLUSTERED_DB', 'Used Capacity By Clustered DB');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'Diskgroup Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'DISKGROUP_OID', 'Diskgroup Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'CAPACITY', 'Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'FREE_CAPACITY', 'Free Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'USED_CAPACITY', 'Used Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'PERCENT_FREE', 'Percent Free');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'PERCENT_USED', 'Percent Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'OVERHEAD_CAPACITY', 'Overhead capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'UNKNOWN_CAPACITY', 'Unknown Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'DB_USED_CAPACITY', 'DB Used capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'DISKGROUP_CAPACITY_', 'USED_CAPACITY_BY_CLUSTERED_DB', 'Used Capacity By Clustered DB');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_DISKGROUP', 'LUN_DISKGROUP');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_DISKGROUP', 'DISKGROUP_KEY', 'Diskgroup Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_DISKGROUP', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_DISKGROUP', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_DISKGROUP', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_DISKGROUP', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'FILESYSTEM_', 'Filesystem Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'FILESYSTEM_OID', 'Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'VOLUME_OID', 'Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'MOUNT_POINT', 'Mount Point');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'DEVICE_PATH', 'Device Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_CREATED_FROM_VOLUME', 'Is Created From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_VCS_CVM', 'Is VCS CVM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_CFS_AWARE', 'IS CFS AWARE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_CHILD', 'Is Child');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'FILESYSTEM_', 'IS_BASED_ON_NAS_SHARE', 'Is Based on NAS share');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'LUN_DISKGROUP_', 'LUN - DISKGROUP Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_DISKGROUP_', 'DISKGROUP_OID', 'Diskgroup Object Identifer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_DISKGROUP_', 'LUN_OID', 'LUN Object Identifer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'LUN_DISKGROUP_', 'ARRAY_OID', 'Array Object Identifer');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DB_VERSION', 'DB_VERSION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DB_VERSION', 'MAJOR', 'MAJOR');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DB_VERSION', 'MINOR', 'MINOR');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DB_VERSION', 'MP_VERSION', 'MP_VERSION');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_QUOTA', 'NAS Quota');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'NAS_QUOTA_KEY', 'NAS Quota Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'NAS_QUOTA_OID', 'NAS Quota Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'TARGET', 'Target');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'NAS Quota Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'NAS_QUOTA_KEY', 'NAS Quota Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'DISK_LIMIT_HARD', 'Disk Limit Hard');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'DISK_LIMIT_SOFT', 'Disk Limit Soft');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'DISK_USED', 'Disk Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'FILE_LIMIT_HARD', 'File Limit Hard');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'FILE_LIMIT_SOFT', 'File Limit Soft');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'FILES_USED', 'Files Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'THRESHOLD', 'Threshold');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'PHYSICAL_DISK_LIMIT_HARD', 'Physical Disk Limit Hard');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'PHYSICAL_DISK_LIMIT_SOFT', 'Physical Disk Limit Soft');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CAPACITY', 'PHYSICAL_DISK_USED', 'Physical Disk Used');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NAS_QUOTA_', 'NAS Quotas Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_', 'NAS_QUOTA_OID', 'NAS Quota Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_', 'NAS_FILESYSTEM_OID', 'NAS Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_', 'TARGET', 'Target');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'NAS Quota Capacity Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'NAS_QUOTA_OID', 'NAS Quota Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'DISK_LIMIT_HARD', 'Disk Limit Hard');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'DISK_LIMIT_SOFT', 'Disk Limit Soft');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'DISK_USED', 'Disk Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'FILE_LIMIT_HARD', 'File Limit Hard');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'FILE_LIMIT_SOFT', 'File Limit Soft');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'FILES_USED', 'Files Used');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'THRESHOLD', 'Threshold');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'PHYSICAL_DISK_LIMIT_HARD', 'Physical Disk Limit Hard');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'PHYSICAL_DISK_LIMIT_SOFT', 'Physical Disk Limit Soft');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_QUOTA_CAPACITY_', 'PHYSICAL_DISK_USED', 'Physical Disk Used');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_VOLUME_FILER', 'NAS Volume Filer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_VOLUME_FILER', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_VOLUME_FILER', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_SHARE_HOST', 'NAS Share Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_SHARE_HOST', 'NETAPP_SHARE_KEY', 'NetApp Share Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_SHARE_HOST', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_SHARE_HOST', 'ACCESS_TYPE', 'Access Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NAS_VOLUME_FILER_', 'NAS Volume Filer_');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_VOLUME_FILER_', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_VOLUME_FILER_', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NAS_SHARE_HOST_', 'NAS Share Host_');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_SHARE_HOST_', 'NETAPP_SHARE_OID', 'NetApp Share Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_SHARE_HOST_', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_SHARE_HOST_', 'ACCESS_TYPE', 'Access Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_SHARE_HOST_FILESYSTEM', 'NAS Share Host Filesystem');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_SHARE_HOST_FILESYSTEM', 'NETAPP_SHARE_KEY', 'NetApp Share Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_SHARE_HOST_FILESYSTEM', 'FILESYSTEM_KEY', 'FileSystem Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NAS_SHARE_HOST_FILESYSTEM_', 'NAS Share Host Filesystem_');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_SHARE_HOST_FILESYSTEM_', 'NETAPP_SHARE_OID', 'NetApp Share Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NAS_SHARE_HOST_FILESYSTEM_', 'FILESYSTEM_OID', 'Filesystem Identifier');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER', 'Cluster');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'CLUSTER_TYPE', 'cluster_type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'CLUSTER_CATEGORY', 'cluster_category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'VERSION', 'version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'PLATFORM', 'platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'USE_FENCE', 'use_fence');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'IS_GLOBAL', 'is_global');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'SECURECLUS', 'secureclus');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'REMOTECLUSTERS', 'remoteclusters');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'T2_IS_CURRENT', 'T2_is_current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'T2_IS_DELETED', 'T2_is_deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'APPLICATION_BE', 'Application_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'APPLICATION_COMPOSITE_BE', 'Application_Composite_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'ORGANIZATION_BE', 'Organization_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER', 'ORGANIZATION_COMPOSITE_BE', 'Organization_Composite_BE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'CLUSTER_', 'Cluster Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'CLUSTER_TYPE', 'cluster_type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'CLUSTER_CATEGORY', 'cluster_category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'VERSION', 'version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'PLATFORM', 'platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'USE_FENCE', 'use_fence');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'IS_GLOBAL', 'is_global');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'SECURECLUS', 'secureclus');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'REMOTECLUSTERS', 'remoteclusters');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'APPLICATION_BE', 'Application_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'APPLICATION_COMPOSITE_BE', 'Application_Composite_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'ORGANIZATION_BE', 'Organization_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_', 'ORGANIZATION_COMPOSITE_BE', 'Organization_Composite_BE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_HOST', 'Cluster Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'CLUSTER_HOST_KEY', 'cluster_host_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'HOST_KEY', 'host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'CLUSTER_HOST_OID', 'cluster_host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'HOST_OID', 'host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'CLUSTER_HOST_', 'Cluster Host Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_HOST_', 'CLUSTER_HOST_OID', 'cluster_host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_HOST_', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_HOST_', 'HOST_OID', 'host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_HOST_', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_INVENTORY', 'Cluster Inventory');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_INVENTORY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_INVENTORY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_INVENTORY', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_INVENTORY', 'NUMBER_OF_NODE', 'number_of_node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_INVENTORY', 'NUMBER_OF_SERVICEGROUP', 'number_of_servicegroup');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_INVENTORY', 'NUMBER_OF_RESOURCE', 'number_of_resource');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'CLUSTER_INVENTORY_', 'Cluster Inventory Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_INVENTORY_', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_INVENTORY_', 'NUMBER_OF_NODE', 'number_of_node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_INVENTORY_', 'NUMBER_OF_SERVICEGROUP', 'number_of_servicegroup');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_INVENTORY_', 'NUMBER_OF_RESOURCE', 'number_of_resource');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'Cluster RemoteCluster');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'CLUSTER_REMOTECLUSTER_KEY', 'cluster_remotecluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'CLUSTER_REMOTECLUSTER_OID', 'cluster_remotecluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'REMOTECLUSTER_OID', 'remotecluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'CLUSTER_REMOTECLUSTER_', 'Cluster RemoteCluster Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_REMOTECLUSTER_', 'CLUSTER_REMOTECLUSTER_OID', 'cluster_remotecluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_REMOTECLUSTER_', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_REMOTECLUSTER_', 'REMOTECLUSTER_OID', 'remotecluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'CLUSTER_REMOTECLUSTER_', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SERVICEGROUP', 'Servicegroup');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'SERVICEGROUP_KEY', 'servicegroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'SG_TYPE', 'sg_type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'SCOPE', 'scope');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'FAILOVERPOLICY', 'failoverpolicy');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'IS_APPHA', 'is_AppHA');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'FAMILY', 'family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'T2_IS_CURRENT', 'T2_is_current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'T2_IS_DELETED', 'T2_is_deleted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'APPLICATION_BE', 'Application_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'APPLICATION_COMPOSITE_BE', 'Application_Composite_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'ORGANIZATION_BE', 'Organization_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP', 'ORGANIZATION_COMPOSITE_BE', 'Organization_Composite_BE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SERVICEGROUP_', 'Servicegroup Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'SG_TYPE', 'sg_type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'SCOPE', 'scope');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'FAILOVERPOLICY', 'failoverpolicy');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'IS_APPHA', 'Is_AppHA');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'FAMILY', 'family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'APPLICATION_BE', 'Application_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'APPLICATION_COMPOSITE_BE', 'Application_Composite_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'ORGANIZATION_BE', 'Organization_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_', 'ORGANIZATION_COMPOSITE_BE', 'Organization_Composite_BE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SERVICEGROUP_INVENTORY', 'Servicegroup Inventory');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_INVENTORY', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_INVENTORY', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_INVENTORY', 'SERVICEGROUP_KEY', 'servicegroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_INVENTORY', 'NUMBER_OF_RESOURCE', 'number_of_resource');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SERVICEGROUP_INVENTORY_', 'Servicegroup Inventory Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_INVENTORY_', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_INVENTORY_', 'NUMBER_OF_RESOURCE', 'number_of_resource');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SERVICEGROUP_HOST', 'Servicegroup Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'SERVICEGROUP_HOST_KEY', 'servicegroup_host_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'SERVICEGROUP_KEY', 'servicegroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'HOST_KEY', 'host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'SERVICEGROUP_HOST_OID', 'servicegroup_host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'HOST_OID', 'host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'T2_IS_CURRENT', 'Type 2 - Is Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST', 'T2_IS_DELETED', 'Type 2 - Is Deleted');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SERVICEGROUP_HOST_', 'Servicegroup Host Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HOST_', 'SERVICEGROUP_HOST_OID', 'servicegroup_host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HOST_', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HOST_', 'HOST_OID', 'host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HOST_', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SERVICEGROUP_BVIEW_NODE', 'Servicegroup BVIEW Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_BVIEW_NODE', 'SERVICEGROUP_KEY', 'servicegroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_BVIEW_NODE', 'Cluster BView Node');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_BVIEW_NODE', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_BVIEW_NODE', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_BVIEW_NODE', 'BVIEW_KEY', 'Business View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_BVIEW_NODE', 'STATUS', 'Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_RULES', 'BVIEW_RULES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'BVIEW_RULES_KEY', 'Business View Rule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'OBJECTTYPE', 'ObjectType');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'APPROVALREQUIRED', 'ApprovalRequired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'ENABLED', 'Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'BVIEW_KEY', 'Busines View Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'BVIEW_NODE_KEY', 'Business View Node Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'DIMENSION', 'Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'APPROVALSTATE', 'ApprovalState');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_RULES', 'SQLQUERY', 'SQLQuery');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_CONDITIONS', 'Business View Rule Condition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_CONDITIONS', 'BVIEW_RULES_KEY', 'Business View Rule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_CONDITIONS', 'BVIEW_CONDITIONS_KEY', 'Business View Rule Condition key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_CONDITIONS', 'ATTRIBUTE', 'Attribute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_CONDITIONS', 'OPERATOR', 'operator');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_CONDITIONS', 'ATTRIBUTEVALUE', 'AttributeValue');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_CONDITIONS', 'CONJUNCTION', 'Conjunction');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'Business View Rule Condition2');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'BVIEW_RULES_KEY', 'Business View Rule Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'BVIEW_APPROVALPENDING_KEY', 'Approval Pending');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'BUSINESSVIEW_NAME', 'BusinessView Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'BUSINESSVIEWNODENAME', 'BusinessViewNodeName');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'OBJECTTYPE', 'ObjectType');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'APPROVALSTATE', 'ApprovalState');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_APPROVAL_PENDING', 'NOOFOBJECTS', 'NoOfObjects');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'FORECASTING_DIMENSION', 'Forecasting Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FORECASTING_DIMENSION', 'DIMENSION', 'Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FORECASTING_DIMENSION', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FORECASTING_DIMENSION', 'METAMODEL_PATH', 'Metamodel Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FORECASTING_DIMENSION', 'FACT_TABLE_NAME', 'Fact Table Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FORECASTING_DIMENSION', 'METAMODEL_KEY', 'Metamodel Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'SERVICEGROUP HIST_EVENTS_');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'DATE', 'date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'PLANNED_FAILOVERS', 'planned_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'UNPLANNED_AUTO_FAILOVERS', 'unplanned_auto_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'UNPLANNED_MAN_FAILOVERS', 'unplanned_man_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TOTAL_SUCCESS_FAILOVERS', 'total_success_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'UNRESOLVED_PLANNED_FAILOVERS', 'unresolved_planned_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'UNRESOLVED_UNPLANNED_AUTO_FAIL', 'unresolved_unplanned_auto_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'UNRESOLVED_UNPLANNED_MAN_FAILO', 'unresolved_unplanned_man_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TOTAL_FAILED_FAILOVERS', 'total_failed_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TIME_ONLINE', 'time_online');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TIME_OFFLINE', 'time_offline');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TIME_PARTIAL', 'time_partial');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TIME_FAULTED', 'time_faulted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TIME_TRANSIENTSTATES', 'time_transientstates');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'SERVICEGROUP_HIST_EVENTS_', 'TIME_NOTKNOWN', 'time_notknown');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'SERVICEGROUP_HIST_EVENTS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'SERVICEGROUP_KEY', 'servicegroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'PLANNED_FAILOVERS', 'planned_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'UNPLANNED_AUTO_FAILOVERS', 'unplanned_auto_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'UNPLANNED_MAN_FAILOVERS', 'unplanned_man_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TOTAL_SUCCESS_FAILOVERS', 'total_success_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'UNRESOLVED_PLANNED_FAILOVERS', 'unresolved_planned_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'UNRESOLVED_UNPLANNED_AUTO_FAIL', 'unresolved_unplanned_auto_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'UNRESOLVED_UNPLANNED_MAN_FAILO', 'unresolved_unplanned_man_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TOTAL_FAILED_FAILOVERS', 'total_failed_failovers');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TIME_ONLINE', 'time_online');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TIME_OFFLINE', 'time_offline');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TIME_PARTIAL', 'time_partial');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TIME_FAULTED', 'time_faulted');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TIME_TRANSIENTSTATES', 'time_transientstates');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HIST_EVENTS', 'TIME_NOTKNOWN', 'time_notknown');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ETL_LOG', 'ETL_LOG');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ETL_LOG', 'ETL_LOG_KEY', 'ETL Log Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ETL_LOG', 'BATCH_KEY', 'Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ETL_LOG', 'TIME', 'Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ETL_LOG', 'PROCEDURE_NAME', 'Procedure Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ETL_LOG', 'SEVERITY', 'Severity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ETL_LOG', 'MESSAGE', 'Message');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'ETL_LOG_', 'ETL Log Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ETL_LOG_', 'BATCH_KEY', 'Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ETL_LOG_', 'TIME', 'Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ETL_LOG_', 'PROCEDURE_NAME', 'Procedure Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ETL_LOG_', 'SEVERITY', 'Severity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ETL_LOG_', 'MESSAGE', 'Message');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW', 'Business View');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW', 'BVIEW_KEY', 'BVIEW_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW', 'NAME', 'NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW', 'DISPLAY_NAME', 'DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW', 'NOTE', 'NOTE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW', 'VIEW_CREATION_REQUIRED', 'VIEW_CREATION_REQUIRED');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_HIERARCHY', 'Business View Hierarchy');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'ANCESTOR_KEY', 'ANCESTOR_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'ANCESTOR_PARENT_KEY', 'ANCESTOR_PARENT_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'ANCESTOR_BVIEW_KEY', 'ANCESTOR_BVIEW_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'ANCESTOR_DISPLAY_NAME', 'ANCESTOR_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'ANCESTOR_LEVEL', 'ANCESTOR_LEVEL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'DESCENDENT_KEY', 'DESCENDENT_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'DESCENDENT_PARENT_KEY', 'DESCENDENT_PARENT_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'DESCENDENT_BVIEW_KEY', 'DESCENDENT_BVIEW_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'DESCENDENT_DISPLAY_NAME', 'DESCENDENT_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_HIERARCHY', 'DESCENDENT_LEVEL', 'DESCENDENT_LEVEL');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'RSOURCE_TYPE_SUPPORTED_DS', 'Rollup Source Type Supported Data Sets');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RSOURCE_TYPE_SUPPORTED_DS', 'ROLLUP_SOURCE_DISPLAY_NAME', 'ROLLUP_SOURCE_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RSOURCE_TYPE_SUPPORTED_DS', 'ROLLUP_DATA_SET_TYPE_NAME', 'ROLLUP_DATA_SET_TYPE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RSOURCE_TYPE_SUPPORTED_DS', 'ROLLUP_SOURCE_TYPE_KEY', 'ROLLUP_SOURCE_TYPE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'RSOURCE_TYPE_SUPPORTED_DS', 'ROLLUP_DATA_SET_TYPE_KEY', 'ROLLUP_DATA_SET_TYPE_KEY');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_STATUS', 'Rollup Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'ROLLUP_BATCH_KEY', 'ROLLUP_BATCH_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_START_TIME', 'BATCH_START_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_START_TIME_ER', 'BATCH_START_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_COMPLETION_TIME', 'BATCH_COMPLETION_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_COMPLETION_TIME_ER', 'BATCH_COMPLETION_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_IS_ROLLUP_NOW', 'BATCH_IS_ROLLUP_NOW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_IS_COMPLETE', 'BATCH_IS_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_IS_SUCCESS', 'BATCH_IS_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_IS_LATEST', 'BATCH_IS_LATEST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_IS_LATEST_COMPLETE', 'BATCH_IS_LATEST_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_IS_LATEST_SUCCESS', 'BATCH_IS_LATEST_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_STATUS', 'BATCH_STATUS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_STATUS_MESSAGE', 'BATCH_STATUS_MESSAGE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'BATCH_LAST_STATUS_UPDATE', 'BATCH_LAST_STATUS_UPDATE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'SOURCE_NAME', 'SOURCE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'SOURCE_TYPE_DISPLAY_NAME', 'SOURCE_TYPE_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_STATUS', 'LOG_ARCHIVE', 'LOG_ARCHIVE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'Rollup Data Set Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'ROLLUP_BATCH_KEY', 'ROLLUP_BATCH_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_START_TIME', 'BATCH_START_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_START_TIME_ER', 'BATCH_START_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_COMPLETION_TIME', 'BATCH_COMPLETION_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_COMPLETION_TIME_ER', 'BATCH_COMPLETION_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_IS_ROLLUP_NOW', 'BATCH_IS_ROLLUP_NOW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_IS_COMPLETE', 'BATCH_IS_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_IS_SUCCESS', 'BATCH_IS_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_IS_LATEST', 'BATCH_IS_LATEST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_IS_LATEST_COMPLETE', 'BATCH_IS_LATEST_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_IS_LATEST_SUCCESS', 'BATCH_IS_LATEST_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_STATUS', 'BATCH_STATUS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_STATUS_MESSAGE', 'BATCH_STATUS_MESSAGE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'BATCH_LAST_STATUS_UPDATE', 'BATCH_LAST_STATUS_UPDATE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'SOURCE_NAME', 'SOURCE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'SOURCE_TYPE_DISPLAY_NAME', 'SOURCE_TYPE_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'LOG_ARCHIVE', 'LOG_ARCHIVE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_TYPE_NAME', 'DS_TYPE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_STATUS', 'DS_STATUS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_STATUS_MESSAGE', 'DS_STATUS_MESSAGE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_UPLOADING_DURATION', 'DS_UPLOADING_DURATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_STAGING_DURATION', 'DS_STAGING_DURATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_LOADING_DURATION', 'DS_LOADING_DURATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_TOTAL_DURATION', 'DS_TOTAL_DURATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_ROWS_PROCESSED', 'DS_ROWS_PROCESSED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_STATUS', 'DS_LAST_STATUS_UPDATE', 'DS_LAST_STATUS_UPDATE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'Rollup Data Set Metadata');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'ROLLUP_DATA_SET_TYPE_KEY', 'ROLLUP_DATA_SET_TYPE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'TYPE_NAME', 'TYPE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'STEREOTYPE', 'STEREOTYPE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'DEST_TABLE', 'DEST_TABLE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'LOAD_SQL', 'LOAD_SQL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'COLUMN_NAME', 'COLUMN_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'LOAD_PROCESSING', 'LOAD_PROCESSING');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_DATA_SET_METADATA', 'SOURCE_EXPRESSION', 'SOURCE_EXPRESSION');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'Enterprise Reporter License Display');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'LICENSE_KEY', 'License Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'LICENSE_TYPE', 'License Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'VERSION', 'Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'CLIENT_ACCESS_LICENSES', 'Client Access Licenses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'DURATION', 'Duration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'EXPIRATION', 'Expiration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'IS_SITE_KEY', 'Is Site Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'IS_EDITION_KEY', 'Is Edition Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_DISPLAY', 'IS_POINT_PRODUCT_KEY', 'Is Point Product Key');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'Rollup Source Latest Success');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'ROLLUP_SOURCE_KEY', 'ROLLUP_SOURCE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'SOURCE_NAME', 'SOURCE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'SOURCE_ENABLED', 'SOURCE_ENABLED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'ROLLUP_SOURCE_TYPE_KEY', 'ROLLUP_SOURCE_TYPE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'SOURCE_TYPE_DISPLAY_NAME', 'SOURCE_TYPE_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'ROLLUP_BATCH_KEY', 'ROLLUP_BATCH_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'START_TIME', 'START_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'START_TIME_ER', 'START_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'COMPLETION_TIME', 'COMPLETION_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'COMPLETION_TIME_ER', 'COMPLETION_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'DURATION', 'DURATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'IS_ROLLUP_NOW', 'IS_ROLLUP_NOW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'IS_COMPLETE', 'IS_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'IS_SUCCESS', 'IS_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'IS_LATEST_COMPLETE', 'IS_LATEST_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'IS_LATEST_SUCCESS', 'IS_LATEST_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'STATUS', 'STATUS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'STATUS_MESSAGE', 'STATUS_MESSAGE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'LAST_STATUS_UPDATE', 'LAST_STATUS_UPDATE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_SUCCESS', 'LOG_ARCHIVE', 'LOG_ARCHIVE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'Business View Supported Dimension');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'DIMENSION', 'DIMENSION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'ALIAS', 'ALIAS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'DIMENSION_DISPLAY_NAME', 'DIMENSION_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'BVIEW_KEY', 'BVIEW_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'BVIEW_NAME', 'BVIEW_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'BVIEW_DISPLAY_NAME', 'BVIEW_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'BVIEW_SUPPORTED_DIMENSION', 'BVIEW_NOTE', 'BVIEW_NOTE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'Rollup Source - Read');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_SOURCE_KEY', 'Roll-up Source Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_SOURCE_TYPE_KEY', 'Roll-up Source Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'UPPER_NAME', 'Upper Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'WEB_SERVER_HOST', 'Web Server Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'WEB_SERVER_PORT', 'Web Server Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'DB_HOST', 'Database Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'DB_PORT', 'Database Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'UTC_ADJUSTMENT', 'UTC Adjustment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_HOUR', 'Roll-up Hour');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_MINUTE', 'Roll-up Minute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_TIME_AS_INTERVAL', 'Roll-up Time as Interval');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_CLOCK', 'Roll-up Clock');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_TIME_REVISION', 'Roll-up Time Revision');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ROLLUP_NOW', 'Roll-up Now');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_READ', 'ENABLED', 'Enabled');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'Rollup Batch - Read');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'ROLLUP_BATCH_KEY', 'Roll-up Batch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'ROLLUP_SOURCE_TYPE_KEY', 'Roll-up Source Type Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'DATES_KEY', 'Dates Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'SESSION_KEY', 'Session Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'SCHEDULE_HASH', 'Schedule Hash');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'ROLLUP_SOURCE_KEY', 'Roll-up Source Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'SOURCE_NAME', 'Source Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'WEB_SERVER_HOST', 'Web Server Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'WEB_SERVER_PORT', 'Web Server Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'DB_HOST', 'Database Host');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'DB_PORT', 'Database Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'LOG_ARCHIVE', 'Log Archive');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'ROLLUP_DAY', 'Roll-up Day');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'START_TIME', 'Start Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'START_TIME_ER', 'Start Time - Enterprise Reporter Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'COMPLETION_TIME', 'Completion Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'COMPLETION_TIME_ER', 'Completion Time - Enterprise Reporter Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'DURATION', 'Duration');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'UTC_ADJUSTMENT', 'UTC Adjustment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_ROLLUP_NOW', 'Is Rollup Now');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_COMPLETE', 'Is Complete');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_SUCCESS', 'Is Success');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_LATEST', 'Is Latest');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_LATEST_COMPLETE', 'Is Latest Complete');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_LATEST_SUCCESS', 'Is Latest Success');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_LAST_OF_MONTH', 'Is Last of Month');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_LAST_OF_QUARTER', 'Is Last of Quarter');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_LAST_OF_YEAR', 'Is Last of Year');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'STATUS_MESSAGE', 'Status Message');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'LAST_STATUS_UPDATE', 'Last Status Update');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_LAST_OF_WEEK', 'Is_Last_Of_Week');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'PERIOD_TYPE', 'PERIOD_TYPE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'VALID_CLUSTER_DATA', 'Valid_Cluster_Data');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'IS_DUMMY', 'Is_Dummy');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_BATCH_READ', 'SHOW_STATUS', 'Show_Status');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ARRAY_CURRENT', 'Array - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'PARENT_ARRAY_KEY', 'Parent Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'ARRAY_OID', 'Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'PARENT_ARRAY_OID', 'Parent Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'VENDOR', 'Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'FAMILY', 'Family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'FIRMWARE_VERSION', 'Firmware Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'SNMP_IP_ADDRESS', 'SNMP IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'NIS_DOMAIN', 'NIS Domain');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'NIS_SERVER', 'NIS Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'IS_AUTO_DISCOVERED', 'Is Auto Discovered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'IS_VIRTUALIZER', 'Is Virtualizer');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'IS_NIS_ENABLED', 'Is NAS NIS Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'VOM_DISCOVER_TYPE', 'VOM Discover type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'IP_ADDRESSES', 'IP Addresses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'CLUSTER_ARRAY_KEY', 'Cluster Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'CLUSTER_ARRAY_NAME', 'Cluster Array Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'CLUSTER_ARRAY_OID', 'Cluster Array Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'FILER_TYPE', 'Filer Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ARRAY_CURRENT', 'PARENT_ARRAY_NAME', 'Parent Array Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'HOST_CURRENT', 'Host - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'HOST_OID', 'Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'OS_DETAILS', 'OS Details');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'OS_TYPE', 'OS Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'IP_ADDRESS', 'IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'VIRTUAL_SERVER', 'Virtual Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'IS_USER_CREATED', 'Is User Created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'IS_VIRTUAL', 'Is Virtual');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'IS_VIRTUAL_SERVER', 'Is Virtual Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'CLUSTER_TYPE', 'Cluster Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'CLUSTER_VERSION', 'Cluster Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'IS_GUEST_INSTALLED', 'Is Guest Installed');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'IS_ACTIVE', 'Is Active');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'CLUSTER_NAME', 'Cluster Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'GUEST_OS_TYPE', 'Guest OS Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'VIRTUALIZATION_TECHNOLOGY', 'Virtualization Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'SF_VERSION', 'SF Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'MH_VERSION', 'MH Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'VOM_LICENSE_REQUIRED', 'VOM License Required');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'VM_HOST_OID', 'VM Host Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'PARENT_VIRTUAL_SERVER_OID', 'Parent Virtual Server OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'HOST_CURRENT', 'CONFIGURATION_TYPE', 'Configuration Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'APP_GROUP_CURRENT', 'Application Group - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CURRENT', 'APP_GROUP_KEY', 'Application Group Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CURRENT', 'APP_GROUP_OID', 'Application Group Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'APP_GROUP_CURRENT', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DATABASES_CURRENT', 'Databases - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'DATABASES_KEY', 'Databases Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'DATABASES_OID', 'Database Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'INSTANCE_DISPLAY_NAME', 'Instance Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'DATABASE_TYPE', 'Database Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'DATABASE_VERSION', 'Database Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'DATABASE_STATE', 'Database State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'IS_RAC_AWARE', 'IS RAC AWARE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DATABASES_CURRENT', 'IS_BASED_ON_DB_VM', 'Is Based on DB VM');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'Filesystem - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'FILESYSTEM_KEY', 'Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'FILESYSTEM_OID', 'Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'VOLUME_KEY', 'Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'MOUNT_POINT', 'Mount Point');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'DEVICE_PATH', 'Device Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_CREATED_FROM_VOLUME', 'Is Created From Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_CLUSTERED', 'Is Clustered');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_VCS_CVM', 'Is VCS CVM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_CFS_AWARE', 'IS CFS AWARE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_CHILD', 'Is Child');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILESYSTEM_CURRENT', 'IS_BASED_ON_NAS_SHARE', 'Is Based on NAS share');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SWITCH_CURRENT', 'Switch - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'SWITCH_KEY', 'Switch Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'SWITCH_OID', 'Switch Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'VENDOR', 'Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'FIRMWARE_VERSION', 'Firmware Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'SNMP_IP_ADDRESS', 'SNMP IP Address(es)');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'SWITCH_ROLE', 'Switch Role');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'FABRIC_NAME', 'Fabric Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'WWN', 'WWN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'IS_PHYSICAL', 'Is Physical');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'HAS_VIRTUAL_SUPPORT', 'Has Virtual Support');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'HAS_MULTIPLEX_SUPPORT', 'Has Multiplex Support');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SWITCH_CURRENT', 'HAS_BUNDLE_SUPPORT', 'Has Bundle Support');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'VOLUME_CURRENT', 'Volume - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'VOLUME_KEY', 'Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'VOLUME_OID', 'Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'DISKGROUP_KEY', 'Diskgroup Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'VOLUME_NAME', 'Volume Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'DISPLAY_NAME', 'Volume Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'VM_VERSION', 'VM Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'VM_TYPE', 'VM Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'DISK_GROUP_NAME', 'Disk Group Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'VOLUME_LAYOUT', 'Volume Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_VCS_CVM', 'Is VCS CVM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_USED_BY_FILESYSTEM', 'Is Used by Filesystem');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_USED_BY_APPLICATION', 'Is Used by Application');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_COMPONENT', 'Is Component');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'VOLUME_TYPE', 'Volume Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'IS_USED_BY_DB_VM', 'Is Used By DB VM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VOLUME_CURRENT', 'DB_VM_NAME', 'DB VM Name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'LUN_CURRENT', 'LUN - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'LUN_KEY', 'LUN Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'LUN_OID', 'LUN Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'ADVISORY_DISPLAY_NAME', 'Advisory Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'DRIVE_TYPE', 'Drive Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'RAID_LEVEL', 'RAID Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'EMULATION_MODE', 'Emulation Mode');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'MASKING_STATUS', 'Masking Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'CLAIMED_STATUS', 'Claimed Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'NUMBER_OF_COPIES', 'Number of Copies');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'PARITY', 'Parity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'RESOURCE_TYPE', 'Resource Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'REVISION', 'Revision');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'STRIPE_WIDTH', 'Stripe Width');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'VENDOR_DEVICE_ID', 'Vendor Device ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'VENDOR_NAME', 'Vendor Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'VENDOR_STATUS', 'Vendor Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'BINDING_STATE', 'Binding State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'COLUMNS', 'Columns');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'TIER_COST_PER_GB', 'Tier Cost per GB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_PRIMARY', 'Is Primary');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_READ_CACHE_ENABLED', 'Is Read Cache Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_SECONDARY', 'Is Secondary');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_SEGMENT', 'Is Segment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_WRITE_CACHE_ENABLED', 'Is Write Cache Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_AGGREGATE', 'Is Aggregate');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_CONTROL_DEVICE', 'Is Control Device');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_FAULT_TOLERANT', 'Is Fault Tolerant');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_ADMINISTRATIVE', 'Is Administrative');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_USED_IN_SNAPSHOT', 'Is Used In Snapshot');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_VIRTUALIZER_SOURCE', 'Is Virtualizer Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_DIRECT_ATTACHED', 'Is Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_BASEDON_EXTERNAL', 'Is Basedon External');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'ARRAY_PORTS', 'Array Ports');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'DISK_VENDOR', 'Disk Vendor');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'DISK_CAPACITY', 'Disk Capacity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'VENDOR_LAYOUT', 'Vendor Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'THIN_STATUS', 'Thin Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'VOM_DISCOVER_TYPE', 'VOM Discover type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_ASSIGNED', 'Is Assigned');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'IS_EXPORTED', 'Is Exported');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'STORAGE_TECHNOLOGY', 'Storage Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'STORAGE_POOL_NAME', 'Storage Pool Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'CONNECTION_PROTOCOL', 'Connection Protocol');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'LUN_CURRENT', 'ARRAY_NAMES', 'Array Names');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'Message Log Summary');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'MESSAGE_LOG_KEY', 'Message Log Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'TIME', 'Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'DURATION_SINCE', 'Duration Since');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'SESSION_KEY', 'Session Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'FUNCTION_NAME', 'Function Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'SEVERITY', 'Severity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'MESSAGE_LOG_SUMMARY', 'MESSAGE', 'MESSAGE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'Rollup Source Time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ROLLUP_SOURCE_KEY', 'ROLLUP_SOURCE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'NAME', 'NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'UTC_ADJUSTMENT', 'UTC_ADJUSTMENT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ROLLUP_HOUR', 'ROLLUP_HOUR');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ROLLUP_MINUTE', 'ROLLUP_MINUTE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ROLLUP_TIME_AS_INTERVAL', 'ROLLUP_TIME_AS_INTERVAL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ROLLUP_CLOCK', 'ROLLUP_CLOCK');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ROLLUP_TIME_REVISION', 'ROLLUP_TIME_REVISION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ROLLUP_NOW', 'ROLLUP_NOW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'ENABLED', 'ENABLED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'CURRENT_UTC_DATETIME', 'CURRENT_UTC_DATETIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'CURRENT_ER_DATETIME', 'CURRENT_ER_DATETIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_TIME', 'CURRENT_SOURCE_DATETIME', 'CURRENT_SOURCE_DATETIME');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'Rollup Schedule Datetime');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'ROLLUP_SOURCE_KEY', 'ROLLUP_SOURCE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'NAME', 'NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'UTC_ADJUSTMENT', 'UTC_ADJUSTMENT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'ROLLUP_HOUR', 'ROLLUP_HOUR');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'ROLLUP_MINUTE', 'ROLLUP_MINUTE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'ROLLUP_TIME_AS_INTERVAL', 'ROLLUP_TIME_AS_INTERVAL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'ROLLUP_CLOCK', 'ROLLUP_CLOCK');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'ROLLUP_TIME_REVISION', 'ROLLUP_TIME_REVISION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'SCHEDULE_DATE', 'SCHEDULE_DATE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'SCHEDULE_DATETIME', 'SCHEDULE_DATETIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'SCHEDULE_UTC_DATETIME', 'SCHEDULE_UTC_DATETIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'SCHEDULE_ER_DATETIME', 'SCHEDULE_ER_DATETIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'SCHEDULE_SOURCE_DATETIME', 'SCHEDULE_SOURCE_DATETIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SCHEDULE_DATETIME', 'SCHEDULE_HASH', 'SCHEDULE_HASH');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'TABLE_METADATA', 'Table Metadata');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA', 'OWNER', 'OWNER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA', 'TABLE_NAME', 'TABLE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA', 'FULL_TABLE_NAME', 'FULL_TABLE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA', 'DISPLAY_NAME', 'DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA', 'TABLE_TYPE', 'TABLE_TYPE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'TABLE_METADATA', 'REMARKS', 'REMARKS');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'COLUMN_METADATA', 'Column Metadata');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA', 'OWNER', 'OWNER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA', 'TABLE_NAME', 'TABLE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA', 'FULL_TABLE_NAME', 'FULL_TABLE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA', 'COLUMN_NAME', 'COLUMN_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA', 'DISPLAY_NAME', 'DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'COLUMN_METADATA', 'REMARKS', 'REMARKS');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ER_LICENSE_USAGE', 'Enterprise Reporter License Usage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_USAGE', 'VALID_CALS', 'Valid Client Access Licenses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_USAGE', 'EXPIRED_CALS', 'Expired Client Access Licenses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_USAGE', 'USED_CALS', 'Used Client Access Licenses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_USAGE', 'UNUSED_CALS', 'Unused Client Access Licenses');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ER_LICENSE_USAGE', 'EXCEEDED_CALS', 'Exceeded Client Access Licenses');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SRC_CUSTOM_ATTRIBUTE', 'Source Custom Attribute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SRC_CUSTOM_ATTRIBUTE', 'CUSTOM_ATTRIBUTE_CONTEXT', 'CUSTOM_ATTRIBUTE_CONTEXT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SRC_CUSTOM_ATTRIBUTE', 'NAME', 'NAME');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'UNUSED_SRC_CUSTOM_ATTRIBUTE', 'Unused Source Custom Attribute');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'UNUSED_SRC_CUSTOM_ATTRIBUTE', 'CUSTOM_ATTRIBUTE_CONTEXT', 'CUSTOM_ATTRIBUTE_CONTEXT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'UNUSED_SRC_CUSTOM_ATTRIBUTE', 'NAME', 'NAME');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'NetApp Volume - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'NETAPP_VOLUME_OID', 'NetApp Volume Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'VENDOR_STATE', 'Vendor State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'VENDOR_LAYOUT', 'Vendor Layout');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'RAID_LEVEL', 'RAID Level');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'IS_AGGREGATE', 'Is Aggregate');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'IS_SEGMENT', 'Is Segment');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'IS_VFILER_VOLUME', 'Is VFiler Volume');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'SUB_TYPE', 'Sub Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'RPM', 'RPM');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'DRIVE_TYPE', 'Drive Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'DISK_SIZE', 'Disk Size');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'IS_NAS_FS_CREATED', 'Is NAS FS Created');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'DEDUP_STATE', 'Dedup State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'LAST_DEDUP_OP_BEGIN_TIME', 'Last Dedup Op begin time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'LAST_DEDUP_OP_END_TIME', 'Last Dedup Op end time');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'STORAGE_POOL_KEY', 'Storage Pool Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_VOLUME_CURRENT', 'STORAGE_POOL_OBJECT_IDENTIFIER', 'Storage Pool Object Identifier');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'Array Storage Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'ARRAY_OID', 'ARRAY_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'TOTAL_PHYSICAL', 'TOTAL_PHYSICAL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'V_PHYSICAL_EXTERNAL', 'V_PHYSICAL_EXTERNAL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'LOCAL_LOGICAL', 'LOCAL_LOGICAL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'V_VIRTUAL_LOGICAL', 'V_VIRTUAL_LOGICAL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'LOGICAL_AVAILABLE', 'LOGICAL_AVAILABLE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'ALLOCATED', 'ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'A_ALLOCATED_TO_HOSTS_DIRECTLY', 'A_ALLOCATED_TO_HOSTS_DIRECTLY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'A_ALLOCATED_TO_VIRTUALIZER', 'A_ALLOCATED_TO_VIRTUALIZER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'A_ALLOC_HOST_VIA_VIRTUALIZER', 'A_ALLOC_HOST_VIA_VIRTUALIZER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'V_LOCAL_LOGICAL_ALLOCATED', 'V_LOCAL_LOGICAL_ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'V_VIRTUAL_LOGICAL_ALLOCATED', 'V_VIRTUAL_LOGICAL_ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'UNALLOCATED', 'UNALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'A_UNALLOCATED_VIA_VIRUALIZER', 'A_UNALLOCATED_VIA_VIRUALIZER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'V_LOCAL_LOGICAL_UNALLOCATED', 'V_LOCAL_LOGICAL_UNALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'V_VIRTUAL_LOGICAL_UNALLOCATED', 'V_VIRTUAL_LOGICAL_UNALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'CLAIMED_BY_HOSTS', 'CLAIMED_BY_HOSTS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'REPLICATION', 'REPLICATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'RAID_OVERHEAD', 'RAID_OVERHEAD');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'ADMINISTRATIVE', 'ADMINISTRATIVE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'SNAPSHOT_RESERVE', 'SNAPSHOT_RESERVE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'WAFL_SPARE', 'WAFL_SPARE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'DAS_CAPACITY', 'DAS_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NAS_USED_CAPACITY', 'NAS_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'UNCONFIGURED_CAPACITY', 'UNCONFIGURED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'INTERNAL_PHYSICAL', 'INTERNAL_PHYSICAL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'LOGICAL_UNKNOWN', 'LOGICAL_UNKNOWN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'TOTAL_UNCLAIMED', 'TOTAL_UNCLAIMED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'UNDISCOVERED_ALLOCATED', 'UNDISCOVERED_ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'PERCENT_ALLOCATED_HOST_CLAIMED', 'PERCENT_ALLOCATED_HOST_CLAIMED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'PERCENT_CONFIGURED', 'PERCENT_CONFIGURED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'PERCENT_CONFIGURED_ALLOCATED', 'PERCENT_CONFIGURED_ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'PERCENT_CONFIGURED_AVAILABLE', 'PERCENT_CONFIGURED_AVAILABLE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'PERCENT_CONFIGURED_OVERHEAD', 'PERCENT_CONFIGURED_OVERHEAD');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'PERCENT_CONFIGURED_UNDISCOVERD', 'PERCENT_CONFIGURED_UNDISCOVERD');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'PERCENT_UNCONFIGURED', 'PERCENT_UNCONFIGURED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'CONSUMPTION_ASSIGNED', 'CONSUMPTION_ASSIGNED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'CONSUMPTION_UNKNOWN', 'CONSUMPTION_UNKNOWN');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'HOST_USAGE_APPS_ON_RAW', 'HOST_USAGE_APPS_ON_RAW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'HOST_USAGE_FS_UN_USED', 'HOST_USAGE_FS_UN_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'HOST_USAGE_FS_USED', 'HOST_USAGE_FS_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'HOST_USAGE_FS_AUSED_APPS', 'HOST_USAGE_FS_AUSED_APPS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'HOST_USAGE_FS_NAUSED_NON_APPS', 'HOST_USAGE_FS_NAUSED_NON_APPS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'HOST_USAGE_OTHER', 'HOST_USAGE_OTHER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'APPL_USED_CAPACITY', 'APPL_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'APPL_UN_USED_CAPACITY', 'APPL_UN_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'THINAWARE_CLAIMED_BY_HOST', 'THINAWARE_CLAIMED_BY_HOST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'THINAWARE_TOTAL_UNCLAIMED', 'THINAWARE_TOTAL_UNCLAIMED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'THINAWARE_ALLOCATED_TOHOST', 'THINAWARE_ALLOCATED_TOHOST');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'THINAWARE_ALC_THGVIRTUALIZER', 'THINAWARE_ALC_THGVIRTUALIZER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'THINAWARE_UNDISC_ALLOCATED', 'THINAWARE_UNDISC_ALLOCATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_VOLUME_CAPACITY', 'NA_VOLUME_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_VOLUME_USED_CAPACITY', 'NA_VOLUME_USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_VOLUME_AVAILABLE_CAPACITY', 'NA_VOLUME_AVAILABLE_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_QTREES', 'NA_QTREES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_SHARES', 'NA_SHARES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_VOLUMES', 'NA_VOLUMES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_FILESYSTEMS', 'NA_FILESYSTEMS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_CONSUMPTION', 'NA_CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_FILES', 'NA_FILES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_FOLDERS', 'NA_FOLDERS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NA_USERS', 'NA_USERS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'SNAPSHOT_USED', 'SNAPSHOT_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'OVERWRITE_RESERVE', 'OVERWRITE_RESERVE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NETAPP_REDUNDANT_CAPACITY', 'NETAPP_REDUNDANT_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'NUMBER_OF_STORAGE_POOLS', 'NUMBER_OF_STORAGE_POOLS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'ARRAY_STORAGE_', 'WAFL_SPARE_NEW', 'WAFL_SPARE_NEW');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'NetApp Share - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'NETAPP_SHARE_KEY', 'NetApp Share Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'NETAPP_SHARE_OID', 'NetApp Share Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'NETAPP_QTREE_KEY', 'NetApp QTree Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'SHARE_PATH', 'Share Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'NFS_EXPORT_PATH', 'NFS Export Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'CIFS_MOUNT_POINT', 'CIFS Mount Point');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'CIFS_SHARE_NAME', 'CIFS Share Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'MOUNT_POINT_TYPE', 'Mount Point Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'ESX_DATA_STORE_NAMES', 'ESX Data Store names');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'HOST_PRIVILEGE', 'Host Privilege');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_SHARE_CURRENT', 'IS_MAPPED_TO_ANY_UNDISCOV_HOST', 'Is Mapped to Any Undiscov Host');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'NetApp QTree - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'NETAPP_QTREE_KEY', 'NetApp QTree Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'NETAPP_QTREE_OID', 'NetApp QTree Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'QTREE_PATH', 'QTree Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'SECURITY_STYLE', 'Security Style');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'FILESYSTEM_ID', 'Filesystem ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NETAPP_QTREE_CURRENT', 'IS_QUOTA_ENABLED', 'Is Quota Enabled');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'NetApp Volume File Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'NETAPP_VOLUME_OID', 'NETAPP_VOLUME_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'CAPACITY', 'CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'USED_CAPACITY', 'USED_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'AVAILABLE_CAPACITY', 'AVAILABLE_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'QTREES', 'QTREES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'TOTAL_CONSUMPTION', 'TOTAL_CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'FILES', 'FILES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'FOLDERS', 'FOLDERS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'USERS', 'USERS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'PHYSICAL_CAPACITY', 'PHYSICAL_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'OVERHEAD_CAPACITY', 'OVERHEAD_CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'PHYSICAL_USED', 'PHYSICAL_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'NUMBER_OF_DATA_DISKS', 'NUMBER_OF_DATA_DISKS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'NUMBER_OF_PARITY_DISKS', 'NUMBER_OF_PARITY_DISKS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'NUMBER_OF_VOLS_CREATED', 'NUMBER_OF_VOLS_CREATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'NUMBER_OF_LUNS_CREATED', 'NUMBER_OF_LUNS_CREATED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'NUMBER_OF_TARGETS', 'NUMBER_OF_TARGETS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'SPACE_SHARED', 'SPACE_SHARED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'SPACE_SAVED', 'SPACE_SAVED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'EFFECTIVE_SPACE_USED', 'EFFECTIVE_SPACE_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'PHYSICAL_SPACE_SHARED', 'PHYSICAL_SPACE_SHARED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'PHYSICAL_SPACE_SAVED', 'PHYSICAL_SPACE_SAVED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'PHYSICAL_EFFECTIVE_SPACE_USED', 'PHYSICAL_EFFECTIVE_SPACE_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'SUBSCRIBED_PERCENT', 'SUBSCRIBED_PERCENT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_', 'SPACE_SAVED_PERCENT', 'SPACE_SAVED_PERCENT');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'NetApp QTree File Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'NETAPP_QTREE_OID', 'NETAPP_QTREE_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'CAPACITY', 'CAPACITY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'QUOTA_LIMIT', 'QUOTA_LIMIT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'QUOTA_USED', 'QUOTA_USED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'TOTAL_CONSUMPTION', 'TOTAL_CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'FILES', 'FILES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'FOLDERS', 'FOLDERS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_', 'USERS', 'USERS');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'NetApp Share File Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'NETAPP_SHARE_OID', 'NETAPP_SHARE_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'TOTAL_CONSUMPTION', 'TOTAL_CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'FILES', 'FILES');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'FOLDERS', 'FOLDERS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'USERS', 'USERS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_', 'NO_OF_HOSTS', 'NO_OF_HOSTS');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'FILE_CATEGORY_CURRENT', 'File Category - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY_CURRENT', 'FILE_CATEGORY_KEY', 'File Category Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY_CURRENT', 'FILE_CATEGORY_OID', 'File Category Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY_CURRENT', 'DESCRIPTION', 'Description');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'FILE_CATEGORY_CURRENT', 'FILE_EXTENSIONS', 'File Extensions');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'Rollup Source Latest Batch');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'ROLLUP_SOURCE_KEY', 'ROLLUP_SOURCE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'SOURCE_NAME', 'SOURCE_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'SOURCE_ENABLED', 'SOURCE_ENABLED');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'ROLLUP_SOURCE_TYPE_KEY', 'ROLLUP_SOURCE_TYPE_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'SOURCE_TYPE_DISPLAY_NAME', 'SOURCE_TYPE_DISPLAY_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'ROLLUP_BATCH_KEY', 'ROLLUP_BATCH_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'START_TIME', 'START_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'START_TIME_ER', 'START_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'COMPLETION_TIME', 'COMPLETION_TIME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'COMPLETION_TIME_ER', 'COMPLETION_TIME_ER');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'DURATION', 'DURATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'IS_ROLLUP_NOW', 'IS_ROLLUP_NOW');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'IS_COMPLETE', 'IS_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'IS_SUCCESS', 'IS_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'IS_LATEST_COMPLETE', 'IS_LATEST_COMPLETE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'IS_LATEST_SUCCESS', 'IS_LATEST_SUCCESS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'STATUS', 'STATUS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'STATUS_MESSAGE', 'STATUS_MESSAGE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'LAST_STATUS_UPDATE', 'LAST_STATUS_UPDATE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'ROLLUP_SOURCE_LATEST_BATCH', 'LOG_ARCHIVE', 'LOG_ARCHIVE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_', 'NetApp QTree File By File Category Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_', 'NETAPP_QTREE_OID', 'NETAPP_QTREE_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_', 'FILE_CATEGORY_OID', 'FILE_CATEGORY_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_', 'CONSUMPTION', 'CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_QTREE_FILE_FCAT_', 'FILES', 'FILES');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_', 'NetApp Volume File By File Category Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_', 'NETAPP_VOLUME_OID', 'NETAPP_VOLUME_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_', 'FILE_CATEGORY_OID', 'FILE_CATEGORY_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_', 'CONSUMPTION', 'CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_VOLUME_FILE_FCAT_', 'FILES', 'FILES');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_', 'NetApp Share File By File Category Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_', 'NETAPP_SHARE_OID', 'NETAPP_SHARE_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_', 'FILE_CATEGORY_OID', 'FILE_CATEGORY_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_', 'CONSUMPTION', 'CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_SHARE_FILE_FCAT_', 'FILES', 'FILES');
call SAHARA.SP_DataDict_TableMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_', 'NetApp File By File Category Stage');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_', 'ARRAY_OID', 'ARRAY_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_', 'FILE_CATEGORY_OID', 'FILE_CATEGORY_OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_', 'DM_BATCH_ID', 'DM_BATCH_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_', 'CONSUMPTION', 'CONSUMPTION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA_APP', 'NETAPP_FILE_FCAT_', 'FILES', 'FILES');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'NAS Filesystem - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'NAS_FILESYSTEM_OID', 'NAS Filesystem Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'NETAPP_VOLUME_KEY', 'NetApp Volume Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'DISCOVERY_STATE', 'Discovery State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'SECURITY_STYLE', 'Security Style');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'FILESYSTEM_ID', 'Filesystem ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'IS_QUOTA_ENABLED', 'Is Quota Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'TIER', 'Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'PATH', 'Path');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'AUTO_EXTEND_ENABLED', 'Auto extend enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'IS_LOCK_ENABLED', 'Is Lock Enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'CWORM_STATE', 'CWORM State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'IS_VIRTUAL_PROVISIONING_ENABLE', 'Is Virtual Provisioning enabled');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'IS_SOURCE', 'Is Source');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'IS_TARGET', 'Is Target');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'REPLICATION_TECHNOLOGY', 'Replication Technology');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'AUTO_EXTEND_MAX_SIZE', 'Auto extend Max size');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_FILESYSTEM_CURRENT', 'HIGH_WATERMARK', 'High watermark');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'SF License Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'SF_LICENSE_KEY', 'SF License Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'SF_LICENSE_OID', 'SF License OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'KEY', 'Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'IS_SITE_LICENSE', 'Is Site License');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'SERIAL_NUMBER', 'Serial Number');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'IS_EXPIRED', 'Is Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_CURRENT', 'IS_KEYLESS', 'Is Keyless');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'SF Product Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'SF_PRODUCT_KEY', 'SF Product Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'SF_PRODUCT_OID', 'SF Product OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'PRODUCT_NAME', 'Product Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'SHORT_NAME', 'Short Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'PRODUCT_CATEGORY', 'Product Category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'PRODUCT_EDITION', 'Product Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'PRODUCT_VERSION', 'Product Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'PLATFORM', 'Platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_PRODUCT_CURRENT', 'LICENSE_TYPE', 'License Type');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'SF License Host Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'SF_LICENSE_HOST_KEY', 'SF License Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'SF_LICENSE_HOST_OID', 'SF License Host OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'ID', 'Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'MODEL', 'Model');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'PROCESSOR_TYPE', 'Processor Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'NUM_PROCESSORS', 'Num Processors');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'PLATFORM', 'Platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'SERVER_PRICE_TIER', 'Server Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'PROCESSOR_PRICE_TIER', 'Processor Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'NUM_PROCESSOR_PRICE_TIER', 'Num Processor Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'OS_PRICE_TIER', 'OS Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'ASSIGNED_PRICE_TIER', 'Assigned Price Tier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'OS_EDITION', 'OS Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'OS', 'OS');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'HAS_ENTERPRISE_EDITION', 'Has Enterprise Edition');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'SPVU', 'SPVU');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_HOST_CURRENT', 'NCORE', 'NCORE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'SF License Deployment Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'SF_LICENSE_DEPLOYMENT_KEY', 'SF License Deployment Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'SF_LICENSE_DEPLOYMENT_OID', 'SF License Deployment OID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'SF_LICENSE_KEY', 'SF License Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'SF_PRODUCT_KEY', 'SF Product Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'SF_LICENSE_HOST_KEY', 'SF License Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'EXPIRY_DATE', 'Expiry Date');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'IS_EXPIRED', 'Is Expired');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SF_LICENSE_DEPLOYMENT_CURRENT', 'IS_ACCOUNTABLE', 'Is Accountable');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'DISKGROUP_CURRENT', 'Diskgroup - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'DISKGROUP_KEY', 'Diskgroup Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'DISKGROUP_OID', 'Diskgroup Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'HOST_KEY', 'Host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'FAMILY', 'Family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'VERSION', 'Version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'IS_SHARED', 'Is Shared');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'IS_BASED_ON_DIRECT_ATTACHED', 'Is Based On Direct Attached');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'IS_BASED_ON_THIN', 'Is Based On Thin');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'STATE', 'State');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'APPLICATION_BE', 'Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Composite Application Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'ORGANIZATION_BE', 'Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Composite Organization Business Entity');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'REDUNDANCY_TYPE', 'Redundancy Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'IS_USED_BY_CLUSTERED_DB', 'Is Used By Clustered DB');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'DISKGROUP_CURRENT', 'USES_VM_VOLUMES', 'Uses VM Volumes');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'VIEW_BROKER_INFO', 'Broker Info View');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'BROKER_ID', 'Broker ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'DOMAIN_NAME', 'Domain Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'DOMAIN_TYPE', 'Domain Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'BROKER_NAME', 'Broker Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'BROKER_TYPE', 'Broker Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'BROKER_PORT', 'Broker Port');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'STATUS', 'Status');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'LDAP_SERVER', 'LDAP Server');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'BIND_USER_NAME', 'Bind User Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'BIND_USER_PASSWORD', 'Bind User Password');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'SEARCH_USER', 'Search User');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'SEARCH_GROUP', 'Search Group');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'SEARCH_BASE', 'Search Base');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_BROKER_INFO', 'SSL_CERT_LOCATION', 'SSL Cert Location');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'VIEW_SECURITY_ROLE', 'Security Role View');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_ROLE', 'ROLE_ID', 'Role Id');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_ROLE', 'NAME', 'Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_ROLE', 'DESCRIPT', 'Descript');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'Security Group View');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'SECURITY_GROUP_ID', 'SECURITY_GROUP_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'SG_NAME', 'SG_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'PRIVILEGE', 'PRIVILEGE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'SG_DESCRIPT', 'SG_DESCRIPT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'BROKER_ID', 'BROKER_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'ROLE_ID', 'ROLE_ID');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'BROKER_NAME', 'BROKER_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'DOMAIN_TYPE', 'DOMAIN_TYPE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'DOMAIN_NAME', 'DOMAIN_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'SR_NAME', 'SR_NAME');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'VIEW_SECURITY_GROUP', 'SR_DESCRIPT', 'SR_DESCRIPT');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'NAS Quota - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'NAS_QUOTA_KEY', 'NAS Quota Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'NAS_QUOTA_OID', 'NAS Quota Object Identifier');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'ARRAY_KEY', 'Array Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'NAS_FILESYSTEM_KEY', 'NAS Filesystem Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'DISPLAY_NAME', 'Display Name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'TYPE', 'Type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'NAS_QUOTA_CURRENT', 'TARGET', 'Target');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_CURRENT', 'Cluster - Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'CLUSTER_TYPE', 'cluster_type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'CLUSTER_CATEGORY', 'cluster_category');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'VERSION', 'version');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'PLATFORM', 'platform');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'USE_FENCE', 'use_fence');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'IS_GLOBAL', 'is_global');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'SECURECLUS', 'secureclus');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'REMOTECLUSTERS', 'remoteclusters');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'APPLICATION_BE', 'Application_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Application_Composite_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'ORGANIZATION_BE', 'Organization_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Organization_Composite_BE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'Servicegroup Current');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'SERVICEGROUP_KEY', 'servicegroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'SG_TYPE', 'sg_type');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'SCOPE', 'scope');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'FAILOVERPOLICY', 'failoverpolicy');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'IS_APPHA', 'is_AppHA');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'FAMILY', 'family');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'APPLICATION_BE', 'Application_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'APPLICATION_COMPOSITE_BE', 'Application_Composite_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'ORGANIZATION_BE', 'Organization_BE');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_CURRENT', 'ORGANIZATION_COMPOSITE_BE', 'Organization_Composite_BE');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_RCLUSTER_RELATION', 'CLUSTER_RCLUSTER_RELATION');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_RCLUSTER_RELATION', 'CLUSTER_KEY', 'CLUSTER_KEY');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_RCLUSTER_RELATION', 'REMOTECLUSTER_KEY', 'REMOTECLUSTER_KEY');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'CLUSTER_HOST_CURRENT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'CLUSTER_HOST_KEY', 'cluster_host_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'HOST_KEY', 'host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'HOST_OID', 'host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_HOST_CURRENT', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER_CURRENT', 'CLUSTER_REMOTECLUSTER_CURRENT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER_CURRENT', 'CLUSTER_REMOTECLUSTER_KEY', 'cluster_remotecluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER_CURRENT', 'CLUSTER_KEY', 'cluster_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER_CURRENT', 'CLUSTER_OID', 'cluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER_CURRENT', 'REMOTECLUSTER_OID', 'remotecluster_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'CLUSTER_REMOTECLUSTER_CURRENT', 'DISPLAY_NAME', 'display_name');
call SAHARA.SP_DataDict_TableMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'SERVICEGROUP_HOST_CURRENT');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'SERVICEGROUP_HOST_KEY', 'servicegroup_host_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'SERVICEGROUP_KEY', 'servicegroup_key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'HOST_KEY', 'host Key');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'SERVICEGROUP_OID', 'servicegroup_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'HOST_OID', 'host_oid');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'CC_STORAGE_URL', 'CC Storage URL');
call SAHARA.SP_DataDict_ColumnMetadata('SAHARA', 'SERVICEGROUP_HOST_CURRENT', 'DISPLAY_NAME', 'display_name');

--
-- NOTE: DO NOT MODIFY THIS FILE.  This file is generated automatically.
--       Any changes will be lost when the file is re-generated.
--
IF EXISTS (SELECT 1 FROM sys.sysprocedure
        WHERE proc_name = 'SP_GrantPermForTables' and user_name(creator) = 'SAHARA')
THEN
        DROP PROCEDURE sahara.SP_GrantPermForTables;
END IF;
CREATE PROCEDURE sahara.SP_GrantPermForTables()
BEGIN
declare @vSQL long varchar;
    for grantcursor as grantloop cursor for select t.table_name as saharatablename from systable t, sysuser u where t.creator = u.user_id and u.user_name = 'SAHARA' and t.table_type in ('BASE','VIEW')
       do
        message  saharatablename to client;
        set @vSQL = 'grant select on SAHARA."'||saharatablename ||'" to SAHARA_REPORTS;';
        
        execute immediate @vSQL;
    end for;  
END;

call sahara.SP_BVIEW_AddRuleKey();

-- Granting the permission for the tables and views;
call sahara.SP_GrantPermForTables();


grant execute on sahara.fn_Session_BeginServerSession to sahara_app;

if not exists (select 1 from sahara.db_version)
then
   insert into SAHARA.DB_VERSION(MAJOR, MINOR, MP_VERSION) on existing update values (5,0,0);
end if;

if exists (select 1 from sys.syscolumns where tname='COLUMN_METADATA_BASE' and cname='AGGREGATE_TYPE') then
   update SAHARA.COLUMN_METADATA_BASE set AGGREGATE_TYPE = 'sum'
   where table_name = 'SERVICEGROUP_HIST_EVENTS' and column_name not like '%_KEY';
end if;

commit;
------------------------------------------------------------------------------------------------------------------
-- procedures that implements loading data into production schema
------------------------------------------------------------------------------------------------------------------

--
-- helper procedure for initializing variables for the batch
--
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataLoad_Initialize_' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataLoad_Initialize_
end if;

CREATE PROCEDURE "SAHARA"."SP_DataLoad_Initialize_" ()
BEGIN

	-- private: variables global to the DataLoad procedures
   if  ( VAREXISTS('@WebServerHost_') = 0 ) then
	CREATE VARIABLE @WebServerHost_ varchar(128);
   end if;
   if  ( VAREXISTS('@WebServerPort_') = 0 ) then
	CREATE VARIABLE @WebServerPort_ numeric(5);
   end if;
  if  ( VAREXISTS('@DBHost_') = 0 ) then
	CREATE VARIABLE @DBHost_ varchar(128);
   end if;
  if  ( VAREXISTS('@URL1') = 0 ) then
    CREATE VARIABLE @URL1 varchar(1024);
   end if;
  if  ( VAREXISTS('@URL2') = 0 ) then
    CREATE VARIABLE @URL2 varchar(1024);
   end if;

    select WEB_SERVER_HOST, WEB_SERVER_PORT, DB_HOST
    into @WebServerHost_, @WebServerPort_, @DBHost_
    from sahara.ROLLUP_BATCH
    where ROLLUP_BATCH_KEY = @gLoadActiveBatchKey;

    call sahara.SP_CUST_ATTR_InitializeLoad();  
	set @URL1 = 'https://' || @WebServerHost_ || ':' || @WebServerPort_ || '/sm/Redirect?action=getlink&Profile=CentralExtension&ContextInfo=';
    set @URL2 = '&host=' || @DBHost_;

END;

grant execute on sahara.SP_DataLoad_Initialize_ to sahara;

--
-- procedure that implements loading the steps
--
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataLoad_Load_' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataLoad_Load_
end if;
CREATE PROCEDURE "SAHARA"."SP_DataLoad_Load_" ()
--CREATE PROCEDURE "SAHARA"."SP_DataLoad_Load_" (@pOwner varchar(32), @pTableName varchar(128), @pDisplayName varchar(128))
BEGIN
	-- tbd - We may need to add roll-back logic here if transactions get
    -- too large.  We want the dimensions processed in one transaction
    -- because we can't implement a "manual" rollback like we can for
    -- facts.

    -- We keep the data source type-specific procedure as simple as
    -- possible.
    --
    -- Logging, status, and transaction management are handled by the
    -- calling and implementation procedures.

    -- load ETL logs in production database generated from VOM MS extraction process
    call sahara.SP_DataLoad_LoadDataSet('ETL_LOG_DIM');

    -- process custom attributes first so they can be used in rules
    call sahara.SP_DataLoad_LoadDataSet('CUSTOM_ATTR');

    -- execute rules that work on the staging tables
    if Fn_Load_ActiveBatchSupportsDataSet('LUN_DIM')=1 then
        call SP_TierRule_Execute('LUN');
    end if;

    -- dimensions (the order matters)
--  call sahara.SP_DataLoad_LoadDataSet('DM_BATCH');
    call sahara.SP_DataLoad_LoadDataSet('ARRAY_DIM');
    call sahara.SP_DataLoad_LoadDataSet('ARRAY_DIM_VFILER');
    call sahara.SP_DataLoad_LoadDataSet('ARRAY_DIM_CLUSTNODE');
    call sahara.SP_DataLoad_LoadDataSet('HOST_DIM');
 
    call sahara.SP_DataLoad_LoadDataSet('NETAPP_VOLUME_DIM');
    call sahara.SP_DataLoad_LoadDataSet('NETAPP_VOLUME_DIM2');	
    call sahara.SP_DataLoad_LoadDataSet('NAS_FS_DIM');

    call sahara.SP_DataLoad_LoadDataSet('LUN_DIM');

    call sahara.SP_DataLoad_LoadDataSet('SF_LICENSE_DIM');
    call sahara.SP_DataLoad_LoadDataSet('SF_PRODUCT_DIM');
    call sahara.SP_DataLoad_LoadDataSet('SF_LICENSE_HOST_DIM');
    call sahara.SP_DataLoad_LoadDataSet('SF_LICENSE_DEPLOYMENT_DIM');

--    call sahara.SP_DataLoad_LoadDataSet('LUN_DUP_DIM');
    call sahara.SP_DataLoad_LoadDataSet('DG_DIM');
    call sahara.SP_DataLoad_LoadDataSet('VOLUME_DIM');
    call sahara.SP_DataLoad_LoadDataSet('FS_DIM');
    call sahara.SP_DataLoad_LoadDataSet('DB_DIM');
--    call sahara.SP_DataLoad_LoadDataSet('APP_GROUP_DIM');
--    call sahara.SP_DataLoad_LoadDataSet('SWITCH_DIM');
--    call sahara.SP_DataLoad_LoadDataSet('FILE_CATEGORY_DIM');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_VOLUME_DIM');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_VOLUME_DIM2');	
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_QTREE_DIM');
--    call sahara.SP_DataLoad_LoadDataSet('NAS_FS_DIM');
    call sahara.SP_DataLoad_LoadDataSet('NETAPP_SHARE_DIM');
    call sahara.SP_DataLoad_LoadDataSet('NAS_QUOTA_DIM');
    call sahara.SP_DataLoad_LoadDataSet('CLUSTER_DIM');
    call sahara.SP_DataLoad_LoadDataSet('SERVICEGROUP_DIM');

    --  pseudo facts
--    call sahara.SP_DataLoad_LoadDataSet('APP_GROUP_HOST');
    call sahara.SP_DataLoad_LoadDataSet('LUN_HOST');
    call sahara.SP_DataLoad_LoadDataSet('LUN_FILESYSTEM');
    call sahara.SP_DataLoad_LoadDataSet('LUN_DISKGROUP');
    call sahara.SP_DataLoad_LoadDataSet('LUN_VOLUME');

	-- many to many join table
	call sahara.SP_DataLoad_LoadDataSet('NAS_VOLUME_FILER');
	call sahara.SP_DataLoad_LoadDataSet('NAS_SHARE_HOST');
	call sahara.SP_DataLoad_LoadDataSet('NAS_SHARE_HOSTFS');

    -- facts
--    call sahara.SP_DataLoad_LoadDataSet('APP_GROUP_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('ARRAY_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('ARRAY_DUP_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('DB_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('FS_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('HOST_ARRAY_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('HOST_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('LUN_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('SF_DEPLOYED_LICENSES_FACTS');    

--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_DM_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_FILE_CATEGORY_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_HAL_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_QTREE_FILE_CATEGORY_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_QTREE_FILE_DM_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_QTREE_FILE_HAL_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('NAS_FS_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_SHARE_FILE_CATEGORY_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_SHARE_FILE_DM_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('NETAPP_SHARE_FILE_HAL_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_VOLUME_FILE_CATEGORY_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('NETAPP_VOLUME_FILE_DM_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('NETAPP_VOLUME_FILE_HAL_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('NAS_QUOTA_FACTS');
--    call sahara.SP_DataLoad_LoadDataSet('SWITCH_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('DG_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('VOLUME_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('CLUSTER_FACTS');
    call sahara.SP_DataLoad_LoadDataSet('SERVICEGROUP_FACTS');    
    call sahara.SP_DataLoad_LoadDataSet('CLUSTER_REMOTECLUSTER_DIM');
    call sahara.SP_DataLoad_LoadDataSet('CLUSTER_HOST_DIM');
    call sahara.SP_DataLoad_LoadDataSet('SERVICEGROUP_HOST_DIM');    
    call sahara.SP_DataLoad_LoadDataSet('SERVICEGROUP_HIST_EVENTS');    

   -- Execute the business view rules to dynamically assign the storage assets  to the different business view nodes based on defined rules.  
   -- The current code goes through all business view and execute the rules associated with each of the rule. Optimization may be required
   -- based on scale testing.
   call  sahara.SP_BVIEW_AutoRunRules();

END;

grant execute on sahara.SP_DataLoad_Load_ to sahara;

--
-- procedure that implements loading of individual dataset
--

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataLoad_LoadDataSet' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataLoad_LoadDataSet
end if;

CREATE PROCEDURE "SAHARA"."SP_DataLoad_LoadDataSet" (@pDataSetName varchar(64))
BEGIN
declare @FUNC varchar(32);
       set @FUNC = 'SP_DataLoad_LoadDataSet';
       call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'entry');
	if fn_Load_ActiveBatchSupportsDataSet(@pDataSetName)=1 then
        --call SP_Log_LogDebug(@gLoadActiveSessionKey, @FUNC, 'Insided the loop'|| @pDataSetName);
	    call sahara.sp_Load_LoadDataSet(@pDataSetName);
    end if;
END;

grant execute on sahara.SP_DataLoad_LoadDataSet to sahara;


--
-- procedure to implement the "custom" load for the custom attribute data set type
--
if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataLoad_LoadCustomAttributes' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataLoad_LoadCustomAttributes
end if;

CREATE PROCEDURE "SAHARA"."SP_DataLoad_LoadCustomAttributes" ()
BEGIN
    -- call sahara.SP_CUST_ATTR_Load('APP_GROUP');
    call sahara.SP_CUST_ATTR_Load('ARRAY');
    call sahara.SP_CUST_ATTR_Load('DATABASES');
    call sahara.SP_CUST_ATTR_Load('FILESYSTEM');
    call sahara.SP_CUST_ATTR_Load('HOST');
    call sahara.SP_CUST_ATTR_Load('LUN');
    -- call sahara.SP_CUST_ATTR_Load('SWITCH');
    call sahara.SP_CUST_ATTR_Load('VOLUME');
    call sahara.SP_CUST_ATTR_Load('DISKGROUP');
    call sahara.SP_CUST_ATTR_Load('CLUSTER');
    call sahara.SP_CUST_ATTR_Load('SERVICEGROUP');    

	-- Auto Load Business Entity as Custom Attributes 
	-- call sahara.SP_CUST_ATTR_BELoad();

    -- if fn_LOADIMPL_ActiveBatchSupportsDataSet('NETAPP_VOLUME_DIM')=0 then
        -- call sahara.SP_CUST_ATTR_Load('NETAPP_VOLUME');
    -- end if;
    -- if fn_LOADIMPL_ActiveBatchSupportsDataSet('NETAPP_QTREE_DIM')=0 then
        -- call sahara.SP_CUST_ATTR_Load('NETAPP_QTREE');
    -- end if;
    -- if fn_LOADIMPL_ActiveBatchSupportsDataSet('NETAPP_SHARE_DIM')=0 then
        -- call sahara.SP_CUST_ATTR_Load('NETAPP_SHARE');
    -- end if;
        -- if fn_LOADIMPL_ActiveBatchSupportsDataSet('NAS_FS_DIM')=0 then
        -- call sahara.SP_CUST_ATTR_Load('NAS_FILESYSTEM');
    -- end if;
END;

grant execute on sahara.SP_DataLoad_LoadCustomAttributes to sahara;


--
-- main load procedure VOM data source type 
--

if exists(select 1 from sys.sysprocedure where proc_name = 'SP_DataLoad_LoadVOM40' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.SP_DataLoad_LoadVOM40
end if;
                           
CREATE PROCEDURE "SAHARA"."SP_DataLoad_LoadVOM40" ()

BEGIN
	call sahara.SP_DataLoad_Initialize_();

   

    call sahara.SP_DataLoad_Load_();
END;

grant execute on sahara.SP_DataLoad_LoadVOM40 to sahara;

/*
 **********************************************************************
 * sp_Load_LoadFact():
 * 
 * This procedure loads facts data set from its staging table into its 
 * warehouse table in accordance with the data set's type metadata rules.
 * Procedure substitutes tokens in a template SQL statement and executes it.  
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Load_LoadFact' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Load_LoadFact
end if;

CREATE PROCEDURE SAHARA.sp_Load_LoadFact 
(
	@pDataSetType varchar(64)
) 
BEGIN
	DECLARE @FUNC varchar(64);
	DECLARE @vDdataSetTypeKey numeric(10);
	DECLARE @vName varchar(64);
	DECLARE @vStereotype varchar(32);
	DECLARE @vDestTable varchar(30);
	DECLARE @vDupFactPurgeKeyCols long varchar;
	DECLARE @vLoadSql long varchar;
	DECLARE @vSrcTable varchar(30);
	DECLARE @vSrcRowCount integer;
	DECLARE	@vDatesKey numeric(10);
	DECLARE	@vSQL long varchar;
    DECLARE @vRowCount integer;
    DECLARE @vMessage long varchar;
	DECLARE	@vSrcColumn long varchar;
	DECLARE	@vDestColumn long varchar;
	DECLARE @vColumn varchar(30);

	SET @FUNC = 'sp_Load_LoadFact'; 

	CALL SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'entry');

	select ROLLUP_DATA_SET_TYPE_KEY, NAME, STEREOTYPE, DEST_TABLE, DUPLICATE_FACT_PURGE_KEY_COLS, LOAD_SQL
	 into @vDdataSetTypeKey, @vName, @vStereotype, @vDestTable, @vDupFactPurgeKeyCols, @vLoadSql
	 from ROLLUP_DATA_SET_TYPE
	where NAME = @pDataSetType;

	set @vSrcTable = @vDestTable || '_';
    set @vSQL = trim(@vLoadSql);

    -- get the source table count for logging, auditing, and error checking
    execute immediate
        'select count(*) into @vSrcRowCount from SAHARA_APP."' || @vSrcTable || '"';
    call SP_Log_LogInfo(@ActiveSessionKey_, @FUNC, 'starting fact load: ' ||
                    @vSrcTable || ' (' || @vSrcRowCount || ' rows) -> ' ||
                    @vDestTable);

    -- the columns that are in both the source and the destination tables
    -- are the measure columns to copy
	FOR @ColumnTypeLoop AS @ColumnTypeCursor CURSOR FOR (   
		select cname as vcolumn
		  from sys.syscolumns
		 where creator = 'SAHARA_APP'
		   and tname = @vSrcTable
		intersect
		 select cname as vcolumn
		   from sys.syscolumns
		  where creator = 'SAHARA'
			and tname = @vDestTable
	)
	DO
		-- MakeList SRC_COLUMNS and DEST_COLUMNS
		if vColumn is not null and vColumn!='' then
			if @vSrcColumn is not null and @vDestColumn is not null then
				set @vSrcColumn = @vSrcColumn ||',SRC."' ||vColumn || '"';
				set @vDestColumn = @vDestColumn ||',"' ||vColumn||'"';
			else
				set @vSrcColumn = 'SRC."' || vColumn || '"';
				set @vDestColumn = '"' || vColumn || '"';
			end if;
		end if;
	END FOR;

    -- the dates key was assigned to the batch when it was created
    select DATES_KEY
      into @vDatesKey
      from ROLLUP_BATCH
     where ROLLUP_BATCH_KEY = @ActiveBatchKey_;

    -- tbd - vMeasureColumns must not be empty

    set @vSQL = replace(@vSQL, '$DATES_KEY$', @vDatesKey);
	if charindex('$DEST_COLUMNS$',@vSQL) <> 0 then 
		set @vSQL = replace(@vSQL, '$DEST_COLUMNS$', @vDestColumn);
	end if;
    set @vSQL = replace(@vSQL, '$DEST_TABLE$', @vDestTable);
    set @vSQL = replace(@vSQL, '$ROLLUP_BATCH_KEY$', @ActiveBatchKey_);
	if charindex('$SRC_COLUMNS$',@vSQL) <> 0 then 
		set @vSQL = replace(@vSQL, '$SRC_COLUMNS$', @vSrcColumn);
	end if;    
    set @vSQL = replace(@vSQL, '$SRC_TABLE$', @vSrcTable);
   /* vSQL = 'begin ' ||
            @vSQL || '; ' ||
            'LOAD_IMPL.RowCount = @@rowcount; ' ||
            'end;';
*/
    -- execute it
    call SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL: ' || @vSQL);
	message @vDestTable || '-' || @vSQL to client;
    execute immediate @vSQL;
    set @vRowCount = @@rowcount;
    call SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL: ' || @vRowCount);
	call sahara.SP_Load_DataSetRowCount(@vRowCount);

    -- make it obvious if we seem to be losing rows
    if @vRowCount <> @vSrcRowCount then
        set @BatchHasPartialLoads_ = 1;
        set @vMessage = 'For fact data set ' || @ActiveDataSetType_ ||
                    ', the number of source rows in ' ||
                    @vSrcTable || ' (' || @vSrcRowCount || ') ' ||
                    'differs from the number of rows inserted in ' ||
                    @vDestTable || ' (' || @vRowCount || ').';
        call sahara.SP_Load_BatchStatusMessage(@vMessage);
        call SP_Log_LogWarn(@ActiveSessionKey_, @FUNC, @vMessage);
    end if;

    call SP_Log_LogInfo(@ActiveSessionKey_, @FUNC, 'completed fact load: ' ||
                    @vSrcTable || ' -> ' || @vDestTable ||
                    ': ' || @vRowCount || ' rows inserted');

    -- get rid of facts for the same day and same objects from
    -- different sources
    call sp_Load_PurgeDuplicateFacts(@pDataSetType);
end;

grant execute on sahara.sp_Load_LoadFact to sahara;

/*
 **********************************************************************
 * sp_Load_PurgeDuplicateFacts():
 * 
 * Purges fact records that were rolled up in previous batches
 * from different roll-up sources for the same day.  These really
 * represent duplicated data generally caused by multiple roll-up
 * sources reporting the same object(s).
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Load_PurgeDuplicateFacts' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Load_PurgeDuplicateFacts
end if;

CREATE PROCEDURE SAHARA.sp_Load_PurgeDuplicateFacts 
(
	@pDataSetType varchar(64)
) 
BEGIN
	DECLARE @FUNC varchar(64);
	DECLARE @PURGE_SQL long varchar;
    DECLARE @vDatesKey numeric(10);
    DECLARE @vSourceKey numeric(10);
    DECLARE @vSQL long varchar;
    DECLARE @vRowCount integer;
    DECLARE @vMessage long varchar;
	
	DECLARE @vDestTable varchar(30);
	DECLARE @vDupFactPurgeKeyCols long varchar;
	
	CALL SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'entry');
	
	SET @FUNC = 'sp_Load_PurgeDuplicateFacts'; 

    SET @PURGE_SQL =
        'delete from "$DEST_TABLE$" ' ||
         'where DATES_KEY = :vDatesKey ' ||
           'and ROLLUP_BATCH_KEY in (select ROLLUP_BATCH_KEY ' ||
                                      'from ROLLUP_BATCH ' ||
                                     'where DATES_KEY = :vDatesKey ' ||
                                       'and ROLLUP_BATCH_KEY <> :ActiveBatchKey_ ' ||
                                       'and ROLLUP_SOURCE_KEY <> :vSourceKey) ' ||
           'and ($KEY_LIST$) in (select $KEY_LIST$ ' ||
                                  'from "$DEST_TABLE$" ' ||
                                 'where ROLLUP_BATCH_KEY = :ActiveBatchKey_)';

    
	select DEST_TABLE, DUPLICATE_FACT_PURGE_KEY_COLS
	 into @vDestTable, @vDupFactPurgeKeyCols
	 from ROLLUP_DATA_SET_TYPE
	where NAME = @pDataSetType;
	
	if @vDupFactPurgeKeyCols is null then
        message @vDupFactPurgeKeyCols to client;
				message 'here' to client;
		return;
    end if;

    -- get some information from the batch to make the SQL a little simpler
    select DATES_KEY, ROLLUP_SOURCE_KEY
      into @vDatesKey, @vSourceKey
      from ROLLUP_BATCH
     where ROLLUP_BATCH_KEY = @ActiveBatchKey_;

    -- form the SQL
    SET @vSQL = replace(@PURGE_SQL, '$DEST_TABLE$', @vDestTable);
    SET @vSQL = replace(@vSQL, '$KEY_LIST$', @vDupFactPurgeKeyCols);
	SET @vSQL = replace(@vSQL, ':vDatesKey', @vDatesKey);
	SET @vSQL = replace(@vSQL, ':vSourceKey', @vSourceKey);
	SET @vSQL = replace(@vSQL, ':ActiveBatchKey_', @ActiveBatchKey_);


    -- execute it
	CALL SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL: ' || @vSQL);
    execute immediate @vSQL;
    set @vRowCount = @@rowcount;

    if @vRowCount > 0 then
        set @BatchHasWarnings_ = 1;
        set @vMessage =
            'For data set ' || @ActiveDataSetType_ || ', ' || @vRowCount ||
            ' duplicate fact row(s) from other roll-up source(s) ' ||
            'and the same date have been purged.';
        call sahara.SP_Load_BatchStatusMessage(@vMessage);
        CALL SP_Log_LogWarn(@ActiveSessionKey_, @FUNC, @vMessage);
    end if;
end;

grant execute on sahara.sp_Load_PurgeDuplicateFacts to sahara;

/*
 **********************************************************************
 * sp_Load_PurgeFactsByDatesKey():
 * 
 * Deletes all fact records previously loaded for the same roll-up
 * source and date as a batch except the specified batch.  There
 * can be only one batch per roll-up source per date.
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Load_PurgeFactsByDatesKey' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Load_PurgeFactsByDatesKey
end if;

CREATE PROCEDURE SAHARA.sp_Load_PurgeFactsByDatesKey() 
BEGIN
	DECLARE @vBatchKey numeric(10);
	DECLARE @vDestTable varchar(30);
	DECLARE @FUNC varchar(64);

	SET @FUNC = 'sp_Load_PurgeFactsByDatesKey'; 

	CALL SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'entry');

    -- find any other successful batches from the same source
    -- for the same roll-up day that have not already been
    -- purged (over-written)
	FOR @BatchKeyLoop AS @BatchKeyTypeCursor CURSOR FOR (   
		select ROLLUP_BATCH_KEY as vBatchKey
		  from ROLLUP_BATCH
		 where (ROLLUP_SOURCE_KEY || '@' ||  DATES_KEY) =
					(select ROLLUP_SOURCE_KEY || '@' ||  DATES_KEY
					   from ROLLUP_BATCH
					  where ROLLUP_BATCH_KEY = @ActiveBatchKey_)
		   and ROLLUP_BATCH_KEY <> @ActiveBatchKey_
		   and IS_SUCCESS = 'T'
		   and STATUS <> 'Over-Written'		
		)
	DO
	    -- there are batches to purge
		-- add the unknown columns
		if vBatchKey is not null and vBatchKey!='' then
			-- get the tables to purge
			FOR @DestTableLoop AS @DestTableTypeCursor CURSOR FOR (   
				select ROLLUP_DATA_SET_TYPE.DEST_TABLE as vDestTable
				  from ROLLUP_BATCH
				  join ROLLUP_DATA_SET
					on ROLLUP_BATCH.ROLLUP_BATCH_KEY = ROLLUP_DATA_SET.ROLLUP_BATCH_KEY
				  join ROLLUP_DATA_SET_TYPE
					on ROLLUP_DATA_SET.ROLLUP_DATA_SET_TYPE_KEY = ROLLUP_DATA_SET_TYPE.ROLLUP_DATA_SET_TYPE_KEY
				   and STEREOTYPE = 'FACT'
				 where ROLLUP_BATCH.ROLLUP_BATCH_KEY = vBatchKey		
				)
			DO
		        -- purge each of the fact tables
		        if vDestTable is not null and vDestTable!='' then
				        -- purge each of the fact tables
						call sahara.sp_Load_PurgeFactByBatch(vBatchKey, vDestTable);
				end if;			
			END FOR;					
		end if;

        -- mark the batch over-written
        call SP_Load_BatchStatus(vBatchKey, 'Over-Written',
                    'Fact measures from this batch have been replaced ' ||
                    'by a newer batch for the same date.',
                     1);
	END FOR;

    -- if we didn't find any batches, we're done
--    if vBatchKeys is null or vBatchKeys.count = 0 then
--        return;
--    end if;

END;

grant execute on sahara.sp_Load_PurgeFactsByDatesKey to sahara;

/*
 **********************************************************************
 * sp_Load_PurgeFactByBatch():
 * 
 * purge fact records by batch
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Load_PurgeFactByBatch' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Load_PurgeFactByBatch
end if;

CREATE PROCEDURE SAHARA.sp_Load_PurgeFactByBatch
(
    @pObsoleteBatchKey numeric(10),
    @pTable varchar(30)
) 
BEGIN
	DECLARE @vSQL long varchar;
	DECLARE @FUNC varchar(64);
    DECLARE @vRowCount integer;

	SET @FUNC = 'sp_Load_PurgeFactByBatch'; 

	set @vRowCount=0;

	CALL SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'entry');

    SET @vSQL = 'delete from "' || @pTable || '" ' ||
         'where ROLLUP_BATCH_KEY = $batchKey$';

    SET @vSQL = replace(@vSQL, '$batchKey$', @pObsoleteBatchKey);
	CALL SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'SQL:' ||@vSQL );
	execute immediate @vSQL;
	set @vRowCount = @@rowcount;

    if @vRowCount > 0 then
        call SP_Log_LogWarn(@ActiveSessionKey_, @FUNC, 'purged ' || @vRowCount ||
                        ' rows from ' || @pTable ||
                        ' from a previous batch ' || @pObsoleteBatchKey);
    end if;
end;

grant execute on sahara.sp_Load_PurgeFactByBatch to sahara;

/*
 **********************************************************************
 * sp_Load_LoadStaging():
 * 
 * This procedure loads facts data set from its staging table into its 
 * warehouse table in accordance with the data set's type metadata rules.
 * Procedure substitutes tokens in a template SQL statement and executes it.  
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Load_LoadStaging' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Load_LoadStaging
end if;

CREATE PROCEDURE SAHARA.sp_Load_LoadStaging 
(
	@pSessionKey numeric(10),
	@pBatchKey numeric(10),
	@pStageLocation long varchar
) 
BEGIN
	DECLARE @vSQL long varchar;
	DECLARE @vname varchar(64);
	DECLARE @vstagetable varchar(30);
	DECLARE @vcsvname varchar(64);
	DECLARE @FUNC varchar(64);
    DECLARE @vRowCount integer;
    DECLARE @vMessage long varchar;

	SET @FUNC = 'sp_Load_LoadStaging'; 

	CALL SP_Log_LogDebug(@pSessionKey, @FUNC, 'entry');

	-- the csvnames present per data set type 
    -- are the stage files to load in staging tables
	FOR @DataSetTypeLoop AS @DataSetTypeCursor CURSOR FOR (   
		select DSTYPE.NAME as vname, DSSTAGE.STAGE_TABLE as vstagetable, CSV_NAME as vcsvname
		  from SAHARA.ROLLUP_DATA_SET_TYPE DSTYPE
		  join SAHARA.ROLLUP_DATA_SET_STAGE DSSTAGE on DSTYPE.ROLLUP_DATA_SET_TYPE_KEY=DSSTAGE.ROLLUP_DATA_SET_TYPE_KEY
		 where csv_name is not null
	)
	DO
		-- log dataset staging status per batch to ROLLUP_DATA_SET table
		call fn_Batch_DataSetStatus(@pSessionKey,@pBatchKey,vname,'Staging');
		set @vSQL= 'LOAD TABLE SAHARA_APP.$STAGE_TABLE$ FROM ''$STAGE_LOC$/$CSV_NAME$'' SKIP 1 ESCAPES OFF FORMAT ASCII';
	    set @vSQL = replace(@vSQL, '$STAGE_TABLE$', vstagetable);
		set @vSQL = replace(@vSQL, '$STAGE_LOC$', @pStageLocation);
		set @vSQL = replace(@vSQL, '$CSV_NAME$', vcsvname);

	    -- execute it
	    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'SQL: ' || @vSQL);
	    execute immediate @vSQL;
		set @vRowCount = @@rowcount;
	    call SP_Log_LogDebug(@pSessionKey, @FUNC, 'Rows Staged: ' || @vRowCount);

		-- dataset staging completed, log completion status to ROLLUP_DATA_SET table
		call fn_Batch_DataSetStatus(@pSessionKey,@pBatchKey,vname,'Staging Complete');
	END FOR;

exception
    when others then
        SELECT ERRORMSG() INTO @vMessage;
		CALL fn_Batch_Status(@pSessionKey, @pBatchKey,'Failed', 'Data rollup failed, because VOM Advanced Enterprise Server could not stage extracted data in the database. Message: ' || @vMessage);	 
        CALL SP_Error_RaiseApplicationErrorDetail(@EStageError,@FUNC,
                         'Staging failed for batch ' || @pBatchKey ||
                         '.  Message: ' || @vMessage, @pSessionKey);
END;

grant execute on sahara.sp_Load_LoadStaging to sahara_app;

/*
 **********************************************************************
 * sp_Load_ResilientCheck():
 * 
 * This procedure loads ETL logs extracted during VOM MS extraction
 * into production table MESSAGE_LOG  
 **********************************************************************
 */
if exists(select 1 from sys.sysprocedure where proc_name = 'sp_Load_ResilientCheck' and user_name(creator) = 'SAHARA') then
   drop procedure sahara.sp_Load_ResilientCheck
end if;

CREATE PROCEDURE SAHARA.sp_Load_ResilientCheck () 
BEGIN
	DECLARE @vSQL long varchar;
	DECLARE @FUNC varchar(64);
    DECLARE @vMessage long varchar;
	DECLARE @vErrorMessage long varchar;

	SET @FUNC = 'sp_Load_ResilientCheck'; 

	CALL SP_Log_LogDebug(@ActiveSessionKey_, @FUNC, 'entry');

	if exists (select 1 from sahara.etl_log where procedure_name= 'sahara.sahara_resilient_data' and batch_key =@ActiveBatchKey_ and ("message" like '%Negative%' or "message" like '%Percentage%')) then
		-- iterate negative and invalid percentage records
		FOR @ResilientLoop AS @DResilientCursor CURSOR FOR (   
			select "message" as vmessage
			from sahara.etl_log where procedure_name= 'sahara.sahara_resilient_data' and batch_key =101 
			 and ("message" like '%Negative%' or "message" like '%Percentage%')			
		)
		DO
			-- log resilient rollup ETL_LOG warnings in MESSAGE_LOG table
	        call SP_Log_LogWarn(@ActiveSessionKey_, @FUNC, vMessage);
		END FOR;
	end if;
exception
    when others then
        SELECT ERRORMSG() INTO @vErrorMessage;
		CALL fn_Batch_Status(@ActiveSessionKey_, @ActiveBatchKey_,'Failed', 'Data rollup failed. Message: ' || @vErrorMessage);	 
        CALL SP_Error_RaiseApplicationErrorDetail(@EStageError,@FUNC,
                         'Rollup failed for batch ' || @ActiveBatchKey_ ||
                         '.  Message: ' || @vErrorMessage, @ActiveSessionKey_);
END;

grant execute on sahara.sp_Load_ResilientCheck to sahara_app;
message 'Populating/updating the data dictionary...' to log;

--
-- load the table and column metadata from PowerDesigner
--
--read saharaPDDump.sql;

--read saharaDataLoad.sql;

-------------------------------------------------------------------------------

BEGIN
DECLARE @CONFIG long varchar;

--
-- possible status value for a roll-up batch
--
begin
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Initializing',
        'Roll-up is beginning.  The roll-up source has not yet been contacted.'
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Remote Processing',
        'The roll-up source has been contacted and data is being processed ' ||
        'remotely in preparation for transfer to the Enterprise Reporter ' ||
        'server.'
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Uploading',
        'The data is being transferred across the network from the roll-up ' ||
        'source to the Enterprise Reporter server.'
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Staging',
        'The data is being loading from files into a temporary area of the ' ||
        'Enterprise Reporter database in preparation for further processing.'
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Loading',
        'The data is undergoing final processing and is being loaded from a ' ||
        'temporary area of the Enterprise Reporter database into the ' ||
        'production area from which it will be reported.'
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Succeeded',
        'Data roll-up has succeeded with no errors or warnings and is ' ||
        'available for reporting.',
        1,       -- is complete
        1        -- is success
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Succeeded with Warnings',
        'Data roll-up has succeeded with some warnings.  It is available ' ||
        'for reporting.',
        1,       -- is complete
        1        -- is success
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Failed',
        'Data roll-up has failed.  No data from the roll-up batch is available ' ||
        'for reporting.',
        1,       -- is complete
        0       -- is success
    );
    call sahara.SP_DataDict_RollupBatchStatus
    (
        'Over-Written',
        'Enterprise Reporter retains only one roll-up batch of data per ' ||
        'roll-up source per roll-up day.  The batch was replaced by a newer ' ||
        'batch which was successfully rolled-up from the same source on the ' ||
        'same roll-up day.',
        1,       -- is complete
        0       -- is success
    );
end;

commit;

-------------------------------------------------------------------------------
--
message 'define which dimensions can be associated with business views' to log;
--
--call sahara.SP_DataDict_BViewDimension('APP_GROUP',        'Application Group');
call sahara.SP_DataDict_BViewDimension('ARRAY',            'Array');
call sahara.SP_DataDict_BViewDimension('FILESYSTEM',       'Filesystem');
call sahara.SP_DataDict_BViewDimension('HOST',             'Host');
--call sahara.SP_DataDict_BViewDimension('NETAPP_QTREE',     'NAS QTree');
call sahara.SP_DataDict_BViewDimension('NETAPP_SHARE',     'NAS Share');
--call sahara.SP_DataDict_BViewDimension('NETAPP_VOLUME',    'NAS Volume',    'NA_VOLUME');
call sahara.SP_DataDict_BViewDimension('NAS_FILESYSTEM',    'NAS Filesystem',    'NAS_FS');
--call sahara.SP_DataDict_BViewDimension('SWITCH',           'Switch');
call sahara.SP_DataDict_BViewDimension('VOLUME',           'Volume');
call sahara.SP_DataDict_BViewDimension('CLUSTER',           'Cluster');
call sahara.SP_DataDict_BViewDimension('SERVICEGROUP',           'Service Group');
commit;

-------------------------------------------------------------------------------
--
message 'define which dimensions are supported for forecasting' to log;
--
call sahara.SP_DataDict_FCastDimension('ARRAY', 'Array', '[Array Analysis].[By Array Attribute].[By Array Attribute].[Array]','ARRAY_STORAGE', '[Array Analysis].[By Array Attribute].[By Array Attribute].[Array].[Array Key]');
call sahara.SP_DataDict_FCastDimension('HOST', 'Host', '[Host Consumption].[By Host Attribute].[By Host Attribute].[Host]','HOST_CAPACITY', '[Host Consumption].[By Host Attribute].[By Host Attribute].[Host].[Host Key]');
call sahara.SP_DataDict_FCastDimension('FILESYSTEM', 'Filesystem','[File System Analysis].[By Host].[By Host].[File System]','FILESYSTEM_CAPACITY','[File System Analysis].[By Host].[By Host].[File System].[File System Key]');
call sahara.SP_DataDict_FCastDimension('DISKGROUP', 'Disk Group','  [Disk Group Analysis].[By Host].[By Host].[Disk Group]','DISKGROUP_CAPACITY','[Disk Group Analysis].[By Host].[By Host].[Disk Group].[Disk Group Key]');
call sahara.SP_DataDict_FCastDimension('DATABASES', 'Database','[Database Analysis].[By Host].[By Host].[Database]','DATABASE_CAPACITY','[Database Analysis].[By Host].[By Host].[Database].[Database Key]');
call sahara.SP_DataDict_FCastDimension('NETAPP_VOLUME', 'NAS Volume','[NAS Volume Analysis].[By NAS Volume].[By NAS Volume].[NAS Volume]','NETAPP_VOLUME_FILE','[NAS Volume Analysis].[By NAS Volume].[By NAS Volume].[NAS Volume].[NAS Volume Key]');
commit;

-------------------------------------------------------------------------------
--
message 'define which dimensions and facts are supported for forecasting' to log;
--
call sahara.SP_DataDict_EnableFCastSchema('HOST','APPLICATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('HOST','APPLICATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('HOST','CLUSTER_NAME');
call sahara.SP_DataDict_EnableFCastSchema('HOST','CLUSTER_TYPE');
call sahara.SP_DataDict_EnableFCastSchema('HOST','CLUSTER_VERSION');
call sahara.SP_DataDict_EnableFCastSchema('HOST','GUEST_OS_TYPE');
call sahara.SP_DataDict_EnableFCastSchema('HOST','IS_USER_CREATED');
call sahara.SP_DataDict_EnableFCastSchema('HOST','IS_VIRTUAL');
call sahara.SP_DataDict_EnableFCastSchema('HOST','IS_VIRTUAL_SERVER');
call sahara.SP_DataDict_EnableFCastSchema('HOST','ORGANIZATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('HOST','ORGANIZATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('HOST','OS_DETAILS');
call sahara.SP_DataDict_EnableFCastSchema('HOST','OS_TYPE');
call sahara.SP_DataDict_EnableFCastSchema('HOST','SF_VERSION');
call sahara.SP_DataDict_EnableFCastSchema('HOST','VIRTUALIZATION_TECHNOLOGY');
call sahara.SP_DataDict_EnableFCastSchema('HOST','VIRTUAL_SERVER');
call sahara.SP_DataDict_EnableFCastSchema('HOST','VOM_LICENSE_REQUIRED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DB_AVAILABLE');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DB_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DB_USED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DISKGROUP_AVAILABLE');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DISKGROUP_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_DISKGROUP_USED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_FS_AVAILABLE');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_FS_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_FS_USED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_NAS_SP');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_NAS_SP_UNUSED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_NAS_SP_USED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_STORAGE_POOL_UNUSED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_STORAGE_POOL_USED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_VIRTUALIZATION_CAP');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_VIRTUALIZATION_UNUSED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_VOLUME_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','ADJUSTED_VOLUME_UNUSED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','C_ADJUSTED_ALLOCATED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','C_ADJUSTED_CLAIMED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','C_ADJUSTED_TOTAL_ALLOCATED');
call sahara.SP_DataDict_EnableFCastSchema('HOST_CAPACITY','C_THIN');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','APPLICATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','APPLICATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','FAMILY');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','FIRMWARE_VERSION');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','MODEL');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','ORGANIZATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','ORGANIZATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','TYPE');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','VENDOR');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY','VOM_DISCOVER_TYPE');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY_STORAGE','ALLOCATED');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY_STORAGE','LOGICAL_AVAILABLE');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY_STORAGE','TOTAL_PHYSICAL');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY_STORAGE','TOTAL_UNCLAIMED');
call sahara.SP_DataDict_EnableFCastSchema('ARRAY_STORAGE','UNALLOCATED');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','APPLICATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','APPLICATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','DEVICE_PATH');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','IS_CFS_AWARE');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','IS_CLUSTERED');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','IS_CREATED_FROM_VOLUME');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','MOUNT_POINT');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','ORGANIZATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','ORGANIZATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM','TYPE');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM_CAPACITY','CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM_CAPACITY','FREE_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('FILESYSTEM_CAPACITY','USED_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('DATABASES','APPLICATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASES','APPLICATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASES','DATABASE_STATE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASES','DATABASE_TYPE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASES','DATABASE_VERSION');
call sahara.SP_DataDict_EnableFCastSchema('DATABASES','ORGANIZATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASES','ORGANIZATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASE_CAPACITY','TOTAL_AVAILABLE');
call sahara.SP_DataDict_EnableFCastSchema('DATABASE_CAPACITY','TOTAL_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('DATABASE_CAPACITY','TOTAL_USED');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP','FAMILY');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP','ORGANIZATION_BE');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP','ORGANIZATION_COMPOSITE_BE');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP','STATE');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP','TYPE');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP_CAPACITY','CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP_CAPACITY','FREE_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('DISKGROUP_CAPACITY','USED_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME_FILE','USED_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME_FILE','CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME_FILE','AVAILABLE_CAPACITY');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME','RAID_LEVEL');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME','TYPE');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME','SUB_TYPE');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME','TIER');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME','IS_NAS_FS_CREATED');
call sahara.SP_DataDict_EnableFCastSchema('NETAPP_VOLUME','DRIVE_TYPE');


commit;

---------------------------------------------------------------------------------
message 'define contexts in which re-usable things may be used' to log;
--
--call sahara.SP_DataDict_CustomAttributeContext('APP_GROUP',        'Application Group');
call sahara.SP_DataDict_CustomAttributeContext('ARRAY',            'Array');
call sahara.SP_DataDict_CustomAttributeContext('DATABASES',        'Database');
call sahara.SP_DataDict_CustomAttributeContext('FILESYSTEM',       'Filesystem');
call sahara.SP_DataDict_CustomAttributeContext('HOST' ,            'Host');
call sahara.SP_DataDict_CustomAttributeContext('LUN',              'LUN');
--call sahara.SP_DataDict_CustomAttributeContext('NETAPP_QTREE',     'NAS QTree');
--call sahara.SP_DataDict_CustomAttributeContext('NETAPP_SHARE',     'NAS Share');
--call sahara.SP_DataDict_CustomAttributeContext('NETAPP_VOLUME',    'NAS Volume');
--call sahara.SP_DataDict_CustomAttributeContext('NAS_FILESYSTEM',    'NAS Filesystem');
--call sahara.SP_DataDict_CustomAttributeContext('SWITCH',           'Switch');
call sahara.SP_DataDict_CustomAttributeContext('DISKGROUP',           'Disk Group');
call sahara.SP_DataDict_CustomAttributeContext('VOLUME',           'Volume');
call sahara.SP_DataDict_CustomAttributeContext('CLUSTER',           'Cluster');
call sahara.SP_DataDict_CustomAttributeContext('SERVICEGROUP',      'Service Group');
commit;

call sahara.SP_DataDict_TierContext
(
    'LUN',      -- context
    'GB',       -- unit cost
    -- rule SQL
    'update SAHARA_APP.LUN_ SRC ' ||
       'set TIER = (select ($TIER_RULE_EXPRESSION$) ' ||
                     'from SAHARA_APP.LUN_ ' ||
                     'left outer join SAHARA_APP.ARRAY_ ' ||
                       'on LUN_.ARRAY_OID = ARRAY_.ARRAY_OID ' ||
                    'where LUN_.LUN_OID = SRC.LUN_OID); ' ||
    'update SAHARA_APP.LUN_ SRC ' ||
       'set TIER_COST_PER_GB = (select TIER.COST ' ||
                                 'from SAHARA_APP.LUN_ ' ||
                                 'join SAHARA.TIER ' ||
                                   'on LUN_.TIER = TIER.DISPLAY_NAME ' ||
                                  'and TIER.TIER_CONTEXT = ''LUN'' ' ||
                                'where LUN_.LUN_OID = SRC.LUN_OID)'
);
commit;


-- TODO: LUN Tier Help
-- read saharaLUNTierRulesHelp.sql
message 'Adding rule context' to log;
call sahara.SP_DataDict_RuleContext('LUN TIER', ':vHTML');
commit;

-------------------------------------------------------------------------------
--
message 'Create the metadata used to create LUN tiers rules' to log;
-- Create the metadata used to create LUN tiers rules.  LUN tier rules can
-- be created from array and LUN attributes.
--
call sahara.SP_DataDict_RuleElementSection('LUN TIER', 'Array', 'Array Attributes');
call sahara.SP_DataDict_RuleElementsFromDimensionSrc
(
    'LUN TIER',     -- context
    'Array',        -- section
    'ARRAY_',       -- staging attribute source table
    'ARRAY',         -- dimension metadata source table
	null,           -- prefix
    -- excluded columns
    'PARENT_ARRAY_OID,ADVISORY_DISPLAY_NAME,DISCOVERY_STATE,NIS_DOMAIN,NIS_SERVER,IS_NIS_ENABLED'
);
commit;

call sahara.SP_DataDict_RuleElementSection('LUN TIER', 'LUN', 'LUN Attributes');
call sahara.SP_DataDict_RuleElementsFromDimensionSrc
(
    'LUN TIER',     -- context
    'LUN',          -- section
    'LUN_',         -- staging attribute source table
    'LUN',          -- dimension metadata source table
    null,           -- prefix
    -- excluded columns
    'TIER,TIER_COST_PER_GB,BINDING_STATE,DRIVE_TYPE,EMULATION_MODE,VENDOR_STATUS,IS_READ_CACHE_ENABLED,IS_WRITE_CACHE_ENABLED,NUMBER_OF_COPIES,PARITY,STRIPE_WIDTH,REVISION,VENDOR_DEVICE_ID,DISCOVERY_STATE'
);
commit;

-------------------------------------------------------------------------------
--
-- Cognos SMTP email configuration: Insert default values if the table
-- is empty.  The table only allows one row.
--
insert into sahara.COGNOS_SMTP(COGNOS_SMTP_KEY, SERVER, PORT, SENDER) on existing update
values ('KEY', 'localhost', 25, 'CCER@veritas.com');
commit;

message 'Populating the Fact mapping table ' to log;
-------------------------------------------------------------------------------
--
-- Cognos dimension to fact mappings
--
-- The COGNOS_FACT_MAPPING table contains a list of all the underlying measures
-- defined in the Cognos metamodel for each of the supported object types.  This
-- information is used during a metamodel update operation to make sure any new
-- dimensions (e.g. business views) that support the Enterprise Reporter Object
-- type are associated properly with all of the measures supported by this type.
-- If this mapping does not happen then the user will get an error when creating
-- a report indicating there's no association between the hierarchy/object and
-- the fact they're trying to report against.
--
-- Columns in this table:
-- OBJECT_TYPE: The underlying Enterprise Reporter object type that supports
--              the measure.
-- NAMESPACE: The namespace to which the measure belongs.
-- FACT_DIMENSION_NAME: The name of the dimension to which underlying fact belongs.
-- FACT_NAME: The name of the fact to which map the object type.
--
-- For example, if we add a new business view that supports object types, then the
-- update_metamodel script needs to create actions that will map this new business
-- view dimension to the "File System Capacity (GB)" measure in the File System
-- Analysis namespace.
--
-- Any XML special characters must be escaped. e.g. % => &#37;
--
-- IMPORTANT - if you make material changes to the metamodel that changes these
-- relationships in any way (e.g. renaming a namespace, adding/removing measures),
-- then you MUST update this configuration information.  Failure to do so will result
-- in possible broken reports.
--
--truncate table SAHARA.COGNOS_FACT_MAPPING;
SET @CONFIG = 'insert into SAHARA.COGNOS_FACT_MAPPING(OBJECT_TYPE, NAMESPACE, FACT_DIMENSION_NAME, FACT_NAME) on existing update values';

/* CLEAN-UP
--
-- application group dimension to fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,
'''APP_GROUP'', ''Application Group Consumption'', ''Application Group Capacity Metrics'', ''Allocated Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''APP_GROUP'', ''Application Group Consumption'', ''Application Group Capacity Metrics'', ''Undiscovered Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''APP_GROUP'', ''Application Group Consumption'', ''Application Group Capacity Metrics'', ''Unused Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''APP_GROUP'', ''Application Group Consumption'', ''Application Group Capacity Metrics'', ''Used Capacity (GB)''');
*/

--
message 'array dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics'',''&#37; Capacity Allocated to Hosts Claimed''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics'', ''Claimed Capacity (GB)''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics'', ''Unclaimed Capacity (GB)''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics'', ''Undiscovered Allocated Capacity (GB)''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Capacity Metrics'', ''&#37; Total Array Capacity Configured''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Capacity Metrics'', ''&#37; Total Array Capacity Unconfigured''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Capacity Metrics'', ''Configured Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Capacity Metrics'', ''External Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Capacity Metrics'', ''Internal Physical Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Capacity Metrics'', ''Total Array Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Capacity Metrics'', ''Unconfigured Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''&#37; Configured Capacity Allocated''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''&#37; Configured Capacity Available for Allocation''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''&#37; Configured Capacity Reserved for Overhead''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''&#37; Configured Capacity Undiscovered''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Administrative Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Available Configured Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Capacity Allocated to Hosts Directly (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Capacity Allocated to Hosts Through Virtualizer (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Capacity Available for Allocation (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Capacity Available to Hosts Through Virtualizer (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Capacity Reserved for Overhead (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''RAID Overhead Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Replication Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Total Allocated Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Unallocated LUN Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Configured Capacity Metrics'', ''Undiscovered Configured Capacity (GB)''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics (Thin Aware)'', ''Claimed Capacity (GB) (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics (Thin Aware)'', ''Unclaimed Capacity (GB) (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics (Thin Aware)'', ''Undiscovered Allocated Capacity (GB) (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Allocated Storage Metrics (Thin Aware)'', ''&#37; Capacity Allocated to Hosts Claimed (Thin Aware)''');


call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''Overwrite Reserve''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''Snapshot Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''Snapshot Reserve''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''WAFL Spare''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''NAS Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''No Of Storage Pools''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''No Of Volumes''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''No Of Shares''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''NAS metrics'', ''No Of NAS Filesystems''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Allocated Host Count''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Capacity Allocated to Hosts Directly (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Claiming Host Count''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Total Cost''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Tiered Storage Analysis'', ''Tiered Storage Metrics'', ''Allocated Hosts''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Tiered Storage Analysis'', ''Tiered Storage Metrics'', ''Allocated To Host Directly (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Tiered Storage Analysis'', ''Tiered Storage Metrics'', ''Claimed Hosts''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Tiered Storage Analysis'', ''Tiered Storage Metrics'', ''LUN Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Tiered Storage Analysis'', ''Tiered Storage Metrics'', ''Total Cost''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Tiered Storage Analysis'', ''Tiered Storage Metrics'', ''Unallocated (GB)''');

--
message 'host dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Database Analysis'', ''Database Analysis Metrics'', ''Database Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Database Analysis'', ''Database Analysis Metrics'', ''Unused Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Database Analysis'', ''Database Analysis Metrics'', ''Used Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Database Analysis'', ''Database Analysis Metrics'', ''&#37; Database Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Database Analysis'', ''Database Analysis Metrics'', ''&#37; Database Capacity Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''Unused Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''Reserved Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''&#37; File System Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''&#37; File System Capacity Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used By Applications (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used By Non-applications (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Undiscovered Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''&#37; Allocated Capacity Undiscovered''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Percent Host Utilization''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Allocated IDE''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Claimed Loop''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Allocated Loop''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Total Allocated''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Allocated Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Claimed Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Unclaimed Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''&#37; Allocated Capacity Claimed''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''&#37; Allocated Capacity Unclaimed''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Allocated SAN''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Claimed SAN''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Allocated DAS''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Allocated ISCSI''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Allocated File Storage''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Claimed ISCSI''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Claimed File Storage''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Storage (LUN) Capacity Directly Allocated to Virtual Machine''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Virtual Pool Allocated''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Claimed By Other Hosts''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Assigned To Volume Manager''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Directly Assigned to File System''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Directly Assigned to Database''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Potentially Unassigned''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Assigned To Other Host''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Uncorrelated SAN Allocated Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Disk Group Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Disk Group Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Disk Group Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Volume Manager Overhead''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''File System Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''File System Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''File System Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''File System On Volume''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''File System On Non Volume''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''File System Used By Application''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''File System Used By Non-Application''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Unused Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Virtualization Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Virtualization Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Storage Pool Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Storage Pool Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Unused VMDK''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Exported to Virtual Machine''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Database Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Database Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Database Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Apps on Raw Volume''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics'', ''Apps on File System''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Thin Subscribed''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Thin Consumed''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Allocated (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Total Allocated (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Claimed (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Unclaimed (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Claimed By Other Host (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Assigned To VM (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Directly Assigned to File System (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Ditectly Assigned to Database (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Potentially Unassigned (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Thin Aware)'', ''Assigned to Other Host (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Thin Subscribed (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Thin Consumed (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Allocated (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Uncorrelated SAN Allocated Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Allocated FC Fibre (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Allocated ISCSI (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Allocated File Storage (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Claimed (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Claimed FC Fibre (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Claimed ISCSI (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Claimed Filestorage (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Unclaimed (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Direct Mapped Allocated (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Virtual Pool Allocated (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Total Allocated (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Assigned to Volume Manager (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Directly Assigned to File System (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Directly Assigned to Database (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Potentially Unassigned (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Allocated (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Total Allocated (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Claimed (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Unclaimed (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Assigned to Volume Manager (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Directly Assigned to File System (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Directly Assigned to Database (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Potentially Unasssigned (Thin Aware, Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Disk Group Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Disk Group Used (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Disk Group Available (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Volume Manager Overhead (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''File System Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''File System Used (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''File System Available (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''File System on Volume (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''File System on Non-Volume (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''File System Used By Applications (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''File System Used by Non-Applications (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Volume Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Unused Volume Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Database Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Database Used (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Database Available (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Apps on Raw Volume (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Apps on File System (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Exported to Virtual Machine (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Unused VMDK (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Storage Pool Unused (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Storage pool Used (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Virtualization Unused (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Capacity Metrics (Cluster Aware)'', ''Virtualization Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Capacity Allocated to Hosts Directly (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Total Cost''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Claiming Host Count''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Allocated Host Count''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Volume Analysis'', ''Volume Analysis Metrics'', ''Volume Size (GB)''');

--
message 'NAS_SHARE dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Total Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Total Logical''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Logical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Logical Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Replication Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''RAID Overhead Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Snapshot Reserve''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Overwrite Reserve''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Snapshot Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''WAFL Spare''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''NAS Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of Volumes''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of Shares''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of NAS Filesystems''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of Storage Pools''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Total Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Available Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Overhead Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''No Of Volumes Created''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''Total Logical''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''Logical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''Logical Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''NAS Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''No Of Volumes''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''No Of NAS Filesystems''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''No Of Shares''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Total Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Used Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Available Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Overhead Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''No Of LUNs Created''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Space Shared''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Space Saved''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Effective Space Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Space Shared''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Space Saved''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Effective Space Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Subscribed Percent''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Space Saved Percent''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''No Of LUNs''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''No Of Targets''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Replication Target Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''No Of Hosts Accessing NAS''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''NAS Allocated Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''NAS Consumed Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''Adjusted NAS Allocated Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''Adjusted NAS Consumed Capacity''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''No Of Hosts Accessing NAS''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Disk Limit Hard''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Disk Limit Soft''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Disk Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''File Limit Hard''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''File Limit Soft''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Files Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Threshold''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Physical Disk Limit Hard''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Physical Disk Limit Soft''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Physical Disk Used''');


--
message 'NAS_FILESYSTEM dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Total Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Total Logical''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Logical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Logical Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Replication Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''RAID Overhead Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Snapshot Reserve''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Overwrite Reserve''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''Snapshot Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''WAFL Spare''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''NAS Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of Volumes''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of Shares''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of NAS Filesystems''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Physical Filer Metrics'', ''No Of Storage Pools''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Total Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Available Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Overhead Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Storage Pool Metrics'', ''No Of Volumes Created''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''Total Logical''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''Logical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''Logical Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''NAS Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''No Of Volumes''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''No Of NAS Filesystems''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Virtual Filter Metrics'', ''No Of Shares''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Total Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Used Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Available Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Overhead Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''No Of LUNs Created''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Space Shared''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Space Saved''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Effective Space Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Space Shared''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Space Saved''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Physical Effective Space Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Subscribed Percent''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS Volume Metrics'', ''Space Saved Percent''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Physical Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''No Of LUNs''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''No Of Targets''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''Replication Target Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''NAS FileSystem Metrics'', ''No Of Hosts Accessing NAS''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''LUN Metrics'', ''LUN Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''LUN Metrics'', ''Physical Capacity''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''NAS Allocated Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''NAS Consumed Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''Adjusted NAS Allocated Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS And Unified Storage Analysis'', ''Host Metrics'', ''Adjusted NAS Consumed Capacity''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''Physical Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Chargeback Analysis'', ''NAS Chargeback Metrics'', ''No Of Hosts Accessing NAS''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Disk Limit Hard''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Disk Limit Soft''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Disk Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''File Limit Hard''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''File Limit Soft''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Files Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Threshold''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Physical Disk Limit Hard''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Physical Disk Limit Soft''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Quota Analysis'', ''NAS Quota Metrics'', ''Physical Disk Used''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Physical Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''No Of Hosts Accessing NAS''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Replication Target Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''No Of Targets''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Physical Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Physical Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Physical Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Physical Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Physical Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''No Of LUNs''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Tree Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Tree Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS FileSystem Analysis'', ''NAS FileSystem Metrics'', ''Volume Capacity''');

--
message 'License Deployments Analysis dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License Deployments Analysis'', ''License Deployments Metrics'', ''Total License Deployments''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License Deployments Analysis'', ''License Deployments Metrics'', ''Expired License Deployments''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License Deployments Analysis'', ''License Deployments Metrics'', ''Accountable License Deployments''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License Deployments Analysis'', ''License Deployments Metrics'', ''Expired Accountable License Deployments''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License Deployments Analysis'', ''License Deployments Metrics'', ''Active License Deployments''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License Deployments Analysis'', ''License Deployments Metrics'', ''Keyless License Deployments''');


--
message 'License And Host Analysis dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License And Host Analysis'', ''License Summary Metrics'', ''License Deployments''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License And Host Analysis'', ''License Summary Metrics'', ''Unique Licenses''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License And Host Analysis'', ''License Summary Metrics'', ''Expired Licenses''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License And Host Analysis'', ''License Summary Metrics'', ''Keyless Licenses''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License And Host Analysis'', ''Host Count Metrics'', ''No Of Hosts With Licenses''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License And Host Analysis'', ''Host Count Metrics'', ''No Of Hosts With Expired Licenses''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''License And Host Analysis'', ''Host Count Metrics'', ''No Of Hosts With Keyless Licenses''');

--
message 'Cluster Inventory dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''CLUSTER'', ''Cluster Inventory'', ''Cluster Inventory Metrics'', ''Number of Clusters''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''CLUSTER'', ''Cluster Inventory'', ''Cluster Inventory Metrics'', ''Number Of Cluster Nodes''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''CLUSTER'', ''Cluster Inventory'', ''Cluster Inventory Metrics'', ''Number Of Service Groups''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''CLUSTER'', ''Cluster Inventory'', ''Cluster Inventory Metrics'', ''Number Of Resources''');

--
message 'Service Group Inventory dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Inventory'', ''Service Group Inventory Metrics'', ''Number Of Resources''');

--
message 'Service Group Uptime Analysis dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time In Online State''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time In Offline State''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time In Partial State''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time In Faulted State''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time In Transient State''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time In Unknown State''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Total''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Online (secs)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Offline (secs)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Partial (secs)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Faulted (secs)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Transient States (secs)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Unknown (secs)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Service Group Uptime Analysis'', ''Service Group Uptime Metrics'', ''Time Total (secs)''');

--
message 'Cluster And Service Group Failover Event Analysis dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Planned Failovers''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Unplanned Automatic Failovers''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Unplanned Manual Failovers''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Successful Failovers''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Unresolved Planned Failovers''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Unresolved Unplanned Automatic Failovers''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Unresolved Unplanned Manual Failovers''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''Cluster And Service Group Failover Event Analysis'', ''Failover Event Metrics'', ''Failed Failovers''');



/*TODO: DG*/
/* CLEAN-UP
--
-- NAS QTree dimension to fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree Analysis'', ''NAS QTree Capacity Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree Analysis'', ''NAS QTree Capacity Metrics'', ''Total Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree Analysis'', ''NAS QTree Inventory Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree Analysis'', ''NAS QTree Inventory Metrics'', ''No Of Folders''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree Analysis'', ''NAS QTree Inventory Metrics'', ''No Of Users''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree File Classification Analysis'', ''NAS QTree File Category Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree File Classification Analysis'', ''NAS QTree File Category Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share Analysis'', ''NAS Share Capacity Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Folders''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Users''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Metrics'', ''Total Files Size (GB)''');

--
-- NAS Share dimension to fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share Analysis'', ''NAS Share Capacity Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Folders''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Users''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Metrics'', ''Total Files Size (GB)''');

--
-- NAS volume dimension to fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree Analysis'', ''NAS QTree Capacity Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree Analysis'', ''NAS QTree Capacity Metrics'', ''Total Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree Analysis'', ''NAS QTree Inventory Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree Analysis'', ''NAS QTree Inventory Metrics'', ''No Of Folders''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree Analysis'', ''NAS QTree Inventory Metrics'', ''No Of Users''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree File Classification Analysis'', ''NAS QTree File Category Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree File Classification Analysis'', ''NAS QTree File Category Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share Analysis'', ''NAS Share Capacity Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Folders''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share Analysis'', ''NAS Share Inventory Metrics'', ''No Of Users''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Capacity Metrics'', ''Available Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Capacity Metrics'', ''Total Files Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Capacity Metrics'', ''Total Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Capacity Metrics'', ''Used Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Inventory Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Inventory Metrics'', ''No Of Folders''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Inventory Metrics'', ''No Of Qtrees''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Inventory Metrics'', ''No Of Users''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume File Classification Analysis'', ''NAS Volume File Category Metrics'', ''No Of Files''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume File Classification Analysis'', ''NAS Volume File Category Metrics'', ''Total Files Size (GB)''');

--
-- switch dimension to fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''Total Port Count''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''Unused Ports''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''Used Ports''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''&#37; Ports Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''&#37; Ports Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''ISL Ports''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''Array Ports''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''Host Ports''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Metrics'', ''Other Used Ports''');

--
-- NAS Filesystem dimension to fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS File System Analysis'', ''NAS File System Capacity Metrics'', ''Total File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS File System Analysis'', ''NAS File System Capacity Metrics'', ''Used File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS File System Analysis'', ''NAS File System Capacity Metrics'', ''Free File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS File System Analysis'', ''NAS File System Capacity Metrics'', ''Physical File System Capacity (GB)''');
*/

--
message ' Chargeback Analysis fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Allocation'', ''Host Storage Allocation Metrics'', ''Adjusted Allocated Capacity''');
/* CLEAN-UP call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''Chargeback Analysis'', ''Volume Capacity Metrics'', ''Volume Size (GB)'''); */
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Chargeback Analysis'', ''Volume Capacity Metrics'', ''Volume Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Chargeback Analysis'', ''Volume Capacity Metrics'', ''Volume Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Total Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Total Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Free Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Free Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Used Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Used Capacity (GB)''');

/* CLEAN-UP
--
-- NAS Usage Analysis fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Total Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Used Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Available Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Physical Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Quota Limit''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Quota Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Total Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Used Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Available Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Physical Volume Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Total File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Used File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Free File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS Usage Analysis'', ''NAS Usage Capacity Metrics'', ''Physical File System Capacity (GB)''');
commit;
*/

--
message 'filesystem dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''Unused Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''Reserved Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''&#37; File System Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''&#37; File System Capacity Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used By Applications (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used By Non-applications (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Total Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Free Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Used Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''Chargeback Analysis'', ''Volume Capacity Metrics'', ''Volume Size (GB)''');
commit;

--
message 'volume dimension to fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''Volume Analysis'', ''Volume Analysis Metrics'', ''Volume Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''Chargeback Analysis'', ''Volume Capacity Metrics'', ''Volume Size (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Total Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Free Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''Chargeback Analysis'', ''File System Capacity Metrics'', ''Used Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''File System Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''Unused Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''Reserved Capacity (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''&#37; File System Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''&#37; File System Capacity Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used By Applications (GB)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''File System Analysis'', ''File System Analysis Metrics'', ''Used By Non-applications (GB)''');
commit;

--
message 'Disk Group Analysis fact mappings' to log;
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Percent Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Percent Used''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Free Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Used Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Percent Free''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Disk Group Analysis'', ''Disk Group Analysis Metrics'', ''Percent Used''');
commit;


--
-- Host Storage Reclamation Analysis fact mappings
--
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Unclaimed Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Potentially Unassigned''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Claimed Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''&#37; LUN Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Total Allocated (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Claimed (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Unclaimed (Thin Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Disk Group Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Disk Group Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Unused Volume Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''&#37; Volume Manager Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''File System Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''File System Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''&#37; File System Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Database Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Database Available''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''&#37; Database Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Virtualization Capacity''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Virtualization Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Storage Pool Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Unused VMDK''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''&#37; Server Virt. Capacity Unused''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Total Allocated''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Allocated Block Storage''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Allocated File Storage''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Storage (LUN) Directly Allocated to Virtual Machine''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Virtual Pool Allocated''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Assigned To Volume Manager''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Directly Assigned to File System''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Directly Assigned to Database''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics'', ''Percent Host Utilization''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Allocated Block Storage (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Claimed (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Unclaimed (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Disk Group Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Disk Group Available (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Volume Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Unused Volume Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''File System Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''File System Available (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Database Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Database Available (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Virtualization Capacity (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Virtualization Unused (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Storage Pool Unused (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Unused VMDK (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Potentially Unassigned (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Allocated File Storage (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Virtual Pool Allocated (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Assigned to Volume Manager (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Directly Assigned to File System (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Directly Assigned to Database (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Storage (LUN) Directly Allocated to Virtual Machine (Cluster Aware)''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Analysis Metrics (Cluster Aware)'', ''Total Allocated (Cluster Aware)''');
commit;


-------------------------------------------------------------------------------
--
-- Location of object attributes based on type.  These values are used to
-- drive the addition and deletion of custom attributes from each hierarchy
-- in the Cognos metamodel.
--
-- IMPORTANT - if you make material changes to the metamodel that changes these
-- relationships in any way (e.g. renaming a namespace, adding/removing dimensions),
-- then you MUST update this configuration information.  Failure to do so will result
-- in failed metamodel updates.
--
message 'populating the cognos hierarchy attribute' to log;
--truncate table SAHARA.COGNOS_HIERARCHY_ATTRIBUTE;
SET @CONFIG = 'insert into SAHARA.COGNOS_HIERARCHY_ATTRIBUTE(NAMESPACE, DIMENSION_NAME, HIERARCHY_NAME, LEVEL_NAME, CUSTOM_ATTRIBUTE_CONTEXT) on existing update values';

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Array Analysis'',                ''By Array Attribute'',             ''By Array Attribute'',             ''Array'',             ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Database Analysis'',             ''By Host'',                        ''By Host'',                        ''Database'',          ''DATABASES'''     );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Database Analysis'',             ''By Host'',                        ''By Host'',                        ''Host'',              ''HOST'''          );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''File System Analysis'',          ''By Host'',                        ''By Host'',                        ''File System'',       ''FILESYSTEM'''    );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''File System Analysis'',          ''By Host'',                        ''By Host'',                        ''Host'',              ''HOST'''          );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''File System Analysis'',          ''By Host'',                        ''By Host'',                        ''Volume'',            ''VOLUME'''        );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Consumption'',              ''By Host Attribute'',              ''By Host Attribute'',              ''Host'',              ''HOST'''          );

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Consumption'',              ''By Cluster and Host'',            ''By Cluster and Host'',            ''Cluster'',           ''CLUSTER'''       );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Consumption'',              ''By Cluster and Host'',            ''By Cluster and Host'',            ''Host'',              ''HOST'''          );

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Host'',                        ''By Host'',                        ''Array'',             ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Host'',                        ''By Host'',                        ''Host'',              ''HOST'''          );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Host'',                        ''By Host'',                        ''LUN'',               ''LUN'''           );

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Cluster and Host'',            ''By Cluster and Host'',            ''Cluster'',           ''CLUSTER'''       );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Cluster and Host'',            ''By Cluster and Host'',            ''Host'',              ''HOST'''          );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Cluster and Host'',            ''By Cluster and Host'',            ''Array'',             ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Cluster and Host'',            ''By Cluster and Host'',            ''LUN'',               ''LUN'''           );

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Cluster, Array and LUN'',       ''By Cluster, Array and LUN'',       ''Cluster'',           ''CLUSTER'''       );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Cluster, Array and LUN'',       ''By Cluster, Array and LUN'',       ''Array'',             ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation'',       ''By Cluster, Array and LUN'',       ''By Cluster, Array and LUN'',       ''LUN'',               ''LUN'''           );


/* CLEAN-UP
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Application Group Consumption'', ''By Application Group Attribute'', ''By Application Group Attribute'', ''Application Group'', ''APP_GROUP'''     );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS QTree Analysis'',         ''By NAS QTree'',                ''By NAS QTree'',                ''NAS QTree'',      ''NETAPP_QTREE'''  );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS QTree Analysis'',         ''By NAS QTree'',                ''By NAS QTree'',                ''NAS Volume'',     ''NETAPP_VOLUME''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS QTree Analysis'',         ''By NAS QTree'',                ''By NAS QTree'',                ''NAS Filer'',            ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Share Analysis'',         ''By NAS Share'',                ''By NAS Share'',                ''NAS QTree'',      ''NETAPP_QTREE'''  );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Share Analysis'',         ''By NAS Share'',                ''By NAS Share'',                ''NAS Share'',      ''NETAPP_SHARE'''  );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Share Analysis'',         ''By NAS Share'',               ''By NAS Share'',               ''NAS Volume'',     ''NETAPP_VOLUME''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Share Analysis'',         ''By NAS Share'',                      ''By NAS Share'',                      ''NAS File System'',            ''NAS_FILESYSTEM'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Share Analysis'',         ''By NAS Share'',                      ''By NAS Share'',                      ''NAS Filer'',            ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Volume Analysis'',        ''By NAS Volume'',               ''By NAS Volume'',               ''NAS Volume'',     ''NETAPP_VOLUME''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Volume Analysis'',        ''By NAS Volume'',               ''By NAS Volume'',               ''NAS Filer'',            ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS File System Analysis'',         ''By NAS File System'',                ''By NAS File System'',                ''NAS File System'',      ''NAS_FILESYSTEM'''  );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS File System Analysis'',         ''By NAS File System'',               ''By NAS File System'',               ''NAS Volume'',     ''NETAPP_VOLUME''' );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS File System Analysis'',         ''By NAS File System'',                      ''By NAS File System'',                      ''NAS Filer'',            ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Switch Analysis'',               ''By Switch Attributes'',           ''By Switch Attributes'',           ''Switch'',            ''SWITCH'''        );
*/
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Storage Analysis'',              ''By Storage Device Attribute'',    ''By Array Attribute'',             ''Array'',             ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Tiered Storage Analysis'',       ''By Array'',                       ''By Array'',                       ''Array'',             ''ARRAY'''         );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Tiered Storage Analysis'',       ''By Array'',                       ''By Array'',                       ''LUN'',               ''LUN'''           );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Volume Analysis'',               ''By Host'',                        ''By Host'',                        ''Host'',              ''HOST'''          );
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Volume Analysis'',               ''By Host'',                        ''By Host'',                        ''Volume'',            ''VOLUME'''        );

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Filesystem'', ''By Filesystem'', ''Host'',''HOST''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Volume'', ''By Volume'', ''Host'',''HOST''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Filesystem'', ''By Filesystem'', ''Filesystem'',''FILESYSTEM''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Volume'', ''By Volume'', ''Volume'',''VOLUME''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Filesystem'', ''By Filesystem'', ''Array'',''ARRAY''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Volume'', ''By Volume'', ''Array'',''ARRAY''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Filesystem'', ''By Filesystem'', ''LUN'',''LUN''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis'', ''By Volume'', ''By Volume'', ''LUN'',''LUN''');

/* CLEAN-UP
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Usage Analysis'', ''By NAS Objects'', ''By NAS Objects'', ''NAS Filer'',''ARRAY''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Usage Analysis'', ''By NAS Objects'', ''By NAS Objects'', ''NAS Volume'',''NETAPP_VOLUME''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Usage Analysis'', ''By NAS Objects'', ''By NAS Objects'', ''NAS QTree'',''NETAPP_QTREE''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Usage Analysis'', ''By NAS Objects'', ''By NAS Objects'', ''NAS File System'',''NAS_FILESYSTEM''');
*/

call sahara.SP_DataDict_CONFIG(@CONFIG,'''License Deployments Analysis'', ''By Product Name'', ''By Product Name'', ''Host'',''HOST''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''License Deployments Analysis'', ''By Host'', ''By Host'', ''Host'',''HOST''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''License And Host Analysis'', ''By Host'', ''By Host'', ''Host'',''HOST''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Disk Group Analysis'', ''By Host'', ''By Host'', ''Host'',''HOST''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Disk Group Analysis'', ''By Host'', ''By Host'', ''Disk Group'',''DISKGROUP''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Disk Group Analysis'', ''By Host'', ''By Host'', ''Array'',''ARRAY''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Disk Group Analysis'', ''By Host'', ''By Host'', ''LUN'',''LUN''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''File System Analysis'', ''By Host'', ''By Host'', ''Disk Group'',''DISKGROUP''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Volume Analysis'', ''By Host'', ''By Host'', ''Disk Group'',''DISKGROUP''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Reclamation Analysis'', ''By Host'', ''By Host'', ''Host'',''HOST''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Cluster Inventory'', ''By Cluster'', ''By Cluster'', ''Cluster'',''CLUSTER''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Cluster Inventory'', ''By Cluster Type'', ''By Cluster Type'', ''Cluster'',''CLUSTER''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Cluster Inventory'', ''By Cluster Platform'', ''By Cluster Platform'', ''Cluster'',''CLUSTER''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Cluster Inventory'', ''By Cluster Version'', ''By Cluster Version'', ''Cluster'',''CLUSTER''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Service Group Inventory'', ''By Cluster and Service Group'', ''By Cluster and Service Group'', ''Cluster'',''CLUSTER''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Service Group Inventory'', ''By Cluster and Service Group'', ''By Cluster and Service Group'', ''Service Group'',''SERVICEGROUP''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Service Group Inventory'', ''By Service Group Type'', ''By Type'', ''Service Group'',''SERVICEGROUP''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Service Group Uptime Analysis'', ''By Service Group'', ''By Service Group'', ''Service Group'',''SERVICEGROUP''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Service Group Uptime Analysis'', ''Cluster'', ''By Cluster'', ''Cluster'',''CLUSTER''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Cluster And Service Group Failover Event Analysis'', ''By Cluster and Service Group'', ''By Cluster and Service Group'', ''Cluster'',''SERVICEGROUP''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Cluster And Service Group Failover Event Analysis'', ''By Cluster and Service Group'', ''By Cluster and Service Group'', ''Service Group'',''CLUSTER''');


commit;

-------------------------------------------------------------------------------
--
-- Cognos packages
--
-- This is a complete list of all the packages we publish in the Cognos content
-- store.  This is used during installation to publish the packages needed by
-- reporting.  It is also used during metamodel updates to publish the updated
-- metamodel information.
--
message 'populating the Cognos package' to log;

--truncate table SAHARA.COGNOS_PACKAGE;
SET @CONFIG = 'insert into SAHARA.COGNOS_PACKAGE(PACKAGE_NAME) on existing update values';

call sahara.SP_DataDict_CONFIG(@CONFIG,'''Advanced Report Package''');
/* CLEAN-UP
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Application Group Consumption''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Switch Analysis''');
*/
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Array Analysis''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Consumption''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Allocation''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Tiered Storage Analysis''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Chargeback Analysis''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''License Analysis''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Disk Group Analysis''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Enterprise Storage Visibility''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Host Storage Reclamation''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''Cluster And Service Group Analysis''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS Analysis''');
commit;

-------------------------------------------------------------------------------
--
-- Cognos presentation shortcuts
--
-- The COGNOS_PRESENTATION_SHORTCUT table contains information needed to support
-- business views.  The underlying business view dimension itself is created in a
-- type-specific folder in the "Presentation View" namespace of the metamodel.
-- For each Enterprise Reporter object type the business view supports, we'll
-- create a shortcut to this business view dimensional hierarchy in one or more
-- folders in the metamodel.  This, in effect, makes this dimensional hierarchy
-- available to the end user for reporting.
--
-- For example, if we create a new business view that supports HOST objects, we'll
-- create the business view hierarchy in the "Host Business Views" folder of the
-- "Presentation View" namespace, and we"ll create shortcuts to this hierarchy
-- in a number of folders, including the "Host Consumption Views" folder in the
-- "Host Consumption" namespace of the metamodel.
--
-- IMPORTANT: If you make material changes to the metamodel that changes these
-- relationships in any way (e.g. renaming a namespace, adding/removing folders),
-- then you MUST update this configuration information.  Failure to do so will result
-- in possible broken reports.
--
message 'populating the Cognos presentation shortcut ' to log;
--truncate table SAHARA.COGNOS_PRESENTATION_SHORTCUT;
SET @CONFIG = 'insert into SAHARA.COGNOS_PRESENTATION_SHORTCUT(OBJECT_TYPE, NAMESPACE, FOLDER_NAME) on existing update values';

-- call sahara.SP_DataDict_CONFIG(@CONFIG,'''APP_GROUP'', ''Application Group Consumption'', ''Application Group Consumption Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Array Analysis'', ''Array Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Host Storage Allocation'', ''Host Storage Allocation Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''Tiered Storage Analysis'', ''Tiered Storage Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Database Analysis'', ''Database Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''File System Analysis'', ''File System Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Consumption'', ''Host Consumption Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Allocation'', ''Host Storage Allocation Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Volume Analysis'', ''Volume Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Disk Group Analysis'', ''Disk Group Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''Host Storage Reclamation Analysis'', ''Host Storage Reclamation Views''');
/* CLEAN-UP
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree Analysis'', ''NAS QTree Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS QTree File Classification Analysis'', ''NAS QTree File Category Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share Analysis'', ''NAS Share Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share Analysis'', ''NAS Share Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree Analysis'', ''NAS QTree Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS QTree File Classification Analysis'', ''NAS QTree File Category Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share Analysis'', ''NAS Share Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Share File Classification Analysis'', ''NAS Share File Category Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume Analysis'', ''NAS Volume Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NAS Volume File Classification Analysis'', ''NAS Volume File Category Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS File System Analysis'', ''NAS File System Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''Switch Analysis'', ''Switch Analysis Views''');
*/
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''File System Analysis'', ''File System Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''Chargeback Analysis'', ''Chargeback Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''Volume Analysis'', ''Volume Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''Chargeback Analysis'', ''Chargeback Analysis Views''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'',''Chargeback Analysis'',''Chargeback Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'',''Chargeback Analysis'',''Chargeback Analysis Views''');
/* CLEAN-UP
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'',''NAS Usage Analysis'',''NAS Object Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'',''NAS Usage Analysis'',''NAS Object Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'',''NAS Usage Analysis'',''NAS Object Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'',''NAS File System Analysis'',''NAS File System Views''');
*/
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'',''License Deployments Analysis'',''License Deployments Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'',''License And Host Analysis'',''Host Summary Views''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'',''NAS And Unified Storage Analysis'',''NAS And Unified Storage Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'',''NAS And Unified Storage Analysis'',''NAS And Unified Storage Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'',''NAS Chargeback Analysis'',''NAS Chargeback Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'',''NAS Chargeback Analysis'',''NAS Chargeback Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'',''NAS Quota Analysis'',''NAS Quota Analysis Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'',''NAS FileSystem Analysis'',''NAS FileSystem Analysis Views''');


call sahara.SP_DataDict_CONFIG(@CONFIG,'''CLUSTER'',''Cluster Inventory'',''Cluster Inventory Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'',''Service Group Inventory'',''Service Group Inventory Views''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'',''Service Group Uptime Analysis'',''Service Group Uptime Views''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'',''Cluster And Service Group Failover Event Analysis'',''Failover Events Views''');

commit;

-------------------------------------------------------------------------------
--
-- Cognos relationship mappings
--
-- The COGNOS_RELATIONSHIP table contains configuration information in support
-- of business views.  For each object type supported by the business view, we
-- must map each of the these object types to the underlying database view query
-- subject for that type.  This table describes this relationship and the
-- cardinality on the left hand side (LHS) and right hand side (RHS) of the
-- relationship.
--
-- For example, if we create a business view that supports HOST objects, then
-- we create a relationship between this new business view and the HOST table.
--
message ' populating the cognos relationship table ..' to log;
--truncate table SAHARA.COGNOS_RELATIONSHIP;
SET @CONFIG = 'insert into SAHARA.COGNOS_RELATIONSHIP(LHS_OBJECT_TYPE, RHS_TABLE, LHS_CARDINALITY_A, LHS_CARDINALITY_B, RHS_CARDINALITY_A, RHS_CARDINALITY_B) on existing update values';

call sahara.SP_DataDict_CONFIG(@CONFIG,'''ARRAY'', ''ARRAY'', ''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''HOST'', ''HOST'', ''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_SHARE'', ''NETAPP_SHARE'', ''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NAS_FILESYSTEM'', ''NAS_FILESYSTEM'',''zero'', ''one'', ''one'', ''one''');
/* CLEAN-UP
call sahara.SP_DataDict_CONFIG(@CONFIG,'''APP_GROUP'', ''APP_GROUP'', ''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_QTREE'', ''NETAPP_QTREE'', ''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''NETAPP_VOLUME'', ''NETAPP_VOLUME'',''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SWITCH'', ''SWITCH'', ''zero'', ''one'', ''one'', ''one''');
*/
call sahara.SP_DataDict_CONFIG(@CONFIG,'''FILESYSTEM'', ''FILESYSTEM'', ''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''VOLUME'', ''VOLUME'', ''zero'', ''one'', ''one'', ''one''');

call sahara.SP_DataDict_CONFIG(@CONFIG,'''CLUSTER'', ''CLUSTER'', ''zero'', ''one'', ''one'', ''one''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''SERVICEGROUP'', ''SERVICEGROUP'', ''zero'', ''one'', ''one'', ''one''');

commit;

-------------------------------------------------------------------------------
--
-- Security Role entries
--
-- The SECURITY_ROLE table stores the roles that are supported in VOM-Advanced.
--
message 'To populate the security roles' to log;
--truncate table SAHARA.SECURITY_ROLE;
SET @CONFIG = 'insert into SAHARA.SECURITY_ROLE(ROLE_ID, NAME) on existing update values';

call sahara.SP_DataDict_CONFIG(@CONFIG,'''0'', ''DomainAdmin''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''1'', ''DomainReportAdmin''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''2'', ''DomainOperator''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''3'', ''Operator''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''4'', ''DomainGuest''');
call sahara.SP_DataDict_CONFIG(@CONFIG,'''5'', ''Guest''');

commit;

-------------------------------------------------------------------------------
--
-- Business Entity Attributes 
--
-- Busines entity attribute values will be populated with custom attribute module stored procedure.
-- For this to work as a tweak manually adding all these BE columns per table name as Source Copy 
-- into P_CUSTOM_ATTRIBUTE (PUBLISHED) table.
--
message 'Populating the business Entity as an Attribute in p_custom_attribute ' to log;
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME) 
on existing update values(10001,'ARRAY','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10002,'ARRAY','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10003,'ARRAY','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10004,'ARRAY','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10005,'HOST','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10006,'HOST','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10007,'HOST','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10008,'HOST','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10009,'LUN','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10010,'LUN','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10011,'LUN','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10012,'LUN','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10013,'DISKGROUP','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10014,'DISKGROUP','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10015,'DISKGROUP','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10016,'DISKGROUP','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10017,'DATABASES','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10018,'DATABASES','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10019,'DATABASES','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10020,'DATABASES','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10021,'FILESYSTEM','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10022,'FILESYSTEM','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10023,'FILESYSTEM','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10024,'FILESYSTEM','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10025,'VOLUME','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10026,'VOLUME','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10027,'VOLUME','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10028,'VOLUME','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10029,'CLUSTER','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10030,'CLUSTER','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10031,'CLUSTER','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10032,'CLUSTER','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10033,'SERVICEGROUP','SOURCE COPY','Business Entity: APPLICATION_BE','APPLICATION_BE','varchar(512)','APPLICATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10034,'SERVICEGROUP','SOURCE COPY','Business Entity: APPLICATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','varchar(512)','APPLICATION_COMPOSITE_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10035,'SERVICEGROUP','SOURCE COPY','Business Entity: ORGANIZATION_BE','ORGANIZATION_BE','varchar(512)','ORGANIZATION_BE');
insert into SAHARA.P_CUSTOM_ATTRIBUTE(CUSTOM_ATTRIBUTE_KEY,CUSTOM_ATTRIBUTE_CONTEXT,TYPE,DISPLAY_NAME,COLUMN_NAME,COLUMN_TYPE,SOURCE_NAME)
on existing update values(10036,'SERVICEGROUP','SOURCE COPY','Business Entity: ORGANIZATION_COMPOSITE_BE','ORGANIZATION_COMPOSITE_BE','varchar(512)','ORGANIZATION_COMPOSITE_BE');

commit;

-------------------------------------------------------------------------------
--
-- roll-up source types

--
message '-- configuring roll-up source types...' to log;
call sahara.SP_DataDict_RollupSourceType('Veritas Operations Manager 4.0 RU1', 'SAHARA.SP_DataLoad_LoadVOM40');
call sahara.SP_DataDict_RollupSourceType('Veritas Operations Manager 4.1', 'SAHARA.SP_DataLoad_LoadVOM40');
call sahara.SP_DataDict_RollupSourceType('Veritas Operations Manager 5.0', 'SAHARA.SP_DataLoad_LoadVOM40');
commit;

-------------------------------------------------------------------------------
--
-- roll-up data set types
--

begin
	message '-- configuring data set APP_GROUP_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'APP_GROUP_DIM',                    -- name
        'DIMENSION',                        -- stereotype
        'APP_GROUP',                        -- destination table
        'APP_GROUP_'                        -- staging table
    );
    call sahara.SP_DataDict_RollupDataSetColumn('APP_GROUP_DIM', 'APP_GROUP_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('APP_GROUP_DIM', 'APP_GROUP_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('APP_GROUP_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.APP_GROUP_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set APP_GROUP_HOST...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'APP_GROUP_HOST',                   -- name
        'FACT',                             -- stereotype
        'APP_GROUP_HOST',                   -- destination table
        'APP_GROUP_HOST_',                  -- staging table


        'APP_GROUP_KEY, HOST_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, APP_GROUP_KEY, HOST_KEY) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, APP_GROUP_CURRENT.APP_GROUP_KEY, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.APP_GROUP_CURRENT on SRC.APP_GROUP_OID = APP_GROUP_CURRENT.APP_GROUP_OID ' ||
          'join SAHARA.HOST_CURRENT on SRC.HOST_OID = HOST_CURRENT.HOST_OID'
    );
end;

commit;

begin
	message '-- configuring data set APP_GROUP_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'APP_GROUP_FACTS',                  -- name
        'FACT',                             -- stereotype
        'APP_GROUP_CAPACITY',               -- destination table
        'APP_GROUP_CAPACITY_',              -- staging table


        'APP_GROUP_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, APP_GROUP_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, APP_GROUP_CURRENT.APP_GROUP_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.APP_GROUP_CURRENT on SRC.APP_GROUP_OID = APP_GROUP_CURRENT.APP_GROUP_OID'
    );
end;

commit;

begin
	message '-- configuring data set ARRAY_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'ARRAY_DIM',                        -- name
        'DIMENSION',                        -- stereotype
        'ARRAY',                            -- destination table
        'ARRAY_',                            -- staging table
		null,
		null,
		'array_dim.csv',
		'vrts_ddl_enclosure'
    );
    call sahara.SP_DataDict_RollupDataSetColumn('ARRAY_DIM', 'ARRAY_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('ARRAY_DIM', 'ARRAY_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('ARRAY_DIM', 'CLUSTER_ARRAY_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('ARRAY_DIM', 'PARENT_ARRAY_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('ARRAY_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.ARRAY_OID || @URL2');
end;

commit;

--
-- This data set doesn't really have a data file loaded for it.  It is
-- a way to arrange for an extra SQL statement to be executed after every
-- array load that might have parent filer array objects.  It was impractical
-- to accomplish this as a trigger as would have been preferred
-- and making it a data set gives us row count logging for free.
--
begin
	message '-- configuring data set ARRAY_DIM_VFILER...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'ARRAY_DIM_VFILER',                       -- name
        'CUSTOM',                           -- stereotype
        'ARRAY',                            -- destination table
		 null,
		 null,

        'update SAHARA.ARRAY CHILD ' ||
           'set PARENT_ARRAY_KEY = (select ARRAY_KEY ' ||
                                     'from SAHARA.ARRAY PARENT ' ||
                                    'where PARENT.ARRAY_OID = CHILD.PARENT_ARRAY_OID ' ||
                                      'and T2_IS_CURRENT = ''T'') ' ||
         'where ARRAY_OID in (select ARRAY_OID ' ||
                               'from SAHARA_APP.ARRAY_ ' ||
                              'where PARENT_ARRAY_OID is not null) ' ||
           'and T2_IS_CURRENT = ''T''; '
    );
end;

commit;

begin
	message '-- configuring data set ARRAY_DIM_CLUSTNODE...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'ARRAY_DIM_CLUSTNODE',                       -- name
        'CUSTOM',                           -- stereotype
        'ARRAY',                            -- destination table
		 null,
		 null,

        'update SAHARA.ARRAY CHILD ' ||
           'set CLUSTER_ARRAY_KEY = (select ARRAY_KEY ' ||
                                     'from SAHARA.ARRAY PARENT ' ||
                                    'where PARENT.ARRAY_OID = CHILD.CLUSTER_ARRAY_OID ' ||
                                      'and T2_IS_CURRENT = ''T'') ' ||
         'where ARRAY_OID in (select ARRAY_OID ' ||
                               'from SAHARA_APP.ARRAY_ ' ||
                              'where CLUSTER_ARRAY_OID is not null) ' ||
           'and T2_IS_CURRENT = ''T''; '
    );
end;

commit;

begin
	message '-- configuring data set ARRAY_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'ARRAY_FACTS',                      -- name
        'FACT',                             -- stereotype
        'ARRAY_STORAGE',                    -- destination table

        'ARRAY_STORAGE_ARRAY_', --TODO: ARRAY_STORAGE_NETAPP_HAL_, ARRAY_STORAGE_NETAPP_DM_,

        'ARRAY_KEY',

        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, ARRAY_KEY, NA_DM_BATCH_KEY, ' ||
             'NA_DM_DATA_AGE, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, ARRAY_CURRENT.ARRAY_KEY, DM_BATCH.DM_BATCH_KEY, ' ||			
            'datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"), $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.ARRAY_CURRENT on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'left outer join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY',
		'array_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set ARRAY_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'ARRAY_DUP_FACTS',                  -- name
        'CUSTOM',                           -- stereotype
        'ARRAY_CLAIMED_CAPACITY',           -- destination table
		null,
		null,

			'merge into SAHARA.ARRAY_CLAIMED_CAPACITY DEST using ' ||
				'(select $ROLLUP_BATCH_KEY$ AS ROLLUP_BATCH_KEY, $DATES_KEY$ AS DATES_KEY, ' ||
				'ROLLUP_BATCH.ROLLUP_SOURCE_KEY as ROLLUP_SOURCE_KEY, ARRAY_CURRENT.ARRAY_KEY as ARRAY_KEY, ' || 
				'PERCENT_ALLOCATED_HOST_CLAIMED AS PERCENT_ALLOCATED_HOST_CLAIMED, CLAIMED_BY_HOSTS AS CLAIMED_BY_HOSTS ' ||  
			'from SAHARA_APP.ARRAY_STORAGE_ARRAY_ SRC ' || 
			'join SAHARA.ARRAY_CURRENT on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' || 
			'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$) SRC on ' || 
				'(SRC.ARRAY_KEY = DEST.ARRAY_KEY AND SRC.ROLLUP_SOURCE_KEY = DEST.ROLLUP_SOURCE_KEY AND SRC.DATES_KEY = DEST.DATES_KEY) ' || 
			'when matched then update set ' || 
				'DEST.ROLLUP_BATCH_KEY = SRC.ROLLUP_BATCH_KEY,  ' ||
				'DEST.PERCENT_ALLOCATED_HOST_CLAIMED = SRC.PERCENT_ALLOCATED_HOST_CLAIMED, ' || 
				'DEST.CLAIMED_BY_HOSTS = SRC.CLAIMED_BY_HOSTS ' || 
			'when not matched then ' || 
				'insert (DEST.ROLLUP_BATCH_KEY, DEST.DATES_KEY, DEST.ROLLUP_SOURCE_KEY, DEST.ARRAY_KEY, ' || 
				'DEST.PERCENT_ALLOCATED_HOST_CLAIMED, DEST.CLAIMED_BY_HOSTS) ' || 
				'values (SRC.ROLLUP_BATCH_KEY, SRC.DATES_KEY, SRC.ROLLUP_SOURCE_KEY, SRC.ARRAY_KEY, ' || 
				'SRC.PERCENT_ALLOCATED_HOST_CLAIMED, SRC.CLAIMED_BY_HOSTS)'
    );
end;

commit;

begin
	message '-- configuring data set CUSTOM_ATTR...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'CUSTOM_ATTR',                      -- name
        'CUSTOM',                           -- stereotype
        'CUSTOM_ATTRIBUTE',                 -- destination table
        'CUSTOM_ATTRIBUTE_',                -- staging table
		null,

        'call SAHARA.SP_DataLoad_LoadCustomAttributes ()',
        'custom_dim.csv'
    );
end;

commit;

begin
	message '-- configuring data set DB_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DB_DIM',                           -- name
        'DIMENSION',                        -- stereotype
        'DATABASES',                        -- destination table
        'DATABASES_',                       -- staging table
		null,

        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
		'database_dim.csv',
		'vrts_dbed_basic'
    );
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'DATABASES_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'DATABASES_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('DB_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.DATABASES_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set DB_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DB_FACTS',                         -- name
        'FACT',                             -- stereotype
        'DATABASE_CAPACITY',                -- destination table
        'DATABASE_CAPACITY_',               -- staging table

        'DATABASES_KEY',

        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, DATABASES_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, DATABASES_CURRENT.DATABASES_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.DATABASES_CURRENT on SRC.DATABASES_OID = DATABASES_CURRENT.DATABASES_OID',
		 'database_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set DM_BATCH...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DM_BATCH',                         -- name
        'CUSTOM',                           -- stereotype
        'DM_BATCH',                         -- destination table
        'DM_BATCH_',                        -- staging table
		null,

        'merge into SAHARA."$DEST_TABLE$" DEST ' ||
        'using ' ||
        '( ' ||
            'select ROLLUP_BATCH.ROLLUP_SOURCE_KEY, SRC.DM_BATCH_ID, ' ||
                   'ROLLUP_BATCH.SOURCE_NAME,  ROLLUP_BATCH.DB_HOST, ROLLUP_BATCH.DB_PORT, ' ||
                   'SRC.TIME, ' ||
                   'ROLLUP_BATCH.UTC_ADJUSTMENT as SOURCE_UTC_ADJUSTMENT, ' ||
                   'CONFIGURATION.DAY_INTERVAL_VALUE as ER_UTC_ADJUSTMENT ' ||
              'from SAHARA_APP.DM_BATCH_ SRC ' ||
              'join SAHARA.ROLLUP_BATCH ' ||
                'on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
              'join SAHARA.CONFIGURATION ' ||
                'on CONFIGURATION.SECTION = ''SERVER'' and CONFIGURATION.NAME = ''UTC_ADJUSTMENT'' ' ||
        ') SRC ' ||
           'on (DEST.ROLLUP_SOURCE_KEY = SRC.ROLLUP_SOURCE_KEY and ' ||
               'DEST.DM_BATCH_ID = SRC.DM_BATCH_ID) ' ||
         'when not matched then ' ||
        'insert(DEST.ROLLUP_SOURCE_KEY, DEST.DM_BATCH_ID, ' ||
               'DEST.SOURCE_NAME, DEST.DB_HOST, DEST.DB_PORT, ' ||
               'DEST.TIME, ' ||
               'DEST.TIME_SOURCE, ' ||
               'DEST.TIME_ER) ' ||
        'values(SRC.ROLLUP_SOURCE_KEY, SRC.DM_BATCH_ID, ' ||
               'SRC.SOURCE_NAME, SRC.DB_HOST, SRC.DB_PORT, ' ||
               'SRC.TIME, ' ||
               'SRC.TIME + SRC.SOURCE_UTC_ADJUSTMENT, ' ||
               'SRC.TIME + SRC.ER_UTC_ADJUSTMENT)'
    );
end;

commit;

begin
	message '-- configuring data set FILE_CATEGORY_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'FILE_CATEGORY_DIM',                -- name
        'DIMENSION',                        -- stereotype
        'FILE_CATEGORY',                    -- destination table
        'FILE_CATEGORY_'                    -- staging table
    );
    call sahara.SP_DataDict_RollupDataSetColumn('FILE_CATEGORY_DIM', 'FILE_CATEGORY_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('FILE_CATEGORY_DIM', 'FILE_CATEGORY_KEY', 'NONE');
end;

commit;

begin
	message '-- configuring data set FS_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'FS_DIM',                           -- name
        'DIMENSION',                        -- stereotype
        'FILESYSTEM',                       -- destination table
        'FILESYSTEM_',                      -- staging table
		null,
	
        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY, VOLUME_CURRENT.VOLUME_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID ' ||
          'left outer join SAHARA.VOLUME_CURRENT ' ||
            'on SRC.VOLUME_OID = VOLUME_CURRENT.VOLUME_OID',
		'filesystem_dim.csv',
		'vrts_vxfs_filesystem'
    );
    call sahara.SP_DataDict_RollupDataSetColumn('FS_DIM', 'FILESYSTEM_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('FS_DIM', 'FILESYSTEM_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('FS_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.FILESYSTEM_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set FS_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'FS_FACTS',                         -- name
        'FACT',                             -- stereotype
        'FILESYSTEM_CAPACITY',              -- destination table
        'FILESYSTEM_CAPACITY_',             -- staging table


        'FILESYSTEM_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, FILESYSTEM_KEY, $DEST_COLUMNS$, ' ||
             'RESERVED_CAPACITY) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, FILESYSTEM_CURRENT.FILESYSTEM_KEY, $SRC_COLUMNS$, ' ||
            -- calculate reserved capacity
            '(CASE WHEN SRC.CAPACITY - SRC.USED_CAPACITY - SRC.FREE_CAPACITY > 0 THEN SRC.CAPACITY - SRC.USED_CAPACITY - SRC.FREE_CAPACITY ELSE 0 END) ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.FILESYSTEM_CURRENT on SRC.FILESYSTEM_OID = FILESYSTEM_CURRENT.FILESYSTEM_OID',
		  'filesystem_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set HOST_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'HOST_DIM',                         -- name
        'DIMENSION',                        -- stereotype
        'HOST',                             -- destination table
        'HOST_',                             -- staging table
		null,
		null,
		'host_dim.csv',
		'vrts_host'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('HOST_DIM', 'HOST_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('HOST_DIM', 'HOST_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('HOST_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.VM_HOST_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set HOST_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'HOST_FACTS',                       -- name
        'FACT',                             -- stereotype
        'HOST_CAPACITY',                    -- destination table
        'HOST_CAPACITY_',                   -- staging table


        'HOST_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, HOST_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, HOST_CURRENT.HOST_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
		  'host_facts.csv'
    );
end;

commit;


/*
 ********************************************
 * Licensing related Dimension/Fact metadata
 ********************************************
 */

/* SF_LICENSE Dimension */
begin
	message '-- configuring data set SF_LICENSE_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SF_LICENSE_DIM',                   -- name
        'DIMENSION',                        -- stereotype
        'SF_LICENSE',                       -- destination table
        'SF_LICENSE_',                      -- staging table
	null,
	null,
	'sf_license_dim.csv'
    );
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_LICENSE_DIM', 'SF_LICENSE_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_LICENSE_DIM', 'SF_LICENSE_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('SF_LICENSE_DIM', 
          'CC_STORAGE_URL', 'TYPE 1', '@URL1 || SRC.SF_LICENSE_OID || @URL2');
end;
commit;

/* SF_PRODUCT Dimension */
begin
	message '-- configuring data set SF_PRODUCT_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SF_PRODUCT_DIM',                   -- name
        'DIMENSION',                        -- stereotype
        'SF_PRODUCT',                       -- destination table
        'SF_PRODUCT_',                      -- staging table
	null,
	null,
	'sf_product_dim.csv'
    );
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_PRODUCT_DIM', 'SF_PRODUCT_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_PRODUCT_DIM', 'SF_PRODUCT_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('SF_PRODUCT_DIM', 
          'CC_STORAGE_URL', 'TYPE 1', '@URL1 || SRC.SF_PRODUCT_OID || @URL2');
end;
commit;

/* SF_LICENSE_HOST Dimension */
begin
	message '-- configuring data set SF_LICENSE_HOST_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SF_LICENSE_HOST_DIM',              -- name
        'DIMENSION',                        -- stereotype
        'SF_LICENSE_HOST',                  -- destination table
        'SF_LICENSE_HOST_',                 -- staging table
	null,
        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
        'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
        'left outer join SAHARA.HOST_CURRENT ' ||
        'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
	'sf_license_host_dim.csv'
    );
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_LICENSE_HOST_DIM', 'SF_LICENSE_HOST_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_LICENSE_HOST_DIM', 'SF_LICENSE_HOST_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('SF_LICENSE_HOST_DIM', 
          'CC_STORAGE_URL', 'TYPE 1', '@URL1 || SRC.SF_LICENSE_HOST_OID || @URL2');
end;
commit;


/* SF_LICENSE_DEPLOYMENT Dimension */
begin
	message '-- configuring data set SF_LICENSE_DEPLOYMENT_DIM...' 
          to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
     'SF_LICENSE_DEPLOYMENT_DIM',        -- name
     'DIMENSION',                        -- stereotype
     'SF_LICENSE_DEPLOYMENT',            -- destination table
     'SF_LICENSE_DEPLOYMENT_',           -- staging table
     null,
     'select $SRC_COLUMNS$, ' ||
     '       SF_LICENSE_CURRENT.SF_LICENSE_KEY, ' ||
     '       SF_PRODUCT_CURRENT.SF_PRODUCT_KEY, ' ||
     '       SF_LICENSE_HOST_CURRENT.SF_LICENSE_HOST_KEY ' ||
     'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
     'join SAHARA.SF_LICENSE_CURRENT ' ||
     'on SRC.SF_LICENSE_OID = SF_LICENSE_CURRENT.SF_LICENSE_OID ' ||
     'join SAHARA.SF_PRODUCT_CURRENT ' ||
     'on SRC.SF_PRODUCT_OID = SF_PRODUCT_CURRENT.SF_PRODUCT_OID ' ||
     'join SAHARA.SF_LICENSE_HOST_CURRENT ' ||
     'on SRC.SF_LICENSE_HOST_OID = SF_LICENSE_HOST_CURRENT.SF_LICENSE_HOST_OID',
     'sf_lic_deployment_dim.csv'
    );
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_LICENSE_DEPLOYMENT_DIM', 'SF_LICENSE_DEPLOYMENT_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn (
          'SF_LICENSE_DEPLOYMENT_DIM', 'SF_LICENSE_DEPLOYMENT_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('SF_LICENSE_DEPLOYMENT_DIM', 
          'CC_STORAGE_URL', 'TYPE 1', '@URL1 || SRC.SF_LICENSE_DEPLOYMENT_OID || @URL2');
end;
commit;

/* SF_DEPLOYED_LICENSES fact */
begin
	message '-- configuring data set SF_DEPLOYED_LICENSES...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
      'SF_DEPLOYED_LICENSES_FACTS',    -- name
      'FACT',                          -- stereotype
      'SF_DEPLOYED_LICENSES',          -- destination table
      'SF_DEPLOYED_LICENSES_',         -- staging table

      'SF_LICENSE_KEY ||''@''|| SF_PRODUCT_KEY ||''@''|| SF_LICENSE_HOST_KEY',

      'insert into SAHARA."$DEST_TABLE$" ' ||
      '    (ROLLUP_BATCH_KEY, DATES_KEY, SF_LICENSE_KEY, SF_PRODUCT_KEY, ' ||
      '     SF_LICENSE_HOST_KEY, SF_LICENSE_DEPLOYMENT_KEY, $DEST_COLUMNS$) ' ||
      'select ' ||
      '   $ROLLUP_BATCH_KEY$, $DATES_KEY$, ' ||
      '   SF_LICENSE_CURRENT.SF_LICENSE_KEY, ' ||
      '   SF_PRODUCT_CURRENT.SF_PRODUCT_KEY, ' ||
      '   SF_LICENSE_HOST_CURRENT.SF_LICENSE_HOST_KEY, ' ||
      '   SF_LICENSE_DEPLOYMENT_CURRENT.SF_LICENSE_DEPLOYMENT_KEY, ' ||
      '      $SRC_COLUMNS$ ' ||
      'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
      'join SAHARA.SF_LICENSE_CURRENT on ' ||
      ' SRC.SF_LICENSE_OID = SF_LICENSE_CURRENT.SF_LICENSE_OID ' ||
      'join SAHARA.SF_PRODUCT_CURRENT on ' ||
      ' SRC.SF_PRODUCT_OID = SF_PRODUCT_CURRENT.SF_PRODUCT_OID ' ||
      'join SAHARA.SF_LICENSE_HOST_CURRENT on ' ||
      ' SRC.SF_LICENSE_HOST_OID = SF_LICENSE_HOST_CURRENT.SF_LICENSE_HOST_OID ' ||
      'join SAHARA.SF_LICENSE_DEPLOYMENT_CURRENT on ' ||
      ' SRC.SF_LICENSE_DEPLOYMENT_OID = SF_LICENSE_DEPLOYMENT_CURRENT.SF_LICENSE_DEPLOYMENT_OID',

      'sf_deployed_licenses_facts.csv'
    );
end;

commit;

/********************* END Licensing DIM/Fact metadata ********************/


begin
	message '-- configuring data set HOST_ARRAY_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'HOST_ARRAY_FACTS',                 -- name
        'FACT',                             -- stereotype
        'HOST_ARRAY_CAPACITY',              -- destination table
        'HOST_ARRAY_CAPACITY_',             -- staging table


        'HOST_KEY, ARRAY_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, HOST_KEY, ARRAY_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, HOST_CURRENT.HOST_KEY, ARRAY_CURRENT.ARRAY_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT on SRC.HOST_OID = HOST_CURRENT.HOST_OID ' ||
          'join SAHARA.ARRAY_CURRENT on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID'
    );
end;

commit;

begin
	message '-- configuring data set LUN_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'LUN_DIM',                          -- name
        'DIMENSION',                        -- stereotype
        'LUN',                              -- destination table
        'LUN_',                             -- staging table
		
		null,

        'select $SRC_COLUMNS$, ARRAY_CURRENT.ARRAY_KEY, NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'left outer join SAHARA.ARRAY_CURRENT ' ||
            'on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' ||
          'left outer join SAHARA.NAS_FILESYSTEM_CURRENT ' ||
            'on SRC.NAS_FILESYSTEM_OID = NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_OID',
		'lun_dim.csv',
		'vrts_sfms_lun'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('LUN_DIM', 'LUN_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('LUN_DIM', 'LUN_KEY', 'NONE');
	call sahara.SP_DataDict_RollupDataSetColumn ('LUN_DIM', 'NAS_FILESYSTEM_KEY', 'TYPE 1');
    call sahara.SP_DataDict_RollupDataSetColumn ('LUN_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.LUN_OID || @URL2');
    call sahara.SP_DataDict_RollupDataSetColumn ('LUN_DIM', 'TIER', 'TYPE 2');
    call sahara.SP_DataDict_RollupDataSetColumn ('LUN_DIM', 'TIER_COST_PER_GB', 'TYPE 2');
end;

commit;

begin
	message '-- configuring data set LUN_HOST...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'LUN_HOST',                         -- name
        'FACT',                             -- stereotype
        'LUN_HOST',                         -- destination table
        'LUN_HOST_',                        -- staging table


        'LUN_KEY ||''@''|| HOST_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, LUN_KEY, HOST_KEY, ' ||
              'LUN_CAPACITY, ALLOCATED_TO_HOST_DIRECTLY, ADJUSTED_ALLOCATED_CAPACITY, ' ||
              'TOTAL_COST, ' ||
              'ALLOCATED_HOSTS, CLAIMED_HOSTS, PHYSICAL_CAPACITY, ' ||
              'C_LUN_CAPACITY, C_ALLOCATED_TO_HOST_DIRECTLY, C_ADJUSTED_ALLOCATED_CAPACITY, ' ||
              'C_TOTAL_COST, ' ||
              'C_PHYSICAL_CAPACITY) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, LUN_CURRENT.LUN_KEY, HOST_CURRENT.HOST_KEY, ' ||
             -- copied columns
             'SRC2.LUN_CAPACITY, SRC2.ALLOCATED_TO_HOST_DIRECTLY, SRC2.ADJUSTED_ALLOCATED_CAPACITY, ' ||
             -- calculate total cost
             'SRC2.LUN_CAPACITY * LUN_CURRENT.TIER_COST_PER_GB, ' ||
             -- more copied columns
             'SRC2.ALLOCATED_HOSTS, SRC2.CLAIMED_HOSTS, SRC2.PHYSICAL_CAPACITY, ' ||
             -- copied thin provision related columns
             'SRC2.C_LUN_CAPACITY, SRC2.C_ALLOCATED_TO_HOST_DIRECTLY, SRC2.C_ADJUSTED_ALLOCATED_CAPACITY, ' ||
             -- calculate thin provision related total cost
             'SRC2.C_LUN_CAPACITY * LUN_CURRENT.TIER_COST_PER_GB, ' ||
             -- more copied thin provision related column
             'SRC2.C_PHYSICAL_CAPACITY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA_APP.LUN_CAPACITY_ SRC2 on SRC.LUN_OID = SRC2.LUN_OID ' ||
          'join SAHARA.LUN_CURRENT on SRC.LUN_OID = LUN_CURRENT.LUN_OID ' ||
          'join SAHARA.HOST_CURRENT on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
		  'lun_host_dim.csv'
    );
end;

commit;

begin
	message '-- configuring data set LUN_FILESYSTEM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'LUN_FILESYSTEM',                         -- name
        'FACT',                                   -- stereotype
        'LUN_FILESYSTEM',                         -- destination table
        'LUN_FILESYSTEM_',                        -- staging table


        'FILESYSTEM_KEY ||''@''|| LUN_KEY ||''@''|| ARRAY_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, FILESYSTEM_KEY, LUN_KEY, ARRAY_KEY) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, FILESYSTEM_CURRENT.FILESYSTEM_KEY, LUN_CURRENT.LUN_KEY, ARRAY_CURRENT.ARRAY_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.FILESYSTEM_CURRENT on SRC.FILESYSTEM_OID = FILESYSTEM_CURRENT.FILESYSTEM_OID ' ||
          'join SAHARA.LUN_CURRENT on SRC.LUN_OID = LUN_CURRENT.LUN_OID ' ||
          'left outer join SAHARA.ARRAY_CURRENT on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID',
		  'filesystemtolun_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set LUN_DISKGROUP...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'LUN_DISKGROUP',                         -- name
        'FACT',                               -- stereotype
        'LUN_DISKGROUP',                         -- destination table
        'LUN_DISKGROUP_',                        -- staging table


        'DISKGROUP_KEY ||''@''|| LUN_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, DISKGROUP_KEY, LUN_KEY, ARRAY_KEY) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, DISKGROUP_CURRENT.DISKGROUP_KEY, LUN_CURRENT.LUN_KEY, ARRAY_CURRENT.ARRAY_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.DISKGROUP_CURRENT on SRC.DISKGROUP_OID = DISKGROUP_CURRENT.DISKGROUP_OID ' ||
          'join SAHARA.LUN_CURRENT on SRC.LUN_OID = LUN_CURRENT.LUN_OID ' ||
          'left outer join SAHARA.ARRAY_CURRENT on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID',
		  'diskgrouptolun_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set LUN_VOLUME...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'LUN_VOLUME',                         -- name
        'FACT',                               -- stereotype
        'LUN_VOLUME',                         -- destination table
        'LUN_VOLUME_',                        -- staging table


        'VOLUME_KEY ||''@''|| LUN_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, VOLUME_KEY, LUN_KEY, ARRAY_KEY) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, VOLUME_CURRENT.VOLUME_KEY, LUN_CURRENT.LUN_KEY, ARRAY_CURRENT.ARRAY_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.VOLUME_CURRENT on SRC.VOLUME_OID = VOLUME_CURRENT.VOLUME_OID ' ||
          'join SAHARA.LUN_CURRENT on SRC.LUN_OID = LUN_CURRENT.LUN_OID ' ||
          'left outer join SAHARA.ARRAY_CURRENT on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID',
		  'volumetolun_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set LUN_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'LUN_FACTS',                        -- name
        'FACT',                             -- stereotype
        'LUN_CAPACITY',                     -- destination table
        'LUN_CAPACITY_',                    -- staging table


        'LUN_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, LUN_KEY, $DEST_COLUMNS$, ' ||
             'TOTAL_COST) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, LUN_CURRENT.LUN_KEY, $SRC_COLUMNS$, ' ||
             'SRC.LUN_CAPACITY * LUN_CURRENT.TIER_COST_PER_GB ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.LUN_CURRENT on SRC.LUN_OID = LUN_CURRENT.LUN_OID',
		  'lun_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set LUN_DUP_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'LUN_DUP_DIM',                      -- name
        'CUSTOM',                           -- stereotype
        'LUN_CLAIMED_STATES',               -- destination table
		null,
		null,


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, LUN_KEY, CLAIMED_STATUS) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, LUN_CURRENT.LUN_KEY, ' ||
			'(CASE WHEN LUN_CURRENT.CLAIMED_STATUS = ''UNKNOWN'' THEN 0 ' ||
				'WHEN LUN_CURRENT.CLAIMED_STATUS = ''UNCLAIMED'' THEN 1 ' ||
				'ELSE 2 END) AS ENUM_CLAIMED_STATE ' ||
          'from SAHARA_APP.LUN_ SRC ' ||
          'join SAHARA.LUN_CURRENT on SRC.LUN_OID = LUN_CURRENT.LUN_OID'
    );
end;

commit;

--
-- this data set gets merged with ARRAY_FACTS via the ARRAY_STORAGE view at
-- load time and is really loaded with that data set
--
begin
	message '-- configuring data set NETAPP_DM_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_DM_FACTS',                  -- name
        'CUSTOM',                           -- stereotype
        'ARRAY_STORAGE',                    -- destination table
        'ARRAY_STORAGE_NETAPP_DM_'          -- staging table
    );
end;

commit;

-- this data set gets merges with ARRAY_FACTS via the ARRAY_STORAGE view at
-- load time and is really loaded with that data set
begin
	message '-- configuring data set NETAPP_HAL_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_HAL_FACTS',                 -- name
        'CUSTOM',                           -- stereotype
        'ARRAY_STORAGE',                    -- destination table
        'ARRAY_STORAGE_NETAPP_HAL_'         -- staging table
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_FILE_CATEGORY_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_FILE_CATEGORY_FACTS',       -- name
        'FACT',                             -- stereotype
        'NETAPP_FILE_FCAT',                 -- destination table
        'NETAPP_FILE_FCAT_DM_',             -- staging table


        'ARRAY_KEY, FILE_CATEGORY_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, ARRAY_KEY, ' ||
             'FILE_CATEGORY_KEY, DM_BATCH_KEY, ' ||
             'DM_DATA_AGE, ' ||
             '$DEST_COLUMNS$) ' ||
        'select ' ||
             '$ROLLUP_BATCH_KEY$, $DATES_KEY$, ARRAY_CURRENT.ARRAY_KEY, ' ||
             'FILE_CATEGORY_CURRENT.FILE_CATEGORY_KEY, DM_BATCH.DM_BATCH_KEY, ' ||			 
             'datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"), ' ||
             '$SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.ARRAY_CURRENT ' ||
            'on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' ||
          'join SAHARA.FILE_CATEGORY_CURRENT ' ||
            'on SRC.FILE_CATEGORY_OID = FILE_CATEGORY_CURRENT.FILE_CATEGORY_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY'
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_QTREE_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_QTREE_DIM',                 -- name
        'DIMENSION',                        -- stereotype
        'NETAPP_QTREE',                     -- destination table
        'NETAPP_QTREE_',                    -- staging table
		null,

        'select $SRC_COLUMNS$, ' ||
               'ARRAY_CURRENT.ARRAY_KEY, ' ||
               'NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.ARRAY_CURRENT ' ||
            'on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' ||
          'join SAHARA.NETAPP_VOLUME_CURRENT ' ||
            'on SRC.NETAPP_VOLUME_OID = NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_OID'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_QTREE_DIM', 'NETAPP_QTREE_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_QTREE_DIM', 'NETAPP_QTREE_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_QTREE_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.NETAPP_QTREE_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set NETAPP_QTREE_FILE_CATEGORY_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_QTREE_FILE_CATEGORY_FACTS', -- name
        'FACT',                             -- stereotype
        'NETAPP_QTREE_FILE_FCAT',           -- destination table
        'NETAPP_QTREE_FILE_FCAT_DM_',       -- staging table


        'NETAPP_QTREE_KEY, FILE_CATEGORY_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NETAPP_QTREE_KEY, ' ||
             'FILE_CATEGORY_KEY, DM_BATCH_KEY, ' ||
             'DM_DATA_AGE, ' ||
             '$DEST_COLUMNS$) ' ||
        'select ' ||
             '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NETAPP_QTREE_CURRENT.NETAPP_QTREE_KEY, ' ||
             'FILE_CATEGORY_CURRENT.FILE_CATEGORY_KEY, DM_BATCH.DM_BATCH_KEY, ' ||
             'datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"), ' ||
             '$SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_QTREE_CURRENT ' ||
            'on SRC.NETAPP_QTREE_OID = NETAPP_QTREE_CURRENT.NETAPP_QTREE_OID ' ||
          'join SAHARA.FILE_CATEGORY_CURRENT ' ||
            'on SRC.FILE_CATEGORY_OID = FILE_CATEGORY_CURRENT.FILE_CATEGORY_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY'
    );
end;

commit;

-- this data set gets merges with NETAPP_QTREE_FILE_HAL_FACTS at load time
-- and is really loaded with that data set
begin
	message '-- configuring data set NETAPP_QTREE_FILE_DM_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_QTREE_FILE_DM_FACTS',       -- name
        'CUSTOM',                           -- stereotype
        'NETAPP_QTREE_FILE',                -- destination table
        'NETAPP_QTREE_FILE_DM_'             -- staging table
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_QTREE_FILE_HAL_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_QTREE_FILE_HAL_FACTS',      -- name
        'FACT',                             -- stereotype
        'NETAPP_QTREE_FILE',                -- destination table


        'NETAPP_QTREE_FILE_DM_',--TODO: NETAPP_QTREE_FILE_HAL_


        'NETAPP_QTREE_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NETAPP_QTREE_KEY, ' ||
             'DM_BATCH_KEY, ' ||
             'DM_DATA_AGE, ' ||
             '$DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NETAPP_QTREE_CURRENT.NETAPP_QTREE_KEY, ' ||
            'DM_BATCH.DM_BATCH_KEY, ' ||
            'datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"), ' ||
            '$SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_QTREE_CURRENT ' ||
            'on SRC.NETAPP_QTREE_OID = NETAPP_QTREE_CURRENT.NETAPP_QTREE_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'left outer join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY'
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_SHARE_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_SHARE_DIM',                 -- name
        'DIMENSION',                        -- stereotype
        'NETAPP_SHARE',                     -- destination table
        'NETAPP_SHARE_',                    -- staging table
		null,


        'select $SRC_COLUMNS$, ' ||
               'ARRAY_CURRENT.ARRAY_KEY, ' ||
               'NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_KEY, ' ||
               'NETAPP_QTREE_CURRENT.NETAPP_QTREE_KEY, ' ||
               'NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.ARRAY_CURRENT ' ||
            'on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' ||
          'left outer join SAHARA.NETAPP_VOLUME_CURRENT ' ||
            'on SRC.NETAPP_VOLUME_OID = NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_OID ' ||
          'left outer join SAHARA.NETAPP_QTREE_CURRENT ' ||
            'on SRC.NETAPP_QTREE_OID = NETAPP_QTREE_CURRENT.NETAPP_QTREE_OID ' ||
          'left outer join SAHARA.NAS_FILESYSTEM_CURRENT ' ||
            'on SRC.NAS_FILESYSTEM_OID = NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_OID',
		'nas_share_dim.csv', 'vrts_array_nasshare'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_SHARE_DIM', 'NETAPP_SHARE_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_SHARE_DIM', 'NETAPP_SHARE_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_SHARE_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.NETAPP_SHARE_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set NETAPP_SHARE_FILE_CATEGORY_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_SHARE_FILE_CATEGORY_FACTS', -- name
        'FACT',                             -- stereotype
        'NETAPP_SHARE_FILE_FCAT',           -- destination table
        'NETAPP_SHARE_FILE_FCAT_DM_',       -- staging table


        'NETAPP_SHARE_KEY, FILE_CATEGORY_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NETAPP_SHARE_KEY, ' ||
             'FILE_CATEGORY_KEY, DM_BATCH_KEY, ' ||
             'DM_DATA_AGE, ' ||
             '$DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NETAPP_SHARE_CURRENT.NETAPP_SHARE_KEY, ' ||
            'FILE_CATEGORY_CURRENT.FILE_CATEGORY_KEY, DM_BATCH.DM_BATCH_KEY, ' ||
            'datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"), ' ||
            '$SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_SHARE_CURRENT ' ||
            'on SRC.NETAPP_SHARE_OID = NETAPP_SHARE_CURRENT.NETAPP_SHARE_OID ' ||
          'join SAHARA.FILE_CATEGORY_CURRENT ' ||
            'on SRC.FILE_CATEGORY_OID = FILE_CATEGORY_CURRENT.FILE_CATEGORY_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY'
    );
end;

commit;

-- this data set gets merges with NETAPP_SHARE_FILE_HAL_FACTS at load time
-- and is really loaded with that data set
begin
	message '-- configuring data set NETAPP_SHARE_FILE_DM_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_SHARE_FILE_DM_FACTS',       -- name
        'CUSTOM',                           -- stereotype
        'NETAPP_SHARE_FILE',                -- destination table
        'NETAPP_SHARE_FILE_DM_'             -- staging table
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_SHARE_FILE_HAL_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_SHARE_FILE_HAL_FACTS',      -- name
        'FACT',                             -- stereotype
        'NETAPP_SHARE_FILE',                -- destination table


        'NETAPP_SHARE_FILE_HAL_', --TODO: NETAPP_SHARE_FILE_DM_',


        'NETAPP_SHARE_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NETAPP_SHARE_KEY, ' ||
             'DM_BATCH_KEY, ' ||
             'DM_DATA_AGE, ' ||
             '$DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NETAPP_SHARE_CURRENT.NETAPP_SHARE_KEY, ' ||
            'isnull(DM_BATCH.DM_BATCH_KEY,0), ' ||
            'isnull(datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"),0), ' ||
            '$SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_SHARE_CURRENT ' ||
            'on SRC.NETAPP_SHARE_OID = NETAPP_SHARE_CURRENT.NETAPP_SHARE_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'left outer join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY',
		  'nas_share_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_VOLUME_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_VOLUME_DIM',                -- name
        'DIMENSION',                        -- stereotype
        'NETAPP_VOLUME',                    -- destination table
        'NETAPP_VOLUME_',                   -- staging table
		null,


        'select $SRC_COLUMNS$, ARRAY_CURRENT.ARRAY_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.ARRAY_CURRENT ' ||
            'on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID',
		'nas_volume_dim.csv', 'vrts_array_storevol'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_VOLUME_DIM', 'NETAPP_VOLUME_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_VOLUME_DIM', 'NETAPP_VOLUME_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('NETAPP_VOLUME_DIM', 'STORAGE_POOL_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('NETAPP_VOLUME_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.NETAPP_VOLUME_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set NETAPP_VOLUME_DIM2...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_VOLUME_DIM2',                       -- name
        'CUSTOM',                           -- stereotype
        'NETAPP_VOLUME',                            -- destination table
		 null,
		 null,

        'update SAHARA.NETAPP_VOLUME_CURRENT CHILD ' ||
           'set STORAGE_POOL_KEY = (select NETAPP_VOLUME_KEY ' ||
                                     'from SAHARA.NETAPP_VOLUME_CURRENT PARENT ' ||
                                    'where PARENT.NETAPP_VOLUME_OID = CHILD.STORAGE_POOL_OBJECT_IDENTIFIER ' ||
                                      ') ' ||
         'where NETAPP_VOLUME_OID in (select NETAPP_VOLUME_OID ' ||
                               'from SAHARA_APP.NETAPP_VOLUME_ ' ||
                              'where STORAGE_POOL_OBJECT_IDENTIFIER is not null); '
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_VOLUME_FILE_CATEGORY_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_VOLUME_FILE_CATEGORY_FACTS',-- name
        'FACT',                             -- stereotype
        'NETAPP_VOLUME_FILE_FCAT',          -- destination table
        'NETAPP_VOLUME_FILE_FCAT_DM_',      -- staging table


        'NETAPP_VOLUME_KEY, FILE_CATEGORY_KEY',

        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NETAPP_VOLUME_KEY, ' ||
             'FILE_CATEGORY_KEY, DM_BATCH_KEY, ' ||
             'DM_DATA_AGE, ' ||
             '$DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_KEY, ' ||
            'FILE_CATEGORY_CURRENT.FILE_CATEGORY_KEY, DM_BATCH.DM_BATCH_KEY, ' ||
            'datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"), ' ||
            '$SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_VOLUME_CURRENT ' ||
            'on SRC.NETAPP_VOLUME_OID = NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_OID ' ||
          'join SAHARA.FILE_CATEGORY_CURRENT ' ||
            'on SRC.FILE_CATEGORY_OID = FILE_CATEGORY_CURRENT.FILE_CATEGORY_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY'
    );
end;

commit;

-- this data set gets merges with NETAPP_VOLUME_FILE_HAL_FACTS at load time
-- and is really loaded with that data set
begin
	message '-- configuring data set NETAPP_VOLUME_FILE_DM_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_VOLUME_FILE_DM_FACTS',      -- name
        'CUSTOM',                           -- stereotype
        'NETAPP_VOLUME_FILE',               -- destination table
        'NETAPP_VOLUME_FILE_DM_'            -- staging table
    );
end;

commit;

begin
	message '-- configuring data set NETAPP_VOLUME_FILE_HAL_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NETAPP_VOLUME_FILE_HAL_FACTS',     -- name
        'FACT',                             -- stereotype
        'NETAPP_VOLUME_FILE',               -- destination table


        'NETAPP_VOLUME_FILE_HAL_',--TODO: NETAPP_VOLUME_FILE_DM_',


        'NETAPP_VOLUME_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NETAPP_VOLUME_KEY, ' ||
             'DM_BATCH_KEY, ' ||
             'DM_DATA_AGE, ' ||
             '$DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_KEY, ' ||
            'DM_BATCH.DM_BATCH_KEY, ' ||
            'datediff(second,ROLLUP_BATCH.START_TIME,DM_BATCH."TIME"), ' ||
            '$SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_VOLUME_CURRENT ' ||
            'on SRC.NETAPP_VOLUME_OID = NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_OID ' ||
          'join SAHARA.ROLLUP_BATCH on ROLLUP_BATCH.ROLLUP_BATCH_KEY = $ROLLUP_BATCH_KEY$ ' ||
          'left outer join SAHARA.DM_BATCH on SRC.DM_BATCH_ID = DM_BATCH.DM_BATCH_ID ' ||
           'and ROLLUP_BATCH.ROLLUP_SOURCE_KEY = DM_BATCH.ROLLUP_SOURCE_KEY',
		   'nas_volume_facts.csv'
    );
end;

commit;

begin
    message '-- configuring data set NAS_QUOTA_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NAS_QUOTA_DIM',                 -- name
        'DIMENSION',                     -- stereotype
        'NAS_QUOTA',                     -- destination table
        'NAS_QUOTA_',                    -- staging table
	null,


        'select $SRC_COLUMNS$, ' ||
               'ARRAY_CURRENT.ARRAY_KEY, ' ||
               'NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.ARRAY_CURRENT ' ||
            'on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' ||
          'join SAHARA.NAS_FILESYSTEM_CURRENT ' ||
            'on SRC.NAS_FILESYSTEM_OID = NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_OID',
	'nas_quota_dim.csv', 'vrts_array_nasquota'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('NAS_QUOTA_DIM', 'NAS_QUOTA_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('NAS_QUOTA_DIM', 'NAS_QUOTA_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('NAS_QUOTA_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.NAS_QUOTA_OID || @URL2');
end;

commit;

begin
    message '-- configuring data set NAS_QUOTA_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NAS_QUOTA_FACTS',	       -- name
        'FACT',                        -- stereotype
        'NAS_QUOTA_CAPACITY',          -- destination table
        'NAS_QUOTA_CAPACITY_',         -- staging table


        'NAS_QUOTA_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NAS_QUOTA_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NAS_QUOTA_CURRENT.NAS_QUOTA_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NAS_QUOTA_CURRENT on SRC.NAS_QUOTA_OID = NAS_QUOTA_CURRENT.NAS_QUOTA_OID',
	'nas_quota_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set NAS_VOLUME_FILER...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NAS_VOLUME_FILER',                      -- name
        'CUSTOM',                           -- stereotype
        'NAS_VOLUME_FILER',               -- destination table
		'NAS_VOLUME_FILER_',			  -- staging table 	
		null,


		'insert into SAHARA."$DEST_TABLE$" ' ||
            '(NETAPP_VOLUME_KEY, ARRAY_KEY) on existing update ' ||
        'select ' ||
            'NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_KEY, ARRAY_CURRENT.ARRAY_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_VOLUME_CURRENT on SRC.NETAPP_VOLUME_OID = NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_OID ' ||
          'join SAHARA.ARRAY_CURRENT on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID; ',
		'volumetofiler_dim.csv'
    );
end;

commit;


begin
	message '-- configuring data set NAS_SHARE_HOST...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NAS_SHARE_HOST',                      -- name
        'CUSTOM',                          -- stereotype
        'NAS_SHARE_HOST',	               -- destination table
		'NAS_SHARE_HOST_',				   -- staging table 	
		null,


		'insert into SAHARA."$DEST_TABLE$" ' ||
            '(NETAPP_SHARE_KEY, HOST_KEY, ACCESS_TYPE) on existing update ' ||
        'select ' ||
            'NETAPP_SHARE_CURRENT.NETAPP_SHARE_KEY, HOST_CURRENT.HOST_KEY, SRC.ACCESS_TYPE ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_SHARE_CURRENT on SRC.NETAPP_SHARE_OID = NETAPP_SHARE_CURRENT.NETAPP_SHARE_OID ' ||
          'join SAHARA.HOST_CURRENT on SRC.HOST_OID = HOST_CURRENT.HOST_OID; ',
		'hosttoshare_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set NAS_SHARE_HOSTFS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NAS_SHARE_HOSTFS',                     -- name
        'CUSTOM',								-- stereotype
        'NAS_SHARE_HOST_FILESYSTEM',            -- destination table
		'NAS_SHARE_HOST_FILESYSTEM_',			-- staging table 	
		null,


		'insert into SAHARA."$DEST_TABLE$" ' ||
            '(NETAPP_SHARE_KEY, FILESYSTEM_KEY) on existing update ' ||
        'select ' ||
            'NETAPP_SHARE_CURRENT.NETAPP_SHARE_KEY, FILESYSTEM_CURRENT.FILESYSTEM_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NETAPP_SHARE_CURRENT on SRC.NETAPP_SHARE_OID = NETAPP_SHARE_CURRENT.NETAPP_SHARE_OID ' ||
          'join SAHARA.FILESYSTEM_CURRENT on SRC.FILESYSTEM_OID = FILESYSTEM_CURRENT.FILESYSTEM_OID; ',
		'hostfilesystemtoshare_dim.csv'
    );
end;

commit;

begin
	message '-- configuring data set SWITCH_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SWITCH_DIM',                       -- name
        'DIMENSION',                        -- stereotype
        'SWITCH',                           -- destination table
        'SWITCH_'                           -- staging table
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('SWITCH_DIM', 'SWITCH_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('SWITCH_DIM', 'SWITCH_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('SWITCH_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.SWITCH_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set SWITCH_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SWITCH_FACTS',                         -- name
        'FACT',                                 -- stereotype
        'SWITCH_PORT_COUNTS',                   -- destination table
        'SWITCH_PORT_COUNTS_',                  -- staging table


        'SWITCH_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, SWITCH_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, SWITCH_CURRENT.SWITCH_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.SWITCH_CURRENT on SRC.SWITCH_OID = SWITCH_CURRENT.SWITCH_OID'
    );
end;

commit;


begin
	message '-- configuring data set DISKGROUP_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DG_DIM',                       -- name
        'DIMENSION',                        -- stereotype
        'DISKGROUP',                           -- destination table
        'DISKGROUP_',                          -- staging table
		null,


        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',
		'diskgroup_dim.csv',
		'vrts_vxvm_diskgroup'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'DISKGROUP_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'DISKGROUP_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('DG_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.DISKGROUP_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set DISKGROUP_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'DG_FACTS',                     -- name
        'FACT',                             -- stereotype
        'DISKGROUP_CAPACITY',                  -- destination table
        'DISKGROUP_CAPACITY_',                 -- staging table


        'DISKGROUP_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, DISKGROUP_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, DISKGROUP_CURRENT.DISKGROUP_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.DISKGROUP_CURRENT on SRC.DISKGROUP_OID = DISKGROUP_CURRENT.DISKGROUP_OID',
		'diskgroup_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set VOLUME_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'VOLUME_DIM',                       -- name
        'DIMENSION',                        -- stereotype
        'VOLUME',                           -- destination table
        'VOLUME_',                          -- staging table
		null,


--        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY ' ||
--          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
--          'join SAHARA.HOST_CURRENT ' ||
--            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID'

        'select $SRC_COLUMNS$, HOST_CURRENT.HOST_KEY, DISKGROUP_CURRENT.DISKGROUP_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID ' ||
          'left outer join SAHARA.DISKGROUP_CURRENT ' ||
            'on SRC.DISKGROUP_OID = DISKGROUP_CURRENT.DISKGROUP_OID',
		'volume_dim.csv',
		'vrts_vxvm_volume'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'VOLUME_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'VOLUME_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('VOLUME_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.VOLUME_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set VOLUME_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'VOLUME_FACTS',                     -- name
        'FACT',                             -- stereotype
        'VOLUME_CAPACITY',                  -- destination table
        'VOLUME_CAPACITY_',                 -- staging table


        'VOLUME_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, VOLUME_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, VOLUME_CURRENT.VOLUME_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.VOLUME_CURRENT on SRC.VOLUME_OID = VOLUME_CURRENT.VOLUME_OID',
		'volume_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set NAS_FS_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NAS_FS_DIM',                         -- name
        'DIMENSION',                          -- stereotype
        'NAS_FILESYSTEM',                     -- destination table
        'NAS_FILESYSTEM_',                    -- staging table
		null,


        'select $SRC_COLUMNS$, ' ||
               'ARRAY_CURRENT.ARRAY_KEY, ' ||
               'NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.ARRAY_CURRENT ' ||
            'on SRC.ARRAY_OID = ARRAY_CURRENT.ARRAY_OID ' ||
          'join SAHARA.NETAPP_VOLUME_CURRENT ' ||
            'on SRC.NETAPP_VOLUME_OID = NETAPP_VOLUME_CURRENT.NETAPP_VOLUME_OID',
		'nas_filesystem_dim.csv', 'vrts_array_nasfilesystem'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('NAS_FS_DIM', 'NAS_FILESYSTEM_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('NAS_FS_DIM', 'NAS_FILESYSTEM_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('NAS_FS_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.NAS_FILESYSTEM_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set NAS_FS_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'NAS_FS_FACTS',		            -- name
        'FACT',                             -- stereotype
        'NAS_FILESYSTEM_CAPACITY',          -- destination table
        'NAS_FILESYSTEM_CAPACITY_',         -- staging table


        'NAS_FILESYSTEM_KEY',


        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, NAS_FILESYSTEM_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.NAS_FILESYSTEM_CURRENT on SRC.NAS_FILESYSTEM_OID = NAS_FILESYSTEM_CURRENT.NAS_FILESYSTEM_OID',
		'nas_filesystem_facts.csv'
    );
end;

commit;


begin
	message '-- configuring data set CLUSTER_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'CLUSTER_DIM',                      -- name
        'DIMENSION',                        -- stereotype
        'CLUSTER',                          -- destination table
        'CLUSTER_',                         -- staging table
		null,
		null,
		'cluster_dim.csv',
		'vrts_cluster'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('CLUSTER_DIM', 'CLUSTER_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('CLUSTER_DIM', 'CLUSTER_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('CLUSTER_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.CLUSTER_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set CLUSTER_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'CLUSTER_FACTS',                    -- name
        'FACT',                             -- stereotype
        'CLUSTER_INVENTORY',                -- destination table
        'CLUSTER_INVENTORY_',               -- staging table
        'CLUSTER_KEY',

        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, CLUSTER_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, CLUSTER_CURRENT.CLUSTER_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.CLUSTER_CURRENT on SRC.CLUSTER_OID = CLUSTER_CURRENT.CLUSTER_OID',
		  'cluster_facts.csv'
    );
end;

commit;


begin
	message '-- configuring data set CLUSTER_REMOTECLUSTER...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'CLUSTER_REMOTECLUSTER_DIM',      -- name
        'DIMENSION',                             -- stereotype
        'CLUSTER_REMOTECLUSTER',            -- destination table
        'CLUSTER_REMOTECLUSTER_',           -- staging table
        null,
        'select $SRC_COLUMNS$, CLUSTER_CURRENT.CLUSTER_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.CLUSTER_CURRENT ' ||
            'on SRC.CLUSTER_OID = CLUSTER_CURRENT.CLUSTER_OID ',
		'clustertoremotecluster_dim.csv'
    );		
    call sahara.SP_DataDict_RollupDataSetColumn('CLUSTER_REMOTECLUSTER_DIM', 'CLUSTER_REMOTECLUSTER_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('CLUSTER_REMOTECLUSTER_DIM', 'CLUSTER_REMOTECLUSTER_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('CLUSTER_REMOTECLUSTER_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.CLUSTER_REMOTECLUSTER_OID || @URL2');
   
end;

commit;


begin
	message '-- configuring data set CLUSTER_HOST...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'CLUSTER_HOST_DIM',               -- name
        'DIMENSION',                             -- stereotype
        'CLUSTER_HOST',                     -- destination table
        'CLUSTER_HOST_',                    -- staging table
        null,
        'select $SRC_COLUMNS$, CLUSTER_CURRENT.CLUSTER_KEY, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.CLUSTER_CURRENT ' ||
            'on SRC.CLUSTER_OID = CLUSTER_CURRENT.CLUSTER_OID ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',            
	'clustertohost_dim.csv'
    );		
    call sahara.SP_DataDict_RollupDataSetColumn('CLUSTER_HOST_DIM', 'CLUSTER_HOST_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('CLUSTER_HOST_DIM', 'CLUSTER_HOST_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('CLUSTER_HOST_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.CLUSTER_HOST_OID || @URL2');    
end;

commit;


begin
	message '-- configuring data set SERVICEGROUP_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SERVICEGROUP_DIM',                 -- name
        'DIMENSION',                        -- stereotype
        'SERVICEGROUP',                     -- destination table
        'SERVICEGROUP_',                    -- staging table
		null,
		'select $SRC_COLUMNS$, CLUSTER_CURRENT.CLUSTER_KEY ' ||
          	'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          	'left outer join SAHARA.CLUSTER_CURRENT ' ||
            	'on SRC.CLUSTER_OID = CLUSTER_CURRENT.CLUSTER_OID ',
		'servicegroup_dim.csv',
		'vrts_vcs_servicegroup'
    );
    call sahara.SP_DataDict_RollupDataSetColumn ('SERVICEGROUP_DIM', 'SERVICEGROUP_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn ('SERVICEGROUP_DIM', 'SERVICEGROUP_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn ('SERVICEGROUP_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.SERVICEGROUP_OID || @URL2');
end;

commit;

begin
	message '-- configuring data set SERVICEGROUP_FACTS...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SERVICEGROUP_FACTS',               -- name
        'FACT',                             -- stereotype
        'SERVICEGROUP_INVENTORY',           -- destination table
        'SERVICEGROUP_INVENTORY_',          -- staging table
        'SERVICEGROUP_KEY',

        'insert into SAHARA."$DEST_TABLE$" ' ||
            '(ROLLUP_BATCH_KEY, DATES_KEY, SERVICEGROUP_KEY, $DEST_COLUMNS$) ' ||
        'select ' ||
            '$ROLLUP_BATCH_KEY$, $DATES_KEY$, SERVICEGROUP_CURRENT.SERVICEGROUP_KEY, $SRC_COLUMNS$ ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.SERVICEGROUP_CURRENT on SRC.SERVICEGROUP_OID = SERVICEGROUP_CURRENT.SERVICEGROUP_OID',
		  'servicegroup_facts.csv'
    );
end;

commit;


begin
	message '-- configuring data set SERVICEGROUP_HOST...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SERVICEGROUP_HOST_DIM',          -- name
        'DIMENSION',                             -- stereotype
        'SERVICEGROUP_HOST',                -- destination table
        'SERVICEGROUP_HOST_',               -- staging table
        null,
        'select $SRC_COLUMNS$, SERVICEGROUP_CURRENT.SERVICEGROUP_KEY, HOST_CURRENT.HOST_KEY ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ' ||
          'join SAHARA.SERVICEGROUP_CURRENT ' ||
            'on SRC.SERVICEGROUP_OID = SERVICEGROUP_CURRENT.SERVICEGROUP_OID ' ||
          'join SAHARA.HOST_CURRENT ' ||
            'on SRC.HOST_OID = HOST_CURRENT.HOST_OID',            
	'servicegrouptohost_dim.csv'
    );		
    call sahara.SP_DataDict_RollupDataSetColumn('SERVICEGROUP_HOST_DIM', 'SERVICEGROUP_HOST_OID', 'STAGING KEY');
    call sahara.SP_DataDict_RollupDataSetColumn('SERVICEGROUP_HOST_DIM', 'SERVICEGROUP_HOST_KEY', 'NONE');
    call sahara.SP_DataDict_RollupDataSetColumn('SERVICEGROUP_HOST_DIM', 'CC_STORAGE_URL', 'TYPE 1',
                            '@URL1 || SRC.SERVICEGROUP_HOST_OID || @URL2');    
end;

begin
        message '-- configuring historical events...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'SERVICEGROUP_HIST_EVENTS',                      -- name
        'CUSTOM',                           -- stereotype
        'SERVICEGROUP_HIST_EVENTS',                 -- destination table
        'SERVICEGROUP_HIST_EVENTS_',                -- staging table
                null,

        'call sahara.SP_SericeGroupHistoricalEvents()',
        'servicegroup_historical_facts.csv'
    );
end;

commit;

begin
	message '-- configuring data set ETL_LOG_DIM...' to log;
    call sahara.SP_DataDict_RollupDataSetType
    (
        'ETL_LOG_DIM',                       -- name
        'CUSTOM',                           -- stereotype
        'ETL_LOG',                            -- destination table
    	'ETL_LOG_',                -- staging table
		 null,

		'insert into SAHARA."$DEST_TABLE$" ' ||
            '(BATCH_KEY, "TIME", PROCEDURE_NAME, SEVERITY, "MESSAGE") ' ||
        'select ' ||
            'BATCH_KEY, "TIME", PROCEDURE_NAME,SEVERITY, "MESSAGE" ' ||
          'from SAHARA_APP."$SRC_TABLE$" SRC ',
		'etl_log.csv'
    );
end;

commit;


-------------------------------------------------------------------------------
--
-- which roll-up source types support which data set types
--

-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'APP_GROUP_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'APP_GROUP_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'APP_GROUP_HOST');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'ARRAY_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'ARRAY_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'CUSTOM_ATTR');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'DB_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'DB_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'FS_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'FS_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'HOST_ARRAY_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'HOST_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SF_LICENSE_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SF_PRODUCT_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SF_LICENSE_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SF_LICENSE_DEPLOYMENT_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SF_DEPLOYED_LICENSES_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'LUN_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'LUN_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'LUN_HOST');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'LUN_FILESYSTEM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'LUN_DISKGROUP');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'LUN_VOLUME');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SWITCH_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SWITCH_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'DG_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'DG_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'VOLUME_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'VOLUME_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'CLUSTER_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'CLUSTER_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'CLUSTER_REMOTECLUSTER_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'CLUSTER_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SERVICEGROUP_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SERVICEGROUP_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'SERVICEGROUP_HOST_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'DM_BATCH');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'FILE_CATEGORY_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_DM_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_FILE_CATEGORY_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_HAL_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_QTREE_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_QTREE_FILE_CATEGORY_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_QTREE_FILE_DM_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_QTREE_FILE_HAL_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_SHARE_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_SHARE_FILE_CATEGORY_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_SHARE_FILE_DM_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_SHARE_FILE_HAL_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_VOLUME_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_VOLUME_FILE_CATEGORY_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_VOLUME_FILE_DM_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NETAPP_VOLUME_FILE_HAL_FACTS');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NAS_FS_DIM');
-- call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.0 RU1', 'NAS_FS_FACTS');
commit;

call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'ARRAY_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'ARRAY_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'CUSTOM_ATTR');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'DB_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'DB_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'FS_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'FS_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'HOST_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SF_LICENSE_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SF_PRODUCT_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SF_LICENSE_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SF_LICENSE_DEPLOYMENT_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SF_DEPLOYED_LICENSES_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'LUN_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'LUN_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'LUN_HOST');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'LUN_FILESYSTEM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'LUN_DISKGROUP');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'LUN_VOLUME');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'DG_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'DG_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'VOLUME_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'VOLUME_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'CLUSTER_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'CLUSTER_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'CLUSTER_REMOTECLUSTER_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'CLUSTER_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SERVICEGROUP_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SERVICEGROUP_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SERVICEGROUP_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 4.1', 'SERVICEGROUP_HIST_EVENTS');

commit;

call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'ARRAY_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'ARRAY_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'CUSTOM_ATTR');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DB_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DB_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'FS_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'FS_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'HOST_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SF_LICENSE_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SF_PRODUCT_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SF_LICENSE_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SF_LICENSE_DEPLOYMENT_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SF_DEPLOYED_LICENSES_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'LUN_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'LUN_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'LUN_HOST');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'LUN_FILESYSTEM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'LUN_DISKGROUP');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'LUN_VOLUME');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DG_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'DG_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'VOLUME_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'VOLUME_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'CLUSTER_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'CLUSTER_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'CLUSTER_REMOTECLUSTER_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'CLUSTER_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SERVICEGROUP_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SERVICEGROUP_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SERVICEGROUP_HOST_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'SERVICEGROUP_HIST_EVENTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NETAPP_VOLUME_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NETAPP_VOLUME_DIM2');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NETAPP_VOLUME_FILE_HAL_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NAS_FS_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NAS_FS_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NETAPP_SHARE_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NETAPP_SHARE_FILE_HAL_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NAS_QUOTA_DIM');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NAS_QUOTA_FACTS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NAS_VOLUME_FILER');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NAS_SHARE_HOST');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'NAS_SHARE_HOSTFS');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'ARRAY_DIM_VFILER');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'ARRAY_DIM_CLUSTNODE');
call sahara.SP_DataDict_BindRDSTypeToRSType('Veritas Operations Manager 5.0', 'ETL_LOG_DIM');
commit;

------------------------------------------------------------------------------------------------------------------

END;
/* --- */

BEGIN
  DECLARE @vSQL LONG VARCHAR;
  DECLARE @vStagingTableName VARCHAR(1024);
  DECLARE @vStagingColExists INTEGER;  
  
  

FOR AttributesLOOP as AttributesCUR dynamic scroll cursor for
      select CUSTOM_ATTRIBUTE_KEY as CUSTOM_ATTRIBUTE_KEY_R,
        CUSTOM_ATTRIBUTE_CONTEXT as CUSTOM_ATTRIBUTE_CONTEXT_R,
        TYPE as TYPE_R,
        DISPLAY_NAME as DISPLAY_NAME_R,
        COLUMN_NAME as COLUMN_NAME_R,
        COLUMN_TYPE as COLUMN_TYPE_R,
        SOURCE_NAME as SOURCE_NAME_R    
      from SAHARA.CUSTOM_ATTRIBUTE CONFIGURED
      where CONFIGURED.SOURCE_NAME not in ('ORGANIZATION_BE','APPLICATION_BE', 'ORGANIZATION_COMPOSITE_BE','APPLICATION_COMPOSITE_BE','BUSINESSENTITY')
	    and exists (select 1
                         from SAHARA.P_CUSTOM_ATTRIBUTE PUBLISHED
                        where PUBLISHED.CUSTOM_ATTRIBUTE_KEY =
                              CONFIGURED.CUSTOM_ATTRIBUTE_KEY)
     DO            

            

                if SAHARA.fn_HELPER_ColumnExists('SAHARA_APP',CUSTOM_ATTRIBUTE_CONTEXT_R || '_',
                    COLUMN_NAME_R) = 0 THEN
                    SET @vSQL = 'lock table ' ||
                    -- source table or view
                    'SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R || '_ ' ||
                            'in exclusive mode';

                    EXECUTE IMMEDIATE @vSQL;

                    SET @vSQL = 'alter table SAHARA_APP.' || CUSTOM_ATTRIBUTE_CONTEXT_R
                        || '_ add ("' || COLUMN_NAME_R || '" ' || COLUMN_TYPE_R || ')';
                    EXECUTE IMMEDIATE @vSQL;                    
                end if;    	        
            -- release the table locks
      COMMIT ;
     END FOR;     
END;

delete from SAHARA.DB_VERSION;
insert into SAHARA.DB_VERSION(MAJOR, MINOR, MP_VERSION) on existing update values (5,0,0);

-- Code to add the status column to the business view table.  Alter table will only happen if the column exists.
BEGIN
declare @SQLQuery long varchar;
declare @tablename varchar(255);


for BviewTablesCursor as BviewTablesLoop cursor for select ALIAS as aliasname
                         from sahara.BVIEW_DIMENSION
    do
            
            set @tablename = aliasname ||'_BVIEW_NODE' ;
            set @SQLQuery = '  if not exists ( select 1 from systable t,syscolumn c
                         where t.table_name = '''|| @tablename ||''' and c.table_id = t.table_id and  c.column_name = ''STATUS'' ) then
                            ALTER TABLE SAHARA.'|| @tablename || ' ADD STATUS integer default 0 ; END IF;';
            message @SQLQuery to client;
            execute immediate @SQLQuery;

end for;
END;
call sahara.SP_BVIEW_AddRuleKey();
-- updating is_last_of_week column of existing ROLLUP_BATCH table entries
call sahara.sp_Update_Rollup_Batch_IsLastOfWeek_columns();
commit;

call sahara.SP_recompile_views();
call SAHARA.SP_Missing_And_Summarization_after_Db_Upgrade();
call SAHARA.sp_summarization_purge_tables();
alter event EV_PURGE_SUMMARIZATION enable;
call sahara.InstallWorkerEvent(10);
